(* ::Package:: *)

(* eudiffgeov2.m *)

(* The second completed Version of eudiffgeo.m. *)
(* Contains the basic concepts of Euclidean Differential Geometry including the definitions
developed in the notebooks EuCurves.nb and euklsf.nb. *)

(* Created by Rolf Sulanke *)

(* Started April 21, 2009. Finisheded 2014.12.10 *)

(* This package contains tools applied in notebooks about Euclidean and
M\[ODoubleDot]bius differential geometry, mainly in three or two dimensions. *)

(* Hint:s. 

1. Be sure, that besides of this package also the packages euvec.m, and 
tensalg.m are contained in the working directory

2. After importing this package define the number
dim (= dimension)  of the Euclidean vector space as constants 
within the Global context of the notebook!!! 
This number may be changed later in the notebook, if wanted. If it is defined before 
importing the notebook it will be fixed for all the modules introduced in the packages!*)

(* Free software: Feel free to apply, distribute or change the packages and
notebooks of the item "Spheres". Please, cite the source and notice the
changes, or new concepts,  which you introduced.*)



(* WARNINGS.  Do not import this package into the notebooks EuCurves.nb, euklsf.nb, and 
loxodromes.nb; this would lead to confusion for Contexts and definitions!*)



BeginPackage["eudiffgeov2`",{"tensalg`", "euvec`"}] 
   
(* The usages *)

circle2D::usage = "circle2D[t, r:1] is the parameter representation of the dircle in the plane with 
radius r and center at the origin.";

circ2D::usage = "circ2D[m, r:1][t] is the parameter representation of the 
dircle in the plane with radius r and center m."

spirgr::usage = "spirgr[t,a:1] is the parameter representation of the spiral 
tansformation group with parameter a in the Euclidean plane; a = 0 yields 
the group of rotattions around the origin.";

spiral::usage = "spiral[a:1,b:1][t] is a parameter representation of the logarithmic spiral 
in orthonormal coordinates of the Euclidean plane; a = 0 yields the circle of radius b with
center at the origin.";

tangent::usage= "tangent[curve][t] yields the tangent unit vector of the  curve[t].";

graph::usage ="graph[f][x,y,...] is the canonical parameter representation of the 
graph {x,y,...f[x,y,...]} of the function f, in arbitrary dimension."; 

arclength::usage="arclength[curve][ x] yields the arc length of the curve curve[x] as an indefinite
Integral; curve[x] must be given in orthonormal Cartesian coordinates.";

arc::usage = "arc[curve,x,y0,y] yields the length of the arc of the curve curve[x]for y0<x<y; 
curve[x] must be given in orthonormal Cartesian coordinates.";

curvature::usage = "curvature[curve][t] yields the absolute value of the curvature 
of the regular curve[t] in any dimension, as a function of an arbitrary parameter t.";

curvaturevector::usage = "curvaturevector[x][t] is the curvature vector of the curve x[t] 
in the Euclidean n-Space, t an arbitrary parameter.";

torsion::usage = "torsion[curve][t] yields the torsion of the curve[t] in 
the Euclidean 3-space; it is defined only at points with non vanishing curvature.";

frenet::usage = "frenet[curve][t] yields the Frenet frame for a generally curved 
curve[t] in the n-dimensional Euclidean space. Generally curved means that the 
rank of the first n-1 derivatives of curve[t] must be n-1 at each point t.";

curvatures::usage = "curvatures[curve][t] yields the n-1 curvatures of a generally curved 
curve[t] in the n-dimensional Euclidean space. Generally curved means that the 
rank of the first n-1 derivatives of curve[t] must be n-1 at each point t.";

assu::usage ="assu[c][s] is a set of assumptions in the first four derivatives of the 
parameter representation c[s] of a smooth curve in the n-dimensional Euclidean space 
giving True if s is the Euclidean arc length of the curve.";

fr::usage ="fr[i][f][u,v], i = 0,1,2,3, is the i-th column of the matrix 
frame[f][u,v]. In particular fr[0] represents the point f[u,v] and fr[i] 
the i-th vector of the adapted orthonormal frame of the surface.";

frame::usage = "frame[f][u,v] is the element of the Euclidean group 
transforming the standard orthonormal frame {stb[i]}, i= 1,2,3,and 
the origin [0,0,0} into the adapted moving frame of the surface f[u.v].";

(* Concepts for immersions *)

d::usage = "d[f][u,v,w,...] is the differential of the function f[u,v,w...] of several 
variables with values in an  n-dimensional manifold or space. For the variables u, v, w,... 
the differentials d[u][u,v,...], d[v][u,v,... ],... yield the standard cobasis in the 
cotangent space of the parameter manifold at the point {u, v,...}. d[f,u,v,w,...][vl] is 
the value of the differential d[f][u,v,w,...] for the tangent vector vl.";

jac::usage = "jac[f][x1,...] is the Jacobian matrix of a map f of the m-dimensional 
psrameter manifold with coordinates {x1,...,xm) into the n-dimensional space.";

identity::usage = "identity[u1,...,um] ={u1,...,um} is the identity function on the 
manifold U with coordinates u1,...,um.";

\[Delta]u::usage = "\[Delta]u[i][u1,...,um] is the i-th standard basis vector field on the paramjeter 
manifold U with coordinates u1,...,um.";

tangsp::usage = "tangsp[f][u,...][x,...] is the tangent vector with standard 
coordinates x,... of the immersion f[u,...] at the point {u,...}.";

tv::usage = "tv[i][f][u,v,...]is the tangent vector of the i-th  parameter line of 
the immersion f[u,v,...] at the point {u, v,...}. If this point is regular these 
vectors for i = 1,...,Length[{u,v,...}] form a basis of the tangent space 
image[d[f][u,v,...]].";

vv::usage = "vv[a][x1,...,xm] is a vector field on the m-manifold with 
coordinates  {x1,...,xm} with components a[i][x1,...,xm], i = 1,...,m.";

tvf::usage = "tvf[a][f][u,v,...] is the tangent vestor field on the 
m-surface f[u] with coordinates a[i][u,v,...] with respect to the 
standard basis tv[i][f][u,v,...].";

rgT::usage = "For the immersion f of an m-dimensional manifold \!\(\*SuperscriptBox[\(X\), \(m\)]\) into the 
n-dimensional Euclidean space, m<=n, the matrix (rgT[i,j][f][u1,...,um]), 1<=i,j<=m, is the matrix of the 
coefficients of the Riemannian metric on \!\(\*SuperscriptBox[\(X\), \(m\)]\) induced by the immersion f.";

fund1::usage = "fund1[f][u1,..,um][x,y] is the value of the first fundamental form 
of the immersion f[u1,...,um] into the n_dimensional Euclidean space, 
for the tangent vectors x,y of the immersion.";

dVol::usage = "dVol[\[Gamma]\[Gamma]][u,v,...] is the volume element of the pseudo-Riemannian manifold 
with scalar product \[Gamma]\[Gamma]. See also dV,dA.";

dV::usage = "dV[f][u,v,...] is the volume element of the immersion f[u,v,...] in 
the Euclidean Space.. See also dVol,dA.";

(* Concepts for surfaces *)

tplane::usage = "tplane[f][uo,vo][x,y] is a parameter representation of the tangent 
plane of the surface f[u,v]in the affine n-space at the point {uo, vo} with parameters x,y.";

amf::usage = "amf[i][f][u,v]is the i-th vector, i = 1,2,3, of the orthonormal moving 
frame adapted to the parameter representation of the surface f[u,v] in the 
Euclidean 3-space.";

dA::usage = "dA[f][u,v] is the area element of the surface f[u,v] in the 
Euclidean n-space.";

dd::usage = "If h[u,v] is a differential form in two variables u,v or a vector of such 
differential forms then dd[h][u,v] is its exterior differential.";

\[Sigma]::usage = "{\[Sigma][i][f][u,v]}, i = 1,2, is the cobasis field to the orthonormal basis field 
in the tangent bundle of the surface f[u,v], \[Sigma][i][f][u,v]= {0,0} for 2<i<= dim. ";

\[Omega]::usage = "\[Omega][i,j][f][u,v] is the (i,j)-th structure form in the derivation equations 
of the surface f[u.v]. If dim = 3 one sets \[Omega][1]=\[Omega][2,3], \[Omega][2]=\[Omega][3,1], \[Omega][3]=\[Omega][1,2].";

gauss::usage = "gauss[f][u,v] is the Gaussian curvature of the surface f[u,v]
in the Euclidean 3-space.";

normalcurvature::usage = "normalcurvature[ff,hu,hv][t]is the normal curvature of 
the curve ff[hu[t],hv[t]], t an arbitrary parameter.";

prcurvatures::usage = "prcurvatures[f][u,v] yields the principal curvatures of 
the surface f[u,v] in the Euclidean 3-space.";

meancurvature::usage = "meancurvature[f][u,v] is the mean curvature of 
the surface f[u,v] in the Euclidean 3-space.";

fd2::usage = "{fd2[i,j][f][u,v], i,j = 1, 2} is the matrix of the coefficients  
with respect to the local standard basis {du,dv} of the second fundamental form 
of the surface f[u,v] in the Euclidean 3-space.";

pluckerconoid::usage = "pluckerconoid[u,v] is a parameter representation of Pl\[UDoubleDot]cker's conoid.";

conesf::usage = "conesf[y][u,v] is the general conical surface with vertex at the origin. 
It is defined by the  curve y[v] on the unit sphere.";

cylindersf::usage = "cylindersf[x,y][u,v] is the cylinder with rulings parallel to 
the z-axis erected over the curve {x[v],y[v],0} in the x,y-plane.";

tangsf::usage = "tangsf[x][u,v] is a parameter representation of the developable 
surface formed by the tangents of the space curve x[v].";

cg::usage= "cg[i][f][u,v], i = 1,2, are the coefficients in the decomposition of 
\[Omega][3][f][u,v] with respect to the cobasis {\[Sigma][i][f][u,v]}."

fund2::usage= "fund2[f][u,v][xx,yy] is the value of the second fundamental form of 
the surface f[u,v] in the Eucliddean 3-space for the tangent vectors xx,yy.";

cf::usage = "cf[i,j][f][u,v] are the coefficients  of the second fundamental form 
with respect to the cobasis {\[Sigma][i][f][u,v]} of the surface f[u,v] in the Euclidean 3-space.";

normals::usage = "normals[f,u,v][s] is the normal congruence of the surface f[u,v]
in the Eucjdean 3-space; s the line parameter.";

(* Riemannian Geometry *)

inverse::usage = "If {g[h,k][u,v,...]}is a field of square matrices the rank of 
which equals the number of variables u,v,..., then {inverse[i,j][g][u,v,...]} is 
the field of the inverse matrices.";

invrgT::usage = "(invrgT[i,j][f][u,v,...]) is the inverse of the matrix(rgT[i,j][f]]u,v,...])
of the coordinates of the first fundamental form of the immersion f[u,v,...].";

pT::usage = "For the immersion f[u,v,...] into the Euclidean n-space the function  
pT[[f][u,v,...][vx]is the orthogonal projection of the vector vx (its tangential component) 
at the point  f[u,v,...].";

christoffel1::usage = "christoffel1[i,j,k][gg][x,y,...] are the Christoffel symbols of the 
first kind of the pseudo-Riemannian metric {gg[i,j][x,y,...]} in local coordinates x,y,...";

\[Gamma]\[Gamma]::usage = "\[Gamma]\[Gamma][j,k,i][f][x,y,...]  are the Christoffel symbols of the first kind of the 
Riemannian metric rgT[i,j][f][x,y,...] of the immersion f[x,y,...] into the Euclidean n-space.";

\[CapitalGamma]::usage = "\[CapitalGamma][k,i,j][g][x,y,...] are the Christoffel symbols of the second kind of the 
pseudo-Riemannian metric {g[i,j][x,y,...]} in local coordinates x,y,...";

\[CapitalGamma]rgT::usage = "\[CapitalGamma]rgT[j,k,i][f][x,y,...]  are the Christoffel symbols of the second  kind of the 
Riemannian metric rgT[i,j][f][x,y,...] of the immersion f[x,y,...] into the Euclidean n-space.";

euclid::usage = "euclid[i,j][x,y,...] denotes the Riemannian metric of the Euclidean n-space.";

DD::usage = "DD[[g][vw][x,y,...] is the absolute Differential of the vector field vw[x,y,...] on 
a pseudo-Riemannian manifold with metric tensor g[i,j][x,y,...].";

DDimm::usage = "DD[[f][vw][x,y,...] is the absolute Differential of the vector field vw[x,y,...] 
on an immersion f[x,y,...]into the Euclidean n-space; here vw is a vector field on the 
parameter manifold. and the images of DDim lie in the Euclidean n-space.";

DD1::usage = "DD1[ff,cu,vw][t] is the absolute derivative of a vector function 
vw[t] ={vw1[t],vw2[t]} defined along a curve with local coordinatess cu[t]={u[t],v[t]} 
of a surface ff[u,v] in the n-dimensional Euclidean space. For higher dimensional 
immersions or Riemannian spaces create a new Module changing the definition of \[CapitalGamma]\[CapitalGamma] 
corresponding to the situation.";

geodcurvature::usage ="geodcurvature[f_,hu_,hv_][t_] is the geodesic curvature of the curve 
f[hu[t],hv[t]] on the surface f[u,v] in the Euclidean 3-space, t an arbitrary parameter.";

R::usage ="R[k,h,i,j][g][x,y,...] is the Riemann curvature tensor of the pseudo-Riemannian 
space with metric tensor g[i,j][x,y,...].";

Rimm::usage ="Rimm[k,h,i,j][f][x,y,...] is the Riemann curvature tensor of the 
immersion f[x,y,...] into the Euclidean n-space.";

covR::usage ="covR[i,j,k,l][g][x,y,...] is the covariant form of the Riemannian curvature tensor 
of the pseudo-Riemannian metric g[i.j][x,y,...].";

covRimm::usage ="covRimm[i,j,k,l][f][x,y,...] is the covariant form of the Riemannian curvature tensor
 of the immersion f[x,y,...] into the n-dimensional Euclidean space.";


Begin["`Private`"] 

(* dimm := Global`dim;*)

(* The spiral group in the plane *)

spirgr[t_,a_:1]:={{Cos[t],-Sin[t]},{Sin[t],Cos[t]}}*Exp[a*t];

spiral[a_:1, b_:1][t_] := spirgr[t,a].{b,0};

tangent[curve_][t_]:= Simplify[dotnorme[curve'[t]], t\[Element] Reals];

(* The graph of a function is formally very easy to define: *)

graph[f_][x___] := {x, f[x]};


arc[curve_,x_,y0_,y_]:= Simplify[Integrate[norm[curve'[x]],{x,y0,y}]];

arclength[curve_][x_]:= Simplify[Integrate[norm[curve'[x]],x]]; 

(* curvature[curve_][t_] :=norm[Cross[D[curve[t],t],D[curve[t],{t,2}]]]/(norm[D[curve[t],t]])^3;
14.11.15 changed to *)

curvature[curve_][t_] :=norm[ D[tangent[curve][t],t]]/norm[D[curve[t],t]];

curvaturevector[curve_][t_] :=  D[tangent[curve][t],t]/norm[D[curve[t],t]];

torsion[curve_][t_] := Det[{D[curve[t],t], D[curve[t],{t,2}],D[curve[t],{t,3}]}]/norm[Cross[D[curve[t],t], D[curve[t],{t,2}]]]^2;

frenet[curve_][t_] := Module[{n = Length[curve[t]], v,i,w},
v = Table[D[curve[t],{t,i}],{i,1,n-1}];
w = Simplify[esorthonorm[v]];
If[Length[w]<n-1, Print[curve[t] , "is not generally curved!"]; Return[]];
v= Append[w, cross[w]]; v];

curvatures[curve_][t_]:= Module[{n = Length[curve[t]],cv, cc,i},
cv = frenet[curve][t];
If[cv == Null, Return[]];
For[i=1, i<n,i++,cc[i] = D[cv[[i]],t].cv[[i+1]]/norm[D[curve[t],t]]];
vec[cc,n-1]];

assu[c_][t_] :=(c'[t].c'[t]== 1&&c'[t].c''[t]==0&&-c'[t].c'''[t]==c''[t].c''[t]&&
-Derivative[1][c][t].Derivative[4][c][t]==3*c'''[t].c''[t]);

circ2D[m_List,r_:1][t_]:= m+r*{Cos[t],Sin[t]};

circle2D[t_, r_:1] := {Cos[t],Sin[t]}*r;

(* Immerssions *)

(* differential *)

d[f_][x___]:= Module[{n=Length[{x}],yy,y,i,ff},
If[n==0, Return [0]];
If[MemberQ[{x},f], Return[D[{x},f]]];
ff=Evaluate[D[f[##],{{##}}]&];
yy =Table[y[i],{i,n}];
ff[yy/.List-> Sequence]/.Table[y[i]->{x}[[i]],{i,n}]];



d[h_,u___][l_]:=Module[{args ={u},n},
n=Length[args];
If[Not[(Head[l]===List)&&(Length[l] == n)],Print["Argument ", l, " incorrect!"];
Return[]];d[h][u].l];

jac[f_] :=Transpose[ d[f][##]]&;

tangsp[f_][u__][x__]:=If[Not[Length[{x}]== Length[{u}]],
Print["invalid arguments!"];Return[],Return[d[f][u].{x}]];
  
(* Standard tangent basis *)
tv[i_][f_][u__]:= If[IntegerQ[i]&&i>0&&i<=Length[{u}], 
If[Head[f[u]]== List&& Length[f[u]]>1,Transpose[d[f][u]][[i]],d[f][u][[i]],
d[f][u][[i]]],Print["Wrong i = ", i]];

identity:={##}&;

\[Delta]u[i_]:= tv[i][identity];

vv[a_]:=Table[a[i][##],{i,Length[{##}]}]&;

tvf[a_][ff_]:= Sum[tv[i][ff][##]*a[i][##],{i,Length[{##}]}]&;

(* Riemannian metric induced by an immersion *)
rgT[i_,j_][f_][u__] :=If[(IntegerQ[i]&&i>0&&i<=Length[{u}])&&
(IntegerQ[j]&&j>0&&j<=Length[{u}]),
If[Head[f[u]]== List&& Length[f[u]]>1,tv[i][f][u].tv[j][f][u]],
 Print["Wrong {i,j} = ", {i,j}]];

\!\(TraditionalForm\`fund1\)[f_][u__][x_List,y_List]:=
If[Length[x] == Length[{u}]&&Length[y] ==  Length[{u}],
Sum[rgT[i,j][f][u]*x[[i]]*y[[j]],{i,Length[{u}]},{j,Length[{u}]}],
Print["Wrong arguments!"];Return[]];

dV[ff_]:= Module[{m,i,j},m = Length[{##}];Sqrt[Det[Table[rgT[i,j][ff][##],{i,m},{j,m}]]]]&;

dVol[\[Gamma]\[Gamma]_]:= Module[{m,i,j},m = Length[{##}];Sqrt[Abs[Det[Table[\[Gamma]\[Gamma][i,j][##],{i,m},{j,m}]]]]]&;

(* Surfaces *)

(* tangent plane *)

tplane[f_][u0_,v0_][x_,y_]:= (f[u,v] + tangsp[f][u,v][x,y])/.{u->u0,v->v0};

(* Area element of a surface *)
dA[f_][u_,v_]:= Sqrt[ Det[Table[ rgT[i,j][f][u,v],{i,2},{j,2}]]];

(* Orthonormal moving frame of a surface *)
amf[1][f_][u_,v_]:= dotnorme[tv[1][f][u,v]]/;
(Head[f[u,v]] == List&&Length[f[u,v]]== 3);
amf[2][f_][u_,v_] := Cross[amf[3][f][u,v],amf[1][f][u,v]]/;
(Head[f[u,v]] == List&&Length[f[u,v]]== 3);
amf[3][f_][u_,v_]:= dotnorme[Cross[tv[1][f][u,v],tv[2][f][u,v]]]/;
(Head[f[u,v]] == List&&Length[f[u,v]]== 3);

fr[0][f_][u_,v_]:=Prepend[f[u,v],1];
fr[i_Integer][f_][u_,v_]:= Prepend[amf[i][f][u,v],0]/;(i>0)&&(i<4);

frame[f_][u_,v_] :=Transpose[Table[fr[j][f][u,v],{j,0,3}]];

(* Exterior differential for 2-forms *)
dd[h_][u_,v_] := Module[{n=1,pp={0,0},i},If[Head[h[u,v]]==List&&Length[h[u,v]]==2,
Return[D[h[u,v][[2]],u]- D[h[u,v][[1]],v]]];
If[MatrixQ[h[u,v]],pp = Dimensions[h[u,v]]; n=pp[[1]]];
If[pp[[2]]==2,
Return[Table[D[h[u,v][[i]][[2]],u]- D[h[u,v][[i]][[1]],v],{i,n}]]];
Print["invalid argument"];Return[]];

dd/: dd[h1_+h2_][u_,v_]:= dd[h1][u,v] +dd[h2][u,v];

(* Cobasis and structure forms*)
\[Sigma][i_][f_][u_,v_] :=If[(Length[f[u,v]]==3)&&(i>= 1)&&(i<= 3),
Return[amf[i][f][u,v].d[f][u,v]],
Print["Invalid argument!"];Return[]];

\[Omega][j_,k_][f_][u_,v_] := 
If[j>=1&&j<=3&&k>=1&&k<=3&&Length[f[u,v]]== 3,
Return[Simplify[amf[k][f][u,v].d[amf[j][f]][u,v]]],
Print["Invalid arguments!"];Return[]];

\[Omega][1][f_][u_,v_]:= \[Omega][2,3][f][u,v]/;Length[f[u,v]]==3;
\[Omega][2][f_][u_,v_]:= \[Omega][3,1][f][u,v]/;Length[f[u,v]]==3;
\[Omega][3][f_][u_,v_]:= \[Omega][1,2][f][u,v]/;Length[f[u,v]]==3;

cg[1][ff_][u_,v_] := dd[\[Sigma][1][ff]][u,v]/extprod[\[Sigma][1][ff][u,v], \[Sigma][2][ff][u,v]][[1,2]];
cg[2][ff_][u_,v_] := dd[\[Sigma][2][ff]][u,v]/extprod[\[Sigma][1][ff][u,v], \[Sigma][2][ff][u,v]][[1,2]];

(*Second fundamental form *)
fund2[ff_][u_,v_][x_List,y_List]:=
(d[ff][u,v].x).(d[amf[3][ff]][u,v].y);

fd2[1,1][ff_][u_,v_] := fund2[ff][u,v][{1,0},{1,0}];
fd2[1,2][ff_][u_,v_] := fund2[ff][u,v][{1,0},{0,1}];
fd2[2,2][ff_][u_,v_] := fund2[ff][u,v][{0,1},{0,1}];
fd2[2,1][ff_][u_,v_] := fund2[ff][u,v][{0,1},{1,0}];

cf[2,1][ff_][u_,v_]:= -extprod[\[Omega][1][ff][u,v],\[Sigma][2][ff][u,v]][[1,2]]/
extprod[\[Sigma][1][ff][u,v],\[Sigma][2][ff][u,v]][[1,2]];
cf[2,2][ff_][u_,v_]:= +extprod[\[Omega][1][ff][u,v],\[Sigma][1][ff][u,v]][[1,2]]/
extprod[\[Sigma][1][ff][u,v],\[Sigma][2][ff][u,v]][[1,2]];
cf[1,1][ff_][u_,v_]:= extprod[\[Omega][2][ff][u,v],\[Sigma][2][ff][u,v]][[1,2]]/
extprod[\[Sigma][1][ff][u,v],\[Sigma][2][ff][u,v]][[1,2]];
cf[1,2][ff_][u_,v_]:= extprod[\[Sigma][1][ff][u,v],\[Omega][2][ff][u,v]][[1,2]]/
extprod[\[Sigma][1][ff][u,v],\[Sigma][2][ff][u,v]][[1,2]];

(*normalcurvature *)
normalcurvature[ff_,h1_,h2_][t_]:= Module[{cvx,tt,vt,tx,aa,bb},
cvx[tt_] :={h1[tt],h2[tt]};
vt[tt_] := D[cvx[tx],tx]/.tx->tt;
aa = -fund2[ff][h1[tt],h2[tt]][vt[tt],vt[tt]]/.tt->t;
bb = fund1[ff][h1[tt],h2[tt]][vt[tt],vt[tt]]/.tt->t;aa/bb];

(* Principal curvatures *)
prcurvatures[ff_][u_,v_] :=Eigenvalues[ Table[cf[i,j][ff][u,v],{i,2},{j,2}]];

(* Mean curvature *)
meancurvature[ff_][u_,v_] := (cf[1,1][ff][u,v] + cf[2,2][ff][u,v])/2;

(* Gauss curvature *)
gauss[ff_][u_,v_] := 
extprod[\[Omega][1][ff][u,v],\[Omega][2][ff][u,v]][[1,2]]/
extprod[\[Sigma][1][ff][u,v],\[Sigma][2][ff][u,v]][[1,2]];

(*normal congruence *)
normals[ff_, u_,v_][s_] := ff[u,v] + amf[3][ff][u,v]*s;

(* Special surfaces *)

pluckerconoid[u_,v_] := {0,0, 2*Cos[v]*Sin[v]} + u*{Cos[v],Sin[v],0};

conesf[y_][u_,v_] := vec[y[v]]*u;

cylindersf[x_,y_][u_,v_] := {x[v],y[v],u};

tangsf[x_][u_,v_] := x[v] + D[x[v],v]*u;

(*Riemannian Geometry *)

inverse[i_,j_][g_][u__]:= Module[{m =Length[{u}],k,h,mM},
mM = Table[g[h,k][u],{h,m},{k,m}];
Inverse[mM][[i,j]]];

invrgT[i_,j_][ff_][u__]:= Module[{gg,m =Length[{u}],k,h,v},
gg[h_,k_][v__] := rgT[h,k][ff][v] ;
inverse[i,j][gg][u]];

pT[ff_][x__]:= 
Sum[tv[k][ff][x]*Sum[invrgT[k,j][ff][x]*(tv[j][ff][x].#),{j,Length[{x}]}],{k,Length[{x}]}]&;

christoffel1[j_,k_,i_][g_][x__]:= 
(D[g[i,j][x],{x}[[k]]] +D[g[j,k][x],{x}[[i]]] -D[g[k,i][x],{x}[[j]]])/2;

\[Gamma]\[Gamma][j_,k_,i_][ff_][x__] := Module[{g,ii,jj,y},
g[ii_,jj_][y__]:= rgT[ii,jj][ff][y];christoffel1[j,k,i][g][x]];

\[CapitalGamma][k_,i_,j_][g_]:=Sum[inverse[k,l][g][##]*christoffel1[l,i,j][g][##],{l,Length[{##}]}]&;

\[CapitalGamma]rgT[k_,i_,j_][ff_]:= Sum[invrgT[k,l][ff][##]*\[Gamma]\[Gamma][l,i,j][ff][##],{l,Length[{##}]}]&;

euclid[i_,j_]:=rgT[i,j][identity][##]&;

DD[g_][vw_][x__]:= Transpose[Simplify[
Table[D[vw[x][[k]],{x}[[j]]]+Sum[\[CapitalGamma][k,i,j][g][x]*vw[x][[i]],
{i,Length[{x}]}],{j,Length[{x}]},{k,Length[{x}]}]]];

DDimm[f_][vw_][x__]:= d[f][x].Transpose[Simplify[Table[D[vw[x][[k]],{x}[[j]]]+
Sum[\[CapitalGamma]rgT[k,i,j][f][x]*vw[x][[i]],{i,Length[{x}]}],{j,Length[{x}]},{k,Length[{x}]}]]];

DD1[ff_,cu_,vw_][t_] := Module[{mf,\[CapitalGamma]\[CapitalGamma],s, i,j,k,u,v},
mf = Length[cu[t]];
\[CapitalGamma]\[CapitalGamma][i_,j_,k_][s_]:=\[CapitalGamma]rgT[i,j,k][ff][u,v]/.{u-> cu[s][[1]],v->cu[s][[2]]};
D[vw[t],t] + Table[Sum[\[CapitalGamma]\[CapitalGamma][i,j,k][t]*vw[t][[j]]*D[cu[t][[k]],t],{j,mf},{k,mf}],{i,mf}]];

geodcurvature[ff_,hu_,hv_][s_]:= Module[{cuv,t,aa,bb,abc,vbc,tt},
cuv[t_]= {hu[t],hv[t]};
abc[t_] =D[cuv[tt],tt]/.tt->t;
bb[t_] = fund1[ff][hu[t],hv[t]][abc[t],abc[t]];
vbc[t_] =abc[t]/Sqrt[bb[t]];
aa[t_] = fund1[ff][hu[t],hv[t]][DD1[ff,cuv,vbc][t],DD1[ff,cuv,vbc][t]];
Sqrt[aa[s]/(bb[s])]];

R[k_,i_,h_,j_][g_][x__] := 
D[\[CapitalGamma][k,i,j][g][x],{x}[[h]]] - D[\[CapitalGamma][k,i,h][g][x],{x}[[j]]] - 
Sum[(\[CapitalGamma][k,l,j][g][x]*\[CapitalGamma][l,i,h][g][x]- \[CapitalGamma][k,l,h][g][x]*\[CapitalGamma][l,i,j][g][x]),{l,Length[{x}]}];

Rimm[k_,i_,h_,j_][ff_][x__] := 
D[\[CapitalGamma]rgT[k,i,j][ff][x],{x}[[h]]] -  D[\[CapitalGamma]rgT[k,i,h][ff][x],{x}[[j]]] - 
Sum[(\[CapitalGamma]rgT[k,l,j][ff][x]*\[CapitalGamma]rgT[l,i,h][ff][x]- \[CapitalGamma]rgT[k,l,h][ff][x]*\[CapitalGamma]rgT[l,i,j][ff][x]),{l,Length[{x}]}];

covR[k_,i_,h_,j_][g_][x__] :=  Sum[g[k,l][x]*R[l,i,h,j][g][x],{l,Length[{x}]}];

covRimm[k_,i_,h_,j_][ff_][x__] :=  Sum[rgT[k,l][ff][x]*Rimm[l,i,h,j][ff][x],{l,Length[{x}]}];


End[]

Protect["amf","arc","arclength","assu","cf","cg","christoffel1","circ2D","circle2D",
"conesf","covR","covRimm","curvature","curvatures","curvaturevector","cylindersf","d",
"dA","dd","DD","DD1","DDimm","dV","dVol","euclid","fd2","fr","frame","frenet","fund1",
"fund2","gauss","geodcurvature","graph","identity","inverse","invrgT","jac",
"meancurvature","normalcurvature","normals","pluckerconoid","prcurvatures","pT",
"R","rgT","Rimm","spiral","spirgr","tangent","tangsf","tangsp","torsion","tplane",
"tv","tvf","vv","\[CapitalGamma]","\[CapitalGamma]rgT","\[Gamma]\[Gamma]","\[Delta]u","\[Sigma]","\[Omega]"];
	

EndPackage[]




