(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 10.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[   7042577,     182964]
NotebookOptionsPosition[   6805214,     176631]
NotebookOutlinePosition[   6821194,     177052]
CellTagsIndexPosition[   6817100,     176952]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Surfaces in the Euclidean 3-Space", "Title",
 CellChangeTimes->{3.588487292274865*^9},
 TextAlignment->Center],

Cell["by Rolf Sulanke", "Subtitle",
 CellChangeTimes->{{3.5942757655247216`*^9, 3.594275786184903*^9}, 
   3.59427585467282*^9, {3.5942759066557937`*^9, 3.594275907880864*^9}, {
   3.6210775010971065`*^9, 3.621077519098136*^9}, {3.6236556769993553`*^9, 
   3.6236556822253647`*^9}},
 TextAlignment->Center],

Cell[TextData[{
 "Finished December 31, 2014\n",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " v. 9, v. 10"
}], "Text",
 CellChangeTimes->{{3.594368087244899*^9, 3.5943680914569063`*^9}, {
  3.614665568276914*^9, 3.6146656059665804`*^9}, {3.6174421006498613`*^9, 
  3.6174421041850634`*^9}, {3.6189956491303053`*^9, 3.618995673793949*^9}, {
  3.620532390543178*^9, 3.620532431056449*^9}, {3.626855473912151*^9, 
  3.626855489044177*^9}, {3.6291885008789363`*^9, 3.6291885014873376`*^9}}],

Cell[CellGroupData[{

Cell["Preface", "Subsection",
 CellChangeTimes->{
  3.541836408310753*^9, 3.5418367181098843`*^9, {3.5992053438926544`*^9, 
   3.5992053481638985`*^9}, {3.5994627461488314`*^9, 3.599462748535635*^9}}],

Cell[TextData[{
 "In this notebook we develop ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " tools for applications  to Euclidean differential geometry of surfaces. We \
construct modules for the calculation of all Euclidean invariants like \
fundamental forms and curvatures for surfaces in the 3-space, and some also \
for immersions into the n-dimensional Euclidean spaces.  This notebook may be \
considered as a continuation of the notebook EuCurves.nb devoted to  \
Euclidean curve theory which may be downloaded from my ",
 ButtonBox["homepage",
  BaseStyle->"Hyperlink",
  ButtonData->"Home"],
 ". It may be used as an interactive introduction into the Euclidean surface \
geometry, accompanying an introductory course or reading a textbook, e.g. the \
Introduction ",
 ButtonBox["[B-R]",
  BaseStyle->"Hyperlink",
  ButtonData->"B-R"],
 "  of  W. Blaschke and H. Reichardt which I often use as a  reference.  It \
may be useful to compare and to complete this notebook studying the \
pioneering work of Alfred Gray on applying ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to differential geometry, see  the book [",
 ButtonBox["G06",
  BaseStyle->"Hyperlink",
  ButtonData->"Gray"],
 "], or the German translation of the first edition [",
 ButtonBox["G94",
  BaseStyle->"Hyperlink",
  ButtonData->"Gray3"],
 "]. The third edition [G06] is accompanied by a series of 27 ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notebooks corresponding to the chapters of the book, which can be \
downloaded from the website ",
 ButtonBox["[G06nb]",
  BaseStyle->"Hyperlink",
  ButtonData->"Gray2"],
 ".  Alfred Gray presented Euclidean differential geometry with many \
applications of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". Writing his book Alfred Gray collected about 200 definitions of concrete \
curves and as many of concrete surfaces with short descriptions (formulated \
as usages) and implementations into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". These catalogs are contained in his ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " packages  CURVES.m and SURFS.m  available from my ",
 ButtonBox["homepage",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www-irm.mathematik.hu-berlin.de/~sulanke/diffgeo.html#Gray"], 
    None},
  ButtonNote->
   "http://www-irm.mathematik.hu-berlin.de/~sulanke/diffgeo.html#Gray"],
 ".  They open many opportunities to apply the functions and miniprograms \
developed here, to deepen the insight into the differential geometry of \
curves and surfaces comparing the calculated invariants, e. g. the \
curvatures, with the shape of the geometric objects using the graphical tools \
of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ".  I am much obliged to Alfred Gray for providing this material and for \
many discussions introducing me into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". \n\nChapter 1 of the notebook is devoted to E. Cartan\[CloseCurlyQuote]s \
method of moving frames using exterior differential forms. It gives the \
structure forms and the Frenet formulas including the integrability \
conditions for surfaces in the Euclidean 3-space. Here I followed the \
presentation in [B-R] using the differential forms ad hoc as elementary as \
possible. Also the first and the second fundamental forms of surface theory \
are treated as fields of bilinear forms not using concepts of tensor \
analysis. Spherical image and Gauss curvature are the subject of Chapter 2 \
containing a proof of Gauss famous Theorema Egregium.  Chapter 3 is devoted \
to the second fundamental form of the surface, studying the geometry of \
curves on the surface, in particular their normal curvature. Chapter 4 \
contains the absolute differentiation on the surface, expressed by the \
Christoffel symbols as a part of the inner geometry of the surface, invariant \
under isometric deformations. It is applied to describe the geodesic \
curvature of a curve on the surface and to define the geodesics as \
auto-parallel curves. The basic concepts are formulated avoiding the tensor \
concept, but admitting a straightforward extension to higher dimensional \
Riemannian and more general geometries, which can be the contents of further ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notebooks.\n\nI hope that the notebook will help students to understand \
the Euclidean differential geometry of surfaces. By its interactive character \
it may serve as a starting point for further excursions into this interesting \
field, which joins algebraic and analytic methods to explore the heart of \
mathematics: the geometry. Studying older or modern textbooks or monographs \
of classical differential geometry it may be an exciting challenge to express \
the objects and concepts in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " terms aiming to explore or illustrate their properties, to verify well \
known facts, and, with some luck, to gain new insight.   \n\nI shall be very \
thankful for comments and corrections, please send them to \n\
sulanke@mathematik.hu-berlin.de.\n\n",
 StyleBox["Important Hint. ",
  FontColor->RGBColor[1, 0, 0]],
 "\n",
 StyleBox["Before starting working with the notebook read the Copyrigth and \
make the necessary adaption of the Section Initialization to your computer! \n\
",
  FontSlant->"Italic"],
 "\n"
}], "Text",
 CellChangeTimes->{{3.446645069921875*^9, 3.44664509953125*^9}, 
   3.446645156828125*^9, {3.446645243484375*^9, 3.446645462125*^9}, {
   3.448159668953125*^9, 3.448159713765625*^9}, 3.449203063109375*^9, {
   3.4542178296875*^9, 3.4542180669375*^9}, {3.454218801*^9, 
   3.454218865875*^9}, {3.4542223853125*^9, 3.4542224753125*^9}, {
   3.454222520640625*^9, 3.45422255271875*^9}, {3.45422428384375*^9, 
   3.45422444978125*^9}, {3.45422448125*^9, 3.454224575171875*^9}, 
   3.541681068146072*^9, {3.541682273824328*^9, 3.5416824190006227`*^9}, {
   3.541827375454864*^9, 3.541827387969111*^9}, {3.5418316529213343`*^9, 
   3.541831653441086*^9}, 3.5418317494023857`*^9, {3.541831858219041*^9, 
   3.541832023482676*^9}, {3.5418320577240677`*^9, 3.5418320930359087`*^9}, {
   3.541832178020227*^9, 3.541832187404354*^9}, {3.5418322197005577`*^9, 
   3.541832219702417*^9}, {3.5418322675411787`*^9, 3.541832354019971*^9}, 
   3.541836408310801*^9, {3.5418366180057*^9, 3.541836621524942*^9}, 
   3.541836718109921*^9, {3.59919611723792*^9, 3.599196122851241*^9}, {
   3.5991962025988026`*^9, 3.5991962289503098`*^9}, {3.5991962615901766`*^9, 
   3.599196336339452*^9}, {3.5991963763887424`*^9, 3.5991964222833676`*^9}, {
   3.5991964577643967`*^9, 3.599196467277941*^9}, {3.599196506140164*^9, 
   3.5991965150836754`*^9}, {3.5991965605252743`*^9, 3.5991965712928905`*^9}, 
   3.5992019728598423`*^9, {3.5992020307651544`*^9, 3.5992021570203757`*^9}, 
   3.599202305491868*^9, 3.5992032715491233`*^9, {3.59920388218805*^9, 
   3.599203905548386*^9}, {3.5992040181558266`*^9, 3.599204028124397*^9}, {
   3.599204085731692*^9, 3.599204105851843*^9}, {3.5992041385407124`*^9, 
   3.59920414934033*^9}, {3.599204190283672*^9, 3.5992042911484413`*^9}, {
   3.5992043640766125`*^9, 3.599204371660046*^9}, {3.59920441124231*^9, 
   3.599204411255311*^9}, {3.5992044716767664`*^9, 3.5992045492682047`*^9}, {
   3.5992045954358454`*^9, 3.599204629315783*^9}, {3.599204741172181*^9, 
   3.599204764548518*^9}, {3.5992047948372507`*^9, 3.5992048358365955`*^9}, {
   3.5992050362200565`*^9, 3.5992050588603516`*^9}, {3.599205134556681*^9, 
   3.599205261883964*^9}, 3.59920536304775*^9, {3.5992053937465057`*^9, 
   3.5992053939325166`*^9}, {3.5992054591412463`*^9, 
   3.5992055378927507`*^9}, {3.5992056833880725`*^9, 
   3.5992057380601997`*^9}, {3.59920582041291*^9, 3.599205820627922*^9}, {
   3.5992058827964783`*^9, 3.599206043644678*^9}, {3.5992060798207474`*^9, 
   3.599206163004505*^9}, {3.5992062112202625`*^9, 3.5992062367237215`*^9}, {
   3.5992063417807302`*^9, 3.5992063865722923`*^9}, {3.599206474428317*^9, 
   3.5992064839718633`*^9}, {3.599206598188396*^9, 3.599206636628595*^9}, {
   3.5992066825932236`*^9, 3.5992066871884866`*^9}, 3.5992067173322105`*^9, {
   3.599206779087743*^9, 3.599206830683694*^9}, {3.5992068737641582`*^9, 
   3.5992069266761847`*^9}, {3.5992069590400352`*^9, 
   3.5992069796602154`*^9}, {3.5992070286610174`*^9, 
   3.5992070416127586`*^9}, {3.5992070903205442`*^9, 3.599207090334545*^9}, {
   3.5992078881661787`*^9, 3.59920812438869*^9}, {3.599226725980919*^9, 
   3.599226730632185*^9}, {3.599226794136817*^9, 3.5992268145319834`*^9}, 
   3.5992268515100985`*^9, {3.599227048665375*^9, 3.599227050600486*^9}, {
   3.599227089888733*^9, 3.599227090432764*^9}, {3.5992274882425175`*^9, 
   3.599227521310409*^9}, {3.5992276061852636`*^9, 3.5992276061962643`*^9}, {
   3.599227931864891*^9, 3.599227961944612*^9}, {3.599227993192399*^9, 
   3.59922802048096*^9}, {3.5992282650969515`*^9, 3.5992282958327093`*^9}, {
   3.59922839935363*^9, 3.5992285024725285`*^9}, 3.59928617084793*^9, {
   3.5992862198164163`*^9, 3.599286314914183*^9}, {3.5994012750208035`*^9, 
   3.599401312258069*^9}, {3.5994013738781776`*^9, 3.5994014292114744`*^9}, {
   3.5994014605675297`*^9, 3.599401482969169*^9}, {3.599401529472851*^9, 
   3.5994018439226027`*^9}, 3.59940195034599*^9, 3.5994020589533806`*^9, 
   3.5994020911986375`*^9, {3.599402465084494*^9, 3.5994026765740657`*^9}, {
   3.599402733092965*^9, 3.5994027337949657`*^9}, {3.5994028421371565`*^9, 
   3.599402909529275*^9}, {3.599402944488936*^9, 3.599403082112378*^9}, {
   3.5994045003111925`*^9, 3.599404706743*^9}, {3.599405087461776*^9, 
   3.5994053763022966`*^9}, {3.5994054519166217`*^9, 
   3.5994054540677443`*^9}, {3.5994056666029005`*^9, 3.5994057112824564`*^9}, 
   3.599405747266515*^9, {3.599405778443298*^9, 3.5994057958012905`*^9}, 
   3.59940586659534*^9, {3.599462817206956*^9, 3.5994629289187517`*^9}, {
   3.599462978480039*^9, 3.59946316121876*^9}, {3.5994633805518007`*^9, 
   3.599463389584317*^9}, 3.614665761857654*^9, {3.6146658697162437`*^9, 
   3.6146659078115106`*^9}, {3.6146660355913353`*^9, 
   3.6146660382433395`*^9}, {3.6189973384975014`*^9, 3.6189973435597906`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Keywords", "Subsection",
 CellChangeTimes->{{3.541686221104368*^9, 3.541686224511538*^9}, 
   3.5418364083108387`*^9, 3.54183671810995*^9}],

Cell["\<\
absolute differential, absolute parallel, arc length, area, asymptotic \
curves, Christoffel symbols, cone, conical surface, cylindrical surface, \
developable surface, diffeomorphism, differential, envelope, Euler\
\[CloseCurlyQuote]s formula, evolute, exterior differential, exterior \
product, flat points, Frenet formulas, fundamental forms, Gauss curvature, \
Gauss map, geodesic, geodesic curvature, graph, helix, hypersurface, \
immersion,  integrability conditions, integral curvature, involute, isogonal \
trajectory, Jacobian matrix, line congruences, loxodrome, mean curvature, \
Meusnier\[CloseCurlyQuote]s proposition,  minimal surface, moving frame, \
normal, normal congruence, normal curvature, orientation, osculating sphere, \
parallel displacement, parameter, Pl\[UDoubleDot]cker\[CloseCurlyQuote]s \
conoid, principal curvatures, principal curves, principal directions,  \
regular, Riemannian curvature tensor, Riemannian manifold, \
R\[UDoubleDot]ckkehrkante,  ruled surface, singular, spherical image, spiral, \
structure forms, submanifold, surface, surface of revolution, surface with \
constant Gauss curvature, tangent plane, tangent surface, tangent vector, \
Theorema egregium, torus, total differential, umbilic point, vector field, \
volume element.\
\>", "Text",
 CellChangeTimes->{
  3.6146702447129803`*^9, {3.6146743760906925`*^9, 3.6146744046253242`*^9}, {
   3.614674471174131*^9, 3.6146744956065283`*^9}, {3.6146745698667755`*^9, 
   3.6146745918020306`*^9}, 3.6146746377306576`*^9, 3.6146748836907253`*^9, {
   3.614774678232683*^9, 3.6147746844320374`*^9}, {3.61477471629686*^9, 
   3.614774744056448*^9}, {3.6151888712415533`*^9, 3.615188905226497*^9}, {
   3.6151890327547913`*^9, 3.615189040579239*^9}, {3.615361391244278*^9, 
   3.6153614969653244`*^9}, {3.615362023628448*^9, 3.615362032140935*^9}, {
   3.617442283553323*^9, 3.6174423273218265`*^9}, {3.6174423670821004`*^9, 
   3.617442374049499*^9}, {3.6174426996831245`*^9, 3.617442706369507*^9}, {
   3.617443035034305*^9, 3.6174430418576956`*^9}, {3.6174431918342733`*^9, 
   3.6174432005217705`*^9}, {3.617443243458226*^9, 3.6174432561699533`*^9}, {
   3.6174433063858256`*^9, 3.6174433213856835`*^9}, 3.617443397977064*^9, {
   3.6174460763694725`*^9, 3.6174461022459526`*^9}, {3.6176052340530386`*^9, 
   3.617605238296246*^9}, {3.6177013931950407`*^9, 3.617701402755587*^9}, {
   3.6179404294297233`*^9, 3.6179404672753897`*^9}, {3.6179405303463*^9, 
   3.617940542997923*^9}, {3.617940949144636*^9, 3.617940969565072*^9}, {
   3.61795167381975*^9, 3.6179516976201115`*^9}, {3.6183981333230515`*^9, 
   3.618398144002662*^9}, {3.6260675001325064`*^9, 3.6260675184001384`*^9}, {
   3.626067551493797*^9, 3.626067557735808*^9}, 3.626855590005945*^9, {
   3.627114607273288*^9, 3.627114624464518*^9}, {3.627187931915095*^9, 
   3.6271879412283115`*^9}, {3.629188541491808*^9, 3.6291885699054585`*^9}, {
   3.629188618322344*^9, 3.629188632489169*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Copyright", "Subsection",
 CellChangeTimes->{{3.541686221104368*^9, 3.541686224511538*^9}, {
   3.541686639857975*^9, 3.541686645161036*^9}, 3.541836408310912*^9, 
   3.541836718110009*^9}],

Cell["\<\
This notebook and the accompanying packages are  public. Authors who intend \
to publish a changed or completed version of them should do it under their \
own names with the condition that they cite the original notebook with the \
Internet address or other source where they got it. I am not responsible for \
errors or damages originated by the use of the procedures contained in my \
notebooks or packages; everybody who applies them should test carefully \
whether they are appropriate for his purposes.\
\>", "Text",
 CellChangeTimes->{{3.470471120664312*^9, 3.470471146997974*^9}, {
   3.498225321006962*^9, 3.498225323459318*^9}, 3.541836408310948*^9, 
   3.541836718110042*^9, 3.5994668492244267`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["Initialization", "Section",
 CellChangeTimes->{{3.572085090826617*^9, 3.572085097846629*^9}, {
   3.614675039233622*^9, 3.614675041977779*^9}, 3.61468526962821*^9}],

Cell["\<\
Before starting the first session insert the address of your working \
directory, e.g. for Windows 7:\
\>", "Text",
 CellChangeTimes->{{3.599463466951742*^9, 3.599463520087782*^9}, {
   3.599463875888132*^9, 3.5994638948642178`*^9}, 3.59946686361625*^9, {
   3.624770686442224*^9, 3.6247707098734655`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetDirectory", "[", 
  "\"\<E:\\\\mymath\\\\diffgeo\\\\EuclSfTheoryv2\>\"", "]"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.571294164375502*^9, 3.5712942516264553`*^9}, 
   3.5919452705433164`*^9, 3.5920226711016054`*^9, 3.6089604686226373`*^9, 
   3.6089669513028173`*^9, 3.609849899827975*^9, 3.6114596276456995`*^9, 
   3.6225992444605618`*^9, {3.622883761555369*^9, 3.622883763286972*^9}, 
   3.623755959921055*^9, 3.6277079613261795`*^9, 3.627712256148449*^9}],

Cell[BoxData["\<\"E:\\\\mymath\\\\diffgeo\\\\EuclSfTheoryv2\"\>"], "Output",
 CellChangeTimes->CompressedData["
1:eJwV0WtIk3EYBfC5zXlNNqoN60NkYnOtrXUxiUzfIavUcEZtlclsoZkrIls6
UDRXyRgyLC+J0xYWWrOpGGpF5bJF2VpRTsy0vE2DXgkrgwamnf+Hhx+c8+08
67XnDuYEMBgMNU51Wd0wxqIpdnKefhbWb5pUBrFpynBUNk5UPG/qj4D9Y8o5
LhTVchU8mDTd0yWGNTszr0lgtF3DlcJDV/mJxIJX5U+OQVMLS2eEc1p5oYXk
nZpd7TCb88I7DCea4+5PwDsJ9UZ2IE1pXalMEUwez2rKgBVOy2kTzBPWbrZD
jzF8Zgh2LC2MBnBo6uuN8/FMOHlGPRIIndYHESEwM7faEAYvTrXaiCJ50TNi
/jLbEQ67KHfACqgaaBcQ+UXGRKKynZNLLC5TlhOp2xO9RIeg8QPRzNMsErO7
e2QRMMmok8ZAubNtP9HLOs4Rwtndv5hiKP2y0pwOPddV6aXwu6hbfwnaa3wG
YsnbkVbi321bH5tgrM7FuAdTJG8q3DBs0Jbhh66n4x5hEE2tbbsbKII1M4YC
Iqtv4FsqlLze7j4A6Q15o+lQn6+YJu4J9f7LglODQyllsIoX5WuGJXz5YTvc
t1yd8x4qC9meYdJ/Wjj1B/INjit+KF/dJF+ExfFHytnB2K1qTXAUPHvC/VEM
48Sr6FRoju3oVMGXVsEFNexzRBbpYQyn/qYN2uqiFaoQ/HX+p0wPK3mV727B
6dIFkQu2JFh7B6BvY8OP3zAldIfODx+l7U1aglssD+cjQ2kq7WTd53XwP0uh
NWI=
  "]]
}, Closed]],

Cell["For Linux:", "Text",
 CellChangeTimes->{{3.6247705161991253`*^9, 3.624770522922737*^9}}],

Cell[BoxData[
 RowBox[{"SetDirectory", "[", "\"\<~/mymath/diffgeo/EuclSfTheoryv2\>\"", 
  "]"}]], "Input",
 CellChangeTimes->{{3.571294164375502*^9, 3.5712942516264553`*^9}, 
   3.5919452705433164`*^9, 3.5920226711016054`*^9, 3.6089604686226373`*^9, 
   3.6089669513028173`*^9, 3.609849899827975*^9, 3.6114596276456995`*^9, 
   3.6225992444605618`*^9, {3.622883761555369*^9, 3.622883763286972*^9}, 
   3.623755959921055*^9, {3.6247705367911615`*^9, 3.624770550487985*^9}, {
   3.624770639532942*^9, 3.6247706546493683`*^9}}],

Cell["\<\
Regard  eventually deviating notations depending on your operating system! \
Ensure that the corresponding cell is an initialization cell, and the other \
cell doesn\[CloseCurlyQuote]t have this property (Menu Cell/ Cell Properties).\
\>", "Text",
 CellChangeTimes->{{3.599463651304287*^9, 3.599463772472217*^9}, 
   3.599466052630864*^9, {3.608967233202941*^9, 3.6089672358820944`*^9}, 
   3.6247707634127593`*^9, {3.6247709006692357`*^9, 3.6247710039882174`*^9}, {
   3.6247711621314993`*^9, 3.624771186016141*^9}, 3.6268556704224873`*^9, {
   3.6271881066198015`*^9, 3.627188135901053*^9}}],

Cell[BoxData["$Path"], "Input"],

Cell[TextData[{
 " Copy all the needed ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " packages into your working directory, or a directory of $Path. \n\
Obligatory are the packages\ntensalg.m, euvec.m, eudiffgeov2.m, Alfred Gray\
\[CloseCurlyQuote]s catalog SURFS.m, and Alfred Gray\[CloseCurlyQuote]s \
catalog\nCURVES.m. "
}], "Text",
 CellChangeTimes->{{3.5994639548406477`*^9, 3.599463992919826*^9}, {
   3.5994640376963873`*^9, 3.599464151447893*^9}, {3.5994642220319304`*^9, 
   3.5994642628482647`*^9}, {3.599464422128375*^9, 3.599464461632635*^9}, 
   3.5994644995758047`*^9, {3.599465959441534*^9, 3.5994660385980616`*^9}, 
   3.5994664734869356`*^9, {3.59948869731579*^9, 3.599488700560596*^9}, {
   3.6089672770904512`*^9, 3.608967282066736*^9}, {3.6089686143229365`*^9, 
   3.6089686872831097`*^9}, {3.6147521287995605`*^9, 3.614752179214444*^9}, {
   3.627188375438616*^9, 3.627188376561818*^9}}],

Cell["\<\
All these packages together with this notebook are contained in the file \
euclidsurfacesv2.tgz which can be downloaded from my homepage.\
\>", "Text",
 CellChangeTimes->{{3.5994661710726385`*^9, 3.5994662775667295`*^9}, {
  3.6190050343161287`*^9, 3.6190050353701887`*^9}, {3.6271884497495513`*^9, 
  3.6271884511691537`*^9}}],

Cell["Now save the notebook with your changes. ", "Text",
 CellChangeTimes->{{3.5994665178194714`*^9, 3.5994666645828657`*^9}, 
   3.599466711174531*^9}],

Cell[CellGroupData[{

Cell["Start Initialization", "Subsection",
 CellChangeTimes->{{3.586059422846844*^9, 3.586059429305256*^9}, {
  3.5994666801267548`*^9, 3.5994666865821238`*^9}}],

Cell[TextData[{
 "Start the session with the menu entry\n",
 StyleBox["Evaluation/Evaluate Initialization Cells",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{3.5994667527829103`*^9, 3.5994668982962337`*^9, 
  3.6174540239292994`*^9}],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<eudiffgeov2`\>\"", "]"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5866138547444267`*^9, 3.5866138692368517`*^9}, {
   3.586613950076194*^9, 3.5866139579230075`*^9}, {3.5866140674976*^9, 
   3.5866140721308084`*^9}, {3.599488731869851*^9, 3.5994887396542645`*^9}, {
   3.6146876112871456`*^9, 3.6146876600459347`*^9}, 3.620195830390541*^9, 
   3.62019876191989*^9, {3.622600738722787*^9, 3.6226007397367887`*^9}}],

Cell["\<\
With this command the packages euvec.m and tensalg.m are loaded automatically.\
\>", "Text",
 CellChangeTimes->{{3.6147522693505993`*^9, 3.614752329286028*^9}, 
   3.6147549420204678`*^9}],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<SURFS`\>\"", "]"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.585969335797983*^9, 3.5859693845168686`*^9}}],

Cell["\<\
We occasionally apply some definitions of  Alfred Gray`s list of curve:\
\>", "Text",
 CellChangeTimes->{{3.585969287625098*^9, 3.585969330509574*^9}, {
   3.599467063031656*^9, 3.599467071758155*^9}, 3.608968714234651*^9, {
   3.614749854875474*^9, 3.614749855920676*^9}, {3.614751974502735*^9, 
   3.614752017430191*^9}}],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<CURVES`\>\"", "]"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.585969335797983*^9, 3.5859693845168686`*^9}, {
  3.599467094937481*^9, 3.599467126408281*^9}}],

Cell["\<\
We will consider surfaces in the 3-space; thus we define the dimension dim = \
3; this value may be changed during the session.\
\>", "Text",
 CellChangeTimes->{{3.5859759313146715`*^9, 3.585975988666952*^9}, {
  3.599464337184517*^9, 3.5994643851282587`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5712056261758223`*^9, 3.571205633008634*^9}, {
   3.571290521047902*^9, 3.5712905431843405`*^9}, 3.572246357078577*^9, {
   3.5859758815878277`*^9, 3.585975898307784*^9}, 3.5994643960898857`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Context", "[", "dim", "]"}]], "Input",
 CellChangeTimes->{{3.6184698263965397`*^9, 3.6184698367237577`*^9}}],

Cell[BoxData["\<\"Global`\"\>"], "Output",
 CellChangeTimes->{3.6270530532257366`*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["The Symbols and their Usages", "Subsection",
 CellChangeTimes->{{3.6271886490659027`*^9, 3.6271886537927113`*^9}, {
  3.6271969984494023`*^9, 3.627197006672617*^9}}],

Cell["\<\
The next Subsections contain tables of the symbols defined in the package \
named in the title. Click the symbol (or use ?symbol) to read the usage: a \
short description of the content and the syntax of the function called with \
the symbol, e.g.\
\>", "Text",
 CellChangeTimes->{{3.627189064555636*^9, 3.6271891325329556`*^9}, {
  3.6271893877476053`*^9, 3.6271895372584705`*^9}, {3.6271896925739117`*^9, 
  3.627189757351226*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "vec"}]], "Input",
 CellChangeTimes->{{3.6271897619064336`*^9, 3.6271897862776766`*^9}}],

Cell[BoxData[
 StyleBox["\<\"vec[x, n:dim] is the n-tuple of the coordinates x[i] of the \
vector vec[x].\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.627189789085682*^9},
 CellTags->"Info3627193388-6571891"]
}, Closed]],

Cell[TextData[{
 "The last method may be applied to all built-in symbols of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " too, e.g. "
}], "Text",
 CellChangeTimes->{{3.627189871044626*^9, 3.6271898728230295`*^9}, {
   3.627189904807086*^9, 3.6271900373493195`*^9}, 3.6271926848071775`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "D"}]], "Input",
 CellChangeTimes->{{3.62718999802925*^9, 3.62718999802925*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox["\<\"\\!\\(\\*RowBox[{\\\"D\\\", \\\"[\\\", RowBox[{StyleBox[\\\"f\
\\\", \\\"TI\\\"], \\\",\\\", StyleBox[\\\"x\\\", \\\"TI\\\"]}], \
\\\"]\\\"}]\\) gives the partial derivative \
\\!\\(\\*RowBox[{\\\"\[PartialD]\\\", StyleBox[\\\"f\\\", \\\"TI\\\"], \
\\\"/\\\", \\\"\[PartialD]\\\", StyleBox[\\\"x\\\", \\\"TI\\\"]}]\\). \
\\n\\!\\(\\*RowBox[{\\\"D\\\", \\\"[\\\", RowBox[{StyleBox[\\\"f\\\", \
\\\"TI\\\"], \\\",\\\", RowBox[{\\\"{\\\", RowBox[{StyleBox[\\\"x\\\", \\\"TI\
\\\"], \\\",\\\", StyleBox[\\\"n\\\", \\\"TI\\\"]}], \\\"}\\\"}]}], \
\\\"]\\\"}]\\) gives the multiple derivative \\!\\(\\*RowBox[{SuperscriptBox[\
\\\"\[PartialD]\\\", StyleBox[\\\"n\\\", \\\"TI\\\"]], StyleBox[\\\"f\\\", \\\
\"TI\\\"], \\\"/\\\", \\\"\[PartialD]\\\", SuperscriptBox[StyleBox[\\\"x\\\", \
\\\"TI\\\"], StyleBox[\\\"n\\\", \\\"TI\\\"]]}]\\). \\n\\!\\(\\*RowBox[{\\\"D\
\\\", \\\"[\\\", RowBox[{StyleBox[\\\"f\\\", \\\"TI\\\"], \\\",\\\", \
StyleBox[\\\"x\\\", \\\"TI\\\"], \\\",\\\", StyleBox[\\\"y\\\", \\\"TI\\\"], \
\\\",\\\", StyleBox[\\\"\[Ellipsis]\\\", \\\"TR\\\"]}], \\\"]\\\"}]\\) \
differentiates \\!\\(\\*StyleBox[\\\"f\\\", \\\"TI\\\"]\\) successively with \
respect to \\!\\(\\*RowBox[{StyleBox[\\\"x\\\", \\\"TI\\\"], \\\",\\\", \
StyleBox[\\\"y\\\", \\\"TI\\\"], \\\",\\\", StyleBox[\\\"\[Ellipsis]\\\", \
\\\"TR\\\"]}]\\).\\n\\!\\(\\*RowBox[{\\\"D\\\", \\\"[\\\", \
RowBox[{StyleBox[\\\"f\\\", \\\"TI\\\"], \\\",\\\", RowBox[{\\\"{\\\", \
RowBox[{\\\"{\\\", RowBox[{SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], \
StyleBox[\\\"1\\\", \\\"TR\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"x\\\", \
\\\"TI\\\"], StyleBox[\\\"2\\\", \\\"TR\\\"]], \\\",\\\", StyleBox[\\\"\
\[Ellipsis]\\\", \\\"TR\\\"]}], \\\"}\\\"}], \\\"}\\\"}]}], \\\"]\\\"}]\\) \
for a scalar \\!\\(\\*StyleBox[\\\"f\\\", \\\"TI\\\"]\\) gives the vector \
derivative \\!\\(\\*RowBox[{\\\"(\\\", RowBox[{RowBox[{\\\"\[PartialD]\\\", \
RowBox[{StyleBox[\\\"f\\\", \\\"TI\\\"], \\\"/\\\", RowBox[{\\\"\[PartialD]\\\
\", SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], \\\"1\\\"]}]}]}], \
\\\",\\\", RowBox[{\\\"\[PartialD]\\\", RowBox[{StyleBox[\\\"f\\\", \
\\\"TI\\\"], \\\"/\\\", RowBox[{\\\"\[PartialD]\\\", SubscriptBox[StyleBox[\\\
\"x\\\", \\\"TI\\\"], \\\"2\\\"]}]}]}], \\\",\\\", \\\"\[Ellipsis]\\\"}], \
\\\")\\\"}]\\). \\n\\!\\(\\*RowBox[{\\\"D\\\", \\\"[\\\", RowBox[{StyleBox[\\\
\"f\\\", \\\"TI\\\"], \\\",\\\", RowBox[{\\\"{\\\", StyleBox[\\\"array\\\", \
\\\"TI\\\"], \\\"}\\\"}]}], \\\"]\\\"}]\\) gives a tensor derivative.\"\>", 
   "MSG"], "\[NonBreakingSpace]", 
  ButtonBox[
   StyleBox["\[RightSkeleton]", "SR"],
   Active->True,
   BaseStyle->"Link",
   ButtonData->"paclet:ref/D"]}]], "Print", "PrintUsage",
 CellChangeTimes->{3.6271899996516533`*^9},
 CellTags->"Info3627193599-6571891"]
}, Closed]],

Cell[TextData[{
 "Click ",
 Cell[BoxData[
  ButtonBox[
   StyleBox["\[RightSkeleton]", "SR"],
   Active->True,
   BaseStyle->"Link",
   ButtonData->"paclet:ref/D"]], "PrintUsage",
  CellChangeTimes->{3.6271899996516533`*^9}],
 " to read the corresponding help page. To see the explicit definition use"
}], "Text",
 CellChangeTimes->{{3.627190069972977*^9, 3.6271900917838154`*^9}, {
  3.62719012261347*^9, 3.6271901802635717`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"??", "d"}]], "Input",
 CellChangeTimes->{{3.6271901901403894`*^9, 3.627190199687606*^9}, {
   3.627190256138506*^9, 3.6271902584941096`*^9}, 3.627190577991473*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 StyleBox["\<\"d[f][u,v,w,...] is the differential of the function \
f[u,v,w...] of several \\nvariables with values in an  n-dimensional manifold \
or space. For the variables u, v, w,... \\nthe differentials d[u][u,v,...], \
d[v][u,v,... ],... yield the standard cobasis in the \\ncotangent space of \
the parameter manifold at the point {u, v,...}. d[f,u,v,w,...][vl] is \\nthe \
value of the differential d[f][u,v,w,...] for the tangent vector vl.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.6271905804718776`*^9},
 CellTags->"Info3627194180-6571891"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {
     RowBox[{
      RowBox[{"Attributes", "[", "d", "]"}], "=", 
      RowBox[{"{", "Protected", "}"}]}]},
    {" "},
    {GridBox[{
       {
        RowBox[{
         RowBox[{
          RowBox[{"d", "[", "eudiffgeov2`Private`f_", "]"}], "[", 
          "eudiffgeov2`Private`x___", "]"}], ":=", 
         RowBox[{"Module", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"eudiffgeov2`Private`n", "=", 
              RowBox[{"Length", "[", 
               RowBox[{"{", "eudiffgeov2`Private`x", "}"}], "]"}]}], ",", 
             "eudiffgeov2`Private`yy", ",", "eudiffgeov2`Private`y", ",", 
             "eudiffgeov2`Private`i", ",", "eudiffgeov2`Private`ff"}], "}"}], 
           ",", 
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{"eudiffgeov2`Private`n", "\[Equal]", "0"}], ",", 
              RowBox[{"Return", "[", "0", "]"}]}], "]"}], ";", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{"MemberQ", "[", 
               RowBox[{
                RowBox[{"{", "eudiffgeov2`Private`x", "}"}], ",", 
                "eudiffgeov2`Private`f"}], "]"}], ",", 
              RowBox[{"Return", "[", 
               RowBox[{
                SubscriptBox["\[PartialD]", "eudiffgeov2`Private`f"], 
                RowBox[{"{", "eudiffgeov2`Private`x", "}"}]}], "]"}]}], "]"}],
             ";", 
            RowBox[{"eudiffgeov2`Private`ff", "=", 
             RowBox[{"Evaluate", "[", 
              RowBox[{
               RowBox[{
                SubscriptBox["\[PartialD]", 
                 RowBox[{"{", 
                  RowBox[{"{", "##1", "}"}], "}"}]], 
                RowBox[{"eudiffgeov2`Private`f", "[", "##1", "]"}]}], "&"}], 
              "]"}]}], ";", 
            RowBox[{"eudiffgeov2`Private`yy", "=", 
             RowBox[{"Table", "[", 
              RowBox[{
               RowBox[{
               "eudiffgeov2`Private`y", "[", "eudiffgeov2`Private`i", "]"}], 
               ",", 
               RowBox[{"{", 
                RowBox[{
                "eudiffgeov2`Private`i", ",", "eudiffgeov2`Private`n"}], 
                "}"}]}], "]"}]}], ";", 
            RowBox[{
             RowBox[{"eudiffgeov2`Private`ff", "[", 
              RowBox[{"eudiffgeov2`Private`yy", "/.", "\[VeryThinSpace]", 
               RowBox[{"List", "\[Rule]", "Sequence"}]}], "]"}], "/.", 
             "\[VeryThinSpace]", 
             RowBox[{"Table", "[", 
              RowBox[{
               RowBox[{
                RowBox[{
                "eudiffgeov2`Private`y", "[", "eudiffgeov2`Private`i", "]"}], 
                "\[Rule]", 
                RowBox[{
                 RowBox[{"{", "eudiffgeov2`Private`x", "}"}], 
                 "\[LeftDoubleBracket]", "eudiffgeov2`Private`i", 
                 "\[RightDoubleBracket]"}]}], ",", 
               RowBox[{"{", 
                RowBox[{
                "eudiffgeov2`Private`i", ",", "eudiffgeov2`Private`n"}], 
                "}"}]}], "]"}]}]}]}], "]"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{
          RowBox[{"d", "[", 
           RowBox[{
           "eudiffgeov2`Private`h_", ",", "eudiffgeov2`Private`u___"}], "]"}],
           "[", "eudiffgeov2`Private`l_", "]"}], ":=", 
         RowBox[{"Module", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"eudiffgeov2`Private`args", "=", 
              RowBox[{"{", "eudiffgeov2`Private`u", "}"}]}], ",", 
             "eudiffgeov2`Private`n"}], "}"}], ",", 
           RowBox[{
            RowBox[{"eudiffgeov2`Private`n", "=", 
             RowBox[{"Length", "[", "eudiffgeov2`Private`args", "]"}]}], ";", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{"!", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{"Head", "[", "eudiffgeov2`Private`l", "]"}], "===", 
                  "List"}], "&&", 
                 RowBox[{
                  RowBox[{"Length", "[", "eudiffgeov2`Private`l", "]"}], 
                  "\[Equal]", "eudiffgeov2`Private`n"}]}], ")"}]}], ",", 
              RowBox[{
               RowBox[{"Print", "[", 
                
                RowBox[{"\<\"Argument \"\>", ",", "eudiffgeov2`Private`l", 
                 ",", "\<\" incorrect!\"\>"}], "]"}], ";", 
               RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", 
            RowBox[{
             RowBox[{
              RowBox[{"d", "[", "eudiffgeov2`Private`h", "]"}], "[", 
              "eudiffgeov2`Private`u", "]"}], ".", 
             "eudiffgeov2`Private`l"}]}]}], "]"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[eudiffgeov2`d],
  Editable->False]], "Print",
 CellChangeTimes->{3.6271905805030775`*^9},
 CellTags->"Info3627194180-6571891"]
}, Closed]]
}, Closed]],

Cell["\<\
I recommend to look into the package and see the definition without the \
prefixed context, here eudiffgeov2`Private`.\
\>", "Text",
 CellChangeTimes->{{3.6271907081877027`*^9, 3.6271909014054437`*^9}, 
   3.6271925396807213`*^9, 3.627192628438278*^9, {3.627192938764891*^9, 
   3.6271929510421124`*^9}, {3.6271930081724496`*^9, 3.62719300846885*^9}}],

Cell[CellGroupData[{

Cell["tensalg.m", "Subsubsection",
 CellChangeTimes->{{3.5860594705829277`*^9, 3.5860594762925377`*^9}, {
  3.5866141508017464`*^9, 3.586614156807757*^9}, {3.5994887803547363`*^9, 
  3.599488783396742*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "tensalg`*"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.571131128509915*^9, 3.57113114290874*^9}, {
   3.571326268599413*^9, 3.5713262712705655`*^9}, 3.5866141747165885`*^9, {
   3.59948905583562*^9, 3.5994890593768263`*^9}}],

Cell[BoxData[
 DynamicModuleBox[{Typeset`open$$ = True}, 
  PaneSelectorBox[{False->
   RowBox[{
    OpenerBox[Dynamic[Typeset`open$$],
     ImageSize->Small], 
    StyleBox["tensalg`", "InfoHeading"]}], True->GridBox[{
     {
      RowBox[{
       OpenerBox[Dynamic[Typeset`open$$],
        ImageSize->Small], 
       StyleBox["tensalg`", "InfoHeading"]}]},
     {GridBox[{
        {
         ButtonBox["andtest",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"andtest", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["covec",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"covec", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["linform",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"linform", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["rank",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"rank", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["trp",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"trp", "tensalg`"}},
          ButtonNote->"tensalg`"]},
        {
         ButtonBox["ArrayTotensor",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3629190372-5022241", {"ArrayTotensor", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["covector",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"covector", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["mathff",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"mathff", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["renorm",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"renorm", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["tstb",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"tstb", "tensalg`"}},
          ButtonNote->"tensalg`"]},
        {
         ButtonBox["basis",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"basis", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["covecTotensor",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3629190372-5022241", {"covecTotensor", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["matrix",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"matrix", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["smoothing",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"smoothing", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["type",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"type", "tensalg`"}},
          ButtonNote->"tensalg`"]},
        {
         ButtonBox["blf",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"blf", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["covpermute",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"covpermute", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["noprops",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"noprops", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["stb",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"stb", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["vec",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"vec", "tensalg`"}},
          ButtonNote->"tensalg`"]},
        {
         ButtonBox["cc",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"cc", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["covs",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"covs", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["null",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"null", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["strictcoords",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"strictcoords", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["vecbasis",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"vecbasis", "tensalg`"}},
          ButtonNote->"tensalg`"]},
        {
         ButtonBox["cc$",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"cc$", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["covsym",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"covsym", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["nullmatrix",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"nullmatrix", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["sum",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"sum", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["vector",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"vector", "tensalg`"}},
          ButtonNote->"tensalg`"]},
        {
         ButtonBox["contra",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"contra", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["cr",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"cr", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["nulltensor",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"nulltensor", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["tensor",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"tensor", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["vecTotensor",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"vecTotensor", "tensalg`"}},
          ButtonNote->"tensalg`"]},
        {
         ButtonBox["contraction",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"contraction", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["dotnorme",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"dotnorme", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["ortest",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"ortest", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["tensorbasis",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"tensorbasis", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["wedge",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"wedge", "tensalg`"}},
          ButtonNote->"tensalg`"]},
        {
         ButtonBox["contralt",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"contralt", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["eps",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"eps", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["outzero",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"outzero", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["tensorproduct",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3629190372-5022241", {"tensorproduct", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["wedgeform",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"wedgeform", "tensalg`"}},
          ButtonNote->"tensalg`"]},
        {
         ButtonBox["contrapermute",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3629190372-5022241", {"contrapermute", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["epst",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"epst", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["plus",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"plus", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["tensorToArray",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3629190372-5022241", {"tensorToArray", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["wedgevector",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"wedgevector", "tensalg`"}},
          ButtonNote->"tensalg`"]},
        {
         ButtonBox["contras",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"contras", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["eps$",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"eps$", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["projpt",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"projpt", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["times",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"times", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["zero",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"zero", "tensalg`"}},
          ButtonNote->"tensalg`"]},
        {
         ButtonBox["contrasym",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"contrasym", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["equi",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"equi", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["projpt3D",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"projpt3D", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["tp",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"tp", "tensalg`"}},
          ButtonNote->"tensalg`"], ""},
        {
         ButtonBox["coords",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"coords", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["extprod",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"extprod", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["ptind",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"ptind", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["trace",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"trace", "tensalg`"}},
          ButtonNote->"tensalg`"], ""},
        {
         ButtonBox["cova",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"cova", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["formbasis",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"formbasis", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["randommatrix",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"randommatrix", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["tradf",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"tradf", "tensalg`"}},
          ButtonNote->"tensalg`"], ""},
        {
         ButtonBox["covalt",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"covalt", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["lf",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"lf", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["randomv",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"randomv", "tensalg`"}},
          ButtonNote->"tensalg`"], 
         ButtonBox["transpose",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190372-5022241", {"transpose", "tensalg`"}},
          ButtonNote->"tensalg`"], ""}
       },
       DefaultBaseStyle->"InfoGrid",
       GridBoxItemSize->{"Columns" -> {{
            Scaled[0.19]}}}]}
    },
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}}]}, 
   Dynamic[Typeset`open$$],
   ImageSize->Automatic]]], "Print", "InfoCell",
 CellChangeTimes->{3.6291867730896215`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Names", "[", "\"\<tensalg`*\>\"", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.6271942446677475`*^9, 3.6271942486633544`*^9}, {
   3.6271963024853754`*^9, 3.6271963626148815`*^9}, 3.6271966915628614`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.627196370947296*^9, 3.6271963824933167`*^9}}],

Cell[BoxData["71"], "Output",
 CellChangeTimes->{3.62719638441212*^9, 3.6271966964612703`*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["euvec.m", "Subsubsection",
 CellChangeTimes->{{3.586059450100092*^9, 3.5860594543120995`*^9}, 
   3.6115594634664087`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "euvec`*"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5711311501159525`*^9, 3.5711311592419686`*^9}}],

Cell[BoxData[
 DynamicModuleBox[{Typeset`open$$ = True}, 
  PaneSelectorBox[{False->
   RowBox[{
    OpenerBox[Dynamic[Typeset`open$$],
     ImageSize->Small], 
    StyleBox["euvec`", "InfoHeading"]}], True->GridBox[{
     {
      RowBox[{
       OpenerBox[Dynamic[Typeset`open$$],
        ImageSize->Small], 
       StyleBox["euvec`", "InfoHeading"]}]},
     {GridBox[{
        {
         ButtonBox["center3pts",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"center3pts", "euvec`"}},
          ButtonNote->"euvec`"], 
         ButtonBox["invsterproj",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"invsterproj", "euvec`"}},
          ButtonNote->"euvec`"], 
         ButtonBox["posvec3pts",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"posvec3pts", "euvec`"}},
          ButtonNote->"euvec`"], 
         ButtonBox["sterproj",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"sterproj", "euvec`"}},
          ButtonNote->"euvec`"]},
        {
         ButtonBox["circle3D",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"circle3D", "euvec`"}},
          ButtonNote->"euvec`"], 
         ButtonBox["neglect",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"neglect", "euvec`"}},
          ButtonNote->"euvec`"], 
         ButtonBox["print",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"print", "euvec`"}},
          ButtonNote->"euvec`"], 
         ButtonBox["subsphere",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"subsphere", "euvec`"}},
          ButtonNote->"euvec`"]},
        {
         ButtonBox["circle3pts",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"circle3pts", "euvec`"}},
          ButtonNote->"euvec`"], 
         ButtonBox["norm",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"norm", "euvec`"}},
          ButtonNote->"euvec`"], 
         ButtonBox["radius3pts",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"radius3pts", "euvec`"}},
          ButtonNote->"euvec`"], 
         ButtonBox["subspheremf",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"subspheremf", "euvec`"}},
          ButtonNote->"euvec`"]},
        {
         ButtonBox["cross",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"cross", "euvec`"}},
          ButtonNote->"euvec`"], 
         ButtonBox["normed",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"normed", "euvec`"}},
          ButtonNote->"euvec`"], 
         ButtonBox["sph3D",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"sph3D", "euvec`"}},
          ButtonNote->"euvec`"], 
         ButtonBox["tg4frame",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"tg4frame", "euvec`"}},
          ButtonNote->"euvec`"]},
        {
         ButtonBox["esorthonorm",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"esorthonorm", "euvec`"}},
          ButtonNote->"euvec`"], 
         ButtonBox["orthocomplement",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3629190373-5022241", {"orthocomplement", "euvec`"}},
          ButtonNote->"euvec`"], 
         ButtonBox["sphere",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"sphere", "euvec`"}},
          ButtonNote->"euvec`"], 
         ButtonBox["tube",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"tube", "euvec`"}},
          ButtonNote->"euvec`"]},
        {
         ButtonBox["gencircle",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"gencircle", "euvec`"}},
          ButtonNote->"euvec`"], 
         ButtonBox["plotcircle3D",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"plotcircle3D", "euvec`"}},
          ButtonNote->"euvec`"], 
         ButtonBox["sphereplot3D",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"sphereplot3D", "euvec`"}},
          ButtonNote->"euvec`"], 
         ButtonBox["unit",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"unit", "euvec`"}},
          ButtonNote->"euvec`"]},
        {
         ButtonBox["hyperplane",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"hyperplane", "euvec`"}},
          ButtonNote->"euvec`"], 
         ButtonBox["plotcircle3pts",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"plotcircle3pts", "euvec`"}},
          ButtonNote->"euvec`"], 
         ButtonBox["sphericalreflection",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3629190373-5022241", {"sphericalreflection", "euvec`"}},
          ButtonNote->"euvec`"], 
         ButtonBox["unitvec",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"unitvec", "euvec`"}},
          ButtonNote->"euvec`"]},
        {
         ButtonBox["innerprod",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"innerprod", "euvec`"}},
          ButtonNote->"euvec`"], 
         ButtonBox["plotgencircle",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"plotgencircle", "euvec`"}},
          ButtonNote->"euvec`"], 
         ButtonBox["spherpt3D",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"spherpt3D", "euvec`"}},
          ButtonNote->"euvec`"], ""}
       },
       DefaultBaseStyle->"InfoGrid",
       GridBoxItemSize->{"Columns" -> {{
            Scaled[0.2375]}}}]}
    },
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}}]}, 
   Dynamic[Typeset`open$$],
   ImageSize->Automatic]]], "Print", "InfoCell",
 CellChangeTimes->{3.629186773230022*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"?", "Norm"}], "\[IndentingNewLine]"}]], "Input",
 CellChangeTimes->{{3.61155672679888*^9, 3.611556761217849*^9}, {
  3.6115568297847705`*^9, 3.61155684060939*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox["\<\"\\!\\(\\*RowBox[{\\\"Norm\\\", \\\"[\\\", StyleBox[\\\"expr\\\
\", \\\"TI\\\"], \\\"]\\\"}]\\) gives the norm of a number, vector or matrix. \
\\n\\!\\(\\*RowBox[{\\\"Norm\\\", \\\"[\\\", RowBox[{StyleBox[\\\"expr\\\", \
\\\"TI\\\"], \\\",\\\", StyleBox[\\\"p\\\", \\\"TI\\\"]}], \\\"]\\\"}]\\) \
gives the \\!\\(\\*StyleBox[\\\"p\\\", \\\"TI\\\"]\\)\[Hyphen]norm. \"\>", 
   "MSG"], "\[NonBreakingSpace]", 
  ButtonBox[
   StyleBox["\[RightSkeleton]", "SR"],
   Active->True,
   BaseStyle->"Link",
   ButtonData->"paclet:ref/Norm"]}]], "Print", "PrintUsage",
 CellChangeTimes->{3.6271931964691815`*^9},
 CellTags->"Info3627196796-6571891"]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Names", "[", "\"\<euvec`*\>\"", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.6271942446677475`*^9, 3.6271942486633544`*^9}, {
   3.6271963024853754`*^9, 3.6271963626148815`*^9}, {3.6271964692550697`*^9, 
   3.627196470893072*^9}, 3.6271966710000253`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.627196370947296*^9, 3.6271963824933167`*^9}}],

Cell[BoxData["31"], "Output",
 CellChangeTimes->{3.6271964809102902`*^9, 3.627196677117236*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["eudiffgeov2", "Subsubsection",
 CellChangeTimes->{{3.6226128203622694`*^9, 3.622612837241499*^9}, 
   3.627188607300629*^9}],

Cell["eudiffgeov2.m is an enlargement of eudiffgeo.m.", "Text",
 CellChangeTimes->{{3.622729922965193*^9, 3.6227299695936747`*^9}, 
   3.627192704106411*^9, {3.6271927442980824`*^9, 3.6271927490092907`*^9}, 
   3.6271927930385685`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "eudiffgeov2`*"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5860674081371207`*^9, 3.586067416311535*^9}, {
  3.6225995153774376`*^9, 3.6225995164850397`*^9}}],

Cell[BoxData[
 DynamicModuleBox[{Typeset`open$$ = True}, 
  PaneSelectorBox[{False->
   RowBox[{
    OpenerBox[Dynamic[Typeset`open$$],
     ImageSize->Small], 
    StyleBox["eudiffgeov2`", "InfoHeading"]}], True->GridBox[{
     {
      RowBox[{
       OpenerBox[Dynamic[Typeset`open$$],
        ImageSize->Small], 
       StyleBox["eudiffgeov2`", "InfoHeading"]}]},
     {GridBox[{
        {
         ButtonBox["amf",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"amf", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["curvatures",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3629190373-5022241", {"curvatures", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["fr",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"fr", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["normalcurvature",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3629190373-5022241", {"normalcurvature", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["torsion",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"torsion", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"]},
        {
         ButtonBox["arc",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"arc", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["curvaturevector",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3629190373-5022241", {"curvaturevector", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["frame",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"frame", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["normals",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"normals", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["tplane",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"tplane", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"]},
        {
         ButtonBox["arclength",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3629190373-5022241", {"arclength", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["cylindersf",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3629190373-5022241", {"cylindersf", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["frenet",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"frenet", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["pluckerconoid",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3629190373-5022241", {"pluckerconoid", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["tv",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"tv", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"]},
        {
         ButtonBox["assu",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"assu", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["d",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"d", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["fund1",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"fund1", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["prcurvatures",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3629190373-5022241", {"prcurvatures", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["tvf",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"tvf", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"]},
        {
         ButtonBox["cf",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"cf", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["dA",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"dA", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["fund2",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"fund2", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["pT",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"pT", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["vv",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"vv", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"]},
        {
         ButtonBox["cg",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"cg", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["dd",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"dd", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["gauss",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"gauss", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["R",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"R", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["\[CapitalGamma]",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3629190373-5022241", {"\[CapitalGamma]", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"]},
        {
         ButtonBox["christoffel1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3629190373-5022241", {"christoffel1", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["DD",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"DD", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["geodcurvature",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3629190373-5022241", {"geodcurvature", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["rgT",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"rgT", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["\[CapitalGamma]rgT",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3629190373-5022241", {"\[CapitalGamma]rgT", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"]},
        {
         ButtonBox["circ2D",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"circ2D", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["DD1",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"DD1", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["graph",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"graph", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["Rimm",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"Rimm", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["\[Gamma]\[Gamma]",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3629190373-5022241", {"\[Gamma]\[Gamma]", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"]},
        {
         ButtonBox["circle2D",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"circle2D", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["DDimm",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"DDimm", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["identity",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"identity", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["spiral",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"spiral", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["\[Delta]u",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3629190373-5022241", {"\[Delta]u", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"]},
        {
         ButtonBox["conesf",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"conesf", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["dV",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"dV", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["inverse",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"inverse", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["spirgr",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"spirgr", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["\[Sigma]",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"\[Sigma]", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"]},
        {
         ButtonBox["covR",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"covR", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["dVol",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"dVol", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["invrgT",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"invrgT", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["tangent",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"tangent", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["\[Omega]",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"\[Omega]", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"]},
        {
         ButtonBox["covRimm",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"covRimm", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["euclid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"euclid", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["jac",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"jac", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["tangsf",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"tangsf", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], ""},
        {
         ButtonBox["curvature",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3629190373-5022241", {"curvature", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["fd2",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"fd2", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["meancurvature",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3629190373-5022241", {"meancurvature", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], 
         ButtonBox["tangsp",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3629190373-5022241", {"tangsp", "eudiffgeov2`"}},
          ButtonNote->"eudiffgeov2`"], ""}
       },
       DefaultBaseStyle->"InfoGrid",
       GridBoxItemSize->{"Columns" -> {{
            Scaled[0.19]}}}]}
    },
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}}]}, 
   Dynamic[Typeset`open$$],
   ImageSize->Automatic]]], "Print", "InfoCell",
 CellChangeTimes->{3.6291867733704224`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"frenet", "[", "curve", "]"}], "[", "s", "]"}], "[", 
  RowBox[{"[", "1", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6282285451661806`*^9, 3.6282285860694523`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{
      SuperscriptBox["x", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], "2"], 
      "+", 
      SuperscriptBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], "2"], 
      "+", 
      SuperscriptBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
       "2"]}]]], ",", 
   FractionBox[
    RowBox[{
     RowBox[{
      SuperscriptBox["x", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], "2"], 
      "+", 
      SuperscriptBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], "2"], 
      "+", 
      SuperscriptBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
       "2"]}]]], ",", 
   FractionBox[
    RowBox[{
     RowBox[{
      SuperscriptBox["x", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], "2"], 
      "+", 
      SuperscriptBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], "2"], 
      "+", 
      SuperscriptBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
       "2"]}]]]}], "}"}]], "Output",
 CellChangeTimes->{3.628228587801055*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Names", "[", "\"\<eudiffgeov2`*\>\"", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.6271942446677475`*^9, 3.6271942486633544`*^9}, {
   3.6271963024853754`*^9, 3.6271963626148815`*^9}, {3.627196431497003*^9, 
   3.627196435867011*^9}, 3.6271966556787977`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.627196370947296*^9, 3.6271963824933167`*^9}}],

Cell[BoxData["63"], "Output",
 CellChangeTimes->{3.6271964431542234`*^9, 3.627196661903209*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Head", "[", "pluckerconoid", "]"}]], "Input",
 CellChangeTimes->{{3.627635308007902*^9, 3.6276353189163218`*^9}, 
   3.6276354207255015`*^9, 3.6276357861763473`*^9}],

Cell[BoxData["Symbol"], "Output",
 CellChangeTimes->{3.627635320647925*^9, 3.6276354220047035`*^9, 
  3.6276357874711494`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Context", "[", "pluckerconoid", "]"}]], "Input",
 CellChangeTimes->{{3.6276353364059525`*^9, 3.627635359374193*^9}, {
  3.6276357266688423`*^9, 3.627635733550454*^9}}],

Cell[BoxData["\<\"eudiffgeov2`\"\>"], "Output",
 CellChangeTimes->{3.627635361121396*^9, 3.6276356837899666`*^9, 
  3.627635735500458*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "pluckerconoid"}]], "Input",
 CellChangeTimes->{{3.62763516519485*^9, 3.6276351671448536`*^9}, {
   3.6276352277161603`*^9, 3.627635234270172*^9}, 3.627635277878249*^9, {
   3.6276357478088794`*^9, 3.627635761742704*^9}}],

Cell[BoxData[
 StyleBox["\<\"pluckerconoid[u,v] is a parameter representation of Pl\
\[UDoubleDot]cker's conoid.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.627635763396307*^9},
 CellTags->"Info3627639363-8560585"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SURFS.m", "Subsubsection",
 CellChangeTimes->{{3.5860594887413597`*^9, 3.586059509489396*^9}, {
  3.5994638339537334`*^9, 3.5994638358238406`*^9}}],

Cell["\<\
Evaluating the next cell yields Alfred Gray\[CloseCurlyQuote]s catalog of \
surfaces:\
\>", "Text",
 CellChangeTimes->{{3.627193786044938*^9, 3.6271938278766127`*^9}, {
  3.6271938611262712`*^9, 3.627193869004285*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "SURFS`*"}]], "Input",
 CellChangeTimes->{{3.6201990028108015`*^9, 3.620199018676029*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 DynamicModuleBox[{Typeset`open$$ = True}, 
  PaneSelectorBox[{False->
   RowBox[{
    OpenerBox[Dynamic[Typeset`open$$],
     ImageSize->Small], 
    StyleBox["SURFS`", "InfoHeading"]}], True->GridBox[{
     {
      RowBox[{
       OpenerBox[Dynamic[Typeset`open$$],
        ImageSize->Small], 
       StyleBox["SURFS`", "InfoHeading"]}]},
     {GridBox[{
        {
         ButtonBox["agnesirev",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"agnesirev", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["epicycloidmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"epicycloidmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["parabolamin",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"parabolamin", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["apple",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"apple", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["equihom1implicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"equihom1implicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["parabolamincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"parabolamincurve", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["astell",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"astell", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["equihom2implicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"equihom2implicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["paraboliccoor",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"paraboliccoor", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["astroidmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"astroidmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["equihom3implicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"equihom3implicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["paraboliccyclide",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"paraboliccyclide", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["bohdom",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"bohdom", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["equihom4implicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"equihom4implicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["paraboloid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"paraboloid", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["bour",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"bour", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["equihom5implicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"equihom5implicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["paraboloidpolar",
          BaseStyle->"InformationLink",
          ButtonData:>{
           "Info3627458084-5102015", {"paraboloidpolar", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["bourmincurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"bourmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["equihom6implicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"equihom6implicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["pear",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"pear", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["bourpolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"bourpolar", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["exptwist",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"exptwist", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["pearbis",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"pearbis", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["boy",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"boy", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["exptwistasym",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"exptwistasym", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["perturbedms",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"perturbedms", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cardioidmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"cardioidmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["flattorus",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"flattorus", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["perturbedmspolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"perturbedmspolar", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["catalan",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"catalan", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["funnel",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"funnel", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["pillow",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"pillow", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["catalandef",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"catalandef", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["funnelasym",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"funnelasym", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["plane",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"plane", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["catalanmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"catalanmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["gencube",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"gencube", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["planepolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"planepolar", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["catenoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"catenoid", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["genhypparab",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"genhypparab", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["planetocat",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"planetocat", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["catenoidasym",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"catenoidasym", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["genoctahedron",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"genoctahedron", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["plucker",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"plucker", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["catenoidiso",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"catenoidiso", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["geopolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"geopolar", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["pluckerpolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"pluckerpolar", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["catenoidmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"catenoidmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["goursatimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"goursatimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["poincare",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"poincare", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cayleysexticmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"cayleysexticmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["handkerchief",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"handkerchief", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["poindisk",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"poindisk", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["circleinvolutemin",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"circleinvolutemin", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["heart",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"heart", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["pseudocrosscap",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"pseudocrosscap", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["circleinvolutemincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"circleinvolutemincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["helicoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"helicoid", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["pseudosphere",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"pseudosphere", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["circularcone",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"circularcone", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["helicoidprin",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"helicoidprin", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["pseudospherebis",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"pseudospherebis", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["circularcylinder",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"circularcylinder", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["heltocat",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"heltocat", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["pseudospheretcheb",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"pseudospheretcheb", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["circularmembrane",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"circularmembrane", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["henneberg",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"henneberg", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["rconoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"rconoid", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["circularmembranemovie",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"circularmembranemovie", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["hennebergmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"hennebergmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["richmond",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"richmond", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["circularmembraneplot",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"circularmembraneplot", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["hy2sheet",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"hy2sheet", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["richmondmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"richmondmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cissoidmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"cissoidmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["hy2sheetimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"hy2sheetimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["richmondpolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"richmondpolar", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["clothoidmin",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"clothoidmin", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["hyperbolamincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"hyperbolamincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["roman",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"roman", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["clothoidmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"clothoidmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["hyperbolicparaboloid",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"hyperbolicparaboloid", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["rosemin",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"rosemin", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cnccosurfrev",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"cnccosurfrev", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["hyperbolicparaboloidimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {
            "hyperbolicparaboloidimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["rosemincurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"rosemincurve", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cnccotcheb",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"cnccotcheb", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["hyperboloid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"hyperboloid", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["scherk",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"scherk", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cnchyprofilemincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"cnchyprofilemincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["hyperboloidimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"hyperboloidimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["scherk5",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"scherk5", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cnchysurfrev",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"cnchysurfrev", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["hypocycloidinvolutemin",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"hypocycloidinvolutemin", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["scherk5implicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"scherk5implicit", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cnchytcheb",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"cnchytcheb", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["hypocycloidinvolutemincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {
            "hypocycloidinvolutemincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["scherkimplicit",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"scherkimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cossurface",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"cossurface", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["hypocycloidmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"hypocycloidmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["scherktowermincurveprime",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"scherktowermincurveprime", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cossurfaceimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"cossurfaceimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["invertedcircularcone",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"invertedcircularcone", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["schwarzschildplane",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"schwarzschildplane", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["costa",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"costa", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["invertedcircularcylinder",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"invertedcircularcylinder", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["scmincurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"scmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["costamincurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"costamincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["invertedtorus",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"invertedtorus", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["shoe",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"shoe", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cpcprofilemincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"cpcprofilemincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["jorgemeeksmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"jorgemeeksmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["shoeasym",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"shoeasym", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cpcsurfrev",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"cpcsurfrev", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["jorgemeeksmincurveprime",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"jorgemeeksmincurveprime", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["sievert",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"sievert", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["crosscap",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"crosscap", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["jorgemeekspolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"jorgemeekspolar", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["sinovaloid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"sinovaloid", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["crosscapimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"crosscapimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["kazoolaimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"kazoolaimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["sinsurface",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"sinsurface", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["deltoidinvolutemin",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"deltoidinvolutemin", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["kidney",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"kidney", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["sinsurfaceimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"sinsurfaceimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["deltoidinvolutemincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"deltoidinvolutemincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["klein4",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"klein4", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["snail",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"snail", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["deltoidmin",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"deltoidmin", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["kleinbottle",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"kleinbottle", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["sphereimplicit",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"sphereimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["deltoidmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"deltoidmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["kleinbottlebis",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"kleinbottlebis", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["sphero",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"sphero", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["dini",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"dini", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["kleinbottlewri",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"kleinbottlewri", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["squaremembrane",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"squaremembrane", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["dinihelicoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"dinihelicoid", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["kuen",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"kuen", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["squaremembranemovie",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"squaremembranemovie", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["dinitcheb",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"dinitcheb", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["kuentcheb",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"kuentcheb", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["squaremembraneplot",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"squaremembraneplot", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["earth",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"earth", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["kummerimplicit",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"kummerimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["stereographicellipsoid",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"stereographicellipsoid", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["eighteight",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"eighteight", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["lemniscatemin",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"lemniscatemin", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["stereographicsphere",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"stereographicsphere", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["eightsurface",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"eightsurface", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["lemniscatemincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"lemniscatemincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["stereographicspherepolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"stereographicspherepolar", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ellhypcyclide",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"ellhypcyclide", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["liouville",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"liouville", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["swallowtail",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"swallowtail", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ellipsemin",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"ellipsemin", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["logmincurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"logmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["tetrahedral",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"tetrahedral", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ellipsemincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"ellipsemincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["logminpolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"logminpolar", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["thomsen",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"thomsen", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ellipsoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"ellipsoid", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["logspiralmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"logspiralmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["torus",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"torus", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ellipsoidimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"ellipsoidimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["menn",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"menn", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["torusasym",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"torusasym", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ellipticalhyperboloidminus",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"ellipticalhyperboloidminus", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["mercatorellipsoid",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"mercatorellipsoid", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["torusbis",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"torusbis", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ellipticalhyperboloidplus",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"ellipticalhyperboloidplus", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["mercatorsphere",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"mercatorsphere", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["torusimplicit",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"torusimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ellipticcoor",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"ellipticcoor", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["moebcirc",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"moebcirc", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["tractrixmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"tractrixmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ellipticparaboloid",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"ellipticparaboloid", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["moebiusstrip",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"moebiusstrip", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["twiflat",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"twiflat", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ellipticparaboloidimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"ellipticparaboloidimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["monkeysaddle",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"monkeysaddle", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["twisphere",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"twisphere", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["enneper",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"enneper", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["monkeysaddlepolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"monkeysaddlepolar", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["twocuspsimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"twocuspsimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ennepercatenoidmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"ennepercatenoidmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["nephroidmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"nephroidmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["veronese",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"veronese", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ennepercatenoidpolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"ennepercatenoidpolar", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["nielsenspiralmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"nielsenspiralmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["wallis",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"wallis", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ennepermincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"ennepermincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["nodoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"nodoid", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["whitneyumbrella",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3627458084-5102015", {"whitneyumbrella", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["enneperpolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"enneperpolar", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["nodoidnds",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"nodoidnds", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["wrinkles",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3627458084-5102015", {"wrinkles", "SURFS`"}},
          ButtonNote->"SURFS`"]}
       },
       DefaultBaseStyle->"InfoGrid",
       GridBoxItemSize->{"Columns" -> {{
            Scaled[0.31666666666666665`]}}}]}
    },
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}}]}, 
   Dynamic[Typeset`open$$],
   ImageSize->Automatic]]], "Print", "InfoCell",
 CellChangeTimes->{3.627454484442663*^9}],

Cell[BoxData[
 StyleBox["\<\"{u,v}->ellipticalhyperboloidplus[a,b,c][u,v] is a \
parametrization of a hyperboloid of one sheet as a ruled surface.  To plot \
try ellipticalhyperboloidplus[1,2,3][-Pi,Pi,-Pi/2,Pi/2].\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellTags->"Info3627458084-5102015"]
}, Closed]]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Names", "[", "\"\<SURFS`*\>\"", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.6271942446677475`*^9, 3.6271942486633544`*^9}, {
   3.6271963024853754`*^9, 3.6271963626148815`*^9}, {3.627196538152791*^9, 
   3.627196539993594*^9}, 3.627196640250371*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.627196370947296*^9, 3.6271963824933167`*^9}}],

Cell[BoxData["198"], "Output",
 CellChangeTimes->{3.627196547764408*^9, 3.6271966457259808`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"sphero", "[", "1", "]"}], "[", 
  RowBox[{"0", ",", 
   RowBox[{"3", 
    RowBox[{"Pi", "/", "2"}]}], ",", 
   RowBox[{
    RowBox[{"-", "Pi"}], "/", "2"}], ",", 
   RowBox[{"Pi", "/", "2"}]}], "]"}]], "Input"],

Cell["\<\
I replaced the name  \[OpenCurlyDoubleQuote]sphere\[CloseCurlyDoubleQuote] as \
defined by Alfred Gray with \[OpenCurlyDoubleQuote]sphero\
\[CloseCurlyDoubleQuote], since sphere has been defined more general in \
euvec.m:\
\>", "Text",
 CellChangeTimes->{{3.585970879065495*^9, 3.5859709333067904`*^9}, {
   3.5859710012426877`*^9, 3.5859710386748285`*^9}, {3.585971099242293*^9, 
   3.585971118058369*^9}, 3.5859754962357864`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "sphere"}]], "Input",
 CellChangeTimes->{{3.585970828271806*^9, 3.58597083093941*^9}}],

Cell[BoxData[
 StyleBox["\<\"sphere[m,r][u,v] is the parameter representation of \\nthe \
sphere with center m and radius r in the euclidean 3-space with \
geographical\\n \\ncoordinates u, 0<=u<=2 Pi, v, -Pi/2 <= v <= Pi/2.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.62719358267658*^9},
 CellTags->"Info3627197182-6571891"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Context", "[", "sphere", "]"}]], "Input",
 CellChangeTimes->{{3.585971045570223*^9, 3.585971062668201*^9}}],

Cell[BoxData["\<\"euvec`\"\>"], "Output",
 CellChangeTimes->{3.6271936141594353`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"sphere", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", "1"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{"sphero", "[", "1", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.58597550835948*^9, 3.5859755988446555`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6271936267018576`*^9}]
}, Closed]],

Cell["Don' t confuse this with the built-in Graphics3D primitive", "Text",
 CellChangeTimes->{{3.5859715472679186`*^9, 3.585971593930588*^9}, {
  3.585971635217949*^9, 3.5859716626585183`*^9}}],

Cell[BoxData[
 RowBox[{"?", "Sphere"}]], "Input",
 CellChangeTimes->{{3.585969549128358*^9, 3.5859695523263636`*^9}}],

Cell[BoxData[
 RowBox[{"Sphere", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "2"}], "}"}]}], "}"}], ",", "1"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.614752453779148*^9, 3.6147525783602743`*^9}, 
   3.6147526639211683`*^9}],

Cell[BoxData[
 RowBox[{"Sphere", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "2"}], "}"}]}], "}"}], ",", 
   RowBox[{"1", "/", "2"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6147526670623474`*^9, 3.6147526670953493`*^9}, {
  3.614752714673071*^9, 3.6147527157761335`*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"Graphics3D", "[", "%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6147525998515034`*^9, 3.6147526487042975`*^9}, 
   3.6147526791610394`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["CURVES.m", "Subsubsection",
 CellChangeTimes->{{3.5860594887413597`*^9, 3.586059509489396*^9}, {
  3.5994638339537334`*^9, 3.5994638358238406`*^9}, {3.599469927037876*^9, 
  3.5994699297330303`*^9}}],

Cell["\<\
Evaluating the next cell yields Alfred Gray\[CloseCurlyQuote]s catalog of \
curves:\
\>", "Text",
 CellChangeTimes->{{3.627193786044938*^9, 3.6271938278766127`*^9}, {
  3.6271938611262712`*^9, 3.627193869004285*^9}, {3.627193916871169*^9, 
  3.627193918540372*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "CURVES`*"}]], "Input",
 CellChangeTimes->{{3.61475153190962*^9, 3.614751542782839*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 DynamicModuleBox[{Typeset`open$$ = True}, 
  PaneSelectorBox[{False->
   RowBox[{
    OpenerBox[Dynamic[Typeset`open$$],
     ImageSize->Small], 
    StyleBox["CURVES`", "InfoHeading"]}], True->GridBox[{
     {
      RowBox[{
       OpenerBox[Dynamic[Typeset`open$$],
        ImageSize->Small], 
       StyleBox["CURVES`", "InfoHeading"]}]},
     {GridBox[{
        {
         ButtonBox["agnesi",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"agnesi", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["deltoidimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"deltoidimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["nephroidunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"nephroidunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["agnesiimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"agnesiimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["deltoidinvolute",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"deltoidinvolute", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["newtonphillipsimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"newtonphillipsimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["airyspiral",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"airyspiral", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["deltoidunitspeed",
          BaseStyle->"InformationLink",
          ButtonData:>{
           "Info3626671618-6153013", {"deltoidunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["ngon",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"ngon", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["alysoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"alysoid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["devilimplicit",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"devilimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["nielsenspiral",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"nielsenspiral", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["alysoidkappa2",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"alysoidkappa2", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["diamond",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"diamond", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["oneparametersubgroup",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"oneparametersubgroup", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["alysoidnds",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"alysoidnds", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["duererimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"duererimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["pacman",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"pacman", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["alysoidprime",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"alysoidprime", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["eight",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"eight", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["parabola",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"parabola", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["archimedesspiral",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"archimedesspiral", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["eightknot",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"eightknot", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["parabolaimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"parabolaimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["archimedesspiralpolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"archimedesspiralpolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["elasticainflect",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"elasticainflect", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["parabolicspiral",
          BaseStyle->"InformationLink",
          ButtonData:>{
           "Info3626671618-6153013", {"parabolicspiral", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["ast3d",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"ast3d", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["elasticanoninflect",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"elasticanoninflect", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["perseusimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"perseusimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["astroid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"astroid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["ellipse",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"ellipse", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["piriform",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"piriform", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["astroidimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"astroidimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["ellipseimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"ellipseimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["piriformimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"piriformimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["baseballseam",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"baseballseam", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["ellipseinc",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"ellipseinc", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["predatorpreyimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"predatorpreyimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["bellcurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"bellcurve", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["epicycloid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"epicycloid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["pseudocatenary",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"pseudocatenary", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["bernstein",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"bernstein", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["epicycloidunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"epicycloidunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["pseudocatenarykappa2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"pseudocatenarykappa2", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["besselcurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"besselcurve", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["epispiral",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"epispiral", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["pseudocatenarynds",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"pseudocatenarynds", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["besseln",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"besseln", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["epispiralpolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"epispiralpolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["pseudocatenaryprime",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"pseudocatenaryprime", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["beziercurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"beziercurve", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["epitrochoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"epitrochoid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["pseudospheregeodesic",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"pseudospheregeodesic", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["bicorn",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"bicorn", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["epitrochoid3d",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"epitrochoid3d", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["pseudosphericalloxodrome",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"pseudosphericalloxodrome", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["bicylinder",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"bicylinder", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["fermatspiral",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"fermatspiral", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["rectangle",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"rectangle", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["biquadratic",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"biquadratic", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["fermatspiralpolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"fermatspiralpolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["regularpolygon",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"regularpolygon", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["bow",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"bow", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["folium",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"folium", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["reuleauxpolygon",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"reuleauxpolygon", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["bowpolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"bowpolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["foliumimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"foliumimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["rose",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"rose", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["bowtie",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"bowtie", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["foliumpolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"foliumpolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["rosepolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"rosepolar", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["bulletnose",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"bulletnose", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["genhelix",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"genhelix", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["sc",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"sc", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["bulletnoseimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"bulletnoseimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["genparabola",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"genparabola", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["scarab",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"scarab", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cardioid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"cardioid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hankelspiral",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"hankelspiral", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["scimplicit",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"scimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cardioidimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"cardioidimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["helix",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"helix", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["scunitspeed",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"scunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cardioidpolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"cardioidpolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["helixunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"helixunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["seiffertspiral",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"seiffertspiral", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cardioidunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"cardioidunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hippopede",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"hippopede", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["semicubic",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"semicubic", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cartesianimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"cartesianimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hippopedeimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"hippopedeimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["semicubicimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"semicubicimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cartesianoval",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"cartesianoval", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["horopter",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"horopter", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["serpentine",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"serpentine", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cassini",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"cassini", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hyperbola",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"hyperbola", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["serpentineimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"serpentineimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cassiniimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"cassiniimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hyperbolabis",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"hyperbolabis", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["sinhspiral",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"sinhspiral", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cassinipolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"cassinipolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hyperbolaimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"hyperbolaimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["sinn",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"sinn", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["catenary",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"catenary", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hyperbolicspiral",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"hyperbolicspiral", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["sinoval",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"sinoval", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["catenaryunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"catenaryunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hyperbolicspiralpolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"hyperbolicspiralpolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["sphericalcardioid",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"sphericalcardioid", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cayleysextic",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"cayleysextic", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hypocycloid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"hypocycloid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["sphericalellipse",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"sphericalellipse", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["circle",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"circle", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hypocycloidinvolute",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"hypocycloidinvolute", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["sphericalhelix",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"sphericalhelix", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["circleimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"circleimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hypocycloidunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"hypocycloidunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["sphericalloxodrome",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"sphericalloxodrome", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["circleinvolute",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"circleinvolute", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hypotrochoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"hypotrochoid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["sphericalloxodromeunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {
            "sphericalloxodromeunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["circleinvoluteunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"circleinvoluteunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["kampyle",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"kampyle", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["sphericalnephroid",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"sphericalnephroid", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["circleunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"circleunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["kampylepolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"kampylepolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["sphericalspiral",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"sphericalspiral", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cissoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"cissoid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["keplerimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"keplerimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["spring",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"spring", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cissoidimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"cissoidimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["keplerorbit",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"keplerorbit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["strophoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"strophoid", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["clelia",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"clelia", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["keplerorbitpolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"keplerorbitpolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["strophoidimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"strophoidimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["clothoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"clothoid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["kochsnowflake",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"kochsnowflake", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["talbot",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"talbot", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["clothoidnds",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"clothoidnds", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lagrangecurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"lagrangecurve", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["tanhspiral",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"tanhspiral", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["clothoidprime",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"clothoidprime", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lehr",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"lehr", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["teardrop",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"teardrop", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cnccoprofile",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"cnccoprofile", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lehrkappa2",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"lehrkappa2", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["teeth",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"teeth", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cnchyprofile",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"cnchyprofile", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lehrnds",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"lehrnds", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["tennisballseam",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"tennisballseam", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cochleoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"cochleoid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lemniscate",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"lemniscate", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["tooth",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"tooth", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cochleoidpolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"cochleoidpolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lemniscatebis",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"lemniscatebis", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["torusknot",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"torusknot", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["conchoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"conchoid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lemniscateimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"lemniscateimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["tractrix",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"tractrix", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["conchoidimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"conchoidimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lemniscatepolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"lemniscatepolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["tractrixminus",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"tractrixminus", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["conicalhelix",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"conicalhelix", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lemniscateunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"lemniscateunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["tractrixplus",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"tractrixplus", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["coshspiral",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"coshspiral", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["limacon",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"limacon", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["tractrixunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"tractrixunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cosn",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"cosn", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["limaconimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"limaconimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["triangle",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"triangle", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cothspiral",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"cothspiral", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["limaconpolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"limaconpolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["trident",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"trident", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cpcprofile",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"cpcprofile", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["line",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"line", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["tridentimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"tridentimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["crosscurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"crosscurve", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["line3d",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"line3d", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["trisectrix",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"trisectrix", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["crosscurveimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"crosscurveimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lissajous",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"lissajous", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["tschirnhausen",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"tschirnhausen", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cubicalellipse",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"cubicalellipse", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lissajous3d",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"lissajous3d", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["twicubic",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"twicubic", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cubicparabola",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"cubicparabola", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lituus",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"lituus", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["twistedn",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"twistedn", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cubicparabolaimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"cubicparabolaimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lituuspolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"lituuspolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["veryflat",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"veryflat", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cycloid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"cycloid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["logistic",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"logistic", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["viviani",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"viviani", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cycloidunitspeed",
          BaseStyle->"InformationLink",
          ButtonData:>{
           "Info3626671618-6153013", {"cycloidunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["logspiral",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"logspiral", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["watt",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"watt", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["delaunay",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"delaunay", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["logspiralpolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"logspiralpolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["wattimplicit",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"wattimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["delaunaykappa2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"delaunaykappa2", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["logspiralunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"logspiralunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["wigglyellipse",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"wigglyellipse", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["delaunaynds",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"delaunaynds", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["nephroid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"nephroid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["zcprofile",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"zcprofile", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["deltoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"deltoid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["nephroidimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3626671618-6153013", {"nephroidimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], ""},
        {
         ButtonBox["deltoidbis",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"deltoidbis", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["nephroidpolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3626671618-6153013", {"nephroidpolar", "CURVES`"}},
          ButtonNote->"CURVES`"], ""}
       },
       DefaultBaseStyle->"InfoGrid",
       GridBoxItemSize->{"Columns" -> {{
            Scaled[0.31666666666666665`]}}}]}
    },
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}}]}, 
   Dynamic[Typeset`open$$],
   ImageSize->Automatic]]], "Print", "InfoCell",
 CellChangeTimes->{3.626668018692275*^9}],

Cell[BoxData[
 StyleBox["\<\"t->sphericalloxodrome[a,b,c][t] is a loxodrome on a sphere of \
radius a. It is the image under the stereographic injection \
{u,v}->stereographicsphere[a][u,v] of the logarithmic spiral \
t->logspiral[b,c][t]. A sphericalloxodrome meets each meridian of the sphere \
at the same angle.\"\>", "MSG"]], "Print", "PrintUsage",
 CellTags->"Info3626671618-6153013"]
}, Closed]]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Names", "[", "\"\<CURVES`*\>\"", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.6271942446677475`*^9, 3.6271942486633544`*^9}, {
   3.6271963024853754`*^9, 3.6271963626148815`*^9}, {3.6271965685300446`*^9, 
   3.6271965796860647`*^9}, 3.627196623057141*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.627196370947296*^9, 3.6271963824933167`*^9}}],

Cell[BoxData["214"], "Output",
 CellChangeTimes->{3.6271965889836807`*^9, 3.6271966285483503`*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Global", "Subsubsection",
 CellChangeTimes->{{3.5860594887413597`*^9, 3.586059509489396*^9}, {
  3.5994638339537334`*^9, 3.5994638358238406`*^9}, {3.599469927037876*^9, 
  3.5994699297330303`*^9}, {3.6201992683798122`*^9, 3.620199272233019*^9}}],

Cell["\<\
Evaluate the next cell to see the symbols actually used in Global Context\
\>", "Text",
 CellChangeTimes->{{3.6201992773654275`*^9, 3.620199340423375*^9}}],

Cell[BoxData[
 RowBox[{"?", "Global`*"}]], "Input",
 CellChangeTimes->{{3.6201992089418144`*^9, 3.6201992164298277`*^9}}]
}, Closed]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["1. The Local Structure Forms of a Surface", "Section",
 CellChangeTimes->{{3.5720851282978826`*^9, 3.5720851431179085`*^9}, {
   3.5722464122558737`*^9, 3.5722464218342905`*^9}, 3.572613861614375*^9, {
   3.5860591355255394`*^9, 3.58605916430759*^9}, 3.5860595807347217`*^9, {
   3.586924985356963*^9, 3.586924992786388*^9}, {3.5872723940085163`*^9, 
   3.5872723973337064`*^9}, {3.5994702008395367`*^9, 3.59947020282165*^9}, 
   3.599487297135331*^9, {3.6245220427340345`*^9, 3.624522046088041*^9}, {
   3.6271999044935236`*^9, 3.627199906381127*^9}, 3.627287277418764*^9}],

Cell[CellGroupData[{

Cell["1.1. Parameter Representation and Differential", "Subsection",
 CellChangeTimes->{{3.572614110263212*^9, 3.5726141131960173`*^9}, {
  3.58605919842485*^9, 3.5860592061936636`*^9}, {3.587272403656068*^9, 
  3.587272408134324*^9}, {3.627104472934962*^9, 3.6271044757117667`*^9}, {
  3.6271046284048347`*^9, 3.6271046373592505`*^9}}],

Cell[TextData[{
 "Definition of a ",
 StyleBox["general surface",
  FontColor->RGBColor[0, 0, 1]],
 " in a  space ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "dim"], TraditionalForm]]],
 " with arbitrary   coordinates"
}], "Text",
 CellChangeTimes->{{3.572855115891306*^9, 3.572855271251192*^9}, 
   3.588919489132287*^9, 3.5994873207225723`*^9, {3.6175282598773465`*^9, 
   3.6175282606261477`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sf", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"f", "[", "i", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "dim"}], "}"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5728552771865315`*^9, 3.5728553233421717`*^9}, {
  3.6175282729813695`*^9, 3.6175282741981716`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "3"}], ";", 
  RowBox[{"sf", "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.57285533188666*^9, 3.5728553521568193`*^9}, {
  3.6175282908278008`*^9, 3.617528294915008*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "1", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "2", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "3", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6272831187023115`*^9}]
}, Closed]],

Cell[TextData[{
 "It is generally assumed that sf is defined on an open domain  U of real ",
 StyleBox["parameters",
  FontColor->RGBColor[0, 0, 1]],
 " u,v:\n {u,v} \[Epsilon] U, and that the components of sf are smooth \
functions. In this notebook it is assumed mostly that sf[u,v] is a point in \
the Euclidean 3-space with  orthonormal Cartesian coordinates f[i][u,v].  \
Essential for the application of analysis  to geometry is the concept of the ",
 StyleBox["differential",
  FontColor->RGBColor[0, 0, 1]],
 ". If f: U \[RightArrow] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]]],
 " is a map of an open set U of the m-dimensional space ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["R", "m"], TraditionalForm]]],
 " with real coordinates {x,y,...} and values in the n-dimensional space ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]]],
 ", its differential is defined in  eudiffgeov2.m:"
}], "Text",
 CellChangeTimes->{{3.5859775598738203`*^9, 3.5859775914046235`*^9}, {
   3.585977684324938*^9, 3.5859776866680717`*^9}, 3.585979278963146*^9, {
   3.585979493235402*^9, 3.585979610219093*^9}, {3.5859796687544413`*^9, 
   3.585979806419315*^9}, {3.5859799516676226`*^9, 3.5859799900988207`*^9}, {
   3.5859805402112856`*^9, 3.585980561378496*^9}, {3.58598075813875*^9, 
   3.5859807637550716`*^9}, 3.588919461957039*^9, {3.5889196555689793`*^9, 
   3.5889199692387304`*^9}, {3.588920019283618*^9, 3.588920055085681*^9}, {
   3.5889201160973883`*^9, 3.5889201311202145`*^9}, {3.614755195935991*^9, 
   3.61475531480779*^9}, {3.61752837015394*^9, 3.6175283750055485`*^9}, {
   3.6271031468078327`*^9, 3.6271031481650352`*^9}, {3.627103252934819*^9, 
   3.627103262029635*^9}, 3.6272831932018433`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "d"}]], "Input",
 CellChangeTimes->{{3.61754602990119*^9, 3.6175460314767923`*^9}}],

Cell[BoxData[
 StyleBox["\<\"d[f][u,v,w,...] is the differential of the function \
f[u,v,w...] of several \\nvariables with values in an  n-dimensional manifold \
or space. For the variables u, v, w,... \\nthe differentials d[u][u,v,...], \
d[v][u,v,... ],... yield the standard cobasis in the \\ncotangent space of \
the parameter manifold at the point {u, v,...}. d[f,u,v,w,...][vl] is \\nthe \
value of the differential d[f][u,v,w,...] for the tangent vector vl.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.627283215821883*^9},
 CellTags->"Info3627286815-6629669"]
}, Closed]],

Cell[TextData[{
 StyleBox["Remark.",
  FontColor->RGBColor[0, 0, 1]],
 " In the next cell one sees the definition of d. The cell is set to be \
unevaluatable to avoid conflicts with the protected definition in \
eudiffgeov2.m. Also further on, to give easy information,  I copied the \
definitions of some important operations from the package into  unevaluatable \
cells."
}], "Text",
 CellChangeTimes->{{3.6272832860008073`*^9, 3.627283326830079*^9}, {
   3.627283379018572*^9, 3.627283483830557*^9}, {3.6272835614734087`*^9, 
   3.6272835689790215`*^9}, {3.62728384450554*^9, 3.6272838469235444`*^9}, {
   3.6272838871736155`*^9, 3.6272839539039326`*^9}, {3.627284077260751*^9, 
   3.6272841039835978`*^9}, {3.627284141384864*^9, 3.627284176208125*^9}, {
   3.62728420719378*^9, 3.6272842142917924`*^9}, {3.627284254949464*^9, 
   3.627284259773873*^9}, {3.6272843147055693`*^9, 3.627284348187228*^9}, {
   3.6272843983337173`*^9, 3.627284412689742*^9}, {3.627284453876815*^9, 
   3.6272844592588243`*^9}, {3.62728505210124*^9, 3.6272851089185405`*^9}, 
   3.627285144067402*^9, 3.6272851952902927`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "f_", "]"}], "[", "x___", "]"}], ":=", " ", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"n", "=", 
        RowBox[{"Length", "[", 
         RowBox[{"{", "x", "}"}], "]"}]}], ",", "yy", ",", "y", ",", "i", ",",
        "ff"}], "}"}], ",", "\n", 
     RowBox[{
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"n", "==", "0"}], ",", " ", 
        RowBox[{"Return", " ", "[", "0", "]"}]}], "]"}], ";", "\n", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"MemberQ", "[", 
         RowBox[{
          RowBox[{"{", "x", "}"}], ",", "f"}], "]"}], ",", " ", 
        RowBox[{"Return", "[", 
         RowBox[{"D", "[", 
          RowBox[{
           RowBox[{"{", "x", "}"}], ",", "f"}], "]"}], "]"}]}], "]"}], ";", 
      "\n", 
      RowBox[{"ff", "=", 
       RowBox[{"Evaluate", "[", 
        RowBox[{
         RowBox[{"D", "[", 
          RowBox[{
           RowBox[{"f", "[", "##", "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"{", "##", "}"}], "}"}]}], "]"}], "&"}], "]"}]}], ";", 
      "\n", 
      RowBox[{"yy", " ", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"y", "[", "i", "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "n"}], "}"}]}], "]"}]}], ";", "\n", 
      RowBox[{
       RowBox[{"ff", "[", 
        RowBox[{"yy", "/.", 
         RowBox[{"List", "->", " ", "Sequence"}]}], "]"}], "/.", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "i", "]"}], "->", 
          RowBox[{
           RowBox[{"{", "x", "}"}], "[", 
           RowBox[{"[", "i", "]"}], "]"}]}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "n"}], "}"}]}], "]"}]}]}]}], "]"}]}], 
  ";"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{3.627285258915205*^9}],

Cell[TextData[{
 StyleBox["Applications",
  FontColor->RGBColor[0, 0, 1]],
 "."
}], "Text",
 CellChangeTimes->{{3.627285228194751*^9, 3.6272852510819907`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "Sin", "]"}], "[", "x", "]"}]], "Input",
 CellChangeTimes->{{3.5954889643645163`*^9, 3.5954889724609303`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"Cos", "[", "x", "]"}], "}"}]], "Output",
 CellChangeTimes->{3.6271033960806704`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"Sin", "[", "x", "]"}], ",", "x"}], "]"}]], "Input",
 CellChangeTimes->{{3.6175279359895773`*^9, 3.617527939203183*^9}}],

Cell[BoxData[
 RowBox[{"Cos", "[", "x", "]"}]], "Output",
 CellChangeTimes->{3.6271034052534866`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "x", "]"}], "[", 
  RowBox[{"x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.609226322838493*^9, 3.6092263283568087`*^9}, {
  3.6092263655099335`*^9, 3.609226372605339*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.6271034086386924`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "x", "]"}], "[", "x", "]"}]], "Input",
 CellChangeTimes->{{3.609226322838493*^9, 3.6092263283568087`*^9}, {
  3.6092263655099335`*^9, 3.609226372605339*^9}, {3.61752795990442*^9, 
  3.617527968266034*^9}}],

Cell[BoxData[
 RowBox[{"{", "1", "}"}]], "Output",
 CellChangeTimes->{3.6271034125854993`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "u", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6092244631351233`*^9, 3.6092244832382736`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.627103416844307*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "y", "]"}], "[", 
  RowBox[{"x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.6092244631351233`*^9, 3.609224525677701*^9}, 
   3.609226415399787*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "1"}], "}"}]], "Output",
 CellChangeTimes->{3.6271034198707123`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "f", "]"}], "[", "]"}]], "Input",
 CellChangeTimes->{3.589005713465705*^9, 3.6175285317078238`*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6271034233963184`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "Pi", "]"}], "[", "x", "]"}]], "Input",
 CellChangeTimes->{{3.588941818019162*^9, 3.5889418578304324`*^9}, {
   3.5889419056445165`*^9, 3.5889419058317165`*^9}, 3.5889419679354258`*^9, {
   3.5889422053522425`*^9, 3.5889422222314725`*^9}}],

Cell[BoxData[
 RowBox[{"{", "0", "}"}]], "Output",
 CellChangeTimes->{3.627103426812724*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "Pi", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.588941818019162*^9, 3.5889418578304324`*^9}, {
   3.5889419056445165`*^9, 3.5889419058317165`*^9}, 3.5889419679354258`*^9, {
   3.5889422053522425`*^9, 3.5889422222314725`*^9}, {3.588942287455187*^9, 
   3.588942303554415*^9}, {3.5890059000743785`*^9, 3.5890059004574003`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.6271034307439313`*^9}]
}, Closed]],

Cell[TextData[{
 "For a function the differential is almost the same as the ",
 StyleBox["gradient",
  FontColor->RGBColor[0, 0, 1]],
 " Grad"
}], "Text",
 CellChangeTimes->{{3.58892064103831*^9, 3.5889206417871113`*^9}, {
  3.5889209049439735`*^9, 3.588920957594066*^9}, {3.608968397882557*^9, 
  3.608968427011223*^9}}],

Cell[BoxData[
 RowBox[{"?", "Grad"}]], "Input",
 CellChangeTimes->{{3.614773387141837*^9, 3.614773389891994*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Grad", "[", 
  RowBox[{
   RowBox[{"h", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "v"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5889209644736786`*^9, 3.5889209987625384`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SuperscriptBox["h", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    SuperscriptBox["h", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6271034439103546`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "h", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5889204141671114`*^9, 3.5889204226535263`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SuperscriptBox["h", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    SuperscriptBox["h", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6271034484655623`*^9}]
}, Closed]],

Cell["\<\
The main differences are 1. d[h][x,...]  defines an object explicitly as a \
function of x,..., and 2. Grad can be related to many concrete coordinate \
systems often used in physical and technical applications:\
\>", "Text",
 CellChangeTimes->{{3.588921241093364*^9, 3.5889214769189787`*^9}, 
   3.588998676512214*^9, 3.5994913695242205`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Grad", "[", 
  RowBox[{
   RowBox[{"h", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "v"}], "}"}], ",", "\"\<Polar\>\""}], "]"}]], "Input",
 CellChangeTimes->{{3.5889209644736786`*^9, 3.5889209987625384`*^9}, {
  3.588921087745095*^9, 3.588921103157922*^9}, {3.5889211371191816`*^9, 
  3.5889211390847845`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SuperscriptBox["h", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   FractionBox[
    RowBox[{
     SuperscriptBox["h", 
      TagBox[
       RowBox[{"(", 
        RowBox[{"0", ",", "1"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "u"]}], "}"}]], "Output",
 CellChangeTimes->{3.6272853404509487`*^9}]
}, Closed]],

Cell["For the parameter representation  of a surface we have", "Text",
 CellChangeTimes->{{3.588921573436348*^9, 3.5889216404540653`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";", 
  RowBox[{
   RowBox[{"d", "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.588921645711275*^9, 3.5889216694545164`*^9}, {
  3.617599579612307*^9, 3.617599585197117*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627103481943221*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "\[Equal]", " ", 
  RowBox[{"Grad", "[", 
   RowBox[{
    RowBox[{"sf", "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"u", ",", "v"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6175286024851484`*^9, 3.6175286391140127`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.627103486248829*^9}]
}, Closed]],

Cell["Now we choose a curve on the surface:", "Text",
 CellChangeTimes->{{3.627103887169533*^9, 3.6271039046571636`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"h1", "[", "s_", "]"}], " ", ":=", " ", 
   RowBox[{"s", "^", "2"}]}], ";", " ", 
  RowBox[{
   RowBox[{"h2", "[", "s_", "]"}], " ", ":=", 
   RowBox[{"Cos", "[", "s", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.627103541379326*^9, 3.627103599442628*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "sf", "]"}], "[", 
  RowBox[{
   RowBox[{"h1", "[", "s", "]"}], ",", 
   RowBox[{"h2", "[", "s", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.627103609052244*^9, 3.627103631079483*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       SuperscriptBox["s", "2"], ",", 
       RowBox[{"Cos", "[", "s", "]"}]}], "]"}], ",", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       SuperscriptBox["s", "2"], ",", 
       RowBox[{"Cos", "[", "s", "]"}]}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       SuperscriptBox["s", "2"], ",", 
       RowBox[{"Cos", "[", "s", "]"}]}], "]"}], ",", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       SuperscriptBox["s", "2"], ",", 
       RowBox[{"Cos", "[", "s", "]"}]}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       SuperscriptBox["s", "2"], ",", 
       RowBox[{"Cos", "[", "s", "]"}]}], "]"}], ",", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       SuperscriptBox["s", "2"], ",", 
       RowBox[{"Cos", "[", "s", "]"}]}], "]"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6271036321402855`*^9}]
}, Closed]],

Cell["\<\
This is the differential along the curve. Grad yields a message and a \
senseless result:\
\>", "Text",
 CellChangeTimes->{{3.627104191916068*^9, 3.627104216220911*^9}, {
  3.6271042699162054`*^9, 3.62710428975944*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Grad", "[", 
  RowBox[{
   RowBox[{"sf", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"h1", "[", "s", "]"}], ",", 
     RowBox[{"h2", "[", "s", "]"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6271036592375326`*^9, 3.6271036603139343`*^9}, {
  3.6271036909367886`*^9, 3.6271037109984236`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Grad", "::", "nocoord"}], "MessageName"], ":", 
  " ", "\<\"\[NoBreak]\\!\\({s\\^2, \\(\\(Cos[s]\\)\\)}\\)\[NoBreak] is not a \
non-empty list of valid variables. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/Grad\\\", ButtonNote -> \
\\\"Grad::nocoord\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.627103712870427*^9}],

Cell[BoxData[
 TemplateBox[{RowBox[{"{", 
     RowBox[{
       RowBox[{
         RowBox[{"f", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{
         RowBox[{"f", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{
         RowBox[{"f", "[", "3", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "}"}],RowBox[{"{", 
     RowBox[{
       SuperscriptBox["s", "2"], ",", 
       RowBox[{"Cos", "[", "s", "]"}]}], "}"}]},
  "Grad"]], "Output",
 CellChangeTimes->{3.627103712886027*^9}]
}, Closed]],

Cell["One obtains the correct result applying", "Text",
 CellChangeTimes->{{3.627104316529087*^9, 3.627104354983155*^9}, 
   3.6272870119346957`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Grad", "[", 
   RowBox[{
    RowBox[{"sf", "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"u", ",", "v"}], "}"}]}], "]"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"u", " ", "->", 
     RowBox[{"h1", "[", "s", "]"}]}], ",", 
    RowBox[{"v", "->", 
     RowBox[{"h2", "[", "s", "]"}]}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.6271036592375326`*^9, 3.6271036603139343`*^9}, {
  3.6271036909367886`*^9, 3.627103769779327*^9}, {3.6271038080929937`*^9, 
  3.6271038306194334`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       SuperscriptBox["s", "2"], ",", 
       RowBox[{"Cos", "[", "s", "]"}]}], "]"}], ",", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       SuperscriptBox["s", "2"], ",", 
       RowBox[{"Cos", "[", "s", "]"}]}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       SuperscriptBox["s", "2"], ",", 
       RowBox[{"Cos", "[", "s", "]"}]}], "]"}], ",", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       SuperscriptBox["s", "2"], ",", 
       RowBox[{"Cos", "[", "s", "]"}]}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       SuperscriptBox["s", "2"], ",", 
       RowBox[{"Cos", "[", "s", "]"}]}], "]"}], ",", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       SuperscriptBox["s", "2"], ",", 
       RowBox[{"Cos", "[", "s", "]"}]}], "]"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6271038418670535`*^9}]
}, Closed]],

Cell["\<\
But if the differential d is used as an inner part of composed definitions \
this method leads to error messages.\
\>", "Text",
 CellChangeTimes->{{3.627104384092806*^9, 3.627104450377322*^9}}],

Cell["The differential of the map f is a family of linear maps", "Text",
 CellChangeTimes->{{3.5889228641358147`*^9, 3.588922920763914*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", "f", "]"}], "[", 
   RowBox[{"x", ",", ".."}], "   ", "]"}], ":", 
  RowBox[{
   SuperscriptBox["R", "m"], " ", "\[LongRightArrow]", "  ", 
   SuperscriptBox["R", "n"]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5889229806212196`*^9, 3.58892300926287*^9}, {
  3.588923056577753*^9, 3.5889230913970137`*^9}, {3.588941572646332*^9, 
  3.5889415857815547`*^9}}],

Cell["\<\
depending on the variables (or \[OpenCurlyDoubleQuote]parameters\
\[CloseCurlyDoubleQuote]) x,..., defined by\
\>", "Text",
 CellChangeTimes->{{3.588923126356675*^9, 3.588923177306365*^9}, 
   3.588923467466874*^9, {3.588998814856127*^9, 3.588998820839469*^9}}],

Cell[BoxData[
 RowBox[{"y", " ", "\[Element]", " ", 
  RowBox[{
   SuperscriptBox["R", "m"], "\[RightTeeArrow]", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", "f", "]"}], "[", 
     RowBox[{"x", ",", ".."}], "]"}], ".", "y"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5889416120676007`*^9, 3.5889417133429785`*^9}, 
   3.588998736145625*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"vec", "[", 
   RowBox[{"s", ",", "2"}], "]"}], " ", "\[RightTeeArrow]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "f", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ".", 
   RowBox[{"vec", "[", 
    RowBox[{"s", ",", "2"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5889232076484184`*^9, 3.5889232636993165`*^9}, {
  3.588923319843815*^9, 3.5889234056907663`*^9}, {3.5889234950945234`*^9, 
  3.5889235095245485`*^9}, {3.5889235406622033`*^9, 3.588923649097994*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"s", "[", "1", "]"}], ",", 
    RowBox[{"s", "[", "2", "]"}]}], "}"}], "\[RightTeeArrow]", 
  RowBox[{
   RowBox[{
    RowBox[{"s", "[", "2", "]"}], " ", 
    RowBox[{
     SuperscriptBox["f", 
      TagBox[
       RowBox[{"(", 
        RowBox[{"0", ",", "1"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "+", 
   RowBox[{
    RowBox[{"s", "[", "1", "]"}], " ", 
    RowBox[{
     SuperscriptBox["f", 
      TagBox[
       RowBox[{"(", 
        RowBox[{"1", ",", "0"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}]}]}]], "Output",
 CellChangeTimes->{3.6272854190850873`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], ".", 
  RowBox[{"vec", "[", 
   RowBox[{"s", ",", "2"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.588940494715638*^9, 3.588940510970867*^9}, 
   3.588945080655693*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"s", "[", "2", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "+", 
    RowBox[{
     RowBox[{"s", "[", "1", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"s", "[", "2", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "+", 
    RowBox[{
     RowBox[{"s", "[", "1", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"s", "[", "2", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "+", 
    RowBox[{
     RowBox[{"s", "[", "1", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6271030244256177`*^9, 3.627285431270709*^9}]
}, Closed]],

Cell["We test the basis forms d[u], d[v]:", "Text",
 CellChangeTimes->{{3.609226511524285*^9, 3.6092265438751354`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "f", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", "f", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "==", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"f", "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", "u"}], "]"}], "*", 
    RowBox[{
     RowBox[{"d", "[", "u", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], " ", "+", 
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"f", "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", "v"}], "]"}], "*", 
    RowBox[{
     RowBox[{"d", "[", "v", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.6092245895043516`*^9, 3.6092247048079467`*^9}, {
  3.6092247607681475`*^9, 3.609224765470416*^9}, {3.6092248053196955`*^9, 
  3.609224817127371*^9}, {3.6092248482401505`*^9, 3.609224859735808*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6271030392924438`*^9}]
}, Closed]],

Cell["For vector function this rule is not valid:", "Text",
 CellChangeTimes->{{3.609226650900257*^9, 3.6092266800509243`*^9}, 
   3.6147549747243385`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "==", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"sf", "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{"{", "u", "}"}]}], "]"}], ".", 
    RowBox[{
     RowBox[{"d", "[", "u", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], " ", "+", 
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"sf", "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{"{", "v", "}"}]}], "]"}], ".", 
    RowBox[{
     RowBox[{"d", "[", "v", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.6092245895043516`*^9, 3.6092247048079467`*^9}, {
  3.6092247607681475`*^9, 3.609224765470416*^9}, {3.6092248053196955`*^9, 
  3.609224817127371*^9}, {3.6092248482401505`*^9, 3.609224859735808*^9}, {
  3.6092266116680126`*^9, 3.6092266305930953`*^9}, {3.6147743020511665`*^9, 
  3.6147743524020467`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Dot", "::", "dotsh"}], "MessageName"], ":", 
  " ", "\<\"Tensors \
\[NoBreak]\\!\\({\\(\\(\\(\\(\\*SuperscriptBox[\\(f[1]\\), TagBox[\\((1, \
0)\\), Derivative], Rule[MultilineFunction, None]]\\)\\)[\\(\\(u, v\\)\\)]\\)\
\\), \\(\\(\\(\\(\\*SuperscriptBox[\\(f[2]\\), TagBox[\\((1, 0)\\), \
Derivative], Rule[MultilineFunction, None]]\\)\\)[\\(\\(u, v\\)\\)]\\)\\), \
\\(\\(\\(\\(\\*SuperscriptBox[\\(f[3]\\), TagBox[\\((1, 0)\\), Derivative], \
Rule[MultilineFunction, None]]\\)\\)[\\(\\(u, v\\)\\)]\\)\\)}\\)\[NoBreak] \
and \[NoBreak]\\!\\({1, 0}\\)\[NoBreak] have incompatible shapes. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/Dot/dotsh\\\", \
ButtonNote -> \\\"Dot::dotsh\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.627103050618064*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Dot", "::", "dotsh"}], "MessageName"], ":", 
  " ", "\<\"Tensors \
\[NoBreak]\\!\\({\\(\\(\\(\\(\\*SuperscriptBox[\\(f[1]\\), TagBox[\\((0, \
1)\\), Derivative], Rule[MultilineFunction, None]]\\)\\)[\\(\\(u, v\\)\\)]\\)\
\\), \\(\\(\\(\\(\\*SuperscriptBox[\\(f[2]\\), TagBox[\\((0, 1)\\), \
Derivative], Rule[MultilineFunction, None]]\\)\\)[\\(\\(u, v\\)\\)]\\)\\), \
\\(\\(\\(\\(\\*SuperscriptBox[\\(f[3]\\), TagBox[\\((0, 1)\\), Derivative], \
Rule[MultilineFunction, None]]\\)\\)[\\(\\(u, v\\)\\)]\\)\\)}\\)\[NoBreak] \
and \[NoBreak]\\!\\({0, 1}\\)\[NoBreak] have incompatible shapes. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/Dot/dotsh\\\", \
ButtonNote -> \\\"Dot::dotsh\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.627103050618064*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "}"}]}], "}"}], "\[Equal]", 
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "}"}], ".", 
    RowBox[{"{", 
     RowBox[{"0", ",", "1"}], "}"}]}], "+", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "}"}], ".", 
    RowBox[{"{", 
     RowBox[{"1", ",", "0"}], "}"}]}]}]}]], "Output",
 CellChangeTimes->{3.627103050618064*^9}]
}, Closed]],

Cell["Instead one has to use the TensorProduct;", "Text",
 CellChangeTimes->{{3.6092267184271193`*^9, 3.609226736707165*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "==", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"sf", "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", "u"}], "]"}], "\[TensorProduct]", 
    RowBox[{
     RowBox[{"d", "[", "u", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "+", 
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"sf", "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", "v"}], "]"}], "\[TensorProduct]", 
    RowBox[{
     RowBox[{"d", "[", "v", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.6092255700064335`*^9, 3.6092255957509055`*^9}, {
   3.6092256366472445`*^9, 3.6092256596085577`*^9}, {3.6092257240252423`*^9, 
   3.6092257378640337`*^9}, {3.6092259323801594`*^9, 3.6092259767666984`*^9}, 
   3.6175997262681646`*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6271030653912897`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.614773800266466*^9, 3.6147738434019337`*^9}, {
  3.6147739088996797`*^9, 3.6147741230419283`*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]},
     {
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]},
     {
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.6271030725361023`*^9}]
}, Closed]],

Cell[TextData[{
 " A point sf[u0,v0] of the parameter representation is said to be",
 StyleBox[" regular",
  FontColor->RGBColor[0, 0, 1]],
 ", if"
}], "Text",
 CellChangeTimes->{{3.5859775598738203`*^9, 3.5859775914046235`*^9}, {
   3.585977684324938*^9, 3.5859776866680717`*^9}, 3.585979278963146*^9, {
   3.585979493235402*^9, 3.585979610219093*^9}, {3.5859796687544413`*^9, 
   3.585979806419315*^9}, {3.5859799516676226`*^9, 3.5859799900988207`*^9}, {
   3.5859805402112856`*^9, 3.585980561378496*^9}, {3.58598075813875*^9, 
   3.5859807637550716`*^9}, 3.588919461957039*^9, 3.588921688034149*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MatrixRank", "[", 
  RowBox[{
   RowBox[{"d", "[", "sf", "]"}], "[", 
   RowBox[{"u0", ",", "v0"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5859805724181275`*^9, 3.5859806608291845`*^9}, {
  3.5859807142702413`*^9, 3.5859807199815674`*^9}, {3.585980775894766*^9, 
  3.585980799256102*^9}, {3.5889217292026215`*^9, 3.588921746565452*^9}, {
  3.627286051265602*^9, 3.627286087914067*^9}}],

Cell[BoxData["2"], "Output",
 CellChangeTimes->{3.6272860804864535`*^9}]
}, Closed]],

Cell[TextData[{
 "is maximal, here equal 2, what ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " supposes generally. A smooth map of manifolds\n f: ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["X", "m"], TraditionalForm]]],
 " \[RightArrow] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["Y", "n"], TraditionalForm]]],
 ", m \[LessEqual] n the dimensions of the manifolds, is said to be an ",
 StyleBox["immersion",
  FontColor->RGBColor[0, 0, 1]],
 ", if it is regular at all points x \[Element] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["X", "m"], TraditionalForm]]],
 ", that means that the differential d[f][x] has maximal rank m at each point \
x.  For the immersions considered in this notebook ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["X", "m"], TraditionalForm]]],
 " is supposed  to be an open connected subset of the m-dimensional real \
parameter space ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalR]", "m"], TraditionalForm]]],
 ".  Such considerations are said to be ",
 StyleBox["local",
  FontColor->RGBColor[0, 0, 1]],
 "; for global results  consult the textbooks."
}], "Text",
 CellChangeTimes->{{3.5859808863310823`*^9, 3.585981001738683*^9}, 
   3.5859812042752676`*^9, {3.58892214405415*^9, 3.5889221553329697`*^9}, {
   3.599489421188262*^9, 3.599489423153865*^9}, {3.6147748706876907`*^9, 
   3.614774889639775*^9}, 3.6175287613558273`*^9, {3.6176041768079815`*^9, 
   3.6176043174578285`*^9}, {3.6176043528230906`*^9, 
   3.6176044069395857`*^9}, {3.6176044518832645`*^9, 
   3.6176046332803836`*^9}, {3.617604677288061*^9, 3.6176047310457554`*^9}, {
   3.6176047705762243`*^9, 3.617604883754423*^9}, {3.617604917356882*^9, 
   3.6176049802249928`*^9}, {3.617605013109851*^9, 3.617605033795487*^9}, {
   3.61760506986275*^9, 3.617605161341311*^9}}],

Cell[TextData[{
 "For concrete surfaces may exist ",
 StyleBox["singular",
  FontColor->RGBColor[0, 0, 1]],
 " (i.e. non regular) points, e. g. the unit sphere"
}], "Text",
 CellChangeTimes->{{3.5859808863310823`*^9, 3.585981001738683*^9}, 
   3.5859812042752676`*^9, {3.58892214405415*^9, 3.5889221553329697`*^9}, {
   3.599489421188262*^9, 3.599489423153865*^9}, {3.6147748706876907`*^9, 
   3.614774889639775*^9}, 3.6175287613558273`*^9, {3.6176041768079815`*^9, 
   3.6176043174578285`*^9}, {3.6176043528230906`*^9, 
   3.6176044069395857`*^9}, {3.6176044518832645`*^9, 
   3.6176046332803836`*^9}, {3.617604677288061*^9, 3.6176047310457554`*^9}, {
   3.6176047705762243`*^9, 3.617604883754423*^9}, {3.617604917356882*^9, 
   3.617604989257409*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"sphero", "[", "1", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5859810215718174`*^9, 3.5859810412209415`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", "u", "]"}], " ", 
    RowBox[{"Cos", "[", "v", "]"}]}], ",", 
   RowBox[{
    RowBox[{"Cos", "[", "v", "]"}], " ", 
    RowBox[{"Sin", "[", "u", "]"}]}], ",", 
   RowBox[{"Sin", "[", "v", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6271046902589436`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       RowBox[{"Cos", "[", "v", "]"}]}], " ", 
      RowBox[{"Sin", "[", "u", "]"}]}], ",", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"Cos", "[", "u", "]"}]}], " ", 
      RowBox[{"Sin", "[", "v", "]"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Cos", "[", "u", "]"}], " ", 
      RowBox[{"Cos", "[", "v", "]"}]}], ",", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"Sin", "[", "u", "]"}]}], " ", 
      RowBox[{"Sin", "[", "v", "]"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{"Cos", "[", "v", "]"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.62710469411215*^9, 3.6272861349667497`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
  RowBox[{"u", ",", 
   RowBox[{"Pi", "/", "2"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6271047226758003`*^9, 3.6271047333774195`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{"-", 
      RowBox[{"Cos", "[", "u", "]"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{"-", 
      RowBox[{"Sin", "[", "u", "]"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627104734719022*^9, 3.6272861424099627`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MatrixRank", "[", 
  RowBox[{
   RowBox[{"d", "[", 
    RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
   RowBox[{"u", ",", 
    RowBox[{"Pi", "/", "2"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.585981072950756*^9, 3.5859810781580544`*^9}, {
  3.5859811219425583`*^9, 3.5859811247027164`*^9}, {3.5889218032715516`*^9, 
  3.588921824487589*^9}, {3.5889218603208513`*^9, 3.588921938492589*^9}, {
  3.6271047782430983`*^9, 3.6271047934687247`*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.627104794545127*^9, 3.6272861462339697`*^9}]
}, Closed]],

Cell["\<\
The \[OpenCurlyDoubleQuote]pole\[CloseCurlyDoubleQuote] is a singularity of \
the parameter representation, not of the sphere.\
\>", "Text",
 CellChangeTimes->{{3.617528858325598*^9, 3.6175288969824657`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sph", "=", 
   RowBox[{"ParametricPlot3D", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"sphero", "[", "1", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"u", ",", "0", ",", 
       RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"v", ",", "0", ",", 
       RowBox[{"Pi", "/", "2"}]}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5859812477817564`*^9, 3.585981348142496*^9}, {
  3.5859815859831*^9, 3.585981590489358*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pn", "=", 
   RowBox[{"Graphics3D", "[", 
    RowBox[{"{", 
     RowBox[{"Black", ",", 
      RowBox[{"PointSize", "[", ".03", "]"}], ",", 
      RowBox[{"Point", "[", 
       RowBox[{
        RowBox[{"sphero", "[", "1", "]"}], "[", 
        RowBox[{"u", ",", 
         RowBox[{"Pi", "/", "2"}]}], "]"}], "]"}]}], "}"}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.585981598798833*^9, 3.585981608132367*^9}, {
  3.5859816511338263`*^9, 3.5859816640615664`*^9}, {3.585981694846327*^9, 
  3.585981764203294*^9}, {3.5859818411426945`*^9, 3.58598188005392*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"{", 
   RowBox[{"pn", ",", "sph"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.5859813522057285`*^9, 3.585981501934293*^9}, {
  3.585981553318232*^9, 3.5859815606696525`*^9}, {3.585981910101639*^9, 
  3.5859819154309435`*^9}}],

Cell[TextData[{
 "We define (in eudiffgeov2.m) the",
 StyleBox[" tangent basis vectors of an immersion",
  FontColor->RGBColor[0, 0, 1]],
 ":"
}], "Text",
 CellChangeTimes->{{3.58598299934694*^9, 3.5859830217632217`*^9}, {
  3.585983083914777*^9, 3.5859831585790477`*^9}, {3.617602206399721*^9, 
  3.617602240641781*^9}, {3.622722556560911*^9, 3.6227225759829454`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "tv"}]], "Input",
 CellChangeTimes->{{3.6147750597295036`*^9, 3.6147752023226595`*^9}, {
  3.617601685889207*^9, 3.617601763280943*^9}, {3.6176018170854373`*^9, 
  3.6176018889547634`*^9}, {3.6176019215588207`*^9, 3.617602075390691*^9}, {
  3.62272259598218*^9, 3.622722607791401*^9}}],

Cell[BoxData[
 StyleBox["\<\"tv[i][f][u,v,...]is the tangent vector of the i-th  parameter \
line of \\nthe immersion f[u,v,...] at the point {u, v,...}. If this point is \
regular these \\nvectors for i = 1,...,Length[{u,v,...}] form a basis of the \
tangent space \\nimage[d[f][u,v,...]].\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.6272861617403965`*^9},
 CellTags->"Info3627289761-6629669"]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"tv", "[", "i_", "]"}], "[", "f_", "]"}], "[", "u__", "]"}], ":=",
   " ", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"IntegerQ", "[", "i", "]"}], "&&", 
     RowBox[{"i", ">", "0"}], "&&", 
     RowBox[{"i", "\[LessEqual]", 
      RowBox[{"Length", "[", 
       RowBox[{"{", "u", "}"}], "]"}]}]}], ",", " ", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Head", "[", 
         RowBox[{"f", "[", "u", "]"}], "]"}], "\[Equal]", " ", "List"}], "&&",
        " ", 
       RowBox[{
        RowBox[{"Length", "[", 
         RowBox[{"f", "[", "u", "]"}], "]"}], ">", "1"}]}], ",", 
      RowBox[{
       RowBox[{"Transpose", "[", 
        RowBox[{
         RowBox[{"d", "[", "f", "]"}], "[", "u", "]"}], "]"}], "[", 
       RowBox[{"[", "i", "]"}], "]"}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"d", "[", "f", "]"}], "[", "u", "]"}], "[", 
       RowBox[{"[", "i", "]"}], "]"}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"d", "[", "f", "]"}], "[", "u", "]"}], "[", 
       RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
    RowBox[{"Print", "[", 
     RowBox[{"\"\<Wrong i = \>\"", ",", " ", "i"}], "]"}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.617600215672224*^9, 3.6176003174156027`*^9}, {
  3.6176006545477953`*^9, 3.617600705856285*^9}, {3.6176007854944253`*^9, 
  3.617600881278593*^9}, {3.617600951853117*^9, 3.6176009588419294`*^9}, {
  3.6176010154232287`*^9, 3.617601022786442*^9}, {3.617601071505327*^9, 
  3.617601080865344*^9}, {3.617601230578807*^9, 3.6176012370996184`*^9}, {
  3.617601271372878*^9, 3.6176014155483313`*^9}, {3.6176014483707895`*^9, 
  3.6176014781356416`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Context", "[", "tv", "]"}]], "Input",
 CellChangeTimes->{{3.62272265838229*^9, 3.622722668647108*^9}}],

Cell[BoxData["\<\"eudiffgeov2`\"\>"], "Output",
 CellChangeTimes->{3.6272863433175173`*^9}]
}, Closed]],

Cell["For a function we have", "Text",
 CellChangeTimes->{{3.617602506435048*^9, 3.617602516840266*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tv", "[", "2", "]"}], "[", "f", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5859821957269754`*^9, 3.5859823002309527`*^9}, {
   3.585999470135942*^9, 3.5859995456400747`*^9}, {3.587561122225507*^9, 
   3.587561164661934*^9}, {3.6141625644876366`*^9, 3.6141626502097874`*^9}, {
   3.614163514950506*^9, 3.6141636632287664`*^9}, {3.614163718749264*^9, 
   3.614163724926875*^9}, 3.617600023838687*^9, 3.6176000856927958`*^9, {
   3.6176015070424924`*^9, 3.6176015352629423`*^9}, 3.6176015768058147`*^9, {
   3.6176016248462996`*^9, 3.61760163099271*^9}, {3.6176023871260386`*^9, 
   3.6176023906516447`*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["f", 
   TagBox[
    RowBox[{"(", 
     RowBox[{"0", ",", "1"}], ")"}],
    Derivative],
   MultilineFunction->None], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Output",
 CellChangeTimes->CompressedData["
1:eJwV0FtIk3EYBvB5mJuHDb0xzENudDBdzszEVLIv1qIsp0FOpaIUpRA3aHPK
hqnDmkwU6qI0qVkIonMSY2aEONEQcqklS50OUpyb8cmwkzZZ0vO/ePjdPLy8
PLxy+bXKAAaDUYKcVdU9dwbRVNm7FqUbTvF8haxgmhqZp74Rx3OME1yYK721
FQnt3ERxFJQ5Zs0CuC2ofZwKHyXWRQqhVpOaR5wd7R4tgyxdYbUWFjhKVR0w
/Ygmewiym+bti3C168rgKpRbhrTBTJpKzSkPTIaiBcWLIqge7r/XCjMMxhMD
MMIft/EVKn9znAEhNNXbrs8KhKJ1mYMJS55Mc0Oh/oaxPhxmL40biMln2seI
mapYUwQ0Z60FcOBD69QBYrSsM4/o6Y2vIqpVVc3Epa7dEeIg1/yF2N1W4yey
hVMnubBzRyc8Sv7InLxEfJpSIz4GA/cUIUmQtjCDBFBoP66XwLRQmeQBfPvK
pmyC2x0/64muSW8fUWC8+L4VSu84Gf1wlxOjs8Gw6eEiHxw79XcmiUVTbp6V
mQwPlj67T6w8vebJhzLqgu0qPBynWZHAzYridWKDeu/fTWixfb/cCPm6HNdr
uMGRXh8gd3b7KuegeSt2ZhHe/sy++wdGyydbfDC3bfi8H3rU3uZgNna2m9h8
uL/gmhfAT/taOh+K4mfeFENrVJpCCjf5GXVKWNZgemmAIQqNuDgUPQ8rXQn9
E/lzPVBdG57yAe4kuEc+woREk/cXXGksqPaRPlVxbh/KljN/xITRlNgxtHwI
/gdQPD07
  "]]
}, Closed]],

Cell["For a surface we get", "Text",
 CellChangeTimes->{{3.6176025506299257`*^9, 3.6176025626575465`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tv", "[", "1", "]"}], "[", "sf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.617600351439263*^9, 3.617600385182122*^9}, 
   3.617601103344983*^9, {3.6176011586470804`*^9, 3.617601175105109*^9}, 
   3.617602309032301*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "1", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "2", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "3", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6271048240447783`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tv", "[", "2", "]"}], "[", "sf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5859821957269754`*^9, 3.5859823002309527`*^9}, {
   3.585999470135942*^9, 3.5859995456400747`*^9}, {3.587561122225507*^9, 
   3.587561164661934*^9}, {3.6141625644876366`*^9, 3.6141626502097874`*^9}, {
   3.614163514950506*^9, 3.6141636632287664`*^9}, {3.614163718749264*^9, 
   3.614163724926875*^9}, 3.617600023838687*^9, 3.6176000856927958`*^9, {
   3.6176015070424924`*^9, 3.6176015352629423`*^9}, 3.6176015768058147`*^9, 
   3.6176023741780157`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "1", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "2", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "3", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->CompressedData["
1:eJwVz11Ik3EUBvCp25xuvswuCk0sK0yWaYaCrWDOamky5iZON+gDTSkMDXv9
yNWco3IsEuwmSfwskHQ6mTRDRqVGkUstWWk1KK1p+JYplrm1rOd/cfhdHM7D
c2IKylRFASwWKx8TulZ1xx3ESGNTr9Hz8FaHNzuYzUiNqvSPxCcHe0YoyJs+
+U0IXdR2WTgUayZt8dBhqWhMgKr2KmEiudMlSIgTjmaHFrZeV5YYYaRGW9kA
N+7pxH0kzzDlmoaWCLnlE8lLthrZHEbqGyoIFEGhmm5RQllK9zkT/Bdt2dsN
Bf4ozxuYRlPuAC4jvRB2IzUQCgvL3nHgzk1OKgTmunuq+ZCbP9xGFB24+YgY
uLa1VwD1D2cDwmC25PkW4tz3JgnRtiu6mChbK64j3o1YHyRWN9heE4v5pX6i
q+9ZEgULK02JsaSHfTSTmDKmPbYbTlyiuXFwMJkbFA99OSKzAv4xlSr00Bjz
kjZAB7VaTew//KOLuC7KGDKRP2bdrPtkL4iqd8K3mXalF5oH1sfjgtGz4zFH
BOdnbpcTkx7MLmRB8fBRpxwuN+s+KODXQvVnYobP9/cENGQtHq+FS9xDXzqh
JywvtxtG/u4qmoT681Hj0/D0K97ZX3BuafSqF/L49nQ/3FyzXMfmoVeOlbcD
jqk9U/Hw5+V6JgsKW8b71dDcuO9iHhzoTK6iofZKb2sbbFrRy9QhjPTIQvB+
Gp5Kl0+2w5oK/p6ncKR1fvAFXGzrXVqF5bWKEi90p51J24DOGfFKRCgjDddY
32+D/wFuez3D
  "]]
}, Closed]],

Cell["\<\
These vectors are linearly independent at regular points, e.g. for the unit \
sphere  we have:\
\>", "Text",
 CellChangeTimes->{{3.585983209687971*^9, 3.585983232980303*^9}, {
   3.5859833216453743`*^9, 3.5859833377072935`*^9}, {3.6176024290121117`*^9, 
   3.6176024402441316`*^9}, {3.617602687020965*^9, 3.6176027192350216`*^9}, {
   3.6176027674079065`*^9, 3.617602782461933*^9}, 3.6176155345117188`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Cross", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", "1", "]"}], "[", 
      RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", "2", "]"}], "[", 
      RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", "u", "]"}], " ", 
    SuperscriptBox[
     RowBox[{"Cos", "[", "v", "]"}], "2"]}], ",", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"Cos", "[", "v", "]"}], "2"], " ", 
    RowBox[{"Sin", "[", "u", "]"}]}], ",", 
   RowBox[{
    RowBox[{"Cos", "[", "v", "]"}], " ", 
    RowBox[{"Sin", "[", "v", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.627104840175207*^9, 3.6272863838035884`*^9}]
}, Closed]],

Cell["Only the points  with v =\[PlusMinus] \[Pi]/2 are singular:", "Text",
 CellChangeTimes->{{3.617602935342201*^9, 3.617602952377431*^9}, {
  3.617602984887888*^9, 3.6176030247771587`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", " ", 
  RowBox[{"v", "\[Rule]", " ", 
   RowBox[{"Pi", "/", "2"}]}]}]], "Input",
 CellChangeTimes->{{3.5859825732805705`*^9, 3.5859825823570895`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.627104844278014*^9, 3.627286392182803*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tv", "[", "1", "]"}], "[", 
   RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
  RowBox[{"u", ",", 
   RowBox[{"Pi", "/", "2"}]}], "]"}]], "Input",
 CellChangeTimes->{
  3.5859826492939177`*^9, 3.617600023854287*^9, {3.6271048543244314`*^9, 
   3.6271048649636507`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.627104866258453*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tv", "[", "2", "]"}], "[", 
   RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
  RowBox[{"u", ",", 
   RowBox[{"Pi", "/", "2"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5859826492939177`*^9, 3.5859826628216915`*^9}, 
   3.6176000856927958`*^9, {3.627104876975672*^9, 3.6271048869752893`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"Cos", "[", "u", "]"}]}], ",", 
   RowBox[{"-", 
    RowBox[{"Sin", "[", "u", "]"}]}], ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.627104887958091*^9}]
}, Closed]],

Cell["\<\
Thus the poles are singularities of the parameter representation in spite of \
the fact that geometrically all points of the sphere have the same \
properties. Remark that singular is a property of the parameter \
representation, not of the surface, considered as the image of the presenting \
function sf.\
\>", "Text",
 CellChangeTimes->{{3.585983439303104*^9, 3.585983656011499*^9}, {
   3.5859837425554495`*^9, 3.585983800018736*^9}, 3.588937868466625*^9, 
   3.5889388391471305`*^9, 3.599489609168592*^9, {3.617603095211282*^9, 
   3.6176031171137204`*^9}}],

Cell["\<\
 The vertex of a cone is an example of a singularity reflecting a geometric \
property of the surface:\
\>", "Text",
 CellChangeTimes->{{3.585983439303104*^9, 3.585983656011499*^9}, {
   3.5859837425554495`*^9, 3.585983800018736*^9}, 3.588937868466625*^9}],

Cell[BoxData[
 RowBox[{"?", "circularcone"}]], "Input",
 CellChangeTimes->{{3.6147755833974557`*^9, 3.614775607648843*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"circularcone", "[", 
   RowBox[{".5", ",", "1"}], "]"}], "[", 
  RowBox[{
   RowBox[{"-", "Pi"}], ",", "Pi", ",", 
   RowBox[{"-", "1"}], ",", "1"}], "]"}]], "Input",
 CellChangeTimes->{{3.5859838112913804`*^9, 3.585983897230296*^9}, {
  3.614775887921873*^9, 3.6147759149784207`*^9}, {3.6147759454011607`*^9, 
  3.6147760046025467`*^9}, {3.614776258627076*^9, 3.6147762664975266`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"con", " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.614776032433139*^9, 3.6147760417846737`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"circularcone", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{3.6147761956744757`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", " ", "v", " ", 
    RowBox[{"Cos", "[", "u", "]"}]}], ",", 
   RowBox[{"a", " ", "v", " ", 
    RowBox[{"Sin", "[", "u", "]"}]}], ",", 
   RowBox[{"b", " ", "v"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6271049118885326`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Cross", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", "1", "]"}], "[", 
      RowBox[{"circularcone", "[", 
       RowBox[{"a", ",", "b"}], "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", "2", "]"}], "[", 
      RowBox[{"circularcone", "[", 
       RowBox[{"a", ",", "b"}], "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5859839125391717`*^9, 3.585983965798218*^9}, {
   3.6147756341853604`*^9, 3.6147756746416745`*^9}, 3.614775705002411*^9, {
   3.6147757402034245`*^9, 3.6147757465467873`*^9}, 3.6176000238698874`*^9, 
   3.617600085708396*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", " ", "b", " ", "v", " ", 
    RowBox[{"Cos", "[", "u", "]"}]}], ",", 
   RowBox[{"a", " ", "b", " ", "v", " ", 
    RowBox[{"Sin", "[", "u", "]"}]}], ",", 
   RowBox[{
    RowBox[{"-", 
     SuperscriptBox["a", "2"]}], " ", "v"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627286473437746*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"v", "\[Rule]", "0"}]}]], "Input",
 CellChangeTimes->{{3.5859840514991198`*^9, 3.5859840750214653`*^9}, {
  3.614775781144766*^9, 3.6147757874811287`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.6271049243217545`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"pv", " ", "=", 
   RowBox[{"Graphics3D", "[", 
    RowBox[{"{", 
     RowBox[{"Black", ",", " ", 
      RowBox[{"PointSize", "[", ".03", "]"}], ",", 
      RowBox[{"Point", "[", 
       RowBox[{
        RowBox[{"circularcone", "[", 
         RowBox[{".5", ",", "1"}], "]"}], "[", 
        RowBox[{"1", ",", "0"}], "]"}], "]"}]}], "}"}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.585984234159567*^9, 3.5859843523663282`*^9}, {
  3.5859844135268264`*^9, 3.585984470358077*^9}, {3.585984555310936*^9, 
  3.585984577831224*^9}, {3.614775846146484*^9, 3.614775861514363*^9}, {
  3.614776134090953*^9, 3.6147761364340878`*^9}, {3.614776216867688*^9, 
  3.614776226506239*^9}, {3.6147763213466635`*^9, 3.614776385643341*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"{", 
   RowBox[{"pv", ",", "con"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.585984508907282*^9, 3.5859845240771494`*^9}}],

Cell[TextData[{
 StyleBox["\nLemma",
  FontColor->RGBColor[0, 0, 1]],
 ". At any regular point f[u,...]  of the immersion f the tangent vectors  \
tv[1][sf], tv[2][sf],... are a basis of the tangent vector space, ",
 StyleBox["often this basis is named the",
  FontColor->GrayLevel[0]],
 StyleBox[" standard basis field of the immersion f.",
  FontColor->RGBColor[0, 0, 1]],
 "  The tangent vector space T[f][u,...]  consists of all vectors"
}], "Text",
 CellChangeTimes->{{3.5859846820571856`*^9, 3.5859849956431217`*^9}, 
   3.588922731067581*^9, 3.6176000239010873`*^9, 3.617600085723996*^9, {
   3.617603202555071*^9, 3.617603223053507*^9}, 3.6176036724278955`*^9, {
   3.6176037379480104`*^9, 3.617603739086813*^9}, {3.6176037930629077`*^9, 
   3.6176038505646086`*^9}, {3.617615688281189*^9, 3.6176157025396137`*^9}, {
   3.6227747642993546`*^9, 3.6227748548575134`*^9}, {3.627286495826786*^9, 
   3.627286519057227*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"tangsp", "[", "f_", "]"}], "[", "u__", "]"}], "[", "x__", "]"}], 
  ":=", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"Not", "[", 
     RowBox[{
      RowBox[{"Length", "[", 
       RowBox[{"{", "x", "}"}], "]"}], "\[Equal]", " ", 
      RowBox[{"Length", "[", 
       RowBox[{"{", "u", "}"}], "]"}]}], "]"}], ",", 
    RowBox[{
     RowBox[{"Print", "[", "\"\<invalid arguments!\>\"", "]"}], ";", 
     RowBox[{"Return", "[", "]"}]}], ",", 
    RowBox[{"Return", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"d", "[", "f", "]"}], "[", "u", "]"}], ".", 
      RowBox[{"{", "x", "}"}]}], "]"}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6227728201235375`*^9, 3.6227728854252524`*^9}, {
   3.622772920978715*^9, 3.6227730892718105`*^9}, {3.622773126056675*^9, 
   3.6227731974268007`*^9}, 3.622774638734734*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "tangsp"}]], "Input",
 CellChangeTimes->{{3.6147768255615034`*^9, 3.614776948794552*^9}, 
   3.614776996347272*^9, {3.622773295051772*^9, 3.622773314317806*^9}, {
   3.6227733699007034`*^9, 3.622773445950837*^9}, {3.6227741322926426`*^9, 
   3.622774141028658*^9}}],

Cell[BoxData[
 StyleBox["\<\"tangsp[f][u,...][x,...] is the tangent vector with standard \
\\ncoordinates x,... of the immersion f[u,...] at the point {u,...}.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.6271050544883833`*^9},
 CellTags->"Info3627108654-6604827"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tangsp", "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "[", 
  RowBox[{"x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.6227732270980525`*^9, 3.6227732613401127`*^9}, {
   3.622773330448234*^9, 3.622773340057851*^9}, 3.622774876635152*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"y", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "+", 
    RowBox[{"x", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"y", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "+", 
    RowBox[{"x", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"y", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "+", 
    RowBox[{"x", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6271050612275953`*^9, 3.627286543143669*^9}]
}, Closed]],

Cell[TextData[{
 "The ",
 StyleBox["tangent plane",
  FontColor->RGBColor[0, 0, 1]],
 " of the surface f at the point f[u0,v0] has the parameter representation \
(with u0, v0  fixed, x ,y the variable parameters)"
}], "Text",
 CellChangeTimes->{{3.5859851459307175`*^9, 3.585985220786999*^9}, {
  3.588946109087499*^9, 3.588946114765909*^9}, {3.5889990371028385`*^9, 
  3.5889990799832916`*^9}, {3.6271053085504293`*^9, 3.6271053144316397`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"tplane", "[", "f_", "]"}], "[", 
    RowBox[{"u0_", ",", "v0_"}], "]"}], "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"f", "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", "+", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"tangsp", "[", "f", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", 
      RowBox[{"x", ",", "y"}], "]"}]}], ")"}], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "\[Rule]", "u0"}], ",", 
     RowBox[{"v", "\[Rule]", "v0"}]}], "}"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.585985258936181*^9, 3.585985353414585*^9}, {
   3.5859991971198626`*^9, 3.585999204311475*^9}, {3.5859996025801744`*^9, 
   3.585999633468229*^9}, {3.588945457053154*^9, 3.5889454978784256`*^9}, 
   3.588945546441311*^9, {3.5889991214026604`*^9, 3.5889991285400686`*^9}, {
   3.6227735926222944`*^9, 3.622773593511496*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "tplane"}]], "Input",
 CellChangeTimes->{{3.6147771023553348`*^9, 3.614777261106415*^9}, 
   3.6174557564293923`*^9, {3.62277389139602*^9, 3.6227739369948997`*^9}, {
   3.6227743400702095`*^9, 3.6227743474646225`*^9}}],

Cell[BoxData[
 StyleBox["\<\"tplane[f][uo,vo][x,y] is a parameter representation of the \
tangent \\nplane of the surface f[u,v]in the affine n-space at the point {uo, \
vo} with parameters x,y.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.6272865808549356`*^9},
 CellTags->"Info3627290180-6629669"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tplane", "[", 
    RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
   RowBox[{
    RowBox[{"Pi", "/", "2"}], ",", "1"}], "]"}], "[", 
  RowBox[{"x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.5889390507927017`*^9, 3.5889390514323034`*^9}, 
   3.5889455743653603`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "x"}], " ", 
    RowBox[{"Cos", "[", "1", "]"}]}], ",", 
   RowBox[{
    RowBox[{"Cos", "[", "1", "]"}], "-", 
    RowBox[{"y", " ", 
     RowBox[{"Sin", "[", "1", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"y", " ", 
     RowBox[{"Cos", "[", "1", "]"}]}], "+", 
    RowBox[{"Sin", "[", "1", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.627105219131072*^9, 3.627286587315347*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"tpl1", " ", "=", 
   RowBox[{"ParametricPlot3D", "[", 
    RowBox[{
     RowBox[{"Evaluate", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"tplane", "[", 
         RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
        RowBox[{"0", ",", "1."}], "]"}], "[", 
       RowBox[{"x", ",", "y"}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"y", ",", 
       RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
     RowBox[{"Mesh", "\[Rule]", " ", "None"}], ",", " ", 
     RowBox[{"PlotStyle", " ", "\[Rule]", " ", 
      RowBox[{"{", 
       RowBox[{"Green", ",", 
        RowBox[{"Opacity", "[", "0.5", "]"}]}], "}"}]}]}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.5859987751547213`*^9, 3.5859990333351746`*^9}, 
   3.5859992311435223`*^9, {3.5859993118268642`*^9, 3.585999327910492*^9}, 
   3.5859996610334773`*^9, 3.5859997184727783`*^9, {3.5859997699684687`*^9, 
   3.585999786676098*^9}, {3.585999823258162*^9, 3.5859998785290594`*^9}, {
   3.5860001388935165`*^9, 3.5860001718407745`*^9}, {3.586000360788306*^9, 
   3.586000380413141*^9}, 3.5860004924525375`*^9, {3.6147774414607306`*^9, 
   3.614777461979904*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"tpn", "=", 
   RowBox[{"Graphics3D", "[", 
    RowBox[{"{", 
     RowBox[{"Black", ",", 
      RowBox[{"PointSize", "[", ".01", "]"}], ",", 
      RowBox[{"Point", "[", 
       RowBox[{
        RowBox[{"sphero", "[", "1", "]"}], "[", 
        RowBox[{"0", ",", "1."}], "]"}], "]"}]}], "}"}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.585981598798833*^9, 3.585981608132367*^9}, {
   3.5859816511338263`*^9, 3.5859816640615664`*^9}, {3.585981694846327*^9, 
   3.585981764203294*^9}, {3.5859818411426945`*^9, 3.58598188005392*^9}, {
   3.5859999785564346`*^9, 3.5860000137656965`*^9}, 3.5860001190502815`*^9, 
   3.586000241026896*^9}],

Cell[BoxData[
 RowBox[{"tpsph", " ", "=", 
  RowBox[{"Show", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"tpl1", ",", " ", "tpn", ",", "sph"}], "}"}], ",", 
    RowBox[{"PlotRange", "\[Rule]", " ", "All"}], ",", 
    RowBox[{"Boxed", "\[Rule]", "False"}], ",", " ", 
    RowBox[{"Axes", "\[Rule]", " ", "False"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.585999915313924*^9, 3.585999934267957*^9}, {
  3.586000024560916*^9, 3.5860001012818503`*^9}, {3.5860004230324154`*^9, 
  3.586000447462058*^9}, {3.58600067040205*^9, 3.5860006744112573`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tplane", "[", "sf", "]"}], "[", 
   RowBox[{"u0", ",", "v0"}], "]"}], "[", 
  RowBox[{"x", ",", "y"}], "]"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.585985258936181*^9, 3.585985353414585*^9}, {
   3.5859991971198626`*^9, 3.585999204311475*^9}, {3.5859996025801744`*^9, 
   3.585999633468229*^9}, {3.588945457053154*^9, 3.5889454978784256`*^9}, 
   3.588945546441311*^9, {3.5889991214026604`*^9, 3.5889991285400686`*^9}, {
   3.5889992556603394`*^9, 3.588999298920814*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"f", "[", "1", "]"}], "[", 
     RowBox[{"u0", ",", "v0"}], "]"}], "+", 
    RowBox[{"y", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u0", ",", "v0"}], "]"}]}], "+", 
    RowBox[{"x", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u0", ",", "v0"}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"f", "[", "2", "]"}], "[", 
     RowBox[{"u0", ",", "v0"}], "]"}], "+", 
    RowBox[{"y", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u0", ",", "v0"}], "]"}]}], "+", 
    RowBox[{"x", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u0", ",", "v0"}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"f", "[", "3", "]"}], "[", 
     RowBox[{"u0", ",", "v0"}], "]"}], "+", 
    RowBox[{"y", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u0", ",", "v0"}], "]"}]}], "+", 
    RowBox[{"x", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u0", ",", "v0"}], "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->CompressedData["
1:eJwV0l1Ik2EAxfH5sbllDfUmpKVOQsSWolmIWPpKjJroptAmUhKVHyE5yJc5
WK05Vg1NQS8izZxYYC2nZuaypFYOZSyT/GiaRjKbge+wzBwtZ3aei8Pv9lz8
heeVRWVBLBZLgbXTmrbFEIb6Zb1Br8BS4V9ZWChD6YtyvxJvv3ryjg+5rlJv
BDQ1xYkjIXt+ckAEjSJVczKMiVNHpMCCreRsYvlI+0gJbDHIq/RwZa5E1QQd
B65m9sKZnakZF6xtze9ZgpzBPn0om6HGMi8EJ8HFWfp+IYx6br5khM6OnkNm
OKfZ75mFOTR/MYjDUFmNDRnBcMGtnGfDtWYnnwfjzvSow+GU662J6HvR+JoY
7Ntn2Q0lGe6gPVDwZnwvsfdyazZR+zCmnBilqqgjPoj+YyWe4D/7SExrqA4Q
O5PHU/kwxWdMSSA/jtpPEdv8NCcRWtM5ISI4Np1ULyVylVIt1Avf0zpobNpQ
EzWjP7qJo+aTL43k/7kvrMewoCH2lhN2OYYK/VB+2D+RGMZQ14Q2dhJ8Wnz3
CjGQ7v6eBzmU2JkPf97TLEjh4LJ8mbjk39o+C3V5q5LrcPhm1rcu2N+oOG2G
HtWjskko8QomXFAk41VuQku13eCH0/VDuQFYo/5dF8olXQxw4+GdT54pEdRV
DjN5cEHwoV8O5ZGpNQqojT9SS8OEbUuHCRYrDWI5j6EiKrhpNFy3ySY7oXgz
/KAdFrR7rQ64arKsbcDjO9IqP+y2Xcz5B1tcx9ajd6GLub7PsfA/OVNB2w==

  "]]
}, Closed]],

Cell["\<\
The tangential plane  tplane[sf][u0, v0][x, y] is defined at regular points \
sf[u0,v0] only; at singular points the function tplane \
doesn\[CloseCurlyQuote]t yield a plane, e.g.\
\>", "Text",
 CellChangeTimes->{{3.6272867521310377`*^9, 3.6272869185877314`*^9}, 
   3.6272870472103577`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tplane", "[", 
    RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
   RowBox[{"u0", ",", 
    RowBox[{"Pi", "/", "2"}]}], "]"}], "[", 
  RowBox[{"x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.6272866582566724`*^9, 3.6272866825811152`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "r"}], " ", "y", " ", 
    RowBox[{"Cos", "[", "u0", "]"}]}], ",", 
   RowBox[{
    RowBox[{"-", "r"}], " ", "y", " ", 
    RowBox[{"Sin", "[", "u0", "]"}]}], ",", "r"}], "}"}]], "Output",
 CellChangeTimes->{3.627286689039527*^9}]
}, Closed]],

Cell[TextData[{
 "The ",
 StyleBox["normal vector",
  FontColor->RGBColor[0, 0, 1]],
 " of the (oriented) surface sf at the regular point sf[u,v] is defined as \
the normal unit vector of  the oriented tangent plane; moreover, we define now"
}], "Text",
 CellChangeTimes->{{3.586000726421748*^9, 3.5860008523139696`*^9}, {
   3.58600106321074*^9, 3.5860010810883713`*^9}, {3.5889225134315987`*^9, 
   3.588922539031244*^9}, {3.588922572540103*^9, 3.5889225905893345`*^9}, 
   3.5889466155267887`*^9, {3.588999331519678*^9, 3.5889993553910437`*^9}, 
   3.617615934153221*^9, {3.6272869386533666`*^9, 3.6272869442069764`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["1.2. An Orthonormal Moving 3-Frame of the Surface", "Subsection",
 CellChangeTimes->{{3.5721739619617743`*^9, 3.5721739801278133`*^9}, {
   3.5860011060172153`*^9, 3.5860011130840273`*^9}, {3.587272413023604*^9, 
   3.5872724205980372`*^9}, {3.588487546773712*^9, 3.588487560267736*^9}, {
   3.5994914425513973`*^9, 3.5994914450615406`*^9}, 3.599552816652402*^9, {
   3.614777404959643*^9, 3.614777407655797*^9}, 3.614951503403132*^9, {
   3.6176159535908546`*^9, 3.617615958161663*^9}, 3.617616389845621*^9, 
   3.6272869628842096`*^9}],

Cell[CellGroupData[{

Cell["1.2.1. The Definition", "Subsubsection",
 CellChangeTimes->{{3.5860011346276655`*^9, 3.586001146530486*^9}, {
   3.5860678825807543`*^9, 3.586067896183978*^9}, {3.5872724267433887`*^9, 
   3.587272433749789*^9}, 3.6176163958204317`*^9, 3.6176185661766634`*^9, 
   3.627287284893177*^9, 3.62730821645765*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";"}]], "Input",
 CellChangeTimes->{{3.61761719716236*^9, 3.6176172042407646`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sf", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.617617219455635*^9, 3.617617242201936*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "1", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "2", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "3", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627306022304499*^9}]
}, Closed]],

Cell["In eudiffgeov2.m are defined", "Text",
 CellChangeTimes->{{3.6226001123748865`*^9, 3.6226001345581255`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"amf", "[", "1", "]"}], "[", "f_", "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"dotnorme", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", "1", "]"}], "[", "f", "]"}], "[", 
     RowBox[{"s", ",", "t"}], "]"}], "]"}], "/;", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"Head", "[", 
       RowBox[{"f", "[", 
        RowBox[{"u", ",", "v"}], "]"}], "]"}], " ", "\[Equal]", " ", "List"}],
      "&&", 
     RowBox[{
      RowBox[{"Length", "[", 
       RowBox[{"f", "[", 
        RowBox[{"u", ",", "v"}], "]"}], "]"}], "\[Equal]", " ", "3"}]}], 
    ")"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{
  3.553950276121236*^9, {3.5860012023005843`*^9, 3.586001242361455*^9}, {
   3.5860012820791245`*^9, 3.586001290471939*^9}, {3.614861848911121*^9, 
   3.6148618489267206`*^9}, {3.6148619944905767`*^9, 3.614862019809421*^9}, 
   3.6149178247118187`*^9, 3.614917951836442*^9, 3.617600023979088*^9, {
   3.6176169240037365`*^9, 3.6176169255228233`*^9}, {3.6176170208992786`*^9, 
   3.6176170992707605`*^9}, 3.617617644972973*^9, {3.617617700828168*^9, 
   3.617617748052869*^9}, 3.627304505615427*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"amf", "[", "1", "]"}], "[", "sf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5718247469642887`*^9, 3.5718247900827646`*^9}, {
   3.5721744197469583`*^9, 3.572174420562005*^9}, 3.6148618489267206`*^9, 
   3.614917824727419*^9, 3.6149179629904613`*^9, {3.6273044458769217`*^9, 
   3.6273044687661624`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"f", "[", "1", "]"}], 
      TagBox[
       RowBox[{"(", 
        RowBox[{"1", ",", "0"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"]}]]], ",", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"f", "[", "2", "]"}], 
      TagBox[
       RowBox[{"(", 
        RowBox[{"1", ",", "0"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"]}]]], ",", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"f", "[", "3", "]"}], 
      TagBox[
       RowBox[{"(", 
        RowBox[{"1", ",", "0"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"]}]]]}], "}"}]], "Output",
 CellChangeTimes->{{3.6273044331316996`*^9, 3.6273044699361644`*^9}, 
   3.627306022335699*^9}]
}, Closed]],

Cell[TextData[{
 "The ",
 StyleBox["normal vector",
  FontColor->RGBColor[0, 0, 1]],
 " of the oriented surface f at the point f[u,v] is"
}], "Text",
 CellChangeTimes->{{3.5718418633550053`*^9, 3.5718418720286207`*^9}, {
  3.586001385070505*^9, 3.586001418438964*^9}, {3.586001462259441*^9, 
  3.586001488514287*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"amf", "[", "3", "]"}], "[", "f_", "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"dotnorme", "[", 
    RowBox[{"Cross", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"tv", "[", "1", "]"}], "[", "f", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"tv", "[", "2", "]"}], "[", "f", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "]"}], "/;", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"Head", "[", 
       RowBox[{"f", "[", 
        RowBox[{"u", ",", "v"}], "]"}], "]"}], " ", "\[Equal]", " ", "List"}],
      "&&", 
     RowBox[{
      RowBox[{"Length", "[", 
       RowBox[{"f", "[", 
        RowBox[{"u", ",", "v"}], "]"}], "]"}], "\[Equal]", " ", "3"}]}], 
    ")"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.571841915895898*^9, 3.5718420523961377`*^9}, {
   3.5860013177251873`*^9, 3.586001351670847*^9}, {3.614861848957921*^9, 
   3.614861848957921*^9}, 3.614862118307994*^9, 3.614917824727419*^9, 
   3.6149179656580667`*^9, 3.6176000239946876`*^9, 3.617600085723996*^9, 
   3.617617128124411*^9, 3.6176248811064234`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"amf", "[", "2", "]"}], "[", "f_", "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"Cross", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "3", "]"}], "[", "f", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "1", "]"}], "[", "f", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "/;", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"Head", "[", 
       RowBox[{"f", "[", 
        RowBox[{"u", ",", "v"}], "]"}], "]"}], " ", "\[Equal]", " ", "List"}],
      "&&", 
     RowBox[{
      RowBox[{"Length", "[", 
       RowBox[{"f", "[", 
        RowBox[{"u", ",", "v"}], "]"}], "]"}], "\[Equal]", " ", "3"}]}], 
    ")"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{
  3.5718421091490374`*^9, 3.5718421511287107`*^9, {3.5718422225768366`*^9, 
   3.571842229035248*^9}, {3.5718433048599377`*^9, 3.5718433055619383`*^9}, {
   3.586001531227162*^9, 3.586001556202806*^9}, {3.614861848973521*^9, 
   3.614861848973521*^9}, {3.614917824743019*^9, 3.614917824758619*^9}, {
   3.614917976219285*^9, 3.614918000305727*^9}, 3.617617135587838*^9, 
   3.6176248570180454`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "amf"}]], "Input",
 CellChangeTimes->{{3.6148635430116963`*^9, 3.6148637222872114`*^9}, {
   3.6148638592242517`*^9, 3.6148639120303445`*^9}, {3.6148639626212335`*^9, 
   3.6148639914500837`*^9}, 3.614917824758619*^9, {3.614918015094553*^9, 
   3.6149180265449734`*^9}, {3.6226001544637604`*^9, 3.622600173183793*^9}}],

Cell[BoxData[
 StyleBox["\<\"amf[i][f][u,v]is the i-th vector, i = 1,2,3, of the \
orthonormal moving \\nframe adapted to the parameter representation of the \
surface f[u,v] in the \\nEuclidean 3-space.\"\>", "MSG"]], "Print", \
"PrintUsage",
 CellChangeTimes->{3.6273060225072994`*^9},
 CellTags->"Info3627309622-3465247"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5718247469642887`*^9, 3.5718247900827646`*^9}, 
   3.571824823950424*^9, {3.571824855134879*^9, 3.5718248677553015`*^9}, 
   3.5718426714520245`*^9, {3.5721742134671593`*^9, 3.5721742146832294`*^9}, 
   3.614861848989121*^9, 3.614917824758619*^9, 3.6149180316617823`*^9, {
   3.627304553807912*^9, 3.627304570800342*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "-", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"\[Sqrt]", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "-", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "-", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "-", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], ")"}]}], ")"}]
     }], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"\[Sqrt]", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "-", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "-", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "-", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], ")"}]}], ")"}]
     }], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "-", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"\[Sqrt]", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "-", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "-", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "-", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], ")"}]}], ")"}]
     }]}], "}"}]], "Output",
 CellChangeTimes->{{3.627304541671091*^9, 3.627304571783144*^9}, 
   3.6273060227101*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"amf", "[", "2", "]"}], "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5718247469642887`*^9, 3.5718247900827646`*^9}, 
   3.571824823950424*^9, {3.571824855134879*^9, 3.5718248677553015`*^9}, 
   3.5718426714520245`*^9, {3.5721742134671593`*^9, 3.5721742146832294`*^9}, 
   3.572174471475917*^9, 3.614861848989121*^9, 3.614917824774219*^9, 
   3.6149180333933854`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "-", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      SqrtBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"]}]], " ", 
      RowBox[{"\[Sqrt]", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], ")"}]}]}], 
     ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      SqrtBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"]}]], " ", 
      RowBox[{"\[Sqrt]", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], ")"}]}]}], 
     ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      SqrtBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"]}]], " ", 
      RowBox[{"\[Sqrt]", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], ")"}]}]}], 
     ")"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.627304577867154*^9, 3.6273060234121013`*^9}]
}, Closed]],

Cell["The orthogonality conditions are fulfilled:", "Text",
 CellChangeTimes->{{3.572174706905383*^9, 3.572174739048221*^9}, 
   3.572175509736302*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "i", "]"}], "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ".", 
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "j", "]"}], "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "3"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5718249735702868`*^9, 3.571825082302478*^9}, {
   3.571825121474147*^9, 3.571825217539116*^9}, {3.5718423315118275`*^9, 
   3.571842382430317*^9}, {3.572174517435546*^9, 3.5721745795881004`*^9}, {
   3.6148618490047207`*^9, 3.6148618490047207`*^9}, 3.6148635013128233`*^9, {
   3.614917824774219*^9, 3.614917824774219*^9}, {3.614918034984588*^9, 
   3.614918038853395*^9}, {3.6149467734835796`*^9, 3.6149467756675835`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "1"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6273045876015716`*^9, 3.6273060258769054`*^9}]
}, Closed]],

Cell["and the frame is positively oriented", "Text",
 CellChangeTimes->{{3.5885663623766937`*^9, 3.5885663832495303`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Det", "[", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "i", "]"}], "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3"}], "}"}]}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5885663928903475`*^9, 3.588566463870472*^9}, {
   3.6148618490047207`*^9, 3.6148618490047207`*^9}, 3.6148635013128233`*^9, 
   3.614917824774219*^9, 3.614918046637809*^9}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.6273045939819827`*^9, 3.627306026797307*^9}]
}, Closed]],

Cell[TextData[{
 "The frame {",
 "amf[",
 "i][f][u,v]} defined above is named the",
 StyleBox[" orthonormal moving frame adapted to the parameter representation",
  FontColor->RGBColor[0, 0, 1]],
 " f[u,v]."
}], "Text",
 CellChangeTimes->{{3.586002264085249*^9, 3.5860023096529293`*^9}, {
   3.586002387465866*^9, 3.5860024607859945`*^9}, 3.614861849035921*^9, 
   3.614917824789819*^9, 3.6149180502102146`*^9}],

Cell[TextData[{
 "The",
 StyleBox[" tangent plane",
  FontColor->RGBColor[0, 0, 1]],
 " of the surface at the point f[u,v] is also obtained by "
}], "Text",
 CellChangeTimes->{{3.572174857303985*^9, 3.572174923024744*^9}, 
   3.572255761617314*^9, 3.586002208315151*^9, 3.58600250944248*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"tplane2", "[", "f_", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], "[", 
   RowBox[{"s1_", ",", "s2_"}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"f", "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", "+", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"amf", "[", "1", "]"}], "[", "f", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "*", "s1"}], " ", "+", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"amf", "[", "2", "]"}], "[", "f", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "*", "s2"}]}], ")"}], "/;", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"Head", "[", 
       RowBox[{"f", "[", 
        RowBox[{"u", ",", "v"}], "]"}], "]"}], " ", "\[Equal]", " ", "List"}],
      "&&", 
     RowBox[{
      RowBox[{"Length", "[", 
       RowBox[{"f", "[", 
        RowBox[{"u", ",", "v"}], "]"}], "]"}], "\[Equal]", " ", "3"}]}], 
    ")"}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5721749454350257`*^9, 3.5721751309256353`*^9}, {
   3.5860025164312925`*^9, 3.58600253769413*^9}, {3.5889995834210863`*^9, 
   3.5889995911395283`*^9}, {3.5889996328979163`*^9, 3.588999681604702*^9}, {
   3.599552093004184*^9, 3.599552117262227*^9}, {3.6148618490515213`*^9, 
   3.6148618490671215`*^9}, {3.614917824805419*^9, 3.614917824805419*^9}, {
   3.614918053626621*^9, 3.614918054999423*^9}, {3.6176173159571543`*^9, 
   3.6176173279088383`*^9}, 3.622781337067371*^9, {3.6273046857825446`*^9, 
   3.6273046926153564`*^9}, 3.6273047310266247`*^9, {3.627305109362291*^9, 
   3.627305125543519*^9}, 3.627305353654521*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tplane2", "::", "usage"}], " ", "=", " ", 
   "\"\<tplane2[f][uo,vo][x,y] is the parameter representation of the tangent \
plane of the surface f[u,v] in the Euclidean 3-space adapted to the moving \
frame of the surface. x,y are orthonormal Cartesian parameters of the tangent \
plane at the point {uo,vo} of the surface.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6148641539087696`*^9, 3.614864405428011*^9}, {
   3.6148644494668884`*^9, 3.6148644830069475`*^9}, 3.614864538574245*^9, 
   3.617617381974931*^9, {3.6227813534629993`*^9, 3.6227813555222034`*^9}, 
   3.622781397377077*^9}],

Cell["\<\
or as the set of the solutions {x,y,z} of the linear equation\
\>", "Text",
 CellChangeTimes->{{3.5721752650483065`*^9, 3.5721752988642406`*^9}, {
   3.5721754466666946`*^9, 3.57217544781676*^9}, {3.5860026178782706`*^9, 
   3.5860026297342916`*^9}, {3.5860027381388817`*^9, 
   3.5860027435364914`*^9}, {3.589000353591138*^9, 3.5890003553912406`*^9}, 
   3.589000402351927*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ".", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "y", ",", "z"}], "}"}], "-", 
       RowBox[{"sf", "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], ")"}]}], " ", "\[Equal]", " ", 
    "0"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5721753037755213`*^9, 3.5721754164589667`*^9}, {
   3.586001703139464*^9, 3.586001727569107*^9}, {3.586002570703788*^9, 
   3.5860025763353977`*^9}, {3.5860026404515104`*^9, 3.5860026441331167`*^9}, 
   3.5860026891235957`*^9, {3.589000060667383*^9, 3.5890000674507713`*^9}, {
   3.589000200076357*^9, 3.58900020817082*^9}, {3.5890004665876007`*^9, 
   3.5890004846996365`*^9}, {3.614861849098321*^9, 3.614861849098321*^9}, 
   3.614864650832042*^9, 3.614917824805419*^9, 3.6149180817846704`*^9, {
   3.6149471868219056`*^9, 3.614947213201552*^9}, {3.617617424587368*^9, 
   3.617617437755121*^9}, 3.6273058974630795`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Solve", "::", "svars"}], "MessageName"], ":", 
  " ", "\<\"Equations may not give solutions for all \\\"solve\\\" variables. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/Solve/svars\\\", \
ButtonNote -> \\\"Solve::svars\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.627305899631483*^9, 3.627306026859707*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"z", "\[Rule]", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"y", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}]}], "-", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], ")"}]}], "-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}]}], "-", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], "-", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"\[Sqrt]", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "-", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"-", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "-", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], ")"}]}], 
        " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"f", "[", "3", "]"}], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "2", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], " ", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}]}], "-", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], " ", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "2", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
             RowBox[{"\[Sqrt]", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "-", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"-", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "-", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], 
               ")"}]}], ")"}]}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"f", "[", "2", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "3", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}]}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"\[Sqrt]", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "-", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"-", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "-", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], 
              ")"}]}], ")"}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"f", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "-", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"\[Sqrt]", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "-", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"-", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "-", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], 
              ")"}]}], ")"}]}]}], ")"}]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], "-", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}]}], "}"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.6273058996626835`*^9, 3.627306026859707*^9}]
}, Closed]],

Cell[TextData[{
 StyleBox["Example",
  FontColor->RGBColor[0, 0, 1]],
 ". ",
 StyleBox["Circular Cone",
  FontColor->GrayLevel[0]]
}], "Text",
 CellChangeTimes->{{3.5995526058523445`*^9, 3.5995526242033944`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "circularcone"}]], "Input",
 CellChangeTimes->{{3.5995522936517363`*^9, 3.5995522964441414`*^9}, {
  3.6148647754450607`*^9, 3.6148648052411137`*^9}}],

Cell[BoxData[
 StyleBox["\<\"{u,v}->circularcone[a,b][u,v] is a parametrization of a \
circular cone of radius a and slope b/a.  To plot try \
circularcone[1,0.5][-Pi,Pi,-1,1].\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.6273060270157075`*^9},
 CellTags->"Info3627309626-3465247"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"circularcone", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6148649822702246`*^9, 3.6148649910374393`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", " ", "v", " ", 
    RowBox[{"Cos", "[", "u", "]"}]}], ",", 
   RowBox[{"a", " ", "v", " ", 
    RowBox[{"Sin", "[", "u", "]"}]}], ",", 
   RowBox[{"b", " ", "v"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627304633906453*^9, 3.627304886636298*^9, 
  3.627305373142956*^9, 3.6273055448840585`*^9, 3.6273060271093073`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"tplane2", "[", 
      RowBox[{"circularcone", "[", 
       RowBox[{"1", ",", "1"}], "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{"s1", ",", "s2"}], "]"}], ",", " ", 
   RowBox[{"Element", "[", 
    RowBox[{"v", ",", "Reals"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.588999525808791*^9, 3.5889995544654303`*^9}, {
   3.5889998476021967`*^9, 3.5889998692254333`*^9}, {3.588999967738068*^9, 
   3.5889999693361597`*^9}, {3.5995521295394487`*^9, 
   3.5995521337046556`*^9}, {3.614864925345724*^9, 3.614864937404546*^9}, {
   3.614865041753129*^9, 3.6148650786939936`*^9}, 3.622781388953062*^9, 
   3.627304843640623*^9, {3.6273053996318026`*^9, 3.62730540400181*^9}, {
   3.6273055995037546`*^9, 3.6273056885743113`*^9}, {3.62730578452048*^9, 
   3.627305809422124*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox["s2", 
        SqrtBox["2"]], "+", "v"}], ")"}], " ", 
     RowBox[{"Cos", "[", "u", "]"}]}], "-", 
    RowBox[{"s1", " ", 
     RowBox[{"Sign", "[", "v", "]"}], " ", 
     RowBox[{"Sin", "[", "u", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"s1", " ", 
     RowBox[{"Cos", "[", "u", "]"}], " ", 
     RowBox[{"Sign", "[", "v", "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox["s2", 
        SqrtBox["2"]], "+", "v"}], ")"}], " ", 
     RowBox[{"Sin", "[", "u", "]"}]}]}], ",", 
   RowBox[{
    FractionBox["s2", 
     SqrtBox["2"]], "+", "v"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.6273046438144703`*^9, 3.627304844935425*^9, {3.627305152983968*^9, 
   3.6273051567163744`*^9}, 3.627305379616967*^9, 3.627305410257421*^9, 
   3.6273056954539237`*^9, 3.6273058254453526`*^9, 3.627306027156108*^9}]
}, Closed]],

Cell["In the case of f = circularcone[1,1][0,1]:", "Text",
 CellChangeTimes->{{3.5721752650483065`*^9, 3.5721752988642406`*^9}, {
   3.5721754466666946`*^9, 3.57217544781676*^9}, {3.5860026178782706`*^9, 
   3.5860026297342916`*^9}, {3.5860027381388817`*^9, 
   3.5860027435364914`*^9}, {3.589000353591138*^9, 3.5890003553912406`*^9}, 
   3.589000402351927*^9, {3.5890004969433365`*^9, 3.589000503406707*^9}, {
   3.6148651258996763`*^9, 3.6148651590185347`*^9}, {3.614947245400008*^9, 
   3.614947251000418*^9}, {3.6149472930736923`*^9, 3.614947322557744*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"amf", "[", "3", "]"}], "[", 
   RowBox[{"circularcone", "[", 
    RowBox[{"1", ",", "1"}], "]"}], "]"}], "[", 
  RowBox[{"0", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.617617511883361*^9, 3.6176175360677443`*^9}, {
   3.6176182028678827`*^9, 3.617618245404316*^9}, {3.6273059976836557`*^9, 
   3.6273060012112617`*^9}, 3.6273060669693775`*^9, 3.627307186882518*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox["v", 
    RowBox[{
     SqrtBox["2"], " ", 
     SqrtBox[
      SuperscriptBox["v", "2"]]}]], ",", "0", ",", 
   RowBox[{"-", 
    FractionBox["v", 
     RowBox[{
      SqrtBox["2"], " ", 
      SqrtBox[
       SuperscriptBox["v", "2"]]}]]}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.627306027702109*^9, 3.6273060688569813`*^9, 3.6273071887077208`*^9, 
   3.6273072929143047`*^9, {3.627307399818693*^9, 3.6273074240495358`*^9}, 
   3.627307977667229*^9}]
}, Closed]],

Cell["\<\
Clearly, the tangential plane of the circular cone doesn\[CloseCurlyQuote]t \
depend on v; it is constant along the generating lines.\
\>", "Text",
 CellChangeTimes->{{3.5890002696153345`*^9, 3.5890003178870955`*^9}, 
   3.6148653107288013`*^9, 3.614948837819605*^9, 3.6151172062957406`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"v", ">", "0"}], ",", 
   RowBox[{"Solve", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"%", ".", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"x", ",", "y", ",", "z"}], "}"}], "-", 
         RowBox[{
          RowBox[{"circularcone", "[", 
           RowBox[{"1", ",", "1"}], "]"}], "[", 
          RowBox[{"0", ",", "v"}], "]"}]}], ")"}]}], " ", "\[Equal]", " ", 
      "0"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5721753037755213`*^9, 3.5721754164589667`*^9}, {
   3.586001703139464*^9, 3.586001727569107*^9}, {3.586002570703788*^9, 
   3.5860025763353977`*^9}, {3.5860026404515104`*^9, 3.5860026441331167`*^9}, 
   3.5860026891235957`*^9, {3.589000060667383*^9, 3.5890000674507713`*^9}, {
   3.589000200076357*^9, 3.58900020817082*^9}, {3.614861849098321*^9, 
   3.614861849098321*^9}, 3.614864659848858*^9, 3.6148647314217834`*^9, {
   3.6148651794857707`*^9, 3.6148652085018215`*^9}, 3.614917824821019*^9, 
   3.6149180935314913`*^9, {3.6149473500449924`*^9, 3.6149473675482225`*^9}, 
   3.6149474635039916`*^9, 3.617618271444805*^9, 3.6176183090049534`*^9, {
   3.6273072196153755`*^9, 3.627307278357479*^9}, {3.627307394889085*^9, 
   3.627307417698325*^9}, 3.6273079680732126`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Solve", "::", "svars"}], "MessageName"], ":", 
  " ", "\<\"Equations may not give solutions for all \\\"solve\\\" variables. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/Solve/svars\\\", \
ButtonNote -> \\\"Solve::svars\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{
  3.6273060277645087`*^9, 3.6273060879262147`*^9, {3.627307268545062*^9, 
   3.6273073000299177`*^9}, {3.6273074036875*^9, 3.6273074276375427`*^9}, 
   3.627307981926037*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"z", "\[Rule]", "x"}], "}"}], "}"}]], "Output",
 CellChangeTimes->{
  3.6273060277645087`*^9, 3.6273060879262147`*^9, {3.627307268545062*^9, 
   3.6273073000299177`*^9}, {3.6273074036875*^9, 3.6273074276375427`*^9}, 
   3.627307981926037*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"circularcone", "[", 
   RowBox[{"1", ",", "1"}], "]"}], "[", 
  RowBox[{"0", ",", "1"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "0", ",", "1"}], "}"}]], "Output",
 CellChangeTimes->{3.6273060277957087`*^9, 3.6273061278340855`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "x"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "3"}], ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", "3"}], ",", "3"}], "}"}], ",", " ", 
   RowBox[{"PlotStyle", " ", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{"Green", ",", 
      RowBox[{"Opacity", "[", ".4", "]"}]}], "}"}]}], ",", 
   RowBox[{"Mesh", "\[Rule]", " ", "None"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.614947511224475*^9, 3.6149475365277195`*^9}, {
   3.614947595277423*^9, 3.6149476982688036`*^9}, {3.6149477572213078`*^9, 
   3.6149477876569605`*^9}, 3.614948425947682*^9, {3.614948471546562*^9, 
   3.6149484829345818`*^9}, 3.6149487622530727`*^9, {3.61494890886213*^9, 
   3.6149489102037325`*^9}, {3.61761845439627*^9, 3.6176184585475073`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"teb", "=", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.6149478113222027`*^9, 3.614947819091016*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Show", "[", 
   RowBox[{"Graphics3D", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"PointSize", "[", ".02", "]"}], ",", 
      RowBox[{"Point", "[", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0", ",", "1"}], "}"}], "]"}]}], "}"}], "]"}], 
   "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.6149478527402754`*^9, 3.614947992469721*^9}, {
   3.614948095773102*^9, 3.6149481286267595`*^9}, 3.61761836961142*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"cpt", " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.614948138938378*^9, 3.614948144850788*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", "t"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", 
     RowBox[{"-", "3"}], ",", "3"}], "}"}], ",", 
   RowBox[{"PlotStyle", " ", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{"Blue", ",", " ", "Thick"}], "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.614948558157914*^9, 3.614948571230737*^9}, {
  3.6149486018379908`*^9, 3.6149486688713083`*^9}, {3.614948926302961*^9, 
  3.6149489275353627`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"cli", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.6149486808833294`*^9, 3.614948690212146*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"circularcone", "[", 
     RowBox[{"1", ",", "1"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "3"}], ",", "3"}], "}"}], ",", " ", 
   RowBox[{"Mesh", "\[Rule]", " ", "None"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.614948162057618*^9, 3.61494827136701*^9}, {
  3.6149489409513865`*^9, 3.614948942246189*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"co", " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.6149483045014687`*^9, 3.6149483106790795`*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"teb", ",", "cpt", ",", "co", ",", "cli"}], "}"}], ",", " ", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6149483147350864`*^9, 3.614948383780808*^9}, {
  3.6149487051413727`*^9, 3.614948706467375*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["1.2.2. Example: Plane", "Subsubsection",
 CellChangeTimes->{{3.587272440743189*^9, 3.5872724436063533`*^9}, 
   3.588661276850272*^9, 3.5995529828209066`*^9, 3.599566093190103*^9, 
   3.6176185715049677`*^9, 3.6176250871982107`*^9, 3.62730822193326*^9, 
   3.627308449269261*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"fpl", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", 
  RowBox[{"{", 
   RowBox[{"u", ",", "v", ",", 
    RowBox[{"u", "+", 
     RowBox[{"2", "*", "v"}]}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.553947791570472*^9, 3.5539477933488746`*^9}, {
  3.586001848219719*^9, 3.586001869981757*^9}, {3.5860020467612677`*^9, 
  3.586002150158249*^9}}],

Cell["\<\
For a plane the vectors of the moving frame can be chosen to be constant:\
\>", "Text",
 CellChangeTimes->{{3.572176177272483*^9, 3.5721762037599983`*^9}, {
   3.572176239688053*^9, 3.572176260632251*^9}, 3.572255951175156*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"amf", "[", "1", "]"}], "[", "fpl", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{
  3.553948967562937*^9, 3.5539502761680355`*^9, {3.586002788948171*^9, 
   3.586002790726574*^9}, 3.6148618491295214`*^9, 3.6149178248366194`*^9, 
   3.6149181005671034`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox["1", 
    SqrtBox["2"]], ",", "0", ",", 
   FractionBox["1", 
    SqrtBox["2"]]}], "}"}]], "Output",
 CellChangeTimes->{3.6273083501794863`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"amf", "[", "2", "]"}], "[", "fpl", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{3.553950304404085*^9, 3.586002826544237*^9, 
  3.6148618491295214`*^9, 3.6149178248366194`*^9, 3.6149181022207065`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", 
     SqrtBox["3"]]}], ",", 
   FractionBox["1", 
    SqrtBox["3"]], ",", 
   FractionBox["1", 
    SqrtBox["3"]]}], "}"}]], "Output",
 CellChangeTimes->{3.627308353876693*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"amf", "[", "3", "]"}], "[", "fpl", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{3.5539489906821775`*^9, 3.5539503233269186`*^9, 
  3.58600282829144*^9, 3.614861849145121*^9, 3.614917824852219*^9, 
  3.614918103531109*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", 
     SqrtBox["6"]]}], ",", 
   RowBox[{"-", 
    SqrtBox[
     FractionBox["2", "3"]]}], ",", 
   FractionBox["1", 
    SqrtBox["6"]]}], "}"}]], "Output",
 CellChangeTimes->{3.627308357279499*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "i", "]"}], "[", "fpl", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ".", 
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "j", "]"}], "[", "fpl", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "3"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.553947825625331*^9, 3.5539478262649326`*^9}, {
   3.5539479129386845`*^9, 3.5539479129386845`*^9}, {3.5539480114528575`*^9, 
   3.5539480249468813`*^9}, {3.5539490104630127`*^9, 
   3.5539490713811197`*^9}, {3.5539503233425183`*^9, 
   3.5539503233425183`*^9}, {3.5721759831323786`*^9, 3.5721760687492757`*^9}, 
   3.5721761324919214`*^9, {3.5860028308186445`*^9, 3.5860028329246483`*^9}, {
   3.6148618491607213`*^9, 3.614861849176321*^9}, 3.614863501344023*^9, {
   3.6149178248678193`*^9, 3.6149178248678193`*^9}, {3.6149181080083165`*^9, 
   3.614918109693119*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "1"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6273083622891073`*^9}]
}, Closed]],

Cell["As we see, the vectors of this moving frame are constant:", "Text",
 CellChangeTimes->{{3.586003209961711*^9, 3.5860032385721607`*^9}, {
  3.5995601670382357`*^9, 3.599560167205245*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "i", "]"}], "[", "fpl", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3"}], "}"}]}], "]"}], ",", "u"}], "]"}]], "Input",
 CellChangeTimes->{{3.5860029528576593`*^9, 3.586003058766245*^9}, {
   3.614861849191921*^9, 3.614861849191921*^9}, 3.614863501359623*^9, 
   3.614917824883419*^9, 3.614918116510331*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6273083690283194`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "i", "]"}], "[", "fpl", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3"}], "}"}]}], "]"}], ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5860029528576593`*^9, 3.5860030749746733`*^9}, {
   3.6148618492075214`*^9, 3.6148618492075214`*^9}, 3.614863501375223*^9, 
   3.6149178248990192`*^9, 3.6149181233275433`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627308396847169*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["1.2.3. Metric Invariants. First Fundamental Form", "Subsubsection",
 CellChangeTimes->{{3.5860653550319138`*^9, 3.586065364922331*^9}, {
   3.5872724489986615`*^9, 3.587272452765877*^9}, {3.5886612605950437`*^9, 
   3.588661268863058*^9}, 3.5886740069241586`*^9, {3.5902137324227285`*^9, 
   3.590213746431553*^9}, 3.6176250938625917`*^9, 3.617693466379898*^9, 
   3.6273084563692737`*^9, 3.627309886928396*^9}],

Cell["Let a smooth curve in the parameter space be given:", "Text",
 CellChangeTimes->{{3.5860654539672875`*^9, 3.5860654965397625`*^9}}],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"\[DoubledGamma]", "[", "t_", "]"}], ":=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
     RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "}"}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5860655187074013`*^9, 3.586065549002654*^9}, {
   3.5860657022885237`*^9, 3.5860657414289923`*^9}, {3.5860658041255026`*^9, 
   3.5860658268391423`*^9}, 3.5890873523715672`*^9, 3.589088281233695*^9}],

Cell["The corresponding curve  on the surface is", "Text",
 CellChangeTimes->{{3.586066192568585*^9, 3.586066201913001*^9}, {
  3.589006059047471*^9, 3.5890060703111153`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";", 
  RowBox[{
   RowBox[{"sf\[DoubledGamma]", "[", "t_", "]"}], " ", ":=", 
   RowBox[{"sf", "[", 
    RowBox[{
     RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
     RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.58606588789765*^9, 3.586065894434061*^9}, {
  3.586066107280835*^9, 3.5860661590597258`*^9}, {3.5860670057980137`*^9, 
  3.586067018200035*^9}, {3.589087429064954*^9, 3.589087468249195*^9}, {
  3.617625177320366*^9, 3.6176251841517563`*^9}}],

Cell["Here is its tangent vector", "Text",
 CellChangeTimes->{{3.5860662109454174`*^9, 3.586066227918247*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"sf", "[", 
    RowBox[{
     RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
     RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ",", "t"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5860662523010902`*^9, 3.586066287057951*^9}, 
   3.5890874822809973`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["\[DoubledGamma]2", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
       RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "+", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[DoubledGamma]1", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
       RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["\[DoubledGamma]2", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
       RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "+", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[DoubledGamma]1", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
       RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["\[DoubledGamma]2", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
       RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "+", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[DoubledGamma]1", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
       RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}]}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.6273084864345264`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", "-", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", "1", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{
      RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
      RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], "*", 
    RowBox[{
     SuperscriptBox["\[DoubledGamma]1", "\[Prime]",
      MultilineFunction->None], "[", "t", "]"}]}], " ", "-", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", "2", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{
      RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
      RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], "*", 
    RowBox[{
     SuperscriptBox["\[DoubledGamma]2", "\[Prime]",
      MultilineFunction->None], "[", "t", "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5860663716724997`*^9, 3.586066533226383*^9}, {
   3.5860665822728696`*^9, 3.586066582912471*^9}, {3.586066701659879*^9, 
   3.5860667132038994`*^9}, {3.58606687879819*^9, 3.5860669453011074`*^9}, {
   3.589087499328973*^9, 3.589087507184422*^9}, 3.589087578555504*^9, 
   3.589087612347437*^9, {3.614163211576773*^9, 3.614163233042411*^9}, 
   3.6141642723006363`*^9, 3.6176000240258875`*^9, 3.617600085739596*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.627308495232942*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%%", " ", "\[Equal]", "  ", 
  RowBox[{
   RowBox[{
    RowBox[{"tangsp", "[", "sf", "]"}], "[", 
    RowBox[{
     RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
     RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], "[", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[DoubledGamma]1", "\[Prime]",
      MultilineFunction->None], "[", "t", "]"}], ",", 
    RowBox[{
     SuperscriptBox["\[DoubledGamma]2", "\[Prime]",
      MultilineFunction->None], "[", "t", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5860679659785004`*^9, 3.5860680404842315`*^9}, 
   3.5890875161849365`*^9, 3.5890875882410583`*^9, 3.5890876187208014`*^9, 
   3.6227808564211283`*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6273085138145747`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "tangsp"}]], "Input",
 CellChangeTimes->{{3.627308563288262*^9, 3.6273085679838705`*^9}}],

Cell[BoxData[
 StyleBox["\<\"tangsp[f][u,...][x,...] is the tangent vector with standard \
\\ncoordinates x,... of the immersion f[u,...] at the point {u,...}.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.6273085691850724`*^9},
 CellTags->"Info3627312169-3465247"]
}, Closed]],

Cell["\<\
Clearly, this is a tangent vector of the surface. The metric properties: the \
arc length of the curve, intersection angles, the area  etc. are defined by \
the scalar products of  tangent vectors at the variable point {u,v} of the \
surface:\
\>", "Text",
 CellChangeTimes->{{3.586068142992011*^9, 3.586068177561672*^9}, {
   3.5860682279341607`*^9, 3.586068343514764*^9}, 3.586151985007841*^9, 
   3.5995528399297333`*^9, 3.61416242396259*^9, {3.614949815192522*^9, 
   3.6149498664854116`*^9}, 3.61762524523025*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "3"}], ";", 
  RowBox[{"ast", " ", "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"tangsp", "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", 
      RowBox[{"x1", ",", "x2"}], "]"}], ".", 
     RowBox[{
      RowBox[{
       RowBox[{"tangsp", "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", 
      RowBox[{"y1", ",", "y2"}], "]"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5860683560727854`*^9, 3.5860684322165194`*^9}, {
   3.5860685573443394`*^9, 3.5860685599183435`*^9}, 3.5886615418167377`*^9, {
   3.589001060562574*^9, 3.5890010784986*^9}, 3.599565556008378*^9, 
   3.599565651376833*^9, 3.6149498906498547`*^9, {3.617683238320698*^9, 
   3.61768324505991*^9}, {3.6227808658279448`*^9, 3.622780867075947*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"x2", " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "+", 
     RowBox[{"x1", " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"y2", " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "+", 
     RowBox[{"y1", " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"x2", " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "+", 
     RowBox[{"x1", " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"y2", " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "+", 
     RowBox[{"y1", " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"x2", " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "+", 
     RowBox[{"x1", " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"y2", " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "+", 
     RowBox[{"y1", " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.6273086112195463`*^9}]
}, Closed]],

Cell["\<\
This scalar product is the Euclidean scalar product restricted to the tangent \
spaces of the surface,that is a symmetric positive definite bilinear form \
whose coefficient matrix is the matrix of the scalar products of the standard \
tangent basis vectors depending on the points {u,v}. In general, they are \
neither normed nor orthogonal:\
\>", "Text",
 CellChangeTimes->{{3.617684210639206*^9, 3.617684599111088*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"rgT", "[", 
      RowBox[{"i_", ",", "j_"}], "]"}], "[", "f_", "]"}], "[", "u__", "]"}], 
   " ", ":=", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"IntegerQ", "[", "i", "]"}], "&&", 
        RowBox[{"i", ">", "0"}], "&&", 
        RowBox[{"i", "\[LessEqual]", 
         RowBox[{"Length", "[", 
          RowBox[{"{", "u", "}"}], "]"}]}]}], ")"}], "&&", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"IntegerQ", "[", "j", "]"}], "&&", 
        RowBox[{"j", ">", "0"}], "&&", 
        RowBox[{"j", "\[LessEqual]", 
         RowBox[{"Length", "[", 
          RowBox[{"{", "u", "}"}], "]"}]}]}], ")"}]}], ",", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"Head", "[", 
          RowBox[{"f", "[", "u", "]"}], "]"}], "\[Equal]", " ", "List"}], "&&",
         " ", 
        RowBox[{
         RowBox[{"Length", "[", 
          RowBox[{"f", "[", "u", "]"}], "]"}], ">", "1"}]}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"tv", "[", "i", "]"}], "[", "f", "]"}], "[", "u", "]"}], 
        ".", 
        RowBox[{
         RowBox[{
          RowBox[{"tv", "[", "j", "]"}], "[", "f", "]"}], "[", "u", "]"}]}]}],
       "]"}], ",", " ", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Wrong {i,j} = \>\"", ",", " ", 
       RowBox[{"{", 
        RowBox[{"i", ",", "j"}], "}"}]}], "]"}]}], "]"}]}], ";"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.586068574332769*^9, 3.5860686078104277`*^9}, {
   3.5860686390572824`*^9, 3.5860687084618044`*^9}, 3.6149501405622935`*^9, 
   3.6149504057159595`*^9, {3.6176000240414877`*^9, 3.6176000240414877`*^9}, 
   3.617625895736457*^9, {3.617683366818124*^9, 3.617683440606253*^9}, {
   3.617692768223966*^9, 3.6176927691900215`*^9}, {3.618028941934537*^9, 
   3.618028984023411*^9}, {3.6180299968551903`*^9, 3.6180300191164293`*^9}, {
   3.61803007644653*^9, 3.6180301031225767`*^9}, 3.6227214881774344`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "rgT "}]], "Input",
 CellChangeTimes->{{3.6176846574395905`*^9, 3.6176847979646373`*^9}, {
   3.617684835311103*^9, 3.6176851458296485`*^9}, {3.61768518668612*^9, 
   3.6176852141889687`*^9}, 3.617685283999091*^9, {3.622721531670311*^9, 
   3.6227215418883286`*^9}}],

Cell[BoxData[
 StyleBox["\<\"\\\"For the immersion f of an m-dimensional manifold \
\\!\\(\\*SuperscriptBox[\\(X\\), \\(m\\)]\\) into the \\\\nn-dimensional \
Euclidean space, m<=n, the matrix (rgT[i,j][f][u1,...,um]), 1<=i,j<=m, \
\\\\nis the matrix of the coefficients of the Riemannian metric on \
\\!\\(\\*SuperscriptBox[\\(X\\), \\(m\\)]\\) induced by the \\\\nimmersion f.\
\\\"\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.6273087457525835`*^9},
 CellTags->"Info3627312345-3465247"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"rgT", "[", 
     RowBox[{"1", ",", "1"}], "]"}], "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], " "}]], "Input",
 CellChangeTimes->{3.6176835523960495`*^9}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "1", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
  SuperscriptBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "2", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
  SuperscriptBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "3", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "2"]}]], "Output",
 CellChangeTimes->{3.6273090499131203`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"rgT", "[", 
     RowBox[{"1", ",", "2"}], "]"}], "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], " "}]], "Input",
 CellChangeTimes->{
  3.6176835523960495`*^9, {3.617684097336207*^9, 3.617684111657032*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "1", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "1", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "2", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "2", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "3", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "3", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}]}]], "Output",
 CellChangeTimes->{3.6273090537507267`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"rgT", "[", 
     RowBox[{"1", ",", "3"}], "]"}], "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], " "}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.586068574332769*^9, 3.5860686078104277`*^9}, {
   3.5860686390572824`*^9, 3.5860687337962494`*^9}, 3.6149501434326982`*^9, 
   3.6149504087735643`*^9, 3.6176000240414877`*^9, 3.617600085739596*^9, 
   3.617625897056532*^9, {3.617683457079882*^9, 3.6176834844579306`*^9}, 
   3.6176840514253263`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Wrong {i,j} = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3"}], "}"}]}],
  SequenceForm["Wrong {i,j} = ", {1, 3}],
  Editable->False]], "Print",
 CellChangeTimes->{3.627712282730896*^9, 3.6277411679054527`*^9, 
  3.627809367352916*^9, 3.6278822739538455`*^9, 3.627897514751646*^9, 
  3.6279734212957134`*^9, 3.6279906329662437`*^9, 3.628051264306637*^9, 
  3.62805968615411*^9, 3.6280653017668467`*^9, 3.6280821919517035`*^9, 
  3.628142661884572*^9, 3.628164759229244*^9, 3.628228445950006*^9, 
  3.6282505024878225`*^9, 3.6282520575645666`*^9, 3.628311481707261*^9, 
  3.6283998614683523`*^9, 3.628773940294592*^9, 3.628835216293781*^9, 
  3.628926586897953*^9, 3.62899900040489*^9, 3.6290103675656805`*^9, 
  3.6290968214569707`*^9, 3.6291080585467076`*^9, 3.6291179552109175`*^9, 
  3.629174666166565*^9, 3.6291867735420227`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"rgT", "[", 
     RowBox[{"1", ",", "2"}], "]"}], "[", "sf", "]"}], "[", 
   RowBox[{"0", ",", "0"}], "]"}], " "}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.586068574332769*^9, 3.5860686078104277`*^9}, {
   3.5860686390572824`*^9, 3.5860687337962494`*^9}, 3.6149501434326982`*^9, 
   3.6149504087735643`*^9, 3.6176000240414877`*^9, 3.617600085739596*^9, 
   3.617625897056532*^9, {3.617683457079882*^9, 3.6176834844579306`*^9}, 
   3.6176840514253263`*^9, {3.6273091347082696`*^9, 3.6273091545057044`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "1", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"0", ",", "0"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "1", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"0", ",", "0"}], "]"}]}], "+", 
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "2", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"0", ",", "0"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "2", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"0", ",", "0"}], "]"}]}], "+", 
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "3", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"0", ",", "0"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "3", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"0", ",", "0"}], "]"}]}]}]], "Output",
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmVkYGBQA2LBiTN4utleO764f59hJZA2b9dsPw2k53jsDPwJpAM2
/T6rwf7a8cGCg6xaQNombF4RiP62+clzbyA9x8H9tC+QDpGuve0PpL0fhz8G
0c6Vf//GAulHp1571QPpzaz2TxYBaY/eiNBVQLqmbGXqeSDtki1/9jqQPuLP
lfEVxM872vITSJ/l3On0B0h/Kv7fyMLx2tHs3B4OJSB95OrzSzpAmitt/2tv
EC1zYUMYkN45wag4HEhXLDQvLwHS+/6smzcfSD/J6XIL43ztuDyN06gESF+x
jj2/AEjf+cKjfQRIL5H8vP0kkG6cv+7dZyCd8y8w+yeQXmSf7vAPSPtccf0o
yfXakefGxlvyQBoA+WqUeg==
  "]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"rgT", "[", 
      RowBox[{"1", ",", "2"}], "]"}], "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], " ", "\[Equal]", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"rgT", "[", 
      RowBox[{"2", ",", "1"}], "]"}], "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5886613827276583`*^9, 3.5886614689802094`*^9}, {
  3.588661500523465*^9, 3.588661516326293*^9}, {3.61762590209282*^9, 
  3.6176259031958833`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6273090779815693`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", " ", 
  RowBox[{"ast", " ", "\[Equal]", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"rgT", "[", 
        RowBox[{"1", ",", "1"}], "]"}], "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "*", "x1", "*", "y1"}], " ", "+", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"rgT", "[", 
        RowBox[{"1", ",", "2"}], "]"}], "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "*", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"x1", "*", "y2"}], " ", "+", 
       RowBox[{"x2", "*", "y1"}]}], ")"}]}], " ", "+", "  ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"rgT", "[", 
        RowBox[{"2", ",", "2"}], "]"}], "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "*", "x2", "*", "y2"}]}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5860688561316643`*^9, 3.5860690418811903`*^9}, {
  3.5860690841260643`*^9, 3.586069106621304*^9}, {3.5890011430442915`*^9, 
  3.5890011879328594`*^9}, {3.617625905967042*^9, 3.617625909573248*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6273090949855995`*^9}]
}, Closed]],

Cell[TextData[{
 "The coefficients rgT[i,j][f][u,v] are the coordinates of a positive \
definite, symmetric bilinear form depending on the point {u,v}  of the \
parameter manifold U. It is the",
 StyleBox[" first",
  FontColor->RGBColor[0, 0, 1]],
 " or ",
 StyleBox["metric fundamental form ",
  FontColor->RGBColor[0, 0, 1]],
 Cell[BoxData[
  FormBox["fund1", TraditionalForm]],
  CellChangeTimes->{{3.5861529045544357`*^9, 3.5861530716929955`*^9}, 
    3.5861532850852013`*^9, {3.5861533247654705`*^9, 3.586153326756584*^9}, 
    3.586153440461088*^9, 3.5861536097777724`*^9}],
 " of a 2-dimensional Riemannian geometry, defining the",
 StyleBox[" inner geometry",
  FontColor->RGBColor[0, 0, 1]],
 " of the surface covering all its properties being invariant under \
isometries. A map between two surfaces is called an ",
 StyleBox["isometry",
  FontColor->RGBColor[0, 0, 1]],
 " if it preserves the arc length of  curves (see the next subsection). \nThe \
classical notation is ",
 Cell[BoxData[
  FormBox["fund1", TraditionalForm]]],
 " = ",
 Cell[BoxData[
  SuperscriptBox["ds", "2"]]],
 ", what may not be reproduced in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " formulas, since the expression  is always interpreted as a power."
}], "Text",
 CellChangeTimes->{{3.586152357579151*^9, 3.586152604679284*^9}, {
   3.5861526839358172`*^9, 3.5861527425681705`*^9}, {3.586152808759957*^9, 
   3.5861528462721024`*^9}, 3.5861533692320137`*^9, {3.586153400847822*^9, 
   3.586153548808285*^9}, 3.58615358683146*^9, {3.5861541794643564`*^9, 
   3.586154180671426*^9}, {3.586154312279953*^9, 3.5861543492690687`*^9}, {
   3.5885676557649*^9, 3.5885677678883133`*^9}, {3.5885680164035273`*^9, 
   3.5885680453391824`*^9}, {3.588661586308016*^9, 3.588661586822817*^9}, {
   3.588661624746483*^9, 3.5886616346525006`*^9}, {3.5886618514772816`*^9, 
   3.588661868418911*^9}, 3.589191183132166*^9, {3.5904957103906093`*^9, 
   3.590495710406209*^9}, 3.5995529202423267`*^9, 3.614950556365424*^9, 
   3.6176259128434353`*^9, 3.6176854156633224`*^9, {3.6176854904342537`*^9, 
   3.6176854977662663`*^9}},
 CellTags->"g(i"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     FormBox["fund1",
      TraditionalForm], "[", "f_", "]"}], "[", "u__", "]"}], "[", 
   RowBox[{"x_List", ",", "y_List"}], "]"}], ":=", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Length", "[", "x", "]"}], " ", "\[Equal]", " ", 
      RowBox[{"Length", "[", 
       RowBox[{"{", "u", "}"}], "]"}]}], "&&", 
     RowBox[{
      RowBox[{"Length", "[", "y", "]"}], " ", "\[Equal]", "  ", 
      RowBox[{"Length", "[", 
       RowBox[{"{", "u", "}"}], "]"}]}]}], ",", 
    RowBox[{"Sum", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"rgT", "[", 
          RowBox[{"i", ",", "j"}], "]"}], "[", "f", "]"}], "[", "u", "]"}], 
       "*", 
       RowBox[{"x", "[", 
        RowBox[{"[", "i", "]"}], "]"}], "*", 
       RowBox[{"y", "[", 
        RowBox[{"[", "j", "]"}], "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", 
        RowBox[{"Length", "[", 
         RowBox[{"{", "u", "}"}], "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", 
        RowBox[{"Length", "[", 
         RowBox[{"{", "u", "}"}], "]"}]}], "}"}]}], "]"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Print", "[", "\"\<Wrong arguments!\>\"", "]"}], ";", 
     RowBox[{"Return", "[", "]"}]}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5861529045544357`*^9, 3.5861530716929955`*^9}, 
   3.5861532850852013`*^9, {3.5861533247654705`*^9, 3.586153326756584*^9}, 
   3.586153440461088*^9, 3.5861536097777724`*^9, {3.5890012608770313`*^9, 
   3.5890012796371045`*^9}, 3.590495710406209*^9, 3.6176259155025873`*^9, {
   3.618028607001949*^9, 3.6180286476868205`*^9}, 3.6180288206443243`*^9, {
   3.618029334727627*^9, 3.6180293682988863`*^9}, {3.618029401433344*^9, 
   3.6180294186401744`*^9}, 3.6180295739228473`*^9, {3.6180296220645313`*^9, 
   3.6180296248413363`*^9}, {3.618029689940251*^9, 3.618029832680502*^9}, {
   3.618030383548669*^9, 3.6180303996322975`*^9}, {3.6180311010875297`*^9, 
   3.61803112412877*^9}, {3.6180313060562897`*^9, 3.618031380031619*^9}, {
   3.618031423196895*^9, 3.6180314644277678`*^9}, {3.618031496064623*^9, 
   3.61803157401796*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "fund1"}]], "Input",
 CellChangeTimes->{{3.6180284552448826`*^9, 3.6180285191581945`*^9}, {
   3.6180286638796487`*^9, 3.6180287896938696`*^9}, 3.618030489800456*^9, {
   3.622698848681225*^9, 3.6226988614732475`*^9}}],

Cell[BoxData[
 StyleBox["\<\"fund1[f][u1,..,um][x,y] is the value of the first fundamental \
form \\nof the immersion f[u1,...,um] into the n_dimensional Euclidean space, \
\\nfor the tangent vectors x,y of the immersion.\"\>", "MSG"]], "Print", \
"PrintUsage",
 CellChangeTimes->{3.6273092551056814`*^9},
 CellTags->"Info3627312854-3465247"]
}, Closed]],

Cell[TextData[StyleBox["Example. The spheres:",
 FontColor->RGBColor[0, 0, 1]]], "Text",
 CellChangeTimes->{{3.5861531338965535`*^9, 3.586153139327864*^9}, {
  3.5886616891901965`*^9, 3.5886617024658194`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "sphero"}]], "Input",
 CellChangeTimes->{{3.6273093145643864`*^9, 3.627309318869994*^9}}],

Cell[BoxData[
 StyleBox["\<\"{u,v}->sphero[a][u,v] is the standard parametrization of a \
sphere of radius a centered at the origin o.  To plot try \
sphero[1][0,3Pi/2,-Pi/2,Pi/2].\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.627309321209998*^9},
 CellTags->"Info3627312921-3465247"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     FormBox["fund1",
      TraditionalForm], "[", 
     RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"x1", ",", "x2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"y1", ",", "y2"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5861531444481573`*^9, 3.5861532434508195`*^9}, {
   3.586153274283583*^9, 3.5861532782268085`*^9}, {3.5861536173652067`*^9, 
   3.586153627083762*^9}, {3.586153777276353*^9, 3.5861537821146297`*^9}, 
   3.5861545793562293`*^9, 3.590495710406209*^9, {3.6176856123328676`*^9, 
   3.617685662018955*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], " ", 
  SuperscriptBox["r", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"x1", " ", "y1"}], "+", 
    RowBox[{"2", " ", "x2", " ", "y2"}], "+", 
    RowBox[{"x1", " ", "y1", " ", 
     RowBox[{"Cos", "[", 
      RowBox[{"2", " ", "v"}], "]"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.627309342413436*^9}]
}, Closed]],

Cell["Another commonly used simplification is", "Text",
 CellChangeTimes->{{3.5861541213060303`*^9, 3.586154139401065*^9}, 
   3.5861543977998447`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      FormBox["fund1",
       TraditionalForm], "[", 
      RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"x1", ",", "x2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"y1", ",", "y2"}], "}"}]}], "]"}], "\[Equal]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"r", "^", "2"}], ")"}], "*", 
    RowBox[{"(", " ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"Cos", "[", "v", "]"}], ")"}], "^", "2"}], "*", "x1", "*", 
       "y1"}], "+", 
      RowBox[{"x2", "*", "y2"}]}], " ", ")"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5861537211471424`*^9, 3.5861537348419256`*^9}, {
   3.5861539275459476`*^9, 3.586154063466722*^9}, {3.586154468125867*^9, 
   3.5861544917242165`*^9}, {3.586154603805628*^9, 3.58615467517371*^9}, 
   3.590495710406209*^9, {3.61768585274489*^9, 3.6176859259714184`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.627309387518115*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FullSimplify", "[", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"rgT", "[", 
        RowBox[{"i", ",", "j"}], "]"}], "[", 
       RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.617685983083119*^9, 3.617686136446988*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       SuperscriptBox["r", "2"], " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", "v", "]"}], "2"]}], "0"},
     {"0", 
      SuperscriptBox["r", "2"]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.6273093982373343`*^9}]
}, Closed]],

Cell["\<\
To define the area of a surface we consider the Gram determinant of two \
vectors:\
\>", "Text",
 CellChangeTimes->{{3.5722510364627957`*^9, 3.5722510654008465`*^9}, {
  3.586154782192831*^9, 3.586154814847698*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"x1", ",", "x2"}], "]"}], ";", 
  RowBox[{
   RowBox[{"gram", "[", 
    RowBox[{"x1_", ",", "x2_"}], "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x1", ".", "x1"}], ")"}], "*", 
     RowBox[{"(", 
      RowBox[{"x2", ".", "x2"}], ")"}]}], "-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x1", ".", "x2"}], ")"}], "^", "2"}]}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.586069943593974*^9, 3.586069984809246*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"gram", "::", "usage"}], " ", "=", " ", 
   "\"\<gram[vx,vy] is the Gram determinant of two vectors vx,vy  in the \
n-dimensional Euclidean Space.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6149507206961126`*^9, 3.6149508409411235`*^9}, 
   3.61495087285878*^9}],

Cell["\<\
In Euclidean geometry this is the square of the area of the parallelogram \
spanned by the vectors x1,  x2:\
\>", "Text",
 CellChangeTimes->{{3.572251109876525*^9, 3.57225117556824*^9}, 
   3.586069358592946*^9, {3.5861548724479933`*^9, 3.5861549045198274`*^9}, 
   3.586156372890814*^9, {3.5890885163591433`*^9, 3.589088550846116*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"Det", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"vec", "[", 
         RowBox[{"x", ",", "2"}], "]"}], ",", 
        RowBox[{"vec", "[", 
         RowBox[{"y", ",", "2"}], "]"}]}], "}"}], "]"}], ")"}], "^", "2"}], 
   " ", "\[Equal]", " ", 
   RowBox[{"gram", "[", 
    RowBox[{
     RowBox[{"vec", "[", 
      RowBox[{"x", ",", "2"}], "]"}], ",", 
     RowBox[{"vec", "[", 
      RowBox[{"y", ",", "2"}], "]"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.572087475918888*^9, 3.5720876163876476`*^9}, {
  3.5720876693809404`*^9, 3.5720877258266935`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6273094343407974`*^9}]
}, Closed]],

Cell["\<\
   Thus the square of  the area of a parallelogram in the tangent plane of \
the surface is defined by\
\>", "Text",
 CellChangeTimes->{{3.572251109876525*^9, 3.57225117556824*^9}, 
   3.586069358592946*^9, {3.5861548724479933`*^9, 3.5861549045198274`*^9}, 
   3.586156372890814*^9, {3.586156413240121*^9, 3.5861564361274304`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dst", "=", " ", 
  RowBox[{"Simplify", "[", 
   RowBox[{"gram", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"tangsp", "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", 
      RowBox[{"s1", ",", "s2"}], "]"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"tangsp", "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", 
      RowBox[{"t1", ",", "t2"}], "]"}]}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.586069422007058*^9, 3.58606947440755*^9}, {
  3.622780870398752*^9, 3.6227808730039573`*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     RowBox[{"s2", " ", "t1"}], "-", 
     RowBox[{"s1", " ", "t2"}]}], ")"}], "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "-", 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.6273094495664244`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"dst", "\[Equal]", "  ", 
   RowBox[{
    RowBox[{"Det", "[", 
     RowBox[{"Table", "[", " ", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"rgT", "[", 
          RowBox[{"i", ",", "j"}], "]"}], "[", "sf", "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}], "*", 
    RowBox[{
     RowBox[{"Det", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"s1", ",", "s2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"t1", ",", "t2"}], "}"}]}], "}"}], "]"}], "^", "2"}]}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5860694992583933`*^9, 3.586069705116355*^9}, {
   3.5860697555356436`*^9, 3.586069761260854*^9}, 3.586070236627984*^9, {
   3.5860702801554737`*^9, 3.586070319856745*^9}, {3.586070594851474*^9, 
   3.5860706219870253`*^9}, 3.5860707138922825`*^9, 3.5860709654996734`*^9, {
   3.586071217676097*^9, 3.5860712227553873`*^9}, 3.6148618492387214`*^9, 
   3.6148635013908234`*^9, 3.6176259295093884`*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6273094586954403`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Det", "[", 
    RowBox[{"Table", "[", " ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"rgT", "[", 
         RowBox[{"i", ",", "j"}], "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}], "\[Equal]", " ", 
   RowBox[{"gram", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"tv", "[", "1", "]"}], "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"tv", "[", "2", "]"}], "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5860694992583933`*^9, 3.586069705116355*^9}, {
   3.5860697555356436`*^9, 3.586069761260854*^9}, 3.586070236627984*^9, {
   3.5860702801554737`*^9, 3.586070319856745*^9}, {3.586070594851474*^9, 
   3.5860706219870253`*^9}, 3.5860707138922825`*^9, 3.5860709654996734`*^9, {
   3.586071217676097*^9, 3.5860712227553873`*^9}, {3.5860748660092196`*^9, 
   3.5860749408935027`*^9}, 3.6148618492543216`*^9, 3.6148635014220233`*^9, 
   3.617600024057088*^9, 3.617600085755196*^9, 3.617625931341493*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6273094650134516`*^9}]
}, Closed]],

Cell[TextData[{
 "We define in eudiffgeov2.m as  the ",
 StyleBox["area element",
  FontColor->RGBColor[0, 0, 1]],
 "  the  ",
 StyleBox["integrand of the",
  FontColor->RGBColor[0, 0, 1]],
 " ",
 StyleBox["area integral",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.586074957018425*^9, 3.5860749831289186`*^9}, {
   3.6149511390512924`*^9, 3.614951164738762*^9}, 3.6176862785320377`*^9, {
   3.6226170862645617`*^9, 3.6226171006789875`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"dA", "[", "f_", "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", 
  RowBox[{"Sqrt", "[", " ", 
   RowBox[{"Det", "[", 
    RowBox[{"Table", "[", " ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"rgT", "[", 
         RowBox[{"i", ",", "j"}], "]"}], "[", "f", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5721028398629103`*^9, 3.5721030771549273`*^9}, 
   3.5721031379326334`*^9, {3.586074746853404*^9, 3.5860747605491877`*^9}, {
   3.5860748158453503`*^9, 3.586074825492902*^9}, {3.5861549510774903`*^9, 
   3.586154964603264*^9}, {3.5862629631854534`*^9, 3.586262985017702*^9}, {
   3.5862630295142474`*^9, 3.5862630744648185`*^9}, {3.586604010786734*^9, 
   3.5866040355439777`*^9}, 3.6176000240726876`*^9, 3.617600085755196*^9, 
   3.617687181505224*^9, 3.6176873242766747`*^9, {3.617687380748774*^9, 
   3.6176874075964212`*^9}, 3.6226169373623004`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "dA"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6149512007068186`*^9, 3.6149512315495825`*^9}, {
   3.6149512646054735`*^9, 3.6149513092050247`*^9}, 3.6149513572857747`*^9, {
   3.6226169727119627`*^9, 3.6226169860343857`*^9}}],

Cell[BoxData[
 StyleBox["\<\"dA[f][u,v] is the area element of the surface f[u,v] in the \
\\nEuclidean n-space.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.629186773713623*^9},
 CellTags->"Info3629190373-5022241"]
}, Closed]],

Cell[TextData[{
 StyleBox["Remark.",
  FontColor->RGBColor[0, 0, 1]],
 " Since the first fundamental form is positively  definite its determinant \
is always positive. Therefore its non oriented integral is positive too. "
}], "Text",
 CellChangeTimes->{{3.617952911137521*^9, 3.6179529253933363`*^9}, {
   3.617953007017005*^9, 3.617953011633269*^9}, {3.6179530458652267`*^9, 
   3.6179531287369666`*^9}, {3.6180305569429736`*^9, 3.6180305659909897`*^9}, 
   3.627308098271442*^9, {3.627309524432357*^9, 3.6273095252747583`*^9}}],

Cell[TextData[{
 StyleBox["Example.",
  FontColor->RGBColor[0, 0, 1]],
 " Circular cone"
}], "Text",
 CellChangeTimes->{{3.6176877651178493`*^9, 3.617687777675871*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"dA", "[", 
    RowBox[{"circularcone", "[", 
     RowBox[{"a", ",", "b"}], "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5866039382466063`*^9, 3.5866039759206724`*^9}, {
  3.5866041540105853`*^9, 3.5866041641662035`*^9}, {3.6148653631292934`*^9, 
  3.6148653750945144`*^9}}],

Cell[BoxData[
 SqrtBox[
  RowBox[{
   SuperscriptBox["a", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["a", "2"], "+", 
     SuperscriptBox["b", "2"]}], ")"}], " ", 
   SuperscriptBox["v", "2"]}]]], "Output",
 CellChangeTimes->{3.6273095459135942`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"dA", "[", 
    RowBox[{"circularcone", "[", 
     RowBox[{"1", ",", "1"}], "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5866039382466063`*^9, 3.5866039759206724`*^9}, {
  3.5866041540105853`*^9, 3.5866041641662035`*^9}, {3.6148653631292934`*^9, 
  3.6148653750945144`*^9}, {3.6148663357515907`*^9, 3.614866346896228*^9}}],

Cell[BoxData[
 RowBox[{
  SqrtBox["2"], " ", 
  SqrtBox[
   SuperscriptBox["v", "2"]]}]], "Output",
 CellChangeTimes->{3.627309552749407*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate", "[", 
  RowBox[{
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"dA", "[", 
       RowBox[{"circularcone", "[", 
        RowBox[{"1", ",", "1"}], "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{"v", "\[GreaterEqual]", " ", "0"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", "h"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.589088773152831*^9, 3.589088850954281*^9}, {
   3.5890888926896687`*^9, 3.5890889221143513`*^9}, 3.6148618492855215`*^9, 
   3.614865446090239*^9, {3.614865726531932*^9, 3.614865741539158*^9}, {
   3.614866292275104*^9, 3.614866301169613*^9}, 3.6176875207278194`*^9}],

Cell[BoxData[
 RowBox[{
  SqrtBox["2"], " ", 
  SuperscriptBox["h", "2"], " ", "\[Pi]"}]], "Output",
 CellChangeTimes->{3.6273095612670217`*^9}]
}, Closed]],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.6176878245383534`*^9, 3.6176878245383534`*^9}}],

Cell[TextData[{
 "The integral over dA[sf][u,v] yields the ",
 StyleBox["area",
  FontColor->RGBColor[0, 0, 1]],
 " of the surface. Since a parameter transformation for the surface amounts \
to the multiplication of the integrand with the Jacobian determinant of the \
parameter transformation, the integral is independent of the choice of the \
parameter representation."
}], "Text",
 CellChangeTimes->{{3.5720898734695654`*^9, 3.572089907259225*^9}, {
  3.572105272437583*^9, 3.5721052835448027`*^9}, {3.572173734631772*^9, 
  3.5721738456001186`*^9}, {3.586074789744858*^9, 3.586074792722028*^9}, {
  3.586155042023692*^9, 3.5861551192241077`*^9}, {3.5861562113125715`*^9, 
  3.5861562665757327`*^9}, {3.5861563043758945`*^9, 3.5861563220399046`*^9}, {
  3.61769233645127*^9, 3.617692342075592*^9}}],

Cell[TextData[{
 StyleBox["Example",
  FontColor->RGBColor[0, 0, 1]],
 ". Spheres"
}], "Text",
 CellChangeTimes->{{3.6176878338827696`*^9, 3.6176878428683853`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"dA", "[", 
     RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
   RowBox[{"r", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6176878844580584`*^9, 3.617687952458578*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["r", "2"], " ", 
  SqrtBox[
   SuperscriptBox[
    RowBox[{"Cos", "[", "v", "]"}], "2"]]}]], "Output",
 CellChangeTimes->{3.6273095991956882`*^9}]
}, Closed]],

Cell["The area of the hemisphere of radius r is", "Text",
 CellChangeTimes->{{3.617691530792863*^9, 3.617691555612507*^9}, {
   3.6176918647894497`*^9, 3.617691867831455*^9}, 3.6176921682166476`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"Pi", "/", "2"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.617687965328601*^9, 3.617688027135909*^9}, 
   3.617691877129071*^9}],

Cell[BoxData[
 RowBox[{"2", " ", "\[Pi]", " ", 
  SuperscriptBox["r", "2"]}]], "Output",
 CellChangeTimes->{3.6273096048584986`*^9}]
}, Closed]],

Cell["\<\
Here is another parameter representation of the hemisphere of radius r:\
\>", "Text",
 CellChangeTimes->{{3.61769158015135*^9, 3.6176916295254364`*^9}, 
   3.6176921682456493`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"sphf", "[", "r_", "]"}], "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], " ", ":=", "  ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"x", "*", 
     RowBox[{"Cos", "[", "y", "]"}]}], ",", 
    RowBox[{"x", "*", 
     RowBox[{"Sin", "[", "y", "]"}]}], ",", 
    RowBox[{"Sqrt", "[", 
     RowBox[{
      RowBox[{"r", "^", "2"}], "-", 
      RowBox[{"x", "^", "2"}]}], "]"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.6176881577081385`*^9, 3.6176882004366136`*^9}, {
  3.617688240637884*^9, 3.617688321227626*^9}, {3.617688750072379*^9, 
  3.6176888089000826`*^9}, {3.61769068046657*^9, 3.6176907256286488`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sphf", "[", "2", "]"}], "[", 
    RowBox[{"x", ",", "y"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.61768835777849*^9, 3.617688479614704*^9}, {
   3.6176888188216996`*^9, 3.617688849678554*^9}, {3.617688879942607*^9, 
   3.61768888749302*^9}, 3.617690751774295*^9, {3.617690789963162*^9, 
   3.617690805313589*^9}}],

Cell["The area element is", "Text",
 CellChangeTimes->{{3.617691660475891*^9, 3.617691667573903*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"dA", "[", 
     RowBox[{"sphf", "[", "r", "]"}], "]"}], "[", 
    RowBox[{"x", ",", "y"}], "]"}], ",", " ", 
   RowBox[{"r", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6176890442264957`*^9, 3.6176890872513714`*^9}}],

Cell[BoxData[
 RowBox[{"r", " ", 
  SqrtBox[
   FractionBox[
    SuperscriptBox["x", "2"], 
    RowBox[{
     SuperscriptBox["r", "2"], "-", 
     SuperscriptBox["x", "2"]}]]]}]], "Output",
 CellChangeTimes->{3.6273096262033362`*^9}]
}, Closed]],

Cell["The area of the  hemisphere is", "Text",
 CellChangeTimes->{{3.617691697073555*^9, 3.617691713562784*^9}, 
   3.6176921682676506`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Integrate", "[", 
    RowBox[{"%", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0", ",", "r"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"y", ",", "0", ",", 
       RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}], ",", 
   RowBox[{"r", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6176891033193994`*^9, 3.6176891346130543`*^9}, {
   3.617689172599121*^9, 3.6176892293676205`*^9}, {3.617690863954092*^9, 
   3.617690886776932*^9}, {3.6176911210581436`*^9, 3.6176911442397842`*^9}, {
   3.6176913540913525`*^9, 3.617691383903005*^9}, 3.617691444025511*^9, {
   3.6176917597700653`*^9, 3.6176917974753313`*^9}}],

Cell[BoxData[
 RowBox[{"2", " ", "\[Pi]", " ", 
  SuperscriptBox["r", "2"]}]], "Output",
 CellChangeTimes->{3.627309631741346*^9}]
}, Closed]],

Cell["This equals the preceding result. \[FilledSquare]", "Text",
 CellChangeTimes->{{3.617691919342746*^9, 3.617691938468379*^9}, {
   3.6176922194665794`*^9, 3.6176922237468243`*^9}, {3.6176922937228265`*^9, 
   3.617692296446982*^9}, 3.6176925683365335`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["1.2.4. Isometries. Inner Geometry", "Subsubsection",
 CellChangeTimes->{{3.586059663992068*^9, 3.586059674943287*^9}, {
   3.5860750543689933`*^9, 3.58607505605709*^9}, {3.5866611656056275`*^9, 
   3.586661191821127*^9}, {3.5872724573591394`*^9, 3.5872724609903474`*^9}, {
   3.588661830058444*^9, 3.588661834145651*^9}, {3.5889467393286066`*^9, 
   3.58894674141901*^9}, 3.589200965451682*^9, {3.5892029279099283`*^9, 
   3.58920294017163*^9}, 3.5995661030936694`*^9, 3.5995680459337935`*^9, 
   3.614951511699607*^9, 3.615021851745468*^9, 3.6176934720992255`*^9, 
   3.617705065201067*^9, {3.6273099762505536`*^9, 3.6273099853609695`*^9}, 
   3.627372246455145*^9}],

Cell["Let sf[u, v] and sh[uu,vv] be two regular surfaces:", "Text",
 CellChangeTimes->{{3.58867436634879*^9, 3.588674446798131*^9}, {
  3.588674735336238*^9, 3.588674760608282*^9}}],

Cell[BoxData[
 RowBox[{"dim", " ", "=", " ", "3"}]], "Input",
 CellChangeTimes->{{3.617701255839184*^9, 3.6177012716340876`*^9}}],

Cell[BoxData[
 RowBox[{"sf", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5886741949824886`*^9, 3.5886742022053013`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sh", "[", 
   RowBox[{"uu_", ",", "vv_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"h", "[", "i", "]"}], "[", 
     RowBox[{"uu", ",", "vv"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "3"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.588674462132958*^9, 3.588674468450969*^9}, {
   3.5886746092880163`*^9, 3.588674654886897*^9}, 3.614861849316722*^9, 
   3.6148635014220233`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sh", "[", 
  RowBox[{"uu", ",", "vv"}], "]"}]], "Input",
 CellChangeTimes->{{3.5886746683497205`*^9, 3.588674708613391*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"h", "[", "1", "]"}], "[", 
    RowBox[{"uu", ",", "vv"}], "]"}], ",", 
   RowBox[{
    RowBox[{"h", "[", "2", "]"}], "[", 
    RowBox[{"uu", ",", "vv"}], "]"}], ",", 
   RowBox[{
    RowBox[{"h", "[", "3", "]"}], "[", 
    RowBox[{"uu", ",", "vv"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.6273705370509424`*^9, 3.6273705475029607`*^9}}]
}, Closed]],

Cell["\<\
A map f: sf \[RightArrow] sh  is defined by the map between the parameter \
regions:\
\>", "Text",
 CellChangeTimes->{{3.588674970288251*^9, 3.588675107786892*^9}, 
   3.5890024944555883`*^9, 3.6176267756617856`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"f", ":", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"u", ",", "v"}], "}"}], " ", "\[Element]", " ", 
    RowBox[{"U", " ", "\[RightArrow]", " ", 
     RowBox[{"f", "[", 
      RowBox[{"u", ",", "v"}], "]"}]}]}]}], " ", "=", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"f1", "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
     RowBox[{"f2", "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "}"}], "  ", "\[Element]", " ", 
   RowBox[{"U", " ", "U"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5886751171313086`*^9, 3.5886752965160236`*^9}, {
   3.5887663043718996`*^9, 3.5887663441987696`*^9}, 3.5887664615265756`*^9, {
   3.617701328276327*^9, 3.6177013359237647`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"f1", ",", "f2", ",", "f"}], "]"}], ";", " ", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"f1", "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
     RowBox[{"f2", "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "}"}]}]}]], "Input",
 CellChangeTimes->{{3.588766400343268*^9, 3.588766437346533*^9}, 
   3.588766471978594*^9, {3.589198575767*^9, 3.5891985917029114`*^9}}],

Cell[TextData[{
 "The functions f1,f2, like all functions and maps in differential geometry, \
are assumed to be continuously differentiable as often as necessary. \
Moreover, f is proposed to be bijective. Such bijective smooth maps are named \
",
 StyleBox["diffeomorphisms",
  FontColor->RGBColor[0, 0, 1]],
 ". The map"
}], "Text",
 CellChangeTimes->{{3.5886753470913124`*^9, 3.588675451190295*^9}, {
   3.5886754866335573`*^9, 3.5886755497668686`*^9}, 3.588675617580188*^9, {
   3.5886756774218926`*^9, 3.5886756818055005`*^9}, 3.5887661518192315`*^9, 
   3.588917638220236*^9, 3.5995674356568875`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"F", "[", 
   RowBox[{"sf", "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], "]"}], ":=", 
  RowBox[{"sh", "[", 
   RowBox[{
    RowBox[{"f1", "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{"f2", "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.588675689121913*^9, 3.5886758305517616`*^9}},
 CellTags->"ptr"],

Cell[TextData[{
 "is said to be an",
 StyleBox[" isometry",
  FontColor->RGBColor[0, 0, 1]],
 " if for any smooth curve sf[ \[DoubledGamma][t]]"
}], "Text",
 CellChangeTimes->{{3.5886761540027294`*^9, 3.5886763175846167`*^9}, 
   3.588766216465745*^9, {3.58900491683214*^9, 3.58900496927914*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"sf\[DoubledGamma]", "[", "t", "]"}]}]], "Input",
 CellChangeTimes->{{3.58867632485423*^9, 3.588676377051922*^9}, 
   3.5886764129163847`*^9, {3.5890048989381166`*^9, 3.589004906552552*^9}, {
   3.5890912050399275`*^9, 3.5890912313764334`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "1", "]"}], "[", 
    RowBox[{
     RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
     RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "2", "]"}], "[", 
    RowBox[{
     RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
     RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "3", "]"}], "[", 
    RowBox[{
     RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
     RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6273712744798374`*^9}]
}, Closed]],

Cell[" in sf its image under F,   namely", "Text",
 CellChangeTimes->{{3.5886764221204004`*^9, 3.5886764349436235`*^9}, {
  3.588676585171887*^9, 3.588676593330701*^9}, {3.588676743387365*^9, 
  3.588676793073452*^9}, {3.588676830435518*^9, 3.588676858265967*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"F\[DoubledGamma]", "[", "t_", "]"}], " ", "=", " ", 
  RowBox[{"sh", "[", 
   RowBox[{
    RowBox[{"f1", "[", 
     RowBox[{
      RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
      RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ",", " ", 
    RowBox[{"f2", "[", 
     RowBox[{
      RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
      RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5886764432116375`*^9, 3.588676487172515*^9}, {
  3.588676519776572*^9, 3.5886765561246357`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"h", "[", "1", "]"}], "[", 
    RowBox[{
     RowBox[{"f1", "[", 
      RowBox[{
       RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
       RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ",", 
     RowBox[{"f2", "[", 
      RowBox[{
       RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
       RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "]"}], ",", 
   RowBox[{
    RowBox[{"h", "[", "2", "]"}], "[", 
    RowBox[{
     RowBox[{"f1", "[", 
      RowBox[{
       RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
       RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ",", 
     RowBox[{"f2", "[", 
      RowBox[{
       RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
       RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "]"}], ",", 
   RowBox[{
    RowBox[{"h", "[", "3", "]"}], "[", 
    RowBox[{
     RowBox[{"f1", "[", 
      RowBox[{
       RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
       RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ",", 
     RowBox[{"f2", "[", 
      RowBox[{
       RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
       RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "]"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.627371282014651*^9}]
}, Closed]],

Cell["\<\
has the same arc length.  The tangent vector of the image curve is\
\>", "Text",
 CellChangeTimes->{{3.588676867469983*^9, 3.5886768700439873`*^9}, 
   3.5886770860263667`*^9, {3.5890026335835457`*^9, 3.5890026564708548`*^9}, 
   3.5995674706898913`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"F\[DoubledGamma]", "'"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.5886768793416033`*^9, 3.5886769396825094`*^9}, {
  3.5890062311063128`*^9, 3.5890062466492014`*^9}, {3.617703363419731*^9, 
  3.6177033783065825`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"h", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"f1", "[", 
        RowBox[{
         RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
         RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ",", 
       RowBox[{"f2", "[", 
        RowBox[{
         RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
         RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["\[DoubledGamma]2", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["f2", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox["\[DoubledGamma]1", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["f2", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], 
    "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["\[DoubledGamma]2", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["f1", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox["\[DoubledGamma]1", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["f1", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}]}], ")"}], 
     " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"h", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"f1", "[", 
        RowBox[{
         RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
         RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ",", 
       RowBox[{"f2", "[", 
        RowBox[{
         RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
         RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "]"}]}]}], 
   ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"h", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"f1", "[", 
        RowBox[{
         RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
         RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ",", 
       RowBox[{"f2", "[", 
        RowBox[{
         RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
         RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["\[DoubledGamma]2", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["f2", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox["\[DoubledGamma]1", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["f2", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], 
    "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["\[DoubledGamma]2", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["f1", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox["\[DoubledGamma]1", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["f1", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}]}], ")"}], 
     " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"h", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"f1", "[", 
        RowBox[{
         RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
         RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ",", 
       RowBox[{"f2", "[", 
        RowBox[{
         RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
         RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "]"}]}]}], 
   ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"h", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"f1", "[", 
        RowBox[{
         RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
         RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ",", 
       RowBox[{"f2", "[", 
        RowBox[{
         RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
         RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["\[DoubledGamma]2", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["f2", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox["\[DoubledGamma]1", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["f2", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], 
    "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["\[DoubledGamma]2", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["f1", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox["\[DoubledGamma]1", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["f1", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}]}], ")"}], 
     " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"h", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"f1", "[", 
        RowBox[{
         RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
         RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ",", 
       RowBox[{"f2", "[", 
        RowBox[{
         RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
         RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "]"}]}]}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.627371305196292*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", " ", "-", 
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", "sh", "]"}], "[", 
     RowBox[{
      RowBox[{"f1", "[", 
       RowBox[{
        RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
        RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ",", 
      RowBox[{"f2", "[", 
       RowBox[{
        RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
        RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "]"}], ".", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"D", "[", 
       RowBox[{
        RowBox[{"f1", "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ",", "t"}], 
       "]"}], ",", 
      RowBox[{"D", "[", 
       RowBox[{
        RowBox[{"f2", "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ",", "t"}], 
       "]"}]}], "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5886777739251747`*^9, 3.5886777838623924`*^9}, 
   3.588677815093647*^9, 3.58867788471657*^9, 3.588678039718442*^9, {
   3.5886784767844095`*^9, 3.5886784915264354`*^9}, 3.588678564362963*^9, 
   3.5886786037374325`*^9, {3.5886786817843695`*^9, 3.588678690676385*^9}, {
   3.588678745182881*^9, 3.5886787995021763`*^9}, {3.5886788985935507`*^9, 
   3.588678939902423*^9}, {3.588947001143866*^9, 3.5889470423591385`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.627371314805908*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%%", " ", "-", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", "sh", "]"}], "[", 
     RowBox[{
      RowBox[{"f1", "[", 
       RowBox[{
        RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
        RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ",", 
      RowBox[{"f2", "[", 
       RowBox[{
        RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
        RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "]"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"d", "[", "f", "]"}], "[", 
       RowBox[{
        RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
        RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ".", 
      RowBox[{
       RowBox[{"\[DoubledGamma]", "'"}], "[", "t", "]"}]}], ")"}]}]}], 
  "]"}]], "Input",
 CellChangeTimes->{3.6177040770355473`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.6273713423555565`*^9}]
}, Closed]],

Cell["\<\
Thus the tangent vector of the image curve is obtained composing the \
differential d[sh] with the differential d[f] applied to the tangent vector \
of the preimage.   \
\>", "Text",
 CellChangeTimes->{{3.5889473808953333`*^9, 3.588947383016937*^9}, {
  3.589002927767372*^9, 3.5890030386547146`*^9}, {3.5890034142311964`*^9, 
  3.58900342374374*^9}, {3.5890036758001575`*^9, 3.5890036774552517`*^9}, {
  3.5891127612773533`*^9, 3.589112761698554*^9}}],

Cell[TextData[{
 "The last differential is the family of linear maps represented by the ",
 StyleBox["Jacobian matrix",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.5890036977964153`*^9, 3.589003791990803*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", "f", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.5890038068086505`*^9, 3.5890038232325897`*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       SuperscriptBox["f1", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], 
      RowBox[{
       SuperscriptBox["f1", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]},
     {
      RowBox[{
       SuperscriptBox["f2", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], 
      RowBox[{
       SuperscriptBox["f2", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.627371495594626*^9}]
}, Closed]],

Cell["\<\
For any diffeomorphism the determinant of the Jacobian matrix isn\
\[CloseCurlyQuote]t zero\
\>", "Text",
 CellChangeTimes->{{3.5890040970872536`*^9, 3.5890041462870674`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Det", "[", 
   RowBox[{
    RowBox[{"d", "[", "f", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "]"}], " ", "\[NotEqual]", " ", 
  "0"}]], "Input",
 CellChangeTimes->{{3.589004155225579*^9, 3.589004184585258*^9}, 
   3.589004258554489*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     SuperscriptBox["f2", 
      TagBox[
       RowBox[{"(", 
        RowBox[{"0", ",", "1"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}], " ", 
    RowBox[{
     SuperscriptBox["f1", 
      TagBox[
       RowBox[{"(", 
        RowBox[{"1", ",", "0"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["f1", 
      TagBox[
       RowBox[{"(", 
        RowBox[{"0", ",", "1"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}], " ", 
    RowBox[{
     SuperscriptBox["f2", 
      TagBox[
       RowBox[{"(", 
        RowBox[{"1", ",", "0"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}]}], "\[NotEqual]", "0"}]], "Output",
 CellChangeTimes->{3.627371505251043*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", "f", "]"}], "[", 
   RowBox[{
    RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
    RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ".", 
  RowBox[{
   RowBox[{"\[DoubledGamma]", "'"}], "[", "t", "]"}]}]], "Input",
 CellChangeTimes->{{3.5890046127287464`*^9, 3.5890047401850367`*^9}, {
  3.589007416467111*^9, 3.58900745281819*^9}, {3.617704166443661*^9, 
  3.617704174970149*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["\[DoubledGamma]2", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}], " ", 
     RowBox[{
      SuperscriptBox["f1", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
       RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "+", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[DoubledGamma]1", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}], " ", 
     RowBox[{
      SuperscriptBox["f1", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
       RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["\[DoubledGamma]2", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}], " ", 
     RowBox[{
      SuperscriptBox["f2", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
       RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "+", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[DoubledGamma]1", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}], " ", 
     RowBox[{
      SuperscriptBox["f2", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
       RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}]}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.6273715146266594`*^9}]
}, Closed]],

Cell["\<\
This equation describes the coordinates of the image of a tangent vector by \
the coordinates of the tangent vector\
\>", "Text",
 CellChangeTimes->{{3.5890916949669495`*^9, 3.589091783910037*^9}, {
   3.5890918352069707`*^9, 3.5890918439904733`*^9}, {3.5890918758222938`*^9, 
   3.589091906262035*^9}, 3.5995675263050723`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[DoubledGamma]", "'"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.617704195682334*^9, 3.6177042018896885`*^9}}],

Cell["\<\
Since any tangent vector of a surface may serve as a tangent vector of a \
curve on the surface one concludes that the differential dF[u,v] is a linear \
isomorphism between the tangent spaces \
\>", "Text",
 CellChangeTimes->{{3.589091964014338*^9, 3.5890920320302286`*^9}, {
   3.5890920656381507`*^9, 3.5890920993430786`*^9}, {3.589092509390532*^9, 
   3.5890925559101925`*^9}, 3.589178529713417*^9, {3.589179168000925*^9, 
   3.5891791748813186`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dF", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], ":=", "  ", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "sh", "]"}], "[", 
    RowBox[{
     RowBox[{"f1", "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{"f2", "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "]"}], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"d", "[", "f", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.5890921347541037`*^9, 3.5890922464404917`*^9}, {
   3.5890922984424667`*^9, 3.5890923536746254`*^9}, {3.589092397100109*^9, 
   3.589092438922501*^9}, 3.589095740955264*^9, 3.589113927504202*^9, {
   3.589178440684325*^9, 3.58917847591634*^9}, {3.589178509732274*^9, 
   3.589178512492432*^9}, 3.589178572611871*^9, {3.589178676029786*^9, 
   3.589178692548731*^9}, {3.5891787467248297`*^9, 3.589178771180228*^9}}],

Cell["\<\
The image of a tangent vector of sf with the coordinates {x,y} is\
\>", "Text",
 CellChangeTimes->{{3.5891791270655837`*^9, 3.5891791611685343`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"imF", " ", "=", " ", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{"dF", "[", 
      RowBox[{"u", ",", "v"}], "]"}], ".", 
     RowBox[{"{", 
      RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5891788967184086`*^9, 3.5891789940119734`*^9}, 
   3.5891790666921306`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"y", " ", 
     RowBox[{
      SuperscriptBox["f2", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"h", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"f1", "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{"f2", "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "]"}]}], "+", 
    RowBox[{"x", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"h", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"f1", "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{"f2", "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "]"}], " ", 
     RowBox[{
      SuperscriptBox["f2", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"y", " ", 
        RowBox[{
         SuperscriptBox["f1", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{"x", " ", 
        RowBox[{
         SuperscriptBox["f1", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"h", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"f1", "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{"f2", "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"y", " ", 
     RowBox[{
      SuperscriptBox["f2", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"h", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"f1", "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{"f2", "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "]"}]}], "+", 
    RowBox[{"x", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"h", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"f1", "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{"f2", "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "]"}], " ", 
     RowBox[{
      SuperscriptBox["f2", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"y", " ", 
        RowBox[{
         SuperscriptBox["f1", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{"x", " ", 
        RowBox[{
         SuperscriptBox["f1", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"h", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"f1", "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{"f2", "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"y", " ", 
     RowBox[{
      SuperscriptBox["f2", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"h", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"f1", "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{"f2", "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "]"}]}], "+", 
    RowBox[{"x", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"h", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"f1", "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{"f2", "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "]"}], " ", 
     RowBox[{
      SuperscriptBox["f2", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"y", " ", 
        RowBox[{
         SuperscriptBox["f1", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{"x", " ", 
        RowBox[{
         SuperscriptBox["f1", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"h", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"f1", "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{"f2", "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.627371627773658*^9}]
}, Closed]],

Cell["The norm square of this vector is ", "Text",
 CellChangeTimes->{{3.5891792418171473`*^9, 3.589179256528989*^9}, 
   3.5995675589289384`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"imfdot", " ", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{"imF", ".", "imF"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5890961486185813`*^9, 3.5890961630174046`*^9}, {
  3.589173102998705*^9, 3.5891731114071198`*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"h", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"f1", "[", 
         RowBox[{"u", ",", "v"}], "]"}], ",", 
        RowBox[{"f2", "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"y", " ", 
         RowBox[{
          SuperscriptBox["f2", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], "+", 
        RowBox[{"x", " ", 
         RowBox[{
          SuperscriptBox["f2", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"y", " ", 
         RowBox[{
          SuperscriptBox["f1", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], "+", 
        RowBox[{"x", " ", 
         RowBox[{
          SuperscriptBox["f1", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"h", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"f1", "[", 
         RowBox[{"u", ",", "v"}], "]"}], ",", 
        RowBox[{"f2", "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"h", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"f1", "[", 
         RowBox[{"u", ",", "v"}], "]"}], ",", 
        RowBox[{"f2", "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"y", " ", 
         RowBox[{
          SuperscriptBox["f2", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], "+", 
        RowBox[{"x", " ", 
         RowBox[{
          SuperscriptBox["f2", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"y", " ", 
         RowBox[{
          SuperscriptBox["f1", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], "+", 
        RowBox[{"x", " ", 
         RowBox[{
          SuperscriptBox["f1", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"h", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"f1", "[", 
         RowBox[{"u", ",", "v"}], "]"}], ",", 
        RowBox[{"f2", "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"h", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"f1", "[", 
         RowBox[{"u", ",", "v"}], "]"}], ",", 
        RowBox[{"f2", "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"y", " ", 
         RowBox[{
          SuperscriptBox["f2", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], "+", 
        RowBox[{"x", " ", 
         RowBox[{
          SuperscriptBox["f2", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"y", " ", 
         RowBox[{
          SuperscriptBox["f1", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], "+", 
        RowBox[{"x", " ", 
         RowBox[{
          SuperscriptBox["f1", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"h", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"f1", "[", 
         RowBox[{"u", ",", "v"}], "]"}], ",", 
        RowBox[{"f2", "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "]"}]}]}], ")"}], "2"]}]], "Output",
 CellChangeTimes->{3.627371640674881*^9}]
}, Closed]],

Cell["\<\
This is a symmetric, positive definite quadratic form on the tangent space of \
sf at the point sf[u,v], expressed as a  quadratic function of the \
coordinates x,y of the tangent vector; the coefficients of this form can be \
read off from the following equation:\
\>", "Text",
 CellChangeTimes->{{3.589179351104398*^9, 3.5891794335371127`*^9}, {
   3.589179471897307*^9, 3.5891794926004915`*^9}, {3.589179532632781*^9, 
   3.589179597400485*^9}, {3.589179633257536*^9, 3.589179648616415*^9}, {
   3.5891796791211596`*^9, 3.5891796974572086`*^9}, 3.589191183175168*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", "-", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"rgT", "[", 
         RowBox[{"i", ",", "j"}], "]"}], "[", "sh", "]"}], "[", 
       RowBox[{
        RowBox[{"f1", "[", 
         RowBox[{"u", ",", "v"}], "]"}], ",", 
        RowBox[{"f2", "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "*", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"d", "[", "f", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], ".", 
         RowBox[{"{", 
          RowBox[{"x", ",", "y"}], "}"}]}], ")"}], "[", 
       RowBox[{"[", "i", "]"}], "]"}], "*", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"d", "[", "f", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], ".", 
         RowBox[{"{", 
          RowBox[{"x", ",", "y"}], "}"}]}], ")"}], "[", 
       RowBox[{"[", "j", "]"}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "2"}], "}"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.589006516431632*^9, 3.5890065543608017`*^9}, {
   3.58900660888992*^9, 3.5890066480421596`*^9}, {3.5890066985860505`*^9, 
   3.589006759490534*^9}, 3.589007352939478*^9, {3.589007550179759*^9, 
   3.5890075695968695`*^9}, {3.589007613876402*^9, 3.589007622562899*^9}, {
   3.5890076612551117`*^9, 3.589007699765315*^9}, {3.589007746557991*^9, 
   3.589007825304495*^9}, 3.5890079005778008`*^9, 3.5890079866817255`*^9, {
   3.5890081122659082`*^9, 3.5890081411845627`*^9}, {3.589111668230233*^9, 
   3.5891117338439484`*^9}, {3.5891117660736055`*^9, 
   3.5891117712528143`*^9}, {3.5891118153228917`*^9, 3.589111820658101*^9}, {
   3.589111873354994*^9, 3.589111932385497*^9}, {3.5891121139386163`*^9, 
   3.5891121239538336`*^9}, 3.6176259331745977`*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.627371695508977*^9}]
}, Closed]],

Cell["namely the coefficients are", "Text",
 CellChangeTimes->{{3.5891797362884293`*^9, 3.589179738016528*^9}, {
  3.5891807603129997`*^9, 3.589180767968438*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fg", "[", 
    RowBox[{"k_", ",", "l_"}], "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"rgT", "[", 
        RowBox[{"i", ",", "j"}], "]"}], "[", "sh", "]"}], "[", 
      RowBox[{
       RowBox[{"f1", "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{"f2", "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "*", 
     RowBox[{
      RowBox[{
       RowBox[{"d", "[", "f", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"i", ",", "k"}], "]"}], "]"}], "*", 
     RowBox[{
      RowBox[{
       RowBox[{"d", "[", "f", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"j", ",", "l"}], "]"}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "2"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5891697933904924`*^9, 3.589169798179701*^9}, {
   3.589170043864532*^9, 3.589170095157422*^9}, {3.589170151333121*^9, 
   3.5891702382564735`*^9}, {3.5891702783641443`*^9, 3.589170311935403*^9}, {
   3.589170343884259*^9, 3.589170424786001*^9}, {3.589170462756468*^9, 
   3.5891704699168806`*^9}, {3.5891705109293523`*^9, 
   3.5891705886798887`*^9}, {3.5891710125170336`*^9, 3.589171030925066*^9}, {
   3.589171522232329*^9, 3.5891715882828445`*^9}, {3.58917227446565*^9, 
   3.589172283201665*^9}, {3.589172315946123*^9, 3.589172380982637*^9}, {
   3.5891724568767705`*^9, 3.589172569274968*^9}, {3.5891727105956163`*^9, 
   3.589172741904871*^9}, {3.589172824366616*^9, 3.589172834662634*^9}, 
   3.617626003982648*^9}],

Cell["Indeed:", "Text",
 CellChangeTimes->{{3.5891797646330504`*^9, 3.5891797678972373`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"imfdot", " ", "-", " ", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"fg", "[", 
        RowBox[{"k", ",", "l"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "*", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "y"}], "}"}], "[", 
       RowBox[{"[", "k", "]"}], "]"}], "*", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "y"}], "}"}], "[", 
       RowBox[{"[", "l", "]"}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", "2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"l", ",", "2"}], "}"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5891716389361334`*^9, 3.5891716916486263`*^9}, 
   3.589171868225336*^9, {3.589173186115651*^9, 3.589173267610194*^9}, {
   3.5891734713933525`*^9, 3.589173522998243*^9}, {3.5891735534026966`*^9, 
   3.589173565289917*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.62737173126424*^9}]
}, Closed]],

Cell["\<\
fg is the special case of the next definition with ff = sh, hh = f:\
\>", "Text",
 CellChangeTimes->{{3.589180411585054*^9, 3.589180488025426*^9}, {
  3.589180806744656*^9, 3.5891808121129627`*^9}, {3.589180947240692*^9, 
  3.5891809669288177`*^9}, {3.5891810706567507`*^9, 3.5891810736889243`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"pullbackF", "[", 
     RowBox[{"ff_", ",", "hh_"}], "]"}], "[", 
    RowBox[{"k_", ",", "l_"}], "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"rgT", "[", 
        RowBox[{"i", ",", "j"}], "]"}], "[", "ff", "]"}], "[", 
      RowBox[{
       RowBox[{
        RowBox[{"hh", "[", 
         RowBox[{"u", ",", "v"}], "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}], ",", 
       RowBox[{
        RowBox[{"hh", "[", 
         RowBox[{"u", ",", "v"}], "]"}], "[", 
        RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "*", 
     RowBox[{
      RowBox[{
       RowBox[{"d", "[", "hh", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"i", ",", "k"}], "]"}], "]"}], "*", 
     RowBox[{
      RowBox[{
       RowBox[{"d", "[", "hh", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"j", ",", "l"}], "]"}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "2"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5891697933904924`*^9, 3.589169798179701*^9}, {
   3.589170043864532*^9, 3.589170095157422*^9}, {3.589170151333121*^9, 
   3.5891702382564735`*^9}, {3.5891702783641443`*^9, 3.589170311935403*^9}, {
   3.589170343884259*^9, 3.589170424786001*^9}, {3.589170462756468*^9, 
   3.5891704699168806`*^9}, {3.5891705109293523`*^9, 
   3.5891705886798887`*^9}, {3.5891710125170336`*^9, 3.589171030925066*^9}, {
   3.589171522232329*^9, 3.5891715882828445`*^9}, {3.58917227446565*^9, 
   3.589172283201665*^9}, {3.589172315946123*^9, 3.589172380982637*^9}, {
   3.5891724568767705`*^9, 3.589172569274968*^9}, {3.5891727105956163`*^9, 
   3.589172741904871*^9}, {3.589172824366616*^9, 3.589172834662634*^9}, {
   3.5891770502016425`*^9, 3.5891770874233074`*^9}, {3.5891771698850527`*^9, 
   3.589177181397873*^9}, {3.589177214532331*^9, 3.589177359612586*^9}, {
   3.5891774076606703`*^9, 3.5891774078322706`*^9}, 3.617626011647086*^9, {
   3.6177024125383434`*^9, 3.617702414332446*^9}}],

Cell["Indeed:", "Text",
 CellChangeTimes->{{3.589180499913106*^9, 3.5891805025442567`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"pullbackF", "[", 
        RowBox[{"sh", ",", "f"}], "]"}], "[", 
       RowBox[{"i", ",", "j"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "-", 
     RowBox[{
      RowBox[{"fg", "[", 
       RowBox[{"i", ",", "j"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5891773678650007`*^9, 3.5891773789098196`*^9}, {
   3.5891774220126953`*^9, 3.58917745896916*^9}, {3.5891775033824387`*^9, 
   3.5891776673075266`*^9}, 3.6148618493635216`*^9, 3.614863501453223*^9, 
   3.617702399920622*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6273717812623277`*^9}]
}, Closed]],

Cell[TextData[{
 "Summarizing we obtain\n",
 StyleBox["Proposition",
  FontColor->RGBColor[0, 0, 1]],
 ". Let sf, sh be two surfaces in the Euclidean space and F[sf[u.v]] = \
sh[f[u,v]] a diffeomorphsm as defined above. Then the differential dF[u,v] is \
a family of linear isomorphisms between the tangent spaces of the surfaces "
}], "Text",
 CellChangeTimes->{{3.5891810884887705`*^9, 3.589181246160789*^9}, {
   3.5891813391141057`*^9, 3.589181539624574*^9}, {3.5891816073614483`*^9, 
   3.5891816585633774`*^9}, 3.589182385073931*^9, {3.5891909517719326`*^9, 
   3.589190958603323*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"tangsp", "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "[", 
   RowBox[{"x", ",", "y"}], "]"}], " ", "\[RightTeeArrow]", " ", 
  RowBox[{
   RowBox[{"dF", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ".", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5891816839978323`*^9, 3.589181696219531*^9}, {
   3.5891817267402763`*^9, 3.589181730315481*^9}, {3.58918179322808*^9, 
   3.5891818181395044`*^9}, {3.5891818644271517`*^9, 3.5891818810271015`*^9}, 
   3.6227808742831593`*^9}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"dF", "[", 
     RowBox[{"u", ",", "v"}], "]"}], ".", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y"}], "}"}]}], "\[Equal]", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", "sh", "]"}], "[", 
     RowBox[{
      RowBox[{"f1", "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
      RowBox[{"f2", "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "]"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"d", "[", "f", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ".", 
      RowBox[{"{", 
       RowBox[{"x", ",", "y"}], "}"}]}], ")"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5891819634118137`*^9, 3.5891820500607696`*^9}, {
  3.5891821021567492`*^9, 3.5891822003973684`*^9}}],

Cell[TextData[{
 "where the Jacobian matrix d[f][u,v] of f is invertible.  The scalar product \
of two image vectors under the map dF[u,v] can be calculated  evaluating the ",
 StyleBox["pull back of the first fundamental form",
  FontColor->RGBColor[0, 0, 1]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"fund1", "[", "sh", "]"}], " ", "to", " ", "the", " ", "surface", 
    " ", "sf"}], TraditionalForm]]],
 ", namely pullback ",
 Cell[BoxData[
  FormBox[
   RowBox[{"fund1", "[", 
    RowBox[{"sh", ",", "f"}], "]"}], TraditionalForm]]],
 ", which is a field of symmetric, positive definite bilinear forms on the \
preimage surface sf. The map F is an isometry iff this pullback coincides \
with the first fundamental form of sf:"
}], "Text",
 CellChangeTimes->{{3.5891822714574327`*^9, 3.58918234099341*^9}, 
   3.5891823920413294`*^9, {3.589182424977214*^9, 3.5891825574487906`*^9}, {
   3.5891826145780582`*^9, 3.5891828892247667`*^9}, {3.589182955201541*^9, 
   3.5891830094166417`*^9}, 3.5891911831031637`*^9, {3.590495710453009*^9, 
   3.590495710453009*^9}, 3.5995676203694525`*^9, 3.6141620282522945`*^9, 
   3.6176274924947863`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"pullbackF", "[", 
     RowBox[{"sh", ",", "f"}], "]"}], "[", 
    RowBox[{"i", ",", "j"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"rgT", "[", "sf", "]"}], "[", 
     RowBox[{"i", ",", "j"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "."}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.589183036155171*^9, 3.5891831006118574`*^9}, 
   3.6176260280920267`*^9, 3.6177023999296227`*^9}],

Cell[TextData[{
 StyleBox["Proof.",
  FontColor->RGBColor[0, 0, 1]],
 " The stated properties of the pull back form follow immediately from the \
corresponding  properties of the first fundamental form. If the corresponding \
quadratic form of the pull back form vanishes for a tangent vector {x,y}, \
then dF[u,v].{x,y] is the null vector, and  then {x,y} = {0,0} since dF[u,v] \
is a linear isomorphism. If the stated equation is true, the scalar products \
of the image vectors equal the scalar products of the preimage; thus, if t is \
the arc length parameter of the curve \[DoubledGamma][t] characterized by |\
\[DoubledGamma]\[CloseCurlyQuote][t]| == 1, then also F\[DoubledGamma][t] has \
this property: "
}], "Text",
 CellChangeTimes->{{3.589191329142517*^9, 3.589191399947567*^9}, {
  3.5891914342835307`*^9, 3.589191466123352*^9}, {3.5891915146831293`*^9, 
  3.5891915159472017`*^9}, {3.5891916046032724`*^9, 3.589191819059539*^9}, {
  3.5891918549235897`*^9, 3.589191867915333*^9}, {3.5891921228519144`*^9, 
  3.5891922318111467`*^9}, {3.5891923724761925`*^9, 3.589192558955858*^9}, {
  3.5995669673581023`*^9, 3.59956699126147*^9}, {3.5995676328671675`*^9, 
  3.5995676404746027`*^9}, {3.627371918604969*^9, 3.6273719225361757`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"F\[DoubledGamma]", "'"}], "[", "t", "]"}], ".", 
  RowBox[{
   RowBox[{"F\[DoubledGamma]", "'"}], "[", "t", "]"}]}]], "Input",
 CellChangeTimes->{{3.61770298558812*^9, 3.6177030001539536`*^9}, {
  3.6177045155556293`*^9, 3.617704574818019*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"h", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"f1", "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ",", 
        RowBox[{"f2", "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["\[DoubledGamma]2", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["f2", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
           RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox["\[DoubledGamma]1", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["f2", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
           RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], 
     "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["\[DoubledGamma]2", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["f1", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
           RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox["\[DoubledGamma]1", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["f1", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
           RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}]}], ")"}], 
      " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"h", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"f1", "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ",", 
        RowBox[{"f2", "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "]"}]}]}], 
    ")"}], "2"], "+", 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"h", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"f1", "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ",", 
        RowBox[{"f2", "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["\[DoubledGamma]2", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["f2", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
           RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox["\[DoubledGamma]1", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["f2", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
           RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], 
     "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["\[DoubledGamma]2", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["f1", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
           RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox["\[DoubledGamma]1", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["f1", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
           RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}]}], ")"}], 
      " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"h", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"f1", "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ",", 
        RowBox[{"f2", "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "]"}]}]}], 
    ")"}], "2"], "+", 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"h", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"f1", "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ",", 
        RowBox[{"f2", "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["\[DoubledGamma]2", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["f2", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
           RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox["\[DoubledGamma]1", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["f2", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
           RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], 
     "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["\[DoubledGamma]2", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["f1", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
           RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox["\[DoubledGamma]1", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["f1", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
           RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}]}], ")"}], 
      " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"h", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"f1", "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], ",", 
        RowBox[{"f2", "[", 
         RowBox[{
          RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
          RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}]}], "]"}]}]}], 
    ")"}], "2"]}]], "Output",
 CellChangeTimes->{3.627371946014217*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"F\[DoubledGamma]", "'"}], "[", "t", "]"}], ".", 
    RowBox[{
     RowBox[{"F\[DoubledGamma]", "'"}], "[", "t", "]"}]}], "\[Equal]", "  ", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"pullbackF", "[", 
         RowBox[{"sh", ",", "f"}], "]"}], "[", 
        RowBox[{"i", ",", "j"}], "]"}], "[", 
       RowBox[{
        RowBox[{"\[DoubledGamma]1", "[", "t", "]"}], ",", 
        RowBox[{"\[DoubledGamma]2", "[", "t", "]"}]}], "]"}], " ", "*", 
      RowBox[{
       RowBox[{
        RowBox[{"\[DoubledGamma]", "'"}], "[", "t", "]"}], "[", 
       RowBox[{"[", "i", "]"}], "]"}], "*", 
      RowBox[{
       RowBox[{
        RowBox[{"\[DoubledGamma]", "'"}], "[", "t", "]"}], "[", 
       RowBox[{"[", "j", "]"}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "2"}], "}"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5891931354458313`*^9, 3.589193142029208*^9}, {
   3.5891932790540457`*^9, 3.58919333071*^9}, {3.5891936984700346`*^9, 
   3.5891937167750816`*^9}, {3.5891937699181213`*^9, 
   3.5891937716942225`*^9}, {3.5891938256453085`*^9, 3.589193873032019*^9}, 
   3.589193903920786*^9, 3.589193971983679*^9, {3.589194039048515*^9, 
   3.589194109337535*^9}, {3.5891941465136614`*^9, 3.589194156657242*^9}, 
   3.589194220249879*^9, 3.589194666835422*^9, 3.617702399934623*^9, 
   3.617704600998516*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6273719516770267`*^9}]
}, Closed]],

Cell[TextData[{
 "and this equals 1 since the pull back form of ",
 Cell[BoxData[
  FormBox["fund1", TraditionalForm]]],
 "[sh] coincides with  ",
 Cell[BoxData[
  FormBox["fund1", TraditionalForm]]],
 "[sf]. Conversely, if the arc length of curves on sf is  preserved under F, \
then at any point sf[u,v] the two symmetric,positive definite bilinear forms \
considered above, for shortness denoted by a[vv,vw], b[vv,vw], have the \
property"
}], "Text",
 CellChangeTimes->{{3.5891943126041613`*^9, 3.5891943570277023`*^9}, {
   3.589194390431613*^9, 3.589194482115857*^9}, 3.5891945521568627`*^9, {
   3.589194711692988*^9, 3.5891947172033033`*^9}, {3.5891949257642317`*^9, 
   3.589195037187605*^9}, {3.5891953686845655`*^9, 3.589195506291436*^9}, {
   3.589195639188038*^9, 3.589195646699467*^9}, {3.590495710484209*^9, 
   3.5904957104998093`*^9}, 3.5995670930302906`*^9, 3.59956764813004*^9}],

Cell["If a[vv, vv] == 1  then  b[vv, vv] == 1, ", "Text",
 CellChangeTimes->{{3.5891950703325005`*^9, 3.5891951412925596`*^9}, {
  3.5891952281335263`*^9, 3.589195290510094*^9}, {3.5891955195901966`*^9, 
  3.5891956239491663`*^9}, {3.589195666484599*^9, 3.589195686715756*^9}, {
  3.589195739051749*^9, 3.589195741803907*^9}}],

Cell["\<\
then it follows for any vector vv:   a[vv, vv] == b[vv, vv].  Applying\
\>", "Text",
 CellChangeTimes->{{3.5891957543636255`*^9, 3.5891958378053975`*^9}, {
  3.5891958964127502`*^9, 3.5891959548750935`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"a", "[", 
    RowBox[{
     RowBox[{"vv", "+", "vw"}], ",", " ", 
     RowBox[{"vv", "+", "vw"}]}], "]"}], " ", "-", " ", 
   RowBox[{"a", "[", 
    RowBox[{
     RowBox[{"vv", "-", "vw"}], ",", " ", 
     RowBox[{"vv", "-", "vw"}]}], "]"}]}], "  ", "\[Equal]", " ", 
  RowBox[{"4", 
   RowBox[{"a", "[", 
    RowBox[{"vv", ",", "vw"}], "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5891959953594093`*^9, 3.5891960269982185`*^9}, {
   3.589196058880042*^9, 3.589196130399133*^9}, 3.589196169014342*^9}],

Cell["\<\
and the same equation for b one concludes a[vv,vw] == b[vv,vw].\[FilledSquare]

A detailed description of an isometry is contained in Subsection 2.1.3 below.\
\>", "Text",
 CellChangeTimes->{{3.589196214305932*^9, 3.5891962400524054`*^9}, {
   3.58919627719753*^9, 3.589196308979347*^9}, {3.5891963441073565`*^9, 
   3.589196358388173*^9}, {3.5896219720007825`*^9, 3.5896220330904894`*^9}, {
   3.5896220670205493`*^9, 3.589622087893386*^9}, {3.617704686345398*^9, 
   3.6177047059625196`*^9}, 3.6273722144126883`*^9}],

Cell[TextData[{
 "The equation defining F, see cell ",
 ButtonBox["ptr",
  BaseStyle->"Hyperlink",
  ButtonData->"ptr"],
 ", can also be interpreted in the following way:"
}], "Text",
 CellChangeTimes->{{3.5891966887550693`*^9, 3.589196765419454*^9}, {
  3.5891968196755576`*^9, 3.5891968461080694`*^9}, {3.5891970194999866`*^9, 
  3.589197069612853*^9}, {3.589197128115199*^9, 3.589197142484021*^9}, {
  3.589197185323471*^9, 3.5891972106999226`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"fsh", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", 
  RowBox[{"sh", "[", 
   RowBox[{
    RowBox[{"f1", "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{"f2", "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.589197328701672*^9, 3.589197346565694*^9}, 
   3.5891977056542325`*^9}],

Cell["\<\
is the same surface like sh, only with another parameter representation \
defined by the parameter transformation f\
\>", "Text",
 CellChangeTimes->{{3.589197261025801*^9, 3.58919729754789*^9}, {
  3.5891973541241264`*^9, 3.589197415771652*^9}, {3.5891977599153357`*^9, 
  3.589197762843504*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sh", "[", 
   RowBox[{"uu", ",", "vv"}], "]"}], " ", "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"uu", " ", "->", " ", 
     RowBox[{"f1", "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], ",", " ", 
    RowBox[{"vv", " ", "->", " ", 
     RowBox[{"f2", "[", 
      RowBox[{"u", ",", "v"}], "]"}]}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.5891974245691557`*^9, 3.589197425132188*^9}, {
  3.5891974678286295`*^9, 3.589197518451525*^9}, {3.5891975508463783`*^9, 
  3.5891976242305756`*^9}}],

Cell["\<\
Then the diffeomorphism F: sf \[LongRightArrow]fsh is a map defined by \
equality of the parameters:\
\>", "Text",
 CellChangeTimes->{{3.589197826723157*^9, 3.589197929196018*^9}, 
   3.615021585437401*^9}],

Cell["\<\
F[sf[u, v]] = fsh[u, v]. Now, after the parameter transformation, the F \
defining map ff is the identity : \
\>", "Text",
 CellChangeTimes->{{3.5891979348993444`*^9, 3.5891979721904774`*^9}, {
   3.5891980628066607`*^9, 3.5891981233641243`*^9}, 3.5891987777965555`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ff1", "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", "u"}], ";", "  ", 
  RowBox[{
   RowBox[{"ff2", "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", "v"}], ";", " ", 
  RowBox[{
   RowBox[{"ff", "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ff1", "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
     RowBox[{"ff2", "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "}"}]}]}]], "Input",
 CellChangeTimes->{{3.589198193981163*^9, 3.5891982524295063`*^9}, {
  3.589198630423126*^9, 3.589198633702314*^9}, {3.5891986683262944`*^9, 
  3.5891987411504593`*^9}, {3.589198814982682*^9, 3.589198826606347*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", "ff", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.5891982672443533`*^9, 3.5891982842843285`*^9}, 
   3.5891987995508003`*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0"},
     {"0", "1"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.6273721362565513`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"Simplify", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"pullbackF", "[", 
         RowBox[{"fsh", ",", "ff"}], "]"}], "[", 
        RowBox[{"k", ",", "l"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], " ", "-", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"rgT", "[", 
         RowBox[{"k", ",", "l"}], "]"}], "[", "fsh", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"l", ",", "2"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5891984270894966`*^9, 3.5891984817756243`*^9}, {
   3.5891988527688437`*^9, 3.5891989219448004`*^9}, {3.589198952870569*^9, 
   3.589199135833034*^9}, 3.614861849379122*^9, 3.6148635014688234`*^9, 
   3.6176260299181314`*^9, 3.617702399939623*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627372148065772*^9}]
}, Closed]],

Cell[TextData[{
 "We conclude:\n",
 StyleBox["Corollary",
  FontColor->RGBColor[0, 0, 1]],
 ". A map F[sf[u,v]] := sh[u,v]    defined by the equality of the parameters \
u,v is an isometry iff the coefficients of the first fundamental forms \
coincide:"
}], "Text",
 CellChangeTimes->{{3.5892004162202682`*^9, 3.589200431100119*^9}, {
  3.5892004646920404`*^9, 3.5892005485078344`*^9}, {3.5892006375559278`*^9, 
  3.589200690859977*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"rgT", "[", 
     RowBox[{"i", ",", "j"}], "]"}], "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"rgT", "[", 
      RowBox[{"i", ",", "j"}], "]"}], "[", "sh", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "."}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5892007019866133`*^9, 3.58920071259722*^9}, {
  3.589200746462157*^9, 3.589200819790351*^9}, {3.617626032620286*^9, 
  3.617626033843356*^9}}],

Cell["\<\
A concrete example for such an isometry is treated in subsection 2.1.4 below.\
\>", "Text",
 CellChangeTimes->{{3.5892012961075945`*^9, 3.589201446532199*^9}, {
   3.589201495700011*^9, 3.589201526347764*^9}, {3.589201570236274*^9, 
   3.5892016393642282`*^9}, {3.589201894731834*^9, 3.589201916108057*^9}, {
   3.5892020651395807`*^9, 3.5892022501801643`*^9}, {3.5892022883003445`*^9, 
   3.589202611939856*^9}, {3.58920274173628*^9, 3.5892027480046387`*^9}, {
   3.589202816884578*^9, 3.589202818035644*^9}, {3.589622502963315*^9, 
   3.589622615283512*^9}, 3.5995677538300858`*^9, 3.599567946702118*^9, 
   3.6150330015639343`*^9}],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{
  "f", ",", "F", ",", " ", "F\[DoubledGamma]", ",", "\[DoubledGamma]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6150330185849085`*^9, 3.615033063812495*^9}}],

Cell[TextData[{
 "The compositions of isometries between surfaces is an isometry again. Thus \
the class of all smooth regular surfaces form a category with respect to the \
isometries. The ",
 StyleBox["metric properties",
  FontColor->RGBColor[0, 0, 1]],
 " of the surfaces are those being  invariant under isometries; they form the \
content of the ",
 StyleBox["inner geometry",
  FontColor->RGBColor[0, 0, 1]],
 " of the surface. The inner geometry consists of all properties of the \
surface which can be described by measurements on the surface not using its \
embedding into the space.  If one cuts a circular cylinder along a generating \
line  one gets a surface which can be rolled out into a stripe of the plane. \
This rolling out process is a continuous family of surfaces which are all \
isometric to the stripe: The arc lengths of curves don't change  under. this \
deformation. Thus",
 StyleBox[" locally",
  FontColor->RGBColor[0, 0, 1]],
 " the geometry of the cylinder  is plane geometry; ",
 StyleBox["globally",
  FontColor->RGBColor[0, 0, 1]],
 ", of course, this isn't the case: on the (uncut) cylinder there exist \
closed curves which can't be contracted to a point."
}], "Text",
 CellChangeTimes->{{3.5892012961075945`*^9, 3.589201446532199*^9}, {
   3.589201495700011*^9, 3.589201526347764*^9}, {3.589201570236274*^9, 
   3.5892016393642282`*^9}, {3.589201894731834*^9, 3.589201916108057*^9}, {
   3.5892020651395807`*^9, 3.5892022501801643`*^9}, {3.5892022883003445`*^9, 
   3.589202611939856*^9}, {3.58920274173628*^9, 3.5892027480046387`*^9}, {
   3.589202816884578*^9, 3.589202818035644*^9}, {3.589622502963315*^9, 
   3.589622615283512*^9}, 3.5995677538300858`*^9, 3.599567946702118*^9, 
   3.6150330015639343`*^9, {3.6150330799094152`*^9, 3.615033082069539*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["1.2.5. The Adapted Moving Frame", "Subsubsection",
 CellChangeTimes->{{3.586059663992068*^9, 3.586059674943287*^9}, {
   3.5860750543689933`*^9, 3.58607505605709*^9}, {3.5866611656056275`*^9, 
   3.586661191821127*^9}, {3.5872724573591394`*^9, 3.5872724609903474`*^9}, 
   3.5886618229136314`*^9, 3.599568052238154*^9, 3.5995684564302726`*^9, 
   3.615021859217881*^9, 3.61503663665985*^9, {3.61762772421404*^9, 
   3.6176277264301662`*^9}, 3.617627765918425*^9, 3.617705071457425*^9, 
   3.61770616075391*^9, 3.627372251618754*^9, 3.62737264801545*^9}],

Cell[TextData[{
 "The following construction joins the point sf[u,v] of the surface with its \
orthonormal moving frame to the ",
 StyleBox[" adapted moving frame",
  FontColor->RGBColor[0, 0, 1]],
 " of the surface:"
}], "Text",
 CellChangeTimes->{{3.5995682124053154`*^9, 3.599568327117876*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"fr", "[", "0", "]"}], "[", "f_", "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], ":=", 
  RowBox[{"Prepend", "[", 
   RowBox[{
    RowBox[{"f", "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", "1"}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5860610379176807`*^9, 3.5860610410532866`*^9}, {
  3.586061137071455*^9, 3.5860612128251877`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fr", "[", "0", "]"}], "[", "sf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5860612266468124`*^9, 3.5860612409208374`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", 
   RowBox[{
    RowBox[{"f", "[", "1", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "2", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "3", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627372404889023*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"fr", "[", "i_Integer", "]"}], "[", "f_", "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"Prepend", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "i", "]"}], "[", "f", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", "0"}], "]"}], "/;", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"i", ">", "0"}], ")"}], "&&", 
    RowBox[{"(", 
     RowBox[{"i", "<", "4"}], ")"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.586061255272863*^9, 3.5860614152199435`*^9}, 
   3.6148618493947215`*^9, 3.614917824961419*^9, 3.6149182629633884`*^9, {
   3.62279638766888*^9, 3.6227963919120874`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "fr"}]], "Input",
 CellChangeTimes->{{3.6150305135993085`*^9, 3.6150306235171013`*^9}, 
   3.6150306611755676`*^9, {3.615030842853463*^9, 3.6150309915679693`*^9}, 
   3.615031036574544*^9, {3.6150310837622423`*^9, 3.6150310923837357`*^9}, 
   3.615031205239191*^9, {3.622794881439434*^9, 3.622794884013439*^9}, {
   3.622795296946164*^9, 3.622795308958185*^9}}],

Cell[BoxData[
 StyleBox["\<\"fr[i][f][u,v], i = 0,1,2,3, is the i-th column of the matrix \
\\nframe[f][u,v]. In particular fr[0] represents the point f[u,v] and fr[i] \
\\nthe i-th vector of the adapted orthonormal frame of the surface.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.627372431190669*^9},
 CellTags->"Info3627376031-6375012"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Context", "[", "fr", "]"}]], "Input",
 CellChangeTimes->{{3.622795315572597*^9, 3.622795327381818*^9}}],

Cell[BoxData["\<\"eudiffgeov2`\"\>"], "Output",
 CellChangeTimes->{3.627372435371476*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"amf", "[", "1", "]"}], "[", "sf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5860614258279624`*^9, 3.5860614510376062`*^9}, {
  3.617705834811267*^9, 3.617705859882701*^9}, {3.6227958443511257`*^9, 
  3.622795846394729*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"f", "[", "1", "]"}], 
      TagBox[
       RowBox[{"(", 
        RowBox[{"1", ",", "0"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"]}]]], ",", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"f", "[", "2", "]"}], 
      TagBox[
       RowBox[{"(", 
        RowBox[{"1", ",", "0"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"]}]]], ",", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"f", "[", "3", "]"}], 
      TagBox[
       RowBox[{"(", 
        RowBox[{"1", ",", "0"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"]}]]]}], "}"}]], "Output",
 CellChangeTimes->{3.627372448225899*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Prepend", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"amf", "[", "1", "]"}], "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", "0"}], "]"}]], "Input",
 CellChangeTimes->{{3.6227958987328215`*^9, 3.62279590923164*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"f", "[", "1", "]"}], 
      TagBox[
       RowBox[{"(", 
        RowBox[{"1", ",", "0"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"]}]]], ",", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"f", "[", "2", "]"}], 
      TagBox[
       RowBox[{"(", 
        RowBox[{"1", ",", "0"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"]}]]], ",", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"f", "[", "3", "]"}], 
      TagBox[
       RowBox[{"(", 
        RowBox[{"1", ",", "0"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"]}]]]}], "}"}]], "Output",
 CellChangeTimes->{3.627372452484707*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"frame", "[", "f_", "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", 
  RowBox[{"Transpose", "[", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"fr", "[", "j", "]"}], "[", "f", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "0", ",", "3"}], "}"}]}], "]"}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.586059709294547*^9, 3.58605971650176*^9}, {
   3.5860602525343018`*^9, 3.5860604795147*^9}, {3.5860605311039906`*^9, 
   3.5860605475620193`*^9}, {3.5860605822876806`*^9, 3.586060606249323*^9}, {
   3.586060813090086*^9, 3.586060828939714*^9}, {3.586061476996052*^9, 
   3.586061557772994*^9}, {3.6148618494103217`*^9, 3.614861849425922*^9}, 
   3.6148635014844236`*^9, {3.6149178249770193`*^9, 3.6149178249926195`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "frame"}]], "Input",
 CellChangeTimes->{{3.6150311612106724`*^9, 3.6150311927404757`*^9}, {
   3.615031307772055*^9, 3.615031452198316*^9}, {3.6150316663265634`*^9, 
   3.6150318326870785`*^9}, 3.615031877439638*^9, {3.6227955599938264`*^9, 
   3.6227955710698457`*^9}}],

Cell[BoxData[
 StyleBox["\<\"frame[f][u,v] is the element of the Euclidean group \
\\ntransforming the standard orthonormal frame {stb[i]}, i= 1,2,3,and \\nthe \
origin [0,0,0} into the adapted moving frame of the surface f[u.v].\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.627372478427552*^9},
 CellTags->"Info3627376078-6375012"]
}, Closed]],

Cell[BoxData[
 RowBox[{"Context", "[", "frame", "]"}]], "Input",
 CellChangeTimes->{{3.622795578807459*^9, 3.62279559044508*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"frame", "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.586060615328539*^9, 3.586060640460183*^9}, 
   3.6148618494415216`*^9, 3.6149178250082197`*^9, 3.6177059560121994`*^9, 
   3.6177060146115513`*^9, {3.6227962126989727`*^9, 3.6227962374874163`*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0", "0"},
     {
      RowBox[{
       RowBox[{"f", "[", "1", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], 
      FractionBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], 
       SqrtBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}]]], 
      FractionBox[
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], "-", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], 
       RowBox[{
        SqrtBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}]], " ", 
        SqrtBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], "-", 
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], "-", 
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], "-", 
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}]]}]], 
      FractionBox[
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], "-", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}]}], 
       SqrtBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}]]]},
     {
      RowBox[{
       RowBox[{"f", "[", "2", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], 
      FractionBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], 
       SqrtBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}]]], 
      FractionBox[
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], 
       RowBox[{
        SqrtBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}]], " ", 
        SqrtBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], "-", 
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], "-", 
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], "-", 
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}]]}]], 
      FractionBox[
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}]}], 
       SqrtBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}]]]},
     {
      RowBox[{
       RowBox[{"f", "[", "3", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], 
      FractionBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], 
       SqrtBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}]]], 
      FractionBox[
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}]}], 
       RowBox[{
        SqrtBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}]], " ", 
        SqrtBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], "-", 
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], "-", 
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], "-", 
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}]]}]], 
      FractionBox[
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], "-", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}]}], 
       SqrtBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}]]]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.6273725094872065`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Det", "[", "%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.586061756330143*^9, 3.5860617723357706`*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.627372570982515*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"frame", "[", 
    RowBox[{"circularcone", "[", 
     RowBox[{"1", ",", "1"}], "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5860619918437567`*^9, 3.586062006351782*^9}, {
   3.5862321451928434`*^9, 3.586232152712056*^9}, 3.614861849457122*^9, 
   3.6149178250082197`*^9, {3.6149183005126543`*^9, 3.6149183068774652`*^9}, {
   3.614918369090375*^9, 3.6149183730995817`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"v", " ", 
      RowBox[{"Cos", "[", "u", "]"}]}], ",", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{"v", " ", 
        RowBox[{"Sin", "[", "u", "]"}]}], 
       SqrtBox[
        SuperscriptBox["v", "2"]]]}], ",", 
     FractionBox[
      RowBox[{"Cos", "[", "u", "]"}], 
      SqrtBox["2"]], ",", 
     FractionBox[
      RowBox[{"v", " ", 
       RowBox[{"Cos", "[", "u", "]"}]}], 
      RowBox[{
       SqrtBox["2"], " ", 
       SqrtBox[
        SuperscriptBox["v", "2"]]}]]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"v", " ", 
      RowBox[{"Sin", "[", "u", "]"}]}], ",", 
     FractionBox[
      RowBox[{"v", " ", 
       RowBox[{"Cos", "[", "u", "]"}]}], 
      SqrtBox[
       SuperscriptBox["v", "2"]]], ",", 
     FractionBox[
      RowBox[{"Sin", "[", "u", "]"}], 
      SqrtBox["2"]], ",", 
     FractionBox[
      RowBox[{"v", " ", 
       RowBox[{"Sin", "[", "u", "]"}]}], 
      RowBox[{
       SqrtBox["2"], " ", 
       SqrtBox[
        SuperscriptBox["v", "2"]]}]]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     FractionBox["1", 
      SqrtBox["2"]], ",", 
     RowBox[{"-", 
      FractionBox["v", 
       RowBox[{
        SqrtBox["2"], " ", 
        SqrtBox[
         SuperscriptBox["v", "2"]]}]]}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627372576692125*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.586062026491417*^9, 3.586062042746646*^9}, 
   3.586232171447689*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0", "0"},
     {
      RowBox[{"v", " ", 
       RowBox[{"Cos", "[", "u", "]"}]}], 
      RowBox[{"-", 
       FractionBox[
        RowBox[{"v", " ", 
         RowBox[{"Sin", "[", "u", "]"}]}], 
        SqrtBox[
         SuperscriptBox["v", "2"]]]}], 
      FractionBox[
       RowBox[{"Cos", "[", "u", "]"}], 
       SqrtBox["2"]], 
      FractionBox[
       RowBox[{"v", " ", 
        RowBox[{"Cos", "[", "u", "]"}]}], 
       RowBox[{
        SqrtBox["2"], " ", 
        SqrtBox[
         SuperscriptBox["v", "2"]]}]]},
     {
      RowBox[{"v", " ", 
       RowBox[{"Sin", "[", "u", "]"}]}], 
      FractionBox[
       RowBox[{"v", " ", 
        RowBox[{"Cos", "[", "u", "]"}]}], 
       SqrtBox[
        SuperscriptBox["v", "2"]]], 
      FractionBox[
       RowBox[{"Sin", "[", "u", "]"}], 
       SqrtBox["2"]], 
      FractionBox[
       RowBox[{"v", " ", 
        RowBox[{"Sin", "[", "u", "]"}]}], 
       RowBox[{
        SqrtBox["2"], " ", 
        SqrtBox[
         SuperscriptBox["v", "2"]]}]]},
     {"v", "0", 
      FractionBox["1", 
       SqrtBox["2"]], 
      RowBox[{"-", 
       FractionBox["v", 
        RowBox[{
         SqrtBox["2"], " ", 
         SqrtBox[
          SuperscriptBox["v", "2"]]}]]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.627372583665337*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"Det", "[", "%", "]"}], ",", 
   RowBox[{"u", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.586062079671911*^9, 3.586062115817174*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.6273725951001573`*^9}]
}, Closed]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["1.3. Differential Forms", "Subsection",
 CellChangeTimes->{{3.5725953757423115`*^9, 3.572595385461128*^9}, {
   3.58666439764649*^9, 3.5866644089011335`*^9}, {3.5872724756311846`*^9, 
   3.58727247921439*^9}, 3.599568542111173*^9, 3.599804993627187*^9, 
   3.6176277747189283`*^9, 3.617632840802114*^9, 3.627372710290759*^9, 
   3.6273740816423683`*^9}],

Cell[TextData[{
 "In the following we will use Cartan\[CloseCurlyQuote]s method of moving \
frames. We apply the calculus of differential forms on the parameter manifold \
U in an elementary form in dimension 2 avoiding tensor algebra, as it is \
presented in the Introduction of W. Blaschke and H. Reichardt [",
 ButtonBox["B-R",
  BaseStyle->"Hyperlink",
  ButtonData->"B-R"],
 "]. "
}], "Text",
 CellChangeTimes->{{3.5866617389574213`*^9, 3.586661854765045*^9}, {
   3.5866619107972503`*^9, 3.5866619417650213`*^9}, {3.586662052197338*^9, 
   3.5866621045253305`*^9}, 3.5866624506461277`*^9, {3.586663494477832*^9, 
   3.586663633164764*^9}, 3.586664441966025*^9, 3.5867576531566353`*^9, {
   3.5998034928760347`*^9, 3.5998035298325*^9}, {3.5998035685049677`*^9, 
   3.599803644882702*^9}}],

Cell[CellGroupData[{

Cell["1.3.1. The Differential of a 0-Form (= Function)", "Subsubsection",
 CellChangeTimes->{{3.572611276502635*^9, 3.572611281323043*^9}, {
   3.586174321969679*^9, 3.5861743363685026`*^9}, {3.5866646931983943`*^9, 
   3.586664749405609*^9}, {3.587272484902715*^9, 3.5872724953183107`*^9}, 
   3.6092316085437975`*^9, 3.609308006813593*^9, 3.6177061733856325`*^9, 
   3.6177065148971663`*^9}],

Cell["\<\
The differential of a function has been defined in section 1.1; it is a \
differential form, more exactly, the coordinate tuple of a covector field. \
The evaluation of the forms for tangent vectors or tangent vector fields is \
obtained by the Dot product with the coordinate vectors\
\>", "Text",
 CellChangeTimes->{{3.586174802937189*^9, 3.586174919432852*^9}, 
   3.5862322201509748`*^9, {3.586609483759947*^9, 3.58660954266565*^9}, {
   3.59021230392822*^9, 3.5902123231630535`*^9}, 3.595484014179422*^9, {
   3.5998037540204935`*^9, 3.599803787295352*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", "h", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], ".", 
  RowBox[{"{", 
   RowBox[{"1", ",", "2"}], "}"}]}]], "Input",
 CellChangeTimes->{{3.590212361710721*^9, 3.590212380820755*^9}, 
   3.6271054821943345`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"2", " ", 
   RowBox[{
    SuperscriptBox["h", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
  RowBox[{
   SuperscriptBox["h", 
    TagBox[
     RowBox[{"(", 
      RowBox[{"1", ",", "0"}], ")"}],
     Derivative],
    MultilineFunction->None], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Output",
 CellChangeTimes->{3.6271053723077416`*^9, 3.627105483520337*^9, 
  3.627372770616065*^9}]
}, Closed]],

Cell["In eudiffgeov2.m we defined the differential as a 1-form by", "Text",
 CellChangeTimes->{{3.586672120886234*^9, 3.5866721226773367`*^9}, {
  3.5902124263728347`*^9, 3.590212459101692*^9}, {3.5998038717695007`*^9, 
  3.5998038774791107`*^9}, {3.6271055140651903`*^9, 3.627105537246831*^9}, {
  3.627105568134886*^9, 3.6271056040617485`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", 
      RowBox[{"h_", ",", "u___"}], "]"}], "[", "l_", "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"args", " ", "=", 
         RowBox[{"{", "u", "}"}]}], ",", "n"}], "}"}], ",", "\n", 
      RowBox[{
       RowBox[{"n", "=", 
        RowBox[{"Length", "[", "args", "]"}]}], ";", "\n", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"Not", "[", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Head", "[", "l", "]"}], "===", "List"}], ")"}], "&&", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Length", "[", "l", "]"}], " ", "==", " ", "n"}], 
            ")"}]}], "]"}], ",", 
         RowBox[{
          RowBox[{"Print", "[", 
           RowBox[{
           "\"\<Argument \>\"", ",", " ", "l", ",", " ", 
            "\"\< incorrect!\>\""}], "]"}], ";", "\n", 
          RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", 
       RowBox[{
        RowBox[{
         RowBox[{"d", "[", "h", "]"}], "[", "u", "]"}], ".", "l"}]}]}], 
     "]"}]}], ";"}], "\n"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5860053895177517`*^9, 3.5860056167477508`*^9}, {
   3.5860058441181498`*^9, 3.5860058472849555`*^9}, {3.5860058953642397`*^9, 
   3.586005916205877*^9}, {3.586005987108001*^9, 3.5860059877476025`*^9}, {
   3.586157360501302*^9, 3.5861574330224495`*^9}, {3.5861574659813347`*^9, 
   3.5861575502311535`*^9}, {3.5861576010810623`*^9, 3.586157618034032*^9}, 
   3.5861672686022496`*^9, {3.586167553217529*^9, 3.5861675586908417`*^9}, {
   3.5861680230984044`*^9, 3.586168040420395*^9}, {3.5861735272862263`*^9, 
   3.586173555796857*^9}, {3.5861738431832943`*^9, 3.5861738489716253`*^9}, {
   3.586173888477885*^9, 3.5861738938371916`*^9}, {3.5861741510469027`*^9, 
   3.5861741786344805`*^9}, {3.586174369246383*^9, 3.5861743854283085`*^9}, {
   3.586174703071477*^9, 3.586174721723544*^9}, {3.5861801571087027`*^9, 
   3.586180190587618*^9}, {3.5866045654457083`*^9, 3.5866045978781652`*^9}, {
   3.5866048796926603`*^9, 3.5866049374907618`*^9}, {3.5866060979455996`*^9, 
   3.5866061415788765`*^9}, {3.5866061887845592`*^9, 
   3.5866061997981787`*^9}, {3.5866062339934387`*^9, 
   3.5866063571400547`*^9}, {3.5866064409902024`*^9, 3.586606541079978*^9}, {
   3.58660687234656*^9, 3.586606910675827*^9}, {3.586607497424058*^9, 
   3.5866075672965803`*^9}, {3.5866081806584578`*^9, 3.586608320278703*^9}, {
   3.5866083550355644`*^9, 3.586608437013708*^9}, {3.586608520614255*^9, 
   3.5866085237186604`*^9}, {3.586666493961392*^9, 3.586666832336746*^9}, {
   3.586666973216804*^9, 3.5866669881356573`*^9}, {3.5866670270078807`*^9, 
   3.5866670286629753`*^9}, {3.586667322887804*^9, 3.5866673449030633`*^9}, {
   3.5866673859834127`*^9, 3.5866674002712297`*^9}, {3.586667504080168*^9, 
   3.5866675524229326`*^9}, {3.586667875759426*^9, 3.586667932094649*^9}, {
   3.586668002592681*^9, 3.5866680095990815`*^9}, 3.5866680408898716`*^9, {
   3.5866680946029434`*^9, 3.586668379732252*^9}, 3.5866684152132816`*^9, {
   3.5866685316519413`*^9, 3.586668568757064*^9}, {3.5866686152287216`*^9, 
   3.5866686618283873`*^9}, {3.5866687358606215`*^9, 3.586668752408568*^9}, 
   3.5866688316411*^9, {3.586668916928978*^9, 3.5866689246484194`*^9}, {
   3.586669160776925*^9, 3.5866693954043455`*^9}, {3.5866694376797633`*^9, 
   3.58666947783606*^9}, 3.5866695936206827`*^9, {3.586669654315154*^9, 
   3.5866697503796487`*^9}, {3.58666978432459*^9, 3.586669873101668*^9}, {
   3.586669929454891*^9, 3.5866699427336507`*^9}, 3.58667177289433*^9, {
   3.5866718081883483`*^9, 3.586671818796955*^9}, {3.5902120419881597`*^9, 
   3.590212061222993*^9}, {3.590212141999935*^9, 3.590212160735568*^9}, 
   3.590212472205715*^9, 3.5954844475325828`*^9, {3.5954845341595354`*^9, 
   3.5954845660927916`*^9}, {3.59548460531126*^9, 3.595484620053286*^9}, {
   3.599804058189828*^9, 3.599804075365458*^9}, {3.609222274667694*^9, 
   3.6092223292498164`*^9}, {3.6092239826166396`*^9, 3.6092239851837864`*^9}, 
   3.6092305807790127`*^9, {3.614055072635099*^9, 3.6140550761451054`*^9}, 
   3.614861849472722*^9, 3.614917782279744*^9, 3.6271056519538326`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "d"}]], "Input",
 CellChangeTimes->{{3.6226166564994073`*^9, 3.622616657404209*^9}}],

Cell[BoxData[
 StyleBox["\<\"d[f][u,v,w,...] is the differential of the function \
f[u,v,w...] of several \\nvariables with values in an  n-dimensional manifold \
or space. For the variables u, v, w,... \\nthe differentials d[u][u,v,...], \
d[v][u,v,... ],... yield the standard cobasis in the \\ncotangent space of \
the parameter manifold at the point {u, v,...}. d[f,u,v,w,...][vl] is \\nthe \
value of the differential d[f][u,v,w,...] for the tangent vector vl.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.627372783564088*^9},
 CellTags->"Info3627376383-6375012"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "h", "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.586608541643092*^9, 3.586608542891094*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SuperscriptBox["h", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
   RowBox[{
    SuperscriptBox["h", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
   RowBox[{
    SuperscriptBox["h", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6270529254771123`*^9, 3.6273728120809383`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"h", ",", "u", ",", "v", ",", "w"}], "]"}], "[", 
  RowBox[{"{", 
   RowBox[{"0", ",", "1", ",", "0"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.6140568667942505`*^9, 3.6140569324703655`*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["h", 
   TagBox[
    RowBox[{"(", 
     RowBox[{"0", ",", "1", ",", "0"}], ")"}],
    Derivative],
   MultilineFunction->None], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Output",
 CellChangeTimes->{3.627052930999522*^9, 3.6271057178015485`*^9, 
  3.6273739621617584`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "sf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.599804000142126*^9, 3.59980400803574*^9}, {
  3.5998041179067326`*^9, 3.5998041241779437`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6270529379571342`*^9, 3.6273739669353666`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "u", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{3.6092241389175797`*^9, 3.609230618873192*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.6270529435731444`*^9, 3.627373970070972*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"u", ",", "u", ",", "v"}], "]"}], "[", 
  RowBox[{"{", 
   RowBox[{"s", ",", "t"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.6092241489101515`*^9, 3.6092241571886244`*^9}, {
   3.6092306631367235`*^9, 3.609230709273362*^9}, 3.614055563458761*^9}],

Cell[BoxData["s"], "Output",
 CellChangeTimes->{3.62705294686475*^9, 3.6271057356791797`*^9, 
  3.6273739732221775`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", "h", "]"}], "[", 
   RowBox[{"u", ",", "v", ",", "w"}], "]"}], ".", 
  RowBox[{"vec", "[", 
   RowBox[{"x", ",", "3"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.586608541643092*^9, 3.586608542891094*^9}, {
   3.5866668851657677`*^9, 3.586666904446871*^9}, 3.586671913536374*^9, {
   3.5899810899432864`*^9, 3.5899811086223545`*^9}, {3.609222408284337*^9, 
   3.6092224095964117`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"x", "[", "3", "]"}], " ", 
   RowBox[{
    SuperscriptBox["h", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], "+", 
  RowBox[{
   RowBox[{"x", "[", "2", "]"}], " ", 
   RowBox[{
    SuperscriptBox["h", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], "+", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], " ", 
   RowBox[{
    SuperscriptBox["h", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}]}]}]], "Output",
 CellChangeTimes->{3.6271057419815907`*^9, 3.627373983689796*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"h", ",", "u", ",", "v", ",", "w"}], "]"}], "[", 
  RowBox[{"vec", "[", 
   RowBox[{"x", ",", "3"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.586608541643092*^9, 3.586608542891094*^9}, {
   3.5866668851657677`*^9, 3.586666904446871*^9}, 3.586671913536374*^9, {
   3.5899810899432864`*^9, 3.5899811086223545`*^9}, {3.5954886029274817`*^9, 
   3.5954886331603346`*^9}, {3.609222422949176*^9, 3.6092224244042587`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"x", "[", "3", "]"}], " ", 
   RowBox[{
    SuperscriptBox["h", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], "+", 
  RowBox[{
   RowBox[{"x", "[", "2", "]"}], " ", 
   RowBox[{
    SuperscriptBox["h", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], "+", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], " ", 
   RowBox[{
    SuperscriptBox["h", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}]}]}]], "Output",
 CellChangeTimes->{3.627105746677199*^9, 3.6273739902886076`*^9}]
}, Closed]],

Cell["\<\
OK.
The differential of a vector function is the vector of the differentials of \
its components:\
\>", "Text",
 CellChangeTimes->{{3.586664784821635*^9, 3.586664824405899*^9}, {
   3.586672145190624*^9, 3.5866721496778803`*^9}, 3.5998049367289324`*^9}],

Cell[BoxData[
 RowBox[{"sf", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5867577346292953`*^9, 3.5867577393255634`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "sf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.586608541643092*^9, 3.586608570113142*^9}}],

Cell["This is a vector of differential forms:", "Text",
 CellChangeTimes->{{3.5861755001530676`*^9, 3.586175567096896*^9}, 
   3.586175620051925*^9, {3.586175719948639*^9, 3.586175730684253*^9}, {
   3.586665765597732*^9, 3.5866657669328084`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "\[Equal]", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", 
      RowBox[{"f", "[", "i", "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "3"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.58617394101589*^9, 3.5861739511384687`*^9}, {
   3.586175358458963*^9, 3.586175475179639*^9}, {3.58617562555624*^9, 
   3.586175682092474*^9}, {3.5862639673368874`*^9, 3.5862639758653755`*^9}, 
   3.6148618495195217`*^9, 3.614863501500024*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6270529663959846`*^9, 3.6271057732440453`*^9, 
  3.6273740068090363`*^9}]
}, Closed]],

Cell["\<\
The local cobasis is defined by the differentials d[u], d[v]:\
\>", "Text",
 CellChangeTimes->{{3.586175113040926*^9, 3.5861751404164915`*^9}, {
  3.599804682830525*^9, 3.5998046926741424`*^9}, {3.609231165950483*^9, 
  3.6092311808063326`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"sf", ",", "u", ",", "v"}], "]"}], "[", 
  RowBox[{"{", 
   RowBox[{"s", ",", "t"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.586672163805689*^9, 3.586672202279889*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"t", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "+", 
    RowBox[{"s", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"t", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "+", 
    RowBox[{"s", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"t", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "+", 
    RowBox[{"s", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.62737401432825*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"sf", "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", " ", "u"}], "]"}], "*", 
    RowBox[{
     RowBox[{"d", "[", 
      RowBox[{"u", ",", "u", ",", "v"}], "]"}], "[", 
     RowBox[{"{", 
      RowBox[{"s", ",", "t"}], "}"}], "]"}]}], " ", "+", 
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"sf", "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", " ", "v"}], "]"}], "*", 
    RowBox[{
     RowBox[{"d", "[", 
      RowBox[{"v", ",", "u", ",", "v"}], "]"}], "[", 
     RowBox[{"{", 
      RowBox[{"s", ",", "t"}], "}"}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.6092308865074997`*^9, 3.609231047065683*^9}, {
  3.6092310939473643`*^9, 3.6092311331686077`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.627374022377864*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", 
    RowBox[{"h", ",", "u", ",", "v"}], "]"}], "[", 
   RowBox[{"{", 
    RowBox[{"s", ",", "t"}], "}"}], " ", "]"}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"h", "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", "u"}], "]"}], "*", 
    RowBox[{
     RowBox[{"d", "[", 
      RowBox[{"u", ",", "u", ",", "v"}], "]"}], "[", 
     RowBox[{"{", 
      RowBox[{"s", ",", "t"}], "}"}], "]"}]}], "+", 
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"h", "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", "v"}], "]"}], "*", 
    RowBox[{
     RowBox[{"d", "[", 
      RowBox[{"v", ",", "u", ",", "v"}], "]"}], "[", 
     RowBox[{"{", 
      RowBox[{"s", ",", "t"}], "}"}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.586232907051381*^9, 3.586232990511528*^9}, {
  3.5866071415562325`*^9, 3.5866071748934913`*^9}, {3.586607643299914*^9, 
  3.586607662534748*^9}, {3.586672926394306*^9, 3.58667295915418*^9}, {
  3.5866730242579036`*^9, 3.586673045131098*^9}, {3.609231247989175*^9, 
  3.609231300964205*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.62737403726029*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ".", 
   RowBox[{"{", 
    RowBox[{"s", ",", "t"}], "}"}]}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"d", "[", 
    RowBox[{"sf", ",", "u", ",", "v"}], "]"}], "[", 
   RowBox[{"{", 
    RowBox[{"s", ",", "t"}], "}"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.586232907051381*^9, 3.586232990511528*^9}, {
  3.586233084673293*^9, 3.586233102675725*^9}, {3.5862332045595036`*^9, 
  3.586233306552483*^9}, {3.586693207585164*^9, 3.586693231078806*^9}, {
  3.5866932664596677`*^9, 3.5866933097341437`*^9}}],

Cell["Remember also  the TensorProduct:", "Text",
 CellChangeTimes->{{3.6092267184271193`*^9, 3.609226736707165*^9}, {
  3.6092315366386847`*^9, 3.609231550774493*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{"sf", "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", "u"}], "]"}], "\[TensorProduct]", 
   RowBox[{
    RowBox[{"d", "[", "u", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
  RowBox[{
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{"sf", "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", "v"}], "]"}], "\[TensorProduct]", 
   RowBox[{
    RowBox[{"d", "[", "v", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6092255700064335`*^9, 3.6092255957509055`*^9}, {
  3.6092256366472445`*^9, 3.6092256596085577`*^9}, {3.6092257240252423`*^9, 
  3.6092257378640337`*^9}, {3.6092259323801594`*^9, 3.6092259767666984`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627374043609501*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "\[Equal]", 
  RowBox[{
   RowBox[{"d", "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6092256755794716`*^9, 3.609225686205079*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6273740631875353`*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["1.3.2. Sum and Product Rules", "Subsubsection",
 CellChangeTimes->{{3.5866934699464254`*^9, 3.586693482956848*^9}, {
   3.587272502702733*^9, 3.5872725077900243`*^9}, 3.5998050004605775`*^9, 
   3.6091386762717667`*^9, 3.6177065204414835`*^9, 3.6177078203058095`*^9, 
   3.6273743154329386`*^9, 3.6274413652668295`*^9}],

Cell["\<\
First we consider a rule for the argumentwise addition of functions.\
\>", "Text",
 CellChangeTimes->{{3.5867582566201515`*^9, 3.586758307524063*^9}, {
   3.5998060968912897`*^9, 3.5998061000514708`*^9}, {3.608969182003406*^9, 
   3.608969183314481*^9}, {3.6089692727475967`*^9, 3.6089692751787357`*^9}, 
   3.615117528078391*^9, 3.6177065469209976`*^9}],

Cell[BoxData[
 RowBox[{"?", "plus"}]], "Input",
 CellChangeTimes->{{3.6089692830481853`*^9, 3.60896928522731*^9}, 
   3.6089693522671447`*^9}],

Cell[BoxData[
 RowBox[{"Context", "[", "plus", "]"}]], "Input",
 CellChangeTimes->{{3.608969363813805*^9, 3.6089693714922442`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"plus", "[", 
   RowBox[{"f", ",", "h"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6089691308184786`*^9, 3.6089691512206454`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"plus", "[", 
   RowBox[{"f", ",", 
    RowBox[{"-", "f"}]}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6089691308184786`*^9, 3.6089691512206454`*^9}, {
  3.6089693858350644`*^9, 3.6089693939485283`*^9}}],

Cell[BoxData[
 RowBox[{"zero", "[", "u", "]"}]], "Input",
 CellChangeTimes->{{3.6089609960595636`*^9, 3.6089610032199764`*^9}}],

Cell[BoxData[
 RowBox[{"zero", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.608961030426424*^9, 3.6089610376336365`*^9}}],

Cell["Adding functions f, h by Plus isn\[CloseCurlyQuote]t defined:", "Text",
 CellChangeTimes->{{3.608969532331444*^9, 3.6089695749308805`*^9}, 
   3.6089719663938813`*^9, 3.615033121402789*^9, 3.6150331679314504`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"h", "+", "f"}], ")"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{3.586697645184559*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Plus", "[", 
   RowBox[{"f", ",", "h"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6089695861105194`*^9, 3.608969601620407*^9}, 
   3.61503319135679*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", 
    RowBox[{"plus", "[", 
     RowBox[{"h", ",", "f"}], "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "h", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], " ", "+", " ", 
   RowBox[{
    RowBox[{"d", "[", "f", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.586170499301035*^9, 3.5861705132268314`*^9}, {
  3.5861758649549327`*^9, 3.5861759534439945`*^9}, {3.5866976110360985`*^9, 
  3.586697618680112*^9}, {3.6089719888831673`*^9, 3.6089720248992276`*^9}}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", 
     RowBox[{
      RowBox[{"plus", "[", 
       RowBox[{"h", ",", "f"}], "]"}], ",", "u", ",", "v"}], "]"}], "[", 
    RowBox[{"{", 
     RowBox[{"s", ",", "t"}], "}"}], "]"}], "\[Equal]", " ", 
   RowBox[{
    RowBox[{"plus", "[", 
     RowBox[{
      RowBox[{"d", "[", 
       RowBox[{"h", ",", "u", ",", "v"}], "]"}], " ", ",", " ", 
      RowBox[{"d", "[", 
       RowBox[{"f", ",", "u", ",", "v"}], "]"}]}], "]"}], "[", 
    RowBox[{"{", 
     RowBox[{"s", ",", "t"}], "}"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.586170499301035*^9, 3.5861705132268314`*^9}, {
  3.5861758649549327`*^9, 3.5861759534439945`*^9}, {3.5866976110360985`*^9, 
  3.586697618680112*^9}, {3.5867587958759947`*^9, 3.5867588868802*^9}, {
  3.586758926592471*^9, 3.5867589322727966`*^9}, {3.6089720585821543`*^9, 
  3.6089721165014668`*^9}}],

Cell["\<\
The function 0[u___]  is not defined; it contradicts the syntax rule: Any \
symbol name must start with a letter.\
\>", "Text",
 CellChangeTimes->{{3.609137726555123*^9, 3.6091378124332733`*^9}, {
  3.6091378564798775`*^9, 3.609137859392044*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"h", "-", "h"}], ")"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6089722793817835`*^9, 3.6089722895793667`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"plus", "[", 
   RowBox[{"h", ",", 
    RowBox[{"-", "h"}]}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6091379234347067`*^9, 3.6091379441628923`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{
    RowBox[{"plus", "[", 
     RowBox[{"h", ",", 
      RowBox[{"-", "h"}]}], "]"}], ",", "u", ",", "v"}], "]"}], "[", 
  RowBox[{"{", 
   RowBox[{"s", ",", "t"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.586695101662491*^9, 3.5866951193685226`*^9}, 
   3.5866969664897666`*^9, {3.5866994488129263`*^9, 3.586699457517742*^9}, {
   3.5902128031758966`*^9, 3.5902128321763477`*^9}, {3.608972365484708*^9, 
   3.6089724264361944`*^9}, 3.6091380787465906`*^9, 3.609138147763538*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"plus", "[", 
    RowBox[{"h", ",", 
     RowBox[{"-", "h"}]}], "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.586695101662491*^9, 3.5866951193685226`*^9}, 
   3.5866969664897666`*^9, {3.5866994488129263`*^9, 3.586699457517742*^9}, {
   3.5902128031758966`*^9, 3.5902128321763477`*^9}, {3.608972365484708*^9, 
   3.608972458475027*^9}, {3.609138183562586*^9, 3.609138185298685*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", 
    RowBox[{
     RowBox[{"plus", "[", 
      RowBox[{"h", ",", 
       RowBox[{"-", "f"}]}], "]"}], ",", "u", ",", "v"}], "]"}], "[", 
   RowBox[{"{", 
    RowBox[{"s", ",", "t"}], "}"}], "]"}], "-", 
  RowBox[{
   RowBox[{"d", "[", 
    RowBox[{"h", ",", "u", ",", "v"}], "]"}], "[", 
   RowBox[{"{", 
    RowBox[{"s", ",", "t"}], "}"}], "]"}], " ", "+", 
  RowBox[{
   RowBox[{"d", "[", 
    RowBox[{"f", ",", "u", ",", "v"}], "]"}], "[", 
   RowBox[{"{", 
    RowBox[{"s", ",", "t"}], "}"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.586695101662491*^9, 3.5866951193685226`*^9}, 
   3.586696958736553*^9, {3.5866994963618097`*^9, 3.5866995074690294`*^9}, {
   3.586759049719514*^9, 3.5867590722788043`*^9}, {3.608972477076091*^9, 
   3.6089724880197167`*^9}, {3.6089725647961082`*^9, 3.608972638157304*^9}, 
   3.6091382607329993`*^9}],

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.60897264542772*^9, 3.6089726519950953`*^9}}],

Cell["\<\
The next two cells led to unsatisfactory results, therefore we made them \
non-evaluable:\
\>", "Text",
 CellChangeTimes->{{3.586699608510407*^9, 3.5866996173244224`*^9}, {
   3.5866996648265057`*^9, 3.5866997222346067`*^9}, {3.586700302084093*^9, 
   3.5867003049882593`*^9}, 3.615117624077882*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Unprotect", "[", "Times", "]"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"h", "*", "f"}], ")"}], "[", "u___", "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{"h", "[", "u", "]"}], "*", 
    RowBox[{"f", "[", "u", "]"}]}]}], ";", 
  RowBox[{"Protect", "[", "Times", "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.586176146106014*^9, 3.5861761604438343`*^9}, {
   3.586176212667821*^9, 3.586176219555215*^9}, {3.5861764457641535`*^9, 
   3.586176448811328*^9}, {3.586179992161268*^9, 3.586180045256305*^9}, 
   3.5861802843079786`*^9, {3.586180325530336*^9, 3.586180335425902*^9}, 
   3.5861809246586046`*^9, {3.5861813951225133`*^9, 3.5861814146896324`*^9}, {
   3.5866948042167687`*^9, 3.5866948729192896`*^9}, {3.5866949295161886`*^9, 
   3.586694942947812*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Unprotect", "[", "Divide", "]"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"h_", "/", "f_"}], ")"}], "[", "u___", "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"h", "[", "u", "]"}], "/", 
    RowBox[{"f", "[", "u", "]"}]}]}], ";", 
  RowBox[{"Protect", "[", "Divide", "]"}], ";"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5861765616997843`*^9, 3.586176568130152*^9}, {
   3.5861766180670085`*^9, 3.5861766375791245`*^9}, 3.5861814251452303`*^9, {
   3.5866948895177183`*^9, 3.5866949166929665`*^9}, {3.5866950043651204`*^9, 
   3.5866950154723396`*^9}, {3.586695137339754*^9, 3.58669518073903*^9}, {
   3.586695359078543*^9, 3.5866954404014864`*^9}, 3.586700369902972*^9}],

Cell["\<\
We prefer to define first a new function corresponding to the wanted \
operation and then to apply the differential.\
\>", "Text",
 CellChangeTimes->{{3.5867003949084024`*^9, 3.5867004820113845`*^9}, {
   3.6151176307982664`*^9, 3.6151176349745054`*^9}, 3.6177068530895095`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"hf", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", 
  RowBox[{
   RowBox[{"h", "[", 
    RowBox[{"u", ",", "v"}], "]"}], "*", 
   RowBox[{"f", "[", 
    RowBox[{"u", ",", "v"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5861765616997843`*^9, 3.586176568130152*^9}, {
   3.5861766180670085`*^9, 3.5861766375791245`*^9}, 3.5861814251452303`*^9, {
   3.5866948895177183`*^9, 3.5866949166929665`*^9}, {3.5867005003014307`*^9, 
   3.5867005604948735`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "hf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5866956108941855`*^9, 3.5866956706890907`*^9}, 
   3.586700547550133*^9}],

Cell[BoxData[
 RowBox[{"%", "-", " ", 
  RowBox[{
   RowBox[{"h", "[", 
    RowBox[{"u", ",", "v"}], "]"}], "*", 
   RowBox[{
    RowBox[{"d", "[", "f", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], " ", "-", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "h", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "*", 
   RowBox[{"f", "[", 
    RowBox[{"u", ",", "v"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.586170588470135*^9, 3.586170596314584*^9}, {
  3.5861804177446103`*^9, 3.5861805309620857`*^9}, {3.5861805934906626`*^9, 
  3.586180596185817*^9}, {3.586180723066074*^9, 3.586180768243658*^9}, {
  3.586180807948929*^9, 3.5861808189555583`*^9}, {3.5861808559476743`*^9, 
  3.586180862659058*^9}, {3.5861811085951247`*^9, 3.5861811264421453`*^9}, {
  3.5861814751470904`*^9, 3.5861814955872593`*^9}}],

Cell["OK. Multiplication with a constant :", "Text",
 CellChangeTimes->{{3.5867592339080486`*^9, 3.586759258348447*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"Pi", "*", "f"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{
  3.5866957036363487`*^9, {3.5866957457408223`*^9, 3.5866957464584236`*^9}, 
   3.5866957931493053`*^9}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", "-", 
   RowBox[{"Pi", "*", 
    RowBox[{
     RowBox[{"d", "[", "f", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5867006053194375`*^9, 3.5867006360301943`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{
    RowBox[{"N", "[", "Pi", "]"}], "*", "f"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{
  3.5866957036363487`*^9, {3.5866957457408223`*^9, 3.5866957464584236`*^9}, 
   3.5866957931493053`*^9, {3.586701009461553*^9, 3.586701015437895*^9}}],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"N", "[", "Pi", "]"}], "*", 
   RowBox[{
    RowBox[{"d", "[", "f", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.586701032414866*^9, 3.5867010529340396`*^9}, {
  3.586759380350425*^9, 3.5867594014706326`*^9}}],

Cell["\<\
OK.
Now we consider the division of functions:\
\>", "Text",
 CellChangeTimes->{{3.586759537684424*^9, 3.5867595598446913`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"fqh", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"u", ",", "v"}], "]"}], "/", 
   RowBox[{"h", "[", 
    RowBox[{"u", ",", "v"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5866959701316166`*^9, 3.586695979398033*^9}, {
  3.586696045042948*^9, 3.5866960986290426`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "fqh", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5866961062886553`*^9, 3.586696119720279*^9}}],

Cell[BoxData[
 RowBox[{"Together", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6091384818996496`*^9, 3.6091384897300973`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"qh", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{"1", "/", 
   RowBox[{"h", "[", 
    RowBox[{"u", ",", "v"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5866962450040994`*^9, 3.5866962911177807`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "qh", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.586696304674204*^9, 3.586696321225833*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"hsf", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"h", "[", 
    RowBox[{"u", ",", "v"}], "]"}], "*", 
   RowBox[{"sf", "[", 
    RowBox[{"u", ",", "v"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.586176146106014*^9, 3.5861761604438343`*^9}, {
   3.586176212667821*^9, 3.586176219555215*^9}, {3.5861764457641535`*^9, 
   3.586176448811328*^9}, {3.586179992161268*^9, 3.586180045256305*^9}, 
   3.5861802843079786`*^9, {3.586180325530336*^9, 3.586180335425902*^9}, 
   3.5861809246586046`*^9, {3.5861813951225133`*^9, 3.5861814146896324`*^9}, {
   3.586182028978768*^9, 3.5861820344580812`*^9}}],

Cell[BoxData[
 RowBox[{"hsf", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5861765616997843`*^9, 3.586176568130152*^9}, {
   3.5861766180670085`*^9, 3.5861766375791245`*^9}, 3.5861814251452303`*^9, 
   3.5861820437846146`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "hsf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.586175998007543*^9, 3.586176056539891*^9}, 
   3.5861761728835454`*^9, {3.5861806944574375`*^9, 3.5861807024008923`*^9}, 
   3.586181021650152*^9, 3.586181462794384*^9, 3.5861818073140893`*^9, 
   3.586182054338218*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"hsf", ",", "u", ",", "v"}], "]"}], "[", 
  RowBox[{"{", 
   RowBox[{"s", ",", "t"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.586175998007543*^9, 3.586176056539891*^9}, 
   3.5861761728835454`*^9, {3.5861806944574375`*^9, 3.5861807024008923`*^9}, 
   3.586181021650152*^9, 3.586181462794384*^9, 3.5861818073140893`*^9, 
   3.586182054338218*^9, 3.586182151961802*^9, {3.5867603175990324`*^9, 
   3.5867603279346237`*^9}}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", "-", " ", 
   RowBox[{
    RowBox[{"h", "[", 
     RowBox[{"u", ",", "v"}], "]"}], "*", 
    RowBox[{
     RowBox[{"d", "[", 
      RowBox[{"sf", ",", "u", ",", "v"}], "]"}], "[", 
     RowBox[{"{", 
      RowBox[{"s", ",", "t"}], "}"}], "]"}]}], "-", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", 
      RowBox[{"h", ",", "u", ",", "v"}], "]"}], "[", 
     RowBox[{"{", 
      RowBox[{"s", ",", "t"}], "}"}], "]"}], "*", 
    RowBox[{"sf", "[", 
     RowBox[{"u", ",", "v"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.586170588470135*^9, 3.586170596314584*^9}, {
   3.5861804177446103`*^9, 3.5861805309620857`*^9}, {3.5861805934906626`*^9, 
   3.586180596185817*^9}, {3.586180723066074*^9, 3.586180768243658*^9}, {
   3.586180807948929*^9, 3.5861808189555583`*^9}, {3.5861808559476743`*^9, 
   3.586180862659058*^9}, {3.5861811085951247`*^9, 3.5861811264421453`*^9}, {
   3.5861814751470904`*^9, 3.5861814955872593`*^9}, {3.586182070963169*^9, 
   3.5861820774905424`*^9}, {3.586182197803424*^9, 3.586182239571813*^9}, {
   3.586182353181311*^9, 3.5861823536443377`*^9}, {3.5867603471997256`*^9, 
   3.586760522599758*^9}, {3.5867605830332146`*^9, 3.5867606005202146`*^9}, 
   3.5867606439676995`*^9, {3.58676070726432*^9, 3.586760716646857*^9}, 
   3.5867607994635935`*^9, {3.6177070210041137`*^9, 3.617707032411766*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["1.3.3. Exterior Product", "Subsubsection",
 CellChangeTimes->{{3.586665161894202*^9, 3.586665168125559*^9}, {
   3.5872725143584003`*^9, 3.587272520677761*^9}, {3.60913869538486*^9, 
   3.6091387007281656`*^9}, 3.6092316026384597`*^9, {3.6093094046875467`*^9, 
   3.6093094080137367`*^9}, 3.609309574326249*^9, {3.6095608566429873`*^9, 
   3.6095608600281935`*^9}, 3.6177078269291887`*^9, 3.6177190550258956`*^9, 
   3.62744137111684*^9, 3.627442434835908*^9}],

Cell["\<\
The exterior product of  differential forms is obtained by the built-in \
\>", "Text",
 CellChangeTimes->{{3.5866638658290715`*^9, 3.586663901221096*^9}, {
   3.609308184685766*^9, 3.6093081991985965`*^9}, 3.615117640053796*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "TensorWedge"}]], "Input",
 CellChangeTimes->{{3.586614301966012*^9, 3.5866143148672347`*^9}, {
  3.6089625997735806`*^9, 3.608962603891988*^9}, {3.608963615959365*^9, 
  3.6089636229637775`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox["\<\"\\!\\(\\*RowBox[{\\\"TensorWedge\\\", \\\"[\\\", \
RowBox[{SubscriptBox[StyleBox[\\\"tensor\\\", \\\"TI\\\"], \
StyleBox[\\\"1\\\", \\\"TR\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"tensor\
\\\", \\\"TI\\\"], StyleBox[\\\"2\\\", \\\"TR\\\"]], \\\",\\\", StyleBox[\\\"\
\[Ellipsis]\\\", \\\"TR\\\"]}], \\\"]\\\"}]\\) represents the antisymmetrized \
tensor product of the \\!\\(\\*SubscriptBox[StyleBox[\\\"tensor\\\", \\\"TI\\\
\"], StyleBox[\\\"i\\\", \\\"TI\\\"]]\\).\"\>", "MSG"], "\[NonBreakingSpace]", 
  ButtonBox[
   StyleBox["\[RightSkeleton]", "SR"],
   Active->True,
   BaseStyle->"Link",
   ButtonData->"paclet:ref/TensorWedge"]}]], "Print", "PrintUsage",
 CellChangeTimes->{3.6274414808318324`*^9},
 CellTags->"Info3627445080-5102015"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Context", "[", "TensorWedge", "]"}]], "Input",
 CellChangeTimes->{{3.586614326286455*^9, 3.5866143340708685`*^9}, {
   3.6089625399630756`*^9, 3.6089625472170877`*^9}, 3.609139331541246*^9}],

Cell[BoxData["\<\"System`\"\>"], "Output",
 CellChangeTimes->{3.6274415099258833`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorWedge", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "h", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{"d", "[", "f", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{
  3.5866075367049265`*^9, {3.5866078946943555`*^9, 3.586607923382806*^9}, {
   3.586609053713991*^9, 3.586609060437603*^9}, {3.5866091609953794`*^9, 
   3.586609167874992*^9}, 3.5866142301590858`*^9, {3.6089625185130377`*^9, 
   3.6089625203070407`*^9}, {3.6089637807424545`*^9, 
   3.6089637866548653`*^9}, {3.608963958769967*^9, 3.608964000765241*^9}, 
   3.609139366379239*^9, 3.6148618495663223`*^9, 3.6149177823421445`*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"StructuredArray", "[", 
   RowBox[{"\<\"SymmetrizedArray\"\>", ",", 
    RowBox[{"{", 
     RowBox[{"2", ",", "2"}], "}"}], ",", "\<\"-Structured Data-\"\>"}], "]"}],
  False,
  Editable->False]], "Output",
 CellChangeTimes->{3.6274415166026955`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Normal", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6089638230497293`*^9, 3.608963830865343*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         SuperscriptBox["h", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "-", 
      RowBox[{
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}]}], ",", "0"}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.6274415240751085`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "Normal"}]], "Input",
 CellChangeTimes->{{3.609139383471216*^9, 3.6091393864273853`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox["\<\"\\!\\(\\*RowBox[{\\\"Normal\\\", \\\"[\\\", StyleBox[\\\"expr\
\\\", \\\"TI\\\"], \\\"]\\\"}]\\) converts \\!\\(\\*StyleBox[\\\"expr\\\", \\\
\"TI\\\"]\\) to a normal expression, from a variety of special forms. \"\>", 
   "MSG"], "\[NonBreakingSpace]", 
  ButtonBox[
   StyleBox["\[RightSkeleton]", "SR"],
   Active->True,
   BaseStyle->"Link",
   ButtonData->"paclet:ref/Normal"]}]], "Print", "PrintUsage",
 CellChangeTimes->{3.6274415439027433`*^9},
 CellTags->"Info3627445143-5102015"]
}, Closed]],

Cell["\<\
The result is an alternating bilinear form in dimension two with  the only \
strict coordinate\
\>", "Text",
 CellChangeTimes->{{3.5867611712118564`*^9, 3.586761221964759*^9}, {
  3.6092151598757515`*^9, 3.609215169273289*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Det", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", "h", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{
     RowBox[{"d", "[", "f", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "}"}], "]"}]], "Input",
 CellChangeTimes->{
  3.5866075367049265`*^9, {3.5866078946943555`*^9, 3.586607923382806*^9}, {
   3.586609053713991*^9, 3.586609060437603*^9}, {3.5866091609953794`*^9, 
   3.586609167874992*^9}, 3.5866142301590858`*^9, {3.5867609939407167`*^9, 
   3.586761018799139*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SuperscriptBox["h", 
      TagBox[
       RowBox[{"(", 
        RowBox[{"0", ",", "1"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], " ", 
   RowBox[{
    SuperscriptBox["f", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
  RowBox[{
   RowBox[{
    SuperscriptBox["f", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox["h", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}]}]], "Output",
 CellChangeTimes->{3.6274419662890854`*^9}]
}, Closed]],

Cell["We defined in tensalg.m", "Text",
 CellChangeTimes->{{3.6091400809761114`*^9, 3.6091400842482986`*^9}, {
  3.6095608806670294`*^9, 3.609560890167446*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"extprod", "[", "w___", "]"}], " ", ":=", " ", 
  RowBox[{"Normal", "[", 
   RowBox[{"TensorWedge", "[", "w", "]"}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.609140140859536*^9, 3.6091402137787075`*^9}, 
   3.609140254443033*^9, {3.609142536672043*^9, 3.6091425573932285`*^9}, {
   3.609143273953213*^9, 3.609143278689484*^9}, 3.6095608991218615`*^9, 
   3.614861849581922*^9, 3.614917782357744*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "extprod"}]], "Input",
 CellChangeTimes->{{3.609561048024123*^9, 3.6095610523765306`*^9}}],

Cell[BoxData[
 StyleBox["\<\"extprod[w1,w2,...] yields the exterior product of \
antisymmetric forms or \\nmultivectors represented by their Arrays of \
coordinates. It avoids the concept of tensor.\"\>", "MSG"]], "Print", \
"PrintUsage",
 CellChangeTimes->{3.6274420049771533`*^9},
 CellTags->"Info3627445604-5102015"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"extprod", "[", "]"}]], "Input",
 CellChangeTimes->{3.609143301679799*^9}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.6274420217159824`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Context", "[", "extprod", "]"}]], "Input",
 CellChangeTimes->{{3.615035389346508*^9, 3.615035402573264*^9}}],

Cell[BoxData["\<\"tensalg`\"\>"], "Output",
 CellChangeTimes->{3.6274420269575915`*^9}]
}, Closed]],

Cell["\<\
A symbolic function isn\[CloseCurlyQuote]t recognized as a 0-form, extprod is \
not evaluated:\
\>", "Text",
 CellChangeTimes->{{3.609215433394396*^9, 3.6092155061055546`*^9}, {
  3.6150338443951416`*^9, 3.6150338459312296`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"extprod", "[", 
  RowBox[{"f", "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{3.6092153726999245`*^9, 3.6150338561808157`*^9}],

Cell[BoxData[
 RowBox[{"TensorWedge", "[", 
  RowBox[{"f", "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Output",
 CellChangeTimes->{3.6274420495308313`*^9}]
}, Closed]],

Cell["A concrete function is treated as a 0-form:", "Text",
 CellChangeTimes->{{3.60921561144158*^9, 3.609215640329232*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"extprod", "[", 
  RowBox[{"Cos", "[", 
   RowBox[{"u", "*", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{
  3.6092153726999245`*^9, {3.6092155294898925`*^9, 3.609215560515667*^9}}],

Cell[BoxData[
 RowBox[{"Cos", "[", 
  RowBox[{"u", " ", "v"}], "]"}]], "Output",
 CellChangeTimes->{3.627442061418052*^9}]
}, Closed]],

Cell["Again", "Text",
 CellChangeTimes->{{3.6092156749692135`*^9, 3.609215677897381*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"extprod", "[", 
  RowBox[{
   RowBox[{"h", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{"d", "[", "f", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6091402188970003`*^9, 3.609140224441317*^9}, {
   3.609142821985362*^9, 3.6091428627446938`*^9}, 3.609142910601431*^9, 
   3.615033913997123*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"h", "[", 
   RowBox[{"u", ",", "v"}], "]"}], "\[TensorWedge]", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     SuperscriptBox["f", 
      TagBox[
       RowBox[{"(", 
        RowBox[{"1", ",", "0"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{
     SuperscriptBox["f", 
      TagBox[
       RowBox[{"(", 
        RowBox[{"0", ",", "1"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "}"}]}]], "Output",
 CellChangeTimes->{3.6274420716360703`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"extprod", "[", 
  RowBox[{
   RowBox[{"Cos", "[", 
    RowBox[{"u", "*", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{"d", "[", "f", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6091402188970003`*^9, 3.609140224441317*^9}, {
   3.609142821985362*^9, 3.6091428627446938`*^9}, 3.609142910601431*^9, {
   3.609143190727453*^9, 3.609143205793315*^9}, {3.609215712988388*^9, 
   3.609215713740431*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", 
     RowBox[{"u", " ", "v"}], "]"}], " ", 
    RowBox[{
     SuperscriptBox["f", 
      TagBox[
       RowBox[{"(", 
        RowBox[{"1", ",", "0"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], ",", 
   RowBox[{
    RowBox[{"Cos", "[", 
     RowBox[{"u", " ", "v"}], "]"}], " ", 
    RowBox[{
     SuperscriptBox["f", 
      TagBox[
       RowBox[{"(", 
        RowBox[{"0", ",", "1"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.627442076206878*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"extprod", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "h", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", "%"}], "]"}]], "Input",
 CellChangeTimes->{{3.6091433804543047`*^9, 3.6091434103450146`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{"Cos", "[", 
         RowBox[{"u", " ", "v"}], "]"}]}], " ", 
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "+", 
      RowBox[{
       RowBox[{"Cos", "[", 
        RowBox[{"u", " ", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"Cos", "[", 
        RowBox[{"u", " ", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "-", 
      RowBox[{
       RowBox[{"Cos", "[", 
        RowBox[{"u", " ", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}]}], ",", "0"}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.6274420855824947`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", "\[Equal]", " ", 
   RowBox[{
    RowBox[{"Cos", "[", 
     RowBox[{"u", "*", "v"}], "]"}], "*", 
    RowBox[{"extprod", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"d", "[", "h", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{
       RowBox[{"d", "[", "f", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6092158552315235`*^9, 3.6092159353341055`*^9}, {
  3.6092159856369824`*^9, 3.6092159966696134`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6274421250193644`*^9}]
}, Closed]],

Cell["\<\
The exterior product is Flat (associative). Clearly, the exterior product of \
three 1-forms of dimension 2 is zero:\
\>", "Text",
 CellChangeTimes->{{3.609216249538077*^9, 3.609216334377929*^9}, {
  3.6093086665503273`*^9, 3.6093086714706087`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"extprod", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "g", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"extprod", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"d", "[", "h", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      RowBox[{"d", "[", "f", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6092160822505083`*^9, 3.609216157741826*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6274421359237833`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"extprod", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "g", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{"d", "[", "h", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{"d", "[", "f", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6092160822505083`*^9, 3.609216157741826*^9}, {
  3.609216355858158*^9, 3.6092163783164425`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627442143661397*^9}]
}, Closed]],

Cell["Now we consider 1-forms in dimension 3:", "Text",
 CellChangeTimes->{{3.609216789849981*^9, 3.609216812481275*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"extprod", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "h", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
   RowBox[{
    RowBox[{"d", "[", "f", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6091402188970003`*^9, 3.609140224441317*^9}, {
   3.609142821985362*^9, 3.6091428627446938`*^9}, 3.609142910601431*^9, {
   3.609142950990741*^9, 3.609142964984541*^9}, {3.6091435169461117`*^9, 
   3.609143530265874*^9}, 3.615034059357437*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         SuperscriptBox["h", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}]}]}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         SuperscriptBox["h", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}]}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], "-", 
      RowBox[{
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}]}]}], ",", "0", ",", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         SuperscriptBox["h", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}]}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], "-", 
      RowBox[{
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}]}]}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], "-", 
      RowBox[{
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}]}]}], ",", "0"}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.6274421488094053`*^9}]
}, Closed]],

Cell["Again we verify the associativity:", "Text",
 CellChangeTimes->{{3.609216843546052*^9, 3.609216862265123*^9}, 
   3.6093087120219283`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"a21", " ", "=", 
   RowBox[{"extprod", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"d", "[", "h", "]"}], "[", 
      RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
     RowBox[{"extprod", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"d", "[", "g", "]"}], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
       RowBox[{
        RowBox[{"d", "[", "f", "]"}], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], "]"}]}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6091433804543047`*^9, 3.6091434103450146`*^9}, {
  3.609143545472743*^9, 3.6091435497609887`*^9}, {3.609143585873054*^9, 
  3.609143586697101*^9}, {3.609216489982829*^9, 3.6092165562066174`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"a111", "=", " ", 
   RowBox[{"extprod", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"d", "[", "h", "]"}], "[", 
      RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
     RowBox[{
      RowBox[{"d", "[", "g", "]"}], "[", 
      RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
     RowBox[{
      RowBox[{"d", "[", "f", "]"}], "[", 
      RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6091440266972675`*^9, 3.609144079570292*^9}, {
  3.6091441215396924`*^9, 3.609144165396201*^9}, {3.6091442799737544`*^9, 
  3.6091443095794477`*^9}, {3.6092166697831135`*^9, 3.6092166863090587`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"a111", " ", "\[Equal]", " ", "a21"}], "]"}]], "Input",
 CellChangeTimes->{{3.6092166921003895`*^9, 3.6092167377400002`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.627442167248638*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"a111", "[", 
     RowBox[{"[", 
      RowBox[{"1", ",", "2", ",", "3"}], "]"}], "]"}], " ", "-", " ", 
    RowBox[{"a21", "[", 
     RowBox[{"[", 
      RowBox[{"1", ",", "2", ",", "3"}], "]"}], "]"}]}], ")"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6091441764198313`*^9, 3.6091442160330973`*^9}, {
  3.609216964916994*^9, 3.609217029803705*^9}, {3.609217095364455*^9, 
  3.609217187740739*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.627442172724248*^9}]
}, Closed]],

Cell["The operation ", "Text",
 CellChangeTimes->{{3.609139990103914*^9, 3.609139996104257*^9}}],

Cell[BoxData[
 RowBox[{"?", "wedgeform"}]], "Input",
 CellChangeTimes->{{3.60913964229502*^9, 3.609139656209816*^9}}],

Cell["produces the corresponding antisymmetric tensor :", "Text",
 CellChangeTimes->{{3.6091400076729183`*^9, 3.6091400438659887`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"wedgeform", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "h", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{"d", "[", "f", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6091397202424784`*^9, 3.609139724370715*^9}}],

Cell[BoxData[
 RowBox[{"tensor", "[", 
  RowBox[{
   RowBox[{"cova", "[", 
    RowBox[{"tp", "[", 
     RowBox[{
      RowBox[{"tensor", "[", 
       RowBox[{"tensalg`Private`xx$5640", ",", "1", ",", "0", ",", "2"}], 
       "]"}], ",", 
      RowBox[{"tensor", "[", 
       RowBox[{"tensalg`Private`xx$5641", ",", "1", ",", "0", ",", "2"}], 
       "]"}]}], "]"}], "]"}], ",", "2", ",", "0", ",", "2"}], "]"}]], "Output",
 CellChangeTimes->{3.62744218539147*^9}]
}, Closed]],

Cell["The coordinates of this tensor", "Text",
 CellChangeTimes->{{3.609217804145995*^9, 3.6092178311615405`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"coords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.609139746633988*^9, 3.6091397593617163`*^9}, {
   3.6092174664596806`*^9, 3.6092174781203475`*^9}, 3.6092176008673687`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "0", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], " ", 
      RowBox[{
       SuperscriptBox["f", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "+", 
     RowBox[{
      RowBox[{
       SuperscriptBox["f", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox["h", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["h", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox["f", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "-", 
     RowBox[{
      RowBox[{
       SuperscriptBox["f", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox["h", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}]}], "}"}], ",", 
   RowBox[{"{", "0", "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6274421989946938`*^9}]
}, Closed]],

Cell["coincide with the coordinates of", "Text",
 CellChangeTimes->{{3.609217843090223*^9, 3.6092178651704855`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"extprod", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "h", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{"d", "[", "f", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6092175258450775`*^9, 3.609217530150324*^9}, 
   3.6092175690455484`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         SuperscriptBox["h", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "-", 
      RowBox[{
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}]}], ",", "0"}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.6274422180111275`*^9}]
}, Closed]],

Cell["up to the Array structure, indeed:", "Text",
 CellChangeTimes->{{3.6092178773931847`*^9, 3.609217882329467*^9}, {
  3.6092183033775496`*^9, 3.609218325625822*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Flatten", "[", "%", "]"}], " ", "\[Equal]", " ", 
  RowBox[{"Flatten", "[", "%%", "]"}]}]], "Input",
 CellChangeTimes->{{3.6092176666411304`*^9, 3.6092176953477726`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.627442234515956*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"coords", "[", 
  RowBox[{"wedgeform", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", "h", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{
     RowBox[{"d", "[", "f", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "0", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], " ", 
      RowBox[{
       SuperscriptBox["f", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "+", 
     RowBox[{
      RowBox[{
       SuperscriptBox["f", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox["h", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["h", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox["f", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "-", 
     RowBox[{
      RowBox[{
       SuperscriptBox["f", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox["h", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}]}], "}"}], ",", 
   RowBox[{"{", "0", "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6274422404127665`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Dimensions", "[", 
  RowBox[{"coords", "[", 
   RowBox[{"wedgeform", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"d", "[", "h", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      RowBox[{"d", "[", "f", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6092185747600718`*^9, 3.609218607205928*^9}, {
  3.609218654523634*^9, 3.6092186639891753`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4", ",", "1"}], "}"}]], "Output",
 CellChangeTimes->{3.6274422448275747`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Dimensions", "[", 
  RowBox[{"extprod", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", "h", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{
     RowBox[{"d", "[", "f", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6092185747600718`*^9, 3.609218607205928*^9}, {
   3.609218654523634*^9, 3.6092186930178356`*^9}, 3.609218730717992*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "2"}], "}"}]], "Output",
 CellChangeTimes->{3.627442265575611*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "astb", "]"}], ";", 
  RowBox[{
   RowBox[{"astb", "[", 
    RowBox[{"i_", ",", "j_"}], "]"}], ":=", " ", 
   RowBox[{"extprod", "[", 
    RowBox[{
     RowBox[{"stb", "[", 
      RowBox[{"i", ",", "2"}], "]"}], ",", 
     RowBox[{"stb", "[", 
      RowBox[{"j", ",", "2"}], "]"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.609218978260151*^9, 3.609219025731866*^9}, {
   3.6092191432765894`*^9, 3.609219180020691*^9}, {3.609219259397231*^9, 
   3.6092192606123004`*^9}, 3.60921947547759*^9, {3.609219528924647*^9, 
   3.6092195414523635`*^9}, {3.6150350891103354`*^9, 3.6150351090304747`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"astb", "[", 
  RowBox[{"1", ",", "2"}], "]"}]], "Input",
 CellChangeTimes->{{3.615034993116845*^9, 3.6150350090367556`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "0"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6274422774784317`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"extprod", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "h", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{"d", "[", "f", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         SuperscriptBox["h", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "-", 
      RowBox[{
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}]}], ",", "0"}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.627442282688841*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", " ", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"extprod", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"d", "[", "h", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], ",", 
        RowBox[{
         RowBox[{"d", "[", "f", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"i", ",", "j"}], "]"}], "]"}], "*", 
     RowBox[{"astb", "[", 
      RowBox[{"i", ",", "j"}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6092190678682756`*^9, 3.6092191330150023`*^9}, {
   3.609219191174329*^9, 3.6092192513927727`*^9}, {3.6092192931361604`*^9, 
   3.6092192956713057`*^9}, {3.6092194987919235`*^9, 3.60921950030301*^9}, {
   3.609219600768756*^9, 3.6092196156726084`*^9}, {3.615034717386074*^9, 
   3.615034718041112*^9}, 3.6150348346557817`*^9, 3.6150349412718797`*^9, 
   3.6150351628795547`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "2"}], " ", 
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "+", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["h", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}]}], ",", "0"}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.627442287244049*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"extprod", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"d", "[", "h", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      RowBox[{"d", "[", "f", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "]"}], " ", "\[Equal]", "  ", 
   RowBox[{
    RowBox[{"Sum", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"extprod", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"d", "[", "h", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], ",", 
          RowBox[{
           RowBox[{"d", "[", "f", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
        RowBox[{"[", 
         RowBox[{"i", ",", "j"}], "]"}], "]"}], "*", 
       RowBox[{"astb", "[", 
        RowBox[{"i", ",", "j"}], "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "/", "2"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6092190678682756`*^9, 3.6092191330150023`*^9}, {
   3.609219191174329*^9, 3.6092192513927727`*^9}, {3.6092192931361604`*^9, 
   3.6092192956713057`*^9}, {3.6092194987919235`*^9, 3.60921950030301*^9}, {
   3.609219600768756*^9, 3.6092196156726084`*^9}, {3.615034717386074*^9, 
   3.615034718041112*^9}, 3.615034776780471*^9, {3.6150351991356287`*^9, 
   3.6150352002156906`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6274423180385027`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"extprod", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"d", "[", "h", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      RowBox[{"d", "[", "f", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "]"}], " ", "-", 
   RowBox[{
    RowBox[{
     RowBox[{"extprod", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"d", "[", "h", "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{
        RowBox[{"d", "[", "f", "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
     RowBox[{"[", 
      RowBox[{"1", ",", "2"}], "]"}], "]"}], "*", 
    RowBox[{"astb", "[", 
     RowBox[{"1", ",", "2"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6092190678682756`*^9, 3.6092191330150023`*^9}, {
   3.609219191174329*^9, 3.6092192513927727`*^9}, {3.6092192931361604`*^9, 
   3.6092192956713057`*^9}, {3.6092194987919235`*^9, 3.60921950030301*^9}, {
   3.609219600768756*^9, 3.6092197225507216`*^9}, 3.6092198184562073`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627442325089715*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"extprod", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "u", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{"d", "[", "v", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.609219902395008*^9, 3.609219945660483*^9}, {
   3.609221187240497*^9, 3.609221221451454*^9}, {3.609308783305005*^9, 
   3.609308810152541*^9}, {3.6093089621622353`*^9, 3.609308967298529*^9}, 
   3.6093090292660737`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "0"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6274423400813417`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"extprod", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", 
     RowBox[{"x", "[", "1", "]"}], "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}]}], "]"}], ",", 
   RowBox[{
    RowBox[{"d", "[", 
     RowBox[{"x", "[", "2", "]"}], "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.609219902395008*^9, 3.609219945660483*^9}, {
   3.609221187240497*^9, 3.609221221451454*^9}, {3.609308783305005*^9, 
   3.609308810152541*^9}, {3.6093089621622353`*^9, 3.609308967298529*^9}, 
   3.6093090292660737`*^9, {3.6093091109147434`*^9, 3.6093091850349827`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "0"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6274423550105677`*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["1.3.4. The Exterior Differential", "Subsubsection",
 CellChangeTimes->{{3.5866649932855587`*^9, 3.586665013117693*^9}, {
   3.5872725258470573`*^9, 3.587272530654332*^9}, {3.5898752162352247`*^9, 
   3.589875218169628*^9}, 3.5899533142260494`*^9, {3.609309548254758*^9, 
   3.6093095610144877`*^9}, 3.6095615905629835`*^9, 3.6177190633933744`*^9, 
   3.6177712155926075`*^9, 3.6274424857387977`*^9, 3.627443068992622*^9}],

Cell[TextData[{
 "We are going to define  the  ",
 StyleBox["exterior differential",
  FontColor->RGBColor[0, 0, 1]],
 " of a differential form in two variables. Of course, the exterior \
differential can be defined more generally using tensor calculus. We restrict \
to the elementary special case which suffices for the applications to surface \
theory and corresponds to the presentation in [",
 ButtonBox["B-R",
  BaseStyle->"Hyperlink",
  ButtonData->"B-R"],
 "]."
}], "Text",
 CellChangeTimes->{{3.5725350648802934`*^9, 3.572535066877097*^9}, {
  3.5725954538048487`*^9, 3.5725954928361173`*^9}, {3.572610955875272*^9, 
  3.5726109814905167`*^9}, {3.5726191833921223`*^9, 3.5726191938753405`*^9}, {
  3.5867612571647725`*^9, 3.5867613047324934`*^9}, {3.615114957662568*^9, 
  3.6151152495158386`*^9}, {3.6151153013033066`*^9, 3.615115302223708*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"dd", "[", "h_", "]"}], "[", 
     RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"n", "=", "1"}], ",", 
        RowBox[{"pp", "=", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}]}], ",", "i"}], "}"}], ",", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"Head", "[", 
            RowBox[{"h", "[", 
             RowBox[{"u", ",", "v"}], "]"}], "]"}], "\[Equal]", "List"}], "&&", 
          RowBox[{
           RowBox[{"Length", "[", 
            RowBox[{"h", "[", 
             RowBox[{"u", ",", "v"}], "]"}], "]"}], "\[Equal]", "2"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Return", "[", 
          RowBox[{
           RowBox[{"D", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"h", "[", 
               RowBox[{"u", ",", "v"}], "]"}], "[", 
              RowBox[{"[", "2", "]"}], "]"}], ",", "u"}], "]"}], "-", " ", 
           RowBox[{"D", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"h", "[", 
               RowBox[{"u", ",", "v"}], "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", "v"}], "]"}]}], "]"}]}], 
        "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"MatrixQ", "[", 
          RowBox[{"h", "[", 
           RowBox[{"u", ",", "v"}], "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"pp", " ", "=", " ", 
           RowBox[{"Dimensions", "[", 
            RowBox[{"h", "[", 
             RowBox[{"u", ",", "v"}], "]"}], "]"}]}], ";", " ", 
          RowBox[{"n", "=", 
           RowBox[{"pp", "[", 
            RowBox[{"[", "1", "]"}], "]"}]}]}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"pp", "[", 
           RowBox[{"[", "2", "]"}], "]"}], "\[Equal]", "2"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Return", "[", 
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"D", "[", 
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"h", "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "[", 
                 RowBox[{"[", "i", "]"}], "]"}], "[", 
                RowBox[{"[", "2", "]"}], "]"}], ",", "u"}], "]"}], "-", " ", 
             RowBox[{"D", "[", 
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"h", "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "[", 
                 RowBox[{"[", "i", "]"}], "]"}], "[", 
                RowBox[{"[", "1", "]"}], "]"}], ",", "v"}], "]"}]}], ",", 
            RowBox[{"{", 
             RowBox[{"i", ",", "n"}], "}"}]}], "]"}], "]"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Print", "[", "\"\<invalid argument\>\"", "]"}], ";", 
       RowBox[{"Return", "[", "]"}]}]}], "]"}]}], ";"}], 
  "\n"}], "\[IndentingNewLine]", 
 RowBox[{"dd", "/:", " ", 
  RowBox[{
   RowBox[{"dd", "[", 
    RowBox[{"h1_", "+", "h2_"}], "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"dd", "[", "h1", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], " ", "+", 
   RowBox[{
    RowBox[{"dd", "[", "h2", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}]}]}], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.586611033529069*^9, 3.5866110693779316`*^9}, {
   3.5866111324644423`*^9, 3.586611457584614*^9}, 3.586611520499524*^9, {
   3.586611943603467*^9, 3.5866119702327137`*^9}, {3.586614520818796*^9, 
   3.5866145367152243`*^9}, 3.586614577431296*^9, 3.586614609473752*^9, {
   3.586762054171359*^9, 3.5867620926175575`*^9}, {3.5867621358820324`*^9, 
   3.5867623292970953`*^9}, {3.5867624177691555`*^9, 3.586762433465053*^9}, {
   3.5867760440221004`*^9, 3.586776094448984*^9}, {3.5867769739352884`*^9, 
   3.5867769950704975`*^9}, {3.589892593479661*^9, 3.589892594446862*^9}, {
   3.5898926941154375`*^9, 3.5898926998406477`*^9}, {3.589892800195624*^9, 
   3.589892805359233*^9}, {3.614861849644322*^9, 3.614861849659922*^9}, 
   3.6148635015156236`*^9, 3.6149177824045444`*^9, {3.615098252957732*^9, 
   3.615098275624572*^9}, 3.6150983436406913`*^9, {3.6151005145716925`*^9, 
   3.6151005965439014`*^9}, 3.6151006554184046`*^9, {3.6151008492175455`*^9, 
   3.6151008715411844`*^9}, {3.615100908201249*^9, 3.615101079286749*^9}, {
   3.6151012150170536`*^9, 3.615101262846738*^9}, {3.6151037500098104`*^9, 
   3.6151037619906316`*^9}, {3.617774172939002*^9, 3.6177741956526413`*^9}, 
   3.622627624325942*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "dd"}]], "Input",
 CellChangeTimes->{{3.6150981806204047`*^9, 3.615098198919237*^9}, {
   3.6150983764163485`*^9, 3.615098563881878*^9}, {3.615098632287998*^9, 
   3.615098639885212*^9}, 3.6150986885572968`*^9, {3.622627709018491*^9, 
   3.622627730702529*^9}}],

Cell[BoxData[
 StyleBox["\<\"If h[u,v] is a differential form in two variables u,v or a \
vector of such \\ndifferential forms then dd[h][u,v] is its exterior \
differential.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.6274425416336956`*^9},
 CellTags->"Info3627446141-5102015"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Context", "[", "dd", "]"}]], "Input",
 CellChangeTimes->{{3.622627671828026*^9, 3.6226276919520607`*^9}}],

Cell[BoxData["\<\"eudiffgeov2`\"\>"], "Output",
 CellChangeTimes->{3.627105923160309*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"dd", "[", 
    RowBox[{"h1_", "+", "h2_"}], "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], "-", " ", 
  RowBox[{
   RowBox[{"dd", "[", "h1", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], " ", "+", 
  RowBox[{
   RowBox[{"dd", "[", "h2", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{3.6226948348296323`*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"invalid argument\"\>"], "Print",
 CellChangeTimes->{3.6274425537237167`*^9}],

Cell[BoxData["\<\"invalid argument\"\>"], "Print",
 CellChangeTimes->{3.627442553739317*^9}],

Cell[BoxData["\<\"invalid argument\"\>"], "Print",
 CellChangeTimes->{3.627442553739317*^9}],

Cell[BoxData["\<\"invalid argument\"\>"], "Print",
 CellChangeTimes->{3.627442553739317*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"2", " ", "Null"}]], "Output",
 CellChangeTimes->{3.627442553739317*^9}]
}, Closed]],

Cell["\<\
In general context, the exterior differential of a differential form is an \
alternating 2-form whose only strict coordinate is just defined.\
\>", "Text",
 CellChangeTimes->{{3.586761454700071*^9, 3.5867615540287523`*^9}}],

Cell[TextData[{
 StyleBox["Hint.",
  FontColor->RGBColor[0, 0, 1]],
 " The exterior differential acts on explicitly defined differential forms \
only, it doesn\[CloseCurlyQuote]t act on pairs of functions, e.g."
}], "Text",
 CellChangeTimes->{{3.6278968177605553`*^9, 3.6278969328247585`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dd", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Cos", "[", 
     RowBox[{"u", "*", "v"}], "]"}], ",", 
    RowBox[{"Sin", "[", 
     RowBox[{"u", "*", "v"}], "]"}]}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.6278969362431645`*^9, 3.627896963779213*^9}}],

Cell[BoxData[
 RowBox[{"dd", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Cos", "[", 
     RowBox[{"u", " ", "v"}], "]"}], ",", 
    RowBox[{"Sin", "[", 
     RowBox[{"u", " ", "v"}], "]"}]}], "}"}], "]"}]], "Output",
 CellChangeTimes->{3.627896965167616*^9}]
}, Closed]],

Cell["With", "Text",
 CellChangeTimes->{{3.6278969793676405`*^9, 3.627896980709243*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"h1", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Cos", "[", 
     RowBox[{"u", "*", "v"}], "]"}], ",", 
    RowBox[{"Sin", "[", 
     RowBox[{"u", "*", "v"}], "]"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.586611682661809*^9, 3.5866117023490434`*^9}, 
   3.589952099935517*^9, {3.615102028135137*^9, 3.6151020402719584`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Cos", "[", 
    RowBox[{"u", " ", "v"}], "]"}], ",", 
   RowBox[{"Sin", "[", 
    RowBox[{"u", " ", "v"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627105934158328*^9, 3.6274425875445766`*^9, 
  3.627896987666855*^9}]
}, Closed]],

Cell["we get the wanted result", "Text",
 CellChangeTimes->{{3.62789700169328*^9, 3.6278970098384943`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "h1", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6278970212577143`*^9, 3.627897029449729*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"v", " ", 
   RowBox[{"Cos", "[", 
    RowBox[{"u", " ", "v"}], "]"}]}], "+", 
  RowBox[{"u", " ", 
   RowBox[{"Sin", "[", 
    RowBox[{"u", " ", "v"}], "]"}]}]}]], "Output",
 CellChangeTimes->{3.6278970302765303`*^9}]
}, Closed]],

Cell[TextData[{
 StyleBox["Additivity",
  FontColor->RGBColor[0, 0, 1]],
 ". Set"
}], "Text",
 CellChangeTimes->{{3.627897049887765*^9, 3.627897060310583*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"h2", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Cosh", "[", 
     RowBox[{"u", "*", "v"}], "]"}], ",", 
    RowBox[{"Sinh", "[", 
     RowBox[{"u", "*", "v"}], "]"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.589952108921133*^9, 3.589952176126051*^9}, {
   3.589952224626536*^9, 3.589952234953754*^9}, 3.615098851343583*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"h1", "+", "h2"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5899519548240623`*^9, 3.589951985041315*^9}, {
  3.5899521931612806`*^9, 3.5899521987928905`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"v", " ", 
   RowBox[{"Cos", "[", 
    RowBox[{"u", " ", "v"}], "]"}]}], "+", 
  RowBox[{"v", " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{"u", " ", "v"}], "]"}]}], "+", 
  RowBox[{"u", " ", 
   RowBox[{"Sin", "[", 
    RowBox[{"u", " ", "v"}], "]"}]}], "-", 
  RowBox[{"u", " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{"u", " ", "v"}], "]"}]}]}]], "Output",
 CellChangeTimes->{3.627105948837954*^9, 3.62744259537579*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"h", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"h1", "[", 
    RowBox[{"u", ",", "v"}], "]"}], "+", " ", 
   RowBox[{"h2", "[", 
    RowBox[{"u", ",", "v"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5899522637670045`*^9, 3.589952293578657*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"dd", "[", "h", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"dd", "[", "h1", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "+", 
   RowBox[{
    RowBox[{"dd", "[", "h2", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.589952307540682*^9, 3.5899523148414946`*^9}, {
   3.6226949498954344`*^9, 3.6226949543102417`*^9}, {3.622694987912701*^9, 
   3.6226949996127214`*^9}, 3.622695049298809*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.627105961988777*^9, 3.6274426042990055`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"u", "*", "h1"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.589952419892079*^9, 3.589952446958126*^9}}],

Cell[BoxData["\<\"invalid argument\"\>"], "Print",
 CellChangeTimes->{3.627105965795184*^9, 3.6274426191034317`*^9, 
  3.6278972099867086`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"u", "*", 
  RowBox[{"h1", "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"u", " ", 
    RowBox[{"Cos", "[", 
     RowBox[{"u", " ", "v"}], "]"}]}], ",", 
   RowBox[{"u", " ", 
    RowBox[{"Sin", "[", 
     RowBox[{"u", " ", "v"}], "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.627442679241537*^9, 3.627897213387515*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Head", "[", 
  RowBox[{"u", "*", 
   RowBox[{"h1", "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData["List"], "Output",
 CellChangeTimes->{3.6271059736263976`*^9, 3.627897218475124*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Head", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"u", "*", "h1"}], ")"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6095592950930424`*^9, 3.609559297058646*^9}, 
   3.6095593286175013`*^9, {3.615102833034151*^9, 3.6151028659502087`*^9}, {
   3.61510390696198*^9, 3.6151039161512938`*^9}}],

Cell[BoxData[
 RowBox[{"h1", " ", "u"}]], "Output",
 CellChangeTimes->{3.6271059769960036`*^9, 3.6274427100359917`*^9, 
  3.6278972220631304`*^9}]
}, Closed]],

Cell["\<\
The reason that dd[u*h1][u,v] is not evaluated is, as above, that the \
argument of dd is a compound expression, and not an explicitly defined \
differential form. To get the result first define\
\>", "Text",
 CellChangeTimes->{{3.615104041892352*^9, 3.6151042091479855`*^9}, {
   3.6278972806920333`*^9, 3.6278973790622067`*^9}, 3.627897429643496*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"h3", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", "=", " ", 
  RowBox[{"u", "*", 
   RowBox[{"h1", "[", 
    RowBox[{"u", ",", "v"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.589955054829507*^9, 3.5899550806319523`*^9}, 
   3.5899551571032867`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"u", " ", 
    RowBox[{"Cos", "[", 
     RowBox[{"u", " ", "v"}], "]"}]}], ",", 
   RowBox[{"u", " ", 
    RowBox[{"Sin", "[", 
     RowBox[{"u", " ", "v"}], "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6274427480064583`*^9, 3.6278973904814267`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "h3", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.589955092331973*^9, 3.589955101255189*^9}, 
   3.589955163858099*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"u", " ", "v", " ", 
   RowBox[{"Cos", "[", 
    RowBox[{"u", " ", "v"}], "]"}]}], "+", 
  RowBox[{"Sin", "[", 
   RowBox[{"u", " ", "v"}], "]"}], "+", 
  RowBox[{
   SuperscriptBox["u", "2"], " ", 
   RowBox[{"Sin", "[", 
    RowBox[{"u", " ", "v"}], "]"}]}]}]], "Output",
 CellChangeTimes->{3.627897393320632*^9}]
}, Closed]],

Cell[TextData[{
 StyleBox["Hint.",
  FontColor->RGBColor[0, 0, 1]],
 " To obtain the exterior differential of a compound expression \
f1{u,v]*w[u,v] + ...  first define the differential form ww[u_,v_] = \
f1{u,v]*w[u,v] + ... and then apply dd: dd[ww][u,v]"
}], "Text",
 CellChangeTimes->{{3.5899554102605314`*^9, 3.5899554428489885`*^9}, {
  3.589955473783843*^9, 3.5899557104518585`*^9}, {3.5899558438758273`*^9, 
  3.5899558469950056`*^9}, {3.6151001589606686`*^9, 3.6151001591634693`*^9}}],

Cell["\<\
The exterior differential of the differential of a function vanishes:\
\>", "Text",
 CellChangeTimes->{{3.5867617995727963`*^9, 3.5867618376129723`*^9}, 
   3.5898679504454346`*^9, 3.615100269508519*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"d", "[", "f", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.586611564975202*^9, 3.5866115972360587`*^9}, {
  3.586611662428573*^9, 3.586611668028983*^9}, {3.5866117129258623`*^9, 
  3.586611719056673*^9}, {3.5866146342933955`*^9, 3.5866146379750023`*^9}, {
  3.5866147131203337`*^9, 3.586614738844779*^9}, {3.5866148159869146`*^9, 
  3.5866148182489185`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.627442891838711*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "3"}], ";", 
  RowBox[{
   RowBox[{"d", "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6151003339210324`*^9, 3.6151003497394605`*^9}, {
  3.617770591201511*^9, 3.61777059667712*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627442895005516*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"d", "[", "sf", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5867618457264366`*^9, 3.586761895606289*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.6274429003875256`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "g3", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5867618457264366`*^9, 3.586761895606289*^9}, {
  3.5867625598042793`*^9, 3.586762588238906*^9}}],

Cell[BoxData["\<\"invalid argument\"\>"], "Print",
 CellChangeTimes->{3.6274429041783323`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"vec", "[", "x", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5867618457264366`*^9, 3.586761895606289*^9}, {
  3.5867625598042793`*^9, 3.586762616862543*^9}}],

Cell[BoxData["\<\"invalid argument\"\>"], "Print",
 CellChangeTimes->{3.6274429156287527`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ff", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", "=", 
  RowBox[{"vec", "[", "x", "]"}]}]], "Input",
 CellChangeTimes->{{3.5867628343769836`*^9, 3.5867628555821967`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}], ",", 
   RowBox[{"x", "[", "3", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627442923912367*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "ff", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5867628680829115`*^9, 3.5867628796065707`*^9}}],

Cell[BoxData["\<\"invalid argument\"\>"], "Print",
 CellChangeTimes->{3.6274429281867743`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ff", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", "=", 
  RowBox[{"vec", "[", 
   RowBox[{"x", "[", 
    RowBox[{"u", ",", "v"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5867628343769836`*^9, 3.5867628555821967`*^9}, {
  3.586775896995691*^9, 3.5867759043721128`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", "1", "]"}], ",", 
   RowBox[{
    RowBox[{"x", "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", "2", "]"}], ",", 
   RowBox[{
    RowBox[{"x", "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", "3", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627106015574871*^9, 3.6274429408851967`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "ff", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5867759283174825`*^9, 3.5867759383390555`*^9}, {
  3.5867771002355123`*^9, 3.586777105603819*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["x", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", "1", "]"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", "1", "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["x", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", "2", "]"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", "2", "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["x", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", "3", "]"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", "3", "]"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.627106018367276*^9, 3.6274429545820208`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "ff", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5867759283174825`*^9, 3.5867759383390555`*^9}, {
  3.5867771002355123`*^9, 3.586777105603819*^9}, {3.617770731164956*^9, 
  3.6177707380757685`*^9}}],

Cell[BoxData["\<\"invalid argument\"\>"], "Print",
 CellChangeTimes->{3.6271060260892897`*^9, 3.6274429663132415`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"d", "[", "ff", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5867759283174825`*^9, 3.5867759383390555`*^9}, {
  3.5867771002355123`*^9, 3.586777105603819*^9}, {3.617770731164956*^9, 
  3.6177707634726133`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["x", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "[", "1", "]"}], ",", 
       RowBox[{
        RowBox[{
         SuperscriptBox["x", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "[", "1", "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["x", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "[", "1", "]"}], ",", 
       RowBox[{
        RowBox[{
         SuperscriptBox["x", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "[", "1", "]"}]}], "}"}]}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["x", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "[", "2", "]"}], ",", 
       RowBox[{
        RowBox[{
         SuperscriptBox["x", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "[", "2", "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["x", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "[", "2", "]"}], ",", 
       RowBox[{
        RowBox[{
         SuperscriptBox["x", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "[", "2", "]"}]}], "}"}]}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["x", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "[", "3", "]"}], ",", 
       RowBox[{
        RowBox[{
         SuperscriptBox["x", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "[", "3", "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["x", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "[", "3", "]"}], ",", 
       RowBox[{
        RowBox[{
         SuperscriptBox["x", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "[", "3", "]"}]}], "}"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.6271060606745505`*^9, 3.627442970509649*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"d", "[", "ff", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5867759467785387`*^9, 3.586775999747568*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.627106031315299*^9, 3.6274429841596727`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"h", ",", "ff"}], "]"}], ";", 
  RowBox[{
   RowBox[{"ff", "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], "=", " ", 
   RowBox[{
    RowBox[{"h", "[", 
     RowBox[{"u", ",", "v"}], "]"}], "*", 
    RowBox[{
     RowBox[{"d", "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.58677719845613*^9, 3.5867773007009783`*^9}, 
   3.615104410515194*^9, 3.617626036060483*^9, {3.6177708820640216`*^9, 
   3.6177708901604357`*^9}, 3.6177709382397203`*^9, {3.6177709850710025`*^9, 
   3.617770986038204*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"h", "[", 
       RowBox[{"u", ",", "v"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], ",", 
     RowBox[{
      RowBox[{"h", "[", 
       RowBox[{"u", ",", "v"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"h", "[", 
       RowBox[{"u", ",", "v"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], ",", 
     RowBox[{
      RowBox[{"h", "[", 
       RowBox[{"u", ",", "v"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"h", "[", 
       RowBox[{"u", ",", "v"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], ",", 
     RowBox[{
      RowBox[{"h", "[", 
       RowBox[{"u", ",", "v"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6271060856813946`*^9, 3.627443008932516*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "ff", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.586777308549427*^9, 3.5867773194350495`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox["h", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "-", 
    RowBox[{
     RowBox[{
      SuperscriptBox["h", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox["h", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "-", 
    RowBox[{
     RowBox[{
      SuperscriptBox["h", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox["h", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "-", 
    RowBox[{
     RowBox[{
      SuperscriptBox["h", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6271060908606033`*^9, 3.6274430153129272`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", "-", " ", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"extprod", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"d", "[", "h", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"d", "[", "sf", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "[", 
         RowBox[{"[", "i", "]"}], "]"}]}], "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"1", ",", "2"}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3"}], "}"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.586777359339332*^9, 3.5867774925729523`*^9}, {
   3.586777531981207*^9, 3.586777539156617*^9}, {3.586777573533583*^9, 
   3.58677763886232*^9}, {3.5867777724309597`*^9, 3.586777818870616*^9}, {
   3.6089650800063367`*^9, 3.6089650838439436`*^9}, {3.609560182594201*^9, 
   3.609560186307008*^9}, 3.614861849691122*^9, 3.6148635015312233`*^9, 
   3.617770966179369*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.6271060976154156`*^9, 3.627443021818139*^9}]
}, Closed]],

Cell["\<\
Since dd yields the only strict coordinate of the alternating 2-form we had \
to specify the strict coordinate of the exterior product. In traditional \
notation the last formula can be written as\
\>", "Text",
 CellChangeTimes->{{3.586778144938266*^9, 3.5867782694653883`*^9}, {
  3.6095602428727074`*^9, 3.6095602647439456`*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"d", "\[Wedge]", 
     RowBox[{"(", 
      RowBox[{"h", " ", "*", 
       RowBox[{"d", "(", "sf", ")"}]}], ")"}]}], "\[LongEqual]", "dh"}], 
   "\[And]", 
   RowBox[{"d", "(", "sf", ")"}]}], TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5867782884074717`*^9, 3.5867784070832596`*^9}, {
   3.5867784387070684`*^9, 3.5867784871388383`*^9}, 3.5867785184666305`*^9, {
   3.61777100630264*^9, 3.6177710104990473`*^9}},
 TextAlignment->Center]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[" 1.4. The Derivation Equations", "Subsection",
 CellChangeTimes->{{3.586839514507707*^9, 3.5868395266264005`*^9}, {
   3.5872725402788825`*^9, 3.5872725434780655`*^9}, 3.6095615970681953`*^9, 
   3.6096567356047544`*^9, 3.615036647915494*^9, 3.6150391086202383`*^9, 
   3.6177712240322223`*^9, 3.617772205182746*^9, 3.6274431059022865`*^9, 
   3.627452822850827*^9, {3.629188873900796*^9, 3.629188876833601*^9}}],

Cell[CellGroupData[{

Cell["1.4.1. The Orthonormal Cobasis {\[Sigma][1], \[Sigma][2]}", \
"Subsubsection",
 CellChangeTimes->{{3.5867788245861397`*^9, 3.586778908304928*^9}, {
   3.5867789597698717`*^9, 3.586778962401022*^9}, 3.586839376202797*^9, {
   3.5868713690839806`*^9, 3.586871375838793*^9}, {3.587272548686363*^9, 
   3.587272552854602*^9}, {3.622724946885109*^9, 3.6227249489287124`*^9}, 
   3.62272763823198*^9, 3.6274528348004484`*^9, 3.6275363610569324`*^9}],

Cell["\<\
The orthonormal cobasis   {\[Sigma][1], \[Sigma][2]} is the dual basis to the \
orthonormal Basis {amf[1], amf[2]} of the tangent spaces. It is defined (in \
eudiffgeov2.m)  by\
\>", "Text",
 CellChangeTimes->{{3.609650787300531*^9, 3.609650922932289*^9}, {
  3.6148618497223225`*^9, 3.6148618498159227`*^9}, {3.6149177824513445`*^9, 
  3.614917782482545*^9}, {3.61491854824109*^9, 3.614918549130291*^9}, {
  3.6150366787472577`*^9, 3.6150367104430704`*^9}, {3.622727589949895*^9, 
  3.6227276048479214`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Sigma]", "[", "i_", "]"}], "[", "f_", "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Length", "[", 
        RowBox[{"f", "[", 
         RowBox[{"u", ",", "v"}], "]"}], "]"}], "===", "dim"}], ")"}], "&&", 
     RowBox[{"(", 
      RowBox[{"i", ">=", " ", "1"}], ")"}], "&&", 
     RowBox[{"(", 
      RowBox[{"i", "<=", " ", "dim"}], ")"}]}], ",", 
    RowBox[{"Return", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"amf", "[", "i", "]"}], "[", "f", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ".", 
      RowBox[{
       RowBox[{"d", "[", "f", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "]"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Print", "[", "\"\<Invalid argument!\>\"", "]"}], ";", 
     RowBox[{"Return", "[", "]"}]}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5862684374261866`*^9, 3.586268590883964*^9}, {
   3.586268694356882*^9, 3.5862687381573877`*^9}, {3.5862689068290353`*^9, 
   3.586268967629513*^9}, {3.5868451322760253`*^9, 3.586845153622246*^9}, {
   3.5868451909133787`*^9, 3.5868454031145163`*^9}, {3.5868455663498526`*^9, 
   3.5868456048040524`*^9}, {3.586845689054871*^9, 3.5868457266850233`*^9}, {
   3.5868458045404763`*^9, 3.5868458267597475`*^9}, {3.586845979053458*^9, 
   3.586845987693952*^9}, {3.5868460253081036`*^9, 3.5868460401399517`*^9}, {
   3.5868461640450387`*^9, 3.586846178851886*^9}, {3.5868462272766557`*^9, 
   3.5868462390363283`*^9}, 3.5868463462864623`*^9, {3.586846553173296*^9, 
   3.58684656827516*^9}, {3.586846604924256*^9, 3.5868466802425632`*^9}, {
   3.586846871252489*^9, 3.5868468892185163`*^9}, {3.586847223476635*^9, 
   3.5868472486350737`*^9}, {3.5868472800588713`*^9, 3.586847288730367*^9}, {
   3.586847402107852*^9, 3.5868474363878126`*^9}, {3.5868475906426353`*^9, 
   3.5868476198363056`*^9}, {3.5868477813165417`*^9, 3.5868477944282913`*^9}, 
   3.609650066236288*^9, {3.6096500990871677`*^9, 3.6096501028473825`*^9}, {
   3.6096501406985474`*^9, 3.6096501659779935`*^9}, {3.609650204137176*^9, 
   3.609650254747071*^9}, {3.6096505070675025`*^9, 3.6096505316539087`*^9}, {
   3.6096505653868384`*^9, 3.6096505818417797`*^9}, 3.6148618498159227`*^9, 
   3.614917782498145*^9, 3.6149184396336985`*^9, {3.615036953864993*^9, 
   3.615036975872252*^9}, {3.617775036650119*^9, 3.6177750639501667`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "\[Sigma]"}]], "Input",
 CellChangeTimes->{{3.622727446679243*^9, 3.6227274648688755`*^9}}],

Cell[BoxData[
 StyleBox["\<\"{\[Sigma][i][f][u,v]}, i = 1,2, is the cobasis field to the \
orthonormal basis field \\nin the tangent bundle of the surface f[u,v], \
\[Sigma][i][f][u,v]= {0,0} for 2<i<= dim. \"\>", "MSG"]], "Print", \
"PrintUsage",
 CellChangeTimes->{3.6274559394260187`*^9},
 CellTags->"Info3627459539-5102015"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Context", "[", "\[Sigma]", "]"}]], "Input",
 CellChangeTimes->{{3.622727476397296*^9, 3.622727495444929*^9}}],

Cell[BoxData["\<\"eudiffgeov2`\"\>"], "Output",
 CellChangeTimes->{3.6274561023839045`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData["dim"], "Input",
 CellChangeTimes->{{3.59733021329095*^9, 3.5973302565784254`*^9}, {
  3.5973819456579657`*^9, 3.597381958777589*^9}}],

Cell[BoxData["3"], "Output",
 CellChangeTimes->{3.627456045490605*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5862685978523626`*^9, 3.586268637618637*^9}, {
  3.5868454727975016`*^9, 3.586845508677554*^9}, {3.586847494630144*^9, 
  3.5868475090139666`*^9}, {3.6096503908708563`*^9, 3.6096504000783834`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   SqrtBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"]}]], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     SqrtBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"]}]], ")"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6274561074227133`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5862685978523626`*^9, 3.586268637618637*^9}, {
  3.5868454727975016`*^9, 3.586845508677554*^9}, {3.586847494630144*^9, 
  3.5868475090139666`*^9}, {3.5868477190429797`*^9, 3.586847749885744*^9}, {
  3.586847865477355*^9, 3.586847871125678*^9}, {3.609650411374029*^9, 
  3.609650424006752*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"\[Sqrt]", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "-", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "-", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "-", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], ")"}]}], 
     ")"}], "/", 
    RowBox[{"(", 
     SqrtBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"]}]], ")"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6274561174535313`*^9}]
}, Closed]],

Cell["By definition we have", "Text",
 CellChangeTimes->{{3.6096511679643035`*^9, 3.609651177323839*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", "-", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"amf", "[", "2", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ".", 
    RowBox[{
     RowBox[{"d", "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6096493267409916`*^9, 3.609649346334112*^9}, {
   3.6096499062221365`*^9, 3.6096499841265917`*^9}, 3.6148618498315225`*^9, 
   3.614917782513745*^9, 3.6149185578819065`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.627456140073571*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Sigma]", "[", "3", "]"}], "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5862685978523626`*^9, 3.586268637618637*^9}, 
   3.5868463633494387`*^9, {3.586847469821725*^9, 3.586847475909073*^9}, {
   3.5868475678933344`*^9, 3.5868475750537443`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.6274561447691793`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Sigma]", "[", "4", "]"}], "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5862685978523626`*^9, 3.586268637618637*^9}, 
   3.5868463633494387`*^9, {3.586847469821725*^9, 3.586847475909073*^9}, {
   3.5868475678933344`*^9, 3.5868475750537443`*^9}, {3.617774888730659*^9, 
   3.6177748964370728`*^9}}],

Cell[BoxData["\<\"Invalid argument!\"\>"], "Print",
 CellChangeTimes->{3.6274561480763855`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Sigma]", "[", "1", "]"}], "[", 
    RowBox[{"vec", "[", "x", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5862685978523626`*^9, 3.586268637618637*^9}, 
   3.5868463633494387`*^9, {3.586847469821725*^9, 3.586847475909073*^9}, {
   3.5868475678933344`*^9, 3.5868475750537443`*^9}, {3.5868476634187984`*^9, 
   3.5868476818378515`*^9}}],

Cell[BoxData["\<\"Invalid argument!\"\>"], "Print",
 CellChangeTimes->{3.627456161851209*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Sigma]", "[", "1", "]"}], "[", 
   RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5862685978523626`*^9, 3.586268637618637*^9}, {
  3.5862690108319836`*^9, 3.5862690363924456`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    FractionBox[
     RowBox[{
      SuperscriptBox["r", "2"], " ", 
      SuperscriptBox[
       RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"Cos", "[", "v", "]"}], "2"]}], 
     SqrtBox[
      RowBox[{
       RowBox[{
        SuperscriptBox["r", "2"], " ", 
        SuperscriptBox[
         RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Cos", "[", "v", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox["r", "2"], " ", 
        SuperscriptBox[
         RowBox[{"Cos", "[", "v", "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "u", "]"}], "2"]}]}]]], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox["r", "2"], " ", 
      SuperscriptBox[
       RowBox[{"Cos", "[", "v", "]"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"Sin", "[", "u", "]"}], "2"]}], 
     SqrtBox[
      RowBox[{
       RowBox[{
        SuperscriptBox["r", "2"], " ", 
        SuperscriptBox[
         RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Cos", "[", "v", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox["r", "2"], " ", 
        SuperscriptBox[
         RowBox[{"Cos", "[", "v", "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "u", "]"}], "2"]}]}]]]}], ",", "0"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.6274561723500276`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"r", ">", "0"}], ",", " ", 
     RowBox[{"v", " ", "\[Element]", " ", "Reals"}]}], "}"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.586834921828021*^9, 3.5868349307725325`*^9}, {
  3.6177716436597595`*^9, 3.6177716527701755`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{"r", " ", 
     SuperscriptBox[
      RowBox[{"Cos", "[", "v", "]"}], "2"]}], 
    RowBox[{"Abs", "[", 
     RowBox[{"Cos", "[", "v", "]"}], "]"}]], ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.6274561774356365`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Sigma]", "[", "2", "]"}], "[", 
     RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"r", ">", "0"}], ",", " ", 
     RowBox[{"v", " ", "\[Element]", " ", "Reals"}]}], "}"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5862685978523626`*^9, 3.586268637618637*^9}, {
  3.5862690108319836`*^9, 3.5862690869863396`*^9}, {3.609650667958705*^9, 
  3.60965070004354*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "r"}], "}"}]], "Output",
 CellChangeTimes->{3.627535222084324*^9}]
}, Closed]],

Cell[TextData[{
 "We verify some classical formulas first expressed in traditional notation, \
then proved by calculation with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "."
}], "Text",
 CellChangeTimes->{{3.5868371669714355`*^9, 3.586837253204368*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   StyleBox["1.",
    FontColor->RGBColor[0, 0, 1]], 
   StyleBox[
    RowBox[{
     StyleBox[" ",
      FontColor->RGBColor[0, 0, 1]], "  "}]], 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"[", 
      RowBox[{"B", "-", "R"}], "]"}], ",", " ", 
     RowBox[{"(", "41.6", ")"}]}], ")"}], "                    ", 
   RowBox[{"d", "[", "sf", "]"}]}], " ", "\[Equal]", "  ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"amf", "[", "1", "]"}], "[", "sf", "]"}], "*", 
    RowBox[{"\[Sigma]", "[", "1", "]"}]}], " ", "+", "  ", 
   RowBox[{
    RowBox[{
     RowBox[{"amf", "[", "2", "]"}], "[", "sf", "]"}], "*", 
    RowBox[{"\[Sigma]", "[", "2", "]"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5868372815939918`*^9, 3.586837377709489*^9}, {
  3.586837463149376*^9, 3.5868375120501733`*^9}, {3.586837863366267*^9, 
  3.5868378687105727`*^9}, {3.614861849847122*^9, 3.614861849847122*^9}, {
  3.6149177825293446`*^9, 3.6149177825293446`*^9}, {3.614918462362939*^9, 
  3.6149184637981415`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"d", "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ".", 
     RowBox[{"{", 
      RowBox[{"s", ",", "t"}], "}"}]}], "-", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"amf", "[", "1", "]"}], "[", "sf", "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "*", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "sf", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], ".", 
        RowBox[{"{", 
         RowBox[{"s", ",", "t"}], "}"}]}]}], " ", "+", " ", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"amf", "[", "2", "]"}], "[", "sf", "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "*", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "sf", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], ".", 
        RowBox[{"{", 
         RowBox[{"s", ",", "t"}], "}"}]}]}]}], ")"}]}], "]"}], " "}]], "Input",
 CellChangeTimes->{{3.586835336082715*^9, 3.586835446246016*^9}, {
   3.5868355063994565`*^9, 3.58683551555098*^9}, {3.586835582798826*^9, 
   3.586835692009073*^9}, {3.5897805003550973`*^9, 3.5897805009010983`*^9}, {
   3.5897806863542237`*^9, 3.589780691627033*^9}, {3.614861849847122*^9, 
   3.6148618498627224`*^9}, {3.614917782544945*^9, 3.614917782544945*^9}, 
   3.6149184651709437`*^9, 3.6149185594419093`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.627456221412114*^9, 3.627535275070017*^9}]
}, Closed]],

Cell["Remark, This formula is analogue to", "Text",
 CellChangeTimes->{{3.627536749229617*^9, 3.6275367729456587`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "sf", "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"tv", "[", "1", "]"}], "[", "sf", "]"}], "*", 
    RowBox[{"d", "[", "u", "]"}]}], " ", "+", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"tv", "[", "2", "]"}], "[", "sf", "]"}], "*", 
    RowBox[{"d", "[", "v", "]"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.627536801045308*^9, 3.627536901658886*^9}, {
  3.627537100957038*^9, 3.6275372116552334`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ".", 
   RowBox[{"{", 
    RowBox[{"s", ",", "t"}], "}"}]}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", "1", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "*", 
    RowBox[{
     RowBox[{
      RowBox[{"d", "[", "u", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ".", 
     RowBox[{"{", 
      RowBox[{"s", ",", "t"}], "}"}]}]}], " ", "+", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", "2", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "*", 
    RowBox[{
     RowBox[{
      RowBox[{"d", "[", "v", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ".", 
     RowBox[{"{", 
      RowBox[{"s", ",", "t"}], "}"}]}]}]}]}]], "Input",
 CellChangeTimes->{{3.627536801045308*^9, 3.627536901658886*^9}, {
  3.627537027722509*^9, 3.6275370808798027`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.627537082346205*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   StyleBox["2.",
    FontColor->RGBColor[0, 0, 1]], "   ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"[", 
      RowBox[{"B", "-", "R"}], "]"}], ",", " ", 
     RowBox[{"(", "42.13", ")"}]}], ")"}], "                ", "fund1"}], " ",
   "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Sigma]", "[", "1", "]"}], "^", "2"}], "  ", "+", "  ", 
   RowBox[{
    RowBox[{"\[Sigma]", "[", "2", "]"}], "^", "2", " "}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5868377489117208`*^9, 3.5868378393648944`*^9}, {
   3.586837873052821*^9, 3.5868379575736556`*^9}, 3.5904957105154095`*^9}],

Cell["We evaluate the corresponding symmetric bilinear form:", "Text",
 CellChangeTimes->{{3.5868380097146378`*^9, 3.586838051907051*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"vs", "=", 
   RowBox[{"{", 
    RowBox[{"s1", ",", "s2"}], "}"}]}], ";", 
  RowBox[{"vt", "=", 
   RowBox[{"{", 
    RowBox[{"t1", ",", "t2"}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.586836626522524*^9, 3.58683667733243*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fund1", "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{"vs", ",", "vt"}], "]"}], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ".", "vs"}], ")"}], "*", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ".", "vt"}], ")"}]}], "-", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ".", "vs"}], ")"}], "*", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ".", "vt"}], ")"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.586836684445837*^9, 3.586836692207281*^9}, {
   3.586836738835948*^9, 3.586836753262773*^9}, {3.5868367939941025`*^9, 
   3.5868368662162333`*^9}, {3.5868369358372154`*^9, 3.586836950725067*^9}, {
   3.586836982972912*^9, 3.5868370924811754`*^9}, 3.5904957105154095`*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{
  3.6274562427529516`*^9, 3.627456277197812*^9, {3.627535439795108*^9, 
   3.62753545798874*^9}}]
}, Closed]],

Cell["\<\
3.   ([B-R], (42.7))   The  area element             dA\[Equal] \[Sigma][1]\
\[Wedge]\[Sigma][2]\
\>", "Text",
 CellChangeTimes->{3.627535506181225*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"dA", "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], " ", "\[Equal]", " ", 
   RowBox[{"Det", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "}"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5868384410843105`*^9, 3.5868384512198906`*^9}, {
   3.586838491404189*^9, 3.586838569777672*^9}, 3.586838627365966*^9, {
   3.586838866175625*^9, 3.5868388901109934`*^9}, {3.586839064039942*^9, 
   3.586839101438081*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6275355246712575`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"extprod", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.588414285618554*^9, 3.5884143135270033`*^9}, {
  3.609651368687784*^9, 3.6096513743031054`*^9}, {3.609651417118554*^9, 
  3.6096514250310073`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{"\[Sqrt]", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "-", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "-", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "-", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], ")"}]}]}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"\[Sqrt]", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], ")"}]}]}], 
     ",", "0"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627535531457269*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"%", "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "2"}], "]"}], "]"}], "\[Equal]", " ", 
   RowBox[{"Det", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "}"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.58841475574098*^9, 3.5884148014646597`*^9}, {
  3.6096514383987713`*^9, 3.6096514445281224`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.627535535840877*^9}]
}, Closed]],

Cell["\<\
Remember that  the determinant on the right side of the equation is the only \
strict coordinate 
of \[Sigma][1]\[Wedge]\[Sigma][2] in the local basis du, dv; since (du,dv) \
and (\[Sigma][1][sf][u,v],\[Sigma][2][sf][u,v]}) define the same orientation \
of the tangent space, this determinant is always assumed to be positive.\
\>", "Text",
 CellChangeTimes->{{3.586839212899456*^9, 3.586839299018382*^9}, {
  3.590736505584259*^9, 3.5907365286878996`*^9}, {3.617953744410181*^9, 
  3.617953914697921*^9}}],

Cell[TextData[{
 StyleBox["The basis transformation.",
  FontColor->RGBColor[0, 0, 1]],
 "\nWe calculate the decomposition of the \[Sigma][i] with respect to the \
basis du,dv, stating"
}], "Text",
 CellChangeTimes->{{3.590721556004931*^9, 3.5907216598688717`*^9}, {
  3.590722468989151*^9, 3.5907224724133463`*^9}, {3.5907350609125214`*^9, 
  3.59073506573293*^9}},
 CellTags->"bastr"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[Sigma]", "(", "1", ")"}], "\[LongEqual]", 
   FractionBox[
    RowBox[{" ", 
     RowBox[{
      RowBox[{
       RowBox[{"rgT", "(", 
        RowBox[{"1", ",", "1"}], ")"}], " ", "du"}], "+", " ", 
      RowBox[{
       RowBox[{"rgT", "(", 
        RowBox[{"1", ",", "2"}], ")"}], " ", "dv"}]}]}], 
    SqrtBox[
     RowBox[{"rgT", "(", 
      RowBox[{"1", ",", "1"}], ")"}]]]}], TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.590735088992571*^9, 3.5907351925767527`*^9}, {
  3.590735232356823*^9, 3.590735292619729*^9}, {3.6176262984584913`*^9, 
  3.617626319160675*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[Sigma]", "(", "2", ")"}], "\[LongEqual]", 
   RowBox[{
    FractionBox[
     SqrtBox[
      TemplateBox[{RowBox[{"{", 
          RowBox[{"rgT", "(", 
            RowBox[{"i", ",", "j"}], ")"}], "}"}]},
       "Det"]], 
     SqrtBox[
      RowBox[{"rgT", "(", 
       RowBox[{"1", ",", "1"}], ")"}]]], " ", "dv"}], " ", "\[Equal]", " ", 
   RowBox[{
    FractionBox["dA", 
     SqrtBox[
      RowBox[{"rgT", "(", 
       RowBox[{"1", ",", "1"}], ")"}]]], " ", "dv"}]}], 
  TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.590735088992571*^9, 3.5907351925767527`*^9}, {
  3.590735232356823*^9, 3.590735292619729*^9}, {3.590735627864318*^9, 
  3.5907357745045753`*^9}, {3.617626330122302*^9, 3.6176263320654135`*^9}, {
  3.6176264002583137`*^9, 3.617626413915095*^9}, {3.6275357480926514`*^9, 
  3.6275357802951083`*^9}}],

Cell[TextData[{
 StyleBox["Proof",
  FontColor->RGBColor[0, 0, 1]],
 ":"
}], "Text",
 CellChangeTimes->{{3.5907358558431177`*^9, 3.5907358594467244`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", " ", "v"}], "]"}], ".", 
    RowBox[{"{", 
     RowBox[{"s1", ",", " ", "s2"}], "}"}]}], " ", "==", " ", "\n", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"rgT", "[", 
          RowBox[{"1", ",", " ", "1"}], "]"}], "[", "sf", "]"}], "[", 
        RowBox[{"u", ",", " ", "v"}], "]"}], "*", 
       RowBox[{
        RowBox[{
         RowBox[{"d", "[", "u", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], ".", 
        RowBox[{"{", 
         RowBox[{"s1", ",", " ", "s2"}], "}"}]}]}], " ", "+", " ", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"rgT", "[", 
          RowBox[{"1", ",", " ", "2"}], "]"}], "[", "sf", "]"}], "[", 
        RowBox[{"u", ",", " ", "v"}], "]"}], "*", 
       RowBox[{
        RowBox[{
         RowBox[{"d", "[", "v", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], ".", 
        RowBox[{"{", 
         RowBox[{"s1", ",", " ", "s2"}], "}"}]}]}]}], ")"}], "/", 
    RowBox[{"Sqrt", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"rgT", "[", 
        RowBox[{"1", ",", " ", "1"}], "]"}], "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", " ", "v"}], "]"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5907173613780117`*^9, 3.5907173699605026`*^9}, {
   3.5907174455358257`*^9, 3.590717473390419*^9}, {3.5907175080954037`*^9, 
   3.5907176306184115`*^9}, {3.5907177498192296`*^9, 3.590717791333604*^9}, {
   3.5907179684707355`*^9, 3.590718084885394*^9}, {3.5907203135178647`*^9, 
   3.5907203282297063`*^9}, {3.590720362173648*^9, 3.590720399942808*^9}, 
   3.590720439646079*^9, 3.5907349903223977`*^9, {3.5907353529138346`*^9, 
   3.5907354339715767`*^9}, {3.6096516615155334`*^9, 
   3.6096516949464455`*^9}, {3.6096517297964387`*^9, 3.609651768851672*^9}, {
   3.61762605087033*^9, 3.6176260543665295`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6274563897988095`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", " ", "v"}], "]"}], ".", 
    RowBox[{"{", 
     RowBox[{"s1", ",", " ", "s2"}], "}"}]}], " ", "==", " ", "\n", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"dA", "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", " ", "v"}], "]"}], "/", 
      RowBox[{"Sqrt", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"rgT", "[", 
          RowBox[{"1", ",", " ", "1"}], "]"}], "[", "sf", "]"}], "[", 
        RowBox[{"u", ",", " ", "v"}], "]"}], "]"}]}], ")"}], "*", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"d", "[", "v", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ".", 
     RowBox[{"{", 
      RowBox[{"s1", ",", " ", "s2"}], "}"}]}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5907173613780117`*^9, 3.5907173699605026`*^9}, {
   3.5907174455358257`*^9, 3.590717473390419*^9}, {3.5907175080954037`*^9, 
   3.5907176306184115`*^9}, {3.5907177498192296`*^9, 3.590717791333604*^9}, {
   3.5907179684707355`*^9, 3.590718084885394*^9}, {3.5907203135178647`*^9, 
   3.5907203282297063`*^9}, {3.590720362173648*^9, 3.590720399942808*^9}, 
   3.590720439646079*^9, {3.590721070544164*^9, 3.590721087006106*^9}, {
   3.5907211626374316`*^9, 3.590721228294187*^9}, 3.590721882841625*^9, {
   3.5907219660703855`*^9, 3.5907219725897584`*^9}, {3.5907220046705933`*^9, 
   3.5907220343012877`*^9}, {3.590722076702713*^9, 3.5907220826040506`*^9}, 
   3.5907355108953123`*^9, {3.6096518755697765`*^9, 3.60965190080122*^9}, 
   3.6176260623649874`*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.627456440249298*^9}]
}, Closed]],

Cell["Here remember", "Text",
 CellChangeTimes->{{3.590735889554777*^9, 3.590735895997588*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"dA", "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", " ", "v"}], "]"}], "\[Equal]", " ", 
   RowBox[{"Sqrt", "[", 
    RowBox[{"Det", "[", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"rgT", "[", 
          RowBox[{"i", ",", "j"}], "]"}], "[", "sf", "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5907359427352705`*^9, 3.590736093166335*^9}, {
   3.590736161681655*^9, 3.590736167609666*^9}, 3.614861849893923*^9, 
   3.614863501578024*^9, {3.6176264560655055`*^9, 3.6176264578096056`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.627456465100142*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", " ", "v"}], "]"}], ".", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"1", "/", 
      RowBox[{"Sqrt", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"rgT", "[", 
          RowBox[{"1", ",", " ", "1"}], "]"}], "[", "sf", "]"}], "[", 
        RowBox[{"u", ",", " ", "v"}], "]"}], "]"}]}], ",", " ", "0"}], 
    "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.609652212502048*^9, 3.6096522185163918`*^9}, {
  3.6096522543684425`*^9, 3.609652287800355*^9}, {3.6176264757526317`*^9, 
  3.6176264781367683`*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.627456470497751*^9}]
}, Closed]],

Cell["\<\
We state and test the following formulas expressing the orthonormal basis \
with respect to the tangent basis t1,t2:\
\>", "Text",
 CellChangeTimes->{{3.6096536113480573`*^9, 3.6096536808440323`*^9}, {
   3.615037408346988*^9, 3.615037421618747*^9}, 3.615117680678119*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tv", "[", "1", "]"}], "[", "sf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6096536860943327`*^9, 3.609653696557931*^9}, 
   3.617600024337888*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "1", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "2", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "3", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627456487252181*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tv", "[", "2", "]"}], "[", "sf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6096537089286385`*^9, 3.609653730893895*^9}, 
   3.617600085770796*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "1", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "2", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "3", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627456491354988*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"amf", "[", "1", "]"}], "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], " ", "==", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", "1", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "/", 
    RowBox[{"Sqrt", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"rgT", "[", 
        RowBox[{"1", ",", " ", "1"}], "]"}], "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", " ", "v"}], "]"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.60965374165051*^9, 3.609653754030218*^9}, {
   3.60965379095033*^9, 3.609653907929021*^9}, 3.6148618499095225`*^9, 
   3.614917782591745*^9, 3.6149185669143224`*^9, 3.617600024353488*^9, 
   3.6176265331969175`*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6274564981722*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"amf", "[", "2", "]"}], "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], " ", "\[Equal]", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         RowBox[{
          RowBox[{"tv", "[", "1", "]"}], "[", "sf", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "*", 
       RowBox[{
        RowBox[{
         RowBox[{"rgT", "[", 
          RowBox[{"1", ",", " ", "2"}], "]"}], "[", "sf", "]"}], "[", 
        RowBox[{"u", ",", " ", "v"}], "]"}]}], "+", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"tv", "[", "2", "]"}], "[", "sf", "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "*", 
       RowBox[{
        RowBox[{
         RowBox[{"rgT", "[", 
          RowBox[{"1", ",", "1"}], "]"}], "[", "sf", "]"}], "[", 
        RowBox[{"u", ",", " ", "v"}], "]"}]}]}], " ", ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"dA", "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", " ", "v"}], "]"}], "*", 
      RowBox[{"Sqrt", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"rgT", "[", 
          RowBox[{"1", ",", " ", "1"}], "]"}], "[", "sf", "]"}], "[", 
        RowBox[{"u", ",", " ", "v"}], "]"}], "]"}]}], ")"}]}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.60965374165051*^9, 3.609653754030218*^9}, {
   3.60965379095033*^9, 3.6096539246169753`*^9}, {3.6096540031054645`*^9, 
   3.609654147059698*^9}, {3.6096541915242414`*^9, 3.6096541999007206`*^9}, 
   3.6096542338676634`*^9, {3.609654281572392*^9, 3.609654289737859*^9}, 
   3.6148618499095225`*^9, 3.6149177826073446`*^9, 3.6149185725303326`*^9, 
   3.617600024353488*^9, 3.617600085786396*^9, {3.6176265366951175`*^9, 
   3.617626541950418*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6274565034138093`*^9}]
}, Closed]],

Cell["\<\
The next four identities verify the duality of the orthonormal basis and \
cobasis:\
\>", "Text",
 CellChangeTimes->{{3.6096566616205225`*^9, 3.6096567048279943`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", " ", "v"}], "]"}], ".", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"1", "/", 
      RowBox[{"Sqrt", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"rgT", "[", 
          RowBox[{"1", ",", " ", "1"}], "]"}], "[", "sf", "]"}], "[", 
        RowBox[{"u", ",", " ", "v"}], "]"}], "]"}]}], ",", "0"}], "}"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.609655949502792*^9, 3.6096559526939745`*^9}, {
   3.609655983711749*^9, 3.6096559907591515`*^9}, {3.6096560255621424`*^9, 
   3.60965603671278*^9}, 3.6176265453316116`*^9}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.627456531681059*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", " ", "v"}], "]"}], ".", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        RowBox[{
         RowBox[{"rgT", "[", 
          RowBox[{"1", ",", " ", "2"}], "]"}], "[", "sf", "]"}], "[", 
        RowBox[{"u", ",", " ", "v"}], "]"}]}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"rgT", "[", 
         RowBox[{"1", ",", "1"}], "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", " ", "v"}], "]"}]}], "}"}]}], "/", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"dA", "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", " ", "v"}], "]"}], "*", 
     RowBox[{"Sqrt", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"rgT", "[", 
         RowBox[{"1", ",", " ", "1"}], "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", " ", "v"}], "]"}], "]"}]}], ")"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.609655949502792*^9, 3.6096559526939745`*^9}, {
  3.609655983711749*^9, 3.6096559907591515`*^9}, {3.6096560255621424`*^9, 
  3.609656054616804*^9}, {3.6096561620579495`*^9, 3.609656183154156*^9}, {
  3.609656261770653*^9, 3.609656267947006*^9}, {3.609656328491469*^9, 
  3.6096563674596977`*^9}, {3.6096564022096853`*^9, 3.609656417282548*^9}, {
  3.617626547645744*^9, 3.617626552389015*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.62745656646912*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", " ", "v"}], "]"}], ".", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        RowBox[{
         RowBox[{"rgT", "[", 
          RowBox[{"1", ",", " ", "2"}], "]"}], "[", "sf", "]"}], "[", 
        RowBox[{"u", ",", " ", "v"}], "]"}]}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"rgT", "[", 
         RowBox[{"1", ",", "1"}], "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", " ", "v"}], "]"}]}], "}"}]}], "/", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"dA", "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", " ", "v"}], "]"}], "*", 
     RowBox[{"Sqrt", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"rgT", "[", 
         RowBox[{"1", ",", " ", "1"}], "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", " ", "v"}], "]"}], "]"}]}], ")"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.609655949502792*^9, 3.6096559526939745`*^9}, {
  3.609655983711749*^9, 3.6096559907591515`*^9}, {3.6096560255621424`*^9, 
  3.6096560682745857`*^9}, {3.6096561208885946`*^9, 3.6096561300971217`*^9}, {
  3.6096565385274825`*^9, 3.60965656017072*^9}, {3.6176265555741973`*^9, 
  3.6176265598944445`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{{3.6274565707279277`*^9, 3.6274565736139326`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", " ", "v"}], "]"}], ".", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"1", "/", 
      RowBox[{"Sqrt", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"rgT", "[", 
          RowBox[{"1", ",", " ", "1"}], "]"}], "[", "sf", "]"}], "[", 
        RowBox[{"u", ",", " ", "v"}], "]"}], "]"}]}], ",", "0"}], "}"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.609656584423107*^9, 3.6096566004730253`*^9}, 
   3.6176265614995365`*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6274565847211523`*^9}]
}, Closed]],

Cell["\<\
In the case of orthogonal parameters u,v, i.e. rgT[1,2][sf][u,v] == 0, the \
formulas simplify to\
\>", "Text",
 CellChangeTimes->{{3.6096570629004745`*^9, 3.6096571596600094`*^9}, 
   3.617626564157688*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"amf", "[", "1", "]"}], " ", "\[Equal]", " ", 
   RowBox[{"tv1", "/", 
    RowBox[{"Sqrt", "[", 
     RowBox[{"rgT", "[", 
      RowBox[{"1", ",", "1"}], "]"}], "]"}]}]}], ";", " ", 
  RowBox[{
   RowBox[{"amf", "[", "2", "]"}], " ", "\[Equal]", " ", 
   RowBox[{"tv2", "/", 
    RowBox[{"Sqrt", "[", 
     RowBox[{"rgT", "[", 
      RowBox[{"2", ",", "2"}], "]"}], "]"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.60965716946657*^9, 3.609657242111725*^9}, {
  3.609657306679418*^9, 3.609657312062726*^9}, {3.6148618499251223`*^9, 
  3.6148618499251223`*^9}, {3.614917782622945*^9, 3.614917782622945*^9}, {
  3.6149185737315345`*^9, 3.6149185765395393`*^9}, {3.6176265667318354`*^9, 
  3.6176265693079824`*^9}, {3.627456912774128*^9, 3.627456919154539*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Sigma]", "[", "1", "]"}], " ", "\[Equal]", " ", 
   RowBox[{
    RowBox[{"d", "[", "u", "]"}], "*", 
    RowBox[{"Sqrt", "[", 
     RowBox[{"rgT", "[", 
      RowBox[{"1", ",", "1"}], "]"}], "]"}]}]}], ";", " ", 
  RowBox[{
   RowBox[{"\[Sigma]", "[", "2", "]"}], " ", "\[Equal]", " ", 
   RowBox[{
    RowBox[{"d", "[", "v", "]"}], "*", 
    RowBox[{"Sqrt", "[", 
     RowBox[{"rgT", "[", 
      RowBox[{"2", ",", "2"}], "]"}], "]"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.60965716946657*^9, 3.6096572875913267`*^9}, {
   3.609657332815913*^9, 3.609657361215537*^9}, 3.6096575886875477`*^9, 
   3.609657638831416*^9, {3.61762657188413*^9, 3.6176265734042172`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["1.4.2. Derivation of the Orthonormal Frame", "Subsubsection",
 CellChangeTimes->{{3.5868395957793555`*^9, 3.586839624635006*^9}, {
   3.587272558157905*^9, 3.587272561670106*^9}, 3.609656777205134*^9, 
   3.609673033665471*^9, 3.6177723089073277`*^9, 3.617778896920499*^9, 
   3.6275363705105495`*^9, 3.627538259815281*^9}],

Cell["\<\
Now we consider the second part of the derivation equations [B-R], (41.6), \
traditionally written as \
\>", "Text",
 CellChangeTimes->{{3.5868413763872004`*^9, 3.586841403410746*^9}, {
  3.586841435555585*^9, 3.5868414605550146`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", 
     RowBox[{"amf", "[", "j", "]"}], "]"}], "[", "sf", "]"}], "[", 
   RowBox[{"u", ".", "v"}], "]"}], "\[Equal]", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"\[Omega]", "[", 
      RowBox[{"j", ",", "k"}], "]"}], "*", 
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "k", "]"}], "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "3"}], "}"}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.586841471104618*^9, 3.586841517365264*^9}, {
   3.5868415742765193`*^9, 3.586841612878727*^9}, {3.5868416500478535`*^9, 
   3.586841798703356*^9}, {3.586842211630974*^9, 3.586842226862845*^9}, 
   3.5868422721024323`*^9, {3.586842430621499*^9, 3.586842442502179*^9}, {
   3.6148618499407225`*^9, 3.6148618499407225`*^9}, {3.614917782622945*^9, 
   3.614917782638545*^9}, {3.614918685490131*^9, 3.614918689499338*^9}}],

Cell["\<\
that means we decompose the derivatives of the frame vectors in the frame \
itself. The main point here is that the formula defines the local structure \
forms of the frame: \
\>", "Text",
 CellChangeTimes->{{3.5868418968969717`*^9, 3.5868419627467384`*^9}, {
   3.586842047258572*^9, 3.5868420475225873`*^9}, {3.5868420867948337`*^9, 
   3.5868421500584517`*^9}, 3.6151176978140993`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Omega]", "[", 
   RowBox[{"j", ",", "k"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"d", "[", 
    RowBox[{"amf", "[", "j", "]"}], "]"}], ".", 
   RowBox[{"amf", "[", "k", "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5868421823663*^9, 3.586842199812298*^9}, {
  3.5868422541254044`*^9, 3.5868422601407485`*^9}, {3.586842300638065*^9, 
  3.586842314316847*^9}, {3.6148618499563227`*^9, 3.6148618499563227`*^9}, {
  3.614917782638545*^9, 3.6149177826541452`*^9}, {3.6149186917613416`*^9, 
  3.6149186932433443`*^9}}],

Cell[TextData[{
 "The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " definition of these differential forms is given in eudiffgeov2.m:"
}], "Text",
 CellChangeTimes->{{3.5868424600031796`*^9, 3.586842469555726*^9}, {
  3.58684251140312*^9, 3.5868425428909206`*^9}, {3.622728161082498*^9, 
  3.6227281812221336`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Omega]", "[", 
      RowBox[{"j_", ",", "k_"}], "]"}], "[", "f_", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", "\[IndentingNewLine]", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"j", ">=", "1"}], "&&", 
      RowBox[{"j", "\[LessEqual]", "3"}], "&&", 
      RowBox[{"k", ">=", "1"}], "&&", 
      RowBox[{"k", "\[LessEqual]", "3"}], "&&", 
      RowBox[{
       RowBox[{"Length", "[", 
        RowBox[{"f", "[", 
         RowBox[{"u", ",", "v"}], "]"}], "]"}], "\[Equal]", " ", "3"}]}], ",",
      "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"j", "\[Equal]", "k"}], ",", 
        RowBox[{"Return", "[", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Return", "[", 
       RowBox[{"Simplify", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"amf", "[", "k", "]"}], "[", "f", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], ".", 
         RowBox[{
          RowBox[{"d", "[", 
           RowBox[{
            RowBox[{"amf", "[", "j", "]"}], "[", "f", "]"}], "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "]"}]}], ",", 
     RowBox[{
      RowBox[{"Print", "[", "\"\<Invalid arguments!\>\"", "]"}], ";", 
      RowBox[{"Return", "[", "]"}]}]}], "]"}]}], 
  "\[IndentingNewLine]"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5868425689884133`*^9, 3.5868427046631737`*^9}, 
   3.586842867103465*^9, {3.586842907943801*^9, 3.5868429135911236`*^9}, {
   3.586843065686823*^9, 3.5868430715351577`*^9}, {3.5868431113754363`*^9, 
   3.586843117103764*^9}, {3.5868431605112467`*^9, 3.586843177967245*^9}, {
   3.586843347214926*^9, 3.586843389720357*^9}, 3.586843549785512*^9, {
   3.586848184577607*^9, 3.5868482812421355`*^9}, {3.5868483466668777`*^9, 
   3.5868483813548617`*^9}, {3.586848414264744*^9, 3.5868486273089294`*^9}, 
   3.5868486965418897`*^9, 3.609658177877248*^9, {3.6096582133592772`*^9, 
   3.6096583241116123`*^9}, {3.6096583805918427`*^9, 3.6096583834950085`*^9}, 
   3.609658418040984*^9, {3.6096584928332624`*^9, 3.6096585432111435`*^9}, {
   3.6096585808842983`*^9, 3.6096586149792485`*^9}, 3.6096587385003138`*^9, {
   3.6148618499563227`*^9, 3.614861849971923*^9}, {3.6149177826541452`*^9, 
   3.614917782669745*^9}, {3.6149186977517524`*^9, 3.614918700466157*^9}, {
   3.6177767763651743`*^9, 3.6177768768137507`*^9}, 3.617776910026209*^9, {
   3.617777034561228*^9, 3.617777046370449*^9}, {3.6257278613107805`*^9, 
   3.6257278841336207`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "3"}], ";"}]], "Input",
 CellChangeTimes->{{3.6177762838567095`*^9, 3.617776319034771*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Omega]", "[", 
    RowBox[{"1", ",", "1"}], "]"}], "[", "sf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5868429252377896`*^9, 3.5868429767887383`*^9}, {
  3.586843433238846*^9, 3.5868434795654955`*^9}, {3.586848719485202*^9, 
  3.5868487233644238`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.62753755556664*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Omega]", "[", 
    RowBox[{"1", ",", "4"}], "]"}], "[", "sf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5868429252377896`*^9, 3.5868429767887383`*^9}, {
  3.586843433238846*^9, 3.5868434795654955`*^9}, {3.586848719485202*^9, 
  3.5868487233644238`*^9}, {3.609672962921425*^9, 3.609672970666868*^9}}],

Cell[BoxData["\<\"Invalid arguments!\"\>"], "Print",
 CellChangeTimes->{3.6275375609486485`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Omega]", "[", 
     RowBox[{"1", ",", "2"}], "]"}], "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5868429252377896`*^9, 3.5868429767887383`*^9}, {
  3.586843433238846*^9, 3.5868434795654955`*^9}, {3.586843579253197*^9, 
  3.5868435857265673`*^9}, {3.5868487299978027`*^9, 3.5868487340450344`*^9}, {
  3.60967251944306*^9, 3.609672531130728*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}]}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}]}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], ")"}],
     "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
      RowBox[{"\[Sqrt]", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], ")"}]}]}], 
     ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}]}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}]}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], ")"}],
     "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
      RowBox[{"\[Sqrt]", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], ")"}]}]}], 
     ")"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.62753756686506*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", "-", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"amf", "[", "2", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ".", 
    RowBox[{
     RowBox[{"d", "[", 
      RowBox[{
       RowBox[{"amf", "[", "1", "]"}], "[", "sf", "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6096579861182795`*^9, 3.609657997869952*^9}, {
  3.60965803910231*^9, 3.609658120935991*^9}, {3.614861849987523*^9, 
  3.614861849987523*^9}, {3.6149177826853447`*^9, 3.614917782700945*^9}, {
  3.6149187024473605`*^9, 3.614918703913763*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.6275375754762745`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%%", "+", 
   RowBox[{
    RowBox[{
     RowBox[{"\[Omega]", "[", 
      RowBox[{"2", ",", "1"}], "]"}], "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5868435996083612`*^9, 3.586843645676996*^9}, {
  3.5868487453106785`*^9, 3.586848749404913*^9}, {3.6096725624665203`*^9, 
  3.609672580851572*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.6275376018443213`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Omega]", "[", 
    RowBox[{"2", ",", "2"}], "]"}], "[", "sf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.586843679460929*^9, 3.58684368874146*^9}, {
  3.586848755308251*^9, 3.58684876770896*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.6275376314883738`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Omega]", "[", 
    RowBox[{"2", ",", "3"}], "]"}], "[", "sf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.586843679460929*^9, 3.586843727773692*^9}, {
  3.5868487753243957`*^9, 3.5868487778365393`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}]}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "3"]}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "3"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "3"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "3"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "-", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
        ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      SqrtBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
        RowBox[{"2", " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], "-", 
        RowBox[{"2", " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], 
     ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}]}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "2"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}]}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "2"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "2"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}]}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "2"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "2"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "2"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], ")"}],
     "/", 
    RowBox[{"(", 
     RowBox[{
      SqrtBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
        RowBox[{"2", " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], "-", 
        RowBox[{"2", " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], 
     ")"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.627537659474823*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Omega]", "[", 
      RowBox[{"2", ",", "3"}], "]"}], "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "+", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"\[Omega]", "[", 
      RowBox[{"3", ",", "2"}], "]"}], "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.586843679460929*^9, 3.586843727773692*^9}, {
  3.5868437628767*^9, 3.5868438044780793`*^9}, {3.5868487931904173`*^9, 
  3.586848796973634*^9}, {3.609672698259287*^9, 3.609672711827063*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.627537692394881*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Omega]", "[", 
    RowBox[{"3", ",", "1"}], "]"}], "[", "sf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5868438496236615`*^9, 3.586843878845333*^9}, {
  3.586848804459062*^9, 3.5868488065651827`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}]}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}]}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      SqrtBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"]}]], " ", 
      RowBox[{"\[Sqrt]", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], ")"}]}]}], 
     ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}]}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}]}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      SqrtBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"]}]], " ", 
      RowBox[{"\[Sqrt]", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], ")"}]}]}], 
     ")"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.627537699152693*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Omega]", "[", 
      RowBox[{"3", ",", "1"}], "]"}], "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "+", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"\[Omega]", "[", 
      RowBox[{"1", ",", "3"}], "]"}], "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5868438496236615`*^9, 3.586843928902196*^9}, {
  3.586848816950776*^9, 3.5868488500336685`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.627537705845104*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Omega]", "[", 
    RowBox[{"3", ",", "3"}], "]"}], "[", "sf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5868438496236615`*^9, 3.586843953133582*^9}, {
  3.586848858750167*^9, 3.586848863541441*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.6275377102443123`*^9}]
}, Closed]],

Cell["Following [B-R], (41.12) we define", "Text",
 CellChangeTimes->{{3.5868493567396507`*^9, 3.586849361010895*^9}, {
  3.586849409451666*^9, 3.5868494290347853`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Omega]", "[", "1", "]"}], "[", "f_", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[Omega]", "[", 
       RowBox[{"2", ",", "3"}], "]"}], "[", "f", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "/;", 
    RowBox[{
     RowBox[{"Length", "[", 
      RowBox[{"f", "[", 
       RowBox[{"u", ",", "v"}], "]"}], "]"}], "\[Equal]", "3"}]}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Omega]", "[", "2", "]"}], "[", "f_", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[Omega]", "[", 
       RowBox[{"3", ",", "1"}], "]"}], "[", "f", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "/;", 
    RowBox[{
     RowBox[{"Length", "[", 
      RowBox[{"f", "[", 
       RowBox[{"u", ",", "v"}], "]"}], "]"}], "\[Equal]", "3"}]}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Omega]", "[", "3", "]"}], "[", "f_", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[Omega]", "[", 
       RowBox[{"1", ",", "2"}], "]"}], "[", "f", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "/;", 
    RowBox[{
     RowBox[{"Length", "[", 
      RowBox[{"f", "[", 
       RowBox[{"u", ",", "v"}], "]"}], "]"}], "\[Equal]", "3"}]}]}], 
  ";"}]}], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5868494748924084`*^9, 3.5868495473585534`*^9}, 
   3.617775849770347*^9, {3.6177759029820404`*^9, 3.617775946038116*^9}, {
   3.6177760265654573`*^9, 3.6177760341158705`*^9}, {3.625727939966119*^9, 
   3.6257279578437505`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "\[Omega]"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.615037843228862*^9, 3.615037851164316*^9}, {
  3.6150378904935656`*^9, 3.615037974870392*^9}, {3.6150381937269096`*^9, 
  3.615038345791607*^9}, {3.615038395609456*^9, 3.615038458136033*^9}, {
  3.617776085533561*^9, 3.6177761088088017`*^9}, {3.6227280311030703`*^9, 
  3.62272804258469*^9}}],

Cell[BoxData[
 StyleBox["\<\"\[Omega][i,j][f][u,v] is the (i,j)-th structure form in the \
derivation equations \\nof the surface f[u.v]. If dim = 3 one sets \
\[Omega][1]=\[Omega][2,3], \[Omega][2]=\[Omega][3,1], \
\[Omega][3]=\[Omega][1,2].\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.629186773900823*^9},
 CellTags->"Info3629190373-5022241"]
}, Closed]],

Cell[TextData[{
 "Clearly, the matrix {\[Omega][j,k][sf][u,v]} is skew symmetric. \n\n",
 StyleBox["Example",
  FontColor->RGBColor[0, 0, 1]],
 ": The local structure forms of a sphere:"
}], "Text",
 CellChangeTimes->{{3.5868440279308605`*^9, 3.586844087339258*^9}, {
   3.586848951379465*^9, 3.5868489860914507`*^9}, 3.6177773205253305`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Omega]", "[", 
      RowBox[{"1", ",", "2"}], "]"}], "[", 
     RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
   RowBox[{
    RowBox[{"r", ">", "0"}], "&&", 
    RowBox[{"v", ">", 
     RowBox[{
      RowBox[{"-", "Pi"}], "/", "2"}]}], "&&", 
    RowBox[{"v", "<", 
     RowBox[{"Pi", "/", "2"}]}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5868443089369326`*^9, 3.5868443716615205`*^9}, 
   3.6177773788694324`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Sin", "[", "v", "]"}], ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.6275378368525352`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Omega]", "[", "1", "]"}], "[", 
     RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
   RowBox[{
    RowBox[{"r", ">", "0"}], "&&", 
    RowBox[{"v", ">", 
     RowBox[{
      RowBox[{"-", "Pi"}], "/", "2"}]}], "&&", 
    RowBox[{"v", "<", 
     RowBox[{"Pi", "/", "2"}]}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5868443089369326`*^9, 3.5868443716615205`*^9}, {
   3.586844407878592*^9, 3.5868444400784335`*^9}, 3.5868495899029865`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{"-", "1"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6275378371021357`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Omega]", "[", "2", "]"}], "[", 
     RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
   RowBox[{
    RowBox[{"r", ">", "0"}], "&&", 
    RowBox[{"v", ">", 
     RowBox[{
      RowBox[{"-", "Pi"}], "/", "2"}]}], "&&", 
    RowBox[{"v", "<", 
     RowBox[{"Pi", "/", "2"}]}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5868443089369326`*^9, 3.5868443716615205`*^9}, {
   3.586844407878592*^9, 3.5868444400784335`*^9}, 3.5868495899029865`*^9, {
   3.5868496805901737`*^9, 3.5868496948869915`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Cos", "[", "v", "]"}], ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.627537837211336*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Omega]", "[", "3", "]"}], "[", 
     RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
   RowBox[{
    RowBox[{"r", ">", "0"}], "&&", 
    RowBox[{"v", ">", 
     RowBox[{
      RowBox[{"-", "Pi"}], "/", "2"}]}], "&&", 
    RowBox[{"v", "<", 
     RowBox[{"Pi", "/", "2"}]}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5868443089369326`*^9, 3.5868443716615205`*^9}, {
   3.586844407878592*^9, 3.5868444400784335`*^9}, 3.5868495899029865`*^9, {
   3.5868496805901737`*^9, 3.5868497038715057`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Sin", "[", "v", "]"}], ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.627537837289336*^9}]
}, Closed]],

Cell[BoxData[
 StyleBox["\[FilledSquare]",
  FontColor->GrayLevel[0]]], "Input",
 CellChangeTimes->{{3.6177774546699657`*^9, 3.6177774857920203`*^9}}],

Cell[TextData[{
 "With the notations introduced above",
 StyleBox[" the derivation equations",
  FontColor->RGBColor[0, 0, 1]],
 " of the moving frame are the following\n(see [B-R], (41.13))"
}], "Text",
 CellChangeTimes->{{3.586870595603422*^9, 3.5868707256452503`*^9}, 
   3.587276591534601*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "sf", "]"}], " ", "\[Equal]", "  ", 
  RowBox[{
   RowBox[{
    RowBox[{"amf", "[", "1", "]"}], "*", 
    RowBox[{"\[Sigma]", "[", "1", "]"}]}], " ", "+", "  ", 
   RowBox[{
    RowBox[{"amf", "[", "2", "]"}], "*", 
    RowBox[{"\[Sigma]", "[", "2", "]"}], " "}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5868707506520944`*^9, 3.5868707637405176`*^9}, {
  3.5868708348454423`*^9, 3.586870922985597*^9}, {3.614861850049923*^9, 
  3.614861850049923*^9}, {3.6149177827477455`*^9, 3.6149177827477455`*^9}, {
  3.614918705364566*^9, 3.6149187071117687`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"amf", "[", "1", "]"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"amf", "[", "2", "]"}], "*", 
    RowBox[{"\[Omega]", "[", "3", "]"}]}], " ", "-", "  ", 
   RowBox[{
    RowBox[{"amf", "[", "3", "]"}], "*", 
    RowBox[{"\[Omega]", "[", "2", "]"}], " "}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.586870881739125*^9, 3.586871024151775*^9}, {
  3.614861850049923*^9, 3.614861850065523*^9}, {3.6149177827633452`*^9, 
  3.6149177827633452`*^9}, {3.6149187088433714`*^9, 3.6149187119165773`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"amf", "[", "2", "]"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     RowBox[{"amf", "[", "1", "]"}]}], "*", 
    RowBox[{"\[Omega]", "[", "3", "]"}]}], " ", "+", "  ", 
   RowBox[{
    RowBox[{"amf", "[", "3", "]"}], "*", 
    RowBox[{"\[Omega]", "[", "1", "]"}], " "}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5868711037119145`*^9, 3.5868711559408064`*^9}, {
  3.614861850065523*^9, 3.614861850065523*^9}, {3.614917782778945*^9, 
  3.614917782778945*^9}, {3.6149187133361797`*^9, 3.6149187168929863`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"amf", "[", "3", "]"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"amf", "[", "1", "]"}], "*", 
    RowBox[{"\[Omega]", "[", "2", "]"}]}], " ", "-", "  ", 
   RowBox[{
    RowBox[{"amf", "[", "2", "]"}], "*", 
    RowBox[{"\[Omega]", "[", "1", "]"}], " "}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5868712300253363`*^9, 3.5868712863882356`*^9}, 
   3.587276317927952*^9, {3.614861850081123*^9, 3.614861850081123*^9}, {
   3.614917782794545*^9, 3.614917782794545*^9}, {3.6149187181409883`*^9, 
   3.6149187213857937`*^9}}],

Cell[TextData[{
 "Prove these identities with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", e.g."
}], "Text",
 CellChangeTimes->{{3.6177787011713552`*^9, 3.617778723697795*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  StyleBox["Simplify",
   FontColor->GrayLevel[0]], 
  StyleBox["[",
   FontColor->GrayLevel[0]], 
  RowBox[{
   StyleBox[
    RowBox[{
     RowBox[{
      RowBox[{"d", "[", 
       RowBox[{
        RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ".", 
     RowBox[{"{", 
      RowBox[{"s", ",", "t"}], "}"}]}],
    FontColor->GrayLevel[0]], 
   StyleBox["-",
    FontColor->GrayLevel[0]], 
   StyleBox[" ",
    FontColor->GrayLevel[0]], 
   StyleBox[
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "1", "]"}], "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "*", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"\[Omega]", "[", "2", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ".", 
      RowBox[{"{", 
       RowBox[{"s", ",", "t"}], "}"}]}]}],
    FontColor->GrayLevel[0]], 
   StyleBox[" ",
    FontColor->GrayLevel[0]], 
   StyleBox["+",
    FontColor->GrayLevel[0]], 
   StyleBox["  ",
    FontColor->GrayLevel[0]], 
   RowBox[{
    StyleBox[
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "2", "]"}], "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}],
     FontColor->GrayLevel[0]], 
    StyleBox["*",
     FontColor->GrayLevel[0]], 
    RowBox[{
     RowBox[{
      RowBox[{
       StyleBox[
        RowBox[{"\[Omega]", "[", "1", "]"}],
        FontColor->GrayLevel[0]], "[", "sf", "]"}], " ", "[", 
      RowBox[{"u", ",", "v"}], "]"}], 
     StyleBox[".",
      FontColor->GrayLevel[0]], 
     StyleBox[
      RowBox[{"{", 
       RowBox[{"s", ",", "t"}], "}"}],
      FontColor->GrayLevel[0]]}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6177774546699657`*^9, 3.6177774857920203`*^9}, 
   3.617777584618194*^9, {3.617777635658284*^9, 3.617777680789163*^9}, {
   3.6177777202260323`*^9, 3.6177778181162043`*^9}, {3.617777882045117*^9, 
   3.617777930467602*^9}, {3.617778075594657*^9, 3.617778118276332*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.62753791907168*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"?", "\[Omega]"}]], "Input",
 CellChangeTimes->{{3.6227277655282035`*^9, 3.6227277797242284`*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["1.5. The Integrability Conditions", "Subsection",
 CellChangeTimes->{{3.586871511559031*^9, 3.58687153347707*^9}, {
   3.5872725777430253`*^9, 3.5872725852784567`*^9}, 3.6150391162116723`*^9, 
   3.6151171023083725`*^9, 3.617779463887895*^9, 3.6178037497835894`*^9}],

Cell[CellGroupData[{

Cell["1.5.1. Summary", "Subsubsection",
 CellChangeTimes->{{3.5872726720544195`*^9, 3.587272684318121*^9}},
 CellTags->"intcond"],

Cell[TextData[{
 "The local structure forms {\[Sigma][1], \[Sigma][2], \[Omega][1], \
\[Omega][2], \[Omega][3] } define all the local geometric properties of the \
surface which are invariant under Euclidean motions. One can prove that if \
the parameter representations of two surfaces have the same local structure \
forms on the same parameter manifold U, then the surfaces are congruent, i.e. \
they differ by an Euclidean motion only. But the forms are not independent of \
each other: they satisfy the so-called integrability conditions, which are \
obtained by exterior differentiation of the derivation equations. This \
operation yields the null vector at the left side; thus the components of the \
frame vectors at the right side after inserting the decomposition of the \
vector differentials using the derivation equations equal zero too, and this \
gives ",
 StyleBox["the integrability conditions",
  FontColor->RGBColor[0, 0, 1]],
 " (see [B-R] (41.14)):"
}], "Text",
 CellChangeTimes->{{3.5868715874843645`*^9, 3.5868718438864145`*^9}, {
  3.5868718815136805`*^9, 3.5868721317693205`*^9}, {3.5868721625013742`*^9, 
  3.586872176073398*^9}, {3.5868722337713647`*^9, 3.5868725303553286`*^9}, {
  3.5868731941242933`*^9, 3.5868732558838263`*^9}, {3.586956213834609*^9, 
  3.5869562430378604`*^9}, {3.609673188706339*^9, 3.6096731893043737`*^9}, {
  3.6178036508950157`*^9, 3.617803683046672*^9}, {3.617803722686342*^9, 
  3.617803723263543*^9}}],

Cell[BoxData[
 RowBox[{"?", "dd"}]], "Input",
 CellChangeTimes->{{3.615097909507529*^9, 3.61509791044353*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"\[Sigma]", "[", "1", "]"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"\[Sigma]", "[", "2", "]"}]}], "\[Wedge]", 
   RowBox[{"\[Omega]", "[", "3", "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.586872685089178*^9, 3.586872778533523*^9}, 
   3.5868730982448096`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"\[Sigma]", "[", "2", "]"}], "]"}], " ", "\[Equal]", "  ", 
  RowBox[{
   RowBox[{"\[Sigma]", "[", "1", "]"}], "\[Wedge]", 
   RowBox[{"\[Omega]", "[", "3", "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.586872802700906*^9, 3.586872836820857*^9}, 
   3.5868730999729085`*^9}],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"0", " ", "\[Equal]", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"\[Sigma]", "[", "1", "]"}], "\[Wedge]", 
     RowBox[{"\[Omega]", "[", "2", "]"}]}], "  ", "-", 
    RowBox[{
     RowBox[{"\[Sigma]", "[", "2", "]"}], "\[Wedge]", 
     RowBox[{"\[Omega]", "[", "1", "]"}]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.58687286695158*^9, 3.5868729460291033`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"\[Omega]", "[", "1", "]"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"\[Omega]", "[", "2", "]"}]}], "\[Wedge]", 
   RowBox[{"\[Omega]", "[", "3", "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5868729989181285`*^9, 3.5868730083976707`*^9}, 
   3.5868731054782233`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"\[Omega]", "[", "2", "]"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"\[Omega]", "[", "3", "]"}]}], "\[Wedge]", 
   RowBox[{"\[Omega]", "[", "1", "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5868729989181285`*^9, 3.586873060821669*^9}, 
   3.5868731077973557`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"\[Omega]", "[", "3", "]"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"\[Omega]", "[", "1", "]"}]}], "\[Wedge]", 
   RowBox[{"\[Omega]", "[", "2", "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5868729989181285`*^9, 3.5868730307969522`*^9}, {
  3.586873067326041*^9, 3.5868731114075623`*^9}}],

Cell["\<\
Remember that dd denotes the exterior derivation of a differential form. In \
the next subsections we check these formulas verifying them for the general \
surface \
\>", "Text",
 CellChangeTimes->{{3.5868731271234617`*^9, 3.586873157483198*^9}, {
   3.5868734330679607`*^9, 3.5868734850189323`*^9}, 3.6096733558729005`*^9}],

Cell[BoxData[
 RowBox[{"sf", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.586873495594537*^9, 3.5868735006288247`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["1.5.2. Proof of the First Three Integrability Conditions", \
"Subsubsection",
 CellChangeTimes->{{3.5868735471634865`*^9, 3.5868736064348764`*^9}, {
   3.587272697133854*^9, 3.5872727162779493`*^9}, 3.609673071961662*^9, 
   3.609673856896558*^9, {3.6275418923074813`*^9, 3.6275418948658857`*^9}, 
   3.627542023143312*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "3"}], ";"}]], "Input",
 CellChangeTimes->{{3.6178040647481422`*^9, 3.6178040701301517`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{
    RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "sf", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"]}]]]}], "-", 
  FractionBox[
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{"f", "[", "2", "]"}], 
      TagBox[
       RowBox[{"(", 
        RowBox[{"1", ",", "0"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}], " ", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"f", "[", "2", "]"}], 
      TagBox[
       RowBox[{"(", 
        RowBox[{"1", ",", "1"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], 
   SqrtBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"]}]]], "-", 
  FractionBox[
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{"f", "[", "3", "]"}], 
      TagBox[
       RowBox[{"(", 
        RowBox[{"1", ",", "0"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}], " ", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"f", "[", "3", "]"}], 
      TagBox[
       RowBox[{"(", 
        RowBox[{"1", ",", "1"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], 
   SqrtBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"]}]]], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], ")"}], "/", 
   RowBox[{"(", 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], 
      RowBox[{"3", "/", "2"}]]}], ")"}]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], ")"}], "/", 
   RowBox[{"(", 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], 
      RowBox[{"3", "/", "2"}]]}], ")"}]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], ")"}], "/", 
   RowBox[{"(", 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], 
      RowBox[{"3", "/", "2"}]]}], ")"}]}], "+", 
  FractionBox[
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{"f", "[", "1", "]"}], 
      TagBox[
       RowBox[{"(", 
        RowBox[{"0", ",", "1"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}], " ", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"f", "[", "1", "]"}], 
      TagBox[
       RowBox[{"(", 
        RowBox[{"2", ",", "0"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], 
   SqrtBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"]}]]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{"f", "[", "2", "]"}], 
      TagBox[
       RowBox[{"(", 
        RowBox[{"0", ",", "1"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}], " ", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"f", "[", "2", "]"}], 
      TagBox[
       RowBox[{"(", 
        RowBox[{"2", ",", "0"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], 
   SqrtBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"]}]]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{"f", "[", "3", "]"}], 
      TagBox[
       RowBox[{"(", 
        RowBox[{"0", ",", "1"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}], " ", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"f", "[", "3", "]"}], 
      TagBox[
       RowBox[{"(", 
        RowBox[{"2", ",", "0"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], 
   SqrtBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"]}]]], "-", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], ")"}], "/", 
   RowBox[{"(", 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], 
      RowBox[{"3", "/", "2"}]]}], ")"}]}], "-", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], ")"}], "/", 
   RowBox[{"(", 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], 
      RowBox[{"3", "/", "2"}]]}], ")"}]}], "-", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], ")"}], "/", 
   RowBox[{"(", 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], 
      RowBox[{"3", "/", "2"}]]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.6275419059419055`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"dd", "[", 
     RowBox[{
      RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "sf", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], " ", "+", 
   RowBox[{
    RowBox[{"extprod", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Omega]", "[", "3", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "2"}], "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.586873642464937*^9, 3.586873909904234*^9}, {
   3.586873966567475*^9, 3.5868740303521233`*^9}, 3.6096736847557116`*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6275419218267336`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"dd", "[", 
     RowBox[{
      RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "sf", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], " ", "-", 
   RowBox[{
    RowBox[{"extprod", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Omega]", "[", "3", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "2"}], "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.586873642464937*^9, 3.586873909904234*^9}, {
   3.586873966567475*^9, 3.5868740303521233`*^9}, {3.5869225150126677`*^9, 
   3.586922546334459*^9}, 3.6096736868738327`*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.627541946178376*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"extprod", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Omega]", "[", "2", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "2"}], "]"}], "]"}], " ", "-", " ", 
   RowBox[{
    RowBox[{"extprod", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Omega]", "[", "1", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "2"}], "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.586922604102763*^9, 3.5869226717776337`*^9}, {
  3.6096736921791363`*^9, 3.6096736932831993`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6275419728272233`*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["1.5.3. Proof of the Last Three Integrability Conditions", \
"Subsubsection",
 CellChangeTimes->{{3.5868735471634865`*^9, 3.5868736064348764`*^9}, {
   3.5869227233315825`*^9, 3.5869227259707336`*^9}, {3.5872727214142427`*^9, 
   3.5872727254704747`*^9}, 3.6275420292741227`*^9, 3.627543603172095*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"dd", "[", 
     RowBox[{
      RowBox[{"\[Omega]", "[", "1", "]"}], "[", "sf", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], " ", "+", 
   RowBox[{
    RowBox[{"extprod", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"\[Omega]", "[", "2", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Omega]", "[", "3", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "2"}], "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.586873642464937*^9, 3.586873909904234*^9}, {
   3.586873966567475*^9, 3.5868740303521233`*^9}, {3.586922786729209*^9, 
   3.586922801385047*^9}, 3.609673696218367*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6275432072269974`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"dd", "[", 
     RowBox[{
      RowBox[{"\[Omega]", "[", "2", "]"}], "[", "sf", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], " ", "+", 
   RowBox[{
    RowBox[{"extprod", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"\[Omega]", "[", "3", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Omega]", "[", "1", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "2"}], "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.586873642464937*^9, 3.586873909904234*^9}, {
   3.586873966567475*^9, 3.5868740303521233`*^9}, {3.586922786729209*^9, 
   3.586922801385047*^9}, {3.586922842405393*^9, 3.5869228588633347`*^9}, 
   3.6096736984744964`*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6275435889740696`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"dd", "[", 
     RowBox[{
      RowBox[{"\[Omega]", "[", "3", "]"}], "[", "sf", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], " ", "+", 
   RowBox[{
    RowBox[{"extprod", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"\[Omega]", "[", "1", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Omega]", "[", "2", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "2"}], "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.586873642464937*^9, 3.586873909904234*^9}, {
   3.586873966567475*^9, 3.5868740303521233`*^9}, {3.586922786729209*^9, 
   3.586922801385047*^9}, {3.586922842405393*^9, 3.586922905792019*^9}, 
   3.609673706290943*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.627543612935113*^9}]
}, Closed]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["1.6. The Local Uniqueness Theorem", "Subsection",
 CellChangeTimes->{{3.586925078299279*^9, 3.5869250892189035`*^9}, {
   3.586956741153708*^9, 3.586956744008871*^9}, {3.58727274251845*^9, 
   3.587272745541623*^9}, 3.609673862642886*^9, 3.6096744232569513`*^9, 
   3.6275436090533047`*^9, 3.62763300809087*^9}],

Cell["\<\
The following theorem shows that the coincidence of the local forms implies \
the congruence of the  surfaces, at least locally:\
\>", "Text",
 CellChangeTimes->{{3.586933018513775*^9, 3.5869331255938997`*^9}}],

Cell[TextData[{
 StyleBox["Theorem",
  FontColor->RGBColor[0, 0, 1]],
 ". Let U be a 2-dimensional smooth, connected manifold and sf, sh: U \
\[LongRightArrow] ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SuperscriptBox["E", "3"]}], TraditionalForm]]],
 " two immersions, i.e. regular surfaces. Assume that on U adapted moving \
frames \n\t\t\t{fr[i][u,v]}  =  {fr[i][sf][u,v]]} , {a[i][u,v]]}  = \
{fr[i][sh] [u,v]} \n are defined such that the local structure forms of these \
frames coincide. Then there exists an Euclidean motion g such that the images \
of the immersion are congruent under g:\n \t\t\t\t\tsh[u,v] = g(sf[u,v])."
}], "Text",
 CellChangeTimes->{{3.5869251578988323`*^9, 3.5869254337306085`*^9}, {
   3.5869254654274216`*^9, 3.586925472082802*^9}, {3.5869320455321236`*^9, 
   3.586932237993132*^9}, {3.5869322892510633`*^9, 3.5869324135211716`*^9}, {
   3.586932453442455*^9, 3.5869324812730465`*^9}, {3.5869325165130625`*^9, 
   3.5869325244015136`*^9}, {3.5869325595775256`*^9, 3.586932617057813*^9}, {
   3.58693264969768*^9, 3.586932690361006*^9}, {3.586932747721287*^9, 
   3.5869327600729933`*^9}, {3.5869328383714714`*^9, 3.586932856682519*^9}, 
   3.5869335946817303`*^9, 3.586956296499154*^9, 3.586956353798055*^9, {
   3.5869568059534144`*^9, 3.5869568158809824`*^9}, {3.614861850127923*^9, 
   3.6148618502215233`*^9}, {3.6149177828413453`*^9, 3.6149177828881454`*^9}, 
   3.614918908632923*^9, {3.614918984511456*^9, 3.6149189923426695`*^9}, {
   3.6151165748299*^9, 3.615116623223332*^9}, {3.6151166671561446`*^9, 
   3.615116667483745*^9}, 3.615117715287099*^9}],

Cell[TextData[{
 StyleBox["Proof",
  FontColor->RGBColor[0, 0, 1]],
 ". We fix a point  {uo,vo} \[Element] U.  Since the Euclidean group acts \
simply transitively on the set of all orthonormal frames {x, {amf[i]}}  there \
exist a uniquely defined Euclidean motion g such that\n \t\t\tsh[uo,vo] =  \
g(sf(uo,vo)),  a[i][uo,vo]]}   = {g(fr[i][uo,vo])}.\n Then the frame { \
{g(fr[i][sf] [u,v]}},i= 0,1,2,3, is an adapted  moving frame of the immersion \
g(sf[u,v]), and since the exterior differentiation permutes with affine \
transformations (constant linear transformations), the local structure  forms \
of the immersions sh and g(sf) coincide too. Now connect the arbitrary point  \
{u,v} \[Element] U and the point  {uo,vo} with a smooth  curve \[Gamma][t]  \
\[Element] U. Let \[Gamma]'[t] denote its tangent vector. Then the moving \
frame  a[i][u,v]]}   = {g(fr[i][u,v])} satisfies along the curve the linear \
system of differential equations defined by the derivation equations:   "
}], "Text",
 CellChangeTimes->{{3.5869332170111284`*^9, 3.5869333239052424`*^9}, {
   3.5869333657456355`*^9, 3.586933515737215*^9}, {3.586933547283019*^9, 
   3.5869336328259115`*^9}, {3.586933666305827*^9, 3.586933708409235*^9}, {
   3.58693374206616*^9, 3.586933749513586*^9}, {3.586933781593421*^9, 
   3.5869338632340903`*^9}, {3.586933899161145*^9, 3.586933919313298*^9}, {
   3.58693394941702*^9, 3.5869341166735864`*^9}, {3.586934188537697*^9, 
   3.586934343241545*^9}, {3.5869344038260107`*^9, 3.586934414377614*^9}, {
   3.5869344666816053`*^9, 3.5869345057538404`*^9}, {3.586934535931566*^9, 
   3.58693461397803*^9}, {3.586934647222932*^9, 3.586934701882058*^9}, {
   3.586934745481552*^9, 3.586934988265438*^9}, {3.586935061969654*^9, 
   3.586935066201896*^9}, {3.5869381562725463`*^9, 3.586938182793063*^9}, 
   3.586956913361558*^9, {3.5872007151327934`*^9, 3.587200734573905*^9}, 
   3.587200779909498*^9, 3.5882493512465467`*^9, {3.6148618504243236`*^9, 
   3.614861850705124*^9}, {3.6149177830753455`*^9, 3.614917783309346*^9}, {
   3.614919002186287*^9, 3.6149190081142974`*^9}, {3.6151167411822543`*^9, 
   3.6151167415098553`*^9}, {3.6151167961591806`*^9, 3.6151168121258764`*^9}, 
   3.6151168598544564`*^9, {3.61511689040298*^9, 3.6151169114786167`*^9}, {
   3.627632759838233*^9, 3.627632780247069*^9}, 3.62763295111657*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", 
    RowBox[{"g", 
     RowBox[{"(", "sf", ")"}]}], "]"}], "[", 
   RowBox[{"\[Gamma]", "'"}], "]"}], " ", "\[Equal]", "  ", 
  RowBox[{
   RowBox[{
    RowBox[{"a", "[", "1", "]"}], "*", 
    RowBox[{
     RowBox[{"\[Sigma]", "[", "1", "]"}], "[", 
     RowBox[{"\[Gamma]", "'"}], "]"}]}], " ", "+", "  ", 
   RowBox[{
    RowBox[{"a", "[", "2", "]"}], "*", 
    RowBox[{
     RowBox[{"\[Sigma]", "[", "2", "]"}], "[", 
     RowBox[{"\[Gamma]", "'"}], "]"}], " "}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5868707506520944`*^9, 3.5868707637405176`*^9}, {
  3.5868708348454423`*^9, 3.586870922985597*^9}, {3.586935121068034*^9, 
  3.586935134571807*^9}, {3.586935166590638*^9, 3.586935170139841*^9}, {
  3.5869352335524683`*^9, 3.586935234159503*^9}, {3.586935284517383*^9, 
  3.586935293012869*^9}, {3.588249351262147*^9, 3.588249351262147*^9}, {
  3.627633302665989*^9, 3.6276333310298395`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", 
    RowBox[{"a", "[", "1", "]"}], "]"}], "[", 
   RowBox[{"\[Gamma]", "'"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"a", "[", "2", "]"}], "*", 
    RowBox[{
     RowBox[{"\[Omega]", "[", "3", "]"}], "[", 
     RowBox[{"\[Gamma]", "'"}], "]"}]}], " ", "-", "  ", 
   RowBox[{
    RowBox[{"a", "[", "3", "]"}], "*", 
    RowBox[{
     RowBox[{"\[Omega]", "[", "2", "]"}], "[", 
     RowBox[{"\[Gamma]", "'"}], "]"}], " "}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.586870881739125*^9, 3.586871024151775*^9}, {
  3.586935234775538*^9, 3.586935235935604*^9}, {3.5869352988932056`*^9, 
  3.5869353290609307`*^9}, {3.588249351277747*^9, 3.588249351277747*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", 
    RowBox[{"a", "[", "2", "]"}], "]"}], "[", 
   RowBox[{"\[Gamma]", "'"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     RowBox[{"a", "[", "1", "]"}]}], "*", 
    RowBox[{
     RowBox[{"\[Omega]", "[", "3", "]"}], "[", 
     RowBox[{"\[Gamma]", "'"}], "]"}]}], " ", "+", "  ", 
   RowBox[{
    RowBox[{"a", "[", "3", "]"}], "*", 
    RowBox[{
     RowBox[{"\[Omega]", "[", "1", "]"}], "[", 
     RowBox[{"\[Gamma]", "'"}], "]"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5868711037119145`*^9, 3.5868711559408064`*^9}, {
  3.586935236511637*^9, 3.5869352376797037`*^9}, {3.586935304164507*^9, 
  3.5869353410446167`*^9}, {3.588249351277747*^9, 3.588249351277747*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", 
    RowBox[{"a", "[", "3", "]"}], "]"}], "[", 
   RowBox[{"\[Gamma]", "'"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"a", "[", "1", "]"}], "*", 
    RowBox[{
     RowBox[{"\[Omega]", "[", "2", "]"}], "[", 
     RowBox[{"\[Gamma]", "'"}], "]"}]}], " ", "-", "  ", 
   RowBox[{
    RowBox[{"a", "[", "2", "]"}], "*", 
    RowBox[{
     RowBox[{"\[Omega]", "[", "3", "]"}], "[", 
     RowBox[{"\[Gamma]", "'"}], "]"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5868712300253363`*^9, 3.5868712863882356`*^9}, {
   3.586935238407746*^9, 3.586935239094785*^9}, {3.586935307740711*^9, 
   3.586935347892008*^9}, {3.588249351277747*^9, 3.588249351277747*^9}, 
   3.6148618507363243`*^9, 3.6149177833249464`*^9, 3.6149190547115793`*^9}],

Cell["\<\
The moving frame  {sh[\[Gamma][t]], {amf[i][sh] [\[Gamma][t]]}} satisfies the \
same system of differential equations since the local structure forms \
coincide. Since the solutions are uniquely defined by the same  initial \
conditions at the point {uo,vo}, the solution coincide for all t, in \
particular we have 
sh[u,v] = g(sf[u,v]). QED.\
\>", "Text",
 CellChangeTimes->{{3.5869354369941044`*^9, 3.5869354442895217`*^9}, {
   3.58693550440296*^9, 3.5869355558179007`*^9}, {3.5869378853850527`*^9, 
   3.5869381324961863`*^9}, 3.586938166736145*^9, {3.5869381971368837`*^9, 
   3.586938204552308*^9}, {3.586956962889391*^9, 3.586957003976741*^9}, 
   3.586957622721131*^9, {3.5992328267612457`*^9, 3.5992328273532796`*^9}, 
   3.614861850767524*^9, 3.614917783356146*^9, 3.614919070732807*^9, {
   3.6178048719871607`*^9, 3.6178048748419657`*^9}, {3.627632896774674*^9, 
   3.627632918525112*^9}}],

Cell[TextData[{
 StyleBox["A technical hint",
  FontColor->RGBColor[0, 0, 1]],
 ". The notation"
}], "Text",
 CellChangeTimes->{{3.5882281307953806`*^9, 3.588228173978851*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"p\.b4", "[", "v", "]"}], "-", " ", 
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{"p", "[", "v", "]"}], ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5882279420925875`*^9, 3.588227998653823*^9}, {
   3.58822803602296*^9, 3.588228040750231*^9}, {3.5882281856145163`*^9, 
   3.5882281962061224`*^9}, 3.58822826364598*^9, {3.5882283076694975`*^9, 
   3.5882283390942945`*^9}, {3.588249511474428*^9, 3.5882495126600304`*^9}, {
   3.5882498746829853`*^9, 3.5882498819464006`*^9}, {3.5883161677869563`*^9, 
   3.588316178531571*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"p\.b4", "[", "v", "]"}], "-", 
  RowBox[{
   SuperscriptBox["p", "\[Prime]",
    MultilineFunction->None], "[", "v", "]"}]}]], "Output",
 CellChangeTimes->{3.6276329738770103`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6151170140388913`*^9, 3.6151170222756543`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"p\.b4", "[", "v", "]"}], "-", 
  RowBox[{
   SuperscriptBox["p", "\[Prime]",
    MultilineFunction->None], "[", "v", "]"}]}]], "Output",
 CellChangeTimes->{3.6276329798548203`*^9}]
}, Closed]],

Cell["can\[CloseCurlyQuote]t be used for denoting a derivation. Use  \
\[OpenCurlyQuote] instead:", "Text",
 CellChangeTimes->{{3.5882495550609045`*^9, 3.5882496043257914`*^9}, 
   3.5882496943756723`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"p", "'"}], "[", "v", "]"}], "-", " ", 
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{"p", "[", "v", "]"}], ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5882279420925875`*^9, 3.588227998653823*^9}, {
   3.58822803602296*^9, 3.588228040750231*^9}, {3.5882281856145163`*^9, 
   3.5882281962061224`*^9}, 3.58822826364598*^9, {3.5882283076694975`*^9, 
   3.5882283390942945`*^9}, {3.588249511474428*^9, 3.5882495126600304`*^9}, {
   3.588249718586057*^9, 3.5882497197461233`*^9}, {3.588249896938258*^9, 
   3.5882499049297156`*^9}, {3.5882506904956474`*^9, 3.5882507097617493`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6276329870308332`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"p", "''"}], "[", "v", "]"}], "-", " ", 
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{"p", "[", "v", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"v", ",", "2"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5882279420925875`*^9, 3.588227998653823*^9}, {
   3.58822803602296*^9, 3.588228040750231*^9}, {3.5882281856145163`*^9, 
   3.5882281962061224`*^9}, 3.58822826364598*^9, {3.5882283076694975`*^9, 
   3.5882283390942945`*^9}, {3.588249511474428*^9, 3.5882495126600304`*^9}, {
   3.588249718586057*^9, 3.5882497197461233`*^9}, {3.588249896938258*^9, 
   3.5882499049297156`*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["2. Spherical Image and Gaussian Curvature", "Section",
 CellChangeTimes->{{3.587201424502367*^9, 3.5872014450525427`*^9}, {
   3.587272807823185*^9, 3.587272810622345*^9}, 3.615117108938384*^9, 
   3.615196425961876*^9, 3.6178038595453825`*^9, 3.617860550133771*^9}],

Cell[CellGroupData[{

Cell["2.1. The Gauss Map", "Subsection",
 CellChangeTimes->{{3.5872730558623724`*^9, 3.58727307645455*^9}, {
  3.5885090107494955`*^9, 3.5885090141486893`*^9}}],

Cell[CellGroupData[{

Cell["2.1.1. Orientation and Spherical Image", "Subsubsection",
 CellChangeTimes->{{3.587201518412738*^9, 3.587201527076234*^9}, {
   3.587272835342759*^9, 3.58727283885396*^9}, {3.587272981302108*^9, 
   3.5872730078536263`*^9}, {3.587273125318345*^9, 3.5872731524058943`*^9}, 
   3.609674435089628*^9, 3.609674810368093*^9, 3.627633117149062*^9, 
   3.6276338957264566`*^9}],

Cell[TextData[{
 "We consider the ",
 StyleBox["normal unit vector",
  FontColor->RGBColor[0, 0, 1]],
 " amf[3][sf][u,v] of the surface. Belonging to the orthogonal complement of \
the tangent plane  it is uniquely defined up to  sign by the surface. A \
choice of the normal vector is equivalent to define an ",
 StyleBox["orientation of the tangent plane",
  FontColor->RGBColor[0, 0, 1]],
 " at the point sf[u,v]: a pair of linearly independent tangent vectors \
{tu,tv} is said to be positively oriented if the triple \n{tu, tv, \
amf[3][sf][u,v]} is positively oriented, what means Det[{tu, tv, \
amf[3][sf][u,v]}]>0. The surface is said to be ",
 StyleBox["orientable",
  FontColor->RGBColor[0, 0, 1]],
 ", if there exists a continuous unit normal vector field defined globally, \
i.e. for all of its points. Locally, on a parameter region U,  any surface is \
orientable. Since almost all our considerations are of a local character we \
may assume that an ",
 StyleBox["oriented surface",
  FontColor->RGBColor[0, 0, 1]],
 " is given, where the orientation is defined by the choice of the normal \
vector field. The map"
}], "Text",
 CellChangeTimes->{{3.5872016123241096`*^9, 3.587201709836687*^9}, {
   3.587201928221178*^9, 3.5872020310770607`*^9}, {3.5872020625648623`*^9, 
   3.587202289884864*^9}, {3.587202330429183*^9, 3.587202334956442*^9}, {
   3.5872023682943487`*^9, 3.5872024447247205`*^9}, {3.587202514020684*^9, 
   3.587202534012827*^9}, 3.5872033283652616`*^9, {3.587203382236343*^9, 
   3.5872035072444925`*^9}, {3.5872036682127*^9, 3.58720378221322*^9}, {
   3.5872038187973127`*^9, 3.5872038959167233`*^9}, {3.587203950612852*^9, 
   3.5872039793004932`*^9}, {3.5872733423347573`*^9, 3.5872733493661594`*^9}, 
   3.5882544812004633`*^9, {3.6148618508767242`*^9, 3.6148618510327244`*^9}, {
   3.614917783449746*^9, 3.6149177835589466`*^9}, {3.6149194945075517`*^9, 
   3.614919498859959*^9}, 3.615180003675227*^9, 3.615263631108482*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"sph", "[", "sf", "]"}], ":", " ", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"u", ",", "v"}], "}"}], " ", "\[Element]", " ", 
    RowBox[{"U", " ", "\[RightTeeArrow]", 
     RowBox[{
      RowBox[{"sph", "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}]}]}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], " ", "\[Element]", " ", 
   SuperscriptBox["S", "2"]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5872734225673466`*^9, 3.5872734983276796`*^9}, {
   3.5872735378889427`*^9, 3.5872735984794083`*^9}, 3.614861851063925*^9, 
   3.614917783574547*^9, 3.6149195004823623`*^9},
 TextAlignment->Center],

Cell[TextData[{
 " is named the ",
 StyleBox["Gauss map",
  FontColor->RGBColor[0, 0, 1]],
 " and  its range the ",
 StyleBox["spherical image",
  FontColor->RGBColor[0, 0, 1]],
 " of the surface. The range is a part of the unit sphere; eidetic, the range \
 becomes larger if the surface crooks more. As a first example we show:"
}], "Text",
 CellChangeTimes->{{3.587273389727468*^9, 3.5872734079835124`*^9}, {
   3.5872737091427374`*^9, 3.5872737288618655`*^9}, 3.587273759398612*^9, {
   3.5872738016460285`*^9, 3.5872738019420457`*^9}, {3.5872744723103886`*^9, 
   3.5872744980938635`*^9}, {3.587274591870227*^9, 3.5872747256548786`*^9}, 
   3.587274844822695*^9, {3.5872748751184273`*^9, 3.587274896542653*^9}, {
   3.5885089495569954`*^9, 3.588508967285009*^9}, {3.588563973357298*^9, 
   3.588564070217868*^9}, 3.6152638209419174`*^9, {3.6178603571017303`*^9, 
   3.617860398341089*^9}, {3.6276334991839356`*^9, 3.627633522396776*^9}}],

Cell[TextData[{
 StyleBox["Lemma.",
  FontColor->RGBColor[0, 0, 1]],
 "  The connected, oriented surface sf[u,v] is a part of a plane iff its \
spherical image consists of a single point only, and this is the case iff the \
local structure forms \[Omega][1][sf] and \[Omega][2][sf] are null forms."
}], "Text",
 CellChangeTimes->{{3.5872749116795187`*^9, 3.58727492778244*^9}, {
   3.5872756902220488`*^9, 3.587275723389946*^9}, {3.587275757518898*^9, 
   3.587275826734857*^9}, {3.587276885118393*^9, 3.5872770007990093`*^9}, 
   3.587278493639395*^9},
 CellTags->"lem2.1.1"],

Cell[TextData[{
 StyleBox["Proof.",
  FontColor->RGBColor[0, 0, 1]],
 " If the surface is a plane, we may choose a constant orthonormal frame"
}], "Text",
 CellChangeTimes->{{3.5872758457349434`*^9, 3.587276064721469*^9}}],

Cell[" {amf[i][sf][uv]} =  {amf[i]}  =  const, ", "Text",
 CellChangeTimes->{{3.5872758457349434`*^9, 3.587276112414197*^9}, {
  3.614861851079525*^9, 3.6148618511263247`*^9}, {3.6149177835901465`*^9, 
  3.6149177836057463`*^9}, {3.6149195029939666`*^9, 3.614919505521171*^9}}],

Cell["\<\
and then the spherical image consists of the point amf[3] only. Conversely, \
assume that this is the case. Then the last derivation equation is\
\>", "Text",
 CellChangeTimes->{{3.5872758457349434`*^9, 3.587276075154066*^9}, {
   3.587276129198157*^9, 3.5872761676863585`*^9}, 3.587276210702819*^9, {
   3.587276689094181*^9, 3.58727670428605*^9}, 3.609674694729479*^9, 
   3.614861851173125*^9, 3.6149177836369467`*^9, 3.614919510154379*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"amf", "[", "3", "]"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"amf", "[", "1", "]"}], "*", 
    RowBox[{"\[Omega]", "[", "2", "]"}]}], " ", "-", "  ", 
   RowBox[{
    RowBox[{"amf", "[", "2", "]"}], "*", 
    RowBox[{"\[Omega]", "[", "1", "]"}]}]}], " ", "\[Equal]", " ", 
  "0"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.587276222756508*^9, 3.587276254568328*^9}, {
  3.5872766188021607`*^9, 3.587276647768818*^9}, {3.6148618511887245`*^9, 
  3.6148618512043247`*^9}, {3.6149177836369467`*^9, 3.614917783652547*^9}, {
  3.614919511620782*^9, 3.614919514491187*^9}}],

Cell["\<\
what means that the structure forms \[Omega][1][sf] and \[Omega][2][sf] are \
zero. We may assume that the constant plane defined by amf[3].vec[x] == 0, \
i.e. the coordinate (x[1],x[2])-plane, contains a point say sf[u0,v0], of the \
surface. Consider a smooth curve \[Gamma][t] \[Epsilon] U with  starting at \
{u0,v0}. Then the third component\
\>", "Text",
 CellChangeTimes->{{3.5872768218717756`*^9, 3.5872768696945105`*^9}, {
   3.587277024326355*^9, 3.5872770369180756`*^9}, {3.587277075214266*^9, 
   3.5872771185187426`*^9}, {3.5872771569349403`*^9, 3.587277405814175*^9}, 
   3.587277488806922*^9, 3.588564173458849*^9, 3.6148618513291254`*^9, 
   3.6149177837773466`*^9, 3.6149195159887896`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"h", "[", "t", "]"}], " ", "=", " ", 
  RowBox[{
   RowBox[{"amf", "[", "3", "]"}], ".", 
   RowBox[{"sf", "[", 
    RowBox[{"\[Gamma]", "[", "t", "]"}], "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5872774239342117`*^9, 3.5872774471485395`*^9}, {
   3.587277494258234*^9, 3.5872774972324038`*^9}, 3.587277620992483*^9, 
   3.614861851344725*^9, 3.6149177837773466`*^9, 3.614919517252392*^9}],

Cell["suffices", "Text",
 CellChangeTimes->{{3.5872775523315554`*^9, 3.587277562951163*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"h", "[", "0", "]"}], " ", "=", " ", "0"}], ",", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"h", "'"}], "[", "t", "]"}], " ", "=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"amf", "[", "3", "]"}], ".", 
      RowBox[{
       RowBox[{
        RowBox[{"d", "[", "sf", "]"}], "[", 
        RowBox[{"\[Gamma]", "[", "t", "]"}], "]"}], "[", 
       RowBox[{
        RowBox[{"\[Gamma]", "'"}], "[", "t", "]"}], "]"}]}], "\[Equal]", " ", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"amf", "[", "3", "]"}], ".", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"amf", "[", "1", "]"}], "[", "sf", "]"}], "[", 
           RowBox[{"\[Gamma]", "[", "t", "]"}], "]"}], "*", 
          RowBox[{
           RowBox[{"\[Sigma]", "[", "1", "]"}], "[", 
           RowBox[{"\[Gamma]", "[", "t", "]"}], "]"}]}], " ", "+", " ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"amf", "[", "2", "]"}], "[", "sf", "]"}], "[", 
           RowBox[{"\[Gamma]", "[", "t", "]"}], "]"}], "*", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"\[Sigma]", "[", "2", "]"}], "[", 
           RowBox[{"\[Gamma]", "[", "t", "]"}], "]"}]}]}], ")"}], "[", 
       RowBox[{
        RowBox[{"\[Gamma]", "'"}], "[", "t", "]"}], "]"}]}], " ", "\[Equal]", 
     " ", "0"}], ":"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.587277570849615*^9, 3.587277746249647*^9}, {
   3.587277786936974*^9, 3.587278019964302*^9}, {3.588249351308947*^9, 
   3.588249351308947*^9}, 3.5885643651051855`*^9, {3.614861851360325*^9, 
   3.614861851360325*^9}, {3.614917783792947*^9, 3.614917783808547*^9}, {
   3.614919518219593*^9, 3.6149195227748013`*^9}, 3.6178605131526556`*^9}],

Cell["\<\
h[t] is the constant 0.  Thus the curve, and especially the arbitrary points \
sf[\[Gamma][t]],  belong to the (x[1],x[2])-plane, the surface must be a part \
of it. \[FilledSquare]\
\>", "Text",
 CellChangeTimes->{{3.587278097846757*^9, 3.5872782393828526`*^9}, {
  3.587278271978717*^9, 3.5872782972861643`*^9}, {3.5872783359033732`*^9, 
  3.587278410622647*^9}, {3.6151804251099677`*^9, 3.6151804335027823`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["2.1.2. Ruled Surfaces", "Subsubsection",
 CellChangeTimes->{{3.587278731126979*^9, 3.5872787512791314`*^9}, {
   3.587278983598419*^9, 3.5872789947820587`*^9}, 3.609674817385494*^9, 
   3.6096757163849144`*^9, 3.6097331138408127`*^9, 3.6178605575971975`*^9, 
   3.617889654026325*^9, 3.627633901248866*^9, 3.627713371971609*^9}],

Cell[TextData[{
 "A ",
 StyleBox["ruled surface",
  FontColor->RGBColor[0, 0, 1]],
 " is defined as a surface generated by a 1-parametric family  of  lines in \
the space:"
}], "Text",
 CellChangeTimes->{{3.5872798729672885`*^9, 3.587279932870714*^9}, {
   3.5872915077116566`*^9, 3.587291508071677*^9}, {3.587291538183399*^9, 
   3.5872915904473886`*^9}, {3.588140984931316*^9, 3.5881409909841266`*^9}, 
   3.6097331232008295`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ruled", "[", 
    RowBox[{"x_", ",", "y_"}], "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"vec", "[", 
    RowBox[{"x", "[", "v", "]"}], "]"}], " ", "+", " ", 
   RowBox[{
    RowBox[{"vec", "[", 
     RowBox[{"y", "[", "v", "]"}], "]"}], "*", " ", "u"}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5872799484616065`*^9, 3.5872801346102533`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ruled", "[", 
   RowBox[{"x", ",", "y"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5872801382974644`*^9, 3.587280151312209*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"x", "[", "v", "]"}], "[", "1", "]"}], "+", 
    RowBox[{"u", " ", 
     RowBox[{
      RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"x", "[", "v", "]"}], "[", "2", "]"}], "+", 
    RowBox[{"u", " ", 
     RowBox[{
      RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"x", "[", "v", "]"}], "[", "3", "]"}], "+", 
    RowBox[{"u", " ", 
     RowBox[{
      RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.627633926883712*^9}]
}, Closed]],

Cell[TextData[{
 "The curve ruled[x,y][0,v] == vec[x[v]] is the so called ",
 StyleBox["directrix",
  FontColor->RGBColor[0, 0, 1]],
 " of the ruled surface; its lines are named the ",
 StyleBox["rulings",
  FontColor->RGBColor[0, 0, 1]],
 " or the ",
 StyleBox["generators",
  FontColor->RGBColor[0, 0, 1]],
 " of the surface."
}], "Text",
 CellChangeTimes->{{3.5872912912692766`*^9, 3.587291449687338*^9}, {
   3.6151807423774195`*^9, 3.615180771745099*^9}, {3.6151808356567545`*^9, 
   3.6151808665295205`*^9}, {3.6151853507540035`*^9, 
   3.6151853884411592`*^9}, {3.6178620539173813`*^9, 3.6178620744445553`*^9}, 
   3.6178621047092867`*^9}],

Cell[TextData[{
 StyleBox["Example.",
  FontColor->RGBColor[0, 0, 1]],
 " Pl\[UDoubleDot]cker's conoid"
}], "Text",
 CellChangeTimes->{{3.6178621217502613`*^9, 3.6178621217502613`*^9}, 
   3.6178621590303936`*^9, {3.617862243166206*^9, 3.6178622692116957`*^9}, 
   3.6178890558120747`*^9}],

Cell[TextData[{
 "See Pl\[UDoubleDot]cker\[CloseCurlyQuote]s conoid in ",
 Cell[BoxData[
  FormBox[
   ButtonBox["MathWorld",
    BaseStyle->"Hyperlink",
    ButtonData->{
      URL["http://mathworld.wolfram.com/RuledSurface.html"], None},
    ButtonNote->"http://mathworld.wolfram.com/RuledSurface.html"], 
   TraditionalForm]]],
 ". Here one finds the following parameter representation:"
}], "Text",
 CellChangeTimes->{{3.617864676993413*^9, 3.6178646913572345`*^9}, {
  3.617864747428441*^9, 3.617864779605282*^9}, {3.6178648119571323`*^9, 
  3.617864811959132*^9}, {3.6178649148010144`*^9, 3.6178649148050146`*^9}, {
  3.617871561964284*^9, 3.617871589563862*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "pluckerconoid"}]], "Input",
 CellChangeTimes->{{3.627635830615226*^9, 3.6276358374012375`*^9}}],

Cell[BoxData[
 StyleBox["\<\"pluckerconoid[u,v] is a parameter representation of Pl\
\[UDoubleDot]cker's conoid.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.6277081204308586`*^9},
 CellTags->"Info3627711720-5071683"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Context", "[", "pluckerconoid", "\[NoBreak]", "]"}]], "Input",
 CellChangeTimes->{{3.6276342521178365`*^9, 3.627634274102275*^9}}],

Cell[BoxData["\<\"eudiffgeov2`\"\>"], "Output",
 CellChangeTimes->{3.6276342752410774`*^9, 3.6276358479644566`*^9, 
  3.627708124206065*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"pluckerconoid", "\[NoBreak]", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.627634622301491*^9, 3.6276346261546974`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"u", " ", 
    RowBox[{"Cos", "[", "v", "]"}]}], ",", 
   RowBox[{"u", " ", 
    RowBox[{"Sin", "[", "v", "]"}]}], ",", 
   RowBox[{"2", " ", 
    RowBox[{"Cos", "[", "v", "]"}], " ", 
    RowBox[{"Sin", "[", "v", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.627634628621502*^9, 3.627634778752967*^9, 
  3.6276358518956633`*^9, 3.627708128589673*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"pluckerconoid", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6178629384309726`*^9, 3.6178630199276342`*^9}, 
   3.6178631166731677`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"plue", " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.627710413806487*^9, 3.6277104211540995`*^9}}],

Cell["In Alfred Gray\[CloseCurlyQuote]s collection SURF.m of surfaces we \
find", "Text",
 CellChangeTimes->{{3.6178716140272617`*^9, 3.617871654019549*^9}, {
  3.6276361523764563`*^9, 3.6276361548724604`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "plucker"}]], "Input",
 CellChangeTimes->{{3.6178651419460063`*^9, 3.61786514672528*^9}}],

Cell[BoxData[
 StyleBox["\<\"{u,v}->plucker[u,v] is the standard parametrization of \
Plucker's surface. {u,v}->plucker[n][u,v] is a parametrization of Plucker's \
surface with n folds. {u,v}->plucker[m,n,a][u,v] is a generalization of the \
monkey saddle and Plucker's surface.  To plot try plucker[-1,1,-1,1].\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.6276358727881002`*^9},
 CellTags->"Info3627639472-8560585"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"plucker", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6178631833399806`*^9, 3.617863203720146*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"u", ",", "v", ",", 
   FractionBox[
    RowBox[{"2", " ", "u", " ", "v"}], 
    RowBox[{
     SuperscriptBox["u", "2"], "+", 
     SuperscriptBox["v", "2"]}]]}], "}"}]], "Output",
 CellChangeTimes->{3.6276359016073513`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"plucker", "[", "n", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6178631833399806`*^9, 3.617863203720146*^9}, {
  3.617865251261259*^9, 3.617865266631138*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"u", ",", "v", ",", 
   RowBox[{"Sin", "[", 
    RowBox[{"n", " ", 
     RowBox[{"ArcTan", "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627635911187768*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"plucker", "[", 
   RowBox[{"m", ",", "n", ",", "a"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6178631833399806`*^9, 3.617863203720146*^9}, {
  3.617865251261259*^9, 3.617865266631138*^9}, {3.6178653096295977`*^9, 
  3.617865327110597*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", " ", "u"}], ",", 
   RowBox[{"a", " ", "v"}], ",", 
   RowBox[{"a", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"u", "+", "v"}], ")"}], 
     RowBox[{"m", "/", "2"}]], " ", 
    RowBox[{"Sin", "[", 
     RowBox[{"n", " ", 
      RowBox[{"ArcTan", "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6276359283829985`*^9}]
}, Closed]],

Cell[TextData[{
 "The parameter transformation u ->  ",
 Cell[BoxData[
  RowBox[{"u", " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["v", "2"]}]]}]],
  CellChangeTimes->{3.6178655826392126`*^9, 3.617865915217235*^9}],
 ", v-> u*v, leads to another representation:"
}], "Text",
 CellChangeTimes->{{3.617871698939118*^9, 3.617871791051387*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"plucker", "[", 
    RowBox[{
     RowBox[{"u", "*", 
      RowBox[{"Sqrt", "[", 
       RowBox[{"1", "-", 
        RowBox[{"v", "^", "2"}]}], "]"}]}], ",", 
     RowBox[{"u", "*", "v"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "\[Element]", "Reals"}], ",", 
     RowBox[{
      RowBox[{"-", "1"}], "<", "v"}], ",", 
     RowBox[{"v", "<", "1"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6178631833399806`*^9, 3.617863203720146*^9}, {
  3.617865251261259*^9, 3.617865266631138*^9}, {3.6178653096295977`*^9, 
  3.617865327110597*^9}, {3.617865529005145*^9, 3.6178655743677397`*^9}, {
  3.617865892209919*^9, 3.6178659117370358`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"u", " ", 
    SqrtBox[
     RowBox[{"1", "-", 
      SuperscriptBox["v", "2"]}]]}], ",", 
   RowBox[{"u", " ", "v"}], ",", 
   RowBox[{"2", " ", "v", " ", 
    SqrtBox[
     RowBox[{"1", "-", 
      SuperscriptBox["v", "2"]}]]}]}], "}"}]], "Output",
 CellChangeTimes->{3.627635946808631*^9, 3.627636290993101*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"u", "*", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Sqrt", "[", 
       RowBox[{"1", "-", 
        RowBox[{"v", "^", "2"}]}], "]"}], ",", "v", ",", "0"}], "}"}]}], " ", 
   "+", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", 
     RowBox[{"2", "v", "*", 
      RowBox[{"Sqrt", "[", 
       RowBox[{"1", "-", 
        RowBox[{"v", "^", "2"}]}], "]"}]}]}], "}"}]}]}]], "Input",
 CellChangeTimes->{{3.6276363173319473`*^9, 3.6276364753580265`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.62763652229091*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", " ", 
      SqrtBox[
       RowBox[{"1", "-", 
        SuperscriptBox["v", "2"]}]]}], ",", 
     RowBox[{"u", " ", "v"}], ",", 
     RowBox[{"2", " ", "v", " ", 
      SqrtBox[
       RowBox[{"1", "-", 
        SuperscriptBox["v", "2"]}]]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6178656197713366`*^9, 3.6178657398552046`*^9}, {
  3.6178659266398883`*^9, 3.617866004487341*^9}, {3.617866533208582*^9, 
  3.6178665589580545`*^9}, {3.627704276657631*^9, 3.6277042993088703`*^9}}],

Cell["Sometimes this is replaced by", "Text",
 CellChangeTimes->{{3.6178718182349415`*^9, 3.617871829883608*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", " ", 
      SqrtBox[
       RowBox[{"1", "-", 
        SuperscriptBox["v", "2"]}]]}], ",", 
     RowBox[{"u", " ", "v"}], ",", 
     RowBox[{"1", "-", 
      RowBox[{"v", "^", "2"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6178656197713366`*^9, 3.6178657398552046`*^9}, {
  3.6178659266398883`*^9, 3.617866004487341*^9}, {3.6178662201736774`*^9, 
  3.6178662267100515`*^9}, {3.6178662869614973`*^9, 3.6178663606737137`*^9}, {
  3.627704398010244*^9, 3.62770440127065*^9}}],

Cell["\<\
The parameter transformation u -> u*Cos[v], v -> u*Sin[v] leads to the first \
representation \
\>", "Text",
 CellChangeTimes->{{3.6178718556990843`*^9, 3.6178719641712885`*^9}, {
  3.627636615209074*^9, 3.627636617517878*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"plucker", "[", 
  RowBox[{
   RowBox[{"u", "*", 
    RowBox[{"Cos", "[", "v", "]"}]}], ",", 
   RowBox[{"u", "*", 
    RowBox[{"Sin", "[", "v", "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6178631833399806`*^9, 3.617863203720146*^9}, {
  3.6178633404619675`*^9, 3.6178633596330643`*^9}, {3.617863499920088*^9, 
  3.6178635314698925`*^9}, {3.6178642016802263`*^9, 3.6178642248475513`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"u", " ", 
    RowBox[{"Cos", "[", "v", "]"}]}], ",", 
   RowBox[{"u", " ", 
    RowBox[{"Sin", "[", "v", "]"}]}], ",", 
   FractionBox[
    RowBox[{"2", " ", 
     SuperscriptBox["u", "2"], " ", 
     RowBox[{"Cos", "[", "v", "]"}], " ", 
     RowBox[{"Sin", "[", "v", "]"}]}], 
    RowBox[{
     RowBox[{
      SuperscriptBox["u", "2"], " ", 
      SuperscriptBox[
       RowBox[{"Cos", "[", "v", "]"}], "2"]}], "+", 
     RowBox[{
      SuperscriptBox["u", "2"], " ", 
      SuperscriptBox[
       RowBox[{"Sin", "[", "v", "]"}], "2"]}]}]]}], "}"}]], "Output",
 CellChangeTimes->{3.627635987970504*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", ",", " ", 
   RowBox[{"u", "\[Element]", "Reals"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6178642461317687`*^9, 3.6178642963036385`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"u", " ", 
    RowBox[{"Cos", "[", "v", "]"}]}], ",", 
   RowBox[{"u", " ", 
    RowBox[{"Sin", "[", "v", "]"}]}], ",", 
   RowBox[{"Sin", "[", 
    RowBox[{"2", " ", "v"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6276359953961167`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"pluckerconoid", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6178719880536547`*^9, 3.6178720015174246`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"u", " ", 
    RowBox[{"Cos", "[", "v", "]"}]}], ",", 
   RowBox[{"u", " ", 
    RowBox[{"Sin", "[", "v", "]"}]}], ",", 
   RowBox[{"2", " ", 
    RowBox[{"Cos", "[", "v", "]"}], " ", 
    RowBox[{"Sin", "[", "v", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6276350952167263`*^9}]
}, Closed]],

Cell["We will find the singularities of \
Pl\[UDoubleDot]cker\[CloseCurlyQuote]s conoid:", "Text",
 CellChangeTimes->{{3.6277045326072807`*^9, 3.6277045719817495`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "pluckerconoid", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.62763666404416*^9, 3.627636689369005*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Cos", "[", "v", "]"}], ",", 
     RowBox[{
      RowBox[{"-", "u"}], " ", 
      RowBox[{"Sin", "[", "v", "]"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Sin", "[", "v", "]"}], ",", 
     RowBox[{"u", " ", 
      RowBox[{"Cos", "[", "v", "]"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", "v", "]"}], "2"]}], "-", 
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"Sin", "[", "v", "]"}], "2"]}]}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6276368792553406`*^9}]
}, Closed]],

Cell["In general, almost all of the points are regular:", "Text",
 CellChangeTimes->{{3.6277054595765085`*^9, 3.627705484240152*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MatrixRank", "[", 
  RowBox[{
   RowBox[{"d", "[", "pluckerconoid", "]"}], "[", 
   RowBox[{"u", ",", 
    RowBox[{"Pi", "/", "4"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.627636783220171*^9, 3.6276368132864237`*^9}, {
  3.6276369338332367`*^9, 3.627636937501243*^9}}],

Cell[BoxData["2"], "Output",
 CellChangeTimes->{3.627636815657628*^9, 3.6276369007873783`*^9, 
  3.6276369448800564`*^9}]
}, Closed]],

Cell["but there exist singularities, e.g. ", "Text",
 CellChangeTimes->{{3.6277054904021626`*^9, 3.6277055388090477`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MatrixRank", "[", 
  RowBox[{
   RowBox[{"d", "[", "pluckerconoid", "]"}], "[", 
   RowBox[{"0", ",", 
    RowBox[{"Pi", "/", "4"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.627636783220171*^9, 3.6276368420256743`*^9}, {
  3.6276369624642873`*^9, 3.6276369687686987`*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{{3.6276368323536577`*^9, 3.6276368435720778`*^9}, 
   3.627636970047901*^9}]
}, Closed]],

Cell["\<\
All singularities of Pl\[UDoubleDot]cker\[CloseCurlyQuote]s conoid can be \
found as the solutions of\
\>", "Text",
 CellChangeTimes->{{3.6276391377005286`*^9, 3.6276391796072025`*^9}, {
  3.6277045172412534`*^9, 3.6277045185048556`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reduce", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Cross", "[", 
     RowBox[{
      RowBox[{"D", "[", 
       RowBox[{
        RowBox[{"pluckerconoid", "[", 
         RowBox[{"u", ",", "v"}], "]"}], ",", "u"}], "]"}], ",", 
      RowBox[{"D", "[", 
       RowBox[{
        RowBox[{"pluckerconoid", "[", 
         RowBox[{"u", ",", "v"}], "]"}], ",", "v"}], "]"}]}], "]"}], 
    "\[Equal]", " ", 
    RowBox[{"null", "[", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "v"}], "}"}], ",", "Reals"}], "]"}]], "Input",
 CellChangeTimes->{{3.6276377756515236`*^9, 3.6276377974507623`*^9}, {
  3.6276378401384373`*^9, 3.6276378469888496`*^9}, {3.627637964227457*^9, 
  3.627638125448541*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"C", "[", "1", "]"}], "\[Element]", "Integers"}], "&&", 
  RowBox[{"u", "\[Equal]", "0"}], "&&", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"v", "\[Equal]", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "2"}], " ", 
       RowBox[{"ArcTan", "[", 
        RowBox[{"1", "+", 
         SqrtBox["2"]}], "]"}]}], "+", 
      RowBox[{"2", " ", "\[Pi]", " ", 
       RowBox[{"C", "[", "1", "]"}]}]}]}], "||", 
    RowBox[{"v", "\[Equal]", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "2"}], " ", 
       RowBox[{"ArcTan", "[", 
        RowBox[{"1", "-", 
         SqrtBox["2"]}], "]"}]}], "+", 
      RowBox[{"2", " ", "\[Pi]", " ", 
       RowBox[{"C", "[", "1", "]"}]}]}]}], "||", 
    RowBox[{"v", "\[Equal]", 
     RowBox[{
      RowBox[{"2", " ", 
       RowBox[{"ArcTan", "[", 
        RowBox[{"1", "-", 
         SqrtBox["2"]}], "]"}]}], "+", 
      RowBox[{"2", " ", "\[Pi]", " ", 
       RowBox[{"C", "[", "1", "]"}]}]}]}], "||", 
    RowBox[{"v", "\[Equal]", 
     RowBox[{
      RowBox[{"2", " ", 
       RowBox[{"ArcTan", "[", 
        RowBox[{"1", "+", 
         SqrtBox["2"]}], "]"}]}], "+", 
      RowBox[{"2", " ", "\[Pi]", " ", 
       RowBox[{"C", "[", "1", "]"}]}]}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.627638134452757*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", 
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{"ArcTan", "[", 
     RowBox[{"1", "+", 
      SqrtBox["2"]}], "]"}]}], "-", 
   RowBox[{"3", "*", 
    RowBox[{"Pi", "/", "4"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6276383924558125`*^9, 3.6276384450203056`*^9}, {
   3.6276384817643704`*^9, 3.62763852707485*^9}, 3.6276385653955183`*^9, 
   3.6276386286188297`*^9, {3.6276386919619417`*^9, 3.627638698313153*^9}}],

Cell[BoxData["0.`"], "Output",
 CellChangeTimes->{{3.6276384013010283`*^9, 3.6276384462371073`*^9}, {
   3.6276384859159775`*^9, 3.627638528650453*^9}, 3.627638568219123*^9, 
   3.6276386299292316`*^9, 3.627638704085163*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", 
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{"ArcTan", "[", 
     RowBox[{"1", "-", 
      SqrtBox["2"]}], "]"}]}], "+", 
   RowBox[{"Pi", "/", "4"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6276383924558125`*^9, 3.6276384450203056`*^9}, {
   3.6276384817643704`*^9, 3.62763852707485*^9}, 3.6276385653955183`*^9, {
   3.6276386286188297`*^9, 3.6276386594786844`*^9}, {3.627638735166418*^9, 
   3.6276387382552233`*^9}, {3.6276387696640787`*^9, 3.627638778776495*^9}, {
   3.6276389958878784`*^9, 3.627639012769108*^9}}],

Cell[BoxData[
 RowBox[{"-", "2.220446049250313`*^-16"}]], "Output",
 CellChangeTimes->{
  3.6276386618508883`*^9, 3.6276387430308313`*^9, 3.6276387809136987`*^9, {
   3.6276390039686923`*^9, 3.6276390140951104`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Cross", "[", 
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"pluckerconoid", "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", "u"}], "]"}], ",", 
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"pluckerconoid", "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", "v"}], "]"}]}], "]"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"u", "\[Rule]", "0"}], ",", 
    RowBox[{"v", "\[Rule]", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", "*", 
         RowBox[{"RandomInteger", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "100"}], ",", "100"}], "}"}], "]"}]}], "+", "1"}], 
       ")"}], "*", 
      RowBox[{"Pi", "/", "4"}]}]}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.627638272456001*^9, 3.6276382915076346`*^9}, {
   3.6276392224664783`*^9, 3.627639225290083*^9}, 3.627639256933939*^9, {
   3.6276392956114073`*^9, 3.6276393412504883`*^9}, {3.6276393727353435`*^9, 
   3.6276393851569653`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.627638298309246*^9, 3.627639227520887*^9, 
  3.6276393953437834`*^9}]
}, Closed]],

Cell["Here is the self intersection set of the surface", "Text",
 CellChangeTimes->{{3.627711158021394*^9, 3.627711189299449*^9}, {
  3.627711637584737*^9, 3.627711661563979*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"pluckerconoid", "[", 
    RowBox[{"0", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", " ", 
   RowBox[{"PlotStyle", " ", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Red", ",", "Thick"}], "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6277104558797607`*^9, 3.6277105779811754`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"plz", " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.6277106014904165`*^9, 3.6277106082452283`*^9}}],

Cell["and here the singular points", "Text",
 CellChangeTimes->{{3.6277116754324036`*^9, 3.627711692828434*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"Graphics3D", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"PointSize", "[", "Large", "]"}], ",", 
     RowBox[{"Point", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"pluckerconoid", "[", 
         RowBox[{"0", ",", 
          RowBox[{"Pi", "/", "4"}]}], "]"}], ",", 
        RowBox[{"pluckerconoid", "[", 
         RowBox[{"0", ",", 
          RowBox[{"3", "*", 
           RowBox[{"Pi", "/", "4"}]}]}], "]"}]}], "}"}], "]"}]}], "}"}], 
   "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6277106381816807`*^9, 3.6277106719557405`*^9}, {
  3.6277107148246155`*^9, 3.627710787973144*^9}, {3.627710976249875*^9, 
  3.627711010975536*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"plupts", "=", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.627711020179552*^9, 3.6277110358731794`*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"plue", ",", "plz", ",", "plupts"}], "}"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.627711044203594*^9, 3.6277111076645055`*^9}}],

Cell["The normal vector of Pl\[UDoubleDot]ckers conoid is/", "Text",
 CellChangeTimes->{{3.627705618790388*^9, 3.627705637682021*^9}, 
   3.6277103163063154`*^9, {3.627711708196461*^9, 3.6277117299116993`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"amf", "[", "3", "]"}], "[", "pluckerconoid", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"Element", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"u", ",", "v"}], "}"}], ",", "Reals"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6276370178241854`*^9, 3.6276371021573343`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{"Cos", "[", 
      RowBox[{"2", " ", "v"}], "]"}], " ", 
     RowBox[{"Sin", "[", "v", "]"}]}], 
    SqrtBox[
     RowBox[{"2", "+", 
      SuperscriptBox["u", "2"], "+", 
      RowBox[{"2", " ", 
       RowBox[{"Cos", "[", 
        RowBox[{"4", " ", "v"}], "]"}]}]}]]], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{"Cos", "[", "v", "]"}], "+", 
      RowBox[{"Cos", "[", 
       RowBox[{"3", " ", "v"}], "]"}]}], 
     SqrtBox[
      RowBox[{"2", "+", 
       SuperscriptBox["u", "2"], "+", 
       RowBox[{"2", " ", 
        RowBox[{"Cos", "[", 
         RowBox[{"4", " ", "v"}], "]"}]}]}]]]}], ",", 
   FractionBox["u", 
    SqrtBox[
     RowBox[{"2", "+", 
      SuperscriptBox["u", "2"], "+", 
      RowBox[{"2", " ", 
       RowBox[{"Cos", "[", 
        RowBox[{"4", " ", "v"}], "]"}]}]}]]]}], "}"}]], "Output",
 CellChangeTimes->{3.6276370467370367`*^9, 3.6276371043101377`*^9, 
  3.6276372757854404`*^9}]
}, Closed]],

Cell["The Gauss map is", "Text",
 CellChangeTimes->{{3.6276374654251757`*^9, 3.627637487579215*^9}, {
  3.6277056821420994`*^9, 3.6277056823293*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"sphim", "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
   RowBox[{"{", 
    RowBox[{
     FractionBox[
      RowBox[{"2", " ", 
       RowBox[{"Cos", "[", 
        RowBox[{"2", " ", "v"}], "]"}], " ", 
       RowBox[{"Sin", "[", "v", "]"}]}], 
      SqrtBox[
       RowBox[{"2", "+", 
        SuperscriptBox["u", "2"], "+", 
        RowBox[{"2", " ", 
         RowBox[{"Cos", "[", 
          RowBox[{"4", " ", "v"}], "]"}]}]}]]], ",", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{
        RowBox[{"Cos", "[", "v", "]"}], "+", 
        RowBox[{"Cos", "[", 
         RowBox[{"3", " ", "v"}], "]"}]}], 
       SqrtBox[
        RowBox[{"2", "+", 
         SuperscriptBox["u", "2"], "+", 
         RowBox[{"2", " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"4", " ", "v"}], "]"}]}]}]]]}], ",", 
     FractionBox["u", 
      SqrtBox[
       RowBox[{"2", "+", 
        SuperscriptBox["u", "2"], "+", 
        RowBox[{"2", " ", 
         RowBox[{"Cos", "[", 
          RowBox[{"4", " ", "v"}], "]"}]}]}]]]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6276372851630573`*^9, 3.6276373458435645`*^9}, 
   3.6277058416991796`*^9}],

Cell["\<\
The normal vector is not defined at the singularities, e.g. for u==0, v==Pi/4\
\>", "Text",
 CellChangeTimes->{{3.6276376036332197`*^9, 3.6276376579157157`*^9}, {
  3.627639506353979*^9, 3.627639520740205*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sphim", "[", 
  RowBox[{"0", ",", 
   RowBox[{"Pi", "/", "4"}]}], "]"}]], "Input",
 CellChangeTimes->{3.627637693114378*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Power", "::", "infy"}], "MessageName"], ":", 
  " ", "\<\"Infinite expression \[NoBreak]\\!\\(1\\/\\@0\\)\[NoBreak] \
encountered. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/infy\\\", ButtonNote -> \
\\\"Power::infy\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.6276376957995825`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Infinity", "::", "indet"}], "MessageName"], ":", 
  " ", "\<\"Indeterminate expression \[NoBreak]\\!\\(\\(0\\\\ ComplexInfinity\
\\)\\/\\@2\\)\[NoBreak] encountered. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/indet\\\", \
ButtonNote -> \\\"Infinity::indet\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.627637695830783*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Power", "::", "infy"}], "MessageName"], ":", 
  " ", "\<\"Infinite expression \[NoBreak]\\!\\(1\\/\\@0\\)\[NoBreak] \
encountered. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/infy\\\", ButtonNote -> \
\\\"Power::infy\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.627637695861983*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Infinity", "::", "indet"}], "MessageName"], ":", 
  " ", "\<\"Indeterminate expression \[NoBreak]\\!\\(0\\\\ ComplexInfinity\\)\
\[NoBreak] encountered. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/indet\\\", ButtonNote -> \
\\\"Infinity::indet\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.627637695908783*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Power", "::", "infy"}], "MessageName"], ":", 
  " ", "\<\"Infinite expression \[NoBreak]\\!\\(1\\/\\@0\\)\[NoBreak] \
encountered. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/infy\\\", ButtonNote -> \
\\\"Power::infy\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.627637695939983*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], ":", 
  " ", "\<\"Further output of \[NoBreak]\\!\\(\\*StyleBox[\\(Power :: \
infy\\), \\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/stop\\\", ButtonNote -> \
\\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.6276376959711833`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Infinity", "::", "indet"}], "MessageName"], ":", 
  " ", "\<\"Indeterminate expression \[NoBreak]\\!\\(0\\\\ ComplexInfinity\\)\
\[NoBreak] encountered. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/indet\\\", ButtonNote -> \
\\\"Infinity::indet\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.627637696002383*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], ":", 
  " ", "\<\"Further output of \[NoBreak]\\!\\(\\*StyleBox[\\(Infinity :: \
indet\\), \\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/stop\\\", ButtonNote -> \
\\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.627637696033583*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"Indeterminate", ",", "Indeterminate", ",", "Indeterminate"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.627637696033583*^9}]
}, Closed]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " neglects the singularities:"
}], "Text",
 CellChangeTimes->{{3.6276396265711913`*^9, 3.627639672472472*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sphim", "[", 
     RowBox[{"u", ",", "v"}], "]"}], ".", 
    RowBox[{"sphim", "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], ",", 
   RowBox[{"Element", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"u", ",", "v"}], "}"}], ",", "Reals"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.627637131491386*^9, 3.62763716789225*^9}, {
  3.6276395650960827`*^9, 3.6276395909669285`*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.6276371702186546`*^9, 3.627639594586135*^9}]
}, Closed]],

Cell["Now we will find the Singularities of the Gauss map:", "Text",
 CellChangeTimes->{{3.627640568548254*^9, 3.627640572526261*^9}, {
   3.6276406312547646`*^9, 3.627640666748827*^9}, 3.6277075401233644`*^9, 
   3.6277134619681673`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "sphim", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"Element", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"u", ",", "v"}], "}"}], ",", "Reals"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.627639717563552*^9, 3.6276397958876905`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{"2", " ", "u", " ", 
        RowBox[{"Cos", "[", 
         RowBox[{"2", " ", "v"}], "]"}], " ", 
        RowBox[{"Sin", "[", "v", "]"}]}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"2", "+", 
          SuperscriptBox["u", "2"], "+", 
          RowBox[{"2", " ", 
           RowBox[{"Cos", "[", 
            RowBox[{"4", " ", "v"}], "]"}]}]}], ")"}], 
        RowBox[{"3", "/", "2"}]]]}], ",", 
     FractionBox[
      RowBox[{"2", " ", 
       RowBox[{"Cos", "[", "v", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "2"}], " ", 
          SuperscriptBox["u", "2"]}], "+", 
         RowBox[{"3", " ", 
          RowBox[{"(", 
           RowBox[{"1", "+", 
            SuperscriptBox["u", "2"]}], ")"}], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"2", " ", "v"}], "]"}]}], "+", 
         RowBox[{"Cos", "[", 
          RowBox[{"6", " ", "v"}], "]"}]}], ")"}]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"2", "+", 
         SuperscriptBox["u", "2"], "+", 
         RowBox[{"2", " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"4", " ", "v"}], "]"}]}]}], ")"}], 
       RowBox[{"3", "/", "2"}]]]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     FractionBox[
      RowBox[{"u", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Cos", "[", "v", "]"}], "+", 
         RowBox[{"Cos", "[", 
          RowBox[{"3", " ", "v"}], "]"}]}], ")"}]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"2", "+", 
         SuperscriptBox["u", "2"], "+", 
         RowBox[{"2", " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"4", " ", "v"}], "]"}]}]}], ")"}], 
       RowBox[{"3", "/", "2"}]]], ",", 
     FractionBox[
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", 
          SuperscriptBox["u", "2"]}], "+", 
         RowBox[{"3", " ", 
          RowBox[{"(", 
           RowBox[{"1", "+", 
            SuperscriptBox["u", "2"]}], ")"}], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"2", " ", "v"}], "]"}]}], "+", 
         RowBox[{"Cos", "[", 
          RowBox[{"6", " ", "v"}], "]"}]}], ")"}], " ", 
       RowBox[{"Sin", "[", "v", "]"}]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"2", "+", 
         SuperscriptBox["u", "2"], "+", 
         RowBox[{"2", " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"4", " ", "v"}], "]"}]}]}], ")"}], 
       RowBox[{"3", "/", "2"}]]]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     FractionBox[
      RowBox[{"4", " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", 
         RowBox[{"2", " ", "v"}], "]"}], "2"]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"2", "+", 
         SuperscriptBox["u", "2"], "+", 
         RowBox[{"2", " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"4", " ", "v"}], "]"}]}]}], ")"}], 
       RowBox[{"3", "/", "2"}]]], ",", 
     FractionBox[
      RowBox[{"4", " ", "u", " ", 
       RowBox[{"Sin", "[", 
        RowBox[{"4", " ", "v"}], "]"}]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"2", "+", 
         SuperscriptBox["u", "2"], "+", 
         RowBox[{"2", " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"4", " ", "v"}], "]"}]}]}], ")"}], 
       RowBox[{"3", "/", "2"}]]]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627640321531618*^9, 3.627640487854912*^9}]
}, Closed]],

Cell["In general, the rank is", "Text",
 CellChangeTimes->{{3.6276406994056845`*^9, 3.627640708812501*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MatrixRank", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6276404634660683`*^9, 3.627640474062487*^9}}],

Cell[BoxData["2"], "Output",
 CellChangeTimes->{3.627640491302518*^9}]
}, Closed]],

Cell["But in reality there exists continua of singularities", "Text",
 CellChangeTimes->{{3.6276407905730143`*^9, 3.6276408133042545`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"Element", "[", 
    RowBox[{"k", ",", "Integers"}], "]"}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{"(", 
     RowBox[{"%%", "/.", " ", 
      RowBox[{"v", "\[Rule]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", "k"}], "+", "1"}], ")"}], "*", " ", 
        RowBox[{"Pi", "/", "4"}]}]}]}], ")"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6276398503251867`*^9, 3.6276398605900044`*^9}, 
   3.627640000325851*^9, {3.6276400313417053`*^9, 3.627640073937781*^9}, {
   3.627640182874173*^9, 3.6276402154188304`*^9}, 3.6276402460310845`*^9, {
   3.6276402929327674`*^9, 3.627640311015199*^9}, 3.627640483235303*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{"Cos", "[", 
         RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{"(", 
           RowBox[{"1", "+", 
            RowBox[{"2", " ", "k"}]}], ")"}], " ", "\[Pi]"}], "]"}]}], 
       SqrtBox[
        SuperscriptBox["u", "2"]]]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", 
     FractionBox[
      RowBox[{"4", " ", 
       RowBox[{"Sin", "[", 
        RowBox[{
         FractionBox["1", "4"], " ", 
         RowBox[{"(", 
          RowBox[{"1", "+", 
           RowBox[{"2", " ", "k"}]}], ")"}], " ", "\[Pi]"}], "]"}]}], 
      SqrtBox[
       SuperscriptBox["u", "2"]]]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.6276398652856126`*^9, {3.6276402285736537`*^9, 3.627640267734723*^9}, 
   3.627640339194849*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MatrixRank", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.627639906213485*^9, 3.6276399193039083`*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.627639922689114*^9, 3.627640397998353*^9}]
}, Closed]],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.6178720672371836`*^9, 3.6178720672391834`*^9}}],

Cell["\<\
Considering a general ruled surface we may assume that the tangent vectors of \
the lines \
\>", "Text",
 CellChangeTimes->{{3.5872802010710545`*^9, 3.587280211198634*^9}, {
   3.5872802433104706`*^9, 3.587280252998025*^9}, 3.5872900805590277`*^9, 
   3.58729163854414*^9, 3.627641063746297*^9, {3.627710251051401*^9, 
   3.627710285511862*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"ruled", "[", 
      RowBox[{"x", ",", "y"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", "u"}], "]"}], "\[Equal]", " ", 
  RowBox[{"vec", "[", 
   RowBox[{"y", "[", "v", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.587280265413735*^9, 3.5872802986966386`*^9}, {
  3.609733214002528*^9, 3.6097332291143923`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6277123886394815`*^9}]
}, Closed]],

Cell["are unit vectors; thus we have", "Text",
 CellChangeTimes->{{3.587280322107978*^9, 3.5872803471424093`*^9}, {
  3.5872804063267946`*^9, 3.58728041062304*^9}, {3.6276410735899143`*^9, 
  3.6276410811599283`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"amf", "[", "1", "]"}], "[", 
      RowBox[{"ruled", "[", 
       RowBox[{"x", ",", "y"}], "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "\[Equal]", " ", 
    RowBox[{"vec", "[", 
     RowBox[{"y", "[", "v", "]"}], "]"}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"vec", "[", 
      RowBox[{"y", "[", "v", "]"}], "]"}], ".", 
     RowBox[{"vec", "[", 
      RowBox[{"y", "[", "v", "]"}], "]"}]}], "\[Equal]", " ", "1"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5872806978644695`*^9, 3.587280778003053*^9}, 
   3.58728083266718*^9, 3.587289862213539*^9, 3.614861851375925*^9, 
   3.614917783808547*^9, 3.614919529482813*^9, {3.6151810048464317`*^9, 
   3.6151810151570215`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.627641098883559*^9, 3.627712405893112*^9}]
}, Closed]],

Cell["We calculate", "Text",
 CellChangeTimes->{{3.587280659127254*^9, 3.5872806747591476`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"amf", "[", "2", "]"}], "[", 
     RowBox[{"ruled", "[", 
      RowBox[{"x", ",", "y"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"vec", "[", 
      RowBox[{"y", "[", "v", "]"}], "]"}], ".", 
     RowBox[{"vec", "[", 
      RowBox[{"y", "[", "v", "]"}], "]"}]}], "\[Equal]", " ", "1"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5872809006570687`*^9, 3.587280909512575*^9}, {
   3.587290118485197*^9, 3.5872901269076786`*^9}, 3.6148618516099253`*^9, 
   3.6149177838397474`*^9, 3.615181162949475*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "+", 
         RowBox[{"u", " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}]}], 
        ")"}]}], "-", 
      RowBox[{
       RowBox[{
        RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
       RowBox[{
        RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "+", 
         RowBox[{"u", " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}]}], 
        ")"}]}], "+", 
      RowBox[{
       RowBox[{
        RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
            "+", 
            RowBox[{"u", " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "v", "]"}], "[", "1", 
              "]"}]}]}], ")"}]}], "-", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
            "+", 
            RowBox[{"u", " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "v", "]"}], "[", "3", 
              "]"}]}]}], ")"}]}]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"\[Sqrt]", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "1", 
                "]"}]}]}], ")"}]}], "-", 
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "2", 
                "]"}]}]}], ")"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "1", 
                "]"}]}]}], ")"}]}], "-", 
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "3", 
                "]"}]}]}], ")"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "2", 
                "]"}]}]}], ")"}]}], "-", 
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "3", 
                "]"}]}]}], ")"}]}]}], ")"}], "2"]}], ")"}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "-", 
      RowBox[{
       RowBox[{
        RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
       RowBox[{
        RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "+", 
         RowBox[{"u", " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}]}], 
        ")"}]}], "+", 
      RowBox[{"u", " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["y", "\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], "-", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "+", 
         RowBox[{"u", " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}]}], 
        ")"}]}], "-", 
      RowBox[{
       RowBox[{
        RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
       RowBox[{
        RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], "+", 
         RowBox[{"u", " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
        ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"\[Sqrt]", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "1", 
                "]"}]}]}], ")"}]}], "-", 
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "2", 
                "]"}]}]}], ")"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "1", 
                "]"}]}]}], ")"}]}], "-", 
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "3", 
                "]"}]}]}], ")"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "2", 
                "]"}]}]}], ")"}]}], "-", 
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "3", 
                "]"}]}]}], ")"}]}]}], ")"}], "2"]}], ")"}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}]}], " ", 
       RowBox[{
        RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "+", 
         RowBox[{"u", " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}]}], 
        ")"}]}], "-", 
      RowBox[{
       RowBox[{
        RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
       RowBox[{
        RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "+", 
         RowBox[{"u", " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}]}], 
        ")"}]}], "-", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox[
          RowBox[{
           RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], "+", 
         RowBox[{"u", " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
        ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"\[Sqrt]", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "1", 
                "]"}]}]}], ")"}]}], "-", 
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "2", 
                "]"}]}]}], ")"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "1", 
                "]"}]}]}], ")"}]}], "-", 
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "3", 
                "]"}]}]}], ")"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "2", 
                "]"}]}]}], ")"}]}], "-", 
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "3", 
                "]"}]}]}], ")"}]}]}], ")"}], "2"]}], ")"}]}], ")"}]}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.6276411253818073`*^9, 3.6277124352835636`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"amf", "[", "2", "]"}], "[", 
   RowBox[{"ruled", "[", 
    RowBox[{"x", ",", "y"}], "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.615181166818696*^9, 3.615181166849698*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"amf", "[", "3", "]"}], "[", 
     RowBox[{"ruled", "[", 
      RowBox[{"x", ",", "y"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"vec", "[", 
      RowBox[{"y", "[", "v", "]"}], "]"}], ".", 
     RowBox[{"vec", "[", 
      RowBox[{"y", "[", "v", "]"}], "]"}]}], "\[Equal]", " ", "1"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5872809006570687`*^9, 3.587280909512575*^9}, {
   3.587290118485197*^9, 3.5872901269076786`*^9}, {3.5872906447672987`*^9, 
   3.5872906711228065`*^9}, 3.6148618516411257`*^9, 3.614917783855347*^9, 
   3.6151811681617727`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}]}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "+", 
         RowBox[{"u", " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}]}], 
        ")"}]}], "+", 
      RowBox[{
       RowBox[{
        RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], "+", 
         RowBox[{"u", " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
        ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"\[Sqrt]", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "1", 
                "]"}]}]}], ")"}]}], "-", 
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "2", 
                "]"}]}]}], ")"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "1", 
                "]"}]}]}], ")"}]}], "-", 
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "3", 
                "]"}]}]}], ")"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "2", 
                "]"}]}]}], ")"}]}], "-", 
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "3", 
                "]"}]}]}], ")"}]}]}], ")"}], "2"]}], ")"}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "+", 
         RowBox[{"u", " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}]}], 
        ")"}]}], "-", 
      RowBox[{
       RowBox[{
        RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], "+", 
         RowBox[{"u", " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
        ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"\[Sqrt]", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "1", 
                "]"}]}]}], ")"}]}], "-", 
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "2", 
                "]"}]}]}], ")"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "1", 
                "]"}]}]}], ")"}]}], "-", 
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "3", 
                "]"}]}]}], ")"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "2", 
                "]"}]}]}], ")"}]}], "-", 
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "3", 
                "]"}]}]}], ")"}]}]}], ")"}], "2"]}], ")"}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}]}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "+", 
         RowBox[{"u", " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}]}], 
        ")"}]}], "+", 
      RowBox[{
       RowBox[{
        RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "+", 
         RowBox[{"u", " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}]}], 
        ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"\[Sqrt]", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "1", 
                "]"}]}]}], ")"}]}], "-", 
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "2", 
                "]"}]}]}], ")"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "1", 
                "]"}]}]}], ")"}]}], "-", 
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "3", 
                "]"}]}]}], ")"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "2", 
                "]"}]}]}], ")"}]}], "-", 
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "3", 
                "]"}]}]}], ")"}]}]}], ")"}], "2"]}], ")"}]}], ")"}]}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.627652538543024*^9, 3.627712448527987*^9}]
}, Closed]],

Cell["\<\
As curve vec[x[v]] we may take any curve intersecting all lines of the \
family. Thus we may assume vec[x[v]] being an orthogonal trajectory of the \
rulings amf[1][ruledsf][u,v] == vec[y[v]] on the ruled surface. Then \
D[vec[x[v]],v].vec[y[v]]\[Equal] 0,  the surface ruled[x,y][u,v] is a family \
of normals of the directrix vec[x[v]]. The normal unit vectors vec[y[v]] in \
general  form a curve on the unit sphere; in the neighborhood of  regular \
points of this curve we may assume that v is the arc length of this spherical \
curve. We collect these assumptions to \
\>", "Text",
 CellChangeTimes->{{3.609733354343555*^9, 3.6097334420075693`*^9}, {
   3.609733472912337*^9, 3.609733575912228*^9}, {3.6097336162885375`*^9, 
   3.6097336413759727`*^9}, {3.609733687194593*^9, 3.609733713808115*^9}, {
   3.609733771040389*^9, 3.6097338766884317`*^9}, {3.609733958888133*^9, 
   3.6097340466561527`*^9}, {3.6097340950399203`*^9, 3.609734099769191*^9}, {
   3.6097341449277735`*^9, 3.6097341633838296`*^9}, {3.609734695385258*^9, 
   3.609734749161334*^9}, 3.614919788302868*^9, 3.615263967187282*^9, {
   3.617889351635394*^9, 3.617889393225067*^9}, {3.6277130956483235`*^9, 
   3.627713100983533*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"ruledass", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"vec", "[", 
        RowBox[{"y", "[", "v", "]"}], "]"}], ".", 
       RowBox[{"vec", "[", 
        RowBox[{"y", "[", "v", "]"}], "]"}]}], "\[Equal]", " ", "1"}], "&&", 
     RowBox[{
      RowBox[{
       RowBox[{"D", "[", 
        RowBox[{
         RowBox[{"vec", "[", 
          RowBox[{"y", "[", "v", "]"}], "]"}], ",", "v"}], "]"}], ".", 
       RowBox[{"vec", "[", 
        RowBox[{"y", "[", "v", "]"}], "]"}]}], "\[Equal]", "0"}], "&&", 
     RowBox[{
      RowBox[{
       RowBox[{"D", "[", 
        RowBox[{
         RowBox[{"vec", "[", 
          RowBox[{"y", "[", "v", "]"}], "]"}], ",", "v"}], "]"}], ".", 
       RowBox[{"D", "[", 
        RowBox[{
         RowBox[{"vec", "[", 
          RowBox[{"y", "[", "v", "]"}], "]"}], ",", "v"}], "]"}]}], 
      "\[Equal]", " ", "1"}], "&&", 
     RowBox[{
      RowBox[{
       RowBox[{"D", "[", 
        RowBox[{
         RowBox[{"vec", "[", 
          RowBox[{"x", "[", "v", "]"}], "]"}], ",", "v"}], "]"}], ".", 
       RowBox[{"vec", "[", 
        RowBox[{"y", "[", "v", "]"}], "]"}]}], "\[Equal]", " ", "0"}]}], 
    "}"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.609731174164606*^9, 3.609731191246636*^9}, {
   3.6097312423211255`*^9, 3.6097313830177727`*^9}, {3.6097314152786293`*^9, 
   3.6097315223728175`*^9}, {3.6097319516387715`*^9, 3.609731998204853*^9}, 
   3.6097348047275124`*^9}],

Cell["\<\
Applying these assumptions leads to a certain simplification of the \
expression for the moving orthonormal frame:\
\>", "Text",
 CellChangeTimes->{{3.6097341810798416`*^9, 3.6097342414882965`*^9}, 
   3.6097348204804134`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"amf", "[", "1", "]"}], "[", 
     RowBox[{"ruled", "[", 
      RowBox[{"x", ",", "y"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", "ruledass"}], "]"}]], "Input",
 CellChangeTimes->{{3.5872806978644695`*^9, 3.587280778003053*^9}, 
   3.58728083266718*^9, 3.587289862213539*^9, {3.609731672757082*^9, 
   3.6097316754558864`*^9}, 3.614861851656726*^9, 3.614917783870947*^9, 
   3.6151811698088675`*^9, {3.617889476638414*^9, 3.617889479259218*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], ",", 
   RowBox[{
    RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], ",", 
   RowBox[{
    RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627713224036549*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"amf", "[", "2", "]"}], "[", 
     RowBox[{"ruled", "[", 
      RowBox[{"x", ",", "y"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", "ruledass"}], "]"}]], "Input",
 CellChangeTimes->{{3.5872809006570687`*^9, 3.587280909512575*^9}, {
   3.587290118485197*^9, 3.5872901269076786`*^9}, {3.60973172877678*^9, 
   3.6097317311635847`*^9}, 3.614861851656726*^9, 3.614917783886547*^9, 
   3.615181170673917*^9, {3.617889487449233*^9, 3.6178894894616365`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "+", 
      RowBox[{"u", " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["y", "\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}]}], 
     ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"\[Sqrt]", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["u", "2"], "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], "2"], 
        "+", 
        RowBox[{"2", " ", "u", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], 
           "+", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], 
           "+", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}],
           ")"}]}]}], ")"}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "+", 
      RowBox[{"u", " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["y", "\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}]}], 
     ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"\[Sqrt]", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["u", "2"], "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], "2"], 
        "+", 
        RowBox[{"2", " ", "u", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], 
           "+", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], 
           "+", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}],
           ")"}]}]}], ")"}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], "+", 
      RowBox[{"u", " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["y", "\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
     ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"\[Sqrt]", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["u", "2"], "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], "2"], 
        "+", 
        RowBox[{"2", " ", "u", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], 
           "+", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], 
           "+", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}],
           ")"}]}]}], ")"}]}], ")"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6277132394961767`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"amf", "[", "3", "]"}], "[", 
     RowBox[{"ruled", "[", 
      RowBox[{"x", ",", "y"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", "ruledass"}], "]"}]], "Input",
 CellChangeTimes->{{3.5872809006570687`*^9, 3.587280909512575*^9}, {
   3.587290118485197*^9, 3.5872901269076786`*^9}, {3.5872906447672987`*^9, 
   3.5872906711228065`*^9}, 3.6097317401336*^9, 3.614861851656726*^9, 
   3.614917783886547*^9, 3.615181171545966*^9, {3.6178895031428604`*^9, 
   3.6178895055608644`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}]}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "+", 
         RowBox[{"u", " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}]}], 
        ")"}]}], "+", 
      RowBox[{
       RowBox[{
        RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], "+", 
         RowBox[{"u", " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
        ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"\[Sqrt]", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["u", "2"], "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], "2"], 
        "+", 
        RowBox[{"2", " ", "u", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], 
           "+", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], 
           "+", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}],
           ")"}]}]}], ")"}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "+", 
         RowBox[{"u", " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}]}], 
        ")"}]}], "-", 
      RowBox[{
       RowBox[{
        RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], "+", 
         RowBox[{"u", " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
        ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"\[Sqrt]", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["u", "2"], "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], "2"], 
        "+", 
        RowBox[{"2", " ", "u", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], 
           "+", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], 
           "+", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}],
           ")"}]}]}], ")"}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}]}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "+", 
         RowBox[{"u", " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}]}], 
        ")"}]}], "+", 
      RowBox[{
       RowBox[{
        RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "+", 
         RowBox[{"u", " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}]}], 
        ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"\[Sqrt]", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["u", "2"], "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], "2"], 
        "+", 
        RowBox[{"2", " ", "u", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], 
           "+", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], 
           "+", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}],
           ")"}]}]}], ")"}]}], ")"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6277132822246513`*^9}]
}, Closed]],

Cell["\<\
For a general ruled surface the tangent plane of the surface considered along \
the generating lines  will not be constant:\
\>", "Text",
 CellChangeTimes->{{3.6097343540487347`*^9, 3.6097344491681757`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"D", "[", 
    RowBox[{"%", ",", "u"}], "]"}], ",", "ruledass"}], "]"}]], "Input",
 CellChangeTimes->{{3.5881411069079304`*^9, 3.5881411142243433`*^9}, {
   3.588141144332396*^9, 3.5881411750956497`*^9}, 3.6097317567632294`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"u", "+", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
        ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
            "+", 
            RowBox[{"u", " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "v", "]"}], "[", "2", 
              "]"}]}]}], ")"}]}], "-", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
            "+", 
            RowBox[{"u", " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "v", "]"}], "[", "3", 
              "]"}]}]}], ")"}]}]}], ")"}]}], ")"}], "/", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["u", "2"], "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], "2"], 
        "+", 
        RowBox[{"2", " ", "u", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], 
           "+", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], 
           "+", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}],
           ")"}]}]}], ")"}], 
      RowBox[{"3", "/", "2"}]]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{
          RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}]}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["y", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["y", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
      ")"}], "/", 
     RowBox[{"(", 
      RowBox[{"\[Sqrt]", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["u", "2"], "+", 
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"],
          "+", 
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"],
          "+", 
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], "2"],
          "+", 
         RowBox[{"2", " ", "u", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "v", "]"}], "[", "3", 
              "]"}]}]}], ")"}]}]}], ")"}]}], ")"}]}]}], ",", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"u", "+", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], 
           "+", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], 
           "+", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}],
           ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "1", 
                "]"}]}]}], ")"}]}], "-", 
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
              "+", 
              RowBox[{"u", " ", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "v", "]"}], "[", "3", 
                "]"}]}]}], ")"}]}]}], ")"}]}], ")"}], "/", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["u", "2"], "+", 
          SuperscriptBox[
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]",
              MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
           "2"], "+", 
          SuperscriptBox[
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]",
              MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
           "2"], "+", 
          SuperscriptBox[
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]",
              MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
           "2"], "+", 
          RowBox[{"2", " ", "u", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
              " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["y", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}],
              "+", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
              " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["y", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}],
              "+", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
              " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["y", "\[Prime]",
                 MultilineFunction->None], "[", "v", "]"}], "[", "3", 
               "]"}]}]}], ")"}]}]}], ")"}], 
        RowBox[{"3", "/", "2"}]]}], ")"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["y", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], "-", 
       RowBox[{
        RowBox[{
         RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["y", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
      ")"}], "/", 
     RowBox[{"(", 
      RowBox[{"\[Sqrt]", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["u", "2"], "+", 
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"],
          "+", 
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"],
          "+", 
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], "2"],
          "+", 
         RowBox[{"2", " ", "u", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "v", "]"}], "[", "3", 
              "]"}]}]}], ")"}]}]}], ")"}]}], ")"}]}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], "+", 
        RowBox[{"u", " ", 
         RowBox[{
          RowBox[{
           SuperscriptBox["y", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
       ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], 
           "-", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}]}],
           ")"}]}], "+", 
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], 
           "+", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}],
           ")"}]}], "+", 
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], 
           "-", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}],
           ")"}]}]}], ")"}]}], ")"}], "/", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["u", "2"], "+", 
       SuperscriptBox[
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"], 
       "+", 
       SuperscriptBox[
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"], 
       "+", 
       SuperscriptBox[
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], "2"], 
       "+", 
       RowBox[{"2", " ", "u", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]",
              MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]",
              MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], 
          "+", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]",
              MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]",
              MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], 
          "+", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]",
              MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]",
              MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
         ")"}]}]}], ")"}], 
     RowBox[{"3", "/", "2"}]]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6277133119739037`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", "/.", 
   RowBox[{"u", "\[Rule]", " ", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6097328746299925`*^9, 3.6097328943484273`*^9}, {
  3.6097330057014227`*^9, 3.6097330357626753`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"],
          "+", 
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"],
          "+", 
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
          "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}]}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
        ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], 
         "-", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
        ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
        ")"}]}]}], ")"}], "/", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"], 
       "+", 
       SuperscriptBox[
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"], 
       "+", 
       SuperscriptBox[
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], "2"]}],
       ")"}], 
     RowBox[{"3", "/", "2"}]]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"],
          "+", 
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"],
          "+", 
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
          "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], 
         "-", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
        ")"}]}], "-", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], 
         "-", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
        ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
        ")"}]}]}], ")"}], "/", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"], 
       "+", 
       SuperscriptBox[
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"], 
       "+", 
       SuperscriptBox[
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], "2"]}],
       ")"}], 
     RowBox[{"3", "/", "2"}]]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], 
           "-", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}]}],
           ")"}]}], "+", 
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], 
           "+", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}],
           ")"}]}], "+", 
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], 
           "-", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
            " ", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}],
           ")"}]}]}], ")"}]}], ")"}], "/", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"], 
       "+", 
       SuperscriptBox[
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"], 
       "+", 
       SuperscriptBox[
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], "2"]}],
       ")"}], 
     RowBox[{"3", "/", "2"}]]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6277133336423416`*^9}]
}, Closed]],

Cell[TextData[{
 "A ruled surface for which the tangent planes along the generating lines are \
constant is the envelope of a 1-parametric family of planes. Such surfaces \
are said to be ",
 StyleBox["developable",
  FontColor->RGBColor[0, 0, 1]],
 ". The next subsections contain examples of developable surfaces."
}], "Text",
 CellChangeTimes->{{3.609734481608031*^9, 3.609734649871655*^9}, {
  3.609734874064478*^9, 3.60973488441607*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["2.1.3. Circular Cones", "Subsubsection",
 CellChangeTimes->{{3.587278731126979*^9, 3.5872787512791314`*^9}, {
   3.587278983598419*^9, 3.5872789947820587`*^9}, {3.5875394069211664`*^9, 
   3.5875394299312067`*^9}, {3.5892067941410646`*^9, 3.589206796884221*^9}, 
   3.5896130524556894`*^9, 3.6096757219372315`*^9, 3.609739435859398*^9, 
   3.6178897837717533`*^9, 3.617890621555625*^9, 3.6277135758373213`*^9, 
   3.6277157139759*^9}],

Cell[TextData[{
 StyleBox["Example. ",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox["Circular Cones",
  FontColor->GrayLevel[0]],
 StyleBox[".",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{
  3.617941903491912*^9, {3.6179445243277154`*^9, 3.6179445260437183`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "circularcone"}]], "Input",
 CellChangeTimes->{{3.6151821603215213`*^9, 3.6151821615225897`*^9}}],

Cell[BoxData[
 StyleBox["\<\"{u,v}->circularcone[a,b][u,v] is a parametrization of a \
circular cone of radius a and slope b/a.  To plot try \
circularcone[1,0.5][-Pi,Pi,-1,1].\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.6277145489778414`*^9},
 CellTags->"Info3627718148-3784632"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Context", "[", "circularcone", "]"}]], "Input",
 CellChangeTimes->{{3.627714597378127*^9, 3.627714607533745*^9}}],

Cell[BoxData["\<\"SURFS`\"\>"], "Output",
 CellChangeTimes->{3.6277146087037473`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"circularcone", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "[", 
  RowBox[{"v", ",", "u"}], "]"}]], "Input",
 CellChangeTimes->{{3.6097399536040115`*^9, 3.6097399569472027`*^9}, 
   3.6148618516879253`*^9, 3.614864731452984*^9, 3.6151811727360344`*^9, 
   3.6151821410514193`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", " ", "u", " ", 
    RowBox[{"Cos", "[", "v", "]"}]}], ",", 
   RowBox[{"a", " ", "u", " ", 
    RowBox[{"Sin", "[", "v", "]"}]}], ",", 
   RowBox[{"b", " ", "u"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627714552786248*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"circularcone", "[", 
   RowBox[{"1", ",", ".5"}], "]"}], "[", 
  RowBox[{
   RowBox[{"-", "Pi"}], ",", "Pi", ",", 
   RowBox[{"-", "1"}], ",", "1"}], "]"}]], "Input",
 CellChangeTimes->{{3.6097399536040115`*^9, 3.6097399569472027`*^9}, 
   3.6148618516879253`*^9, 3.614864731452984*^9, 3.6151811727360344`*^9, 
   3.6151821410514193`*^9, {3.61518224160217*^9, 3.6151823043477592`*^9}}],

Cell[TextData[{
 "Here is the parameter representation of a",
 StyleBox[" circular cone with the opening  angle parameter a",
  FontColor->RGBColor[0, 0, 1]],
 ";"
}], "Text",
 CellChangeTimes->{{3.5875390653897667`*^9, 3.5875391467751093`*^9}, {
  3.6097367551930723`*^9, 3.609736762248476*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"u", ",", "v", ",", "a"}], "]"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"cone", "[", "a_", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", " ", 
      RowBox[{"Cos", "[", "v", "]"}], " ", 
      RowBox[{"Sin", "[", 
       FractionBox["a", "2"], "]"}]}], ",", 
     RowBox[{"u", " ", 
      RowBox[{"Sin", "[", 
       FractionBox["a", "2"], "]"}], " ", 
      RowBox[{"Sin", "[", "v", "]"}]}], ",", 
     RowBox[{"u", " ", 
      RowBox[{"Cos", "[", 
       FractionBox["a", "2"], "]"}]}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cone", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"u", " ", 
     RowBox[{"Cos", "[", "v", "]"}]}], ",", 
    RowBox[{"u", " ", 
     RowBox[{"Sin", "[", "v", "]"}]}], ",", "u"}], "}"}]}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5872948975265427`*^9, 3.5872949467553587`*^9}, {
   3.587295055234563*^9, 3.5872951071875343`*^9}, {3.58729514766685*^9, 
   3.5872951496909657`*^9}, {3.587295297660429*^9, 3.5872953178275824`*^9}, {
   3.5872953816602335`*^9, 3.587295400275298*^9}, 3.58729564402924*^9, {
   3.587296119420431*^9, 3.58729612220759*^9}, 3.587296180140904*^9, {
   3.5872968322122*^9, 3.58729691385187*^9}, {3.5872969927723837`*^9, 
   3.587297041619178*^9}, 3.5872995662082434`*^9, 3.5873007900452433`*^9, {
   3.587300836185882*^9, 3.587300857290089*^9}, {3.5873009825162516`*^9, 
   3.587301025218694*^9}, 3.5873015813395023`*^9, {3.5873016792821045`*^9, 
   3.5873016953790255`*^9}, 3.5873017301390133`*^9, {3.5873018736832237`*^9, 
   3.587301885521901*^9}, 3.5873023656363616`*^9, {3.587362824728996*^9, 
   3.587362858495927*^9}, {3.5873655403923225`*^9, 3.587365560400467*^9}, {
   3.587365642144143*^9, 3.5873656656024847`*^9}, 3.5873659192889943`*^9, {
   3.6097353257093105`*^9, 3.6097353561790533`*^9}, 3.60973566592377*^9, {
   3.6097357072661343`*^9, 3.6097358183264866`*^9}, {3.6097358569586964`*^9, 
   3.609735895822919*^9}, {3.6148618517035255`*^9, 3.6148618517035255`*^9}, {
   3.614864731468584*^9, 3.614864731484184*^9}}],

Cell["expressed with circularcone :", "Text",
 CellChangeTimes->{{3.609738511000499*^9, 3.609738550375751*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"cone", "[", "a", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{"circularcone", "[", 
    RowBox[{
     RowBox[{"Sin", "[", 
      RowBox[{"a", "/", "2"}], "]"}], ",", " ", 
     RowBox[{"Cos", "[", 
      RowBox[{"a", "/", "2"}], "]"}]}], "]"}], "[", 
   RowBox[{"v", ",", "u"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.609736354019127*^9, 3.6097364236911116`*^9}, {
  3.6148618517035255`*^9, 3.6148618517035255`*^9}, {3.614864731499784*^9, 
  3.614864731499784*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6277146678856516`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"cone", "[", 
   RowBox[{"u", ",", "v"}], "]"}], "==", 
  RowBox[{
   RowBox[{"circularcone", "[", 
    RowBox[{"1", ",", " ", "1"}], "]"}], "[", 
   RowBox[{"v", ",", "u"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5872932058577847`*^9, 3.587293215873358*^9}, 
   3.5872948874259653`*^9, {3.6097364393220053`*^9, 3.6097364881637993`*^9}, {
   3.6097367155078025`*^9, 3.6097367297066145`*^9}, {3.6148618517191257`*^9, 
   3.6148618517191257`*^9}, {3.6148647315153837`*^9, 3.6148647315153837`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.627714674422063*^9}]
}, Closed]],

Cell["\<\
Here a denotes the angle between two opposite generators of the circular cone\
\>", "Text",
 CellChangeTimes->{{3.5873628884356394`*^9, 3.5873628961790824`*^9}, {
  3.5873631146025753`*^9, 3.587363137978913*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Sin", "[", 
       RowBox[{"a", "/", "2"}], "]"}], "*", 
      RowBox[{"Cos", "[", "v", "]"}]}], ",", 
     RowBox[{
      RowBox[{"Sin", "[", 
       RowBox[{"a", "/", "2"}], "]"}], "*", 
      RowBox[{"Sin", "[", "v", "]"}]}], ",", 
     RowBox[{"Cos", "[", 
      RowBox[{"a", "/", "2"}], "]"}]}], "}"}], ".", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Sin", "[", 
       RowBox[{"a", "/", "2"}], "]"}], "*", 
      RowBox[{"Cos", "[", 
       RowBox[{"v", "+", "Pi"}], "]"}]}], ",", 
     RowBox[{
      RowBox[{"Sin", "[", 
       RowBox[{"a", "/", "2"}], "]"}], "*", 
      RowBox[{"Sin", "[", 
       RowBox[{"v", "+", "Pi"}], "]"}]}], ",", 
     RowBox[{"Cos", "[", 
      RowBox[{"a", "/", "2"}], "]"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.587363187949771*^9, 3.5873632063528233`*^9}, {
  3.587363405322204*^9, 3.587363414033702*^9}, {3.587364073017394*^9, 
  3.5873640870571966`*^9}, {3.587364145360532*^9, 3.5873641558491316`*^9}, {
  3.5873650971939735`*^9, 3.587365103712346*^9}, {3.5873651748564157`*^9, 
  3.587365181856816*^9}, {3.5873655843958397`*^9, 3.5873656085542216`*^9}, {
  3.587365705545769*^9, 3.5873657138572445`*^9}}],

Cell[BoxData[
 RowBox[{"Cos", "[", "a", "]"}]], "Output",
 CellChangeTimes->{3.627714714446134*^9}]
}, Closed]],

Cell["In the next cell try some values of a:", "Text",
 CellChangeTimes->{{3.5873652832916174`*^9, 3.587365315627467*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"cone", "[", 
     RowBox[{"8", "*", 
      RowBox[{"Pi", "/", "10"}]}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.587292881024205*^9, 3.587293043506499*^9}, {
   3.5872931006277657`*^9, 3.5872931337866626`*^9}, {3.587293221739693*^9, 
   3.5872932468991323`*^9}, {3.5872932802280383`*^9, 3.587293303739383*^9}, {
   3.5872933712442446`*^9, 3.587293384002974*^9}, 3.587295167828003*^9, {
   3.587295346019195*^9, 3.5872953481313157`*^9}, {3.587295417195266*^9, 
   3.5872954187073526`*^9}, 3.5872955288836546`*^9, {3.587295665403463*^9, 
   3.5872956745789876`*^9}, {3.5872957076028767`*^9, 3.587295718787516*^9}, 
   3.587295950548772*^9, {3.587296019203699*^9, 3.587296034386567*^9}, 
   3.587296273172225*^9, {3.5873020711465178`*^9, 3.5873020813291*^9}, 
   3.5873021189782534`*^9, {3.5873021523621635`*^9, 3.587302157585462*^9}, {
   3.587302190002316*^9, 3.5873021999618855`*^9}, 3.587302265473633*^9, {
   3.5873023851534777`*^9, 3.5873024626499104`*^9}, {3.587365325152012*^9, 
   3.5873653364146557`*^9}, {3.5873659389441185`*^9, 
   3.5873659632795105`*^9}, {3.5873775764752693`*^9, 3.587377577659337*^9}, 
   3.6148618517191257`*^9, 3.614864731530984*^9, {3.617890074603064*^9, 
   3.6178900946490993`*^9}, {3.62771476283982*^9, 3.6277147774922457`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"cone", "[", 
    RowBox[{"u", ",", "v"}], "]"}], "-", 
   RowBox[{
    RowBox[{"cone", "[", 
     RowBox[{"Pi", "/", "2"}], "]"}], "[", 
    RowBox[{
     RowBox[{"u", "*", 
      RowBox[{"Sqrt", "[", "2", "]"}]}], ",", "v"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.587292881024205*^9, 3.587293043506499*^9}, {
   3.5872931006277657`*^9, 3.5872931337866626`*^9}, {3.587293221739693*^9, 
   3.5872932468991323`*^9}, {3.5872932802280383`*^9, 3.587293303739383*^9}, {
   3.5872933712442446`*^9, 3.587293384002974*^9}, 3.587295167828003*^9, {
   3.587295346019195*^9, 3.5872953481313157`*^9}, {3.587295417195266*^9, 
   3.5872954187073526`*^9}, 3.5872955288836546`*^9, {3.587295665403463*^9, 
   3.5872956745789876`*^9}, {3.5872957076028767`*^9, 3.587295718787516*^9}, 
   3.587295950548772*^9, {3.587296019203699*^9, 3.587296034386567*^9}, 
   3.587296273172225*^9, {3.5873020711465178`*^9, 3.5873020813291*^9}, 
   3.5873021189782534`*^9, {3.5873021523621635`*^9, 3.587302157585462*^9}, {
   3.587302190002316*^9, 3.5873021999618855`*^9}, 3.587302265473633*^9, {
   3.5873023851534777`*^9, 3.587302489929471*^9}, {3.5873659905340695`*^9, 
   3.5873660240139847`*^9}, 3.5873660610071*^9, {3.5873663265912905`*^9, 
   3.5873663338547063`*^9}, {3.614861851734726*^9, 3.614861851734726*^9}, {
   3.614864731546584*^9, 3.614864731546584*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.6277147958554783`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"amf", "[", "3", "]"}], "[", 
     RowBox[{"cone", "[", "a", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"u", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.587367193391869*^9, 3.5873672317900653`*^9}, {
   3.5873672813919024`*^9, 3.587367295423705*^9}, {3.614861851734726*^9, 
   3.614861851734726*^9}, 3.6148647315621843`*^9, 3.614917783933347*^9, 
   3.615181206984994*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{"Cos", "[", "v", "]"}], " ", 
      RowBox[{"Sin", "[", "a", "]"}]}], 
     RowBox[{"2", " ", 
      SqrtBox[
       SuperscriptBox[
        RowBox[{"Sin", "[", 
         FractionBox["a", "2"], "]"}], "2"]]}]]}], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{"Sin", "[", "a", "]"}], " ", 
      RowBox[{"Sin", "[", "v", "]"}]}], 
     RowBox[{"2", " ", 
      SqrtBox[
       SuperscriptBox[
        RowBox[{"Sin", "[", 
         FractionBox["a", "2"], "]"}], "2"]]}]]}], ",", 
   SqrtBox[
    SuperscriptBox[
     RowBox[{"Sin", "[", 
      FractionBox["a", "2"], "]"}], "2"]]}], "}"}]], "Output",
 CellChangeTimes->{3.627714814378711*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", ".", "%"}], "]"}]], "Input",
 CellChangeTimes->{{3.61518282362146*^9, 3.615182837275241*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.627714829261137*^9}]
}, Closed]],

Cell["\<\
The spherical image is a circle on the unit sphere. The vector \
amf[3][cone[a]][u,v] does not depend on u, it is constant along the \
generators.The tangent planes form a 1-parameter family;  tangent to  the \
cone along the generating lines. The cone is the envelope of this family of \
planes.\
\>", "Text",
 CellChangeTimes->{{3.5873769759596725`*^9, 3.5873769972388897`*^9}, {
   3.5873770459446754`*^9, 3.587377192903081*^9}, {3.5873775421113043`*^9, 
   3.5873775659356666`*^9}, {3.5873780405038104`*^9, 3.587378092671794*^9}, 
   3.589204165124693*^9, {3.614861851765926*^9, 3.614861851812726*^9}, 
   3.614864731593384*^9, 3.614917783964547*^9, 3.61518121426541*^9}],

Cell["\<\
Here is the first fundamental form of the circular cone with parameter a:\
\>", "Text",
 CellChangeTimes->{{3.5875392110940228`*^9, 3.5875392468024855`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    FormBox["fund1",
     TraditionalForm], "[", 
    RowBox[{"cone", "[", "a", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"s1", ",", "s2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s1", ",", "s2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.587302621247982*^9, 3.5873026846016054`*^9}, 
   3.5873027534025407`*^9, {3.5873033248071203`*^9, 3.5873033282391267`*^9}, 
   3.590495710531009*^9, 3.614861851843926*^9, 3.614864731624584*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["s1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"Cos", "[", 
       FractionBox["a", "2"], "]"}], "2"], "+", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"Cos", "[", "v", "]"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"Sin", "[", 
        FractionBox["a", "2"], "]"}], "2"]}], "+", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"Sin", "[", 
        FractionBox["a", "2"], "]"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"Sin", "[", "v", "]"}], "2"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["s2", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["u", "2"], " ", 
      SuperscriptBox[
       RowBox[{"Cos", "[", "v", "]"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"Sin", "[", 
        FractionBox["a", "2"], "]"}], "2"]}], "+", 
     RowBox[{
      SuperscriptBox["u", "2"], " ", 
      SuperscriptBox[
       RowBox[{"Sin", "[", 
        FractionBox["a", "2"], "]"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"Sin", "[", "v", "]"}], "2"]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.627714859546791*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.5873027745887527`*^9, 3.5873027831682434`*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["s1", "2"], "+", 
  FractionBox[
   RowBox[{
    SuperscriptBox["s2", "2"], " ", 
    SuperscriptBox["u", "2"]}], "2"], "-", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox["s2", "2"], " ", 
   SuperscriptBox["u", "2"], " ", 
   RowBox[{"Cos", "[", "a", "]"}]}]}]], "Output",
 CellChangeTimes->{3.6277148794740257`*^9}]
}, Closed]],

Cell["Since", "Text",
 CellChangeTimes->{{3.5896106720295086`*^9, 3.589610673963912*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", 
      RowBox[{"Cos", "[", "a", "]"}]}], " ", ")"}], "/", "2"}], "\[Equal]", 
   " ", 
   RowBox[{
    RowBox[{"Sin", "[", 
     RowBox[{"a", "/", "2"}], "]"}], "^", "2"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.58961068427553*^9, 3.589610786065709*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.627714901926466*^9}]
}, Closed]],

Cell["\<\
the first fundamental form of the cone with parameter a can be written in \
traditional form as\
\>", "Text",
 CellChangeTimes->{{3.5896108644714465`*^9, 3.589610910959528*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["ds", "2"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["du", "2"], " ", "+", " ", 
    RowBox[{
     SuperscriptBox["u", "2"], " ", 
     RowBox[{
      SuperscriptBox["sin", "2"], "(", 
      FractionBox["a", "2"], ")"}], " ", 
     SuperscriptBox["dv", "2"]}]}]}], TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5896109202103443`*^9, 3.5896110848218336`*^9}}],

Cell[TextData[{
 "The ",
 StyleBox["plane in polar coordinates",
  FontColor->RGBColor[0, 0, 1]],
 " r, \[Phi] has the parameter representation"
}], "Text",
 CellChangeTimes->{{3.589611215674863*^9, 3.5896112858281865`*^9}},
 CellTags->"polco"],

Cell[BoxData[
 RowBox[{
  RowBox[{"polcoord", "[", 
   RowBox[{"r_", ",", "\[Phi]_"}], "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"r", "*", 
     RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", " ", 
    RowBox[{"r", "*", 
     RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], " "}]], "Input",
 CellChangeTimes->{{3.589611309758629*^9, 3.589611399879987*^9}, 
   3.5896227230329013`*^9}],

Cell["\<\
The first fundamental form of the plane in these coordinates is\
\>", "Text",
 CellChangeTimes->{{3.5896114233736277`*^9, 3.5896114457284675`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     FormBox["fund1",
      TraditionalForm], "[", "polcoord", "]"}], "[", 
    RowBox[{"r", ",", "\[Phi]\[Phi]"}], "]"}], "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"s1", ",", "s2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"s1", ",", "s2"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5896115173949933`*^9, 3.5896115745690937`*^9}, 
   3.5896116780440755`*^9, 3.5904957105466094`*^9}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["s1", "2"], "+", 
  RowBox[{
   SuperscriptBox["r", "2"], " ", 
   SuperscriptBox["s2", "2"]}]}]], "Output",
 CellChangeTimes->{3.6277149274833107`*^9}]
}, Closed]],

Cell["In traditional form:", "Text",
 CellChangeTimes->{{3.5896116022123423`*^9, 3.5896116119155593`*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["ds", "2"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["dr", "2"], "+", " ", 
    RowBox[{
     SuperscriptBox["r", "2"], 
     SuperscriptBox["d\[Phi]", "2"]}]}]}], TraditionalForm]], "Input",
 CellChangeTimes->{{3.589611617359969*^9, 3.5896117282137637`*^9}}],

Cell["Thus the map ", "Text",
 CellChangeTimes->{{3.5896117816906576`*^9, 3.5896117861366653`*^9}, 
   3.5896118362283535`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"f", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{"u", ",", " ", 
    RowBox[{
     RowBox[{"Sin", "[", 
      RowBox[{"a", "/", "2"}], "]"}], "*", "v"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.5896118084291043`*^9, 3.589611809895507*^9}, {
  3.589611866227206*^9, 3.589611949827753*^9}}],

Cell["\<\
 is an isometry of the cone with parameter a onto a certain region of the \
plane. We consider the part of  cone[1]  with u< R=1 and obtain the picture\
\>", "Text",
 CellChangeTimes->{{3.58961199938904*^9, 3.589612149695304*^9}, {
   3.58961222168943*^9, 3.5896122512046824`*^9}, {3.5896123690160894`*^9, 
   3.5896123697492905`*^9}, {3.5896124644102564`*^9, 3.589612466797061*^9}, 
   3.614861851875126*^9, 3.6148647316557837`*^9}],

Cell[BoxData[
 RowBox[{"ParametricPlot", "[", 
  RowBox[{
   RowBox[{"polcoord", "[", 
    RowBox[{"u", ",", " ", 
     RowBox[{
      RowBox[{"Sin", "[", ".5", "]"}], "*", "v"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.589612155046113*^9, 3.589612207477805*^9}, {
   3.589612261875101*^9, 3.5896123416224413`*^9}, 3.589612379109307*^9, 
   3.5896124778106804`*^9, 3.589622759817766*^9}],

Cell["\<\
The map can be obtained by cutting the cone along the line v== 0 and laying \
down all its other generating lines into the x,y-plane; this often is said as \
\[OpenCurlyDoubleQuote]developing it into the plane\[CloseCurlyDoubleQuote]. \
Of course, the isometry is valid only locally: The circle u == 1 must be cut \
and is only then developable into a circle arc whose length depends on a. \
\>", "Text",
 CellChangeTimes->{{3.5896126509449844`*^9, 3.5896127547447667`*^9}, {
  3.589612787770025*^9, 3.589612852681739*^9}, {3.5896129059714327`*^9, 
  3.589612984767171*^9}, {3.5896131463990545`*^9, 3.5896132386264167`*^9}, {
  3.615264023461501*^9, 3.615264024773576*^9}}],

Cell["\<\
The built-in function Cone yields a cone as a  Graphics3D object:\
\>", "Text",
 CellChangeTimes->{{3.587555743989889*^9, 3.5875557629299726`*^9}, {
  3.587555807577526*^9, 3.5875558655058393`*^9}, {3.587556515250003*^9, 
  3.587556566753948*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "Cone"}]], "Input",
 CellChangeTimes->{{3.5875558693020563`*^9, 3.5875558720292125`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox["\<\"\\!\\(\\*RowBox[{\\\"Cone\\\", \\\"[\\\", \
RowBox[{RowBox[{\\\"{\\\", RowBox[{RowBox[{\\\"{\\\", \
RowBox[{SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], StyleBox[\\\"1\\\", \
\\\"TR\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"y\\\", \\\"TI\\\"], \
StyleBox[\\\"1\\\", \\\"TR\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"z\\\", \
\\\"TI\\\"], StyleBox[\\\"1\\\", \\\"TR\\\"]]}], \\\"}\\\"}], \\\",\\\", \
RowBox[{\\\"{\\\", RowBox[{SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], \
StyleBox[\\\"2\\\", \\\"TR\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"y\\\", \
\\\"TI\\\"], StyleBox[\\\"2\\\", \\\"TR\\\"]], \\\",\\\", \
SubscriptBox[StyleBox[\\\"z\\\", \\\"TI\\\"], StyleBox[\\\"2\\\", \
\\\"TR\\\"]]}], \\\"}\\\"}]}], \\\"}\\\"}], \\\",\\\", StyleBox[\\\"r\\\", \\\
\"TI\\\"]}], \\\"]\\\"}]\\) represents a cone with a base of radius \
\\!\\(\\*StyleBox[\\\"r\\\", \\\"TI\\\"]\\) centered at \
\\!\\(\\*RowBox[{\\\"(\\\", RowBox[{SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\
\"], StyleBox[\\\"1\\\", \\\"TR\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"y\
\\\", \\\"TI\\\"], StyleBox[\\\"1\\\", \\\"TR\\\"]], \\\",\\\", \
SubscriptBox[StyleBox[\\\"z\\\", \\\"TI\\\"], StyleBox[\\\"1\\\", \
\\\"TR\\\"]]}], \\\")\\\"}]\\) and a tip at \\!\\(\\*RowBox[{\\\"(\\\", \
RowBox[{SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], StyleBox[\\\"2\\\", \
\\\"TR\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"y\\\", \\\"TI\\\"], \
StyleBox[\\\"2\\\", \\\"TR\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"z\\\", \
\\\"TI\\\"], StyleBox[\\\"2\\\", \\\"TR\\\"]]}], \\\")\\\"}]\\). \
\\n\\!\\(\\*RowBox[{\\\"Cone\\\", \\\"[\\\", RowBox[{\\\"{\\\", \
RowBox[{RowBox[{\\\"{\\\", RowBox[{SubscriptBox[StyleBox[\\\"x\\\", \
\\\"TI\\\"], StyleBox[\\\"1\\\", \\\"TR\\\"]], \\\",\\\", \
SubscriptBox[StyleBox[\\\"y\\\", \\\"TI\\\"], StyleBox[\\\"1\\\", \
\\\"TR\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"z\\\", \\\"TI\\\"], \
StyleBox[\\\"1\\\", \\\"TR\\\"]]}], \\\"}\\\"}], \\\",\\\", \
RowBox[{\\\"{\\\", RowBox[{SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], \
StyleBox[\\\"2\\\", \\\"TR\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"y\\\", \
\\\"TI\\\"], StyleBox[\\\"2\\\", \\\"TR\\\"]], \\\",\\\", \
SubscriptBox[StyleBox[\\\"z\\\", \\\"TI\\\"], StyleBox[\\\"2\\\", \
\\\"TR\\\"]]}], \\\"}\\\"}]}], \\\"}\\\"}], \\\"]\\\"}]\\) represents a cone \
with a base of radius 1.\"\>", "MSG"], "\[NonBreakingSpace]", 
  ButtonBox[
   StyleBox["\[RightSkeleton]", "SR"],
   Active->True,
   BaseStyle->"Link",
   ButtonData->"paclet:ref/Cone"]}]], "Print", "PrintUsage",
 CellChangeTimes->{3.6277150267124863`*^9},
 CellTags->"Info3627718626-1674846"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cone", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "0"}], "}"}]}], "}"}], ",", 
   RowBox[{"Tan", "[", 
    FractionBox["1", "2"], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{3.5875574251940484`*^9, 3.6148618519063263`*^9, 
  3.614917783995747*^9}],

Cell[BoxData[
 RowBox[{"Cone", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "0"}], "}"}]}], "}"}], ",", 
   RowBox[{"Tan", "[", 
    FractionBox["1", "2"], "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.6277152201732273`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"co1", " ", "=", 
  RowBox[{"Show", "[", 
   RowBox[{"Graphics3D", "[", "%", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5875564103430023`*^9, 3.587556438523614*^9}, {
  3.587556962923608*^9, 3.5875569664438095`*^9}}],

Cell["We plot the surface  opposite to co1  by cone", "Text",
 CellChangeTimes->{{3.587558692569538*^9, 3.587558711641629*^9}, {
  3.587558788978052*^9, 3.5875588102972717`*^9}, {3.6277152642513056`*^9, 
  3.627715276548127*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"cone", "[", "a", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5875559643094907`*^9, 3.587555973026989*^9}, 
   3.614861851921926*^9, 3.614864731686984*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"u", " ", 
    RowBox[{"Cos", "[", "v", "]"}], " ", 
    RowBox[{"Sin", "[", 
     FractionBox["a", "2"], "]"}]}], ",", 
   RowBox[{"u", " ", 
    RowBox[{"Sin", "[", 
     FractionBox["a", "2"], "]"}], " ", 
    RowBox[{"Sin", "[", "v", "]"}]}], ",", 
   RowBox[{"u", " ", 
    RowBox[{"Cos", "[", 
     FractionBox["a", "2"], "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6277152833653393`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", 
  RowBox[{"1", "/", 
   RowBox[{"Cos", "[", 
    RowBox[{"1", "/", "2"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6151842927934914`*^9, 3.615184323403242*^9}}],

Cell[BoxData["1.139493927324549`"], "Output",
 CellChangeTimes->{3.627715287766547*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"co2", " ", "=", " ", 
  RowBox[{"ParametricPlot3D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"cone", "[", "1", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"u", ",", "0", ",", 
      RowBox[{"-", "1.139"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"v", ",", "0", ",", 
      RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
    RowBox[{"AspectRatio", "\[Rule]", "1"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5875566103584423`*^9, 3.5875566887569265`*^9}, {
   3.5875575681262236`*^9, 3.587557569437299*^9}, {3.587557626693574*^9, 
   3.5875576269725895`*^9}, {3.5875577747100396`*^9, 3.587557780154351*^9}, {
   3.5875578458291073`*^9, 3.5875578481092377`*^9}, {3.5875584166217546`*^9, 
   3.5875584181818438`*^9}, 3.587558498653447*^9, 3.587558539606789*^9, {
   3.5875585741337643`*^9, 3.58755858018111*^9}, 3.614861851921926*^9, 
   3.6148647317025843`*^9, {3.6151836825005846`*^9, 3.615183683916666*^9}, {
   3.6151837288772373`*^9, 3.6151837305003304`*^9}, {3.615184362476477*^9, 
   3.615184364692604*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"{", 
   RowBox[{"co1", ",", "co2"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.5875575114239807`*^9, 3.5875575243397193`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["2.1.4. Conical Surfaces", "Subsubsection",
 CellChangeTimes->{{3.587201518412738*^9, 3.587201527076234*^9}, {
   3.587272835342759*^9, 3.58727283885396*^9}, {3.587272981302108*^9, 
   3.5872730078536263`*^9}, 3.5872731046621633`*^9, 3.587278763622837*^9, {
   3.5875395682098494`*^9, 3.58753958547908*^9}, 3.589613294661715*^9, 
   3.5896228998436117`*^9, 3.6097394807369647`*^9, 3.6097411436570787`*^9, 
   3.6178906549864836`*^9, 3.617944252481638*^9, 3.627715806580064*^9, 
   3.6277165361065516`*^9}],

Cell[TextData[{
 "The next three examples treat important special cases of developable \
surfaces.\n\n",
 StyleBox["Example 1",
  FontColor->RGBColor[0, 0, 1]],
 ". Conical surface\n A ruled surface is called ",
 StyleBox["conical ",
  FontColor->RGBColor[0, 0, 1]],
 "if all its rulings  have a common point, say the origin. Thus the surface \
is defined by  vec[y[v]] being a curve on the unit sphere:"
}], "Text",
 CellChangeTimes->{{3.587291182439052*^9, 3.587291253231101*^9}, {
   3.5872918463990283`*^9, 3.587291860454832*^9}, {3.6097350506905804`*^9, 
   3.609735056359905*^9}, 3.6179419782004433`*^9, 3.617942024688525*^9, {
   3.6179420573861823`*^9, 3.6179420643125944`*^9}, {3.6179434846950893`*^9, 
   3.617943490045899*^9}, {3.6179445410041447`*^9, 3.6179445500833607`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"conesf", "[", "y_", "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"vec", "[", 
    RowBox[{"y", "[", "v", "]"}], "]"}], "*", "u"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.587292095259262*^9, 3.587292166330327*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "conesf"}]], "Input",
 CellChangeTimes->{{3.6151879081844697`*^9, 3.6151881733726377`*^9}, {
   3.6179435529452095`*^9, 3.617943629603744*^9}, {3.617943683735839*^9, 
   3.617943707759881*^9}, 3.617943754403963*^9, 3.617943886052594*^9, 
   3.617948041571166*^9, {3.622616200775807*^9, 3.6226162254706497`*^9}}],

Cell[BoxData[
 StyleBox["\<\"conesf[y][u,v] is the general conical surface with vertex at \
the origin. \\nIt is defined by the  curve y[v] on the unit sphere.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.6277162318892145`*^9},
 CellTags->"Info3627719831-1674846"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Context", "[", "conesf", "]"}]], "Input",
 CellChangeTimes->{{3.622616261803114*^9, 3.6226162857959557`*^9}}],

Cell[BoxData["\<\"eudiffgeov2`\"\>"], "Output",
 CellChangeTimes->{3.627716236257222*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"conesf", "[", "y", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.587292408188161*^9, 3.587292436585785*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"u", " ", 
    RowBox[{
     RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}]}], ",", 
   RowBox[{"u", " ", 
    RowBox[{
     RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}]}], ",", 
   RowBox[{"u", " ", 
    RowBox[{
     RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6277162418928328`*^9}]
}, Closed]],

Cell["\<\
As before we assume that the curve vec[y[v]] lies on the unit sphere and  \
that v is its arc length; then ruledass is satisfied;\
\>", "Text",
 CellChangeTimes->{{3.587540430813365*^9, 3.5875404624814205`*^9}, {
   3.5875404962086797`*^9, 3.5875405929912496`*^9}, {3.609740743897213*^9, 
   3.6097407692406626`*^9}, 3.615184574313594*^9, {3.6151846406823893`*^9, 
   3.6151846992017365`*^9}}],

Cell[CellGroupData[{

Cell[BoxData["ruledass"], "Input",
 CellChangeTimes->{{3.6097406879370127`*^9, 3.6097406914172115`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{
       RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{
       RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{
       RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], "2"]}], "\[Equal]", 
    "1"}], "&&", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["y", "\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["y", "\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["y", "\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
    "\[Equal]", "0"}], "&&", 
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{
       RowBox[{
        SuperscriptBox["y", "\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"], 
     "+", 
     SuperscriptBox[
      RowBox[{
       RowBox[{
        SuperscriptBox["y", "\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"], 
     "+", 
     SuperscriptBox[
      RowBox[{
       RowBox[{
        SuperscriptBox["y", "\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], "2"]}], 
    "\[Equal]", "1"}], "&&", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
    "\[Equal]", "0"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6277162855799093`*^9}]
}, Closed]],

Cell[" The simplification under these condition yields:", "Text",
 CellChangeTimes->{{3.609740786200633*^9, 3.6097407934400473`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"u", ">", "0"}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "3", "]"}], "[", 
       RowBox[{"conesf", "[", "y", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", "ruledass"}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.587539794488247*^9, 3.5875399826401777`*^9}, {
   3.5875400133566313`*^9, 3.5875400957715764`*^9}, 3.58754036527765*^9, {
   3.587548649578514*^9, 3.587548649812515*^9}, {3.588249351340147*^9, 
   3.588249351340147*^9}, {3.6097400255921288`*^9, 3.609740054356774*^9}, {
   3.609740095708139*^9, 3.60974011669934*^9}, {3.6097401494602137`*^9, 
   3.609740164915097*^9}, {3.6097402113237514`*^9, 3.609740268059997*^9}, 
   3.614861851953126*^9, 3.6149177840113473`*^9, 3.6151812556737785`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}]}], " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["y", "\[Prime]",
        MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}], " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["y", "\[Prime]",
        MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}], " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["y", "\[Prime]",
        MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], "-", 
    RowBox[{
     RowBox[{
      RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["y", "\[Prime]",
        MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}]}], " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["y", "\[Prime]",
        MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}], " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["y", "\[Prime]",
        MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}]}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.6277162992143335`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"u", ">", "0"}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{"%", ".", "%"}], ",", "ruledass"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.615184854845639*^9, 3.61518486657131*^9}, {
  3.615184929956935*^9, 3.6151849709232783`*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.627716304365343*^9}]
}, Closed]],

Cell["\<\
The normal vector of the surface is constant along the generating lines, the \
surface is developable, once more: \
\>", "Text",
 CellChangeTimes->{{3.5875406082012763`*^9, 3.587540632225319*^9}, {
   3.587548159254053*^9, 3.587548167506468*^9}, 3.587548689857785*^9, {
   3.6097404018726506`*^9, 3.6097404565847797`*^9}, {3.6097411060729284`*^9, 
   3.6097411181606197`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"D", "[", 
    RowBox[{"%%", ",", "u"}], "]"}], ",", "ruledass"}], "]"}]], "Input",
 CellChangeTimes->{{3.5875401840365314`*^9, 3.5875402950931263`*^9}, {
   3.5875485494419384`*^9, 3.587548549675939*^9}, {3.588249351355747*^9, 
   3.5882493513713474`*^9}, {3.609740484835396*^9, 3.6097404878005657`*^9}, 
   3.6151850264194527`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.6277163572262363`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"amf", "[", "1", "]"}], "[", 
      RowBox[{"conesf", "[", "y", "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "-", " ", 
    RowBox[{"vec", "[", 
     RowBox[{"y", "[", "v", "]"}], "]"}]}], ",", "ruledass"}], "]"}]], "Input",
 CellChangeTimes->{{3.5875483343955607`*^9, 3.5875483387791686`*^9}, 
   3.5875483973260717`*^9, {3.5875484361857395`*^9, 3.5875485398323216`*^9}, 
   3.5875487665475197`*^9, {3.5882493513713474`*^9, 3.588249351386947*^9}, {
   3.6097405338642*^9, 3.609740552621273*^9}, 3.614861851953126*^9, 
   3.6149177840269475`*^9, 3.615181257120861*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.627716389976694*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"u", ">", "0"}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "2", "]"}], "[", 
       RowBox[{"conesf", "[", "y", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", "ruledass"}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5875483343955607`*^9, 3.5875483387791686`*^9}, 
   3.5875483973260717`*^9, {3.5875484361857395`*^9, 3.5875485398323216`*^9}, {
   3.5875487665475197`*^9, 3.58754880069598*^9}, {3.587548900848156*^9, 
   3.5875489599566593`*^9}, {3.5875489994247293`*^9, 3.587549030718384*^9}, 
   3.5875490897488875`*^9, {3.6097405719843807`*^9, 3.6097405966487913`*^9}, 
   3.6097406267725143`*^9, 3.614861851968726*^9, 3.614917784042547*^9, 
   3.6151812583689327`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     SuperscriptBox["y", "\[Prime]",
      MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], ",", 
   RowBox[{
    RowBox[{
     SuperscriptBox["y", "\[Prime]",
      MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], ",", 
   RowBox[{
    RowBox[{
     SuperscriptBox["y", "\[Prime]",
      MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.627716404174719*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", " ", "\[Equal]", "  ", 
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{"vec", "[", 
     RowBox[{"y", "[", "v", "]"}], "]"}], ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.587549103554912*^9, 3.5875491348797665`*^9}, {
  3.587549217154311*^9, 3.5875492390099497`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6277164112415314`*^9}]
}, Closed]],

Cell["Thus, under the above assumptions, we have: The vectors ", "Text",
 CellChangeTimes->{{3.5875493479449415`*^9, 3.58754937602499*^9}, {
  3.5875496573714848`*^9, 3.5875496662635*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"amf", "[", "1", "]"}], "[", 
     RowBox[{"conesf", "[", "y", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "\[Equal]", " ", 
   RowBox[{"vec", "[", 
    RowBox[{"y", "[", "v", "]"}], "]"}]}], ";", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"amf", "[", "2", "]"}], "[", 
     RowBox[{"conesf", "[", "y", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "\[Equal]", " ", 
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{"vec", "[", 
      RowBox[{"y", "[", "v", "]"}], "]"}], ",", "v"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"amf", "[", "3", "]"}], "[", 
     RowBox[{"conesf", "[", "y", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "\[Equal]", 
   RowBox[{"Cross", "[", 
    RowBox[{
     RowBox[{"vec", "[", 
      RowBox[{"y", "[", "v", "]"}], "]"}], ",", " ", 
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{"vec", "[", 
        RowBox[{"y", "[", "v", "]"}], "]"}], ",", "v"}], "]"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.587549409190649*^9, 3.587549645827464*^9}, {
   3.587549711363179*^9, 3.587549807303348*^9}, 3.587549850172223*^9, {
   3.6148618519843264`*^9, 3.6148618519999266`*^9}, {3.614917784042547*^9, 
   3.6149177840581474`*^9}, {3.6151812596670065`*^9, 3.615181270946652*^9}}],

Cell["\<\
form the orthonormal moving 3-frame of the surface; it does not depend on u.\
\>", "Text",
 CellChangeTimes->{{3.5875498669734526`*^9, 3.5875499120107317`*^9}, {
  3.587549954926407*^9, 3.587549970245634*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"u", ">", "0"}], ",", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"amf", "[", "3", "]"}], "[", 
        RowBox[{"conesf", "[", "y", "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "\[Equal]", 
      RowBox[{"Cross", "[", 
       RowBox[{
        RowBox[{"vec", "[", 
         RowBox[{"y", "[", "v", "]"}], "]"}], ",", " ", 
        RowBox[{"D", "[", 
         RowBox[{
          RowBox[{"vec", "[", 
           RowBox[{"y", "[", "v", "]"}], "]"}], ",", "v"}], "]"}]}], "]"}]}], 
     ",", "ruledass"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.587549976423245*^9, 3.5875500258129315`*^9}, {
   3.6097408644981112`*^9, 3.609740903912366*^9}, 3.6148618520155263`*^9, 
   3.6149177840737476`*^9, 3.6151812746748652`*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.627716455459009*^9}]
}, Closed]],

Cell["Here is the first fundamental form of the  conical surface:", "Text",
 CellChangeTimes->{{3.5875392110940228`*^9, 3.5875392468024855`*^9}, {
   3.5875503815403566`*^9, 3.5875504304308424`*^9}, 3.6179442107983646`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"u", ">", "0"}], ",", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        FormBox["fund1",
         TraditionalForm], "[", 
        RowBox[{"conesf", "[", "y", "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"s1", ",", "s2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"s1", ",", "s2"}], "}"}]}], "]"}], ",", "ruledass"}], "]"}]}],
   "]"}]], "Input",
 CellChangeTimes->{{3.587302621247982*^9, 3.5873026846016054`*^9}, 
   3.5873027534025407`*^9, {3.5873033248071203`*^9, 3.5873033282391267`*^9}, {
   3.5875504420840626`*^9, 3.5875505150141907`*^9}, 3.590495710562209*^9, {
   3.609740930352878*^9, 3.609740964005803*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["s1", "2"], "+", 
  RowBox[{
   SuperscriptBox["s2", "2"], " ", 
   SuperscriptBox["u", "2"]}]}]], "Output",
 CellChangeTimes->{3.627716469001833*^9}]
}, Closed]],

Cell["Thus, in traditional notation, the first fundamental form is", "Text",
 CellChangeTimes->{{3.588144859952071*^9, 3.5881448887037153`*^9}, {
  3.5896222176076136`*^9, 3.5896222211800203`*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["ds", "2"], " ", "==", 
  RowBox[{
   SuperscriptBox["du", 
    RowBox[{"2", "  "}]], "+", " ", 
   RowBox[{
    SuperscriptBox["u", "2"], 
    SuperscriptBox["dv", "2"]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.588144893206973*^9, 3.588144994745781*^9}}],

Cell[TextData[{
 "and with the map defined by the equality of the parameters   r = u, \[Phi]  \
= v we obtain a local  isometry of the conical surface with the Euclidean \
plane in polar coordinates see ",
 ButtonBox["above",
  BaseStyle->"Hyperlink",
  ButtonData->"polco"],
 " ."
}], "Text",
 CellChangeTimes->{{3.58814502199934*^9, 3.5881450378552465`*^9}, {
  3.58814511029739*^9, 3.5881451394640584`*^9}, {3.588145246302169*^9, 
  3.5881452463101697`*^9}, {3.588145293663878*^9, 3.588145374903525*^9}, {
  3.5881609609546366`*^9, 3.588160965305886*^9}, {3.5896223283834085`*^9, 
  3.5896223388510265`*^9}, {3.589622379957099*^9, 3.589622382078703*^9}, {
  3.589622940793684*^9, 3.5896229828357573`*^9}, {3.5896230368274527`*^9, 
  3.5896230368274527`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["2.1.5. Cylindrical Surfaces", "Subsubsection",
 CellChangeTimes->{{3.587201518412738*^9, 3.587201527076234*^9}, {
   3.587272835342759*^9, 3.58727283885396*^9}, {3.587272981302108*^9, 
   3.5872730078536263`*^9}, 3.5872731046621633`*^9, 3.587278763622837*^9, {
   3.5875395682098494`*^9, 3.58753958547908*^9}, {3.5875506376928062`*^9, 
   3.58755065084363*^9}, 3.6097411500414433`*^9, 3.609744994379327*^9, 
   3.617944348905407*^9, 3.6179453910184374`*^9, {3.627716730155094*^9, 
   3.6277167346655025`*^9}, 3.6277171001395473`*^9}],

Cell[TextData[{
 StyleBox["Example 2",
  FontColor->RGBColor[0, 0, 1]],
 ". Cylindrical surface"
}], "Text",
 CellChangeTimes->{{3.617944654400744*^9, 3.6179446785963864`*^9}}],

Cell[TextData[{
 "A ruled surface is said to be ",
 StyleBox["cylindrical",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox[" or a",
  FontColor->GrayLevel[0]],
 StyleBox[" cylinder,",
  FontColor->RGBColor[0, 0, 1]],
 " if all its generating lines are parallel. In this case its parameter \
representation has the form vec[x[v]] + u vec[a], vec[a] = constant, not the \
null vector. We assume vec[a] = stb[3], that the directrix intersects the \
generating lines orthogonally and lies in the plane z==0. Thus we suppose the \
parameter representation to be"
}], "Text",
 CellChangeTimes->{{3.587553102264791*^9, 3.587553151345598*^9}, {
   3.587553246298029*^9, 3.5875534525058236`*^9}, {3.587553671769365*^9, 
   3.587553876378068*^9}, {3.587554060673609*^9, 3.5875540834409113`*^9}, {
   3.588564643440874*^9, 3.5885646536744924`*^9}, {3.5885647572430744`*^9, 
   3.588564762640684*^9}, {3.5885648533236427`*^9, 3.588564937423391*^9}, 
   3.5885649858302755`*^9, {3.5885650220535393`*^9, 3.5885650255167456`*^9}, {
   3.588565093548465*^9, 3.588565129428528*^9}, 3.5885651668217936`*^9, {
   3.588565473877133*^9, 3.588565491099563*^9}, 3.6097437815449567`*^9, 
   3.615185309945669*^9, {3.615264079771722*^9, 3.6152640993158393`*^9}, {
   3.617944759092528*^9, 3.617944795518592*^9}, 3.617944826391046*^9, {
   3.617944945918456*^9, 3.6179449649660892`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"cylindersf", "[", 
    RowBox[{"x_", ",", "y_"}], "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"x", "[", "v", "]"}], ",", 
    RowBox[{"y", "[", "v", "]"}], ",", "u"}], "}"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5875539174934196`*^9, 3.5875540237964993`*^9}, {
   3.5875544463496685`*^9, 3.587554446916701*^9}, 3.5875549322674613`*^9, {
   3.5875550888534174`*^9, 3.587555104389306*^9}, {3.6097443219728675`*^9, 
   3.6097443494924417`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "cylindersf"}]], "Input",
 CellChangeTimes->{{3.615187599736828*^9, 3.6151877880615993`*^9}, 
   3.6179448499938874`*^9, {3.622616460594263*^9, 3.622616472216284*^9}}],

Cell[BoxData[
 StyleBox["\<\"cylindersf[x,y][u,v] is the cylinder with rulings parallel to \
\\nthe z-axis erected over the curve {x[v],y[v],0} in the x,y-plane.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.6277168255340624`*^9},
 CellTags->"Info3627720425-9230554"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Context", "[", "cylindersf", "]"}]], "Input",
 CellChangeTimes->{{3.6226164762098904`*^9, 3.622616493728721*^9}}],

Cell[BoxData["\<\"eudiffgeov2`\"\>"], "Output",
 CellChangeTimes->{3.627716828453268*^9}]
}, Closed]],

Cell["A special case is", "Text",
 CellChangeTimes->{{3.6097443626391935`*^9, 3.609744379281145*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "circularcylinder"}]], "Input",
 CellChangeTimes->{{3.6097439348607264`*^9, 3.609743944041251*^9}}],

Cell[BoxData[
 StyleBox["\<\"{u,v}->circularcylinder[a][u,v] is a parametrization of a \
circular cylinder of radius a.  To plot try circularcylinder[1][-Pi,Pi,-4,4].\
\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.627716846054099*^9},
 CellTags->"Info3627720445-9230554"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"circularcylinder", "[", "a", "]"}], "[", 
   RowBox[{"v", ",", "u"}], "]"}], " "}]], "Input",
 CellChangeTimes->{{3.609744115587063*^9, 3.6097441296198654`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", " ", 
    RowBox[{"Cos", "[", "v", "]"}]}], ",", 
   RowBox[{"a", " ", 
    RowBox[{"Sin", "[", "v", "]"}]}], ",", "u"}], "}"}]], "Output",
 CellChangeTimes->{3.6277168503909063`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"cylinder", "[", "a_", "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"circularcylinder", "[", "a", "]"}], "[", 
   RowBox[{"v", ",", "u"}], "]"}], " "}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5875539174934196`*^9, 3.5875540237964993`*^9}, {
   3.5875544463496685`*^9, 3.5875545218449864`*^9}, {3.587554553540799*^9, 
   3.587554574661007*^9}, 3.5875549421500263`*^9, {3.58755512963875*^9, 
   3.5875551798056197`*^9}, {3.587555221229989*^9, 3.5875552221160393`*^9}, 
   3.5875552711018414`*^9, 3.587555335188507*^9, {3.5875607561415677`*^9, 
   3.58756076387001*^9}, {3.5875609656385503`*^9, 3.587560970365821*^9}, {
   3.6097442549010315`*^9, 3.6097442800674706`*^9}}],

Cell[BoxData[
 RowBox[{"cy1", " ", "=", 
  RowBox[{"ParametricPlot3D", "[", 
   RowBox[{
    RowBox[{"Evaluate", "[", 
     RowBox[{
      RowBox[{"cylinder", "[", ".5", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"v", ",", "0", ",", 
      RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"u", ",", 
      RowBox[{"-", "1"}], ",", "2.3"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.587554597538316*^9, 3.587554678861967*^9}, {
   3.5875548571821666`*^9, 3.5875548631575084`*^9}, {3.587555015485221*^9, 
   3.587555031589142*^9}, {3.587560057349599*^9, 3.587560095149761*^9}, {
   3.609744228741535*^9, 3.6097442294445753`*^9}, 3.6148618520311265`*^9, 
   3.6149177840737476`*^9}],

Cell["In Mathematica Cylinder is a Graphics3D object :", "Text",
 CellChangeTimes->{{3.588565563780091*^9, 3.5885656192069883`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "Cylinder"}]], "Input",
 CellChangeTimes->{{3.587559881636549*^9, 3.5875598989385386`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox["\<\"\\!\\(\\*RowBox[{\\\"Cylinder\\\", \\\"[\\\", \
RowBox[{RowBox[{\\\"{\\\", RowBox[{RowBox[{\\\"{\\\", \
RowBox[{SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], StyleBox[\\\"1\\\", \
\\\"TR\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"y\\\", \\\"TI\\\"], \
StyleBox[\\\"1\\\", \\\"TR\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"z\\\", \
\\\"TI\\\"], StyleBox[\\\"1\\\", \\\"TR\\\"]]}], \\\"}\\\"}], \\\",\\\", \
RowBox[{\\\"{\\\", RowBox[{SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], \
StyleBox[\\\"2\\\", \\\"TR\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"y\\\", \
\\\"TI\\\"], StyleBox[\\\"2\\\", \\\"TR\\\"]], \\\",\\\", \
SubscriptBox[StyleBox[\\\"z\\\", \\\"TI\\\"], StyleBox[\\\"2\\\", \
\\\"TR\\\"]]}], \\\"}\\\"}]}], \\\"}\\\"}], \\\",\\\", StyleBox[\\\"r\\\", \\\
\"TI\\\"]}], \\\"]\\\"}]\\) represents a cylinder of radius \
\\!\\(\\*StyleBox[\\\"r\\\", \\\"TI\\\"]\\) around the line from \
\\!\\(\\*RowBox[{\\\"(\\\", RowBox[{SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\
\"], StyleBox[\\\"1\\\", \\\"TR\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"y\
\\\", \\\"TI\\\"], StyleBox[\\\"1\\\", \\\"TR\\\"]], \\\",\\\", \
SubscriptBox[StyleBox[\\\"z\\\", \\\"TI\\\"], StyleBox[\\\"1\\\", \
\\\"TR\\\"]]}], \\\")\\\"}]\\) to \\!\\(\\*RowBox[{\\\"(\\\", \
RowBox[{SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], StyleBox[\\\"2\\\", \
\\\"TR\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"y\\\", \\\"TI\\\"], \
StyleBox[\\\"2\\\", \\\"TR\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"z\\\", \
\\\"TI\\\"], StyleBox[\\\"2\\\", \\\"TR\\\"]]}], \\\")\\\"}]\\). \
\\n\\!\\(\\*RowBox[{\\\"Cylinder\\\", \\\"[\\\", RowBox[{\\\"{\\\", \
RowBox[{RowBox[{\\\"{\\\", RowBox[{SubscriptBox[StyleBox[\\\"x\\\", \
\\\"TI\\\"], StyleBox[\\\"1\\\", \\\"TR\\\"]], \\\",\\\", \
SubscriptBox[StyleBox[\\\"y\\\", \\\"TI\\\"], StyleBox[\\\"1\\\", \
\\\"TR\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"z\\\", \\\"TI\\\"], \
StyleBox[\\\"1\\\", \\\"TR\\\"]]}], \\\"}\\\"}], \\\",\\\", \
RowBox[{\\\"{\\\", RowBox[{SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], \
StyleBox[\\\"2\\\", \\\"TR\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"y\\\", \
\\\"TI\\\"], StyleBox[\\\"2\\\", \\\"TR\\\"]], \\\",\\\", \
SubscriptBox[StyleBox[\\\"z\\\", \\\"TI\\\"], StyleBox[\\\"2\\\", \
\\\"TR\\\"]]}], \\\"}\\\"}]}], \\\"}\\\"}], \\\"]\\\"}]\\) represents a \
cylinder of radius 1. \"\>", "MSG"], "\[NonBreakingSpace]", 
  ButtonBox[
   StyleBox["\[RightSkeleton]", "SR"],
   Active->True,
   BaseStyle->"Link",
   ButtonData->"paclet:ref/Cylinder"]}]], "Print", "PrintUsage",
 CellChangeTimes->{3.6277168798809586`*^9},
 CellTags->"Info3627720479-9230554"]
}, Closed]],

Cell[BoxData[
 RowBox[{"cy2", " ", "=", 
  RowBox[{"Show", "[", 
   RowBox[{"Graphics3D", "[", 
    RowBox[{"Cylinder", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "0", ",", 
          RowBox[{"-", "1.3"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0", ",", "2"}], "}"}]}], "}"}], ",", ".5"}], 
     "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.587559923827962*^9, 3.587560042114728*^9}, {
  3.58756010946058*^9, 3.5875601130607853`*^9}, {3.5875601522060246`*^9, 
  3.58756016296564*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"cy1", ",", "cy2"}], "}"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.587560120740225*^9, 3.587560134066987*^9}, {
  3.587560249348581*^9, 3.587560256972017*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"amf", "[", "3", "]"}], "[", 
     RowBox[{"cylinder", "[", "a", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"(", 
    RowBox[{"a", ">", "0"}], ")"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5875610023946524`*^9, 3.58756104606015*^9}, {
   3.58756133208951*^9, 3.587561367965562*^9}, {3.5875614118850737`*^9, 
   3.5875614266689196`*^9}, {3.5875615312299004`*^9, 
   3.5875615491479254`*^9}, {3.60974449055751*^9, 3.6097444971398864`*^9}, 
   3.6148618520311265`*^9, 3.614917784089348*^9, 3.615181284609433*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"Cos", "[", "v", "]"}]}], ",", 
   RowBox[{"-", 
    RowBox[{"Sin", "[", "v", "]"}]}], ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.6277169409016666`*^9}]
}, Closed]],

Cell["\<\
The normal vector doesn\[CloseCurlyQuote]t depend on u, the cylinder is \
developable. The spherical image is the unit circle.  For a cylindrical \
surface the spherical image is a part of the equator:\
\>", "Text",
 CellChangeTimes->{{3.5875615964976335`*^9, 3.5875616142576494`*^9}, {
   3.5875617607620287`*^9, 3.5875618261617694`*^9}, 3.5876201866455812`*^9, {
   3.6097446080412292`*^9, 3.60974465385685*^9}, 3.6152641126426015`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"amf", "[", "3", "]"}], "[", 
    RowBox[{"cylindersf", "[", 
     RowBox[{"x", ",", "y"}], "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.587561671403918*^9, 3.5875617060689*^9}, 
   3.614861852046726*^9, 3.6149177841049476`*^9, 3.615181286408536*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["y", "\[Prime]",
       MultilineFunction->None], "[", "v", "]"}], 
     SqrtBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["x", "\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["y", "\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "2"]}]]]}], ",", 
   FractionBox[
    RowBox[{
     SuperscriptBox["x", "\[Prime]",
      MultilineFunction->None], "[", "v", "]"}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["y", "\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "2"]}]]], ",", "0"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.6277170080083847`*^9}]
}, Closed]],

Cell["\<\
These surfaces are developable too. The first fundamental form is\
\>", "Text",
 CellChangeTimes->{{3.587620372457508*^9, 3.5876203935175447`*^9}, {
  3.609744685168641*^9, 3.609744702696644*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      FormBox["fund1",
       TraditionalForm], "[", 
      RowBox[{"cylindersf", "[", 
       RowBox[{"x", ",", "y"}], "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"s1", ",", "s2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"s1", ",", "s2"}], "}"}]}], "]"}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", "v", 
       "]"}], "^", "2"}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "v", 
       "]"}], "^", "2"}]}], "\[Equal]", "1"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.588145723022436*^9, 3.588145758972492*^9}, 
   3.5881457897422523`*^9, 3.5904957105778093`*^9, {3.6148618520623264`*^9, 
   3.6148618520779266`*^9}, {3.6149177841205473`*^9, 3.6149177841205473`*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["s1", "2"], "+", 
  SuperscriptBox["s2", "2"]}]], "Output",
 CellChangeTimes->{3.627717017243601*^9}]
}, Closed]],

Cell["\<\
Obviously, this metric equals the metric of the Euclidean plane in \
orthonormal Cartesian coordinates. Thus the surface is locally isometric to \
the Euclidean plane, the isometry is defined by equal  parameters.\
\>", "Text",
 CellChangeTimes->{{3.588145812464552*^9, 3.5881458169518085`*^9}, {
   3.588160786594664*^9, 3.588160926090643*^9}, 3.589623261935848*^9, {
   3.5896233630623236`*^9, 3.589623446423092*^9}, {3.609744915512816*^9, 
   3.6097449577452316`*^9}, 3.615185967266266*^9, 3.6152641629144773`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["2.1.6. Tangent Surfaces", "Subsubsection",
 CellChangeTimes->{{3.587201518412738*^9, 3.587201527076234*^9}, {
   3.587272835342759*^9, 3.58727283885396*^9}, {3.587272981302108*^9, 
   3.5872730078536263`*^9}, 3.5872731046621633`*^9, 3.587278763622837*^9, {
   3.5875395682098494`*^9, 3.58753958547908*^9}, {3.5875506376928062`*^9, 
   3.58755065084363*^9}, {3.5875621007464747`*^9, 3.5875621112900777`*^9}, 
   3.609745012249349*^9, 3.6098188216334867`*^9, 3.6179455096254463`*^9, 
   3.617947264841502*^9, 3.627717223226165*^9, 3.627745740101883*^9}],

Cell[TextData[{
 StyleBox["Example 3. ",
  FontColor->RGBColor[0, 0, 1]],
 "Tangent surfaces\nLet x{v] be an arbitrary smooth regular curve in the \
space. Then its ",
 StyleBox["tangent surface",
  FontColor->RGBColor[0, 0, 1]],
 " (also named ",
 StyleBox["tangent developable",
  FontColor->RGBColor[0, 0, 1]],
 ") is defined by"
}], "Text",
 CellChangeTimes->{{3.587620429179208*^9, 3.587620528239382*^9}, {
  3.58762060146591*^9, 3.5876206099835253`*^9}, {3.5882547189130597`*^9, 
  3.5882547566322174`*^9}, {3.6179455507315183`*^9, 3.6179456100116224`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tangsf", "[", "x_", "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"x", "[", "v", "]"}], " ", "+", " ", 
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"x", "[", "v", "]"}], ",", "v"}], "]"}], "*", 
    "u"}]}]}]], "Input",
 Editable->False,
 CellChangeTimes->{{3.5876205386446*^9, 3.587620546663014*^9}, {
   3.587620618813141*^9, 3.587620638874776*^9}, {3.587620784657032*^9, 
   3.5876208247491026`*^9}, 3.587731804014976*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "tangsf"}]], "Input",
 CellChangeTimes->{{3.6151860940185156`*^9, 3.61518611857092*^9}, {
   3.615187275916306*^9, 3.6151873529487123`*^9}, {3.615187448988205*^9, 
   3.615187498012009*^9}, 3.6151875548302593`*^9, 3.622722251673976*^9, 
   3.6227223103456783`*^9}],

Cell[BoxData[
 StyleBox["\<\"tangsf[x][u,v] is a parameter representation of the \
developable \\nsurface formed by the tangents of the space curve x[v].\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.6277412191671424`*^9},
 CellTags->"Info3627744819-7142630"]
}, Closed]],

Cell[BoxData[
 RowBox[{"Context", "[", "tangsf", "]"}]], "Input",
 CellChangeTimes->{{3.6227223600005655`*^9, 3.6227223788141985`*^9}}],

Cell["\<\
The curve itself lies on its tangent surface: set u=0. A general space curve \
is \
\>", "Text",
 CellChangeTimes->{{3.587621303264343*^9, 3.587621319441571*^9}, {
  3.587621391529298*^9, 3.5876214238837547`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"curve", "[", "v_", "]"}], ":=", 
  RowBox[{"vec", "[", 
   RowBox[{"x", "[", "v", "]"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5876209324048915`*^9, 3.5876209401893053`*^9}, {
   3.587620972294162*^9, 3.587620981076977*^9}, 3.6148618520779266`*^9, 
   3.6149177841361475`*^9}],

Cell[TextData[{
 StyleBox["All points of the space curve ",
  FontColor->RGBColor[0, 0, 1]],
 "vec[x[v]]",
 StyleBox[" are singularities of its tangent surface",
  FontColor->RGBColor[0, 0, 1]],
 ":"
}], "Text",
 CellChangeTimes->{{3.587621445770593*^9, 3.5876214903866715`*^9}, {
  3.6277412917072697`*^9, 3.6277412932672725`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Cross", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", "1", "]"}], "[", 
      RowBox[{"tangsf", "[", "curve", "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", "2", "]"}], "[", 
      RowBox[{"tangsf", "[", "curve", "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.587620867742778*^9, 3.587620899972434*^9}, {
   3.587620992870598*^9, 3.5876210110446296`*^9}, {3.5876210566279097`*^9, 
   3.5876210894971676`*^9}, 3.6176000248214893`*^9, 3.617600085801996*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"u", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}], " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], "+", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
     ")"}]}], ",", 
   RowBox[{"u", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], "-", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
     ")"}]}], ",", 
   RowBox[{"u", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], "+", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}]}], 
     ")"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6277413118157053`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"u", "\[Rule]", " ", "0"}]}]], "Input",
 CellChangeTimes->{{3.5876211520064774`*^9, 3.587621171943312*^9}, {
  3.5876212035333676`*^9, 3.5876212069809732`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.627741316901314*^9}]
}, Closed]],

Cell["We shall assume that v is the arc length of the space curve:", "Text",
 CellChangeTimes->{{3.5876215279827375`*^9, 3.587621564190401*^9}, 
   3.6152641666586914`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"arcl", " ", "=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"D", "[", 
        RowBox[{
         RowBox[{"curve", "[", "v", "]"}], ",", "v"}], "]"}], ".", " ", 
       RowBox[{"D", "[", 
        RowBox[{
         RowBox[{"curve", "[", "v", "]"}], ",", "v"}], "]"}]}], "==", "1"}], 
     ",", 
     RowBox[{
      RowBox[{
       RowBox[{"D", "[", 
        RowBox[{
         RowBox[{"curve", "[", "v", "]"}], ",", "v"}], "]"}], ".", " ", 
       RowBox[{"D", "[", 
        RowBox[{
         RowBox[{"curve", "[", "v", "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"v", ",", "2"}], "}"}]}], "]"}]}], "==", " ", "0"}]}], 
    "}"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.58762160342447*^9, 3.58762167150299*^9}, {
  3.5876218099844327`*^9, 3.5876218138688393`*^9}, {3.587621898998189*^9, 
  3.587621913833815*^9}, {3.5876219516638813`*^9, 3.5876219707739153`*^9}, {
  3.587622105526952*^9, 3.587622222090357*^9}, {3.587732305290656*^9, 
  3.5877323565367465`*^9}, {3.5877346417807617`*^9, 3.5877346617487965`*^9}, {
  3.5877347212941017`*^9, 3.5877347303421173`*^9}, {3.588162712104797*^9, 
  3.5881627655508537`*^9}, {3.588162871047888*^9, 3.588162886310761*^9}, {
  3.614861852109126*^9, 3.6148618521247263`*^9}, {3.6149177841361475`*^9, 
  3.614917784167348*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"], 
     "+", 
     SuperscriptBox[
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"], 
     "+", 
     SuperscriptBox[
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], "2"]}], 
    "\[Equal]", "1"}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
    "\[Equal]", "0"}]}], "}"}]], "Output",
 CellChangeTimes->CompressedData["
1:eJwV0GtIU3EABfA7nXMuW6ZpapBUZDa3WUZiL3RCammbk1whahEsqkUjW7op
PrqZsxlBFqJbZhiJmq8kTM2ZWJQhZo6ZOUUl0z7cnLlhOBDt/D8cft/OgbPr
kjpVyaIo6jwyuVhunHJnJKERJs0CtATz5J5sRmIbSZwh/ozuGOBDgVX5x4f4
IDR+K9RZZjqE8ED9vUdi2B9o8ImAf52xMcScqFe96bAim1bRsE9+PechrN9Z
frQVhhdMWMfh2ccZzbNwrqWPZnswku9vbroJoCKFrpFDe2v31TLYUPVO1ASz
VvfOj8F+ddAUi8NIAvXGaDcYnFE84QEpbxvfCwrTzNpNME8+WkvMP1zZR5So
hS3e0PDawdoM93X92E5cXGiMITpqxJeJqUu5d4h5FV5dxLv6D6NEf07+GlG6
33qQD+uWqiJCoW/b2CnicUcpJwxa27e4C+GL4SMGGeTQRbJCKDBNaoph1H1K
RyztpRqIxj3pPWVQbFukGqGfVqofgi0Dn+UuGCnifg3zZCSD/ic8BTBR1pZN
fNm6/DsJlpulQ2egXzU9KYM3ZrPmiIMrHuuZsNm8EpAFvyU4TxfB1Y3kX3Uw
incxrQmqnO3KEehQCofHYWGy75UVuH5tuMQFE0o+xa3BemsSzebi/247dzcc
k/2zCKFuepRJgikB0+0KKDLE3ToHHTtO5mqg1tX5rBb2zJviFV7oyQyJ1MBe
ccHIc1i5vC38I9Q/2Xj7BR572ml3wo7VCyoXsft27DrMOFS5HMTDjuW9LQT+
B5clQrg=
  "]]
}, Closed]],

Cell["\<\
Now we calculate the spherical image of the regular part of the tangent \
surface:\
\>", "Text",
 CellChangeTimes->{{3.5876231220247374`*^9, 3.5876231738636284`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"amf", "[", "3", "]"}], "[", 
     RowBox[{"tangsf", "[", "curve", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", "arcl"}], "]"}]], "Input",
 CellChangeTimes->{{3.5881630905624437`*^9, 3.5881631391492224`*^9}, 
   3.6148618521403265`*^9, 3.6149177841829476`*^9, 3.6151813113289614`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{"u", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
      ")"}]}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox["u", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
         "2"]}], ")"}]}]]], ",", 
   FractionBox[
    RowBox[{"u", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], "-", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
      ")"}]}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox["u", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
         "2"]}], ")"}]}]]], ",", 
   FractionBox[
    RowBox[{"u", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}]}], 
      ")"}]}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox["u", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
         "2"]}], ")"}]}]]]}], "}"}]], "Output",
 CellChangeTimes->{3.627741390205843*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "3", "]"}], "[", 
       RowBox[{"tangsf", "[", "curve", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", "u"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", ">", "0"}], ",", "arcl"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.587621684622613*^9, 3.587621793027203*^9}, 
   3.5876222348823795`*^9, {3.5876223777942305`*^9, 3.5876223968730636`*^9}, {
   3.5881629216867847`*^9, 3.5881629433340225`*^9}, {3.5881630472379656`*^9, 
   3.5881630616367893`*^9}, 3.6148618521559267`*^9, 3.6149177841985474`*^9, 
   3.615181313257072*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.627741404526668*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "3", "]"}], "[", 
       RowBox[{"tangsf", "[", "curve", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", "u"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "<", "0"}], ",", "arcl"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.587621684622613*^9, 3.587621793027203*^9}, 
   3.5876222348823795`*^9, {3.5876223777942305`*^9, 3.5876223968730636`*^9}, {
   3.5881629216867847`*^9, 3.5881629433340225`*^9}, {3.588162991834797*^9, 
   3.5881630291659317`*^9}, 3.609815167604488*^9, 3.614861852171527*^9, 
   3.6149177841985474`*^9, 3.61518131444914*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.627741413746284*^9}]
}, Closed]],

Cell["\<\
Obviously, this vector does not depend on u. It is proportional to the \
binormal vector of the curve. Indeed, denote  the orthonormal Frenet frame of \
the curve by frc[i][v]\
\>", "Text",
 CellChangeTimes->{{3.587732407077636*^9, 3.587732462603733*^9}, {
   3.5881617314507065`*^9, 3.588161813658409*^9}, {3.5881619205945253`*^9, 
   3.5881620015621567`*^9}, 3.588162068818003*^9, 3.5881621797223463`*^9, {
   3.5881632090832224`*^9, 3.5881632112343454`*^9}, {3.6098152094808826`*^9, 
   3.609815219712468*^9}, 3.6277416758579445`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "frenet"}]], "Input",
 CellChangeTimes->{{3.5881621858806987`*^9, 3.5881621891138835`*^9}}],

Cell[BoxData[
 StyleBox["\<\"frenet[curve][t] yields the Frenet frame for a generally \
curved \\ncurve[t] in the n-dimensional Euclidean space. Generally curved \
means that the \\nrank of the first n-1 derivatives of curve[t] must be n-1 \
at each point t.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.627741443869937*^9},
 CellTags->"Info3627745043-7142630"]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"frc", "[", "i_", "]"}], "[", "v_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"frenet", "[", "curve", "]"}], "[", "v", "]"}], "[", 
   RowBox[{"[", "i", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5881620864630127`*^9, 3.5881621696497707`*^9}, {
   3.588162213157259*^9, 3.5881622519004745`*^9}, 3.6277415330552936`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"frc", "[", "1", "]"}], "[", "v", "]"}], ",", "arcl"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.588162260438963*^9, 3.588162294247897*^9}, 
   3.62774161653104*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     SuperscriptBox["x", "\[Prime]",
      MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], ",", 
   RowBox[{
    RowBox[{
     SuperscriptBox["x", "\[Prime]",
      MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], ",", 
   RowBox[{
    RowBox[{
     SuperscriptBox["x", "\[Prime]",
      MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.6277417247640305`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"frc", "[", "2", "]"}], "[", "v", "]"}], ",", "arcl"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.588162260438963*^9, 3.58816232507666*^9}, 
   3.627741618200243*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{
      SuperscriptBox["x", "\[Prime]\[Prime]",
       MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"], 
      "+", 
      SuperscriptBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"], 
      "+", 
      SuperscriptBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
       "2"]}]]], ",", 
   FractionBox[
    RowBox[{
     RowBox[{
      SuperscriptBox["x", "\[Prime]\[Prime]",
       MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"], 
      "+", 
      SuperscriptBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"], 
      "+", 
      SuperscriptBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
       "2"]}]]], ",", 
   FractionBox[
    RowBox[{
     RowBox[{
      SuperscriptBox["x", "\[Prime]\[Prime]",
       MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"], 
      "+", 
      SuperscriptBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"], 
      "+", 
      SuperscriptBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
       "2"]}]]]}], "}"}]], "Output",
 CellChangeTimes->{3.6277417458396673`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"frc", "[", "3", "]"}], "[", "v", "]"}], "-", 
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "3", "]"}], "[", 
       RowBox[{"tangsf", "[", "curve", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "/.", 
    RowBox[{"u", "\[Rule]", "1"}]}], ",", "arcl"}], "]"}]], "Input",
 CellChangeTimes->{{3.5881624193480525`*^9, 3.5881624769663477`*^9}, 
   3.588162509207192*^9, 3.588163534773851*^9, {3.588163584366688*^9, 
   3.5881636160775013`*^9}, {3.6098154243721743`*^9, 3.609815444524327*^9}, 
   3.6148618522027264`*^9, 3.6149177841985474`*^9, 3.6151813235776625`*^9, 
   3.62774162194425*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.627741777304923*^9}]
}, Closed]],

Cell[TextData[{
 StyleBox["Lemma",
  FontColor->RGBColor[0, 0, 1]],
 ". The tangent  planes of the tangent surface of a regular generally curved  \
curve form a 1-parametric family of planes, namely the osculating planes of \
the curve.",
 "\[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.588163746058936*^9, 3.588163855090172*^9}, {
   3.588163888538085*^9, 3.588163931786559*^9}, 3.609815337807223*^9, 
   3.6098153803756576`*^9, {3.6098158536767287`*^9, 3.6098158536827292`*^9}}],

Cell["\<\
Again the spherical image  is a curve on the unit sphere only.  The first \
fundamental form of the tangent surface is\
\>", "Text",
 CellChangeTimes->{{3.5876231834576454`*^9, 3.5876232754354067`*^9}, {
  3.587732519481433*^9, 3.5877325282954483`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"u", ">", "0"}], ",", " ", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        FormBox["fund1",
         TraditionalForm], "[", 
        RowBox[{"tangsf", "[", "curve", "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"s1", ",", "s2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"s1", ",", "s2"}], "}"}]}], "]"}], ",", "arcl"}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5876233192090836`*^9, 3.5876233215178876`*^9}, {
   3.58762344825251*^9, 3.5876235183590336`*^9}, 3.5904957105934095`*^9, {
   3.609815592204774*^9, 3.6098156350922265`*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["s1", "2"], "+", 
  RowBox[{"2", " ", "s1", " ", "s2"}], "+", 
  RowBox[{
   SuperscriptBox["s2", "2"], " ", 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     RowBox[{
      SuperscriptBox["u", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}], 
         "2"]}], ")"}]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.627741835664625*^9}]
}, Closed]],

Cell[TextData[{
 "The coefficient of ",
 Cell[BoxData[
  SuperscriptBox["s2", "2"]],
  CellChangeTimes->{3.587623484054573*^9, 3.587623520870638*^9, 
   3.587731983789692*^9, 3.5877325449718776`*^9, 3.588143353745921*^9, 
   3.5881434770949764`*^9}],
 " depends on the curvature of the curve:"
}], "Text",
 CellChangeTimes->{{3.5881611650823126`*^9, 3.5881612328341875`*^9}, 
   3.5881615556586523`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"1", "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"u", "*", 
        RowBox[{
         RowBox[{"curvature", "[", "curve", "]"}], "[", "v", "]"}]}], ")"}], 
      "^", "2"}]}], "\[Equal]", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"rgT", "[", 
       RowBox[{"2", ",", "2"}], "]"}], "[", 
      RowBox[{"tangsf", "[", "curve", "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], ",", "arcl"}], "]"}]], "Input",
 CellChangeTimes->{{3.587732635249236*^9, 3.587732665138889*^9}, {
   3.587734459322841*^9, 3.5877344750476685`*^9}, {3.587734557213013*^9, 
   3.587734573624242*^9}, {3.588143552923313*^9, 3.588143610522608*^9}, {
   3.588143658443349*^9, 3.588143660107444*^9}, {3.5881437021078463`*^9, 
   3.5881437113943777`*^9}, 3.6098156989718804`*^9, 3.6148618522183266`*^9, 
   3.6148631365061827`*^9, {3.61518849458801*^9, 3.6151885227476206`*^9}, {
   3.617946380169375*^9, 3.6179464749551415`*^9}, {3.6179465495700727`*^9, 
   3.617946552346877*^9}, {3.617946583468932*^9, 3.617946594357751*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6277418806395044`*^9}]
}, Closed]],

Cell["\<\
Denote the curvature of the generating curve by k[v]. Then the first \
fundamental form is in traditional notation\
\>", "Text",
 CellChangeTimes->{{3.58816433061037*^9, 3.588164431618148*^9}, {
  3.588166702827053*^9, 3.5881667068342824`*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["ds", "2"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["du", "2"], "+", 
    RowBox[{"2", " ", "du", " ", "dv"}], "+", 
    RowBox[{
     SuperscriptBox["dv", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["k", "2"], " ", 
        SuperscriptBox["u", "2"]}], "+", "1"}], ")"}]}]}]}], 
  TraditionalForm]], "Input",
 CellChangeTimes->{{3.5881644831580954`*^9, 3.5881645610145483`*^9}, {
   3.588164594589469*^9, 3.5881646515087247`*^9}, 3.588164796428014*^9, {
   3.588166618812248*^9, 3.5881666271647253`*^9}, {3.588166660402627*^9, 
   3.588166674374426*^9}}],

Cell[TextData[{
 StyleBox["Lemma",
  FontColor->RGBColor[0, 0, 1]],
 ". A tangent surface of a generally curved curve is locally isometric to the \
Euclidean plane."
}], "Text",
 CellChangeTimes->{{3.5881661309633446`*^9, 3.5881661511865015`*^9}, {
   3.5881661837943664`*^9, 3.588166194841998*^9}, {3.5881662727544546`*^9, 
   3.5881663097305694`*^9}, {3.5881663506989126`*^9, 
   3.5881664278183236`*^9}, {3.588166502170576*^9, 3.5881666479229126`*^9}, 
   3.588166770114902*^9}],

Cell[TextData[{
 StyleBox["Proof",
  FontColor->RGBColor[0, 0, 1]],
 ". Let k[v] be the curvature of the curve as a function of the arc length.  \
We consider a plane curve with the same curvature.Then the tangent surface of \
this curve is a part of the plane and this part with the coordinates u,v of \
the tangent surface has the same first fundamental form as ",
 Cell[BoxData[
  StyleBox[
   SuperscriptBox["ds", "2"],
   FontWeight->"Bold"]],
  CellChangeTimes->{{3.5881644831580954`*^9, 3.5881645610145483`*^9}, {
    3.588164594589469*^9, 3.5881646515087247`*^9}, 3.588164796428014*^9, {
    3.588166618812248*^9, 3.5881666271647253`*^9}}],
 " above; the map defined by equal parameters  is an isometry.\[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.5881661309633446`*^9, 3.5881661511865015`*^9}, {
   3.5881661837943664`*^9, 3.588166194841998*^9}, {3.5881662727544546`*^9, 
   3.5881663097305694`*^9}, {3.5881663506989126`*^9, 
   3.5881664278183236`*^9}, {3.588166502170576*^9, 3.5881666479229126`*^9}, {
   3.588166770114902*^9, 3.588166781762568*^9}, {3.589623715895505*^9, 
   3.589623784606435*^9}, {3.609815789441055*^9, 3.6098157902561016`*^9}, {
   3.6098158333205643`*^9, 3.609815833331565*^9}, 3.615264176211238*^9, {
   3.6179466864915133`*^9, 3.617946694260327*^9}, 3.6179473026156626`*^9}],

Cell[TextData[{
 StyleBox["Example.",
  FontColor->RGBColor[0, 0, 1]],
 " To plot an image of a tangent surface it isn\[CloseCurlyQuote]t necessary \
to assume v to be the arc length of the curve. If not done yet, first load ",
 ButtonBox["CURVES`",
  BaseStyle->"Hyperlink",
  ButtonData->"curves"],
 " Take a space curve of A. Gray\[CloseCurlyQuote]s list, e. g."
}], "Text",
 CellChangeTimes->{{3.6098159294240613`*^9, 3.6098160902722616`*^9}, {
   3.6098165374328375`*^9, 3.60981659482412*^9}, {3.6098166414167852`*^9, 
   3.6098166414217854`*^9}, 3.6152641791634064`*^9, {3.627745547535145*^9, 
   3.6277455529327545`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "twistedn"}]], "Input",
 CellChangeTimes->{{3.6151885869542933`*^9, 3.61518858813136*^9}}],

Cell[BoxData[
 StyleBox["\<\"t->twistedn[n,a][t] is the curve t->a*{t,t^2,...,t^n}.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.6277455639619737`*^9},
 CellTags->"Info3627749163-7142630"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"twistedn", "[", 
   RowBox[{"3", ",", "1"}], "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.6098163918435106`*^9, 3.6098164178099957`*^9}, 
   3.6098166758507547`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"t", ",", 
   SuperscriptBox["t", "2"], ",", 
   SuperscriptBox["t", "3"]}], "}"}]], "Output",
 CellChangeTimes->{3.627745567253579*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"ParametricPlot3D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"twistedn", "[", 
      RowBox[{"3", ",", "1"}], "]"}], "[", "t", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"t", ",", 
      RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
    RowBox[{"PlotStyle", " ", "\[Rule]", " ", 
     RowBox[{"{", 
      RowBox[{"Thick", ",", " ", "Red"}], "}"}]}]}], "]"}], "\n"}]], "Input",
 CellChangeTimes->{{3.6098167011682024`*^9, 3.6098167997948437`*^9}, {
   3.6098172238280973`*^9, 3.609817253964821*^9}, 3.6098184802129583`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"cubic", "=", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.6098172685086527`*^9, 3.6098172828574734`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"tangsf", "[", 
   RowBox[{"twistedn", "[", 
    RowBox[{"3", ",", "1"}], "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{3.609817055443466*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"u", "+", "v"}], ",", 
   RowBox[{
    RowBox[{"2", " ", "u", " ", "v"}], "+", 
    SuperscriptBox["v", "2"]}], ",", 
   RowBox[{
    RowBox[{"3", " ", "u", " ", 
     SuperscriptBox["v", "2"]}], "+", 
    SuperscriptBox["v", "3"]}]}], "}"}]], "Output",
 CellChangeTimes->{3.627745580732003*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"ParametricPlot3D", "[", 
   RowBox[{
    RowBox[{"Evaluate", "[", 
     RowBox[{
      RowBox[{"tangsf", "[", 
       RowBox[{"twistedn", "[", 
        RowBox[{"3", ",", "1"}], "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"u", ",", 
      RowBox[{"-", "3"}], ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"v", ",", 
      RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}], "\n"}]], "Input",
 CellChangeTimes->{{3.6098167011682024`*^9, 3.6098167997948437`*^9}, {
   3.609816865372595*^9, 3.6098169849714355`*^9}, {3.609817087804317*^9, 
   3.6098171001240215`*^9}, {3.609817172053136*^9, 3.609817178644513*^9}, {
   3.6098175673657465`*^9, 3.6098175700919023`*^9}, 3.6148618522183266`*^9, 
   3.6149177842141476`*^9, {3.61794697485802*^9, 3.617946978024825*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"cubic", ",", "%"}], "}"}], ",", " ", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6098174162581034`*^9, 3.6098174565804095`*^9}}],

Cell[TextData[{
 "As  shown at the beginning of this subsection, the curve itself consists of \
singularities of the surface; in German such a singular curve is named the ",
 StyleBox["R\[UDoubleDot]ckkehrkante",
  FontColor->RGBColor[0, 0, 1]],
 " of the tangent surface."
}], "Text",
 CellChangeTimes->{{3.609818220856124*^9, 3.6098183795682015`*^9}, 
   3.615264200418622*^9, 3.6277457031922183`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["2.1.7. Developable Surfaces 1", "Subsubsection",
 CellChangeTimes->{{3.587201518412738*^9, 3.587201527076234*^9}, {
   3.587272835342759*^9, 3.58727283885396*^9}, {3.587272981302108*^9, 
   3.5872730078536263`*^9}, 3.5872731046621633`*^9, 3.587278763622837*^9, {
   3.5875395682098494`*^9, 3.58753958547908*^9}, {3.5875506376928062`*^9, 
   3.58755065084363*^9}, {3.5875621007464747`*^9, 3.5875621112900777`*^9}, {
   3.5881675117713223`*^9, 3.5881675315864553`*^9}, {3.590039110793*^9, 
   3.590039112899004*^9}, 3.609818829057911*^9, 3.6098201375367517`*^9, 
   3.617947335801561*^9, 3.6179498973303094`*^9, 3.627745769523535*^9, 
   3.627812689599845*^9}],

Cell[TextData[{
 "The surfaces considered in Subsections 2.1.3 - 6 have the following common \
properties:\n1. Locally, in a neighborhood of its regular points, they are \
isometric to open sets in the Euclidean plane. \n2. They are ruled surfaces \
whose tangent planes are constant along the rulings; the tangent planes form \
a 1-parameter family, and the surface is the ",
 StyleBox["envelope",
  FontColor->RGBColor[0, 0, 1]],
 " of this family. Such surfaces are named ",
 StyleBox["developable",
  FontColor->RGBColor[0, 0, 1]],
 " ",
 StyleBox["surfaces",
  FontColor->RGBColor[0, 0, 1]],
 " (also ",
 StyleBox["torses",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox[").",
  FontColor->GrayLevel[0]]
}], "Text",
 CellChangeTimes->{{3.588167702162212*^9, 3.588167785762994*^9}, {
   3.5881678992664857`*^9, 3.5881680912674675`*^9}, {3.588168243835194*^9, 
   3.5881683815790725`*^9}, {3.5881684601945686`*^9, 
   3.5881684888672085`*^9}, {3.5881685413312097`*^9, 
   3.5881685562750645`*^9}, {3.5881687263187904`*^9, 
   3.5881687325821486`*^9}, {3.588313711952491*^9, 3.5883137251212435`*^9}, {
   3.617947741961583*^9, 3.617947772433326*^9}, {3.6179478375380497`*^9, 
   3.617947840217203*^9}, 3.627809399644973*^9}],

Cell[TextData[{
 StyleBox["Remark",
  FontColor->RGBColor[0, 0, 1]],
 ". Sometimes the surfaces having the property 1 are called developable. We \
shall see that both properties are equivalent."
}], "Text",
 CellChangeTimes->{{3.588168501626939*^9, 3.5881685296505413`*^9}, {
   3.588168564514536*^9, 3.588168628546198*^9}, 3.588168834290966*^9, 
   3.588168905955065*^9}],

Cell[TextData[{
 "We specify the concept of the ",
 StyleBox["envelope",
  FontColor->RGBColor[0, 0, 1]],
 " of a 1-parametric family of planes. Consider this family as defined by the \
Hesse normal form of the plane equation"
}], "Text",
 CellChangeTimes->{{3.5881689795302734`*^9, 3.58816911077878*^9}, 
   3.588223189320817*^9, {3.627809545158792*^9, 3.6278095461259937`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"(", "1", ")"}], "         ", 
    RowBox[{"H", "[", "v", "]"}]}], ":", "   ", 
   RowBox[{
    RowBox[{
     RowBox[{"vec", "[", 
      RowBox[{"n", "[", "v", "]"}], "]"}], ".", 
     RowBox[{"vec", "[", "x", "]"}]}], " ", "\[Equal]", " ", 
    RowBox[{"p", "[", "v", "]"}]}]}], ";"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.58816914451871*^9, 3.588169184516998*^9}, {
  3.588169223453225*^9, 3.5881692616134076`*^9}, {3.588169784484314*^9, 
  3.588169790045632*^9}, {3.588170630071679*^9, 3.588170636168028*^9}, {
  3.5882232147176614`*^9, 3.5882232961654043`*^9}}],

Cell[BoxData[
 RowBox[{"         ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"vec", "[", 
      RowBox[{"n", "[", "v", "]"}], "]"}], ".", 
     RowBox[{"vec", "[", 
      RowBox[{"n", "[", "v", "]"}], "]"}]}], "\[Equal]", " ", "1"}], ";"}], 
  "                              "}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.58816914451871*^9, 3.588169184516998*^9}, {
  3.588169223453225*^9, 3.5881692616134076`*^9}, {3.588169784484314*^9, 
  3.588169790045632*^9}, {3.588170630071679*^9, 3.588170636168028*^9}, {
  3.5882232147176614`*^9, 3.5882232961654043`*^9}}],

Cell["\<\
vec[n[v]] is the unit normal vector of the plane  H[v], and p[v] is its \
oriented distance to the origin. We assume that in general two neighbored \
planes of the family are not parallel. Then the neighbored plane\
\>", "Text",
 CellChangeTimes->{{3.5881692817635603`*^9, 3.588169296634411*^9}, {
   3.5881693277151885`*^9, 3.5881693600910397`*^9}, {3.5881694707303686`*^9, 
   3.5881694890984187`*^9}, {3.588169563994703*^9, 3.5881696304745054`*^9}, 
   3.5882233336678705`*^9, 3.6151891048899174`*^9, {3.6278096679774113`*^9, 
   3.627809669428214*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"H", "[", 
   RowBox[{"v", "+", "\[CapitalDelta]v"}], "]"}], ":", "   ", 
  RowBox[{
   RowBox[{
    RowBox[{"vec", "[", 
     RowBox[{"n", "[", 
      RowBox[{"v", "+", "\[CapitalDelta]v"}], "]"}], "]"}], ".", 
    RowBox[{"vec", "[", "x", "]"}]}], " ", "\[Equal]", " ", 
   RowBox[{"p", "[", 
    RowBox[{"v", "+", "\[CapitalDelta]v"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5881696746290307`*^9, 3.5881697198396163`*^9}, {
  3.5881706549351006`*^9, 3.5881706928302684`*^9}}],

Cell["\<\
and the plane (1) intersect in a line. Subtracting (1) from the last equation \
we get the equivalent system with (1) and\
\>", "Text",
 CellChangeTimes->{{3.588169817044176*^9, 3.5881699449544926`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"vec", "[", 
      RowBox[{"n", "[", 
       RowBox[{"v", "+", "\[CapitalDelta]v"}], "]"}], "]"}], "-", 
     RowBox[{"vec", "[", 
      RowBox[{"n", "[", "v", "]"}], "]"}]}], ")"}], ".", 
   RowBox[{"vec", "[", "x", "]"}]}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"p", "[", 
    RowBox[{"v", "+", "\[CapitalDelta]v"}], "]"}], " ", "-", 
   RowBox[{"p", "[", "v", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5881696746290307`*^9, 3.5881697198396163`*^9}, {
   3.5881699584912663`*^9, 3.5881700206618223`*^9}, 3.5881709190392065`*^9}],

Cell["\<\
Dividing by \[CapitalDelta]v and considering the limit for \[CapitalDelta]v \
-> 0 we obtain\
\>", "Text",
 CellChangeTimes->{{3.5881701424827905`*^9, 3.5881702165230255`*^9}, {
  3.5882235164354544`*^9, 3.588223517291504*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"(", "2", ")"}], "                 ", 
    RowBox[{
     RowBox[{"vec", "[", 
      RowBox[{
       RowBox[{"n", "'"}], "[", "v", "]"}], "]"}], ".", 
     RowBox[{"vec", "[", "x", "]"}]}]}], " ", "\[Equal]", " ", 
   RowBox[{
    RowBox[{"p", "'"}], "[", "v", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5881703486705837`*^9, 3.5881704729966946`*^9}, {
  3.5882233724339385`*^9, 3.5882233992659855`*^9}, {3.588249351418147*^9, 
  3.588249351418147*^9}}],

Cell[TextData[{
 "The linear system (1), (2) defines the limit of the intersection line of \
two neighbored planes; it is a line L[v] \[Subset] H[v].  The family of these \
lines  describes a surface, called the ",
 StyleBox["envelope",
  FontColor->RGBColor[0, 0, 1]],
 " of the plane family.We will find a parameter representation of this \
surface. \n First we exclude the trivial case: vec[n[v]] = constant  Then by \
(2)  there exists a solution only iff also p[v)== constant, and the solution \
is the whole plane given by (1).  Now we assume"
}], "Text",
 CellChangeTimes->{{3.588170493386861*^9, 3.588170564282916*^9}, 
   3.5881706131627116`*^9, {3.5881707202988396`*^9, 3.5881708395146585`*^9}, 
   3.588223422057626*^9, {3.588223503955741*^9, 3.588223504307761*^9}, 
   3.5882235387797327`*^9, 3.588223578346996*^9, {3.588223678451721*^9, 
   3.588223703267141*^9}, {3.5882237384271517`*^9, 3.5882239335233107`*^9}, {
   3.588223980171979*^9, 3.588224014403937*^9}, {3.588224092035377*^9, 
   3.5882240997228165`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"vec", "[", 
   RowBox[{
    RowBox[{"n", "'"}], "[", "v", "]"}], "]"}], "\[NotEqual]", " ", 
  RowBox[{"null", "[", "]"}]}]], "Input",
 CellChangeTimes->{{3.588224154310939*^9, 3.5882241641175003`*^9}, 
   3.588249351418147*^9}],

Cell[TextData[{
 "for all values of v; that means we exclude the",
 StyleBox[" singular planes",
  FontColor->RGBColor[0, 0, 1]],
 " of the family not satisfying this assumption. Further we assume that v is \
the arc length of the curve vec[n[v]] on the unit sphere. It follows"
}], "Text",
 CellChangeTimes->{{3.588224215228423*^9, 3.5882242746508217`*^9}, {
  3.5882243074997005`*^9, 3.588224418731063*^9}, {3.6152642253870506`*^9, 
  3.6152642280992055`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"vec", "[", 
    RowBox[{
     RowBox[{"n", "'"}], "[", "v", "]"}], "]"}], ".", 
   RowBox[{"vec", "[", 
    RowBox[{
     RowBox[{"n", "'"}], "[", "v", "]"}], "]"}]}], "\[Equal]", "1"}]], "Input",
 CellChangeTimes->{{3.588224458469336*^9, 3.5882244813016415`*^9}, {
  3.588249351418147*^9, 3.5882493514337473`*^9}}],

Cell["and by derivation", "Text",
 CellChangeTimes->{{3.588224538131892*^9, 3.5882245496755524`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"vec", "[", 
    RowBox[{"n", "[", "v", "]"}], "]"}], ".", 
   RowBox[{"vec", "[", 
    RowBox[{
     RowBox[{"n", "'"}], "[", "v", "]"}], "]"}]}], "\[Equal]", " ", 
  "0"}]], "Input",
 CellChangeTimes->{{3.5882245658614783`*^9, 3.588224670981491*^9}, 
   3.588249351464947*^9}],

Cell[BoxData[
 RowBox[{"  ", 
  RowBox[{
   RowBox[{
    RowBox[{"vec", "[", 
     RowBox[{
      RowBox[{"n", "'"}], "[", "v", "]"}], "]"}], ".", 
    RowBox[{"vec", "[", 
     RowBox[{
      RowBox[{"n", "''"}], "[", "v", "]"}], "]"}]}], "\[Equal]", " ", 
   "0"}]}]], "Input",
 CellChangeTimes->{{3.5882245658614783`*^9, 3.588224670981491*^9}, {
  3.588249351480547*^9, 3.588249351480547*^9}}],

Cell["\<\
Since vec[n[v]], vec[n'[v]] are orthogonal unit vectors the lines L[v] as \
solutions of the system (1), (2) exist and are uniquely defined. We choose an \
arbitrary smooth curve of solution points vec[x[v]] \[Element] L[v]. Then the \
difference vector with any solution vector\
\>", "Text",
 CellChangeTimes->{{3.588224756027355*^9, 3.588225057963625*^9}, {
   3.5882250974998865`*^9, 3.588225159196415*^9}, 3.5882493515117474`*^9, {
   3.5883143728092895`*^9, 3.5883144417842345`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"vec", "[", "x", "]"}], "-", 
  RowBox[{"vec", "[", 
   RowBox[{"x", "[", "v", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.588225162159585*^9, 3.5882251621605844`*^9}}],

Cell["satisfies", "Text",
 CellChangeTimes->{{3.5882251698270235`*^9, 3.588225182027721*^9}}],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{
    RowBox[{"vec", "[", 
     RowBox[{"n", "[", "v", "]"}], "]"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"vec", "[", "x", "]"}], "-", 
      RowBox[{"vec", "[", 
       RowBox[{"x", "[", "v", "]"}], "]"}]}], ")"}]}], "\[Equal]", " ", 
   "0"}]}]], "Input",
 CellChangeTimes->{{3.5882252507816534`*^9, 3.588225301341545*^9}}],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{
    RowBox[{"vec", "[", 
     RowBox[{
      RowBox[{"n", "'"}], "[", "v", "]"}], "]"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"vec", "[", "x", "]"}], "-", 
      RowBox[{"vec", "[", 
       RowBox[{"x", "[", "v", "]"}], "]"}]}], ")"}]}], "\[Equal]", " ", 
   "0"}]}]], "Input",
 CellChangeTimes->{{3.5882252507816534`*^9, 3.588225337870635*^9}, 
   3.5882493515273476`*^9}],

Cell[" and with", "Text",
 CellChangeTimes->{{3.588225361011958*^9, 3.588225364643166*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"m", "[", "v_", "]"}], ":=", " ", 
  RowBox[{"Cross", "[", 
   RowBox[{
    RowBox[{"vec", "[", 
     RowBox[{"n", "[", "v", "]"}], "]"}], ",", 
    RowBox[{"vec", "[", 
     RowBox[{
      RowBox[{"n", "'"}], "[", "v", "]"}], "]"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5882253883235207`*^9, 3.588225509749466*^9}, 
   3.5882493515273476`*^9}],

Cell["\<\
we obtain the wanted parameter representation of the envelope as the ruled \
surface\
\>", "Text",
 CellChangeTimes->{{3.5882255299016185`*^9, 3.588225543363388*^9}, {
  3.588225639010859*^9, 3.5882256690675783`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"solsf", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"vec", "[", 
    RowBox[{"x", "[", "v", "]"}], "]"}], " ", "+", " ", 
   RowBox[{"u", "*", 
    RowBox[{"m", "[", "v", "]"}]}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5882255749011917`*^9, 3.588225629869336*^9}, {
  3.588227285079009*^9, 3.5882272865500927`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"solsf", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5882256864725733`*^9, 3.5882256943570247`*^9}, {
  3.5882272973097086`*^9, 3.588227298808794*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"x", "[", "v", "]"}], "[", "1", "]"}], "+", 
    RowBox[{"u", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{
          RowBox[{"n", "[", "v", "]"}], "[", "3", "]"}]}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["n", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"n", "[", "v", "]"}], "[", "2", "]"}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["n", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
      ")"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"x", "[", "v", "]"}], "[", "2", "]"}], "+", 
    RowBox[{"u", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"n", "[", "v", "]"}], "[", "3", "]"}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["n", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], "-", 
       RowBox[{
        RowBox[{
         RowBox[{"n", "[", "v", "]"}], "[", "1", "]"}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["n", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "3", "]"}]}]}], 
      ")"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"x", "[", "v", "]"}], "[", "3", "]"}], "+", 
    RowBox[{"u", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{
          RowBox[{"n", "[", "v", "]"}], "[", "2", "]"}]}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["n", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "1", "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"n", "[", "v", "]"}], "[", "1", "]"}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["n", "\[Prime]",
           MultilineFunction->None], "[", "v", "]"}], "[", "2", "]"}]}]}], 
      ")"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6278106783745756`*^9, 3.6278107689447355`*^9}]
}, Closed]],

Cell["\<\
Regard that (vec[n[v]] ,vec[n'[v]], m[v]) form a family of positively \
oriented orthonormal frames of the Euclidean vector space.\
\>", "Text",
 CellChangeTimes->{{3.588226562763695*^9, 3.5882266954272823`*^9}, 
   3.5882493515741477`*^9, {3.5883145374247046`*^9, 3.5883145665123687`*^9}, 
   3.615264263011202*^9}],

Cell["\<\
We will specify the choice of the up to now arbitrary point vec[x[v]] \
\[Element] L[v]. The lines L(v) \[Subset] solsf form a 1-parameter family of \
lines on solsf and this family may have an envelope being a curve in solsf. \
Considering the intersection point of two neighbored lines and forming the \
limit as before for the planes we obtain the new equation\
\>", "Text",
 CellChangeTimes->{{3.5882270884837637`*^9, 3.588227163644063*^9}, {
  3.5882272044033947`*^9, 3.5882272723632812`*^9}, {3.5882273138996572`*^9, 
  3.588227314427687*^9}, {3.5882273797874255`*^9, 3.5882274552117395`*^9}, {
  3.588227606035366*^9, 3.588227705715068*^9}, {3.5882277412511005`*^9, 
  3.5882277753560505`*^9}, {3.588314581137205*^9, 3.5883146185293436`*^9}, {
  3.5896240502636294`*^9, 3.589624065926525*^9}, {3.627810895632959*^9, 
  3.6278108972397614`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", "3", ")"}], "        ", 
   RowBox[{
    RowBox[{"vec", "[", 
     RowBox[{
      RowBox[{"n", "''"}], "[", "v", "]"}], "]"}], ".", 
    RowBox[{"vec", "[", "x", "]"}]}]}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"p", "''"}], "[", "v", "]"}]}]], "Input",
 CellChangeTimes->{{3.5882277882617893`*^9, 3.588227862166016*^9}, {
  3.588227911814856*^9, 3.5882279124458923`*^9}, {3.5882493515897474`*^9, 
  3.5882493515897474`*^9}, {3.588249855162869*^9, 3.588249862274276*^9}}],

Cell["\<\
Now considering the system (1), (2), (3) we assume that the vectors \
vec[n[v]],  vec[n\[CloseCurlyQuote][v]], vec[n\[CloseCurlyQuote]\
\[CloseCurlyQuote][v]] are linearly independent. Then for any value of v \
there exists a uniquely defined solution of the system, say vec[z[v]]. \
Inserting this solution and differentiating (1) we obtain using (2):\
\>", "Text",
 CellChangeTimes->{{3.5882507314879923`*^9, 3.5882508650476313`*^9}, {
   3.588250898919568*^9, 3.588250988743706*^9}, {3.588251026695877*^9, 
   3.5882510652160797`*^9}, {3.5882515321277857`*^9, 3.5882515797755113`*^9}, 
   3.5882544666876335`*^9, {3.588314657265559*^9, 3.588314702873168*^9}, 
   3.617948569314351*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"vec", "[", 
     RowBox[{"n", "[", "v", "]"}], "]"}], ".", 
    RowBox[{"vec", "[", 
     RowBox[{
      RowBox[{"z", "'"}], "[", "v", "]"}], "]"}]}], " ", "+", " ", 
   RowBox[{
    RowBox[{"vec", "[", 
     RowBox[{
      RowBox[{"n", "'"}], "[", "v", "]"}], "]"}], ".", 
    RowBox[{"vec", "[", 
     RowBox[{"z", "[", "v", "]"}], "]"}]}]}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"p", "'"}], "[", "v", "]"}]}]], "Input",
 CellChangeTimes->{{3.588251647891407*^9, 3.5882517007304296`*^9}, {
  3.588252009659099*^9, 3.5882520141383553`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"vec", "[", 
    RowBox[{"n", "[", "v", "]"}], "]"}], ".", 
   RowBox[{"vec", "[", 
    RowBox[{
     RowBox[{"z", "'"}], "[", "v", "]"}], "]"}]}], "\[Equal]", " ", 
  "0"}]], "Input",
 CellChangeTimes->{{3.5882517374495296`*^9, 3.5882517400336776`*^9}}],

Cell["Analogously, differentiating (2) and using (3) we obtain", "Text",
 CellChangeTimes->{{3.588251764577081*^9, 3.588251813207863*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"vec", "[", 
      RowBox[{
       RowBox[{"n", "''"}], "[", "v", "]"}], "]"}], ".", 
     RowBox[{"vec", "[", 
      RowBox[{"z", "[", "v", "]"}], "]"}]}], " ", "+", " ", 
    RowBox[{
     RowBox[{"vec", "[", 
      RowBox[{
       RowBox[{"n", "'"}], "[", "v", "]"}], "]"}], ".", 
     RowBox[{"vec", "[", 
      RowBox[{
       RowBox[{"z", "'"}], "[", "v", "]"}], "]"}]}]}], " ", "\[Equal]", " ", 
   RowBox[{
    RowBox[{"p", "''"}], "[", "v"}]}], "}"}]], "Input",
 CellChangeTimes->{{3.588251647891407*^9, 3.5882517007304296`*^9}, {
  3.5882519593002186`*^9, 3.588252024290936*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"vec", "[", 
    RowBox[{
     RowBox[{"n", "'"}], "[", "v", "]"}], "]"}], ".", 
   RowBox[{"vec", "[", 
    RowBox[{
     RowBox[{"z", "'"}], "[", "v", "]"}], "]"}]}], "\[Equal]", " ", 
  "0"}]], "Input",
 CellChangeTimes->{{3.5882517374495296`*^9, 3.5882517400336776`*^9}, 
   3.588252071458634*^9}],

Cell["\<\
Remembering that vec[n[v]], vec[n\[CloseCurlyQuote][v]],  m[v] form an \
orthonormal frame we conclude\
\>", "Text",
 CellChangeTimes->{{3.5882520842993684`*^9, 3.588252111823943*^9}, {
  3.588252162215825*^9, 3.5882522530560207`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"vec", "[", 
   RowBox[{
    RowBox[{"z", "'"}], "[", "v", "]"}], "]"}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{"h", "[", "v", "]"}], "*", 
   RowBox[{"m", "[", "v", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.588252291249205*^9, 3.5882523068100953`*^9}}],

Cell["\<\
 for a certain function h[v]. If vec[z\[CloseCurlyQuote][v]] \[NotEqual] \
null[], the tangents of the curve vec[z[v]] coincide with L[v], we may take \
x[v].= z[v] in the description of the envelope and see that the envelope is \
the tangent surface of the solution vec[z[v]]. If, on the other hand, vec[z\
\[CloseCurlyQuote][v]] == null[] the solution vec[z[v]] ==  vec[z] is \
constant, and the envelope is a conical surface with the vertex vec[z].  It \
remains to consider the case that vec[n[v]], vec[n\[CloseCurlyQuote][v]], \
vec[n\[CloseCurlyQuote]\[CloseCurlyQuote][v]] are linearly dependent. Since \
vec[n[v]], vec[n\[CloseCurlyQuote][v]] are linearly independent there exist \
two functions h[v], f[v] such that\
\>", "Text",
 CellChangeTimes->{{3.5882523323675575`*^9, 3.5882523659594784`*^9}, {
   3.588252403119604*^9, 3.5882525301118674`*^9}, {3.588252562376713*^9, 
   3.588252669647848*^9}, {3.588252736247658*^9, 3.5882527644802723`*^9}, {
   3.5882527968651247`*^9, 3.5882528233836412`*^9}, {3.588252933663949*^9, 
   3.588252998519659*^9}, {3.5882537708398333`*^9, 3.5882538299282126`*^9}, {
   3.588254100159669*^9, 3.5882541141364684`*^9}, {3.58825416206621*^9, 
   3.588254201264452*^9}, {3.588254344448642*^9, 3.588254427728405*^9}, 
   3.588254475112115*^9, {3.588254820552873*^9, 3.588254859344092*^9}, 
   3.5882558381370754`*^9, {3.5883147947924256`*^9, 3.5883148574810114`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", "4", ")"}], "        ", 
   RowBox[{"vec", "[", 
    RowBox[{
     RowBox[{"n", "''"}], "[", "v", "]"}], "]"}]}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "v", "]"}], "*", 
    RowBox[{"vec", "[", 
     RowBox[{"n", "[", "v", "]"}], "]"}]}], "+", " ", 
   RowBox[{
    RowBox[{"h", "[", "v", "]"}], "*", 
    RowBox[{"vec", "[", 
     RowBox[{
      RowBox[{"n", "'"}], "[", "v", "]"}], "]"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5882548846685405`*^9, 3.588254949778264*^9}, {
  3.5882554002110276`*^9, 3.5882554059073534`*^9}}],

Cell["\<\
The Dot product with vec[n\[CloseCurlyQuote][v]] yields h[v] == 0.  \
Differentiating vec[n[v]].vec[n\[CloseCurlyQuote][v]] == 0 we obtain\
\>", "Text",
 CellChangeTimes->{{3.588254991015623*^9, 3.5882551163277903`*^9}, {
  3.5882552475522957`*^9, 3.588255277408004*^9}}],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{
    RowBox[{"vec", "[", 
     RowBox[{"n", "[", "v", "]"}], "]"}], ".", 
    RowBox[{"vec", "[", 
     RowBox[{
      RowBox[{"n", "''"}], "[", "v", "]"}], "]"}]}], " ", "\[Equal]", " ", 
   RowBox[{"-", 
    RowBox[{
     RowBox[{"vec", "[", 
      RowBox[{
       RowBox[{"n", "'"}], "[", "v", "]"}], "]"}], ".", 
     RowBox[{"vec", "[", 
      RowBox[{
       RowBox[{"n", "'"}], "[", "v", "]"}], "]"}]}]}], " ", "\[Equal]", " ", 
   RowBox[{"-", "1"}]}]}]], "Input",
 CellChangeTimes->{{3.58825529726814*^9, 3.5882553831390514`*^9}, 
   3.588255433762947*^9}],

Cell["\<\
and the Dot product of  (4) with vec[n[v]]  yields f[v] == -1, thus we have \
\>", "Text",
 CellChangeTimes->{{3.588255445336609*^9, 3.58825551791176*^9}, {
  3.588255961705143*^9, 3.5882559648163214`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", "5", ")"}], "       ", 
   RowBox[{"vec", "[", 
    RowBox[{
     RowBox[{"n", "''"}], "[", "v", "]"}], "]"}]}], "\[Equal]", " ", 
  RowBox[{"-", " ", 
   RowBox[{"vec", "[", 
    RowBox[{"n", "[", "v", "]"}], "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5882556594018526`*^9, 3.588255677505888*^9}, {
  3.5882559714417*^9, 3.5882559760659647`*^9}}],

Cell["\<\
Now we consider the tangent vector m[v] = Cross[vec[n[v]], vec[n\
\[CloseCurlyQuote][v]]] of the line L[v]. We derive this equation and obtain \
\>", "Text",
 CellChangeTimes->{{3.5882557291788435`*^9, 3.588255736385256*^9}, {
  3.588255779688732*^9, 3.58825579258447*^9}, {3.5882560164642754`*^9, 
  3.5882560741835766`*^9}, {3.5882561426084905`*^9, 3.588256154536173*^9}, {
  3.588256323640845*^9, 3.5882563745037537`*^9}, {3.5883149684323573`*^9, 
  3.588315003656372*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"m", "'"}], "[", "v", "]"}], " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"Cross", "[", 
     RowBox[{
      RowBox[{"vec", "[", 
       RowBox[{
        RowBox[{"n", "'"}], "[", "v", "]"}], "]"}], ",", " ", 
      RowBox[{"vec", "[", 
       RowBox[{
        RowBox[{"n", "'"}], "[", "v", "]"}], "]"}]}], "]"}], "  ", "+", " ", 
    RowBox[{"Cross", "[", 
     RowBox[{
      RowBox[{"vec", "[", 
       RowBox[{"n", "[", "v", "]"}], "]"}], ",", " ", 
      RowBox[{"vec", "[", 
       RowBox[{
        RowBox[{"n", "''"}], "[", "v", "]"}], "]"}]}], "]"}]}], " ", 
   "\[Equal]", " ", 
   RowBox[{"null", "[", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5882561811636953`*^9, 3.5882562189628572`*^9}, {
   3.5882562738379965`*^9, 3.5882563086289864`*^9}, 3.5883150319099874`*^9}],

Cell["\<\
Therefore m[v] is constant, and the envelope is a cylindrical surface.  \
Summarizing we proved:\
\>", "Text",
 CellChangeTimes->{{3.5882567953318243`*^9, 3.58825682305641*^9}, {
   3.5882568771925063`*^9, 3.588256903752025*^9}, 3.588315098768812*^9, {
   3.5883151461535225`*^9, 3.5883151633765073`*^9}}],

Cell[TextData[{
 StyleBox["Proposition",
  FontColor->RGBColor[0, 0, 1]],
 ". The envelope of a family of planes (1) is the set of  all solutions of \
the system (1), (2).  If vec[n[v]] is constant the envelope is a plane. \
Otherwise the envelope is locally, in the neighborhood of a plane with "
}], "Text",
 CellChangeTimes->{{3.5883151680647755`*^9, 3.588315182672611*^9}, {
   3.5883152262491035`*^9, 3.588315279185131*^9}, {3.5883153113609715`*^9, 
   3.588315350529212*^9}, {3.5883153994650106`*^9, 3.588315515048622*^9}, {
   3.5883159339855833`*^9, 3.588315946921324*^9}, 3.6152643301310415`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"vec", "[", 
    RowBox[{
     RowBox[{"n", "'"}], "[", "v", "]"}], "]"}], " ", "\[NotEqual]", " ", 
   RowBox[{"null", "[", "]"}]}], ","}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5883155376899166`*^9, 3.588315562603342*^9}}],

Cell["\<\
a developable surface, namely a tangent surface, a conical or a cylindrical \
surface. Any such surface is the envelope of a 1-parametric family of planes, \
namely its tangent planes. Thus its spherical image is a curve. These \
surfaces are locally isometric to plane regions.\[FilledSquare]\
\>", "Text",
 CellChangeTimes->{{3.5883155891288586`*^9, 3.5883156337204094`*^9}, {
   3.588315687208469*^9, 3.588315892569215*^9}, 3.589624400446659*^9, {
   3.609820118876684*^9, 3.6098201188846846`*^9}}],

Cell["\<\
The last statement has been proved for each of the three types in subsections \
2.1.4-6.\
\>", "Text",
 CellChangeTimes->{{3.589624420686816*^9, 3.589624461254137*^9}, {
  3.5896244950710707`*^9, 3.589624519638476*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["2.1.8. One-Sheeted Hyperboloids", "Subsubsection",
 CellChangeTimes->{{3.587201518412738*^9, 3.587201527076234*^9}, {
   3.587272835342759*^9, 3.58727283885396*^9}, {3.587272981302108*^9, 
   3.5872730078536263`*^9}, 3.5872731046621633`*^9, 3.587278763622837*^9, {
   3.5875395682098494`*^9, 3.58753958547908*^9}, {3.5875506376928062`*^9, 
   3.58755065084363*^9}, {3.5875621007464747`*^9, 3.5875621112900777`*^9}, {
   3.5881675117713223`*^9, 3.5881675315864553`*^9}, {3.58831735340977*^9, 
   3.588317395129156*^9}, 3.6098201445611534`*^9, 3.6098236513987246`*^9, 
   3.6179500274097495`*^9, 3.617951459497492*^9, 3.627812925363861*^9, 
   3.6278137765121903`*^9}],

Cell[TextData[{
 "One-sheeted hyperboloids are the standard examples of  ruled, non \
developable surfaces, see ",
 ButtonBox["MathWorld",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://mathworld.wolfram.com/One-SheetedHyperboloid.html"], None},
  ButtonNote->"http://mathworld.wolfram.com/One-SheetedHyperboloid.html"],
 ". We consider the surface generated by rotating a hyperbola in the x, z- \
plane around the z-axis. A parameter representation is"
}], "Text",
 CellChangeTimes->{{3.588317415403316*^9, 3.5883175018172584`*^9}, {
   3.5883175511190786`*^9, 3.5883175511240788`*^9}, {3.5883175887452307`*^9, 
   3.5883175964246693`*^9}, {3.5883176462165174`*^9, 
   3.5883176956253433`*^9}, {3.5883177767289824`*^9, 3.5883178420087166`*^9}, 
   3.588336208915674*^9, 3.589624589599478*^9, {3.6278132508740597`*^9, 
   3.627813260858077*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "hyperboloid"}]], "Input",
 CellChangeTimes->{{3.5883362179324903`*^9, 3.5883362240945005`*^9}}],

Cell[BoxData[
 StyleBox["\<\"{u,v}->hyperboloid[a,c][u,v] is the standard parametrization \
of a circular hyperboloid of one sheet. {u,v}->hyperboloid[a,b,c][u,v] is the \
standard parametrization of an elliptical hyperboloid of one sheet.  To plot \
try hyperboloid[1,2,3][0,3Pi/2,-2,2].\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.627813279970111*^9},
 CellTags->"Info3627816879-6530549"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"hyperboloid", "[", 
   RowBox[{"a", ",", "b", ",", "c"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{
  3.5883403874176536`*^9, {3.5883404244667726`*^9, 3.5883404263938828`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", " ", 
    RowBox[{"Cos", "[", "u", "]"}], " ", 
    RowBox[{"Cosh", "[", "v", "]"}]}], ",", 
   RowBox[{"b", " ", 
    RowBox[{"Cosh", "[", "v", "]"}], " ", 
    RowBox[{"Sin", "[", "u", "]"}]}], ",", 
   RowBox[{"c", " ", 
    RowBox[{"Sinh", "[", "v", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6278132868985233`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"hyperboloid", "[", 
   RowBox[{"1", ",", "2", ",", "3"}], "]"}], "[", 
  RowBox[{"0", ",", 
   RowBox[{"3", 
    RowBox[{"Pi", "/", "2"}]}], ",", 
   RowBox[{"-", "2"}], ",", "2"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"hyperboloid", "[", 
   RowBox[{"1", ",", "1", ",", "1"}], "]"}], "[", 
  RowBox[{"0", ",", 
   RowBox[{"2", "*", "Pi"}], ",", 
   RowBox[{"-", "2"}], ",", "2"}], "]"}]], "Input",
 CellChangeTimes->{{3.5883386193385253`*^9, 3.58833863724955*^9}}],

Cell["We consider", "Text",
 CellChangeTimes->{{3.6151958465297346`*^9, 3.6151958510249915`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"hypsf", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", 
  RowBox[{
   RowBox[{"hyperboloid", "[", 
    RowBox[{"1", ",", "1", ",", "1"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.588318033245654*^9, 3.5883180445303*^9}, {
  3.588318457179902*^9, 3.5883184740108647`*^9}, {3.588336256043357*^9, 
  3.588336269974181*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hypsf", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.588336161585191*^9, 3.5883361828948283`*^9}, {
  3.588336306119445*^9, 3.588336306805846*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", "u", "]"}], " ", 
    RowBox[{"Cosh", "[", "v", "]"}]}], ",", 
   RowBox[{
    RowBox[{"Cosh", "[", "v", "]"}], " ", 
    RowBox[{"Sin", "[", "u", "]"}]}], ",", 
   RowBox[{"Sinh", "[", "v", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627813336575011*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"hyp", " ", "=", " ", 
   RowBox[{"ParametricPlot3D", "[", 
    RowBox[{
     RowBox[{"hypsf", "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
     RowBox[{"{", 
      RowBox[{"u", ",", "0", ",", 
       RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"v", ",", 
       RowBox[{"-", "2"}], ",", "2"}], "}"}], ",", 
     RowBox[{"PlotStyle", " ", "\[Rule]", " ", 
      RowBox[{"Opacity", "[", ".6", "]"}]}], ",", " ", 
     RowBox[{"Mesh", " ", "\[Rule]", " ", "None"}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5883386948088417`*^9, 3.5883387920834055`*^9}, {
   3.5883388681147547`*^9, 3.588338902323711*^9}, {3.5883389574828663`*^9, 
   3.588338974385833*^9}, 3.5883404603178234`*^9, 3.5883414040037985`*^9}],

Cell["It is the quadric defined by the equation", "Text",
 CellChangeTimes->{{3.5883182455697985`*^9, 3.588318261936735*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"x", "^", "2"}], " ", "+", " ", 
   RowBox[{"y", "^", "2"}], " ", "-", 
   RowBox[{"z", "^", "2"}]}], " ", "\[Equal]", " ", "1"}]], "Input",
 CellChangeTimes->{{3.5883182820748863`*^9, 3.5883183021390343`*^9}, {
  3.5883926010801964`*^9, 3.588392611438615*^9}}],

Cell["The normal vector field is", "Text",
 CellChangeTimes->{{3.5883183493287334`*^9, 3.588318360512373*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"nhyp", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"amf", "[", "3", "]"}], "[", "hypsf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"u", ">", "0"}], ",", 
      RowBox[{"v", ">", "0"}]}], "}"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.588318365192641*^9, 3.588318372047033*^9}, {
   3.5883184180426636`*^9, 3.58831842515207*^9}, {3.5883184870836124`*^9, 
   3.5883185219316053`*^9}, {3.5883188504443955`*^9, 3.588318860715983*^9}, 
   3.588392645727475*^9, 3.614861852249527*^9, 3.614917784229748*^9, 
   3.6151813598587375`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"Cos", "[", "u", "]"}], " ", 
     RowBox[{"Cosh", "[", "v", "]"}]}], 
    SqrtBox[
     RowBox[{"Cosh", "[", 
      RowBox[{"2", " ", "v"}], "]"}]]], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"Cosh", "[", "v", "]"}], " ", 
     RowBox[{"Sin", "[", "u", "]"}]}], 
    SqrtBox[
     RowBox[{"Cosh", "[", 
      RowBox[{"2", " ", "v"}], "]"}]]], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"Sinh", "[", "v", "]"}], 
     SqrtBox[
      RowBox[{"Cosh", "[", 
       RowBox[{"2", " ", "v"}], "]"}]]]}]}], "}"}]], "Output",
 CellChangeTimes->CompressedData["
1:eJwVzl1Ik2EAxfFtbtOV2YzFlC4iL3KsqSmWEaS+VmpNGwuaqZgk09TKYA01
K7/TESMrwwvBj4XkXGo5FXRMkaGWvaypWFZqLrOS3lVaDhR0dJ6Lw+/i3PwP
ZN04n81msVgXMc/Px43zXgzVIzNov8N1sb/Sm8tQiYnpi8Sp/kGbHwxQXncJ
iYLQeH9YTa+aZTA5qO5RKCxvqBOGwTtr8THEzIh+axrUL325VglFDk3hA2hr
fHK8Gz4vXp6Zhc6HWZ1OGGl6VcnlMdTIy9scKYxO0jcp4XgwnaeDoXtHQ0ww
wB3y7R2kc6XzbD5Ducs7jnHgs9SajzyoFzj9BNA1bSveCX0nZ1qI+eamYSL7
V3iXLzwbtsHeBVeOfBYTu9QvYojmfZE5xAXX3Qri1G7hAPEkb2KKqOGWbRFN
wR/C/eC4qyXsIGzunDtDfP1Hz5dATnuwl4z8b2LvK6B2rV1RCq2Bi9pyqNZx
bhH3WPhGYpoh06KD91LWWR3QYu+vpWH60Ixyk/RKRG8l3gylq4/zlkIfe5+G
aO3Y/iGHpy0qOhm2iWvmFFCykPOVGFUQ4MmA0SM8eRk86sg7VQHnS1KXn0I1
q/iCCZ4rGMp2QM3lKPssLLJJc93wStZS9Sa8VLgStwVVyxlVXB+GEpcM+wbB
uiTPtAyGdye45DBCx+pRwcHaxJsp0CNSFWmhc5VuboEn/PMTVAKGKn1/OEIL
t3uNk62wYUV6aBQa1dqBCdhWT//+B71cpVc34UZvVawH9o0Z/gbuQL9j7NN+
+B/+60Hg
  "]]
}, Closed]],

Cell["\<\
We fix a point {u,v} and consider the tangent plane of the hyperboloid at \
this point with the variable point vec[x]. Its equation is \
\>", "Text",
 CellChangeTimes->{{3.588318711689459*^9, 3.5883187975213685`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"h", "[", 
   RowBox[{"u_", ",", " ", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"nhyp", "[", 
     RowBox[{"u", ",", " ", "v"}], "]"}], ".", 
    RowBox[{"vec", "[", "x", "]"}]}], " ", "==", " ", 
   RowBox[{
    RowBox[{"nhyp", "[", 
     RowBox[{"u", ",", " ", "v"}], "]"}], ".", 
    RowBox[{"hypsf", "[", 
     RowBox[{"u", ",", " ", "v"}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.58833644559929*^9, 3.58833644559929*^9}, {
  3.5883368501704006`*^9, 3.588336863664424*^9}, {3.5883926475838785`*^9, 
  3.588392650391883*^9}}],

Cell["\<\
The intersection of the hyperboloid with the tangent plane at {u,v} = {0,0} \
is the solution of\
\>", "Text",
 CellChangeTimes->{{3.588336386537586*^9, 3.588336413541233*^9}, {
  3.588336480340551*^9, 3.588336555641883*^9}, {3.609822617083565*^9, 
  3.6098226359956465`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reduce", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"h", "[", 
      RowBox[{"0", ",", "0"}], "]"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"x", "[", "1", "]"}], "^", "2"}], " ", "+", " ", 
       RowBox[{
        RowBox[{"x", "[", "2", "]"}], "^", "2"}], " ", "-", 
       RowBox[{
        RowBox[{"x", "[", "3", "]"}], "^", "2"}]}], "\[Equal]", " ", "1"}]}], 
    "}"}], ",", " ", 
   RowBox[{"vec", "[", "x", "]"}], ",", "Reals"}], "]"}]], "Input",
 CellChangeTimes->{{3.5883189765386076`*^9, 3.588319000498978*^9}, {
   3.5883190488517437`*^9, 3.5883191283982935`*^9}, {3.588319186566621*^9, 
   3.588319199897383*^9}, {3.588319458461172*^9, 3.58831946104532*^9}, 
   3.6148618522651267`*^9, 3.614917784245348*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], "\[Equal]", "1"}], "&&", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"x", "[", "3", "]"}], "\[Equal]", 
     RowBox[{"-", 
      SqrtBox[
       SuperscriptBox[
        RowBox[{"x", "[", "2", "]"}], "2"]]}]}], "||", 
    RowBox[{
     RowBox[{"x", "[", "3", "]"}], "\[Equal]", 
     SqrtBox[
      SuperscriptBox[
       RowBox[{"x", "[", "2", "]"}], "2"]]}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.6278134609986315`*^9}]
}, Closed]],

Cell["The solution consists of two lines intersecting in the point", "Text",
 CellChangeTimes->{{3.5883368982808847`*^9, 3.588336928685338*^9}, {
   3.5883370207430906`*^9, 3.5883370393991575`*^9}, 3.615264421498866*^9}],

Cell[BoxData[
 RowBox[{"hypsf", "[", 
  RowBox[{"0", ",", "0"}], "]"}]], "Input",
 CellChangeTimes->{{3.588337052581912*^9, 3.5883370617754374`*^9}, 
   3.58833711234533*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"lines", " ", "=", " ", 
   RowBox[{"ParametricPlot3D", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "t", ",", "t"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "t", ",", 
         RowBox[{"-", "t"}]}], "}"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", 
       RowBox[{"-", "3"}], ",", "3"}], "}"}], ",", " ", 
     RowBox[{"PlotStyle", " ", "\[Rule]", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"Red", ",", "Thick"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"Red", ",", "Thick"}], "}"}]}], "}"}]}]}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.5883391787715225`*^9, 3.588339406986576*^9}, 
   3.588340016380431*^9, 3.5883400791880236`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"tpl", " ", "=", " ", 
   RowBox[{"ParametricPlot3D", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"u", "+", "v"}], ",", 
       RowBox[{"u", "-", "v"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"u", ",", 
       RowBox[{"-", "3"}], ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"v", ",", 
       RowBox[{"-", "3"}], ",", "3"}], "}"}], ",", " ", 
     RowBox[{"PlotStyle", "\[Rule]", " ", 
      RowBox[{"{", 
       RowBox[{"Green", ",", 
        RowBox[{"Opacity", "[", "0.3", "]"}]}], "}"}]}], ",", " ", 
     RowBox[{"Mesh", "\[Rule]", " ", "False"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5883395599233236`*^9, 3.588339574290145*^9}, {
   3.58833960514691*^9, 3.588339605156911*^9}, {3.588339682473333*^9, 
   3.588339886557006*^9}, 3.5883400673403463`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pt", " ", "=", 
   RowBox[{"Graphics3D", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"PointSize", "[", "0.015", "]"}], ",", 
      RowBox[{"Point", "[", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0", ",", "0"}], "}"}], "]"}]}], "}"}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.588340740746863*^9, 3.5883407765549107`*^9}, {
   3.5883408387954707`*^9, 3.5883408522202387`*^9}, {3.5883409048192472`*^9, 
   3.588341005627013*^9}, {3.5883410445882416`*^9, 3.5883410657394514`*^9}, 
   3.588341104810686*^9, 3.614861852280727*^9, 3.6149177842609477`*^9}],

Cell["\<\
The next figure shows the hyperboloid and the intersection with its tangent \
plane at the point pt:\
\>", "Text",
 CellChangeTimes->{{3.5883405593284864`*^9, 3.5883406616313376`*^9}, {
  3.5883411405207286`*^9, 3.588341154064503*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"hyp", ",", "lines", ",", "tpl", ",", "pt"}], "}"}], ",", " ", 
   RowBox[{"Axes", "\[Rule]", " ", "None"}], ",", 
   RowBox[{"Boxed", "\[Rule]", " ", "False"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5883394200973263`*^9, 3.588339474577442*^9}, {
  3.588339905786106*^9, 3.588339907934229*^9}, {3.588339937962946*^9, 
  3.588339947194474*^9}, {3.588341018018722*^9, 3.5883410194828053`*^9}}],

Cell["\<\
Through any point of hypsf  two lines belong to the surface, for example\
\>", "Text",
 CellChangeTimes->{{3.5883377675518055`*^9, 3.5883378334235735`*^9}, {
  3.5883390557674875`*^9, 3.588339089159397*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"h", "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"x", "[", "1", "]"}], "^", "2"}], " ", "+", " ", 
       RowBox[{
        RowBox[{"x", "[", "2", "]"}], "^", "2"}], " ", "-", 
       RowBox[{
        RowBox[{"x", "[", "3", "]"}], "^", "2"}]}], "\[Equal]", " ", "1"}]}], 
    "}"}], ",", " ", 
   RowBox[{"vec", "[", "x", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5883189765386076`*^9, 3.588319000498978*^9}, {
   3.5883190488517437`*^9, 3.5883191283982935`*^9}, {3.588319186566621*^9, 
   3.588319199897383*^9}, {3.588319458461172*^9, 3.58831946104532*^9}, {
   3.588319535468577*^9, 3.58831954670922*^9}, {3.588319614765112*^9, 
   3.588319619853403*^9}, {3.588336590819945*^9, 3.5883366074651737`*^9}, 
   3.5883368036823187`*^9, {3.5883371369567375`*^9, 3.588337149963482*^9}, {
   3.5896249832169914`*^9, 3.5896249944856358`*^9}, {3.58962515837101*^9, 
   3.5896251697996635`*^9}, 3.6148618522963266`*^9, 3.6148646598800583`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Solve", "::", "svars"}], "MessageName"], ":", 
  " ", "\<\"Equations may not give solutions for all \\\"solve\\\" variables. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/Solve/svars\\\", \
ButtonNote -> \\\"Solve::svars\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.6278135696178246`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"x", "[", "2", "]"}], "\[Rule]", 
      FractionBox[
       RowBox[{
        RowBox[{"2", " ", 
         RowBox[{"Cosh", "[", "v", "]"}]}], "-", 
        RowBox[{"2", " ", 
         RowBox[{"Cos", "[", "u", "]"}], " ", 
         RowBox[{"x", "[", "1", "]"}]}], "+", 
        RowBox[{"\[ImaginaryI]", " ", 
         RowBox[{"Cos", "[", 
          RowBox[{"u", "-", 
           RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}], " ", 
         RowBox[{"x", "[", "1", "]"}]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
         RowBox[{"Cos", "[", 
          RowBox[{"u", "+", 
           RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}], " ", 
         RowBox[{"x", "[", "1", "]"}]}]}], 
       RowBox[{
        RowBox[{"2", " ", 
         RowBox[{"Sin", "[", "u", "]"}]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
         RowBox[{"Sin", "[", 
          RowBox[{"u", "-", 
           RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}]}], "+", 
        RowBox[{"\[ImaginaryI]", " ", 
         RowBox[{"Sin", "[", 
          RowBox[{"u", "+", 
           RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}]}]}]]}], ",", 
     RowBox[{
      RowBox[{"x", "[", "3", "]"}], "\[Rule]", 
      FractionBox[
       RowBox[{
        RowBox[{"-", 
         RowBox[{"Cos", "[", "u", "]"}]}], "+", 
        RowBox[{
         FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
         RowBox[{"Cos", "[", 
          RowBox[{"u", "-", 
           RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}]}], "-", 
        RowBox[{
         FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
         RowBox[{"Cos", "[", 
          RowBox[{"u", "+", 
           RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}]}], "+", 
        RowBox[{
         RowBox[{"Cosh", "[", "v", "]"}], " ", 
         RowBox[{"x", "[", "1", "]"}]}]}], 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"Sin", "[", "u", "]"}]}], "+", 
        RowBox[{
         FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
         RowBox[{"Sin", "[", 
          RowBox[{"u", "-", 
           RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}]}], "-", 
        RowBox[{
         FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
         RowBox[{"Sin", "[", 
          RowBox[{"u", "+", 
           RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}]}]}]]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"x", "[", "2", "]"}], "\[Rule]", 
      FractionBox[
       RowBox[{
        RowBox[{"2", " ", 
         RowBox[{"Cosh", "[", "v", "]"}]}], "-", 
        RowBox[{"2", " ", 
         RowBox[{"Cos", "[", "u", "]"}], " ", 
         RowBox[{"x", "[", "1", "]"}]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
         RowBox[{"Cos", "[", 
          RowBox[{"u", "-", 
           RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}], " ", 
         RowBox[{"x", "[", "1", "]"}]}], "+", 
        RowBox[{"\[ImaginaryI]", " ", 
         RowBox[{"Cos", "[", 
          RowBox[{"u", "+", 
           RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}], " ", 
         RowBox[{"x", "[", "1", "]"}]}]}], 
       RowBox[{
        RowBox[{"2", " ", 
         RowBox[{"Sin", "[", "u", "]"}]}], "+", 
        RowBox[{"\[ImaginaryI]", " ", 
         RowBox[{"Sin", "[", 
          RowBox[{"u", "-", 
           RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
         RowBox[{"Sin", "[", 
          RowBox[{"u", "+", 
           RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}]}]}]]}], ",", 
     RowBox[{
      RowBox[{"x", "[", "3", "]"}], "\[Rule]", 
      FractionBox[
       RowBox[{
        RowBox[{"-", 
         RowBox[{"Cos", "[", "u", "]"}]}], "-", 
        RowBox[{
         FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
         RowBox[{"Cos", "[", 
          RowBox[{"u", "-", 
           RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}]}], "+", 
        RowBox[{
         FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
         RowBox[{"Cos", "[", 
          RowBox[{"u", "+", 
           RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}]}], "+", 
        RowBox[{
         RowBox[{"Cosh", "[", "v", "]"}], " ", 
         RowBox[{"x", "[", "1", "]"}]}]}], 
       RowBox[{
        RowBox[{"Sin", "[", "u", "]"}], "+", 
        RowBox[{
         FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
         RowBox[{"Sin", "[", 
          RowBox[{"u", "-", 
           RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}]}], "-", 
        RowBox[{
         FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
         RowBox[{"Sin", "[", 
          RowBox[{"u", "+", 
           RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}]}]}]]}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.6278135696178246`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", ">", "0"}], ",", 
     RowBox[{"v", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5883371576409206`*^9, 3.5883371965681477`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"x", "[", "2", "]"}], "\[Rule]", 
      FractionBox[
       RowBox[{
        RowBox[{"Cosh", "[", "v", "]"}], "-", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Cos", "[", "u", "]"}], "+", 
           RowBox[{
            RowBox[{"Sin", "[", "u", "]"}], " ", 
            RowBox[{"Sinh", "[", "v", "]"}]}]}], ")"}], " ", 
         RowBox[{"x", "[", "1", "]"}]}]}], 
       RowBox[{
        RowBox[{"Sin", "[", "u", "]"}], "-", 
        RowBox[{
         RowBox[{"Cos", "[", "u", "]"}], " ", 
         RowBox[{"Sinh", "[", "v", "]"}]}]}]]}], ",", 
     RowBox[{
      RowBox[{"x", "[", "3", "]"}], "\[Rule]", 
      FractionBox[
       RowBox[{
        RowBox[{"Cos", "[", "u", "]"}], "+", 
        RowBox[{
         RowBox[{"Sin", "[", "u", "]"}], " ", 
         RowBox[{"Sinh", "[", "v", "]"}]}], "-", 
        RowBox[{
         RowBox[{"Cosh", "[", "v", "]"}], " ", 
         RowBox[{"x", "[", "1", "]"}]}]}], 
       RowBox[{
        RowBox[{"Sin", "[", "u", "]"}], "-", 
        RowBox[{
         RowBox[{"Cos", "[", "u", "]"}], " ", 
         RowBox[{"Sinh", "[", "v", "]"}]}]}]]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"x", "[", "2", "]"}], "\[Rule]", 
      FractionBox[
       RowBox[{
        RowBox[{"Cosh", "[", "v", "]"}], "-", 
        RowBox[{
         RowBox[{"Cos", "[", "u", "]"}], " ", 
         RowBox[{"x", "[", "1", "]"}]}], "+", 
        RowBox[{
         RowBox[{"Sin", "[", "u", "]"}], " ", 
         RowBox[{"Sinh", "[", "v", "]"}], " ", 
         RowBox[{"x", "[", "1", "]"}]}]}], 
       RowBox[{
        RowBox[{"Sin", "[", "u", "]"}], "+", 
        RowBox[{
         RowBox[{"Cos", "[", "u", "]"}], " ", 
         RowBox[{"Sinh", "[", "v", "]"}]}]}]]}], ",", 
     RowBox[{
      RowBox[{"x", "[", "3", "]"}], "\[Rule]", 
      FractionBox[
       RowBox[{
        RowBox[{"-", 
         RowBox[{"Cos", "[", "u", "]"}]}], "+", 
        RowBox[{
         RowBox[{"Sin", "[", "u", "]"}], " ", 
         RowBox[{"Sinh", "[", "v", "]"}]}], "+", 
        RowBox[{
         RowBox[{"Cosh", "[", "v", "]"}], " ", 
         RowBox[{"x", "[", "1", "]"}]}]}], 
       RowBox[{
        RowBox[{"Sin", "[", "u", "]"}], "+", 
        RowBox[{
         RowBox[{"Cos", "[", "u", "]"}], " ", 
         RowBox[{"Sinh", "[", "v", "]"}]}]}]]}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627813579104642*^9}]
}, Closed]],

Cell[TextData[{
 "Surfaces being ruled in a twofold way are said to be ",
 StyleBox["doubly ruled",
  FontColor->RGBColor[0, 0, 1]],
 ", see ",
 ButtonBox["MathWorld",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://mathworld.wolfram.com/DoublyRuledSurface.html"], None},
  ButtonNote->"http://mathworld.wolfram.com/DoublyRuledSurface.html"],
 ". The spherical image of the surface hypsf is given by its normal vector \
nhyp[u,v] defined above. We have"
}], "Text",
 CellChangeTimes->{{3.588337859712077*^9, 3.588337952823403*^9}, {
   3.588338081911786*^9, 3.588338171823929*^9}, 3.588338215392421*^9, 
   3.5896254035730343`*^9, {3.589625489598955*^9, 3.589625585046414*^9}, {
   3.609822804716297*^9, 3.609822809275558*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "nhyp", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5896256072246823`*^9, 3.5896256187763433`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{
        RowBox[{"Cosh", "[", "v", "]"}], " ", 
        RowBox[{"Sin", "[", "u", "]"}]}], 
       SqrtBox[
        RowBox[{"Cosh", "[", 
         RowBox[{"2", " ", "v"}], "]"}]]]}], ",", 
     RowBox[{
      FractionBox[
       RowBox[{
        RowBox[{"Cos", "[", "u", "]"}], " ", 
        RowBox[{"Sinh", "[", "v", "]"}]}], 
       SqrtBox[
        RowBox[{"Cosh", "[", 
         RowBox[{"2", " ", "v"}], "]"}]]], "-", 
      FractionBox[
       RowBox[{
        RowBox[{"Cos", "[", "u", "]"}], " ", 
        RowBox[{"Cosh", "[", "v", "]"}], " ", 
        RowBox[{"Sinh", "[", 
         RowBox[{"2", " ", "v"}], "]"}]}], 
       SuperscriptBox[
        RowBox[{"Cosh", "[", 
         RowBox[{"2", " ", "v"}], "]"}], 
        RowBox[{"3", "/", "2"}]]]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"Cos", "[", "u", "]"}], " ", 
       RowBox[{"Cosh", "[", "v", "]"}]}], 
      SqrtBox[
       RowBox[{"Cosh", "[", 
        RowBox[{"2", " ", "v"}], "]"}]]], ",", 
     RowBox[{
      FractionBox[
       RowBox[{
        RowBox[{"Sin", "[", "u", "]"}], " ", 
        RowBox[{"Sinh", "[", "v", "]"}]}], 
       SqrtBox[
        RowBox[{"Cosh", "[", 
         RowBox[{"2", " ", "v"}], "]"}]]], "-", 
      FractionBox[
       RowBox[{
        RowBox[{"Cosh", "[", "v", "]"}], " ", 
        RowBox[{"Sin", "[", "u", "]"}], " ", 
        RowBox[{"Sinh", "[", 
         RowBox[{"2", " ", "v"}], "]"}]}], 
       SuperscriptBox[
        RowBox[{"Cosh", "[", 
         RowBox[{"2", " ", "v"}], "]"}], 
        RowBox[{"3", "/", "2"}]]]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{"Cosh", "[", "v", "]"}], 
        SqrtBox[
         RowBox[{"Cosh", "[", 
          RowBox[{"2", " ", "v"}], "]"}]]]}], "+", 
      FractionBox[
       RowBox[{
        RowBox[{"Sinh", "[", "v", "]"}], " ", 
        RowBox[{"Sinh", "[", 
         RowBox[{"2", " ", "v"}], "]"}]}], 
       SuperscriptBox[
        RowBox[{"Cosh", "[", 
         RowBox[{"2", " ", "v"}], "]"}], 
        RowBox[{"3", "/", "2"}]]]}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.62781360642629*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"u", "\[Rule]", " ", 
     RowBox[{"Random", "[", "]"}]}], ",", 
    RowBox[{"v", "\[Rule]", " ", 
     RowBox[{"Random", "[", "]"}]}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.5896256589816427`*^9, 3.58962570072103*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.5934982587725776`"}], ",", 
     RowBox[{"-", "0.009026126434595036`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.8047570884925795`", ",", 
     RowBox[{"-", "0.006656655031678769`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{"-", "0.9996854004706323`"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6278136114182987`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MatrixRank", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.5896257130727367`*^9, 3.5896257222002587`*^9}}],

Cell[BoxData["2"], "Output",
 CellChangeTimes->{3.627813615692706*^9}]
}, Closed]],

Cell["\<\
Indeed, the spherical image of the surface covers a whole open region of the \
sphere. Therefore the surface may not be developable; remember that the \
spherical image of a developable surface is a curve or a point only.\
\>", "Text",
 CellChangeTimes->{{3.5896257957964683`*^9, 3.589625951590379*^9}, 
   3.6151962208821464`*^9}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"nhyp", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", " ", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "7"}], ",", "7"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5896259599918594`*^9, 3.589626106817258*^9}, {
  3.5896265303224807`*^9, 3.589626550265621*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Limit", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"nhyp", "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{"[", "3", "]"}], "]"}], ",", " ", 
   RowBox[{"v", "\[Rule]", " ", 
    RowBox[{"-", "Infinity"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5896261379230366`*^9, 3.589626182777602*^9}, 
   3.589626268130484*^9, {3.615196311548332*^9, 3.6151963255081306`*^9}}],

Cell[BoxData[
 FractionBox["1", 
  SqrtBox["2"]]], "Output",
 CellChangeTimes->{3.627813659224783*^9}]
}, Closed]],

Cell[TextData[{
 "Thus the spherical image of the whole hyperboloid hypsf consists of the \
points of the unit sphere whose third coordinate  satisfies -",
 Cell[BoxData[
  FractionBox["1", 
   SqrtBox["2"]]],
  CellChangeTimes->{{3.58962617067691*^9, 3.5896261837426577`*^9}, 
    3.5896262696565714`*^9}],
 "< x[3] < ",
 Cell[BoxData[
  FractionBox["1", 
   SqrtBox["2"]]],
  CellChangeTimes->{{3.58962617067691*^9, 3.5896261837426577`*^9}, 
    3.5896262696565714`*^9}],
 "."
}], "Text",
 CellChangeTimes->{{3.58962632295862*^9, 3.589626485630924*^9}}],

Cell[TextData[{
 "Another doubly ruled surface is the ",
 StyleBox["hyperbolic paraboloid",
  FontColor->RGBColor[0, 0, 1]],
 ", see ",
 ButtonBox["MathWorld",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://mathworld.wolfram.com/HyperbolicParaboloid.html"], None},
  ButtonNote->"http://mathworld.wolfram.com/HyperbolicParaboloid.html"],
 "."
}], "Text",
 CellChangeTimes->{{3.588337859712077*^9, 3.588337952823403*^9}, {
   3.588338081911786*^9, 3.588338171823929*^9}, 3.588338215392421*^9, {
   3.5896254035730343`*^9, 3.589625428646468*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "hyperbolicparaboloid"}]], "Input",
 CellChangeTimes->{3.5883384595453854`*^9}],

Cell[BoxData[
 StyleBox["\<\"{u,v}->hyperbolicparaboloid[u,v] is a parametrization of a \
hyperbolic paraboloid.  To plot try hyperbolicparaboloid[-1,1,-1,1].\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.6278136777616158`*^9},
 CellTags->"Info3627817277-6530549"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hyperbolicparaboloid", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.588338474241226*^9, 3.5883384780094414`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"u", ",", "v", ",", 
   RowBox[{"u", " ", "v"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6278136821296234`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"hyperbolicparaboloid", "[", 
  RowBox[{
   RowBox[{"-", "1"}], ",", "1", ",", 
   RowBox[{"-", "1"}], ",", "1"}], "]"}]], "Input"],

Cell["\<\
As an exercise find  the line families contained in this surface, calculate \
its first fundamental form and plot its spherical image.\
\>", "Text",
 CellChangeTimes->{{3.5896266873664627`*^9, 3.589626792142456*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["2.2. The Gaussian Curvature", "Subsection",
 CellChangeTimes->{{3.5872730558623724`*^9, 3.58727307645455*^9}, {
   3.5883415613617992`*^9, 3.5883415766636744`*^9}, {3.5883416340959597`*^9, 
   3.5883416520479865`*^9}, 3.6151964340823407`*^9, 3.6152119753114877`*^9, 
   3.6179516333704367`*^9, 3.61795637897787*^9}],

Cell[CellGroupData[{

Cell["\<\
2.2.1. Integral Curvature and Curvature Integral. Orientation\
\>", "Subsubsection",
 CellChangeTimes->{{3.589779006964474*^9, 3.5897790435933385`*^9}, {
   3.5897790965554314`*^9, 3.589779110049455*^9}, 3.609823738997735*^9, 
   3.6098312806720943`*^9, 3.6278139692135315`*^9, 3.62789661193079*^9}],

Cell["Here is the unit sphere:", "Text",
 CellChangeTimes->{{3.588487873500686*^9, 3.5884878829387026`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sph1", " ", "=", " ", 
   RowBox[{"ParametricPlot3D", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"sphero", "[", "1", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"u", ",", "0", ",", 
       RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"v", ",", 
       RowBox[{
        RowBox[{"-", "Pi"}], "/", "2"}], ",", 
       RowBox[{"Pi", "/", "2"}]}], "}"}], ",", 
     RowBox[{"PlotStyle", " ", "\[Rule]", " ", 
      RowBox[{"Opacity", "[", ".4", "]"}]}], ",", " ", 
     RowBox[{"Mesh", "\[Rule]", " ", "False"}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5884876724319334`*^9, 3.588487820210993*^9}, 
   3.5896342644259577`*^9}],

Cell["The spherical image of the part hyp of the hyperboloid is", "Text",
 CellChangeTimes->{{3.588393319929459*^9, 3.5883933833435707`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"nhyp", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "2"}], ",", "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5883416562532268`*^9, 3.588341656255227*^9}, {
   3.5883416876340218`*^9, 3.588341775107025*^9}, 3.5883418201235995`*^9, 
   3.588392652560287*^9, {3.5883932721777754`*^9, 3.588393273675378*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"sph1", ",", " ", "%"}], "}"}], ",", " ", 
   RowBox[{"Axes", " ", "\[Rule]", " ", "None"}], ",", " ", 
   RowBox[{"Boxed", " ", "\[Rule]", " ", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.588487907992347*^9, 3.588487947772417*^9}}],

Cell[TextData[{
 "The area of the spherical image of a surface region is a measure  for the \
curvature of the surface in this region; it is called its",
 StyleBox[" integral curvature",
  FontColor->RGBColor[0, 0, 1]],
 ". For developable surfaces the integral curvature is zero since the \
spherical image is a point (in case of a plane) or a curve. "
}], "Text",
 CellChangeTimes->{{3.5883934642453127`*^9, 3.588393512823798*^9}, {
   3.5883935504978642`*^9, 3.5883936377488174`*^9}, {3.588393710195345*^9, 
   3.588393869144024*^9}, 3.5883939316533337`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "Integrate"}]], "Input",
 CellChangeTimes->{{3.5883419502820444`*^9, 3.5883419756504955`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox["\<\"\\!\\(\\*RowBox[{\\\"Integrate\\\", \\\"[\\\", \
RowBox[{StyleBox[\\\"f\\\", \\\"TI\\\"], \\\",\\\", StyleBox[\\\"x\\\", \
\\\"TI\\\"]}], \\\"]\\\"}]\\) gives the indefinite integral \\!\\(\\*RowBox[{\
\\\"\[Integral]\\\", StyleBox[\\\"f\\\", \\\"TI\\\"], \\\" \\\", \
StyleBox[\\\"d\\\", \\\"TI\\\"], StyleBox[\\\"x\\\", \\\"TI\\\"]}]\\). \\n\\!\
\\(\\*RowBox[{\\\"Integrate\\\", \\\"[\\\", RowBox[{StyleBox[\\\"f\\\", \
\\\"TI\\\"], \\\",\\\", RowBox[{\\\"{\\\", RowBox[{StyleBox[\\\"x\\\", \\\"TI\
\\\"], \\\",\\\", SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], \
StyleBox[\\\"min\\\", \\\"TI\\\"]], \\\",\\\", \
SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], StyleBox[\\\"max\\\", \
\\\"TI\\\"]]}], \\\"}\\\"}]}], \\\"]\\\"}]\\) gives the definite integral \\!\
\\(\\*RowBox[{SubsuperscriptBox[\\\"\[Integral]\\\", SubscriptBox[StyleBox[\\\
\"x\\\", \\\"TI\\\"], StyleBox[\\\"min\\\", \\\"TI\\\"]], \
SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], StyleBox[\\\"max\\\", \
\\\"TI\\\"]]], \\\" \\\", StyleBox[\\\"f\\\", \\\"TI\\\"], \\\" \\\", \
StyleBox[\\\"d\\\", \\\"TI\\\"], StyleBox[\\\"x\\\", \\\"TI\\\"]}]\\). \\n\\!\
\\(\\*RowBox[{\\\"Integrate\\\", \\\"[\\\", RowBox[{StyleBox[\\\"f\\\", \
\\\"TI\\\"], \\\",\\\", RowBox[{\\\"{\\\", RowBox[{StyleBox[\\\"x\\\", \\\"TI\
\\\"], \\\",\\\", SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], \
StyleBox[\\\"min\\\", \\\"TI\\\"]], \\\",\\\", \
SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], StyleBox[\\\"max\\\", \
\\\"TI\\\"]]}], \\\"}\\\"}], \\\",\\\", RowBox[{\\\"{\\\", \
RowBox[{StyleBox[\\\"y\\\", \\\"TI\\\"], \\\",\\\", \
SubscriptBox[StyleBox[\\\"y\\\", \\\"TI\\\"], StyleBox[\\\"min\\\", \
\\\"TI\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"y\\\", \\\"TI\\\"], \
StyleBox[\\\"max\\\", \\\"TI\\\"]]}], \\\"}\\\"}], \\\",\\\", StyleBox[\\\"\
\[Ellipsis]\\\", \\\"TR\\\"]}], \\\"]\\\"}]\\) gives the multiple integral \
\\!\\(\\*RowBox[{SubsuperscriptBox[\\\"\[Integral]\\\", \
SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], StyleBox[\\\"min\\\", \
\\\"TI\\\"]], SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], \
StyleBox[\\\"max\\\", \\\"TI\\\"]]], RowBox[{StyleBox[\\\"d\\\", \\\"TI\\\"], \
StyleBox[\\\"x\\\", \\\"TI\\\"], \
RowBox[{SubsuperscriptBox[\\\"\[Integral]\\\", \
SubscriptBox[StyleBox[\\\"y\\\", \\\"TI\\\"], StyleBox[\\\"min\\\", \
\\\"TI\\\"]], SubscriptBox[StyleBox[\\\"y\\\", \\\"TI\\\"], \
StyleBox[\\\"max\\\", \\\"TI\\\"]]], RowBox[{StyleBox[\\\"d\\\", \\\"TI\\\"], \
\\\"\[InvisibleSpace]\\\", StyleBox[\\\"y\\\", \\\"TI\\\"], \\\" \\\", \\\"\
\[Ellipsis]\\\", \\\" \\\", StyleBox[\\\"f\\\", \\\"TI\\\"]}]}]}]}]\\). \
\\n\\!\\(\\*RowBox[{\\\"Integrate\\\", \\\"[\\\", RowBox[{StyleBox[\\\"f\\\", \
\\\"TI\\\"], \\\",\\\", RowBox[{RowBox[{\\\"{\\\", \
RowBox[{StyleBox[\\\"x\\\", \\\"TI\\\"], \\\",\\\", StyleBox[\\\"y\\\", \
\\\"TI\\\"], \\\",\\\", StyleBox[\\\"\[Ellipsis]\\\", \\\"TR\\\"]}], \
\\\"}\\\"}], \\\"\[Element]\\\", StyleBox[\\\"reg\\\", \\\"TI\\\"]}]}], \\\"]\
\\\"}]\\) integrates over the geometric region \\!\\(\\*StyleBox[\\\"reg\\\", \
\\\"TI\\\"]\\).\"\>", "MSG"], "\[NonBreakingSpace]", 
  ButtonBox[
   StyleBox["\[RightSkeleton]", "SR"],
   Active->True,
   BaseStyle->"Link",
   ButtonData->"paclet:ref/Integrate"]}]], "Print", "PrintUsage",
 CellChangeTimes->{3.6278270156706204`*^9},
 CellTags->"Info3627830615-6530549"]
}, Closed]],

Cell["\<\
The integration of the area element of the spherical image nhyp[u,v] yields \
for the region hyp:\
\>", "Text",
 CellChangeTimes->{{3.588394687349861*^9, 3.5883947193611174`*^9}, {
  3.588394751699974*^9, 3.588394762619993*^9}, {3.588394847452942*^9, 
  3.588394860198165*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"dA", "[", "nhyp", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "2"}], ",", "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.588342032832766*^9, 3.5883420805934973`*^9}, {
   3.5883422997950354`*^9, 3.588342314618883*^9}, 3.588394079635194*^9, {
   3.5883947964096527`*^9, 3.588394797220854*^9}, 3.614861852327527*^9, 
   3.614865446121439*^9}],

Cell[BoxData[
 RowBox[{"4", " ", "\[Pi]", " ", 
  SqrtBox[
   RowBox[{"Sech", "[", "4", "]"}]], " ", 
  RowBox[{"Sinh", "[", "2", "]"}]}]], "Output",
 CellChangeTimes->{3.627827046564675*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.588342444491311*^9, 3.5883424563769913`*^9}}],

Cell[BoxData["8.721554642121092`"], "Output",
 CellChangeTimes->{3.627827054339489*^9}]
}, Closed]],

Cell["The area of the spherical image of the whole hyperboloid is", "Text",
 CellChangeTimes->{{3.588394955373932*^9, 3.5883949821591787`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Limit", "[", 
  RowBox[{
   RowBox[{"Integrate", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"dA", "[", "nhyp", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"u", ",", "0", ",", 
       RowBox[{"2", "*", "Pi"}]}], "}"}], ",", " ", 
     RowBox[{"{", 
      RowBox[{"v", ",", 
       RowBox[{"-", "x"}], ",", "x"}], "}"}]}], "]"}], ",", " ", 
   RowBox[{"x", "\[Rule]", " ", "Infinity"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5883426748503556`*^9, 3.5883427271728477`*^9}, 
   3.5883940826303988`*^9, {3.5883948978722305`*^9, 3.588394899385433*^9}, 
   3.614861852327527*^9, 3.614865446137039*^9}],

Cell[BoxData[
 RowBox[{"2", " ", 
  SqrtBox["2"], " ", "\[Pi]"}]], "Output",
 CellChangeTimes->{3.627827065243908*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.5883427886993556`*^9, 3.588342798730173*^9}}],

Cell[BoxData["8.885765876316732`"], "Output",
 CellChangeTimes->{3.627827074385524*^9}]
}, Closed]],

Cell["\<\
and this is less than the area 4\[DoubledPi] of the whole unit sphere.  \
Indeed, the spherical image is the region on the unit sphere between the two \
limit circles\
\>", "Text",
 CellChangeTimes->{{3.588395009084826*^9, 3.588395145163865*^9}, {
  3.588395195895154*^9, 3.5883951978139577`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Limit", "[", 
  RowBox[{
   RowBox[{"nhyp", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
   RowBox[{"v", "\[Rule]", " ", "Infinity"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.588342841177848*^9, 3.5883428656074905`*^9}, 
   3.58839265435429*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{"Cos", "[", "u", "]"}], 
    SqrtBox["2"]], ",", 
   FractionBox[
    RowBox[{"Sin", "[", "u", "]"}], 
    SqrtBox["2"]], ",", 
   RowBox[{"-", 
    FractionBox["1", 
     SqrtBox["2"]]}]}], "}"}]], "Output",
 CellChangeTimes->{3.627827092846357*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Limit", "[", 
  RowBox[{
   RowBox[{"nhyp", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
   RowBox[{"v", "\[Rule]", " ", 
    RowBox[{"-", "Infinity"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.588342841177848*^9, 3.5883428656074905`*^9}, 
   3.58839265435429*^9, {3.588395162370695*^9, 3.5883951674563046`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{"Cos", "[", "u", "]"}], 
    SqrtBox["2"]], ",", 
   FractionBox[
    RowBox[{"Sin", "[", "u", "]"}], 
    SqrtBox["2"]], ",", 
   FractionBox["1", 
    SqrtBox["2"]]}], "}"}]], "Output",
 CellChangeTimes->{3.6278270991643677`*^9}]
}, Closed]],

Cell["\<\
Since the area element of any surface by its definition is always  positive, \
the integral curvature is always non negative. For a finer consideration of \
the curvature the orientation of the spherical image has to be considered, \
and a finer integration on the surface must be introduced, namely the \
integration of alternating biforms.
As has been shown at the end  of subsection 1.4.1 we have the equation 
dA[sf][u,v] ==Det[{\[Sigma][1][sf][u,v], \[Sigma][2][sf][u,v]}]], and this is \
the strict coordinate of the biform 
\[Sigma][1][sf][u,v]\[And]\[Sigma][2][sf][u,v]. The left hand side of this \
equation is always positive, whereas the sign of the right side depends on \
the signs of the basic vectors and on their order. The ordered cobasis 
{\[Sigma][1][sf][u,v], \[Sigma][2][sf][u,v]}], or equivalently the ordered \
basis  {amf[1][sf][u,v], amf[2][sf][u,v]}] represents the orientation of the \
tangent planes; in the natural order this orientation is considered to be \
positive or counter clockwise. \
\>", "Text",
 CellChangeTimes->{{3.5883956383758087`*^9, 3.5883958423764772`*^9}, {
   3.588395877112464*^9, 3.5883959244321704`*^9}, {3.5884149596021376`*^9, 
   3.5884151284880342`*^9}, {3.5884152076581736`*^9, 
   3.5884152696526823`*^9}, {3.588415339525205*^9, 3.5884153419276094`*^9}, {
   3.5884154774918475`*^9, 3.588415512420309*^9}, {3.588415721613336*^9, 
   3.588415786285035*^9}, {3.588415925629005*^9, 3.5884159356215763`*^9}, {
   3.58841620247784*^9, 3.5884162121253915`*^9}, {3.5884162594771*^9, 
   3.588416445276727*^9}, {3.5884236681491313`*^9, 3.5884236686571603`*^9}, {
   3.588488086924661*^9, 3.5884880975326796`*^9}, {3.5884882789641457`*^9, 
   3.588488288460689*^9}, {3.58848868499737*^9, 3.5884886898446465`*^9}, 
   3.609823956908199*^9, 3.6098240272362213`*^9, {3.614861852405527*^9, 
   3.614861852483527*^9}, {3.6149177843389482`*^9, 3.6149177844169483`*^9}, 
   3.6152107773853836`*^9, {3.615210839067892*^9, 3.615210881359566*^9}, 
   3.6152644556048174`*^9, {3.6278271307136235`*^9, 3.627827140479241*^9}}],

Cell["\<\
Consider the sequence of colors (Red, Green, Blue, Black) and the sequence of \
points in the parameter neighborhood of the point {0,0}:\
\>", "Text",
 CellChangeTimes->{{3.5884285649172106`*^9, 3.5884286217354603`*^9}, {
  3.5884286562264333`*^9, 3.5884287192820396`*^9}, {3.588488460204512*^9, 
  3.5884884633086896`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{".3", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", ".3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", ".3"}], ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{"-", ".3"}]}], "}"}]}], "}"}]], "Input",
 CellChangeTimes->{{3.5884287354409637`*^9, 3.5884288748109355`*^9}, {
  3.589634138656764*^9, 3.589634151975526*^9}}],

Cell["\<\
Running on the hyperboloid through these points in the given order is a \
positive or counter clockwise turn:\
\>", "Text",
 CellChangeTimes->{{3.588428907825824*^9, 3.5884289947457952`*^9}, {
  3.588488794187615*^9, 3.5884888100045195`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pt1", " ", "=", 
   RowBox[{"Graphics3D", "[", 
    RowBox[{"{", 
     RowBox[{"Red", ",", 
      RowBox[{"PointSize", "[", "0.02", "]"}], ",", 
      RowBox[{"Point", "[", 
       RowBox[{"hypsf", "[", 
        RowBox[{".3", ",", "0"}], "]"}], "]"}]}], "}"}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.5884290046673627`*^9, 3.5884291367649183`*^9}, 
   3.5884295120143814`*^9, 3.5884883695753284`*^9, 3.6148618524991274`*^9, 
   3.6149177844169483`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pt2", " ", "=", 
   RowBox[{"Graphics3D", "[", 
    RowBox[{"{", 
     RowBox[{"Green", ",", 
      RowBox[{"PointSize", "[", "0.02", "]"}], ",", 
      RowBox[{"Point", "[", 
       RowBox[{"hypsf", "[", 
        RowBox[{"0", ",", ".3"}], "]"}], "]"}]}], "}"}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.5884290046673627`*^9, 3.58842918871689*^9}, 
   3.5884295183647447`*^9, 3.5884883917355957`*^9, 3.614861852514727*^9, 
   3.614917784432548*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pt3", "=", 
   RowBox[{"Graphics3D", "[", 
    RowBox[{"{", 
     RowBox[{"Blue", ",", 
      RowBox[{"PointSize", "[", "0.02", "]"}], ",", 
      RowBox[{"Point", "[", 
       RowBox[{"hypsf", "[", 
        RowBox[{
         RowBox[{"-", ".3"}], ",", "0"}], "]"}], "]"}]}], "}"}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.5884290046673627`*^9, 3.5884292509564495`*^9}, 
   3.5884295237650537`*^9, 3.588488417024042*^9, 3.614861852514727*^9, 
   3.614917784432548*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pt4", "=", 
   RowBox[{"Graphics3D", "[", 
    RowBox[{"{", 
     RowBox[{"Black", ",", 
      RowBox[{"PointSize", "[", "0.02", "]"}], ",", 
      RowBox[{"Point", "[", 
       RowBox[{"hypsf", "[", 
        RowBox[{"0", ",", 
         RowBox[{"-", ".3"}]}], "]"}], "]"}]}], "}"}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.5884290046673627`*^9, 3.5884293145250854`*^9}, 
   3.5884295299654083`*^9, 3.588488430007785*^9, 3.614861852514727*^9, 
   3.614917784448148*^9}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"hyp", ",", "pt1", ",", "pt2", ",", "pt3", ",", "pt4"}], "}"}], 
   ",", 
   RowBox[{"Axes", "\[Rule]", "None"}], ",", 
   RowBox[{"Boxed", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.58842939432465*^9, 3.5884294656937323`*^9}}],

Cell["Now we consider the spherical images of these points:", "Text",
 CellChangeTimes->{{3.588488839700218*^9, 3.5884888813005977`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"arr1", "=", 
   RowBox[{"Graphics3D", "[", 
    RowBox[{"{", 
     RowBox[{"Red", ",", 
      RowBox[{"Arrow", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
         RowBox[{"nhyp", "[", 
          RowBox[{"0.3", ",", "0"}], "]"}]}], "}"}], "]"}]}], "}"}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.588427392477151*^9, 3.5884274494664106`*^9}, {
   3.5884275184193544`*^9, 3.5884275979879055`*^9}, {3.588427688125061*^9, 
   3.5884276984686527`*^9}, {3.588427772996915*^9, 3.58842777378096*^9}, {
   3.5884280016129913`*^9, 3.5884280146927395`*^9}, {3.5884280699088974`*^9, 
   3.5884280748041773`*^9}, 3.588429567725568*^9, 3.588488547303494*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"arr2", "=", 
   RowBox[{"Graphics3D", "[", 
    RowBox[{"{", 
     RowBox[{"Green", ",", 
      RowBox[{"Arrow", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
         RowBox[{"nhyp", "[", 
          RowBox[{"0", ",", "0.3"}], "]"}]}], "}"}], "]"}]}], "}"}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.588427392477151*^9, 3.5884274494664106`*^9}, {
   3.5884275184193544`*^9, 3.5884275979879055`*^9}, {3.588427688125061*^9, 
   3.5884276984686527`*^9}, {3.588427772996915*^9, 3.58842777378096*^9}, {
   3.5884280016129913`*^9, 3.5884280146927395`*^9}, {3.5884280699088974`*^9, 
   3.588428119556737*^9}, 3.588429575366005*^9, 3.5884885668076096`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"arr3", "=", 
   RowBox[{"Graphics3D", "[", 
    RowBox[{"{", 
     RowBox[{"Blue", ",", 
      RowBox[{"Arrow", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
         RowBox[{"nhyp", "[", 
          RowBox[{
           RowBox[{"-", ".3"}], ",", "0"}], "]"}]}], "}"}], "]"}]}], "}"}], 
    "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.588427392477151*^9, 3.5884274494664106`*^9}, {
   3.5884275184193544`*^9, 3.5884275979879055`*^9}, {3.588427688125061*^9, 
   3.5884276984686527`*^9}, {3.588427772996915*^9, 3.58842777378096*^9}, {
   3.5884280016129913`*^9, 3.5884280146927395`*^9}, {3.5884280699088974`*^9, 
   3.5884281805252247`*^9}, {3.5884282118210144`*^9, 3.588428289323447*^9}, 
   3.588429582853433*^9, 3.5884885870717688`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"arr4", "=", 
   RowBox[{"Graphics3D", "[", 
    RowBox[{"{", 
     RowBox[{"Black", ",", 
      RowBox[{"Arrow", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
         RowBox[{"nhyp", "[", 
          RowBox[{"0", ",", 
           RowBox[{"-", ".3"}]}], "]"}]}], "}"}], "]"}]}], "}"}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.588427392477151*^9, 3.5884274494664106`*^9}, {
   3.5884275184193544`*^9, 3.5884275979879055`*^9}, {3.588427688125061*^9, 
   3.5884276984686527`*^9}, {3.588427772996915*^9, 3.58842777378096*^9}, {
   3.5884280016129913`*^9, 3.5884280146927395`*^9}, {3.5884280699088974`*^9, 
   3.5884281805252247`*^9}, {3.5884282118210144`*^9, 3.5884283493828826`*^9}, 
   3.588429589597819*^9, 3.588488601991622*^9}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"sph1", ",", "arr1", ",", "arr2", ",", "arr3", ",", "arr4"}], 
    "}"}], ",", " ", 
   RowBox[{"Axes", "\[Rule]", " ", "None"}], ",", " ", 
   RowBox[{"Boxed", " ", "\[Rule]", " ", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5884283602025013`*^9, 3.588428386155986*^9}, {
  3.5884886535505705`*^9, 3.5884886580278273`*^9}, {3.5884889416150475`*^9, 
  3.588488975566989*^9}}],

Cell["\<\
The order of the colors  (Red, Green, Blue, Black)  shows that the sequence \
of the corresponding arrows induce the clockwise, i. e. negative, orientation \
on the unit sphere:  in case of the hyperboloid the Gauss map inverses the \
natural orientations. The reason is that the curvature of the hyperboloid is \
negative, as we shall see now.
The space of alternating biforms on a two-dimensional vector space is \
one-dimensional; the strict coordinate \[CapitalOmega][[1,2]] of the biform \
\[CapitalOmega] serves as a Cartesian coordinate. The integral of the biform \
\[CapitalOmega] is defined to be\
\>", "Text",
 CellChangeTimes->{{3.5884893334124565`*^9, 3.5884893557247334`*^9}, {
  3.588489422220536*^9, 3.5884897365085125`*^9}, {3.588489801484229*^9, 
  3.588489872100268*^9}, {3.5884899097884235`*^9, 3.588489962276426*^9}, {
  3.5884900049308653`*^9, 3.588490046281231*^9}, {3.588490080052162*^9, 
  3.5884900844844155`*^9}, {3.5884901378524685`*^9, 3.5884902140368257`*^9}, {
  3.5884995387898827`*^9, 3.588499554920311*^9}, {3.615211108371165*^9, 
  3.615211113316374*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Integral]", "\[CapitalOmega]"}], " ", ":=", " ", 
  RowBox[{"\[Integral]", 
   RowBox[{
    RowBox[{"\[CapitalOmega]", "[", 
     RowBox[{"[", 
      RowBox[{"1", ",", "2"}], "]"}], "]"}], " ", 
    RowBox[{
     RowBox[{"\[Sigma]", "[", "1", "]"}], "\[Wedge]", 
     RowBox[{"\[Sigma]", "[", "2", "]"}]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{3.588500380185361*^9}],

Cell["\<\
As in any 1-dimensional vector spaces the ratio of two elements can be \
defined as the ratio of its coordinates with respect to an arbitrary basis, \
e.g.  we have .\
\>", "Text",
 CellChangeTimes->{
  3.5884998191067753`*^9, {3.5884998502368298`*^9, 3.5884999423861914`*^9}, {
   3.588500051430383*^9, 3.588500125826914*^9}, 3.6098243018919306`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[CapitalOmega]", " ", "\[Equal]", " ", 
   RowBox[{
    RowBox[{"\[CapitalOmega]", "[", 
     RowBox[{"[", 
      RowBox[{"1", ",", "2"}], "]"}], "]"}], " ", 
    RowBox[{
     RowBox[{"\[Sigma]", "[", "1", "]"}], "\[Wedge]", 
     RowBox[{"\[Sigma]", "[", "2", "]"}]}]}]}], ",", " ", 
  RowBox[{
   RowBox[{"or", "  ", 
    RowBox[{"\[CapitalOmega]", "[", 
     RowBox[{"[", 
      RowBox[{"1", ",", "2"}], "]"}], "]"}]}], " ", "\[Equal]", " ", 
   RowBox[{
    RowBox[{"\[CapitalOmega]", "/", 
     RowBox[{"\[Sigma]", "[", "1", "]"}]}], "\[Wedge]", 
    RowBox[{"\[Sigma]", "[", "2", "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.5885001751074004`*^9, 3.5885002704079676`*^9}, 
   3.5885003734305487`*^9}],

Cell["\<\
Since dA = \[Sigma][1]\[Wedge]\[Sigma][2]   the integral can be written as a \
surface integral over the ratio, which is a function on the surface\
\>", "Text",
 CellChangeTimes->{{3.5885013705531*^9, 3.5885014825768967`*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[Integral]", "\[CapitalOmega]"}], "=", 
   RowBox[{
    RowBox[{"\[Integral]", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"\[CapitalOmega]", "/", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"\[Sigma]", "[", "1", "]"}], "\[Wedge]", 
          RowBox[{"\[Sigma]", "[", "2", "]"}]}], ")"}]}], ")"}], 
      RowBox[{
       RowBox[{"\[Sigma]", "[", "1", "]"}], "\[Wedge]", 
       RowBox[{"\[Sigma]", "[", "2", "]"}]}]}]}], "  ", "=", " ", 
    RowBox[{"\[Integral]", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"\[CapitalOmega]", "/", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"\[Sigma]", "[", "1", "]"}], "\[Wedge]", 
          RowBox[{"\[Sigma]", "[", "2", "]"}]}], ")"}]}], ")"}], "dA"}]}]}]}],
   TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{
  3.5885011117954454`*^9, {3.588501145444705*^9, 3.5885011624331346`*^9}, {
   3.588501272460128*^9, 3.588501317419407*^9}}],

Cell[TextData[{
 "Now we apply this to the oriented area element of the spherical image \
amf[3][sf][u,v]. By the third derivation equation this area element is   \
\[CapitalOmega] = \[Omega][1] ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Wedge]", " "}], TraditionalForm]]],
 "\[Omega][2]. The ratio"
}], "Text",
 CellChangeTimes->{{3.5885016075799165`*^9, 3.5885016542863984`*^9}, {
   3.588501705220488*^9, 3.5885019030600357`*^9}, {3.5896350200141745`*^9, 
   3.589635126429261*^9}, 3.6148618526083274`*^9, 3.6149177845261483`*^9, {
   3.615211292326688*^9, 3.615211294042691*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"gauss", "[", "ff_", "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"extprod", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"\[Omega]", "[", "1", "]"}], "[", "ff", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Omega]", "[", "2", "]"}], "[", "ff", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "2"}], "]"}], "]"}], "/", 
   RowBox[{
    RowBox[{"extprod", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "ff", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "ff", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "2"}], "]"}], "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5885019269280777`*^9, 3.5885021178568125`*^9}, {
  3.588502172800109*^9, 3.5885022401922274`*^9}, {3.609673711884263*^9, 
  3.609673712924323*^9}},
 CellTags->"gaussK"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "gauss"}]], "Input",
 CellChangeTimes->{{3.6152113763484354`*^9, 3.615211448826163*^9}, {
   3.615211504674261*^9, 3.6152115152822795`*^9}, 3.61521155540555*^9, {
   3.622781860423489*^9, 3.6227818755399156`*^9}, {3.6227822926690483`*^9, 
   3.622782302122665*^9}}],

Cell[BoxData[
 StyleBox["\<\"gauss[f][u,v] is the Gaussian curvature of the surface \
f[u,v]\\nin the Euclidean 3-space.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.627827690359412*^9},
 CellTags->"Info3627831290-6705403"]
}, Closed]],

Cell[TextData[{
 "Is named the ",
 StyleBox["Gaussian curvature",
  FontColor->RGBColor[0, 0, 1]],
 " of the surface at the point  ff[u,v].  Remark that the Gaussian curvature \
doesn\[CloseCurlyQuote]t depend on the orientation of the surface: A change \
of the orientation would cause the change of signs of the numerator and the \
denominator in the same way.\nWe will calculate it for our example:"
}], "Text",
 CellChangeTimes->{{3.5885022915319176`*^9, 3.588502353635627*^9}, {
   3.5896344813413644`*^9, 3.5896344946851273`*^9}, {3.5896352910086946`*^9, 
   3.589635419818121*^9}, 3.615264508027816*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"gauss", "[", "hypsf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5885023564748316`*^9, 3.588502394772899*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  SuperscriptBox[
   RowBox[{"Sech", "[", 
    RowBox[{"2", " ", "v"}], "]"}], "2"]}]], "Output",
 CellChangeTimes->{3.627827740691101*^9}]
}, Closed]],

Cell["\<\
Remember Sech[x] == 1/Cosh[x].  Since hypsf is a surface of revolution the \
Gaussian curvature does not depend on u.  It reflects very well the visual \
impression of the curvature of the hyperboloid:\
\>", "Text",
 CellChangeTimes->{{3.588502593392448*^9, 3.58850274255991*^9}, 
   3.615264540124651*^9}],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     RowBox[{"Sech", "[", 
      RowBox[{"2", " ", "v"}], "]"}], "^", "2"}]}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5884250453039*^9, 3.5884250954547687`*^9}, 
   3.588425134172983*^9, 3.588502546139965*^9}],

Cell["\<\
For a sphere of radius r the Gaussian curvature is constant, namely\
\>", "Text",
 CellChangeTimes->{{3.627827807484621*^9, 3.627827884538557*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"gauss", "[", 
     RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"r", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6278275913456373`*^9, 3.6278276071796656`*^9}, {
  3.627827637682719*^9, 3.627827647699937*^9}}],

Cell[BoxData[
 FractionBox["1", 
  SuperscriptBox["r", "2"]]], "Output",
 CellChangeTimes->{3.6278276104712715`*^9, 3.627827650728342*^9}]
}, Closed]],

Cell["\<\
Mathematica also calculates the Gaussian curvature for a general surface:\
\>", "Text",
 CellChangeTimes->{{3.588502870776535*^9, 3.5885029205094223`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"gauss", "[", "sf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.588502749330322*^9, 3.5885027917311964`*^9}, {
  3.609825995110777*^9, 3.609825999670038*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}]}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "2"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "3"]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "2"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "3"]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "2"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "2"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "2"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "2"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "2"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "3"]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "2"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "3"]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "2"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "2"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "2"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "2"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "2"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "2"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "2"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "2"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "2"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "3"]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "2"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "3"]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], ")"}], "/", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], "-", 
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], "-", 
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], "-", 
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], ")"}], 
        RowBox[{"3", "/", "2"}]]}], ")"}]}], "-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "3"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "3"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "3"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "3"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "3"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "3"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], ")"}], "/", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], "-", 
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], "-", 
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], "-", 
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], ")"}], 
        RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}], "/", 
  RowBox[{"(", 
   RowBox[{"\[Sqrt]", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}]}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], ")"}]}], 
   ")"}]}]], "Output",
 CellChangeTimes->{3.6278279619578943`*^9, 3.62788232195513*^9, 
  3.6278977020091763`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"Together", "[", "%", "]"}], ",", 
    RowBox[{"TimeConstraint", " ", "\[Rule]", " ", "Infinity"}]}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6098262724436398`*^9, 3.6098262802690873`*^9}, {
   3.6098268413811812`*^9, 3.6098268828225517`*^9}, 3.6098309235986705`*^9, {
   3.627827986394537*^9, 3.6278280029345665`*^9}, {3.627882451248157*^9, 
   3.6278824652413816`*^9}, 3.6278958180536346`*^9}],

Cell[BoxData["$Aborted"], "Output",
 CellChangeTimes->{3.6279167169316807`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"Together", "[", "%", "]"}], ",", 
    RowBox[{"TimeConstraint", " ", "\[Rule]", " ", "Infinity"}]}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6098262724436398`*^9, 3.6098262802690873`*^9}, {
   3.6098268413811812`*^9, 3.6098268828225517`*^9}, 3.6098309235986705`*^9, {
   3.627827986394537*^9, 3.6278280029345665`*^9}, {3.627882451248157*^9, 
   3.6278824652413816`*^9}, 3.6278958180536346`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "12707.91946049999933165963739156723022460938`10.124674366997587", ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"]}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "2"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "-", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "2"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "-", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "2"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{"2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "2"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "2"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
        ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], "2"], "-", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"-", "2"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"-", "2"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "-", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "-", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "-", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "-", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "-", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"-", "2"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"-", "2"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}]}], ")"}]}]}], ")"}], "/", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "-", 
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}], 
     "2"]}]}], "}"}]], "Output",
 CellChangeTimes->{3.62783131606742*^9, 3.6278824419661407`*^9, 
  3.6278951878125277`*^9}]
}, Closed]],

Cell[TextData[{
 "Evaluating with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " version 9.0.1.0 he needed processing time was 12708 s. We shall give \
simpler expression for the Gaussian curvature later."
}], "Text",
 CellChangeTimes->{{3.6098309366054144`*^9, 3.6098310648917522`*^9}, 
   3.615211712373026*^9, 3.6278958350888643`*^9, {3.6278958679893227`*^9, 
   3.6278959661602945`*^9}, {3.627896520991829*^9, 3.6278965538328867`*^9}}],

Cell[TextData[{
 "We define the ",
 StyleBox["curvature integral ",
  FontColor->RGBColor[0, 0, 1]],
 "as the integral of K(u,v)*dA[sf][u,v]  (K[u,v]== Gaussian Curvature); for \
surfaces with K(u,v)\[NotEqual]0  it coincides up to sign with the integral \
curvature. "
}], "Text",
 CellChangeTimes->{{3.588502989586344*^9, 3.5885031522946296`*^9}, {
   3.588503185959489*^9, 3.5885032187663465`*^9}, {3.6098311102523465`*^9, 
   3.6098311450603375`*^9}, 3.615264590355524*^9, {3.617954311481616*^9, 
   3.6179544126024*^9}}],

Cell["\<\
The Gaussian curvature of any developable surface is zero. Examples:\
\>", "Text",
 CellChangeTimes->{{3.5885081700865965`*^9, 3.5885082334071083`*^9}, 
   3.588508375289357*^9}],

Cell[BoxData[
 RowBox[{"?", "plane"}]], "Input",
 CellChangeTimes->{{3.5899584114106817`*^9, 3.589958414279846*^9}, {
  3.615211799577179*^9, 3.615211819997615*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pl", "[", 
   RowBox[{"u_", ",", "v_"}], " ", "]"}], "=", " ", 
  RowBox[{
   RowBox[{"plane", "[", 
    RowBox[{"1", ",", "0", ",", "0", ",", "1", ",", "0", ",", "0"}], "]"}], 
   "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5899581112285123`*^9, 3.589958113401637*^9}, {
   3.5899581556930556`*^9, 3.5899582483363543`*^9}, {3.589958278543082*^9, 
   3.589958359030686*^9}, 3.6148618527175274`*^9, 3.614862652983333*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"gauss", "[", "pl", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.589958375121606*^9, 3.589958385221184*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"gauss", "[", 
    RowBox[{"tangsf", "[", "curve", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.588502749330322*^9, 3.5885027917311964`*^9}, {
  3.5885080347251587`*^9, 3.588508057048798*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"gauss", "[", 
    RowBox[{"conesf", "[", "y", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.588502749330322*^9, 3.5885027917311964`*^9}, {
   3.5885080347251587`*^9, 3.588508057048798*^9}, 3.5885082818295927`*^9, {
   3.588508346647707*^9, 3.588508358332127*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"gauss", "[", 
    RowBox[{"cylindersf", "[", 
     RowBox[{"c", ",", "y"}], "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.588502749330322*^9, 3.5885027917311964`*^9}, {
   3.5885080347251587`*^9, 3.588508057048798*^9}, 3.5885082818295927`*^9, {
   3.588508346647707*^9, 3.5885083677857437`*^9}, {3.5885084318706565`*^9, 
   3.588508443430277*^9}}],

Cell[TextData[{
 "The Gaussian curvature of a sphere of radius r equals ",
 Cell[BoxData[
  FormBox[
   FractionBox["1", 
    SuperscriptBox["r", "2"]], TraditionalForm]]],
 ":"
}], "Text",
 CellChangeTimes->{{3.5885086629246006`*^9, 3.588508697213562*^9}, {
   3.588508729758423*^9, 3.5885088016285343`*^9}, 3.615264594986789*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"gauss", "[", 
    RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.588502749330322*^9, 3.5885027917311964`*^9}, {
   3.5885080347251587`*^9, 3.588508057048798*^9}, 3.5885082818295927`*^9, {
   3.588508346647707*^9, 3.5885083677857437`*^9}, {3.5885084318706565`*^9, 
   3.588508443430277*^9}, {3.5885088284560685`*^9, 3.5885088477761736`*^9}}],

Cell["\<\
Clearly, for the unit sphere the Gauss map is the identity; it is orientation \
preserving, and the Gaussian curvature is positive.\
\>", "Text",
 CellChangeTimes->{{3.5885090527488976`*^9, 3.5885091182126417`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["2.2.2. Theorema Egregium", "Subsubsection",
 CellChangeTimes->{{3.5897791459451184`*^9, 3.5897791554299345`*^9}, {
   3.589779201169215*^9, 3.5897792115588336`*^9}, 3.609831296628007*^9, 
   3.6112105016421847`*^9, 3.6152119829399014`*^9, 3.615212872250663*^9, 
   3.6179564158099766`*^9, 3.617965383514293*^9, 3.6278966188572016`*^9, 
   3.6278967271093955`*^9}],

Cell["\<\
The Gaussian curvature has been defined by the normal vector and the \
spherical image  of the surface. This is a construction using the embedding \
of the surface into the Euclidean space. It is an surprising result, found by \
Gauss in the year 1816,  that this curvature doesn\[CloseCurlyQuote]t depend \
on the embedding but on the metric of the surface only. Gauss named this \
result the\
\>", "Text",
 CellChangeTimes->{{3.589779294238979*^9, 3.589779629748768*^9}, {
   3.589780134955655*^9, 3.5897801705861177`*^9}, {3.5898648454159813`*^9, 
   3.589864846336383*^9}, 3.589864883526848*^9}],

Cell[TextData[{
 StyleBox["Theorema egregium",
  FontColor->RGBColor[0, 0, 1]],
 ". If F[sf[u,v]] = sh[f[u,v]] is an isometry then the Gaussian curvatures at \
corresponding points coincide:"
}], "Text",
 CellChangeTimes->{{3.589779644319194*^9, 3.5897796537572103`*^9}, {
  3.58977983591873*^9, 3.589779957817344*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"gauss", "[", 
    RowBox[{"sh", "[", "f", "]"}], "]"}], "[", 
   RowBox[{"u", ",", " ", "v"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"gauss", "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.589779962700153*^9, 3.5897800217150564`*^9}, {
  3.61795650503608*^9, 3.6179565144676194`*^9}, {3.6179565867477536`*^9, 
  3.6179565931711206`*^9}}],

Cell[TextData[{
 StyleBox["Proof",
  FontColor->RGBColor[0, 0, 1]],
 ".  In subsection 1.4.1, 2. we expressed the first fundamental form as the \
sum of the squares of the basis forms \[Sigma][1], \[Sigma][2]. Therefore it \
suffices to show that the Gaussian curvature may be expressed through these \
basis forms and their exterior derivatives. From the last integrability \
condition ",
 ButtonBox["1.5.1",
  BaseStyle->"Hyperlink",
  ButtonData->"intcond"],
 " it follows"
}], "Text",
 CellChangeTimes->{{3.5897802415506425`*^9, 3.589780247556653*^9}, {
   3.589781031988431*^9, 3.5897811125001726`*^9}, {3.589781144308628*^9, 
   3.5897811764446845`*^9}, {3.5897813023213053`*^9, 3.589781390586261*^9}, {
   3.5898649435557537`*^9, 3.589864944928556*^9}, {3.589865042054326*^9, 
   3.589865062708763*^9}, {3.589865161098136*^9, 3.589865165263343*^9}, {
   3.5898652061666145`*^9, 3.589865238209071*^9}, 3.589868621090613*^9}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"gauss", "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", "+", 
     RowBox[{
      RowBox[{
       RowBox[{"dd", "[", 
        RowBox[{
         RowBox[{"\[Omega]", "[", "3", "]"}], "[", "sf", "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "/", 
      RowBox[{
       RowBox[{"extprod", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "sf", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "sf", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
       RowBox[{"[", 
        RowBox[{"1", ",", "2"}], "]"}], "]"}]}]}], ")"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "\[Element]", " ", "Reals"}], ",", " ", 
     RowBox[{"v", "\[Element]", " ", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5898653290948305`*^9, 3.5898654271098027`*^9}, {
   3.589865496529925*^9, 3.5898655792100697`*^9}, 3.589865737519148*^9, {
   3.5898682853312235`*^9, 3.589868290292032*^9}, {3.5898683717241745`*^9, 
   3.589868393626613*^9}, {3.58986851160962*^9, 3.5898685574737015`*^9}, 
   3.6096737157884865`*^9, 3.6099111290994644`*^9, 3.609911671609494*^9}],

Cell["\<\
Thus it suffices to show that  \[Omega][3] may be expressed by \[Sigma][1], \
\[Sigma][2]. This follows from the first two integrability conditions. \
Indeed, inserting for the basis representation of  \[Omega][3]:\
\>", "Text",
 CellChangeTimes->{{3.589865870649782*^9, 3.589866048770895*^9}, {
   3.589866265190075*^9, 3.589866267186878*^9}, 3.6152646117317467`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Omega]", "[", "3", "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"cg", "[", "1", "]"}], "*", 
    RowBox[{"\[Sigma]", "[", "1", "]"}]}], " ", "+", "  ", 
   RowBox[{
    RowBox[{"cg", "[", "2", "]"}], "*", 
    RowBox[{"\[Sigma]", "[", "2", "]"}], " "}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.589866097895381*^9, 3.589866168033104*^9}, {
  3.6179744737467556`*^9, 3.6179744768909354`*^9}}],

Cell[TextData[{
 "into the integrability conditions we obtain the formulas ",
 ButtonBox["[B-R]",
  BaseStyle->"Hyperlink",
  ButtonData->"B-R"],
 ", (46.5-8),used  as definitions in eudiffgeov2,m"
}], "Text",
 CellChangeTimes->{{3.5898662781068974`*^9, 3.589866329103387*^9}, {
  3.5898673198299274`*^9, 3.5898673426683674`*^9}, {3.589867391465253*^9, 
  3.589867431931724*^9}, {3.6226157712134523`*^9, 3.622615791384288*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"cg", "[", "1", "]"}], "[", "ff_", "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"dd", "[", 
     RowBox[{
      RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "ff", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "/", 
   RowBox[{
    RowBox[{"extprod", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "ff", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "ff", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "2"}], "]"}], "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5898663353121977`*^9, 3.5898664498007994`*^9}, {
   3.589866942293664*^9, 3.5898669583616924`*^9}, {3.5898669907161493`*^9, 
   3.589867004756174*^9}, 3.5898681119369183`*^9, {3.5898682177831044`*^9, 
   3.589868228796724*^9}, 3.6096737250530167`*^9, 3.6179744789150515`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"cg", "[", "2", "]"}], "[", "ff_", "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"dd", "[", 
     RowBox[{
      RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "ff", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "/", 
   RowBox[{
    RowBox[{"extprod", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "ff", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "ff", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "2"}], "]"}], "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.589866492544874*^9, 3.5898665075833006`*^9}, {
   3.589866902544794*^9, 3.5898669300476427`*^9}, 3.5898689735732317`*^9, {
   3.589869375273937*^9, 3.58986938255915*^9}, 3.609911786091042*^9, 
   3.609911905885894*^9, 3.6179744802981305`*^9}],

Cell[BoxData[
 RowBox[{"?", "cg"}]], "Input",
 CellChangeTimes->{{3.6179750758591948`*^9, 3.617975187419575*^9}, {
   3.6179752579396086`*^9, 3.617975284484127*^9}, 3.6179754698357286`*^9, 
   3.617976052148035*^9, {3.6179762354295177`*^9, 3.6179763154430943`*^9}, 
   3.617976361330719*^9, {3.6226158991336765`*^9, 3.622615913142502*^9}}],

Cell[BoxData[
 RowBox[{"Context", "[", "cg", "]"}]], "Input",
 CellChangeTimes->{{3.6226159252169228`*^9, 3.6226159541861734`*^9}}],

Cell["and this shows", "Text",
 CellChangeTimes->{{3.5898665879858418`*^9, 3.589866602790268*^9}}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[Omega]", "[", "3", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], " ", "-", " ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"cg", "[", "1", "]"}], "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "*", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], " ", "-", " ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"cg", "[", "2", "]"}], "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "*", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}]}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "\[Element]", " ", "Reals"}], ",", " ", 
     RowBox[{"v", "\[Element]", " ", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.589866608874278*^9, 3.5898667830485845`*^9}, {
   3.589867075549098*^9, 3.5898671069051533`*^9}, {3.5898676092260356`*^9, 
   3.5898676576329203`*^9}, {3.589869404633189*^9, 3.5898694693577027`*^9}, 
   3.609911828578472*^9, {3.617974482131235*^9, 3.6179744835703173`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"dd", "[", 
        RowBox[{
         RowBox[{"\[Omega]", "[", "3", "]"}], "[", "sf", "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "/", 
      RowBox[{
       RowBox[{"extprod", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "sf", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "sf", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
       RowBox[{"[", 
        RowBox[{"1", ",", "2"}], "]"}], "]"}]}], "+", " ", 
     RowBox[{
      RowBox[{"gauss", "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"u", "\[Element]", " ", "Reals"}], ",", " ", 
      RowBox[{"v", "\[Element]", " ", "Reals"}]}], "}"}]}], "]"}], 
  " "}]], "Input",
 CellChangeTimes->{
  3.589893961196463*^9, 3.6179587350616302`*^9, {3.617958812949085*^9, 
   3.617958909461605*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"gauss", "[", "hypsf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], " ", "\[Equal]", " ", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       RowBox[{"dd", "[", 
        RowBox[{
         RowBox[{"\[Omega]", "[", "3", "]"}], "[", "hypsf", "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "/", 
     RowBox[{
      RowBox[{"extprod", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "hypsf", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "hypsf", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"1", ",", "2"}], "]"}], "]"}]}]}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "\[Element]", " ", "Reals"}], ",", " ", 
     RowBox[{"v", "\[Element]", " ", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{
  3.5898707934412284`*^9, {3.5898708344381003`*^9, 3.5898708579785414`*^9}, {
   3.5898940993971057`*^9, 3.5898941417199802`*^9}, 3.609911786284053*^9, 
   3.6099119062469144`*^9, {3.617964848782708*^9, 3.617964872742079*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6278966920189333`*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["2.2.3. Developable Surfaces 2", "Subsubsection",
 CellChangeTimes->{{3.58995791782845*^9, 3.5899579288990836`*^9}, {
   3.58995796082791*^9, 3.589957970067438*^9}, {3.5900391023377857`*^9, 
   3.5900391053953905`*^9}, 3.611210506158443*^9, 3.6112125836092663`*^9, 
   3.6152128786622744`*^9, 3.615263576317348*^9, 3.6179654195943565`*^9, 
   3.617973709425148*^9, 3.6278967582042503`*^9, 3.6279023898942275`*^9}],

Cell["\<\
In subsection 2.2.1 we have seen that all developable surfaces have Gaussian \
curvature zero. From the theorema egregium it follows that any surface \
isometric to the plane has this property. Now we will prove the converse: \
\>", "Text",
 CellChangeTimes->{{3.5899580312989407`*^9, 3.5899580758994913`*^9}, {
  3.5899585563559723`*^9, 3.5899586364995565`*^9}, {3.589958678715971*^9, 
  3.589958706467558*^9}, {3.589978337180153*^9, 3.5899783553853855`*^9}, {
  3.590039159901887*^9, 3.590039159901887*^9}, {3.590039354761829*^9, 
  3.590039368786254*^9}}],

Cell[TextData[{
 StyleBox["Proposition",
  FontColor->RGBColor[0, 0, 1]],
 ". Let sf[u,v] be a surface with Gaussian curvature zero. Then the rank \
rk[u,v] of the differential of the Gauss map  d[",
 "amf[",
 "3][sf]][u,v] is less than 2. If the rank for all points sf[u,v] equals \
zero, the surface is a part of a plane. If the rank  for all points sf[u,v] \
equals one, the surface is developable."
}], "Text",
 CellChangeTimes->{{3.5899587166831427`*^9, 3.5899587244515867`*^9}, {
   3.5899588196920342`*^9, 3.589958880251498*^9}, {3.589958924227013*^9, 
   3.589958945747244*^9}, {3.5899589894277425`*^9, 3.5899590581636744`*^9}, {
   3.5899590896194735`*^9, 3.5899591359231215`*^9}, 3.589978431482319*^9, {
   3.589979743507023*^9, 3.589979771041072*^9}, {3.5899800276459227`*^9, 
   3.5899800708423977`*^9}, 3.614861852826728*^9, 3.6149177846821485`*^9, 
   3.615261926990012*^9}],

Cell[TextData[{
 StyleBox["Proof",
  FontColor->RGBColor[0, 0, 1]],
 ".  The differential of the Gauss map is \nd[amf[3] ]== amf[1] \[Omega][2] - \
 amf[2] \[Omega][1]. \nThe rank  rk[u,v] of this map at a point sf[u,v] is \
less than 2 iff \[Omega][1] ",
 Cell[BoxData[
  FormBox["\[And]", TraditionalForm]]],
 " \[Omega][2] == 0, and this is the case iff gauss[sf][u,v]== 0. If the rank \
equals 0 for the whole connected surface  the normal vector amf[3] is \
constant, and the statement follows from",
 ButtonBox[" Lemma 2.1.1",
  BaseStyle->"Hyperlink",
  ButtonData->"lem2.1.1"],
 ".  If the rank of the Gauss map is less than two then the set of all points \
sf[u,v] of the surface with rk[u,v] = 1 is open. We assume that rk[u,v] == 1 \
for all points of the surface. Then the kernel ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{"d", "[", 
        RowBox[{
         RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ")"}], 
     RowBox[{"-", "1"}]], "[", 
    RowBox[{"null", "[", " ", "]"}]}], TraditionalForm]]],
 "] of d[amf[3][sf]][u,v]  is one-dimensional. We put the vector \
amf[1][sf][u,v] into the kernel. Then the frame is uniquely defined since \
amf[2] == amf[3] \[Times] amf[1]. The integral curves of the Pfaffian \
equation \[Sigma][2] == 0 have the tangent vector amf[1]=d[sf][u, v].vt, \
where vt is the corresponding vector in the parameter space. Thus \
d[amf[3][sf]][u, v].vt== 0, and the normal vector is constant along the \
integral curves. We will show that also amf[1] is constant along these \
curves: d[amf[1][sf]][u,v].vt == 0 if \[Sigma][2](vt) = 0. To this aim we \
consider the basis decompositions of \[Omega][1], \[Omega][2] with the \
notations [B-R] (61.9) p. 87:"
}], "Text",
 CellChangeTimes->{{3.589978656980715*^9, 3.589978703499996*^9}, {
   3.5899787359636536`*^9, 3.589978816974596*^9}, {3.5899788639306784`*^9, 
   3.589979198488866*^9}, {3.5899796086915865`*^9, 3.589979638050838*^9}, {
   3.589979676146105*^9, 3.5899797209025836`*^9}, 3.5899800551799707`*^9, {
   3.589980091294034*^9, 3.5899801718057756`*^9}, {3.589980202428629*^9, 
   3.5899802665915422`*^9}, {3.5899803831008573`*^9, 
   3.5899805423809676`*^9}, {3.5899805879755754`*^9, 
   3.5899807208211737`*^9}, {3.589980760477442*^9, 3.58998077250113*^9}, {
   3.589980874364956*^9, 3.5899809564446507`*^9}, 3.5899811643895445`*^9, 
   3.589981878013362*^9, {3.589981948653402*^9, 3.589981996293127*^9}, {
   3.589982186453003*^9, 3.589982221292996*^9}, {3.5899822553059416`*^9, 
   3.5899823342284555`*^9}, {3.590039496051277*^9, 3.5900394960668774`*^9}, {
   3.590039566969002*^9, 3.590039575065416*^9}, {3.590039605189069*^9, 
   3.590039677604396*^9}, 3.59003973123729*^9, {3.5900397746677666`*^9, 
   3.5900397784741735`*^9}, {3.5900398450082903`*^9, 3.590039878158348*^9}, {
   3.5900399682641068`*^9, 3.590040111987159*^9}, {3.5900407431174674`*^9, 
   3.59004083581283*^9}, {3.590040875140499*^9, 3.590040965620658*^9}, {
   3.59004100086112*^9, 3.590041048722004*^9}, {3.614861853481929*^9, 
   3.6148618583803377`*^9}, {3.6149177853529496`*^9, 
   3.6149177902357583`*^9}, {3.6152619289561243`*^9, 3.615261948601248*^9}, {
   3.615262603651715*^9, 3.615262646526167*^9}, {3.615262816362881*^9, 
   3.6152628196860714`*^9}, {3.627900345151836*^9, 3.6279003527802496`*^9}, {
   3.6279012717154636`*^9, 3.627901321385951*^9}, {3.627901352398805*^9, 
   3.6279014732834177`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", " ", 
    RowBox[{"\[Omega]", "[", "1", "]"}]}], " ", "\[Equal]", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"c", "[", 
      RowBox[{"2", ",", "1"}], "]"}], "*", 
     RowBox[{"\[Sigma]", "[", "1", "]"}]}], " ", "+", "  ", 
    RowBox[{
     RowBox[{"c", "[", 
      RowBox[{"2", ",", "2"}], "]"}], "*", 
     RowBox[{"\[Sigma]", "[", "2", "]"}]}]}]}], ";", " ", 
  RowBox[{
   RowBox[{"\[Omega]", "[", "2", "]"}], " ", "\[Equal]", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"c", "[", 
      RowBox[{"1", ",", "1"}], "]"}], "*", 
     RowBox[{"\[Sigma]", "[", "1", "]"}]}], " ", "+", "  ", 
    RowBox[{
     RowBox[{"c", "[", 
      RowBox[{"1", ",", "2"}], "]"}], "*", 
     RowBox[{"\[Sigma]", "[", "2", "]"}]}]}]}], ";"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5899823839693003`*^9, 3.589982562800529*^9}}],

Cell[TextData[{
 "leading to the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " definitions of the coefficients cf defined in eudiffgeov2.m."
}], "Text",
 CellChangeTimes->{{3.589982622308933*^9, 3.5899826449002247`*^9}, {
   3.6226135412863355`*^9, 3.622613608160453*^9}, 3.627901605961651*^9},
 CellTags->"cij"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "cf"}]], "Input",
 CellChangeTimes->{{3.622613719685049*^9, 3.622613721900253*^9}}],

Cell[BoxData[
 StyleBox["\<\"cf[i,j][f][u,v] are the coefficients  of the second \
fundamental form \\nwith respect to the cobasis {\[Sigma][i][f][u,v]} of the \
surface f[u,v] in the Euclidean 3-space.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.627901610610459*^9},
 CellTags->"Info3627905210-4669561"]
}, Closed]],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"cf", "[", 
       RowBox[{"2", ",", "1"}], "]"}], "[", "ff_", "]"}], "[", 
     RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       RowBox[{"extprod", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Omega]", "[", "1", "]"}], "[", "ff", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "ff", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
       RowBox[{"[", 
        RowBox[{"1", ",", "2"}], "]"}], "]"}]}], "/", 
     RowBox[{
      RowBox[{"extprod", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "ff", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "ff", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"1", ",", "2"}], "]"}], "]"}]}]}], ";"}], 
  "\[IndentingNewLine]"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"cf", "[", 
       RowBox[{"2", ",", "2"}], "]"}], "[", "ff_", "]"}], "[", 
     RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", 
    RowBox[{
     RowBox[{"+", 
      RowBox[{
       RowBox[{"extprod", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Omega]", "[", "1", "]"}], "[", "ff", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "ff", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
       RowBox[{"[", 
        RowBox[{"1", ",", "2"}], "]"}], "]"}]}], "/", 
     RowBox[{
      RowBox[{"extprod", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "ff", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "ff", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"1", ",", "2"}], "]"}], "]"}]}]}], ";"}], 
  "\n"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"cf", "[", 
       RowBox[{"1", ",", "1"}], "]"}], "[", "ff_", "]"}], "[", 
     RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"extprod", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[Omega]", "[", "2", "]"}], "[", "ff", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "ff", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"1", ",", "2"}], "]"}], "]"}], "/", 
     RowBox[{
      RowBox[{"extprod", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "ff", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "ff", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"1", ",", "2"}], "]"}], "]"}]}]}], ";"}], 
  "\n"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"cf", "[", 
      RowBox[{"1", ",", "2"}], "]"}], "[", "ff_", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"extprod", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "ff", "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Omega]", "[", "2", "]"}], "[", "ff", "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
     RowBox[{"[", 
      RowBox[{"1", ",", "2"}], "]"}], "]"}], "/", 
    RowBox[{
     RowBox[{"extprod", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "ff", "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "ff", "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
     RowBox[{"[", 
      RowBox[{"1", ",", "2"}], "]"}], "]"}]}]}], ";"}]}], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5899830368256416`*^9, 3.589983216612925*^9}, 
   3.589983350955609*^9, 3.589983976627395*^9, {3.59221330302858*^9, 
   3.5922133147682514`*^9}, 3.592315502531618*^9, {3.609911786486065*^9, 
   3.609911786498065*^9}, {3.609911906425925*^9, 3.6099119064319253`*^9}, 
   3.6179720239937468`*^9, {3.6257282958419447`*^9, 3.6257283174635825`*^9}}],

Cell["For this coefficients one has ", "Text",
 CellChangeTimes->{{3.5900401716884637`*^9, 3.590040188489693*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"cf", "[", 
      RowBox[{"2", ",", "1"}], "]"}], "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "==", 
   RowBox[{
    RowBox[{
     RowBox[{"cf", "[", 
      RowBox[{"1", ",", "2"}], "]"}], "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5899837285372057`*^9, 3.589983810223878*^9}, {
  3.5900402150409403`*^9, 3.5900402169441433`*^9}, {3.617972032119211*^9, 
  3.6179720334632883`*^9}, {3.625726673503094*^9, 3.625726701817144*^9}, {
  3.6279017111838355`*^9, 3.6279017345370765`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6279017946751823`*^9}]
}, Closed]],

Cell["\<\
 for all surfaces; it is a consequence of the third integrability condition. \
We conclude, since \[Sigma][1][sf][u,v][vt] ==1 and \[Sigma][2][sf][u,v][vt] \
== 0,\
\>", "Text",
 CellChangeTimes->{{3.589984123908819*^9, 3.589984178348933*^9}, {
   3.589984231596979*^9, 3.5899843220211506`*^9}, {3.5899843621814475`*^9, 
   3.5899843678207703`*^9}, {3.5899843991745634`*^9, 3.5899844344285803`*^9}, 
   3.590040616492045*^9, {3.5900406549929123`*^9, 3.590040669547738*^9}, 
   3.5900411030413*^9, {3.590041232427927*^9, 3.5900412522711616`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", 
     RowBox[{
      RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], 
   RowBox[{"(", "vt", ")"}]}], " ", "\[Equal]", " ", "0", " ", "\[Equal]", 
  " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"amf", "[", "1", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], " ", "*", 
    RowBox[{
     RowBox[{"cf", "[", 
      RowBox[{"1", ",", "1"}], "]"}], "[", "sf", "]"}]}], " ", "+", "  ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"amf", "[", "2", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], " ", "*", 
    RowBox[{
     RowBox[{"cf", "[", 
      RowBox[{"2", ",", "1"}], "]"}], "[", "sf", "]"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.589984447519329*^9, 3.5899845634239583`*^9}, {
   3.589984611100685*^9, 3.589984633600972*^9}, {3.589984668727981*^9, 
   3.589984696433566*^9}, 3.589984851882457*^9, 3.5899848983511147`*^9, {
   3.6148618584115376`*^9, 3.6148618584271374`*^9}, {3.6149177902513585`*^9, 
   3.614917790266958*^9}, {3.61526194986032*^9, 3.615261953404523*^9}, {
   3.6179720352323895`*^9, 3.617972036935487*^9}, {3.6279019504258556`*^9, 
   3.6279019720630937`*^9}}],

Cell["\<\
We conclude cf[1,1] == cf[2,1] ==cf[1,2] ==0, and therefore \
\[Omega][2][sf][u,v] == 0 is the null form. Furthermore we have that the \
assumption implies \
\>", "Text",
 CellChangeTimes->{{3.589984961956753*^9, 3.589984971221283*^9}, {
  3.589985019637052*^9, 3.5899851567168922`*^9}, {3.5899852224376516`*^9, 
  3.5899852821410666`*^9}, {3.61797203839657*^9, 3.617972045323967*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Omega]", "[", "1", "]"}], "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     RowBox[{
      RowBox[{"cf", "[", 
       RowBox[{"2", ",", "2"}], "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "*", 
   RowBox[{
    RowBox[{
     RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5899852947197857`*^9, 3.5899853374092274`*^9}, {
   3.5899853776785307`*^9, 3.589985379710647*^9}, {3.589985414166618*^9, 
   3.5899855089280376`*^9}, 3.617972047079067*^9}],

Cell["We conclude from the derivation equations", "Text",
 CellChangeTimes->{{3.5899857875329733`*^9, 3.589985865828451*^9}, 
   3.5899859187814803`*^9, {3.5899859747406807`*^9, 3.5899859794209485`*^9}, {
   3.5900414890483775`*^9, 3.5900415027764015`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", 
     RowBox[{
      RowBox[{"amf", "[", "1", "]"}], "[", "sf", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], 
   RowBox[{"(", "vt", ")"}]}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"amf", "[", "2", "]"}], "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "*", 
   RowBox[{
    RowBox[{
     RowBox[{"\[Omega]", "[", "3", "]"}], "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], 
   RowBox[{"(", "vt", ")"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.589985616280178*^9, 3.5899857305607147`*^9}, {
  3.6148618584271374`*^9, 3.6148618584427376`*^9}, {3.6149177902825584`*^9, 
  3.6149177902825584`*^9}, {3.615261954652594*^9, 3.6152619562126837`*^9}}],

Cell["\<\
since \[Omega][2] == 0. The last equation implies. since \[Sigma][1][vt]== 1, \
\[Sigma][2][vt]==0:\
\>", "Text",
 CellChangeTimes->{
  3.589985928237021*^9, {3.5899860674129815`*^9, 3.5899861255893087`*^9}, 
   3.590041447006304*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", 
     RowBox[{
      RowBox[{"amf", "[", "1", "]"}], "[", "sf", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], 
   RowBox[{"(", "vt", ")"}]}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"amf", "[", "2", "]"}], "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "*", 
   RowBox[{
    RowBox[{
     RowBox[{"cg", "[", "1", "]"}], "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5899860290237856`*^9, 3.589986048991928*^9}, {
   3.6148618584427376`*^9, 3.6148618584427376`*^9}, {3.6149177902825584`*^9, 
   3.6149177902981586`*^9}, {3.6152619573567486`*^9, 3.6152619587168264`*^9}, 
   3.6179744883795924`*^9}],

Cell["Now we apply the fifth integrability condition :", "Text",
 CellChangeTimes->{{3.5899861640295076`*^9, 3.5899861937332067`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"dd", "[", 
     RowBox[{"\[Omega]", "[", "2", "]"}], "]"}], " ", "\[Equal]", " ", 
    RowBox[{"-", " ", 
     RowBox[{"\[Omega]", "[", "3", "]"}]}]}], "\[And]", " ", 
   RowBox[{"\[Omega]", "[", "1", "]"}]}], " ", "=", " ", 
  RowBox[{"0", " ", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"cg", "[", "1", "]"}], "*", " ", 
     RowBox[{"cf", "[", 
      RowBox[{"2", ",", "2"}], "]"}], "*", 
     RowBox[{"\[Sigma]", "[", "1", "]"}]}], "\[And]", 
    RowBox[{
     RowBox[{"\[Sigma]", "[", "2", "]"}], "."}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.589986244895133*^9, 3.5899863882873344`*^9}, {
   3.5899864285356364`*^9, 3.589986483231765*^9}, {3.5899865213349447`*^9, 
   3.589986633432356*^9}, 3.6179720482881365`*^9, 3.617974492023801*^9}],

Cell["\<\
Since MatrixRank[{\[Omega][1], \[Omega][2]}] == 1 we have \
cf[2,2]\[NotEqual]0, and thus cg[1] == 0. It follows\
\>", "Text",
 CellChangeTimes->{{3.5899866745767093`*^9, 3.589986807629319*^9}, {
   3.589986839149122*^9, 3.5899868475086*^9}, 3.61797204974422*^9, 
   3.617974494186925*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", 
     RowBox[{
      RowBox[{"amf", "[", "1", "]"}], "[", "sf", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], 
   RowBox[{"(", "vt", ")"}]}], " ", "\[Equal]", " ", "0"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5899868535589466`*^9, 3.589986858447226*^9}, 
   3.6148618584427376`*^9, 3.6149177902981586`*^9, 3.6152619598278904`*^9}],

Cell["\<\
amf[1] is constant along the integral curves of \[Sigma][2]== 0, and thus \
these curves must be lines. Since the normal vector of the surface is also \
constant along these lines, the surface is developable. \[FilledSquare]\
\>", "Text",
 CellChangeTimes->{{3.5899869117332735`*^9, 3.5899870684532375`*^9}, 
   3.6148618584739375`*^9, 3.614917790329358*^9, 3.615261961097963*^9, {
   3.6152634663810596`*^9, 3.6152634704812946`*^9}, {3.617972839882413*^9, 
   3.617972846952817*^9}}],

Cell[TextData[{
 StyleBox["Remark",
  FontColor->RGBColor[0, 0, 1]],
 ". Generally, a surface may have singularities. Furthermore, the subset of \
points at which the rank of the Gauss map is zero, is a closed set. The \
points of the last set not being interior points of the set are not \
considered in the proposition.  Having in mind this, one often formulates - \
with a grain of salt:\n",
 StyleBox["Corollary",
  FontColor->RGBColor[0, 0, 1]],
 ". A surface in the Euclidean 3-space is locally isometric to an Euclidean \
plane iff its Gauss curvature is zero."
}], "Text",
 CellChangeTimes->{{3.589959160829546*^9, 3.5899591830038147`*^9}, {
   3.5899592232671175`*^9, 3.5899593889235926`*^9}, {3.5899594189633102`*^9, 
   3.58995943381116*^9}, {3.589959612019353*^9, 3.589959622683963*^9}, 
   3.5899785101376567`*^9, {3.5900420955373964`*^9, 3.590042246961057*^9}, 
   3.617972882304839*^9}]
}, Closed]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["3. The Second Fundamental Form", "Section",
 CellChangeTimes->{{3.590213457675046*^9, 3.5902135002631207`*^9}, 
   3.5902137672731895`*^9, {3.5905025808142767`*^9, 3.5905025824834795`*^9}, 
   3.5907394960471115`*^9, {3.6112125916717277`*^9, 3.6112126039264283`*^9}, {
   3.611212757083188*^9, 3.6112127596153336`*^9}, 3.6152635866689405`*^9, 
   3.61527332575016*^9}],

Cell[CellGroupData[{

Cell["3.1. The Definition ", "Subsection",
 CellChangeTimes->{{3.5906466020506277`*^9, 3.590646615061051*^9}, {
   3.5906466475091076`*^9, 3.5906466547319202`*^9}, {3.6112127683308315`*^9, 
   3.6112127708729773`*^9}, 3.611213166180588*^9, {3.6179741428428288`*^9, 
   3.617974156080586*^9}, 3.6180341576632977`*^9, 3.6279029363281584`*^9, {
   3.6279070864876823`*^9, 3.627907086968285*^9}, {3.6279073204952974`*^9, 
   3.62790733313132*^9}, 3.6279079091849365`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";"}]], "Input",
 CellChangeTimes->{{3.6180338973768406`*^9, 3.6180339102624636`*^9}}],

Cell[TextData[{
 "Following ",
 ButtonBox["[B-R]",
  BaseStyle->"Hyperlink",
  ButtonData->"B-R"],
 ",(61..8) we introduce now the ",
 StyleBox["second fundamental form",
  FontColor->RGBColor[0, 0, 1]],
 ":"
}], "Text",
 CellChangeTimes->{{3.5906404910830946`*^9, 3.590640514810736*^9}, {
  3.5906405514552007`*^9, 3.590640594776477*^9}, {3.5906406796562257`*^9, 
  3.5906406796562257`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"fund2", "[", "ff_", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], "[", 
   RowBox[{"x_List", ",", "y_List"}], "]"}], ":=", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"d", "[", "ff", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ".", "x"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"d", "[", 
       RowBox[{
        RowBox[{"amf", "[", "3", "]"}], "[", "ff", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ".", "y"}], ")"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.590214197740346*^9, 3.590214224307193*^9}, {
   3.5902142903889084`*^9, 3.590214400400302*^9}, {3.590218010839443*^9, 
   3.590218051961116*^9}, {3.5902180860783753`*^9, 3.5902181389780684`*^9}, {
   3.5904956764293494`*^9, 3.5904956764293494`*^9}, {3.5905011101060934`*^9, 
   3.5905011431313515`*^9}, {3.5907157938532677`*^9, 3.59071579522607*^9}, 
   3.6148618585051374`*^9, 3.6149177903449583`*^9, 3.6152619715185585`*^9, 
   3.622699047316374*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "fund2"}]], "Input",
 CellChangeTimes->{{3.618026345918378*^9, 3.618026378569235*^9}, {
   3.6180264185365047`*^9, 3.6180265050698566`*^9}, 3.618026535786311*^9, {
   3.618026569950371*^9, 3.618026638153691*^9}, {3.622699083212036*^9, 
   3.622699098281663*^9}}],

Cell[BoxData[
 StyleBox["\<\"fund2[f][u,v][xx,yy] is the value of the second fundamental \
form of \\nthe surface f[u,v] in the Eucliddean 3-space for the tangent \
vectors xx,yy.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.627907445012117*^9},
 CellTags->"Info3627911044-4669561"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Context", "[", "fund2", "]"}]], "Input",
 CellChangeTimes->{{3.6226991761414*^9, 3.622699201304244*^9}}],

Cell[BoxData["\<\"eudiffgeov2`\"\>"], "Output",
 CellChangeTimes->{3.62790746350915*^9}]
}, Closed]],

Cell[TextData[{
 "Clearly, fund2 is invariant with respect to orientation preserving \
Euclidean motions. It changes the sign if the orientation of the space or the \
orientation of the surface (direction of amf[3][sf])  changes.  It is a field \
of bilinear forms with  linear transformation under parameter \
transformations. Thus, as well as the first fundamental form, it defines a \
field of twofold covariant tensors on the surface. The coordinates of this \
tensor with respect to the local cobasis \[Sigma][1], \[Sigma][2]  are the \
coefficients cf[i,j], considered already in subsection ",
 ButtonBox["2.2.3",
  BaseStyle->"Hyperlink",
  ButtonData->"cij"],
 ":"
}], "Text",
 CellChangeTimes->{{3.590640970830737*^9, 3.5906414247915344`*^9}, {
   3.5906420395730143`*^9, 3.590642063846657*^9}, {3.5907159302287073`*^9, 
   3.5907160598961353`*^9}, {3.5907161361179314`*^9, 3.5907161917331123`*^9}, 
   3.6112129366894617`*^9, 3.6148618585987377`*^9, 3.6149177904385586`*^9, 
   3.6152619732326565`*^9, {3.61526726018423*^9, 3.615267267071624*^9}, 
   3.617972051489319*^9, 3.6180344092752137`*^9, {3.622613841349663*^9, 
   3.6226138600696955`*^9}, {3.62790750892583*^9, 3.627907528646265*^9}, 
   3.6279075760139484`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"xx", "=", " ", 
   RowBox[{"{", 
    RowBox[{"s1", ",", "s2"}], "}"}]}], ";", " ", 
  RowBox[{"yy", "=", 
   RowBox[{"{", 
    RowBox[{"t1", ",", "t2"}], "}"}]}], ";", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"fund2", "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", 
      RowBox[{"xx", ",", "yy"}], "]"}], "-", 
     RowBox[{"Sum", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"cf", "[", 
           RowBox[{"i", ",", "j"}], "]"}], "[", "sf", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "*", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[Sigma]", "[", "i", "]"}], "[", "sf", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], ".", "xx"}], ")"}], "*", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[Sigma]", "[", "j", "]"}], "[", "sf", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], ".", "yy"}], ")"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "2"}], "}"}]}], "]"}]}], ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"u", "\[Element]", " ", "Reals"}], ",", 
      RowBox[{"v", "\[Element]", " ", "Reals"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5902162888928185`*^9, 3.5902165073244023`*^9}, {
   3.5902167767524757`*^9, 3.590216796829711*^9}, {3.590217454916867*^9, 
   3.590217511061365*^9}, {3.5902175775954823`*^9, 3.5902176033823276`*^9}, 
   3.5902178500811605`*^9, {3.590218591566063*^9, 3.590218738206321*^9}, 
   3.5902188211048665`*^9, 3.5902198527502785`*^9, 3.590220631597246*^9, 
   3.59049567686615*^9, {3.5904994302171426`*^9, 3.5904995144260902`*^9}, {
   3.5904995882298203`*^9, 3.5904996219258795`*^9}, {3.5904996601615467`*^9, 
   3.5904996603643465`*^9}, {3.5904997007840176`*^9, 3.590499704403224*^9}, 
   3.5905023971551538`*^9, {3.590502450304447*^9, 3.590502485420109*^9}, {
   3.5906415477197504`*^9, 3.5906415910566263`*^9}, {3.590818432344045*^9, 
   3.5908184468832707`*^9}, 3.6179720537374477`*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6279078063697553`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "cf"}]], "Input",
 CellChangeTimes->{{3.6226140882668962`*^9, 3.6226140906693*^9}}],

Cell[BoxData[
 StyleBox["\<\"cf[i,j][f][u,v] are the coefficients  of the second \
fundamental form \\nwith respect to the cobasis {\[Sigma][i][f][u,v]} of the \
surface f[u,v] in the Euclidean 3-space.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.627907813155767*^9},
 CellTags->"Info3627911413-4669561"]
}, Closed]],

Cell[TextData[{
 "The bilinear form fund2 is ",
 StyleBox["symmetric",
  FontColor->RGBColor[0, 0, 1]],
 ":"
}], "Text",
 CellChangeTimes->{{3.5906417309108725`*^9, 3.590641761923727*^9}, 
   3.5906418653519087`*^9, 3.5907162291482525`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"fund2", "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "[", 
     RowBox[{"xx", ",", "yy"}], "]"}], "-", 
    RowBox[{
     RowBox[{
      RowBox[{"fund2", "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "[", 
     RowBox[{"yy", ",", "xx"}], "]"}]}], ",", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "\[Element]", " ", "Reals"}], ",", 
     RowBox[{"v", "\[Element]", " ", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.590214680093193*^9, 3.5902147816961718`*^9}, {
  3.5904956764449496`*^9, 3.5904956764449496`*^9}, {3.590641834853855*^9, 
  3.5906418500950813`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.627907826809791*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"cf", "[", 
       RowBox[{"1", ",", "2"}], "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "-", 
    RowBox[{
     RowBox[{
      RowBox[{"cf", "[", 
       RowBox[{"2", ",", "1"}], "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], ",", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "\[Element]", " ", "Reals"}], ",", 
     RowBox[{"v", "\[Element]", " ", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.590214680093193*^9, 3.5902147816961718`*^9}, {
  3.5904956764449496`*^9, 3.5904956764449496`*^9}, {3.590641834853855*^9, 
  3.5906418500950813`*^9}, {3.6226141555186143`*^9, 3.6226142522699842`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6279078865316963`*^9}]
}, Closed]],

Cell[TextData[{
 "We remark that very often the second fundamental form is defined with the \
opposite sign, see e.g. Alfred  Gray [",
 ButtonBox["G06",
  BaseStyle->"Hyperlink",
  ButtonData->"Gray"],
 "], or Erwin Kreyszig [",
 ButtonBox["Kr",
  BaseStyle->"Hyperlink",
  ButtonData->"Kr"],
 "], formula (38.6), p 146."
}], "Text",
 CellChangeTimes->{{3.590642754147869*^9, 3.59064281138437*^9}, {
   3.590643336512492*^9, 3.5906434798299437`*^9}, {3.5906435852705293`*^9, 
   3.590643647140238*^9}, 3.590644042975333*^9, {3.5991973443311057`*^9, 
   3.5991973496114078`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["3.2. Gaussian Curvature and Second Fundamental Form", "Subsection",
 CellChangeTimes->{{3.5906466724535513`*^9, 3.5906467470684824`*^9}, 
   3.590671674118125*^9, 3.6112131733619986`*^9, 3.6114693957095017`*^9, 
   3.6152733312304735`*^9, 3.6152757102455454`*^9, 3.6180344955531483`*^9, 
   3.6180373458971786`*^9, 3.627907951651211*^9, 3.6279130329859734`*^9}],

Cell["\<\
For the Gaussian curvature we obtain the formula, see [B-R], (61.12):\
\>", "Text",
 CellChangeTimes->{{3.590644144063511*^9, 3.5906441679783525`*^9}, {
  3.5906444821005044`*^9, 3.590644503784542*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"gauss", "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], " ", "-", " ", 
    RowBox[{"Det", "[", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"cf", "[", 
          RowBox[{"i", ",", "j"}], "]"}], "[", "sf", "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "\[Element]", " ", "Reals"}], ",", 
     RowBox[{"v", "\[Element]", " ", "Reals"}]}], "}"}], ",", 
   RowBox[{"TimeConstraint", " ", "\[Rule]", " ", "Infinity"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5906441721591597`*^9, 3.59064420053561*^9}, {
   3.590644236665273*^9, 3.5906442773657446`*^9}, 3.5906443350702457`*^9, {
   3.6114672135436893`*^9, 3.611467262235474*^9}, 3.614861858629938*^9, 
   3.6148635015936236`*^9, 3.617972055328539*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.627908552911071*^9}]
}, Closed]],

Cell[TextData[{
 "Remember that the definition of the Gaussian curvature is the ratio of two \
alternating biforms, invariant under Euclidean motions and parameter \
transformations. In the local basis \[Sigma][1], \[Sigma][2] the strict \
coordinate of  \[Sigma][1] \[And] \[Sigma][2] is 1, and the formula just \
calculated follows immediately from the ",
 ButtonBox["definition",
  BaseStyle->"Hyperlink",
  ButtonData->"gaussK"],
 "."
}], "Text",
 CellChangeTimes->{{3.590738951294155*^9, 3.5907390645503535`*^9}, {
   3.59073910233362*^9, 3.590739262452301*^9}, {3.590739416346572*^9, 
   3.5907394544574385`*^9}, {3.618034565778165*^9, 3.6180345719545183`*^9}, 
   3.6180373847834024`*^9}],

Cell[TextData[{
 "The coordinates of the second fundamental form with respect to the local ",
 "cobasis",
 " ",
 ButtonBox["du, dv",
  BaseStyle->"Hyperlink",
  ButtonData->"dudv"],
 " are"
}], "Text",
 CellChangeTimes->{{3.5907165401090384`*^9, 3.5907165927970514`*^9}, {
   3.5907167704052105`*^9, 3.5907168262464046`*^9}, 3.618034459904109*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fd2", "[", 
      RowBox[{"1", ",", "1"}], "]"}], "[", "ff_", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"fund2", "[", "ff", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fd2", "[", 
      RowBox[{"1", ",", "2"}], "]"}], "[", "ff_", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"fund2", "[", "ff", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fd2", "[", 
      RowBox[{"2", ",", "2"}], "]"}], "[", "ff_", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"fund2", "[", "ff", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fd2", "[", 
      RowBox[{"2", ",", "1"}], "]"}], "[", "ff_", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"fund2", "[", "ff", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0"}], "}"}]}], "]"}]}], ";"}]}], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5906718894921036`*^9, 3.5906720131535206`*^9}, {
   3.590719635842104*^9, 3.590719636032115*^9}, 3.5907196778325057`*^9, 
   3.618035398484793*^9, 3.6180354370499988`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "fd2"}]], "Input",
 CellChangeTimes->{{3.6180354694598527`*^9, 3.6180357108316584`*^9}, 
   3.6180359090899982`*^9, {3.6226953261836953`*^9, 3.6226953478989334`*^9}}],

Cell[BoxData[
 StyleBox["\<\"{fd2[i,j][f][u,v], i,j = 1, 2} is the matrix of the \
coefficients  \\nwith respect to the local standard basis {du,dv} of the \
second fundamental form \\nof the surface f[u,v] in the Euclidean \
3-space.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.627910688478037*^9},
 CellTags->"Info3627914288-4669561"]
}, Closed]],

Cell[TextData[StyleBox["The determinant of the matrix {fd2[i,j]} is the \
product of determinants",
 FontColor->RGBColor[0, 0, 1]]], "Text",
 CellChangeTimes->{{3.590716987076603*^9, 3.590717076748732*^9}, {
  3.592113156600597*^9, 3.5921131570841975`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Det", "[", 
   RowBox[{"{", 
    RowBox[{"fd2", "[", 
     RowBox[{"i", ",", "j"}], "]"}], "}"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"Det", "[", 
    RowBox[{"{", 
     RowBox[{"cf", "[", 
      RowBox[{"i", ",", "j"}], "]"}], "}"}], "]"}], "*", 
   RowBox[{"Det", "[", 
    RowBox[{"{", 
     RowBox[{"rgT", "[", 
      RowBox[{"i", ",", "j"}], "]"}], "}"}], "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5907170998510537`*^9, 3.5907171567763095`*^9}, {
   3.590719718049806*^9, 3.5907197185128326`*^9}, 3.617628080916442*^9, 
   3.6179720574146585`*^9}],

Cell[TextData[{
 StyleBox["Proof",
  FontColor->RGBColor[0, 0, 1]],
 ":"
}], "Text",
 CellChangeTimes->{{3.5907172529178085`*^9, 3.5907172560609884`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Det", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"fd2", "[", 
         RowBox[{"i", ",", "j"}], "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}], "==", 
   RowBox[{
    RowBox[{"Det", "[", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"cf", "[", 
          RowBox[{"i", ",", "j"}], "]"}], "[", "sf", "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}], "*", 
    RowBox[{"Det", "[", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"rgT", "[", 
          RowBox[{"i", ",", "j"}], "]"}], "[", "sf", "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5906723438117013`*^9, 3.5906724411714725`*^9}, {
   3.590672471809926*^9, 3.5906725129315987`*^9}, {3.59071972720533*^9, 
   3.59071972808438*^9}, {3.5907368705001*^9, 3.5907368707185*^9}, {
   3.614861858629938*^9, 3.6148618586455383`*^9}, {3.6148635016092234`*^9, 
   3.6148635016092234`*^9}, 3.6176280830785656`*^9, 3.617972058857741*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6279111657208776`*^9}]
}, Closed]],

Cell["Thus it follows the general formula, see [B-R], (61.12)", "Text",
 CellChangeTimes->{{3.5909099752525454`*^9, 3.590909987588251*^9}, {
   3.5909888126325254`*^9, 3.590988812960126*^9}, {3.5909889041422863`*^9, 
   3.59098893481194*^9}, 3.5909890097856717`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"gauss", "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], " ", "-", 
   RowBox[{
    RowBox[{"Det", "[", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"fd2", "[", 
          RowBox[{"i", ",", "j"}], "]"}], "[", "sf", "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
       RowBox[{"{", 
        RowBox[{"i", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}], "/", 
    RowBox[{"Det", "[", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"rgT", "[", 
          RowBox[{"i", ",", "j"}], "]"}], "[", "sf", "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
       RowBox[{"{", 
        RowBox[{"i", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.59091000400919*^9, 3.590910046910644*^9}, {
   3.590910117662691*^9, 3.5909101521126614`*^9}, 3.6114690139486666`*^9, {
   3.6148618586455383`*^9, 3.614861858661138*^9}, {3.614863501640424*^9, 
   3.614863501640424*^9}, 3.617628084765662*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.627911452696184*^9}]
}, Closed]],

Cell[TextData[{
 StyleBox["Example",
  FontColor->RGBColor[0, 0, 1]],
 ". ",
 StyleBox["The sphere of radius r",
  FontColor->RGBColor[0, 0, 1]],
 ":"
}], "Text",
 CellChangeTimes->{{3.590645549360779*^9, 3.5906455664584093`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Det", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"fd2", "[", 
         RowBox[{"i", ",", "j"}], "]"}], "[", 
        RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}], "-", 
   RowBox[{
    RowBox[{"Det", "[", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"cf", "[", 
          RowBox[{"i", ",", "j"}], "]"}], "[", 
         RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}], "*", 
    RowBox[{"Det", "[", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"rgT", "[", 
          RowBox[{"i", ",", "j"}], "]"}], "[", 
         RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5906723438117013`*^9, 3.5906724411714725`*^9}, {
   3.590672471809926*^9, 3.5906725129315987`*^9}, {3.5906725949253426`*^9, 
   3.5906726627230616`*^9}, {3.590673581970276*^9, 3.5906735987247057`*^9}, {
   3.5907197413651395`*^9, 3.5907197418371663`*^9}, {3.5907368468192587`*^9, 
   3.5907368597516813`*^9}, {3.590738130810714*^9, 3.5907381619795685`*^9}, {
   3.614861858661138*^9, 3.614861858661138*^9}, {3.614863501656024*^9, 
   3.6148635016716237`*^9}, 3.6176280865107617`*^9, 3.617972060681845*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6279129185975714`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"cf", "[", 
        RowBox[{"i", ",", "j"}], "]"}], "[", 
       RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "2"}], "}"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"r", ">", "0"}], ",", " ", 
     RowBox[{
      RowBox[{"Cos", "[", "v", "]"}], ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5906720266943445`*^9, 3.5906720991096716`*^9}, {
   3.5906721323533297`*^9, 3.590672139934943*^9}, 3.5906722123658705`*^9, {
   3.5907382355024977`*^9, 3.590738290320994*^9}, 3.614861858676738*^9, 
   3.614863501687224*^9, 3.6179720618959146`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     FractionBox["1", "r"], ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", 
     FractionBox["1", "r"]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627912944309417*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fund2", "[", 
      RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{"xx", ",", "xx"}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "\[Element]", " ", "Reals"}], ",", 
     RowBox[{"v", "\[Element]", " ", "Reals"}], ",", 
     RowBox[{
      RowBox[{"Cos", "[", "v", "]"}], ">", "0"}], ",", 
     RowBox[{"r", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.590220048343422*^9, 3.5902200805418787`*^9}, {
   3.590220507904629*^9, 3.590220528902266*^9}, 3.5902205880731697`*^9, {
   3.5902221438171024`*^9, 3.5902221985575986`*^9}, 3.5904956768973503`*^9, 
   3.5906439031210876`*^9, 3.5906439367703466`*^9}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], " ", "r", " ", 
  RowBox[{"(", 
   RowBox[{
    SuperscriptBox["s1", "2"], "+", 
    RowBox[{"2", " ", 
     SuperscriptBox["s2", "2"]}], "+", 
    RowBox[{
     SuperscriptBox["s1", "2"], " ", 
     RowBox[{"Cos", "[", 
      RowBox[{"2", " ", "v"}], "]"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.6279129536538334`*^9}]
}, Closed]],

Cell["Thus we see", "Text",
 CellChangeTimes->{{3.5906455044951*^9, 3.59064551025151*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"fund2", "[", 
       RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "[", 
     RowBox[{"xx", ",", "xx"}], "]"}], "\[Equal]", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "/", "r"}], ")"}], "*", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"fund1", "[", 
        RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", 
      RowBox[{"xx", ",", "xx"}], "]"}]}]}], ",", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "\[Element]", " ", "Reals"}], ",", 
     RowBox[{"v", "\[Element]", " ", "Reals"}], ",", 
     RowBox[{
      RowBox[{"Cos", "[", "v", "]"}], ">", "0"}], ",", 
     RowBox[{"r", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.590220048343422*^9, 3.5902200805418787`*^9}, {
   3.590220507904629*^9, 3.590220528902266*^9}, 3.5902205880731697`*^9, {
   3.5902221438171024`*^9, 3.5902221985575986`*^9}, 3.5904956768973503`*^9, 
   3.5906439031210876`*^9, 3.5906439367703466`*^9, {3.5906453634084525`*^9, 
   3.590645369289663*^9}, {3.590645419552951*^9, 3.590645467944236*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6279129713784647`*^9}]
}, Closed]],

Cell[TextData[{
 StyleBox["Example",
  FontColor->RGBColor[0, 0, 1]],
 ". The second fundamental forms for all ",
 StyleBox["developable surfaces",
  FontColor->RGBColor[0, 0, 1]],
 " degenerate. For the ",
 StyleBox["circular cones",
  FontColor->RGBColor[0, 0, 1]],
 " we have"
}], "Text",
 CellChangeTimes->{{3.5906456117920885`*^9, 3.590645688778224*^9}, {
  3.5906459738999243`*^9, 3.5906459850383444`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fund2", "[", 
      RowBox[{"cone", "[", "a", "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"s1", ",", "s2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"t1", ",", "t2"}], "}"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Sin", "[", 
       RowBox[{"a", "/", "2"}], "]"}], ">", "0"}], ",", 
     RowBox[{"u", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.590214476700036*^9, 3.590214562578187*^9}, 
   3.5904956764449496`*^9, {3.5904958754076986`*^9, 3.590495889354123*^9}, {
   3.5906437534388247`*^9, 3.590643762767641*^9}, {3.5906447896394444`*^9, 
   3.5906448165806923`*^9}, {3.590644867826782*^9, 3.590644932863296*^9}, 
   3.614861858676738*^9, 3.6148647317181845`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox["1", "2"]}], " ", "s2", " ", "t2", " ", "u", " ", 
  RowBox[{"Sin", "[", "a", "]"}]}]], "Output",
 CellChangeTimes->{3.6279129960265083`*^9}]
}, Closed]],

Cell["For the plane we have ", "Text",
 CellChangeTimes->{{3.5907384391608553`*^9, 3.590738465088101*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pl", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", 
  RowBox[{
   RowBox[{"plane", "[", 
    RowBox[{"a1", ",", "a2", ",", "b1", ",", "b2", ",", "c1", ",", "c2"}], 
    "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], " "}]], "Input",
 CellChangeTimes->{{3.5907384667573037`*^9, 3.5907384667573037`*^9}, {
   3.5907385125277843`*^9, 3.5907385253198066`*^9}, 3.590738675891271*^9, 
   3.614861858692338*^9, 3.6148626542469354`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fund2", "[", "pl", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"s1", ",", "s2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"t1", ",", "t2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5907385438214393`*^9, 3.5907385878915167`*^9}, 
   3.5907386904304967`*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.627913013782339*^9}]
}, Closed]],

Cell["\<\
Since the plane and the cones are locally isometric surfaces it is clear that \
the second fundamental form is not invariant under isometries; it doesn\
\[CloseCurlyQuote]t belong to the inner geometry of the surfaces.\
\>", "Text",
 CellChangeTimes->{{3.590738712442135*^9, 3.590738740100984*^9}, {
  3.590738784498662*^9, 3.5907388582555914`*^9}, {3.611469357789333*^9, 
  3.611469360549491*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
3.3.  An Equivalent Definition of the Second Fundamental Form\
\>", "Subsection",
 CellChangeTimes->{{3.590818296109006*^9, 3.590818381035555*^9}, 
   3.590910415997754*^9, 3.611469400327766*^9, 3.611471134269942*^9, 
   3.615275717765976*^9, 3.615276391918535*^9, 3.618037428649912*^9, 
   3.6180379274334407`*^9, 3.6279130679888353`*^9, 3.627913632095426*^9}],

Cell["\<\
The following definition is often used as the primary. We start with the  \
identity\
\>", "Text",
 CellChangeTimes->{{3.590818477116124*^9, 3.590818522715004*^9}, {
  3.590818558439067*^9, 3.590818570607088*^9}, {3.5908187338572936`*^9, 
  3.5908187527853765`*^9}, {3.590820605355337*^9, 3.590820657793337*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"xx", "=", " ", 
    RowBox[{"{", 
     RowBox[{"s1", ",", "s2"}], "}"}]}], ";", " ", 
   RowBox[{"yy", "=", 
    RowBox[{"{", 
     RowBox[{"u1", ",", "u2"}], "}"}]}], ";", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"d", "[", "sf", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], ".", "xx"}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"d", "[", 
          RowBox[{
           RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}], "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], ".", "yy"}], ")"}]}], "\[Equal]", 
     " ", 
     RowBox[{"-", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"d", "[", 
             RowBox[{"d", "[", "sf", "]"}], "]"}], "[", 
            RowBox[{"u", ",", "v"}], "]"}], ".", "xx"}], ")"}], ".", "yy"}], 
        ")"}], ".", 
       RowBox[{
        RowBox[{
         RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}]}]}], "]"}]}], "\n"}]], "Input",
 CellChangeTimes->{{3.6152760579194317`*^9, 3.6152760614466333`*^9}, {
  3.6180375677928705`*^9, 3.6180375998537045`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6279132442061453`*^9}]
}, Closed]],

Cell["\<\
and it follows the new expression for the second fundamental form\
\>", "Text",
 CellChangeTimes->{{3.5909086612622857`*^9, 3.5909086901379366`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fund2", "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{"xx", ",", "yy"}], "]"}], " ", "\[Equal]", " ", 
   RowBox[{"-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"d", "[", 
           RowBox[{"d", "[", "sf", "]"}], "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], ".", "xx"}], ")"}], ".", "yy"}], 
      ")"}], ".", 
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5908231690709734`*^9, 3.590823267325593*^9}, 
   3.590823302446602*^9, {3.5908233571987333`*^9, 3.5908233692144213`*^9}, {
   3.590823756951598*^9, 3.590823799359024*^9}, {3.5908238384552603`*^9, 
   3.590823914906633*^9}, {3.590905864085401*^9, 3.5909059654855795`*^9}, {
   3.5909082182371073`*^9, 3.590908268110395*^9}, {3.590908312008872*^9, 
   3.5909083780749884`*^9}, 3.614861858707938*^9, 3.6149177904853587`*^9, 
   3.6152622106372356`*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6279134761837473`*^9}]
}, Closed]],

Cell["For the coefficients fd2[i,j][sf][u,v] we obtain", "Text",
 CellChangeTimes->{{3.5909087665624704`*^9, 3.590908833970189*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"sf", "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"u", ",", "2"}], "}"}]}], "]"}], ".", 
    RowBox[{
     RowBox[{
      RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "\[Equal]", " ", 
   RowBox[{"-", 
    RowBox[{
     RowBox[{
      RowBox[{"fd2", "[", 
       RowBox[{"1", ",", "1"}], "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.590837626280514*^9, 3.5908376661217923`*^9}, {
   3.5908379583135047`*^9, 3.5908380325077486`*^9}, 3.614861858707938*^9, 
   3.6149177905009584`*^9, 3.6152622119333096`*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6279134932521777`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"sf", "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", "u", ",", "v"}], "]"}], ".", 
    RowBox[{
     RowBox[{
      RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "\[Equal]", " ", 
   RowBox[{"-", 
    RowBox[{
     RowBox[{
      RowBox[{"fd2", "[", 
       RowBox[{"1", ",", "2"}], "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.590837626280514*^9, 3.5908376661217923`*^9}, {
   3.5908379583135047`*^9, 3.5908380517038465`*^9}, {3.5908381960561028`*^9, 
   3.5908382285549617`*^9}, 3.614861858723538*^9, 3.6149177905165586`*^9, 
   3.6152622130683746`*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6279135055957994`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"sf", "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", "v", ",", "u"}], "]"}], ".", 
    RowBox[{
     RowBox[{
      RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "\[Equal]", " ", 
   RowBox[{"-", 
    RowBox[{
     RowBox[{
      RowBox[{"fd2", "[", 
       RowBox[{"2", ",", "1"}], "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.590837626280514*^9, 3.5908376661217923`*^9}, {
   3.5908379583135047`*^9, 3.5908380517038465`*^9}, {3.5908381960561028`*^9, 
   3.590838288707402*^9}, 3.614861858723538*^9, 3.6149177905165586`*^9, 
   3.6152622141574373`*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.627913505783*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"sf", "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"v", ",", "2"}], "}"}]}], "]"}], ".", 
    RowBox[{
     RowBox[{
      RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "\[Equal]", " ", 
   RowBox[{"-", 
    RowBox[{
     RowBox[{
      RowBox[{"fd2", "[", 
       RowBox[{"2", ",", "2"}], "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.590837626280514*^9, 3.5908376661217923`*^9}, {
   3.5908379583135047`*^9, 3.5908380517038465`*^9}, {3.5908381960561028`*^9, 
   3.5908383295867405`*^9}, 3.614861858723538*^9, 3.614917790532159*^9, 
   3.6152622154055085`*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6279135060014*^9}]
}, Closed]],

Cell["Using the cobasis du, dv we may write", "Text",
 CellChangeTimes->{{3.590909029266932*^9, 3.590909052214572*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{"sf", "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", "u", ",", "u"}], "]"}], ".", 
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "*", 
    RowBox[{
     RowBox[{"d", "[", 
      RowBox[{"u", ",", "u", ",", "v"}], "]"}], "[", "xx", "]"}], "*", 
    RowBox[{
     RowBox[{"d", "[", 
      RowBox[{"u", ",", "u", ",", "v"}], "]"}], "[", "yy", "]"}]}], "+", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{"sf", "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", "u", ",", "v"}], "]"}], ".", 
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "*", 
    RowBox[{
     RowBox[{"d", "[", 
      RowBox[{"u", ",", "u", ",", "v"}], "]"}], "[", "xx", "]"}], "*", 
    RowBox[{
     RowBox[{"d", "[", 
      RowBox[{"v", ",", "u", ",", "v"}], "]"}], "[", "yy", "]"}]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{"sf", "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", "v", ",", "u"}], "]"}], ".", 
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "*", 
    RowBox[{
     RowBox[{"d", "[", 
      RowBox[{"v", ",", "u", ",", "v"}], "]"}], "[", "xx", "]"}], "*", 
    RowBox[{
     RowBox[{"d", "[", 
      RowBox[{"u", ",", "u", ",", "v"}], "]"}], "[", "yy", "]"}]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{"sf", "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", "v", ",", "v"}], "]"}], ".", 
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "*", 
    RowBox[{
     RowBox[{"d", "[", 
      RowBox[{"v", ",", "u", ",", "v"}], "]"}], "[", "xx", "]"}], "*", 
    RowBox[{
     RowBox[{"d", "[", 
      RowBox[{"v", ",", "u", ",", "v"}], "]"}], "[", "yy", "]"}]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"fund2", "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{"xx", ",", "yy"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.590838820986847*^9, 3.5908389558845625`*^9}, {
   3.590838990723555*^9, 3.5908390265996075`*^9}, {3.5908390586154385`*^9, 
   3.5908390676709566`*^9}, {3.5908391213030243`*^9, 
   3.5908391710558696`*^9}, {3.5908392565527596`*^9, 3.590839257639822*^9}, {
   3.590839445520568*^9, 3.59083947264812*^9}, 3.5908415910502853`*^9, {
   3.611470519818797*^9, 3.61147052284297*^9}, {3.611470586316601*^9, 
   3.6114705918189154`*^9}, {3.6114708904849977`*^9, 
   3.6114708942192116`*^9}, {3.611470943300019*^9, 3.6114709523105345`*^9}, {
   3.6148618587391376`*^9, 3.614861858754738*^9}, {3.614917790547759*^9, 
   3.614917790563359*^9}, {3.615262216609577*^9, 3.615262220241785*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.627913566859909*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fund2", "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{"xx", ",", "yy"}], "]"}], " ", "-", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"fd2", "[", 
         RowBox[{"i", ",", "j"}], "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "*", 
      RowBox[{"xx", "[", 
       RowBox[{"[", "i", "]"}], "]"}], "*", 
      RowBox[{"yy", "[", 
       RowBox[{"[", "j", "]"}], "]"}]}], ",", " ", 
     RowBox[{"{", 
      RowBox[{"i", ",", "2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "2"}], "}"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5908401746002693`*^9, 3.5908402249931517`*^9}, {
   3.5908402920429864`*^9, 3.5908403707404876`*^9}, {3.5908404013022356`*^9, 
   3.5908404929334764`*^9}, 3.590840842709483*^9, 3.5908409435162487`*^9, {
   3.5908411319660273`*^9, 3.59084131367642*^9}, {3.590841362764228*^9, 
   3.590841376276001*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6279135946015596`*^9}]
}, Closed]],

Cell["For possible future applications we define", "Text",
 CellChangeTimes->{{3.5909091109174757`*^9, 3.590909168762377*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "fund2a", "]"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fund2a", "[", "ff_", "]"}], "[", 
     RowBox[{"u_", ",", "v_"}], "]"}], "[", 
    RowBox[{"x_List", ",", "y_List"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"d", "[", 
            RowBox[{"d", "[", "ff", "]"}], "]"}], "[", 
           RowBox[{"u1", ",", "v1"}], "]"}], "/.", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"u1", "\[Rule]", "u"}], ",", 
            RowBox[{"v1", "\[Rule]", "v"}]}], "}"}]}], ")"}], ".", "x"}], 
       ")"}], ".", "y"}], ")"}], ".", 
    RowBox[{
     RowBox[{
      RowBox[{"amf", "[", "3", "]"}], "[", "ff", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5908418218494864`*^9, 3.590841851618189*^9}, {
   3.590845349630325*^9, 3.590845703112543*^9}, {3.590907786584349*^9, 
   3.590907800437174*^9}, {3.590907905596958*^9, 3.5909079379046154`*^9}, {
   3.614057154177955*^9, 3.6140572705541596`*^9}, 3.614861858754738*^9, 
   3.6149177905789585`*^9, 3.615262223526973*^9}],

Cell["Regard that fund2 and fund2a differ by the sign only:", "Text",
 CellChangeTimes->{{3.5909093606603928`*^9, 3.590909391316146*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fund2a", "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", " ", "v"}], "]"}], "[", 
    RowBox[{"xx", ",", "yy"}], "]"}], " ", "+", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"fund2", "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", " ", "v"}], "]"}], "[", 
    RowBox[{"xx", ",", "yy"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5908457241747475`*^9, 3.5908458264865994`*^9}, {
   3.5908458587194433`*^9, 3.59084586740694*^9}, 3.590846092119793*^9, {
   3.5908461398635235`*^9, 3.590846173752462*^9}, {3.59090782689482*^9, 
   3.5909078458800535`*^9}, 3.590907989353505*^9, 3.61803783090792*^9, 
   3.6180378859720693`*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6279136176760006`*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["3.4. The Curvature Vector", "Subsection",
 CellChangeTimes->{{3.5911636991239314`*^9, 3.591163726106475*^9}, 
   3.6114711409343233`*^9, 3.611640870360384*^9, 3.615276399973996*^9, 
   3.615434797034876*^9, 3.618037933137767*^9, 3.618406755097189*^9, {
   3.6246867791918316`*^9, 3.624686781531836*^9}, 3.6251124003687673`*^9, {
   3.6254878485050135`*^9, 3.6254878765266633`*^9}, {3.627917374326639*^9, 
   3.6279173763234425`*^9}, 3.6279748461554165`*^9}],

Cell[CellGroupData[{

Cell["3.4.1. Curves on Surfaces", "Subsubsection",
 CellChangeTimes->{{3.625112377592727*^9, 3.6251123907903504`*^9}}],

Cell["\<\
Consider a general curve on the surface (or a two-dimensional manifold) with \
parameter s\
\>", "Text",
 CellChangeTimes->{{3.591163790323148*^9, 3.5911637979785852`*^9}, {
   3.5911638677065735`*^9, 3.591163893043023*^9}, 3.615276463125608*^9, {
   3.618293875822893*^9, 3.6182939377394023`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Gamma]", "[", "s_", "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"hu", "[", "s", "]"}], ",", 
    RowBox[{"hv", "[", "s", "]"}]}], "}"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5911640806537533`*^9, 3.5911641202370176`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]], "Input",
 CellChangeTimes->{{3.6254682628083925`*^9, 3.625468273322811*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SuperscriptBox["hu", "\[Prime]",
     MultilineFunction->None], "[", "s", "]"}], ",", 
   RowBox[{
    SuperscriptBox["hv", "\[Prime]",
     MultilineFunction->None], "[", "s", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627973459562581*^9}]
}, Closed]],

Cell["Such a curve on the surface ff[u,v] is given by", "Text",
 CellChangeTimes->{{3.6182945196204243`*^9, 3.6182945562960887`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"cxx", "[", "ff_", "]"}], "[", "s_", "]"}], " ", ":=", 
  RowBox[{"ff", "[", 
   RowBox[{
    RowBox[{"hu", "[", "s", "]"}], ",", 
    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.618294110182105*^9, 3.6182941903194456`*^9}, 
   3.618294366069354*^9, 3.6246953556850033`*^9}],

Cell["For the non specified general surface we set", "Text",
 CellChangeTimes->{{3.618294651565456*^9, 3.618294677352301*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"cx", "[", "s_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"cxx", "[", "sf", "]"}], "[", "s", "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5911639056057415`*^9, 3.5911639586847773`*^9}, 
   3.5911644865179677`*^9, 3.611471276833096*^9, {3.6182942572903633`*^9, 
   3.6182943057128487`*^9}, 3.618294381731782*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"cx", "[", "s", "]"}]], "Input",
 CellChangeTimes->{{3.6182943281612883`*^9, 3.6182943327788963`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "1", "]"}], "[", 
    RowBox[{
     RowBox[{"hu", "[", "s", "]"}], ",", 
     RowBox[{"hv", "[", "s", "]"}]}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "2", "]"}], "[", 
    RowBox[{
     RowBox[{"hu", "[", "s", "]"}], ",", 
     RowBox[{"hv", "[", "s", "]"}]}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "3", "]"}], "[", 
    RowBox[{
     RowBox[{"hu", "[", "s", "]"}], ",", 
     RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6279735037730584`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"cx", "'"}], "[", "s", "]"}]], "Input",
 CellChangeTimes->{{3.6182943281612883`*^9, 3.6182943327788963`*^9}, {
  3.6254683143508835`*^9, 3.6254683271585054`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["hv", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
    RowBox[{
     RowBox[{
      SuperscriptBox["hu", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["hv", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
    RowBox[{
     RowBox[{
      SuperscriptBox["hu", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["hv", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
    RowBox[{
     RowBox[{
      SuperscriptBox["hu", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6279735084998665`*^9}]
}, Closed]],

Cell["\<\
A parameter s is  the arc length parameter,  if the following condition is \
satisfied\
\>", "Text",
 CellChangeTimes->{{3.5911640248865643`*^9, 3.5911640470268307`*^9}, {
   3.6152764886770697`*^9, 3.615276498893654*^9}, {3.618294718099573*^9, 
   3.618294723200782*^9}, {3.6182948073161297`*^9, 3.6182949029754972`*^9}, 
   3.6250336541598353`*^9, {3.6252310696115417`*^9, 3.6252310712651443`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"cx", "'"}], "[", "s", "]"}], ".", 
    RowBox[{
     RowBox[{"cx", "'"}], "[", "s", "]"}]}], " ", "==", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"fund1", "[", "sf", "]"}], "[", 
     RowBox[{
      RowBox[{"hu", "[", "s", "]"}], ",", 
      RowBox[{"hv", "[", "s", "]"}]}], "]"}], "[", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}], ",", 
     RowBox[{
      RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.591164184167674*^9, 3.5911642643002577`*^9}, {
   3.5911643068696923`*^9, 3.591164405895356*^9}, 3.5911644724711647`*^9, {
   3.5911646118721375`*^9, 3.5911646213996825`*^9}, {3.5911647404074893`*^9, 
   3.59116474548778*^9}, {3.611471280412301*^9, 3.611471281483362*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.627973544021129*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fund1", "[", "sf", "]"}], "[", 
     RowBox[{
      RowBox[{"hu", "[", "s", "]"}], ",", 
      RowBox[{"hv", "[", "s", "]"}]}], "]"}], "[", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}], ",", 
     RowBox[{
      RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]}], "]"}], "\[Equal]", "1"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6252309845717916`*^9, 3.6252310031046247`*^9}}],

Cell["\<\
A general test for  a curve parameter to be the arc length is defined in the \
package eudiffgeov2.m:\
\>", "Text",
 CellChangeTimes->{{3.5911647999228935`*^9, 3.591164817426895*^9}, 
   3.61719839476588*^9, 3.6250336697286625`*^9, {3.62523110835041*^9, 
   3.6252311933121595`*^9}, {3.6252312302570243`*^9, 3.625231240039242*^9}, 
   3.6252363978051243`*^9, 3.62523643468559*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "assu"}]], "Input",
 CellChangeTimes->{{3.6252312475740547`*^9, 3.6252312495396585`*^9}}],

Cell[BoxData[
 StyleBox["\<\"assu[c][s] is a set of assumptions in the first four \
derivatives of the \\nparameter representation c[s] of a smooth curve in the \
n-dimensional Euclidean space \\ngiving True if s is the Euclidean arc length \
of the curve.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.627973560681958*^9},
 CellTags->"Info3627977160-2492953"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"assu", "[", "x", "]"}], "[", "s", "]"}], "[", 
  RowBox[{"[", "1", "]"}], "]"}]], "Input",
 CellChangeTimes->{3.625231297750744*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SuperscriptBox["x", "\[Prime]",
     MultilineFunction->None], "[", "s", "]"}], ".", 
   RowBox[{
    SuperscriptBox["x", "\[Prime]",
     MultilineFunction->None], "[", "s", "]"}]}], "\[Equal]", "1"}]], "Output",
 CellChangeTimes->{3.6279735811335945`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"assu", "[", "cx", "]"}], "[", "s", "]"}], "[", 
  RowBox[{"[", "1", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6251520146981506`*^9, 3.6251520764118586`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["hv", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{
         RowBox[{"hu", "[", "s", "]"}], ",", 
         RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox["hu", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{
         RowBox[{"hu", "[", "s", "]"}], ",", 
         RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["hv", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{
         RowBox[{"hu", "[", "s", "]"}], ",", 
         RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox["hu", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{
         RowBox[{"hu", "[", "s", "]"}], ",", 
         RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["hv", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{
         RowBox[{"hu", "[", "s", "]"}], ",", 
         RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox["hu", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{
         RowBox[{"hu", "[", "s", "]"}], ",", 
         RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
  "\[Equal]", "1"}]], "Output",
 CellChangeTimes->{3.6279735890272083`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"cspherx", "[", "s_", "]"}], " ", ":=", 
  RowBox[{
   RowBox[{"sphero", "[", "1", "]"}], "[", 
   RowBox[{
    RowBox[{"hu", "[", "s", "]"}], ",", 
    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.625223954903874*^9, 3.625224005323162*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"assu", "[", "cspherx", "]"}], "[", "s", "]"}], "[", 
  RowBox[{"[", "1", "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"Cos", "[", 
      RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
    SuperscriptBox[
     RowBox[{
      SuperscriptBox["hv", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], "2"]}], "+", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{"Cos", "[", 
         RowBox[{"hv", "[", "s", "]"}], "]"}]}], " ", 
       RowBox[{"Sin", "[", 
        RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["hu", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}]}], "-", 
      RowBox[{
       RowBox[{"Cos", "[", 
        RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
       RowBox[{"Sin", "[", 
        RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["hv", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}]}]}], ")"}], "2"], "+", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"Cos", "[", 
        RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
       RowBox[{"Cos", "[", 
        RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["hu", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}]}], "-", 
      RowBox[{
       RowBox[{"Sin", "[", 
        RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
       RowBox[{"Sin", "[", 
        RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["hv", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}]}]}], ")"}], "2"]}], 
  "\[Equal]", "1"}]], "Output",
 CellChangeTimes->{3.627973696043396*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"assu", "[", "cspherx", "]"}], "[", "s", "]"}], "[", 
     RowBox[{"[", "1", "]"}], "]"}], ",", 
    RowBox[{"Element", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"hu", "[", "s", "]"}], ",", 
        RowBox[{"hv", "[", "s", "]"}], ",", 
        RowBox[{
         RowBox[{"hu", "'"}], "[", "s", "]"}], ",", 
        RowBox[{
         RowBox[{"hv", "'"}], "[", "s", "]"}], ",", "s"}], "}"}], ",", 
      "Reals"}], "]"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6251526255172234`*^9, 3.6251527012865562`*^9}, {
   3.625221223261076*^9, 3.6252212246494784`*^9}, {3.625221359230915*^9, 
   3.625221392817774*^9}, 3.6252229737245502`*^9, {3.625224027724802*^9, 
   3.6252240326232104`*^9}, 3.625230376260124*^9, {3.625231401152725*^9, 
   3.625231404959132*^9}, {3.627973783169549*^9, 3.627973835913242*^9}, {
   3.627973904272562*^9, 3.6279739211361914`*^9}}],

Cell[BoxData["$Aborted"], "Output",
 CellChangeTimes->{3.6279736882433825`*^9, 3.6279738806073203`*^9, 
  3.627974437356698*^9}]
}, Closed]],

Cell["\<\
after waiting a long time.  assu has difficulties to be evaluated for \
symbolically defined  curves on certain specified surfaces. Thus we try \
another more direct definition of an arc length condition better adapted to \
general curves on surfaces:\
\>", "Text",
 CellChangeTimes->{{3.625234093019863*^9, 3.625234116065104*^9}, {
   3.625234164473989*^9, 3.625234315217455*^9}, {3.6252364755012617`*^9, 
   3.625236490524088*^9}, 3.6254684356566963`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "scond", "]"}], ";", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"scond", "[", "ff_", "]"}], "[", "s_", "]"}], ":=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"fund1", "[", "ff", "]"}], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}], "[", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}], ",", 
      RowBox[{
       RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]}], "]"}], "\[Equal]", " ", 
    "1"}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.625234324405871*^9, 3.6252343731639566`*^9}, {
  3.6252344399848747`*^9, 3.6252344762433386`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"scond", "[", 
    RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", "t", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.598944128504306*^9, 3.5989441321885166`*^9}, {
  3.6136199311429863`*^9, 3.613619945604212*^9}, {3.613620401281012*^9, 
  3.6136204109530287`*^9}, {3.61728282398886*^9, 3.617282859771202*^9}, {
  3.6252345050731893`*^9, 3.62523453399564*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"Cos", "[", 
      RowBox[{"hv", "[", "t", "]"}], "]"}], "2"], " ", 
    SuperscriptBox[
     RowBox[{
      SuperscriptBox["hu", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
   SuperscriptBox[
    RowBox[{
     SuperscriptBox["hv", "\[Prime]",
      MultilineFunction->None], "[", "t", "]"}], "2"]}], "\[Equal]", 
  "1"}]], "Output",
 CellChangeTimes->{3.6279744765439672`*^9}]
}, Closed]],

Cell["\<\
The  condition scond evaluates immediately, also for an unspecified general \
surface\
\>", "Text",
 CellChangeTimes->{{3.6252305236803837`*^9, 3.6252305403724127`*^9}, {
   3.6252306316723647`*^9, 3.6252306713286343`*^9}, {3.6252345876969347`*^9, 
   3.6252346438182335`*^9}, {3.6252347187687654`*^9, 3.6252347214675703`*^9}, 
   3.625468519803244*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"scond", "[", "sf", "]"}], "[", "s", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6251526255172234`*^9, 3.6251527012865562`*^9}, {
   3.625221223261076*^9, 3.6252212246494784`*^9}, {3.625221359230915*^9, 
   3.625221392817774*^9}, 3.6252229737245502`*^9, {3.625224027724802*^9, 
   3.6252240326232104`*^9}, {3.625230376260124*^9, 3.625230469735489*^9}, {
   3.6252346718710833`*^9, 3.625234672963085*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.26520169999999998466932993324007838964`5.444176216942424", ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["hv", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}], "+", 
     RowBox[{"2", " ", 
      RowBox[{
       SuperscriptBox["hu", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{
       SuperscriptBox["hv", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["hu", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}]}], 
    "\[Equal]", "1"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6279744928927956`*^9}]
}, Closed]],

Cell[TextData[{
 "Now the derivation of this equation must be zero; this is an analytic \
consequence not recognized automatically by ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ":"
}], "Text",
 CellChangeTimes->{{3.599025515164632*^9, 3.5990256061127915`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"fund1", "[", 
        RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
       RowBox[{
        RowBox[{"hu", "[", "s", "]"}], ",", 
        RowBox[{"hv", "[", "s", "]"}]}], "]"}], "[", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}], ",", 
       RowBox[{
        RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]}], "]"}], ",", "s"}], 
    "]"}], ",", 
   RowBox[{
    RowBox[{"scond", "[", 
     RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", "s", "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5989442396766644`*^9, 3.5989442862613287`*^9}, {
   3.6136206379490275`*^9, 3.6136206408506327`*^9}, {3.617282881985641*^9, 
   3.617282884887246*^9}, {3.618217879816814*^9, 3.6182178831708198`*^9}, 
   3.618218489091484*^9, {3.618295111095543*^9, 3.6182951433683887`*^9}, {
   3.6252348254029536`*^9, 3.625234827493357*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", 
    RowBox[{"Sin", "[", 
     RowBox[{"2", " ", 
      RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], " ", 
   SuperscriptBox[
    RowBox[{
     SuperscriptBox["hu", "\[Prime]",
      MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
   RowBox[{
    SuperscriptBox["hv", "\[Prime]",
     MultilineFunction->None], "[", "s", "]"}]}], "+", 
  RowBox[{"2", " ", 
   SuperscriptBox[
    RowBox[{"Cos", "[", 
     RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
   RowBox[{
    SuperscriptBox["hu", "\[Prime]",
     MultilineFunction->None], "[", "s", "]"}], " ", 
   RowBox[{
    SuperscriptBox["hu", "\[Prime]\[Prime]",
     MultilineFunction->None], "[", "s", "]"}]}], "+", 
  RowBox[{"2", " ", 
   RowBox[{
    SuperscriptBox["hv", "\[Prime]",
     MultilineFunction->None], "[", "s", "]"}], " ", 
   RowBox[{
    SuperscriptBox["hv", "\[Prime]\[Prime]",
     MultilineFunction->None], "[", "s", "]"}]}]}]], "Output",
 CellChangeTimes->{3.6279745218464465`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"fund1", "[", "sf", "]"}], "[", 
       RowBox[{
        RowBox[{"hu", "[", "s", "]"}], ",", 
        RowBox[{"hv", "[", "s", "]"}]}], "]"}], "[", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}], ",", 
       RowBox[{
        RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]}], "]"}], ",", "s"}], 
    "]"}], ",", 
   RowBox[{
    RowBox[{"scond", "[", "sf", "]"}], "[", "s", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5989442396766644`*^9, 3.5989442862613287`*^9}, {
   3.6136206379490275`*^9, 3.6136206408506327`*^9}, {3.617282881985641*^9, 
   3.617282884887246*^9}, {3.618217879816814*^9, 3.6182178831708198`*^9}, 
   3.618218489091484*^9, {3.618295111095543*^9, 3.6182951433683887`*^9}, {
   3.618295278430114*^9, 3.6182952966481557`*^9}, {3.6252348547964053`*^9, 
   3.6252348855928593`*^9}}],

Cell[BoxData[
 RowBox[{"2", " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["hv", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "3"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
    RowBox[{
     RowBox[{
      SuperscriptBox["hu", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["hv", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
    RowBox[{
     RowBox[{
      SuperscriptBox["hv", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["hv", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox["hu", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["hu", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{"2", " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{"2", " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"2", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"2", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"2", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], ")"}]}], 
    "+", 
    RowBox[{
     RowBox[{
      SuperscriptBox["hu", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["hv", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox["hu", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["hu", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"2", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"2", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"2", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], ")"}]}]}],
    ")"}]}]], "Output",
 CellChangeTimes->{3.627974560799715*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"Evaluate", "[", 
    RowBox[{
     RowBox[{"scond", "[", 
      RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", "s", "]"}], "]"}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"fund1", "[", "sf", "]"}], "[", 
        RowBox[{
         RowBox[{"hu", "[", "s", "]"}], ",", 
         RowBox[{"hv", "[", "s", "]"}]}], "]"}], "[", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}], ",", 
        RowBox[{
         RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]}], "]"}], ",", "s"}], 
     "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5989442396766644`*^9, 3.5989442862613287`*^9}, {
   3.6136206379490275`*^9, 3.6136206408506327`*^9}, {3.617282881985641*^9, 
   3.617282884887246*^9}, {3.618217879816814*^9, 3.6182178831708198`*^9}, 
   3.618218489091484*^9, {3.618295111095543*^9, 3.6182951433683887`*^9}, {
   3.618295278430114*^9, 3.6182952966481557`*^9}, {3.625061428050082*^9, 
   3.62506154631389*^9}, {3.6252349137705092`*^9, 3.625234916110513*^9}}],

Cell[BoxData[
 RowBox[{"2", " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["hv", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "3"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
    RowBox[{
     RowBox[{
      SuperscriptBox["hv", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["hv", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox["hu", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], ")"}]}], 
    "+", 
    RowBox[{
     RowBox[{
      SuperscriptBox["hu", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["hv", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox["hu", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}]}], 
      ")"}]}], "+", 
    RowBox[{
     RowBox[{
      SuperscriptBox["hu", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["hv", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["hu", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
     RowBox[{
      SuperscriptBox["hv", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["hu", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "3"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
   ")"}]}]], "Output",
 CellChangeTimes->{3.6279745725933356`*^9}]
}, Closed]],

Cell["\<\
Therefore we have to formulate the condition for the derivative explicitly \
and join it to the condition scond:\
\>", "Text",
 CellChangeTimes->{{3.5990256677016993`*^9, 3.599025684378129*^9}, 
   3.61728290995649*^9, 3.617441777844398*^9, {3.6252349592641892`*^9, 
   3.625235026054907*^9}, {3.625468841733322*^9, 3.6254688427785244`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Clear", "[", "scond2", "]"}], ";"}], " "}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"scond2", "[", "ff_", "]"}], "[", "s_", "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"scond", "[", "ff", "]"}], "[", "s", "]"}], ",", " ", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"Evaluate", "[", 
       RowBox[{
        RowBox[{"D", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"fund1", "[", "ff", "]"}], "[", 
            RowBox[{
             RowBox[{"hu", "[", "t", "]"}], ",", 
             RowBox[{"hv", "[", "t", "]"}]}], "]"}], "[", 
           RowBox[{
            RowBox[{
             RowBox[{"\[Gamma]", "'"}], "[", "t", "]"}], ",", 
            RowBox[{
             RowBox[{"\[Gamma]", "'"}], "[", "t", "]"}]}], "]"}], ",", "t"}], 
         "]"}], "/.", 
        RowBox[{"t", "->", "s"}]}], " ", "]"}], ")"}], "==", " ", "0"}]}], 
   "}"}]}], "\n", 
 RowBox[{"  "}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5989444311036134`*^9, 3.598944452091814*^9}, {
   3.598944491622075*^9, 3.5989445892626595`*^9}, {3.613620483305956*^9, 
   3.613620520215621*^9}, {3.6136205570784855`*^9, 3.6136205617428937`*^9}, {
   3.6173372915542564`*^9, 3.6173373145675726`*^9}, 3.618208860062937*^9, {
   3.6182090179508142`*^9, 3.6182090390576515`*^9}, {3.618209092534545*^9, 
   3.6182090956701508`*^9}, 3.6182091600046635`*^9, {3.6182091936227226`*^9, 
   3.6182092224515734`*^9}, {3.61820972123125*^9, 3.618209830852642*^9}, {
   3.6182099255604086`*^9, 3.618209961814872*^9}, {3.618211324789266*^9, 
   3.6182113780477595`*^9}, {3.6182114114942183`*^9, 3.618211437405864*^9}, 
   3.618211471148723*^9, {3.6182164338814745`*^9, 3.618216468357535*^9}, {
   3.6182165257656355`*^9, 3.618216533191249*^9}, {3.6182185538627977`*^9, 
   3.618218577262839*^9}, 3.6182186344057393`*^9, {3.618218786227606*^9, 
   3.6182187922804165`*^9}, {3.6182217002815247`*^9, 
   3.6182217279247727`*^9}, {3.618221842928175*^9, 3.6182218524753914`*^9}, {
   3.625235042468136*^9, 3.6252350593805656`*^9}}],

Cell["Now we obtain the expected result:", "Text",
 CellChangeTimes->{{3.6180604278910885`*^9, 3.618060462419063*^9}, {
   3.6182196402511063`*^9, 3.6182196579103374`*^9}, {3.61829593994195*^9, 
   3.6182959707107096`*^9}, 3.6184068236311083`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"fund1", "[", "sf", "]"}], "[", 
       RowBox[{
        RowBox[{"hu", "[", "s", "]"}], ",", 
        RowBox[{"hv", "[", "s", "]"}]}], "]"}], "[", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}], ",", 
       RowBox[{
        RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]}], "]"}], ",", "s"}], 
    "]"}], ",", 
   RowBox[{
    RowBox[{"scond2", "[", "sf", "]"}], "[", "s", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5989442396766644`*^9, 3.5989442862613287`*^9}, {
  3.6136206379490275`*^9, 3.6136206408506327`*^9}, {3.617282881985641*^9, 
  3.617282884887246*^9}, {3.618059910127474*^9, 3.618059926725423*^9}, {
  3.6182186201629143`*^9, 3.6182186312857337`*^9}, {3.6182187056510644`*^9, 
  3.618218714121879*^9}, {3.6252351518841286`*^9, 3.62523515798374*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.627974613793008*^9}]
}, Closed]],

Cell[TextData[{
 StyleBox["Example",
  FontColor->RGBColor[0, 0, 1]],
 ". The unit sphere"
}], "Text",
 CellChangeTimes->{{3.6180604278910885`*^9, 3.618060462419063*^9}, {
  3.6182196402511063`*^9, 3.6182196579103374`*^9}, {3.61829593994195*^9, 
  3.618296005188682*^9}, {3.618296062100937*^9, 3.6182960712054577`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"fund1", "[", 
     RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
    RowBox[{
     RowBox[{"hu", "[", "s", "]"}], ",", 
     RowBox[{"hv", "[", "s", "]"}]}], "]"}], "[", 
   RowBox[{
    RowBox[{
     RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}], ",", 
    RowBox[{
     RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.618220361128372*^9, 3.6182203959788337`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox[
    RowBox[{"Cos", "[", 
     RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{
     SuperscriptBox["hu", "\[Prime]",
      MultilineFunction->None], "[", "s", "]"}], "2"]}], "+", 
  SuperscriptBox[
   RowBox[{
    SuperscriptBox["hv", "\[Prime]",
     MultilineFunction->None], "[", "s", "]"}], "2"]}]], "Output",
 CellChangeTimes->{3.62797466014069*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{"%", ",", "s"}], "]"}]], "Input",
 CellChangeTimes->{{3.618220420798477*^9, 3.618220429269292*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "2"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
   RowBox[{"Sin", "[", 
    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
   SuperscriptBox[
    RowBox[{
     SuperscriptBox["hu", "\[Prime]",
      MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
   RowBox[{
    SuperscriptBox["hv", "\[Prime]",
     MultilineFunction->None], "[", "s", "]"}]}], "+", 
  RowBox[{"2", " ", 
   SuperscriptBox[
    RowBox[{"Cos", "[", 
     RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
   RowBox[{
    SuperscriptBox["hu", "\[Prime]",
     MultilineFunction->None], "[", "s", "]"}], " ", 
   RowBox[{
    SuperscriptBox["hu", "\[Prime]\[Prime]",
     MultilineFunction->None], "[", "s", "]"}]}], "+", 
  RowBox[{"2", " ", 
   RowBox[{
    SuperscriptBox["hv", "\[Prime]",
     MultilineFunction->None], "[", "s", "]"}], " ", 
   RowBox[{
    SuperscriptBox["hv", "\[Prime]\[Prime]",
     MultilineFunction->None], "[", "s", "]"}]}]}]], "Output",
 CellChangeTimes->{3.6279746634946957`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"fund1", "[", 
        RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
       RowBox[{
        RowBox[{"hu", "[", "s", "]"}], ",", 
        RowBox[{"hv", "[", "s", "]"}]}], "]"}], "[", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}], ",", 
       RowBox[{
        RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]}], "]"}], ",", "s"}], 
    "]"}], ",", 
   RowBox[{
    RowBox[{"scond2", "[", 
     RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", "s", "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5989442396766644`*^9, 3.5989442862613287`*^9}, {
   3.6136206379490275`*^9, 3.6136206408506327`*^9}, {3.617282881985641*^9, 
   3.617282884887246*^9}, {3.6182102141921153`*^9, 3.618210263519402*^9}, 
   3.6182116456038294`*^9, 3.618220220603326*^9, {3.6182360303187585`*^9, 
   3.618236096026074*^9}, {3.6252357283153443`*^9, 3.625235730156148*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", 
    RowBox[{"Sin", "[", 
     RowBox[{"2", " ", 
      RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], " ", 
   SuperscriptBox[
    RowBox[{
     SuperscriptBox["hu", "\[Prime]",
      MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
   RowBox[{
    SuperscriptBox["hv", "\[Prime]",
     MultilineFunction->None], "[", "s", "]"}]}], "+", 
  RowBox[{"2", " ", 
   SuperscriptBox[
    RowBox[{"Cos", "[", 
     RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
   RowBox[{
    SuperscriptBox["hu", "\[Prime]",
     MultilineFunction->None], "[", "s", "]"}], " ", 
   RowBox[{
    SuperscriptBox["hu", "\[Prime]\[Prime]",
     MultilineFunction->None], "[", "s", "]"}]}], "+", 
  RowBox[{"2", " ", 
   RowBox[{
    SuperscriptBox["hv", "\[Prime]",
     MultilineFunction->None], "[", "s", "]"}], " ", 
   RowBox[{
    SuperscriptBox["hv", "\[Prime]\[Prime]",
     MultilineFunction->None], "[", "s", "]"}]}]}]], "Output",
 CellChangeTimes->{3.6279746726831117`*^9}]
}, Closed]],

Cell["\<\
Unexpectedly, scond2[sphero[1]][s] has not been applied. 
First we have to evaluate and to simplify the specialized condition:\
\>", "Text",
 CellChangeTimes->{{3.618236148270566*^9, 3.6182362271599045`*^9}, {
  3.618296137309239*^9, 3.6182961587914677`*^9}, {3.618296206613203*^9, 
  3.6182962413651905`*^9}, {3.6252358055152807`*^9, 3.6252358079332848`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Evaluate", "[", 
   RowBox[{
    RowBox[{"scond2", "[", 
     RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", "s", "]"}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6182358571116543`*^9, 3.6182358646152678`*^9}, {
  3.61823632717168*^9, 3.6182363467965145`*^9}, {3.6252357642336082`*^9, 
  3.6252357678996143`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"Cos", "[", 
        RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["hu", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], "2"]}], "+", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["hv", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "2"]}], "\[Equal]", "1"}], 
   ",", 
   RowBox[{
    RowBox[{
     RowBox[{"Sin", "[", 
      RowBox[{"2", " ", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["hu", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
     RowBox[{
      SuperscriptBox["hv", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}]}], "\[Equal]", 
    RowBox[{
     RowBox[{"2", " ", 
      SuperscriptBox[
       RowBox[{"Cos", "[", 
        RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
      RowBox[{
       SuperscriptBox["hu", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{
       SuperscriptBox["hu", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "s", "]"}]}], "+", 
     RowBox[{"2", " ", 
      RowBox[{
       SuperscriptBox["hv", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{
       SuperscriptBox["hv", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "s", "]"}]}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.627974704226367*^9}]
}, Closed]],

Cell["Now we get the expected result:", "Text",
 CellChangeTimes->{{3.618236269311178*^9, 3.6182362919936185`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{"%", ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"fund1", "[", 
         RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
        RowBox[{
         RowBox[{"hu", "[", "s", "]"}], ",", 
         RowBox[{"hv", "[", "s", "]"}]}], "]"}], "[", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}], ",", 
        RowBox[{
         RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]}], "]"}], ",", "s"}], 
     "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.618217765187813*^9, 3.618217802393878*^9}, {
   3.6182179873790026`*^9, 3.6182180275178733`*^9}, 3.6182181114772205`*^9, 
   3.6182182611126833`*^9, 3.6182183126707745`*^9, 3.6182188601405363`*^9, {
   3.6182200229821787`*^9, 3.6182200436834145`*^9}, {3.618235708989394*^9, 
   3.6182357770055137`*^9}, 3.61823583762722*^9, {3.6182358763308883`*^9, 
   3.618235921149767*^9}, {3.61823636102374*^9, 3.618236386326984*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6279747134147835`*^9}]
}, Closed]],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.61822320072336*^9, 3.61822320072336*^9}}],

Cell[TextData[{
 StyleBox["Example",
  FontColor->RGBColor[0, 0, 1]],
 ". The  hyperboloid"
}], "Text",
 CellChangeTimes->{{3.6182368296017623`*^9, 3.618236847713394*^9}}],

Cell["\<\
The same situation we meet with other explicitly given surfaces, e. g.\
\>", "Text",
 CellChangeTimes->{{3.61829635272456*^9, 3.618296403549467*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hypsf", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.618236856090609*^9, 3.618236874139841*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", "u", "]"}], " ", 
    RowBox[{"Cosh", "[", "v", "]"}]}], ",", 
   RowBox[{
    RowBox[{"Cosh", "[", "v", "]"}], " ", 
    RowBox[{"Sin", "[", "u", "]"}]}], ",", 
   RowBox[{"Sinh", "[", "v", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.627974748218444*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"fund1", "[", "hypsf", "]"}], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}], "[", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}], ",", 
      RowBox[{
       RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]}], "]"}], ",", "s"}], "]"}],
   "]"}]], "Input",
 CellChangeTimes->{3.618296326096037*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Sinh", "[", 
    RowBox[{"2", " ", 
     RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
   SuperscriptBox[
    RowBox[{
     SuperscriptBox["hu", "\[Prime]",
      MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
   RowBox[{
    SuperscriptBox["hv", "\[Prime]",
     MultilineFunction->None], "[", "s", "]"}]}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{"2", " ", 
     RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
   SuperscriptBox[
    RowBox[{
     SuperscriptBox["hv", "\[Prime]",
      MultilineFunction->None], "[", "s", "]"}], "3"]}], "+", 
  RowBox[{"2", " ", 
   SuperscriptBox[
    RowBox[{"Cosh", "[", 
     RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
   RowBox[{
    SuperscriptBox["hu", "\[Prime]",
     MultilineFunction->None], "[", "s", "]"}], " ", 
   RowBox[{
    SuperscriptBox["hu", "\[Prime]\[Prime]",
     MultilineFunction->None], "[", "s", "]"}]}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{"2", " ", 
     RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
   RowBox[{
    SuperscriptBox["hv", "\[Prime]",
     MultilineFunction->None], "[", "s", "]"}], " ", 
   RowBox[{
    SuperscriptBox["hv", "\[Prime]\[Prime]",
     MultilineFunction->None], "[", "s", "]"}]}]}]], "Output",
 CellChangeTimes->{3.627974752711252*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"Simplify", "[", 
    RowBox[{"Evaluate", "[", 
     RowBox[{
      RowBox[{"scond2", "[", "hypsf", "]"}], "[", "s", "]"}], "]"}], "]"}], 
   ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"fund1", "[", "hypsf", "]"}], "[", 
        RowBox[{
         RowBox[{"hu", "[", "s", "]"}], ",", 
         RowBox[{"hv", "[", "s", "]"}]}], "]"}], "[", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}], ",", 
        RowBox[{
         RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]}], "]"}], ",", "s"}], 
     "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.618217765187813*^9, 3.618217802393878*^9}, {
   3.6182179873790026`*^9, 3.6182180275178733`*^9}, 3.6182181114772205`*^9, 
   3.6182182611126833`*^9, 3.6182183126707745`*^9, 3.6182188601405363`*^9, {
   3.6182200229821787`*^9, 3.6182200436834145`*^9}, {3.618235708989394*^9, 
   3.6182357770055137`*^9}, 3.61823583762722*^9, {3.6182358763308883`*^9, 
   3.618235921149767*^9}, {3.61823636102374*^9, 3.618236386326984*^9}, {
   3.6182369218915243`*^9, 3.6182369801108265`*^9}, {3.6252358876710253`*^9, 
   3.6252358922886333`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.627974764348872*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "hypsf", "]"}], "[", 
        RowBox[{
         RowBox[{"hu", "[", "s", "]"}], ",", 
         RowBox[{"hv", "[", "s", "]"}]}], "]"}], ".", 
       RowBox[{
        RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]}], ")"}], "^", "2"}], " ", 
    "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "hypsf", "]"}], "[", 
        RowBox[{
         RowBox[{"hu", "[", "s", "]"}], ",", 
         RowBox[{"hv", "[", "s", "]"}]}], "]"}], ".", 
       RowBox[{
        RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]}], ")"}], "^", "2"}]}], 
   ",", 
   RowBox[{
    RowBox[{"scond2", "[", "hypsf", "]"}], "[", "s", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6182370761445956`*^9, 3.6182371296214895`*^9}, {
  3.6252359178620787`*^9, 3.625235920919684*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox[
    RowBox[{"Cosh", "[", 
     RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{
     SuperscriptBox["hu", "\[Prime]",
      MultilineFunction->None], "[", "s", "]"}], "2"]}], "+", 
  RowBox[{
   RowBox[{"Cosh", "[", 
    RowBox[{"2", " ", 
     RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
   SuperscriptBox[
    RowBox[{
     SuperscriptBox["hv", "\[Prime]",
      MultilineFunction->None], "[", "s", "]"}], "2"]}]}]], "Output",
 CellChangeTimes->{3.6279747700896826`*^9}]
}, Closed]],

Cell["\<\
The same as before: First evaluate and simplify the specialized condition:\
\>", "Text",
 CellChangeTimes->{{3.6182372867293653`*^9, 3.618237333201847*^9}, {
  3.6182964786051564`*^9, 3.618296480141244*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"Simplify", "[", 
    RowBox[{"Evaluate", "[", 
     RowBox[{
      RowBox[{"scond2", "[", "hypsf", "]"}], "[", "s", "]"}], "]"}], "]"}], 
   ",", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "hypsf", "]"}], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], ".", 
        RowBox[{
         RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]}], ")"}], "^", "2"}], " ",
      "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "hypsf", "]"}], "[", 
         RowBox[{
          RowBox[{"hu", "[", "s", "]"}], ",", 
          RowBox[{"hv", "[", "s", "]"}]}], "]"}], ".", 
        RowBox[{
         RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]}], ")"}], "^", "2"}]}], 
    "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6182370761445956`*^9, 3.6182371296214895`*^9}, {
   3.6182371678259563`*^9, 3.6182371946892037`*^9}, 3.618237242128887*^9, {
   3.6252359480823317`*^9, 3.6252359500811357`*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.627974783084505*^9}]
}, Closed]],

Cell["\<\
\[FilledSquare]

The following general formula is obtained directly:\
\>", "Text",
 CellChangeTimes->{{3.6182373795027285`*^9, 3.6182374399996347`*^9}, {
  3.624696435316099*^9, 3.624696438716905*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "sf", "]"}], "[", 
        RowBox[{
         RowBox[{"hu", "[", "s", "]"}], ",", 
         RowBox[{"hv", "[", "s", "]"}]}], "]"}], ".", 
       RowBox[{
        RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]}], ")"}], "^", "2"}], " ", 
    "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "sf", "]"}], "[", 
        RowBox[{
         RowBox[{"hu", "[", "s", "]"}], ",", 
         RowBox[{"hv", "[", "s", "]"}]}], "]"}], ".", 
       RowBox[{
        RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]}], ")"}], "^", "2"}]}], 
   ",", 
   RowBox[{
    RowBox[{"scond2", "[", "sf", "]"}], "[", "s", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.591164978147087*^9, 3.59116512593254*^9}, 
   3.6180441542132874`*^9, {3.6252359797377872`*^9, 3.6252359858217983`*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.6279748178413663`*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["3.4.2. Angles", "Subsubsection",
 CellChangeTimes->{{3.625112340729862*^9, 3.6251123601050963`*^9}}],

Cell[TextData[{
 "There exists a uniquely defined ",
 StyleBox["angle",
  FontColor->RGBColor[0, 0, 1]],
 " \[Tau] between two tangent vectors  with coordinates {s1,s2}, {t1,t2} \
defined by"
}], "Text",
 CellChangeTimes->{{3.591165792643674*^9, 3.591165861738626*^9}, {
   3.5911660657072926`*^9, 3.5911660947709546`*^9}, 3.618296655345346*^9, {
   3.6182972690779467`*^9, 3.6182973948131385`*^9}, 3.618298044670308*^9, {
   3.618298078357235*^9, 3.6182980826774817`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"angle", "[", "ff_", "]"}], "[", 
    RowBox[{"u_", ",", " ", "v_"}], "]"}], "[", 
   RowBox[{"s_List", ",", " ", "t_List"}], "]"}], " ", ":=", " ", 
  RowBox[{"ArcCos", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"fund1", "[", "ff", "]"}], "[", 
      RowBox[{"u", ",", " ", "v"}], "]"}], "[", 
     RowBox[{"s", ",", " ", "t"}], "]"}], "/", " ", 
    RowBox[{"Sqrt", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"fund1", "[", "ff", "]"}], "[", 
        RowBox[{"u", ",", " ", "v"}], "]"}], "[", 
       RowBox[{"s", ",", " ", "s"}], "]"}], "*", 
      RowBox[{
       RowBox[{
        RowBox[{"fund1", "[", "ff", "]"}], "[", 
        RowBox[{"u", ",", " ", "v"}], "]"}], "[", 
       RowBox[{"t", ",", " ", "t"}], "]"}]}], "]"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6182985837181396`*^9, 3.61829858371914*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"angle", "[", 
    RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"s1", ",", "s2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"t1", ",", "t2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6183113497195683`*^9, 3.6183114408887825`*^9}}],

Cell[BoxData[
 RowBox[{"ArcCos", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"s1", " ", "t1", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"Cos", "[", "v", "]"}], "2"]}], "+", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"Cos", "[", "v", "]"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"Sin", "[", "u", "]"}], "2"]}]}], ")"}]}], "+", 
     RowBox[{"s2", " ", "t2", " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", "v", "]"}], "2"], "+", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"Sin", "[", "v", "]"}], "2"]}], "+", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"Sin", "[", "v", "]"}], "2"]}]}], ")"}]}]}], ")"}], "/", 
   RowBox[{"(", 
    RowBox[{"\[Sqrt]", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["s1", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
             SuperscriptBox[
              RowBox[{"Cos", "[", "v", "]"}], "2"]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", "v", "]"}], "2"], " ", 
             SuperscriptBox[
              RowBox[{"Sin", "[", "u", "]"}], "2"]}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["s2", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"Cos", "[", "v", "]"}], "2"], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
             SuperscriptBox[
              RowBox[{"Sin", "[", "v", "]"}], "2"]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
             SuperscriptBox[
              RowBox[{"Sin", "[", "v", "]"}], "2"]}]}], ")"}]}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["t1", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
             SuperscriptBox[
              RowBox[{"Cos", "[", "v", "]"}], "2"]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", "v", "]"}], "2"], " ", 
             SuperscriptBox[
              RowBox[{"Sin", "[", "u", "]"}], "2"]}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["t2", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"Cos", "[", "v", "]"}], "2"], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
             SuperscriptBox[
              RowBox[{"Sin", "[", "v", "]"}], "2"]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
             SuperscriptBox[
              RowBox[{"Sin", "[", "v", "]"}], "2"]}]}], ")"}]}]}], ")"}]}], 
      ")"}]}], ")"}]}], "]"}]], "Output",
 CellChangeTimes->{3.6279751071620646`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6183114751387415`*^9, 3.618311485000306*^9}}],

Cell[BoxData[
 RowBox[{"ArcCos", "[", 
  FractionBox[
   RowBox[{
    RowBox[{"s1", " ", "t1"}], "+", 
    RowBox[{"2", " ", "s2", " ", "t2"}], "+", 
    RowBox[{"s1", " ", "t1", " ", 
     RowBox[{"Cos", "[", 
      RowBox[{"2", " ", "v"}], "]"}]}]}], 
   SqrtBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["s1", "2"], "+", 
       RowBox[{"2", " ", 
        SuperscriptBox["s2", "2"]}], "+", 
       RowBox[{
        SuperscriptBox["s1", "2"], " ", 
        RowBox[{"Cos", "[", 
         RowBox[{"2", " ", "v"}], "]"}]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["t1", "2"], "+", 
       RowBox[{"2", " ", 
        SuperscriptBox["t2", "2"]}], "+", 
       RowBox[{
        SuperscriptBox["t1", "2"], " ", 
        RowBox[{"Cos", "[", 
         RowBox[{"2", " ", "v"}], "]"}]}]}], ")"}]}]]], "]"}]], "Output",
 CellChangeTimes->{3.627976254840481*^9}]
}, Closed]],

Cell["\<\
The direction angle between the tangent vector of a curve on the surface with \
the tangent   vector amf[1]{ff][u,v] of the surface is defined as the \
solution of the equations\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.6182974023185673`*^9, 3.618297482128133*^9}, {
   3.6182975428396053`*^9, 3.6182977356406326`*^9}, {3.618297912209732*^9, 
   3.618297956537267*^9}, {3.6182979883630877`*^9, 3.6182979985366697`*^9}, {
   3.6182980955372176`*^9, 3.61829810676186*^9}, {3.6182981910096784`*^9, 
   3.6182982298669004`*^9}, {3.618298282223895*^9, 3.6182983847377586`*^9}, 
   3.6182984255310917`*^9, 3.6182985196004725`*^9, {3.6182985972059116`*^9, 
   3.618298654213172*^9}, {3.6182986943974705`*^9, 3.618298766141574*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"dirangle", "[", 
    RowBox[{"ff_", ",", "hu_", ",", "hv_"}], "]"}], "[", "s_", "]"}], " ", ":=",
   " ", 
  RowBox[{"Assuming", "[", 
   RowBox[{
    RowBox[{"Simplify", "[", 
     RowBox[{"Evaluate", "[", 
      RowBox[{
       RowBox[{"scond2", "[", "ff", "]"}], "[", "s", "]"}], "]"}], "]"}], ",", 
    RowBox[{"FullSimplify", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", "\[Tau]", "]"}], " ", "\[Equal]", " ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "ff", "]"}], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], ".", 
         RowBox[{
          RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]}]}], ",", " ", 
       RowBox[{
        RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", "\[Equal]", " ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "ff", "]"}], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], ".", 
         RowBox[{
          RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"0", "\[LessEqual]", " ", "\[Tau]"}], ",", " ", 
       RowBox[{"\[Tau]", " ", "<", " ", 
        RowBox[{"2", "*", "Pi"}]}]}], "}"}], "]"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5911658904282665`*^9, 3.591166056496765*^9}, {
   3.5911664298171177`*^9, 3.591166476456786*^9}, {3.5911665164810753`*^9, 
   3.5911665297628345`*^9}, 3.5911673587612505`*^9, 3.6148618587859383`*^9, 
   3.614917790610159*^9, {3.615262227734214*^9, 3.6152622420480323`*^9}, 
   3.6152757802705507`*^9, {3.6152765968182545`*^9, 3.6152765992253923`*^9}, {
   3.6181181173599977`*^9, 3.618118117952799*^9}, {3.6182375175941706`*^9, 
   3.6182376369811807`*^9}, {3.618298818994597*^9, 3.618298822635805*^9}, {
   3.6250340281548924`*^9, 3.6250340517421336`*^9}, {3.625236692271044*^9, 
   3.6252366944550476`*^9}}],

Cell["It is the angle ", "Text",
 CellChangeTimes->{{3.618296717043454*^9, 3.6182967229246645`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dirangle", "[", 
   RowBox[{
    RowBox[{"sphero", "[", "r", "]"}], ",", "hu", ",", "hv"}], "]"}], "[", 
  "s", "]"}]], "Input",
 CellChangeTimes->{{3.6116354915177364`*^9, 3.6116355074453645`*^9}, {
   3.611635948395739*^9, 3.611635961468562*^9}, 3.6148618587859383`*^9, 
   3.614917790610159*^9, {3.6180442929732237`*^9, 3.618044310582231*^9}, 
   3.61804546640534*^9, {3.6180605496920547`*^9, 3.61806057566054*^9}, 
   3.61811479371176*^9, {3.618117354035057*^9, 3.6181173736598916`*^9}, {
   3.6181174175271683`*^9, 3.618117436574802*^9}, {3.618118143271643*^9, 
   3.618118149449254*^9}, {3.618237684498864*^9, 3.6182376950132823`*^9}, {
   3.6183189630340247`*^9, 3.618319022601432*^9}, {3.625034125311863*^9, 
   3.6250341303038716`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", "\[Tau]", "]"}], "\[Equal]", 
    RowBox[{
     SqrtBox[
      RowBox[{
       SuperscriptBox["r", "2"], " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", 
         RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}]], " ", 
     RowBox[{
      SuperscriptBox["hu", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"Sin", "[", "\[Tau]", "]"}], "\[Equal]", 
    FractionBox[
     RowBox[{
      SqrtBox[
       RowBox[{
        SuperscriptBox["r", "4"], " ", 
        SuperscriptBox[
         RowBox[{"Cos", "[", 
          RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}]], " ", 
      RowBox[{
       SuperscriptBox["hv", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}]}], 
     SqrtBox[
      RowBox[{
       SuperscriptBox["r", "2"], " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", 
         RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}]]]}], ",", 
   RowBox[{"\[Tau]", "\[GreaterEqual]", "0"}], ",", 
   RowBox[{"\[Tau]", "<", 
    RowBox[{"2", " ", "\[Pi]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.627976371637886*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reduce", "[", 
  RowBox[{
   RowBox[{"%", "/.", 
    RowBox[{"r", "\[Rule]", "1"}]}], ",", "\[Tau]", ",", "Reals"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.611635580391093*^9, 3.611635587114704*^9}, {
   3.6116356236031685`*^9, 3.6116356431344028`*^9}, 3.6148618588015385`*^9, 
   3.6149177906413593`*^9, {3.618119717361208*^9, 3.618119726877225*^9}, {
   3.618320086843303*^9, 3.618320092458624*^9}, 3.618375224935257*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"Cos", "[", 
      RowBox[{"hv", "[", "s", "]"}], "]"}], "\[Equal]", "0"}], "&&", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["hv", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "\[Equal]", 
         RowBox[{"-", "1"}]}], "&&", 
        RowBox[{"\[Tau]", "\[Equal]", 
         FractionBox[
          RowBox[{"3", " ", "\[Pi]"}], "2"]}]}], ")"}], "||", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["hv", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "\[Equal]", "1"}], "&&", 
        RowBox[{"\[Tau]", "\[Equal]", 
         FractionBox["\[Pi]", "2"]}]}], ")"}]}], ")"}]}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"Cos", "[", 
      RowBox[{"hv", "[", "s", "]"}], "]"}], ">", "0"}], "&&", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"Sec", "[", 
           RowBox[{"hv", "[", "s", "]"}], "]"}]}], "<", 
         RowBox[{
          SuperscriptBox["hu", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "<", 
         RowBox[{"Sec", "[", 
          RowBox[{"hv", "[", "s", "]"}], "]"}]}], "&&", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["hv", "\[Prime]",
               MultilineFunction->None], "[", "s", "]"}], "\[Equal]", 
             RowBox[{"-", 
              SqrtBox[
               RowBox[{"1", "-", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"]}]}]]}]}], 
            "&&", 
            RowBox[{"\[Tau]", "\[Equal]", 
             RowBox[{
              RowBox[{"2", " ", "\[Pi]"}], "-", 
              RowBox[{"2", " ", 
               RowBox[{"ArcTan", "[", 
                SqrtBox[
                 FractionBox[
                  RowBox[{"1", "-", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                    RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}]}]}], 
                  RowBox[{"1", "+", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                    RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}]}]}]]], 
                "]"}]}]}]}]}], ")"}], "||", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["hv", "\[Prime]",
               MultilineFunction->None], "[", "s", "]"}], "\[Equal]", 
             SqrtBox[
              RowBox[{"1", "-", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"]}]}]]}], "&&", 
            RowBox[{"\[Tau]", "\[Equal]", 
             RowBox[{"2", " ", 
              RowBox[{"ArcTan", "[", 
               SqrtBox[
                FractionBox[
                 RowBox[{"1", "-", 
                  RowBox[{
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}]}]}], 
                 RowBox[{"1", "+", 
                  RowBox[{
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}]}]}]]], 
               "]"}]}]}]}], ")"}]}], ")"}]}], ")"}], "||", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["hu", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "\[Equal]", 
         RowBox[{"Sec", "[", 
          RowBox[{"hv", "[", "s", "]"}], "]"}]}], "&&", 
        RowBox[{
         RowBox[{
          SuperscriptBox["hv", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "\[Equal]", "0"}], "&&", 
        RowBox[{"\[Tau]", "\[Equal]", "0"}]}], ")"}], "||", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["hu", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "\[Equal]", 
         RowBox[{"-", 
          RowBox[{"Sec", "[", 
           RowBox[{"hv", "[", "s", "]"}], "]"}]}]}], "&&", 
        RowBox[{
         RowBox[{
          SuperscriptBox["hv", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "\[Equal]", "0"}], "&&", 
        RowBox[{"\[Tau]", "\[Equal]", "\[Pi]"}]}], ")"}]}], ")"}]}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"Cos", "[", 
      RowBox[{"hv", "[", "s", "]"}], "]"}], "<", "0"}], "&&", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"Sec", "[", 
          RowBox[{"hv", "[", "s", "]"}], "]"}], "<", 
         RowBox[{
          SuperscriptBox["hu", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "<", 
         RowBox[{"-", 
          RowBox[{"Sec", "[", 
           RowBox[{"hv", "[", "s", "]"}], "]"}]}]}], "&&", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["hv", "\[Prime]",
               MultilineFunction->None], "[", "s", "]"}], "\[Equal]", 
             RowBox[{"-", 
              SqrtBox[
               RowBox[{"1", "-", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"]}]}]]}]}], 
            "&&", 
            RowBox[{"\[Tau]", "\[Equal]", 
             RowBox[{
              RowBox[{"2", " ", "\[Pi]"}], "-", 
              RowBox[{"2", " ", 
               RowBox[{"ArcTan", "[", 
                SqrtBox[
                 FractionBox[
                  RowBox[{
                   RowBox[{"-", "1"}], "-", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                    RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}]}]}], 
                  RowBox[{
                   RowBox[{"-", "1"}], "+", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                    RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}]}]}]]], 
                "]"}]}]}]}]}], ")"}], "||", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["hv", "\[Prime]",
               MultilineFunction->None], "[", "s", "]"}], "\[Equal]", 
             SqrtBox[
              RowBox[{"1", "-", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"]}]}]]}], "&&", 
            RowBox[{"\[Tau]", "\[Equal]", 
             RowBox[{"2", " ", 
              RowBox[{"ArcTan", "[", 
               SqrtBox[
                FractionBox[
                 RowBox[{
                  RowBox[{"-", "1"}], "-", 
                  RowBox[{
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}]}]}], 
                 RowBox[{
                  RowBox[{"-", "1"}], "+", 
                  RowBox[{
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}]}]}]]], 
               "]"}]}]}]}], ")"}]}], ")"}]}], ")"}], "||", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["hu", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "\[Equal]", 
         RowBox[{"-", 
          RowBox[{"Sec", "[", 
           RowBox[{"hv", "[", "s", "]"}], "]"}]}]}], "&&", 
        RowBox[{
         RowBox[{
          SuperscriptBox["hv", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "\[Equal]", "0"}], "&&", 
        RowBox[{"\[Tau]", "\[Equal]", "0"}]}], ")"}], "||", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["hu", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "\[Equal]", 
         RowBox[{"Sec", "[", 
          RowBox[{"hv", "[", "s", "]"}], "]"}]}], "&&", 
        RowBox[{
         RowBox[{
          SuperscriptBox["hv", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "\[Equal]", "0"}], "&&", 
        RowBox[{"\[Tau]", "\[Equal]", "\[Pi]"}]}], ")"}]}], ")"}]}], 
   ")"}]}]], "Output",
 CellChangeTimes->{3.627976394538726*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"solangle", " ", "=", 
  RowBox[{"Assuming", "[", 
   RowBox[{
    RowBox[{"Evaluate", "[", 
     RowBox[{
      RowBox[{"scond2", "[", 
       RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", "s", "]"}], "]"}], ",", 
    RowBox[{"FullSimplify", "[", 
     RowBox[{"%", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"hu", "[", "s", "]"}], "\[Element]", " ", "Reals"}], ",", 
        RowBox[{
         RowBox[{"hv", "[", "s", "]"}], "\[Element]", " ", "Reals"}]}], 
       "}"}]}], "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6183191347148447`*^9, 3.6183191944412603`*^9}, 
   3.618319228977236*^9, {3.6183193357953453`*^9, 3.6183193716903987`*^9}, {
   3.6183194292416906`*^9, 3.61831947505031*^9}, 3.61837523803928*^9, {
   3.6250342829189396`*^9, 3.625034296178963*^9}, {3.6252367505295467`*^9, 
   3.6252367540239525`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["hv", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], "\[Equal]", "0"}], "&&", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"\[Tau]", "\[Equal]", "0"}], "&&", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"Sec", "[", 
              RowBox[{"hv", "[", "s", "]"}], "]"}], "\[Equal]", 
             RowBox[{
              SuperscriptBox["hu", "\[Prime]",
               MultilineFunction->None], "[", "s", "]"}]}], "&&", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{"hv", "[", "s", "]"}], "]"}], ">", "0"}]}], ")"}], "||", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"Sec", "[", 
               RowBox[{"hv", "[", "s", "]"}], "]"}], "+", 
              RowBox[{
               SuperscriptBox["hu", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}]}], "\[Equal]", 
             "0"}], "&&", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{"hv", "[", "s", "]"}], "]"}], "<", "0"}]}], ")"}]}], 
         ")"}]}], ")"}], "||", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"\[Tau]", "\[Equal]", "\[Pi]"}], "&&", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"Sec", "[", 
              RowBox[{"hv", "[", "s", "]"}], "]"}], "\[Equal]", 
             RowBox[{
              SuperscriptBox["hu", "\[Prime]",
               MultilineFunction->None], "[", "s", "]"}]}], "&&", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{"hv", "[", "s", "]"}], "]"}], "<", "0"}]}], ")"}], "||", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"Sec", "[", 
               RowBox[{"hv", "[", "s", "]"}], "]"}], "+", 
              RowBox[{
               SuperscriptBox["hu", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}]}], "\[Equal]", 
             "0"}], "&&", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{"hv", "[", "s", "]"}], "]"}], ">", "0"}]}], ")"}]}], 
         ")"}]}], ")"}]}], ")"}]}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     SqrtBox[
      RowBox[{"1", "-", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", 
          RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["hu", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "2"]}]}]], "\[Equal]", 
     RowBox[{
      SuperscriptBox["hv", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}]}], "&&", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"\[Tau]", "\[Equal]", 
         RowBox[{"2", " ", 
          RowBox[{"ArcTan", "[", 
           SqrtBox[
            RowBox[{
             RowBox[{"-", "1"}], "+", 
             FractionBox["2", 
              RowBox[{"1", "+", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}]}]}]]}]], 
           "]"}]}]}], "&&", 
        RowBox[{
         RowBox[{"Cos", "[", 
          RowBox[{"hv", "[", "s", "]"}], "]"}], ">", "0"}], "&&", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"Sec", "[", 
           RowBox[{"hv", "[", "s", "]"}], "]"}]}], "<", 
         RowBox[{
          SuperscriptBox["hu", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "<", 
         RowBox[{"Sec", "[", 
          RowBox[{"hv", "[", "s", "]"}], "]"}]}]}], ")"}], "||", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"\[Tau]", "\[Equal]", 
         RowBox[{"2", " ", 
          RowBox[{"ArcTan", "[", 
           SqrtBox[
            RowBox[{
             RowBox[{"-", "1"}], "-", 
             FractionBox["2", 
              RowBox[{
               RowBox[{"-", "1"}], "+", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}]}]}]]}]], 
           "]"}]}]}], "&&", 
        RowBox[{
         RowBox[{"Sec", "[", 
          RowBox[{"hv", "[", "s", "]"}], "]"}], "<", 
         RowBox[{
          SuperscriptBox["hu", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "<", 
         RowBox[{"-", 
          RowBox[{"Sec", "[", 
           RowBox[{"hv", "[", "s", "]"}], "]"}]}]}], "&&", 
        RowBox[{
         RowBox[{"Cos", "[", 
          RowBox[{"hv", "[", "s", "]"}], "]"}], "<", "0"}]}], ")"}]}], 
     ")"}]}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"Cos", "[", 
      RowBox[{"hv", "[", "s", "]"}], "]"}], "\[Equal]", "0"}], "&&", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"2", " ", "\[Tau]"}], "\[Equal]", "\[Pi]"}], "&&", 
        RowBox[{
         RowBox[{
          SuperscriptBox["hv", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "\[Equal]", "1"}]}], 
       ")"}], "||", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"2", " ", "\[Tau]"}], "\[Equal]", 
         RowBox[{"3", " ", "\[Pi]"}]}], "&&", 
        RowBox[{
         RowBox[{
          SuperscriptBox["hv", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "\[Equal]", 
         RowBox[{"-", "1"}]}]}], ")"}]}], ")"}]}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{
      SqrtBox[
       RowBox[{"1", "-", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"Cos", "[", 
           RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["hu", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], "2"]}]}]], "+", 
      RowBox[{
       SuperscriptBox["hv", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}]}], "\[Equal]", "0"}], "&&", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Tau]", "+", 
          RowBox[{"2", " ", 
           RowBox[{"ArcTan", "[", 
            SqrtBox[
             RowBox[{
              RowBox[{"-", "1"}], "+", 
              FractionBox["2", 
               RowBox[{"1", "+", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}]}]}]]}]], 
            "]"}]}]}], "\[Equal]", 
         RowBox[{"2", " ", "\[Pi]"}]}], "&&", 
        RowBox[{
         RowBox[{"Cos", "[", 
          RowBox[{"hv", "[", "s", "]"}], "]"}], ">", "0"}], "&&", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"Sec", "[", 
           RowBox[{"hv", "[", "s", "]"}], "]"}]}], "<", 
         RowBox[{
          SuperscriptBox["hu", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "<", 
         RowBox[{"Sec", "[", 
          RowBox[{"hv", "[", "s", "]"}], "]"}]}]}], ")"}], "||", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Tau]", "+", 
          RowBox[{"2", " ", 
           RowBox[{"ArcTan", "[", 
            SqrtBox[
             RowBox[{
              RowBox[{"-", "1"}], "-", 
              FractionBox["2", 
               RowBox[{
                RowBox[{"-", "1"}], "+", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}]}]}]]}]], 
            "]"}]}]}], "\[Equal]", 
         RowBox[{"2", " ", "\[Pi]"}]}], "&&", 
        RowBox[{
         RowBox[{"Sec", "[", 
          RowBox[{"hv", "[", "s", "]"}], "]"}], "<", 
         RowBox[{
          SuperscriptBox["hu", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "<", 
         RowBox[{"-", 
          RowBox[{"Sec", "[", 
           RowBox[{"hv", "[", "s", "]"}], "]"}]}]}], "&&", 
        RowBox[{
         RowBox[{"Cos", "[", 
          RowBox[{"hv", "[", "s", "]"}], "]"}], "<", "0"}]}], ")"}]}], 
     ")"}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.627976524252954*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"solangle", "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"hu", "[", "s", "]"}], " ", "\[Rule]", " ", "0"}], ",", 
    RowBox[{
     RowBox[{"hv", "[", "s", "]"}], " ", "\[Rule]", " ", 
     RowBox[{"1", "+", "s"}]}], ",", 
    RowBox[{
     RowBox[{
      SuperscriptBox["hu", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], "\[Rule]", " ", "0"}], ",", 
    RowBox[{
     RowBox[{
      SuperscriptBox["hv", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], "\[Rule]", "1"}]}], 
   "}"}]}]], "Input",
 CellChangeTimes->{{3.611635580391093*^9, 3.611635587114704*^9}, {
   3.6116356236031685`*^9, 3.6116356431344028`*^9}, 3.6148618588015385`*^9, 
   3.6149177906413593`*^9, 3.618118629384097*^9, {3.61811879586759*^9, 
   3.618118838892465*^9}, {3.618119433066309*^9, 3.618119489522808*^9}, {
   3.6181195503005147`*^9, 3.618119577226162*^9}, 3.61811986070986*^9, {
   3.6181199797380686`*^9, 3.618120032622162*^9}, 3.6182378003914676`*^9, {
   3.618320127722641*^9, 3.618320133946997*^9}, {3.618320467129054*^9, 
   3.6183204861621428`*^9}, {3.627978129901374*^9, 3.627978159120225*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"\[Tau]", "\[Equal]", 
     FractionBox["\[Pi]", "2"]}], "&&", 
    RowBox[{
     RowBox[{"Cos", "[", 
      RowBox[{"1", "+", "s"}], "]"}], ">", "0"}], "&&", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"Sec", "[", 
       RowBox[{"1", "+", "s"}], "]"}]}], "<", "0", "<", 
     RowBox[{"Sec", "[", 
      RowBox[{"1", "+", "s"}], "]"}]}]}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"\[Tau]", "\[Equal]", 
     FractionBox["\[Pi]", "2"]}], "&&", 
    RowBox[{
     RowBox[{"Sec", "[", 
      RowBox[{"1", "+", "s"}], "]"}], "<", "0", "<", 
     RowBox[{"-", 
      RowBox[{"Sec", "[", 
       RowBox[{"1", "+", "s"}], "]"}]}]}], "&&", 
    RowBox[{
     RowBox[{"Cos", "[", 
      RowBox[{"1", "+", "s"}], "]"}], "<", "0"}]}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"Cos", "[", 
      RowBox[{"1", "+", "s"}], "]"}], "\[Equal]", "0"}], "&&", 
    RowBox[{
     RowBox[{"2", " ", "\[Tau]"}], "\[Equal]", "\[Pi]"}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.6279766300211396`*^9, 3.627978161132629*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dirangle", "[", 
   RowBox[{
    RowBox[{"sphero", "[", "1", "]"}], ",", "hu", ",", "hv"}], "]"}], "[", 
  "s", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", "\[Tau]", "]"}], "\[Equal]", 
    RowBox[{
     SqrtBox[
      SuperscriptBox[
       RowBox[{"Cos", "[", 
        RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]], " ", 
     RowBox[{
      SuperscriptBox["hu", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"Sin", "[", "\[Tau]", "]"}], "\[Equal]", 
    RowBox[{
     SuperscriptBox["hv", "\[Prime]",
      MultilineFunction->None], "[", "s", "]"}]}], ",", 
   RowBox[{"\[Tau]", "\[GreaterEqual]", "0"}], ",", 
   RowBox[{"\[Tau]", "<", 
    RowBox[{"2", " ", "\[Pi]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6279777188562517`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"dirangle", "[", 
    RowBox[{
     RowBox[{"sphero", "[", "1", "]"}], ",", "hu", ",", "hv"}], "]"}], "[", 
   "s", "]"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["hu", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], "\[Rule]", " ", 
     RowBox[{"1", "/", 
      RowBox[{"(", 
       RowBox[{
        SqrtBox[
         SuperscriptBox[
          RowBox[{"Cos", "[", 
           RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]], "*", 
        RowBox[{"Sqrt", "[", "2", "]"}]}], ")"}]}]}], ",", 
    RowBox[{
     RowBox[{
      SuperscriptBox["hv", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], "\[Rule]", 
     RowBox[{"1", "/", 
      RowBox[{"Sqrt", "[", "2", "]"}]}]}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.627977400818494*^9, 3.6279774178381233`*^9}, 
   3.6279774628130026`*^9, {3.6279775345887284`*^9, 3.627977568643588*^9}, {
   3.627977801302397*^9, 3.627977876151328*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", "\[Tau]", "]"}], "\[Equal]", 
    FractionBox["1", 
     SqrtBox["2"]]}], ",", 
   RowBox[{
    RowBox[{"Sin", "[", "\[Tau]", "]"}], "\[Equal]", 
    FractionBox["1", 
     SqrtBox["2"]]}], ",", 
   RowBox[{"\[Tau]", "\[GreaterEqual]", "0"}], ",", 
   RowBox[{"\[Tau]", "<", 
    RowBox[{"2", " ", "\[Pi]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6279771012511673`*^9, 3.627977489270649*^9, 
  3.627977571919594*^9, 3.6279778834365416`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Reduce", "[", 
   RowBox[{"%", ",", "\[Tau]", ",", "Reals"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.611635580391093*^9, 3.611635587114704*^9}, {
   3.6116356236031685`*^9, 3.6116356431344028`*^9}, 3.6148618588015385`*^9, 
   3.6149177906413593`*^9, 3.618118629384097*^9, {3.61811879586759*^9, 
   3.618118838892465*^9}, {3.618119433066309*^9, 3.618119489522808*^9}, {
   3.6181195503005147`*^9, 3.618119577226162*^9}, 3.61811986070986*^9, 
   3.618119948070013*^9, 3.618120040656176*^9, {3.618120115208707*^9, 
   3.6181201569699802`*^9}, 3.618237828970718*^9, {3.6183201627306433`*^9, 
   3.618320166970886*^9}, {3.618320507146343*^9, 3.6183205123936434`*^9}, {
   3.6183206598570776`*^9, 3.6183207264948893`*^9}, {3.618320972341951*^9, 
   3.6183209890159044`*^9}, {3.618321068420446*^9, 3.6183210771309443`*^9}, 
   3.618321121093459*^9, {3.618321162740841*^9, 3.6183211834050226`*^9}, {
   3.625237028674836*^9, 3.625237032374043*^9}, 3.6279776359109063`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"4", " ", "\[Tau]"}], "\[Equal]", "\[Pi]"}]], "Output",
 CellChangeTimes->{3.6279766537643814`*^9, 3.6279776394521127`*^9, 
  3.627977896493764*^9}]
}, Closed]],

Cell[" \[FilledSquare]", "Text",
 CellChangeTimes->{{3.591168964923118*^9, 3.5911689898665447`*^9}, {
   3.598367986622035*^9, 3.5983680078382483`*^9}, 3.611471327168975*^9, {
   3.611636294653947*^9, 3.6116363786600947`*^9}, {3.6181209493889723`*^9, 
   3.618120953039378*^9}, {3.6181215261999855`*^9, 3.61812154033361*^9}, {
   3.6182378794368067`*^9, 3.6182379036012487`*^9}, {3.6250617487710457`*^9, 
   3.625061750159448*^9}, 3.627977948894256*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["3.4.3. Curvature", "Subsubsection",
 CellChangeTimes->{{3.6251124466228485`*^9, 3.6251124651400805`*^9}, 
   3.6254886914533005`*^9, {3.6279789572020273`*^9, 3.6279789586840296`*^9}, 
   3.6280013306607757`*^9, 3.628001365312437*^9, {3.628051487277828*^9, 
   3.628051497012245*^9}}],

Cell["We consider the curvature vector of the curve cx[s] ", "Text",
 CellChangeTimes->{{3.591168964923118*^9, 3.5911689898665447`*^9}, {
   3.598367986622035*^9, 3.5983680078382483`*^9}, 3.611471327168975*^9, {
   3.611636294653947*^9, 3.6116363786600947`*^9}, {3.6181209493889723`*^9, 
   3.618120953039378*^9}, {3.6181215261999855`*^9, 3.61812154033361*^9}, {
   3.6182378794368067`*^9, 3.6182379036012487`*^9}, 3.625040520855096*^9, 
   3.6280013867176743`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "curvaturevector"}]], "Input",
 CellChangeTimes->{{3.6251386786559267`*^9, 3.6251387118683853`*^9}}],

Cell[BoxData[
 StyleBox["\<\"curvaturevector[x][t] is the curvature vector of the curve \
x[t] \\nin the Euclidean n-Space, t an arbitrary parameter.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.627983606667103*^9},
 CellTags->"Info3627987206-2313691"]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"curvaturevector", "[", "curve_", "]"}], "[", "t_", "]"}], " ", ":=",
   "  ", 
  RowBox[{
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"tangent", "[", "curve", "]"}], "[", "t", "]"}], ",", "t"}], 
    "]"}], "/", 
   RowBox[{"norm", "[", 
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"curve", "[", "t", "]"}], ",", "t"}], "]"}], "]"}]}]}]], "Input",
 Evaluatable->False],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "tangent"}]], "Input",
 CellChangeTimes->{{3.6279836598943963`*^9, 3.6279836633108025`*^9}}],

Cell[BoxData[
 StyleBox["\<\"tangent[curve][t] yields the tangent unit vector of the  \
curve[t].\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.62798366785041*^9},
 CellTags->"Info3627987267-2313691"]
}, Closed]],

Cell["\<\
Now we define the curvature vector of a general curve on the surface ff[u,v]\
\>", "Text",
 CellChangeTimes->{{3.6279837558345647`*^9, 3.6279838478123264`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"curvaturevec", "[", "f_", "]"}], "[", "t_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"curvaturevector", "[", 
    RowBox[{"cxx", "[", "f", "]"}], "]"}], "[", "t", "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6279839265924644`*^9, 3.627983933503277*^9}, {
  3.6279839852329674`*^9, 3.627984040769065*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"curvaturevec", "[", "sf", "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{
  3.5983684150655403`*^9, {3.611471358282755*^9, 3.6114713598908463`*^9}, {
   3.61147376236126*^9, 3.611473789919836*^9}, {3.611477071370826*^9, 
   3.6114770742179885`*^9}, {3.6181290507857103`*^9, 3.618129058819724*^9}, {
   3.6181309876539125`*^9, 3.6181309895883155`*^9}, {3.618323987210391*^9, 
   3.618323997915004*^9}, {3.618380132688277*^9, 3.61838013449788*^9}, {
   3.6250364470327406`*^9, 3.6250364807911997`*^9}, {3.625036525251278*^9, 
   3.6250365334724927`*^9}, {3.6279836978024626`*^9, 
   3.6279837038708735`*^9}, {3.6279843682604403`*^9, 3.6279843800072613`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["hv", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"hu", "[", "t", "]"}], ",", 
            RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox["hu", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"hu", "[", "t", "]"}], ",", 
            RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox["hv", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["hv", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "t", "]"}], ",", 
               RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox["hu", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "t", "]"}], ",", 
               RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox["hu", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["hv", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "t", "]"}], ",", 
               RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox["hu", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "t", "]"}], ",", 
               RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}]}], ")"}], 
       "/", 
       RowBox[{"(", 
        RowBox[{"\[Sqrt]", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
          ")"}]}], ")"}]}], "-", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}]}], 
             ")"}]}], "+", 
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}]}], 
             ")"}]}], "+", 
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}]}], 
             ")"}]}]}], ")"}]}], ")"}], "/", 
       RowBox[{"(", 
        RowBox[{"2", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "t", "]"}], ",", 
                  RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "t", "]"}], ",", 
                  RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"], 
            "+", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "t", "]"}], ",", 
                  RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "t", "]"}], ",", 
                  RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"], 
            "+", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "t", "]"}], ",", 
                  RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "t", "]"}], ",", 
                  RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
           ")"}], 
          RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"\[Sqrt]", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
       ")"}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["hv", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"hu", "[", "t", "]"}], ",", 
            RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox["hu", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"hu", "[", "t", "]"}], ",", 
            RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox["hv", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["hv", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "t", "]"}], ",", 
               RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox["hu", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "t", "]"}], ",", 
               RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox["hu", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["hv", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "t", "]"}], ",", 
               RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox["hu", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "t", "]"}], ",", 
               RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}]}], ")"}], 
       "/", 
       RowBox[{"(", 
        RowBox[{"\[Sqrt]", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
          ")"}]}], ")"}]}], "-", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}]}], 
             ")"}]}], "+", 
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}]}], 
             ")"}]}], "+", 
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}]}], 
             ")"}]}]}], ")"}]}], ")"}], "/", 
       RowBox[{"(", 
        RowBox[{"2", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "t", "]"}], ",", 
                  RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "t", "]"}], ",", 
                  RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"], 
            "+", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "t", "]"}], ",", 
                  RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "t", "]"}], ",", 
                  RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"], 
            "+", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "t", "]"}], ",", 
                  RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "t", "]"}], ",", 
                  RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
           ")"}], 
          RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"\[Sqrt]", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
       ")"}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["hv", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"hu", "[", "t", "]"}], ",", 
            RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox["hu", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"hu", "[", "t", "]"}], ",", 
            RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox["hv", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["hv", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "t", "]"}], ",", 
               RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox["hu", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "t", "]"}], ",", 
               RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox["hu", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["hv", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "t", "]"}], ",", 
               RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox["hu", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "t", "]"}], ",", 
               RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}]}], ")"}], 
       "/", 
       RowBox[{"(", 
        RowBox[{"\[Sqrt]", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
          ")"}]}], ")"}]}], "-", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}]}], 
             ")"}]}], "+", 
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}]}], 
             ")"}]}], "+", 
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "t", "]"}], ",", 
                 RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "t", "]"}], ",", 
                    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}]}]}], 
             ")"}]}]}], ")"}]}], ")"}], "/", 
       RowBox[{"(", 
        RowBox[{"2", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "t", "]"}], ",", 
                  RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "t", "]"}], ",", 
                  RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"], 
            "+", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "t", "]"}], ",", 
                  RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "t", "]"}], ",", 
                  RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"], 
            "+", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "t", "]"}], ",", 
                  RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "t", "]"}], ",", 
                  RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
           ")"}], 
          RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"\[Sqrt]", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "t", "]"}], ",", 
              RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
       ")"}]}], ")"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6279837053216763`*^9, 3.62798438503047*^9, 
  3.6279907126667833`*^9}]
}, Closed]],

Cell["\<\
The result  is contained in the following definition giving the curvature \
vector of the curve  cx[t], t an arbitrary parameter, on the arbitrary \
surface sf[u,v]:\
\>", "Text",
 CellChangeTimes->{{3.62503661705744*^9, 3.6250366330318675`*^9}, {
   3.6250367671921034`*^9, 3.6250368411362333`*^9}, 3.6251126016715207`*^9, {
   3.627984579406811*^9, 3.6279846420565214`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"curvatvec", "[", "t_", "]"}], " ", ":=", 
  RowBox[{
   RowBox[{"curvaturevec", "[", "sf", "]"}], "[", "t", "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6250366399270797`*^9, 3.6250367196744194`*^9}, 
   3.6251126527460103`*^9, {3.6279844774918323`*^9, 3.62798449348186*^9}, {
   3.628051575402383*^9, 3.628051599270425*^9}}],

Cell[TextData[{
 "Now we test the power of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " simplifying large symbolic expressions. Sometimes this power is great, but \
sometimes the simplification remains without success,"
}], "Text",
 CellChangeTimes->{{3.5984278441866727`*^9, 3.5984278488354807`*^9}, {
   3.6253017820189505`*^9, 3.625301782252951*^9}, 3.6279993615091743`*^9, {
   3.62799939308363*^9, 3.6279993938948317`*^9}, {3.6279994710910797`*^9, 
   3.6279996145557337`*^9}, 3.6279996744812546`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "norm"}]], "Input",
 CellChangeTimes->{{3.6280598655856256`*^9, 3.6280598690176315`*^9}}],

Cell[BoxData[
 StyleBox["\<\"norm[v] is the norm of the vector v with respect to a positive\
\\nsemi-definite scalar product which can be specified as an option for \
innerprod; the\\ndefault is Dot.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.628059870281234*^9},
 CellTags->"Info3628063470-3762495"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Context", "[", "norm", "]"}]], "Input",
 CellChangeTimes->{{3.628059906785298*^9, 3.628059920606922*^9}}],

Cell[BoxData["\<\"euvec`\"\>"], "Output",
 CellChangeTimes->{3.6280599215273237`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"norm", "[", 
  RowBox[{
   RowBox[{"tangent", "[", "cx", "]"}], "[", "s", "]"}], "]"}]], "Input",
 CellChangeTimes->{3.627998951961655*^9}],

Cell[BoxData[
 RowBox[{"\[Sqrt]", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["hv", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox["hu", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"], "/", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["hv", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox["hu", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["hv", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox["hu", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["hv", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox["hu", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
      ")"}]}], "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["hv", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox["hu", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"], "/", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["hv", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox["hu", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["hv", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox["hu", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["hv", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox["hu", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
      ")"}]}], "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["hv", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox["hu", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"], "/", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["hv", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox["hu", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["hv", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox["hu", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["hv", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox["hu", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"hu", "[", "s", "]"}], ",", 
             RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
      ")"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.627998184471507*^9, 3.6279992134337144`*^9, 
  3.6279996834044704`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", ",", 
   RowBox[{"Element", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"hu", "[", "t", "]"}], ",", 
       RowBox[{"hv", "[", "t", "]"}], ",", 
       RowBox[{
        RowBox[{"hu", "'"}], "[", "t", "]"}], ",", 
       RowBox[{
        RowBox[{"hv", "'"}], "[", "t", "]"}], ",", "t"}], "}"}], ",", 
     "Reals"}], "]"}], ",", 
   RowBox[{"TimeConstraint", " ", "\[Rule]", " ", "Infinity"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6279982036283407`*^9, 3.6279982443444123`*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.627998263392046*^9, 3.6279992217485294`*^9}]
}, Closed]],

Cell["Try the same with the built-in function ", "Text",
 CellChangeTimes->{{3.627999692158086*^9, 3.627999724639343*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "Norm"}]], "Input",
 CellChangeTimes->{{3.6279997278061485`*^9, 3.627999729958952*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox["\<\"\!\(\*RowBox[{\\\"Norm\\\", \\\"[\\\", StyleBox[\\\"expr\\\", \
\\\"TI\\\"], \\\"]\\\"}]\) gives the norm of a number, vector or matrix. \
\\n\!\(\*RowBox[{\\\"Norm\\\", \\\"[\\\", RowBox[{StyleBox[\\\"expr\\\", \
\\\"TI\\\"], \\\",\\\", StyleBox[\\\"p\\\", \\\"TI\\\"]}], \\\"]\\\"}]\) \
gives the \!\(\*StyleBox[\\\"p\\\", \\\"TI\\\"]\)\[Hyphen]norm. \"\>", "MSG"],
   "\[NonBreakingSpace]", 
  ButtonBox[
   StyleBox["\[RightSkeleton]", "SR"],
   Active->True,
   BaseStyle->"Link",
   ButtonData->"paclet:ref/Norm"]}]], "Print", "PrintUsage",
 CellChangeTimes->{3.6280599331337442`*^9},
 CellTags->"Info3628063532-3762495"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Context", "[", "Norm", "]"}]], "Input",
 CellChangeTimes->{{3.6280599391241546`*^9, 3.6280599483125706`*^9}}],

Cell[BoxData["\<\"System`\"\>"], "Output",
 CellChangeTimes->{3.628059949435773*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Norm", "[", 
    RowBox[{
     RowBox[{"tangent", "[", "cx", "]"}], "[", "s", "]"}], "]"}], ",", 
   RowBox[{"Element", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"hu", "[", "t", "]"}], ",", 
       RowBox[{"hv", "[", "t", "]"}], ",", 
       RowBox[{
        RowBox[{"hu", "'"}], "[", "t", "]"}], ",", 
       RowBox[{
        RowBox[{"hv", "'"}], "[", "t", "]"}], ",", "t"}], "}"}], ",", 
     "Reals"}], "]"}], ",", 
   RowBox[{"TimeConstraint", " ", "\[Rule]", " ", "Infinity"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6279982036283407`*^9, 3.6279982443444123`*^9}, {
   3.6279997912312603`*^9, 3.627999802980081*^9}, 3.627999845012555*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"\[Sqrt]", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"Abs", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["hv", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"hu", "[", "s", "]"}], ",", 
            RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox["hu", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"hu", "[", "s", "]"}], ",", 
            RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{"Abs", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["hv", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"hu", "[", "s", "]"}], ",", 
            RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox["hu", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"hu", "[", "s", "]"}], ",", 
            RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{"Abs", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["hv", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"hu", "[", "s", "]"}], ",", 
            RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox["hu", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"hu", "[", "s", "]"}], ",", 
            RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], "]"}], "2"]}], 
     ")"}]}], ")"}], "/", 
  RowBox[{"(", 
   RowBox[{"\[Sqrt]", 
    RowBox[{"Abs", "[", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["hv", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"hu", "[", "s", "]"}], ",", 
            RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox["hu", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"hu", "[", "s", "]"}], ",", 
            RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["hv", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"hu", "[", "s", "]"}], ",", 
            RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox["hu", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"hu", "[", "s", "]"}], ",", 
            RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["hv", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"hu", "[", "s", "]"}], ",", 
            RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox["hu", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"hu", "[", "s", "]"}], ",", 
            RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
     "]"}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.6279998576661777`*^9}]
}, Closed]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " doesn\[CloseCurlyQuote]t confirm that this expression equals 1."
}], "Text",
 CellChangeTimes->{{3.627999943493929*^9, 3.6279999750567846`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"Assuming", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"scond2", "[", "sf", "]"}], "[", "s", "]"}], ",", " ", 
    RowBox[{"Simplify", "[", 
     RowBox[{
      RowBox[{"curvatvec", "[", "s", "]"}], ",", 
      RowBox[{"Element", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"hu", "[", "t", "]"}], ",", 
          RowBox[{"hv", "[", "t", "]"}], ",", 
          RowBox[{
           RowBox[{"hu", "'"}], "[", "t", "]"}], ",", 
          RowBox[{
           RowBox[{"hv", "'"}], "[", "t", "]"}], ",", "t"}], "}"}], ",", 
        "Reals"}], "]"}], ",", 
      RowBox[{"TimeConstraint", " ", "\[Rule]", " ", "Infinity"}]}], "]"}]}], 
   "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.625036910712355*^9, 3.6250369422556105`*^9}, {
  3.62503698740209*^9, 3.625037006075323*^9}, {3.625040832060043*^9, 
  3.625040833573245*^9}, {3.6252914589882417`*^9, 3.6252914910618978`*^9}, {
  3.625301652476323*^9, 3.625301662694341*^9}, {3.6254739705053406`*^9, 
  3.6254739930473804`*^9}, {3.6254740463646746`*^9, 3.6254740667267103`*^9}, {
  3.627984731288678*^9, 3.6279847685259438`*^9}}],

Cell[BoxData["$Aborted"], "Output",
 CellChangeTimes->{3.6279905933265743`*^9, 3.6279981787930975`*^9}]
}, Closed]],

Cell["\<\
Aborted after waiting a long time. The simplification of large symbolic \
expressions is difficult. sometimes impossible, also if using the condition \
scond2[sf][s]. We shall tests such simplifications for general curves on \
specified surfaces, e.g..(and later)\
\>", "Text",
 CellChangeTimes->{{3.6250401786840954`*^9, 3.625040182303302*^9}, 
   3.62547825758156*^9, {3.625478369673415*^9, 3.6254784676571875`*^9}, 
   3.6254785007760453`*^9, {3.625478534924505*^9, 3.6254785805077853`*^9}, {
   3.6254899430977087`*^9, 3.6254900201894445`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"cxx", "[", 
   RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.627983866532359*^9, 3.6279838862507935`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"r", " ", 
    RowBox[{"Cos", "[", 
     RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
    RowBox[{"Cos", "[", 
     RowBox[{"hv", "[", "t", "]"}], "]"}]}], ",", 
   RowBox[{"r", " ", 
    RowBox[{"Cos", "[", 
     RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
    RowBox[{"Sin", "[", 
     RowBox[{"hu", "[", "t", "]"}], "]"}]}], ",", 
   RowBox[{"r", " ", 
    RowBox[{"Sin", "[", 
     RowBox[{"hv", "[", "t", "]"}], "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6279838872803955`*^9, 3.627990694664352*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"curvaturevec", "[", 
   RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.627984075463526*^9, 3.6279841041675763`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"r", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
             RowBox[{"Cos", "[", 
              RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox["hu", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "2"]}], "-", 
            RowBox[{"2", " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox["hu", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], " ", 
             RowBox[{
              SuperscriptBox["hv", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}]}], "+", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
             RowBox[{"Cos", "[", 
              RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox["hv", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox["hu", "\[Prime]\[Prime]",
               MultilineFunction->None], "[", "t", "]"}]}], "+", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox["hv", "\[Prime]\[Prime]",
               MultilineFunction->None], "[", "t", "]"}]}]}], ")"}]}], ")"}], 
        "/", 
        RowBox[{"(", 
         SqrtBox[
          RowBox[{
           SuperscriptBox["r", "2"], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"Cos", "[", 
                RowBox[{"hv", "[", "t", "]"}], "]"}], "2"], " ", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox["hv", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "2"]}], ")"}]}]], 
         ")"}]}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["r", "3"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}], "+", 
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}]}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "2"}], " ", 
            RowBox[{"Cos", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox["hu", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], "2"], " ", 
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}], "+", 
           RowBox[{"2", " ", 
            SuperscriptBox[
             RowBox[{"Cos", "[", 
              RowBox[{"hv", "[", "t", "]"}], "]"}], "2"], " ", 
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox["hu", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}], "+", 
           RowBox[{"2", " ", 
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox["hv", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}]}], ")"}]}], ")"}], 
       "/", 
       RowBox[{"(", 
        RowBox[{"2", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["r", "2"], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "t", "]"}], "]"}], "2"], " ", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], "2"]}], ")"}]}], 
           ")"}], 
          RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"\[Sqrt]", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["r", "2"], " ", 
         SuperscriptBox[
          RowBox[{"Cos", "[", 
           RowBox[{"hv", "[", "t", "]"}], "]"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["hv", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "r"}], " ", 
            RowBox[{"Cos", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}], "-", 
           RowBox[{"r", " ", 
            RowBox[{"Cos", "[", 
             RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"r", " ", 
            RowBox[{"Cos", "[", 
             RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
            RowBox[{"Cos", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}], "-", 
           RowBox[{"r", " ", 
            RowBox[{"Sin", "[", 
             RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}]}], ")"}], "2"]}], 
       ")"}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"r", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", 
             RowBox[{"Cos", "[", 
              RowBox[{"hv", "[", "t", "]"}], "]"}]}], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox["hu", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], "2"]}], "-", 
           RowBox[{"2", " ", 
            RowBox[{"Cos", "[", 
             RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}], "-", 
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox["hv", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
            RowBox[{"Cos", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox["hu", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}], "-", 
           RowBox[{
            RowBox[{"Sin", "[", 
             RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox["hv", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}]}], ")"}]}], ")"}], 
       "/", 
       RowBox[{"(", 
        SqrtBox[
         RowBox[{
          SuperscriptBox["r", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"hv", "[", "t", "]"}], "]"}], "2"], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox["hu", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox["hv", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], "2"]}], ")"}]}]], 
        ")"}]}], "-", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["r", "3"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
            RowBox[{"Cos", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}], "-", 
           RowBox[{
            RowBox[{"Sin", "[", 
             RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}]}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "2"}], " ", 
            RowBox[{"Cos", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox["hu", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], "2"], " ", 
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}], "+", 
           RowBox[{"2", " ", 
            SuperscriptBox[
             RowBox[{"Cos", "[", 
              RowBox[{"hv", "[", "t", "]"}], "]"}], "2"], " ", 
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox["hu", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}], "+", 
           RowBox[{"2", " ", 
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox["hv", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}]}], ")"}]}], ")"}], 
       "/", 
       RowBox[{"(", 
        RowBox[{"2", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["r", "2"], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "t", "]"}], "]"}], "2"], " ", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], "2"]}], ")"}]}], 
           ")"}], 
          RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"\[Sqrt]", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["r", "2"], " ", 
         SuperscriptBox[
          RowBox[{"Cos", "[", 
           RowBox[{"hv", "[", "t", "]"}], "]"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["hv", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "r"}], " ", 
            RowBox[{"Cos", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}], "-", 
           RowBox[{"r", " ", 
            RowBox[{"Cos", "[", 
             RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"r", " ", 
            RowBox[{"Cos", "[", 
             RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
            RowBox[{"Cos", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}], "-", 
           RowBox[{"r", " ", 
            RowBox[{"Sin", "[", 
             RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}]}], ")"}], "2"]}], 
       ")"}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{"r", " ", 
         RowBox[{"Sin", "[", 
          RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["hv", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "2"]}], 
        SqrtBox[
         RowBox[{
          SuperscriptBox["r", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"hv", "[", "t", "]"}], "]"}], "2"], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox["hu", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox["hv", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], "2"]}], ")"}]}]]]}],
       "+", 
      FractionBox[
       RowBox[{"r", " ", 
        RowBox[{"Cos", "[", 
         RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox["hv", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "t", "]"}]}], 
       SqrtBox[
        RowBox[{
         SuperscriptBox["r", "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"Cos", "[", 
              RowBox[{"hv", "[", "t", "]"}], "]"}], "2"], " ", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox["hu", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "2"]}], ")"}]}]]], 
      "-", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["r", "3"], " ", 
         RowBox[{"Cos", "[", 
          RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox["hv", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "2"}], " ", 
            RowBox[{"Cos", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox["hu", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], "2"], " ", 
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}], "+", 
           RowBox[{"2", " ", 
            SuperscriptBox[
             RowBox[{"Cos", "[", 
              RowBox[{"hv", "[", "t", "]"}], "]"}], "2"], " ", 
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox["hu", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}], "+", 
           RowBox[{"2", " ", 
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], " ", 
            RowBox[{
             SuperscriptBox["hv", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}]}], ")"}]}], ")"}], 
       "/", 
       RowBox[{"(", 
        RowBox[{"2", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["r", "2"], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "t", "]"}], "]"}], "2"], " ", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], "2"]}], ")"}]}], 
           ")"}], 
          RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"\[Sqrt]", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["r", "2"], " ", 
         SuperscriptBox[
          RowBox[{"Cos", "[", 
           RowBox[{"hv", "[", "t", "]"}], "]"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["hv", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "r"}], " ", 
            RowBox[{"Cos", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}], "-", 
           RowBox[{"r", " ", 
            RowBox[{"Cos", "[", 
             RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}]}], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"r", " ", 
            RowBox[{"Cos", "[", 
             RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
            RowBox[{"Cos", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}], "-", 
           RowBox[{"r", " ", 
            RowBox[{"Sin", "[", 
             RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}]}]}], ")"}], "2"]}], 
       ")"}]}], ")"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.627984105446779*^9, 3.628000043814106*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", ",", 
   RowBox[{"Element", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"hu", "[", "t", "]"}], ",", 
       RowBox[{"hv", "[", "t", "]"}], ",", 
       RowBox[{
        RowBox[{"hu", "'"}], "[", "t", "]"}], ",", 
       RowBox[{
        RowBox[{"hv", "'"}], "[", "t", "]"}], ",", "t"}], "}"}], ",", 
     "Reals"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.627984133979229*^9, 3.6279842634906564`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{"Cos", "[", 
         RowBox[{"hu", "[", "t", "]"}], "]"}]}], " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", 
         RowBox[{"hv", "[", "t", "]"}], "]"}], "3"], " ", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["hu", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], "4"]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Cos", "[", 
         RowBox[{"hv", "[", "t", "]"}], "]"}], "2"], " ", 
       RowBox[{"Sin", "[", 
        RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
       RowBox[{"Sin", "[", 
        RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["hu", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], "3"], " ", 
       RowBox[{
        SuperscriptBox["hv", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "-", 
      RowBox[{
       RowBox[{"Cos", "[", 
        RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["hv", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"Cos", "[", 
           RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox["hv", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
         RowBox[{
          RowBox[{"Sin", "[", 
           RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox["hu", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "t", "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox["hu", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], " ", 
       RowBox[{
        SuperscriptBox["hv", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"Sin", "[", 
           RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox["hv", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"hv", "[", "t", "]"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox["hu", "\[Prime]\[Prime]",
               MultilineFunction->None], "[", "t", "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{"Cos", "[", 
           RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox["hv", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "t", "]"}]}]}], ")"}]}], "-", 
      RowBox[{
       RowBox[{"Cos", "[", 
        RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
       RowBox[{"Cos", "[", 
        RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["hu", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox["hv", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
         RowBox[{
          RowBox[{"Cos", "[", 
           RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox["hv", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "t", "]"}]}]}], ")"}]}]}], ")"}], 
    "/", 
    RowBox[{"(", 
     RowBox[{"r", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"hv", "[", "t", "]"}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox["hu", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["hv", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "2"]}], ")"}], "2"]}], 
     ")"}]}], ",", 
   RowBox[{"-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", 
         RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox["hv", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Cos", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], "2"], " ", 
           RowBox[{"Sin", "[", 
            RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "3"]}], "-", 
          RowBox[{
           RowBox[{"Cos", "[", 
            RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
           RowBox[{
            SuperscriptBox["hv", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], " ", 
           RowBox[{
            SuperscriptBox["hu", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox["hu", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"2", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
             RowBox[{
              RowBox[{"Cos", "[", 
               RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
              RowBox[{
               SuperscriptBox["hv", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}]}]}], ")"}]}]}], 
         ")"}]}], "+", 
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"Cos", "[", 
         RowBox[{"hv", "[", "t", "]"}], "]"}], " ", 
        RowBox[{"Sin", "[", 
         RowBox[{"hu", "[", "t", "]"}], "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", 
           SuperscriptBox[
            RowBox[{"Cos", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "4"]}], "+", 
          RowBox[{"2", " ", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "4"]}], "-", 
          RowBox[{
           RowBox[{"Sin", "[", 
            RowBox[{"2", " ", 
             RowBox[{"hv", "[", "t", "]"}]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox["hu", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], " ", 
           RowBox[{
            SuperscriptBox["hv", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], " ", 
           RowBox[{
            SuperscriptBox["hu", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}]}], "+", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "2"], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
             RowBox[{
              RowBox[{"Sin", "[", 
               RowBox[{"2", " ", 
                RowBox[{"hv", "[", "t", "]"}]}], "]"}], " ", 
              RowBox[{
               SuperscriptBox["hv", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}]}]}], ")"}]}]}], 
         ")"}]}]}], ")"}], "/", 
     RowBox[{"(", 
      RowBox[{"r", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Cos", "[", 
             RowBox[{"hv", "[", "t", "]"}], "]"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox["hv", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "2"]}], ")"}], "2"]}],
       ")"}]}]}], ",", 
   FractionBox[
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       RowBox[{"Sin", "[", 
        RowBox[{"hv", "[", "t", "]"}], "]"}]}], " ", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["hv", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "4"]}], "-", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"Cos", "[", 
        RowBox[{"hv", "[", "t", "]"}], "]"}], "3"], " ", 
      RowBox[{
       SuperscriptBox["hu", "\[Prime]",
        MultilineFunction->None], "[", "t", "]"}], " ", 
      RowBox[{
       SuperscriptBox["hv", "\[Prime]",
        MultilineFunction->None], "[", "t", "]"}], " ", 
      RowBox[{
       SuperscriptBox["hu", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "t", "]"}]}], "+", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"Cos", "[", 
        RowBox[{"hv", "[", "t", "]"}], "]"}], "3"], " ", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["hu", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "2"], " ", 
      RowBox[{
       SuperscriptBox["hv", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "t", "]"}]}]}], 
    RowBox[{"r", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"Cos", "[", 
           RowBox[{"hv", "[", "t", "]"}], "]"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["hu", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["hv", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "2"]}], ")"}], 
      "2"]}]]}], "}"}]], "Output",
 CellChangeTimes->{3.627984314003545*^9, 3.628000061615737*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Together", "[", 
   RowBox[{
    RowBox[{"%%%", ".", "%%%"}], " ", "-", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"curvature", "[", 
       RowBox[{"cxx", "[", 
        RowBox[{"sphero", "[", "r", "]"}], "]"}], "]"}], "[", "t", "]"}], "^",
      "2"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.625489561092635*^9, 3.625489636030567*^9}, {
  3.6280003079555717`*^9, 3.628000350797247*^9}, {3.628001243087821*^9, 
  3.6280012547098417`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{
  3.628000297549353*^9, {3.628001231758201*^9, 3.628001261137053*^9}}]
}, Closed]],

Cell["\<\
The norm of the curvature vector equals the curvature of the curve:\
\>", "Text",
 CellChangeTimes->{{3.6254877745060835`*^9, 3.6254878051797376`*^9}, {
   3.6280000900429873`*^9, 3.628000090292588*^9}, 3.628000276487316*^9, 
   3.6280014229001384`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"curvatvec", "[", "s", "]"}], ".", 
    RowBox[{"curvatvec", "[", "s", "]"}]}], "-", 
   RowBox[{
    RowBox[{
     RowBox[{"curvature", "[", "cx", "]"}], "[", "s", "]"}], "^", "2"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.625037026838959*^9, 3.6250370385077796`*^9}, {
  3.625041055468035*^9, 3.625041065467653*^9}, {3.6252915316999693`*^9, 
  3.6252915382363806`*^9}, {3.6252955519362307`*^9, 3.625295571295864*^9}, {
  3.6252956122771363`*^9, 3.6252957028664956`*^9}, {3.6252957379041567`*^9, 
  3.6252957399009604`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.628001279439885*^9}]
}, Closed]],

Cell["OK.", "Text",
 CellChangeTimes->{{3.6254900729889383`*^9, 3.6254900744885406`*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.5. Normal and Principal Curvatures", "Subsection",
 CellChangeTimes->{{3.6253019673316765`*^9, 3.62530196942208*^9}, {
   3.625487887886483*^9, 3.6254878944560947`*^9}, {3.6254955731301117`*^9, 
   3.6254955894477406`*^9}, 3.6258042239622874`*^9, 3.628001374908454*^9, 
   3.628085057551937*^9}],

Cell["The curvature vector of a curve ", "Text",
 CellChangeTimes->{{3.6256503633483934`*^9, 3.625650415327685*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"cx", "[", "s", "]"}]], "Input",
 CellChangeTimes->{{3.6256504203040934`*^9, 3.6256504203040934`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "1", "]"}], "[", 
    RowBox[{
     RowBox[{"hu", "[", "s", "]"}], ",", 
     RowBox[{"hv", "[", "s", "]"}]}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "2", "]"}], "[", 
    RowBox[{
     RowBox[{"hu", "[", "s", "]"}], ",", 
     RowBox[{"hv", "[", "s", "]"}]}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "3", "]"}], "[", 
    RowBox[{
     RowBox[{"hu", "[", "s", "]"}], ",", 
     RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.628051293977889*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SuperscriptBox["hu", "\[Prime]",
     MultilineFunction->None], "[", "s", "]"}], ",", 
   RowBox[{
    SuperscriptBox["hv", "\[Prime]",
     MultilineFunction->None], "[", "s", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.628051350340788*^9}]
}, Closed]],

Cell["on the surface ", "Text",
 CellChangeTimes->{{3.6256504346561184`*^9, 3.62565045821216*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sf", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6256504686329784`*^9, 3.625650474607789*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "1", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "2", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "3", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6280512979714956`*^9}]
}, Closed]],

Cell["\<\
decomposes orthogonally into two components: the normal and the tangential \
component. The normal component is the projection of the second derivative of \
the curve  cx\[CloseCurlyQuote]\[CloseCurlyQuote][s], s its arc length, to \
the normal amf[3][sf] of the surface. We calculate for an arbitrary parameter \
s\
\>", "Text",
 CellChangeTimes->{{3.6256505100978513`*^9, 3.6256505142474585`*^9}, {
   3.62565070715102*^9, 3.6256507566967072`*^9}, {3.625650795868376*^9, 
   3.6256508451556644`*^9}, 3.625650879445525*^9, {3.6256509102897797`*^9, 
   3.6256509309286156`*^9}, {3.6256509643906746`*^9, 3.625650999289936*^9}, 
   3.6258042728995733`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"cx", "''"}], "[", "s", "]"}], ".", 
    RowBox[{
     RowBox[{
      RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{
      RowBox[{"hu", "[", "s", "]"}], ",", 
      RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
   RowBox[{
    RowBox[{
     RowBox[{"fund2a", "[", "sf", "]"}], "[", 
     RowBox[{
      RowBox[{"hu", "[", "s", "]"}], ",", 
      RowBox[{"hv", "[", "s", "]"}]}], "]"}], "[", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}], ",", 
     RowBox[{
      RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.591167006058077*^9, 3.591167099582426*^9}, {
   3.5911671805020547`*^9, 3.5911672617777033`*^9}, 3.6114713759407644`*^9, 
   3.6148618589731383`*^9, 3.6149177907973595`*^9, 3.615262266573435*^9, 
   3.6180441624927607`*^9, {3.625041188676669*^9, 3.625041192171075*^9}, 
   3.6250412242915316`*^9, 3.6250413414477377`*^9, 3.6253020700578566`*^9, {
   3.6253021015543118`*^9, 3.6253021613024173`*^9}, {3.6253022616885934`*^9, 
   3.6253022634045963`*^9}, {3.6253023696407833`*^9, 3.6253023712787857`*^9}, 
   3.625303907085883*^9, {3.6256418568786397`*^9, 3.6256419171415453`*^9}, {
   3.6256419513836055`*^9, 3.6256419649868293`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.628051393708864*^9}]
}, Closed]],

Cell["In particular, we obtain for the curvature vector:", "Text",
 CellChangeTimes->{{3.625645256701811*^9, 3.625645335341549*^9}, {
  3.6256510287593875`*^9, 3.625651085137887*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "curvatvec"}]], "Input",
 CellChangeTimes->{{3.628051466108591*^9, 3.6280514715530005`*^9}}],

Cell[CellGroupData[{

Cell["Global`curvatvec", "Print", "PrintUsage",
 CellChangeTimes->{3.6280516620137353`*^9},
 CellTags->"Info3628055261-3902486"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"curvatvec", "[", "t_", "]"}], ":=", 
         RowBox[{
          RowBox[{"curvaturevec", "[", "sf", "]"}], "[", "t", "]"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`curvatvec],
  Editable->False]], "Print",
 CellChangeTimes->{3.6280516620137353`*^9},
 CellTags->"Info3628055261-3902486"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"curvatvec", "[", "t", "]"}], ".", 
    RowBox[{
     RowBox[{
      RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{
      RowBox[{"hu", "[", "t", "]"}], ",", 
      RowBox[{"hv", "[", "t", "]"}]}], "]"}]}], "-", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"fund2a", "[", "sf", "]"}], "[", 
      RowBox[{
       RowBox[{"hu", "[", "t", "]"}], ",", 
       RowBox[{"hv", "[", "t", "]"}]}], "]"}], "[", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Gamma]", "'"}], "[", "t", "]"}], ",", 
      RowBox[{
       RowBox[{"\[Gamma]", "'"}], "[", "t", "]"}]}], "]"}], "/", 
    RowBox[{
     RowBox[{
      RowBox[{"fund1", "[", "sf", "]"}], "[", 
      RowBox[{
       RowBox[{"hu", "[", "t", "]"}], ",", 
       RowBox[{"hv", "[", "t", "]"}]}], "]"}], "[", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Gamma]", "'"}], "[", "t", "]"}], ",", 
      RowBox[{
       RowBox[{"\[Gamma]", "'"}], "[", "t", "]"}]}], "]"}]}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.591167006058077*^9, 3.591167099582426*^9}, {
   3.5911671805020547`*^9, 3.5911672617777033`*^9}, 3.6114713759407644`*^9, 
   3.6148618589731383`*^9, 3.6149177907973595`*^9, 3.615262266573435*^9, 
   3.6180441624927607`*^9, {3.625041188676669*^9, 3.625041192171075*^9}, 
   3.6250412242915316`*^9, 3.6250413414477377`*^9, 3.6253020700578566`*^9, {
   3.6253021015543118`*^9, 3.6253021613024173`*^9}, {3.6253022616885934`*^9, 
   3.6253022634045963`*^9}, {3.6253023696407833`*^9, 3.6253023712787857`*^9}, 
   3.625303907085883*^9, {3.6256418568786397`*^9, 3.6256419171415453`*^9}, {
   3.6256419513836055`*^9, 3.6256419649868293`*^9}, 3.6256438308593063`*^9, {
   3.6256438615133605`*^9, 3.6256439269866753`*^9}, {3.628051718719835*^9, 
   3.628051768483922*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6280514103696933`*^9, 3.628051648769312*^9, 
  3.6280517752387342`*^9}]
}, Closed]],

Cell[TextData[{
 "This formula is valid for an arbitrary parameter t. We conclude\n",
 StyleBox["Proposition",
  FontColor->RGBColor[0, 0, 1]],
 ". ",
 StyleBox["All curves ",
  FontSlant->"Italic"],
 "cx",
 StyleBox[" on the surface through the point sf[u,v] with the same tangent  \
at this point have the same ",
  FontSlant->"Italic"],
 StyleBox["normal curvature",
  FontSlant->"Italic",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox[" defined by",
  FontSlant->"Italic"],
 " fund2a[sf][u,v][\[Gamma]\[CloseCurlyQuote][s],\[Gamma]\[CloseCurlyQuote][s]\
], s  ",
 StyleBox["the arc length of the curve.  ",
  FontSlant->"Italic"],
 "\[FilledRectangle]"
}], "Text",
 CellChangeTimes->{{3.59117015034192*^9, 3.591170175986387*^9}, {
   3.5911702324906187`*^9, 3.5911703699304795`*^9}, {3.591170402012315*^9, 
   3.591170414571033*^9}, {3.5983664215985203`*^9, 3.598366434774274*^9}, {
   3.598428209586115*^9, 3.5984282141101227`*^9}, {3.5984287710467005`*^9, 
   3.5984287755863085`*^9}, {3.598443070436406*^9, 3.598443070450407*^9}, 
   3.6114713772008367`*^9, 3.6152810347747393`*^9, {3.6256511613908205`*^9, 
   3.625651224758132*^9}, {3.625651318217896*^9, 3.6256513230383043`*^9}, 
   3.6280517867827544`*^9}],

Cell["\<\
We defined the normal curvature for a curve {h1[t],h2[t]}  with an arbitrary \
parameter t on the surface ff[u,v] in diffgeov2.m by\
\>", "Text",
 CellChangeTimes->{{3.614310317474022*^9, 3.6143103199232264`*^9}, {
   3.6143104210818157`*^9, 3.6143106065544243`*^9}, 3.6152810492845693`*^9, {
   3.625651367825983*^9, 3.625651386062415*^9}, {3.6256514204448757`*^9, 
   3.6256514532517333`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "normalcurvature"}]], "Input",
 CellChangeTimes->{{3.6256514626741495`*^9, 3.625651470708164*^9}}],

Cell[BoxData[
 StyleBox["\<\"normalcurvature[ff,hu,hv][t]is the normal curvature of \\nthe \
curve ff[hu[t],hv[t]], t an arbitrary parameter.\"\>", "MSG"]], "Print", \
"PrintUsage",
 CellChangeTimes->{3.6280518361724415`*^9},
 CellTags->"Info3628055436-3902486"]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"normalcurvature", "[", 
    RowBox[{"ff_", ",", "h1_", ",", "h2_"}], "]"}], "[", "t_", "]"}], ":=", 
  " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"cvx", ",", "tt", ",", "vt", ",", "tx", ",", "aa", ",", "bb"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"cvx", "[", "tt_", "]"}], " ", ":=", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"h1", "[", "tt", "]"}], ",", 
        RowBox[{"h2", "[", "tt", "]"}]}], "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"vt", "[", "tt_", "]"}], " ", ":=", " ", 
      RowBox[{
       RowBox[{"D", "[", 
        RowBox[{
         RowBox[{"cvx", "[", "tx", "]"}], ",", "tx"}], "]"}], "/.", 
       RowBox[{"tx", "\[Rule]", "tt"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"aa", " ", "=", " ", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         RowBox[{
          RowBox[{"fund2", "[", "ff", "]"}], "[", 
          RowBox[{
           RowBox[{"h1", "[", "tt", "]"}], ",", 
           RowBox[{"h2", "[", "tt", "]"}]}], "]"}], "[", 
         RowBox[{
          RowBox[{"vt", "[", "tt", "]"}], ",", 
          RowBox[{"vt", "[", "tt", "]"}]}], "]"}]}], "/.", 
       RowBox[{"tt", "\[Rule]", "t"}]}]}], ";", 
     RowBox[{"bb", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"fund1", "[", "ff", "]"}], "[", 
         RowBox[{
          RowBox[{"h1", "[", "tt", "]"}], ",", 
          RowBox[{"h2", "[", "tt", "]"}]}], "]"}], "[", 
        RowBox[{
         RowBox[{"vt", "[", "tt", "]"}], ",", 
         RowBox[{"vt", "[", "tt", "]"}]}], "]"}], "/.", 
       RowBox[{"tt", "\[Rule]", "t"}]}]}], ";", 
     RowBox[{"aa", "/", "bb"}]}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6143106214582767`*^9, 3.614310705541086*^9}, {
   3.6143107438282757`*^9, 3.6143107815564337`*^9}, {3.614310816436429*^9, 
   3.614310915165076*^9}, {3.614310967790086*^9, 3.614311041454299*^9}, {
   3.614311075774262*^9, 3.614311217126347*^9}, {3.6143112637990165`*^9, 
   3.6143112898545065`*^9}, {3.614311337239217*^9, 3.6143113487278743`*^9}, {
   3.614311448903604*^9, 3.614311576977929*^9}, {3.6148618589731383`*^9, 
   3.6148618589731383`*^9}, 3.6148631365217824`*^9, 3.614917790812959*^9, {
   3.618395996020481*^9, 3.6183960036020947`*^9}, {3.618396341607888*^9, 
   3.6183963564903145`*^9}}],

Cell["Indeed, we have", "Text",
 CellChangeTimes->{{3.6256515285530653`*^9, 3.6256515378662815`*^9}, {
  3.625651572825943*^9, 3.6256515747135468`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"curvatvec", "[", "s", "]"}], ".", 
    RowBox[{
     RowBox[{
      RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{
      RowBox[{"hu", "[", "s", "]"}], ",", 
      RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", " ", 
   RowBox[{
    RowBox[{"normalcurvature", "[", 
     RowBox[{"sf", ",", "hu", ",", "hv"}], "]"}], "[", "s", "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.625303629467796*^9, 3.625303731413975*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.62805187537531*^9}]
}, Closed]],

Cell["\<\
The expression aa/bb in the Module  is invariant under a parameter \
transformation, and it coincides with the result of the proposition if t = s \
is the arc length parameter.\
\>", "Text",
 CellChangeTimes->{{3.6143128012499533`*^9, 3.6143129209067974`*^9}, 
   3.615281888172551*^9, 3.6184068592621465`*^9, {3.6256516497964783`*^9, 
   3.625651666394908*^9}, {3.628051900054553*^9, 3.628051901817356*^9}}],

Cell[TextData[{
 StyleBox["Example.",
  FontColor->RGBColor[0, 0, 1]],
 " Normal curvature of a general curve on the sphere of radius r"
}], "Text",
 CellChangeTimes->{{3.61431205516628*^9, 3.6143121144266696`*^9}, {
   3.6143125946581373`*^9, 3.614312626209942*^9}, 3.615281900524257*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"normalcurvature", "[", 
     RowBox[{
      RowBox[{"sphero", "[", "r", "]"}], ",", "hu", ",", "hv"}], "]"}], "[", 
    "t", "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"hu", "[", "t", "]"}], "\[Element]", "Reals"}], ",", 
     RowBox[{
      RowBox[{"hv", "[", "t", "]"}], "\[Element]", "Reals"}], ",", 
     RowBox[{"r", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.614311584076335*^9, 3.614311620931443*^9}, {
   3.614312164300522*^9, 3.6143122685814867`*^9}, 3.6148618589887385`*^9, 
   3.614863136537382*^9}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{
    RowBox[{"Abs", "[", 
     RowBox[{"Sec", "[", 
      RowBox[{"hv", "[", "t", "]"}], "]"}], "]"}], " ", 
    RowBox[{"Cos", "[", 
     RowBox[{"hv", "[", "t", "]"}], "]"}]}], "r"]}]], "Output",
 CellChangeTimes->{3.6280519283686028`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"%", "\[Equal]", " ", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"Sign", "[", 
       RowBox[{"Cos", "[", 
        RowBox[{"hv", "[", "t", "]"}], "]"}], "]"}]}], "/", "r"}]}], ",", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"r", ">", "0"}], ",", 
     RowBox[{
      RowBox[{"hv", "[", "t", "]"}], "\[Element]", "Reals"}]}], "}"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6143116405155635`*^9, 3.614311725388418*^9}, {
  3.614312338661495*^9, 3.614312447581725*^9}}],

Cell[TextData[{
 "All curves through  a given point of the sphere have at this point the same \
normal curvature! A point on a surface with this property is said to be ",
 StyleBox["umbilic",
  FontColor->RGBColor[0, 0, 1]],
 ". On a general surface umbilic points don\[CloseCurlyQuote]t need to exist. \
On a sphere all points are umbilic. \[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.6183972736471252`*^9, 3.618397349026458*^9}, {
   3.6183975386560383`*^9, 3.6183975462724743`*^9}, {3.6256519947920847`*^9, 
   3.625652012154915*^9}, {3.6256520442753716`*^9, 3.625652138889538*^9}, {
   3.625652231788701*^9, 3.6256522402283154`*^9}, {3.625652273955575*^9, 
   3.6256522761239786`*^9}, 3.6256599037945337`*^9}],

Cell["\<\
The sign of the normal curvature as well as the sign of the second \
fundamental form depends on the orientation of the surface: it is reversed if \
the orientation is changed.\
\>", "Text",
 CellChangeTimes->{{3.614312497097557*^9, 3.614312497099557*^9}, 
   3.6256522687763653`*^9, {3.6256523066844325`*^9, 3.625652402703601*^9}}],

Cell["\<\
Now we consider the normal plane of the curve at the point cx[s]. It can be \
spanned by the normal vector of the curve belonging to the tangent plane of \
the surface:\
\>", "Text",
 CellChangeTimes->{{3.591182851545838*^9, 3.5911830541589937`*^9}, {
   3.5913402274775476`*^9, 3.5913402300671525`*^9}, 3.6114713785929165`*^9, 
   3.6116379545436625`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"a", "[", "2", "]"}], " ", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "  ", 
     RowBox[{
      RowBox[{"amf", "[", "1", "]"}], "[", "sf", "]"}]}], "*", 
    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], " ", "+", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"amf", "[", "2", "]"}], "[", "sf", "]"}], "*", 
    RowBox[{"Cos", "[", "\[Tau]", "]"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5911830364997625`*^9, 3.5911830411173706`*^9}, {
  3.591183096435068*^9, 3.5911832179904814`*^9}, {3.6148618589887385`*^9, 
  3.6148618589887385`*^9}, {3.614917790828559*^9, 3.614917790828559*^9}, {
  3.6152622786691265`*^9, 3.615262279580179*^9}, {3.6280521694826264`*^9, 
  3.628052224971924*^9}}],

Cell["\<\
and the normal vector amf[3][sf] of the surface; here \[Tau] denotes the \
angle between the tangent of the curve and the vector amf[1][sf][u,v]. We \
decompose the curvature vector cx\[CloseCurlyQuote]\[CloseCurlyQuote][s]  \
lying in the normal plane into its normal and its tangential components:\
\>", "Text",
 CellChangeTimes->{{3.5911833479543095`*^9, 3.591183380074766*^9}, {
   3.59118341636043*^9, 3.591183463690913*^9}, {3.591183535118173*^9, 
   3.5911835543982763`*^9}, {3.591415122160385*^9, 3.591415157112384*^9}, 
   3.611471379982996*^9, 3.6148618590199385`*^9, 3.6149177908597593`*^9, 
   3.6152622808812532`*^9, {3.6256525658360877`*^9, 3.6256526363326116`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"cx", "''"}], "[", "s", "]"}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["k", "n"], "[", "s", "]"}], "*", 
    RowBox[{
     RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}]}], " ", "+", " ", 
   RowBox[{
    RowBox[{
     SubscriptBox["k", "g"], "[", "s", "]"}], " ", "*", 
    RowBox[{"a", "[", "2", "]"}]}]}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{"k", "[", "s", "]"}], "*", 
   RowBox[{
    RowBox[{
     RowBox[{"frenet", "[", "cx", "]"}], "[", "s", "]"}], "[", "2", 
    "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5911839809576735`*^9, 3.5911840523367567`*^9}, {
   3.59118409034593*^9, 3.5911842270017467`*^9}, {3.591184262162758*^9, 
   3.591184277106612*^9}, {3.59118467761052*^9, 3.5911846817297554`*^9}, {
   3.591247611926702*^9, 3.5912476277139297`*^9}, {3.6114713820901165`*^9, 
   3.6114713829871674`*^9}, 3.6148618590199385`*^9, 3.6149177908753595`*^9, 
   3.615262281932314*^9}],

Cell[TextData[{
 "Here ",
 Cell[BoxData[
  SubscriptBox["k", "n"]],
  Evaluatable->False,
  CellChangeTimes->{{3.5911839809576735`*^9, 3.5911840523367567`*^9}, {
   3.59118409034593*^9, 3.5911842270017467`*^9}, {3.591184262162758*^9, 
   3.591184277106612*^9}}],
 " denotes the normal curvature defined above, ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["k", "g"], " "}]],
  Evaluatable->False,
  CellChangeTimes->{{3.5911839809576735`*^9, 3.5911840523367567`*^9}, {
   3.59118409034593*^9, 3.5911842270017467`*^9}, {3.591184262162758*^9, 
   3.591184277106612*^9}}],
 " the so called ",
 StyleBox["geodesic  curvature",
  FontColor->RGBColor[0, 0, 1]],
 " considered later in Subsection 2.4.6, and k = k[s] the curvature of the \
curve. A trivial but remarkable consequence of this formula is"
}], "Text",
 CellChangeTimes->{{3.5911844346546235`*^9, 3.5911845590627394`*^9}, {
   3.5911846124467926`*^9, 3.5911846528151016`*^9}, {3.5911846911002913`*^9, 
   3.591184738406997*^9}, 3.591184783935601*^9, {3.591184882295227*^9, 
   3.591184892598817*^9}, {3.591184945407837*^9, 3.5911850142957773`*^9}, 
   3.5912474146331553`*^9, {3.591247670036804*^9, 3.5912477102692747`*^9}, {
   3.624771935816691*^9, 3.6247719464403095`*^9}},
 CellTags->"geodc"],

Cell[TextData[{
 StyleBox["Lemma.",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox[" The geodesic curvature satisfies the formula",
  FontSlant->"Italic"]
}], "Text",
 CellChangeTimes->{{3.591247751796548*^9, 3.5912479321172647`*^9}, {
   3.5912479914753685`*^9, 3.5912480101954017`*^9}, 3.611471384289242*^9, {
   3.6116387366198363`*^9, 3.61163878415312*^9}, {3.6116388403912187`*^9, 
   3.611638846319229*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{
     SubscriptBox["k", "g"], "(", "s", ")"}], "2"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"k", "(", "s", ")"}], "2"], "-", 
    SuperscriptBox[
     RowBox[{
      SubscriptBox["k", "n"], "(", "s", ")"}], "2"]}]}], 
  TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6116388736348767`*^9, 3.611638942649398*^9}}],

Cell[TextData[{
 StyleBox["A point ",
  FontSlant->"Italic"],
 "cx",
 StyleBox["[s] is a",
  FontSlant->"Italic"],
 StyleBox[" flat point of the curve",
  FontSlant->"Italic",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox[" (what means",
  FontSlant->"Italic"],
 " k[s] = 0) ",
 StyleBox["iff ",
  FontSlant->"Italic"]
}], "Text",
 CellChangeTimes->{{3.591247751796548*^9, 3.5912479321172647`*^9}, {
   3.5912479914753685`*^9, 3.5912480101954017`*^9}, 3.611471384289242*^9, {
   3.6116387366198363`*^9, 3.61163878415312*^9}, 3.611638959684628*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["k", "n"], "[", "s", "]"}], "=", " ", 
  RowBox[{
   RowBox[{
    SubscriptBox["k", "g"], "[", "s", "]"}], "=", " ", "0.", 
   "  "}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5912479532553015`*^9, 3.5912479532553015`*^9}, 
   3.5912480620186925`*^9, {3.5984431524080944`*^9, 3.598443158216427*^9}, 
   3.61839774271371*^9}],

Cell["\<\
In particular, any line lying in the surface has vanishing normal and \
geodesic curvatures. \[FilledSquare]\
\>", "Text",
 CellChangeTimes->{{3.5912480764175177`*^9, 3.5912480831255293`*^9}, {
   3.591248205517848*^9, 3.5912482771579456`*^9}, 3.598429103155684*^9, {
   3.618397761840804*^9, 3.618397761842804*^9}}],

Cell["\<\
 The angle \[Theta] between cx''[s] and a[2] equals the angle between the \
tangent plane of the surface and the osculating plane of the curve. By \
Pythagoras it follows\
\>", "Text",
 CellChangeTimes->{{3.5911844346546235`*^9, 3.5911845590627394`*^9}, {
   3.5911846124467926`*^9, 3.5911846528151016`*^9}, {3.5911846911002913`*^9, 
   3.591184738406997*^9}, 3.591184783935601*^9, {3.591184882295227*^9, 
   3.591184892598817*^9}, {3.591184945407837*^9, 3.5911850142957773`*^9}, 
   3.5912474146331553`*^9, {3.591247670036804*^9, 3.5912477102692747`*^9}, 
   3.611471385545314*^9, 3.628052596321707*^9}],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["k", "n"], "=", 
    RowBox[{"k", " ", 
     RowBox[{"sin", "(", "\[Theta]", ")"}]}]}], ",", "      ", 
   RowBox[{
    RowBox[{"or", "   ", 
     FractionBox["1", 
      SubscriptBox["r", "n"]]}], "=", 
    FractionBox[
     RowBox[{" ", 
      RowBox[{"sin", "(", "\[Theta]", ")"}]}], "r"]}], ","}], 
  TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.591185108512166*^9, 3.591185329658815*^9}, {
   3.591185370764166*^9, 3.591185390363287*^9}, {3.5911854204750094`*^9, 
   3.5911854560440435`*^9}, 3.591185489753972*^9}],

Cell[TextData[{
 "where ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "n"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.591185108512166*^9, 3.591185329658815*^9}, {
    3.591185370764166*^9, 3.591185390363287*^9}, {3.5911854204750094`*^9, 
    3.5911854560440435`*^9}, 3.591185489753972*^9}],
 " = 1/",
 Cell[BoxData[
  FormBox[
   SubscriptBox["k", "n"], TraditionalForm]]],
 " and r = 1/k are the corresponding curvature radii. This formula is the \
content of \n\n",
 StyleBox["Meusnier\[CloseCurlyQuote]s Proposition",
  FontColor->RGBColor[0, 0, 1]],
 ". ",
 StyleBox["All curves ",
  FontSlant->"Italic"],
 "cx",
 StyleBox[" on the surface sf whose osculating plane at the point ",
  FontSlant->"Italic"],
 "cx",
 StyleBox["[s] has an angle \[Theta] \[NotEqual] 0, \[Pi]   with the tangent \
plane have at this point the same curvature, namely\n ",
  FontSlant->"Italic"],
 "k = ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["k", "n"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.591185108512166*^9, 3.591185329658815*^9}, {
    3.591185370764166*^9, 3.591185390363287*^9}, {3.5911854204750094`*^9, 
    3.5911854560440435`*^9}, 3.591185489753972*^9},
  FontSlant->"Plain"],
 "/sin(\[Theta]). \[FilledRectangle]"
}], "Text",
 CellChangeTimes->{{3.5911854976874256`*^9, 3.5911855165425043`*^9}, {
  3.5911855569068127`*^9, 3.591185646350929*^9}, {3.59118578763901*^9, 
  3.59118583029445*^9}, {3.591186005119449*^9, 3.591186281990285*^9}, {
  3.598429373769359*^9, 3.5984293827549753`*^9}, {3.5984432845576525`*^9, 
  3.598443288191861*^9}, {3.611471386802386*^9, 3.6114713884584804`*^9}, {
  3.6280526662750316`*^9, 3.6280526713606405`*^9}}],

Cell[TextData[{
 "In particular one obtains such a curve intersecting the surface with a \
plane containing the point x = sf[u,v]: a ",
 StyleBox["plane section",
  FontColor->RGBColor[0, 0, 1]],
 " of the surface. The curvature of the plane section through x containing \
the tangent vector"
}], "Text",
 CellChangeTimes->{{3.5911866587728357`*^9, 3.591186787255185*^9}, {
   3.591186817454912*^9, 3.5911868375110593`*^9}, {3.5911868704389424`*^9, 
   3.5911871717431765`*^9}, 3.6116396765994873`*^9}],

Cell[" tt[\[Tau]]  = amf[1][sf]*Cos[\[Tau]] + amf[2][sf]*Sin[\[Tau]] ", "Text",
 CellChangeTimes->{{3.5911866587728357`*^9, 3.591186787255185*^9}, {
  3.591186817454912*^9, 3.5911868375110593`*^9}, {3.5911868704389424`*^9, 
  3.5911871717431765`*^9}, {3.6116396765994873`*^9, 3.611639688003107*^9}, {
  3.6153607884512267`*^9, 3.6153608190829787`*^9}}],

Cell["is defined by the normal curvature ", "Text",
 CellChangeTimes->{{3.5911866587728357`*^9, 3.591186787255185*^9}, {
  3.591186817454912*^9, 3.5911868375110593`*^9}, {3.5911868704389424`*^9, 
  3.5911871717431765`*^9}, {3.6116396765994873`*^9, 3.611639688003107*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["k", "n"], "[", "\[Tau]", "]"}], "=", " ", 
   RowBox[{"-", 
    RowBox[{"fund2", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"sf", "[", 
        RowBox[{"u", ",", "v"}], "]"}], 
       RowBox[{"tt", "[", "\[Tau]", "]"}]}], ",", 
      RowBox[{"tt", "[", "\[Tau]", "]"}]}], "]"}], " "}]}], 
  TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5911872243951874`*^9, 3.591187294696209*^9}, 
   3.5911873290491734`*^9, {3.6183982560420704`*^9, 3.618398263809515*^9}}],

Cell[TextData[{
 "and the angle \[Theta] by Meusnier\[CloseCurlyQuote]s formula. The case  \
\[Theta] = 0, \[Pi]  is excluded since the intersection of the tangent plane \
with the surface yields a curve  (or a point) being singular at this point. \
The special sections containing the normal of the surface (corresponding to  \
\[Theta] = \[Pi]/2) at a considered point are called ",
 StyleBox["normal sections ",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox["at this point.",
  FontColor->GrayLevel[0]],
 " For the normal sections the curvature at this point coincides with the \
normal curvature. This property is an appropriate geometric interpretation of \
the second fundamental form."
}], "Text",
 CellChangeTimes->{{3.591187367878394*^9, 3.5911875588153152`*^9}, {
   3.5911875983665776`*^9, 3.591187677583109*^9}, {3.591187709710946*^9, 
   3.591187782231094*^9}, {3.5911878983987384`*^9, 3.591187971934944*^9}, 
   3.5913413340342913`*^9, {3.611639842490179*^9, 3.611639889290261*^9}, {
   3.611639940676751*^9, 3.611639947384763*^9}}],

Cell["\<\
By the last formula the normal curvature of curves through a point sf[u,v] on \
the surface is a quadratic form of the tangent vectors tt at this point. Its \
extreme values under the condition\
\>", "Text",
 CellChangeTimes->{{3.591255092426669*^9, 3.5912552697677803`*^9}, 
   3.591255317035863*^9, {3.591255404676817*^9, 3.5912554445504875`*^9}, 
   3.5914189635761013`*^9, {3.5914191306166553`*^9, 3.591419151610856*^9}, 
   3.618406972686634*^9, 3.6256531094878626`*^9, {3.628052873575998*^9, 
   3.6280528741063986`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fund1", "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{"tt", ",", "tt"}], "]"}], " ", "\[Equal]", " ", "1"}], 
  ","}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5914192915048575`*^9, 3.591419294121007*^9}, {
  3.5914193733635397`*^9, 3.591419415121928*^9}}],

Cell[TextData[{
 " i.e. varying \[Tau], are called the ",
 StyleBox["principal curvatures",
  FontColor->RGBColor[0, 0, 1]],
 " of the surface at this point. They are the eigenvalues of the second \
fundamental form"
}], "Text",
 CellChangeTimes->{{3.591255092426669*^9, 3.5912552697677803`*^9}, 
   3.591255317035863*^9, {3.591255404676817*^9, 3.5912554445504875`*^9}, 
   3.5914189635761013`*^9, {3.5914191306166553`*^9, 3.591419151610856*^9}, {
   3.5914194705841007`*^9, 3.591419508504269*^9}, 3.59150772294116*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "prcurvatures", "]"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"prcurvatures", "[", "ff_", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", 
   RowBox[{"Eigenvalues", "[", " ", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"cf", "[", 
         RowBox[{"i", ",", "j"}], "]"}], "[", "ff", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5912555454202642`*^9, 3.591255666928878*^9}, 
   3.5912557322461925`*^9, {3.5923133603434553`*^9, 3.5923135255321455`*^9}, {
   3.592313755554549*^9, 3.5923137596573563`*^9}, {3.592313883084773*^9, 
   3.5923138845043755`*^9}, {3.5923139232080436`*^9, 
   3.5923139658897185`*^9}, {3.5923140976943502`*^9, 3.59231414305923*^9}, {
   3.5923141935877185`*^9, 3.592314214320155*^9}, {3.592314548823943*^9, 
   3.592314676946968*^9}, 3.614861859051139*^9, 3.614863501734024*^9, 
   3.6179720955858417`*^9, 3.62565314221672*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "prcurvatures"}]], "Input",
 CellChangeTimes->{{3.6153610380975056`*^9, 3.6153611275666227`*^9}, {
  3.6227207800409913`*^9, 3.622720789713008*^9}}],

Cell[BoxData[
 StyleBox["\<\"prcurvatures[f][u,v] yields the principal curvatures of \\nthe \
surface f[u,v] in the Euclidean 3-space.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.628059821921149*^9},
 CellTags->"Info3628063421-3762495"]
}, Closed]],

Cell["For a sphere of radius r we obtain ", "Text",
 CellChangeTimes->{{3.591419571946898*^9, 3.591419599568478*^9}, {
   3.591419672176631*^9, 3.591419681112142*^9}, 3.5914215436806746`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"cf", "[", 
        RowBox[{"i", ",", "j"}], "]"}], "[", 
       RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "2"}], "}"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"r", ">", "0"}], ",", 
     RowBox[{
      RowBox[{"Cos", "[", "v", "]"}], ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.625660071276428*^9, 3.625660086439655*^9}, 
   3.625660132771736*^9, {3.625728855734329*^9, 3.625728862379941*^9}, {
   3.6257288999916067`*^9, 3.6257289195580416`*^9}, {3.62572896420532*^9, 
   3.6257290078113966`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     FractionBox["1", "r"], ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", 
     FractionBox["1", "r"]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6280600067658734`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"prcurvatures", "[", 
     RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"r", ">", "0"}], ",", " ", 
     RowBox[{
      RowBox[{"Cos", "[", "v", "]"}], ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5912556786912985`*^9, 3.59125568522771*^9}, {
  3.591255752541828*^9, 3.5912558042715187`*^9}, {3.591419629908213*^9, 
  3.5914196592438912`*^9}, {3.592314726508255*^9, 3.5923147697359304`*^9}, {
  3.611640033496914*^9, 3.611640064338168*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox["1", "r"], ",", 
   FractionBox["1", "r"]}], "}"}]], "Output",
 CellChangeTimes->{3.628060022662301*^9}]
}, Closed]],

Cell["\<\
As expected by symmetry reasons, the normal curvature on a sphere is \
constant. In general the principal curvatures  differ and vary; e. g. for the \
hyperbolic paraboloid one has\
\>", "Text",
 CellChangeTimes->{{3.591419786766185*^9, 3.591419851071863*^9}, {
  3.5914200225206695`*^9, 3.591420025927864*^9}, {3.591420094999815*^9, 
  3.591420137944271*^9}, {3.591420194544509*^9, 3.5914202251602592`*^9}, {
  3.5914202937611837`*^9, 3.591420296000312*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"prcurvatures", "[", "hypsf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
   RowBox[{"v", "\[Element]", " ", "Reals"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5912556786912985`*^9, 3.59125568522771*^9}, {
  3.591255752541828*^9, 3.5912558042715187`*^9}, {3.5913414958377757`*^9, 
  3.5913415065081944`*^9}, {3.5914198948743687`*^9, 3.591419913586439*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", 
     SuperscriptBox[
      RowBox[{"Cosh", "[", 
       RowBox[{"2", " ", "v"}], "]"}], 
      RowBox[{"3", "/", "2"}]]]}], ",", 
   FractionBox["1", 
    SqrtBox[
     RowBox[{"Cosh", "[", 
      RowBox[{"2", " ", "v"}], "]"}]]]}], "}"}]], "Output",
 CellChangeTimes->{3.628060047497545*^9}]
}, Closed]],

Cell[TextData[{
 "A point sf[u, v]  is  an ",
 StyleBox["umbilic",
  FontColor->RGBColor[0, 0, 1]],
 " (or an umbilic point) if the principal curvatures at this point coincide. \
Special umbilic points are the",
 StyleBox[" flat points",
  FontColor->RGBColor[0, 0, 1]],
 " defined by the property that both principal curvatures at this point are \
zero, or that the second fundamental form is zero at this point. On a sphere \
all points are umbilic, and all points of a plane are flat points.as shown at \
the end of Section 3.2. "
}], "Text",
 CellChangeTimes->{{3.5914206128734355`*^9, 3.591420717416415*^9}, 
   3.5914207584237604`*^9, {3.591421160424754*^9, 3.591421316560684*^9}, {
   3.591421360529199*^9, 3.5914214038246756`*^9}, {3.591421572585328*^9, 
   3.5914215837359657`*^9}, {3.5914216161288185`*^9, 3.5914216186169605`*^9}, 
   3.5914216810255303`*^9, {3.5914417174874754`*^9, 3.5914417322295017`*^9}, {
   3.6153612636999826`*^9, 3.6153612654830847`*^9}, {3.618398779056985*^9, 
   3.6183988121208763`*^9}, 3.6183988661469665`*^9, 3.6257294760404215`*^9}],

Cell[TextData[{
 "The directions in the tangent plane defined by the eigenvectors of the \
second fundamental form are called ",
 StyleBox["principal directions",
  FontColor->RGBColor[0, 0, 1]],
 ". At an umbilic the principal directions are not defined, or any direction \
can be considered as a principal direction. For a non umbilic point there \
exist exactly two principal directions orthogonal to each other. Curves on \
the surface  whose tangents at each point have principal directions are \
called ",
 StyleBox["principal curves",
  FontColor->RGBColor[0, 0, 1]],
 ". Outside the set of the umbilic points there exist two families of \
principal curves orthogonal to each other, namely the integral curves of the \
principal  directions. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has two functions admitting the calculation of the principal directions:"
}], "Text",
 CellChangeTimes->{{3.591421836304412*^9, 3.5914220785842695`*^9}, {
   3.5914222012562857`*^9, 3.591422254088308*^9}, {3.591422290704402*^9, 
   3.5914223437044334`*^9}, {3.5914223746402025`*^9, 
   3.5914223980085397`*^9}, {3.5914224586890106`*^9, 3.5914225164723153`*^9}, 
   3.5914418208220572`*^9, {3.5984546613739953`*^9, 3.5984546967180166`*^9}, {
   3.6116402144728317`*^9, 3.611640215564834*^9}, 3.618406981744152*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Eigenvectors", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"cf", "[", 
         RowBox[{"i", ",", "j"}], "]"}], "[", "hypsf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}], ",", " ", 
   RowBox[{"v", "\[Element]", " ", "Reals"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.591186146479534*^9, 3.591186166391673*^9}, {
   3.5914225738175955`*^9, 3.591422608636587*^9}, 3.614861859051139*^9, 
   3.6148635017496243`*^9, 3.617972096893916*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.628060127868886*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Eigensystem", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"cf", "[", 
         RowBox[{"i", ",", "j"}], "]"}], "[", "hypsf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}], ",", " ", 
   RowBox[{"v", "\[Element]", " ", "Reals"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5913416993869333`*^9, 3.5913417963723035`*^9}, 
   3.614861859066739*^9, 3.614863501765224*^9, 3.6179720982649946`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"Cosh", "[", 
         RowBox[{"2", " ", "v"}], "]"}], 
        RowBox[{"3", "/", "2"}]]]}], ",", 
     FractionBox["1", 
      SqrtBox[
       RowBox[{"Cosh", "[", 
        RowBox[{"2", " ", "v"}], "]"}]]]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0"}], "}"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6280601401305075`*^9}]
}, Closed]],

Cell["\<\
From this result it follows that on the parabolic hyperboloid hypsf[u,v]  \
there don\[CloseCurlyQuote]t exist umbilic points, and that the parameter \
lines {u ,t} and {t,v} are the principal curves.\
\>", "Text",
 CellChangeTimes->{{3.5914229744565105`*^9, 3.591423241632792*^9}, {
  3.6183990352646394`*^9, 3.6183990356326604`*^9}, {3.6258043878023934`*^9, 
  3.6258043903295975`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fund1", "[", "hypsf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"s1", ",", "s2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"s1", ",", "s2"}], "}"}]}], "]"}], ",", " ", 
   RowBox[{"v", "\[Element]", " ", "Reals"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5913416993869333`*^9, 3.5913417963723035`*^9}, {
  3.5914420245290146`*^9, 3.5914421216703854`*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    SuperscriptBox["s1", "2"], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["s1", "2"], "+", 
       RowBox[{"2", " ", 
        SuperscriptBox["s2", "2"]}]}], ")"}], " ", 
     RowBox[{"Cosh", "[", 
      RowBox[{"2", " ", "v"}], "]"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.628060168163757*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fund2", "[", "hypsf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"s1", ",", "s2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"s1", ",", "s2"}], "}"}]}], "]"}], ",", " ", 
   RowBox[{"v", "\[Element]", " ", "Reals"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5913416993869333`*^9, 3.5913417963723035`*^9}, {
  3.5914420245290146`*^9, 3.5914421216703854`*^9}, {3.591442168205267*^9, 
  3.5914421830252934`*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   SuperscriptBox["s1", "2"], "-", 
   RowBox[{"2", " ", 
    SuperscriptBox["s2", "2"]}], "+", 
   RowBox[{
    SuperscriptBox["s1", "2"], " ", 
    RowBox[{"Cosh", "[", 
     RowBox[{"2", " ", "v"}], "]"}]}]}], 
  RowBox[{"2", " ", 
   SqrtBox[
    RowBox[{"Cosh", "[", 
     RowBox[{"2", " ", "v"}], "]"}]]}]]], "Output",
 CellChangeTimes->{3.6280601748405685`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "hypsf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ".", 
    RowBox[{"{", 
     RowBox[{"s1", ",", "s2"}], "}"}]}], ",", " ", 
   RowBox[{"v", "\[Element]", " ", "Reals"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5913416993869333`*^9, 3.5913417963723035`*^9}, {
   3.5914420245290146`*^9, 3.5914421216703854`*^9}, {3.591442168205267*^9, 
   3.5914421830252934`*^9}, 3.5914423088239145`*^9, {3.5914427746251326`*^9, 
   3.591442808165191*^9}, {3.59144284725886*^9, 3.591442868974098*^9}}],

Cell[BoxData[
 RowBox[{"s1", " ", 
  RowBox[{"Cosh", "[", "v", "]"}]}]], "Output",
 CellChangeTimes->{3.628060188833793*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "hypsf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ".", 
    RowBox[{"{", 
     RowBox[{"s1", ",", "s2"}], "}"}]}], ",", " ", 
   RowBox[{"v", "\[Element]", " ", "Reals"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5913416993869333`*^9, 3.5913417963723035`*^9}, {
   3.5914420245290146`*^9, 3.5914421216703854`*^9}, {3.591442168205267*^9, 
   3.5914421830252934`*^9}, 3.5914423088239145`*^9, {3.5914427746251326`*^9, 
   3.591442808165191*^9}, {3.59144284725886*^9, 3.5914429074437656`*^9}}],

Cell[BoxData[
 FractionBox["s2", 
  SqrtBox[
   RowBox[{"Sech", "[", 
    RowBox[{"2", " ", "v"}], "]"}]]]], "Output",
 CellChangeTimes->{3.6280601923593993`*^9}]
}, Closed]],

Cell[TextData[{
 "The product of the eigenvalues is the determinant of the matrix Table[",
 "cf[",
 "i,j][sf][u,v] ], and this is the Gaussian curvature, see Section 3.2. Using \
a long working time ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " verified"
}], "Text",
 CellChangeTimes->{{3.591345466125239*^9, 3.591345517773193*^9}, {
   3.5914233072975483`*^9, 3.5914233377202883`*^9}, {3.5914233679840193`*^9, 
   3.5914234270003943`*^9}, {3.5914235564968014`*^9, 3.591423599200244*^9}, {
   3.5915079289499435`*^9, 3.5915079404446006`*^9}, 3.5984298919242697`*^9, 
   3.614861859082339*^9, 3.614863501812024*^9, 3.617972099835085*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"gauss", "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "-", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"prcurvatures", "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", 
      RowBox[{"[", "1", "]"}], "]"}], "*", 
     RowBox[{
      RowBox[{
       RowBox[{"prcurvatures", "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", 
      RowBox[{"[", "2", "]"}], "]"}]}]}], ",", 
   RowBox[{"{", " ", 
    RowBox[{
     RowBox[{"v", "\[Element]", " ", "Reals"}], ",", " ", 
     RowBox[{"v", "\[Element]", " ", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5913423152136145`*^9, 3.5913424135873876`*^9}, {
   3.59134244879665*^9, 3.591342507515153*^9}, 3.5913425460004206`*^9}],

Cell[BoxData["$Aborted"], "Output",
 CellChangeTimes->{3.6280622010033274`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"gauss", "[", "hypsf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "-", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"prcurvatures", "[", "hypsf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", 
      RowBox[{"[", "1", "]"}], "]"}], "*", 
     RowBox[{
      RowBox[{
       RowBox[{"prcurvatures", "[", "hypsf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", 
      RowBox[{"[", "2", "]"}], "]"}]}]}], ",", 
   RowBox[{"{", " ", 
    RowBox[{
     RowBox[{"v", "\[Element]", " ", "Reals"}], ",", " ", 
     RowBox[{"v", "\[Element]", " ", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5913423152136145`*^9, 3.5913424135873876`*^9}, {
   3.59134244879665*^9, 3.591342507515153*^9}, 3.5913425460004206`*^9, {
   3.6257375890655355`*^9, 3.625737630177608*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.62806026117112*^9}]
}, Closed]],

Cell[TextData[{
 "If amf[1][sf][u,v], ",
 "amf[",
 "2][sf][u,v] have principal directions and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "1"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "2"], TraditionalForm]]],
 " denote the principal curvature radii,  the normal form of the second \
fundamental form gives ",
 StyleBox["Euler\[CloseCurlyQuote]s formula",
  FontColor->RGBColor[0, 0, 1]],
 " for the normal curvature"
}], "Text",
 CellChangeTimes->{{3.591508114652565*^9, 3.5915083854050508`*^9}, 
   3.5984299357759466`*^9, {3.6148618591135387`*^9, 3.614861859144739*^9}, {
   3.6149177909221597`*^9, 3.6149177909533596`*^9}, {3.615262300306364*^9, 
   3.6152623012024155`*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", 
    SubscriptBox["r", "n"]], "\[LongEqual]", 
   RowBox[{
    FractionBox[
     RowBox[{
      SuperscriptBox["cos", "2"], "(", "\[Tau]", ")"}], 
     SubscriptBox["r", "1"]], " ", "+", " ", 
    RowBox[{
     FractionBox[
      RowBox[{
       SuperscriptBox["sin", "2"], "(", "\[Tau]", ")"}], 
      SubscriptBox["r", "2"]], "."}]}]}], TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.591508404647152*^9, 3.5915084060942345`*^9}, {
   3.5915084395291467`*^9, 3.5915084509828014`*^9}, {3.5915084879509163`*^9, 
   3.5915086366044188`*^9}, 3.591508698468957*^9, {3.5915087375851946`*^9, 
   3.5915087569603024`*^9}},
 TextAlignment->Center],

Cell["\<\
where \[Tau] denotes the angle of the tangent vector of the curve with the \
first principal direction.\
\>", "Text",
 CellChangeTimes->{{3.5914236744605484`*^9, 3.591423751543957*^9}, {
   3.5914243571205945`*^9, 3.591424357792633*^9}, {3.598451012192274*^9, 
   3.5984510125012918`*^9}, {3.615362094107479*^9, 3.615362161155314*^9}, 
   3.628060562329649*^9}],

Cell[TextData[{
 " A curve on the surface is said to be an ",
 StyleBox["asymptotic curve",
  FontColor->RGBColor[0, 0, 1]],
 " if its normal curvature at all points is identically zero. An implicit \
differential equation for the asymptotic curves is"
}], "Text",
 CellChangeTimes->{{3.5912513948692684`*^9, 3.5912515055345984`*^9}, {
   3.5912515441348057`*^9, 3.5912516055903206`*^9}, 3.591341039365374*^9, {
   3.6183978275525627`*^9, 3.61839783503199*^9}, 3.6184068933910985`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fund2", "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}], ",", 
     RowBox[{
      RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]}], "]"}], "\[Equal]", " ", 
   "0"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}], " ", "\[NotEqual]", " ", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}]}], ";"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5912516204781723`*^9, 3.591251639128239*^9}, {
   3.591251688890085*^9, 3.5912517049450035`*^9}, 3.591251797316287*^9, {
   3.5912544370822725`*^9, 3.5912544610826454`*^9}}],

Cell["\<\
Any line contained in a surface is an asymptotic curve. Asymptotic curves \
exist only at points with an indefinite second fundamental form, that means \
with non positive Gaussian curvature; therefore a ruled surface has \
non-negative curvature at all its points. On  spheres there don\
\[CloseCurlyQuote]t exist  asymptotic curves since the second fundamental \
form is definite:\
\>", "Text",
 CellChangeTimes->{{3.591254508871379*^9, 3.5912545339588137`*^9}, {
   3.5912545724790173`*^9, 3.5912546562148066`*^9}, {3.5915138824234624`*^9, 
   3.5915139679813557`*^9}, 3.598429175227811*^9, {3.615359907563843*^9, 
   3.6153599694273815`*^9}, {3.6153600463797827`*^9, 3.615360088875213*^9}, {
   3.615360418947092*^9, 3.615360511867407*^9}, {3.615360549322549*^9, 
   3.615360598043336*^9}, 3.618406903278664*^9, {3.628060652201407*^9, 
   3.6280607016066937`*^9}, {3.628062123564791*^9, 3.628062170895274*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reduce", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Simplify", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"fund2", "[", 
          RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"s1", ",", "s2"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"s1", ",", "s2"}], "}"}]}], "]"}], "]"}], "\[Equal]", " ", 
      "0"}], ")"}], "&&", "  ", 
    RowBox[{"(", 
     RowBox[{"v", "<", 
      RowBox[{"Pi", "/", "2"}]}], ")"}], "&&", "  ", 
    RowBox[{"(", 
     RowBox[{"v", ">", 
      RowBox[{
       RowBox[{"-", "Pi"}], "/", "2"}]}], ")"}], "&&", 
    RowBox[{"(", 
     RowBox[{"r", ">", "0"}], ")"}]}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"s1", ",", "s2", ",", "v"}], "}"}], ",", "Reals"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.591251918655227*^9, 3.591251930794922*^9}, {
   3.5912519649228735`*^9, 3.591252048466652*^9}, {3.591252241953719*^9, 
   3.5912522623128834`*^9}, {3.5912522937856836`*^9, 3.591252347257742*^9}, 
   3.591252871539729*^9, {3.5912537907793064`*^9, 3.591253941118905*^9}, 
   3.591253983300318*^9, {3.591254121708235*^9, 3.591254134867987*^9}, {
   3.611639104484082*^9, 3.6116391257157197`*^9}, 3.6148618590355387`*^9, 
   3.6149177908909597`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"r", ">", "0"}], "&&", 
  RowBox[{"s1", "\[Equal]", "0"}], "&&", 
  RowBox[{"s2", "\[Equal]", "0"}], "&&", 
  RowBox[{
   RowBox[{"-", 
    FractionBox["\[Pi]", "2"]}], "<", "v", "<", 
   FractionBox["\[Pi]", "2"]}]}]], "Output",
 CellChangeTimes->{3.6280622101293435`*^9}]
}, Closed]],

Cell[TextData[{
 "Another important invariant of the surface is the ",
 StyleBox["mean curvature",
  FontColor->RGBColor[0, 0, 1]],
 " defined as the trace of the second fundamental form"
}], "Text",
 CellChangeTimes->{{3.5914236744605484`*^9, 3.591423751543957*^9}, {
   3.5914243571205945`*^9, 3.591424357792633*^9}, {3.598451012192274*^9, 
   3.5984510125012918`*^9}, {3.615362094107479*^9, 3.615362161155314*^9}, 
   3.628060562329649*^9, 3.6280622939638906`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"meancurvature", "[", "ff_", "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"cf", "[", 
        RowBox[{"1", ",", "1"}], "]"}], "[", "ff", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", "+", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"cf", "[", 
        RowBox[{"2", ",", "2"}], "]"}], "[", "ff", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], ")"}], "/", "2"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.591453039732647*^9, 3.591453186263705*^9}, 
   3.591453315572332*^9, 3.614861859144739*^9, 3.614863136537382*^9, {
   3.617972101448177*^9, 3.6179721027672524`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "meancurvature"}]], "Input",
 CellChangeTimes->{{3.6153650924458*^9, 3.6153652075853853`*^9}, 
   3.615365241170306*^9, {3.6226994759115267`*^9, 3.622699487720747*^9}}],

Cell[BoxData[
 StyleBox["\<\"meancurvature[f][u,v] is the mean curvature of \\nthe surface \
f[u,v] in the Euclidean 3-space.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.628062399030075*^9},
 CellTags->"Info3628065998-8815218"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Context", "[", "meancurvature", "]"}]], "Input",
 CellChangeTimes->{{3.6153650924458*^9, 3.6153652075853853`*^9}, 
   3.615365241170306*^9, {3.6226994759115267`*^9, 3.622699487720747*^9}, {
   3.62269954043324*^9, 3.6226995583264713`*^9}}],

Cell[BoxData["\<\"eudiffgeov2`\"\>"], "Output",
 CellChangeTimes->{3.628062405145286*^9}]
}, Closed]],

Cell[TextData[{
 StyleBox["Remark,",
  FontColor->RGBColor[0, 0, 1]],
 " The signs of the mean curvature, of  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "1"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.591508404647152*^9, 3.5915084060942345`*^9}, {
    3.5915084395291467`*^9, 3.5915084509828014`*^9}, {3.5915084879509163`*^9, 
    3.5915086366044188`*^9}, 3.591508698468957*^9, {3.5915087375851946`*^9, 
    3.5915087569603024`*^9}},
  TextAlignment->Center],
 ", and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "2"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.591508404647152*^9, 3.5915084060942345`*^9}, {
    3.5915084395291467`*^9, 3.5915084509828014`*^9}, {3.5915084879509163`*^9, 
    3.5915086366044188`*^9}, 3.591508698468957*^9, {3.5915087375851946`*^9, 
    3.5915087569603024`*^9}},
  TextAlignment->Center],
 ", are opposite to the definitions in [B-R], see (61.26), p. 89."
}], "Text",
 CellChangeTimes->{{3.5915132001654396`*^9, 3.5915133912293673`*^9}, {
   3.615365277023357*^9, 3.6153652880139856`*^9}, {3.6154345328328114`*^9, 
   3.6154345703196774`*^9}, 3.6184070924544845`*^9, 3.6257307007247796`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"meancurvature", "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "-", 
  RowBox[{
   RowBox[{"Tr", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"cf", "[", 
         RowBox[{"i", ",", "j"}], "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}], "/", "2"}]}]], "Input",
 CellChangeTimes->{{3.591424392201601*^9, 3.591424478700548*^9}, {
   3.5914245920050287`*^9, 3.5914246060768337`*^9}, 3.5914432841844273`*^9, {
   3.591443426487877*^9, 3.5914434841143785`*^9}, {3.591443521382844*^9, 
   3.5914435245496497`*^9}, 3.591444524511406*^9, 3.591452112560619*^9, 
   3.591453529043107*^9, {3.5914535906632147`*^9, 3.5914535921140175`*^9}, {
   3.591509215258516*^9, 3.591509217562648*^9}, {3.614861859160339*^9, 
   3.614861859160339*^9}, 3.6148631365529823`*^9, 3.614863501843224*^9, 
   3.617972104512352*^9, {3.6280625047202606`*^9, 3.6280625121926737`*^9}}],

Cell[BoxData[
 InterpretationBox[
  TagBox[
   FrameBox[GridBox[{
      {
       ItemBox[
        TagBox[
         RowBox[{
          RowBox[{
           FractionBox["1", "2"], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"-", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                   " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                    ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "-", 
                  TemplateBox[{"1"},
                   "OutputSizeLimit`Skeleton",
                   DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                  "-", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "/", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"(", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
                    SuperscriptBox[
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
                    SuperscriptBox[
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
                  RowBox[{"\[Sqrt]", 
                   RowBox[{"(", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "-", 
                    RowBox[{
                    RowBox[{
                    SuperscriptBox[
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{"\" \[CenterEllipsis]\"", #, 
                    "\"\[CenterEllipsis] \""}], Background -> GrayLevel[0.75],
                     BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                    TagBox[
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{"\" \[CenterEllipsis]\"", #, 
                    "\"\[CenterEllipsis] \""}], Background -> GrayLevel[0.75],
                     BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{"\" \[CenterEllipsis]\"", #, 
                    "\"\[CenterEllipsis] \""}], Background -> GrayLevel[0.75],
                     BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 
                    7]& )]}]}], ")"}], "2"], "+", 
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"-", 
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{"\" \[CenterEllipsis]\"", #, 
                    "\"\[CenterEllipsis] \""}], Background -> GrayLevel[0.75],
                     BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )]}],
                     " ", 
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{"\" \[CenterEllipsis]\"", #, 
                    "\"\[CenterEllipsis] \""}], Background -> GrayLevel[0.75],
                     BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )]}],
                     "+", 
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )]}],
                     ")"}], "2"], "+", 
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{"\" \[CenterEllipsis]\"", #, 
                    "\"\[CenterEllipsis] \""}], Background -> GrayLevel[0.75],
                     BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                    " ", 
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{"\" \[CenterEllipsis]\"", #, 
                    "\"\[CenterEllipsis] \""}], Background -> GrayLevel[0.75],
                     BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )]}],
                     "-", 
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )]}],
                     ")"}], "2"]}], ")"}]}]}], ")"}]}], ")"}]}], "-", 
             FractionBox[
              RowBox[{
               FractionBox[
                TemplateBox[{"1"},
                 "OutputSizeLimit`Skeleton",
                 DisplayFunction->(FrameBox[
                   
                   RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                   Background -> GrayLevel[0.75], 
                   BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                   Baseline, ContentPadding -> False, FrameMargins -> 1, 
                   FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                TemplateBox[{"1"},
                 "OutputSizeLimit`Skeleton",
                 DisplayFunction->(FrameBox[
                   
                   RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                   Background -> GrayLevel[0.75], 
                   BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                   Baseline, ContentPadding -> False, FrameMargins -> 1, 
                   FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )]], 
               "-", 
               TemplateBox[{"1"},
                "OutputSizeLimit`Skeleton",
                DisplayFunction->(FrameBox[
                  
                  RowBox[{
                   "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                  Background -> GrayLevel[0.75], 
                  BaseStyle -> {
                   "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                  Baseline, ContentPadding -> False, FrameMargins -> 1, 
                  FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )]}], 
              SqrtBox[
               TemplateBox[{"1"},
                "OutputSizeLimit`Skeleton",
                DisplayFunction->(FrameBox[
                  
                  RowBox[{
                   "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                  Background -> GrayLevel[0.75], 
                  BaseStyle -> {
                   "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                  Baseline, ContentPadding -> False, FrameMargins -> 1, 
                  FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )]]]}],
             ")"}]}], "+", 
          RowBox[{
           FractionBox["1", "2"], " ", 
           RowBox[{"(", 
            RowBox[{
             FractionBox[
              TemplateBox[{"1"},
               "OutputSizeLimit`Skeleton",
               DisplayFunction->(FrameBox[
                 
                 RowBox[{
                  "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                 Background -> GrayLevel[0.75], 
                 BaseStyle -> {
                  "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                   ShowStringCharacters -> False}, BaselinePosition -> 
                 Baseline, ContentPadding -> False, FrameMargins -> 1, 
                 FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
              TemplateBox[{"1"},
               "OutputSizeLimit`Skeleton",
               DisplayFunction->(FrameBox[
                 
                 RowBox[{
                  "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                 Background -> GrayLevel[0.75], 
                 BaseStyle -> {
                  "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                   ShowStringCharacters -> False}, BaselinePosition -> 
                 Baseline, ContentPadding -> False, FrameMargins -> 1, 
                 FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )]], 
             "+", 
             TemplateBox[{"1"},
              "OutputSizeLimit`Skeleton",
              DisplayFunction->(FrameBox[
                
                RowBox[{
                 "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                Background -> GrayLevel[0.75], 
                BaseStyle -> {
                 "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                  ShowStringCharacters -> False}, BaselinePosition -> 
                Baseline, ContentPadding -> False, FrameMargins -> 1, 
                FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )]}], 
            ")"}]}]}],
         Short[#, 20]& ],
        BaseStyle->{Deployed -> False},
        StripOnInput->False]},
      {GridBox[{
         {
          TagBox[
           TooltipBox[
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource[
                "FEStrings", "sizeBriefExplanation"], StandardForm],
               ImageSizeCache->{183., {7., 26.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarCategory",
             StripOnInput->False],
            StyleBox[
             DynamicBox[
              ToBoxes[
               FEPrivate`FrontEndResource["FEStrings", "sizeExplanation"], 
               StandardForm]], DynamicUpdating -> True, StripOnInput -> 
             False]],
           Annotation[#, 
            Style[
             Dynamic[
              FEPrivate`FrontEndResource["FEStrings", "sizeExplanation"]], 
             DynamicUpdating -> True], "Tooltip"]& ], 
          ButtonBox[
           PaneSelectorBox[{False->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeShowLess"], 
                StandardForm],
               ImageSizeCache->{167., {1., 26.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControl",
             StripOnInput->False], True->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeShowLess"], 
                StandardForm],
               ImageSizeCache->{180., {1., 26.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControlActive",
             StripOnInput->False]}, Dynamic[
             CurrentValue["MouseOver"]],
            Alignment->Center,
            FrameMargins->0,
            ImageSize->{Automatic, 25}],
           Appearance->None,
           
           ButtonFunction:>OutputSizeLimit`ButtonFunction[
            Identity, 90, 26395590910928283107, 20/2],
           Enabled->True,
           Evaluator->Automatic,
           Method->"Queued"], 
          ButtonBox[
           PaneSelectorBox[{False->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeShowMore"], 
                StandardForm],
               ImageSizeCache->{189., {1., 26.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControl",
             StripOnInput->False], True->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeShowMore"], 
                StandardForm],
               ImageSizeCache->{191., {1., 26.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControlActive",
             StripOnInput->False]}, Dynamic[
             CurrentValue["MouseOver"]],
            Alignment->Center,
            FrameMargins->0,
            ImageSize->{Automatic, 25}],
           Appearance->None,
           
           ButtonFunction:>OutputSizeLimit`ButtonFunction[
            Identity, 90, 26395590910928283107, 20 2],
           Enabled->True,
           Evaluator->Automatic,
           Method->"Queued"], 
          ButtonBox[
           PaneSelectorBox[{False->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeShowAll"], 
                StandardForm],
               ImageSizeCache->{141., {1., 26.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControl",
             StripOnInput->False], True->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeShowAll"], 
                StandardForm],
               ImageSizeCache->{184., {59., 34.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControlActive",
             StripOnInput->False]}, Dynamic[
             CurrentValue["MouseOver"]],
            Alignment->Center,
            FrameMargins->0,
            ImageSize->{Automatic, 25}],
           Appearance->None,
           
           ButtonFunction:>OutputSizeLimit`ButtonFunction[
            Identity, 90, 26395590910928283107, Infinity],
           Enabled->True,
           Evaluator->Automatic,
           Method->"Queued"], 
          ButtonBox[
           PaneSelectorBox[{False->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeChangeLimit"], 
                StandardForm],
               ImageSizeCache->{247., {1., 27.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControl",
             StripOnInput->False], True->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeChangeLimit"], 
                StandardForm],
               ImageSizeCache->{247., {1., 27.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControlActive",
             StripOnInput->False]}, Dynamic[
             CurrentValue["MouseOver"]],
            Alignment->Center,
            FrameMargins->0,
            ImageSize->{Automatic, 25}],
           Appearance->None,
           ButtonFunction:>FrontEndExecute[{
              FrontEnd`SetOptions[
              FrontEnd`$FrontEnd, 
               FrontEnd`PreferencesSettings -> {"Page" -> "Evaluation"}], 
              FrontEnd`FrontEndToken["PreferencesDialog"]}],
           Evaluator->None,
           Method->"Preemptive"]}
        },
        AutoDelete->False,
        FrameStyle->GrayLevel[0.85],
        GridBoxDividers->{"Columns" -> {False, {True}}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{"Columns" -> {{2}}}]}
     },
     DefaultBaseStyle->"Column",
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxDividers->{
      "Columns" -> {{False}}, "ColumnsIndexed" -> {}, "Rows" -> {{False}}, 
       "RowsIndexed" -> {}},
     GridBoxItemSize->{
      "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], 
         Offset[1.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Background->RGBColor[0.9657, 0.9753, 0.9802],
    FrameMargins->{{12, 12}, {0, 15}},
    FrameStyle->GrayLevel[0.85],
    RoundingRadius->5,
    StripOnInput->False],
   Deploy,
   DefaultBaseStyle->"Deploy"],
  Out[90]]], "Output",
 CellChangeTimes->{3.628062495906245*^9, 3.628062645463708*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Together", "[", "%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.628062670064951*^9, 3.6280626859769793`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.628062689876986*^9}]
}, Closed]],

Cell[TextData[{
 "Also the following general equation has been confirmed by ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", using about 20 min of evaluating time:"
}], "Text",
 CellChangeTimes->{{3.5914271254729347`*^9, 3.5914271808651032`*^9}, {
   3.591453651908922*^9, 3.5914536546389275`*^9}, 3.5914579284684405`*^9, {
   3.615365395470132*^9, 3.615365402398528*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"meancurvature", "[", "hypsf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"prcurvatures", "[", "hypsf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", 
      RowBox[{"[", "1", "]"}], "]"}], " ", "+", 
     RowBox[{
      RowBox[{
       RowBox[{"prcurvatures", "[", "hypsf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", 
      RowBox[{"[", "2", "]"}], "]"}]}], ")"}], "/", "2"}]}]], "Input",
 CellChangeTimes->{
  3.591425022610658*^9, {3.5914250548025*^9, 3.5914251408384204`*^9}, {
   3.591425286552755*^9, 3.5914253533975782`*^9}, 3.591453535875919*^9, 
   3.614861859160339*^9, 3.6148631365685825`*^9, 3.615365573002286*^9, {
   3.6153732193236303`*^9, 3.6153732221637926`*^9}, {3.628062711670224*^9, 
   3.6280627217790422`*^9}, {3.6280728281519146`*^9, 3.628072872974794*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{
  3.628065254152563*^9, {3.6280728781696033`*^9, 3.628072885909217*^9}}]
}, Closed]],

Cell["\<\
The traditional notations of the Gaussian and the mean curvature are\
\>", "Text",
 CellChangeTimes->{{3.591510504133235*^9, 3.5915105337009263`*^9}, 
   3.6280631605828657`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"K", "=", 
   RowBox[{
    RowBox[{"gauss", "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], ";", 
  RowBox[{"H", "=", " ", 
   RowBox[{
    RowBox[{"meancurvature", "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.591509404812358*^9, 3.5915094861910124`*^9}, {
   3.5915098286475997`*^9, 3.591509840613284*^9}, 3.591510578280476*^9, 
   3.6148618591759386`*^9, 3.6148631365841827`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"tt", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"s1", ",", "s2"}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.591509404812358*^9, 3.5915094861910124`*^9}, {
  3.5915098286475997`*^9, 3.591509840613284*^9}, {3.591510578280476*^9, 
  3.5915105854378853`*^9}, {3.615365503896333*^9, 3.6153655065834866`*^9}}],

Cell["\<\
We verify the well known relation, proved e.g. in [B-R], (62.19), p.93:\
\>", "Text",
 CellChangeTimes->{{3.591512624125492*^9, 3.5915126308448763`*^9}, {
  3.591512932918154*^9, 3.5915129927415752`*^9}, {3.591514476205425*^9, 
  3.591514484732912*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"K", "*", 
     RowBox[{
      RowBox[{
       RowBox[{"fund1", "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", 
      RowBox[{"tt", ",", "tt"}], "]"}]}], " ", "+", " ", 
    RowBox[{"2", "*", "H", "*", 
     RowBox[{
      RowBox[{
       RowBox[{"fund2a", "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", 
      RowBox[{"tt", ",", "tt"}], "]"}]}], " ", "+", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"d", "[", 
         RowBox[{
          RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}], "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}], ".", "tt"}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"d", "[", 
         RowBox[{
          RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}], "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}], ".", "tt"}], ")"}]}]}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5915095082782755`*^9, 3.591509568263706*^9}, {
  3.591509610320112*^9, 3.591509645545127*^9}, {3.591509733351149*^9, 
  3.591509753367294*^9}, {3.5915098645436525`*^9, 3.591509870040967*^9}, {
  3.591510188080158*^9, 3.591510252920867*^9}, {3.591510335568594*^9, 
  3.5915103952670083`*^9}, {3.5915125071888037`*^9, 3.59151253055614*^9}, {
  3.6148618591759386`*^9, 3.6148618591759386`*^9}, {3.6149177909533596`*^9, 
  3.61491779096896*^9}, {3.61526232208561*^9, 3.6152623239587173`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "2133.96887920000017402344383299350738525391`9.349787994867484", ",", "0"}],
   "}"}]], "Output",
 CellChangeTimes->{3.6280844755929146`*^9}]
}, Closed]],

Cell["\<\
The result is zero. The time needed to verify this identity was 2134 s. \
\>", "Text",
 CellChangeTimes->{{3.6280847273149567`*^9, 3.6280848058454947`*^9}, {
  3.628084868074004*^9, 3.628084885717635*^9}}],

Cell[TextData[{
 "The last Dot product is named the",
 StyleBox[" third fundamental form",
  FontColor->RGBColor[0, 0, 1]],
 " of the surface:"
}], "Text",
 CellChangeTimes->{{3.591510744132962*^9, 3.591510787156423*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"fund3", "[", "ff_", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], "[", 
   RowBox[{"x_List", ",", "y_List"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"d", "[", 
       RowBox[{
        RowBox[{"amf", "[", "3", "]"}], "[", "ff", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ".", "x"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"d", "[", 
       RowBox[{
        RowBox[{"amf", "[", "3", "]"}], "[", "ff", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ".", "y"}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.5915107902566004`*^9, 3.591510920463048*^9}, {
  3.5915111307120733`*^9, 3.591511134304279*^9}, {3.614861859191539*^9, 
  3.614861859191539*^9}, {3.61491779096896*^9, 3.6149177909845595`*^9}, {
  3.6152623260058346`*^9, 3.615262327597925*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"fund3", "[", 
     RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "[", 
   RowBox[{"tt", ",", "tt"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5915135006626267`*^9, 3.591513610408904*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    SuperscriptBox["s1", "2"], "+", 
    RowBox[{"2", " ", 
     SuperscriptBox["s2", "2"]}], "+", 
    RowBox[{
     SuperscriptBox["s1", "2"], " ", 
     RowBox[{"Cos", "[", 
      RowBox[{"2", " ", "v"}], "]"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.628084963826972*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", "\[Equal]", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"fund1", "[", 
      RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{"tt", ",", "tt"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5915136280619135`*^9, 3.591513662912907*^9}, 
   3.5915136981689234`*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.628084972172987*^9}]
}, Closed]],

Cell["\<\
It degenerates if K = 0, and it is the first fundamental form of the unit \
sphere under the parameter representation of the Gauss map otherwise.\
\>", "Text",
 CellChangeTimes->{{3.5915109744131336`*^9, 3.5915110763489637`*^9}, {
  3.6154347044799128`*^9, 3.6154347134031286`*^9}}],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"K", ",", "H", ",", "cx", ",", "tt", ",", " ", "\[Gamma]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.591511821736598*^9, 3.5915118333552628`*^9}, 
   3.5915119055813932`*^9, {3.5915119646777735`*^9, 3.591511972552224*^9}, {
   3.591513428718512*^9, 3.591513460286318*^9}, {3.591513764215701*^9, 
   3.5915137673668814`*^9}, 3.6114713927527266`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["3.6. Graphs of Functions", "Subsection",
 CellChangeTimes->{{3.591592516559106*^9, 3.591592541566536*^9}, 
   3.611640876709595*^9, 3.6116416079920797`*^9, 3.615434814350906*^9, 
   3.6154364406381626`*^9, {3.6184074755410967`*^9, 3.618407498629137*^9}, {
   3.628085165379326*^9, 3.6280851714945374`*^9}, 3.6280860846577406`*^9}],

Cell["\<\
In this section we consider surfaces as graphs of functions of two variables: \
We shall calculate the invariants of the graphs and apply the results to \
visualize the relation of the curvature with the local shape of a surface.\
\>", "Text",
 CellChangeTimes->{{3.5915973790152225`*^9, 3.591597457390705*^9}, {
  3.5916065041192923`*^9, 3.5916066114464307`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "graph"}]], "Input",
 CellChangeTimes->{{3.5915926925891743`*^9, 3.5915926960793734`*^9}}],

Cell[BoxData[
 StyleBox["\<\"graph[f][x,y,...] is the canonical parameter representation of \
the \\ngraph {x,y,...f[x,y,...]} of the function f, in arbitrary dimension.\"\
\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.628085387851317*^9},
 CellTags->"Info3628088987-5827330"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"graph", "[", "f", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5915927300373163`*^9, 3.591592744128122*^9}, 
   3.5984303193338203`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"u", ",", "v", ",", 
   RowBox[{"f", "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6280854011425405`*^9}]
}, Closed]],

Cell["The moving frame of the surface graph[f] is", "Text",
 CellChangeTimes->{{3.5915977489033785`*^9, 3.5915978063746657`*^9}, {
  3.5915982482159376`*^9, 3.591598249815029*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"amf", "[", "1", "]"}], "[", 
   RowBox[{"graph", "[", "f", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.591592810221902*^9, 3.5915928362723923`*^9}, {
   3.5915975202423*^9, 3.5915975270816913`*^9}, {3.591597567498003*^9, 
   3.5915975708171926`*^9}, 3.614861859207139*^9, 3.6149177909845595`*^9, 
   3.615275815323556*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox["1", 
    SqrtBox[
     RowBox[{"1", "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"]}]]], ",", "0", ",", 
   FractionBox[
    RowBox[{
     SuperscriptBox["f", 
      TagBox[
       RowBox[{"(", 
        RowBox[{"1", ",", "0"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}], 
    SqrtBox[
     RowBox[{"1", "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"]}]]]}], "}"}]], "Output",
 CellChangeTimes->{3.628085410518157*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"amf", "[", "2", "]"}], "[", 
   RowBox[{"graph", "[", "f", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.591592810221902*^9, 3.5915928362723923`*^9}, {
   3.5915975202423*^9, 3.5915975270816913`*^9}, {3.591597567498003*^9, 
   3.591597605425172*^9}, 3.614861859207139*^9, 3.6149177910001593`*^9, 
   3.615275816315613*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{
       SuperscriptBox["f", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox["f", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], 
     RowBox[{
      SqrtBox[
       RowBox[{"1", "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["f", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"]}]], " ", 
      SqrtBox[
       RowBox[{"1", "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["f", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["f", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"]}]]}]]}], ",", 
   RowBox[{
    FractionBox["1", 
     RowBox[{
      SqrtBox[
       RowBox[{"1", "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["f", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"]}]], " ", 
      SqrtBox[
       RowBox[{"1", "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["f", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["f", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"]}]]}]], "+", 
    FractionBox[
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["f", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"], 
     RowBox[{
      SqrtBox[
       RowBox[{"1", "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["f", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"]}]], " ", 
      SqrtBox[
       RowBox[{"1", "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["f", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["f", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "2"]}]]}]]}], ",", 
   FractionBox[
    RowBox[{
     SuperscriptBox["f", 
      TagBox[
       RowBox[{"(", 
        RowBox[{"0", ",", "1"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"u", ",", "v"}], "]"}], 
    RowBox[{
     SqrtBox[
      RowBox[{"1", "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["f", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"]}]], " ", 
     SqrtBox[
      RowBox[{"1", "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["f", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["f", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"]}]]}]]}], "}"}]], "Output",
 CellChangeTimes->{3.6280854145117636`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"amf", "[", "3", "]"}], "[", 
   RowBox[{"graph", "[", "f", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.591592810221902*^9, 3.5915928362723923`*^9}, {
   3.5915975202423*^9, 3.5915975270816913`*^9}, {3.591597567498003*^9, 
   3.591597605425172*^9}, {3.5915976544389753`*^9, 3.5915976736180725`*^9}, 
   3.614861859207139*^9, 3.6149177910157595`*^9, 3.6152758172926683`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["f", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], 
     SqrtBox[
      RowBox[{"1", "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["f", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["f", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"]}]]]}], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["f", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], 
     SqrtBox[
      RowBox[{"1", "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["f", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["f", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"]}]]]}], ",", 
   FractionBox["1", 
    SqrtBox[
     RowBox[{"1", "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "2"]}]]]}], "}"}]], "Output",
 CellChangeTimes->{3.628085419066972*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"Simplify", "[", 
     RowBox[{"Together", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"amf", "[", "i", "]"}], "[", 
         RowBox[{"graph", "[", "f", "]"}], "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}], ".", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"amf", "[", "j", "]"}], "[", 
         RowBox[{"graph", "[", "f", "]"}], "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "3"}], "}"}]}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.6280854466478205`*^9, 3.6280855191255474`*^9}, {
  3.628085564958428*^9, 3.628085661070197*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0"},
     {"0", "1", "0"},
     {"0", "0", "1"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.6280855216371517`*^9, {3.6280855981396866`*^9, 3.628085616766119*^9}, 
   3.6280856652666044`*^9}]
}, Closed]],

Cell["\<\
We consider the special position that the tangent plane of the surface at the \
point {u, v}={0, 0}  is the u, v-plane z = 0 and the point graph[f][0,0] is \
the origin. These conditions are\
\>", "Text",
 CellChangeTimes->{{3.5915982913974075`*^9, 3.5915985371594644`*^9}, 
   3.5915987400710697`*^9, 3.59168583181394*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "tcond", "]"}], ";", 
  RowBox[{
   RowBox[{"tcond", "[", "ff_", "]"}], ":=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"ff", "[", 
         RowBox[{"u", ",", "v"}], "]"}], "/.", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"u", "\[Rule]", "0"}], ",", 
          RowBox[{"v", "\[Rule]", "0"}]}], "}"}]}], ")"}], "\[Equal]", "0"}], 
     ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["ff", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "/.", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"u", "\[Rule]", "0"}], ",", 
          RowBox[{"v", "\[Rule]", "0"}]}], "}"}]}], ")"}], "\[Equal]", "0"}], 
     ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["ff", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "/.", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"u", "\[Rule]", "0"}], ",", 
          RowBox[{"v", "\[Rule]", "0"}]}], "}"}]}], ")"}], "\[Equal]", 
      "0"}]}], "}"}]}]}]], "Input",
 CellChangeTimes->{{3.5915929268945756`*^9, 3.5915930649064693`*^9}, {
  3.591593102353611*^9, 3.5915931751407747`*^9}, {3.591593263948854*^9, 
  3.5915933381660986`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tcond", "[", "f", "]"}]], "Input",
 CellChangeTimes->{{3.5915931846653194`*^9, 3.591593224104575*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", 
     RowBox[{"0", ",", "0"}], "]"}], "\[Equal]", "0"}], ",", 
   RowBox[{
    RowBox[{
     SuperscriptBox["f", 
      TagBox[
       RowBox[{"(", 
        RowBox[{"1", ",", "0"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"0", ",", "0"}], "]"}], "\[Equal]", "0"}], ",", 
   RowBox[{
    RowBox[{
     SuperscriptBox["f", 
      TagBox[
       RowBox[{"(", 
        RowBox[{"0", ",", "1"}], ")"}],
       Derivative],
      MultilineFunction->None], "[", 
     RowBox[{"0", ",", "0"}], "]"}], "\[Equal]", "0"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6280857085410805`*^9}]
}, Closed]],

Cell["\<\
Each surface sf locally, in the neighborhood of an arbitrary regular point,  \
can be represented in this way. The function f is the distance of the point \
sf[u,v] from the tangent plane at the point sf[0,0]. The frame at the point \
{0,0} then is the standard basis:\
\>", "Text",
 CellChangeTimes->{{3.591598628758703*^9, 3.591598711359428*^9}, {
  3.591598745871402*^9, 3.59159889951919*^9}, {3.5915991083351336`*^9, 
  3.591599135631695*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"amf", "[", "1", "]"}], "[", 
        RowBox[{"graph", "[", "f", "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"amf", "[", "2", "]"}], "[", 
        RowBox[{"graph", "[", "f", "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"amf", "[", "3", "]"}], "[", 
        RowBox[{"graph", "[", "f", "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "}"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"u", "\[Rule]", "0"}], ",", 
      RowBox[{"v", "\[Rule]", "0"}]}], "}"}]}], ",", 
   RowBox[{"tcond", "[", "f", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5915989328190947`*^9, 3.5915990016440315`*^9}, {
  3.591599046445594*^9, 3.591599054701066*^9}, {3.614861859222739*^9, 
  3.614861859222739*^9}, {3.6149177910313597`*^9, 3.6149177910313597`*^9}, {
  3.6152758184227333`*^9, 3.6152758205978575`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "1"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.628085760208371*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"ss", "=", 
   RowBox[{"{", 
    RowBox[{"s1", ",", "s2"}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5915944041560698`*^9, 3.5915944155127196`*^9}, 
   3.591599225201818*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fund1", "[", 
    RowBox[{"graph", "[", "f", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "[", 
  RowBox[{"ss", ",", "ss"}], "]"}]], "Input",
 CellChangeTimes->{{3.5915942364314766`*^9, 3.591594247940135*^9}, {
   3.591594278312872*^9, 3.591594297073945*^9}, {3.5915944248652544`*^9, 
   3.5915944299525456`*^9}, 3.5915993268266306`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["s2", "2"], " ", 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["f", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
  RowBox[{"2", " ", "s1", " ", "s2", " ", 
   RowBox[{
    SuperscriptBox["f", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox["f", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
  RowBox[{
   SuperscriptBox["s1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["f", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.628085769771188*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"%", "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"u", "\[Rule]", "0"}], ",", 
      RowBox[{"v", "\[Rule]", "0"}]}], "}"}]}], ",", 
   RowBox[{"tcond", "[", "f", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5915944688857727`*^9, 3.591594505640875*^9}, {
  3.591599368362006*^9, 3.5915993755534177`*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["s1", "2"], "+", 
  SuperscriptBox["s2", "2"]}]], "Output",
 CellChangeTimes->{3.6280857797864056`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fund2a", "[", 
    RowBox[{"graph", "[", "f", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "[", 
  RowBox[{"ss", ",", "ss"}], "]"}]], "Input",
 CellChangeTimes->{{3.5915945394578085`*^9, 3.59159454052787*^9}, {
   3.591594591249771*^9, 3.5915946618878117`*^9}, {3.591594739650259*^9, 
   3.591594762532568*^9}, 3.5916053890825157`*^9, 3.5916054716252365`*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"s2", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"s2", " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "2"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "+", 
      RowBox[{"s1", " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
   RowBox[{"s1", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"s2", " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "+", 
      RowBox[{"s1", " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"2", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], 
  SqrtBox[
   RowBox[{"1", "+", 
    SuperscriptBox[
     RowBox[{
      SuperscriptBox["f", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
    SuperscriptBox[
     RowBox[{
      SuperscriptBox["f", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "2"]}]]]], "Output",
 CellChangeTimes->{3.628085786322817*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"%", "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"u", "\[Rule]", "0"}], ",", 
      RowBox[{"v", "\[Rule]", "0"}]}], "}"}]}], ",", 
   RowBox[{"tcond", "[", "f", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5915944688857727`*^9, 3.591594505640875*^9}, {
  3.591599368362006*^9, 3.5915993755534177`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["s2", "2"], " ", 
   RowBox[{
    SuperscriptBox["f", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "2"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"0", ",", "0"}], "]"}]}], "+", 
  RowBox[{"2", " ", "s1", " ", "s2", " ", 
   RowBox[{
    SuperscriptBox["f", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"0", ",", "0"}], "]"}]}], "+", 
  RowBox[{
   SuperscriptBox["s1", "2"], " ", 
   RowBox[{
    SuperscriptBox["f", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"2", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"0", ",", "0"}], "]"}]}]}]], "Output",
 CellChangeTimes->{3.628085790488024*^9}]
}, Closed]],

Cell["\<\
We will compare this expression with the first non vanishing term of the \
Taylor series of f at {0,0}:\
\>", "Text",
 CellChangeTimes->{{3.5916010136001086`*^9, 3.5916010984949646`*^9}, {
  3.591605432806016*^9, 3.5916054543502483`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "Series"}]], "Input",
 CellChangeTimes->{{3.5915936651197996`*^9, 3.5915936737732944`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox["\<\"\\!\\(\\*RowBox[{\\\"Series\\\", \\\"[\\\", RowBox[{StyleBox[\
\\\"f\\\", \\\"TI\\\"], \\\",\\\", RowBox[{\\\"{\\\", \
RowBox[{StyleBox[\\\"x\\\", \\\"TI\\\"], \\\",\\\", \
SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], StyleBox[\\\"0\\\", \
\\\"TR\\\"]], \\\",\\\", StyleBox[\\\"n\\\", \\\"TI\\\"]}], \\\"}\\\"}]}], \\\
\"]\\\"}]\\) generates a power series expansion for \\!\\(\\*StyleBox[\\\"f\\\
\", \\\"TI\\\"]\\) about the point \\!\\(\\*RowBox[{StyleBox[\\\"x\\\", \
\\\"TI\\\"], \\\"=\\\", SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], \
StyleBox[\\\"0\\\", \\\"TR\\\"]]}]\\) to order \
\\!\\(\\*SuperscriptBox[RowBox[{\\\"(\\\", RowBox[{StyleBox[\\\"x\\\", \\\"TI\
\\\"], \\\"-\\\", SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], \
StyleBox[\\\"0\\\", \\\"TR\\\"]]}], \\\")\\\"}], StyleBox[\\\"n\\\", \\\"TI\\\
\"]]\\). \\n\\!\\(\\*RowBox[{\\\"Series\\\", \\\"[\\\", \
RowBox[{StyleBox[\\\"f\\\", \\\"TI\\\"], \\\",\\\", RowBox[{\\\"{\\\", \
RowBox[{StyleBox[\\\"x\\\", \\\"TI\\\"], \\\",\\\", \
SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], StyleBox[\\\"0\\\", \
\\\"TR\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"n\\\", \\\"TI\\\"], \
StyleBox[\\\"x\\\", \\\"TI\\\"]]}], \\\"}\\\"}], \\\",\\\", \
RowBox[{\\\"{\\\", RowBox[{StyleBox[\\\"y\\\", \\\"TI\\\"], \\\",\\\", \
SubscriptBox[StyleBox[\\\"y\\\", \\\"TI\\\"], StyleBox[\\\"0\\\", \
\\\"TR\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"n\\\", \\\"TI\\\"], \
StyleBox[\\\"y\\\", \\\"TI\\\"]]}], \\\"}\\\"}], \\\",\\\", StyleBox[\\\"\
\[Ellipsis]\\\", \\\"TR\\\"]}], \\\"]\\\"}]\\) successively finds series \
expansions with respect to \\!\\(\\*StyleBox[\\\"x\\\", \\\"TI\\\"]\\), then \
\\!\\(\\*StyleBox[\\\"y\\\", \\\"TI\\\"]\\), etc. \"\>", "MSG"], 
  "\[NonBreakingSpace]", 
  ButtonBox[
   StyleBox["\[RightSkeleton]", "SR"],
   Active->True,
   BaseStyle->"Link",
   ButtonData->"paclet:ref/Series"]}]], "Print", "PrintUsage",
 CellChangeTimes->{3.628085821344878*^9},
 CellTags->"Info3628089421-5827330"]
}, Closed]],

Cell["The first term of this series along an arbitrary line is", "Text",
 CellChangeTimes->{{3.591601114977907*^9, 3.5916011479037905`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"u", "[", "t_", "]"}], " ", ":=", " ", 
   RowBox[{"s1", "*", "t"}]}], ";", 
  RowBox[{
   RowBox[{"v", "[", "t_", "]"}], " ", ":=", 
   RowBox[{"s2", "*", "t"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5916003676521626`*^9, 3.5916004260495024`*^9}, {
  3.591600594235122*^9, 3.591600630105174*^9}, {3.6184687864831133`*^9, 
  3.618468807839551*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"Series", "[", 
    RowBox[{
     RowBox[{"f", "[", 
      RowBox[{
       RowBox[{"u", "[", "t", "]"}], ",", 
       RowBox[{"v", "[", "t", "]"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", "0", ",", "2"}], "}"}]}], "]"}], ",", 
   RowBox[{"tcond", "[", "f", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5915937069431915`*^9, 3.591593725240238*^9}, {
  3.59159376058526*^9, 3.5915937987624435`*^9}, {3.591593849634353*^9, 
  3.591593853378567*^9}, {3.591594000297971*^9, 3.591594019612075*^9}, {
  3.5915998108033123`*^9, 3.591599837041813*^9}, {3.5916003026874466`*^9, 
  3.5916003256267586`*^9}, {3.5916004415553894`*^9, 3.5916004544021244`*^9}, {
  3.5916006394597087`*^9, 3.591600644394991*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["s2", "2"], " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "2"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"0", ",", "0"}], "]"}]}], "+", 
      RowBox[{"2", " ", "s1", " ", "s2", " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"0", ",", "0"}], "]"}]}], "+", 
      RowBox[{
       SuperscriptBox["s1", "2"], " ", 
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"2", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"0", ",", "0"}], "]"}]}]}], ")"}], " ", 
    SuperscriptBox["t", "2"]}], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "t", "]"}], "3"],
    SeriesData[$CellContext`t, 0, {}, 2, 3, 1],
    Editable->False]}],
  SeriesData[$CellContext`t, 0, {
   Rational[1, 2] ($CellContext`s2^2 
      Derivative[0, 2][$CellContext`f][
       0, 0] + ((2 $CellContext`s1) $CellContext`s2) 
      Derivative[1, 1][$CellContext`f][0, 0] + $CellContext`s1^2 
      Derivative[2, 0][$CellContext`f][0, 0])}, 2, 3, 1],
  Editable->False]], "Output",
 CellChangeTimes->{3.6280858376781073`*^9}]
}, Closed]],

Cell["\<\
This is the half of the second fundamental form of graph[f] at the point \
{0,0}. We consider simple test functions with given principal curvatures of \
their graphs:\
\>", "Text",
 CellChangeTimes->{{3.591601196950596*^9, 3.5916012447353287`*^9}, {
  3.591601283679556*^9, 3.5916013527195053`*^9}, {3.6154352752225156`*^9, 
  3.615435283849331*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"testf", "[", 
    RowBox[{"k1_", ",", "k2_"}], "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"k1", "*", 
       RowBox[{"u", "^", "2"}]}], "+", 
      RowBox[{"k2", "*", 
       RowBox[{"v", "^", "2"}]}]}], ")"}]}], "/", "2"}]}]], "Input",
 CellChangeTimes->{{3.5915953083397865`*^9, 3.591595409400567*^9}, {
   3.591596024913772*^9, 3.5915960290890107`*^9}, 3.591686360140068*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tcond", "[", 
  RowBox[{"testf", "[", 
   RowBox[{"k1", ",", "k2"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.591595419727157*^9, 3.5915954602054725`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True", ",", "True"}], "}"}]], "Output",
 CellChangeTimes->{3.6280858987678146`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"prcurvatures", "[", 
      RowBox[{"graph", "[", 
       RowBox[{"testf", "[", 
        RowBox[{"k1", ",", "k2"}], "]"}], "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"u", "\[Rule]", "0"}], ",", 
      RowBox[{"v", "\[Rule]", "0"}]}], "}"}]}], ",", " ", 
   RowBox[{"k1", "\[GreaterEqual]", " ", "k2"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.591601370840542*^9, 3.591601405881546*^9}, 
   3.591604992271819*^9, {3.591605166785801*^9, 3.5916051749442673`*^9}, 
   3.591605221897953*^9, {3.5916055418982563`*^9, 3.5916055638985147`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"k2", ",", "k1"}], "}"}]], "Output",
 CellChangeTimes->{3.6280859101870346`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"gauss", "[", 
      RowBox[{"graph", "[", 
       RowBox[{"testf", "[", 
        RowBox[{"k1", ",", "k2"}], "]"}], "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"u", "\[Rule]", "0"}], ",", 
      RowBox[{"v", "\[Rule]", "0"}]}], "}"}]}], ",", " ", 
   RowBox[{"k1", "\[GreaterEqual]", " ", "k2"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.591601370840542*^9, 3.591601405881546*^9}, 
   3.591604992271819*^9, {3.591605166785801*^9, 3.5916051749442673`*^9}, 
   3.591605221897953*^9, {3.5916055418982563`*^9, 3.5916055638985147`*^9}, {
   3.5916056076470165`*^9, 3.5916056199587207`*^9}}],

Cell[BoxData[
 RowBox[{"k1", " ", "k2"}]], "Output",
 CellChangeTimes->{3.6280859284078665`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"meancurvature", "[", 
      RowBox[{"graph", "[", 
       RowBox[{"testf", "[", 
        RowBox[{"k1", ",", "k2"}], "]"}], "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"u", "\[Rule]", "0"}], ",", 
      RowBox[{"v", "\[Rule]", "0"}]}], "}"}]}], ",", " ", 
   RowBox[{"k1", "\[GreaterEqual]", " ", "k2"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.591601370840542*^9, 3.591601405881546*^9}, 
   3.591604992271819*^9, {3.591605166785801*^9, 3.5916051749442673`*^9}, 
   3.591605221897953*^9, {3.5916055418982563`*^9, 3.5916055638985147`*^9}, {
   3.5916056076470165`*^9, 3.591605642184992*^9}, 3.614861859238339*^9, 
   3.6148631365841827`*^9, 3.615275821972936*^9, 3.615275880248269*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{"k1", "+", "k2"}], "2"]], "Output",
 CellChangeTimes->{3.6280859360206795`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"fund2", "[", 
     RowBox[{"graph", "[", 
      RowBox[{"testf", "[", 
       RowBox[{"k1", ",", "k2"}], "]"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "[", 
   RowBox[{"ss", ",", "ss"}], "]"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"u", "\[Rule]", "0"}], ",", 
    RowBox[{"v", "\[Rule]", "0"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.591596189028159*^9, 3.5915962745710516`*^9}, 
   3.5916064079967937`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"k1", " ", 
   SuperscriptBox["s1", "2"]}], "+", 
  RowBox[{"k2", " ", 
   SuperscriptBox["s2", "2"]}]}]], "Output",
 CellChangeTimes->{3.62808594168349*^9}]
}, Closed]],

Cell[TextData[{
 "In the near of  an ",
 StyleBox["elliptic point",
  FontColor->RGBColor[0, 0, 1]],
 ", i.e. K[u,v] >0, the shape of the surface is similar to"
}], "Text",
 CellChangeTimes->{{3.5916056834863544`*^9, 3.591605780221887*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"graph", "[", 
     RowBox[{"testf", "[", 
      RowBox[{"1", ",", "3"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", ".3"}], ",", ".3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", ".3"}], ",", ".3"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.591595505721076*^9, 3.591595605058758*^9}, 
   3.5915957711872597`*^9}],

Cell["\<\
In particular, for an umbilic point with K > 0 we have the shape nearly to a \
spherical surface\
\>", "Text",
 CellChangeTimes->{{3.591605792010562*^9, 3.5916059000707426`*^9}, {
  3.5916860947056017`*^9, 3.591686125422056*^9}, {3.618469149698551*^9, 
  3.6184691517889547`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"graph", "[", 
     RowBox[{"testf", "[", 
      RowBox[{"3", ",", "3"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", ".3"}], ",", ".3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", ".3"}], ",", ".3"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.591595505721076*^9, 3.591595605058758*^9}, {
  3.591595742887641*^9, 3.591595750138056*^9}, {3.591605870594056*^9, 
  3.5916058715061083`*^9}}],

Cell[TextData[{
 "Near an",
 StyleBox[" hyperbolic point",
  FontColor->RGBColor[0, 0, 1]],
 ", K<0, the shape of the surface is near to"
}], "Text",
 CellChangeTimes->{{3.5916060217467017`*^9, 3.5916060856463566`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"graph", "[", 
     RowBox[{"testf", "[", 
      RowBox[{
       RowBox[{"-", "4"}], ",", "3"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", ".3"}], ",", ".3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", ".3"}], ",", ".3"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.591595505721076*^9, 3.591595645378064*^9}, {
  3.591596110219651*^9, 3.5915961157469673`*^9}, {3.591686418702571*^9, 
  3.591686425004982*^9}}],

Cell[TextData[{
 "This is a ",
 StyleBox["saddle point",
  FontColor->RGBColor[0, 0, 1]],
 " of the function ftest.  For a ",
 StyleBox["parabolic point",
  FontColor->RGBColor[0, 0, 1]],
 ", i.e. k1>k2 = 0, (or k1<k2 = 0) , the shape of the surface is of type"
}], "Text",
 CellChangeTimes->{{3.591606125706648*^9, 3.591606367550481*^9}, 
   3.591686581754058*^9}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"graph", "[", 
     RowBox[{"testf", "[", 
      RowBox[{"0", ",", "3"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", ".3"}], ",", ".3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", ".3"}], ",", ".3"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.591595505721076*^9, 3.591595645378064*^9}, {
   3.5915956756417947`*^9, 3.591595686778432*^9}, 3.5916064444268775`*^9, 
   3.591686453256632*^9}],

Cell["\<\
As an exercise plot the intersection of the surface with its tangent plane at \
the point {0,0}  in each of the cases:\
\>", "Text",
 CellChangeTimes->{{3.5916867209219017`*^9, 3.5916868015584435`*^9}, {
   3.5916868490917273`*^9, 3.591686866766558*^9}, 3.5916875684089904`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reduce", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"graph", "[", 
        RowBox[{"testf", "[", 
         RowBox[{"k1", ",", "k2"}], "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ")"}], ".", 
     RowBox[{"stb", "[", "3", "]"}]}], " ", "\[Equal]", "0"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"u", ",", "v"}], "}"}], ",", " ", "Reals"}], "]"}]], "Input",
 CellChangeTimes->{{3.591687035543255*^9, 3.591687042812867*^9}, {
   3.591687140562639*^9, 3.5916871955839357`*^9}, 3.6148618592539387`*^9, 
   3.6149177910625596`*^9, {3.618469330019268*^9, 3.618469340970487*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"k2", "<", "0"}], "&&", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"k1", "<", "0"}], "&&", 
        RowBox[{"u", "\[Equal]", "0"}], "&&", 
        RowBox[{"v", "\[Equal]", "0"}]}], ")"}], "||", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"k1", "\[GreaterEqual]", "0"}], "&&", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"v", "\[Equal]", 
           RowBox[{"-", 
            SqrtBox[
             RowBox[{"-", 
              FractionBox[
               RowBox[{"k1", " ", 
                SuperscriptBox["u", "2"]}], "k2"]}]]}]}], "||", 
          RowBox[{"v", "\[Equal]", 
           SqrtBox[
            RowBox[{"-", 
             FractionBox[
              RowBox[{"k1", " ", 
               SuperscriptBox["u", "2"]}], "k2"]}]]}]}], ")"}]}], ")"}]}], 
     ")"}]}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"k2", "\[Equal]", "0"}], "&&", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"k1", "<", "0"}], "&&", 
        RowBox[{"u", "\[Equal]", "0"}]}], ")"}], "||", 
      RowBox[{"k1", "\[Equal]", "0"}], "||", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"k1", ">", "0"}], "&&", 
        RowBox[{"u", "\[Equal]", "0"}]}], ")"}]}], ")"}]}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"k2", ">", "0"}], "&&", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"k1", "\[LessEqual]", "0"}], "&&", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"v", "\[Equal]", 
           RowBox[{"-", 
            SqrtBox[
             RowBox[{"-", 
              FractionBox[
               RowBox[{"k1", " ", 
                SuperscriptBox["u", "2"]}], "k2"]}]]}]}], "||", 
          RowBox[{"v", "\[Equal]", 
           SqrtBox[
            RowBox[{"-", 
             FractionBox[
              RowBox[{"k1", " ", 
               SuperscriptBox["u", "2"]}], "k2"]}]]}]}], ")"}]}], ")"}], "||", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"k1", ">", "0"}], "&&", 
        RowBox[{"u", "\[Equal]", "0"}], "&&", 
        RowBox[{"v", "\[Equal]", "0"}]}], ")"}]}], ")"}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.628086009028808*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"u", ",", "v", ",", "ss", ",", "testf"}], "]"}]], "Input",
 CellChangeTimes->{{3.5916885145818524`*^9, 3.5916885328650846`*^9}, {
  3.618470302669377*^9, 3.6184703062573833`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["3.7. Ruled Surfaces: Criterion of Developability", "Subsection",
 CellChangeTimes->{{3.591771408757987*^9, 3.5917714528555093`*^9}, {
   3.594270351054023*^9, 3.5942703667164507`*^9}, 3.611641617461296*^9, 
   3.611667344308116*^9, 3.6154364525565834`*^9, 3.6154517959096875`*^9, {
   3.6184703670506897`*^9, 3.6184703699678946`*^9}, 3.618561968272961*^9, 
   3.628085177921748*^9, 3.6280861398350377`*^9, 3.628176382410313*^9}],

Cell["Ruled surface have been introduced in Subsection 2.1.2;", "Text",
 CellChangeTimes->{{3.59202391500099*^9, 3.592023996339533*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ruled", "[", 
   RowBox[{"x", ",", "y"}], "]"}], "[", 
  RowBox[{"s", ",", "t"}], "]"}]], "Input",
 CellChangeTimes->{{3.5920228263062778`*^9, 3.5920228604391375`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"x", "[", "t", "]"}], "[", "1", "]"}], "+", 
    RowBox[{"s", " ", 
     RowBox[{
      RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"x", "[", "t", "]"}], "[", "2", "]"}], "+", 
    RowBox[{"s", " ", 
     RowBox[{
      RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"x", "[", "t", "]"}], "[", "3", "]"}], "+", 
    RowBox[{"s", " ", 
     RowBox[{
      RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.628086814286622*^9}]
}, Closed]],

Cell["\<\
For  shortness we set for a general, not specified ruled surface\
\>", "Text",
 CellChangeTimes->{{3.6280868843463454`*^9, 3.6280869280888224`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"rsf", "[", 
   RowBox[{"s_", ",", "t_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"ruled", "[", 
    RowBox[{"x", ",", " ", "y"}], "]"}], "[", 
   RowBox[{"s", ",", "t"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5917723210491667`*^9, 3.591772332008794*^9}, {
   3.591772366105744*^9, 3.591772383985767*^9}, {3.591971789454519*^9, 
   3.5919717908585215`*^9}, 3.5919720609641957`*^9, {3.5920231716284847`*^9, 
   3.592023191643319*^9}, {3.5920234488097715`*^9, 3.592023471414211*^9}, 
   3.5920240269935865`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rsf", "[", 
  RowBox[{"s", ",", " ", "t"}], "]"}]], "Input",
 CellChangeTimes->{{3.591772390090116*^9, 3.591772396432479*^9}, 
   3.5919717922625237`*^9, 3.591972063741001*^9, 3.592023652046928*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"x", "[", "t", "]"}], "[", "1", "]"}], "+", 
    RowBox[{"s", " ", 
     RowBox[{
      RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"x", "[", "t", "]"}], "[", "2", "]"}], "+", 
    RowBox[{"s", " ", 
     RowBox[{
      RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"x", "[", "t", "]"}], "[", "3", "]"}], "+", 
    RowBox[{"s", " ", 
     RowBox[{
      RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.628086936544037*^9, 3.6281428076142035`*^9, 
  3.6281649242517357`*^9}]
}, Closed]],

Cell["Furthermore we set", "Text",
 CellChangeTimes->{{3.592024050081627*^9, 3.59202405156363*^9}, {
  3.6280869500068607`*^9, 3.628086963656885*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"xx", "[", "t_", "]"}], " ", "=", 
   RowBox[{"vec", "[", 
    RowBox[{"x", "[", "t", "]"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"yy", "[", "t_", "]"}], " ", "=", 
   RowBox[{"vec", "[", 
    RowBox[{"y", "[", "t", "]"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.591771503590411*^9, 3.5917715096997604`*^9}, {
   3.591771552281196*^9, 3.5917715868411727`*^9}, {3.591771626377434*^9, 
   3.591771658025244*^9}, {3.5917719866660414`*^9, 3.5917720404511175`*^9}, {
   3.591772073435004*^9, 3.5917722000862484`*^9}, {3.591772279558794*^9, 
   3.591772307915416*^9}, {3.5919717709996862`*^9, 3.5919717880817165`*^9}, 
   3.591971995686881*^9, {3.592023012071404*^9, 3.5920230840655303`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"rsf", "[", 
   RowBox[{"s", ",", "t"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"xx", "[", "t", "]"}], "+", 
   RowBox[{"s", "*", 
    RowBox[{"yy", "[", "t", "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.6281428797487297`*^9, 3.6281429318996215`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{
  3.6281429330852237`*^9, {3.628164908805708*^9, 3.628164938775361*^9}}]
}, Closed]],

Cell["\<\
and assume that the following three conditions are satisfied:\
\>", "Text",
 CellChangeTimes->{{3.592024086991292*^9, 3.5920240913905*^9}, {
  3.5920241521058064`*^9, 3.592024182026659*^9}}],

Cell["Conditions. 1. t is the arc length of the curve xx[t]: ", "Text",
 CellChangeTimes->{{3.5917724731278653`*^9, 3.5917725324312572`*^9}, 
   3.591971800358938*^9, 3.591972152161956*^9, {3.6154366314108973`*^9, 
   3.615436634546503*^9}, 3.6184912621753073`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"xx", "'"}], "[", "t", "]"}], ".", 
   RowBox[{
    RowBox[{"xx", "'"}], "[", "t", "]"}]}], "\[Equal]", " ", "1"}]], "Input",
 CellChangeTimes->{{3.5917725466500707`*^9, 3.5917725635370364`*^9}, 
   3.5917727670656776`*^9, {3.5919718012949395`*^9, 3.5919718020593405`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox[
    RowBox[{
     RowBox[{
      SuperscriptBox["x", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], "2"], "+", 
   SuperscriptBox[
    RowBox[{
     RowBox[{
      SuperscriptBox["x", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], "2"], "+", 
   SuperscriptBox[
    RowBox[{
     RowBox[{
      SuperscriptBox["x", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], "2"]}], 
  "\[Equal]", "1"}]], "Output",
 CellChangeTimes->{{3.6280869842645206`*^9, 3.6280869927509356`*^9}}]
}, Closed]],

Cell["2.s is the arc length of the rulings (the generating lines)", "Text",
 CellChangeTimes->{{3.5917726476638484`*^9, 3.5917727400471325`*^9}, 
   3.5919721589635677`*^9, {3.592024220246726*^9, 3.5920242610719976`*^9}, 
   3.6184912986169715`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"yy", "[", "t", "]"}], ".", 
   RowBox[{"yy", "[", "t", "]"}]}], "\[Equal]", " ", "1"}]], "Input",
 CellChangeTimes->{{3.5917727481735973`*^9, 3.591772758993216*^9}, {
  3.5919718461606183`*^9, 3.5919718469562197`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox[
    RowBox[{
     RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], "2"], "+", 
   SuperscriptBox[
    RowBox[{
     RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "2"], "+", 
   SuperscriptBox[
    RowBox[{
     RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "2"]}], "\[Equal]", 
  "1"}]], "Output",
 CellChangeTimes->{3.6280870081013627`*^9}]
}, Closed]],

Cell["3. The directrix intersects the rulings orthogonal :", "Text",
 CellChangeTimes->{{3.5917728253350105`*^9, 3.5917728708076115`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"xx", "'"}], "[", "t", "]"}], ".", 
   RowBox[{"yy", "[", "t", "]"}]}], "\[Equal]", " ", "0"}]], "Input",
 CellChangeTimes->{{3.5917728840893707`*^9, 3.5917728999052753`*^9}, {
  3.5919718565346365`*^9, 3.5919718575018387`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["x", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["x", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["x", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
  "\[Equal]", "0"}]], "Output",
 CellChangeTimes->{3.6280870233269897`*^9}]
}, Closed]],

Cell["\<\
These conditions mean 1, that t is the arc length of the directrix and that \
the rulings  are a family of normals of the directrix spanned by unit normal \
vectors of the directrix.
We collect the conditions and their derivatives into\
\>", "Text",
 CellChangeTimes->{{3.591772976055631*^9, 3.5917730201351523`*^9}, 
   3.6281472022053223`*^9, {3.628147237792985*^9, 3.628147262928629*^9}, {
   3.628147298422692*^9, 3.6281474116636915`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"ruledcond", "[", 
   RowBox[{"f_", ",", "g_", ",", "t_"}], "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"f", "'"}], "[", "t", "]"}], ".", 
      RowBox[{
       RowBox[{"f", "'"}], "[", "t", "]"}]}], "\[Equal]", " ", "1"}], ",", 
    " ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"f", "'"}], "[", "t", "]"}], ".", 
      RowBox[{
       RowBox[{"f", "''"}], "[", "t", "]"}]}], "\[Equal]", " ", "0"}], ",", 
    " ", 
    RowBox[{
     RowBox[{
      RowBox[{"g", "[", "t", "]"}], ".", 
      RowBox[{"g", "[", "t", "]"}]}], "\[Equal]", " ", "1"}], ",", " ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"g", "'"}], "[", "t", "]"}], ".", 
      RowBox[{"g", "[", "t", "]"}]}], "\[Equal]", "  ", "0"}], ",", " ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"f", "'"}], "[", "t", "]"}], ".", 
      RowBox[{"g", "[", "t", "]"}]}], " ", "\[Equal]", " ", "0"}], ",", " ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"f", "''"}], "[", "t", "]"}], ".", 
       RowBox[{"g", "[", "t", "]"}]}], " ", "+", 
      RowBox[{
       RowBox[{
        RowBox[{"f", "'"}], "[", "t", "]"}], ".", 
       RowBox[{
        RowBox[{"g", "'"}], "[", "t", "]"}]}]}], " ", "\[Equal]", " ", 
     "0"}]}], "}"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5917730578503094`*^9, 3.591773257268716*^9}, {
  3.591773294164826*^9, 3.5917733554303303`*^9}, {3.5917734069742785`*^9, 
  3.5917734221761475`*^9}, {3.5917735277751875`*^9, 3.5917735384467983`*^9}, {
  3.5919718795134773`*^9, 3.5919719042083206`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ruledcond", "[", 
  RowBox[{"xx", ",", "yy", ",", "t"}], "]"}]], "Input",
 CellChangeTimes->{{3.5917734434103622`*^9, 3.5917734747451544`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], "2"], 
     "+", 
     SuperscriptBox[
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], "2"], 
     "+", 
     SuperscriptBox[
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], "2"]}], 
    "\[Equal]", "1"}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
    "\[Equal]", "0"}], ",", 
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "2"]}], "\[Equal]", 
    "1"}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["y", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["y", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["y", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
    "\[Equal]", "0"}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
    "\[Equal]", "0"}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["y", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["y", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["y", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
    "\[Equal]", "0"}]}], "}"}]], "Output",
 CellChangeTimes->{3.628087037148614*^9, 3.628142963177676*^9}]
}, Closed]],

Cell["\<\
Now we calculate the \[OpenCurlyDoubleQuote]fundamental\
\[CloseCurlyDoubleQuote] invariants of the ruled surface and express them by \
the defining vectors. For the first fundamental form we obtain\
\>", "Text",
 CellChangeTimes->{{3.592024910672739*^9, 3.5920249298295727`*^9}, {
   3.5920258232431417`*^9, 3.592025891040861*^9}, 3.618491368785895*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"Evaluate", "[", 
    RowBox[{"ruledcond", "[", 
     RowBox[{"xx", ",", "yy", ",", "t"}], "]"}], "]"}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"fund1", "[", "rsf", "]"}], "[", 
     RowBox[{"s", ",", " ", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"u1", ",", "u2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"u1", ",", "u2"}], "}"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.591773629631014*^9, 3.591773714170849*^9}, {
   3.5917737484678106`*^9, 3.5917737560822463`*^9}, {3.591773808723257*^9, 
   3.591773814242573*^9}, {3.591775768244335*^9, 3.5917757816751037`*^9}, {
   3.5917786327132173`*^9, 3.591778658722705*^9}, {3.591788739317565*^9, 
   3.591788759352711*^9}, 3.5917889229130664`*^9, 3.5917890378886423`*^9, {
   3.59178994401447*^9, 3.5917900186787405`*^9}, 3.5917933838012147`*^9, {
   3.591793669191538*^9, 3.5917937009843564`*^9}, {3.5917951837611666`*^9, 
   3.5917952037283087`*^9}, {3.591795236712195*^9, 3.591795257680394*^9}, {
   3.5917967300736103`*^9, 3.59179673880011*^9}, 3.5919455581142216`*^9, 
   3.591945593588684*^9, {3.5919476014838104`*^9, 3.5919476613255157`*^9}, {
   3.5919499489159336`*^9, 3.59195003789849*^9}, 3.5920236628265467`*^9, 
   3.592024567659336*^9, {3.6184922379570217`*^9, 3.6184923055051403`*^9}, {
   3.61849235670443*^9, 3.61849236233604*^9}, 3.618493122774976*^9, {
   3.6280871721356506`*^9, 3.6280871832740707`*^9}, 3.6280872203085356`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["u1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "2"]}], ")"}]}], "+", 
  RowBox[{"2", " ", "u1", " ", "u2", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], "+", 
        RowBox[{"s", " ", 
         RowBox[{
          RowBox[{
           SuperscriptBox["y", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}]}], 
       ")"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], "+", 
        RowBox[{"s", " ", 
         RowBox[{
          RowBox[{
           SuperscriptBox["y", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}]}], 
       ")"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], "+", 
        RowBox[{"s", " ", 
         RowBox[{
          RowBox[{
           SuperscriptBox["y", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
       ")"}]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["u2", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], "+", 
        RowBox[{"s", " ", 
         RowBox[{
          RowBox[{
           SuperscriptBox["y", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}]}], 
       ")"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], "+", 
        RowBox[{"s", " ", 
         RowBox[{
          RowBox[{
           SuperscriptBox["y", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}]}], 
       ")"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], "+", 
        RowBox[{"s", " ", 
         RowBox[{
          RowBox[{
           SuperscriptBox["y", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
       ")"}], "2"]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.62808722275774*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", ",", 
   RowBox[{"ruledcond", "[", 
    RowBox[{"xx", ",", "yy", ",", "t"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.628087324594719*^9, 3.6280873683215957`*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["u1", "2"], "+", 
  RowBox[{
   SuperscriptBox["u2", "2"], " ", 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     RowBox[{
      SuperscriptBox["s", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["y", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["y", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["y", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], 
         "2"]}], ")"}]}], "-", 
     RowBox[{"2", " ", "s", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], "+", 
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], "+", 
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
       ")"}]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.62808744485533*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], "+", 
      RowBox[{
       RowBox[{
        RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], "+", 
      RowBox[{
       RowBox[{
        RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
     ")"}], "\[Equal]", 
    RowBox[{"-", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"y", "'"}], "[", "t", "]"}], "[", "1", "]"}], " ", 
        RowBox[{
         RowBox[{
          RowBox[{"x", "'"}], "[", "t", "]"}], "[", "1", "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"y", "'"}], "[", "t", "]"}], "[", "2", "]"}], " ", 
        RowBox[{
         RowBox[{
          RowBox[{"x", "'"}], "[", "t", "]"}], "[", "2", "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"y", "'"}], "[", "t", "]"}], "[", "3", "]"}], " ", 
        RowBox[{
         RowBox[{
          RowBox[{"x", "'"}], "[", "t", "]"}], "[", "3", "]"}]}]}], ")"}]}]}],
    ",", 
   RowBox[{"ruledcond", "[", 
    RowBox[{"xx", ",", "yy", ",", "t"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6184932156419387`*^9, 3.618493233987571*^9}, {
   3.618493313578911*^9, 3.6184933488349724`*^9}, 3.6184959796391935`*^9, 
   3.6184960168452587`*^9, 3.6184960623505383`*^9, {3.6280875750375586`*^9, 
   3.6280875828687725`*^9}, {3.628088359812537*^9, 3.6280884551443043`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.62808759286839*^9, 3.628088457562309*^9}]
}, Closed]],

Cell["The coefficients are", "Text",
 CellChangeTimes->{{3.5920256205363855`*^9, 3.592025648600835*^9}, {
  3.5920259057828865`*^9, 3.592025917529707*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"Evaluate", "[", 
    RowBox[{"ruledcond", "[", 
     RowBox[{"xx", ",", "yy", ",", "t"}], "]"}], "]"}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"rgT", "[", 
         RowBox[{"1", ",", "1"}], "]"}], "[", "rsf", "]"}], "[", 
       RowBox[{"s", ",", " ", "t"}], "]"}], "\[Equal]", " ", "1"}], ",", 
     RowBox[{"TimeConstraint", "\[Rule]", " ", "Infinity"}]}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.591773629631014*^9, 3.591773714170849*^9}, {
   3.5917737484678106`*^9, 3.5917737560822463`*^9}, {3.591773808723257*^9, 
   3.591773814242573*^9}, {3.591775768244335*^9, 3.5917757816751037`*^9}, {
   3.5917786327132173`*^9, 3.591778658722705*^9}, {3.591788739317565*^9, 
   3.591788759352711*^9}, 3.5917889229130664`*^9, 3.5917890378886423`*^9, {
   3.59178994401447*^9, 3.5917900186787405`*^9}, 3.5917933838012147`*^9, {
   3.591793669191538*^9, 3.5917937009843564`*^9}, {3.5917951837611666`*^9, 
   3.5917952037283087`*^9}, {3.591795236712195*^9, 3.591795257680394*^9}, {
   3.5917967300736103`*^9, 3.59179673880011*^9}, 3.5919455581142216`*^9, 
   3.591945593588684*^9, {3.5919476014838104`*^9, 3.5919476613255157`*^9}, {
   3.5919499489159336`*^9, 3.59195003789849*^9}, 3.5920236628265467`*^9, 
   3.592024567659336*^9, {3.592024950874009*^9, 3.5920250921166573`*^9}, 
   3.5920251260467167`*^9, 3.5920257017501283`*^9, 3.6176282336851797`*^9, 
   3.6184923352699924`*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6280884831463537`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"ruledcond", "[", 
    RowBox[{"xx", ",", "yy", ",", "t"}], "]"}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"rgT", "[", 
         RowBox[{"1", ",", "2"}], "]"}], "[", "rsf", "]"}], "[", 
       RowBox[{"s", ",", " ", "t"}], "]"}], "\[Equal]", " ", "0"}], ",", 
     RowBox[{"TimeConstraint", "\[Rule]", " ", "Infinity"}]}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.591773629631014*^9, 3.591773714170849*^9}, {
   3.5917737484678106`*^9, 3.5917737560822463`*^9}, {3.591773808723257*^9, 
   3.591773814242573*^9}, {3.591775768244335*^9, 3.5917757816751037`*^9}, {
   3.5917786327132173`*^9, 3.591778658722705*^9}, {3.591788739317565*^9, 
   3.591788759352711*^9}, 3.5917889229130664`*^9, 3.5917890378886423`*^9, {
   3.59178994401447*^9, 3.5917900186787405`*^9}, 3.5917933838012147`*^9, {
   3.591793669191538*^9, 3.5917937009843564`*^9}, {3.5917951837611666`*^9, 
   3.5917952037283087`*^9}, {3.591795236712195*^9, 3.591795257680394*^9}, {
   3.5917967300736103`*^9, 3.59179673880011*^9}, 3.5919455581142216`*^9, 
   3.591945593588684*^9, {3.5919476014838104`*^9, 3.5919476613255157`*^9}, {
   3.5919499489159336`*^9, 3.59195003789849*^9}, 3.5920236628265467`*^9, 
   3.592024567659336*^9, {3.592024950874009*^9, 3.5920250784978333`*^9}, 
   3.5920251338467307`*^9, 3.5920251948428383`*^9, 3.6176282415886316`*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.628088490431566*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"ruledcond", "[", 
    RowBox[{"xx", ",", "yy", ",", "t"}], "]"}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"rgT", "[", 
         RowBox[{"2", ",", "2"}], "]"}], "[", "rsf", "]"}], "[", 
       RowBox[{"s", ",", " ", "t"}], "]"}], "\[Equal]", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"xx", "'"}], "[", "t", "]"}], "+", " ", 
         RowBox[{"s", "*", 
          RowBox[{
           RowBox[{"yy", "'"}], "[", "t", "]"}]}]}], ")"}], ".", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"xx", "'"}], "[", "t", "]"}], "+", " ", 
         RowBox[{"s", "*", 
          RowBox[{
           RowBox[{"yy", "'"}], "[", "t", "]"}]}]}], ")"}]}]}], ",", 
     RowBox[{"TimeConstraint", "\[Rule]", " ", "Infinity"}]}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.591773629631014*^9, 3.591773714170849*^9}, {
   3.5917737484678106`*^9, 3.5917737560822463`*^9}, {3.591773808723257*^9, 
   3.591773814242573*^9}, {3.591775768244335*^9, 3.5917757816751037`*^9}, {
   3.5917786327132173`*^9, 3.591778658722705*^9}, {3.591788739317565*^9, 
   3.591788759352711*^9}, 3.5917889229130664`*^9, 3.5917890378886423`*^9, {
   3.59178994401447*^9, 3.5917900186787405`*^9}, 3.5917933838012147`*^9, {
   3.591793669191538*^9, 3.5917937009843564`*^9}, {3.5917951837611666`*^9, 
   3.5917952037283087`*^9}, {3.591795236712195*^9, 3.591795257680394*^9}, {
   3.5917967300736103`*^9, 3.59179673880011*^9}, 3.5919455581142216`*^9, 
   3.591945593588684*^9, {3.5919476014838104`*^9, 3.5919476613255157`*^9}, {
   3.5919499489159336`*^9, 3.59195003789849*^9}, 3.5920236628265467`*^9, 
   3.592024567659336*^9, {3.592024950874009*^9, 3.5920250784978333`*^9}, 
   3.5920251338467307`*^9, {3.5920251948428383`*^9, 3.5920252157780743`*^9}, {
   3.5920254784357357`*^9, 3.5920255461554546`*^9}, 3.6176282474619675`*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.628088563112094*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"rgT", "[", 
      RowBox[{"2", ",", "2"}], "]"}], "[", "rsf", "]"}], "[", 
    RowBox[{"s", ",", " ", "t"}], "]"}], ",", 
   RowBox[{"ruledcond", "[", 
    RowBox[{"xx", ",", "yy", ",", "t"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.591773629631014*^9, 3.591773714170849*^9}, {
   3.5917737484678106`*^9, 3.5917737560822463`*^9}, {3.591773808723257*^9, 
   3.591773814242573*^9}, {3.591775768244335*^9, 3.5917757816751037`*^9}, {
   3.5917786327132173`*^9, 3.591778658722705*^9}, {3.591788739317565*^9, 
   3.591788759352711*^9}, 3.5917889229130664`*^9, 3.5917890378886423`*^9, {
   3.59178994401447*^9, 3.5917900186787405`*^9}, 3.5917933838012147`*^9, {
   3.591793669191538*^9, 3.5917937009843564`*^9}, {3.5917951837611666`*^9, 
   3.5917952037283087`*^9}, {3.591795236712195*^9, 3.591795257680394*^9}, {
   3.5917967300736103`*^9, 3.59179673880011*^9}, 3.5919455581142216`*^9, 
   3.591945593588684*^9, {3.5919476014838104`*^9, 3.5919476613255157`*^9}, {
   3.5919499489159336`*^9, 3.59195003789849*^9}, 3.5920236628265467`*^9, 
   3.592024567659336*^9, {3.592024950874009*^9, 3.5920250784978333`*^9}, 
   3.5920251338467307`*^9, {3.5920251948428383`*^9, 3.5920252157780743`*^9}, {
   3.5920254784357357`*^9, 3.5920255461554546`*^9}, 3.6176282474619675`*^9, {
   3.618495351020489*^9, 3.61849537412413*^9}, {3.628088659754264*^9, 
   3.628088690221117*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]",
        MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], "+", 
     RowBox[{"s", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["y", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}]}], 
    ")"}], "2"], "+", 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]",
        MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], "+", 
     RowBox[{"s", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["y", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}]}], 
    ")"}], "2"], "+", 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]",
        MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], "+", 
     RowBox[{"s", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["y", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
    ")"}], "2"]}]], "Output",
 CellChangeTimes->{3.628088649208645*^9, 3.6280887151343613`*^9}]
}, Closed]],

Cell["The Gaussian curvature:", "Text",
 CellChangeTimes->{{3.5920274989668846`*^9, 3.5920275363757505`*^9}, {
  3.6154383312678833`*^9, 3.615438345729109*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"gauss", "[", "rsf", "]"}], "[", 
    RowBox[{"s", ",", " ", "t"}], "]"}], ",", 
   RowBox[{"ruledcond", "[", 
    RowBox[{"xx", ",", "yy", ",", "t"}], "]"}], ",", 
   RowBox[{"TimeConstraint", "\[Rule]", "Infinity"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.591773629631014*^9, 3.591773714170849*^9}, {
   3.5917737484678106`*^9, 3.5917737560822463`*^9}, {3.591773808723257*^9, 
   3.591773814242573*^9}, {3.591775768244335*^9, 3.5917757816751037`*^9}, {
   3.5917786327132173`*^9, 3.591778658722705*^9}, 3.591945531859376*^9, 
   3.5919662569712014`*^9, {3.591966314535302*^9, 3.5919663241605196`*^9}, 
   3.5920236574601374`*^9, {3.62808877711327*^9, 3.628088801464913*^9}, {
   3.628088837953377*^9, 3.6280888486081953`*^9}, {3.6280890791454*^9, 
   3.6280890878658156`*^9}, {3.6280894318464203`*^9, 3.628089467274082*^9}, {
   3.628091090318059*^9, 3.628091101238078*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Simplify", "::", "time"}], "MessageName"], ":", 
  " ", "\<\"Time spent on a transformation exceeded \[NoBreak]\\!\\(300.`\\)\
\[NoBreak] seconds, and the transformation was aborted. Increasing the value \
of TimeConstraint option may improve the result of simplification. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/Simplify/time\\\", \
ButtonNote -> \\\"Simplify::time\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.6280914067178144`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Simplify", "::", "time"}], "MessageName"], ":", 
  " ", "\<\"Time spent on a transformation exceeded \[NoBreak]\\!\\(300.`\\)\
\[NoBreak] seconds, and the transformation was aborted. Increasing the value \
of TimeConstraint option may improve the result of simplification. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/Simplify/time\\\", \
ButtonNote -> \\\"Simplify::time\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.6280929353956995`*^9}],

Cell[BoxData[
 RowBox[{"-", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{
       RowBox[{
        SuperscriptBox["y", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], "2"], 
     "+", 
     SuperscriptBox[
      RowBox[{
       RowBox[{
        SuperscriptBox["y", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], "2"], 
     "+", 
     SuperscriptBox[
      RowBox[{
       RowBox[{
        SuperscriptBox["y", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], "2"], 
     "-", 
     SuperscriptBox[
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], "2"], 
     "+", 
     RowBox[{
      SuperscriptBox[
       RowBox[{
        RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], "2"]}], 
     "+", 
     RowBox[{
      SuperscriptBox[
       RowBox[{
        RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], "2"]}], 
     "-", 
     RowBox[{"2", " ", 
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], "-", 
     RowBox[{
      SuperscriptBox[
       RowBox[{
        RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], "2"]}], 
     "-", 
     RowBox[{"2", " ", 
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}], "-", 
     RowBox[{"2", " ", 
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}], "-", 
     RowBox[{
      SuperscriptBox[
       RowBox[{
        RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], 
       "2"]}]}], ")"}], "/", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "+", 
      RowBox[{
       SuperscriptBox["s", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], "2"],
          "+", 
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], "2"],
          "+", 
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], 
          "2"]}], ")"}]}], "-", 
      RowBox[{"2", " ", "s", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
        ")"}]}]}], ")"}], "2"]}]}]], "Output",
 CellChangeTimes->{3.628093757142743*^9}]
}, Closed]],

Cell["The result is entered in the following definition:", "Text",
 CellChangeTimes->{{3.615440149154676*^9, 3.6154401960015583`*^9}, 
   3.6184976289988904`*^9, {3.6281430842494893`*^9, 3.6281430990383153`*^9}, {
   3.6281435810479617`*^9, 3.6281435826703644`*^9}, {3.628143657542761*^9, 
   3.6281436783591976`*^9}, {3.6281442165307493`*^9, 3.628144221602758*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"gaussrsf", "[", 
   RowBox[{"s_", ",", " ", "t_"}], "]"}], " ", ":=", " ", "\n", "  ", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
         "]"}], "[", "1", "]"}], "^", "2"}], " ", "+", " ", "\n", "    ", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
         "]"}], "[", "2", "]"}], "^", "2"}], " ", "+", " ", "\n", "    ", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
         "]"}], "[", "3", "]"}], "^", "2"}], " ", "-", " ", "\n", "    ", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
         "]"}], "[", "1", "]"}], "^", "2"}], " ", "+", " ", "\n", "    ", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "^", "2"}], "*", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
          "]"}], "[", "1", "]"}], "^", "\n", "      ", "2"}]}], " ", "+", " ", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "^", "2"}], "*", "\n",
        "     ", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
          "]"}], "[", "1", "]"}], "^", "2"}]}], " ", "-", " ", "\n", "    ", 
      RowBox[{"2", "*", 
       RowBox[{
        RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], "*", 
       RowBox[{
        RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "*", "\n", "     ", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
         "]"}], "[", "1", "]"}], "*", "\n", "     ", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
         "]"}], "[", "2", "]"}]}], " ", "-", " ", "\n", "    ", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "^", "2"}], "*", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
          "]"}], "[", "2", "]"}], "^", "\n", "      ", "2"}]}], " ", "-", " ", 
      RowBox[{"2", "*", 
       RowBox[{
        RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], "*", 
       RowBox[{
        RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "*", "\n", "     ", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
         "]"}], "[", "1", "]"}], "*", "\n", "     ", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
         "]"}], "[", "3", "]"}]}], " ", "-", " ", "\n", "    ", 
      RowBox[{"2", "*", 
       RowBox[{
        RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "*", 
       RowBox[{
        RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "*", "\n", "     ", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
         "]"}], "[", "2", "]"}], "*", "\n", "     ", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
         "]"}], "[", "3", "]"}]}], " ", "-", " ", "\n", "    ", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "^", "2"}], "*", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
          "]"}], "[", "3", "]"}], "^", "\n", "      ", "2"}]}]}], ")"}]}], 
   "/", "\n", "   ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", " ", "+", " ", 
      RowBox[{
       RowBox[{"s", "^", "2"}], "*", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t",
             "]"}], "[", "1", "]"}], "^", "\n", "        ", "2"}], " ", "+", 
         " ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t",
             "]"}], "[", "2", "]"}], "^", "\n", "        ", "2"}], " ", "+", 
         " ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t",
             "]"}], "[", "3", "]"}], "^", "\n", "        ", "2"}]}], ")"}]}], 
      " ", "-", " ", 
      RowBox[{"2", "*", "s", "*", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], "*", "\n", 
          "        ", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t",
             "]"}], "[", "1", "]"}]}], " ", "+", " ", "\n", "       ", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "*", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t",
             "]"}], "[", "\n", "         ", "2", "]"}]}], " ", "+", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "*", "\n", 
          "        ", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t",
             "]"}], "[", "3", "]"}]}]}], ")"}]}]}], ")"}], "^", 
    "2"}]}]}]], "Input",
 CellChangeTimes->{{3.6280982673446646`*^9, 3.6280982786078844`*^9}, 
   3.628143725874282*^9}],

Cell["\<\
Clearly, the Gaussian curvature of a ruled surface is ever non-positive. 
We  calculate the coefficients of the second fundamental form with respect to \
the cobasis \[Sigma][1], \[Sigma][2]:\
\>", "Text",
 CellChangeTimes->{{3.591971612569808*^9, 3.591971643707463*^9}, {
   3.5920275682310066`*^9, 3.592027618712695*^9}, {3.5920276500999503`*^9, 
   3.592027688772418*^9}, 3.592046285230348*^9, {3.6154383708919525`*^9, 
   3.6154383797527685`*^9}, {3.628144104260951*^9, 3.628144171317869*^9}}],

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"ruledcond", "[", 
    RowBox[{"xx", ",", "yy", ",", "t"}], "]"}], ",", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"cf", "[", 
        RowBox[{"1", ",", "1"}], "]"}], "[", "rsf", "]"}], "[", 
      RowBox[{"s", ",", " ", "t"}], "]"}], ",", 
     RowBox[{"TimeConstraint", " ", "->", " ", "Infinity"}]}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.591773629631014*^9, 3.591773714170849*^9}, {
   3.5917737484678106`*^9, 3.5917737560822463`*^9}, {3.591773808723257*^9, 
   3.591773814242573*^9}, {3.591775768244335*^9, 3.5917757816751037`*^9}, {
   3.5917786327132173`*^9, 3.591778658722705*^9}, {3.591788739317565*^9, 
   3.591788759352711*^9}, 3.5919455433721952`*^9, 3.59202365875494*^9, 
   3.5920277181784697`*^9, 3.5920277515937285`*^9, 3.617972115784997*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"rsfc", "[", 
    RowBox[{"1", ",", "1"}], "]"}], "[", 
   RowBox[{"s_", ",", "t_"}], "]"}], ":=", "0"}]], "Input",
 CellChangeTimes->{{3.5917888627836275`*^9, 3.591788891982297*^9}, 
   3.592027795305005*^9, {3.5920463146675997`*^9, 3.592046340828846*^9}, 
   3.592046432603807*^9, 3.592046856191351*^9}],

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"ruledcond", "[", 
    RowBox[{"xx", ",", "yy", ",", "t"}], "]"}], ",", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"cf", "[", 
        RowBox[{"1", ",", "2"}], "]"}], "[", "rsf", "]"}], "[", 
      RowBox[{"s", ",", " ", "t"}], "]"}], ",", 
     RowBox[{"TimeConstraint", " ", "->", " ", "Infinity"}]}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.591773629631014*^9, 3.591773714170849*^9}, {
   3.5917737484678106`*^9, 3.5917737560822463`*^9}, {3.591773808723257*^9, 
   3.591773814242573*^9}, {3.591775768244335*^9, 3.5917757816751037`*^9}, {
   3.5917786327132173`*^9, 3.591778658722705*^9}, {3.591788739317565*^9, 
   3.591788759352711*^9}, 3.5919455433721952`*^9, 3.59202365875494*^9, 
   3.5920277181784697`*^9, 3.5920277515937285`*^9, {3.5920278491563*^9, 
   3.5920278705283375`*^9}, 3.592046850528541*^9, 3.617972162048643*^9}],

Cell["The result is entered in the following definition:", "Text",
 CellChangeTimes->{{3.615440149154676*^9, 3.6154401960015583`*^9}, 
   3.6184976289988904`*^9, {3.6281430842494893`*^9, 3.6281430990383153`*^9}, {
   3.6281435810479617`*^9, 3.6281435826703644`*^9}, {3.628143657542761*^9, 
   3.6281436783591976`*^9}, {3.6281442165307493`*^9, 3.628144221602758*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"rsfc", "[", 
    RowBox[{"1", ",", " ", "2"}], "]"}], "[", 
   RowBox[{"s_", ",", " ", "t_"}], "]"}], " ", ":=", " ", "\n", "  ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "*", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "2", "]"}], "*", "\n", "       ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
           "]"}], "[", "1", "]"}]}], " ", "-", " ", "\n", "      ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "1", "]"}], "*", "\n", "       ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
           "]"}], "[", "2", "]"}]}]}], ")"}]}], " ", "+", " ", "\n", "    ", 
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "*", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", 
             "t", "]"}], "[", "\n", "         ", "3", "]"}]}], ")"}], "*", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
           "]"}], "[", "\n", "        ", "1", "]"}]}], " ", "+", " ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "1", "]"}], "*", "\n", "       ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
           "]"}], "[", "3", "]"}]}]}], ")"}]}], " ", "+", " ", "\n", "    ", 
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], "*", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "3", "]"}], "*", "\n", "       ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
           "]"}], "[", "2", "]"}]}], " ", "-", " ", "\n", "      ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "2", "]"}], "*", "\n", "       ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
           "]"}], "[", "3", "]"}]}]}], ")"}]}]}], ")"}], "/", "\n", "   ", 
   RowBox[{"(", 
    RowBox[{"1", " ", "+", " ", 
     RowBox[{
      RowBox[{"s", "^", "2"}], "*", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
           "]"}], "[", "1", "]"}], "^", "\n", "       ", "2"}], " ", "+", " ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
           "]"}], "[", "2", "]"}], "^", "\n", "       ", "2"}], " ", "+", " ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
           "]"}], "[", "3", "]"}], "^", "\n", "       ", "2"}]}], ")"}]}], 
     " ", "-", " ", 
     RowBox[{"2", "*", "s", "*", "\n", "     ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], "*", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "\n", "        ", "1", "]"}]}], " ", "+", " ", 
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "*", "\n", "       ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "2", "]"}]}], " ", "+", " ", "\n", "      ", 
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "*", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "\n", "        ", "3", "]"}]}]}], ")"}]}]}], 
    ")"}]}]}]], "Input",
 CellChangeTimes->{{3.592046383806921*^9, 3.592046453539044*^9}, {
  3.614861859285139*^9, 3.614861859347539*^9}, {3.6149177910937595`*^9, 
  3.6149177911561604`*^9}, {3.615439089538415*^9, 3.615439122516873*^9}, {
  3.615439153966528*^9, 3.6154392339010687`*^9}, {3.628144258159622*^9, 
  3.628144268689641*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"rsfc", "[", 
    RowBox[{"2", ",", "1"}], "]"}], "[", 
   RowBox[{"s_", ",", " ", "t_"}], "]"}], " ", ":=", 
  RowBox[{
   RowBox[{"rsfc", "[", 
    RowBox[{"1", ",", "2"}], "]"}], "[", 
   RowBox[{"s", ",", " ", "t"}], "]"}], " "}]], "Input",
 CellChangeTimes->{{3.5920285948376093`*^9, 3.5920286219348574`*^9}, {
  3.5920464791854887`*^9, 3.5920465062983365`*^9}}],

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"ruledcond", "[", 
    RowBox[{"xx", ",", "yy", ",", "t"}], "]"}], ",", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"cf", "[", 
        RowBox[{"2", ",", "2"}], "]"}], "[", "rsf", "]"}], "[", 
      RowBox[{"s", ",", " ", "t"}], "]"}], ",", 
     RowBox[{"TimeConstraint", " ", "->", " ", "Infinity"}]}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.591773629631014*^9, 3.591773714170849*^9}, {
   3.5917737484678106`*^9, 3.5917737560822463`*^9}, {3.591773808723257*^9, 
   3.591773814242573*^9}, {3.591775768244335*^9, 3.5917757816751037`*^9}, {
   3.5917786327132173`*^9, 3.591778658722705*^9}, {3.591788739317565*^9, 
   3.591788759352711*^9}, 3.5919455433721952`*^9, 3.59202365875494*^9, 
   3.5920277181784697`*^9, 3.5920277515937285`*^9, {3.5920278491563*^9, 
   3.5920278705283375`*^9}, {3.5920279197152243`*^9, 3.592027932912847*^9}, 
   3.592046889169809*^9, 3.617972174952381*^9}],

Cell["The result is entered in the following definition:", "Text",
 CellChangeTimes->{{3.615440149154676*^9, 3.6154401960015583`*^9}, 
   3.6184976289988904`*^9, {3.6281430842494893`*^9, 3.6281430990383153`*^9}, {
   3.6281435810479617`*^9, 3.6281435826703644`*^9}, {3.628143657542761*^9, 
   3.6281436783591976`*^9}, {3.6281442165307493`*^9, 3.628144221602758*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"rsfc", "[", 
    RowBox[{"2", ",", " ", "2"}], "]"}], "[", 
   RowBox[{"s_", ",", " ", "t_"}], "]"}], " ", ":=", " ", "\n", "  ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "*", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", 
             "t", "]"}], "[", "1", "]"}]}], ")"}], "*", "\n", "       ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "2", "]"}]}], " ", "+", " ", "\n", "      ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "2", "]"}], "*", "\n", "       ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
             "t", "]"}], "[", "1", "]"}], " ", "+", " ", "\n", "        ", 
           RowBox[{"s", "*", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "1", "]"}]}]}], ")"}]}], " ", "+", " ", "\n", 
        "      ", 
        RowBox[{"s", "*", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "2", "]"}], "*", "\n", "         ", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
              "t", "]"}], "[", "1", "]"}]}], " ", "-", " ", "\n", "        ", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "1", "]"}], "*", "\n", "         ", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
              "t", "]"}], "[", "2", "]"}]}], " ", "-", " ", "\n", "        ", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", 
              "t", "]"}], "[", "1", "]"}], "*", "\n", "         ", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "2", "]"}]}]}], ")"}]}], " ", "+", " ", "\n", 
        "      ", 
        RowBox[{
         RowBox[{"s", "^", "2"}], "*", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "2", "]"}], "*", "\n", "         ", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "1", "]"}]}], " ", "-", " ", "\n", "        ", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "1", "]"}], "*", "\n", "         ", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "2", "]"}]}]}], ")"}]}]}], ")"}]}], " ", "+", 
     " ", "\n", "    ", 
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "*", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "1", "]"}], "*", "\n", "       ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "3", "]"}]}], " ", "-", " ", "\n", "      ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "3", "]"}], "*", "\n", "       ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
             "t", "]"}], "[", "1", "]"}], " ", "+", " ", "\n", "        ", 
           RowBox[{"s", "*", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "1", "]"}]}]}], ")"}]}], " ", "+", " ", "\n", 
        "      ", 
        RowBox[{"s", "*", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"-", 
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", 
                "t", "]"}], "[", "3", "]"}]}], ")"}], "*", "\n", "         ", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
              "t", "]"}], "[", "1", "]"}]}], " ", "+", " ", "\n", "        ", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "1", "]"}], "*", "\n", "         ", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
              "t", "]"}], "[", "3", "]"}]}], " ", "+", " ", "\n", "        ", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", 
              "t", "]"}], "[", "1", "]"}], "*", "\n", "         ", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "3", "]"}]}]}], ")"}]}], " ", "+", " ", "\n", 
        "      ", 
        RowBox[{
         RowBox[{"s", "^", "2"}], "*", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"-", 
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", 
                "t", "]"}], "[", "3", "]"}]}], ")"}], "*", "\n", "         ", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "1", "]"}]}], " ", "+", " ", "\n", "        ", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "1", "]"}], "*", "\n", "         ", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "3", "]"}]}]}], ")"}]}]}], ")"}]}], " ", "+", 
     " ", "\n", "    ", 
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], "*", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", 
             "t", "]"}], "[", "\n", "         ", "2", "]"}]}], ")"}], "*", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "3", "]"}]}], " ", "+", " ", "\n", "      ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "3", "]"}], "*", "\n", "       ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
             "t", "]"}], "[", "2", "]"}], " ", "+", " ", "\n", "        ", 
           RowBox[{"s", "*", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "2", "]"}]}]}], ")"}]}], " ", "+", " ", "\n", 
        "      ", 
        RowBox[{"s", "*", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "3", "]"}], "*", "\n", "         ", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
              "t", "]"}], "[", "2", "]"}]}], " ", "-", " ", "\n", "        ", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "2", "]"}], "*", "\n", "         ", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
              "t", "]"}], "[", "3", "]"}]}], " ", "-", " ", "\n", "        ", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", 
              "t", "]"}], "[", "2", "]"}], "*", "\n", "         ", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "3", "]"}]}]}], ")"}]}], " ", "+", " ", "\n", 
        "      ", 
        RowBox[{
         RowBox[{"s", "^", "2"}], "*", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "3", "]"}], "*", "\n", "         ", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "2", "]"}]}], " ", "-", " ", "\n", "        ", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "2", "]"}], "*", "\n", "         ", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "3", "]"}]}]}], ")"}]}]}], ")"}]}]}], ")"}], 
   "/", "\n", "   ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", " ", "+", " ", 
      RowBox[{
       RowBox[{"s", "^", "2"}], "*", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t",
             "]"}], "[", "1", "]"}], "^", "\n", "        ", "2"}], " ", "+", 
         " ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t",
             "]"}], "[", "2", "]"}], "^", "\n", "        ", "2"}], " ", "+", 
         " ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t",
             "]"}], "[", "3", "]"}], "^", "\n", "        ", "2"}]}], ")"}]}], 
      " ", "-", " ", 
      RowBox[{"2", "*", "s", "*", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], "*", "\n", 
          "        ", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t",
             "]"}], "[", "1", "]"}]}], " ", "+", " ", "\n", "       ", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "*", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t",
             "]"}], "[", "\n", "         ", "2", "]"}]}], " ", "+", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "*", "\n", 
          "        ", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t",
             "]"}], "[", "3", "]"}]}]}], ")"}]}]}], ")"}], "^", 
    RowBox[{"(", 
     RowBox[{"3", "/", "2"}], ")"}]}]}]}]], "Input",
 CellChangeTimes->{{3.618548391200186*^9, 3.61854839374299*^9}, 
   3.6185484318850574`*^9}],

Cell["\<\
 The Gaussian curvature is the determinant of these coefficients, see Section \
3.2\
\>", "Text",
 CellChangeTimes->{{3.5920281482400255`*^9, 3.592028194182106*^9}, {
  3.5920282896542735`*^9, 3.5920282970642867`*^9}}],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"Assuming", "[", 
   RowBox[{
    RowBox[{"ruledcond", "[", 
     RowBox[{"xx", ",", "yy", ",", "t"}], "]"}], ",", 
    RowBox[{"FullSimplify", "[", 
     RowBox[{
      RowBox[{"Det", "[", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"rsfc", "[", 
           RowBox[{"i", ",", "j"}], "]"}], "[", 
          RowBox[{"s", ",", "t"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "2"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}], ",", 
      RowBox[{"TimeConstraint", " ", "->", " ", "Infinity"}]}], "]"}]}], 
   "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.591773629631014*^9, 3.591773714170849*^9}, {
   3.5917737484678106`*^9, 3.5917737560822463`*^9}, {3.591773808723257*^9, 
   3.591773814242573*^9}, {3.591775768244335*^9, 3.5917757816751037`*^9}, {
   3.5917786327132173`*^9, 3.591778658722705*^9}, {3.591788739317565*^9, 
   3.591788759352711*^9}, 3.5919455433721952`*^9, 3.59202365875494*^9, 
   3.5920277181784697`*^9, 3.5920277515937285`*^9, {3.5920278491563*^9, 
   3.5920278705283375`*^9}, {3.5920279197152243`*^9, 3.592027932912847*^9}, {
   3.5920283473743753`*^9, 3.592028382022036*^9}, {3.5920284192281013`*^9, 
   3.5920285315326986`*^9}, 3.592046635310563*^9, {3.5920469265942745`*^9, 
   3.592046955766326*^9}, 3.6148618596127396`*^9, 3.614863501843224*^9}],

Cell["The result is entered in the following definition:", "Text",
 CellChangeTimes->{{3.615440149154676*^9, 3.6154401960015583`*^9}, 
   3.6184976289988904`*^9, {3.6281430842494893`*^9, 3.6281430990383153`*^9}, {
   3.6281435810479617`*^9, 3.6281435826703644`*^9}, {3.628143657542761*^9, 
   3.6281436783591976`*^9}, {3.6281442165307493`*^9, 3.628144221602758*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Krsf", "[", 
   RowBox[{"s_", ",", " ", "t_"}], "]"}], " ", ":=", " ", "\n", "  ", 
  RowBox[{"-", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
          "]"}], "[", "1", "]"}], "^", "2"}], " ", "+", " ", "\n", "     ", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
          "]"}], "[", "2", "]"}], "^", "2"}], " ", "+", " ", "\n", "     ", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
          "]"}], "[", "3", "]"}], "^", "2"}], " ", "+", " ", "\n", "     ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], " ", "+", " ", 
          RowBox[{
           RowBox[{
            RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "^", "2"}], " ", 
          "+", " ", 
          RowBox[{
           RowBox[{
            RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "^", "2"}]}], 
         ")"}], "*", "\n", "      ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "1", "]"}], "^", "2"}]}], " ", "-", " ", "\n", "     ", 
       RowBox[{"2", "*", 
        RowBox[{
         RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], "*", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
          "]"}], "[", "1", "]"}], "*", "\n", "      ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "*", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
             "t", "]"}], "[", "\n", "         ", "2", "]"}]}], " ", "+", " ", 
          RowBox[{
           RowBox[{
            RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "*", "\n", 
           "        ", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
             "t", "]"}], "[", "3", "]"}]}]}], ")"}]}], " ", "-", " ", "\n", 
       "     ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "*", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
             "t", "]"}], "[", "\n", "         ", "2", "]"}]}], " ", "+", " ", 
          RowBox[{
           RowBox[{
            RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "*", "\n", 
           "        ", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
             "t", "]"}], "[", "3", "]"}]}]}], ")"}], "^", "2"}]}], ")"}], "/",
      "\n", "    ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", " ", "+", " ", 
        RowBox[{
         RowBox[{"s", "^", "2"}], "*", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "1", "]"}], "^", "\n", "         ", "2"}], " ",
            "+", " ", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "2", "]"}], "^", "\n", "         ", "2"}], " ",
            "+", " ", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "3", "]"}], "^", "\n", "         ", "2"}]}], 
          ")"}]}], " ", "-", " ", 
        RowBox[{"2", "*", "s", "*", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], "*", "\n", 
            "         ", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
              "t", "]"}], "[", "1", "]"}]}], " ", "+", " ", "\n", "        ", 
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "*", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
              "t", "]"}], "[", "\n", "          ", "2", "]"}]}], " ", "+", 
           " ", 
           RowBox[{
            RowBox[{
             RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "*", "\n", 
            "         ", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
              "t", "]"}], "[", "3", "]"}]}]}], ")"}]}]}], ")"}], "^", "2"}]}],
     ")"}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5920466545921965`*^9, 3.592046673967431*^9}, {
  3.6148618596127396`*^9, 3.61486185967514*^9}, {3.6149177914057603`*^9, 
  3.6149177914681606`*^9}, {3.618549408914773*^9, 3.618549437447223*^9}}],

Cell["We control", "Text",
 CellChangeTimes->{{3.615440232958023*^9, 3.61544023660843*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"ruledcond", "[", 
    RowBox[{"xx", ",", "yy", ",", "t"}], "]"}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{"Krsf", "[", 
      RowBox[{"s", ",", "t"}], "]"}], "-", " ", 
     RowBox[{"gaussrsf", "[", 
      RowBox[{"s", ",", "t"}], "]"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.592031391735322*^9, 3.5920314016257396`*^9}, {
   3.615440245282045*^9, 3.6154403257781863`*^9}, 3.618549469052879*^9, {
   3.625739388698906*^9, 3.6257393971229205`*^9}, {3.628144604976634*^9, 
   3.6281446107954445`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{{3.6281445978786216`*^9, 3.6281446226086655`*^9}}]
}, Closed]],

Cell[TextData[{
 StyleBox["It follows the",
  FontColor->GrayLevel[0]],
 StyleBox["\nCriterion.",
  FontColor->RGBColor[0, 0, 1]],
 " The ruled surface rsf[s, t] := xx[t] + s*yy][t], where xx, yy satisfy the \
conditions "
}], "Text",
 CellChangeTimes->{{3.5919695796873407`*^9, 3.5919695872065535`*^9}, {
   3.591969771286877*^9, 3.5919698988795013`*^9}, {3.5919699317799587`*^9, 
   3.5919699517635937`*^9}, 3.592023584436409*^9, {3.592029928671152*^9, 
   3.5920299343651624`*^9}, {3.5920470783513412`*^9, 3.592047080753745*^9}, {
   3.5922183538404703`*^9, 3.592218371311469*^9}, {3.625804454478113*^9, 
   3.6258044586121206`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"ruledcond", "[", 
   RowBox[{"xx", ",", "yy", ",", "t"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"xx", "'"}], ".", 
      RowBox[{"xx", "'"}]}], "\[Equal]", "1"}], ",", " ", 
    RowBox[{
     RowBox[{"yy", ".", "yy"}], " ", "\[Equal]", " ", "1"}], ",", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"xx", "'"}], ".", "yy"}], "\[Equal]", " ", "0"}]}], 
   "}"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.591969981590846*^9, 3.5919701060166655`*^9}, {
   3.592047093514568*^9, 3.5920470950589705`*^9}, 3.6185546260787363`*^9}],

Cell["has the Gaussian curvature", "Text",
 CellChangeTimes->{{3.5920299951116695`*^9, 3.5920300221777163`*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"Krsf", "(", 
    RowBox[{"s", ",", "t"}], ")"}], "\[LongEqual]", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["yy", "\[Prime]",
         MultilineFunction->None], "(", "t", ")"}], ".", 
       RowBox[{
        SuperscriptBox["yy", "\[Prime]",
         MultilineFunction->None], "(", "t", ")"}]}], "-", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["xx", "\[Prime]",
           MultilineFunction->None], "(", "t", ")"}], ".", 
         RowBox[{
          SuperscriptBox["yy", "\[Prime]",
           MultilineFunction->None], "(", "t", ")"}]}], ")"}], "2"]}], 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"s", " ", 
           RowBox[{
            SuperscriptBox["yy", "\[Prime]",
             MultilineFunction->None], "(", "t", ")"}]}], "+", 
          RowBox[{
           SuperscriptBox["xx", "\[Prime]",
            MultilineFunction->None], "(", "t", ")"}]}], ")"}], ".", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"s", " ", 
           RowBox[{
            SuperscriptBox["yy", "\[Prime]",
             MultilineFunction->None], "(", "t", ")"}]}], "+", 
          RowBox[{
           SuperscriptBox["xx", "\[Prime]",
            MultilineFunction->None], "(", "t", ")"}]}], ")"}]}], ")"}], 
      "2"]]}]}], TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.592030032660935*^9, 3.5920301076346664`*^9}, {
   3.592030144840732*^9, 3.5920302698905516`*^9}, 3.592032530350122*^9, 
   3.5920471394254484`*^9}],

Cell["The surface is developable iff ", "Text",
 CellChangeTimes->{{3.591970202222034*^9, 3.591970227213278*^9}, {
  3.5919702655562363`*^9, 3.591970270403514*^9}, {3.5920303196546392`*^9, 
  3.592030324241047*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{"(*", 
   RowBox[{
    RowBox[{")", "                          ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["yy", "\[Prime]",
        MultilineFunction->None], "(", "t", ")"}], ".", 
      RowBox[{
       SuperscriptBox["yy", "\[Prime]",
        MultilineFunction->None], "(", "t", ")"}]}]}], "\[LongEqual]", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["yy", "\[Prime]",
          MultilineFunction->None], "(", "t", ")"}], ".", 
        RowBox[{
         SuperscriptBox["xx", "\[Prime]",
          MultilineFunction->None], "(", "t", ")"}]}], ")"}], "2"], "."}]}]}],
   TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5919703342231636`*^9, 3.591970334456177*^9}, 
   3.5919703812638545`*^9, 3.592047182216324*^9, {3.592047410741125*^9, 
   3.5920474186035385`*^9}, {3.6116464842946773`*^9, 3.611646496306698*^9}}],

Cell[TextData[{
 StyleBox["Proof",
  FontColor->RGBColor[0, 0, 1]],
 ". First we check the formula for the curvature  Krsf[s,t]:"
}], "Text",
 CellChangeTimes->{{3.591970572095769*^9, 3.591970666820187*^9}, {
   3.5919707419874864`*^9, 3.591970840188103*^9}, {3.5919710737404613`*^9, 
   3.5919710961077414`*^9}, {3.591971168891904*^9, 3.591971219956825*^9}, {
   3.5919712500365453`*^9, 3.591971385892316*^9}, {3.5919714409454646`*^9, 
   3.591971459415521*^9}, {3.5919714898212605`*^9, 3.591971502003957*^9}, {
   3.592030403785587*^9, 3.5920304817545238`*^9}, {3.5920305450906353`*^9, 
   3.5920305474462395`*^9}, {3.5920326139818687`*^9, 
   3.5920326269922915`*^9}, {3.592047221075992*^9, 3.5920472221835938`*^9}, {
   3.615442227593127*^9, 3.6154423098988714`*^9}, 3.618553138569324*^9, {
   3.618553195618624*^9, 3.618553225009076*^9}, {3.61855355192325*^9, 
   3.6185535876161127`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"ruledcond", "[", 
    RowBox[{"xx", ",", "yy", ",", "t"}], "]"}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Krsf", "[", 
       RowBox[{"s", ",", "t"}], "]"}], " ", "\[Equal]", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"yy", "'"}], "[", "t", "]"}], ".", 
           RowBox[{
            RowBox[{"yy", "'"}], "[", "t", "]"}]}], " ", "-", 
          RowBox[{
           RowBox[{"(", " ", 
            RowBox[{
             RowBox[{
              RowBox[{"xx", "'"}], "[", "t", "]"}], ".", 
             RowBox[{
              RowBox[{"yy", "'"}], "[", "t", "]"}]}], ")"}], "^", "2"}]}], 
         ")"}]}], "/", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"xx", "'"}], "[", "t", "]"}], " ", "+", 
            RowBox[{"s", "*", 
             RowBox[{
              RowBox[{"yy", "'"}], "[", "t", "]"}]}]}], ")"}], ".", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"xx", "'"}], "[", "t", "]"}], " ", "+", 
            RowBox[{"s", "*", 
             RowBox[{
              RowBox[{"yy", "'"}], "[", "t", "]"}]}]}], ")"}]}], ")"}], "^", 
        "2"}]}]}], ",", 
     RowBox[{"TimeConstraint", " ", "->", " ", "Infinity"}]}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.592030563233467*^9, 3.592030592951519*^9}, {
   3.5920314597202415`*^9, 3.5920314666778536`*^9}, {3.5920315132439356`*^9, 
   3.592031539623582*^9}, 3.592031682098632*^9, {3.592032371151842*^9, 
   3.5920323779846544`*^9}, {3.592032515015295*^9, 3.5920325452325478`*^9}, {
   3.592032592266631*^9, 3.5920325936082335`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.62814759221021*^9}]
}, Closed]],

Cell["\<\
 A surface, in particular a ruled surface, is developable iff its Gaussian \
curvature equals zero, see  Subsection 2.2.3. Thus the ruled surface is \
developable iff condition (*) is valid   \[FilledSquare]\
\>", "Text",
 CellChangeTimes->{{3.5920326603919506`*^9, 3.5920326808747864`*^9}, {
   3.5920327676577387`*^9, 3.5920328681687155`*^9}, {3.5920473610394373`*^9, 
   3.5920473823490753`*^9}, {3.592047433064764*^9, 3.592047457588007*^9}, {
   3.5920477355725155`*^9, 3.5920477384756813`*^9}, 3.6154423422533283`*^9, {
   3.615443194014824*^9, 3.615443196276828*^9}, {3.618553735753973*^9, 
   3.6185537650040245`*^9}, 3.628147613802648*^9}],

Cell[TextData[{
 StyleBox["Corollary.",
  FontColor->RGBColor[0, 0, 1]],
 " The condition (*) can be written in the form"
}], "Text",
 CellChangeTimes->{{3.615443353993105*^9, 3.615443453926881*^9}, 
   3.615443697817709*^9, 3.6154437382841797`*^9, 3.6185539467131433`*^9}],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(*", "*)"}], "                       ", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["xx", "\[Prime]",
         MultilineFunction->None], "(", "t", ")"}], " ", ".", 
       RowBox[{
        SuperscriptBox["xx", "\[Prime]",
         MultilineFunction->None], "(", "t", ")"}]}], ")"}], "*", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["yy", "\[Prime]",
         MultilineFunction->None], "(", "t", ")"}], ".", 
       RowBox[{
        SuperscriptBox["yy", "\[Prime]",
         MultilineFunction->None], "(", "t", ")"}]}], ")"}]}], "\[LongEqual]", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["yy", "\[Prime]",
          MultilineFunction->None], "(", "t", ")"}], ".", 
        RowBox[{
         SuperscriptBox["xx", "\[Prime]",
          MultilineFunction->None], "(", "t", ")"}]}], ")"}], "2"], "."}]}]}],
   TraditionalForm]],
 Evaluatable->False,
 CellChangeTimes->{{3.5919703342231636`*^9, 3.591970334456177*^9}, 
   3.5919703812638545`*^9, 3.592047182216324*^9, {3.592047410741125*^9, 
   3.5920474186035385`*^9}, {3.6116464842946773`*^9, 
   3.611646496306698*^9}}]], "Text",
 CellChangeTimes->{{3.6154435063429728`*^9, 3.615443534313822*^9}, {
  3.615443574858293*^9, 3.6154436171811676`*^9}, {3.6154436514544277`*^9, 
  3.6154436537008314`*^9}, {3.6154437181133447`*^9, 3.6154437181133447`*^9}, {
  3.615443791183873*^9, 3.615443791183873*^9}}],

Cell["A ruled surface", "Text",
 CellChangeTimes->{{3.615444030473133*^9, 3.615444048054364*^9}, {
  3.618554659743596*^9, 3.6185546844384394`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"ruled", "[", 
   RowBox[{"xx", ",", "yy"}], "]"}], "[", 
  RowBox[{"s", ",", "t"}], "]"}]], "Input",
 CellChangeTimes->{{3.615444057507981*^9, 3.6154440603159857`*^9}, {
  3.6154441029508605`*^9, 3.615444114292081*^9}}],

Cell[TextData[{
 " satisfying  the conditions yy(t).yy(t)==1 and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"yy", "(", "t", ")"}], ".", 
    RowBox[{
     SuperscriptBox["xx", "\[Prime]",
      MultilineFunction->None], "(", "t", ")"}]}], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.5919703342231636`*^9, 3.591970334456177*^9}, 
    3.5919703812638545`*^9, 3.592047182216324*^9, {3.592047410741125*^9, 
    3.5920474186035385`*^9}, {3.6116464842946773`*^9, 3.611646496306698*^9}}],
 " == 0 is developable iff condition (**) is  satisfied, t an arbitrary \
parameter.\n",
 StyleBox["Proof.",
  FontColor->RGBColor[0, 0, 1]],
 " Remember: the conditions yy(t).yy(t)==1 and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"yy", "(", "t", ")"}], ".", 
    RowBox[{
     SuperscriptBox["xx", "\[Prime]",
      MultilineFunction->None], "(", "t", ")"}]}], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.5919703342231636`*^9, 3.591970334456177*^9}, 
    3.5919703812638545`*^9, 3.592047182216324*^9, {3.592047410741125*^9, 
    3.5920474186035385`*^9}, {3.6116464842946773`*^9, 3.611646496306698*^9}}],
 " == 0 being assumed in addition to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["xx", "\[Prime]",
      MultilineFunction->None], "(", "t", ")"}], " ", ".", 
    RowBox[{
     SuperscriptBox["xx", "\[Prime]",
      MultilineFunction->None], "(", "t", ")"}]}], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.5919703342231636`*^9, 3.591970334456177*^9}, 
    3.5919703812638545`*^9, 3.592047182216324*^9, {3.592047410741125*^9, 
    3.5920474186035385`*^9}, {3.6116464842946773`*^9, 3.611646496306698*^9}}],
 " == 1 as we calculated the Gaussian curvature; the other three conditions \
in ruledcond follow by deriving those.  If t is the arc length of the curve \
xx[t] condition [**) reduces to (*) and is proved. Now condition (**) is \
invariant under a parameter transformation. Thus the criterion is valid for \
any parametrizations  of xx[t], yy[t].\[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.615444168190175*^9, 3.6154444057473927`*^9}, {
   3.615444442173456*^9, 3.6154444708775067`*^9}, 3.615444591231718*^9, {
   3.6154446991215076`*^9, 3.615444904449068*^9}, {3.6154449408595324`*^9, 
   3.6154449687991815`*^9}, {3.615445015068863*^9, 3.6154450421193104`*^9}, {
   3.618554242692463*^9, 3.6185542543768835`*^9}, {3.618554289866946*^9, 
   3.6185543196317987`*^9}, {3.618554692082453*^9, 3.6185547207709026`*^9}, 
   3.618554780019807*^9, {3.6185548133258657`*^9, 3.618554836772707*^9}, {
   3.6185548901092005`*^9, 3.618554909624835*^9}, {3.6185549522753096`*^9, 
   3.6185550958111615`*^9}}],

Cell["\<\
Now we control our result using the expression of the Gaussian curvature as \
the quotient of two determinants, namely the determinants of the coefficients \
of the second  over the first fundamental form, see Section 3.2. For the \
second fundamental form we obtain\
\>", "Text",
 CellChangeTimes->{{3.592112863023681*^9, 3.5921130169023514`*^9}, {
   3.592113231402728*^9, 3.59211328907603*^9}, 3.6116512417882333`*^9, {
   3.61856207403701*^9, 3.618562085612673*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"ruledcond", "[", 
    RowBox[{"xx", ",", "yy", ",", "t"}], "]"}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"fund2", "[", "rsf", "]"}], "[", 
       RowBox[{"s", ",", " ", "t"}], "]"}], "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"u1", ",", "u2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"u1", ",", "u2"}], "}"}]}], "]"}], ",", 
     RowBox[{"TimeConstraint", "\[Rule]", " ", "Infinity"}]}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.591773629631014*^9, 3.591773714170849*^9}, {
   3.5917737484678106`*^9, 3.5917737560822463`*^9}, {3.591773808723257*^9, 
   3.591773814242573*^9}, {3.591775768244335*^9, 3.5917757816751037`*^9}, {
   3.5917786327132173`*^9, 3.591778658722705*^9}, {3.591788739317565*^9, 
   3.591788759352711*^9}, 3.5917889229130664`*^9, 3.5917890378886423`*^9, {
   3.59178994401447*^9, 3.5917900186787405`*^9}, 3.5917933838012147`*^9, {
   3.591793669191538*^9, 3.5917937009843564`*^9}, {3.5917951837611666`*^9, 
   3.5917952037283087`*^9}, {3.591795236712195*^9, 3.591795257680394*^9}, {
   3.5917967300736103`*^9, 3.59179673880011*^9}, 3.5919455581142216`*^9, 
   3.591945593588684*^9, {3.5919476014838104`*^9, 3.5919476613255157`*^9}, {
   3.5919499489159336`*^9, 3.59195003789849*^9}, 3.5920236628265467`*^9, 
   3.592047774872763*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"u2", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"2", " ", "u1", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              RowBox[{
               RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}]}], " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "2", 
              "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], 
            "-", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "2", 
              "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], 
            "-", 
            RowBox[{
             RowBox[{
              RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "2", 
              "]"}]}]}], ")"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
        ")"}]}], "+", 
      RowBox[{"u2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}],
              " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], 
             " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["x", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], 
               "+", 
               RowBox[{"s", " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["y", "\[Prime]\[Prime]",
                   MultilineFunction->None], "[", "t", "]"}], "[", "1", 
                 "]"}]}]}], ")"}]}], "+", 
            RowBox[{"s", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["y", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], "[", "1", 
                  "]"}]}], " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", "\[Prime]\[Prime]",
                   MultilineFunction->None], "[", "t", "]"}], "[", "2", 
                 "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  SuperscriptBox["y", "\[Prime]",
                   MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}],
                 " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox["x", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], "[", "1", 
                   "]"}], "+", 
                  RowBox[{"s", " ", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox["y", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], "[", "1", 
                    "]"}]}]}], ")"}]}], "-", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], "[", "1", 
                   "]"}], "+", 
                  RowBox[{"s", " ", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox["y", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], "[", "1", 
                    "]"}]}]}], ")"}], " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["y", "\[Prime]\[Prime]",
                   MultilineFunction->None], "[", "t", "]"}], "[", "2", 
                 "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}], 
            "-", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], 
             " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["x", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], 
               "+", 
               RowBox[{"s", " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["y", "\[Prime]\[Prime]",
                   MultilineFunction->None], "[", "t", "]"}], "[", "1", 
                 "]"}]}]}], ")"}]}], "+", 
            RowBox[{"s", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  SuperscriptBox["y", "\[Prime]",
                   MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}],
                 " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", "\[Prime]\[Prime]",
                   MultilineFunction->None], "[", "t", "]"}], "[", "3", 
                 "]"}]}], "-", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  SuperscriptBox["y", "\[Prime]",
                   MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}],
                 " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox["x", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], "[", "1", 
                   "]"}], "+", 
                  RowBox[{"s", " ", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox["y", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], "[", "1", 
                    "]"}]}]}], ")"}]}], "+", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], "[", "1", 
                   "]"}], "+", 
                  RowBox[{"s", " ", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox["y", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], "[", "1", 
                    "]"}]}]}], ")"}], " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["y", "\[Prime]\[Prime]",
                   MultilineFunction->None], "[", "t", "]"}], "[", "3", 
                 "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}],
              " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], 
             " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["x", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], 
               "+", 
               RowBox[{"s", " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["y", "\[Prime]\[Prime]",
                   MultilineFunction->None], "[", "t", "]"}], "[", "2", 
                 "]"}]}]}], ")"}]}], "+", 
            RowBox[{"s", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["y", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], "[", "2", 
                  "]"}]}], " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", "\[Prime]\[Prime]",
                   MultilineFunction->None], "[", "t", "]"}], "[", "3", 
                 "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  SuperscriptBox["y", "\[Prime]",
                   MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}],
                 " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox["x", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], "[", "2", 
                   "]"}], "+", 
                  RowBox[{"s", " ", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox["y", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], "[", "2", 
                    "]"}]}]}], ")"}]}], "-", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], "[", "2", 
                   "]"}], "+", 
                  RowBox[{"s", " ", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox["y", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], "[", "2", 
                    "]"}]}]}], ")"}], " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["y", "\[Prime]\[Prime]",
                   MultilineFunction->None], "[", "t", "]"}], "[", "3", 
                 "]"}]}]}], ")"}]}]}], ")"}]}]}], ")"}]}]}], ")"}]}], ")"}], 
  "/", 
  RowBox[{"(", 
   RowBox[{"\[Sqrt]", 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      RowBox[{
       SuperscriptBox["s", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], "2"],
          "+", 
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], "2"],
          "+", 
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], 
          "2"]}], ")"}]}], "-", 
      RowBox[{"2", " ", "s", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
        ")"}]}]}], ")"}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.6281485263210564`*^9}]
}, Closed]],

Cell["\<\
We calculate the coefficients of the second fundamental form:\
\>", "Text",
 CellChangeTimes->{{3.5920481697323475`*^9, 3.592048223499423*^9}, {
  3.611651325841181*^9, 3.611651328352785*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"ruledcond", "[", 
    RowBox[{"xx", ",", "yy", ",", "t"}], "]"}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"fd2", "[", 
        RowBox[{"1", ",", "1"}], "]"}], "[", "rsf", "]"}], "[", 
      RowBox[{"s", ",", " ", "t"}], "]"}], ",", 
     RowBox[{"TimeConstraint", "\[Rule]", " ", "Infinity"}]}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.591773629631014*^9, 3.591773714170849*^9}, {
   3.5917737484678106`*^9, 3.5917737560822463`*^9}, {3.591773808723257*^9, 
   3.591773814242573*^9}, {3.591775768244335*^9, 3.5917757816751037`*^9}, {
   3.5917786327132173`*^9, 3.591778658722705*^9}, {3.591788739317565*^9, 
   3.591788759352711*^9}, 3.5917889229130664`*^9, 3.5917890378886423`*^9, {
   3.59178994401447*^9, 3.5917900186787405`*^9}, 3.5917933838012147`*^9, {
   3.591793669191538*^9, 3.5917937009843564`*^9}, {3.5917951837611666`*^9, 
   3.5917952037283087`*^9}, {3.591795236712195*^9, 3.591795257680394*^9}, {
   3.5917967300736103`*^9, 3.59179673880011*^9}, 3.5919455581142216`*^9, 
   3.591945593588684*^9, {3.5919476014838104`*^9, 3.5919476613255157`*^9}, {
   3.5919499489159336`*^9, 3.59195003789849*^9}, {3.59195236660818*^9, 
   3.5919524075738525`*^9}, {3.5919527647832794`*^9, 
   3.5919527686988864`*^9}, {3.591972928029319*^9, 3.59197293456573*^9}, {
   3.5919729663585863`*^9, 3.5919729992434435`*^9}, {3.591973035076707*^9, 
   3.591973039694315*^9}, 3.5920236728261642`*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.628149788639681*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fa", "[", 
    RowBox[{"1", ",", " ", "1"}], "]"}], "[", 
   RowBox[{"s_", ",", " ", "t_"}], "]"}], " ", ":=", " ", "0"}]], "Input",
 CellChangeTimes->{
  3.592048429597211*^9, {3.5921117118976593`*^9, 3.5921117176696696`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"ruledcond", "[", 
    RowBox[{"xx", ",", "yy", ",", "t"}], "]"}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"fd2", "[", 
        RowBox[{"1", ",", "2"}], "]"}], "[", "rsf", "]"}], "[", 
      RowBox[{"s", ",", " ", "t"}], "]"}], ",", 
     RowBox[{"TimeConstraint", "\[Rule]", " ", "Infinity"}]}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.591773629631014*^9, 3.591773714170849*^9}, {
   3.5917737484678106`*^9, 3.5917737560822463`*^9}, {3.591773808723257*^9, 
   3.591773814242573*^9}, {3.591775768244335*^9, 3.5917757816751037`*^9}, {
   3.5917786327132173`*^9, 3.591778658722705*^9}, {3.591788739317565*^9, 
   3.591788759352711*^9}, 3.5917889229130664`*^9, 3.5917890378886423`*^9, {
   3.59178994401447*^9, 3.5917900186787405`*^9}, 3.5917933838012147`*^9, {
   3.591793669191538*^9, 3.5917937009843564`*^9}, {3.5917951837611666`*^9, 
   3.5917952037283087`*^9}, {3.591795236712195*^9, 3.591795257680394*^9}, {
   3.5917967300736103`*^9, 3.59179673880011*^9}, 3.5919455581142216`*^9, 
   3.591945593588684*^9, {3.5919476014838104`*^9, 3.5919476613255157`*^9}, {
   3.5919499489159336`*^9, 3.59195003789849*^9}, {3.59195236660818*^9, 
   3.5919524075738525`*^9}, {3.591972358191518*^9, 3.591972401185193*^9}, {
   3.5919725408522387`*^9, 3.5919725457818475`*^9}, 3.592023669799759*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]",
        MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}]}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["y", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["y", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}]}], 
      ")"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["y", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], "-", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["y", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}]}], 
      ")"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], "-", 
       RowBox[{
        RowBox[{
         RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}]}], 
      ")"}], " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["y", "\[Prime]",
        MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], ")"}],
   "/", 
  RowBox[{"(", 
   RowBox[{"\[Sqrt]", 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      RowBox[{
       SuperscriptBox["s", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], "2"],
          "+", 
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], "2"],
          "+", 
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], 
          "2"]}], ")"}]}], "-", 
      RowBox[{"2", " ", "s", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
        ")"}]}]}], ")"}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.62814981630153*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fa", "[", 
    RowBox[{"1", ",", " ", "2"}], "]"}], "[", 
   RowBox[{"s_", ",", " ", "t_"}], "]"}], " ", ":=", " ", "\n", "  ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", "t", 
        "]"}], "[", "3", "]"}], "*", "\n", "     ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{
            RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}]}], ")"}], "*", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
           "]"}], "[", "\n", "        ", "1", "]"}]}], " ", "+", " ", 
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], "*", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "\n",
            "         ", "t", "]"}], "[", "2", "]"}]}]}], ")"}]}], " ", "+", 
     " ", 
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "*", "\n", "     ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "2", "]"}], "*", "\n", "       ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
           "]"}], "[", "1", "]"}]}], " ", "-", " ", "\n", "      ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "1", "]"}], "*", "\n", "       ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
           "]"}], "[", "2", "]"}]}]}], ")"}]}], " ", "+", " ", "\n", "    ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "*", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "1", "]"}]}], " ", "-", " ", "\n", "      ", 
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], "*", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "2", "]"}]}]}], ")"}], "*", "\n", "     ", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
        "]"}], "[", "3", "]"}]}]}], ")"}], "/", "\n", "   ", 
   RowBox[{"Sqrt", "[", 
    RowBox[{"1", " ", "+", " ", 
     RowBox[{
      RowBox[{"s", "^", "2"}], "*", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
           "]"}], "[", "\n", "         ", "1", "]"}], "^", "2"}], " ", "+", 
        " ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
           "]"}], "[", "\n", "         ", "2", "]"}], "^", "2"}], " ", "+", 
        " ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
           "]"}], "[", "\n", "         ", "3", "]"}], "^", "2"}]}], ")"}]}], 
     " ", "-", " ", "\n", "     ", 
     RowBox[{"2", "*", "s", "*", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], "*", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "\n", "         ", "1", "]"}]}], " ", "+", " ", 
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "*", "\n", 
         "        ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "2", "]"}]}], " ", "+", " ", "\n", "       ", 
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "*", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "\n", "         ", "3", "]"}]}]}], ")"}]}]}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6185565731181564`*^9, 3.618556640307474*^9}, {
  3.618556692723566*^9, 3.61855670039878*^9}, {3.618556736512843*^9, 
  3.618556750131667*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fa", "[", 
    RowBox[{"2", ",", "1"}], "]"}], "[", 
   RowBox[{"s_", ",", "t_"}], "]"}], " ", ":=", 
  RowBox[{
   RowBox[{"fa", "[", 
    RowBox[{"1", ",", "2"}], "]"}], "[", 
   RowBox[{"s", ",", " ", "t"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5919524988028126`*^9, 3.5919525572249155`*^9}, {
   3.5919526620726995`*^9, 3.591952664225503*^9}, {3.591973412129369*^9, 
   3.5919734181665792`*^9}, 3.592023671406562*^9, {3.592111790209797*^9, 
   3.5921117943438044`*^9}, 3.6185562000123014`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"ruledcond", "[", 
    RowBox[{"xx", ",", "yy", ",", "t"}], "]"}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"fd2", "[", 
        RowBox[{"2", ",", "2"}], "]"}], "[", "rsf", "]"}], "[", 
      RowBox[{"s", ",", " ", "t"}], "]"}], ",", 
     RowBox[{"TimeConstraint", "\[Rule]", " ", "Infinity"}]}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.591773629631014*^9, 3.591773714170849*^9}, {
   3.5917737484678106`*^9, 3.5917737560822463`*^9}, {3.591773808723257*^9, 
   3.591773814242573*^9}, {3.591775768244335*^9, 3.5917757816751037`*^9}, {
   3.5917786327132173`*^9, 3.591778658722705*^9}, {3.591788739317565*^9, 
   3.591788759352711*^9}, 3.5917889229130664`*^9, 3.5917890378886423`*^9, {
   3.59178994401447*^9, 3.5917900186787405`*^9}, 3.5917933838012147`*^9, {
   3.591793669191538*^9, 3.5917937009843564`*^9}, {3.5917951837611666`*^9, 
   3.5917952037283087`*^9}, {3.591795236712195*^9, 3.591795257680394*^9}, {
   3.5917967300736103`*^9, 3.59179673880011*^9}, 3.5919455581142216`*^9, 
   3.591945593588684*^9, {3.5919476014838104`*^9, 3.5919476613255157`*^9}, {
   3.5919499489159336`*^9, 3.59195003789849*^9}, {3.59195236660818*^9, 
   3.5919524075738525`*^9}, {3.5919527647832794`*^9, 
   3.5919527686988864`*^9}, {3.5919532628765545`*^9, 3.591953272751372*^9}, {
   3.591973168425741*^9, 3.591973191108181*^9}, {3.591973224133439*^9, 
   3.5919732446630745`*^9}, 3.592023674042967*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], "+", 
          RowBox[{"s", " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}]}], 
         ")"}]}], "+", 
       RowBox[{"s", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], 
           " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], 
          "+", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], 
             "+", 
             RowBox[{"s", " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["y", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], "[", "1", 
               "]"}]}]}], ")"}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], 
             "+", 
             RowBox[{"s", " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["y", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], "[", "1", 
               "]"}]}]}], ")"}], " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}]}], 
         ")"}]}]}], ")"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}], "-", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], "+", 
          RowBox[{"s", " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}]}], 
         ")"}]}], "+", 
       RowBox[{"s", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}], 
          "-", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], 
             "+", 
             RowBox[{"s", " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["y", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], "[", "1", 
               "]"}]}]}], ")"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], 
             "+", 
             RowBox[{"s", " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["y", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], "[", "1", 
               "]"}]}]}], ")"}], " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
         ")"}]}]}], ")"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], "+", 
          RowBox[{"s", " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}]}], 
         ")"}]}], "+", 
       RowBox[{"s", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], 
           " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}], 
          "+", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], 
             "+", 
             RowBox[{"s", " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["y", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], "[", "2", 
               "]"}]}]}], ")"}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], 
             "+", 
             RowBox[{"s", " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["y", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], "[", "2", 
               "]"}]}]}], ")"}], " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
         ")"}]}]}], ")"}]}]}], ")"}], "/", 
  RowBox[{"(", 
   RowBox[{"\[Sqrt]", 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      RowBox[{
       SuperscriptBox["s", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], "2"],
          "+", 
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], "2"],
          "+", 
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["y", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], 
          "2"]}], ")"}]}], "-", 
      RowBox[{"2", " ", "s", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
        ")"}]}]}], ")"}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.628150012225675*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fa", "[", 
    RowBox[{"2", ",", " ", "2"}], "]"}], "[", 
   RowBox[{"s_", ",", " ", "t_"}], "]"}], " ", ":=", " ", "\n", "  ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "*", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", 
             "t", "]"}], "[", "1", "]"}]}], ")"}], "*", "\n", "       ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "2", "]"}]}], " ", "+", " ", "\n", "      ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "2", "]"}], "*", "\n", "       ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
             "t", "]"}], "[", "1", "]"}], " ", "+", " ", "\n", "        ", 
           RowBox[{"s", "*", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "1", "]"}]}]}], ")"}]}], " ", "+", " ", "\n", 
        "      ", 
        RowBox[{"s", "*", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"-", 
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", 
                "t", "]"}], "[", "1", "]"}]}], ")"}], "*", "\n", "         ", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
              "t", "]"}], "[", "2", "]"}]}], " ", "+", " ", "\n", "        ", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "2", "]"}], "*", "\n", "         ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
                "t", "]"}], "[", "1", "]"}], " ", "+", " ", "\n", 
              "          ", 
              RowBox[{"s", "*", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"Derivative", "[", "2", "]"}], "[", "y", "]"}], "[",
                  "t", "]"}], "[", "1", "]"}]}]}], ")"}]}], " ", "-", " ", 
           "\n", "        ", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", 
                "t", "]"}], "[", "1", "]"}], " ", "+", " ", "\n", 
              "          ", 
              RowBox[{"s", "*", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[",
                  "t", "]"}], "[", "1", "]"}]}]}], ")"}], "*", "\n", 
            "         ", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "2", "]"}]}]}], ")"}]}]}], ")"}]}], " ", "+", 
     " ", "\n", "    ", 
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "*", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "1", "]"}], "*", "\n", "       ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "3", "]"}]}], " ", "-", " ", "\n", "      ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "3", "]"}], "*", "\n", "       ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
             "t", "]"}], "[", "1", "]"}], " ", "+", " ", "\n", "        ", 
           RowBox[{"s", "*", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "1", "]"}]}]}], ")"}]}], " ", "+", " ", "\n", 
        "      ", 
        RowBox[{"s", "*", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "1", "]"}], "*", "\n", "         ", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
              "t", "]"}], "[", "3", "]"}]}], " ", "-", " ", "\n", "        ", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "3", "]"}], "*", "\n", "         ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
                "t", "]"}], "[", "1", "]"}], " ", "+", " ", "\n", 
              "          ", 
              RowBox[{"s", "*", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"Derivative", "[", "2", "]"}], "[", "y", "]"}], "[",
                  "t", "]"}], "[", "1", "]"}]}]}], ")"}]}], " ", "+", " ", 
           "\n", "        ", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", 
                "t", "]"}], "[", "1", "]"}], " ", "+", " ", "\n", 
              "          ", 
              RowBox[{"s", "*", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[",
                  "t", "]"}], "[", "1", "]"}]}]}], ")"}], "*", "\n", 
            "         ", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "3", "]"}]}]}], ")"}]}]}], ")"}]}], " ", "+", 
     " ", "\n", "    ", 
     RowBox[{
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], "*", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", 
             "t", "]"}], "[", "\n", "         ", "2", "]"}]}], ")"}], "*", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "3", "]"}]}], " ", "+", " ", "\n", "      ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "3", "]"}], "*", "\n", "       ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
             "t", "]"}], "[", "2", "]"}], " ", "+", " ", "\n", "        ", 
           RowBox[{"s", "*", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "2", "]"}]}]}], ")"}]}], " ", "+", " ", "\n", 
        "      ", 
        RowBox[{"s", "*", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"-", 
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", 
                "t", "]"}], "[", "2", "]"}]}], ")"}], "*", "\n", "         ", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
              "t", "]"}], "[", "3", "]"}]}], " ", "+", " ", "\n", "        ", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "3", "]"}], "*", "\n", "         ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", 
                "t", "]"}], "[", "2", "]"}], " ", "+", " ", "\n", 
              "          ", 
              RowBox[{"s", "*", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"Derivative", "[", "2", "]"}], "[", "y", "]"}], "[",
                  "t", "]"}], "[", "2", "]"}]}]}], ")"}]}], " ", "-", " ", 
           "\n", "        ", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"Derivative", "[", "1", "]"}], "[", "x", "]"}], "[", 
                "t", "]"}], "[", "2", "]"}], " ", "+", " ", "\n", 
              "          ", 
              RowBox[{"s", "*", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[",
                  "t", "]"}], "[", "2", "]"}]}]}], ")"}], "*", "\n", 
            "         ", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Derivative", "[", "2", "]"}], "[", "y", "]"}], "[", 
              "t", "]"}], "[", "3", "]"}]}]}], ")"}]}]}], ")"}]}]}], ")"}], 
   "/", "\n", "   ", 
   RowBox[{"Sqrt", "[", 
    RowBox[{"1", " ", "+", " ", 
     RowBox[{
      RowBox[{"s", "^", "2"}], "*", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
           "]"}], "[", "\n", "         ", "1", "]"}], "^", "2"}], " ", "+", 
        " ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
           "]"}], "[", "\n", "         ", "2", "]"}], "^", "2"}], " ", "+", 
        " ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
           "]"}], "[", "\n", "         ", "3", "]"}], "^", "2"}]}], ")"}]}], 
     " ", "-", " ", "\n", "     ", 
     RowBox[{"2", "*", "s", "*", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], "*", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "\n", "         ", "1", "]"}]}], " ", "+", " ", 
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "*", "\n", 
         "        ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "2", "]"}]}], " ", "+", " ", "\n", "       ", 
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "*", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "\n", "         ", "3", "]"}]}]}], ")"}]}]}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5919732754107285`*^9, 3.5919733017747755`*^9}, {
   3.5920485906614237`*^9, 3.5920486099175253`*^9}, 3.592048655164113*^9, {
   3.5921118349662757`*^9, 3.5921118422982883`*^9}, {3.61486185979994*^9, 
   3.6148618599871407`*^9}, {3.6149177915773606`*^9, 3.6149177917645607`*^9}}],

Cell["\<\
The determinant Det[fa[i,j][s,t]] of the second fundamental form is\
\>", "Text",
 CellChangeTimes->{{3.5921133369837136`*^9, 3.5921133636441603`*^9}, {
  3.611652241578389*^9, 3.6116522849620657`*^9}, {3.6116523173789225`*^9, 
  3.6116523304049454`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"ruledcond", "[", 
    RowBox[{"xx", ",", "yy", ",", "t"}], "]"}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{"Det", "[", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"fa", "[", 
          RowBox[{"i", ",", "j"}], "]"}], "[", 
         RowBox[{"s", ",", " ", "t"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}], ",", 
     RowBox[{"TimeConstraint", "\[Rule]", " ", "Infinity"}]}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.591953703343328*^9, 3.591953781561866*^9}, 
   3.592023675415769*^9, 3.6148618600027404`*^9, 3.614863501858824*^9}],

Cell[BoxData[
 RowBox[{"-", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{
       RowBox[{
        SuperscriptBox["y", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], "2"], 
     "+", 
     SuperscriptBox[
      RowBox[{
       RowBox[{
        SuperscriptBox["y", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], "2"], 
     "+", 
     SuperscriptBox[
      RowBox[{
       RowBox[{
        SuperscriptBox["y", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], "2"], 
     "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "2"]}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], "2"]}], 
     "-", 
     RowBox[{"2", " ", 
      RowBox[{
       RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], "+", 
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
       ")"}]}], "-", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], "+", 
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
       ")"}], "2"]}], ")"}], "/", 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     RowBox[{
      SuperscriptBox["s", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["y", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["y", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["y", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], 
         "2"]}], ")"}]}], "-", 
     RowBox[{"2", " ", "s", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], "+", 
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], "+", 
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
       ")"}]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.6281515143671207`*^9, 3.6281516515655637`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"detfa", "[", 
   RowBox[{"s_", ",", " ", "t_"}], "]"}], " ", ":=", " ", "\n", "  ", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
         "]"}], "[", "1", "]"}], "^", "2"}], " ", "+", " ", "\n", "     ", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
         "]"}], "[", "2", "]"}], "^", "2"}], " ", "+", " ", "\n", "     ", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
         "]"}], "[", "3", "]"}], "^", "2"}], " ", "+", " ", "\n", "     ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], " ", "+", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "^", "2"}], " ", 
         "+", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "^", "2"}]}], ")"}],
        "*", "\n", "      ", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
          "]"}], "[", "1", "]"}], "^", "2"}]}], " ", "-", " ", "\n", "     ", 
      RowBox[{"2", "*", 
       RowBox[{
        RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], "*", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
         "]"}], "[", "1", "]"}], "*", "\n", "      ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "*", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t",
             "]"}], "[", "\n", "         ", "2", "]"}]}], " ", "+", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "*", "\n", 
          "        ", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t",
             "]"}], "[", "3", "]"}]}]}], ")"}]}], " ", "-", " ", "\n", 
      "     ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "*", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t",
             "]"}], "[", "\n", "         ", "2", "]"}]}], " ", "+", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "*", "\n", 
          "        ", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t",
             "]"}], "[", "3", "]"}]}]}], ")"}], "^", "2"}]}], ")"}]}], "/", 
   "\n", "   ", 
   RowBox[{"(", 
    RowBox[{"1", " ", "+", " ", 
     RowBox[{
      RowBox[{"s", "^", "2"}], "*", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
           "]"}], "[", "1", "]"}], "^", "\n", "       ", "2"}], " ", "+", " ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
           "]"}], "[", "2", "]"}], "^", "2"}], " ", "+", " ", "\n", "      ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "1", "]"}], "[", "y", "]"}], "[", "t", 
           "]"}], "[", "3", "]"}], "^", "2"}]}], ")"}]}], " ", "-", " ", "\n",
      "    ", 
     RowBox[{"2", "*", "s", "*", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], "*", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "\n", "        ", "1", "]"}]}], " ", "+", " ", 
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "*", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "\n",
            "         ", "t", "]"}], "[", "2", "]"}]}], " ", "+", " ", 
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "*", "\n", "       ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Derivative", "[", "2", "]"}], "[", "x", "]"}], "[", "t", 
           "]"}], "[", "3", "]"}]}]}], ")"}]}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.591954021537087*^9, 3.591954073875179*^9}, {
  3.5920489980117226`*^9, 3.592049027986437*^9}, {3.59211216177125*^9, 
  3.592112191146101*^9}, {3.6148618600027404`*^9, 3.6148618600651407`*^9}, {
  3.614917791780161*^9, 3.6149177918425612`*^9}}],

Cell["The quotient of the determinants is", "Text",
 CellChangeTimes->{{3.592113393424613*^9, 3.5921134297102766`*^9}}],

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"ruledcond", "[", 
    RowBox[{"xx", ",", "yy", ",", "t"}], "]"}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"detfa", "[", 
       RowBox[{"s", ",", "t"}], "]"}], "/", 
      RowBox[{"Det", "[", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"rgT", "[", 
            RowBox[{"i", ",", "j"}], "]"}], "[", "rsf", "]"}], "[", 
          RowBox[{"s", ",", "t"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "2"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}]}], ",", 
     RowBox[{"TimeConstraint", "\[Rule]", " ", "Infinity"}]}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5919676261801605`*^9, 3.591967669766637*^9}, {
   3.591967706021101*^9, 3.5919677109039097`*^9}, {3.591967770386814*^9, 
   3.5919678540029607`*^9}, {3.5920491350405607`*^9, 
   3.5920492534233313`*^9}, {3.592049573543641*^9, 3.5920496692571154`*^9}, {
   3.592049704033105*^9, 3.5920497102974634`*^9}, 3.592050945314102*^9, {
   3.5920511461695905`*^9, 3.592051226626192*^9}, 3.592055181657555*^9, 
   3.6148618600807405`*^9, 3.614863501874424*^9, 3.6176282733734493`*^9}],

Cell["Comparing with the expression Krsf[s,t] defined above gives", "Text",
 CellChangeTimes->{{3.5921134527515173`*^9, 3.5921135289264507`*^9}}],

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"ruledcond", "[", 
    RowBox[{"xx", ",", "yy", ",", "t"}], "]"}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{"%", "-", 
      RowBox[{"Krsf", "[", 
       RowBox[{"s", ",", "t"}], "]"}]}], ",", 
     RowBox[{"TimeConstraint", "\[Rule]", " ", "Infinity"}]}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5919676261801605`*^9, 3.591967669766637*^9}, {
   3.591967706021101*^9, 3.5919677109039097`*^9}, {3.591967770386814*^9, 
   3.5919678540029607`*^9}, {3.5920491350405607`*^9, 
   3.5920492534233313`*^9}, {3.592049573543641*^9, 3.5920496692571154`*^9}, {
   3.592049704033105*^9, 3.5920497102974634`*^9}, 3.592050945314102*^9, {
   3.5920511461695905`*^9, 3.592051226626192*^9}, 3.592055181657555*^9, 
   3.592055268737536*^9, {3.5920553221935935`*^9, 3.592055377169738*^9}, 
   3.592058731034568*^9, {3.592112568931965*^9, 3.592112574875575*^9}, 
   3.5921126374472847`*^9}],

Cell["OK. We yet calculate the mean curvature", "Text",
 CellChangeTimes->{{3.5921135377560663`*^9, 3.592113576865335*^9}, {
  3.6185605679688683`*^9, 3.6185605756163054`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"meancurvature", "[", "rsf", "]"}], "[", 
     RowBox[{"s", ",", "t"}], "]"}], ",", 
    RowBox[{"ruledcond", "[", 
     RowBox[{"xx", ",", "yy", ",", "t"}], "]"}], ",", 
    RowBox[{"TimeConstraint", "\[Rule]", " ", "Infinity"}]}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5919676261801605`*^9, 3.591967669766637*^9}, {
   3.591967706021101*^9, 3.5919677109039097`*^9}, {3.591967770386814*^9, 
   3.5919678540029607`*^9}, {3.5920491350405607`*^9, 
   3.5920492534233313`*^9}, {3.592049573543641*^9, 3.5920496692571154`*^9}, {
   3.592049704033105*^9, 3.5920497102974634`*^9}, 3.592050945314102*^9, {
   3.5920511461695905`*^9, 3.592051226626192*^9}, 3.592055181657555*^9, 
   3.592055268737536*^9, {3.5920553221935935`*^9, 3.592055377169738*^9}, 
   3.592058731034568*^9, {3.592112568931965*^9, 3.592112574875575*^9}, 
   3.5921126374472847`*^9, {3.592113589844558*^9, 3.592113639499445*^9}, 
   3.61486186009634*^9, 3.614863136599783*^9, {3.628160648918606*^9, 
   3.628160689917478*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Simplify", "::", "time"}], "MessageName"], ":", 
  " ", "\<\"Time spent on a transformation exceeded \[NoBreak]\\!\\(300.`\\)\
\[NoBreak] seconds, and the transformation was aborted. Increasing the value \
of TimeConstraint option may improve the result of simplification. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/Simplify/time\\\", \
ButtonNote -> \\\"Simplify::time\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.628161053188717*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2543.2999030999999376945197582244873046875`9.425997487997954", ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], 
          " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], 
            "+", 
            RowBox[{"s", " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "1", 
              "]"}]}]}], ")"}]}], "+", 
         RowBox[{"s", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              RowBox[{
               RowBox[{
                SuperscriptBox["y", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}],
              " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], 
             " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["x", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], 
               "+", 
               RowBox[{"s", " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["y", "\[Prime]\[Prime]",
                   MultilineFunction->None], "[", "t", "]"}], "[", "1", 
                 "]"}]}]}], ")"}]}], "-", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["x", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], 
               "+", 
               RowBox[{"s", " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["y", "\[Prime]",
                   MultilineFunction->None], "[", "t", "]"}], "[", "1", 
                 "]"}]}]}], ")"}], " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "2", 
              "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}], 
         "-", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], 
            "+", 
            RowBox[{"s", " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "1", 
              "]"}]}]}], ")"}]}], "+", 
         RowBox[{"s", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}], 
            "-", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], 
             " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["x", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], 
               "+", 
               RowBox[{"s", " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["y", "\[Prime]\[Prime]",
                   MultilineFunction->None], "[", "t", "]"}], "[", "1", 
                 "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["x", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], 
               "+", 
               RowBox[{"s", " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["y", "\[Prime]",
                   MultilineFunction->None], "[", "t", "]"}], "[", "1", 
                 "]"}]}]}], ")"}], " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "3", 
              "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], 
          " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], 
            "+", 
            RowBox[{"s", " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "2", 
              "]"}]}]}], ")"}]}], "+", 
         RowBox[{"s", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              RowBox[{
               RowBox[{
                SuperscriptBox["y", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}],
              " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], 
             " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["x", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], 
               "+", 
               RowBox[{"s", " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["y", "\[Prime]\[Prime]",
                   MultilineFunction->None], "[", "t", "]"}], "[", "2", 
                 "]"}]}]}], ")"}]}], "-", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["x", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], 
               "+", 
               RowBox[{"s", " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["y", "\[Prime]",
                   MultilineFunction->None], "[", "t", "]"}], "[", "2", 
                 "]"}]}]}], ")"}], " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "3", 
              "]"}]}]}], ")"}]}]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"2", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{
          SuperscriptBox["s", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], 
             "2"], "+", 
            SuperscriptBox[
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], 
             "2"], "+", 
            SuperscriptBox[
             RowBox[{
              RowBox[{
               SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], 
             "2"]}], ")"}]}], "-", 
         RowBox[{"2", " ", "s", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "3", 
              "]"}]}]}], ")"}]}]}], ")"}], 
       RowBox[{"3", "/", "2"}]]}], ")"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.628163267536619*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"ruledcond", "[", 
    RowBox[{"xx", ",", "yy", ",", "t"}], "]"}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"rsfc", "[", 
          RowBox[{"1", ",", "1"}], "]"}], "[", 
         RowBox[{"s", ",", "t"}], "]"}], "+", 
        RowBox[{
         RowBox[{"rsfc", "[", 
          RowBox[{"2", ",", "2"}], "]"}], "[", 
         RowBox[{"s", ",", "t"}], "]"}]}], ")"}], "/", "2"}], ",", 
     RowBox[{"TimeConstraint", "\[Rule]", " ", "Infinity"}]}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5919676261801605`*^9, 3.591967669766637*^9}, {
   3.591967706021101*^9, 3.5919677109039097`*^9}, {3.591967770386814*^9, 
   3.5919678540029607`*^9}, {3.5920491350405607`*^9, 
   3.5920492534233313`*^9}, {3.592049573543641*^9, 3.5920496692571154`*^9}, {
   3.592049704033105*^9, 3.5920497102974634`*^9}, 3.592050945314102*^9, {
   3.5920511461695905`*^9, 3.592051226626192*^9}, 3.592055181657555*^9, 
   3.592055268737536*^9, {3.5920553221935935`*^9, 3.592055377169738*^9}, 
   3.592058731034568*^9, {3.592112568931965*^9, 3.592112574875575*^9}, 
   3.5921126374472847`*^9, {3.592113589844558*^9, 3.592113639499445*^9}, {
   3.5921139773804383`*^9, 3.592114013931303*^9}, {3.5921140833046246`*^9, 
   3.592114101431856*^9}, 3.5921143285994554`*^9, {3.5921143809843473`*^9, 
   3.592114405835191*^9}, 3.6116650552848954`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], "+", 
          RowBox[{"s", " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}]}], 
         ")"}]}], "+", 
       RowBox[{"s", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], 
           " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], 
          "+", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], 
             "+", 
             RowBox[{"s", " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["y", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], "[", "1", 
               "]"}]}]}], ")"}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], 
             "+", 
             RowBox[{"s", " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["y", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], "[", "1", 
               "]"}]}]}], ")"}], " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}]}], 
         ")"}]}]}], ")"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}], "-", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], "+", 
          RowBox[{"s", " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}]}], 
         ")"}]}], "+", 
       RowBox[{"s", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}], 
          "-", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], 
             "+", 
             RowBox[{"s", " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["y", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], "[", "1", 
               "]"}]}]}], ")"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], 
             "+", 
             RowBox[{"s", " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["y", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], "[", "1", 
               "]"}]}]}], ")"}], " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
         ")"}]}]}], ")"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], "+", 
          RowBox[{"s", " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}]}], 
         ")"}]}], "+", 
       RowBox[{"s", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], 
           " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}], 
          "+", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], 
             "+", 
             RowBox[{"s", " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["y", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], "[", "2", 
               "]"}]}]}], ")"}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], 
             "+", 
             RowBox[{"s", " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["y", "\[Prime]",
                 MultilineFunction->None], "[", "t", "]"}], "[", "2", 
               "]"}]}]}], ")"}], " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
         ")"}]}]}], ")"}]}]}], ")"}], "/", 
  RowBox[{"(", 
   RowBox[{"2", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "+", 
       RowBox[{
        SuperscriptBox["s", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], 
           "2"], "+", 
          SuperscriptBox[
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], 
           "2"], "+", 
          SuperscriptBox[
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], 
           "2"]}], ")"}]}], "-", 
       RowBox[{"2", " ", "s", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], 
          "+", 
          RowBox[{
           RowBox[{
            RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], 
          "+", 
          RowBox[{
           RowBox[{
            RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
         ")"}]}]}], ")"}], 
     RowBox[{"3", "/", "2"}]]}], ")"}]}]], "Output",
 CellChangeTimes->{3.6281636681713243`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"ruledcond", "[", 
    RowBox[{"xx", ",", "yy", ",", "t"}], "]"}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{"%", "-", 
     RowBox[{"%%", "[", 
      RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6185612597444353`*^9, 3.6185613193158426`*^9}, 
   3.628164622736604*^9, 3.6281647920391026`*^9, 3.6281648369545813`*^9, {
   3.6281649802910337`*^9, 3.628165010511287*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6281649612716007`*^9}]
}, Closed]],

Cell["OK.", "Text",
 CellChangeTimes->{{3.592114541290229*^9, 3.592114545346236*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["3.8. The Normal Congruences", "Subsection",
 CellChangeTimes->{{3.591688363105586*^9, 3.5916883848832245`*^9}, 
   3.5917714021446085`*^9, 3.5985964668405457`*^9, 3.615451802305699*^9, 
   3.615559274768277*^9, 3.6280851857685623`*^9, 3.628176387948323*^9, 
   3.629013005046713*^9}],

Cell[CellGroupData[{

Cell["3.8.1. Normal Congruence of a Regular Curve", "Subsubsection",
 CellChangeTimes->{{3.592115389572919*^9, 3.592115426638584*^9}, 
   3.5921354646825233`*^9, 3.5922182112163124`*^9, 3.5942701628864923`*^9, 
   3.598596137570768*^9, 3.6116673526853304`*^9, 3.6119986168238125`*^9, {
   3.6155592477178297`*^9, 3.615559249355833*^9}, 3.6161383195804777`*^9, 
   3.618564015369048*^9, 3.6186374644950294`*^9, 3.6280852651103015`*^9, 
   3.6282292445934086`*^9, {3.6282293236387477`*^9, 3.6282293292235575`*^9}, 
   3.6282293736836357`*^9, {3.6282294187833147`*^9, 3.6282294761602154`*^9}},
 CellTags->"helix"],

Cell[TextData[{
 "The family of all normals to a given curve forms a 2-dimensional family of \
lines: ",
 StyleBox["The normal congruence of the curve",
  FontColor->RGBColor[0, 0, 1]],
 ". Let s be the arc length of a general curve xx[s] in the Euclidean \
3-space. "
}], "Text",
 CellChangeTimes->{
  3.6282296190564666`*^9, {3.6282299000909605`*^9, 3.6282299640354724`*^9}, {
   3.628230007684349*^9, 3.6282300308659897`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"xx", "[", "s_", "]"}], " ", ":=", " ", 
  RowBox[{"vec", "[", 
   RowBox[{"x", "[", "s", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.62823004226961*^9, 3.6282301137957354`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"xx", "[", "s", "]"}]], "Input",
 CellChangeTimes->{{3.628230120300947*^9, 3.628230123873353*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "[", "s", "]"}], "[", "1", "]"}], ",", 
   RowBox[{
    RowBox[{"x", "[", "s", "]"}], "[", "2", "]"}], ",", 
   RowBox[{
    RowBox[{"x", "[", "s", "]"}], "[", "3", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6282301250277553`*^9, 3.628399958594123*^9}]
}, Closed]],

Cell["We assume", "Text",
 CellChangeTimes->{{3.6282302200475225`*^9, 3.6282302457251673`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"assu", "[", "xx", "]"}], "[", "s", "]"}], " "}]], "Input",
 CellChangeTimes->{{3.6282302540711823`*^9, 3.6282302674716053`*^9}, 
   3.628399978749358*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], "2"], "+", 
    SuperscriptBox[
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], "2"], "+", 
    SuperscriptBox[
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], "2"]}], 
   "\[Equal]", "1"}], "&&", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}]}]}], 
   "\[Equal]", "0"}], "&&", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}], " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", 
        TagBox[
         RowBox[{"(", "3", ")"}],
         Derivative],
        MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}], "-", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", 
        TagBox[
         RowBox[{"(", "3", ")"}],
         Derivative],
        MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}]}], "-", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", 
        TagBox[
         RowBox[{"(", "3", ")"}],
         Derivative],
        MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}]}]}], 
   "\[Equal]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], "2"], "+", 
    SuperscriptBox[
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], "2"], "+", 
    SuperscriptBox[
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], "2"]}]}], 
  "&&", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}], " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", 
        TagBox[
         RowBox[{"(", "4", ")"}],
         Derivative],
        MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}], "-", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", 
        TagBox[
         RowBox[{"(", "4", ")"}],
         Derivative],
        MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}]}], "-", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", 
        TagBox[
         RowBox[{"(", "4", ")"}],
         Derivative],
        MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}]}]}], 
   "\[Equal]", 
   RowBox[{"3", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["x", 
          TagBox[
           RowBox[{"(", "3", ")"}],
           Derivative],
          MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}], "+", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["x", 
          TagBox[
           RowBox[{"(", "3", ")"}],
           Derivative],
          MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}]}], "+", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["x", 
          TagBox[
           RowBox[{"(", "3", ")"}],
           Derivative],
          MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}]}]}], 
     ")"}]}]}]}]], "Output",
 CellChangeTimes->{3.62823027566162*^9, 3.6282520775824013`*^9, 
  3.6283999812453623`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "assu"}]], "Input",
 CellChangeTimes->{{3.628230160658218*^9, 3.6282301799710517`*^9}}],

Cell[BoxData[
 StyleBox["\<\"assu[c][s] is a set of assumptions in the first four \
derivatives of the \\nparameter representation c[s] of a smooth curve in the \
n-dimensional Euclidean space \\ngiving True if s is the Euclidean arc length \
of the curve.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.628230181281454*^9},
 CellTags->"Info3628233781-2217871"]
}, Closed]],

Cell[TextData[{
 " A general parameter representation of the ",
 StyleBox["normal congruence",
  FontColor->RGBColor[0, 0, 1]],
 "  of the curve xx[s] is"
}], "Text",
 CellChangeTimes->{{3.628230407809452*^9, 3.6282304976812096`*^9}, {
  3.628230763240476*^9, 3.628230770603689*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"normalcongruence", "[", "x_", "]"}], "[", 
   RowBox[{"a_", ",", "s_", ",", "t_"}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"x", "[", "s", "]"}], " ", "+", " ", 
   RowBox[{"t", "*", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"frenet", "[", "x", "]"}], "[", "s", "]"}], "[", 
        RowBox[{"[", "2", "]"}], "]"}], "*", 
       RowBox[{"Cos", "[", "a", "]"}]}], "+", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"frenet", "[", "x", "]"}], "[", "s", "]"}], "[", 
        RowBox[{"[", "3", "]"}], "]"}], "*", 
       RowBox[{"Sin", "[", "a", "]"}]}]}], ")"}]}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6282305071348267`*^9, 3.628230645195069*^9}, {
  3.628230675849123*^9, 3.628230714568391*^9}}],

Cell[TextData[{
 "Here t is the line parameter and a is the angle between the normal and the \
binormale at the same point of the curve.\n\nAs an application we can give  a \
parameter representation for the ",
 StyleBox["tube of radius r ",
  FontColor->RGBColor[0, 0, 1]],
 "of the curve x[s]"
}], "Text",
 CellChangeTimes->{{3.6282323211528125`*^9, 3.6282324109777703`*^9}, {
  3.628250775620305*^9, 3.6282508493540354`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "tube"}]], "Input",
 CellChangeTimes->{{3.6282320050494576`*^9, 3.628232007249061*^9}, 
   3.6282322482070847`*^9}],

Cell[BoxData[
 StyleBox["\<\"tube[x,a,b,r,opt] plots the tube of the curve x (Enter only \
\\nthe name, not the parameter!)  with tube radius r for the parameter s, \
\\na<s<b, with options opt of ParametricPlot3D.\"\>", "MSG"]], "Print", \
"PrintUsage",
 CellChangeTimes->{3.6282322519198914`*^9},
 CellTags->"Info3628235851-2217871"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Context", "[", "tube", "]"}]], "Input",
 CellChangeTimes->{{3.6282319533977666`*^9, 3.6282319788570113`*^9}}],

Cell[BoxData["\<\"euvec`\"\>"], "Output",
 CellChangeTimes->{{3.6282319618061814`*^9, 3.6282319799178133`*^9}}]
}, Closed]],

Cell["A parameter representation of the tube is", "Text",
 CellChangeTimes->{{3.628400068168715*^9, 3.628400082785941*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tubepr", "[", 
    RowBox[{"x_", ",", "r_"}], "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"normalcongruence", "[", "x", "]"}], "[", 
   RowBox[{"u", ",", "v", ",", "r"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6282317535458155`*^9, 3.628231854462393*^9}, {
   3.628231918765706*^9, 3.628231926284919*^9}, {3.6282322056190095`*^9, 
   3.628232206711012*^9}, 3.6282328058924637`*^9}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"Evaluate", "[", 
    RowBox[{
     RowBox[{"tubepr", "[", 
      RowBox[{
       RowBox[{"helix", "[", 
        RowBox[{"1", ",", "1"}], "]"}], ",", "0.5"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"6", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6282325317219825`*^9, 3.628232644775381*^9}, {
  3.628232735770341*^9, 3.6282327467059603`*^9}, {3.6282328547985497`*^9, 
  3.628232896965424*^9}}],

Cell["The same picture can be obtained directly using tube:", "Text",
 CellChangeTimes->{{3.6282506795237355`*^9, 3.6282507070773845`*^9}}],

Cell[BoxData[
 RowBox[{"tube", "[", 
  RowBox[{
   RowBox[{"helix", "[", 
    RowBox[{"1", ",", "1"}], "]"}], ",", "0", ",", 
   RowBox[{"6", "*", "Pi"}], ",", "0.5"}], "]"}]], "Input",
 CellChangeTimes->{{3.6282329207554655`*^9, 3.628232939023098*^9}, {
  3.6282329695835514`*^9, 3.628233007320018*^9}}],

Cell["\<\
Our aim now is to answer the following question: Choose a normal at each \
point of the curve fixing 
a = a[s].  For which functions a = a[s]  the ruled surface formed by the \
chosen normals is developable?  We will apply the criterion of Subsection 3.7 \
with xx[s] and\
\>", "Text",
 CellChangeTimes->{{3.628250887457303*^9, 3.6282509321709814`*^9}, {
   3.6282509730470533`*^9, 3.628251000538302*^9}, {3.628251066394018*^9, 
   3.628251250685543*^9}, {3.6282512808599963`*^9, 3.6282513758605638`*^9}, 
   3.6282516328798175`*^9, {3.628400123018411*^9, 3.6284001464808526`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"yy", "[", "s_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"frenet", "[", "xx", "]"}], "[", "s", "]"}], "[", 
     RowBox[{"[", "2", "]"}], "]"}], "*", 
    RowBox[{"Cos", "[", 
     RowBox[{"a", "[", "s", "]"}], "]"}]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"frenet", "[", "xx", "]"}], "[", "s", "]"}], "[", 
     RowBox[{"[", "3", "]"}], "]"}], "*", 
    RowBox[{"Sin", "[", 
     RowBox[{"a", "[", "s", "]"}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.6282513797957706`*^9, 3.628251400372207*^9}, {
  3.6282514540599017`*^9, 3.62825150394119*^9}}],

Cell["We calculate", "Text",
 CellChangeTimes->{{3.628251737427602*^9, 3.628251742856411*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"yy", "[", "s", "]"}], ",", "s"}], "]"}]], "Input",
 CellChangeTimes->{{3.628251747130819*^9, 3.628251761564844*^9}}],

Cell[BoxData[
 InterpretationBox[
  TagBox[
   PanelBox[GridBox[{
      {
       StyleBox[
        StyleBox[
         DynamicBox[ToBoxes[
           FEPrivate`FrontEndResource["FEStrings", "sizeExplanation"], 
           StandardForm],
          ImageSizeCache->{900., {7., 26.}}],
         StripOnInput->False,
         DynamicUpdating->True], "Panel",
        StripOnInput->False,
        Background->None]},
      {
       ItemBox[
        TagBox[
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{"\[LeftSkeleton]", "11", "\[RightSkeleton]"}], "+", 
            FractionBox[
             RowBox[{
              RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], " ", 
              RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}], 
             RowBox[{
              RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], " ", 
              RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}]]}], ",", 
           RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], ",", 
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{
                   RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                   "]"}], "2"], "+", 
                 SuperscriptBox[
                  RowBox[{
                   RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                   "]"}], "2"], "+", 
                 SuperscriptBox[
                  RowBox[{
                   RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                   "]"}], "2"]}], ")"}], " ", 
               SqrtBox[
                FractionBox[
                 RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                 RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]]]}]]}], 
            "-", 
            FractionBox[
             RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "2"], 
              " ", 
              SqrtBox[
               RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]]}]], "+", 
            RowBox[{"\[LeftSkeleton]", "8", "\[RightSkeleton]"}], "+", 
            FractionBox[
             RowBox[{
              RowBox[{"Sin", "[", 
               RowBox[{"a", "[", "s", "]"}], "]"}], " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"-", 
                 FractionBox[
                  RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                  SqrtBox[
                   RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]]]}], 
                "+", 
                RowBox[{"\[LeftSkeleton]", "10", "\[RightSkeleton]"}]}], 
               ")"}]}], 
             RowBox[{
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], ")"}], 
               RowBox[{"3", "/", "2"}]], " ", 
              SqrtBox[
               FractionBox[
                RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]]]}]]}]}],
           "}"}],
         Short[#, 5]& ],
        Background->GrayLevel[1],
        BaseStyle->{Deployed -> False},
        Frame->True,
        FrameStyle->GrayLevel[0, 0.2],
        StripOnInput->False]},
      {
       RowBox[{
        ButtonBox[
         StyleBox[
          StyleBox[
           DynamicBox[ToBoxes[
             FEPrivate`FrontEndResource["FEStrings", "sizeShowLess"], 
             StandardForm],
            ImageSizeCache->{144., {1., 26.}}],
           StripOnInput->False,
           DynamicUpdating->True], "Panel",
          StripOnInput->False,
          Background->None],
         Appearance->Automatic,
         ButtonFunction:>OutputSizeLimit`ButtonFunction[
          Identity, 75, 24766985315704358400, 5/2],
         Enabled->True,
         Evaluator->Automatic,
         Method->"Queued"], "\[ThinSpace]", 
        ButtonBox[
         StyleBox[
          StyleBox[
           DynamicBox[ToBoxes[
             FEPrivate`FrontEndResource["FEStrings", "sizeShowMore"], 
             StandardForm],
            ImageSizeCache->{165., {1., 26.}}],
           StripOnInput->False,
           DynamicUpdating->True], "Panel",
          StripOnInput->False,
          Background->None],
         Appearance->Automatic,
         ButtonFunction:>OutputSizeLimit`ButtonFunction[
          Identity, 75, 24766985315704358400, 5 2],
         Enabled->True,
         Evaluator->Automatic,
         Method->"Queued"], "\[ThinSpace]", 
        ButtonBox[
         StyleBox[
          StyleBox[
           DynamicBox[ToBoxes[
             FEPrivate`FrontEndResource["FEStrings", "sizeShowAll"], 
             StandardForm],
            ImageSizeCache->{122., {1., 26.}}],
           StripOnInput->False,
           DynamicUpdating->True], "Panel",
          StripOnInput->False,
          Background->None],
         Appearance->Automatic,
         ButtonFunction:>OutputSizeLimit`ButtonFunction[
          Identity, 75, 24766985315704358400, Infinity],
         Enabled->True,
         Evaluator->Automatic,
         Method->"Queued"], "\[ThinSpace]", 
        ButtonBox[
         StyleBox[
          StyleBox[
           DynamicBox[ToBoxes[
             FEPrivate`FrontEndResource["FEStrings", "sizeChangeLimit"], 
             StandardForm],
            ImageSizeCache->{216., {1., 26.}}],
           StripOnInput->False,
           DynamicUpdating->True], "Panel",
          StripOnInput->False,
          Background->None],
         Appearance->Automatic,
         ButtonFunction:>FrontEndExecute[{
            FrontEnd`SetOptions[
            FrontEnd`$FrontEnd, 
             FrontEnd`PreferencesSettings -> {"Page" -> "Evaluation"}], 
            FrontEnd`FrontEndToken["PreferencesDialog"]}],
         Evaluator->None,
         Method->"Preemptive"]}]}
     },
     DefaultBaseStyle->"Column",
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxDividers->{
      "Columns" -> {{False}}, "ColumnsIndexed" -> {}, "Rows" -> {{False}}, 
       "RowsIndexed" -> {}},
     GridBoxItemSize->{
      "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[1.2]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    DefaultBaseStyle->{},
    FrameMargins->5],
   Deploy,
   DefaultBaseStyle->"Deploy"],
  Out[75]]], "Output",
 CellChangeTimes->{3.6282517656676517`*^9, 3.628252110313259*^9, 
  3.628252237219883*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", 
         RowBox[{"a", "[", "s", "]"}], "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"3", " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], " ", 
           SuperscriptBox[
            RowBox[{
             RowBox[{
              SuperscriptBox["x", 
               TagBox[
                RowBox[{"(", "3", ")"}],
                Derivative],
               MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], 
            "2"]}], "+", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"3", " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "3", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], 
              " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "3", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}]}],
              "+", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
              " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "4", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}],
              "-", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], 
              " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "4", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "2", 
               "]"}]}]}], ")"}]}], "+", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"3", " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "3", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], 
              " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "3", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}]}],
              "+", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
              " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "4", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}],
              "-", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], 
              " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "4", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "3", 
               "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
       RowBox[{
        RowBox[{"Sin", "[", 
         RowBox[{"a", "[", "s", "]"}], "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
              " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 RowBox[{"-", "3"}], " ", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", 
                    TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                    MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                  "]"}], " ", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", 
                    TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                    MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                  "]"}]}], "+", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                  "]"}], " ", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", 
                    TagBox[
                    RowBox[{"(", "4", ")"}],
                    Derivative],
                    MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                  "]"}]}]}], ")"}]}], "+", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
              " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"3", " ", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", 
                    TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                    MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                  "]"}], " ", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", 
                    TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                  "]"}]}], "-", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                  "]"}], " ", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", 
                    TagBox[
                    RowBox[{"(", "4", ")"}],
                    Derivative],
                    MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                  "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
            "2"], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"3", " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "3", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
              " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "3", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}]}],
              "-", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
              " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "4", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "2", 
               "]"}]}]}], ")"}]}], "+", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
            "2"], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"-", "3"}], " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "3", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
              " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "3", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}]}],
              "+", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
              " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "4", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "3", 
               "]"}]}]}], ")"}]}], "+", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"-", "3"}], " ", 
              SuperscriptBox[
               RowBox[{
                RowBox[{
                 SuperscriptBox["x", 
                  TagBox[
                   RowBox[{"(", "3", ")"}],
                   Derivative],
                  MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
               "2"]}], "+", 
             RowBox[{"3", " ", 
              SuperscriptBox[
               RowBox[{
                RowBox[{
                 SuperscriptBox["x", 
                  TagBox[
                   RowBox[{"(", "3", ")"}],
                   Derivative],
                  MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
               "2"]}], "+", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
              " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "4", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}]}],
              "-", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
              " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "4", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "3", 
               "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox["a", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "3"}], " ", 
           RowBox[{"Cos", "[", 
            RowBox[{"a", "[", "s", "]"}], "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
              " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}]}],
              "-", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
              " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "[", "3", 
               "]"}]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], 
              " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "3", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}],
              "+", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
              " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "3", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}]}],
              "+", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
              " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "3", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "3", 
               "]"}]}]}], ")"}]}], "+", 
          RowBox[{
           RowBox[{"Sin", "[", 
            RowBox[{"a", "[", "s", "]"}], "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"-", "3"}], " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
              " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], 
              " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "3", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}]}],
              "-", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                RowBox[{
                 SuperscriptBox["x", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
               "2"], " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "4", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}],
              "-", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"-", "1"}], "+", 
                SuperscriptBox[
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                  "]"}], "2"]}], ")"}], " ", 
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "4", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}],
              "+", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
              " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 RowBox[{"-", "3"}], " ", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                  "]"}], " ", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", 
                    TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                    MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                  "]"}]}], "+", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                  "]"}], " ", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", 
                    TagBox[
                    RowBox[{"(", "4", ")"}],
                    Derivative],
                    MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                  "]"}]}]}], ")"}]}], "+", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], 
              " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"3", " ", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                  "]"}], " ", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", 
                    TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                    MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                  "]"}]}], "+", 
                RowBox[{"3", " ", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                  "]"}], " ", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", 
                    TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                  "]"}]}], "+", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                  "]"}], " ", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", 
                    TagBox[
                    RowBox[{"(", "4", ")"}],
                    Derivative],
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                  "]"}]}]}], ")"}]}]}], ")"}]}]}], ")"}]}]}], ")"}], "/", 
     RowBox[{"(", 
      RowBox[{"3", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", 
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}], 
           " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["x", 
              TagBox[
               RowBox[{"(", "3", ")"}],
               Derivative],
              MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}], 
          "-", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["x", 
              TagBox[
               RowBox[{"(", "3", ")"}],
               Derivative],
              MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}]}], 
          "-", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["x", 
              TagBox[
               RowBox[{"(", "3", ")"}],
               Derivative],
              MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}]}]}], 
         ")"}], 
        RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"Cos", "[", 
        RowBox[{"a", "[", "s", "]"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "3"}], " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "3", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "3", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "4", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "1", 
              "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "3"}], " ", 
             SuperscriptBox[
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "3", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
              "2"]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "4", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "2", 
              "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "3"}], " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "3", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "3", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "4", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "3", 
              "]"}]}]}], ")"}]}]}], ")"}]}], "-", 
      RowBox[{
       RowBox[{"Sin", "[", 
        RowBox[{"a", "[", "s", "]"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"3", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              RowBox[{
               RowBox[{
                SuperscriptBox["x", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
              "2"]}]}], ")"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", 
             TagBox[
              RowBox[{"(", "3", ")"}],
              Derivative],
             MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", 
             TagBox[
              RowBox[{"(", "3", ")"}],
              Derivative],
             MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", 
             TagBox[
              RowBox[{"(", "4", ")"}],
              Derivative],
             MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}], 
         "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
           "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"3", " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "3", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "3", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}]}], 
            "-", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "4", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "1", 
              "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"3", " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "3", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "3", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "4", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}], 
            "-", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "4", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "2", 
              "]"}]}]}], ")"}]}], "-", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
           "2"], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", 
             TagBox[
              RowBox[{"(", "4", ")"}],
              Derivative],
             MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
             " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "3"}], " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", 
                   TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                   MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}],
                 " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", 
                   TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                   MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                 "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", "\[Prime]\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}],
                 " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", 
                   TagBox[
                    RowBox[{"(", "4", ")"}],
                    Derivative],
                   MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                 "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
             " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"3", " ", 
                SuperscriptBox[
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", 
                    TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                    MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                  "]"}], "2"]}], "-", 
               RowBox[{"3", " ", 
                SuperscriptBox[
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", 
                    TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                  "]"}], "2"]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", "\[Prime]\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}],
                 " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", 
                   TagBox[
                    RowBox[{"(", "4", ")"}],
                    Derivative],
                   MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                 "]"}]}]}], ")"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox["a", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"3", " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"a", "[", "s", "]"}], "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "3", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "3", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "3", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "3", 
              "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{"Cos", "[", 
           RowBox[{"a", "[", "s", "]"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"3", " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
             " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"-", "1"}], "+", 
                   SuperscriptBox[
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                    "]"}], "2"], "+", 
                   SuperscriptBox[
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                    "]"}], "2"]}], ")"}]}], " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", 
                   TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                   MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                 "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}],
                 " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                    "]"}], " ", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox["x", 
                    TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                    MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                    "]"}]}], "+", 
                  RowBox[{"2", " ", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                    "]"}], " ", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox["x", 
                    TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                    "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
             " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "3"}], " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}],
                 " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", "\[Prime]\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}],
                 " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", 
                   TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                   MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                 "]"}]}], "-", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                  "]"}], "2"], " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", 
                   TagBox[
                    RowBox[{"(", "4", ")"}],
                    Derivative],
                   MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                 "]"}]}], "-", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", "1"}], "+", 
                  SuperscriptBox[
                   RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                    "]"}], "2"]}], ")"}], " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", 
                   TagBox[
                    RowBox[{"(", "4", ")"}],
                    Derivative],
                   MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                 "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}],
                 " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"-", "3"}], " ", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                    "]"}], " ", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox["x", 
                    TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                    MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                    "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                    "]"}], " ", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox["x", 
                    TagBox[
                    RowBox[{"(", "4", ")"}],
                    Derivative],
                    MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                    "]"}]}]}], ")"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}],
                 " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"3", " ", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                    "]"}], " ", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox["x", 
                    TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                    MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                    "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                    "]"}], " ", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox["x", 
                    TagBox[
                    RowBox[{"(", "4", ")"}],
                    Derivative],
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                    "]"}]}]}], ")"}]}]}], ")"}]}]}], ")"}]}]}], ")"}]}]}], 
     ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"3", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}], 
          " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", 
             TagBox[
              RowBox[{"(", "3", ")"}],
              Derivative],
             MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}], 
         "-", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", 
             TagBox[
              RowBox[{"(", "3", ")"}],
              Derivative],
             MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}]}], 
         "-", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", 
             TagBox[
              RowBox[{"(", "3", ")"}],
              Derivative],
             MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}]}]}], 
        ")"}], 
       RowBox[{"3", "/", "2"}]]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"3", " ", 
       RowBox[{
        SuperscriptBox["a", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"Sin", "[", 
           RowBox[{"a", "[", "s", "]"}], "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "3", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "3", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "3", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "3", 
              "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{"Cos", "[", 
           RowBox[{"a", "[", "s", "]"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"-", "1"}], "+", 
                   SuperscriptBox[
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                    "]"}], "2"]}], ")"}]}], " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", "\[Prime]\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                 "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}],
                 " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}],
                 " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", "\[Prime]\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                 "]"}]}]}], ")"}], " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "3", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
             " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", "1"}], "+", 
                  SuperscriptBox[
                   RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                    "]"}], "2"]}], ")"}], " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", 
                   TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                   MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                 "]"}]}], "-", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}],
                 " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}],
                 " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", 
                   TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                   MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                 "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
             " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                  "]"}]}], " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", 
                   TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                   MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                 "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", "\[Prime]\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}],
                 " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", 
                   TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                   MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                 "]"}]}]}], ")"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"Cos", "[", 
        RowBox[{"a", "[", "s", "]"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "3"}], " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "3", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "3", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "4", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "1", 
              "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "3"}], " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "3", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "3", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "4", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "2", 
              "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "3"}], " ", 
             SuperscriptBox[
              RowBox[{
               RowBox[{
                SuperscriptBox["x", 
                 TagBox[
                  RowBox[{"(", "3", ")"}],
                  Derivative],
                 MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
              "2"]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "4", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "3", 
              "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"Sin", "[", 
        RowBox[{"a", "[", "s", "]"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"3", " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", 
             TagBox[
              RowBox[{"(", "3", ")"}],
              Derivative],
             MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", "1"}], "+", 
               RowBox[{"2", " ", 
                SuperscriptBox[
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                  "]"}], "2"]}], "+", 
               SuperscriptBox[
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}],
                 "2"]}], ")"}], " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "3", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
             " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", 
                TagBox[
                 RowBox[{"(", "3", ")"}],
                 Derivative],
                MultilineFunction->None], "[", "s", "]"}], "[", "3", 
              "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", 
             TagBox[
              RowBox[{"(", "4", ")"}],
              Derivative],
             MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}], 
         "-", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", 
             TagBox[
              RowBox[{"(", "4", ")"}],
              Derivative],
             MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
             " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["x", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                  "]"}]}], " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", 
                   TagBox[
                    RowBox[{"(", "4", ")"}],
                    Derivative],
                   MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                 "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", "\[Prime]\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}],
                 " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", 
                   TagBox[
                    RowBox[{"(", "4", ")"}],
                    Derivative],
                   MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                 "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
             " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", "\[Prime]\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}],
                 " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", 
                   TagBox[
                    RowBox[{"(", "4", ")"}],
                    Derivative],
                   MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                 "]"}]}], "-", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", "\[Prime]\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}],
                 " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", 
                   TagBox[
                    RowBox[{"(", "4", ")"}],
                    Derivative],
                   MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                 "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"3", " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], 
             " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", 
                   TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                   MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}],
                 "2"], "-", 
               SuperscriptBox[
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", 
                   TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                   MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}],
                 "2"]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], 
             " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "3"}], " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", 
                   TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                   MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}],
                 " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", 
                   TagBox[
                    RowBox[{"(", "3", ")"}],
                    Derivative],
                   MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                 "]"}]}], "-", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", "\[Prime]\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}],
                 " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", 
                   TagBox[
                    RowBox[{"(", "4", ")"}],
                    Derivative],
                   MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                 "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", "\[Prime]\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}],
                 " ", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox["x", 
                   TagBox[
                    RowBox[{"(", "4", ")"}],
                    Derivative],
                   MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                 "]"}]}]}], ")"}]}]}], ")"}]}]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"3", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            RowBox[{
             SuperscriptBox["x", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}], 
          " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", 
             TagBox[
              RowBox[{"(", "3", ")"}],
              Derivative],
             MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}], 
         "-", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", 
             TagBox[
              RowBox[{"(", "3", ")"}],
              Derivative],
             MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}]}], 
         "-", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", 
             TagBox[
              RowBox[{"(", "3", ")"}],
              Derivative],
             MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}]}]}], 
        ")"}], 
       RowBox[{"3", "/", "2"}]]}], ")"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.628253441454808*^9}]
}, Closed]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " wasn\[CloseCurlyQuote]t very helpful in simplifying this expression and \
its  Dot square  using assu[xx][s] or the corresponding rules"
}], "Text",
 CellChangeTimes->{{3.6284023224350743`*^9, 3.6284023325750923`*^9}, {
  3.628402364757949*^9, 3.6284024706665354`*^9}, {3.628774378701568*^9, 
  3.6287743814003725`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"xxrules", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], "2"], 
       "+", 
       SuperscriptBox[
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], "2"], 
       "+", 
       SuperscriptBox[
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], "2"]}],
       "->", "1"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}]}]}], "->",
       "0"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", 
           TagBox[
            RowBox[{"(", "3", ")"}],
            Derivative],
           MultilineFunction->None], "[", "s", "]"}], "[", "1", "]"}]}], "-", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", 
           TagBox[
            RowBox[{"(", "3", ")"}],
            Derivative],
           MultilineFunction->None], "[", "s", "]"}], "[", "2", "]"}]}], "-", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}], " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", 
           TagBox[
            RowBox[{"(", "3", ")"}],
            Derivative],
           MultilineFunction->None], "[", "s", "]"}], "[", "3", "]"}]}]}], 
      "\[Rule]", 
      RowBox[{
       RowBox[{"xxk", "[", "s", "]"}], "^", "2"}]}]}], "}"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6283436227912064`*^9, 3.6283436864861183`*^9}, {
   3.628343724300585*^9, 3.6283437409146137`*^9}, {3.628343834998379*^9, 
   3.6283438836860647`*^9}, {3.628343926071339*^9, 3.6283439722942204`*^9}, 
   3.628402533831046*^9}],

Cell["\<\
Therefore we solve the problem directly using the Frenet formulas of curve \
theory. We calculate\
\>", "Text",
 CellChangeTimes->{{3.6284026131571856`*^9, 3.6284026911573224`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"yy", "'"}], "[", "s", "]"}], " ", "==", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"frenet", "[", "xx", "]"}], "[", "s", "]"}], "[", 
       RowBox[{"[", "2", "]"}], "]"}], ",", "s"}], "]"}], "*", 
    RowBox[{"Cos", "[", 
     RowBox[{"a", "[", "s", "]"}], "]"}]}], "+", 
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"frenet", "[", "xx", "]"}], "[", "s", "]"}], "[", 
       RowBox[{"[", "3", "]"}], "]"}], ",", "s"}], "]"}], "*", 
    RowBox[{"Sin", "[", 
     RowBox[{"a", "[", "s", "]"}], "]"}]}], "-", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"frenet", "[", "xx", "]"}], "[", "s", "]"}], "[", "2", "]"}], 
    "*", 
    RowBox[{"Sin", "[", 
     RowBox[{"a", "[", "s", "]"}], "]"}], "*", 
    RowBox[{
     RowBox[{"a", "'"}], "[", "s", "]"}]}], " ", "+", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"frenet", "[", "xx", "]"}], "[", "s", "]"}], "[", "3", "]"}], 
    "*", 
    RowBox[{"Cos", "[", 
     RowBox[{"a", "[", "s", "]"}], "]"}], "*", 
    RowBox[{
     RowBox[{"a", "'"}], "[", "s", "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.628402744587416*^9, 3.6284029951862564`*^9}, {
  3.6284030756511974`*^9, 3.6284030880844193`*^9}}],

Cell[BoxData[
 InterpretationBox[
  TagBox[
   FrameBox[GridBox[{
      {
       ItemBox[
        TagBox[
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"Sin", "[", 
                   RowBox[{"a", "[", "s", "]"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["a", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                    "]"}], "2"], " ", 
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                    "]"}]}], "-", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                    "]"}], " ", 
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                    "]"}], " ", 
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                    "]"}]}], "+", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                    "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                    "]"}], " ", 
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                    "]"}]}], "-", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                    "]"}], " ", 
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                    "]"}]}]}], ")"}]}]}], ")"}]}], ")"}], "/", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"(", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                    "]"}], "2"], "+", 
                    SuperscriptBox[
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                    "]"}], "2"], "+", 
                    SuperscriptBox[
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                    "]"}], "2"]}], ")"}], " ", 
                  SqrtBox[
                   FractionBox[
                    RowBox[{
                    TemplateBox[{"6"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                    "+", 
                    RowBox[{
                    SuperscriptBox[
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                    "2"], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    SuperscriptBox[
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{"\" \[CenterEllipsis]\"", #, 
                    "\"\[CenterEllipsis] \""}], Background -> GrayLevel[0.75],
                     BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                    "2"], "+", 
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )]}],
                     ")"}]}]}], 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "1", 
                    "]"}], "2"], "+", 
                    SuperscriptBox[
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "2", 
                    "]"}], "2"], "+", 
                    SuperscriptBox[
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                    "]"}], "2"]}]]]}], ")"}]}], ")"}]}], "-", 
             FractionBox[
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{"a", "[", "s", "]"}], "]"}], " ", 
               RowBox[{"(", 
                TemplateBox[{"1"},
                 "OutputSizeLimit`Skeleton",
                 DisplayFunction->(FrameBox[
                   
                   RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                   Background -> GrayLevel[0.75], 
                   BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                   Baseline, ContentPadding -> False, FrameMargins -> 1, 
                   FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                ")"}], " ", 
               RowBox[{"(", 
                TemplateBox[{"1"},
                 "OutputSizeLimit`Skeleton",
                 DisplayFunction->(FrameBox[
                   
                   RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                   Background -> GrayLevel[0.75], 
                   BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                   Baseline, ContentPadding -> False, FrameMargins -> 1, 
                   FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                ")"}]}], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 TemplateBox[{"1"},
                  "OutputSizeLimit`Skeleton",
                  DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                 ")"}], "2"], " ", 
               SqrtBox[
                FractionBox[
                 TemplateBox[{"1"},
                  "OutputSizeLimit`Skeleton",
                  DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                 TemplateBox[{"1"},
                  "OutputSizeLimit`Skeleton",
                  DisplayFunction->(FrameBox[
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 
                    7]& )]]]}]], "+", 
             TemplateBox[{"8"},
              "OutputSizeLimit`Skeleton",
              DisplayFunction->(FrameBox[
                
                RowBox[{
                 "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                Background -> GrayLevel[0.75], 
                BaseStyle -> {
                 "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                  ShowStringCharacters -> False}, BaselinePosition -> 
                Baseline, ContentPadding -> False, FrameMargins -> 1, 
                FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], "+", 
             FractionBox[
              RowBox[{
               RowBox[{"Sin", "[", 
                RowBox[{"a", "[", "s", "]"}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"-", 
                  FractionBox[
                   RowBox[{
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["x", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "[", "3", 
                    "]"}], " ", 
                    SqrtBox[
                    FractionBox[
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )]]],
                     " ", 
                    RowBox[{"(", 
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                    ")"}]}], 
                   SqrtBox[
                    FractionBox[
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 
                    7]& )]]]]}], "+", 
                 TemplateBox[{"10"},
                  "OutputSizeLimit`Skeleton",
                  DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )]}],
                 ")"}]}], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                    "[", "1", "]"}], "2"], "+", 
                  SuperscriptBox[
                   TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                   "2"], "+", 
                  SuperscriptBox[
                   TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                   "2"]}], ")"}], 
                RowBox[{"3", "/", "2"}]], " ", 
               SqrtBox[
                FractionBox[
                 TemplateBox[{"1"},
                  "OutputSizeLimit`Skeleton",
                  DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                 RowBox[{
                  SuperscriptBox[
                   TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                   "2"], "+", 
                  SuperscriptBox[
                   TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                   "2"], "+", 
                  SuperscriptBox[
                   TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton",
                    DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                   "2"]}]]]}]]}], ",", 
            TemplateBox[{"1"},
             "OutputSizeLimit`Skeleton",
             DisplayFunction->(FrameBox[
               
               RowBox[{
                "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
               Background -> GrayLevel[0.75], 
               BaseStyle -> {
                "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                 ShowStringCharacters -> False}, BaselinePosition -> Baseline,
                ContentPadding -> False, FrameMargins -> 1, FrameStyle -> 
               GrayLevel[0.75], RoundingRadius -> 7]& )], ",", 
            TemplateBox[{"1"},
             "OutputSizeLimit`Skeleton",
             DisplayFunction->(FrameBox[
               
               RowBox[{
                "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
               Background -> GrayLevel[0.75], 
               BaseStyle -> {
                "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                 ShowStringCharacters -> False}, BaselinePosition -> Baseline,
                ContentPadding -> False, FrameMargins -> 1, FrameStyle -> 
               GrayLevel[0.75], RoundingRadius -> 7]& )]}], "}"}], "\[Equal]", 
          RowBox[{"{", 
           TemplateBox[{"1"},
            "OutputSizeLimit`Skeleton",
            DisplayFunction->(FrameBox[
              RowBox[{"\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}],
               Background -> GrayLevel[0.75], 
              BaseStyle -> {
               "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                ShowStringCharacters -> False}, BaselinePosition -> Baseline, 
              ContentPadding -> False, FrameMargins -> 1, FrameStyle -> 
              GrayLevel[0.75], RoundingRadius -> 7]& )], "}"}]}],
         Short[#, 20]& ],
        BaseStyle->{Deployed -> False},
        StripOnInput->False]},
      {GridBox[{
         {
          TagBox[
           TooltipBox[
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource[
                "FEStrings", "sizeBriefExplanation"], StandardForm],
               ImageSizeCache->{183., {7., 26.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarCategory",
             StripOnInput->False],
            StyleBox[
             DynamicBox[
              ToBoxes[
               FEPrivate`FrontEndResource["FEStrings", "sizeExplanation"], 
               StandardForm]], DynamicUpdating -> True, StripOnInput -> 
             False]],
           Annotation[#, 
            Style[
             Dynamic[
              FEPrivate`FrontEndResource["FEStrings", "sizeExplanation"]], 
             DynamicUpdating -> True], "Tooltip"]& ], 
          ButtonBox[
           PaneSelectorBox[{False->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeShowLess"], 
                StandardForm],
               ImageSizeCache->{167., {1., 26.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControl",
             StripOnInput->False], True->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeShowLess"], 
                StandardForm]],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControlActive",
             StripOnInput->False]}, Dynamic[
             CurrentValue["MouseOver"]],
            Alignment->Center,
            FrameMargins->0,
            ImageSize->{Automatic, 25}],
           Appearance->None,
           
           ButtonFunction:>OutputSizeLimit`ButtonFunction[
            Identity, 72, 26400271531097435981, 20/2],
           Enabled->True,
           Evaluator->Automatic,
           Method->"Queued"], 
          ButtonBox[
           PaneSelectorBox[{False->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeShowMore"], 
                StandardForm],
               ImageSizeCache->{189., {1., 26.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControl",
             StripOnInput->False], True->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeShowMore"], 
                StandardForm],
               ImageSizeCache->{189., {1., 26.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControlActive",
             StripOnInput->False]}, Dynamic[
             CurrentValue["MouseOver"]],
            Alignment->Center,
            FrameMargins->0,
            ImageSize->{Automatic, 25}],
           Appearance->None,
           
           ButtonFunction:>OutputSizeLimit`ButtonFunction[
            Identity, 72, 26400271531097435981, 20 2],
           Enabled->True,
           Evaluator->Automatic,
           Method->"Queued"], 
          ButtonBox[
           PaneSelectorBox[{False->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeShowAll"], 
                StandardForm],
               ImageSizeCache->{141., {1., 26.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControl",
             StripOnInput->False], True->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeShowAll"], 
                StandardForm],
               ImageSizeCache->{141., {1., 26.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControlActive",
             StripOnInput->False]}, Dynamic[
             CurrentValue["MouseOver"]],
            Alignment->Center,
            FrameMargins->0,
            ImageSize->{Automatic, 25}],
           Appearance->None,
           
           ButtonFunction:>OutputSizeLimit`ButtonFunction[
            Identity, 72, 26400271531097435981, Infinity],
           Enabled->True,
           Evaluator->Automatic,
           Method->"Queued"], 
          ButtonBox[
           PaneSelectorBox[{False->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeChangeLimit"], 
                StandardForm],
               ImageSizeCache->{247., {1., 27.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControl",
             StripOnInput->False], True->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeChangeLimit"], 
                StandardForm],
               ImageSizeCache->{247., {1., 27.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControlActive",
             StripOnInput->False]}, Dynamic[
             CurrentValue["MouseOver"]],
            Alignment->Center,
            FrameMargins->0,
            ImageSize->{Automatic, 25}],
           Appearance->None,
           ButtonFunction:>FrontEndExecute[{
              FrontEnd`SetOptions[
              FrontEnd`$FrontEnd, 
               FrontEnd`PreferencesSettings -> {"Page" -> "Evaluation"}], 
              FrontEnd`FrontEndToken["PreferencesDialog"]}],
           Evaluator->None,
           Method->"Preemptive"]}
        },
        AutoDelete->False,
        FrameStyle->GrayLevel[0.85],
        GridBoxDividers->{"Columns" -> {False, {True}}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{"Columns" -> {{2}}}]}
     },
     DefaultBaseStyle->"Column",
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxDividers->{
      "Columns" -> {{False}}, "ColumnsIndexed" -> {}, "Rows" -> {{False}}, 
       "RowsIndexed" -> {}},
     GridBoxItemSize->{
      "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], 
         Offset[1.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Background->RGBColor[0.9657, 0.9753, 0.9802],
    FrameMargins->{{12, 12}, {0, 15}},
    FrameStyle->GrayLevel[0.85],
    RoundingRadius->5,
    StripOnInput->False],
   Deploy,
   DefaultBaseStyle->"Deploy"],
  Out[72]]], "Output",
 CellChangeTimes->{3.6287745336722403`*^9}]
}, Closed]],

Cell["Trying once more the simplification:", "Text",
 CellChangeTimes->{{3.62883535794203*^9, 3.628835378580866*^9}, 
   3.629013093857669*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"assu", "[", "xx", "]"}], "[", "s", "]"}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{"%", "//.", "xxrules"}], ",", " ", 
     RowBox[{"TimeConstraint", " ", "\[Rule]", " ", "Infinity"}]}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.628774568678702*^9, 3.6287746673800755`*^9}}],

Cell[BoxData["$Aborted"], "Output",
 CellChangeTimes->{3.628781739792897*^9}]
}, Closed]],

Cell["we aborted after waiting more than 2 hours.", "Text",
 CellChangeTimes->{{3.628781743817704*^9, 3.6287817835353737`*^9}, {
  3.6288353998437033`*^9, 3.6288354117933245`*^9}}],

Cell["For shortness we omit the arc length variable s and set", "Text",
 CellChangeTimes->{{3.628403568955264*^9, 3.628403608844534*^9}, 
   3.628774476981741*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "frenet"}]], "Input",
 CellChangeTimes->{{3.6288355030534844`*^9, 3.62883550625149*^9}}],

Cell[BoxData[
 StyleBox["\<\"frenet[curve][t] yields the Frenet frame for a generally \
curved \\ncurve[t] in the n-dimensional Euclidean space. Generally curved \
means that the \\nrank of the first n-1 derivatives of curve[t] must be n-1 \
at each point t.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.628835508825495*^9},
 CellTags->"Info3628839108-8634717"]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fre", "[", "i", "]"}], " ", "=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"frenet", "[", "xx", "]"}], "[", "s", "]"}], "[", 
    RowBox[{"[", "i", "]"}], "]"}]}], ";", 
  RowBox[{"k", " ", "=", " ", 
   RowBox[{
    RowBox[{"curvature", "[", "xx", "]"}], "[", "s", "]"}]}], ";", " ", 
  RowBox[{"\[Kappa]", " ", "=", " ", 
   RowBox[{"torsion", "[", 
    RowBox[{"xx", "[", "s", "]"}], "]"}]}], ";"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.628403621652157*^9, 3.628403775998828*^9}, 
   3.628835461027011*^9}],

Cell["Applying the Frenet formulas we obtain", "Text",
 CellChangeTimes->{{3.6284037918484554`*^9, 3.6284038282589197`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"yy", "'"}], "[", "s", "]"}], " ", "\[Equal]", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     RowBox[{"fre", "[", "1", "]"}]}], "*", "k", "*", 
    RowBox[{"Cos", "[", "a", "]"}]}], " ", "-", 
   RowBox[{
    RowBox[{"fre", "[", "2", "]"}], "*", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"\[Kappa]", "*", 
       RowBox[{"Sin", "[", "a", "]"}]}], "+", 
      RowBox[{
       RowBox[{"a", "'"}], "*", 
       RowBox[{"Sin", "[", "a", "]"}]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"fre", "[", "3", "]"}], "*", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"\[Kappa]", "*", 
       RowBox[{"Cos", "[", "a", "]"}]}], " ", "+", 
      RowBox[{
       RowBox[{"a", "'"}], "*", 
       RowBox[{"Cos", "[", "a", "]"}]}]}], ")"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.628403876431804*^9, 3.628403940735117*^9}, {
   3.6284040564561205`*^9, 3.628404065784937*^9}, {3.628404196700367*^9, 
   3.6284042602392783`*^9}, {3.6284043016417513`*^9, 3.6284043993291225`*^9}, 
   3.628404622097514*^9}],

Cell["\<\
Since the Frenet frame is orthonormal we get the scalar square of this vector \
by\
\>", "Text",
 CellChangeTimes->{{3.6284044139775486`*^9, 3.6284044875472775`*^9}, {
   3.628835724199473*^9, 3.628835726461477*^9}, {3.6290131026716843`*^9, 
   3.6290131068992915`*^9}, 3.6290131751182117`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sqy", " ", "=", " ", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"k", "*", 
        RowBox[{"Cos", "[", "a", "]"}]}], ")"}], "^", "2"}], " ", "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"\[Kappa]", "*", 
         RowBox[{"Sin", "[", "a", "]"}]}], "+", 
        RowBox[{
         RowBox[{"a", "'"}], "*", 
         RowBox[{"Sin", "[", "a", "]"}]}]}], ")"}], "^", "2"}], "+", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"\[Kappa]", "*", 
         RowBox[{"Cos", "[", "a", "]"}]}], " ", "+", 
        RowBox[{
         RowBox[{"a", "'"}], "*", 
         RowBox[{"Cos", "[", "a", "]"}]}]}], ")"}], "^", "2"}]}], ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a", ">", "0"}], ",", " ", 
      RowBox[{
       RowBox[{"a", "'"}], ">", "0"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.628404489918482*^9, 3.6284045923638616`*^9}, {
   3.6284046467455573`*^9, 3.6284047233572917`*^9}, 3.62840481358785*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["k", "2"], "+", 
     RowBox[{"2", " ", 
      SuperscriptBox["\[Kappa]", "2"]}], "+", 
     RowBox[{
      SuperscriptBox["k", "2"], " ", 
      RowBox[{"Cos", "[", 
       RowBox[{"2", " ", "a"}], "]"}]}]}], ")"}]}], "+", 
  RowBox[{"2", " ", "\[Kappa]", " ", 
   SuperscriptBox["a", "\[Prime]",
    MultilineFunction->None]}], "+", 
  SuperscriptBox[
   RowBox[{"(", 
    SuperscriptBox["a", "\[Prime]",
     MultilineFunction->None], ")"}], "2"]}]], "Output",
 CellChangeTimes->{3.6284047307829046`*^9, 3.628404814679852*^9, 
  3.628835772138357*^9}]
}, Closed]],

Cell["Obviously, this can be written as", "Text",
 CellChangeTimes->{{3.628405373675634*^9, 3.6284053877468586`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"sqy", " ", "\[Equal]", "  ", 
   RowBox[{
    RowBox[{
     RowBox[{"k", "^", "2"}], "  ", "*", 
     RowBox[{
      RowBox[{"Cos", "[", "a", "]"}], "^", "2"}]}], " ", "+", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"\[Kappa]", " ", "+", 
       RowBox[{"a", "'"}]}], ")"}], "^", "2"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.628404790343809*^9, 3.6284050227998176`*^9}, {
  3.628405319449939*^9, 3.62840534295918*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{{3.6284050029877825`*^9, 3.628405024843421*^9}, 
   3.6284053476859884`*^9, 3.6288357767247653`*^9}]
}, Closed]],

Cell["Since xx\[CloseCurlyQuote][s] == fre[1], we have", "Text",
 CellChangeTimes->{{3.6284050532354712`*^9, 3.6284051018295565`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"xx", "'"}], "[", "s", "]"}], ".", 
     RowBox[{
      RowBox[{"yy", "'"}], "[", "s", "]"}]}], ")"}], "^", "2"}], " ", "=", 
  " ", 
  RowBox[{
   RowBox[{"k", "^", "2"}], "*", 
   RowBox[{
    RowBox[{"Cos", "[", "a", "]"}], "^", "2"}]}]}]], "Input",
 CellChangeTimes->{{3.6284051121411743`*^9, 3.6284051842757015`*^9}, {
  3.6284052725250564`*^9, 3.6284052839910765`*^9}}],

Cell[TextData[{
 "Using the criterion of Subsection 3.7 we conclude \n",
 StyleBox["Proposition.",
  FontColor->RGBColor[0, 0, 1]],
 " A family of normals  of  the regular curve xx[s], s its arc length \
parameter, defined by the angle a[s] of the normal with the principal normal \
at the point xx[s], is developable iff"
}], "Text",
 CellChangeTimes->{{3.6284055015334587`*^9, 3.628405719824642*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{"a", "[", "s", "]"}], ",", "s"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{"-", 
   RowBox[{
    RowBox[{"torsion", "[", "xx", "]"}], "[", "s", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.62840573583027*^9, 3.6284057675607257`*^9}, 
   3.628405920986995*^9}],

Cell["is satisfied.", "Text",
 CellChangeTimes->{{3.6284057912883673`*^9, 3.628405811038002*^9}}],

Cell["\<\
Indeed, the criterion is satisfied iff sqy == (xx\[CloseCurlyQuote][s].yy\
\[CloseCurlyQuote][s])^2 . \[FilledSquare]\
\>", "Text",
 CellChangeTimes->{{3.6284058365596466`*^9, 3.62840590095656*^9}, {
  3.628405944730237*^9, 3.628405956648658*^9}}],

Cell[TextData[{
 StyleBox["Example.",
  FontColor->RGBColor[0, 0, 1]],
 " Plane curve.\nConsider a plane curve embedded into the 3-space as a space \
curve, e.g."
}], "Text",
 CellChangeTimes->{{3.628837813612527*^9, 3.628837819495737*^9}, {
   3.628837851354993*^9, 3.628837922782919*^9}, 3.6290131318437357`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"plcurve", "[", 
    RowBox[{"f_", ",", "g_"}], "]"}], "[", "s_", "]"}], ":=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"f", "[", "s", "]"}], ",", 
    RowBox[{"g", "[", "s", "]"}], ",", "0"}], "}"}]}]], "Input",
 CellChangeTimes->{{3.6288379330945377`*^9, 3.628838029202707*^9}}],

Cell["\<\
Its principal normal is the normal of the plane curve, and its binormal is \
the constant normal vector of the embedded plane, in our case \
\[PlusMinus]stb[3]:\
\>", "Text",
 CellChangeTimes->{{3.6288380721711826`*^9, 3.628838201589411*^9}, {
  3.628838433289233*^9, 3.628838433289233*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"frenet", "[", 
      RowBox[{"plcurve", "[", 
       RowBox[{"f", ",", "g"}], "]"}], "]"}], "[", "s", "]"}], "[", 
    RowBox[{"[", "3", "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["f", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["g", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "2"]}], " ", ">", "0"}], 
     ",", " ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{
          SuperscriptBox["g", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}]}], " ", 
        RowBox[{
         SuperscriptBox["f", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "s", "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox["f", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], " ", 
        RowBox[{
         SuperscriptBox["g", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "s", "]"}]}]}], "<", "0"}]}], 
    "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6288382088590236`*^9, 3.62883825766391*^9}, {
  3.6288383057493973`*^9, 3.628838354168291*^9}, {3.628838412258396*^9, 
  3.628838417968006*^9}, {3.6288384675664935`*^9, 3.6288384951669416`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", 
   RowBox[{"-", "1"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.6288382592239122`*^9, 3.6288384397496443`*^9, {3.628838470748899*^9, 
   3.628838496648945*^9}, 3.629010455222234*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"torsion", "[", 
     RowBox[{"plcurve", "[", 
      RowBox[{"f", ",", "g"}], "]"}], "]"}], "[", "s", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["f", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["g", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "2"]}], " ", ">", "0"}], 
     ",", " ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{
          SuperscriptBox["g", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}]}], " ", 
        RowBox[{
         SuperscriptBox["f", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "s", "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox["f", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], " ", 
        RowBox[{
         SuperscriptBox["g", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "s", "]"}]}]}], "<", "0"}]}], 
    "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6288385166209803`*^9, 3.62883859589472*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6288385968931217`*^9, 3.6290104584202394`*^9}]
}, Closed]],

Cell[TextData[{
 "Thus the angle between the principal normal and the normal of a developable \
normal surface of the curve must be constant. For a = 0 we obtain the region \
of the plane covered by the principal normals (the normals of the plane \
curve). The envelope of these normals is the locus of the centers of \
curvature of the curve, called its ",
 StyleBox["evolute",
  FontColor->RGBColor[0, 0, 1]],
 " (see ",
 ButtonBox["evolute",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://mathworld.wolfram.com/Evolute.html"], None},
  ButtonNote->"http://mathworld.wolfram.com/Evolute.html"],
 "). The curve itself then is called the ",
 ButtonBox["involute",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://mathworld.wolfram.com/Involute.html"], None},
  ButtonNote->"http://mathworld.wolfram.com/Involute.html"],
 " of its evolute. The developable normal surface of the involute is the \
tangent surface of the evolute. For \na[s] =Pi/2 we obtain the cylindrical \
surface generated by the curve with rulings spanned  by stb[3]. \n\nUsually \
one considers the  evolute-involute relation for plane  curves in dimension 2 \
only. Considering the plane curve embedded in the 3-space  we  may consider \
evolutes depending on the angle a, as we do now; we speak about",
 StyleBox[" a-evolutes",
  FontColor->RGBColor[0, 0, 1]],
 ". Again the a-evolute is the curve whose tangent surface is the developable \
normal surface of the given involute, it is the locus of singularities, the \
\[OpenCurlyDoubleQuote]R\[UDoubleDot]ckkehrkante, of the surface, we will \
find its equation. The parameter representation of the surface is"
}], "Text",
 CellChangeTimes->{{3.628838712128925*^9, 3.628838715529731*^9}, {
   3.628838747387987*^9, 3.628838748105588*^9}, {3.628840123825511*^9, 
   3.628840370957348*^9}, {3.6288404856389093`*^9, 3.6288404873237123`*^9}, {
   3.628840517872566*^9, 3.6288406501888*^9}, {3.6288407539358625`*^9, 
   3.6288407899527264`*^9}, {3.6288408687733097`*^9, 
   3.6288408687733097`*^9}, {3.628841083558158*^9, 3.62884115726262*^9}, {
   3.6288420287840567`*^9, 3.6288420287840567`*^9}, {3.6288423029120092`*^9, 
   3.628842310296521*^9}, {3.628842359964644*^9, 3.628842359965644*^9}, {
   3.628842441725917*^9, 3.628842641079469*^9}, 3.6288427322952304`*^9, 
   3.62884862992288*^9, {3.628848772024931*^9, 3.628848803182186*^9}, {
   3.6288488645596943`*^9, 3.628848994353323*^9}, {3.6288490565556326`*^9, 
   3.628849092129695*^9}, {3.628999299317421*^9, 3.628999540211647*^9}, {
   3.6289996149517794`*^9, 3.628999631207008*^9}, 3.6290132389067235`*^9, 
   3.629013272821183*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"congr", "[", 
    RowBox[{"f_", ",", "g_", ",", "a_"}], "]"}], "[", 
   RowBox[{"s_", ",", "t_"}], "]"}], " ", ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"plcurve", "[", 
     RowBox[{"f", ",", "g"}], "]"}], "[", "s", "]"}], " ", "+", 
   RowBox[{"t", "*", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{
           RowBox[{"g", "'"}], "[", "s", "]"}]}], ",", 
         RowBox[{
          RowBox[{"f", "'"}], "[", "s", "]"}], ",", "0"}], "}"}], "*", 
       RowBox[{
        RowBox[{"Cos", "[", "a", "]"}], "/", 
        RowBox[{"Sqrt", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"f", "'"}], "[", "s", "]"}], "^", "2"}], " ", "+", 
          RowBox[{
           RowBox[{
            RowBox[{"g", "'"}], "[", "s", "]"}], "^", "2"}]}], "]"}]}]}], "+", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", 
        RowBox[{"Sin", "[", "a", "]"}]}], "}"}]}], ")"}]}]}]}]], "Input",
 CellChangeTimes->{{3.6288491287313595`*^9, 3.6288493832914085`*^9}, {
  3.628849470269562*^9, 3.628849502737219*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tv", "[", "1", "]"}], "[", 
   RowBox[{"congr", "[", 
    RowBox[{"f", ",", "g", ",", "a"}], "]"}], "]"}], "[", 
  RowBox[{"s", ",", "t"}], "]"}]], "Input",
 CellChangeTimes->{{3.6288496146512165`*^9, 3.628849689366748*^9}, {
  3.628849748452052*^9, 3.6288497553628645`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     SuperscriptBox["f", "\[Prime]",
      MultilineFunction->None], "[", "s", "]"}], "-", 
    FractionBox[
     RowBox[{"t", " ", 
      RowBox[{"Cos", "[", "a", "]"}], " ", 
      RowBox[{
       SuperscriptBox["g", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "s", "]"}]}], 
     SqrtBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["f", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["g", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "2"]}]]], "+", 
    FractionBox[
     RowBox[{"t", " ", 
      RowBox[{"Cos", "[", "a", "]"}], " ", 
      RowBox[{
       SuperscriptBox["g", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", 
         RowBox[{
          SuperscriptBox["f", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], " ", 
         RowBox[{
          SuperscriptBox["f", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "s", "]"}]}], "+", 
        RowBox[{"2", " ", 
         RowBox[{
          SuperscriptBox["g", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], " ", 
         RowBox[{
          SuperscriptBox["g", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "s", "]"}]}]}], ")"}]}], 
     RowBox[{"2", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["f", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["g", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], "2"]}], ")"}], 
       RowBox[{"3", "/", "2"}]]}]]}], ",", 
   RowBox[{
    RowBox[{
     SuperscriptBox["g", "\[Prime]",
      MultilineFunction->None], "[", "s", "]"}], "+", 
    FractionBox[
     RowBox[{"t", " ", 
      RowBox[{"Cos", "[", "a", "]"}], " ", 
      RowBox[{
       SuperscriptBox["f", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "s", "]"}]}], 
     SqrtBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["f", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["g", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "2"]}]]], "-", 
    FractionBox[
     RowBox[{"t", " ", 
      RowBox[{"Cos", "[", "a", "]"}], " ", 
      RowBox[{
       SuperscriptBox["f", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", 
         RowBox[{
          SuperscriptBox["f", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], " ", 
         RowBox[{
          SuperscriptBox["f", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "s", "]"}]}], "+", 
        RowBox[{"2", " ", 
         RowBox[{
          SuperscriptBox["g", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], " ", 
         RowBox[{
          SuperscriptBox["g", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "s", "]"}]}]}], ")"}]}], 
     RowBox[{"2", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["f", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["g", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], "2"]}], ")"}], 
       RowBox[{"3", "/", "2"}]]}]]}], ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.6288496922527533`*^9, 3.628849756860467*^9, 
  3.6288499540658145`*^9, 3.629000259890319*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tv", "[", "2", "]"}], "[", 
   RowBox[{"congr", "[", 
    RowBox[{"f", ",", "g", ",", "a"}], "]"}], "]"}], "[", 
  RowBox[{"s", ",", "t"}], "]"}]], "Input",
 CellChangeTimes->{{3.6288497772693033`*^9, 3.628849805164152*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{"Cos", "[", "a", "]"}], " ", 
      RowBox[{
       SuperscriptBox["g", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}]}], 
     SqrtBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["f", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["g", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "2"]}]]]}], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"Cos", "[", "a", "]"}], " ", 
     RowBox[{
      SuperscriptBox["f", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}]}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["f", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["g", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], "2"]}]]], ",", 
   RowBox[{"Sin", "[", "a", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.628849806991356*^9, 3.6288499569986196`*^9, 
  3.6290002644611273`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Together", "[", 
    RowBox[{"Cross", "[", 
     RowBox[{"%", ",", "%%"}], "]"}], "]"}], ",", 
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["f", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["g", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "2"]}], ">", "0"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.628849831265998*^9, 3.6288499420225935`*^9}, {
  3.6290002900471725`*^9, 3.6290003028275948`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["f", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["g", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], "2"]}], ")"}], 
       RowBox[{"3", "/", "2"}]]], 
     RowBox[{
      RowBox[{"Sin", "[", "a", "]"}], " ", 
      RowBox[{
       SuperscriptBox["g", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["f", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
         SqrtBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox["f", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox["g", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], "2"]}]]}], "+", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["g", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
         SqrtBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox["f", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox["g", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], "2"]}]]}], "+", 
        RowBox[{"t", " ", 
         RowBox[{"Cos", "[", "a", "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["g", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox["f", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "s", "]"}]}], "-", 
           RowBox[{
            RowBox[{
             SuperscriptBox["f", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox["g", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "s", "]"}]}]}], ")"}]}]}], 
       ")"}]}]}]}], ",", 
   RowBox[{
    FractionBox["1", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["f", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["g", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "2"]}], ")"}], 
      RowBox[{"3", "/", "2"}]]], 
    RowBox[{
     RowBox[{"Sin", "[", "a", "]"}], " ", 
     RowBox[{
      SuperscriptBox["f", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["f", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
        SqrtBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox["f", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox["g", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "2"]}]]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["g", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
        SqrtBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox["f", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox["g", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "2"]}]]}], "+", 
       RowBox[{"t", " ", 
        RowBox[{"Cos", "[", "a", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["g", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], " ", 
           RowBox[{
            SuperscriptBox["f", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "s", "]"}]}], "-", 
          RowBox[{
           RowBox[{
            SuperscriptBox["f", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], " ", 
           RowBox[{
            SuperscriptBox["g", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "s", "]"}]}]}], ")"}]}]}], 
      ")"}]}]}], ",", 
   RowBox[{
    RowBox[{"Cos", "[", "a", "]"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       SqrtBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["f", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["g", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], "2"]}]]}], "+", 
      FractionBox[
       RowBox[{"t", " ", 
        RowBox[{"Cos", "[", "a", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", 
            RowBox[{
             SuperscriptBox["g", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}]}], " ", 
           RowBox[{
            SuperscriptBox["f", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "s", "]"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox["f", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], " ", 
           RowBox[{
            SuperscriptBox["g", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "s", "]"}]}]}], ")"}]}], 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["f", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["g", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "2"]}]]}], ")"}]}]}], 
  "}"}]], "Output",
 CellChangeTimes->{{3.628849945360999*^9, 3.6288499637710314`*^9}, 
   3.6290003077104034`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"nvector", " ", "=", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.6290003608394976`*^9, 3.629000383243137*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{"nvector", "\[Equal]", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0", ",", "0"}], "}"}]}], ",", "t"}], "]"}]], "Input",
 CellChangeTimes->{{3.6290003965227604`*^9, 3.6290004286034174`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"t", "\[Rule]", 
    FractionBox[
     RowBox[{
      RowBox[{"Sec", "[", "a", "]"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["f", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["g", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], "2"]}], ")"}], 
       RowBox[{"3", "/", "2"}]]}], 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         SuperscriptBox["g", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}]}], " ", 
       RowBox[{
        SuperscriptBox["f", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "s", "]"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox["f", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], " ", 
       RowBox[{
        SuperscriptBox["g", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "s", "]"}]}]}]]}], "}"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.6290004343754272`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "curvatures"}]], "Input",
 CellChangeTimes->{{3.6290009344649105`*^9, 3.629000941937324*^9}, 
   3.6290010944087934`*^9}],

Cell[BoxData[
 StyleBox["\<\"curvatures[curve][t] yields the n-1 curvatures of a generally \
curved \\ncurve[t] in the n-dimensional Euclidean space. Generally curved \
means that the \\nrank of the first n-1 derivatives of curve[t] must be n-1 \
at each point t.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.6290010969047976`*^9},
 CellTags->"Info3629004696-3111076"]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"curve2D", "[", 
    RowBox[{"f_", ",", "g_"}], "]"}], "[", "s_", "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"f", "[", "s", "]"}], ",", 
    RowBox[{"g", "[", "s", "]"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.6290011512048936`*^9, 3.6290012007565813`*^9}}],

Cell[TextData[{
 Cell[BoxData[
  RowBox[{
  "Thus", " ", "for", " ", "the", " ", "oriented", " ", "plane", " ", "curve",
    " "}]], "Input"],
 "curve2D the curvature ",
 "with the correct sign is the only element of"
}], "Text",
 CellChangeTimes->{{3.6290015897156677`*^9, 3.6290016212649236`*^9}, {
  3.629001652205778*^9, 3.629001663347198*^9}, {3.6290017335125217`*^9, 
  3.629001734698124*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"curvatures", "[", 
    RowBox[{"curve2D", "[", 
     RowBox[{"f", ",", "g"}], "]"}], "]"}], "[", "s", "]"}], " "}]], "Input",
 CellChangeTimes->{{3.629001212415802*^9, 3.629001235301042*^9}, {
  3.6290016810084295`*^9, 3.629001727878912*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox["1", 
    SqrtBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["f", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["g", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], "2"]}]]], 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{
        RowBox[{
         SuperscriptBox["g", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          FractionBox[
           RowBox[{
            SuperscriptBox["f", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], 
           SqrtBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox["f", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox["g", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "2"]}]]], "-", 
          FractionBox[
           RowBox[{
            RowBox[{
             SuperscriptBox["f", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"2", " ", 
               RowBox[{
                SuperscriptBox["f", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], " ", 
               RowBox[{
                SuperscriptBox["f", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}]}], "+", 
              RowBox[{"2", " ", 
               RowBox[{
                SuperscriptBox["g", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], " ", 
               RowBox[{
                SuperscriptBox["g", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}]}]}], ")"}]}], 
           RowBox[{"2", " ", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox["f", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox["g", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], "2"]}], ")"}], 
             RowBox[{"3", "/", "2"}]]}]]}], ")"}]}], 
       SqrtBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["f", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["g", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], "2"]}]]]}], "+", 
     FractionBox[
      RowBox[{
       RowBox[{
        SuperscriptBox["f", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{
           SuperscriptBox["g", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "s", "]"}], 
          SqrtBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox["f", "\[Prime]",
               MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox["g", "\[Prime]",
               MultilineFunction->None], "[", "s", "]"}], "2"]}]]], "-", 
         FractionBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["g", "\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"2", " ", 
              RowBox[{
               SuperscriptBox["f", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], " ", 
              RowBox[{
               SuperscriptBox["f", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "s", "]"}]}], "+", 
             RowBox[{"2", " ", 
              RowBox[{
               SuperscriptBox["g", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], " ", 
              RowBox[{
               SuperscriptBox["g", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "s", "]"}]}]}], ")"}]}], 
          RowBox[{"2", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox["f", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox["g", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "2"]}], ")"}], 
            RowBox[{"3", "/", "2"}]]}]]}], ")"}]}], 
      SqrtBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["f", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["g", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "2"]}]]]}], ")"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.6290012394516497`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Together", "[", "%", "]"}], ",", " ", 
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["f", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["g", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "2"]}], ">", "0"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6290012850907297`*^9, 3.6290013545812526`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  FractionBox[
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SuperscriptBox["g", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}]}], " ", 
     RowBox[{
      SuperscriptBox["f", "\[Prime]\[Prime]",
       MultilineFunction->None], "[", "s", "]"}]}], "+", 
    RowBox[{
     RowBox[{
      SuperscriptBox["f", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{
      SuperscriptBox["g", "\[Prime]\[Prime]",
       MultilineFunction->None], "[", "s", "]"}]}]}], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["f", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["g", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], "2"]}], ")"}], 
    RowBox[{"3", "/", "2"}]]], "}"}]], "Output",
 CellChangeTimes->{3.629001369060078*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"k2D", "[", 
    RowBox[{"f_", ",", "g_"}], "]"}], "[", "s_", "]"}], " ", ":=", " ", 
  FractionBox[
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SuperscriptBox["g", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}]}], " ", 
     RowBox[{
      SuperscriptBox["f", "\[Prime]\[Prime]",
       MultilineFunction->None], "[", "s", "]"}]}], "+", 
    RowBox[{
     RowBox[{
      SuperscriptBox["f", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{
      SuperscriptBox["g", "\[Prime]\[Prime]",
       MultilineFunction->None], "[", "s", "]"}]}]}], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["f", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["g", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], "2"]}], ")"}], 
    RowBox[{"3", "/", "2"}]]]}]], "Input",
 CellChangeTimes->{{3.6290018263606863`*^9, 3.6290018536983356`*^9}, 
   3.629001891379402*^9}],

Cell["Thus this vector is the null vector  iff", "Text",
 CellChangeTimes->{{3.6288506737594833`*^9, 3.628850688065708*^9}, {
  3.628850735604992*^9, 3.628850738350597*^9}}],

Cell[BoxData[
 RowBox[{"t", " ", "\[Equal]", " ", 
  RowBox[{"1", "/", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"k2D", "[", 
       RowBox[{"f", ",", "g"}], "]"}], "[", "s", "]"}], "*", 
     RowBox[{"Cos", "[", "a", "]"}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.628850764156042*^9, 3.6288508063571167`*^9}, {
   3.628850837856572*^9, 3.628850863054617*^9}, 3.628850951078972*^9, {
   3.6290019373450837`*^9, 3.6290019466583*^9}, 3.6290019931573825`*^9}],

Cell["\<\
For a == \[PlusMinus]Pi/2 it follows t == Infinity, and this corresponds to \
the cylindrical surface: the ecolute is empty. The flat points, k[s] == 0, \
must be excluded. For a == 0 the evolute consists of the curvature centers of \
the involute. For arbitrary a the parameter representation of the evolute is \
(with s the arc length parameter of the given involute)\
\>", "Text",
 CellChangeTimes->{{3.6288516468381596`*^9, 3.628851893235795*^9}, {
  3.6288519719873266`*^9, 3.6288519743741302`*^9}, {3.6288520750349927`*^9, 
  3.62885209606583*^9}, {3.628852129578689*^9, 3.6288521651839523`*^9}, {
  3.6288524813639107`*^9, 3.6288524863423195`*^9}, {3.6288528450719533`*^9, 
  3.6288528712548*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"plevolute", "[", 
    RowBox[{"f_", ",", "g_", ",", "a_"}], "]"}], "[", "s_", "]"}], " ", ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"congr", "[", 
     RowBox[{"f", ",", "g", ",", "a"}], "]"}], "[", 
    RowBox[{"s", ",", "t"}], "]"}], "/.", 
   RowBox[{"t", "->", " ", 
    RowBox[{"1", "/", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"k2D", "[", 
         RowBox[{"f", ",", "g"}], "]"}], "[", "s", "]"}], "*", 
       RowBox[{"Cos", "[", "a", "]"}]}], ")"}]}]}]}]}]], "Input",
 CellChangeTimes->{{3.628852635461183*^9, 3.628852648038805*^9}, {
   3.628852704347305*^9, 3.62885270721971*^9}, 3.6288528126646957`*^9, {
   3.6288528847976236`*^9, 3.6288529286895013`*^9}, {3.629002130381624*^9, 
   3.629002158608074*^9}, {3.6290023516602154`*^9, 3.629002360681031*^9}, {
   3.629002519093111*^9, 3.629002564963192*^9}, {3.6290026001004543`*^9, 
   3.6290026283883038`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"plevolute", "[", 
   RowBox[{"f", ",", "g", ",", "a"}], "]"}], "[", "s", "]"}]], "Input",
 CellChangeTimes->{{3.629002643617931*^9, 3.629002657739956*^9}, 
   3.629002688821211*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "s", "]"}], "-", 
    FractionBox[
     RowBox[{
      RowBox[{
       SuperscriptBox["g", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["f", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["g", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "2"]}], ")"}]}], 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         SuperscriptBox["g", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}]}], " ", 
       RowBox[{
        SuperscriptBox["f", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "s", "]"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox["f", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], " ", 
       RowBox[{
        SuperscriptBox["g", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "s", "]"}]}]}]]}], ",", 
   RowBox[{
    RowBox[{"g", "[", "s", "]"}], "+", 
    FractionBox[
     RowBox[{
      RowBox[{
       SuperscriptBox["f", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["f", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["g", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "2"]}], ")"}]}], 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         SuperscriptBox["g", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}]}], " ", 
       RowBox[{
        SuperscriptBox["f", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "s", "]"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox["f", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], " ", 
       RowBox[{
        SuperscriptBox["g", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "s", "]"}]}]}]]}], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"Tan", "[", "a", "]"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["f", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["g", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "2"]}], ")"}], 
      RowBox[{"3", "/", "2"}]]}], 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SuperscriptBox["g", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}]}], " ", 
      RowBox[{
       SuperscriptBox["f", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "s", "]"}]}], "+", 
     RowBox[{
      RowBox[{
       SuperscriptBox["f", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{
       SuperscriptBox["g", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "s", "]"}]}]}]]}], "}"}]], "Output",
 CellChangeTimes->{3.629002659471559*^9, 3.629002690584014*^9, 
  3.6290105294003644`*^9}]
}, Closed]],

Cell["For the unit circle we get", "Text",
 CellChangeTimes->{{3.62901060080169*^9, 3.6290106128137107`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"plevolute", "[", 
   RowBox[{"Cos", ",", "Sin", ",", "a"}], "]"}], "[", "s", "]"}]], "Input",
 CellChangeTimes->{{3.629003320262925*^9, 3.6290033296913424`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", 
   RowBox[{
    SqrtBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{"Cos", "[", "s", "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{"Sin", "[", "s", "]"}], "2"]}]], " ", 
    RowBox[{"Tan", "[", "a", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.629003365326805*^9, 3.6290106200833235`*^9}]
}, Closed]],

Cell["This is the vertex of a circular cone:", "Text",
 CellChangeTimes->{{3.6290034227097063`*^9, 3.629003493717431*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"congr", "[", 
     RowBox[{"Cos", ",", "Sin", ",", 
      RowBox[{"Pi", "/", "3"}]}], "]"}], "[", 
    RowBox[{"s", ",", "t"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", 
     RowBox[{"-", "3"}], ",", "6"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6290034964680367`*^9, 3.6290034964836364`*^9}, 
   3.629003536207307*^9, {3.6290106549337845`*^9, 3.6290107423095384`*^9}}],

Cell["\<\
The a-evolute of the unit circle degenerates to the vertex {0, 0, Tan[a]} of  \
the normal surface; a circular cone. \[FilledSquare]\
\>", "Text",
 CellChangeTimes->{{3.6290107804516053`*^9, 3.629010858217742*^9}, {
  3.629010987432769*^9, 3.6290110266356378`*^9}, {3.629012881104495*^9, 
  3.629012886970105*^9}}],

Cell[TextData[{
 StyleBox["Example.",
  FontColor->RGBColor[0, 0, 1]],
 " a-Evolutes of  an ellipse."
}], "Text",
 CellChangeTimes->{{3.6290111135589905`*^9, 3.6290111465998487`*^9}}],

Cell["We consider the ellipse defined by fc and Sin", "Text",
 CellChangeTimes->{{3.6290111978459387`*^9, 3.629011256158841*^9}, 
   3.62901343550743*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fc", "[", "s_", "]"}], " ", ":=", " ", 
   RowBox[{"2", "*", 
    RowBox[{"Cos", "[", "s", "]"}]}]}], ";", " ", 
  RowBox[{"ell", "=", 
   RowBox[{"ParametricPlot3D", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"fc", "[", "s", "]"}], ",", 
       RowBox[{"Sin", "[", "s", "]"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"s", ",", "0", ",", 
       RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.629011263537654*^9, 3.6290113435501947`*^9}, {
  3.6290123228575144`*^9, 3.6290123329975324`*^9}, {3.629012402979255*^9, 
  3.6290124110132694`*^9}}],

Cell["For a = Pi/4 the developable normal surface is", "Text",
 CellChangeTimes->{{3.629011429818346*^9, 3.6290114688652143`*^9}, 
   3.6290118770399313`*^9}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"congr", "[", 
     RowBox[{"fc", ",", "Sin", ",", 
      RowBox[{"Pi", "/", "4"}]}], "]"}], "[", 
    RowBox[{"s", ",", "t"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", 
     RowBox[{"-", "3"}], ",", "7"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.629011478069231*^9, 3.629011486852046*^9}, {
   3.629011538176136*^9, 3.6290115817158127`*^9}, 3.6290116392175136`*^9, {
   3.629011699246419*^9, 3.629011792877784*^9}}],

Cell["The \[Pi]/4-evolute of the ellipse is", "Text",
 CellChangeTimes->{{3.629011907693985*^9, 3.6290119475520554`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"plevolute", "[", 
     RowBox[{"fc", ",", "Sin", ",", 
      RowBox[{"Pi", "/", "4"}]}], "]"}], "[", "s", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", " ", "0", ",", " ", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6290119657884874`*^9, 3.6290120194057817`*^9}}],

Cell["This is not a plane curve; its torsion is", "Text",
 CellChangeTimes->{{3.629012090073906*^9, 3.629012105861133*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"torsion", "[", 
     RowBox[{"plevolute", "[", 
      RowBox[{"fc", ",", "Sin", ",", 
       RowBox[{"Pi", "/", "4"}]}], "]"}], "]"}], "[", "s", "]"}], ",", " ", 
   RowBox[{"s", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6290121132711463`*^9, 3.6290121657496386`*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{"4", " ", 
   SqrtBox["2"], " ", 
   RowBox[{"Csc", "[", "s", "]"}], " ", 
   RowBox[{"Sec", "[", "s", "]"}]}], 
  RowBox[{"9", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"5", "-", 
      RowBox[{"3", " ", 
       RowBox[{"Cos", "[", 
        RowBox[{"2", " ", "s"}], "]"}]}]}], ")"}], 
    RowBox[{"3", "/", "2"}]]}]]], "Output",
 CellChangeTimes->{3.6290121739552526`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.629012177434059*^9, 3.62901221183212*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"plevolute", "[", 
     RowBox[{"fc", ",", "Sin", ",", "0"}], "]"}], "[", "s", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", " ", "0", ",", " ", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"PlotStyle", " ", "\[Rule]", " ", "Red"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6290119657884874`*^9, 3.6290120194057817`*^9}, {
  3.629012279177438*^9, 3.62901230903589*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ell", ",", "%"}], "}"}], ",", " ", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.629012350625563*^9, 3.629012375429607*^9}}],

Cell["\<\
\[FilledSquare]
Find developable normal surfaces and a-evolutes   for other plane or not \
plane 3D-curves! In the next subsection we do this for a helix.\
\>", "Text",
 CellChangeTimes->{{3.6290125932995896`*^9, 3.629012596591195*^9}, {
  3.6290126359656644`*^9, 3.629012835412015*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["3.8.2. Normal Congruence of the Helix", "Subsubsection",
 CellChangeTimes->{{3.592115389572919*^9, 3.592115426638584*^9}, 
   3.5921354646825233`*^9, 3.5922182112163124`*^9, 3.5942701628864923`*^9, 
   3.598596137570768*^9, 3.6116673526853304`*^9, 3.6119986168238125`*^9, {
   3.6155592477178297`*^9, 3.615559249355833*^9}, 3.6161383195804777`*^9, 
   3.618564015369048*^9, 3.6186374644950294`*^9, 3.6280852651103015`*^9, 
   3.6282292445934086`*^9, {3.6282293236387477`*^9, 3.6282293292235575`*^9}, 
   3.6282293736836357`*^9, 3.628229447222165*^9, 3.629013014422329*^9, 
   3.6291254008855953`*^9},
 CellTags->"helix"],

Cell[" As example we consider the normal congruence of a helix.", "Text",
 CellChangeTimes->{{3.598596193044465*^9, 3.5985962126692996`*^9}, {
   3.598596242824153*^9, 3.5985963424459276`*^9}, 3.5986252137382*^9, 
   3.5986252462330575`*^9, {3.6156418819882107`*^9, 3.615641952219534*^9}, {
   3.6282295708991823`*^9, 3.628229591974819*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"hel", "[", "s_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"helix", "[", 
     RowBox[{"1", ",", "1"}], "]"}], "[", "s", "]"}], "/", 
   RowBox[{"Sqrt", "[", "2", "]"}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.592115431786593*^9, 3.5921154360142*^9}, {
  3.59211546966346*^9, 3.5921154800374775`*^9}, {3.592115539411182*^9, 
  3.592115660030594*^9}, {3.5921157321963205`*^9, 3.5921157646287775`*^9}, {
  3.614750259680585*^9, 3.6147502813958235`*^9}, {3.614750440531703*^9, 
  3.614750474570963*^9}, {3.6290971891808167`*^9, 3.629097200802837*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hel", "[", "s", "]"}]], "Input",
 CellChangeTimes->{{3.614750403200837*^9, 3.614750411578052*^9}, 
   3.6290972426109104`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{"Cos", "[", "s", "]"}], 
    SqrtBox["2"]], ",", 
   FractionBox[
    RowBox[{"Sin", "[", "s", "]"}], 
    SqrtBox["2"]], ",", 
   FractionBox["s", 
    SqrtBox["2"]]}], "}"}]], "Output",
 CellChangeTimes->{3.6290968681166525`*^9, 3.6290972546229315`*^9, 
  3.629108113412004*^9}]
}, Closed]],

Cell["\<\
With the norming by 1/Sqrt[2] s becomes the arc length parameter of the curve \
hel:\
\>", "Text",
 CellChangeTimes->{{3.6147505405122786`*^9, 3.6147506061883936`*^9}, 
   3.629097410982006*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"hel", "'"}], "[", "s", "]"}], ".", 
   RowBox[{
    RowBox[{"hel", "'"}], "[", "s", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.592115673696218*^9, 3.5921156966750584`*^9}, {
  3.5921157757515974`*^9, 3.5921157855640144`*^9}, {3.614750345465136*^9, 
  3.614750352344748*^9}, {3.629097455348484*^9, 3.629097456159685*^9}, {
  3.6290977914042745`*^9, 3.629097792340276*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.629096880300274*^9, 3.629108116313609*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"helixnormalcongr", "[", "a_", "]"}], "[", 
   RowBox[{"s_", ",", "t_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"normalcongruence", "[", "hel", "]"}], "[", 
   RowBox[{"a", ",", "s", ",", "t"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.592115877026975*^9, 3.592115984230363*^9}, {
   3.592116019377225*^9, 3.5921161327894244`*^9}, {3.59859658850516*^9, 
   3.5985966192840137`*^9}, {3.614750762812669*^9, 3.61475076927108*^9}, {
   3.629097457189287*^9, 3.629097462696097*^9}, {3.6290977932450776`*^9, 
   3.6290978027922945`*^9}, 3.6290979253461094`*^9, {3.6290984935147076`*^9, 
   3.629098538239986*^9}, 3.629098582216463*^9, {3.6290986665190115`*^9, 
   3.6290986687810154`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"helixnormalcongr", "[", "a", "]"}], "[", 
  RowBox[{"s", ",", "t"}], "]"}]], "Input",
 CellChangeTimes->{{3.6290986002344947`*^9, 3.6290986190637283`*^9}, {
  3.629098685863045*^9, 3.629098733193528*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    FractionBox[
     RowBox[{"Cos", "[", "s", "]"}], 
     SqrtBox["2"]], "+", 
    RowBox[{"t", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"Cos", "[", "a", "]"}]}], " ", 
        RowBox[{"Cos", "[", "s", "]"}]}], "+", 
       FractionBox[
        RowBox[{
         RowBox[{"Sin", "[", "a", "]"}], " ", 
         RowBox[{"Sin", "[", "s", "]"}]}], 
        SqrtBox["2"]]}], ")"}]}]}], ",", 
   RowBox[{
    FractionBox[
     RowBox[{"Sin", "[", "s", "]"}], 
     SqrtBox["2"]], "+", 
    RowBox[{"t", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{
          RowBox[{"Cos", "[", "s", "]"}], " ", 
          RowBox[{"Sin", "[", "a", "]"}]}], 
         SqrtBox["2"]]}], "-", 
       RowBox[{
        RowBox[{"Cos", "[", "a", "]"}], " ", 
        RowBox[{"Sin", "[", "s", "]"}]}]}], ")"}]}]}], ",", 
   RowBox[{
    FractionBox["s", 
     SqrtBox["2"]], "+", 
    FractionBox[
     RowBox[{"t", " ", 
      RowBox[{"Sin", "[", "a", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", "s", "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "s", "]"}], "2"]}], ")"}]}], 
     SqrtBox["2"]]}]}], "}"}]], "Output",
 CellChangeTimes->{3.62909873422313*^9, 3.62911813896364*^9}]
}, Closed]],

Cell["\<\
Here a[s] denotes the angle of the normal chosen at the point hel[s] with the \
principal normal at this point. The surface generated by the principal \
normals is\
\>", "Text",
 CellChangeTimes->{{3.598596667690899*^9, 3.598596740636627*^9}, {
   3.598596885264481*^9, 3.5985969100217247`*^9}, 3.614750778131896*^9, {
   3.6290974653325014`*^9, 3.6290974663777037`*^9}, {3.629097804289897*^9, 
   3.6290978052570987`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"f0", "[", "s_", "]"}], " ", ":=", " ", "0"}], ";", 
  RowBox[{
   RowBox[{"helixnormalcongr", "[", 
    RowBox[{"f0", "[", "s", "]"}], "]"}], "[", 
   RowBox[{"s", ",", "t"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5985967629914665`*^9, 3.598596787421109*^9}, 
   3.6290979997270403`*^9, 3.629098866776563*^9, {3.6290989519059124`*^9, 
   3.6290989680207405`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    FractionBox[
     RowBox[{"Cos", "[", "s", "]"}], 
     SqrtBox["2"]], "-", 
    RowBox[{"t", " ", 
     RowBox[{"Cos", "[", "s", "]"}]}]}], ",", 
   RowBox[{
    FractionBox[
     RowBox[{"Sin", "[", "s", "]"}], 
     SqrtBox["2"]], "-", 
    RowBox[{"t", " ", 
     RowBox[{"Sin", "[", "s", "]"}]}]}], ",", 
   FractionBox["s", 
    SqrtBox["2"]]}], "}"}]], "Output",
 CellChangeTimes->{3.629098007261853*^9, 3.6290989204250574`*^9, 
  3.629098973652351*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"Evaluate", "[", 
    RowBox[{
     RowBox[{"helixnormalcongr", "[", 
      RowBox[{"f0", "[", "s", "]"}], "]"}], "[", 
     RowBox[{"s", ",", "t"}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "0", ",", 
     RowBox[{"4", "*", "Pi"}]}], "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"t", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5921161521646585`*^9, 3.5921162529720354`*^9}, 
   3.5921162866056943`*^9, {3.5921163851354675`*^9, 3.59211639789629*^9}, 
   3.598596854392027*^9, 3.614861860143141*^9, 3.6149177919205613`*^9, {
   3.6290980374323063`*^9, 3.629098042034314*^9}, 3.629098866792163*^9, {
   3.62909899034438*^9, 3.629098993167985*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"gauss", "[", 
    RowBox[{"helixnormalcongr", "[", 
     RowBox[{"f0", "[", "s", "]"}], "]"}], "]"}], "[", 
   RowBox[{"s", ",", "t"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.592116441529567*^9, 3.592116487518447*^9}, 
   3.5985969281489563`*^9, 3.629098866807763*^9, {3.6290990076136103`*^9, 
   3.6290990129176197`*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox["1", 
   RowBox[{"2", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RowBox[{
        SqrtBox["2"], " ", "t"}], "+", 
       SuperscriptBox["t", "2"]}], ")"}], "2"]}]]}]], "Output",
 CellChangeTimes->{3.62909818744217*^9, 3.629099150151061*^9}]
}, Closed]],

Cell["\<\
Obviously, this surface isn\[CloseCurlyQuote]t developable.  To find the \
developable normal surfaces we apply the criterion of Proposition 3.8.1. The \
torsion of hel is\
\>", "Text",
 CellChangeTimes->{{3.6155454978738985`*^9, 3.6155455235827436`*^9}, 
   3.615642092604181*^9, {3.629099653501545*^9, 3.6290997312988815`*^9}, {
   3.629099785196976*^9, 3.6290998144782276`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"torsion", "[", "hel", "]"}], "[", "s", "]"}], ",", 
   RowBox[{"s", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.629099818003834*^9, 3.629099864101915*^9}}],

Cell[BoxData[
 FractionBox["1", 
  SqrtBox["2"]]], "Output",
 CellChangeTimes->{{3.6290998384086695`*^9, 3.6290998652095165`*^9}, 
   3.629118153596466*^9}]
}, Closed]],

Cell["\<\
Thus the developable normal surfaces of hel correspond to the  parameter \
function\
\>", "Text",
 CellChangeTimes->{{3.629099959792483*^9, 3.629100051037043*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"fa", "[", 
   RowBox[{"s_", ",", "c_"}], "]"}], ":=", " ", 
  RowBox[{"c", "-", 
   RowBox[{"s", "/", 
    RowBox[{"Sqrt", "[", "2", "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.62910006624707*^9, 3.629100160455635*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"helixnormalcongr", "[", 
   RowBox[{"fa", "[", 
    RowBox[{"s", ",", "c"}], "]"}], "]"}], "[", 
  RowBox[{"s", ",", "t"}], "]"}]], "Input",
 CellChangeTimes->{3.629103152572091*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    FractionBox[
     RowBox[{"Cos", "[", "s", "]"}], 
     SqrtBox["2"]], "+", 
    RowBox[{"t", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"Cos", "[", "s", "]"}]}], " ", 
        RowBox[{"Cos", "[", 
         RowBox[{"c", "-", 
          FractionBox["s", 
           SqrtBox["2"]]}], "]"}]}], "+", 
       FractionBox[
        RowBox[{
         RowBox[{"Sin", "[", "s", "]"}], " ", 
         RowBox[{"Sin", "[", 
          RowBox[{"c", "-", 
           FractionBox["s", 
            SqrtBox["2"]]}], "]"}]}], 
        SqrtBox["2"]]}], ")"}]}]}], ",", 
   RowBox[{
    FractionBox[
     RowBox[{"Sin", "[", "s", "]"}], 
     SqrtBox["2"]], "+", 
    RowBox[{"t", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"Cos", "[", 
          RowBox[{"c", "-", 
           FractionBox["s", 
            SqrtBox["2"]]}], "]"}]}], " ", 
        RowBox[{"Sin", "[", "s", "]"}]}], "-", 
       FractionBox[
        RowBox[{
         RowBox[{"Cos", "[", "s", "]"}], " ", 
         RowBox[{"Sin", "[", 
          RowBox[{"c", "-", 
           FractionBox["s", 
            SqrtBox["2"]]}], "]"}]}], 
        SqrtBox["2"]]}], ")"}]}]}], ",", 
   RowBox[{
    FractionBox["s", 
     SqrtBox["2"]], "+", 
    FractionBox[
     RowBox[{"t", " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", "s", "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "s", "]"}], "2"]}], ")"}], " ", 
      RowBox[{"Sin", "[", 
       RowBox[{"c", "-", 
        FractionBox["s", 
         SqrtBox["2"]]}], "]"}]}], 
     SqrtBox["2"]]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6291031558480964`*^9, 3.6291082245465994`*^9, 
  3.6291083324831886`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"c", "\[Element]", "Reals"}], ",", 
     RowBox[{"s", "\[Element]", "Reals"}], ",", 
     RowBox[{"t", "\[Element]", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6291031707617226`*^9, 3.6291032089193897`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Cos", "[", "s", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox["1", 
        SqrtBox["2"]], "-", 
       RowBox[{"t", " ", 
        RowBox[{"Cos", "[", 
         RowBox[{"c", "-", 
          FractionBox["s", 
           SqrtBox["2"]]}], "]"}]}]}], ")"}]}], "+", 
    FractionBox[
     RowBox[{"t", " ", 
      RowBox[{"Sin", "[", "s", "]"}], " ", 
      RowBox[{"Sin", "[", 
       RowBox[{"c", "-", 
        FractionBox["s", 
         SqrtBox["2"]]}], "]"}]}], 
     SqrtBox["2"]]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox["1", 
        SqrtBox["2"]], "-", 
       RowBox[{"t", " ", 
        RowBox[{"Cos", "[", 
         RowBox[{"c", "-", 
          FractionBox["s", 
           SqrtBox["2"]]}], "]"}]}]}], ")"}], " ", 
     RowBox[{"Sin", "[", "s", "]"}]}], "-", 
    FractionBox[
     RowBox[{"t", " ", 
      RowBox[{"Cos", "[", "s", "]"}], " ", 
      RowBox[{"Sin", "[", 
       RowBox[{"c", "-", 
        FractionBox["s", 
         SqrtBox["2"]]}], "]"}]}], 
     SqrtBox["2"]]}], ",", 
   FractionBox[
    RowBox[{"s", "+", 
     RowBox[{"t", " ", 
      RowBox[{"Sin", "[", 
       RowBox[{"c", "-", 
        FractionBox["s", 
         SqrtBox["2"]]}], "]"}]}]}], 
    SqrtBox["2"]]}], "}"}]], "Output",
 CellChangeTimes->{3.6291032129129963`*^9, 3.62910835607043*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"helcgr", "[", "c_", "]"}], "[", 
   RowBox[{"s_", ",", "t_"}], "]"}], ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Cos", "[", "s", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        FractionBox["1", 
         SqrtBox["2"]], "-", 
        RowBox[{"t", " ", 
         RowBox[{"Cos", "[", 
          RowBox[{"c", "-", 
           FractionBox["s", 
            SqrtBox["2"]]}], "]"}]}]}], ")"}]}], "+", 
     FractionBox[
      RowBox[{"t", " ", 
       RowBox[{"Sin", "[", "s", "]"}], " ", 
       RowBox[{"Sin", "[", 
        RowBox[{"c", "-", 
         FractionBox["s", 
          SqrtBox["2"]]}], "]"}]}], 
      SqrtBox["2"]]}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox["1", 
         SqrtBox["2"]], "-", 
        RowBox[{"t", " ", 
         RowBox[{"Cos", "[", 
          RowBox[{"c", "-", 
           FractionBox["s", 
            SqrtBox["2"]]}], "]"}]}]}], ")"}], " ", 
      RowBox[{"Sin", "[", "s", "]"}]}], "-", 
     FractionBox[
      RowBox[{"t", " ", 
       RowBox[{"Cos", "[", "s", "]"}], " ", 
       RowBox[{"Sin", "[", 
        RowBox[{"c", "-", 
         FractionBox["s", 
          SqrtBox["2"]]}], "]"}]}], 
      SqrtBox["2"]]}], ",", 
    FractionBox[
     RowBox[{"s", "+", 
      RowBox[{"t", " ", 
       RowBox[{"Sin", "[", 
        RowBox[{"c", "-", 
         FractionBox["s", 
          SqrtBox["2"]]}], "]"}]}]}], 
     SqrtBox["2"]]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.6291032498382616`*^9, 3.6291032907883334`*^9}, {
   3.6291033253267937`*^9, 3.6291033326432066`*^9}, 3.629108366709649*^9, {
   3.62911819574774*^9, 3.6291181992265463`*^9}}],

Cell["We control the Gaussian curvature:", "Text",
 CellChangeTimes->{{3.629108422214546*^9, 3.62910845850021*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"gauss", "[", 
     RowBox[{"helcgr", "[", "c", "]"}], "]"}], "[", 
    RowBox[{"s", ",", "t"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"c", "\[Element]", "Reals"}], ",", 
     RowBox[{"s", "\[Element]", "Reals"}], ",", 
     RowBox[{"t", "\[Element]", "Reals"}]}], "}"}], ",", 
   RowBox[{"TimeConstraint", " ", "\[Rule]", " ", "Infinity"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.592116441529567*^9, 3.5921165134768925`*^9}, {
   3.5921165735837984`*^9, 3.5921166331759033`*^9}, {3.5985969638886194`*^9, 
   3.5985969657918224`*^9}, {3.6119269126930046`*^9, 3.611926926561429*^9}, 
   3.6290974676101055`*^9, 3.6290978063959007`*^9, 3.629098866807763*^9, {
   3.62909905305649*^9, 3.629099056426096*^9}, {3.6291002016085076`*^9, 
   3.6291002111401243`*^9}, 3.6291002494849916`*^9, {3.6291033455600295`*^9, 
   3.6291033721268764`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6291031227916384`*^9, 3.6291080271594524`*^9, 
  3.6291083900784903`*^9}]
}, Closed]],

Cell["OK. We consider some examples", "Text",
 CellChangeTimes->{{3.615642127345442*^9, 3.615642168201914*^9}, 
   3.615894763370123*^9, {3.6291084712142324`*^9, 3.629108514535508*^9}, 
   3.6291255059517794`*^9}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"helcgr", "[", "0", "]"}], "[", 
    RowBox[{"s", ",", "t"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", 
     RowBox[{"-", "10"}], ",", "10"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.629108703701441*^9, 3.6291087923719964`*^9}, 
   3.629108829141261*^9, 3.629108870138133*^9, {3.6291182299274*^9, 
   3.6291182700350704`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"helcgr", "[", 
     RowBox[{"Pi", "/", "4"}], "]"}], "[", 
    RowBox[{"s", ",", "t"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", 
     RowBox[{"-", "10"}], ",", "10"}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.629108703701441*^9, 3.6291087923719964`*^9}, 
   3.629108829141261*^9, 3.629108870138133*^9, {3.6291089550958824`*^9, 
   3.6291090883825164`*^9}, {3.6291183025299273`*^9, 3.6291183196275578`*^9}, 
   3.6291183693292446`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"tv", "[", "1", "]"}], "[", 
     RowBox[{"helcgr", "[", "c", "]"}], "]"}], "[", 
    RowBox[{"s", ",", "t"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"c", "\[Element]", "Reals"}], ",", 
     RowBox[{"s", "\[Element]", "Reals"}], ",", 
     RowBox[{"t", "\[Element]", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.629118429763751*^9, 3.629118447469782*^9}, 
   3.6291185314759293`*^9, {3.629118564688388*^9, 3.6291185966684437`*^9}, {
   3.629118632361307*^9, 3.6291186514557405`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "2"]}], " ", 
    RowBox[{"(", 
     RowBox[{
      SqrtBox["2"], "-", 
      RowBox[{"t", " ", 
       RowBox[{"Cos", "[", 
        RowBox[{"c", "-", 
         FractionBox["s", 
          SqrtBox["2"]]}], "]"}]}]}], ")"}], " ", 
    RowBox[{"Sin", "[", "s", "]"}]}], ",", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"Cos", "[", "s", "]"}], " ", 
    RowBox[{"(", 
     RowBox[{
      SqrtBox["2"], "-", 
      RowBox[{"t", " ", 
       RowBox[{"Cos", "[", 
        RowBox[{"c", "-", 
         FractionBox["s", 
          SqrtBox["2"]]}], "]"}]}]}], ")"}]}], ",", 
   RowBox[{
    FractionBox["1", 
     SqrtBox["2"]], "-", 
    RowBox[{
     FractionBox["1", "2"], " ", "t", " ", 
     RowBox[{"Cos", "[", 
      RowBox[{"c", "-", 
       FractionBox["s", 
        SqrtBox["2"]]}], "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.629118653592944*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"tv", "[", "2", "]"}], "[", 
     RowBox[{"helcgr", "[", "c", "]"}], "]"}], "[", 
    RowBox[{"s", ",", "t"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"c", "\[Element]", "Reals"}], ",", 
     RowBox[{"s", "\[Element]", "Reals"}], ",", 
     RowBox[{"t", "\[Element]", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.629118429763751*^9, 3.629118447469782*^9}, 
   3.6291185314759293`*^9, {3.629118564688388*^9, 3.6291185966684437`*^9}, {
   3.629118632361307*^9, 3.6291186764469843`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{"Cos", "[", "s", "]"}]}], " ", 
     RowBox[{"Cos", "[", 
      RowBox[{"c", "-", 
       FractionBox["s", 
        SqrtBox["2"]]}], "]"}]}], "+", 
    FractionBox[
     RowBox[{
      RowBox[{"Sin", "[", "s", "]"}], " ", 
      RowBox[{"Sin", "[", 
       RowBox[{"c", "-", 
        FractionBox["s", 
         SqrtBox["2"]]}], "]"}]}], 
     SqrtBox["2"]]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{"Cos", "[", 
       RowBox[{"c", "-", 
        FractionBox["s", 
         SqrtBox["2"]]}], "]"}]}], " ", 
     RowBox[{"Sin", "[", "s", "]"}]}], "-", 
    FractionBox[
     RowBox[{
      RowBox[{"Cos", "[", "s", "]"}], " ", 
      RowBox[{"Sin", "[", 
       RowBox[{"c", "-", 
        FractionBox["s", 
         SqrtBox["2"]]}], "]"}]}], 
     SqrtBox["2"]]}], ",", 
   FractionBox[
    RowBox[{"Sin", "[", 
     RowBox[{"c", "-", 
      FractionBox["s", 
       SqrtBox["2"]]}], "]"}], 
    SqrtBox["2"]]}], "}"}]], "Output",
 CellChangeTimes->{3.629118678755788*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Cross", "[", 
    RowBox[{"%", ",", "%%"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"c", "\[Element]", "Reals"}], ",", 
     RowBox[{"s", "\[Element]", "Reals"}], ",", 
     RowBox[{"t", "\[Element]", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.629118429763751*^9, 3.629118447469782*^9}, 
   3.6291185314759293`*^9, {3.629118564688388*^9, 3.6291185966684437`*^9}, {
   3.629118632361307*^9, 3.629118722326665*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"t", " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", 
         RowBox[{"c", "-", 
          FractionBox["s", 
           SqrtBox["2"]]}], "]"}], "2"], " ", 
       RowBox[{"Sin", "[", "s", "]"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{"Cos", "[", "s", "]"}], " ", 
       RowBox[{"Sin", "[", 
        RowBox[{"c", "-", 
         FractionBox["s", 
          SqrtBox["2"]]}], "]"}]}], "+", 
      RowBox[{
       SqrtBox["2"], " ", 
       RowBox[{"Cos", "[", 
        RowBox[{"c", "-", 
         FractionBox["s", 
          SqrtBox["2"]]}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"Sin", "[", "s", "]"}]}], "+", 
         RowBox[{"t", " ", 
          RowBox[{"Cos", "[", "s", "]"}], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"c", "-", 
            FractionBox["s", 
             SqrtBox["2"]]}], "]"}]}]}], ")"}]}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", "2"]}], " ", "t", " ", 
     RowBox[{"Cos", "[", "s", "]"}], " ", 
     SuperscriptBox[
      RowBox[{"Cos", "[", 
       RowBox[{"c", "-", 
        FractionBox["s", 
         SqrtBox["2"]]}], "]"}], "2"]}], "-", 
    RowBox[{
     RowBox[{"Sin", "[", "s", "]"}], " ", 
     RowBox[{"Sin", "[", 
      RowBox[{"c", "-", 
       FractionBox["s", 
        SqrtBox["2"]]}], "]"}]}], "+", 
    FractionBox[
     RowBox[{
      RowBox[{"Cos", "[", 
       RowBox[{"c", "-", 
        FractionBox["s", 
         SqrtBox["2"]]}], "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Cos", "[", "s", "]"}], "+", 
        RowBox[{"t", " ", 
         RowBox[{"Sin", "[", "s", "]"}], " ", 
         RowBox[{"Sin", "[", 
          RowBox[{"c", "-", 
           FractionBox["s", 
            SqrtBox["2"]]}], "]"}]}]}], ")"}]}], 
     SqrtBox["2"]]}], ",", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"Cos", "[", 
     RowBox[{"c", "-", 
      FractionBox["s", 
       SqrtBox["2"]]}], "]"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       SqrtBox["2"]}], "+", 
      RowBox[{"t", " ", 
       RowBox[{"Cos", "[", 
        RowBox[{"c", "-", 
         FractionBox["s", 
          SqrtBox["2"]]}], "]"}]}]}], ")"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6291187324666824`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{"%", "\[Equal]", " ", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0", ",", "0"}], "}"}]}], ",", "t"}], "]"}]], "Input",
 CellChangeTimes->{{3.62911875929873*^9, 3.6291187888139815`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"t", "\[Rule]", 
    RowBox[{
     SqrtBox["2"], " ", 
     RowBox[{"Sec", "[", 
      RowBox[{"c", "-", 
       FractionBox["s", 
        SqrtBox["2"]]}], "]"}]}]}], "}"}], "}"}]], "Output",
 CellChangeTimes->{3.6291187922303877`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"helaevolute", "[", "c_", "]"}], "[", "s_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"helcgr", "[", "c", "]"}], "[", 
    RowBox[{"s", ",", "t"}], "]"}], "/.", " ", 
   RowBox[{"t", "->", 
    RowBox[{
     SqrtBox["2"], " ", 
     RowBox[{"Sec", "[", 
      RowBox[{"c", "-", 
       FractionBox["s", 
        SqrtBox["2"]]}], "]"}]}]}]}]}]], "Input",
 CellChangeTimes->{{3.6291188366592655`*^9, 3.6291188564557004`*^9}, {
  3.6291188920237627`*^9, 3.6291189516158676`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"helaevolute", "[", "c", "]"}], "[", "s", "]"}]], "Input",
 CellChangeTimes->{{3.6291189605390835`*^9, 3.6291189720987034`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox["1", 
        SqrtBox["2"]], "-", 
       SqrtBox["2"]}], ")"}], " ", 
     RowBox[{"Cos", "[", "s", "]"}]}], "+", 
    RowBox[{
     RowBox[{"Sin", "[", "s", "]"}], " ", 
     RowBox[{"Tan", "[", 
      RowBox[{"c", "-", 
       FractionBox["s", 
        SqrtBox["2"]]}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox["1", 
        SqrtBox["2"]], "-", 
       SqrtBox["2"]}], ")"}], " ", 
     RowBox[{"Sin", "[", "s", "]"}]}], "-", 
    RowBox[{
     RowBox[{"Cos", "[", "s", "]"}], " ", 
     RowBox[{"Tan", "[", 
      RowBox[{"c", "-", 
       FractionBox["s", 
        SqrtBox["2"]]}], "]"}]}]}], ",", 
   FractionBox[
    RowBox[{"s", "+", 
     RowBox[{
      SqrtBox["2"], " ", 
      RowBox[{"Tan", "[", 
       RowBox[{"c", "-", 
        FractionBox["s", 
         SqrtBox["2"]]}], "]"}]}]}], 
    SqrtBox["2"]]}], "}"}]], "Output",
 CellChangeTimes->{3.6291189731907053`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"helaevolute", "[", "0", "]"}], "[", "s", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "0", ",", 
     RowBox[{"4", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6291191718726544`*^9, 3.6291192474703875`*^9}, {
  3.6291192990284777`*^9, 3.629119345407359*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"curvature", "[", 
     RowBox[{"helaevolute", "[", "c", "]"}], "]"}], "[", "s", "]"}], ",", 
   RowBox[{"s", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6291195077412443`*^9, 3.6291195894385877`*^9}, {
  3.6291196547091026`*^9, 3.6291196667055235`*^9}}],

Cell[BoxData[
 FractionBox[
  SqrtBox[
   RowBox[{"1", "+", 
    RowBox[{"Cos", "[", 
     RowBox[{
      RowBox[{"2", " ", "c"}], "-", 
      RowBox[{
       SqrtBox["2"], " ", "s"}]}], "]"}]}]], 
  RowBox[{"2", " ", 
   SqrtBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"Sec", "[", 
       RowBox[{"c", "-", 
        FractionBox["s", 
         SqrtBox["2"]]}], "]"}], "2"], " ", 
     SuperscriptBox[
      RowBox[{"Tan", "[", 
       RowBox[{"c", "-", 
        FractionBox["s", 
         SqrtBox["2"]]}], "]"}], "2"]}]]}]]], "Output",
 CellChangeTimes->{{3.62911956826935*^9, 3.6291195938377953`*^9}, 
   3.6291196707771306`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"acurvaturehel", "[", 
    RowBox[{"s_", ",", "c_"}], "]"}], " ", ":=", " ", 
   FractionBox[
    SqrtBox[
     RowBox[{"1", "+", 
      RowBox[{"Cos", "[", 
       RowBox[{
        RowBox[{"2", " ", "c"}], "-", 
        RowBox[{
         SqrtBox["2"], " ", "s"}]}], "]"}]}]], 
    RowBox[{"2", " ", 
     SqrtBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"Sec", "[", 
         RowBox[{"c", "-", 
          FractionBox["s", 
           SqrtBox["2"]]}], "]"}], "2"], " ", 
       SuperscriptBox[
        RowBox[{"Tan", "[", 
         RowBox[{"c", "-", 
          FractionBox["s", 
           SqrtBox["2"]]}], "]"}], "2"]}]]}]]}], 
  "\[IndentingNewLine]"}]], "Input",
 CellChangeTimes->{{3.6291196963767757`*^9, 3.629119755126479*^9}, {
  3.6291221908367567`*^9, 3.6291222294780245`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"torsion", "[", 
     RowBox[{"helaevolute", "[", "c", "]"}], "]"}], "[", "s", "]"}], ",", 
   RowBox[{"s", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6291224354607863`*^9, 3.6291224386431923`*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   SuperscriptBox[
    RowBox[{"Cos", "[", 
     RowBox[{"c", "-", 
      FractionBox["s", 
       SqrtBox["2"]]}], "]"}], "2"], 
   SqrtBox["2"]]}]], "Output",
 CellChangeTimes->{3.6291224427615995`*^9}]
}, Closed]],

Cell["\<\
Obviously, the a-evolute of the helix hel are generally curved curves in the \
3-space with isolated singularities.\
\>", "Text",
 CellChangeTimes->{
  3.616065523621974*^9, 3.6185806840470195`*^9, 3.6291190521892443`*^9, {
   3.629125290421801*^9, 3.6291253653331327`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["3.8.3. Normal Surfaces of a Surface", "Subsubsection",
 CellChangeTimes->{{3.5922181089604635`*^9, 3.5922181546640778`*^9}, {
   3.5988619755006013`*^9, 3.598861982021413*^9}, 3.6185622090977354`*^9, 
   3.6185639613379583`*^9, 3.628085258168289*^9, 3.6282293786288443`*^9, 
   3.629125440884065*^9, 3.6291863673197055`*^9}],

Cell[TextData[{
 "The normals of a surface form a 2-parameter family of lines, named the ",
 StyleBox["normal congruence of the surface",
  FontColor->RGBColor[0, 0, 1]],
 ":"
}], "Text",
 CellChangeTimes->{{3.5922049711314583`*^9, 3.592205009202636*^9}, {
   3.592205189986976*^9, 3.592205219635672*^9}, {3.592205300843317*^9, 
   3.5922053260107565`*^9}, 3.5922053597396855`*^9, 3.592217768303979*^9, {
   3.598596409822446*^9, 3.598596446888111*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"normals", "[", 
    RowBox[{"ff_", ",", " ", "u_", ",", "v_"}], "]"}], "[", "s_", "]"}], " ", 
  ":=", " ", 
  RowBox[{
   RowBox[{"ff", "[", 
    RowBox[{"u", ",", "v"}], "]"}], " ", "+", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"amf", "[", "3", "]"}], "[", "ff", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "*", "s"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.592205394333664*^9, 3.5922054044162407`*^9}, {
   3.592205454349097*^9, 3.59220549234127*^9}, 3.5922055233200417`*^9, 
   3.5922056857913346`*^9, 3.61486186009634*^9, 3.614917791858161*^9, 
   3.615531673064457*^9, {3.629174897118985*^9, 3.62917490005179*^9}, {
   3.629175105839157*^9, 3.629175110175965*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "normals"}]], "Input",
 CellChangeTimes->{{3.615531879804282*^9, 3.6155319923537197`*^9}, {
  3.615532027924754*^9, 3.615532036194227*^9}, {3.6228848660061088`*^9, 
  3.6228848794845324`*^9}}],

Cell[BoxData[
 StyleBox["\<\"normals[f,u,v][s] is the normal congruence of the surface \
f[u,v]\\nin the Eucjdean 3-space; s the line parameter.\"\>", "MSG"]], "Print",\
 "PrintUsage",
 CellChangeTimes->{3.6282289229676437`*^9},
 CellTags->"Info3628232522-2217871"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Context", "[", "normals", "]"}]], "Input",
 CellChangeTimes->{{3.622884925286213*^9, 3.6228849373918343`*^9}}],

Cell[BoxData["\<\"eudiffgeov2`\"\>"], "Output",
 CellChangeTimes->{3.6282289300344563`*^9, 3.6291750719956975`*^9}]
}, Closed]],

Cell["\<\
Consider a curve on the surface sf[u, v], given in the parameter  \
neighborhood by \
\>", "Text",
 CellChangeTimes->{{3.5922057532761946`*^9, 3.592205856435095*^9}, {
  3.598595786398551*^9, 3.5985957881769543`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Gamma]", "[", "t_", "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"hu", "[", "t", "]"}], ",", 
    RowBox[{"hv", "[", "t", "]"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.5922059133623505`*^9, 3.592205943501075*^9}, 
   3.6185624445392017`*^9}],

Cell["\<\
Then the normals along the curve form a ruled surface with directrix\
\>", "Text",
 CellChangeTimes->{{3.592206047180005*^9, 3.592206078795813*^9}, {
  3.5922061699950294`*^9, 3.592206174531289*^9}, {3.5985957986133723`*^9, 
  3.5985957997521744`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"xx", "[", "t_", "]"}], " ", ":=", " ", 
  RowBox[{"sf", "[", 
   RowBox[{
    RowBox[{"hu", "[", "t", "]"}], ",", 
    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5922060867402678`*^9, 3.5922061155499153`*^9}}],

Cell["The normal vectors of the surface along the curve are", "Text",
 CellChangeTimes->{{3.5922062089312563`*^9, 3.5922062280593505`*^9}, {
   3.611665413773525*^9, 3.6116654420563745`*^9}, 3.6146693565981827`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"yy", "[", "t_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"amf", "[", "3", "]"}], "[", "sf", "]"}], "[", 
   RowBox[{
    RowBox[{"hu", "[", "t", "]"}], ",", 
    RowBox[{"hv", "[", "t", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5922062360348067`*^9, 3.5922063003584857`*^9}, 
   3.6148618601119404`*^9, 3.614917791873761*^9, 3.615531676222638*^9}],

Cell["\<\
The parameter representation of the ruled surface generated by the normals of \
the surface along the curve is\
\>", "Text",
 CellChangeTimes->{{3.611665474582432*^9, 3.611665527762925*^9}, 
   3.618562101492581*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"rsf", "[", 
   RowBox[{"s_", ",", "t_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"xx", "[", "t", "]"}], " ", "+", "  ", 
   RowBox[{"s", "*", 
    RowBox[{"yy", "[", "t", "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.5922063337603965`*^9, 3.592206366014241*^9}}],

Cell[TextData[{
 "Such a surface is named a ",
 StyleBox["normal surface.",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.5922180596636443`*^9, 3.592218076359599*^9}, {
   3.6116655888370323`*^9, 3.6116656122682734`*^9}, 3.6146694043429136`*^9, 
   3.618562954160351*^9}],

Cell[BoxData[
 RowBox[{"rsf", "[", 
  RowBox[{"s", ",", "t"}], "]"}]], "Input",
 CellChangeTimes->{{3.5922064062155404`*^9, 3.5922064138929796`*^9}}],

Cell[TextData[{
 "We will answer the question: \n",
 StyleBox["Which curves on the surface generate developable normal surfaces?",
  FontSlant->"Italic",
  FontColor->RGBColor[0, 0, 1]],
 "\nTo this aim we have to consider the Gaussian curvature gauss[rsf][s,t] \
and find those curves for which this Gaussian curvature equals zero. The \
direct calculation of gauss[rsf][s,t]  symbolically in this generality is \
hopeless: "
}], "Text",
 CellChangeTimes->{{3.5922067006993837`*^9, 3.5922067612428465`*^9}, {
   3.59220679293966*^9, 3.592206827362629*^9}, {3.5922068605635276`*^9, 
   3.5922071163711596`*^9}, 3.5922074624679546`*^9, {3.5922178556159735`*^9, 
   3.592217870047799*^9}, {3.5922179504163957`*^9, 3.592217957559804*^9}, 
   3.6116657781901655`*^9, 3.615532199768268*^9}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"gauss", "[", "rsf", "]"}], "[", 
    RowBox[{"s", ",", "t"}], "]"}], ",", 
   RowBox[{"TimeConstraint", " ", "\[Rule]", " ", "Infinity"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.592206528380528*^9, 3.592206541620285*^9}, {
   3.61553224945711*^9, 3.615532286417224*^9}, {3.618563047210673*^9, 
   3.6185630693639402`*^9}, {3.62573886182058*^9, 3.625738869916994*^9}, 
   3.6257412565877867`*^9}],

Cell["\<\
Therefore we will apply Criterion 3.7 The condition of developability we will \
apply is\
\>", "Text",
 CellChangeTimes->{{3.5922072468236203`*^9, 3.5922073230269794`*^9}, {
  3.5922074788518915`*^9, 3.5922075577394037`*^9}, {3.592218413391876*^9, 
  3.592218423719467*^9}, {3.6291753155153275`*^9, 3.6291753645386143`*^9}}],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{"(*", 
   RowBox[{
    RowBox[{")", "                      ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["yy", "\[Prime]",
        MultilineFunction->None], "(", "t", ")"}], ".", 
      RowBox[{
       SuperscriptBox["yy", "\[Prime]",
        MultilineFunction->None], "(", "t", ")"}]}]}], "\[LongEqual]", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["yy", "\[Prime]",
          MultilineFunction->None], "(", "t", ")"}], ".", 
        RowBox[{
         SuperscriptBox["xx", "\[Prime]",
          MultilineFunction->None], "(", "t", ")"}]}], ")"}], "2"], "."}]}]}],
   TraditionalForm]],
 Evaluatable->False,
 CellChangeTimes->{{3.5919703342231636`*^9, 3.591970334456177*^9}, 
   3.5919703812638545`*^9, 3.592047182216324*^9, {3.592047410741125*^9, 
   3.5920474186035385`*^9}, {3.6116464842946773`*^9, 
   3.611646496306698*^9}}]], "Text",
 CellChangeTimes->{{3.6154435063429728`*^9, 3.615443534313822*^9}, {
  3.615443574858293*^9, 3.6154436171811676`*^9}, {3.6154436514544277`*^9, 
  3.6154436537008314`*^9}, {3.6154437181133447`*^9, 3.6154437181133447`*^9}, {
  3.615443791183873*^9, 3.615443791183873*^9}, {3.6155330280896454`*^9, 
  3.6155330478407745`*^9}}],

Cell["\<\
We consider a fixed value t and assume that the frame of sf[\[Gamma][t]] at \
this point has principal directions, such that\
\>", "Text",
 CellChangeTimes->{{3.5922081614029317`*^9, 3.592208197498996*^9}, {
   3.592208436427662*^9, 3.592208587483302*^9}, {3.615532908975832*^9, 
   3.6155329097028737`*^9}, 3.618562485131524*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"cf", "[", 
    RowBox[{"1", ",", "1"}], "]"}], " ", "=", " ", "k1"}], ";", " ", 
  RowBox[{
   RowBox[{"cf", "[", 
    RowBox[{"1", ",", "2"}], "]"}], " ", "=", " ", "0"}], ";", " ", 
  RowBox[{
   RowBox[{"cf", "[", 
    RowBox[{"2", ",", "2"}], "]"}], " ", "=", " ", "k2"}], ";"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.592208596248803*^9, 3.5922086683489275`*^9}, {
  3.6179721880391293`*^9, 3.6179721906402783`*^9}}],

Cell["\<\
are given by the principal curvatures of sf at this point. For the tangent \
vector of the curve xx[t] at this point we have\
\>", "Text",
 CellChangeTimes->{{3.5922086876910334`*^9, 3.5922087121154304`*^9}, {
  3.5922087602831855`*^9, 3.592208787339733*^9}, {3.6155329249997487`*^9, 
  3.6155329518872867`*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["xx", "\[Prime]",
     MultilineFunction->None], "(", "t", ")"}], "\[LongEqual]", " ", 
   RowBox[{
    RowBox[{"x", " ", 
     RowBox[{
      RowBox[{"amf", "(", "1", ")"}], "\[InvisibleApplication]", 
      RowBox[{"(", "sf", ")"}]}]}], "+", 
    RowBox[{"y", " ", 
     RowBox[{
      RowBox[{"amf", "(", "2", ")"}], "\[InvisibleApplication]", 
      RowBox[{"(", "sf", ")"}]}], "  ", "with", "  ", 
     SuperscriptBox["x", "2"]}], " ", "+", " ", 
    SuperscriptBox["y", "2"]}], "\[LongEqual]", "1"}], 
  TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5922087972623005`*^9, 3.5922088957339325`*^9}, {
   3.5922089455897846`*^9, 3.5922090179659243`*^9}, {3.6155329861842484`*^9, 
   3.615532999689021*^9}, 3.6185626038273125`*^9}],

Cell["\<\
assuming that t is the arc length of the curve.  Using the third derivation \
equation we obtain for the derivative of yy[t]\
\>", "Text",
 CellChangeTimes->{{3.592209073251086*^9, 3.5922091739078436`*^9}, {
   3.615533014047842*^9, 3.6155330161909647`*^9}, 3.618563604760563*^9}],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["yy", "\[Prime]",
     MultilineFunction->None], "(", "t", ")"}], " ", "\[LongEqual]", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"d", "(", 
      RowBox[{
       RowBox[{"amf", "(", "3", ")"}], "\[InvisibleApplication]", 
       RowBox[{"(", "sf", ")"}]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", 
      RowBox[{"\[Gamma]", "(", "t", ")"}], ")"}]}], ".", 
    SuperscriptBox["\[Gamma]", "\[Prime]",
     MultilineFunction->None]}], "\[LongEqual]", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"amf", "(", "1", ")"}], "\[InvisibleApplication]", 
        RowBox[{"(", "sf", ")"}]}], 
       RowBox[{"\[Omega]", "(", "2", ")"}]}], " ", "-", 
      RowBox[{
       RowBox[{
        RowBox[{"amf", "(", "2", ")"}], "\[InvisibleApplication]", 
        RowBox[{"(", "sf", ")"}]}], 
       RowBox[{"\[Omega]", "(", "1", ")"}]}]}], " ", ")"}], ".", 
    SuperscriptBox["\[Gamma]", "\[Prime]",
     MultilineFunction->None]}]}], TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5922091796091695`*^9, 3.592209399005718*^9}, 
   3.5922094301304984`*^9, {3.5922094720768976`*^9, 3.5922095483502607`*^9}, {
   3.6155331102723455`*^9, 3.615533147672485*^9}, {3.618562644897662*^9, 
   3.6185626640177555`*^9}, 3.6185636420766973`*^9}],

Cell[TextData[{
 "In the chosen frame with principal directions we have, using the formulas \
for the ",
 ButtonBox["\[Omega][i]",
  BaseStyle->"Hyperlink",
  ButtonData->"cij"],
 " in Subsection 2.2.3, Inserting \[Sigma][1] = x, \[Sigma][2] = y"
}], "Text",
 CellChangeTimes->{{3.5922096102918034`*^9, 3.5922096605156755`*^9}, {
  3.592213357239681*^9, 3.5922134378152895`*^9}, {3.5922134816957994`*^9, 
  3.5922134817037997`*^9}, {3.592215535215254*^9, 3.5922155360162997`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", 
    RowBox[{"\[Omega]", "[", "1", "]"}]}], " ", "==", " ", 
   RowBox[{"k2", "*", "y"}]}], ";", "  ", 
  RowBox[{
   RowBox[{"\[Omega]", "[", "2", "]"}], " ", "==", " ", 
   RowBox[{"k1", "*", "x"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5922135413852134`*^9, 3.5922136907857585`*^9}, {
  3.59221373679439*^9, 3.5922137769206853`*^9}, {3.592215489306628*^9, 
  3.5922155094177785`*^9}}],

Cell["It follows:", "Text",
 CellChangeTimes->{{3.592213810783622*^9, 3.592213814695846*^9}, {
  3.5922138524090033`*^9, 3.5922138799195766`*^9}, {3.592215530463982*^9, 
  3.59221555909562*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"yy", "'"}], ".", 
    RowBox[{"yy", "'"}]}], " ", "\[Equal]", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"k1", "^", "2"}], "*", 
     RowBox[{"x", "^", "2"}]}], " ", "+", " ", 
    RowBox[{
     RowBox[{"k2", "^", "2"}], "*", 
     RowBox[{"y", "^", "2"}]}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"xx", "'"}], ".", 
   RowBox[{"yy", "'"}]}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"k1", "*", 
    RowBox[{"x", "^", "2"}]}], "+", " ", 
   RowBox[{"k2", "*", 
    RowBox[{"y", "^", "2"}]}]}]}]}], "Input",
 CellChangeTimes->{{3.5922155975878215`*^9, 3.592215639401213*^9}, {
   3.592215732299526*^9, 3.592215762529256*^9}, 3.592215806385764*^9}],

Cell["\<\
In the case k1 == k2 any curve on the surface is a principal  curves, and the \
corresponding normal surface is always developable:\
\>", "Text",
 CellChangeTimes->{{3.592216110336149*^9, 3.592216217967305*^9}, {
  3.592216888759672*^9, 3.592216900167325*^9}, {3.592217137663909*^9, 
  3.5922171386879673`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"x", "^", "2"}], " ", "+", " ", 
       RowBox[{"y", "^", "2"}]}], " ", "==", " ", "1"}], ",", " ", 
     RowBox[{"k1", " ", "==", " ", "k2"}]}], "}"}], ",", " ", "\n", "  ", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"k1", "^", "2"}], "*", 
       RowBox[{"x", "^", "2"}]}], " ", "+", " ", 
      RowBox[{
       RowBox[{"k2", "^", "2"}], "*", 
       RowBox[{"y", "^", "2"}]}]}], " ", "==", " ", "\n", "    ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"k1", "*", 
         RowBox[{"x", "^", "2"}]}], " ", "+", " ", 
        RowBox[{"k2", "*", 
         RowBox[{"y", "^", "2"}]}]}], ")"}], "^", "2"}]}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.592215866233187*^9, 3.5922159659308896`*^9}, {
  3.59221601646778*^9, 3.5922160588312035`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.6291862637039227`*^9}]
}, Closed]],

Cell[TextData[{
 "We used ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SuperscriptBox["x", "2"], " ", "+", " ", 
    SuperscriptBox["y", "2"]}], "\[LongEqual]", "1"}]],
  Evaluatable->False,
  CellChangeTimes->{{3.5922087972623005`*^9, 3.5922088957339325`*^9}, {
   3.5922089455897846`*^9, 3.5922090179659243`*^9}}],
 " since t must be assumed to be  the arc length of the curve xx[t]. Now we \
assume \nk1 <k2 and solve the condition"
}], "Text",
 CellChangeTimes->{{3.5922162494631066`*^9, 3.5922163814636564`*^9}, 
   3.592216485983635*^9, {3.615537646480226*^9, 3.6155376744510756`*^9}, {
   3.6155377729340477`*^9, 3.6155377904996786`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reduce", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"k1", "^", "2"}], "*", 
        RowBox[{"x", "^", "2"}]}], " ", "+", " ", 
       RowBox[{
        RowBox[{"k2", "^", "2"}], "*", 
        RowBox[{"y", "^", "2"}]}]}], "\[Equal]", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"k1", "*", 
          RowBox[{"x", "^", "2"}]}], "+", " ", 
         RowBox[{"k2", "*", 
          RowBox[{"y", "^", "2"}]}]}], ")"}], "^", "2"}]}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"x", "^", "2"}], "+", 
       RowBox[{"y", "^", "2"}]}], "\[Equal]", "1"}], ",", 
     RowBox[{"k1", "<", "k2"}]}], "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}], ",", "Reals"}], "]"}]], "Input",
 CellChangeTimes->{{3.592200842980342*^9, 3.592200961997149*^9}, {
   3.592201251093684*^9, 3.5922013459261084`*^9}, {3.5922015293455997`*^9, 
   3.592201554663048*^9}, {3.5922020007245607`*^9, 3.5922020071349277`*^9}, {
   3.592202175170539*^9, 3.592202182919982*^9}, 3.5922023318775015`*^9, 
   3.5922033977764673`*^9, {3.5922035067917027`*^9, 3.5922035132950745`*^9}, {
   3.5922035968798556`*^9, 3.592203648999837*^9}, 3.6148618601275406`*^9, 
   3.614917791904961*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"k1", "<", "k2"}], "&&", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"x", "\[Equal]", 
       RowBox[{"-", "1"}]}], "&&", 
      RowBox[{"y", "\[Equal]", "0"}]}], ")"}], "||", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"x", "\[Equal]", "0"}], "&&", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"y", "\[Equal]", 
         RowBox[{"-", "1"}]}], "||", 
        RowBox[{"y", "\[Equal]", "1"}]}], ")"}]}], ")"}], "||", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"x", "\[Equal]", "1"}], "&&", 
      RowBox[{"y", "\[Equal]", "0"}]}], ")"}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.6291863018851905`*^9}]
}, Closed]],

Cell[TextData[{
 "Thus we proved\n",
 StyleBox["Proposition.",
  FontColor->RGBColor[0, 0, 1]],
 " ",
 StyleBox["A curve on a regular surface is a principal curve of the surface \
iff its corresponding normal surface is developable",
  FontSlant->"Italic"],
 ". \[FilledRectangle]\n\nWe remember, that on a surface without umbilic \
points there exist two families of principal lines intersecting orthogonally, \
namely the integral curves of the two fields of principal directions."
}], "Text",
 CellChangeTimes->{{3.59221665199113*^9, 3.5922166772315736`*^9}, {
   3.5922167386160846`*^9, 3.5922167741271152`*^9}, {3.592217151807718*^9, 
   3.592217208774976*^9}, {3.592217254623598*^9, 3.5922173034473915`*^9}, {
   3.592217335319214*^9, 3.592217420312075*^9}, {3.5922175010716944`*^9, 
   3.5922175148644834`*^9}, {3.592218543152298*^9, 3.5922185677917075`*^9}, {
   3.5984545080142236`*^9, 3.5984545145705986`*^9}, {3.598454584037572*^9, 
   3.5984545866857233`*^9}, 3.6155384990217237`*^9}],

Cell[TextData[{
 StyleBox["Corollary",
  FontColor->RGBColor[0, 0, 1]],
 ". ",
 StyleBox["On a developable surface a family of generating lines and their \
orthogonal trajectories are ",
  FontSlant->"Italic"],
 "principal curve",
 StyleBox["s",
  FontSlant->"Italic"],
 ".\n",
 StyleBox["Proof",
  FontColor->RGBColor[0, 0, 1]],
 ". The normal vector of the surface along a generating line is constant. \
Thus the normal surface over a generating line is a plane. \[FilledRectangle]"
}], "Text",
 CellChangeTimes->{{3.598595326447343*^9, 3.598595355291794*^9}, {
   3.5985954204219084`*^9, 3.598595640085894*^9}, {3.5985957006764*^9, 
   3.5985957064952106`*^9}, 3.6174428163857994`*^9, 3.618564000414193*^9}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.9. Surfaces of Revolution", "Subsection",
 CellChangeTimes->{{3.5922186227278495`*^9, 3.592218655535726*^9}, 
   3.5924598940730705`*^9, 3.611998625801326*^9, 3.6120759619398127`*^9, 
   3.616138350874133*^9, 3.616236325855336*^9, 3.6280851914781723`*^9}],

Cell["\<\
Any body produced on a lathe or a potter\[CloseCurlyQuote]s wheel is bordered \
by a surface of revolution. Such surfaces are not only easy to produce;  they \
may serve also as easy to handle examples in surface geometry. The reason is \
that the geometric properties of such a surface are defined by the properties \
of its generating curve, the profile curve or meridian of the surface. Thus \
two-dimensional problems for surfaces can be reduced to one-dimensional \
problems for curves, e.g. partial differential equations for surfaces  to \
ordinary differential equations for the meridians, which are much easier to \
solve. In this section we consider some of these applications.\
\>", "Text",
 CellChangeTimes->{{3.594271376925708*^9, 3.594271399649008*^9}, 
   3.594271494168414*^9, {3.5942716242798557`*^9, 3.594271650616362*^9}, 
   3.5942717013762655`*^9, {3.5942718587122645`*^9, 3.5942719851514964`*^9}, {
   3.5942721101926484`*^9, 3.594272209720341*^9}, {3.5942723600879416`*^9, 
   3.5942724882642727`*^9}, {3.594272641160018*^9, 3.5942727229756975`*^9}, {
   3.5942727668722086`*^9, 3.5942729100653987`*^9}, {3.594272942136233*^9, 
   3.5942729601122613`*^9}, {3.5942729958403044`*^9, 3.594273047392253*^9}, {
   3.5942747450093517`*^9, 3.5942747465524397`*^9}, 3.594274784344601*^9}],

Cell[CellGroupData[{

Cell["3.9.1. Definition and Fundamental Invariants", "Subsubsection",
 CellChangeTimes->{{3.5924599150863075`*^9, 3.5924599499211693`*^9}, 
   3.618637581542035*^9, 3.62048228513097*^9, 3.628085323922405*^9, {
   3.629186596970511*^9, 3.62918663603998*^9}, {3.6291867838712406`*^9, 
   3.6291867870088463`*^9}, 3.6291877482677407`*^9}],

Cell[TextData[{
 "Rotating a curve in the x,z-plane around the z-axis generates a ",
 StyleBox["surface of revolution",
  FontColor->RGBColor[0, 0, 1]],
 ". A parameter representation of a surface of revolution is"
}], "Text",
 CellChangeTimes->{{3.5922246929608603`*^9, 3.5922247958887477`*^9}, 
   3.618637861515727*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"sfrev", "[", 
    RowBox[{"f_", ",", "h_"}], "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"f", "[", "u", "]"}], "*", 
     RowBox[{"Cos", "[", "v", "]"}]}], ",", 
    RowBox[{
     RowBox[{"f", "[", "u", "]"}], "*", 
     RowBox[{"Sin", "[", "v", "]"}]}], ",", " ", 
    RowBox[{"h", "[", "u", "]"}]}], "}"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5922251535142026`*^9, 3.5922251573054194`*^9}, {
  3.592225626315245*^9, 3.59222574073979*^9}, {3.5927415105422573`*^9, 
  3.592741513818263*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sfrev", "[", 
   RowBox[{"f", ",", "h"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.592226758859023*^9, 3.592226768058549*^9}, {
   3.5922268420587816`*^9, 3.59222685356244*^9}, {3.592227075467132*^9, 
   3.5922270836586003`*^9}, {3.592316504708578*^9, 3.592316515347797*^9}, 
   3.592741514426664*^9}],

Cell[TextData[{
 "The rotating curves v = const. are the ",
 StyleBox["meridians",
  FontColor->RGBColor[0, 0, 1]],
 " of the surface of revolution, e.g."
}], "Text",
 CellChangeTimes->{{3.592455608074743*^9, 3.5924556618168373`*^9}, 
   3.5942709406726584`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"sfrev", "[", 
    RowBox[{"f", ",", "h"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "/.", " ", 
  RowBox[{"v", "\[Rule]", " ", "0"}]}]], "Input",
 CellChangeTimes->{{3.592226758859023*^9, 3.592226768058549*^9}, {
   3.5922268420587816`*^9, 3.59222685356244*^9}, {3.592227075467132*^9, 
   3.5922270836586003`*^9}, {3.592316504708578*^9, 3.592316515347797*^9}, {
   3.592455691269689*^9, 3.592455708289319*^9}, 3.5927415323198953`*^9}],

Cell["The sphere and the torus are such surfaces", "Text",
 CellChangeTimes->{{3.5922271717606397`*^9, 3.592227193319873*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sfrev", "[", 
   RowBox[{"Cos", ",", "Sin"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.592226758859023*^9, 3.592226768058549*^9}, {
  3.5922268420587816`*^9, 3.59222685356244*^9}, {3.592227075467132*^9, 
  3.5922270836586003`*^9}, {3.612072328365831*^9, 3.6120723342938414`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sphero", "[", "1", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5923061930991216`*^9, 3.592306214393159*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "torus"}]], "Input",
 CellChangeTimes->{{3.5923062653272486`*^9, 3.592306284000481*^9}}],

Cell[BoxData[
 StyleBox["\<\"{u,v}->torus[a,b][u,v] is a parametrization of the torus \
formed by revolving a circle of radius b in the xz-plane about the z-axis \
along a circle of radius a in the xy-plane. {u,v}->torus[a,b,c][u,v] is a \
parametrization of the torus formed by revolving an ellipse with axes of \
lengths b and c in the xz-plane about the z-axis along a circle of radius a \
in the xy-plane. {u,v}->torus[a,b,b,d,d][u,v] is a surface formed by moving a \
circle of radius b in the xz-plane along an ellipse in the xy-plane. \
{u,v}->torus[a,b,c,d,e][u,v] is a surface formed by moving an ellipse in the \
xz-plane along an ellipse in the xy-plane.  To plot try \
torus[8,3][0,3Pi/2,-Pi,Pi].\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.629186902991451*^9},
 CellTags->"Info3629190502-5022241"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"torus", "[", 
   RowBox[{"3", ",", "1"}], "]"}], "[", 
  RowBox[{"v", ",", "u"}], "]"}]], "Input",
 CellChangeTimes->{{3.5923063610022163`*^9, 3.5923063772574453`*^9}, {
  3.612072444539235*^9, 3.6120724721980834`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"3", "+", 
      RowBox[{"Cos", "[", "u", "]"}]}], ")"}], " ", 
    RowBox[{"Cos", "[", "v", "]"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"3", "+", 
      RowBox[{"Cos", "[", "u", "]"}]}], ")"}], " ", 
    RowBox[{"Sin", "[", "v", "]"}]}], ",", 
   RowBox[{"Sin", "[", "u", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.629186960501152*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ftor", "[", "v_", "]"}], " ", ":=", 
   RowBox[{"3", "+", 
    RowBox[{"Cos", "[", "v", "]"}]}]}], ";", 
  RowBox[{
   RowBox[{"htor", "[", "v_", "]"}], " ", ":=", " ", 
   RowBox[{"Sin", "[", "v", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5922272678111334`*^9, 3.592227311142612*^9}, {
   3.592227345322567*^9, 3.592227440772026*^9}, 3.5927415350966997`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"sfrev", "[", 
   RowBox[{"ftor", ",", "htor"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{3.5927415501507263`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"3", "+", 
      RowBox[{"Cos", "[", "u", "]"}]}], ")"}], " ", 
    RowBox[{"Cos", "[", "v", "]"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"3", "+", 
      RowBox[{"Cos", "[", "u", "]"}]}], ")"}], " ", 
    RowBox[{"Sin", "[", "v", "]"}]}], ",", 
   RowBox[{"Sin", "[", "u", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6291868676845884`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"torus", "[", 
    RowBox[{"8", ",", "3"}], "]"}], "[", 
   RowBox[{"0", ",", 
    RowBox[{"3", 
     RowBox[{"Pi", "/", "2"}]}], ",", 
    RowBox[{"-", "Pi"}], ",", "Pi"}], "]"}]}]], "Input"],

Cell[BoxData[
 Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJy0vXVYlk3QN4yJQXdIg4nYrdeO3d3d3d23jYGN3d3tbSvn2o3dAWInigGi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   "], {{
     {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJxFnHf8l9P/xu99zi2RrRTSkhWyQ/aeZUs0RcjeUlaZWdmj7E1GCKmkrOxN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         "]], Polygon3DBox[CompressedData["
1:eJw1nHf8VuP/x+8z7uucIxkVIaGE0FKULaNCRGWvdrJaFBERUrJXUtGSTbaG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         "]], Polygon3DBox[CompressedData["
1:eJwl13n4zWUax/HDOcc5qOxE/CxRkcykdHVlmqWmIZqkkolQjApJsi/xSxI/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         "]]}]}, {}, {}, {}, {}}, {{}, {}, {}, {
      Line3DBox[{236, 234, 2017, 576, 238, 2040, 599, 277, 2054, 613, 302, 
       2071, 627, 2072, 318, 641, 2096, 334, 655, 2110, 354, 669, 2124, 383, 
       2138, 683, 412, 2152, 697, 441, 2166, 711, 466, 2183, 725, 2184, 482, 
       739, 2208, 498, 753, 2222, 518, 547}], 
      Line3DBox[{779, 987, 1388, 988, 2338, 1390, 991, 1033, 992, 2344, 1406, 
       1035, 1036, 1413, 1065, 2406, 1066, 1420, 1086, 2420, 1087, 1427, 2505,
        1107, 1131, 1108, 1434, 2508, 1133, 1166, 1134, 1441, 2514, 1168, 
       1201, 1169, 2349, 1448, 1203, 1236, 1204, 2355, 1455, 1238, 1239, 2359,
        1462, 1268, 2461, 1269, 1469, 1289, 2475, 1290, 1476, 2533, 1310, 
       1334, 1311, 1483, 2536, 1336, 1366, 1337, 1368}], 
      Line3DBox[{780, 989, 1389, 990, 2339, 1392, 995, 1034, 996, 2345, 1407, 
       1038, 2393, 1039, 1414, 1067, 2407, 1068, 1421, 1088, 2421, 1089, 1428,
        1109, 2429, 1132, 1110, 1435, 2509, 1136, 1167, 1137, 1442, 2515, 
       1171, 1202, 1172, 2350, 1449, 1206, 1237, 1207, 2356, 1456, 1241, 2448,
        1242, 1463, 1270, 2462, 1271, 1470, 1291, 2476, 1292, 1477, 1312, 
       1335, 1313, 1484, 2537, 1339, 1367, 1340, 1370}], 
      Line3DBox[{781, 993, 1391, 994, 2340, 1394, 1000, 1037, 2392, 1001, 
       1408, 1041, 2395, 1042, 1415, 1069, 2408, 1070, 1422, 1090, 2422, 1091,
        1429, 1111, 2430, 1135, 1112, 1436, 2510, 1139, 1170, 1140, 1443, 
       2516, 1174, 1205, 1175, 2351, 1450, 1209, 1240, 2447, 1210, 1457, 1244,
        2450, 1245, 1464, 1272, 2463, 1273, 1471, 1293, 2477, 1294, 1478, 
       1314, 2484, 1338, 1315, 1485, 1342, 1369, 1343, 1372}], 
      Line3DBox[{782, 997, 1393, 2496, 999, 1396, 1004, 1040, 2394, 1006, 
       1409, 1044, 2397, 1046, 1416, 1071, 2410, 1073, 1423, 1092, 2424, 1094,
        1430, 1113, 2431, 1138, 1115, 1437, 1142, 2436, 1173, 1144, 1444, 
       2517, 1177, 1208, 2442, 1179, 1451, 1212, 1243, 2449, 1214, 1458, 1247,
        2452, 1249, 1465, 1274, 2465, 1276, 1472, 1295, 2479, 1297, 1479, 
       1316, 2485, 1341, 1318, 1486, 1345, 2490, 1371, 1347, 1374}], 
      Line3DBox[{787, 1014, 1400, 1015, 2341, 1402, 1019, 1052, 1020, 2346, 
       1410, 1055, 1056, 2348, 1417, 1077, 2414, 1078, 1424, 2504, 1098, 1099,
        1431, 2506, 1119, 1150, 1120, 1438, 2511, 1153, 1185, 1154, 1445, 
       2518, 1188, 1220, 1189, 2352, 1452, 1223, 1255, 1224, 2357, 1459, 1258,
        1259, 2360, 1466, 1280, 2469, 1281, 1473, 1301, 1302, 1480, 2534, 
       1322, 1353, 1323, 1487, 2538, 1356, 1378, 1357, 1380}], 
      Line3DBox[{788, 1017, 1401, 1018, 2342, 1404, 1023, 1054, 1024, 2347, 
       1411, 1058, 2401, 1059, 1418, 1079, 2415, 1080, 1425, 1100, 2427, 1101,
        1432, 2507, 1121, 1152, 1122, 1439, 2512, 1156, 1187, 1157, 1446, 
       2519, 1191, 1222, 1192, 2353, 1453, 1226, 1257, 1227, 2358, 1460, 1261,
        2456, 1262, 1467, 1282, 2470, 1283, 1474, 1303, 2482, 1304, 1481, 
       2535, 1324, 1355, 1325, 1488, 2539, 1359, 1379, 1360, 1382}], 
      Line3DBox[{789, 1021, 1403, 1022, 2343, 1405, 1025, 1057, 2400, 1026, 
       1412, 1060, 2402, 1061, 1419, 1081, 2416, 1082, 1426, 1102, 2428, 1103,
        1433, 1123, 2432, 1155, 1124, 1440, 2513, 1158, 1190, 1159, 1447, 
       2520, 1193, 1225, 1194, 2354, 1454, 1228, 1260, 2455, 1229, 1461, 1263,
        2457, 1264, 1468, 1284, 2471, 1285, 1475, 1305, 2483, 1306, 1482, 
       1326, 2486, 1358, 1327, 1489, 2540, 1361, 1381, 1362, 1383}], 
      Line3DBox[{874, 972, 1363, 2492, 1329, 867, 965, 1328, 2487, 1307, 860, 
       958, 2472, 1286, 853, 951, 2458, 1265, 846, 944, 2445, 1231, 839, 937, 
       2441, 1230, 1196, 832, 2521, 930, 1195, 1161, 2370, 825, 923, 1160, 
       2438, 1126, 818, 916, 1125, 2433, 1104, 811, 909, 2417, 1083, 804, 902,
        2403, 1062, 797, 895, 2390, 1028, 790, 888, 2386, 1027, 981, 776, 881,
        1385, 979, 1384}], 
      Line3DBox[{875, 973, 1364, 1331, 2381, 868, 966, 1330, 2488, 1308, 861, 
       959, 2473, 1287, 854, 952, 2459, 1266, 847, 945, 2446, 1233, 840, 2527,
        938, 1232, 1198, 833, 2522, 931, 1197, 1163, 2371, 826, 924, 1162, 
       1128, 2365, 819, 917, 1127, 2434, 1105, 812, 910, 2418, 1084, 805, 903,
        2404, 1063, 798, 896, 2391, 1030, 791, 889, 1029, 984, 777, 2494, 882,
        1386, 980, 982}], 
      Line3DBox[{876, 974, 1365, 1333, 2382, 869, 967, 1332, 1309, 2378, 862, 
       960, 2474, 1288, 855, 953, 2460, 1267, 848, 2531, 946, 1235, 841, 2528,
        939, 1234, 1200, 834, 2523, 932, 1199, 1165, 2372, 827, 925, 1164, 
       1130, 2366, 820, 918, 1129, 1106, 2362, 813, 911, 2419, 1085, 806, 904,
        2405, 1064, 799, 897, 1032, 792, 2500, 890, 1031, 986, 778, 2495, 883,
        1387, 983, 985}], 
      Line3DBox[{877, 975, 1373, 2493, 1346, 870, 968, 1344, 2489, 1317, 863, 
       961, 2478, 1296, 856, 954, 2464, 1275, 849, 947, 2451, 1248, 842, 940, 
       2443, 1246, 1213, 835, 933, 2440, 1211, 1178, 2373, 828, 926, 1176, 
       2439, 1143, 821, 919, 1141, 2435, 1114, 814, 912, 2423, 1093, 807, 905,
        2409, 1072, 800, 898, 2396, 1045, 793, 891, 2388, 1043, 1005, 783, 
       884, 2387, 1395, 998, 1002}], 
      Line3DBox[{878, 976, 1375, 1349, 2383, 871, 969, 1348, 2491, 1319, 864, 
       962, 2480, 1298, 857, 955, 2466, 1277, 850, 948, 2453, 1251, 843, 941, 
       2444, 1250, 1216, 836, 2524, 934, 1215, 1181, 2374, 829, 927, 1180, 
       1146, 2367, 822, 920, 1145, 2437, 1116, 815, 913, 2425, 1095, 808, 906,
        2411, 1074, 801, 899, 2398, 1048, 794, 892, 2389, 1047, 1009, 784, 
       2497, 885, 1397, 1003, 1007}], 
      Line3DBox[{879, 977, 1376, 1351, 2384, 872, 970, 1350, 1320, 2379, 865, 
       963, 2481, 1299, 858, 956, 2467, 1278, 851, 949, 2454, 1253, 844, 2529,
        942, 1252, 1218, 837, 2525, 935, 1217, 1183, 2375, 830, 928, 1182, 
       1148, 2368, 823, 921, 1147, 1117, 2363, 816, 914, 2426, 1096, 809, 907,
        2412, 1075, 802, 900, 2399, 1050, 795, 2501, 893, 1049, 1012, 785, 
       2498, 886, 1398, 1008, 1010}], 
      Line3DBox[{880, 978, 1377, 1354, 2385, 873, 971, 1352, 1321, 2380, 866, 
       964, 1300, 2377, 859, 957, 2468, 1279, 852, 2532, 950, 1256, 845, 2530,
        943, 1254, 1221, 838, 2526, 936, 1219, 1186, 2376, 831, 929, 1184, 
       1151, 2369, 824, 922, 1149, 1118, 2364, 817, 915, 1097, 2361, 810, 908,
        2413, 1076, 803, 2503, 901, 1053, 796, 2502, 894, 1051, 1016, 786, 
       2499, 887, 1399, 1011, 1013}]}, {
      Line3DBox[{1490, 1725, 2011, 881, 2010, 1726, 2013, 2494, 1648, 2012, 
       1728, 2015, 2495, 1649, 2014, 1730, 2017, 1650, 2016, 2265, 2338, 2019,
        1651, 2018, 2266, 2339, 2021, 1652, 2020, 2268, 2340, 2023, 1653, 
       2022, 2496, 1733, 2025, 2233, 2387, 2024, 1735, 2027, 2497, 1654, 2026,
        1737, 2029, 2498, 1655, 2028, 1739, 2031, 2499, 1656, 2030, 2270, 
       2341, 2033, 1657, 2032, 2271, 2342, 2035, 1658, 2034, 2273, 2343, 2036,
        1659, 1742}], 
      Line3DBox[{1491, 1727, 2037, 2234, 2386, 1744, 1729, 2038, 889, 1746, 
       1731, 2039, 2500, 1660, 1748, 1732, 2040, 1661, 1750, 2267, 2344, 2041,
        1662, 1752, 2269, 2345, 2042, 1663, 1753, 2392, 1734, 2043, 1664, 
       1755, 2394, 1736, 2044, 2235, 2388, 1757, 1738, 2045, 2236, 2389, 1759,
        1740, 2046, 2501, 1665, 1761, 1741, 2047, 2502, 1666, 1763, 2272, 
       2346, 2048, 1667, 1765, 2274, 2347, 2049, 1668, 1766, 2400, 1743, 2050,
        1669, 1768}], 
      Line3DBox[{1493, 1745, 2051, 2237, 2390, 1492, 1747, 2052, 2238, 2391, 
       1494, 1749, 2053, 897, 1495, 1751, 2054, 1670, 1496, 1036, 2055, 1671, 
       1497, 2393, 1754, 2056, 1672, 1498, 2395, 1756, 2057, 1673, 1499, 2397,
        1758, 2058, 2239, 2396, 1500, 1760, 2059, 2240, 2398, 1501, 1762, 
       2060, 2241, 2399, 1502, 1764, 2061, 2503, 1674, 1503, 2275, 2348, 2062,
        1675, 1504, 2401, 1767, 2063, 1676, 1505, 2402, 1769, 2064, 1677, 
       1783}], Line3DBox[{1507, 1770, 2065, 2242, 2403, 1506, 1771, 2067, 
       2243, 2404, 1508, 1772, 2069, 2244, 2405, 1509, 1773, 2071, 1678, 1510,
        2406, 1774, 2073, 1679, 1511, 2407, 1775, 2075, 1680, 1512, 2408, 
       1776, 2077, 1681, 1513, 2410, 1777, 2079, 2245, 2409, 1514, 1778, 2081,
        2246, 2411, 1515, 1779, 2083, 2247, 2412, 1516, 1780, 2085, 2248, 
       2413, 1517, 2414, 1781, 2087, 1682, 1518, 2415, 1782, 2089, 1683, 1519,
        2416, 1784, 2091, 1684, 1809}], 
      Line3DBox[{1533, 1811, 2092, 1810, 2428, 1532, 1808, 2090, 1807, 2427, 
       1531, 1806, 2088, 2504, 2277, 1530, 2276, 2361, 2086, 1805, 1529, 2426,
        1804, 2084, 1803, 1528, 2425, 1802, 2082, 1801, 1527, 2423, 1800, 
       2080, 1799, 2424, 1526, 1798, 2078, 1797, 2422, 1525, 1796, 2076, 1795,
        2421, 1524, 1794, 2074, 1793, 2420, 1523, 1792, 2072, 1791, 1522, 
       2419, 1790, 2070, 1789, 1521, 2418, 1788, 2068, 1787, 1520, 2417, 1786,
        2066, 1785, 1814}], 
      Line3DBox[{1547, 1834, 2106, 1833, 2432, 1546, 1832, 2105, 2507, 2283, 
       1545, 1831, 2104, 2506, 2282, 1544, 2281, 2364, 2103, 1830, 1543, 2280,
        2363, 2102, 1829, 1542, 2437, 1828, 2101, 1827, 1541, 2435, 1826, 
       2100, 1825, 2431, 1540, 1824, 2099, 1823, 2430, 1539, 1822, 2098, 1821,
        2429, 1538, 1820, 2097, 2505, 2279, 1537, 1819, 2096, 1818, 1536, 
       2278, 2362, 2095, 1817, 1535, 2434, 1816, 2094, 1815, 1534, 2433, 1813,
        2093, 1812, 1837}], 
      Line3DBox[{1561, 1852, 2120, 2513, 2294, 1560, 1851, 2119, 2512, 2293, 
       1559, 1850, 2118, 2511, 2292, 1558, 2291, 2369, 2117, 1849, 1557, 2290,
        2368, 2116, 1848, 1556, 2289, 2367, 2115, 1847, 1555, 2439, 1846, 
       2114, 1845, 2436, 1554, 1844, 2113, 2510, 2288, 1553, 1843, 2112, 2509,
        2287, 1552, 1842, 2111, 2508, 2286, 1551, 1841, 2110, 1840, 1550, 
       2285, 2366, 2109, 1839, 1549, 2284, 2365, 2108, 1838, 1548, 2438, 1836,
        2107, 1835, 1855}], 
      Line3DBox[{1563, 1854, 2135, 2521, 1685, 1876, 1857, 2136, 2522, 1686, 
       1879, 1859, 2137, 2523, 1687, 1881, 1862, 2138, 1688, 1883, 2299, 2349,
        2139, 1689, 1885, 2301, 2350, 2140, 1690, 1886, 2303, 2351, 2141, 
       1691, 1887, 2442, 1866, 2142, 2249, 2440, 1889, 1868, 2143, 2524, 1692,
        1891, 1870, 2144, 2525, 1693, 1893, 1872, 2145, 2526, 1694, 1895, 
       2310, 2352, 2146, 1695, 1897, 2312, 2353, 2147, 1696, 1898, 2314, 2354,
        2148, 1697, 1899}], 
      Line3DBox[{1576, 1875, 2134, 2520, 2313, 1575, 1874, 2133, 2519, 2311, 
       1574, 1873, 2132, 2518, 2309, 1573, 2308, 2376, 2131, 1871, 1572, 2307,
        2375, 2130, 1869, 1571, 2306, 2374, 2129, 1867, 1570, 2305, 2373, 
       2128, 2517, 2304, 1569, 1865, 2127, 2516, 2302, 1568, 1864, 2126, 2515,
        2300, 1567, 1863, 2125, 2514, 2298, 1566, 1861, 2124, 1860, 1565, 
       2297, 2372, 2123, 1858, 1564, 2296, 2371, 2122, 1856, 1562, 2295, 2370,
        2121, 1853, 1878}], 
      Line3DBox[{1577, 1877, 2149, 2250, 2441, 1901, 1880, 2150, 2527, 1698, 
       1903, 1882, 2151, 2528, 1699, 1905, 1884, 2152, 1700, 1907, 2315, 2355,
        2153, 1701, 1909, 2316, 2356, 2154, 1702, 1910, 2447, 1888, 2155, 
       1703, 1912, 2449, 1890, 2156, 2251, 2443, 1914, 1892, 2157, 2252, 2444,
        1916, 1894, 2158, 2529, 1704, 1918, 1896, 2159, 2530, 1705, 1920, 
       2317, 2357, 2160, 1706, 1922, 2318, 2358, 2161, 1707, 1923, 2455, 1900,
        2162, 1708, 1925}], 
      Line3DBox[{1579, 1902, 2163, 2253, 2445, 1578, 1904, 2164, 2254, 2446, 
       1580, 1906, 2165, 2531, 1709, 1581, 1908, 2166, 1710, 1582, 2319, 2359,
        2167, 1711, 1583, 2448, 1911, 2168, 1712, 1584, 2450, 1913, 2169, 
       1713, 1585, 2452, 1915, 2170, 2255, 2451, 1586, 1917, 2171, 2256, 2453,
        1587, 1919, 2172, 2257, 2454, 1588, 1921, 2173, 2532, 1714, 1589, 
       2320, 2360, 2174, 1715, 1590, 2456, 1924, 2175, 1716, 1591, 2457, 1926,
        2176, 1717, 1940}], 
      Line3DBox[{1593, 1927, 2177, 2258, 2458, 1592, 1928, 2179, 2259, 2459, 
       1594, 1929, 2181, 2260, 2460, 1595, 1930, 2183, 1718, 1596, 2461, 1931,
        2185, 1719, 1597, 2462, 1932, 2187, 1720, 1598, 2463, 1933, 2189, 
       1721, 1599, 2465, 1934, 2191, 2261, 2464, 1600, 1935, 2193, 2262, 2466,
        1601, 1936, 2195, 2263, 2467, 1602, 1937, 2197, 2264, 2468, 1603, 
       2469, 1938, 2199, 1722, 1604, 2470, 1939, 2201, 1723, 1605, 2471, 1941,
        2203, 1724, 1966}], 
      Line3DBox[{1619, 1968, 2204, 1967, 2483, 1618, 1965, 2202, 1964, 2482, 
       1617, 1963, 2200, 1301, 1616, 2321, 2377, 2198, 1962, 1615, 2481, 1961,
        2196, 1960, 1614, 2480, 1959, 2194, 1958, 1613, 2478, 1957, 2192, 
       1956, 2479, 1612, 1955, 2190, 1954, 2477, 1611, 1953, 2188, 1952, 2476,
        1610, 1951, 2186, 1950, 2475, 1609, 1949, 2184, 1948, 1608, 2474, 
       1947, 2182, 1946, 1607, 2473, 1945, 2180, 1944, 1606, 2472, 1943, 2178,
        1942, 1971}], 
      Line3DBox[{1633, 1990, 2218, 1989, 2486, 1632, 1988, 2217, 2535, 2327, 
       1631, 1987, 2216, 2534, 2326, 1630, 2325, 2380, 2215, 1986, 1629, 2324,
        2379, 2214, 1985, 1628, 2491, 1984, 2213, 1983, 1627, 2489, 1982, 
       2212, 1981, 2485, 1626, 1980, 2211, 1979, 2484, 1625, 1978, 2210, 1312,
        1624, 1977, 2209, 2533, 2323, 1623, 1976, 2208, 1975, 1622, 2322, 
       2378, 2207, 1974, 1621, 2488, 1973, 2206, 1972, 1620, 2487, 1970, 2205,
        1969, 1993}], 
      Line3DBox[{1647, 2008, 2232, 2540, 2337, 1646, 2007, 2231, 2539, 2336, 
       1645, 2006, 2230, 2538, 2335, 1644, 2334, 2385, 2229, 2005, 1643, 2333,
        2384, 2228, 2004, 1642, 2332, 2383, 2227, 2003, 1641, 2493, 2002, 
       2226, 2001, 2490, 1640, 2000, 2225, 1342, 1639, 1999, 2224, 2537, 2331,
        1638, 1998, 2223, 2536, 2330, 1637, 1997, 2222, 1996, 1636, 2329, 
       2382, 2221, 1995, 1635, 2328, 2381, 2220, 1994, 1634, 2492, 1992, 2219,
        1991, 2009}]}}},
   VertexNormals->CompressedData["
1:eJx0HWV4VMsO14v7pTgUdytyN7i7u7u7u7u7OxQrUKDoBig1pEKFFkq9hdIW
d33nbDKDfC/84ds0mzOTycSzp8iAsR0Gp0iWLFli+mTJUhr/T/ny0/j3Eq/l
GfPjypYt1gotDx5rU3CP1edEtd0tqr3A8U39XrQsFogKXsh+ZXHXVPF4MWZK
0/1vozW8bY9zd4amicQFXt/X9s6bgBUZvm7nt/NrZ97DFRc25d5V5aWGd43b
f/+C4zUIs9+1ZPa9X3C7Ph9m3rj9GBIjt+65FpCI5RneptbcIXv2xUFz+l/j
vyI8yEbf0/Bwogu56Dl6nctpHeBM6wKFP5vWDZ1pH6Cee4b2CSVo3xp/KPEF
/IhPGn/my2omHy3MV1DPTbunYjtzP48+7SiWpccma9HOW2/eNNabY5/blNU1
EnBwvp4nh2X0RwV3mDZ16ewVcdjOveMm38+RGl6s4KqoDafCsPPLqGsThj/D
Igy/e8Tl7VRHL6xyvvtkp/yJGj9lKsj1j/9lmDdx0sWORxI1/rSocu1dU4XA
4hkTcvt+jtf43auMevxxQAx0pv81/lLCg1H0PY3PdMEzpe05Gl6J1gHPaV2g
6LSndUMF2gco/Ba0T/iP9q3xexJfICfxScOPTB9q8tHCfNV0ijX/3nOfIZc7
XJqeeF1mlTVNsU4RprwmxNe6kSwwCvc06ZDHvecdTM3wTwFbvHdUCUe7kFa7
80cGo8JPX/hLmYe1A7HnOeeeN2IiNLz55ZE9aty6iQdyju28fmK0hl8tvav9
t7CzcNX155UF/tGa/rueT5bUaecHPh6ZX/sY56jwa81xbtqkZijUoP813I/w
IIm+94s+0YX19BxNfz+tA/rQukDhd6N1Q3bah4bnoH3Cd9o3pGL4euILMJ9A
0Z85PpnJRwvzVdPJ7JjzzhrjXg8qE+eZP/8ka7Yi34qZ97tl5mJZ+kfdwZ9z
3oyZ53sOFbzW4Apnbzdyx6zVV283+afg6/rO8T3dxoov0z4a9aqAp4ZP8nrm
nqeHI9as3L699fZdDY/ZH9Cq7MLtcGDSgmsXWt/T8BHF3bf1c7kEe1I/GWze
AwVvc7ncm2fFbkML+l/D9xEe9KPvafghoguO9BwNr0HrgPW0LlDwBFo37KR9
aHhG2ifUpX1r+AfiC7QhPmn49BbOJh8tzFcNHzTl/OQNxud+H8v3K2j8vdKc
Pi92GvCB7dMtGWR8v8ydvY8XGPQU/GAOn7UexvNKhw6OdjCer+DfOrW7dM5Y
X+d2k5a9Ndar4Fk96mXMb+wnZ7YBsTeN/VVkeIjz1C0VjP0Xu11rwGWDHwo+
op6950CDX5PnjD0/w+CforM/YsuKBIO/u+l/DZ9KeNiPvqfhRYku7qbnaHgO
WgdWpHWhem4HWjem62zbByr8YrRPPEb71vCixBccTHzSdMbk+drX4KOV+arx
L23sk9fU8xEl6154Y8h7ylv5WphyXvF9ugnJjfvQY2XlrB7G/VDwQQn273ca
96fkg2rL7Iz7pOCe8aPemvftn1rVv5r3T8HnWvKmqGmch0fDPJk2GPdVwbP2
7PbFvM9l/StPWWjc71QMT3LuXKaucf+TN/LcYepFhe/Yw65DU0NfHKT/NTwF
4WEMfU/DmS4+7WF7joa70zpwE60LFTwdrRv9aB8aXpD2iSNp3xreiviClYhP
Gn4j8t15g49W5quGtzh05ZKpr699HTLa1N8jB43MYtrtPXcnhJj6/cDh4a9N
fT+K4SkPVQ0x7cHBegsCTT6MYHhuJ6/2Gw37sSpoaK6Jhj1RdIp+ut1umiGX
Iy83fmzaHwV3zBc9wbRPryp3LdjpyC94/k8xJ0x7Vq/IwUmmfVP0Gy7sVuKT
Yf+A/tfwBoSHOeh7Gv6a6OIYeg6MZvgIWgfWp3Wh2tdyWjcWpX2gWs9e2iem
o31r/O3EF9xHfNL4S2dVNfloZb5qeIof5H/U/dfm11mS8b+STuSveDYn/0XB
Y0uRf5MQR/6OglfqRf7QgnvkHyn4pT3kP/24RP6Ugi+MJ39rYBnyvzSd/uSf
FYolf03BfeuQP3eH/tfwIoSHJeh7Gj6Y6GIDeo6Gf6N14CNaFyr4HFo3OtA+
NDyW9okJtG8NtxJfsDTxScO/vrb5dVbmq4YL8gyCPIMgzyDIMwjyDII8gyDP
IMgzCPIMgjyDIM8gyDMI8gyCPIMgzyDoZxD0Mwj6GQT9DIJ+BkE/g6CfQdDP
IOhnEPQzCPoZBP0Mgn4GQT+DoJ9B8DdA8DdA8DdA8DdA8DdA8DdA8DdA8DdA
8DdA8DdA8DdA8DdA8DdA8DdA8DdA8J9B8J9B8J9B8J9B8J9B8J9B8J9B8J9B
8J9B8J9B8J9B8J9B8J9B8J9B8J9BiAdBiAdBiAdBiAdBiAdBiAdBiAdBiAdB
iAdBiAdBiAdBiAdBiAdBiAdBiAdByG+AkN8AIb8BQn4DhPwGCPkNEPIbIOQ3
QMhvgJDfACG/AUJ+A4T8Bgj5DRDyGyDk6yxCvs4i5OssQr7OIuTrLEK+ziLk
6yxCvs4i5OssQr7OIuTrLEK+ziLk6yxCvs4i5OsswblnJbt98wVudC67ou7T
2zi316KWmwy+jMg1PzprmwTs2PTIh1UVwnA+w2tVXeLa6PgzPBvRal1P1xu4
1WHtFJPfNY6t9Tfjy6X18h23pt5iVXTKemdoFpA8EeuldUpX/32Yzk8+GlfG
LsQrBnek/Fy/ZMOn+rlLvuS4vi02Fh1KTLSrUS0YtzH9iOKDLl3oEItj7eY2
aB36VOc/115aEnM64iG28YmsfPxBoqbTxz2/5c6JCBzwpV1Lx6NRms6kpld+
Rs4Oxu093V+PXJek865Ph9h//fbtBD7udaV7RvuXms6agr1WfC/hj51aLlxe
u8BTTSeofpuvcOEkFvDr5W339lde94RjtcnvGvvB2Rrjx0/7kqTpFHKO3xg+
/ghW8G+y2enLM823M4+Hbj7p4gc3YxsdCsQXms4Zn+PZ9maKghrVHXDd+Hic
w3Rauvyc3/D8XfjwJnPDCRme6fVkqJj5wvRFUfCxRYacOfI91/uqRd+Hk0RP
r6f4rHMf5r0LhRm1UmafPjRGr+cLfR9SEz19Xs60H9hL+9PrmUPfBzuip+m4
0n7gAO1P74v5C8uG2vgNis5H2g840P40HTviL0xvYOO3zj+3ovOGHXT+MI/p
MH9hH/EbFH+20nnDPDp/nd/eRPIHUSSPoPjTns4bttL5azpDSf7gKcmjzns3
p3sCo+neaDo9Sf5gEMkjqH1Vo/sAFeh+aDqL6R4C30vNn0p0H2AZ3Q9Nx92u
xv5H49ygyxOXRvVXvdT8KbY00tG4hxa+l3o9R+jeQm26x3pfz3cGRXbb/RSz
OM9acWmJr+b/P/OnHGrT7zHudC7il3HnOQywT5Zk6r+I1d9+3DL0bb3ybzYW
dFxvVfgzt5edP35tNPa//D7LLsO/0Peu2cwT11sH4/RVuTI0eXJH01nZ8EjD
HHsf4agb3pCtYqx+7uu0c7tEz3uAxYq72m0+H4D+jF/VfVhTN8M/7Pnf+n4V
Bv26R8csH7Leq+eGB0r3bb/vaIjG73zq4LOQYRfh0+v3g2eH/rov0Y7zT4+N
34ZOrpFN111+rPG/3v1aIfhCIJQu8eKAabcV/bqV//nqi1b40qD/t+enH+n1
70354kLImUjYSf9r+jN69ruxL5svTGp+fe/uw0GafhmiC2/oORp/KuHBFPqe
xv9I64bCtI9f95TWASVpXXo9zBdoS3zS58v7hAnHbfsGRX848R220Dlo/L3E
RzhLfAVFvxudK8ylc9b4dnRO8JHOTdNPR/IEiSRfGn8syQFMJbnQ9FvNdjPl
ycLypeV8HckfZCJ51PTP1r5eq8/Z1ZYVp8tNy19hkKXAT/IbUg7rXAMNu3sx
U4YvWXYstCr8Rod2bviv1wrL4o5Zf1iKzdb4ha3V2l/LtsgSWmHe7iKTl2j4
sCJL89TINNUSGHCm4WD/lRr+9fvxaXapW1qico+ZXD77Gg3P4/dqxeeT46yj
hr50+fhqlYYPPdfZ9+ro+dYBbh1X1lq3TMMH0WfrAPq7ho+m71szEj0N5+dZ
t/6wPd+q4P60PuskWq+GB9F+rCVpfxo+g/ZvbUL80PBOk4I/G/yyMP80n+cQ
f63OxG+NnzPjvHbtjPPwuJr+3T/G+dwsNnmRiW/fwtRDayylSjYrfr5B9f8U
fq+1M0qicd6JHQuebmqcv8IvmN17aowhN3duzu23xZAjBV9ctmHv+4b8vZw1
J2q/IY83GP68Zcpq4w05nhHa5/N6Q64Vfuk0/o8fGPdhRb3NuxON+6Hw68ze
c3W/ca+uVQtMvte4ZwrfSp+xOv1d46+i72Nuoqfxp9PzcHor2/NRwZNofbiW
1qvht2k/WJz2p+FRtH/sT/zQ8GY9s5v8us7803y+SvzFXMRvjX+9+v6ujQ09
/jzZ+3c2vf6q/BUzDvTtOf+Z6a9O6bW4XDbjPBV+3WNr62037IddQ69Tpj2p
xfgbBtQ4YNqn2hU6XzXtlYJPjOry3bR/9Wu2yVnHsIeKfuazTv9GGPZ1ysVG
Zc8Y9lbB17fvUaqRYb8/Nx3hb9pzRedhqnWH5hv+wZX+y1aZ/oKCW+kz+tDf
NfwbfR/nEj0N5+fhWno+KriF1oczab0aXo32g9tpf6jWmYP2jxbiBzowPItr
R5NfVuaf5nME8RetxG9NZ2/QgbHdDb0WeelSkqnnFJ3MPl2rm/bbz5L3tGnP
W26I2bvOtL+va2w2/fURr79/N/WcWmfIhr0fTH36el2Lu6Z+VfB1ecZkz2b4
E3V+Liqz2vAvWjGdz+PnRZv6+tDQFGjqb82fGTFg+jETxr0dbvo1Cn/Lq0L3
TXuQekjat6Z9UPh3wtdmcjL8p07HRgw0/Sm1zjlzqo59ZNibkossX037o/B7
pq1X9rvhtw0f2WCt6ccp/HHXu5017ZnXu6Zl5hj2TeHfKhWzwfQX147+cMj0
H9V67lQtVfiRYS/d6X+NP2Pi2Uqmnzq77Zvnpt/agvHvEl0cRs/R+PMIDyfT
9zQ+rxtb0j40/npaB16kden1pyS+4Bnik5Yf3id+o32jwt9HfMdUE2znoPE7
Eh8xiPiq8ePpXDGUzlnLzyg6J5xD54aKP8EkT7iP5EvTr0JygBtJLjT+lPp+
pjxZWb40/m2SPyWPqPgTHjO50egJSRD1oojn3p0Bmg+C3MLqNuXTm/FpuYpv
fEy/Uj3X5afzDFOe0oQermLmkxUdQW6h7ODDG4psj4A+lesGTBrxXMuDILew
q9zQMnj7LtR0HjO8z4gXmr4gt7DSNSDz9ddXsUTeVeUGJfsln4LcQr5Bn1su
zv4YyxRxuWDmORS+ILdww7/VjQFZ4vAq/a/XL8gtlCe6mIWeo+kLcgslad3Y
kvah8QW5herEF7xOfNLnK8gt9CC+YxU6By0/gtxCMjpXvEbnrPEFuYUQkieM
IPnS6xHkFjwmXzHlycrypekLcguC3IJgByW5BUFuQbCPktyCYDcluQXBnkpy
C4KdleQWBPsryS0IdlmSWxDstSS3INhxSW5BsO+S3IJg9yW5BcEfkOQWBD9B
klsQ/AQQ/AQQ/GcQ/AQQ/AQQ/GoQ/AQQ/G0Q/AQQ/HAQ/AQQ/HMQ/AQQ/HYQ
/AQQ/HkQ/AQQ/HwQ/AQQ/H8Q/AQQ4gIQ/AQQ4gUQ/AQQ4ggQ/AQQ4gsQ/AQQ
4gsQ4msQ4gsQ4msQ4msQ4msQ4msQ4msQ4msQ4msQ4msQ4msQ4msQ4msQ4msQ
4gsQ4msQ8m8gxNcg5NlAyKeBkDcDIT8GQh4MhHwXCHktEPJXIOSpQMhHgZB3
AiG/BEJ8DUJ+CYR6Bwj5JRDqFyDUI0CoL4BQLwAh/w9CPh+E/DwI+XYQ8ucg
5MNByG+DkK8GIb8EQl4ahLw0CHUrEPLSIOSlQahzgZCXBqFuBUJeGoT6FAh5
aRDqUCDkpUGoN4GQlwahrgRCXhqE+hEIeWkQ6kQg5KVBqAeBkJcGoe4DQl4a
hPoOCHlpEOo4IOSlQajXwLWZj7KZfmT2Li6HTL9SwblOgwFUt9H1X6F+Cvke
e7Yz5Sk6k0M2059VcKGuaukSX3CK6R+n7hF0xPSX9b34//VTS4dqp7+YfTN1
KpSubvrjCl+ok1q8p3mHXTP8+1PY6ONAw99X+xLqoZbDLhMKmPFDaRy22Iwn
FH2h7mnBjgN3mPHJFfof1fkK9U1LOaIL2+k5mr5Qx7TwumEV7UPjC/VKSy3i
C0wkPml5E+qSluTEd+hD56DlR6g/Wp7QuUJBOmdNX6gzWtKRPIGV5EvLm1BP
tAh1QBDqhhZBPq2CPGPfypvfThyViIMiahVt3t5Vz8UIcoutSnY6fadOPI79
HB0zNOYxFmN8QW5xdNet4wrXiMakl3kWDZwbq+kLcoun8r0K+Zg1CFustdSK
P/Bcz/UIcovRhVefH//EEe3e3HhYyOvXPJEgt5i1Z6k6yZJ8IHuOynebnE/Q
9AW5xeRnRhdI7RQBPwcdHDRu4VONL8gtphxsw4NvTrbv6fUIcos5aR3wpYdt
XRpfkFu1T5hWxLZvPWckyC02Iz7CZeKrnj8S5Baf0jnBJDo3UOcryC0OJjmA
tiQXmn53kifoR/Kl4XXjBz+bfCIaPSxnZs66dkXPDYV4jW/WbkokJi5pbN+u
tb+G51617FjslkfYr8yqS4k3nqCaA+p946TT8BXemK9yjzcBY6L0nNGqwSE1
K8Ttx1aTT3fbmvBrvin50GLZYvK4g9eU8U7Jy/2CT2x9JHeTPA8hcH3ZxZsa
R2j6wfQZRtHfNf4d+j68HmKjp/H5efCOnq/nlfLS+mA0rVfPK3Wn/UAR2p+e
V4ql/cMj4ofGv0D8Auafhi8dtOvbspz30GfkzpvLch3Uc0BZU37B60me6N58
4PUia65qeNkjC7Kd/3ILHa+OfBxT103DvZq6xF68dwHDj6abs7jdHQ33zzDc
ddzq5fhhxZM6ucf9mmN675ajaL0VTjBvTZ8Fm/v8mnsKP5Cu44iZNyB38Osl
F2p7aHg++gyP6O8aPp++D2FET8M/0/NgWEbb8/WcUSitD/xpvRp+kPYDVWh/
Gn6d9g85iR8a7kr8guXEPw3PufL9jlXG55WZKtRcafxdzd0kW93zww3j+/3W
jmhRwqCn4G3t9y28aDxvVp+M254az9fzQVPCz1821udYpmjUMmO9Cp7rfZ/M
k4z91EpKEZfP2J+C58m/rkojY//pGzROv93gh4I3GXt7wiiDXx/eNlx8yeCf
gn+mz1if/q7hGen7mILoaTg/Dx3p+XrO6BitDwvSevWc0WTaD3ah/Wl4F9o/
piR+aPg84hfmIv5p+r39CxyYYsjr/aOFp8025FfN1zxwap9kyvfGRYtCTXlX
8CL/bDkYZ9yH7/em+yUZ90PBq9xP5zvCuD/vVw/aF2jcJwW/vTgkwrxvLyJd
c25L+DWX9KXN9Ujzfub7OnlQinK/5pjezzrxyrzPN2t7F9zc+Nfckyt9xlf0
dw3/l76PCURPw/l5WHaJ7fl6bugtrQ+B1otqPW9pP1iW9qfxV9L+0Z/4oeHX
iV/Yh/in4XE5j/SYZOjTbvcbDzP1q5qjOejun/6uoX/Lr0rbYpihjxV8sle9
i6Ye37cp4wNTr6s5oNt735b+ZNiD2n0Dtpv2QcFvxnfJPsGwK8nDitQrbNgZ
NdczpWLiadM+XSh8pGJTw14p+O7Tid1NO5e6cdxl0+4pOunpM26jv2v4Rfo+
9iN6mk4yeh52fW57Pir8mrQ+9KH1avgO2g/Opv3p+aNitH88QvzQ8FbEL3xG
/NPwwAjyn6oVIn9Kzcs08iM/zCkD+WUKXrck+XM5CpF/p+DlWpBfGNGH/EQF
T9mY/MujAeRvKvjhJ+Snlo4nv1XB81Qmf/dRa/J/FTycPmN2+vuv59L3cTvR
+0WfnoeO9Hw9BxRO68OatF4N/4f2g41ofxp+mPaPzYkfGm5P/FL803BBPkGQ
TxDkEwT5BEE+QZBPEOQTBPkEQT5BkE8Q5BME+QRBPkGQTxD0Jwj6EwT9CYL+
BEF/gqA/QdCfIOhPEPQnCPoTBP0Jgv4EQX+CoD9BsO8g2HcQ7DsI9h0E+w6C
fQfBvoNg30Gw7yDYdxDsOwj2HQT7DoJ9B8H/BMH/BMH/BMH/BMH/BMH/BMH/
BMH/BMH/BMH/BMH/BMH/BMH/BMH/BCE+AiE+AiE+AiE+AiE+AiE+AiE+AiE+
AiE+AiE+AiE+AiE+AiE+AiE+AiG/AUIeA4R8BQh5CRDyDyDkGUDIJ4CQNwAh
PwBCHgCEeB+EuB6E+B2E+B3c9+Vs5OT0Es9OXbfgUv8L2HFDrRFXHa/hTvqs
/g4KLuCDgA+jp/SbYM7trCiXd1efCVusnZhOtoNxdXsuugABFaeXShn9Uvf/
82e0p7/rvF/LfuE7je9bmB4wHRDwrQI+CusBYT0W/7Y75tXamIDh0y9seHXV
F0t1q5qYJ+AxNpg5qfSNd0+xVo+bE92TR+Dr1lNa1w4x/NKQi3n6NIvCiI2v
HpVvEqfxl8+94Ome/TKmLrDg567glziqQbGtvlX2WA9vOJJ06ZMnBKYv6bF1
wEtcmiJrnFOtPdYm9Qf0bF4mDPp+We/4zSERRzJ+e55vavLX7xF17mnnV2tE
PHTIYDcvU5nH+rnPhxwIy94gEVJcvJv3zkx3fMPrTFG/zu31pxLB+8P0Ug6b
LoLCb8zy5v5Xfumz19uRBa49g8SIAa4jW8eAWn+qgX2P1/GIhJtdbj0ZvPM5
LOH17x1d8lGVzQ9gWoTzlRx7X4Ba/+C8i8sezhKNJZrk6DKnqAfaXz2Uvm/G
eGwRFDJ9/+oI/GB3JO57wSCsNS9T4sZM8bg4zzbvVU1C8LzvWLvKd8M0/trr
YQMGro+E6RWTCr5xf6DhyVYsWrpvSTRE2ReytMuOmo5fj6qTfB9EQ8VRMU9y
1nQChZ+b9dXcv/I5ey8Wcw+oeBdnDMrVcO/cM1hgftFBx6q+xOa7+3q13eGB
fgneGc6uQEzyfxyQvNpLHOudM3xqmlt4prXHqpj3bhr/WuDIHnsye8H3wo1D
i3le0fBymULcBz28Cx0GrvJ/Zn9E02lcMleXqk3uQcXlDmsDG2wBhR/Cdiri
rzzJrfb1mgZXvAt+y+YN3D/3DKyu6VnBxF/dtXlAhx0eUC6VS/4LKxCWJNu2
16RfLm9k5RlpbsHh7WP6P33vpvHLLqi/eX9mLzxTNFW7kp5XNLzDlIO9hz28
iw0WbD6dYH9E0xkzOMfXGk3uYVe8Mzq4wRZU+IfZPyn7V/5hXUJouHHucMP7
faW5RT0g5T6c2Mfgf8zHDbkPrI6AiDa5L/0oGATuHxp1Mc/r67ur11c3CYFl
Bb2gyt0wjV/wea70g9ZHomf5zb7GuWv4tcMD6u1fYtjZsq0Htc+Oms73qzcr
+j2IxlTza6fOXdMJFf5r9kvb/xXXT8yVJta473B9xBEf477D2cpLa+U27sWF
2Q/mGfcdLoQUOW3cdwgatHeHg3GPbk9r2cO472DXK3O5Ck3i4Azjp29wvHzt
EfGY3TtdtHEfwYnhrvFnGudokIhz394ZYdxHeMh0jsGJchtOJeKGN84zjPuI
ik51jkeecDyi8gBPouleeY2le6bitTPd6R5mGkz3UsGLRdO9db1H91jBV3wn
/bKsEekbBe+eifRRtc2knxR8XWHSXx4LSJ8p+GOOQxv/FYcK/ASBnyDwEwR+
gsBPEPgJAj9BkE8Q5BME+QRBPkGQTxDkEwT5BOG+g3DfQbjvINx3EO47CPcd
hPsOgv4EQX+CoD9B0J8g6E8Q9CcI+hMEewSCPQLBHoFgj0CwRyDYIxDsEQh+
CAh+CAh+CAh+Agh+Agh+Agh+Akwie48uZP+V32K5RP4BZid/Qfk5lkjyJzDF
HZteUn6RZQj5PVCO/CAczfBg8pNgJPlNyP6GJRX5VfCN/Czlb1g2c513Hdd5
K7Nf1+vYwRoTWyVipkS7DhVT+aHjrcLPeq4Nxi7lZ/ZMKBWPWaDBC7czEfg4
+cZD/asFYuXXWTINWxeNYaWyLV3l8hRPlc9waCP64ciZAdmtaZ7DuYIOy59/
e4znmc7ouw79Lm9NgqtVbtpPeueBT5nOpFu16lzc8QKm707WL/UcF7jAdJpy
H8Vh7qM4zvW45lW8plwZHoMejkUa3frXE/1Khlgh9inuTb2ncMO4KNyB+5ZN
vR6MwU/SjcXjcXip36x/O8wNw+63PlYu0tGIE7cfWn+rWyyOtAvo8ONsFOxJ
U73ezm4BGMB0Ag/uihtgFwcr8jz6x3vqLQxlOrC7eZ2x15/C5AfZKm4echbS
MZ3m3He0k/uO9vM84A/f8qt2pvDDd3MGPy70+Bz2z1+6YuNbL3DOqlubugb5
4aGRB76FTXbHK02ufvtQ4wWufGG3fo6zD/p9dk2MOvAAXaq6NH+wPgnrPHg/
5Pslb1heqOYO54E3cQDTeT9lUTqP1QHg/tkpedoz5/Aa07nT8Pg4rwHBULTN
txo1F+2Fy0xnFfenOXJ/Woiak3Vdn7pug9two/jVAp97nAT7Wv6ZU656iZfs
0p5+POwi2Hfpk/FH2Fko//P9XDNOqFYtfdLkiXtgZaa1J6vUOQaRVRasrfz2
JXY76l50XBdXdO5dIGPKfeegBNOJG3C9/qaZ1zA2j1+/JSeXazp9Xn+eOjbu
BD7x+zQsaNpyjGA6D1O/aGz2MQ6Z2PXf3+cEx8KeHx0PRYJXhyo/89d0hx6V
063KXuA5DnrfISnzu8eQbezDzGda+sPmL8uT5y6UgLmWvX1erYU/+JTu7NE1
fzBExlZZeDltIrqfbO0SPiQCPcefd+uZ1w+6MZ2JAXHRs89HoMu34icfzboE
G5jO0XFneh90CsP8zosWHYETGM50vnJ/7Fnuj7XyHNbxzIWX7bzxHHa2DF3n
NcsXQrpZKp9KHo7dNuw6tXBEHFx2dK1o6RYGu9yPZhyTLRILuy1J12xCBJzu
d21A7doxkGKYZfvuNVHYcHX1Mp+szzCg7HdrSLlH4M10VlvD0tXZ9Bzx2rO6
aWbfhi1MJ0Ozst6nXsRj+m1T3y8ceAF/DCU6Dbj/fA33n1fhuZvaRdbaP1n3
AsLaPXteaccDKP3qRuWGZ09ipS7hN0aOfA5Rr0a4HzsbCSUezGs/2vUaFpwx
M2Hoj2hoOO/arIJDnoHd+XaD3yxyw58FUx53rpSIs6Iaxh3NHgY+L4lO9zJn
rw+v/BLPZ1v0OsNpT3jpR3S+N/d5+GKg4af//LmqecIlTMd0OvB8RxzPdxTj
OYsUE4PWGHoJDp1NsdLQS7BmcvUyPdYGw75iIbsMvQTXHZbZuZ+JgJJZD4f2
rRYI97q3qmHoJTi2OuSqoZfg3dzcAevRDxaFtx1s6CW86OPyxNBLsIXpjAgt
7WToJSy54lYvQy9BdaZT7/P52YZewpElOuw39BJ+ZzpteJ2RvM7J3Fdfcdau
oleHx8DpPS75XP/1hLT/bS9o6BNYUf3gKUMvQeaN75pNux4M3ft+vm09HgfZ
9m2KMfQSBHh8Pm3oJbiwxD7sZrdYqLkj1xtDL+Gi2DNPDb0EGZlOxyUd1wy0
i8NkNy+vNPQS9GE6Yd+KPDb0EiY22XXR0Et4ienU4HOfy+fegPuoK0dUaLI7
hR/0qrnmU5HH52BpzbDERrdewM+91TN1D/KDKbXafoyY7A4L7+0/ZegTCKpv
cZ3n7AOu1T40ijnwAJ5b/JL81ifB2nn9tia77I3tj31ueWHgTVjGdDLmDnbz
XG3oU2tc73RnzsESpjPp6J0ydwcE4833tZPXXrQXE5nOY75HS/keeXC/8aFl
s2c4NLiNrfGz/7seJ7FEstw1DX0Clvc4PXDYRXxUcs2zz2FnccRE3GHmM86m
vR8xYOIeHJbzbe6SdY5hvWqBuwx9Avvztw8f1cUVWqcavern3nNYnOkMuPbh
9aqZ16Dh+/OnZpxcjsOZjjX8w4p+cScgRfvkm32nLQdFx/PgxPqmXtrlczHn
7/3VzqNHNDD0EqY+t6agoZewQu9CeXIUeA6Nr8Vty/TuMdau37yCU0t/vJOi
ynxDn0D/nv+EVW3hjwc8Ny7okj8YfZc2/2joE/AavDCDoZfgbMTHDIZewnJM
52CIvZ2hl6BnwshBIbMuoQfTCfF8dnm/UxgcKryh2WE4Ad5M5xTr+Qes5325
fzVdhbA8hl7CBlVvtTX0Ev6o3uzh6eThMHHpgvqGXsIUK98e/K9bGD7v5PbQ
0CdQq+fR000nRKBlxuKPtWrHYMmBt3rvWRMFjSf5un60PgOP3o3aGHoJ3zOd
CX4/V9Te9BxSDIjwTT37NsYwnf5nvRsYegkWlO6z0dBLUIzpcD8xOLLdXMf9
imnS2eUz9BKW8G8VZ+glDN/i36jD2ZNQ3ieVo6GXcG7VmpcNvYRHO83zn+R6
Ddr3G+Jr6CWclGpUH0Mv4X9pQgp8WOQGY6aF7zH0EsTtnxdk6CVczHSu9Jp9
0NBL0GLFQH9DL+EYpvNPz1pOhl6C9YdcZhh6CUozHdXPeYL9kO7cnyasE4V1
osB/FPxAq7AvFM4LBb/RKpwjFqJ9YibaNzZg+pnID8Op5JfhcfbH5hG/4ALx
D6/yvt6SPwdjyL/DQPbritB5Yz86f/1ce/KfsDH5U5iK/aifdE4whM4Nq28l
fnqTHwnDyK/EvexPViA5gxEkd5jIfHAhvw1CyI/DIPbf2v2VJ8y7rJWjmScU
5AT78O9TdeLfp8rLeb9gknuoTfcA09bgcyQ/EsaQX4n32Z8U/HarIIfYg5/b
gZ9blp8r3DtswL9XVpN/ryyF6u/6/3GBVZBzFO4vCnGEVbgXKNx3FOIO6wiS
b+hF8g49eD1BFFfAfxRnwF6WQ0Fv4Hi6b1CN7h+k6Uz8TE1xCwygOAZcWT6r
0n2A+nQ/oByvswvFCeBHcQMkY/nMRPccMtC9hxUsn58SbPESDKL4CTaxfMau
sN1PmEL3Fd4xP9dTfAKnKF4Bb5bPUqQXIA3pCcjKcnWK4hzoSHEPeLFcOcS8
/Tn3+VO8fDjh4oc09zEh2/gfCV+jUbBHWDMw6EZK9xisHJ5qrtutx7gg75v7
Zw39Itgp3BSfavaui+EIqzYWXNo+GhOZvmC/0B7KD6mx2RkTD5xI0z/8mV7P
tWYpvxeZ6wrNC96KSzP/mX5uKbJb6Et2DP2YzsbWtrgFgyiO0XFQJI473Wzt
Q9hbdt3RyAxxmv5Ksouwg+wk3uX1f6K4CFJPtcVJeJnjo/08d7D9r9/DuUh2
F6xkh7EK8+0UxV3QmeIw7Mvx1yFaB9yndWn+VOf7WJ7vY3q+j4J9xwH8u46d
+Hcds3IeuxXxC9YT/zTfBH8AE4jvMI/OAdR6BD8B2/I9Osj3aLv6nTQ6bzhG
5w+Kz4JfgQVInsBC8gXzeZ2rST5gMMkLBPBzAym+hZMU78IFPt8NJMdQm+Ra
r78IySUAySn48X5PUvwMWyiehot8vlFnbXIPJ+keQHXm83OKwyGXny0uhz7q
HP+/n4mCn4mCn4kr29n8RexP/qOGD6f4FlJSvKvj5Trkd0JO8kM1/XCKk3EX
xc1QgePliuS/wjTyZ9Ge17mF4m2sTPE3lOS4ux7LVV2WqxwsV4I/jBv4d1zX
8O+4VuY6heAno+Ano+An448cNn5BHeKfxi9BeQR0q2rLK6DizzbiO7x/ZzsH
GMnPje1sy0fgBMpPYEXmTzo6P5hL5wkleV+zKK+BVyjPgaWYP0IchEIchEIc
hBUpnoF5FN9APMN3nbPlC2Av5Q8givMGqdEWF2F+ipNgAdOPo7wDjqA8BGzl
/MMUiq/wBMVbsJjX6UP5C7xK+Qzow3mMC3yOR/kcVZ+qEK/hEf6d5J38O8lp
ue4jxHEoxHEoxHHoQfzCj8Q/fMr4lykvg/koT4OxzJ/ZxHfMt892DjiXn/t+
jC2/g30p34O7mD8d6fywKZ0nLuJ9HaY8EQ6nvBH2Z/4IcToKcToKcTrupHgb
oij+hvMMP7LVln+BtJSPgZSczylHcTt2pTgeujH9RMrj4FLK68Aezufsofgf
a1E+AFLxOqMpH4S1KD8EkZwXcuZzPMznmIzPUcgnYCP+HfK6/Dvkk7mOJuQZ
UMgzoJBnQF/iF5Yi/qET4x+kPBeWorwXpmP+pCK+4zY6B2zHz91N+TLsQfkz
PML82UTnh7XpPDEZ72sh5d3wGOXh8Dnzp2tImnXzDLuSMoXX4I+GnfmvfYkG
pn0R8ktYslxBSyrDnt13HGvvbti320dOZTPtmpB3Qkf3rHV3G/byXO13G0z7
aWH6Qj4KU54sEWra6ZJe83eZdlvh12m1dUJRw947hCybkdaw/+q5P4vb8lDQ
kPJS8IbptKM8GvygvBoU4XyaR7U+dZsbfskc/+7logw/RdHvQ3ku5LwXFOP1
P6c8HXamvB3U4bzfff59Azf+fQP1u0B+lEfDBZRXg4XMt42UB8RPlBeEWZxX
XEDrQGdal15PM5ZDC8vhAJZDIV+HD/g9BR78noJ8Rzxhd0m3a7WJX5iV+Kf5
JuT3kPmOPekcUK1HyPthXZbzpSznig+OdN54jc5f0xHyhHiZ5AnLk3yhWud+
kg8MIXnBOH6uA+VbsTTlX9GBz/dRcpscYw+Sa/1cZ5JLPEFyitl4v4GUz0UH
yu/iEj7fNST3mJ7uAU5nPrtTXhj/ozwxPuBzdMxcO83SkCRI8ezHxHYFHkDF
7g+O/1h9T8onw4xcP0InvoyHWn7bb3XtFAmHhr9sV3bTPSnPDOervug0/k40
7O/z5MQKu2dQmekL+Wdomzf8Tarxl6DHo4Q1pTO+hEqMH9ap78ZuXT0xcP6d
H7e3voAj/FzhvoBwv+BG7ix2p4c+wYgdw0/GL0jQ9IX7AsL9gtv8+zaX+fdt
WrM8CPcFhPsFcbQOPEbr0vwX7kVdIT8Pwr2DYOIXHiP+wWHmm5DPh27Ed0Q6
B1T8EfL80I7vRSzfC/V7StvovPEGnb+mI9QFoBzJEy4k+UK1TuG+gHC/IOGp
TY7xJMm1fq5wX0C4XyDcFxDuFwj3CIT6kXSPQKgrSfcIhHqTdI9AuEcg+Dkg
+EXSPQLB/wHBX5LuEQh+EQh+lHSPQPCLQKjHgeB3SfcIhPqddI9AqOtJ9wiE
ewRCHVC6RyD4aSD4ddI9AsF/A8HfA8F/A8HfA8GvA6FeDIJfB0IdGQS/DoT6
Mgh+HQh+HQhxIghxJQh+HQjxIwjxJgh+HQhxJQhxKAh+HQhxJQj1dxDiVhD8
OhDq9SD4dSDU8UHw60Dw60Co+4Pg14EQ54IQF4Pg14EQ/4IQL4MQ/4IQL4PQ
BwJCHwgIfSAg5LtAyI+BkO8CIT8GQr4LhPwYCHktEPpVQMibgdDHAkIfCwh9
LCDku0DIj4GQ7wIhPwZCvguE/BgIfUog9CmB0KcEQt4ehDw/CPl5EPL5IOTh
Qcjbg5BXB6GfCoS8PQh9ViD0WYHQZwVC3huEPDkI+W0Q8uEg5LdByIeD0EcH
Qh8dCH10INRzQaj/glCfBaGeC0JdFYQ6LAj1UBD6/UCot4LQBwhCHyAIfYAg
1BNBqD+CUB8EoZ4IQn0QhHoiCHVDEPo/QagbgtAXCkLdEIR+URDqhiDUDUHo
TwChnwGEuiEIfQsg9DmAUDcEoT8BhH4GEOqGIPQbgNBPC0I/g0WoG4LQfwtC
3RCEvlwQ6oYg1A1B6OMFoW4IQt0fhD4BEOqGINT9QegTAKGOD0LdH8p+P/5g
iRHnHHx/vpMZ95RasaFbijX3IHBuwTNmHPU2055DZlyVJmrwzwpGPLUlLVYx
47EqR2suMOOz0oxffcpRTzPe+/qkVXcz/lPwkY/qDzfjxiOfH3uZcWRqpiPM
zVmuHm8Uf8qIY2svjJllxrWKjjBPZ7nFvwfl8tfvmAlzdpb6RBcO0HM0fWH+
zuJI64bqtA/Nhy+0TzhM+wZFpznLiSPLifqdw1LELzhA/APF5zjiL4QRv0Hx
R5jLs6yic4KKdG76ucK8nkWY17MI/fxWQR5QkAcU5AEFeUBBHnAwzSFgKppL
wNI8tyjIA1aleQZ4SvMN+IrnHwV5wBE0FwFtaU5Cz1EK8oC9WI91ZD2m5iUF
eUBBHlCQBxTkAQV5wGCa94AuNP8Bij+CPGA5mhuBFjRHAmqO9Q7NmcBDmjvR
86F7R9rmVfABza9gCZ6jLFPSNt8CP5fb5l303OUKmoeBFTQfo+c067LfUuXP
906iI83VwGaas4GSjB9HczjQjuZyQNFPQ3M7MJTmePRcp9cy2/wPNqR5ID13
uZjmhaA0zQ/pucscRWzzRXCR5o00fjv2q5v++R5SPEpzSjCd5pb0nOZVmmuC
9jTnBIr+IJqDgkM0F6XxX1pt81Qwluar9NzlJJq/wnY0j6XnLj1oXgtL0PyW
xj/Icd+OP99Li7to7gtr0hyYntPMSXNiuIXmxlDRv0ZzZehBc2Yaf/I823ya
mleDVDxHmamcbZ4Nr9J8m567DKb5N/yX5uH0nOZJzpPs/es98jNpjg7/eW+b
q0NF35vm7vA5zeGhor+P5vRwI83t6bnOUjTvB440/6fnMc/RfCDeoHlBPb9Z
muYJMRXNF2r8JpwX/Y/zour3hXLQXCJ605yintM8RnOMeJ3mGlHRP0pzjziV
5iA1fkean4T1NE+p5ytH07wldqH5Sw3fR/OZuJDmNTXc/686iIJfoTlPrEhz
n3oeMxnNheJlmhPV8LM0R4oRNFeq4QI/QeAnCPwEgZ8g8BMEfoLATxDkEwT5
BEE+QZBPEOQTBPkEQT5BuO8g3HcQ7jsI9x2E+w7CfQfhvoOgP0HQnyDoTxD0
Jwj6EwT9CYL+BMEegWCPQLBHINgjEOwRCPYIBHsEgt8Cgn8Cgh8Cgr8Bgt0H
wb6DYN+hS+WOHZ6sTcLqZVs3Lhm2yeqUf8i0DfOC0a2US9xuQ8/Uj1/ZLeWb
tdZtLZrGmXrnXpVixypme4he69Pf2F/8Bg4JHB5Td2MS9i27P3St8b3znwpc
utdyvrX1vrOhJt3u859s65bfH+d//u42brq3xp/V2ndstuR38L1z3WmdzwZo
eO7y2epuv3EBp3jFJhvu/1DDg/zOX3wYfRiCGk96m9QsWMMnl90w6Vs1N3ix
dYSHc6ogDV866t2IkKZ+sJj+1/BXhAej6Xsa/pDown56jobzOqAirQsU/A2t
G5bSPjR8Cu0T+tC+Nfwm8QuYfxoe3HmCw/7RqyxO6b3q2sVO1O8ZDKxZ1NF8
v8A441u/v78vKiBlfKlVSy35r4w73MpjgcafMySpVJE08ywHLzcsPX/ecg3f
XT3rJl/vEZZPW6xTf/RareFXerROVu9LD+uWy13X+Cf9gn8s+qnMzaIzrK8W
jCmORX+9b/Fk0gnnFK0WW/l/DWc8K39Pw+u1a2rStfJzNPxA+dPmOqxv9xQ2
16X7J3nd1tI735v70HDep9XBktPct4annhhi8svK/NPw+qcWt6ps8NejTd24
Awa/w+sMW2byuUDPHO+7G+fRc3Wf1xOM81Hw1P0vN89hnF+6cd1fdzHOU8Ht
V8323GGc94nI/M4jjPNX8DJzDjYOMeSjfO50p18a8qLg/1XK/PG7IU8FC5zp
dsGQLwX/Mirq8CND/j7S/xrOeFiNvqfhTBc/zrY9R8OP0TqwHq0LFTwVrRuz
0D40vD3tE4vQvjUciS/YkPik4XetVd7cN+5twyI+29cb93jR6QnzzPtbeHXz
UWt6xsLM/q8eX4m5D3VqPK1j6oHw416vzXxReUd/L9Nu1mV4urcN57jkCIZ/
538YfqNUtMa/gpWs8YZd6B3Y+q0Zhyn8Se9bbFjZ6Dymepq6Q5VZv+BBrVKe
PXzAHy17/qtl+g8KvrZNwYLJkkXgKvpfw4Hw0Ju+p+FMF5vQczS8J60DH9C6
UMFz07oxB+1Dw4vTPjGK9q3hI4kvWIT4hGq/O+F2s1SG3jwEedabnxV+sUc+
yys8TYAProFfUi31h3mlHK6uM/i9pfnyCXftnsJtn5i8s83fA2D4y9P165+d
FA4J4U79YtPGa/jhkq8upm13B7L1dL5eyIgjFbzGmxoujjOu4IQiJ0Kb9kvS
8FH1fDLYHw3BilMKdB5+4LmG10o+KVuLpzFYnf7X8MqEh4PoezCH4ROJLn5+
bXuOxs9K60A3Whcq+DNaN36jfWg40j5xB+1bw5OIL1iC+KThA0vVWGLap0H2
G06bcqnWU8qJ3rPu1Zzeux5/5ebTdfdc6g4gPAh8P8L8nmU54UNsKXpfe0Ic
vb/9GeNX7kXvd194j973/pzhl/fQ++B/XqL3wyv6C+Pp/fEDy9D75BWdSv3p
ffOFYun98wruy377HfbbFZ0ihIcl6HsafzDRxQb0HA3/QevAUFoXKvgcWjc6
0D5Q0Y+lfWIC7VvDrcQvLE38w6cM//ra9p56a4ofpE8V/j6SY3DJmau5IdeW
eiTPUIP1LesViCR9ArVIj8COBX1MvWJZSvoEcrIebs16WOF/6kd6+PtY0sMK
np/18F7WwwpemPVwUdbDCs76FLOzHlbwt6yHX7IeVvCcrIfLsB7W9FkPJ7Ie
VvA9rIdrsB5W8M+0bkzBeljBG7Iezst6WMFdWA87sB5W8PHN0tc0/YGDTx7U
+c0fAI8xh4+a/sDWd5uK/f6+oTkp7z01/YGr/XYe+s0fgDv1C9v8gWdzvpb8
zR+AmC8eG0x/IO5itSm/+QPQZXmun2D4A5VOvlj1mz8AdZ85lTb9gYiwXMV+
8wfg3YtN50x/gP/XcMaz8vc0fGHIfZOu9Weh1OZzftEv3Nhch7X2oOANv/kD
YHmWsZTpD/A+NHzwxxXmPq0Bo0Kf/uYPwCHil9Uu/FON3/wBCGR/1Zf91YHk
d8FA9kvXsl+q4MvYL/3JfqmCF2a/dD77pQoexn5pJPulCj6X/dLP7Jcq+Dr2
S1ezX6rgX9kvncJ+qYJHsV96gv1SBed1QC32SzUd9ks3sF+q4EvZLx3KfqmC
e7JfGsR+qYI/r/bGxfDjLSPIr4eW5M9D88zT76427NzFYY++XDbs3maKCyC1
ZfMo0y62atz3vRlPbWH4nMOfPS8adnS7s3MIGnZV4de/v8LetLvpWlRtbdph
hZ+mjdPnFYad/pLl0rHKht1W+OVPlilg2vUhkdf2mnGQguewb9Hy589wyET/
azrDCA9K0Pc0nOnCw9a252h4WloH9KB1gYJvoXXDMtqHhjekfUI62reGOxJf
oCXxScPtc9Yx4yiLF8VVGp4yj7WKaecGO8ddN+3eUYq/4GrfEWVMu9i4UP/w
WYadVPD1ofnCzxh2tM6iTdlNu3qE4Z8jAgaZdrfn9NsDTTus8Gsd3DLItNOP
jiWbb9pthV/p/a0rJQy7/rpg8hSmnVf43S/U8m1u+AGd6X8Nf0t4UIy+p+FM
F74esD1Hw3vQOiBvpG1doJ5bg9YNu2gfoPDr0T7hJu1b4/ckvkBq4pOGu/XO
a8arlm4Uv2o6hexvLCv87Cku7fTfVPe+kZirWI4m/VYG49OnO7qc8o/AYaGz
o5a2fIbec97M6m/Au+duV8l+mA/uybn+5JNZifp3gR7VDj+c/qMTHDj6NXLW
zCQ8yvCexV8OTQcPIVUa76tpyyZo/COnM+U89Doa3II2rfl2P1bjO+e3X3eg
UhLM+LnscNEZtzAHr2f6/h+tQnMkweb6m/svLeQGPryesQVeni5xLh5uLw8p
lLHiE1D0K80JdfhvbDTYe71rs7hBHCj6JyNL54v+GgiddrpffDc6AY4xfFvz
/OebZDoMvhfHVW25OEnTKT2+aP6gRY9x48LnRyYnRKLnrpH9xjrG4Y73o98V
rH8P96ytNGPypFj9+0JTyqX0rOezAjJUrfffjM1x+FD97tCYIzOjRvjAvymH
ld1b/hd+ppd+07P5hUIeXLyy6eBIDf+ascaNVUMMPbG0AN5e5gLu/NwiOdqW
G1omBqp3yt6wRo8AUPjT4pNd3dItHA73d8j1tmEEqOcuybv0gfXhA2gUlWbj
5KUxGt74075F2eLOQI7BPQ8XmR2n6bi7PXu/f9dNXHD6Xdfu/kF63nZ9Ys7g
dNeWYY5eo+r/mPhrDnd0vw8/cg+8Dg1Hf7+RuctDDa+SpnbYiWb3Yfj2aw0O
bPfX8MOTijs87x0EHwIGHzjq7woKnmZYn41Dg/xgd6VMqddZfTW82OAxszx3
ucP1ZSciqp4L1PA1q6t0rffPSbgzJd/T6TWCNXzcHrS9nzFoY7lGv72fEffv
9Jhqvp+xvkPi7+9nxGwf+qw038/o4Nr/0m/vZ8T4CU629zNGLT71+/sZsUMH
H9v7E69/2fr7+xNxUtlBtvcnLp6QY+Nv70804kJ6T+KAUnVr//aeREzGc0ZH
ec5oI/eBvx218YFxHyHl+7JOxn2E4Dk5lvVZGQxXpvsuN+4jBKQ5UXFZy2fw
MeqNta8BH9/s2ErjPkJvv/5pwmYlajobXfaWzPDRCQ+/bmWZPTMJNjD8XK9a
5437iLlHNcyarmyChl+Oe9vRuI/oft7Zz7iPGt74YlyCcR+xadYSwcZ9hBBe
z6xE96PGfcRmU9OeMO4jqvUMd9j6w7iPuHzHhjnGfUS1nmGeQycZ9xG9Un7a
bNxHDZ/j4NjDuI+4sea0TO9HJ+B6hs+zrA427iMmPcy1zriPGn85z+Vl4bm8
ntwnnCFVWKxxT+H0kZqVpiREwmvv1hfHOMZB11x7rheqfw+6dW1VYMqkWOjB
+KevDOvfwGcFWqf6PTXuqaYzIVfxydEjfNAhe4s7xj3V+McaLOqW3S8UT++K
/qfZ4EgNr7Lw87DVQ+Jw9t1BK9yWueArfm72ndVdjXuKO8buijLuKSr6qTKV
P7i1Wzg+6thjg3FPUdHx35ayJz58gIMnzKg4ZWmMht+dtua5cU9x7aT1bYvO
jtNwX54/vcnzpwncL+r+T9ESB3fdhMAzxz/38A/S8OKdjsxKf20ZZN96+/vP
icEa/nDPxhr5Bl7HpNz5hmTt8lDDu+Vukux0s/vYc1m3/Qe3+2t4xecX7yT0
DsLTSS7/Ovq7ooIn71bNb1iQHyZU2V1nvdVXwyu0aJ72zi53XDHfY0z1c4Ea
fuR67swN/jmJNdPl2DezRrCGty6Z23Z/A7pcbvD7+1W73Bo/w7y/xQLGT/r9
/arll+a0vV/1gVOGP96vOqTyAR/z/vZutG/F7+9XffQ0cpd5f382GtPu9/ef
zjo47Lt5f73Snlz/+/tPe/H97dQXHX5/zynbCbxMdgOPs71oT3YFL5CdQWVf
WM9iJ9K7Ws8XIfuENcleaToNSF/jKtLfGv8K2TmcSHZP099Peh8Xkh3Q+OXI
TmBxshsafofsLuwnO4xHmM4lh4EfLW0fgmVGJv/rXaOxnPXWoCwn4zAt2W9o
R/Zc2/F/yW5BerJj2t4dJD8AepJfoPHzk/2Do2QP9Xp2kT8BU8i/0HaZ7Sjc
LWOzq7/wye7CabLD2n65sp/DeXpQ+XnfhJQfnaxuUO1HoRb2CbFQlvfVlP1z
Z/bPVd7ek+wK7iU7o+1XCvbbm7Pfvp3xL5N9wrJkrzT+TPbn2S/W+NvIzuE/
ZPc0PrCfn5r9fIWfRPYSj5L91Pip2P//yP6/Wn8Z9v8Hsf+v4FViJ5Ybd+sS
uJV2GDz42EPcFdlw6PANSZiV44IMHBco/BVZDjb70vYu9BrUaHrZ5gGYbt6r
JiY+xxVQlOMFtc6RZO+hLNl/vc5PHEcEcByh6DcivwF6kB+h8Xn/0IXjC4XP
/gcsSbL5I9rub+S4YzHHHWo9c8mPAW/yazR+PY5H0nA8ougf4XikGccjis7F
dJu3ZD3vCfmzXp+dPH8gKL4V4TiF6z+aTuz2oyN63TgHzzIO2Fc6WTAovvUj
P8Cy7s/3U+NC8hssc8mP0PCL5GdYupHfoeFulTLnNfVkmjZ2f/g5BTKQngwb
+fwPP2foyEw2P8eu85g//Jz9aVfb/Jw7J9z/8HNiyO+xxpAfpOG1yE+ypia/
ScPZr7L+t8vmZ2k/J4D8MOss8ss0vGWZRJsfdXLrtD/8qJAdTjY/asf8n7+/
hxqjOL7muhmoetnl0oE2/Wy/o/of/lV9sivwiOyM/h2GKisudShrnMf6eQOi
ehvn4xKRO2GYcS5ryT5BY7JXGv+U+6lPKYzzTjfOZ0h24/zrNx6R0TzHJLJz
kJnsHjxj/Gi2y15slxWdE2Qv4T+ynxr+gO11ONtrBS/F8z6LeN5H0Z/OdrwA
23EFf7Dp+t3xty6hn3erXEOPPdTrfMz2/RPbd4X/b5qMZb61vYurl/i8KNc8
AC4yH/qz3R/Mdl/hD6PPyH6BXuda+j5mI3qan5/peXiHnq/xeX2ItF69Tt4P
juls2x+q5y7k35c4xL8voeiEEr/wEfFPwzOwH/Ka/RBF5+dY2/nhOTpPVM/9
j/2TDeyfKPx170ba3rvNdUUtV0tIbpQcodrvbfZnGrA/o+hsIL8NPpAfB134
dwymnUr20rRD27Z8OHrGsEs319y8ktmwR1xPggOcB1Z1pSHkF0IM+Ymazpq/
fqeiK8MDyb+EfORvavy15I/Cv+Sfavgw9pN7sZ+s4EmXyU92ZT9Z0a/C85jd
eR5TwUex/2xh/1nRyRX8cz+0NeKORglDrF2j4RbvV/nVzuxXK/yL9Bn30981
PB99HzMTPc23evQ87EHP1/g9+PcrCvDvV6h18n6w8SXb/n7h0/5xJvEDFTwz
++2P2W9X8FD224ey367gq+hccQadM6r9PmV/nuVC47+kOAeeUtwDy3juPhXH
ifs5TlzOcK6TwXWuL6h62XqKo2ADxVWajivFXTCe4jBYyvBpFKfBOIrbNP4i
jiv7cFyp4O05rjzKcaWCn+S4Mh/HlYq+C8eVdzmuVHAf+ozO9He9r8X8Ow9u
/DsPir4d0cUD9BwNP0brQO+LtnVpOMfDuI/2gQo+iuNQT45DFXz5X3GoWqcn
x6F8Phqf6z0w+KOzWf+xqLoPx6twgM9Rxa1ch4OHXF9T9biSXI+7z/U4VUdb
x+fI69J0Yrgel8T1OIV/m893NJ+vit8rcp1uEdfp1DpvCXW6OSwPo1ge1HMn
cP1uENfvFH65v+p3Cu7N9bu7XL9T9TKWA7zA567oF+W6XlGu6yk6efncHfnc
1b5Uva/SX/W+IywP2UhONf3vXAeM4Tqgot+L5WQNy4nCj+M6YPxfdUDXv+qA
iv/FfWb8UQdU6ynC9et3XL9eyXXVNWwXQtguqLheyVXdf8f8IVebuN7tyvXu
FUxnINuFx2wXFJ0kroM/5zq4wld2IQvbBYV/iOvjWbg+rvDXs73IxfZC4Vfn
uvk4rpsr/JFcNy/PdXNVR3bgunlVrpsr/IxsF3KyXXAnPQmVuJ4+gOvpCv8s
24V9bBfUesZznf0j19kVfm22C/3YLij8zFx/d+X6uzoXZLuQiuyexo/juvwX
rssr+l3YXoxge6HwrVyv38b1eoX/nOv1Rbler/jTn+v1XIfX8E1sRyazHVH8
seM+k8ncZwKqvsx+7xX2e1VeJZT7T0py/4mqRy9hv7cm+70KPxX3pXB/h8aP
Y783Dfu9Ct+F+1W6c7+Kwj/Cfm8V9nsV/jjuY0nGfSwK35/7WGpzH4uCX2f/
1p3924bkv8Fq7m9Zzv0tCj8D+7eL2b+9RH4a1OW+lzvc96Lwe7N/25b9W7VO
Xh824H4Yhf+C/Vt39m8Vflfuk/HmPhmFn43923/Zv1X42bl/JjP3zyj8B+zf
erN/q/ALcV8N9xNp/CHcV1OA+2oU/CP7vUfZ71V828z9NtynoPGXsn9bkv3b
y8y3DpT/sgz7M24FD8qXWZZS/kzDP1F+zRJB+TYNd0kxPbcZt46LCvkjv5c5
ruBkM259PmryH/m9kbtcbPm9zKteX/w9v/ci7VNbfm/zp9F/5Pf6Ur7P2p/y
fxruT/lBaxHKF2o45xOtbyi/qPN7Dyj/aO1O+UgNf1XzmC1/mDfjqT/yh263
/b6ZcevOeP8/8oeu3CfG/R2o+joKXHo8xYxb786890de0Z/zTsh5JzUX/Jry
CPiK8gqYhvIJcJvzTnU476TwS1CeAj0ob4E7KF8BeznvlJvzTgp/N89xB/Mc
t4K/4byTM+edFNyf60SruE6k4LN57juS574V/CnXjwpy/UjBm3I+yofzUWqd
47mu1JbrSgp/O+ejBnI+KjXzoS7XmyZxvUnhT+V8VHXORyn4UPo+bCJ6mg4/
D/pyPkrh+9L6oDqtV6+T9wNhnI9S+Lx/+Ej80PDlnI8K5XyUgmfl+tchrn8p
uJXzTvacd1LPrcx1MXeuiyl8l1J9zX6X69z/ouVK5aPecj5Kyc8prqMFcB1N
0cnF+eqhnK8O5LnjmpQ/xUDKp2IpyqMC9z3gNe5nVv0PXHfEjZzHVnQO/jVX
rua+93EeewXnsRV+Wc5j23MeW8EPcx12L9dhFfxzWarDZuI6rII7cx3Wjuuw
Cn6D8971OO9dmveVheuznOfW+AX/ynsHMLwBfR8uEz3Nn0Kc9z7IeW9FJ5Tn
yo/zXLmiw+uGFpz3Vvhcb4YrnPdWfJvH9d8jXP9VdLh+AI25/qvo3OF8OJ8n
qHV25bown7/G9+N6RzzXOw7wPGwfnhNPx3Pi+xnOfRvYhPvhVf9GB66PXOf6
iMIvxHWQ2lwH2cfwi1zvmML1DoU/lPsQ9nIfglrPGe5DOMR9CAq/K/chcB1D
4x/iPgRP7kNQ+Pf/qo+oOW41f408f811E0jP9ZGWXB/R9LkOUoHrIAq+m+sd
i7jeoZ5bjfsWSnDfguKD8199Cwrfk/sW+HxA0ec+D6zJcwqq3yM5988M5P6Z
01wfWRZ/Pf+r4ma/87Hrx71dUdVNrnBfTSPuq1HwDO13NtgQHo+lliWkHb89
VMPXcL9NLe63OcXwUPv/3PzORWOaub0WJWSI0/ifuA+nO/fhKPie49mtlfYF
4dlD707fbZug6Thwf04w9+coeHhCk7kdFh/HOq1eznq/MEnTqcB9Oy+5b0fB
g0ZevJa/jy+8WrhyZZ5hiZpOV+7n6cj9PIo/o/Iv6NMmfyScuHXlYSa7Zxr/
Dff5FOU+H0X/NOHBUPqehvO64Qv3/yj4G1oH3KV16ed2476gPNwXpOC8T5iW
aNs3qPVU536hndwvpOg7ER/Bkfiq4RbuI0LuI1L0f86xnRNE07lp/B7cX8R9
WPq5BUkOIBPJhcZ3JnmC5SRfmn5sdBk/tzZxaLdu4Y1hHay6HrRhV1DzwPvR
+Pbunus/owN1Xamhz/3BHiXDMCAprpdDeITGPzw72aHp7f0wtd3xqqef/qpz
/WydEOPc6hA2rZE29fytcbiV4YGzM8UOy+wFL13Setayxmr8TQffBWeo9Aha
zOnRqM6kKP3c1vQZVtLfNf5r+j64ET0N5+fB5Da254N6bipaHzjRenVdyYf2
A21of7p+lED7h03EDw1PR/yCOOKfhj/aHn3vZclgrN96g4fHiDPYhusjudaM
GRczPhAnO2X/1O68h4aPcLv3/VxHX/zSOXZk7sYPUNVTUveuXMznxi1snqXT
mjwHgzQ8aWmrf5p83YwpC505c3ZKsIY7zj+1IbzIVXCqnt5n+MqHmv6DtZtD
5/W8B/m+dnp59Ku/xs9Pn+Eu/V3D+fuwhehpOvw8GLHM9nxd92lK64N/aL2g
8N/RfmAc7U/jj6L9Q17ih8Z3IH7BY+Kfxm/Su7atTlfqyvw/6nT2nT0Wvjbw
Mkx/9NrT+J7Kz8+9msFWvytZaOUf9bv4mlk/xBrP23vx5pEOxvMVfvkOy211
vaX+YX/U9fZtGm9/3lj34vIPXfIY+1D435o8s/VF5M5n/0e9L2O+hKm+xv4d
o/aUzmfwYzHjR4faTTfjqYl+f/Y72Tc+vbOpwcefd8qMdjb4qug/y1PTVh8s
03f4H/XBJUGH5kYa53Euu9+Mkcb5KPrrS+y11Q0/NL/6R93wWy2f8AXGudYo
NbSHo3HOiv4nwrOuou9pfAfCwzf0PY1fltZhDaV1aXxeB06kdWl83qd17hPb
vnW95jvtEw/TvvWcTk7iozVd02d/1CWPEh8xB/EV1X7n0zlZq9G5afy5dE54
mM5N07cjObDOJ7nQ+JtJDjCJ5OIXPsmZtRnJncb/Oc0mZ1iS5E7jv7lcdLq7
cf/9hvgfGm7oA5X/9xv9LszUFx1OTBiQLCZQzxO5HQtt5mnolzovXj029Y3C
H+U1xH6GoY/CCmX3M/WTgpcZcnv2eUN/zXAq6mTqMwWPH+Oyf7ih7waUw5G1
Df2n55iGFNuT0dCPi7K18TH1pYIvpc+YnP6u1zOIvo8PiZ6G8/NwHz1fzwGF
0vpwEq0X/2N4ddoP+tH+NH4L2j/6Ez80HIlf+Jb4p+GLajhNMe1TqTdTvpr2
Ss0Z9T3VdaNpzwasLN3ftG9qviYk5nyKB4ZdPO9d3Mu0kwru53oyVWXDvtZP
XSHVPcPeKnjQnHQPTDuN/6YPMe22or+uq9dL0963W14xxrT/Ct9naPRG02/4
UPlD0cyGH6Hgn+kzetHf9ZxRB/o+LiZ6Gp+fh+Pn2p6v54MstD4MpfXq+aBT
tB+MoP1p/G60f+xP/NDw/MQvXEz80/DACJdG9Ve9hGqFaux/NM5N5+cb+2Vo
FpA8Ec5kcEpX/32Yrl/ULTno0oUOsZCj0NwGrUOf/qov0PuSIYLen6zppGzc
5itcOAlHA3p52739NU90+MnQzSdd/LB0fKNDgfhC08lTOfOF6Yui8FHrDDlz
5Huu6xHh9Bmz0981vBx9H7cTvV/1BXoeHqfn67pAOL/fuSa/31nh/0P7wUa0
P41/mPaPzYkfGm5P/FL803NGC1g+S7J8qnx1b5bP/iyfKs8czPLpzPKp8sk+
LJ98/rCM4QEsn9dYPhWdNSyfbVg+Fdyb5fMdy6eCf2T59GT5VM9l+cZFLJ8K
n5+HY1g+FX5dls9HLJ8KfoLlM4zlU9HpyvLZj+VT8Scfy+dClk+F/4L15z3W
nyr/6c36sy3rTwW/wfrTgfWngg9j/fmI9aeC27P+nML6U8FjWH/2Yf2p4KlY
f85n/angrE/x+2DSnwrej/WnP+tPlf/n5+Eu1p8KP5j15zjWnwpehfXnPdaf
Ct6E9acv608Fv8L68yXrTwW3Y78oBftFKs8Zwf7PJvZ/FHw7+zmz2c9R8BTs
z+xlf0bnS9lvect+i4JPZ//kKPsnCv6S/ZBy7IcoeEX2N+LY31DwY+xXDGO/
QsHfsP+wk/0HBd/NfgL7Xxo+jf2BPewPKPhKtvsxbPcV/BPb9wJs3xV86ftb
tnx7660Lpv6eb49lP785+/mtOM/mPfm6LQ9f79q/f+ThC7H/P5f9f4WfZstK
W35+SeytP/LzEzguSNmF4gKVx4Og27a8fcdW7n/k7TNxvNCJ4wWFX7Ye9dvj
/nF/5PM/chyRleMIhT+54SBbnv/F1K9/5PkvcHxxmeMLtf4c927Z8v+1ry36
I/8fznFHEY47FP26hGfNTN/T+MU5HgnieEThv6R1WIfTujQ+rwMOcpyi1sP7
tHrU+6P/DXifMInjF0W/PfHR2pL4qvHbc1yTi+MahT+Pzsmaic5N43/neGcq
xzsKvzrJgdWX5ELjT+U4qCjHQWr99UnOrMtJ7jR+PY6P4jg+UvSfcfxeiON3
NV/G8Sq+4/hd5f2acvwexPG7gh/j+D09x+8KnoLiZ2zO8buiH8Lx+xuO3xV8
K8fvHK9rOu04fl/L8buCv+P43ZPjd0WHnwdTOX5X8LQcv5/l+F3R8eP4vT3H
7wr/BcfvWzh+V/j/cPwez/G7gq/g/NsNzr+pfF1GzrOV5jzbMYb/nU9T+Hs5
b3aO82Zq/iuC82N1OT+m8B9yHuw158EU/mjOd53kfJeCn/krr6Xgb//KXyn6
/DyYznkqTYfzUcc5H6Xwk839M++k59Q4v5SZ80sKfp7zS8w/DVd52kucpz3J
/eTc/42PuX6h+sDrc97+HOftFfwR5+f3cn5e9bfH/VWPU/29+bjuhlx3U33C
fbjetIjrTQrfnetKRbiupPADuX70D/+esOpbLk59rhY36nvVcfE26pe1xFD/
rIYfpv5aS3vqt9Xwe9Tva91F/b+/+oepP9jak/qFNZz7ia2XM/xRJ8Xa04rZ
4ujgynny/h5Hqz6uq9zHpfp2JnHf4z7ue3TjfgbuC8RU3Nen4Py+B8zM/Xtq
jqbkX328qk59gPt1k3G/rqp3L+U+1WTcp6rwPbgf9QT3oyr8adx3eoX7TlX9
3YvqtZbSVL/VdmE71X0tvlQH1vBsVCe2fKC6sYZvpbq1NZ7q2Bqeherc1r5U
99ZwrosbW7fVybV+TrHmrs2OuCQb+ocdGU9zkjiD5ib1/OZ2mqsEnrPE7Dxf
GbjENmeo5g5RzRt6/PV7tmpO5CT/bnMQ/25zCN+LgTQvCi9ofhR8+bnzab4U
itG8KeTk566huUpwoDlLPeeYhupJ0Jt/f1XNj2zk30Pm+qGeZ/Tm39Pewb+n
re5LHZ5XcuZ5JdUnzO/tw0T+vX0Fz8lzZGd4jiyJ5rmwKs13QQ2a99Jw/ozD
ef4rkeF5eP5rO89/qT7Pk0QXC9FzUNFJze+/2MDvv1D4H3keMDnPA96n+TtE
ngf053nABJq/wzw0jwfzaD5Pw6fQ/B42oXk+8GY69/96r5Dqb2Q87M5zfwp/
Ls/91eK5v0SmP5rn/hby3J/u86T1oRutF9V6vr2z7Qe/0v5Q0d/K78dJze/H
UXTU75bc5j5Jn+ue6cd+dqn7lPshn3M/5D8fKi7cle5S3arc9zif+x59Gf8s
9z0mu0x9j4qOPfNtNvPtA88zbv7rd0syMf1BzM9GzE81F1mN+yELcj/kPaZ/
m/m8kPms+gz9uE/S868+ycLcD1ma+yHvMx3uv8SO3PeYkdejfufEn/sbvRl/
Lv/OSQ3+nROVb5nF8NoMV/R9+bxu8nkpPkRzP2Qi90MqPv/gc/zA56j4cIX7
JMtwn6Siv4fPl9+zh8ouNOX7VZnv1xueoyzKfeaduc+8F+v/43yPcvE9Uvhp
+R6t4nuk+vEsPE8xnecplD6/xnoghPWAgp/lOeWvPKes+iIq8Tz4Jp4HdyN9
BR6sP3+w/rzNcP4Mn+jv4MrwEjzHXY3nuFWdfS3rQwvrQ4W/gfXeUdZ7qr5f
lOf3F/P8fub/1XXtQVVWQRzMcLLQSGqyFMwas8EHmWUK3ZOVTlReL2mG0AiZ
iBiipCQCkigCISgZmkFmmtwyURnl5VW+A1gKBVo8fAQ+wLz5LM1XjsPU97G/
/eZ6ZvyL+Q47e87Zs2d3f3u/3Y/sqjiv1O8fIDssZsEvLIZf4HE7/EIJ/ML9
4FOKevwM1ON7YjwJ9fgrUI9fCz4foR6/CvX4/Ds1+4Ur8At1oP8EfmEQ/ALz
d4f9D4f959+1ZzYOia6+Vib3RAReq42+JH9ZM/T4Id1uVA6r3Zk19oDYHBIR
sunfizK+7bPJ1blHZT38V5TivzxeDz9d3+QU83sPiLj4Y4eoBx+nxd//bPgp
8eYkr4j6dWdFAvjcuHDnerhO81S5244RrcVyjluHFpPqlCfbz+d+ZXfKkrv4
x7VfB80Ii2oXK7wetfQ61iaYPgvna1f8Wj382nrFrwXXdyvT5SD899lsddGX
hPuYKb4HdbntiP/jhi4HWZS9fJcuB5E2OGmtzD0qDsL+LFHsT0Kf8X4NTU6Z
Zo/L1+Ug3cBnd+e6Gboc5MWylUW6HORy8MnG/e2u3N8Jv/QIeK61WIw9t2TW
nFSnOJxf7SywO8UzuL+TcX85fvtrd/5ZXQ6yfPqLD/c+1iaZ/vZqur+5yv19
5i7314H7e1K5v4XIz/RFfiYdv5tspfnETZpfNNK8chytW66nfcjfMI68nkxW
fndwklzEfpKTWETykUkkR7GM5Cquj+6Spyym85ApdD4iEfQ4P1lJ5ymZPgD5
z3NKfv449Z8VddSP1vRfxdSvVnhS/1rTLzxJ/W3FPup3a9JnUZ9cmUl9c03/
MpX66sqR1GfX5JNLfXjlAerLa9rzVuS9xyHvzf7FhjxGPyWPkUx6L9fTPZAn
SP/FPLo/YnhF130SrRjH7/rCAdzN+PpVup/yKt1XOY/uqZhN91n2pvstf6J7
Lb4juyBWkJ0w6WFHRCPZFcH024BPy4FPGbd2prdv9bOfkfG/hl7c3VQnW2Kf
v35Ij99igDd3KXjziWmOiJKTp8XgSd0PjUpukYdB3wpcmaPgyrkXyvymRh6R
BUHum4469siYe6zac2suydjFp1clf9wspy+obIz8+mdZeaaweXeeHt8SX9GP
5jH59827el9RQInw/jd6QEz7EamB3ht1vh5Kne+4b2riZr5+QHz2SWdJ949a
zHm/pGfxLP3fHP8UfZmWKP1CMZ+48HnX/Oa8jGeDFDxrIzmK/0iugtc/gfYp
4mnfgve7gOQi5pOcBK+nYMoa39+rsi0TP3zkysMz4yy+wKGHB1fum3gr0+Iz
rjnQ077EHB/UI2jHiT7LLB7xV4JTR2ZYfDC+I9jP+4Wp71pqVw2ydJ7JMceD
J8xf45sdpb0cGPF9YmyOyacs/t6N1veStdEf9O/1Q1KWOR412NN41srp/+b4
O2kFXX1B8dfE11MG2gy+2pHxc415TPqhKwca69ASrU8a69J4PaOAuzsU3B2Q
cMvYjxbWM9jYn0nfn/avyVcaDHlozL8i08eQl7b/8eWG/MzxwO2jWkJ1+U5b
1Ou733V5f7Awf7oh56qGPu4p+nkUn3uvOko/n13ptxOMc4H9FT8rda+Bd6lj
vX9i0fRtun5siA5pnqPrC/OZ+2Cp9yxdzxY2vfaSh653PG8SPcsQ+r853ok+
dVeVfqHfEF95wto1j8l/KXD9NaWedB3tR9bR/iTTj6D9SwvJQ/K8Q+HXQpT6
x6W+i2Nf1O95qNvmm8a9//GNPm8a+rwadQpZSh/O2xOKSh26HckIv2UdotsV
pk+jZzHQ2vV/k08O6j0fUuo9w2k+OZfmN+kDUXfjr/SrDENepbtSF3kP6g0L
UW/IeDAT+RyHUpeXjvEKJc+TivX3w/praT1iDNYzXKkDCsP6Y7B+puf8Tzcl
/zMC+mmDfkbSuYgK6OdG6GcpnaNIwblMwbnUgb8H9DAPesj0K3FeGUpdSST0
Mxb6yfPGQT+t0M+ZGL8G/byg9FFcC/08Bv3keTnvdFDJO92AniyDnrB8sqC3
1dBb5vMU9HYk9JbXudZzgI9hJ1NaH7vsYidFZOCKGsNOhlZ1BLjYSfF+afJ2
w07+8/dmm4udFOdfSvQy7KTXrW2BLnZS7Hphf55hJ8ccPG53sZNiyPGqDYad
bFtV+4CLnRQ936kwnrXQTcUbXOzkXfsl2koaDL7aaP+aPBc7KRbe3GKsQ0sd
NdvLxU4KN+THpJIfe/pqkbEfra586XYXOykmNpw39q+FBn1e42InRTLJS8sj
+Zn0i+Cvt8BfzyZ/JBLgr+fBX+8h/yXWoc6iFXUWZcCVDtRNdCh1E/3hx33g
x5mPFf76K/hrnvdb+Osx8Nc8zn0U4bflDO7TCH99Gf56L/ifgr+eZ72zfiEU
fjkFfpnXkwa/nAK/zPNaEJ/4ID5pJv8u+ir9Rjium4R4oFsGxQNNoHejZ5mI
uKsR4xtRR7AedQSMo59C3OWHuIv5hOH93mC831uIuO4E4rGVynv4/H57b7zf
zjg0FvFeqfIe+CuIS/9BXMr4MRpxaS/EpYxbQ7EeG9bD79PaEa9+inh1Ifg4
lHiVcWsR4tUy5f3eBMTbBYi3TwFvAq+KYYi3EYfLboi3yxFvc3w4HvG8N+J5
fk+yBvjrkpJHtQIPAleZOOsycN9e4D4e/w+4L0/Jf74NnDsCOPcG8NFO4Nzt
wLmMv34Fzk1V8pmJwLnpwLmMsxzAuX8B5zIuywHO9VDykJPxXZLV+C4J4yn1
uyScH9uI75Kk4bskjJvU77Nz3k/9LgnTq98lYf478V2SdnyXhPN+7vDjm5X+
cn8At/4E3Mr5gUTg1lTgVs4DQL4yCbiV8wDDgFv3Arcy/Wjg1rPArezfuW+b
J+IWxvV24PHrwOOcB0AeQRYAj7dg/DDw+CLg8bHwy78h/qxS+qHlIw/wIPIA
mXi/oh15G+BO2Qa8+QXyM6uQn+HxbORntij5tyjko/YiH7UP+FFDPqoQ+ag5
wJs9kHdagLzTftD/ibzTW8g7xYH+JvJO7yPv9C3szP+kXczs
    "]],
  Axes->None,
  Method->{},
  PlotRange->{{-10.982695039340884`, 
   10.999999999999376`}, {-10.999999999999376`, 
   10.982695039340886`}, {-2.9999999999999245`, 2.9999999999999245`}},
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.6291870058740325`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"torus", "[", 
     RowBox[{"3", ",", "1"}], "]"}], "[", 
    RowBox[{"v", ",", "u"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "Pi"}], ",", "Pi"}], "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5922257630910683`*^9, 3.592225775968805*^9}, {
   3.592225989373011*^9, 3.5922259964974184`*^9}, {3.5922260927779255`*^9, 
   3.5922261304900827`*^9}, {3.5922261650030565`*^9, 
   3.5922261971228933`*^9}, {3.592226229242731*^9, 3.5922263441073008`*^9}, {
   3.592226740379966*^9, 3.592226750132524*^9}, {3.592226787731674*^9, 
   3.59222679937234*^9}, {3.59222687175648*^9, 3.592226888444435*^9}, {
   3.5922269726212497`*^9, 3.5922269743323474`*^9}, 3.5922270304075546`*^9, {
   3.5922271192936387`*^9, 3.5922271563647594`*^9}, {3.592227492203968*^9, 
   3.59222750168251*^9}, {3.5922275492762327`*^9, 3.592227574379668*^9}, {
   3.5922279260677834`*^9, 3.5922279324121466`*^9}, {3.5922280294676976`*^9, 
   3.5922280530840483`*^9}, 3.5923064930564485`*^9, {3.6120725609778395`*^9, 
   3.6120725888394885`*^9}}],

Cell[BoxData[
 Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJyFfXVcFs33NiWKhd1id3fhPQN2dwcGdne3j93d3d36GOza8djYomKDoKIg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   "], {{
     {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJxFnHf8l9P//1/Xuda5UiJbpGRFNolEWvb4IESpKJIkKiNSmYlCpGHvlb03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         "]], Polygon3DBox[CompressedData["
1:eJxFnAf8l9P//u/7fc+3pDJCQ4UkMhMZpUEpaWkJaUhpaElC2kVaNGSWWZQt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         "]], Polygon3DBox[CompressedData["
1:eJwl13mYllUZx/F35n2HedXMFIISTDYJlUyDNokAMYHAKJEQl9hlURw1FQEh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         "]]}]}, {}, {}, {}, {}}, {{}, {}, {}, {
      Line3DBox[{780, 1017, 1439, 1018, 2273, 1441, 1022, 1065, 1023, 2279, 
       1457, 1068, 1069, 1464, 1099, 2341, 1100, 1471, 2427, 1121, 1122, 1478,
        2429, 1143, 1168, 1144, 1485, 2432, 1171, 1205, 1172, 1492, 1208, 
       1242, 1209, 2284, 1499, 1245, 1279, 1246, 2290, 1506, 1282, 1283, 1513,
        1313, 2389, 1314, 1520, 1335, 1336, 1527, 2451, 1357, 1382, 1358, 
       1534, 2453, 1385, 1416, 1386, 1418}], 
      Line3DBox[{781, 1020, 1440, 1021, 2274, 1443, 1026, 1067, 1027, 2280, 
       1458, 1071, 2328, 1072, 1465, 1101, 2342, 1102, 1472, 1123, 2354, 1124,
        1479, 2430, 1145, 1170, 1146, 1486, 2433, 1174, 1207, 1175, 1493, 
       1211, 1244, 1212, 2285, 1500, 1248, 1281, 1249, 2291, 1507, 1285, 2377,
        1286, 1514, 1315, 2390, 1316, 1521, 1337, 2401, 1338, 1528, 1359, 
       1384, 1360, 1535, 2454, 1388, 1417, 1389, 1420}], 
      Line3DBox[{782, 1024, 1442, 1025, 2275, 1445, 1031, 1070, 2327, 1032, 
       1459, 1074, 2330, 1075, 1466, 1103, 2343, 1104, 1473, 1125, 2355, 1126,
        1480, 1147, 2361, 1173, 1148, 1487, 2434, 1177, 1210, 1178, 1494, 
       1214, 1247, 1215, 2286, 1501, 1251, 1284, 2376, 1252, 1508, 1288, 2379,
        1289, 1515, 1317, 2391, 1318, 1522, 1339, 2402, 1340, 1529, 1361, 
       2408, 1387, 1362, 1536, 2455, 1391, 1419, 1392, 1422}], 
      Line3DBox[{783, 1028, 1444, 2417, 1030, 1447, 1035, 1073, 2329, 1037, 
       1460, 1077, 2332, 1079, 1467, 1105, 2345, 1107, 1474, 1127, 2357, 1129,
        1481, 1149, 2362, 1176, 1151, 1488, 1180, 2366, 1213, 1182, 1495, 
       1217, 1250, 2371, 1219, 1502, 1254, 1287, 2378, 1256, 1509, 1291, 2381,
        1293, 1516, 1319, 2393, 1321, 1523, 1341, 2404, 1343, 1530, 1363, 
       2409, 1390, 1365, 1537, 1394, 2413, 1421, 1396, 1424}], 
      Line3DBox[{788, 1045, 1451, 1046, 2276, 1453, 1050, 1085, 1051, 2281, 
       1461, 1088, 1089, 2283, 1468, 1111, 2349, 1112, 1475, 2428, 1133, 1134,
        1482, 2431, 1155, 1188, 1156, 1489, 2435, 1191, 1225, 1192, 1496, 
       1228, 1262, 1229, 2287, 1503, 1265, 1299, 1266, 2292, 1510, 1302, 1303,
        2293, 1517, 1325, 2396, 1326, 1524, 2450, 1347, 1348, 1531, 2452, 
       1369, 1402, 1370, 1538, 2456, 1405, 1428, 1406, 1430}], 
      Line3DBox[{789, 1048, 1452, 1049, 2277, 1455, 1054, 1087, 1055, 2282, 
       1462, 1091, 2336, 1092, 1469, 1113, 2350, 1114, 1476, 1135, 2359, 1136,
        1483, 1157, 1190, 1158, 1490, 2436, 1194, 1227, 1195, 1497, 1231, 
       1264, 1232, 2288, 1504, 1268, 1301, 1269, 1511, 1305, 2384, 1306, 1518,
        1327, 2397, 1328, 1525, 1349, 2406, 1350, 1532, 1371, 1404, 1372, 
       1539, 2457, 1408, 1429, 1409, 1432}], 
      Line3DBox[{790, 1052, 1454, 1053, 2278, 1456, 1056, 1090, 2335, 1057, 
       1463, 1093, 2337, 1094, 1470, 1115, 2351, 1116, 1477, 1137, 2360, 1138,
        1484, 1159, 2363, 1193, 1160, 1491, 2437, 1196, 1230, 1197, 1498, 
       1233, 1267, 1234, 2289, 1505, 1270, 1304, 2383, 1271, 1512, 1307, 2385,
        1308, 1519, 1329, 2398, 1330, 1526, 1351, 2407, 1352, 1533, 1373, 
       2410, 1407, 1374, 1540, 2458, 1410, 1431, 1411, 1433}], 
      Line3DBox[{887, 999, 1412, 1376, 879, 991, 1375, 2411, 1353, 871, 983, 
       2399, 1331, 863, 975, 2386, 1309, 855, 967, 2374, 1273, 847, 959, 2370,
        1272, 1236, 839, 951, 1235, 1199, 831, 943, 1198, 1162, 823, 935, 
       1161, 2364, 1139, 815, 927, 2352, 1117, 807, 919, 2338, 1095, 799, 911,
        2324, 1059, 791, 903, 2319, 1058, 1009, 776, 895, 1435, 1007, 1434}], 
      Line3DBox[{888, 1000, 1413, 1378, 2313, 880, 992, 1377, 1354, 872, 984, 
       2400, 1332, 864, 976, 2387, 1310, 856, 968, 2375, 1275, 848, 960, 1274,
        1238, 840, 2438, 952, 1237, 1201, 832, 944, 1200, 1164, 2301, 824, 
       936, 1163, 1140, 816, 928, 2353, 1118, 808, 920, 2339, 1096, 800, 912, 
       2325, 1061, 792, 904, 2320, 1060, 1012, 777, 2414, 896, 1436, 1008, 
       1010}], Line3DBox[{889, 1001, 1414, 1380, 2314, 881, 993, 1379, 1355, 
       2309, 873, 985, 1333, 865, 977, 2388, 1311, 857, 969, 1277, 849, 2444, 
       961, 1276, 1240, 841, 2439, 953, 1239, 1203, 833, 945, 1202, 1166, 
       2302, 825, 937, 1165, 1141, 2297, 817, 929, 1119, 2294, 809, 921, 2340,
        1097, 801, 913, 2326, 1063, 793, 2421, 905, 1062, 1015, 778, 2415, 
       897, 1437, 1011, 1013}], 
      Line3DBox[{890, 1002, 1415, 1383, 2315, 882, 994, 1381, 1356, 2310, 874,
        986, 1334, 2307, 866, 978, 1312, 858, 2448, 970, 1280, 850, 2445, 962,
        1278, 1243, 842, 2440, 954, 1241, 1206, 834, 946, 1204, 1169, 2303, 
       826, 938, 1167, 1142, 2298, 818, 930, 1120, 2295, 810, 922, 1098, 802, 
       2425, 914, 1066, 794, 2422, 906, 1064, 1019, 779, 2416, 898, 1438, 
       1014, 1016}], 
      Line3DBox[{891, 1003, 1423, 1395, 883, 995, 1393, 2412, 1364, 875, 987, 
       2403, 1342, 867, 979, 2392, 1320, 859, 971, 2380, 1292, 851, 963, 2372,
        1290, 1255, 843, 955, 2369, 1253, 1218, 835, 947, 1216, 2368, 1181, 
       827, 939, 1179, 2365, 1150, 819, 931, 2356, 1128, 811, 923, 2344, 1106,
        803, 915, 2331, 1078, 795, 907, 2322, 1076, 1036, 784, 899, 2321, 
       1446, 1029, 1033}], 
      Line3DBox[{892, 1004, 1425, 1398, 2316, 884, 996, 1397, 1366, 876, 988, 
       2405, 1344, 868, 980, 2394, 1322, 860, 972, 2382, 1295, 852, 964, 2373,
        1294, 1258, 844, 2441, 956, 1257, 1221, 836, 948, 1220, 1184, 2304, 
       828, 940, 1183, 2367, 1152, 820, 932, 2358, 1130, 812, 924, 2346, 1108,
        804, 916, 2333, 1081, 796, 908, 2323, 1080, 1040, 785, 2418, 900, 
       1448, 1034, 1038}], 
      Line3DBox[{893, 1005, 1426, 1400, 2317, 885, 997, 1399, 1367, 2311, 877,
        989, 1345, 869, 981, 2395, 1323, 861, 973, 1297, 853, 2446, 965, 1296,
        1260, 845, 2442, 957, 1259, 1223, 837, 949, 1222, 1186, 2305, 829, 
       941, 1185, 1153, 2299, 821, 933, 1131, 813, 925, 2347, 1109, 805, 917, 
       2334, 1083, 797, 2423, 909, 1082, 1043, 786, 2419, 901, 1449, 1039, 
       1041}], Line3DBox[{894, 1006, 1427, 1403, 2318, 886, 998, 1401, 1368, 
       2312, 878, 990, 1346, 2308, 870, 982, 1324, 862, 2449, 974, 1300, 854, 
       2447, 966, 1298, 1263, 846, 2443, 958, 1261, 1226, 838, 950, 1224, 
       1189, 2306, 830, 942, 1187, 1154, 2300, 822, 934, 1132, 2296, 814, 926,
        2348, 1110, 806, 2426, 918, 1086, 798, 2424, 910, 1084, 1047, 787, 
       2420, 902, 1450, 1042, 1044}]}, {
      Line3DBox[{1541, 1744, 1994, 895, 1993, 1745, 1996, 2414, 1684, 1995, 
       1747, 1998, 2415, 1685, 1997, 1749, 2000, 2416, 1686, 1999, 2207, 2273,
        2002, 1687, 2001, 2208, 2274, 2004, 1688, 2003, 2210, 2275, 2006, 
       1689, 2005, 2417, 1752, 2008, 2188, 2321, 2007, 1754, 2010, 2418, 1690,
        2009, 1756, 2012, 2419, 1691, 2011, 1758, 2014, 2420, 1692, 2013, 
       2212, 2276, 2016, 1693, 2015, 2213, 2277, 2018, 1694, 2017, 2215, 2278,
        2019, 1695, 1761}], 
      Line3DBox[{1542, 1746, 2020, 2189, 2319, 1763, 1748, 2021, 2190, 2320, 
       1765, 1750, 2022, 2421, 1696, 1767, 1751, 2023, 2422, 1697, 1769, 2209,
        2279, 2024, 1698, 1771, 2211, 2280, 2025, 1699, 1772, 2327, 1753, 
       2026, 1700, 1774, 2329, 1755, 2027, 2191, 2322, 1776, 1757, 2028, 2192,
        2323, 1778, 1759, 2029, 2423, 1701, 1780, 1760, 2030, 2424, 1702, 
       1782, 2214, 2281, 2031, 1703, 1784, 2216, 2282, 2032, 1704, 1785, 2335,
        1762, 2033, 1705, 1787}], 
      Line3DBox[{1544, 1764, 2034, 2193, 2324, 1543, 1766, 2036, 2194, 2325, 
       1545, 1768, 2038, 2195, 2326, 1546, 1770, 2040, 2425, 1706, 1547, 1069,
        2042, 1707, 1548, 2328, 1773, 2044, 1708, 1549, 2330, 1775, 2046, 
       1709, 1550, 2332, 1777, 2048, 2196, 2331, 1551, 1779, 2050, 2197, 2333,
        1552, 1781, 2052, 2198, 2334, 1553, 1783, 2054, 2426, 1710, 1554, 
       2217, 2283, 2056, 1711, 1555, 2336, 1786, 2058, 1712, 1556, 2337, 1788,
        2060, 1713, 1814}], 
      Line3DBox[{1570, 1816, 2061, 1815, 2351, 1569, 1813, 2059, 1812, 2350, 
       1568, 1811, 2057, 1810, 2349, 1567, 2348, 1809, 2055, 1808, 1566, 2347,
        1807, 2053, 1806, 1565, 2346, 1805, 2051, 1804, 1564, 2344, 1803, 
       2049, 1802, 2345, 1563, 1801, 2047, 1800, 2343, 1562, 1799, 2045, 1798,
        2342, 1561, 1797, 2043, 1796, 2341, 1560, 1098, 2041, 1795, 1559, 
       2340, 1794, 2039, 1793, 1558, 2339, 1792, 2037, 1791, 1557, 2338, 1790,
        2035, 1789, 1819}], 
      Line3DBox[{1584, 1839, 2075, 1838, 2360, 1583, 1837, 2074, 1836, 2359, 
       1582, 1835, 2073, 2428, 2222, 1581, 2221, 2296, 2072, 1834, 1580, 1131,
        2071, 1833, 1579, 2358, 1832, 2070, 1831, 1578, 2356, 1830, 2069, 
       1829, 2357, 1577, 1828, 2068, 1827, 2355, 1576, 1826, 2067, 1825, 2354,
        1575, 1824, 2066, 2427, 2220, 1574, 2219, 2295, 2065, 1823, 1573, 
       2218, 2294, 2064, 1822, 1572, 2353, 1821, 2063, 1820, 1571, 2352, 1818,
        2062, 1817, 1842}], 
      Line3DBox[{1598, 1859, 2089, 1858, 2363, 1597, 1857, 2088, 1157, 1596, 
       1856, 2087, 2431, 2229, 1595, 2228, 2300, 2086, 1855, 1594, 2227, 2299,
        2085, 1854, 1593, 2367, 1853, 2084, 1852, 1592, 2365, 1851, 2083, 
       1850, 2362, 1591, 1849, 2082, 1848, 2361, 1590, 1847, 2081, 2430, 2226,
        1589, 1846, 2080, 2429, 2225, 1588, 2224, 2298, 2079, 1845, 1587, 
       2223, 2297, 2078, 1844, 1586, 1140, 2077, 1843, 1585, 2364, 1841, 2076,
        1840, 1861}], 
      Line3DBox[{106, 666, 831, 107, 667, 832, 108, 668, 833, 109, 669, 834, 
       110, 1492, 670, 111, 1493, 671, 112, 1494, 672, 113, 1495, 673, 835, 
       114, 674, 836, 115, 675, 837, 116, 676, 838, 117, 1496, 677, 118, 1497,
        678, 119, 1498, 679, 120}], 
      Line3DBox[{1612, 1875, 2103, 2437, 2241, 1611, 1874, 2102, 2436, 2240, 
       1610, 1873, 2101, 2435, 2239, 1609, 2238, 2306, 2100, 1872, 1608, 2237,
        2305, 2099, 1871, 1607, 2236, 2304, 2098, 1870, 1606, 2368, 1869, 
       2097, 1868, 2366, 1605, 1867, 2096, 2434, 2235, 1604, 1866, 2095, 2433,
        2234, 1603, 1865, 2094, 2432, 2233, 1602, 2232, 2303, 2093, 1864, 
       1601, 2231, 2302, 2092, 1863, 1600, 2230, 2301, 2091, 1862, 1599, 1162,
        2090, 1860, 1877}], 
      Line3DBox[{1613, 1876, 2104, 951, 1885, 1878, 2105, 2438, 1714, 1887, 
       1879, 2106, 2439, 1715, 1889, 1880, 2107, 2440, 1716, 1891, 2242, 2284,
        2108, 1717, 1893, 2243, 2285, 2109, 1718, 1894, 2244, 2286, 2110, 
       1719, 1895, 2371, 1881, 2111, 2199, 2369, 1897, 1882, 2112, 2441, 1720,
        1899, 1883, 2113, 2442, 1721, 1901, 1884, 2114, 2443, 1722, 1903, 
       2245, 2287, 2115, 1723, 1905, 2246, 2288, 2116, 1724, 1906, 2247, 2289,
        2117, 1725, 1907}], 
      Line3DBox[{1614, 1886, 2118, 2200, 2370, 1909, 1888, 2119, 960, 1911, 
       1890, 2120, 2444, 1726, 1913, 1892, 2121, 2445, 1727, 1915, 2248, 2290,
        2122, 1728, 1917, 2249, 2291, 2123, 1729, 1918, 2376, 1896, 2124, 
       1730, 1920, 2378, 1898, 2125, 2201, 2372, 1922, 1900, 2126, 2202, 2373,
        1924, 1902, 2127, 2446, 1731, 1926, 1904, 2128, 2447, 1732, 1928, 
       2250, 2292, 2129, 1733, 1930, 1269, 2130, 1734, 1931, 2383, 1908, 2131,
        1735, 1933}], 
      Line3DBox[{1616, 1910, 2132, 2203, 2374, 1615, 1912, 2133, 2204, 2375, 
       1617, 1914, 2134, 969, 1618, 1916, 2135, 2448, 1736, 1619, 1283, 2136, 
       1737, 1620, 2377, 1919, 2137, 1738, 1621, 2379, 1921, 2138, 1739, 1622,
        2381, 1923, 2139, 2205, 2380, 1623, 1925, 2140, 2206, 2382, 1624, 
       1927, 2141, 973, 1625, 1929, 2142, 2449, 1740, 1626, 2251, 2293, 2143, 
       1741, 1627, 2384, 1932, 2144, 1742, 1628, 2385, 1934, 2145, 1743, 
       1935}], Line3DBox[{479, 462, 2386, 1629, 464, 2387, 1630, 465, 2388, 
       1631, 466}], 
      Line3DBox[{492, 477, 2398, 1641, 475, 2397, 1640, 474, 2396, 1639}], 
      Line3DBox[{1655, 1957, 2159, 1956, 2407, 1654, 1955, 2158, 1954, 2406, 
       1653, 1953, 2157, 2450, 2254, 1652, 2253, 2308, 2156, 1952, 1651, 1345,
        2155, 1951, 1650, 2405, 1950, 2154, 1949, 1649, 2403, 1948, 2153, 
       1947, 2404, 1648, 1946, 2152, 1945, 2402, 1647, 1944, 2151, 1943, 2401,
        1646, 1942, 2150, 1335, 1645, 2252, 2307, 2149, 1941, 1644, 1333, 
       2148, 1940, 1643, 2400, 1939, 2147, 1938, 1642, 2399, 1937, 2146, 1936,
        1960}], 
      Line3DBox[{1669, 1976, 2173, 1975, 2410, 1668, 1974, 2172, 1371, 1667, 
       1973, 2171, 2452, 2260, 1666, 2259, 2312, 2170, 1972, 1665, 2258, 2311,
        2169, 1971, 1664, 1366, 2168, 1970, 1663, 2412, 1969, 2167, 1968, 
       2409, 1662, 1967, 2166, 1966, 2408, 1661, 1965, 2165, 1359, 1660, 1964,
        2164, 2451, 2257, 1659, 2256, 2310, 2163, 1963, 1658, 2255, 2309, 
       2162, 1962, 1657, 1354, 2161, 1961, 1656, 2411, 1959, 2160, 1958, 
       1978}], Line3DBox[{1683, 1991, 2187, 2458, 2272, 1682, 1990, 2186, 
       2457, 2271, 1681, 1989, 2185, 2456, 2270, 1680, 2269, 2318, 2184, 1988,
        1679, 2268, 2317, 2183, 1987, 1678, 2267, 2316, 2182, 1986, 1677, 
       1395, 2181, 1985, 2413, 1676, 1984, 2180, 2455, 2266, 1675, 1983, 2179,
        2454, 2265, 1674, 1982, 2178, 2453, 2264, 1673, 2263, 2315, 2177, 
       1981, 1672, 2262, 2314, 2176, 1980, 1671, 2261, 2313, 2175, 1979, 1670,
        1376, 2174, 1977, 1992}], 
      Line3DBox[{467, 1633, 2390, 468, 1634, 2391, 469, 1635, 2393, 470, 2392,
        1636, 471, 2394, 1637, 472, 2395, 1638, 473}], 
      Line3DBox[{467, 2389, 1632}]}}},
   VertexNormals->CompressedData["
1:eJxsvXk8Vd/3P06kqDRQSRIq0ShFyXBEpUyVaJR5KDRIpkoaNJAkETI0SSSV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    "]],
  Axes->True,
  Method->{},
  PlotRange->{{-3.9999999999994964`, 
   3.999999999999899}, {-3.9999999999997984`, 
   3.9999999999997984`}, {-0.9999999999999748, 0.9999999999999748}},
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.6291870172152524`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sfrev", "[", 
     RowBox[{"ftor", ",", "htor"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "Pi"}], ",", "Pi"}], "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5922257630910683`*^9, 3.592225775968805*^9}, {
   3.592225989373011*^9, 3.5922259964974184`*^9}, {3.5922260927779255`*^9, 
   3.5922261304900827`*^9}, {3.5922261650030565`*^9, 
   3.5922261971228933`*^9}, {3.592226229242731*^9, 3.5922263441073008`*^9}, {
   3.592226740379966*^9, 3.592226750132524*^9}, {3.592226787731674*^9, 
   3.59222679937234*^9}, {3.59222687175648*^9, 3.592226888444435*^9}, {
   3.5922269726212497`*^9, 3.5922269743323474`*^9}, 3.5922270304075546`*^9, {
   3.5922271192936387`*^9, 3.5922271563647594`*^9}, {3.592227492203968*^9, 
   3.59222750168251*^9}, {3.5922275492762327`*^9, 3.592227574379668*^9}, {
   3.5922279260677834`*^9, 3.5922279324121466`*^9}, {3.5922280294676976`*^9, 
   3.5922280530840483`*^9}, 3.5927415515391293`*^9}],

Cell[BoxData[
 Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJyNXXVcFtnXpwQLsVbs1rW7fXzOKGsrBnahKHagrl1rd2G3romKqBiI+szY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   "], {{
     {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJxFnHccT+X//s+8z3kjUREqshqSFoWoRDQkRdFe0tYQkfpoGkk7RYv2VGnS
oqEp7a20qYh2Gr7X03V+j98f53Fe5z73ud9n3Pfrvl7X67rfLY86Zb9hSRRF
J4coSrWvZVF0Zx5FX8ruFEdRK207yT5RlTrL3ll2o1oUPVlG0UuqW1/2A7Lv
kd1Y9lOyX5G9tuyHZN8newttj6nN/3RtPZXfr/KbVVZov3YRRc1UvoPsg2Xv
od/oLPtQ2XvJ3kr1ntC1ieyTdQ9dtd9F9Ueozguq87mOD9b592T/IvtS1TlN
+yGq01N1Rqn8JB2fQj1tR6t8pMrnq/wLHY+W/brs72VvqHt7TccHqb3r9LsP
ahuk8hu0f1jbgbI3Up3XVecE1TlL+zd07Y8q31H2MbL3l91N9rGFr91F9mmy
j5LdSG3cp+uW6x7Wl32/7BWyV+l80LaO7HVVfq/Kf5R9ON9E5Q1k/6N9qW1d
3pXaaqGtu+wt1H5XlXeW3VH7Ftq2l72ZyneQvYPsrmrzGbW5lq75U2WZtoYq
f1Ht/yT7LdnzZa+Q/bbsnXXtKbKPUP0jVJ7Trsq3VDuPq51I5anq1FV5Ex2u
o3fyuI6f0vkG2jdWeSuV15e9nuyWspvq3I7aVsq+RvsHMr+rXHXWUp2mKm+o
dh7V8RydW0dlhbaNVT5Ix6/pd9uofi+dH63yYbIn6lsfp/1hqnOh6nyuOr11
3FZ1tlOdzWXXUZv36Pgyne+g/U4q31blNZXfrePJKt9a+x4q76jy09VmT+17
q82Lde7WzP1wgva3ZO6fM7T/Ub91qOwzVH837fuo/tn0O20nMF5U56LMv9Va
7W+t9jelD6tsXObf6qdte219Vb+v9l207S37YJ0fkbl+D21battN5Weo7EP9
7o46LnX/r6rdbXV8mMpHZn7e7zSI5+i3FsgexviT/bjso3R+NGNR9p3az83c
J2O187Lq5WrnGj3LOSo7Sb+1ncr21rU76fgY7ovxo/LLVWc4ZfQ31dlHdXbR
8e1q79nMfWa86gzV/lDV6aM656rOmTpupt+ap+OPVG+mtp/1m8erfHeVnac6
oxmf2vprO0TX5qr/is41Ur3XVH+19ufq3CzZr2ce14N0/lJde6Xss7Qdq+1E
XXugyi9X+TU6/gdfpe1PXXOt7m0s30F1ntHxX2rzDB0/KntB5n71ld7hI7r2
OdlNdQ9zde0HOnes9reo/D7GheypsqfzLmRPl3237MH0UW0D1f6xuuZttb8V
Y191mqvOZiqfpjF1Ps8r+xPVCTqepDrPyf4087t6QftFmd8J36ap6syUPVnX
XaTtLF27ROfrqfxalR+k9m9V+SSV70l/lz2GfiX7Ctlny16o+ku1XaD6P/K7
anea7Ldlf09/lX198PucoPof6niutjtkv6/98/Qd2Qv0Dj+SfZfsAWp/mupf
IHuK9hdrO0/2viq/Tvb5kd/7umr3Fl1zc+HvMlHlg1VnBtfI/kJ1Vmm7XHU+
57tou1T27MJ9+GbmqNJ9e7rs42Q/Knua7Bu5fz3LQao/Irjf9sTPB/fPXpHn
mx9U5xDVGaf9wTq3Lv09VHON6oyRPYw5KLJf+Er19ontO76WvS/jXXWGqM6u
jPfgOWtP7oG2ddxebd6Eb5DdQfYL2v+hOm+qTiuVb6LjF5mPdP9PqnxG5DG8
pcrfVPnQ0n1vqsrvLtzHLpd9pMrvp58zL8i+V/YVzAvMu7r+dl07Vf32r8Jz
9w3BffLayONnYx3PVp2HC/fh61S+sdppL3sb2Ucwn2n7nTGk/f+0rcIP4l9y
z5U9ea7cc+4h2u8SPOd+qef8W/tn1M4y7T/W9pjsv7T/RtvTsn/S/gPGGfOL
rt1O136K/5PdX9s7zPu8Y+Zy2Xtqf6S2r2QvVPufaH8340X2CtmPy75J9sV8
E+ZTPcvG+Hz6v97Ji7q2CXO6yurje1W+Db5cW6ryRbr2Z+2fUPlGunZz1dma
90y/kN0BH6L3mamdc3ifqj9B9YfLnq023tR2qo43Ud1G2rZS+eaq20Pbq5Tz
frS9jP/Wvpe212WvUDvraFug+hNVNlC/10jHf2jbWNv7Kt9f5cNy44qr6deq
00rnMm0rVfai6vwgu6btJdkNtP9H5a/LHq36e6t+HZUt1VZom6/yoP1y/Irs
c/D14AHegbb1tL2h8t+1b67tPdnvav+16j/A2Jf9rewHZe+j67qq/e/47tre
0DZT5V+rzr+yn5V9kur0UZ3/dDxA9sm58dV/qrORtlzb0Sobo+1PlW+g41Lb
u7r2dJVN0pbq+FTtL9YWyz438fjeX3VayCe/oW82QufOTzymD1D5KcxljF+V
36yyexJ/r/fBo7qfc3lvwb4Jv/Szyg9K7a8O1zUP6NycyHhyuux7aFP2Y7Ln
yf4x2N/h62ao/fsS972TmSP0u11V93TZb4L9cvuyGaozKvKcFen6U7Qbp/OX
BuOWkapf0/GGOj9K9hayB8o+X/ZJsm/KPBeUqn+m6k9g3ld539z+ehp4N7JP
n8o7kz1edZ5TnW6557U4uPxp5gXZh8v+TeUHp34PN+DLE78f5oM7mSPBG2pj
rOr3i+xb/qxwI/PzhMRzdFv91kK1O1K/NVP7s1R/d8aCyt8r7WvH6reWaD+A
d8W8F4yXTpA9KhhrXa863VLPm2CtZao/GB8ou0fq3wKDjEuMQ8Bmy1XnQNnf
BM95zHdXq86tiTHMV2DM1HPlcN3Px/q9obrmG5UfkLr+bbInBv/Wt8HzJXPl
hao7ORjDnKVru+h4SGZ8MSUxxviOuaa0f5gCBkqMK/hm3VP31dG6treuPVVl
Y2Xvz/wo+5oKn7fHP8huLXsD/IbsjWSvJ/sttbEHzyW7lfZNiQVi45Y9dW6o
2j9MbV6j8p/Ac7Jvkv2f7CvA26rTn76t8muJjyrctVvqPgBWZB5iDmqj/QbE
IGp/K9ltZW8EHis9xzC/DCnd/p1gRGI+2Y+Ct2TfJfvh2Fh9U7W7Ld9X5Xeo
/KHY+G0flQ+LjNW3T91PwLTbyO4h+8rKD7eMPVeerPLrVT5U9/8gMYLKjqn5
dxvIPrrm9mu5sVOh7zEyMm6Ynni8TAIDBuPV59T+lOD39nFq/AH2OEDtjFG9
JzL79iR4TL0su2/qvnSO6uyuOqep7GzZg8A2st/T1l91Tsefq+zq4NiTbze5
wmmn635uTFznWF07i2+dG2eCY8AwQ2p+h/VUfkds34HfOFDlEwr7MOKOXXTN
IPwzmCh4TvxQ5fum9nVjVH+Azp2rsgGyD5d9ieyBso8vPM7GaPtOv7NfZKx6
XuJxN5LrwEyy5zNXpu7n43SPS3VubdVrp3YWlcY7jzF/JvYha+t4SGqs8jO4
SHUukX2R9ksKzzWb6tpPdHyl6q7Dt0yNYe4HlyQedxvjP1XnTLASYxb8mrvP
PaA64yP34w31G5dF7vcbgFnBV6r/rupfpPpbyP6sNBabWDrObaLrW6v8fR2P
Z+7Q/ovCcw39e6b24yL310cTt3mO6rxfGBfRvx9L/Lv/U/mHheev82R/Xnie
itTu4anx7cpgXA4mT1R+ROr+9gyYSHWngD2IgVLjMcZtK11zDe88dr+hzzD+
Wqr86sjj+hu+nc6NToxP8c/42Cnado39fa/KHJeN0v7KzHHcmMQYlu/7lPZP
Jf6tP9T2UbrPq2Q/ofI5iec4xtjmOncT2CAYg4I/71WdhxL7zxdkz0v8jH+r
zvGlxyxjclMd3xAZD72QuB1iqmcTv4f24JzU2L6F7BNS417GwGa69sbI+On5
xDYx7fzEfYb5b3RqfAI2AaPgx5bpuoG6hwsj8za/qd5Rkcd279T8T3/1gX31
vf7HWNZ+XLCv3kPl2+v4SJX3k91T9in48pp98m6ZY6A3EsdBzK9npI6DiIeI
i7j/XWv2n/2Zu2r27f0yc0q/gmu5H93jhOCxTOy8SuXH0q9Ud3wwnmmiax8j
TsjMK1ydmFtYD39IDJA5lv9b1x6n8t1Uvpmu3y9z3PZ64thtFvF18FwAB3ZZ
4vcAD/ML/TIy/3cgGCYy1gZz45fA4q8xPvBFxAWpsTe4fUddMysyrv84MbYH
+3+aGP+Dgc5Jbb8Nlgl+V+CzUalxI/jxvaTCkLFx5/TImHl71b+fub40N9Us
cYzQTeUPR8bbnWTfFxlvd5R9r+yOeg91VL+5nm+h2nw5cb/dSnVOSd1+J9Wp
pzqt4RXAR8T2Ku9a8zy7E/NnzXNij8xzTwfVuzUyzhueug8Qt7yq9m+L7NO3
CrY769qGunY7XXtKML/UTeXDgjmoHZmndO6n3HjqN7BGaSz2Q+z3fnvVn//G
16TGn6vgIlLPpd9lnk/vTcxDgGPrq87yzPgNf7yo8sng35WZMd7S2PfNfeKD
P6v88JLYuJ5n3Fv31SV4bC3V/qrCbcKz3BObu8BvfVH5Lnzdat3bJzpeHHvM
M96XqPxKxprsgTr3h+5hRPUtdgj+/t/j2wrf26+yDymNQ9eS/UPma/dS/c7B
fWQi82nlG+8O5m/pD6fDQxTGh2eAv3PHGmDTQcExxs66h7nVHAqXd3hsDAnH
d6S2gyLPTwcGx2NgzDsy1wGHzquw6KTE/CVjmflpcHDsBCd4WGw8eWViPoyx
CZ4Fa1IOTl5L/WM6flntPZt6vIOH4TLWxDmpMQT44UT6Y+mx20dl81NjDPzZ
i5VPG6g6ZwbzhItix9Y3VD62TxV7Mudfm3neB49Py4zJL0nMVeMnd1U7RxTm
lmcl5romVnN6pHv+NDWWmVi9/9sSc2zc54xgHp4xeGswx874XYE/K41p4O/O
ix0Hwd+dHzs2AVP/Lzbe6KffeL3CisQs6xPz6F4fpo1gfwXmv0jlb6veLYn7
JX2Sufejav4doP2bqfvbdcE8JN/iWtmXFPa3xIPjwASpf/+K6h5+Yk4ojSXB
31fF9r3wcZdV2Ix44TfVeYd7CM5H4IuI1RZnxp/76dzC1HHiI2CZ4D7DmOpX
Om7jt24ojD3WY7zDxcleFYy5wT7kG3oHx9iN2Geuv1z29YXrk7fYOZjXuCc4
N/FI5Hn1y2pufTAxTwm/d5zq7xbM3fCNbiyMr/4Kxv1gImKc32W/mzpWfq+K
l8HI72bGyefouquC+yEx7huZv+9gtb9TMF9DDPpW5m/NeN+ndPwKJvq4wkX0
o7uqvrS3yl5LHU/ByV4Y+7fu0rWfFZ53npb9q+xXZHdR3SdTY5vDVNaldLy4
ru6hRTCueTr2u+QZn4z9fLzDObG5Md4h+G3D3O+oofbNg3Eu+LZd7ngHrLJ1
MBYAv3XJPd++Gxvbgeteio0jwZCvVNgR3Dg/Nv83WXZzXdcmGBPdBycBNxG5
n59L/ixzTH9SNUbwNydWPmdjXds2GH91l71t8LzdLjGv+UlkzNk6eAyfy9jP
jeEv1H6qtobYif0g/u3t2LgQTLi1zrcPnj/fis1Ngj9TlTcJxuy3xuZ0ibl+
1b02DOZdHojNVeO34VjW0jU36fjB2H6deSRW2frBWPK22OOM8UJs3ji4LxC/
r848tn/Xfp1g3oW4rBY8XolTvsw8Rr/Vvm5wLL6j2t8mGJd1IB4Ixo/EHe2C
cyJwIBsFz52zYs+9zEFTY49L3vn1sfspfe9r1a8TzDHgL0+ufOZ1sTl4+jzx
4Apt42PHZQ2CfSU8zbCqHfzAefq+j+j8Z7ExLvgWnml5Ya7pqsT5meOZ31W/
rupfTozPXBqcJ+qq/v5UFePvof3LFScATtszmCtkbPYuHUsQU/UN5g3JeYyI
7QPBouuq/aczxxUDYvfDJ4Ljo+dlPxMcx70q+wtiqsy8dx+4jeAc8ePBcdBz
si9KPKcynxKHHBB7ruH+u5bGPBeoneurfrir7v251HwLsewbVTzLuPif7u3q
KoYZGHv+ejY4Jn0tcly/AbFiZk5wr2Dek7h8QRWb99L+hYr3ID7cL5iHJb5/
tYrxe2r/fGpeboTa3rU034M/GV44TwfG6FEaj5O7OiH2fATGbkBuu4qbX6li
5+P1W/sG87PEnAOC+Vbm9IuC425ilMcz96WHguPKpyLzkd8V5iQfDI5Dn6zu
7ZJgHEJMMKaam8AaI2PPcfCOo6t5ag0XUo2v3XVfL6XmQ2Yz/5CTy9zXRlX9
bQ7xk8qfz+ybyaUxHslRXVKNd+owp/JbbVXn3tR5hFbEoNr+AEskzm+XbLpu
49h55A6qOyu1lqA3MZWev5HObamyh1L3pa/UdrcKJ38pe8cqNmkMJ0bML3tT
eNPSufDTVfZBbn1AO5W3KK1d+DqYz4LL+lR2G53bsmof3g3Obb6u21vXb4Hf
lr2P7Pay6+pc89g56LYqe0Tn/pL9ia5tVbjORmA3nWst+2OVtyzczgYqe0j1
f8FXg11k/y67s84fH5yj/4T4KHN+Cu7z6Yr/3Dp2zpB2DijMNZFjhlN8puIV
t4v9TDxLe5VdoHa60M+1Pzwzf8h7OCo4NzEycS6aMfuufvekzO8WvLdN6Tz0
mYnz0uTCOqnNJyqObpvY74nn7Uj8l5q//VjtnJU513aM9m8RI+H3E495xvs5
icctY5b89M6xn/cj1T0zcx5tv8KcFXl37nH36j4XgWV0b/eBcVX3zdx8KVh3
+9LcFfn+PrFzdnzrnas467NgLhLNQAPt79G130fmescWxuEXB+scmPsuDNZs
cM/oGP7MnedmTj6zsD9El/BHbn3JscE5/Y4q/5ecRmyNSl3V6ajta5UPVZ29
iAFlHxesSegke3udf0rt1MEHButGeD+TgrUi+Ey0Nf/mzsejp/kvd94anc2K
3Pl1sCT6FvAAOpuVufUQYOTxhf0S/PmLFX8AX/BPbp8Gp5AGzwvktrPg/NaV
wfoKfFF9lT+p+kvot7Kfll2Pb5NYy7OWtp0Ta1UID96AS8j8jHW0vwuOk/lC
J3/R93ow9XfZsnonrZivVK8d/RmeIHUuYFvtH00dyzBe2pXWQLSD70mtdWnM
cey83udqs0P1LdZSe3frd7+T/br2B2X+rfqx9S+NZK8dW4fSWFtD2b9qv37k
vGBfclSy94LL03F97Rar/W2qb7eZ2ntU7f4jexPZz8lexRyXWNejatEpibUb
jIsFOn9o5ns7NfEYoP+/A9+V2adRt3tVfwc93+zU8/ugxM/JM34INiAW0LkN
VRbwTdRXGy/n9lVztd9Fx225T9W7P7WfmafynirfRHYbld2TOte5Sex8Qtvq
GckBwP/zPOvG1vb0T9w2PvAD1WlWuJ211EYX/Izs91XepHA7C2V/q3am6/y/
kd9TXeYd1X0BzKe2OsneN/Oc8KzKujNOVadZ7G/A+3+X2F/t3EbMIXv9wnWa
xlVet/ru5EV4hndkfy/7VuJm2XVLfz+2uPqOc8DJmdt/W3UaFH6HA1T2Su68
A3NEs9Lvl35fxM55oVnJg/PNi1V3fGafw7w6uZpbmU+JLcFU+NE9K1/6V2Jd
wKey+yV+bubB/dXGq2D02P6iFlvrtb6eoX3m8cV91ivtW5qovEPmZ2kb27fx
ThiLq6vxSP/cXFs92Y1Uv11lt5Z9d+q5gLls08zfJU7tvzK1NZu4JvO7ZY5r
Wvp7vEw8C6+ruksjj62U70Xcm1rftbb2jTP3/5nECJnHWkP4gtR96XW1k5TG
BA1U1iwzNngN51F6/NRX+XqZ/cnfkefbOpHn0xbZGgoielDtb5R5/HLNT9W4
o92f3dSaPrlO6XG+udqcmVoLh3aqW+x5hzzV8My5qt0K5+nQOIIrehXWAGyt
Oo+kzk911n5Oar59j8KYA7zRR/uFucfqG/rdb/S7N8NDyP6J3BjvXdvDqbWL
YOZtS+u9yFl9lhlT9Uk89nhXYIY2mZ+9qa6bkXoMbhZ7jDIGp+s3D2MeSuxv
62X+LqsSa08Wyf4h8ryXRZ4HG2ZrQtyoW+L+3bDymWtnrkMD/+jcNzJfAf/D
SarsMl10a24OdrL2M3LnU6jbObHm5M/IGIdv1FzX3J7ad81ibGX2sQuYT8lT
6lwLbXek1kQxhyyL/N3rqWyd6j67Jh4bDSr/s25pf/2S7BFqZ5rqlqnz0zzL
t5HnHNw3OpMzU+fxpwZrzMC3xBx5VsW/Ov9H+v91C+Nkz5U9Wfv/Uus00FRM
Ir6SfZH2f1Z1PoWzLI3t0Tqsjh27zQzOYc2OjEFPSoxp0UBwY8RHuwdrC6iP
lua81PWfC86vocfYAzyQuz44ed3UWln63pBgvP0lnGdpnvMKnY8z6yjQllyu
44Wy34GTC+bh8cPrpcbP5DI7VDgcvoRcIZzJponzdR9FjnXurvADzzsrWM+z
eWIu4WPZv6jstsKcxjWp3y38A5wlGgZ4S/o7+VP6fH/VPz937PZb4vwbulN8
7t6JffI75Gb1fceq3kJd91Xh/MIClTdX+Ym540g4dGLJ7qXzm2h0wYbk2cGH
E3U/f6fWycCtdqyw6yX4vdSamU6l80HkzsDC5H3Aw78mzumhj12cWANFXEbe
FW6L3Cv5n+9T5zXuC469yY/MTcwPwY3vVjq/jF7oX7iHwvH+/cE6FjRI5M6C
nuvr1PzbA4V5GL7tGRWP/afKZxbmYcaW1jOg0UIvCp7etfJjRwfP72jB0A8z
Nv8LztPDQ40vHXvT/8md1s3NU9GXiVfpz+hf0EIzdo7Gx+fO5z4ZnJPlfc4J
1n7Mj5wPWZo6J5LgG0tzYXCYDxXmxMCQdVNjhmcSc2bo06hLTpz6cFXPxuZv
ye3slptTfT44n0t/RheVJOYlwNJ1UuuiyZE2yf1+5gXrTxi/TwXnnRm/aGWJ
W8Ba5Fqb5X7P/zA+CnPmJ6bOzU+rytEiwNnFOn9aaY4MfnBebB6VHHD9xP35
bZ2fqO94tq75RvUvLJ13AHfDM4C94avg9onbyaHtl5vPPEv196+ZC/he9hk1
jxPwPrwEmB9NKHE5sT1aUWJ3MAhaWrgHOAniALgOYgE4v7EV7wd3BVcNf4Xm
FL6EmB5cA08OtnknM2eypq/m5mCJQdDbwkfCRcI94BPgH9DMwmPBbZFTIrdE
LubbxP4O30UsBTdFPIXeFp4J/qm17DI3T9omdz+kf36X+BvzfW/Q+ztC7+Gb
1H2fXDjfF+6TsQL/uV3u7853ITc4MPc9oLVsWX27nsRZucfv2qW15cx/6CLX
y83/wjvCd8B1EBvBQREfEfPBmRD3bSq7fm4eGK4RXhy+EZ7489RcMZpccnNw
gnDPi1Pzzzfqd4/Us3yr408K8+9w72hdwQBwoMRh5AuIxYh70B0R+zwSrG1g
nhpcOFdL7IsGZYvUsf+mpfPXaLz3LBx7gnceDh7X6DPRIcHXMHfAkaA3Y44C
y2ySGguhHWlXxVkXgwequY8+ie/Ab4BdN0zNexCbb5xaewkfQw4aTgZcvUHq
tQBXap9k1hY+GqzBYA6FN2iZOgZ8S2Ut9X5G5X43Z6XWHDIXwD/CPcLBgK3A
hXDA5H/ggdGGkyMiv4S2kfHHHEbcTC6J2HlZ5pwF3CxaY3I05KTgaMn/wNNe
q98aXHMOCQ1l3cT5BXSdm+XOa6Kv5B7XaCwz5zvgi4kLiXuIeS5TO9vUjG9O
LbwOBI4NHUm9xM/CmhjwOfgd/TX5Ebhm9JnXaftQ5VN03aCa8y6M4c2qOXdU
aR0g+lpyx3cWzi80Cx5TzMVH44eDMSeagNYV/qcfsjYADpzxhZaL/vyH9i+W
1pUTCx2aGNv/qHYmlM4rTFEbIXMs8yvYtrTOnfw56xHIoe9VWqPOd0FDh1YB
Xpe1D3DHcDtoHNtUceil2q9OjXngmQ5KzDWh40C3f2Hk+XhUYg6WnDo83Rr+
jfkvMQ92HP06t47lJf3+maX1w+T3C73DJXyL1Lq3GdRJnH9Gf3Kd6h5WOk+B
rmR5am0JeOWsxPE4c+gThfPa9BPWHpCbQKtC32BOfztxHyHvib6DdTHkmpk7
ni4qnUBpjTFx4JHMKcEY7MXCa5PAGeCazlWsgZaKNRrMv88XXvuEBrt5ae0x
8TkB0FOFc/docJiz0OGgWeiUu5w48/DE4xEdKGu14MPRorKeC947lF7XA44n
rh6QOK6Hnz0g8Tod4p0WFW5HW8f6iGMia9/QqwyJrLNjrcfQyPF+0yp+If4f
nBjHwm2eGOxnWC9DXE1MjSYDzmINz1B6/RExQOPSWm5iJLSYm1cx3Zd8t9Jr
p+AHhyS+N+KkgYnb+VHn7yi9FgqOtHkVm6ARgTfhGaeWXtdBPghubp/Ea7LQ
a8JPwU2hDYJbgVdBFwK/A7eDXgS+Ca4Jfz08MX+Fno71ceQl0Q/BkcEjocdh
fdya3E3p9SDgAbQ/rIFCD/Ad46n0+jmwATiSeRm+9ITgZ4f3YM0V3xRubt/E
WAj+pGFp/wyP2rK0b4cvalbF7HtX4/2cys/0L+0biVublPbJ8E5o6/HtaFLQ
ppBTRlMIV8W4mFZ6DQyY6BLtt6w5Xr0j2I/jw+Hk4A/gDlhfSLwGLoUjJIYj
ZrszeG3dQ5H1UnBk8GNo/Zhz8W+Tdb5DzbwDa+ngCuAAmG9OSKxbQKuW1jwf
k68cnPoZyTkOSq0zhJNsX3reRBPEnDw9ss9AQ47fmJ3YR6KfYU0eMSlxKlwy
MTFxNWva4DrgGeD4iWsZO5eqna1rjoFfK7zOEDy0oPC6QfANXPnQxO/2g8Ka
QrDk2aXjCPJ36GFuL4zP0aKRIyGWHFNaZ768iinQ2oKj0JzB78Ptf1Q4TiBG
oE8/Ejt3f1hqzINfZQ0ivA3cyc6Z43RyN4sL6yOJzR7IzdWQy0ArmNWMabpl
5mGIx9HxgdHwpZN03VY18z7kbdFhoMEgn35Iam0h2sC2ufH25qXXLcKfkAsl
JwrmR58HF8w4Qs+CrgVtCWvZyJ30ipwHPyq1bySffmRqLSI+eHHlh0eq7MeK
ByBXflJq30iO+9jU8wW6rR1y+2e++WvVd0f/uEVun49mbVlqrRo55aGpdWUf
Jsa+YE708vhu/DaaStav0T5rQXheno/46JnC+q7rS68rI+75PLFfJ8d3RmkN
P3o6NHc/p1VcGXtdFnHTszo/vLT+jtw0mgn0EvMKX8+15OXRoKA/GaH9D6nj
1rcS3wvv+e/Ca0IZg7THWgbaRLu3MjVvQG4dXQ6aHHR2u+b+FujETks975Nz
P7GKm4h9WTNG/It269RqziVuHlHdA/runap5Cp3Goan75EeJfQ1+Bp0F8TXt
rdC93Vl6fQ+6CWI0tBM/F45FiUNX69odEvNUVzH+M6+pgTfAv+BbiMdaJOZJ
4M9uzs2hsVYFXmaeyo8JxsT4zP907Qul16hlpddrMBb+BJeWXlfXS/V/T43H
WB/TL7eWjvF+Us0xCX7gxJrjvQ90XRu03bL/AteXXofHeGta8SHoyVomxsys
CYIPIp76CCygay/NzQ2MT70m6CuV31x6nfHXzGOl1xmzlor4h9iH+/sqMYdD
DHJ7MM5hndeuwX2PNRLnp67TvfC6WfJySeo1PBsmXgvWIzg33bfwelHyfPij
JYnXGf2rsudLr+fbpfA6W3hxdIUrUmsLEz3H/NLrEfsVXlNK7nBR4fmDuePr
wtwK/Z845oLUsQzfukj9vcF7J6DRz60T/Cm1VhBtDrwE4wYdHOMXLdynxCmq
f1VubQiYEH0I67QY7/+vDtwBsQ9aRTAhePH3wjwdeJW1a+Arxv8vhfkRuJHt
EvepzyPrbW7IjdvBb8fVPH54l7zTd1R+tfaZ3sEHkfl4ePm1I/Ow/1T8Knv4
WeDe56U5EfiQLPU5yuEDVxZ+/78V5kEWVO8qVb0vIq83a5x4TeJ6pXWw+HzW
/7EOihgF3dYvqbVbl6XRGnJ4YTVPDdP9H6V6Q7T9lnqu5FlC6vVdw+nTqfsC
fC1aXOYI1guytoo1g2typqnzpqx9BLejMeI5eJ7OFfe2fsW/oUUCv6NH6pSY
e16M71X7k3Q/l+TWvKxKPW+ig/s1tRaOPsLz8+wrVf/20msBqfd94rHD/x7c
kZs3xg92SWyn1bXbJ16rO7Cay/4rqrEWWbfLGhz0Q4xN1tVzz/8WxqXwXXDm
t+XuD3Dpt+e+/5rO31o65/Jm8DE2ebSyNE/LfIrfweeQX+B/E8DPxJm7J87j
gN1YJwJ+W1147JMfIbfCMXZZ4UnarKP9baVzYVeUjiWII8BX8CNLInNdrLuB
VwHrdE+cIyAfxP8RgDPBczsmzjuQL+CZeV5yx7XU+Llhhb3BouBDMD14nniL
dUnEXOAl+iL9sEnl58GfxNHgfjA/+Y5btH0WOcdXp7p/+kLHqj+QF+D/FPhd
cvHkBsgLvBr8zfhel5eOwYi/wMWDKj9PPned1DEOcTP/D7ALY7M0RgGfsCZx
/+Cx078wdwxvjD+H15ir8h8Kcy74KPIw2yTOxeAPtq18AvwrHDT8Mzzh7OB+
uLT6LvzvxZKqb/C/C1sV/h8BcsWskTyg6mPLVDaj9P8rfFuYU4ZPhsNdmbj9
ZYX5I7ijToV5Ijgi8O9dFQa+qvTaM3Qj2xb+DwVy1YyvF6rnhZ8ht0GegvHc
M3FOh7xeXNoGB/dI3N8OKfxfHmt0YIX/4wOtIflVtPHw/cOD/5+hR2ROp3fi
nNfQwv+jgc6ONYX8Z8jukTXU/D/Fmv+m0HvoF8zPsH6SPBP3AP7ulTgPdVzh
3AB5AbRJ5CPJr7EGkXwAuQDWO5OTgL8lP8V/UtAHWKO6T8U74RdZC4RvJE4/
vmZuGr+wusIb9Ktta86zvkKsH4wV+2bOO/O85DLA7vhG1iKTbyXHylzKbzCf
suZ3Su65AA07OR34f/4HhfwvuWC06a0TY5tDMuf3+Xb01R1qzndyD/BEzPlo
h8lboR9GH0dOBI0c63TJpZC/QqdGzou+A4/9dPVu6Q871axj+DS3ToMYlz7Z
rWaNBfp6+D2wFbHDaTXnrtB2oVmAPxicOdcPrzA3eAzQP9smXt8M37Us8Tvm
nfOfJWgM0G2gLUI/QsyEFoA+3yayrnVa7lwV/7WAngTtFP9TQg6dSRH/tl3N
ugfWl4PzwZdoAOGd0AGix0E7QNzPmmNyc8w/jIvta9ZSoH9Bj0Cczf8HwIPB
iR2YWXcB3wB380dsrIXWD4eJ3g8OjWfmecEvv1cYhnWgu9Ssa4F/OLjm9Ris
twATrllzURhnoqdlTQwYh7gGPSzrQtDEoqVl7QV6Wv5rgXUWaHPR6qLZR69L
rHFozVpwYvZDal6/wf9AsM4CbS4aXtaXoOM9PTWOBduDZVkfQp6iY+68Cfma
Ivd6E+Y+coBgObhHYpAbg/NB/HcF6+jRBNfJvQ4Fnw/ndlDNa1T4fxd0dGga
wFisLSeWRNsOhrwosraOuRtdHOv70QGiBTkus34P7nBMamwJzkdvQu4dzgnd
Kzw22lc0F2jK4MPwhzvXrFs6O7PuCN+4fm6NPXrx4zNrAuEaWaPapWa9I7oq
dJ7kRvkPBnT7aMrR8hMvjNM9vJdb6wjHDh+9SeK+yn+l4MvwY2Dt3yq8PSyz
BhI+A/4bHAsGZs0f+l/0uYyVNtV4WZ6YM8B3kRfYq2Yt4KLcOj00dZ/l1vKh
IWTO2rNm3SFrXMCc+JJzU+MlfPvUzFpE5rWluTWH5FLB+X1q1u2RVwfLEVOD
7XvVrCkcmxo3MgehFyBGwTdOSI3NiNGIKXrWrH+6Fj+UeX7kfw7A4zwf+gIw
HXM7ccHuNWsNWWdDTEdsxNjbMnGuGf07MT4a+FDFKeDqrMKa+MzuwXE0cTH/
P0HsCa9wQO44l5iX2IZr8wqfci04fFJmXRY8NLpy4mi05ayv/Da3PhCef7+a
1wvxPwlnV/2QNRZoDMj1wMmcWnMuar/S/5dC3gF/wzpSfM55uXPT8DZov5kj
mR/J6fSuWbMCXiD2AzN8mVuXAlfNOjtibHA7c+g+Na/5QRuLjgW9Cv+Vwpoj
1hldp32ZGVMRR/5VxZL8lwnfBd3S77nXHJHH53+PWLfEOidw/K+x5yx4i99i
x4/8b8qatRix/zcFjTrrTvnPJ3TcrGN4LPPaJeZl1sCh/SaPTwxKbEE7/IcK
9waGAkvsXbPu/K7M67DQKnB/WyXOR/D/Lqzt4v8g4EbeKrz+Ef0DnBp4//3c
Ohx0Pvz3BvE38RD4pG/Nui5iq/Y18wLgh31rXiv1fyP1sBE=
         "]], Polygon3DBox[CompressedData["
1:eJxFnAPU7NjWtSuVVFLV5m3jtG3btm7btm3btm3btm3ctu3/ec7MP74xOn0y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         "]], Polygon3DBox[CompressedData["
1:eJwl13u0zlUex/HHOec552HMxJhGJ0YjlELqSLpQphYjZLk1OEkYMmS5pGIo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         "]]}]}, {}, {}, {}, {}}, {{}, {}, {}, {
      Line3DBox[{780, 1042, 1451, 1043, 2274, 781, 1081, 1470, 1082, 2284, 
       797, 1110, 1483, 1111, 807, 1139, 2374, 1496, 1140, 817, 2315, 1168, 
       1509, 1169, 827, 2320, 1197, 1522, 1198, 837, 2327, 1226, 1535, 1227, 
       847, 1255, 1548, 1256, 2295, 857, 1284, 1561, 1285, 2305, 867, 1313, 
       1574, 1314, 877, 1342, 1587, 1343, 887, 1371, 1600, 1372, 897, 2337, 
       1400, 1613, 1401, 907, 2344, 1427, 1626, 1428, 1430}], 
      Line3DBox[{782, 1044, 1452, 1045, 2275, 1454, 1048, 1083, 1049, 2285, 
       1471, 1085, 1112, 2362, 1086, 1484, 1114, 2364, 1141, 1115, 1497, 1143,
        2375, 1170, 1144, 1510, 2430, 1172, 1199, 1173, 1523, 2432, 1201, 
       1228, 1202, 1536, 1230, 1257, 1231, 2296, 1549, 1259, 1286, 1260, 2306,
        1562, 1288, 1315, 2403, 1289, 1575, 1317, 1344, 1318, 1588, 1346, 
       2408, 1373, 1347, 1601, 1375, 1402, 1376, 1614, 2441, 1404, 1429, 1405,
        1432}], 
      Line3DBox[{783, 1046, 1453, 1047, 2276, 1456, 1053, 1084, 2356, 1054, 
       1472, 1088, 1113, 2363, 1089, 1485, 1117, 2366, 1142, 1118, 1498, 1146,
        2376, 1171, 1147, 1511, 1175, 2383, 1200, 1176, 1524, 2433, 1204, 
       1229, 1205, 1537, 1233, 1258, 1234, 2297, 1550, 1262, 1287, 2398, 1263,
        1563, 1291, 1316, 2404, 1292, 1576, 1320, 1345, 1321, 1589, 1349, 
       2409, 1374, 1350, 1602, 1378, 2414, 1403, 1379, 1615, 2442, 1407, 1431,
        1408, 1434}], 
      Line3DBox[{784, 1050, 1455, 2422, 1052, 1458, 1057, 1087, 2357, 1059, 
       1473, 1091, 1116, 2365, 1093, 1486, 1120, 2367, 1145, 1122, 1499, 1149,
        2378, 1174, 1151, 1512, 1178, 2385, 1203, 1180, 1525, 1207, 2389, 
       1232, 1209, 1538, 1236, 1261, 2393, 1238, 1551, 1265, 1290, 2399, 1267,
        1564, 1294, 1319, 2405, 1296, 1577, 1323, 1348, 1325, 1590, 1352, 
       2410, 1377, 1354, 1603, 1381, 2416, 1406, 1383, 1616, 1410, 2420, 1433,
        1412, 1436}], 
      Line3DBox[{789, 1066, 1462, 1067, 2279, 790, 1097, 1477, 1098, 2288, 
       802, 1126, 1490, 1127, 2292, 812, 1155, 2379, 1503, 1156, 822, 2317, 
       1184, 1516, 1185, 832, 2323, 1213, 1529, 1214, 842, 2331, 1242, 1542, 
       1243, 852, 1271, 1555, 1272, 2300, 862, 1300, 1568, 1301, 2309, 872, 
       1329, 1581, 1330, 2312, 882, 1358, 1594, 1359, 892, 2334, 1387, 1607, 
       1388, 902, 2340, 1416, 1620, 1417, 912, 2348, 1439, 1630, 1440, 1442}],
       Line3DBox[{791, 1068, 1463, 1069, 2280, 1465, 1072, 1099, 1073, 2289, 
       1478, 1101, 1128, 2369, 1102, 1491, 1130, 2371, 1157, 1131, 1504, 1159,
        2380, 1186, 1160, 1517, 1188, 1215, 1189, 1530, 2435, 1217, 1244, 
       1218, 1543, 1246, 1273, 1247, 2301, 1556, 1275, 1302, 1276, 1569, 1304,
        1331, 2406, 1305, 1582, 1333, 1360, 1334, 1595, 1362, 2411, 1389, 
       1363, 1608, 1391, 1418, 1392, 1621, 2443, 1420, 1441, 1421, 1444}], 
      Line3DBox[{792, 1070, 1464, 1071, 2281, 1466, 1074, 1100, 2360, 1075, 
       1479, 1103, 1129, 2370, 1104, 1492, 1132, 2372, 1158, 1133, 1505, 1161,
        2381, 1187, 1162, 1518, 1190, 2386, 1216, 1191, 1531, 2436, 1219, 
       1245, 1220, 1544, 1248, 1274, 1249, 2302, 1557, 1277, 1303, 2402, 1278,
        1570, 1306, 1332, 2407, 1307, 1583, 1335, 1361, 1336, 1596, 1364, 
       2412, 1390, 1365, 1609, 1393, 2417, 1419, 1394, 1622, 2444, 1422, 1443,
        1423, 1445}], 
      Line3DBox[{913, 1025, 1424, 1396, 903, 1017, 1395, 2418, 1367, 893, 
       1009, 1366, 2413, 1338, 883, 1001, 1337, 1309, 873, 993, 2396, 1308, 
       1280, 863, 985, 2392, 1279, 1251, 853, 977, 1250, 1222, 843, 969, 1221,
        1193, 833, 961, 1192, 2387, 1164, 823, 953, 1163, 2382, 1135, 813, 
       945, 1134, 2373, 1106, 803, 937, 2354, 1105, 1077, 793, 929, 2349, 
       1076, 1035, 776, 921, 1447, 1033, 1446}], 
      Line3DBox[{914, 1026, 1623, 1398, 2341, 904, 1018, 1610, 1369, 894, 
       1010, 1597, 2439, 1340, 884, 1002, 1584, 1311, 874, 994, 2397, 1571, 
       1282, 864, 986, 1558, 1253, 854, 2437, 978, 1545, 1224, 844, 970, 1532,
        1195, 2324, 834, 962, 1519, 1166, 824, 954, 1506, 2429, 1137, 814, 
       946, 1493, 2424, 1108, 804, 938, 2355, 1480, 1079, 794, 930, 2350, 
       1467, 1038, 777, 2421, 922, 1448, 1034, 1036}], 
      Line3DBox[{915, 1027, 1624, 1425, 2342, 905, 1019, 1611, 1397, 2335, 
       895, 1011, 1598, 1368, 885, 1003, 1585, 1339, 875, 995, 1572, 1310, 
       865, 2303, 987, 1559, 1281, 855, 2293, 979, 1546, 1252, 845, 971, 1533,
        1223, 2325, 835, 963, 1520, 1194, 2318, 825, 955, 1507, 1165, 2313, 
       815, 947, 1494, 2425, 1136, 805, 939, 2361, 1481, 1107, 795, 2282, 931,
        1468, 1078, 778, 2272, 923, 1449, 1037, 1039}], 
      Line3DBox[{916, 1028, 1625, 1426, 2343, 906, 1020, 1612, 1399, 2336, 
       896, 1012, 1599, 1370, 2332, 886, 1004, 1586, 1341, 876, 2310, 996, 
       1573, 1312, 866, 2304, 988, 1560, 1283, 856, 2294, 980, 1547, 1254, 
       846, 972, 1534, 1225, 2326, 836, 964, 1521, 1196, 2319, 826, 956, 1508,
        1167, 2314, 816, 948, 1495, 1138, 806, 2290, 940, 1482, 1109, 796, 
       2283, 932, 1469, 1080, 779, 2273, 924, 1450, 1040, 1041}], 
      Line3DBox[{917, 1029, 1435, 1411, 908, 1021, 1409, 2419, 1382, 898, 
       1013, 1380, 2415, 1353, 888, 1005, 1351, 1324, 878, 997, 2400, 1322, 
       1295, 868, 989, 2394, 1293, 1266, 858, 981, 2391, 1264, 1237, 848, 973,
        1235, 2390, 1208, 838, 965, 1206, 2388, 1179, 828, 957, 1177, 2384, 
       1150, 818, 949, 1148, 2377, 1121, 808, 941, 2358, 1119, 1092, 798, 933,
        2352, 1090, 1058, 785, 925, 2351, 1457, 1051, 1055}], 
      Line3DBox[{918, 1030, 1627, 1414, 2345, 909, 1022, 1617, 1385, 899, 
       1014, 1604, 2440, 1356, 889, 1006, 1591, 1327, 879, 998, 2401, 1578, 
       1298, 869, 990, 2395, 1565, 1269, 859, 2438, 982, 1552, 1240, 849, 974,
        1539, 1211, 2328, 839, 966, 1526, 2434, 1182, 829, 958, 1513, 2431, 
       1153, 819, 950, 1500, 2426, 1124, 809, 942, 2359, 1487, 1095, 799, 934,
        2353, 1474, 1062, 786, 2423, 926, 1459, 1056, 1060}], 
      Line3DBox[{919, 1031, 1628, 1437, 2346, 910, 1023, 1618, 1413, 2338, 
       900, 1015, 1605, 1384, 890, 1007, 1592, 1355, 880, 999, 1579, 1326, 
       870, 2307, 991, 1566, 1297, 860, 2298, 983, 1553, 1268, 850, 975, 1540,
        1239, 2329, 840, 967, 1527, 1210, 2321, 830, 959, 1514, 1181, 820, 
       951, 1501, 2427, 1152, 810, 943, 2368, 1488, 1123, 800, 2286, 935, 
       1475, 1094, 787, 2277, 927, 1460, 1061, 1063}], 
      Line3DBox[{920, 1032, 1629, 1438, 2347, 911, 1024, 1619, 1415, 2339, 
       901, 1016, 1606, 1386, 2333, 891, 1008, 1593, 1357, 881, 2311, 1000, 
       1580, 1328, 871, 2308, 992, 1567, 1299, 861, 2299, 984, 1554, 1270, 
       851, 976, 1541, 1241, 2330, 841, 968, 1528, 1212, 2322, 831, 960, 1515,
        1183, 2316, 821, 952, 1502, 2428, 1154, 811, 2291, 944, 1489, 1125, 
       801, 2287, 936, 1476, 1096, 788, 2278, 928, 1461, 1064, 1065}]}, {
      Line3DBox[{1631, 1795, 2041, 921, 2040, 1796, 2043, 2421, 1735, 2042, 
       1798, 2272, 1736, 2044, 1800, 2273, 1737, 2045, 2206, 2274, 1738, 2046,
        2207, 2275, 2048, 1739, 2047, 2208, 2276, 2050, 1740, 2049, 2422, 
       1801, 2052, 2187, 2351, 2051, 1803, 2054, 2423, 1741, 2053, 1805, 2277,
        1742, 2055, 1807, 2278, 1743, 2056, 2210, 2279, 1744, 2057, 2211, 
       2280, 2059, 1745, 2058, 2212, 2281, 2060, 1746, 1808}], 
      Line3DBox[{1632, 1797, 2061, 2188, 2349, 1810, 1799, 2063, 2189, 2350, 
       2062, 1812, 2282, 1747, 2064, 1814, 2283, 1748, 2065, 2214, 2284, 1749,
        1815, 2209, 2285, 2066, 1750, 1816, 2356, 1802, 2067, 1751, 1818, 
       2357, 1804, 2068, 2190, 2352, 1820, 1806, 2070, 2191, 2353, 2069, 1822,
        2286, 1752, 2071, 1824, 2287, 1753, 2072, 2215, 2288, 1754, 1825, 
       2213, 2289, 2073, 1755, 1826, 2360, 1809, 2074, 1756, 1828}], 
      Line3DBox[{1633, 1811, 2075, 2192, 2354, 1830, 1813, 2078, 2193, 2355, 
       2077, 1833, 2194, 2361, 2080, 1836, 2290, 1757, 2081, 1111, 1758, 1837,
        2362, 1817, 2082, 1759, 1838, 2363, 1819, 2084, 1760, 1841, 2365, 
       1821, 2086, 2195, 2358, 1844, 1823, 2089, 2196, 2359, 2088, 1847, 2197,
        2368, 2091, 1850, 2291, 1761, 2092, 2216, 2292, 1762, 1851, 2369, 
       1827, 2093, 1763, 1852, 2370, 1829, 2095, 1764, 1855}], 
      Line3DBox[{1647, 1857, 2096, 1856, 2372, 1646, 1854, 2094, 1853, 2371, 
       1645, 1881, 1880, 2379, 1644, 2428, 1879, 1878, 1643, 2427, 1875, 1874,
        1642, 2426, 1849, 2090, 1848, 1641, 2377, 1846, 2087, 1845, 2367, 
       1640, 1843, 2085, 1842, 2366, 1639, 1840, 2083, 1839, 2364, 1638, 1867,
        1866, 2374, 1637, 1138, 1865, 1636, 2425, 1862, 1861, 1635, 2424, 
       1835, 2079, 1834, 1634, 2373, 1832, 2076, 1831, 1860}], 
      Line3DBox[{1661, 1885, 2104, 1884, 2381, 1660, 1883, 2103, 1882, 2380, 
       1659, 1902, 2317, 2222, 1658, 2221, 2316, 1901, 1657, 1181, 1898, 1656,
        2431, 1877, 2102, 1876, 1655, 2384, 1873, 2101, 1872, 2378, 1654, 
       1871, 2100, 1870, 2376, 1653, 1869, 2099, 1868, 2375, 1652, 1892, 2315,
        2219, 1651, 2218, 2314, 1891, 1650, 2217, 2313, 1889, 1649, 2429, 
       1864, 2098, 1863, 1648, 2382, 1859, 2097, 1858, 1888}], 
      Line3DBox[{1675, 1905, 2112, 1904, 2386, 1674, 1903, 2111, 1188, 1673, 
       1919, 2323, 2232, 1672, 2231, 2322, 1918, 1671, 2229, 2321, 1916, 1670,
        2434, 1900, 2110, 1899, 1669, 2388, 1897, 2109, 1896, 2385, 1668, 
       1895, 2108, 1894, 2383, 1667, 1893, 2107, 2430, 2220, 1666, 1911, 2320,
        2226, 1665, 2225, 2319, 1910, 1664, 2223, 2318, 1908, 1663, 1166, 
       2106, 1890, 1662, 2387, 1887, 2105, 1886, 1907}], 
      Line3DBox[{106, 660, 843, 107, 662, 844, 108, 845, 109, 846, 110, 847, 
       111, 1536, 665, 112, 1537, 666, 113, 1538, 667, 848, 114, 669, 849, 
       115, 850, 116, 851, 117, 852, 118, 1543, 672, 119, 1544, 673, 120}], 
      Line3DBox[{1689, 1921, 2120, 2436, 2234, 1688, 1920, 2119, 2435, 2233, 
       1687, 1932, 2331, 2242, 1686, 2241, 2330, 1931, 1685, 2240, 2329, 1929,
        1684, 2230, 2328, 2118, 1917, 1683, 2390, 1915, 2117, 1914, 2389, 
       1682, 1913, 2116, 2433, 2228, 1681, 1912, 2115, 2432, 2227, 1680, 1927,
        2327, 2237, 1679, 2236, 2326, 1926, 1678, 2235, 2325, 1924, 1677, 
       2224, 2324, 2114, 1909, 1676, 1193, 2113, 1906, 1923}], 
      Line3DBox[{1690, 1922, 2121, 977, 1933, 1925, 2123, 2437, 1765, 2122, 
       1935, 2293, 1766, 2124, 1937, 2294, 1767, 2125, 2245, 2295, 1768, 1938,
        2238, 2296, 2126, 1769, 1939, 2239, 2297, 2127, 1770, 1940, 2393, 
       1928, 2128, 2198, 2391, 1942, 1930, 2130, 2438, 1771, 2129, 1944, 2298,
        1772, 2131, 1946, 2299, 1773, 2132, 2247, 2300, 1774, 1947, 2243, 
       2301, 2133, 1775, 1948, 2244, 2302, 2134, 1776, 1949}], 
      Line3DBox[{1691, 1934, 2135, 2199, 2392, 1951, 1936, 2137, 986, 2136, 
       1953, 2303, 1777, 2138, 1955, 2304, 1778, 2139, 2248, 2305, 1779, 1956,
        2246, 2306, 2140, 1780, 1957, 2398, 1941, 2141, 1781, 1959, 2399, 
       1943, 2142, 2200, 2394, 1961, 1945, 2144, 2201, 2395, 2143, 1963, 2307,
        1782, 2145, 1965, 2308, 1783, 2146, 2249, 2309, 1784, 1966, 1276, 
       2147, 1785, 1967, 2402, 1950, 2148, 1786, 1969}], 
      Line3DBox[{1692, 1952, 2149, 2202, 2396, 1971, 1954, 2151, 2203, 2397, 
       2150, 1972, 995, 2152, 1973, 2310, 1787, 2153, 1314, 1788, 1974, 2403, 
       1958, 2154, 1789, 1975, 2404, 1960, 2155, 1790, 1976, 2405, 1962, 2156,
        2204, 2400, 1977, 1964, 2158, 2205, 2401, 2157, 1978, 999, 2159, 1979,
        2311, 1791, 2160, 2250, 2312, 1792, 1980, 2406, 1968, 2161, 1793, 
       1981, 2407, 1970, 2162, 1794, 1982}], 
      Line3DBox[{472, 448, 1337, 470, 451, 1584, 717, 473, 1585, 719, 475}], 
      Line3DBox[{491, 469, 1361, 489, 467, 1360, 488, 487, 1594, 727, 1593}], 
      Line3DBox[{1706, 1998, 2170, 1997, 2412, 1705, 1996, 2169, 1995, 2411, 
       1704, 2014, 2334, 2253, 1703, 2252, 2333, 2013, 1702, 1384, 2011, 1701,
        2440, 1994, 2168, 1993, 1700, 2415, 1992, 2167, 1991, 2410, 1699, 
       1990, 2166, 1989, 2409, 1698, 1988, 2165, 1987, 2408, 1697, 2005, 1371,
        1696, 2251, 2332, 2004, 1695, 1368, 2002, 1694, 2439, 1986, 2164, 
       1985, 1693, 2413, 1984, 2163, 1983, 2001}], 
      Line3DBox[{1720, 2017, 2178, 2016, 2417, 1719, 2015, 2177, 1391, 1718, 
       2030, 2340, 2263, 1717, 2262, 2339, 2029, 1716, 2260, 2338, 2027, 1715,
        1385, 2176, 2012, 1714, 2419, 2010, 2175, 2009, 2416, 1713, 2008, 
       2174, 2007, 2414, 1712, 2006, 2173, 1375, 1711, 2023, 2337, 2257, 1710,
        2256, 2336, 2022, 1709, 2254, 2335, 2020, 1708, 1369, 2172, 2003, 
       1707, 2418, 2000, 2171, 1999, 2019}], 
      Line3DBox[{1734, 2032, 2186, 2444, 2265, 1733, 2031, 2185, 2443, 2264, 
       1732, 2039, 2348, 2271, 1731, 2270, 2347, 2038, 1730, 2269, 2346, 2037,
        1729, 2261, 2345, 2184, 2028, 1728, 1411, 2183, 2026, 2420, 1727, 
       2025, 2182, 2442, 2259, 1726, 2024, 2181, 2441, 2258, 1725, 2036, 2344,
        2268, 1724, 2267, 2343, 2035, 1723, 2266, 2342, 2034, 1722, 2255, 
       2341, 2180, 2021, 1721, 1396, 2179, 2018, 2033}], 
      Line3DBox[{476, 477, 1344, 456, 478, 1345, 458, 480, 1348, 460, 1351, 
       482, 462, 1591, 724, 484, 1592, 726, 486}], 
      Line3DBox[{1586, 720, 1587, 476}]}}},
   VertexNormals->CompressedData["
1:eJxkfXk8Vl3/taRSihRJpNKkNMiQSq5T0UgpUwkJoURFIkNShhAypCSFEmmO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    "]],
  Axes->True,
  Method->{},
  PlotRange->{{-4., 3.999999999999597}, {-3.999999999999899, 
   3.999999999999899}, {-0.9999999999999748, 0.9999999999999748}},
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.6291870229716625`*^9}]
}, Closed]],

Cell["Example of rotating function graphs", "Text",
 CellChangeTimes->{{3.59427376520831*^9, 3.594273780472183*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"id", "[", "u_", "]"}], " ", ":=", " ", "u"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5922276737623525`*^9, 3.5922277054741664`*^9}, {
  3.5942736968433995`*^9, 3.594273702465721*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"sfrev", "[", 
   RowBox[{"Cos", ",", "id"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", "u", "]"}], " ", 
    RowBox[{"Cos", "[", "v", "]"}]}], ",", 
   RowBox[{
    RowBox[{"Cos", "[", "u", "]"}], " ", 
    RowBox[{"Sin", "[", "v", "]"}]}], ",", "u"}], "}"}]], "Output",
 CellChangeTimes->{3.629187051241912*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sfrev", "[", 
     RowBox[{"Cos", ",", "id"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{
      RowBox[{"-", "Pi"}], "/", "2"}], ",", 
     RowBox[{"4", "*", 
      RowBox[{"Pi", "/", "2"}]}]}], "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5922257630910683`*^9, 3.592225775968805*^9}, {
   3.592225989373011*^9, 3.5922259964974184`*^9}, {3.5922260927779255`*^9, 
   3.5922261304900827`*^9}, {3.5922261650030565`*^9, 
   3.5922261971228933`*^9}, {3.592226229242731*^9, 3.5922263441073008`*^9}, {
   3.592226740379966*^9, 3.592226750132524*^9}, {3.592226787731674*^9, 
   3.59222679937234*^9}, {3.59222687175648*^9, 3.592226888444435*^9}, {
   3.5922269726212497`*^9, 3.5922269743323474`*^9}, 3.5922270304075546`*^9, {
   3.5922271192936387`*^9, 3.5922271563647594`*^9}, {3.592227492203968*^9, 
   3.59222750168251*^9}, {3.5922275492762327`*^9, 3.592227574379668*^9}, {
   3.592227739908136*^9, 3.5922277753561635`*^9}, {3.592228075636338*^9, 
   3.592228080619623*^9}, {3.5942731171882453`*^9, 3.594273179040783*^9}, {
   3.594273287980014*^9, 3.59427333423666*^9}, {3.5942733772041173`*^9, 
   3.5942733783931856`*^9}, {3.5942734515083675`*^9, 
   3.5942734539215055`*^9}, {3.594273661022351*^9, 3.594273673524066*^9}, {
   3.5942737331404757`*^9, 3.594273734280541*^9}, {3.594273822492586*^9, 
   3.594273824035675*^9}}],

Cell[BoxData[
 Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJx0XXc819/3V9o70TJKSlrag/S+GlTKaIn2XtKgKIRCQ2UlJSklipZQMnpf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   "], {{
     {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJxFnXXcFsXXxvfee2vWQEXFllARAwS7EFsRERvjp4SFHdhYqIAtiomJCaJi
g4mFAVhYYDdigtj1Xt/nGj/vH/PsuWdnZ2dr5pzrXOc87QYctcuRaZIkf1dJ
0tT2ozJJ+qksKvktbeuQJC9JflNykPyi5B2LJBmo0l4H9tR2gEpbyXtqe7DK
apJ3UWerqvyl9u/o2AV17FTJ07Mk+VFtPpPcVnUb6LxLS35FbdaQ/LTkQ7T/
bR27mfp5VfVNtXtG9WeqbhfVLdLQOVW/Wp0kS6h+wTxJjtExleqXU9t11U8b
1b+vNmtKfk3yLMm7qP0rkl+XvJ3aPy/5Ncm5jnlW8kDJ3dRmGa5dfX6v8oj6
DNoerfa55G6St1PpKPkwte+h/jur/eLa3q/ft0juq7avaqyra6zTVNdbfT7F
tUtOdK7J2nyj499SuULyD+qv0jGv6Pe2Ou4NbS9X/V+qX0L17+n3dyrvqFyp
+u10rvVVVpHcS9uNVFaVvIjar6VSq90hqutNO9UfofOupzFsIHmA6nZQWZf7
oGcxT/1/IfkAtVlHbZaTfKDkrmrTVvLC6q+zSqBP1a+tNp1Uf5DkLmqzouRD
JW8huYvkIRr/9rruVO0/4D3S9b6u+vckt5L8quQ+2r+g9uuQ5Fu1X1jlU8mZ
znOkxtPQvr3UZhltF+beqv541S+k31+XHivjPEF1e+nY3dT2q9LPj2e3hura
qO2CknMde1Th8XxW+r3hndkx9fPU7mQj/RmqYzrr91KSN1BZVPJqnL/hd+mH
0tfPta/Ne6ayvOS5qhuksrrkH9XnCqpfVqVQH6urNCV3Ud9LUa8262q7osoK
jFP7t1bpoN+Dua74XI6VvL/ktSX/JvlIlY0k91S7dVTW4Llru57KmpK7qf+u
jJfXS8fdpfYXSZ6quodUzpT8oepa6/6/kfj7XqvyN95fcmfJi0jeTWM5T31t
rWP+ldxG9+0DyZurrN7w+/az2h9eemzfqb8f9D5cqmOOUd3/1E831XfRsduq
rKxjTmL8qt9Q9adKPkHyJpJPkXyc5I0ld9Xxi6l8KXlDHbeLypo69mS1Wb7y
tW/a8DfXQfL2atte5RfJD6rNx6q/jrFp+5HKaMn36Vk8J/k8yfeo7bOST+Ub
UX+nMY/p90p6/+/XvrvUdoLK06oboTbHqf4RlWskj1PdM4xF8qdMkCrP63em
/YOinKrPx1WGave9TZ/vdvXXQWPbp3S7y9XuLpVD1eYO/R4v+XC+ce1/sPLY
Dmc8mcd8m46/W21O4ftV3QMq50v+R+0f5hokf6J+/lG7e9Suqf0HNy0/rO1D
KvdrX6W2L6hcqPar6Nh9VQ7TvmPV/mGVq1V/g57zc7rnJ0ou1fZo3Z+x6ufW
1NfPtc9T3VU8Y8k3qs09KvtIvkVtLuM5x2t5U+/Dvdw71V/JHKn6P1U/Q/X3
qP4Wnet9netcjtX+iSrHMc+ov5N1jif5diQ/pTJM9WPU/l21P5vx81w09kfU
dyvtP4W5lO9c8kmSH5O8gOR3dK6H1PZWHfth7n5+Unm84fPuof3fqvyr/ePV
16MNPzuu/WmV4ZJr3kGVQvtuYj7Pfc/HpV4T+KZWUv3TGs/frBGsh8ylkn9n
rlG5QW2+0HF/q0zhPdJ9mKv+H1D908wZrBmSn5X8k+QHJa+lPrZXCTrPZNV/
r/r7+H4bfmd5jouqbkLpd/5T9f1n7nd9q6bvwTnUaztN5RKuRf09yvel3wvp
2Lt17KjE92q85Mskb6P9bVXmS3674fvKfXiz4WfCPfxR53lAbV7V77XV5w4q
C2icU9THb+prktq8JPkvvgfJO6ntKip/SG6obmLl+7al6h5QH2dIbtXwvM5a
wLt6nJ7jfQ3PY4Ml38/9V3lQ5Ri1OUrP9UGVS7kutTk+eI67WWN5I3efXzBv
BK8xd+lcD2t7LPO8jltMbViGH9WfL5grWXf0Lvyk8qLaPqX6ldXXBNW/IPl3
tXlUcqrr/JZvQvLDqv818VrfU8d8rfoxkpfWcV1V5uv342ozm3tN/zr2e9Xd
JnmS6j9X/Tj61HgO1PFXMbc2vMYuzryqNgerrISeoP2doh7VRf38mVifeZG1
V/Uvx2f6WuV3fBG1+avh+7+B+l8g9xrHWlfG9e50HXeaSi+1XVjn2Y15Sr9f
1845OuZ02qu/J1XmSn6EexD8zJ7Q9t3K7wk6z/sqPzNv67r/yHyvOutePqvy
j37fpmN+YK2mH/X/peqOkryszned9n3PNaJrZV6zxqq/vjr24rheLxus13Xk
XUr9Ln3LOhx8vzqrfgFtl2Q91XG3q3yqvvqqfk+VTPt6pl6HWe8+1rEvqs3l
mc/xneoO5l2trauhp/2tfVXpe40Ot6TKdMkdtW2dWo/dLrWegu68u87zLYJ+
b6Fjr8ysO5W6rmcLj+FTdJPg58DzXS1YT1tE271qP1t07xckX6bjt9Xvf5O4
LjPHaPsR3472XaXSLvE5t1H5hzVL52mV+/1ZSnW/J9alV9P9/KHy75XVTyvt
+03yaTr3gQ3P4SfquItUDtDvfVW2VdlG9TN1nl/U79eSL1Uf52l8OzL/q+1X
qh/e8D2/FpuBevV/Omsc67Ha3K9yjuQrVL83far0ZZ6iqP3O2v+5+tmCdU79
D1E/3flO1ec5kreV/IHG8JvafCP5f2p/nMqOfEvaf3fltamX6gaodFf9nqo7
qrQuOlLn7aO6TVR6c30qm0m+VtsJrCmSr0dHiHPLOLW/UfIFzCfMtXFt2lTj
Gak+92DN1Xj+KPzenK/9x6scIHkF1a2j8qt+t9P2W74HyY3UcwXz/Iqq+yXz
nJNLnpd5Du+g8/7WsD4zUuO6Q+VUnoPa/E9lOfVxi9qMUN1g1h7tv1LlcMlb
amw3l36H71Ld07nX4y1Uf6PqD2Q+1Hn+Kvwd3q5+rtD+s7hG3asr0QlUP1TH
Xa1yBHM986XKANWP0nZI1FvO5ftVP0czV6mfdTWu5dExgm0q7KnjVb9eattw
uOrO1zl6qn6E5AuxPSQPk9xO8vb0KXm45O0kX6X+7809NtaIFVTmJX6/bqn8
jm2uulu575KbqrujtG7A+7Jh7XdmrsYzV/JVanuG6k9Xuy0SX2v32tc7Sue5
m/VCfY1iLVWbvVR/heQbJO8t+ULWRsk7872o7RW53+1TtL1c5RDuo9qcVfl9
vkbyGMn7s4Zq//im9Zse2i6n8qPqH1P9m7nXNObYO0vrJGO0HaffhyR+nitV
fqY3Sb5T8kGsN5LHSx4k+T718RLPTP3co+0LufWtgbw7KrtKPl3tT1P7zRPr
YLeW1iev0/a2ys/3cskdmHMlT+DbUDky8buzee3354WG12T0n6ca1g3R957Q
9o6G9capOudnPD/0FG0/ya2/sm7fEL+vx1V3W9Pf1qbaLtP0/P+vznV76XXn
ZrV5NPc3ydy+iMrnqr9M+6/ROPdkfsCeYs5lLdR2xdR6yHp6N3thJ0reQNve
Kgvz/em459CbmatV973Kqqo/Qcduqu0aKpukxi1mqc0SrB06/2zWBY3l7cz2
zgraX2BXSF5TfWwJJsDagP3CPKxj+qjuAJWVwEC0PVBl5dRrInhKC5bCmq66
JVQ2Zi5RWVTy0qo/t+HrWpz5H91L8u86f1F6Du9VW0dBP8E2PErX9lVirGRN
lfclL4cOX3ne7tfwlm9tUV3LK5nt3wf1jWxb+d6ujR4avI5iX29cGZ/ZSb/X
51lJnq+6S4Kv/wK1GanffVR/seQrJO/KO6z+b2h6XuyvaxqksnbquXOT2vPn
rrWxGHCY3WpjNOgzZ+vYDzN/a23Ux5HqY47kQaqflfn7PbHhdYB3ta3OOaLy
70c0rn5VtB/V9mXuV2LbfKfK+Ma66ueNzDpVA4yk9Jr6MbpVah2M9eCa+P63
1f69S9tUl6aeF/vFd4+x7SL5vczjZsyjVX+c6ndnnVL9nSoDJe+luqMavj8b
gz+pnKVjLtEAz8ndz9DK8whzyLI67nr1NZz7pjYX577eK/X75IbnB9ajV1XO
17791X5s5u90mNrvm9umvlp97h68Pg2TfHbpNX15tR2DPar+BvAdNzzvXajj
fi98j6/g22h4zhmZer5nrvhDfQyO9vuV2u4WvE6gN5zS8Jx5kepvUumVeH3Z
O3iO5dp5b3hnRqe+19znfhrPuMx2Yn/Wxsx28fl871GfaHAvJZ/Z8Jo3tOE5
9pLS94p18Xbm3eB54/rUuglzzkfcz9zP7qrU6yrz52W61ktzX/t1qfUU9FJs
272C17weGvNrKqOZ+9X+Bm37x2vnveeZHh3XS+7PUNX9qGOPYe5Vnzs0vPZt
rzGMVllL8pY69g21WQl9VHXDNP6B2AfYg8F401+q+0F1izWMfbXJW2DDZLDG
sERuXXfX1BgV2FQ77fxIZXfmkNyYGMdujX2v0lFtzlLdGqpbSvIyarOsygKJ
ca1pusZr1G53beeozNV5uuv38rHNHujADeO0fEtLN4y/9VCb9mqzkOQ52E3B
mN3mfGe5sbtvVL9KMCZ2svpdJfexrTKPafeI3b2s817B3KR2rVX0M5mHHheM
te2rdm0bxhu3U7trM+t4i6rN7uAS6usncK1gfA3duV3D+jO2TnvJi/HNolOk
vv//S429tUUPUf05Kvulti+2row1LS95r9J22mka/3rgWsn/44Pc/yHMc7nb
c73dK+N7ezeNCS7fMG62vsa2quT5ktcNxrNOYS3KjSXyDoDtgrOBiXVq+Fyv
Zp6/mLt+Qc8JxuP20fYnlYuxHbCzU38jl6T+TvhGsBVWDbZ/wKSnlrZpnq9t
d/QpjeOdUFr3f6n2u3QGNoe2b2k8sxq2kZ8svS6BRz9QGjt6UPXv6vcf6DTY
bWAxkq+VfJLk/STvoHK7+vpFxz6ofp+rjMehpzEvMW+yblxaeu24W8dOVpsP
dNw3anOg5C2Zl1R/OPMaeqjq9wBXVP2lqt8ZPUf1IyXvjj6WGxcEH0Tf5oWa
xXrH2CW/IflXnqX2vxuMiZ2o7ROV8aATwJOqiPVInlYZnx8i+fnKeMup+A0q
44Y9gnHO7STvHLw+8gxYLy6v4poRrE+ylmwQjPFij9xZ2+bdXmPZKBgHxiY6
C7xc8uuSH9L+2douoPflPrArsDL0S56p2kzXvodrYxELqc0Tkn9Wfedol/9R
2XbGDvg8sy0wsTamsajqJkn+WG0W5x0PxnKxp2ajHwbj6cxn/eOcdn5cP5g/
abdbw3rmbqwvme3E3bDPKvtHdtf+wxqe/09U3dbqs3fDdt82kneWfLTO9VWw
zfavrmt/lV1YU8ENVXqjJ6Tui36OV/tvg21L1tCekvdBZ4n625Zq805mHQL9
4YzUtiM2yDmS92h47R6B3yr3+LnvLTYqc0Lq9Zt5/jy1OTR3nwdVxuvB6tto
u6dKf56zymUqJ6CP1n7nGf+eYJEquyU+/8A4hgvV5wm57+cFqddR1tDzJO/f
sK67FxhKZt2JdXwH9bmf9h2r434trN9hL09XOVp1y2TWRdAfFlTdnWo/SG0u
UNvBufXAa5teW4eCWWQeO214LpdU1vcOqzwn4D8a1fTcgP6wt9rfntmGWgqs
Q/tOTb3u7xQipqD6OzJjxeAGfYJtpdPU3/eSD5a8jOS+pdf1H7XtHOwjOVHj
XDG3H2dAaj8A8+G7qW1r8KV3JP/SMN74RsTlwFSX1Lk+rIyzgZGeju9S+z7g
OlP73VZFZy9tl6NLv6uyCjq42l8QrHNP0O/FCmN992q7SGGsbwnmpMr196tu
+cI43keqOxdfSMNz4Y8qV6n+9dTfI/jtwmrb1Hnn6Pc9khdUuTkxNkAd/Z+m
e/ZEZt1+RsQMwYS5PvBHrvEh/KuF8YQPwbVS+0Y/Sm2LPMtarz4m05fk5yM+
fW/i+QP5JsnPpf6NvEbmdq9iA3Gs5Gcl3wlGkRsfnq9tQ+Ofwfqs6x0a7Kf8
WvL5wZgjvhd8MNwT+vhT8o2Se0V8Yzzfr/p/OjPOxvWBjXKND+tcaxS+xi7a
P03HvKd9P2XGTZjD/8PWwdXXVP0LccyfxXqw1gfwRRc+18zUGAxYaKfM8yXX
tYauYz9sJ2xLMEJ0h4Z9K4cE28DoftjzrKFgLG/wLekejNHxlzZs84I/vK36
+9T3Tan9GeBF1+n8T6rtYMmHq78/gn1M16r+odx6L7r8JQ3r87eD4RfGI2/m
W2oY37ia9yt3m6tT+5DAYe5s2od0bWqs586G7fEDNY67M58X/Od1lVt1fFts
BLCtqP+ObFgHvkPj7x+Mc2WsUcHYymQd83dh7Bb8ly34Ldd9e7z228EFc/sj
5uu460vbOzekxjbQvTfWeUdl1jk/Uf/D1G6e9p3B88eXo/pC2yPi/T+dZ5TZ
Tn9cdXMb9sHhOzg0GEd4jmcI7q3zDOJbQY9MrO9fH8/bLjPuz/Ver3E+k3v8
+JWOCsYsuPYbSmN3YyMmwbPDjgAfQZ8Hlxik9qO177PM+DU2Qo+IXR8tebT6
n5hbRo9hbsYGPFHtJ2X+xvFtvV7Z13+62m+S24+8I7ZJZr88+jX+Z3TsM9Sm
R26f8q8aw2bB/l304T6V/b/dtf8nteum+sXBJkuvl2AB1HHsYup7RLQvDlBZ
q2GewOJg7ao/SnU7Sb5RZbPE2Ok6OtdGDevC6FbtWKckX6ByKPiCtvPBLVj7
1f7NYBmddt/K/t/fwY6CfdVngu/mHvNAnWd8Zl8M60v3YKx5V+5TZuyuTeb1
hjUFXRe+AuvsOqrrobbrcG9T6xys16wPm8c14mydq0/uNlw3OiO2zw7grCrr
JeZLfB58r09Ovd5vFdf0TYKxaezcLYP960PV5465r+uk1PoausQ5qt8j97Nb
Qn1fEu8nvIs5wT7001Lrg2CbfdX2Z3x7+n2utnvn9mvvzNyhsnXi57N1fEbc
w2Mq2/IzMuNQYFD4WpiD8d9h7+xSmQOAbbRBw/bR9Mp+Uvx0+LfRDflGFlP9
GcF+wTvAqNXnxao/Xv1PzDxXs4aeE8w1eTi1zosP8STtf5R1nW8z9TrHGvdN
Zv8Nvhv0Zs45NLE/Ed8U/eOrnVGZuzJO5/0t97cwFjw897Wwtp4dzImZmFrn
Han6jnxHTft80cXhNJyZ2Pf3UuWx0fbL2J5rnFl5jWDsn8bxwxn4hDkisb4+
s2Ff/xNNzzVPso5om6q8z7uh8z4GjpOYM/BOZV5BJ2zN0nNWP43352Aspbfa
3pDZfzEmYmDMLeAw2DXMCfjksaXw46MbdwvG156INhX21NTU6yhr6PS4trGu
3a17VRQe/w+Z/Qf4Drg32DT4Q9+rbBdwL05Vm8dV7lH9eHSPwuOfltouYj19
JvWayr16u7IdxLX3Dsbq0UevaxrP+J/KYXpWXxTWiR/TdqvCPiZ0kKWjHvK1
tm1S83y20P6ytD+qUJ/XBPsLvknt04CLMkfbZVJzhNBflo06zOM6dpvCfZ6v
65ii8mZiHLLE58v8Wdv3jB6I/26WSo/C/ohrgzlplY47RtfQDpsXnCHixV9q
W8X60eCa+A2Z11Jjt+8kxpE7pOa8fax+d9HxC+qYBdXnKdruABaKHqLyVmLb
Y1hqewFblO+cb3w4uE3peWvX3PMRc9HFcc5AXj/3/MXcha8Km4T5DV/D7tH2
eRe8PPV5sZfOSm1nXR11edqsk3vOYr46P87Z4E69MvN0OB82ee/4rvL+DU6t
82NX7xjXI9aA41O36aO6qYXtmpHRvmG96Kn6lwqf4yx0Zl3jYZL75V5XaLNl
blySeew89MzM2O+/rIfBvtVuqpvB2iU5ZL7HG0g+sWlcG0x7tuQFYj3PZ6P4
jHauzU3kGYHpg+OD4R+kc3Yt/TzB9bvH+oWC+Yj4cPFVbZj62fGurJ/6HVtY
bWaW9vmCYx+m8rHkrqwXqeUFMvsPeuh3qms8vLAvuMn6k9nXjP8ZLlwzMTa/
ebwWeG5HFK7HH71kMDZVYo+Ax0n+U33sWthfMIn3SOUxyV+k9hVMje//TJX1
C/s4Lg3mjZSZddta8kRsHbWpJC+k7d3B9VOb/hbeZo5J/a5zD7GrFkz9TXVB
pyj9/ZyrPp/lO0yM7+wWbXzu/3vgL4X5HviXGSd6N1wEdO9fM/MDsN2wF+BG
MKc9oW2ztu8E/umoYP/6cLV/LvN9wA4Ac8IPgo0GBoUtsxbvUbQRPk/t5+Fe
7aH66YW/h4u0Xao0ZnAIa11h3AC8GNwYXyp4+w1Rph4/K3osOPktsR498Zfg
+Zx+sZ351pbO/O3ha8Pepk9sfOzbq2Ob+XGNuBVsMfPaQD/4SsbE/vHlbYw/
IjWni+0RanOqxtuKZ5CafwXWd1PT/lu2+EmXy+x3PzTW3Rzrwc2pb7HBS18j
a9Ayut/nVp7PaXNNPJbttVEekRtDoM30qENMims7/B/W9w46732SxzRs29AG
HaM9NkjTfkn08PGxDfbKnbF+fPyNzLXgkwbzf1Tj+jO4H7BRrrtDYn7dWxr3
hKZ5dfDr4Mv1jVgH13VMXKu4/2Ag18Xx48/n3uPTXyGz/xEZDBIskrUY/t1v
wX2CS9L/vf9to4w/d0K8lpO1XUhlK41lK+b/wrjx2U1v0ZnB7+Eqg9t/WZrr
jG4Ptg1fFXwbjBIsGxwb3unG0VZaTvKmmTGQTTJzU8HJwdb7psbVwdtXCsaN
t8ntY2Ce31Py+YV16X1y4z5gPnAnwJvAmsDdzkztozks+qLwQ6H7npJaHz4i
t26KXgpPA/xry8T6+pDUWNAumX3s6OpXxC3twXH2TM13hfeydDAvZT3m/yLy
jtRm59T4eafcdgI2Ajr3wal12o65bSFsGWyJg1K3Afvun1rnXzI3Lg9ODu6/
T2rsfbXcXJy2iXm28GZ4LthRB6bmyoLF90uNq2+sNttmtstWzo3Lc2/h6OyU
2mcK36h36ms5lrW69O/WuX0z+GXgssFvrhLz0OCjc43Dox2GPbJNZp4tv/Ft
7J36XHBf8dnQBtv+jvhd7M8aW5j/k4NFlJ6Hn8ht82PvY8ODQWDD4gPDDscW
hm8wMNpct+XGJmgPVwTeJ7Y8Nj+4A5jD2Nw2M/YydjJ4B20OiN8yNvW4uKXP
S9T+qsJchWMkX1oYq4QjADcUXIJ5CB8FcxH28YjU3+zxuX2c+DfhIOGTA0++
QPKVhX2a4IkXpuYpgSFelBp7BAu4Pl7j5Wq7bGm/B9wkbHLscfyO8FDxLcKl
eaawnlnrHs6vHWOwueomF9ZJtkafLeyPfqdpfpW6aNG7vktboNIWPvmCpX+j
u8xOrSfQkPZwsZh7WA+Yi+CFHh9tK3T5p1Pr2+Nz+z4ZG5wfsB5wHvyOYD3g
PPA68InyDsCnAj8CO2K+5P2gfoD2v1749zXarlDavwqGxZb24Bf4OMEufgWD
qT0P4hselXo9Yq3DjmHtm5SbTwmXEv7lnam5GXBA7kjNA4HjcXDmOZNYiUER
hzkSXbgw9xhe5tjU9hqxDPCqwXPgsT6QGju6UW3ble73EPTAwnP8bdquVLrd
87m5vMMS81vvTs1xhZd6T+p6+Kr3puasfpybS31e4jmesTDPH5p5/WK9ekdt
DsvMHYJDC8eaNvix4EZji32U217FVn0s2oiMf7D2v1P491y1OSYzRgoeOil1
7MbRmTm32K1T1OagzNwbOFT4kvsn5lyNKowxwnuBN8D9x6d+WWoMal/VvVr4
+5kY+8PWezPqy+hIxI/Ai6Uenxl2NTb1d7ntRmxGbMqnUtuAd6muY+nf2IKT
U9uSc3LbirTB/n8stc0+IzdegC0PdvBQapsaW/3R1PcHnP3B1DgAsTJVbq4j
/BzsKuarYRrjB4V/ty6M+YL34qMC+wP3g0c3sbD+CY9uUtR7wdbfTG33wZEG
QwQ/BE98K/WxZ6ruvcI67SParlnaV7B6YXsVW7Ujemhh/Bms//2oW8L5BX8H
eweXfiG1HwGc40XJtyf2qz1YuM2mhXF2MPbpUQ9FHprZz8pv/Grg9WD14OOz
oj78UtS7GX/fqH8yXwzP7Sti3sCn8Evl875cWteFM4TuDw8V/f+kpm0kbBDs
mhuDbZ2L1ceL3Au1uUl1Ve13BRt88WD+EvbW6nHuwhbDL8x9QA+Hw8048QWe
FDGfpyTvpvJyYg7P+yp76/el2IA8V95V/Q6luazY9Gulnm/h+Xygsh86RGo7
6uPEGEC31DgAc3O/OD+jW22dmjt0XNNcw3VS22U3B9tSxNesm9lu+j23z+bW
xD5dfDa3SB4H7h7M74HveU1u3snVGuNY/GiJeZRn5PbVwj/cSPv+jLobOhyx
Zh/q/L9K7pE4LmBdtZmbmkvAtnVifuCGqv89NRcVnzcYJX556nZivda41ipt
b2GnfZKaM0nc3PBo78Bbe7KwLYYt9GV81vDEnor3fwjrfm4/6FNN+zRYX2Y2
PWZ0TjgHcA/gXWyvtuvrvD/r96ym61sljutaKLdec4rOP7Mwrj6lab8K2NGQ
zLx4fi9ZGBcDE4P/BkbzTGKfNHj3zYn9Xc+n5sOfkDk+gvGBb72U+l3C7/Ra
ajsO39aU1M+uQ2HcivOWhX05+HE+aHrsXRPHFq2na/kpNRbNtn3imLyuue0G
YvvOyXxv4at9nPqdv0/1q5XG2J5p+lq5lqywPw/MDTzsldT2KdxNdB50qhUz
r7vYVvAYx0Y7Yk94TSqLlL6vcD64twfltj0HNM3TXLG0vfO3fu/b9Jj/wV5o
+rrgbC5Z2paEzwxe1E/7+ktuXZrD/HdpXhqctNboL3rQQxLzmVn3D2qat7Vx
09wteAfwD+ATXlRaRs+HT3VA0+8/XGJ0J36j4DBe3lViE+gPOxRO7nKlrx0O
7NKl7UR89IyR7wKuyqJN6+Hv4Aeuva6vF9/5Y7UvR5dBd2EtxtasvcYTHwMP
gxgZsIujtW9i4vizVUvHoKXx2R3ZdIzF6qXjLIjRWah0nA6+JPaPZW6J8z99
EafSuTRHFL4nuA1zIzFcjOOCxDghY3wh8XzDvDMjMR41mPk6MbcTTInfcJzB
j96K8zB9wiMldvXEKNeZ28CFhhvbvrQt+bDuz8TgmDd4mA8F37tHYn3rpjmk
OxXGumbouHuCx8Y8yPjAUb+KcyPjvFf7JwRjZF9wr+N9IFZ3cx1fN83R3UTy
vzr2c8nTc+t0cGbXLYwBdUOXLowDEc+yWem+eL5wkr5M7JN6Njim7/um4xux
9biervEdwACaFKxD4isj3gOdHNy1jM+XGKIepX/DBaYNfOClVN8orLMQc8d4
ec/B5xvxeRFXtWnp33CBl47vOfxZZPSrZWO/nBdO7+JN27D4kbjHYK0PxPvf
wjMuzYNtFeX7gm064lOp+zRx30vF/uHpwbVjjYDf0yZ+X5yH881O7F9uxnee
mMrupX//I/mr3LowsdQfFX5Pvm4a/+R9A19gLWQdnNO0/ElibssS8Vo455Lx
vGz5DW8WvvS8pueCn5uODUNnnopvqmkdFfyG2CgwjYc1lmm5bYWxOvahyvoz
vOw1a+v/xDjeVxn/+Vrv9j+F/cNw8LeurUvDZ9+mNqd9HPp7ZT8LsV8TKtsC
2BezC9tws3PfC+4Dfiv4V/hxXsn9jvJ+oveCQ4FBEa+2XW19+zsMstJxT/eU
5mW1cNKaxnH41l5TH1/mtmPuLc3Xwm+FrgtJGL0VTgRrEusRax+8ha8a5hSw
HrAWPKj2L+e2q+B3oBeim8GDY21jXQP3Attinodrf29lnR/s8fTcfCNwQfBB
5vmDGsZiesX1Ykhc01mb8H0z7+Hjeiz6ue5Tnz1rr7P4uKZFP1eZWj9GN0av
7lRYh/1Z9ycrzTFhPuwWv2v0T/Bg5k/88o+UHj/+I/hmN8U1cZnC9gFr8TeF
7Vfm4aXitwlv5+lgLIX19pm45r5YGgtGb+f92ySuR/Objt8AZyCOBs4D+0ar
/QvB3xTch+6xDdvNorxI7VgI+oGr8FLw+0xsC1wF2oGr8T2CrbH/5diGOBLi
SZiLiGFDnhffYWJAt2g6hpRvg++C7+OI2nwXYiLxZWL74M9EJvbtl6Zj9Wk/
NzO3Z7umuTHTgvUf5j24AsR9EP/RI85FxC7jj8QuwyeJTOwPzwtu0g6sxbqW
HZvmLJE34HfJfZqON2Q/nKIvda5OleO5iBVdtPYxcP24//D9+N079oNfjj5a
YvC13TnKxBfgp6MdsW/o8OzDf0EcHbHzi9WO4ad9Z/Wzq+o/TKzv9Yzj+TWO
j2t/m7bB+uoC9Fc6jo1YOPrlEsFQ9mgaVwE7IY4OvAWfyZ7xvE+pj8nB68sP
Tce6876RRwKe8prN/8fNiX2GA4vOjQ7ze5ThxP6WWo9HV8EuwAZA/5kX9Xh0
ePR+4gGIBWDNYe15PurR6M3E9t8RzKuDL/efDgr/4T97AF7x12pzUW2OBX43
tvDo9tZzmR2sy60UsQjmtJUzz1nMV8wxD8R5Zn7sH10R7ix6MLribcH8QriF
2IRgAdgO+NGQiSsZkdnGw77DJsA2gBM+X/fszcw8EXI/vJaZ071a07kYWOOI
rTgNGy9zLMmp+KUyr4O0KRNzpj6P6xprMGszvmNyfhCzSrxqpyjPjNcOtxMe
Cfk/pmWOMeFF4D6zvqObvdW07oQvDLuV3+TTmBGvC79cqH1da2fG8GiPX/Td
+C79E9+HI+J7BZZHnOR3sQ3+0xZ8qmnMZ3ywbwy/WBttt6qcu2CxiNN2aRiz
hasPNwNuDHxx2sCROSfKYP/I7dBV1Ofw2sesIPnjYGwfHB/eO/LifFPB7x68
FzB0YiSXUH23yjotnH3aL67twplzoKBvsG7DcWLtxscBJgseC06LbYLNAkZ7
V6yHA0YbbJZzKscpwN2EB4N/Ad9Eq+hr4Fxg+ewHz39fbfeujXd/ojIr2P+N
XwO+OFxxcmx0K/3Nwse9Ih47ROf6IXi9Q4b/CUcW3wE8n23U5sg8ymozqDKn
Fz4NfmXeF/waSzG3VX4W+LDZj98cXzT94WuGO3NefEZwf0bEZ7dkZr813x2+
B+IBwA0WypyvhDHjg/0g+mGJFaAN958cI2fEe07d6bEejvLIeF5wfM4Fbr+k
xrlh5Xu4tORDK18XMUDwA+AGNOO3A+ctjd8OOi2x8h3jvN0S79z0/Enc7kpN
r6esHR3iekE8dMfYhu9s1fit/avr+kHjfjfxN7Z60zEacDA5FpsaP+3KsU/6
RmYdwXY5KX5T2PxwLLHH0YmmRJm8B/BF0ZE6Z9Yn0CW+qezP5Rj0C+rBuE6I
ffaI+iz+M/TY5eGfV46bw14gxnx5xhQc2866zbpJHWsoY28fxw8/eoOmOdJn
6NgVK+u9xCzDi16v6dhhYpo2bLrthrE9MaetatcR80ac4eeJbQBkdP7vms4j
Qw6Z/up72cr6PHgI146u+FZmTtfa+r1u0+eE6/W4xv9ksO6AzsC1XBz1EOR5
cd1v17QO8ERs3y7Or6vFOZbcKQvX7vuSzM+mhQtRel1kTYRftm48L/cWOwHs
DE7vlPiM4Lvg68f+5TlOjvXgMshgR8vpPK/oGl9rmA8FLwr9c27UR8FJ0JPB
c9APV4vYDv0sCw+nsv0+Nx7PseDjyGDmYEFPxmP/CY5h4DccHGxC7MEyynBy
iqbtWGxYbFxsXeKN4eFsUZq3gc2LbYxdjD3+etM2FNs3oozdC3cE3gi+M3g+
7CeXBRw51l/WAXKmMP5Z8TcyGC64K7gunGRwXt7Vz3Stae13vlNsw9oEhgr2
C47aNTP2vUC891OjvFbEh2nPc6EN2Hj3wlwH+n87HsO3AzdiehwD54XLAf+V
XCgT4/iJ/0AG/8fvwHVhQ+ErmBTrO0Z/BM8XftijsQ0Y2LPx+a6e+Vnz7OCU
TYrtwbSnxevaNTjek7g34uA/zhwL/2vtdWHt0v41cOev4nrOOr1G1FPQUagn
3mK7YJtoRqznnhDHuVRlnIHY1qUr6yqf18Y3PuSctfW8VyImfUMw94e4qO0r
+4fxM4CZ8Px5PmvF/l+NMs+W+Nc2lb997nGXWA/XAM42/n5i4oiHJeaNnCLr
VM4rQkzbJpXxVWLUNq3sy+V58uzxcRCLtlllvzSxX9tU9jkT57deZR/4Z7W/
gfcLc053rewDhz9LTBN+7+fU5p/KPofJkudUxkqJPyCfCLgl9wGdm7XgSbhs
lfHaZ9TP6vq9Fu8TXKnKeDn5hOBAoP/AoYZ7cRbrpvrYLPOaAX6DrY7d9GZh
vAh97Y3COip6Fs+B5wGGDt+ZmCX4mJ/Uxt5fK+y/Am9Hv34qNzcen/MOjLM2
3oQOBt4E1vRuxH/QMd8ujJOwTjLOw4L55Vz7iOCYEO4f9xEMC44eWEoLh0x1
H9XGyrneY4Nj0onj3C/EGNGmsT/mkJnBPEj02J/i2gT2zbcId5DvEbwQ3BC7
AA4omCNzFHwrsEvmMdY91kW+4yNjG9Z6bDhsucmJcUnao3P+UnvfTtHPg7+H
7/2QpvM6Yde8orE9VnqtBn8El2RO+yQ4/hO9csFYD3YEVgxGjF6NLYhNjj7x
RbB+Ah9hULwWdA/84SH6xImFA6tFtz823h/m5GnxG2GOAlsCY2I9JQfPpMr+
XnzLcKXwL/8ezKsBF/krmNsD5gRPithd/FzEGLJ24F+dG7lJcEoOje8eGBF8
ZN5bfJJwhIndw/dI7pmOlTnR5HQiXw05npj/pkccCV4TscRwsYgZRLeHF5nE
9Q7/Cbl0Vq68Lv9c+9q2K52PijhE/Mzkmnqmsp+ZuCLwI7CjH6KOB7eSvFn3
V+Y+EPt/YWaOIjGM6K7wPeFfv1zZj0oenVUq4wk/antRMJf2/trnwv88j+OC
ubH9a+ffa6v6A2rn9NsUfgfvfu3chP207Vo7F+CA2t8+ef72Zw6onc+PZ4MP
A1sA3yM+yJa49dr5D1fQvn207VI7t+J+tXMDdpC8b+28i1Oj3xIeGxw27Np9
KuvYh9TGA/nWT9C2T+08kayZfOPz4/wM75Z5kueNPYAtgE6BroCecHBt7PGr
zPcXWxo7Gp8Mdg42zkHYa7XxyxNr53jcq/B3i66GnsZaQy6MzRPP/9iTrAVb
lbZ/wbB3kPxtZqyUbwAcnzVr69L2MtgDcyN2K7ohMZXkh4LTuG1pW3lu0+8O
7xDrIvGcxFr+pLqekmdnxsC2KW1P/9h0fO7mlW1GxsZayzp7We31Aj4vMbok
bvpD+0ervm9lztf1kg+uzCneEX9IZgzmqtrrCNxb5hXml7+07xYw2srcLuZA
cLf+8Z6j06PL9cKPlRnfulntT63M5+2t+t9U/1vTugN6Bt8XMvnF0BOOr53r
crPCPHl8iugXswr73vC7fVF7DYUzNS1eL2vul7XXRPhcP0pev3YuE7hwxKYT
H/NcYV4Btn6qd+eqYD8zvqMWHRQbpbBPE1ynU+3YXeIAvqqtG8Cznl17voLX
QCzYmXGuHl/7+30+rmXgrTcnXpvhE8Al+FP7Lw/2wz8f3zHspK/Ay4N5r1/X
/sbhX7xY2C8JlvNkfLfRu7+prdtgQ3xXex4DR8TWxw7HfgRr4j7ge15M8q3B
voAjuZba2Plx3O/a+WyOqo0P/4Q/uzZOPlPysbXz3lwmeVBtfP5ZyYdpu21t
jHxw7Xwvt+Rer4kth3MCDgoeCtaOv471D2z+mNo5Pc6TfHTt3DtH5f424UqC
S0ytrSPBp8vA3IJ9/uhw+MLxg8MxHhb1pQ9rzyHbR/85ccjdo344IuqNd6jN
VZW5Ei8Xbgc29kDU6dGHuTZivuCvUQ/vB58F2AA4DvjA0HifWTex78FWsPHJ
gUR+DmK0R+k8Y2rn6eA6ybMBZ4r5ithXMBL4p/BN4ZqC7cMJhiN8U+4cIHB5
Pimcdw698dPCORzQ/ZgTiLUjvm5MnEuJG4GDwDNAX4ILR54P/LqsSeTpIEfH
Pto/MjPvAt4dsb/YxMxFxAYTF0x+I/JpwINjzYXrzFxNDpyLMmMFo3WNt9bO
M3JdvBa4KLwr5OWAj3a92txWO19G+8p+ZXxt5NuAt3ReXEeWjceSF4RndGzE
YMDRwGHwPcLvAlsFH2oT7yF4KfhOl4jf0B5MhXhu5itwkw1KxyDwLbAWEHMF
DgReBM4CxkIcOfMVsRGstYvHNqz3rTPjSfjrwfjA6/Dn49eHOwd3A944mDHx
ysRdwDckppmcBvAQ4THCt+a74HktH+8n3G04eODQ8OuIC4JLB84G35p3g2+D
OG74j7fEY2mDDxO+HBgqPGU4yqzLo+L94bp4x+BkE194Hj6f2rHcrE3EARLv
1zN3DgrsC3JXoAN00DM6U/J1tWPXTyqclwObZVjumHTu7bFqc3XtmLSxUTcg
HpK5gHhSuKXHq801tfPLwCMlrhRfDbmaiOnjHUA/IB8IPGXW1na57aZ2zBO1
c3asUBmvmaOxbZY79wX2UXvtP792npTPCufZwI5rq7phtfOYoOsQOwmHFx2I
OHG4xugr5CohR0EHtb2wdrzfSrzXtWMs98odh4htBceVWF7iWclRR34X8r/s
nzsWcqN4/8EliXuHq0nuUuJvmSvABPE1kXOLXKfk3ULH5J1BzyQ3GPHvvD8D
da2jasclXiD5xtox//vxHdWOLTw8dwwg/Gt8sMQI44c9Qm2urB1niL90YtSH
T86dD4H3HF2C2ELiCsnJx7qOXXMXemjt+GF8j+hA2NrkcGW9gIuLfUR+AHwj
xO/yLQ9U/TKVMS/wLjjGxDcT20zeMmww7K8JcZ6HIwMOCsYKFgs3lpyHYO28
S23j+4z9gq+KWHBytcB9wZdxcDCnBc7nW8ExpfhTZgTnysGfQlwrXJMeiXMV
APLj35ke7GvDD/dasL+JvOP9Ynv8IwNie/w1+Kz+bNpvRQ4ofFL8hh+BPgpW
AG4Phsv6C58ObhacOuylReO8xDs8pHZeKvRg8k3hN4H/v0hmPwD38JTaucPq
qL+RbwpfDLwB1hpybsIHap/Yb0k9PpoXg/2h5Ehj7cUvif8+iToq/m64OXB0
8FVN0f4zS/v/4fjA9YGjANYIltoS49k0HwXc7HnVHVeap/Ahc28wLk49OCex
ts8Fczy6xX5ejPbRqU3Hv+FDwReAD6N1xHiJiYJXc2AcG/6XT4N9GdybT6If
BB8G3AH0YfgD5NpEPyXf5ocRh2/B5aMuin2X/affJuYqwWHCr0dMOVgHPNCB
sR6/Hnkvz66ts3J/wIfJCwKfZWbEGcjZQM4TfOfEJsIbg1eD7bBm1Ovga8D1
BbNH3wNXRU9eVccNrZ3PET4aMSD4A8HAwbWxkafE9q9GHA4MDj0Ze4Y8qGDa
4Phn1c4FScxXS66WxDjc6lF/Rj9fLeqN2MmrZsZWJ8d6xoMd2ym2gZ9InAU+
SWKasRGYW4i3Jn8OMRxgsGC+jIdcLOTqgYvwaBwn2OG4+P3iRyJWlbw6xAvC
JSTuD04e8eLYI8yZYE3wAeADYfvzbfNdg02uEvuE7wNHF6wXXeL02nnfiK8n
byv8H+LAwLvgP64AJlA7j8AjsR90PPyl+E1buN3BmBgcW3J4wOWF20u+OnLg
wAEkNg7ecIi6Iv5XeODkaMcGIz8SuefJC0teV/xmxC+hxxLfR05WuP34Vshr
i3+TXPvkXK3jfNghtgfrIO8D/fN/CcjBSw7b++N5W3gssX/u7YromLXzxmLj
rJ85V+/rwbnCmCcuqY0lEitD/i5yZRFLdEZt7II4bGyTp2rHCq4R/D8CFo36
W9/g3A/oEn2jnobut2dwLoo1g/+PAHYWmB0xFcRTkKdl36jzgEU8XjtPxyq1
cyQRk02e8yPjswCvg7sPb5/8i8QmEpuHDo/dDef4kWg/8j7sUTjWj2cNBvJY
7fwm2IZg4sT+bVI4nwnPd9/gHJvweNsH//+I1nG9OyA4Hwa22KO184Cgp+0X
dS3wRuIf8S0Q90GsMJyIdSKWCpYItkkcBbET5P4aEPVDvnf4FvjIWNcOiM8O
nwy+GXxb5O0mRpJ3bGWd65jg/OnkWN4qrgv4UvH5Mo/CESBXAPwA3oGOwXnL
yE0CvxFMEfuROEBi4bBZ+F8FrE27lva34odFR+J586zJ9cX/buB9HhbtHdYs
fNHkPaCOORnMAT8nGDpxksRIdqyd04r4eHDCnpX9z9hB4P5g/sxVzFnkZSGf
/OA4FzEvkxcbrjvrObEcfPtn1sa4iFkHQybeg1gPMF7iEIhBAHfFnoUbSs5F
uLJwXMmdD2cSnJK89dj5YJPE38MLBX/9X3BuWPQ3eKdwYsFNwSvI9UOeH7ha
5CkjfwH/T4O4fjhb4ERw1+GT8z7fVTu3OP/DBP486z55TMGOwY15puNq5+h/
L3fOfLA+8vGDQREXA9eMXPHwzeYUzrEA1tc1+H+FkKeBGF/0CjgbXxfOW/5Q
tH34HwzYQY9Ee5aYS/LKE8sPF401ilh41ikwaeY0cGniWch5Tw4GsKxZubHQ
76OPBP8IuUiJ0yUf6c+5c+ODZ/Lc76n9/wBW1fbe2v//AH4cOV7AObHfn6kd
k8//EwA3IH4HHObp2rkAVgz+3yvEwPE/YYiLJ64aDIIc/sTuwCMjjwFcMvCZ
CbVzT7QJ/p8UzL2bBv/vEvJhkHcBrJWYFHAl4seJuwIrJ78AuQU2Dv6fJuQC
AfsgZxGxOMTmgOPgj1gi+P9cMIfzXcJL5NtsVTgfBdgysSHkgoJ7BQ5GziNy
bbCekLOINWXt4P91Qt5A7hX54MgFB1eOHAvw5YjXI9aBOAdi9cA/pyTmSpGj
jdxP8A3J6Ua+J3wacCPRq9ExwOLwtR4RzLdE9x5R28dETCRY0JTa8RLgfuSW
IofUgcE5sbFTDgrmH2KbwLEn1xt5rOAGktsKfiBzFzgnMfbgCcRzwO1iHuJ/
uzAXdSmcq2detD13DM77dllwbnzm9pNr++zIUzIy+H8BwCUGDyMHPpgYvhry
7TPHYvuQ/45cKgOD8x5ji10X/D+G4Dy38Ncq8xT2ijbXItG+43nzrHsF544G
98AHyP8Y4j3HVt0qONcaOcbgq6Mbgq/z/xfA2MGi8ohHga/z/xfyqG/wHvMO
Xx/8/4/4XrDdyItBToxG8P8qQl/dNjjHMrzNLYL/pwl58jg//+uEMQwIzuGM
vTwm+P/gdI7XRb7OOnHee3I+k5ePvLbkxya3LbbArZV5zcOD/3fAL9GuJFcg
+WvwZZHPGbsPbJL/vwA+SV6gPtEuI888/wsJGwRbm/eYdxgMlf+vBI5KLs2e
ESchfzD/e+XrxDgb+R3I7QCesH1wDhDyyvM/LMhLTB15sKknHx6xDNhq4LHE
yhIni02HD4L4hz2C8zmDa5G/k/gJ7EL0EXJxo5MQN4wPgnhbfMXk8caGRZch
Rzf6DDYjvg/iKLCXycMIJwKslP93A156beUc1/ip8U8Th4y9Q84kYiuwLy8M
/l8bfPtXhqQliAbd4Jbg/2HEt0yuP/ILgO+BD+CzAL9C7yAvN7rHf5w/cCfy
hhI3gT10Um1fM7k8WceJ64ZzhU8SrjXrCzbsAsG8HrAL5kHmQPAZ3i3eK7AU
1hLWEb5F8qvzPU4rnGeEHCPkbCDeAd8mOuT3tWMk/g8JOL+8
         "]], Polygon3DBox[CompressedData["
1:eJwtnHkcDtUbxWfed/aivaR9oRCpaI+ESFRS+WkVqbQpJe1aUApJaNFCVChL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         "]], Polygon3DBox[CompressedData["
1:eJwt1nmQz3Ucx/Ev9ufY0phqVCpESITYbI51W1YtNmdptDkq5Y7GFWpRiyiU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         "]]}]}, {}, {}, {}, {}}, {{}, {}, {}, {Line3DBox[CompressedData["
1:eJwV0L8rxHEcx/H3CYdCqaurk7pSShGXH/+CckVGixx3g+HISmEymZQ6i8lk
Icpkk3RxSZIkP6/4K+RhePTu+3p+pm+2sDxTTkTEPOONEZfNES1NEStuqzvb
FjGSjFj1fUW7bZAO5rRObc1+zbBtghwLWpe2Za8yaSuQJ2Xftt3w5l3Rtk6J
tLZjr/Gubdj22CSj7drv+NAqtmP26dEq9ntOfVc5o+hdVjuwP3Brq1NjSevV
Du2PfNt++aGs9WlH9icSpGmg337iPvPpXcbN0c2Adu6+8KWNunnGGNIu3Ffq
2pS7yPT/v9NK7h94NjKw
       "]], 
      Line3DBox[{1080, 1291, 1649, 1292, 2651, 1651, 1295, 1328, 1296, 2656, 
       1660, 1331, 1354, 2706, 1332, 1666, 1357, 2707, 1380, 1358, 1672, 1383,
        1406, 1384, 1678, 2766, 1409, 1432, 1410, 1684, 2768, 1435, 1458, 
       1436, 1690, 1461, 1484, 1462, 2662, 1696, 1487, 1510, 1488, 2667, 1702,
        1513, 1536, 1514, 1708, 1539, 1562, 1540, 1714, 1565, 1588, 1566, 
       1720, 2777, 1591, 1614, 1592, 1726, 2779, 1617, 1637, 1618, 1164}], 
      Line3DBox[{1081, 1226, 1227, 2652, 1297, 1230, 2680, 1231, 1333, 1234, 
       2681, 1235, 1359, 1238, 2682, 1239, 1385, 1242, 2684, 1243, 1411, 1246,
        2686, 1247, 1437, 2726, 1250, 1251, 1463, 1254, 1255, 2663, 1489, 
       1258, 2687, 1259, 1515, 1262, 2688, 1263, 1541, 1266, 2689, 1267, 1567,
        1270, 2690, 1271, 1593, 1274, 1275, 1619, 2755, 1278, 1279, 1775}], 
      Line3DBox[{1082, 1298, 1761, 2789, 1299, 1652, 1301, 1762, 2790, 1302, 
       1661, 1334, 1763, 2791, 1335, 1667, 1360, 2708, 1764, 1361, 1673, 1386,
        2716, 1765, 1387, 1679, 1412, 2722, 1766, 1413, 1685, 1438, 1767, 
       1439, 1691, 1464, 1768, 1465, 1697, 1490, 1769, 2792, 1491, 1703, 1516,
        1770, 2793, 1517, 1709, 1542, 1771, 1543, 1715, 1568, 2746, 1772, 
       1569, 1721, 1594, 2751, 1773, 1595, 1727, 1620, 1774, 1621, 1165}], 
      Line3DBox[{8, 592, 2332, 23, 298, 2120, 38, 321, 2146, 53, 1908, 344, 
       68, 1921, 367, 83, 1934, 390, 98, 1948, 413, 113, 436, 2232, 128, 459, 
       2249, 143, 482, 2265, 158, 505, 173, 1980, 528, 188, 1993, 551, 203, 
       2007, 572, 218}], 
      Line3DBox[{1086, 1309, 1310, 2653, 1655, 1311, 1312, 2657, 1662, 1340, 
       1341, 2660, 1668, 1366, 2710, 1367, 1674, 2765, 1392, 1393, 1680, 2767,
        1418, 1419, 1686, 2769, 1444, 1445, 1692, 1470, 1471, 2664, 1698, 
       1496, 1497, 2668, 1704, 1522, 1523, 2671, 1710, 1548, 2742, 1549, 1716,
        2776, 1574, 1575, 1722, 2778, 1600, 1601, 1728, 2780, 1626, 1627, 
       1169}], Line3DBox[CompressedData["
1:eJwVzssuAwEYR/Fv6IVKvEN3noLE/X5bWVjNHomtWNl6DvEYNnYkJCQICQmJ
SyUtbbW0+M1icv7fOTNpy+nW6mYSEalnJBcxVow4wWoh4hRr+FqKKOcjxrUz
7oM7x5T/tIe1Ce2Cq7vf+Eu7Ya9pk9qVe4dvctf2F25rU9qte1drcXd2G/e1
aa3B33Pf3AP+4IF2yM/oj1yHe8IuHmmz/LP71/2Cf1jx/o02p1W4wHfc4BO7
rs1jlevJGl+ze+1B3y9k77hz2X/CPA7xi9hy73m/YLftIo5qS9jkO1yf3cV+
XNeO+WU7/G4JE1zBAc8/Uow/Ag==
       "]], 
      Line3DBox[{1088, 1314, 1315, 2654, 1657, 1318, 2696, 1319, 1663, 1344, 
       2704, 1345, 1669, 1370, 2712, 1371, 1675, 1396, 2718, 1397, 1681, 1422,
        2724, 1423, 1687, 2770, 1448, 1449, 1693, 1474, 1475, 2665, 1699, 
       1500, 2733, 1501, 1705, 1526, 2739, 1527, 1711, 1552, 2744, 1553, 1717,
        1578, 2747, 1579, 1723, 1604, 2753, 1605, 1729, 2781, 1630, 1631, 
       1642}], Line3DBox[{1089, 1316, 1656, 2763, 1317, 1658, 1320, 1343, 
       2703, 1321, 1664, 1346, 1369, 2711, 1347, 1670, 1372, 2713, 1395, 1373,
        1676, 1398, 2719, 1421, 1399, 1682, 1424, 2725, 1447, 1425, 1688, 
       1450, 2729, 1473, 1451, 1694, 1476, 1499, 2732, 1477, 1700, 1502, 1525,
        2738, 1503, 1706, 1528, 1551, 2743, 1529, 1712, 1554, 1577, 1555, 
       1718, 1580, 2748, 1603, 1581, 1724, 1606, 2754, 1629, 1607, 1730, 1632,
        2758, 1641, 1633, 1643}], 
      Line3DBox[{1162, 1222, 1608, 1156, 1218, 2749, 1582, 1150, 1214, 2745, 
       1556, 1144, 1210, 2740, 1530, 1138, 1206, 2734, 1504, 1132, 1202, 2730,
        1478, 1126, 2771, 1198, 1452, 1120, 1194, 1426, 2674, 1114, 1190, 
       2720, 1400, 1108, 1186, 2714, 1374, 1102, 1182, 2705, 1348, 1096, 1178,
        2697, 1322, 1090, 1174, 2692, 1283, 1076, 2759, 1170, 1282, 1644}], 
      Line3DBox[{1163, 1223, 1634, 1610, 2678, 1157, 1219, 1609, 1584, 1151, 
       1215, 1583, 2750, 1558, 1145, 1211, 1557, 1532, 1139, 1207, 2735, 1531,
        1506, 1133, 1203, 1505, 1480, 1127, 2772, 1199, 1479, 1454, 1121, 
       1195, 1453, 1428, 2675, 1115, 1191, 1427, 1402, 1109, 1187, 1401, 2721,
        1376, 1103, 1183, 1375, 2715, 1350, 1097, 1179, 2698, 1349, 1324, 
       1091, 1175, 1323, 1286, 1077, 2760, 1171, 1646, 1284, 1645}], 
      Line3DBox[{1166, 1224, 1639, 1623, 1159, 1220, 1622, 2756, 1597, 1153, 
       1216, 1596, 2752, 1571, 1147, 1212, 1570, 1545, 1141, 1208, 2736, 1544,
        1519, 1135, 1204, 2731, 1518, 1493, 1129, 1200, 1492, 1467, 1123, 
       1196, 1466, 1441, 1117, 1192, 1440, 2727, 1415, 1111, 1188, 1414, 2723,
        1389, 1105, 1184, 1388, 2717, 1363, 1099, 1180, 2700, 1362, 1337, 
       1093, 1176, 2694, 1336, 1304, 1083, 1172, 2693, 1653, 1300, 1303}], 
      Line3DBox[{1167, 887, 1640, 982, 2804, 1160, 979, 1624, 975, 1154, 972, 
       1598, 2803, 968, 1148, 965, 1572, 961, 1142, 958, 2737, 1546, 954, 
       1136, 951, 1520, 947, 1130, 2773, 944, 1494, 940, 1124, 937, 1468, 933,
        2802, 1118, 930, 1442, 926, 1112, 923, 1416, 2801, 919, 1106, 916, 
       1390, 2799, 912, 1100, 909, 2701, 1364, 905, 1094, 902, 2695, 1338, 
       898, 1084, 2761, 895, 1654, 857, 1305}], 
      Line3DBox[{1168, 1225, 1625, 2679, 1161, 1221, 1599, 2677, 1155, 1217, 
       1573, 1149, 1213, 2741, 1547, 1143, 1209, 1521, 1137, 2775, 1205, 1495,
        1131, 2774, 1201, 1469, 1125, 1197, 1443, 2676, 1119, 1193, 1417, 
       2673, 1113, 1189, 1391, 2672, 1107, 1185, 2709, 1365, 1101, 1181, 2702,
        1339, 1095, 2764, 1177, 1308, 1085, 2762, 1173, 1306, 1307}], 
      Line3DBox[CompressedData["
1:eJwV0EkrhWEYgOFX4ZQQER2RKSJTyDxLCSkd8xxnY8VGFqSsbKzlZ8hvkCRJ
kiQZSraysxLXWdw99/PeT331la5vxTaTQgg7iKSEcJ4aQnokhBS+xZv4JH7S
Qmg2W5NDuPDewtPc1CGDf+lTZiaG3Fy6aeUN+jCO9Cz7NLIx6ubKTRsf0V/1
NTPHPoNcTLi5dtPOv/W4voc8+yzyE7dubtx08H3tGFE+hwIs6bd6Jz/RTlHI
51GEdf1O7+Jn2hWK+QJKsKHf6938WvtAGV9EOTb1B72Hf2q/qEh8F5XY1h/1
Xv6n5fMqvoxq7OpP3vp41GxEDV9BLQ70Z2/9vNkcQz1fTfxbHOov3gb4uBnH
G2L2d3PQ/AeoAjNS
       "]]}, {Line3DBox[CompressedData["
1:eJwVzU0ug1EchfG/keqYqqKU1sCAhkokxBIMJJ34Cm+NWYUFtRtgpmoHlNZn
Sxdg2t87eHLOee69uaXk5uh6IiIOcTkZsZGJmJmKeMhG9OwLVLlZrs317Zz+
ijfk8c8/y3dn0/qL3sWxXpAf/JzctE8wr39yCdbtU2z7o8V/cU35zdW4xbRz
C3LHXpJFZ7co6v30X5TQwyP/4/6yvuv+iuxwA66BLf0Mq/zQvpP3+OPq2PPm
l79CzT5HJT23y/LJ3ndnTR9xB/oYfnAqRQ==
       "]], Line3DBox[CompressedData["
1:eJwNzE0ug2EYheFH0qR0jqF2RFv6Q0I0EZGmIU0bTTeAJv4mTC1AqjNDC6EL
YGyCHZQaVRUbcA3unHPu53u/TOeyfTEVEU2cJCOK0xGLMxELqYhrHHNlbom7
S0Ts6AN9Vc66j/Fiv6LKvckSd4CsPvK+xpftQ+S4L+4UV/a82y4e+D7u8cmv
c3l97Ls9fQPLdtqti7o9tDflnP2u38gP2eAqWNGfuW//OMOafoQCP7Ef5RPy
XAf73vzw5+jZW3bR/ddu6bfctixxf1xb/wdfgidT
       "]], Line3DBox[CompressedData["
1:eJwNzEtKAmAYheHPUekGsqHpBiIsQqLUECISNCiCBmEFoaUETlpCdiGatBNt
bkuoLSTptPvIZ/Byznn/ny/T6NTbiYioojUTsTEbsZiMyKQirtHkStwSt2D3
sGe/25tyzt7HDV65A+5N5u1j5PVPNw75R7siV7gv7hxXXJo7woB/Rh8jfotb
1b/9a+jbKNhZb7c4tSf2jkzbH/qdHMszroo1Pcfd48KdZXmCdf7HHsoXTLh5
/zv45dt44Gp20fuffak/cbuyzP1zXX0KU9kpTA==
       "]], Line3DBox[CompressedData["
1:eJwVzUtKAmAYheGvQdgFnFc2aAshIRER0SyciFEUFmUUWnnPvKwiKknEYe3E
bTTVQdEWehoc3nPe74d/46qaq8xFRFb6iYj2QsTBYsQjDpcielxT3+ca+M51
uXW9Jinpc0np8BV7z9sHHHBP3Bve27v8Hba5H/2XL9srWMLv/3/dxvatvWPf
YIu7xm27iK/uTW5ir9mXsio9vsFf6Bm3c3zh6lxB3+IO5yM+cZn/wC88c5vp
pzjFmvcj/sRO28dY5Z65I32Ty2NX/gAuaCXU
       "]], Line3DBox[CompressedData["
1:eJwVzD1OAmEUheFrYUBjQm+BsgrEXhQFC5dAQ6d2GGX4cXBANLE3BmModCV2
rsEdIOzAZ4qTc973fjOV9tXF5UZEtGRaiHguRpxvRTzp9+2ICTezW9yjnnMZ
V7Gnsi8JV5IHfoKb3v7gQxlzNX0tZ3zmnnJre51/g3fzd3rF3bstcIpPc9Yj
bqQbeKjf3IfcN97DAylLjx/wffvErbNp4wQf41/3A/myd/SnruqV/Nk975Y6
8c0Hd4fr+Db/N/fK3dhHXFe/yD9Coim+
       "]], 
      Line3DBox[{1941, 2206, 2392, 2205, 2725, 1940, 2204, 2391, 2203, 2724, 
       1939, 2083, 2202, 2805, 1010, 1938, 2201, 2390, 2767, 2621, 1937, 2620,
        2673, 2389, 2200, 1936, 926, 1051, 925, 1935, 2727, 2199, 2388, 2198, 
       1934, 2197, 2387, 2196, 2722, 1933, 826, 2195, 2082, 2686, 1932, 2194, 
       2386, 2766, 2619, 1931, 2193, 2385, 2784, 762, 991, 761, 1849, 922, 
       921, 1930, 1402, 2384, 2192, 1929, 2720, 2190, 2383, 2189, 2208}], 
      Line3DBox[{106, 664, 1120, 107, 665, 1121, 108, 666, 1122, 109, 1689}], 
      Line3DBox[{1955, 2220, 2404, 2219, 2729, 1954, 2218, 2403, 2770, 2627, 
       1953, 2084, 2217, 2648, 2728, 2576, 1952, 2216, 2402, 2769, 2626, 1951,
        2625, 2676, 2401, 2215, 1950, 2550, 2802, 2551, 2400, 2549, 2548, 
       1949, 1441, 2399, 2214, 1948, 2213, 2398, 1438, 1947, 2503, 2504, 2212,
        2726, 2604, 1946, 2211, 2397, 2768, 2624, 1945, 2210, 2396, 2785, 
       2475, 2474, 1944, 2473, 1743, 2395, 2547, 2546, 1943, 2623, 2675, 2394,
        2209, 1942, 2622, 2674, 2393, 2207, 2222}], 
      Line3DBox[{1957, 2221, 2405, 2771, 2043, 1956, 2223, 2406, 2772, 2044, 
       2228, 2552, 2553, 2407, 1824, 2045, 2579, 2518, 2519, 2661, 2408, 2046,
        2230, 2628, 2662, 2409, 2047, 1958, 2605, 2663, 2231, 2490, 2048, 
       2489, 1465, 2410, 2049, 2232, 2224, 2411, 1200, 2234, 2555, 2556, 2412,
        2554, 2773, 2050, 2236, 2225, 2413, 2774, 2051, 1959, 2629, 2664, 
       2414, 2052, 2505, 2506, 2796, 2646, 2238, 2053, 1960, 2630, 2665, 2415,
        2054, 2239, 2732, 2226, 2416, 2055, 2241}], 
      Line3DBox[{120, 675, 1694, 119, 674, 1693, 118, 421, 1472, 117, 673, 
       1692, 116, 1125, 672, 115, 1124, 671, 114, 1123, 670, 113, 669, 1691, 
       112, 412, 1463, 111, 668, 1690, 110, 667}], 
      Line3DBox[{1962, 2227, 2417, 2597, 2730, 1961, 2229, 2418, 1203, 2244, 
       948, 1061, 1883, 873, 1028, 874, 2666, 2419, 2056, 2247, 2631, 2667, 
       2420, 2057, 1963, 2687, 2085, 2248, 799, 2491, 2792, 2233, 2421, 2058, 
       2249, 2235, 2422, 2598, 2731, 2251, 952, 1062, 951, 2253, 2237, 2423, 
       2775, 2059, 1964, 2632, 2668, 2424, 2060, 2507, 837, 2669, 2256, 2061, 
       1965, 2733, 2240, 2425, 2062, 2257, 2738, 2242, 2426, 2063, 2259}], 
      Line3DBox[{1967, 2243, 2427, 2599, 2734, 1966, 2245, 2428, 2600, 2735, 
       2261, 2557, 2246, 2587, 1829, 2520, 2262, 2521, 2633, 2670, 2429, 2064,
        2263, 1514, 2430, 2065, 1968, 2688, 2086, 2264, 2493, 2494, 2492, 
       2793, 2250, 2431, 2066, 2265, 2252, 2432, 2601, 2736, 2266, 2560, 2254,
        2588, 2558, 2737, 2559, 2267, 2255, 2433, 1209, 1969, 2634, 2671, 
       2434, 2067, 2508, 2509, 2797, 2087, 2268, 2068, 1970, 2739, 2258, 2435,
        2069, 2269, 2743, 2260, 2436, 2070, 2270}], 
      Line3DBox[{496, 472, 2740, 1971, 475, 1557, 497, 477, 1559, 499, 1560}],
       Line3DBox[{516, 494, 1577, 514, 492, 2744, 1974, 247, 1796, 842, 489, 
       2742, 1973, 2741, 488, 509, 1572, 487, 507, 1570, 485, 505, 483, 1771, 
       802, 246, 2689, 1972, 480, 1562, 501, 479}], 
      Line3DBox[{1987, 2291, 2446, 2290, 2748, 1986, 2289, 2445, 2288, 2747, 
       1985, 2091, 2287, 2090, 2691, 2577, 1984, 2286, 2444, 2776, 2636, 1983,
        1573, 2443, 2285, 1982, 2564, 2803, 2284, 2589, 2283, 2563, 1981, 
       2752, 2282, 2442, 2281, 1980, 2280, 2441, 2279, 2746, 1979, 2510, 2089,
        2278, 2088, 2690, 1978, 2277, 2440, 1565, 1977, 2276, 2439, 2786, 
       2635, 2477, 2573, 2476, 1561, 2562, 2275, 2561, 1976, 2750, 2274, 2438,
        2273, 1975, 2745, 2272, 2437, 2271, 2294}], 
      Line3DBox[{2000, 2309, 2456, 2308, 2754, 1999, 2307, 2455, 2306, 2753, 
       1998, 2092, 2305, 2806, 1017, 1997, 2304, 2454, 2778, 2639, 1996, 2638,
        2677, 2453, 2303, 1995, 975, 1072, 974, 1994, 2756, 2302, 2452, 2301, 
       1993, 2300, 2451, 2299, 2751, 1992, 847, 2298, 1274, 1991, 2297, 2450, 
       2777, 2637, 1990, 2296, 2449, 2787, 776, 998, 775, 1856, 971, 970, 
       1989, 1584, 2448, 2295, 1988, 2749, 2293, 2447, 2292, 2311}], 
      Line3DBox[{2014, 2323, 2468, 2322, 2758, 2013, 2321, 2467, 2781, 2644, 
       2012, 2093, 2320, 2649, 2757, 2578, 2011, 2319, 2466, 2780, 2643, 2010,
        2642, 2679, 2465, 2318, 2009, 2569, 2804, 2570, 2464, 2568, 2567, 
       2008, 1623, 2463, 2317, 2007, 2316, 2462, 1620, 2006, 2511, 2512, 2315,
        2755, 2606, 2005, 2314, 2461, 2779, 2641, 2004, 2313, 2460, 2788, 
       2480, 2479, 2003, 2478, 1757, 2459, 2566, 2565, 2002, 2640, 2678, 2458,
        2312, 2001, 1608, 2457, 2310, 2324}]}}},
   VertexNormals->CompressedData["
1:eJxMXHlcTd37RQMNogkNhiTJVBQJ3Y3SRNEkRSQRohTNRTKkFImkJJUxoqSB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    "]],
  Axes->True,
  Method->{},
  PlotRange->{{-0.9999999999998427, 0.999999999999742}, {-0.9999999999998175, 
   0.9999999999998175}, {-1.5707957657962084`, 6.283184746180898}},
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.6291870555163193`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"sfrev", "[", 
   RowBox[{"id", ",", "Sin"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"u", " ", 
    RowBox[{"Cos", "[", "v", "]"}]}], ",", 
   RowBox[{"u", " ", 
    RowBox[{"Sin", "[", "v", "]"}]}], ",", 
   RowBox[{"Sin", "[", "u", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.629187061679331*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sfrev", "[", 
     RowBox[{"id", ",", "Sin"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"4", "*", "Pi"}]}], "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5922257630910683`*^9, 3.592225775968805*^9}, {
   3.592225989373011*^9, 3.5922259964974184`*^9}, {3.5922260927779255`*^9, 
   3.5922261304900827`*^9}, {3.5922261650030565`*^9, 
   3.5922261971228933`*^9}, {3.592226229242731*^9, 3.5922263441073008`*^9}, {
   3.592226740379966*^9, 3.592226750132524*^9}, {3.592226787731674*^9, 
   3.59222679937234*^9}, {3.59222687175648*^9, 3.592226888444435*^9}, {
   3.5922269726212497`*^9, 3.5922269743323474`*^9}, 3.5922270304075546`*^9, {
   3.5922271192936387`*^9, 3.5922271563647594`*^9}, {3.592227492203968*^9, 
   3.59222750168251*^9}, {3.5922275492762327`*^9, 3.592227574379668*^9}, {
   3.592227739908136*^9, 3.5922277753561635`*^9}, {3.592228075636338*^9, 
   3.592228080619623*^9}, {3.5942731171882453`*^9, 3.594273179040783*^9}, {
   3.594273287980014*^9, 3.59427333423666*^9}, {3.5942733772041173`*^9, 
   3.5942733783931856`*^9}, {3.5942734515083675`*^9, 3.594273490251583*^9}, {
   3.594273530348877*^9, 3.5942735469958286`*^9}}],

Cell[BoxData[
 Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJx0fXc8V+/7v509y94iSguVSq9zQtoaNClt2kuDFu2dliJFU0vvdiivc5BI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   "], {{
     {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJxNnXXcFdX3/efOzL1TdjeChYGNia2ABYqK3diBrRiIrSh2t4DdYn1sbMXu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         "]], Polygon3DBox[CompressedData["
1:eJwtnGMY7EgThTPKJDNr27Zt27ZtG3dt27xr27Zte/dbm9/73jM/+pmuru5O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         "]], Polygon3DBox[CompressedData["
1:eJwl1XtMlmUYx/FHEBDMrIAO6kotpmkemplmbQxYRrZWqZvrQBkIhAdIEU94
AAEl0DIHGpWrZVrZH+oqpMxDphIWZkOn01UazbX+KGvWXHT63Hv/+O29ru/1
u677fp/nfp5nSH7Z1NK4KIryKIGW9o2itSlRtDg+ik4lRdGqxCiq6hNFz+OT
xJ3YHLXp8oU0HSvHPjKkLTmKnsXOG/QyxWPL5Q3mLdNzmm8D/9V8R6i/+ma+
Ydi38lx6VHwD30i1xXpr9S7U24WVqjXYywL8IfHXgamtl68Je+4VRdnqOTTc
rEqsSL2b7x7+CiwOm2d2NvY69pjeiXiOOE9fnv2cl0+jGdgAvjv5R+mr5/1Y
Pht/wlrLsXXYaH1jqNhazX4rsKd4vuGdj5WZ9TcVm5OJNas18i3ie1q8ENuj
Npfnz3ANxZOwRrWz8vtosvg6LEPtX/lzen+z18NUgp3GRlMmX3++W7Ai+e/0
pHgCVhvWlWfZ90lsP3YgXG+zrqcR9lSEvaV2Ld9tfNvFJdgdfHOCD2vBZmKJ
4WzwxWNlWLu97AxnBJvF2y1/ldLVvsTSwh7Vk/SmYoPCOdCb2tsMa49Tq5E3
6P2Zp0Je6Dpf4ivEfjFrL83Qe0D9hPpR9Rw9Z7G+fF10kzglXCv1k/LBND4x
xkLtH2sdwvqJ47A0tfvNeMEaB+UX5GfM/YPvU74kvv9CP23ka+Tr4ftJfpHv
Mk8R9qu97ac8/lL5IN43xLN4B4ZzH3Le4+KpWJ1aOl8aDbfWUSzfebkSzgp2
yazPqIAvQ++IcBYSY/cj3JdlPGPxz8NzhlWrlWGDsa1YMTYUi9d3LDl2zcO1
vzk8s3yb+D7EFmDTsDa/j/vN9Z9Gqu0KZ5JvgHiLuED9GvU+4Vkwb594CrZU
LQFbjbViD2Dl4X5gTVg79kh4l2AfyGea+b3/tY56rFWHJfJWqB/D2vhbsKuw
auwE1oF94hpV6+0SXzHvfddqHN8ZGsL3Jl+W2l6+mnD+xE18F9VekvejOp75
4X1G4/WtSIk9tz/yZfPvwlfznRJvCGdB743hniXHznE4z2PVXgzz9ZaE+6Mn
n2qxdKzbe3Sz3lfoVn1Dsbf1tson6J3Ntygldq/bsd5YFt85Gsa31cwHsXl8
S/kK+TrC2cUe5vmORvFt55uKbbL2Et4y8RK+feGZlZfrfUbvESxO7R2/79IX
6uPVVrr2GWadooHYFvMm8h2WF+j/ISH2fg7v6Xv512Lv6d9Bx8P7gy8fu8DT
RMnYCvndvB3iKr6N5lVid2Ht2CqsHluDjcHm2sO2cJ7DOwFr919SXb968zLE
PdRqnV56/wrvQLqM7cauhGcvnEfaiVXpPxS+JWbV+N/bsFXYbuwcVoVl6rvd
ujvM24Plqq3HkrFKrNO6B7Eh8jq9r4XvRFLs/q0M30C8U1yDtfDNx5pTYt/E
r7Ap2P+dnOox
         "]]}]}, {}, {}, {}, {}}, {{}, {}, {}, {
      Line3DBox[{1417, 1537, 1989, 1538, 2960, 1777, 1539, 1991, 1540, 2963, 
       1789, 1560, 1993, 1561, 2966, 1797, 1577, 1995, 1578, 1805, 3021, 1594,
        1997, 1595, 1813, 3024, 1611, 1999, 1612, 1821, 3027, 1628, 2001, 
       1629, 1829, 1645, 2003, 1646, 2967, 1837, 1662, 2005, 1663, 2970, 1845,
        1679, 2007, 1680, 2973, 1853, 1696, 2009, 1697, 1861, 3029, 1713, 
       2011, 1714, 1869, 3031, 1730, 2013, 1731, 1877, 3034, 1747, 2040, 1748,
        1499}], 
      Line3DBox[{1418, 1541, 1888, 1542, 2961, 1889, 1562, 1892, 1563, 2964, 
       1893, 1579, 1896, 3037, 1580, 1897, 1596, 2993, 1900, 1597, 1901, 1613,
        1904, 1614, 1905, 3039, 1630, 1908, 1631, 1909, 3040, 1647, 1912, 
       1648, 1913, 1664, 1916, 1665, 2968, 1917, 1681, 1920, 1682, 2971, 1921,
        1698, 1924, 1699, 1925, 1715, 1928, 1716, 1929, 1732, 1932, 1733, 
       1933, 3042, 1749, 1936, 1750, 1937, 3043, 1763, 1940, 1764, 1765}], 
      Line3DBox[{833, 832, 1778, 1233, 3069, 2044, 1009, 1790, 3071, 1241, 
       2047, 1023, 1798, 3073, 1249, 2050, 1037, 2994, 1806, 1257, 2053, 1051,
        2999, 1814, 1265, 2056, 1065, 1822, 1273, 2059, 3055, 1079, 1830, 
       1281, 2062, 1093, 1838, 1289, 3075, 2065, 1107, 1846, 1297, 2068, 1121,
        1854, 3077, 1305, 2071, 1135, 1862, 1313, 2074, 1149, 3014, 1870, 
       1321, 2077, 1163, 1878, 1329, 2081, 3060, 1179, 1885, 944, 1228}], 
      Line3DBox[{1419, 1543, 1779, 3018, 1544, 1780, 1545, 1564, 2984, 1546, 
       1791, 1565, 1581, 2988, 1566, 1799, 1582, 2989, 1598, 1583, 1807, 1599,
        2995, 1615, 1600, 1815, 1616, 3000, 1632, 1617, 1823, 1633, 1649, 
       1634, 1831, 1650, 1666, 1651, 1839, 1667, 1683, 3006, 1668, 1847, 1684,
        1700, 3010, 1685, 1855, 1701, 1717, 1702, 1863, 1718, 3012, 1734, 
       1719, 1871, 1735, 3015, 1751, 1736, 1879, 1752, 1766, 1753, 1767}], 
      Line3DBox[{8, 667, 2584, 23, 673, 2590, 38, 679, 2598, 53, 2169, 685, 
       68, 2179, 691, 83, 2189, 697, 98, 2201, 703, 113, 709, 2610, 128, 715, 
       2616, 143, 721, 2622, 158, 727, 173, 2216, 733, 188, 2226, 739, 203, 
       2238, 743, 218}], 
      Line3DBox[{910, 247, 2484, 509, 248, 2496, 523, 268, 2512, 535, 2513, 
       286, 547, 2521, 304, 559, 2527, 322, 571, 2533, 340, 583, 358, 2541, 
       595, 376, 2549, 607, 394, 2560, 619, 412, 631, 2567, 430, 643, 2573, 
       448, 655, 2579, 466, 902}], Line3DBox[CompressedData["
1:eJwVzd8qw3EcxvGPZCOrSSmlVtslcAlcgSh/htnYMPYjZQcOHDjYagcrIis3
wLEjLoFLmJ3jyIkz5eXg3fN53s+33y9fThYbQxFRRHckIpuKqKQjJuTVeMSv
e2U0YhV5rqYX5I0tuDWsY5ar2wb8nHuYK2ID8/oTn9gX3CluE1tIZyKWuFPb
shzjStjGrt7kqzKjf/tGWVZwxp3bmjKr7/y/R0u/4Nuy5/0kV0UNd9ylrSfv
bVPcHvbxwLVsj3JaP0Adz3qHf5Ez+iGO8Kp3+TeZ0xtI0Nev+XdZ0H/851ie
4IO7tX3KL/wBcj8svg==
       "]], 
      Line3DBox[{840, 839, 1784, 1238, 3070, 2046, 1017, 1793, 3072, 1246, 
       2049, 1031, 1801, 3074, 1254, 2052, 1045, 2997, 1809, 1262, 2055, 1059,
        3001, 1817, 1270, 2058, 1073, 1825, 1278, 2061, 3058, 1087, 1833, 
       1286, 2064, 1101, 1841, 1294, 3076, 2067, 1115, 1849, 1302, 2070, 1129,
        1857, 3078, 1310, 2073, 1143, 1865, 1318, 2076, 1157, 3016, 1873, 
       1326, 2079, 1171, 1881, 1334, 2083, 3063, 1190, 1887, 950, 1231}], 
      Line3DBox[{1424, 1553, 1785, 3019, 1554, 1786, 1555, 1571, 2986, 1556, 
       1794, 1572, 1588, 2991, 1573, 1802, 1589, 2992, 1605, 1590, 1810, 1606,
        2998, 1622, 1607, 1818, 1623, 3002, 1639, 1624, 1826, 1640, 1656, 
       1641, 1834, 1657, 1673, 1658, 1842, 1674, 1690, 3008, 1675, 1850, 1691,
        1707, 3011, 1692, 1858, 1708, 1724, 1709, 1866, 1725, 3013, 1741, 
       1726, 1874, 1742, 3017, 1758, 1743, 1882, 1759, 1772, 1760, 1773}], 
      Line3DBox[{1497, 1528, 1883, 1745, 1491, 1526, 1875, 3033, 1728, 1485, 
       1524, 1867, 3030, 1711, 1479, 1522, 1859, 1694, 1473, 1520, 3005, 1851,
        1677, 1467, 1518, 3003, 1843, 1660, 1461, 1516, 1835, 1643, 1455, 
       1514, 1827, 1626, 1449, 1512, 1819, 3026, 1609, 1443, 1510, 1811, 3023,
        1592, 1437, 1508, 1803, 3020, 1575, 1431, 1506, 2983, 1795, 1558, 
       1425, 1504, 2980, 1787, 1533, 1414, 1502, 1775, 1531, 1774}], 
      Line3DBox[CompressedData["
1:eJwVzr8rxHEcx/G3wY/yB5gVdYqiKFcURR25QixsZvEX2E0W0y3oymRhusmg
04WELi5dSkcSkiS56wYehlev9+v5/Pbp2760OrvSEBELUm6MqMlPc8RDk1vf
2PMtEXNyZxfxbGvEt/uSm8GnpWwfcV/6FE9jU3Jr7+OfOo9PYCkp2dv4hz7E
x7G6d8f0NbbBvescN4qNSNFew9/0AT6MDcmVvYy/6D08iQ3Khb2I73j32b3L
DeD9cm5Pclvc0/+/cH14r5zZr3iSf3RnuB68W07sBF7Rm3gXVvNtQhewNu5e
r3OdWJXr0MfYLxb8H67mOgE=
       "]], 
      Line3DBox[{1500, 1530, 1886, 1754, 1494, 1527, 1880, 3035, 1737, 1488, 
       1525, 1872, 3032, 1720, 1482, 1523, 1864, 1703, 1476, 1521, 3007, 1856,
        1686, 1470, 1519, 3004, 1848, 1669, 1464, 1517, 1840, 1652, 1458, 
       1515, 1832, 1635, 1452, 1513, 1824, 3028, 1618, 1446, 1511, 1816, 3025,
        1601, 1440, 1509, 1808, 3022, 1584, 1434, 1507, 2985, 1800, 1567, 
       1428, 1505, 2982, 1792, 1548, 1420, 1503, 2981, 1782, 1547, 1781}], 
      Line3DBox[{1187, 859, 1360, 1185, 3061, 1495, 1331, 1358, 1167, 1489, 
       1323, 1356, 2892, 1153, 1483, 1315, 1354, 1139, 1477, 1307, 2890, 1352,
        1125, 1471, 1299, 1350, 1111, 1465, 3059, 1291, 1348, 1097, 1459, 
       1283, 1346, 1083, 3056, 1453, 1275, 1344, 1069, 1447, 1267, 1342, 2888,
        1055, 1441, 1259, 1340, 2886, 1041, 1435, 1251, 2884, 1338, 1027, 
       1429, 1243, 2882, 1336, 1013, 1421, 3054, 1235, 1783, 835, 908}], 
      Line3DBox[CompressedData["
1:eJwVzb8rxHEcx/G3wVHib2ChKEJRFHXKFUXdFaO6ZDCcUgwGg+EGww0KixKW
m678WJhMBplMSkqSDkeSDMfgYXj2+ryfz+t7LdmFdK4mImZwVxtxVBdxiE2U
ExE/9q0h4kabro+YwgZ3qx3w397XWoZPo+C+1L7sFT/JTWDdfcp/2gt+nBtD
3l3kP+w5n+J+fXfUrnHb2rs90575ET6JVS6vVeyJNswNYcW9xL/Yit+XtEF+
AMvcrLbHl///V+vn+7Dozmi72pP3vtbL9yDnfuWT+qP3jtbFd6LEz3Pd2oPd
0jr4Kt9u57hm7d4WtDauFVl3I39sm+wfeBE5EA==
       "]], Line3DBox[CompressedData["
1:eJwVzytLA2AYhuFX8AQatRkUhskoWIwGg0ksgkGMC9sP8AdYTBbP5/NxBp2n
6ZwWkxiMIgieolmGeH3h5nl47nfbt46x/GCuJiJG8VcXUWmIeEVV/6mPmNZn
8NIUMWE70VvtVczqc5iyn8pOW63cdzsvF9JnuaLs4d7tzfoilhLcmeznPrgW
fRkr2ODO5TDXJlexhj37p9sLPctl5Hq6xzF3KcftXXITW3hzX+Su9EmuW25j
ByV7Kb3J3isrbnfT7+COu5ZHXF/6XzjAg/1Glu1f7gf0w3SHR64sn7ghWUjv
wrP92+1teg83kr4fv7hHe2PEP0IMP+4=
       "]]}, {Line3DBox[CompressedData["
1:eJwVzkkug3EYx/HHQlIsmxoaSg1xlG6lvYDYNYQEvYEDYK9l0aUaqj1CUYru
pUewEDGkJT5dfPObnv+bN7uxk98eiYg17CYi3lEbi+iNRpzT/HhEASkcTES8
ouVmlp7oytjHt5zErW1PLvKHuPCNO92kblM+whRfwb1+Xp7DLzKo69v6S+/a
8gd/xT/Qadsnrcvrtiqu+S/djW1BfsOjnKWnujOUsKrrYIb/Gd57l+Of+DTd
cn+MBr9In/UrdAl/WEbT9qJvDP9H7vNNvksH+AeQ7zKh
       "]], Line3DBox[CompressedData["
1:eJwVzMkug3EYhfFTEdqyNCzMG+NdsMGy6Q3o0qJdSHAfWNhSIm6gg9kloG5E
JEgMP4snz3nP+X/fXK1RqReSVLBTTN7QLiVbQ0kTLblaTn70E3w8mKxzR3/C
4T3+9HYEo9h1b/MBuvYn307q6u5DTMmnaOLKPq/7RUFu6Z6975MvdO/ytfzC
07YPvnXXbOe4kfvRti24V7GIO92ZboD3+Us3hnHM/N/+c2/bkDcxi4btCB15
idewjAfvVjjDSVHu2nu+L8mX+m/5UX7lMv8Bi/AtAg==
       "]], Line3DBox[CompressedData["
1:eJwNzr8uQ3EYxvG3RPzpSRokogmaE4KhlHtgwtKkN4C1LJaK1WbuJTgawhUY
RFUqpMKACrfCZ/jmed7vc07yS3cPqwe5iKjiaCTiD+3RiL18RIZ7vTYWkchU
nnKX2NI73LksyGNZ9u+0raHXZROPtg9+nvtxj7sX9AwX6LrLfC6JmNBvuU/f
T+pX/KB8whe3aBvSX7Bva+FZn8KdbcW9gVX0uBZXlCfyjLtGEUvuYf4V2/oO
lvFrm+Ee9Iq+iTW8cetywBtn9Y697z1z+g2fl+/45kryH+1QKTU=
       "]], Line3DBox[CompressedData["
1:eJwVzr0uw3EYhuG3i1AhBk2FRSstpRKSSnzsbKg0mM1Sk291EpyGBKmJqPiI
UmkicQgmTsJluHM/7/P0908zWztr1UREVHDaGZHpivjCMC67I2q6Bfk1GTHP
N7qm/I43LLnn9GWe5Tqv4sS7dfuQ7hODONad637tM+6aXOJHfsYTFm3TuhWe
4mU+8q7NA+420jjUpbiFflzYD3ST8r3vFPla15AfsI0JXY7H+cq27/d9chOF
jog995n9xzb2/3/lUb7lO2zi2zaiy8pZ3uBd7z70ve4X9CCPPz68Kig=
       "]], Line3DBox[CompressedData["
1:eJwVzL8uw2EUxvFjQXETEoZ2ICFhKZIKpUL4qUFtdkqY1d9yA4xchETjGtBq
aZXdffgYvnme8z3veYd3y8l+T0QUcdMfMZOK+EEWT4MR11yifwxEbMhn7lPv
oI1l8zq/JfuGIo5lEVV32/bTdt+YwhV3xx3ZP6Cir/HvsoUmlvgVblMWZCIv
3TXlpLmLCVxw4/ILY3i0P+fy+pt/FmWNq+sN7GGBy8jV3ogzb9Pmzv977w5w
yt3aH+r3ONHn7V/kK3bwy89xaX1WlmTFXYMfNbcxghz+AJCtLkM=
       "]], 
      Line3DBox[{2194, 2388, 2528, 2387, 3002, 2193, 2396, 2776, 1073, 1075, 
       1074, 1381, 2145, 1380, 2192, 791, 2527, 2383, 2191, 923, 2977, 2689, 
       1068, 1070, 1069, 1379, 1378, 2190, 3028, 2381, 2526, 2380, 2189, 2377,
        2525, 2376, 3000, 2188, 2392, 2775, 1065, 1066, 846, 2604, 3039, 2934,
        2187, 2372, 2524, 3024, 2930, 2186, 922, 2976, 2688, 1062, 1063, 
       793}], Line3DBox[{106, 575, 1455, 107, 700, 1456, 108, 882, 1457, 109, 
       1829, 577, 110, 1913, 702, 111, 2062, 1092, 112, 1831, 579, 113, 581, 
       1458, 114, 1095, 1459, 115, 883, 1460, 116, 583, 117, 2063, 1099, 118, 
       2064, 1100, 119, 1834, 585, 120}], Line3DBox[CompressedData["
1:eJwVz8suA3EUB+Bjo8pT9AF4FUmTKkOV2li1K4k1G7GwcZlpaXknK7fSm5JI
VJ/AN4sv55zf/zZTarTKzYWIWOdhKaJWjPjkcDHi3nygHy/LmK9ETNSefKp+
MeJC/scmXWs7shoTZ7e5k2XmH+vnnOo/eOfW2kCdyYdqn6p+g07+vu8YuyOh
bd5SR+zLM/Oe/sWZN36deVVTeVNtsZa/JT/mRl61f8ij+YhrWWrPdyHiRH3m
iav87vy/7TljVV9nl0trifcH7qlQ5x+70TZw
       "]], Line3DBox[CompressedData["
1:eJwVjzkOgVEURj8xxLQJKzD8ZpUFUFiAhEQkJBQKCS06NUqtaUXmeW5swHnF
yXffue++//6BcrPQsEnKwdQtFT3SHd5O6UaWvFIZgjDwSQdYcy9CznBz6EMW
t4EedZveCB7Mb3EhXIfzGMLUC9jhY5wtsPulqLmP3+MrzLVc0pM8mG/hS2Yn
+LDXi6zCEx9n7gdH7iXIJW5ldoWheRNO9CzzH2Ye8tRnXJTs0p/AizpJXvAZ
MgUO9kqTb3pXfI3ZDnt9yRvnOvkHTVgxKg==
       "]], Line3DBox[{2210, 2404, 2544, 2917, 3003, 2543, 806, 2613}], 
      Line3DBox[CompressedData["
1:eJwNzjlOAlAUheGLiUSFFWjFOmTsCBYoS9BCGyWYKDK2JsIWrOhsHHpjkPWg
oiY44PAVf/7zzn335WV267WjRERUMVqJOFyN+MB+KuIec3lvLeKUN/lcN0ZV
/tRdc5MHXLa/bnYhd/kSX2ZTfVY3Skbk+Aa3+DbLurOcjjiTZ7ond1vyo77O
Czzr8mYN+RcHZg/4kdt4Ncs5V5DHn+5O1+EhV+xv6AvysS54Gzso4sq/3rjg
zhaKSDiXOOlvXTvvzi/e6ckT/Qkv6Wa6vvwPDlIu8Q==
       "]], 
      Line3DBox[{420, 401, 1859, 622, 418, 1860, 624, 421, 2008, 893, 2009}], 
      Line3DBox[{434, 417, 1724, 433, 432, 1865, 632, 1355, 431, 1931, 729, 
       412, 728, 1930, 429, 630, 1354, 427, 628, 1864, 410, 727, 408, 1717, 
       425, 424, 1862, 626, 423, 1928, 725, 404}], Line3DBox[CompressedData["
1:eJwNzUsuw2EUhvHTaOtfVmEpjBnUzGUkGGIHVb24RQWJRqNG7tpFSFqRGlfs
wArcr7/Bm+e8z3dOvpG55cmlVETkpZ1EbOYikqGIDewOR7S4E/MP18R77tcc
+IcTeMzP4kM2YgZv3Mx7W+cHsYrXXMPcs1PDBveJ3/Il4+7q3DQe4hReuXnC
ip61U8ZLroQZfQ073i+4A/Mbt4933Lv5Q1ZkjxvDjr/P7Rb1tN63V5Az7ki/
9b6Du95f8FUW5dnONjdq3sIFPHXzyK/qA3oBa/IP8UU0Ng==
       "]], 
      Line3DBox[{2231, 2464, 2574, 2463, 3017, 2230, 2472, 2818, 1171, 1173, 
       1172, 1409, 2159, 1408, 2229, 819, 2573, 2459, 2228, 937, 2979, 2704, 
       1166, 1168, 1167, 1407, 1406, 2227, 3035, 2457, 2572, 2456, 2226, 2453,
        2571, 2452, 3015, 2225, 2468, 2817, 1163, 1164, 853, 2628, 3042, 2946,
        2224, 2448, 2570, 3031, 2944, 2223, 936, 2978, 2703, 1160, 1161, 
       821}], Line3DBox[CompressedData["
1:eJwNzr0uw2EUB+DTVL/RD27AxbBK7CWxtCWpcAfSqTEIK0vTIMoFqK/W1wV0
ky4GSxcbGptneHLO+Z33ff//pc2dtWYiIlZ5y0Zc5iIW8xEf6YhX87V5vhBR
YkqRF3lZrTDHIb9s8GxXc7/OAhfuP8lO9V/2Bxzr82qOoV1B/WGWLOtUGdiN
/UfF+XPvPJrP1LL5Xf5gvjKnnM3wTZp7+a4ze6zwKWtxJ+86X5KNzfvcyk7M
f5mII3VGlqQv3zZP9G2W9Vs0uLEb+X5R3/Fej38K0yu2
       "]], Line3DBox[{2390, 2368, 2523, 2369, 3026, 2185, 792, 2603}], 
      Line3DBox[{2420, 2284, 2550, 2412, 3008, 2419, 2283, 2800, 1302, 1303, 
       1301, 2799, 2972, 2942, 2618, 763, 2549, 2409, 2617, 1213, 3068, 2697, 
       981, 1300, 1299, 2798, 2416, 2547, 3004, 2918, 2548, 2408, 2616, 2282, 
       2546, 2406, 3006, 2414, 2281, 2797, 1297, 1298, 980, 2615, 2971, 2941, 
       2614, 2280, 2545, 2970, 2937, 2695, 1212, 3067, 2696, 979, 1296, 762}],
       Line3DBox[{2466, 2444, 2569, 2445, 3033, 2222, 820, 2627}]}}},
   VertexNormals->CompressedData["
1:eJxMXXc8lt/7p9IuWorMZEWSLXluq5DspIzsraFNZI8IDZJNw8pIlIye2yoh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    "]],
  AutomaticImageSize->True,
  Axes->True,
  ImageSize->{334.14645191542564`, 146.38631786643256`},
  Method->{},
  PlotRange->{{-12.56636971676127, 
   12.566369716760006`}, {-12.566369716760954`, 
   12.566369716760954`}, {-0.9999999999999748, 0.9999999999999748}},
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]},
  ViewPoint->{0.9514712620717689, -2.95472313165787, 1.3469646070693408`},
  ViewVertical->{0.0866817536459293, -0.20448693722164318`, 
   12.252509266833048`}]], "Output",
 CellChangeTimes->{3.6291870652683372`*^9}]
}, Closed]],

Cell["\<\
We return to the general surfaces of revolution. The intersections of the \
surface with the axis are singularities:\
\>", "Text",
 CellChangeTimes->{{3.592307003801346*^9, 3.592307178864853*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Cross", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", "1", "]"}], "[", 
      RowBox[{"sfrev", "[", 
       RowBox[{"f", ",", "h"}], "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", "2", "]"}], "[", 
      RowBox[{"sfrev", "[", 
       RowBox[{"f", ",", "h"}], "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.592307183295261*^9, 3.592307206632902*^9}, 
   3.5923114948741784`*^9, {3.5927415609927454`*^9, 3.592741561928747*^9}, 
   3.61760002546109*^9, 3.617600085817596*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     RowBox[{"Cos", "[", "v", "]"}]}], " ", 
    RowBox[{"f", "[", "u", "]"}], " ", 
    RowBox[{
     SuperscriptBox["h", "\[Prime]",
      MultilineFunction->None], "[", "u", "]"}]}], ",", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"f", "[", "u", "]"}]}], " ", 
    RowBox[{"Sin", "[", "v", "]"}], " ", 
    RowBox[{
     SuperscriptBox["h", "\[Prime]",
      MultilineFunction->None], "[", "u", "]"}]}], ",", 
   RowBox[{
    RowBox[{"f", "[", "u", "]"}], " ", 
    RowBox[{
     SuperscriptBox["f", "\[Prime]",
      MultilineFunction->None], "[", "u", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.629187082537567*^9}]
}, Closed]],

Cell["\<\
This vector vanishes for f[u] == 0. If f[u] \[NotEqual]0 the singularities of \
the surface are circles generated by rotating a singularity of the meridian. \
\>", "Text",
 CellChangeTimes->{{3.5988833745252876`*^9, 3.5988834041653395`*^9}, {
   3.5988834859718833`*^9, 3.5988836255765285`*^9}, 3.612073729544692*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"amf", "[", "3", "]"}], "[", 
     RowBox[{"sfrev", "[", 
      RowBox[{"f", ",", "h"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", " ", "\[Element]", "Reals"}], ",", " ", 
     RowBox[{"v", "\[Element]", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5988828503643675`*^9, 3.5988829588157578`*^9}, 
   3.614861860314741*^9, 3.6149177921077614`*^9, 3.6155317601264367`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{"Cos", "[", "v", "]"}], " ", 
      RowBox[{"f", "[", "u", "]"}], " ", 
      RowBox[{
       SuperscriptBox["h", "\[Prime]",
        MultilineFunction->None], "[", "u", "]"}]}], 
     SqrtBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "u", "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["f", "\[Prime]",
            MultilineFunction->None], "[", "u", "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["h", "\[Prime]",
            MultilineFunction->None], "[", "u", "]"}], "2"]}], ")"}]}]]]}], 
   ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{"f", "[", "u", "]"}], " ", 
      RowBox[{"Sin", "[", "v", "]"}], " ", 
      RowBox[{
       SuperscriptBox["h", "\[Prime]",
        MultilineFunction->None], "[", "u", "]"}]}], 
     SqrtBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "u", "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["f", "\[Prime]",
            MultilineFunction->None], "[", "u", "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["h", "\[Prime]",
            MultilineFunction->None], "[", "u", "]"}], "2"]}], ")"}]}]]]}], 
   ",", 
   FractionBox[
    RowBox[{
     RowBox[{"f", "[", "u", "]"}], " ", 
     RowBox[{
      SuperscriptBox["f", "\[Prime]",
       MultilineFunction->None], "[", "u", "]"}]}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "u", "]"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["f", "\[Prime]",
           MultilineFunction->None], "[", "u", "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["h", "\[Prime]",
           MultilineFunction->None], "[", "u", "]"}], "2"]}], ")"}]}]]]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.629187110572817*^9}]
}, Closed]],

Cell["This is the null vector iff", "Text",
 CellChangeTimes->{{3.5923116177555943`*^9, 3.59231164965765*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"f", "[", "u", "]"}], "\[Equal]", " ", "0"}], ")"}], "  ", "||", 
  " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"f", "'"}], "[", "u", "]"}], "\[Equal]", " ", "0"}], ")"}], "&&", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"h", "'"}], "[", "u", "]"}], "\[Equal]", " ", "0"}], ")"}]}], 
   ")"}]}]], "Input",
 CellChangeTimes->{{3.592311665117277*^9, 3.592311835422776*^9}, 
   3.592741566343555*^9}],

Cell["\<\
We calculate general formulas for the fundamental invariants of the surfaces \
of revolution. To this aim we often assume that u is the arc length of the \
meridians v == const.\
\>", "Text",
 CellChangeTimes->{{3.592312523212384*^9, 3.5923126353141813`*^9}, {
  3.5924558966908503`*^9, 3.5924558969404507`*^9}, {3.618637873730548*^9, 
  3.6186378991117926`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"fund1", "[", 
     RowBox[{"sfrev", "[", 
      RowBox[{"f", ",", "h"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"s1", ",", "s2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"s1", ",", "s2"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.592293344680974*^9, 3.592293457656372*^9}, 
   3.5927415729891667`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["s2", "2"], " ", 
   SuperscriptBox[
    RowBox[{"f", "[", "u", "]"}], "2"]}], "+", 
  RowBox[{
   SuperscriptBox["s1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["f", "\[Prime]",
        MultilineFunction->None], "[", "u", "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["h", "\[Prime]",
        MultilineFunction->None], "[", "u", "]"}], "2"]}], 
    ")"}]}]}]], "Output",
 CellChangeTimes->{3.629187154381694*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Derivative", "[", "1", "]"}], "[", "f", "]"}], "[", "u", 
       "]"}], "^", "2"}], " ", "+", " ", "\n", "    ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Derivative", "[", "1", "]"}], "[", "h", "]"}], "[", "u", 
       "]"}], "^", "2"}]}], " ", "==", " ", "1"}], ",", " ", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"fund1", "[", 
       RowBox[{"sfrev", "[", 
        RowBox[{"f", ",", "h"}], "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"s1", ",", "s2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"s1", ",", "s2"}], "}"}]}], "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5923126741270494`*^9, 3.592312682005063*^9}, {
   3.5927415848451877`*^9, 3.59274158606199*^9}, {3.614861860330341*^9, 
   3.614861860330341*^9}, {3.6149177921233616`*^9, 3.6149177921233616`*^9}, 
   3.6155317601744394`*^9, 3.616235037932674*^9}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["s1", "2"], "+", 
  RowBox[{
   SuperscriptBox["s2", "2"], " ", 
   SuperscriptBox[
    RowBox[{"f", "[", "u", "]"}], "2"]}]}]], "Output",
 CellChangeTimes->{3.6291871735113277`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"fund2", "[", 
     RowBox[{"sfrev", "[", 
      RowBox[{"f", ",", "h"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"s1", ",", "s2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"s1", ",", "s2"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.592293344680974*^9, 3.592293492662834*^9}, 
   3.592741587465992*^9}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{
    RowBox[{"f", "[", "u", "]"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["h", "\[Prime]",
         MultilineFunction->None], "[", "u", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["s2", "2"], " ", 
          RowBox[{"f", "[", "u", "]"}]}], "-", 
         RowBox[{
          SuperscriptBox["s1", "2"], " ", 
          RowBox[{
           SuperscriptBox["f", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "u", "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox["s1", "2"], " ", 
       RowBox[{
        SuperscriptBox["f", "\[Prime]",
         MultilineFunction->None], "[", "u", "]"}], " ", 
       RowBox[{
        SuperscriptBox["h", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "u", "]"}]}]}], ")"}]}], 
   SqrtBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"f", "[", "u", "]"}], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["f", "\[Prime]",
          MultilineFunction->None], "[", "u", "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["h", "\[Prime]",
          MultilineFunction->None], "[", "u", "]"}], "2"]}], 
      ")"}]}]]]}]], "Output",
 CellChangeTimes->{3.629187180125739*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Derivative", "[", "1", "]"}], "[", "f", "]"}], "[", "u", 
         "]"}], "^", "2"}], " ", "+", " ", "\n", "    ", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Derivative", "[", "1", "]"}], "[", "h", "]"}], "[", "u", 
         "]"}], "^", "2"}]}], " ", "==", " ", "1"}], ")"}], "&&", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"f", "[", "u", "]"}], ">", "0"}], ")"}]}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"fund2", "[", 
       RowBox[{"sfrev", "[", 
        RowBox[{"f", ",", "h"}], "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"s1", ",", "s2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"s1", ",", "s2"}], "}"}]}], "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.592293344680974*^9, 3.592293492662834*^9}, {
  3.592312753687189*^9, 3.5923127859948463`*^9}, {3.592741600554415*^9, 
  3.592741602722819*^9}, {3.612073960315897*^9, 3.612074035164829*^9}, {
  3.614861860330341*^9, 3.6148618603459406`*^9}, {3.614917792138962*^9, 
  3.614917792138962*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SuperscriptBox["h", "\[Prime]",
     MultilineFunction->None], "[", "u", "]"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       SuperscriptBox["s2", "2"]}], " ", 
      RowBox[{"f", "[", "u", "]"}]}], "+", 
     RowBox[{
      SuperscriptBox["s1", "2"], " ", 
      RowBox[{
       SuperscriptBox["f", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "u", "]"}]}]}], ")"}]}], "-", 
  RowBox[{
   SuperscriptBox["s1", "2"], " ", 
   RowBox[{
    SuperscriptBox["f", "\[Prime]",
     MultilineFunction->None], "[", "u", "]"}], " ", 
   RowBox[{
    SuperscriptBox["h", "\[Prime]\[Prime]",
     MultilineFunction->None], "[", "u", "]"}]}]}]], "Output",
 CellChangeTimes->{3.629187187007352*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Derivative", "[", "1", "]"}], "[", "f", "]"}], "[", "u", 
         "]"}], "^", "2"}], " ", "+", " ", "\n", "    ", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Derivative", "[", "1", "]"}], "[", "h", "]"}], "[", "u", 
         "]"}], "^", "2"}]}], " ", "==", " ", "1"}], ")"}], "&&", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"f", "[", "u", "]"}], ">", "0"}], ")"}]}], ",", " ", "\n", "  ", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"cf", "[", 
       RowBox[{"1", ",", " ", "1"}], "]"}], "[", 
      RowBox[{"sfrev", "[", 
       RowBox[{"f", ",", " ", "h"}], "]"}], "]"}], "[", "\n", "    ", 
     RowBox[{"u", ",", " ", "v"}], "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.592293344680974*^9, 3.592293492662834*^9}, {
   3.592293728987649*^9, 3.592293822790614*^9}, {3.5927416150780406`*^9, 
   3.5927416171528444`*^9}, {3.612074104850151*^9, 3.6120741149901686`*^9}, {
   3.6148618603459406`*^9, 3.6148618603459406`*^9}, {3.6149177921545615`*^9, 
   3.6149177921545615`*^9}, 3.617972391888789*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SuperscriptBox["h", "\[Prime]",
     MultilineFunction->None], "[", "u", "]"}], " ", 
   RowBox[{
    SuperscriptBox["f", "\[Prime]\[Prime]",
     MultilineFunction->None], "[", "u", "]"}]}], "-", 
  RowBox[{
   RowBox[{
    SuperscriptBox["f", "\[Prime]",
     MultilineFunction->None], "[", "u", "]"}], " ", 
   RowBox[{
    SuperscriptBox["h", "\[Prime]\[Prime]",
     MultilineFunction->None], "[", "u", "]"}]}]}]], "Output",
 CellChangeTimes->{3.6291872146994*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Derivative", "[", "1", "]"}], "[", "f", "]"}], "[", "u", 
       "]"}], "^", "2"}], " ", "+", " ", "\n", "    ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Derivative", "[", "1", "]"}], "[", "h", "]"}], "[", "u", 
       "]"}], "^", "2"}]}], " ", "==", " ", "1"}], ",", " ", "\n", "  ", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"cf", "[", 
       RowBox[{"1", ",", " ", "2"}], "]"}], "[", 
      RowBox[{"sfrev", "[", 
       RowBox[{"f", ",", " ", "h"}], "]"}], "]"}], "[", "\n", "    ", 
     RowBox[{"u", ",", " ", "v"}], "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.592293344680974*^9, 3.592293492662834*^9}, {
   3.592293728987649*^9, 3.592293822790614*^9}, {3.592293870589098*^9, 
   3.59229388912193*^9}, {3.5927416267468615`*^9, 3.5927416281196637`*^9}, {
   3.614861860361541*^9, 3.614861860361541*^9}, {3.6149177921701612`*^9, 
   3.6149177921701612`*^9}, 3.6179723934888806`*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6291872289306254`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Derivative", "[", "1", "]"}], "[", "f", "]"}], "[", "u", 
         "]"}], "^", "2"}], " ", "+", " ", "\n", "    ", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Derivative", "[", "1", "]"}], "[", "h", "]"}], "[", "u", 
         "]"}], "^", "2"}]}], " ", "==", " ", "1"}], ")"}], "&&", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"f", "[", "u", "]"}], ">", "0"}], ")"}]}], ",", " ", "\n", "  ", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"cf", "[", 
       RowBox[{"2", ",", " ", "2"}], "]"}], "[", 
      RowBox[{"sfrev", "[", 
       RowBox[{"f", ",", " ", "h"}], "]"}], "]"}], "[", "\n", "    ", 
     RowBox[{"u", ",", " ", "v"}], "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.592293344680974*^9, 3.592293492662834*^9}, {
   3.592293728987649*^9, 3.592293822790614*^9}, {3.592293870589098*^9, 
   3.592293909916767*^9}, {3.592741633018072*^9, 3.5927416341256742`*^9}, {
   3.612074196281912*^9, 3.6120742080599327`*^9}, {3.614861860361541*^9, 
   3.614861860361541*^9}, {3.6149177921857615`*^9, 3.6149177921857615`*^9}, 
   3.61797239487296*^9}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{
    SuperscriptBox["h", "\[Prime]",
     MultilineFunction->None], "[", "u", "]"}], 
   RowBox[{"f", "[", "u", "]"}]]}]], "Output",
 CellChangeTimes->{3.6291872331582327`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Derivative", "[", "1", "]"}], "[", "f", "]"}], "[", "u", 
       "]"}], "^", "2"}], " ", "+", " ", "\n", "    ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Derivative", "[", "1", "]"}], "[", "h", "]"}], "[", "u", 
       "]"}], "^", "2"}]}], " ", "==", " ", "1"}], ",", " ", "\n", "  ", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"prcurvatures", "[", 
       RowBox[{"sfrev", "[", 
        RowBox[{"f", ",", " ", "h"}], "]"}], "]"}], "[", " ", 
      RowBox[{"u", ",", " ", "v"}], "]"}], ",", " ", 
     RowBox[{
      RowBox[{"f", "[", "u", "]"}], ">", "0"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.592293344680974*^9, 3.592293492662834*^9}, {
  3.592293728987649*^9, 3.592293822790614*^9}, {3.592293870589098*^9, 
  3.592293909916767*^9}, {3.5923129860183973`*^9, 3.592313012959645*^9}, {
  3.5923130719745483`*^9, 3.5923131018642006`*^9}, {3.592741640880486*^9, 
  3.5927416422688885`*^9}, {3.614861860377141*^9, 3.614861860377141*^9}, {
  3.6149177922013617`*^9, 3.6149177922013617`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["h", "\[Prime]",
       MultilineFunction->None], "[", "u", "]"}], 
     RowBox[{"f", "[", "u", "]"}]]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["h", "\[Prime]",
       MultilineFunction->None], "[", "u", "]"}], " ", 
     RowBox[{
      SuperscriptBox["f", "\[Prime]\[Prime]",
       MultilineFunction->None], "[", "u", "]"}]}], "-", 
    RowBox[{
     RowBox[{
      SuperscriptBox["f", "\[Prime]",
       MultilineFunction->None], "[", "u", "]"}], " ", 
     RowBox[{
      SuperscriptBox["h", "\[Prime]\[Prime]",
       MultilineFunction->None], "[", "u", "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6291872415666475`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"prcurvatures", "[", 
    RowBox[{"sfrev", "[", 
     RowBox[{"f", ",", " ", "h"}], "]"}], "]"}], "[", " ", 
   RowBox[{"u", ",", " ", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.592315825904586*^9, 3.592315836169404*^9}, 
   3.592315870785865*^9, 3.592741650505703*^9, 3.5988843282017965`*^9, {
   3.612074334747755*^9, 3.612074353998189*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["h", "\[Prime]",
       MultilineFunction->None], "[", "u", "]"}], 
     SqrtBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "u", "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["f", "\[Prime]",
            MultilineFunction->None], "[", "u", "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["h", "\[Prime]",
            MultilineFunction->None], "[", "u", "]"}], "2"]}], ")"}]}]]]}], 
   ",", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"f", "[", "u", "]"}], "3"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["h", "\[Prime]",
          MultilineFunction->None], "[", "u", "]"}], " ", 
        RowBox[{
         SuperscriptBox["f", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "u", "]"}]}], "-", 
       RowBox[{
        RowBox[{
         SuperscriptBox["f", "\[Prime]",
          MultilineFunction->None], "[", "u", "]"}], " ", 
        RowBox[{
         SuperscriptBox["h", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "u", "]"}]}]}], ")"}]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "u", "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["f", "\[Prime]",
            MultilineFunction->None], "[", "u", "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["h", "\[Prime]",
            MultilineFunction->None], "[", "u", "]"}], "2"]}], ")"}]}], ")"}], 
     RowBox[{"3", "/", "2"}]]]}], "}"}]], "Output",
 CellChangeTimes->{3.6291872625058846`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Derivative", "[", "1", "]"}], "[", "f", "]"}], "[", "u", 
       "]"}], "^", "2"}], " ", "+", " ", "\n", "    ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Derivative", "[", "1", "]"}], "[", "h", "]"}], "[", "u", 
       "]"}], "^", "2"}]}], " ", "==", " ", "1"}], ",", " ", "\n", "  ", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"gauss", "[", 
       RowBox[{"sfrev", "[", 
        RowBox[{"f", ",", " ", "h"}], "]"}], "]"}], "[", " ", 
      RowBox[{"u", ",", " ", "v"}], "]"}], ",", " ", 
     RowBox[{
      RowBox[{"f", "[", "u", "]"}], ">", "0"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5923159480528*^9, 3.5923159491604023`*^9}, {
  3.592741687072167*^9, 3.5927416925009766`*^9}, {3.614861860392741*^9, 
  3.614861860392741*^9}, {3.614917792216962*^9, 3.614917792216962*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["h", "\[Prime]",
     MultilineFunction->None], "[", "u", "]"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SuperscriptBox["h", "\[Prime]",
         MultilineFunction->None], "[", "u", "]"}]}], " ", 
      RowBox[{
       SuperscriptBox["f", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "u", "]"}]}], "+", 
     RowBox[{
      RowBox[{
       SuperscriptBox["f", "\[Prime]",
        MultilineFunction->None], "[", "u", "]"}], " ", 
      RowBox[{
       SuperscriptBox["h", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "u", "]"}]}]}], ")"}]}], 
  RowBox[{"f", "[", "u", "]"}]]], "Output",
 CellChangeTimes->{3.6291872698534975`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", " ", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", " ", "\[Element]", "Reals"}], ",", " ", 
     RowBox[{"v", "\[Element]", "Reals"}]}], "}"}], ",", "\n", "  ", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{"gauss", "[", 
      RowBox[{"sfrev", "[", 
       RowBox[{"Cos", ",", " ", "Sin"}], "]"}], "]"}], "[", " ", 
     RowBox[{"u", ",", " ", "v"}], "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5923159480528*^9, 3.5923159491604023`*^9}, {
   3.592741687072167*^9, 3.5927416925009766`*^9}, 3.5988845224279056`*^9, 
   3.5988845565158553`*^9, {3.598884601500428*^9, 3.598884656618581*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.62918727702951*^9}]
}, Closed]],

Cell[TextData[{
 "This is the curvature of the unit sphere; the singularities u = \
\[PlusMinus] \[Pi]/2 of the parameter representation (not the surface!) are \
neglected by ",
 StyleBox["Mathematica.",
  FontSlant->"Italic"]
}], "Text",
 CellChangeTimes->{{3.598884719488177*^9, 3.5988849026396523`*^9}, {
   3.5988849344074697`*^9, 3.5988849347754908`*^9}, 3.5988849765198784`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Derivative", "[", "1", "]"}], "[", "f", "]"}], "[", "u", 
       "]"}], "^", "2"}], " ", "+", " ", "\n", "    ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Derivative", "[", "1", "]"}], "[", "h", "]"}], "[", "u", 
       "]"}], "^", "2"}]}], " ", "==", " ", "1"}], ",", " ", "\n", "  ", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"meancurvature", "[", 
       RowBox[{"sfrev", "[", 
        RowBox[{"f", ",", " ", "h"}], "]"}], "]"}], "[", " ", 
      RowBox[{"u", ",", " ", "v"}], "]"}], ",", " ", 
     RowBox[{
      RowBox[{"f", "[", "u", "]"}], ">", "0"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5923159480528*^9, 3.5923159491604023`*^9}, {
   3.5923161606031733`*^9, 3.5923161764528017`*^9}, {3.592741702859395*^9, 
   3.5927417040449967`*^9}, {3.614861860392741*^9, 3.6148618604083414`*^9}, 
   3.6148631366153827`*^9, {3.614917792232562*^9, 3.614917792232562*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{
       SuperscriptBox["h", "\[Prime]",
        MultilineFunction->None], "[", "u", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        RowBox[{
         RowBox[{"f", "[", "u", "]"}], " ", 
         RowBox[{
          SuperscriptBox["f", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "u", "]"}]}]}], ")"}]}], 
     RowBox[{"f", "[", "u", "]"}]], "-", 
    RowBox[{
     RowBox[{
      SuperscriptBox["f", "\[Prime]",
       MultilineFunction->None], "[", "u", "]"}], " ", 
     RowBox[{
      SuperscriptBox["h", "\[Prime]\[Prime]",
       MultilineFunction->None], "[", "u", "]"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.629187298405548*^9}]
}, Closed]],

Cell[TextData[{
 StyleBox["The parameter curves u =const, resp. v= const. are the ",
  FontSlant->"Italic"],
 StyleBox["principal curves",
  FontSlant->"Italic",
  FontColor->RGBColor[0, 0, 1]],
 ". For u= const the normal surfaces are circular cones, or cylinders if the \
tangent to the meridian at u is parallel to the  z-axis, or planes if the \
tangent at u to the meridian is parallel to the x,y-plane. They are planes \
containing the z-axis for the curves v = const."
}], "Text",
 CellChangeTimes->{{3.5923166334400043`*^9, 3.5923167850410705`*^9}, {
   3.592316831903553*^9, 3.592316919622507*^9}, {3.592316964098185*^9, 
   3.592317059741953*^9}, {3.598885239647928*^9, 3.5988852870966425`*^9}, 
   3.6162356507017508`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Derivative", "[", "1", "]"}], "[", "f", "]"}], "[", "u", 
       "]"}], "^", "2"}], " ", "+", " ", "\n", "    ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Derivative", "[", "1", "]"}], "[", "h", "]"}], "[", "u", 
       "]"}], "^", "2"}]}], " ", "==", " ", "1"}], ",", " ", "\n", "  ", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "3", "]"}], "[", 
       RowBox[{"sfrev", "[", 
        RowBox[{"f", ",", " ", "h"}], "]"}], "]"}], "[", " ", 
      RowBox[{"u", ",", " ", "v"}], "]"}], ",", " ", 
     RowBox[{
      RowBox[{"f", "[", "u", "]"}], ">", "0"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5923159480528*^9, 3.5923159491604023`*^9}, {
  3.5923161606031733`*^9, 3.5923161764528017`*^9}, {3.5923175406751976`*^9, 
  3.592317559504431*^9}, {3.5927417169150195`*^9, 3.592741717944621*^9}, {
  3.6148618604083414`*^9, 3.614861860423941*^9}, {3.614917792248162*^9, 
  3.614917792248162*^9}, {3.6162357121814585`*^9, 3.6162357144434624`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     RowBox[{"Cos", "[", "v", "]"}]}], " ", 
    RowBox[{
     SuperscriptBox["h", "\[Prime]",
      MultilineFunction->None], "[", "u", "]"}]}], ",", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"Sin", "[", "v", "]"}]}], " ", 
    RowBox[{
     SuperscriptBox["h", "\[Prime]",
      MultilineFunction->None], "[", "u", "]"}]}], ",", 
   RowBox[{
    SuperscriptBox["f", "\[Prime]",
     MultilineFunction->None], "[", "u", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.629187314820777*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"sfrev", "[", 
   RowBox[{"f", ",", " ", "h"}], "]"}], "[", " ", 
  RowBox[{"u", ",", " ", "v"}], "]"}]], "Input",
 CellChangeTimes->{3.616235837855279*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", "v", "]"}], " ", 
    RowBox[{"f", "[", "u", "]"}]}], ",", 
   RowBox[{
    RowBox[{"f", "[", "u", "]"}], " ", 
    RowBox[{"Sin", "[", "v", "]"}]}], ",", 
   RowBox[{"h", "[", "u", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6291873385484185`*^9}]
}, Closed]],

Cell["\<\
A parameter representation of the meridian in the x,z-plane is\
\>", "Text",
 CellChangeTimes->{{3.592318386851884*^9, 3.592318411250327*^9}, 
   3.5923192904366713`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"xx", "[", "u_", "]"}], " ", ":=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"f", "[", "u", "]"}], ",", 
     RowBox[{"h", "[", "u", "]"}]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5923177760796113`*^9, 3.592317802069257*^9}, 
   3.592741726399836*^9}],

Cell["\<\
We see that the normal vector of the surface coincides with the normal vector \
of the meridian. For the curvature k[u] of xx[u] we have\
\>", "Text",
 CellChangeTimes->{{3.592318436849972*^9, 3.5923184469743896`*^9}, {
   3.59231853820335*^9, 3.592318550979772*^9}, {3.592319354537184*^9, 
   3.592319416235292*^9}, 3.598454226766137*^9}],

Cell[BoxData[
 RowBox[{"?", "curvatures"}]], "Input",
 CellChangeTimes->{{3.5923178150952797`*^9, 3.59231782085169*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Derivative", "[", "1", "]"}], "[", "f", "]"}], "[", "u", 
       "]"}], "^", "2"}], " ", "+", " ", "\n", "    ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Derivative", "[", "1", "]"}], "[", "h", "]"}], "[", "u", 
       "]"}], "^", "2"}]}], " ", "==", " ", "1"}], ",", " ", "\n", "  ", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{"curvatures", "[", "xx", "]"}], "[", "u", "]"}], 
    "\[IndentingNewLine]", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5923159480528*^9, 3.5923159491604023`*^9}, {
   3.5923161606031733`*^9, 3.5923161764528017`*^9}, {3.5923175406751976`*^9, 
   3.592317559504431*^9}, {3.5923177112926974`*^9, 3.592317725878723*^9}, {
   3.5923178594149575`*^9, 3.592317965136343*^9}, 3.5927417433258657`*^9, {
   3.614861860423941*^9, 3.614861860439541*^9}, {3.614917792263762*^9, 
   3.614917792263762*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     RowBox[{
      SuperscriptBox["h", "\[Prime]",
       MultilineFunction->None], "[", "u", "]"}]}], " ", 
    RowBox[{
     SuperscriptBox["f", "\[Prime]\[Prime]",
      MultilineFunction->None], "[", "u", "]"}]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["f", "\[Prime]",
      MultilineFunction->None], "[", "u", "]"}], " ", 
    RowBox[{
     SuperscriptBox["h", "\[Prime]\[Prime]",
      MultilineFunction->None], "[", "u", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6291873723576784`*^9}]
}, Closed]],

Cell["\<\
It follows for the  principal curvature: k2 [u,v] == -k[u]. To obtain a \
geometric interpretation of the  principal curvature k1[u] = -h\
\[CloseCurlyQuote](u)/f(u) we set\
\>", "Text",
 CellChangeTimes->{{3.592318462059616*^9, 3.5923185135553064`*^9}, {
   3.5923197207946267`*^9, 3.592319804972375*^9}, {3.592457358429017*^9, 
   3.5924573688498354`*^9}, 3.5924574456175704`*^9, 3.592457476022024*^9, {
   3.59245750784608*^9, 3.592457516067294*^9}, 3.592741757755891*^9, 
   3.5988856176475487`*^9, {3.598885840976322*^9, 3.5988858679358644`*^9}, 
   3.6120752354153366`*^9, {3.612075287004627*^9, 3.612075303540656*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"f", "'"}], ",", 
    RowBox[{"h", "'"}]}], "}"}], " ", "\[Equal]", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Cos", "[", "\[Alpha]", "]"}], ",", " ", 
    RowBox[{"Sin", "[", "\[Alpha]", "]"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.5923198184663982`*^9, 3.5923198916461267`*^9}, 
   3.5923199762294755`*^9, 3.5923201472057757`*^9, {3.5923201953474607`*^9, 
   3.592320207624682*^9}, 3.592741759237894*^9, {3.5984543124800396`*^9, 
   3.598454319863462*^9}}],

Cell["\<\
where \[Alpha] is the intersection angle of the tangent at xx[u] with the \
x-axis. For the normal curvature radius r1 = 1/k1 we obtain 
r1 = -f[u]/ Sin[\[Alpha]], and this equals the length of the normal segment \
from the point xx[u] to the z-axis with  appropriate sign.\
\>", "Text",
 CellChangeTimes->{{3.592319980987484*^9, 3.592320109001309*^9}, {
   3.5923201533209867`*^9, 3.5923201626030025`*^9}, {3.5923202265163155`*^9, 
   3.5923202504623575`*^9}, {3.5923203898999653`*^9, 3.5923204463161917`*^9}, 
   3.592320613900777*^9, {3.5924575738653955`*^9, 3.5924575840366135`*^9}, 
   3.5942749734884195`*^9, {3.612075640095648*^9, 3.6120756606876836`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"gauss", "[", 
   RowBox[{"sfrev", "[", 
    RowBox[{"g", ",", "h"}], "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6126851006025085`*^9, 3.612685155985676*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"g", "[", "u", "]"}], " ", 
     RowBox[{
      SuperscriptBox["h", "\[Prime]",
       MultilineFunction->None], "[", "u", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["h", "\[Prime]",
          MultilineFunction->None], "[", "u", "]"}], " ", 
        RowBox[{
         SuperscriptBox["g", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "u", "]"}]}], "-", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", "\[Prime]",
          MultilineFunction->None], "[", "u", "]"}], " ", 
        RowBox[{
         SuperscriptBox["h", "\[Prime]\[Prime]",
          MultilineFunction->None], "[", "u", "]"}]}]}], ")"}]}], ")"}], "/", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["g", "\[Prime]",
          MultilineFunction->None], "[", "u", "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["h", "\[Prime]",
          MultilineFunction->None], "[", "u", "]"}], "2"]}], ")"}], " ", 
     SqrtBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"g", "[", "u", "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["g", "\[Prime]",
            MultilineFunction->None], "[", "u", "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["h", "\[Prime]",
            MultilineFunction->None], "[", "u", "]"}], "2"]}], ")"}]}]], " ", 
     SqrtBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"g", "[", "u", "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"Cos", "[", "v", "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"Sin", "[", "v", "]"}], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", "v", "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox["g", "\[Prime]",
             MultilineFunction->None], "[", "u", "]"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Sin", "[", "v", "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox["g", "\[Prime]",
             MultilineFunction->None], "[", "u", "]"}], "2"]}], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox["h", "\[Prime]",
            MultilineFunction->None], "[", "u", "]"}], "2"]}], ")"}]}]]}], 
    ")"}]}]}]], "Output",
 CellChangeTimes->{3.6291876917604413`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"g", "[", "u", "]"}], ">", "0"}], ")"}], "&&", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["g", "\[Prime]",
          MultilineFunction->None], "[", "u", "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["h", "\[Prime]",
          MultilineFunction->None], "[", "u", "]"}], "2"]}], "\[Equal]", 
      "1"}], ")"}], "&&", 
    RowBox[{"(", 
     RowBox[{"v", "\[Element]", " ", "Reals"}], ")"}]}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{"gauss", "[", 
      RowBox[{"sfrev", "[", 
       RowBox[{"g", ",", "h"}], "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6126853156718097`*^9, 3.612685489498752*^9}, 
   3.6126855397236247`*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["h", "\[Prime]",
     MultilineFunction->None], "[", "u", "]"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SuperscriptBox["h", "\[Prime]",
         MultilineFunction->None], "[", "u", "]"}]}], " ", 
      RowBox[{
       SuperscriptBox["g", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "u", "]"}]}], "+", 
     RowBox[{
      RowBox[{
       SuperscriptBox["g", "\[Prime]",
        MultilineFunction->None], "[", "u", "]"}], " ", 
      RowBox[{
       SuperscriptBox["h", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "u", "]"}]}]}], ")"}]}], 
  RowBox[{"g", "[", "u", "]"}]]], "Output",
 CellChangeTimes->{3.6291876965652494`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"g", "[", "u", "]"}], ">", "0"}], ")"}], "&&", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["g", "\[Prime]",
          MultilineFunction->None], "[", "u", "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["h", "\[Prime]",
          MultilineFunction->None], "[", "u", "]"}], "2"]}], "\[Equal]", 
      "1"}], ")"}], "&&", 
    RowBox[{"(", 
     RowBox[{"v", "\[Element]", " ", "Reals"}], ")"}]}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{"meancurvature", "[", 
      RowBox[{"sfrev", "[", 
       RowBox[{"g", ",", "h"}], "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6126853156718097`*^9, 3.612685489498752*^9}, 
   3.6126855397236247`*^9, {3.612685581324004*^9, 3.6126856003970947`*^9}, 
   3.614861860439541*^9, 3.6148631366309824`*^9}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{
       SuperscriptBox["h", "\[Prime]",
        MultilineFunction->None], "[", "u", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        RowBox[{
         RowBox[{"g", "[", "u", "]"}], " ", 
         RowBox[{
          SuperscriptBox["g", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "u", "]"}]}]}], ")"}]}], 
     RowBox[{"g", "[", "u", "]"}]], "-", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", "\[Prime]",
       MultilineFunction->None], "[", "u", "]"}], " ", 
     RowBox[{
      SuperscriptBox["h", "\[Prime]\[Prime]",
       MultilineFunction->None], "[", "u", "]"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.629187705894066*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["3.9.2. Minimal Surfaces of Revolution", "Subsubsection",
 CellChangeTimes->{{3.592460039870927*^9, 3.592460088511812*^9}, 
   3.612075971237429*^9, 3.6120777693766346`*^9, 3.6162363473209743`*^9, 
   3.616249192488927*^9, 3.6280853246868057`*^9, 3.629187754059351*^9, 
   3.6291884415088315`*^9}],

Cell[TextData[{
 "Now we ask: ",
 StyleBox["Do there exist minimal surfaces of revolution? ",
  FontColor->RGBColor[0, 0, 1]],
 " Locally we always can assume that the surface is obtained  rotating the \
graph f[z] of a function of the z coordinate."
}], "Text",
 CellChangeTimes->{{3.5923922154218073`*^9, 3.592392309933213*^9}, {
   3.5923923561738577`*^9, 3.592392418821441*^9}, {3.5923924792458973`*^9, 
   3.592392484629205*^9}, {3.5924037899994526`*^9, 3.5924038106956367`*^9}, {
   3.5924038678159037`*^9, 3.5924039006877837`*^9}, 3.616236447972351*^9},
 CellTags->"sfrev"],

Cell[BoxData[
 RowBox[{"??", "id"}]], "Input",
 CellChangeTimes->{{3.614336919250351*^9, 3.6143369213794727`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"sfr", "[", "f_", "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"sfrev", "[", 
    RowBox[{"f", ",", "id"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.592373484017357*^9, 3.5923736033263664`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sfr", "[", "f", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{3.6143369034754486`*^9}],

Cell["For the mean curvature we calculate", "Text",
 CellChangeTimes->{{3.5924039232400737`*^9, 3.592403949191558*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"meancurvature", "[", 
   RowBox[{"sfr", "[", "f", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5923736246204042`*^9, 3.5923736590028644`*^9}, 
   3.614861860455141*^9, 3.6148631366465826`*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Together", "[", "%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5923737127917585`*^9, 3.5923737360513997`*^9}}],

Cell["\<\
To find the functions f for which the surface is minimal we have to solve the \
differential equation\
\>", "Text",
 CellChangeTimes->{{3.5924039805513515`*^9, 3.5924040254469194`*^9}}],

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "1"}], "-", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["f", "\[Prime]",
         MultilineFunction->None], "[", "u", "]"}], "2"], "+", 
      RowBox[{
       RowBox[{"f", "[", "u", "]"}], " ", 
       RowBox[{
        SuperscriptBox["f", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "u", "]"}]}]}], "\[Equal]", " ", 
     "0"}], "}"}], ",", " ", 
   RowBox[{"f", "[", "u", "]"}], ",", "u"}], "]"}]], "Input",
 CellChangeTimes->{{3.59237394119176*^9, 3.5923739467141695`*^9}, 
   3.592374104087246*^9, {3.592374137658505*^9, 3.5923741653017535`*^9}, {
   3.5923744127337885`*^9, 3.5923746021493206`*^9}, 3.5923746475922008`*^9, {
   3.5923746813506603`*^9, 3.5923747107879114`*^9}, 3.5923748965530376`*^9, 
   3.614861860455141*^9, 3.614864659895658*^9}],

Cell["The two solutions are", "Text",
 CellChangeTimes->{{3.5924040563586874`*^9, 3.5924040733436594`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"f1", "[", "u_", "]"}], " ", "=", " ", 
   RowBox[{
    RowBox[{"f", "[", "u", "]"}], "/.", 
    RowBox[{"%", "[", 
     RowBox[{"[", "1", "]"}], "]"}]}]}], ";", 
  RowBox[{
   RowBox[{"f2", "[", "u_", "]"}], " ", "=", " ", 
   RowBox[{
    RowBox[{"f", "[", "u", "]"}], "/.", 
    RowBox[{"%", "[", 
     RowBox[{"[", "2", "]"}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.5923749685003643`*^9, 3.592375012102441*^9}, {
  3.592375074783351*^9, 3.592375088948176*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"fcc", "[", "u_", "]"}], "=", " ", 
  RowBox[{
   RowBox[{"f1", "[", "u", "]"}], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"C", "[", "1", "]"}], "\[Rule]", "0"}], ",", 
     RowBox[{
      RowBox[{"C", "[", "2", "]"}], "\[Rule]", "0"}]}], "}"}]}]}]], "Input",
 CellChangeTimes->{{3.5923757214668865`*^9, 3.59237579740782*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", "\[Equal]", 
   RowBox[{"Cosh", "[", "u", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.592375859355529*^9, 3.592375867186743*^9}, {
  3.5924577623917265`*^9, 3.5924577724225445`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"fcd", "[", "u_", "]"}], "=", " ", 
  RowBox[{
   RowBox[{"f2", "[", "u", "]"}], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"C", "[", "1", "]"}], "\[Rule]", "0"}], ",", 
     RowBox[{
      RowBox[{"C", "[", "2", "]"}], "\[Rule]", "0"}]}], "}"}]}]}]], "Input",
 CellChangeTimes->{{3.5923759386972685`*^9, 3.592375946450482*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"fcc", "[", "u", "]"}], "\[Equal]", 
   RowBox[{"fcd", "[", "u", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5923759852009497`*^9, 3.592376005075385*^9}}],

Cell[TextData[{
 "The curve y = a*Cosh[u/a+c] is named a ",
 ButtonBox["catenary",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://mathworld.wolfram.com/Catenary.html"], None},
  ButtonNote->"http://mathworld.wolfram.com/Catenary.html"],
 ", and the corresponding surface of revolution is a ",
 StyleBox["catenoid",
  FontColor->RGBColor[0, 0, 1]],
 "."
}], "Text",
 CellChangeTimes->{{3.592403369800419*^9, 3.592403375974772*^9}, {
   3.592403409599695*^9, 3.5924034140069475`*^9}, 3.59240355794518*^9, {
   3.592403598175481*^9, 3.5924036565918226`*^9}, 3.592403704551565*^9, {
   3.592404230990676*^9, 3.5924042309966764`*^9}, 3.592404299751609*^9}],

Cell[BoxData[
 RowBox[{"?", "catenoid"}]], "Input",
 CellChangeTimes->{{3.61623708531147*^9, 3.6162371066523075`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"catenoid", "[", "1", "]"}], "[", 
  RowBox[{"0", ",", " ", 
   RowBox[{"3", "*", 
    RowBox[{"Pi", "/", "2"}]}], ",", 
   RowBox[{"-", "1.5"}], ",", "1.5"}], "]"}]], "Input",
 CellChangeTimes->{{3.6162371338275557`*^9, 3.61623721036129*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sfr", "[", "Cosh", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.592376018163808*^9, 3.592376106974764*^9}}],

Cell["All the catenary functions solve the differential equation :", "Text",
 CellChangeTimes->{{3.592406210985925*^9, 3.5924062689602413`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fch", "[", 
    RowBox[{"a_", ",", "c_"}], "]"}], "[", "u_", "]"}], " ", "=", " ", 
  RowBox[{"a", "*", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     RowBox[{"u", "/", "a"}], "+", "c"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5924063384082136`*^9, 3.592406369394986*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "1"}], "-", " ", 
    RowBox[{
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"fch", "[", 
         RowBox[{"a", ",", "c"}], "]"}], "[", "u", "]"}], ",", "u"}], "]"}], 
     "^", "2"}], " ", "+", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"fch", "[", 
       RowBox[{"a", ",", "c"}], "]"}], "[", "u", "]"}], "*", " ", 
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"fch", "[", 
         RowBox[{"a", ",", "c"}], "]"}], "[", "u", "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"u", ",", "2"}], "}"}]}], "]"}]}]}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", ">", "0"}], ",", 
     RowBox[{"a", ">", "0"}], ",", 
     RowBox[{"c", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5924058117980933`*^9, 3.5924058226167116`*^9}, {
  3.5924059291928077`*^9, 3.5924061164775195`*^9}, {3.592406313278776*^9, 
  3.5924063256014805`*^9}, {3.592406455974938*^9, 3.5924065108300753`*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"meancurvature", "[", 
     RowBox[{"sfr", "[", 
      RowBox[{"fch", "[", 
       RowBox[{"a", ",", "c"}], "]"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", ">", "0"}], ",", 
     RowBox[{"c", "\[Element]", " ", "Reals"}], ",", 
     RowBox[{"u", "\[Element]", " ", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5923761453976316`*^9, 3.5923762008245287`*^9}, {
   3.592406400466763*^9, 3.592406406930133*^9}, {3.6120767595224133`*^9, 
   3.6120767901920676`*^9}, {3.612076855306582*^9, 3.612076902091064*^9}, 
   3.614861860501941*^9, 3.6148631366465826`*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"gauss", "[", 
     RowBox[{"sfr", "[", 
      RowBox[{"fch", "[", 
       RowBox[{"a", ",", "c"}], "]"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", ">", "0"}], ",", 
     RowBox[{"c", "\[Element]", " ", "Reals"}], ",", 
     RowBox[{"u", "\[Element]", " ", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5923761453976316`*^9, 3.5923762008245287`*^9}, {
   3.5923763196031375`*^9, 3.5923763375119686`*^9}, 3.5924065540515475`*^9, {
   3.6120769821348047`*^9, 3.6120769963620296`*^9}}],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"%", "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"c", "\[Rule]", "0"}], ",", 
      RowBox[{"a", "\[Rule]", "1"}]}], "}"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "5"}], ",", "5"}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.592376419677313*^9, 3.592376439489348*^9}, {
  3.592376495290646*^9, 3.592376538362322*^9}, {3.5924066466428432`*^9, 
  3.592406682546897*^9}}],

Cell["\<\
Now we consider the first solution family depending on the integration \
constants C[1], C[2]\
\>", "Text",
 CellChangeTimes->{{3.5924067279284925`*^9, 3.5924067953433485`*^9}, 
   3.6162455668285513`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fc", "[", 
    RowBox[{"c1_", ",", "c2_"}], "]"}], "[", "u_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"f1", "[", "u", "]"}], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"C", "[", "1", "]"}], "\[Rule]", "c1"}], ",", 
     RowBox[{
      RowBox[{"C", "[", "2", "]"}], "\[Rule]", "c2"}]}], "}"}]}]}]], "Input",
 CellChangeTimes->{{3.5923766072208424`*^9, 3.5923766754709625`*^9}, {
  3.592376706561817*^9, 3.592376714455431*^9}, {3.5923767748899374`*^9, 
  3.5923767911919656`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"fc", "[", 
   RowBox[{"0", ",", "0"}], "]"}], "[", "u", "]"}]], "Input",
 CellChangeTimes->{{3.5923767197594404`*^9, 3.592376729462657*^9}}],

Cell["\<\
We want to show that all solutions are catenoids; for c1=c2=0 we have fc[0,0] \
= Cosh[u]. Insert arbitrary values for c1,c2 in the next cell and plot the \
corresponding surfaces:\
\>", "Text",
 CellChangeTimes->{{3.5924044154162245`*^9, 3.592404454479459*^9}, {
   3.592404494679758*^9, 3.5924046417831717`*^9}, 3.592406810392209*^9, {
   3.6186381829491167`*^9, 3.6186382067574778`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sfr", "[", 
     RowBox[{"fc", "[", 
      RowBox[{"1", ",", ".1"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.592376018163808*^9, 3.592376106974764*^9}, {
   3.5923768369936457`*^9, 3.592376917848588*^9}, 3.592376962090266*^9, {
   3.592381577598248*^9, 3.592381672414671*^9}, {3.5923817308470135`*^9, 
   3.5923817497980976`*^9}, {3.592381785999168*^9, 3.59238179215052*^9}, {
   3.592381892599265*^9, 3.592381938686901*^9}}],

Cell["All these surfaces are minimal:", "Text",
 CellChangeTimes->{{3.59240467654416*^9, 3.592404687135766*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Together", "[", 
   RowBox[{
    RowBox[{"meancurvature", "[", 
     RowBox[{"sfr", "[", 
      RowBox[{"fc", "[", 
       RowBox[{"c1", ",", "c2"}], "]"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{
  3.5923770400280027`*^9, {3.592402645102968*^9, 3.592402685762294*^9}, {
   3.6120771413018847`*^9, 3.6120771477446957`*^9}, 3.614861860517541*^9, 
   3.614863136662183*^9}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sfr", "[", "Cosh", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "5"}], ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.592376018163808*^9, 3.592376106974764*^9}, {
  3.5923778949095044`*^9, 3.592377901227515*^9}}],

Cell["", "Text"],

Cell["\<\
To show that any function fc[c1,c2][u] is a catenary we consider the function \
\>", "Text",
 CellChangeTimes->{{3.592406927479906*^9, 3.5924069740795717`*^9}, {
  3.5924070154639387`*^9, 3.592407048239814*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"ff", "[", "u", "]"}], " ", "=", " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"a", "*", 
     RowBox[{"Cosh", "[", 
      RowBox[{
       RowBox[{"u", "/", "a"}], "+", "c"}], "]"}]}], " ", "-", " ", 
    RowBox[{
     RowBox[{"fc", "[", 
      RowBox[{"c1", ",", "c2"}], "]"}], "[", "u", "]"}]}], ")"}]}]], "Input",
 CellChangeTimes->{{3.5924070587674155`*^9, 3.5924070657398143`*^9}}],

Cell["\<\
and solve the necessary condition expressed in the equations  {ff[0] == 0,ff\
\[CloseCurlyQuote][0] == 0} for the unknowns a>0, c. If such a solution exist \
then the function fc[c1,c2] [u] equals fch[a,c][u], since both function solve \
the differental equation with the same integration constants c1,c2. \
\>", "Text",
 CellChangeTimes->{{3.59240708559795*^9, 3.592407252647505*^9}, {
   3.5924072923117733`*^9, 3.592407416431873*^9}, 3.6162457001251755`*^9, 
   3.618638246605757*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"a", "*", 
     RowBox[{"Cosh", "[", 
      RowBox[{
       RowBox[{"u", "/", "a"}], "+", "c"}], "]"}]}], " ", "-", " ", 
    RowBox[{
     RowBox[{"fc", "[", 
      RowBox[{"c1", ",", "c2"}], "]"}], "[", "u", "]"}]}], ")"}], "/.", 
  RowBox[{"u", "\[Rule]", " ", "0"}]}]], "Input",
 CellChangeTimes->{3.592391432480026*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"c1", "\[Element]", "Reals"}], ",", 
     RowBox[{"c2", "\[Element]", "Reals"}], ",", 
     RowBox[{"a", "\[Element]", "Reals"}], ",", 
     RowBox[{"c", "\[Element]", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.592391170736055*^9, 3.5923913059287877`*^9}, {
  3.5923913602958975`*^9, 3.592391377799898*^9}, {3.5923914969207115`*^9, 
  3.5923915113365364`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"a", "*", 
       RowBox[{"Cosh", "[", 
        RowBox[{
         RowBox[{"u", "/", "a"}], "+", "c"}], "]"}]}], " ", "-", " ", 
      RowBox[{
       RowBox[{"fc", "[", 
        RowBox[{"c1", ",", "c2"}], "]"}], "[", "u", "]"}]}], ")"}], ",", 
    "u"}], "]"}], "/.", 
  RowBox[{"u", "\[Rule]", " ", "0"}]}]], "Input",
 CellChangeTimes->{{3.5923910346082687`*^9, 3.5923910392705355`*^9}, {
  3.5923910704973216`*^9, 3.5923911040812426`*^9}, {3.5923911540981035`*^9, 
  3.592391157953324*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"c1", "\[Element]", "Reals"}], ",", 
     RowBox[{"c", "\[Element]", "Reals"}], ",", 
     RowBox[{"c2", "\[Element]", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.592391170736055*^9, 3.5923913059287877`*^9}, {
  3.5923913602958975`*^9, 3.592391377799898*^9}}],

Cell[BoxData[
 RowBox[{"Reduce", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"a", " ", 
        RowBox[{"Cosh", "[", "c", "]"}]}], "+", 
       RowBox[{
        RowBox[{"Cosh", "[", 
         RowBox[{"c1", "-", 
          RowBox[{"c2", " ", 
           SuperscriptBox["\[ExponentialE]", "c1"]}]}], "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"Cosh", "[", "c1", "]"}]}], "+", 
          RowBox[{"Sinh", "[", "c1", "]"}]}], ")"}]}]}], "\[Equal]", "0"}], 
     ",", 
     RowBox[{
      RowBox[{
       RowBox[{"Sinh", "[", "c", "]"}], "+", 
       RowBox[{"Sinh", "[", 
        RowBox[{"c1", "-", 
         RowBox[{"c2", " ", 
          SuperscriptBox["\[ExponentialE]", "c1"]}]}], "]"}]}], "\[Equal]", 
      "0"}], ",", 
     RowBox[{"a", ">", "0"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "c"}], "}"}], ",", " ", "Reals"}], "]"}]], "Input",
 CellChangeTimes->{{3.5923790170437927`*^9, 3.5923791509614525`*^9}, {
   3.5923822007118883`*^9, 3.5923822065432215`*^9}, {3.5923905062420483`*^9, 
   3.5923905404490047`*^9}, {3.592391583304652*^9, 3.5923915990655537`*^9}, {
   3.5923916412429667`*^9, 3.5923916766269903`*^9}, {3.592404722684799*^9, 
   3.592404773292694*^9}, 3.614861860533141*^9, 3.614917792341762*^9}],

Cell["\<\
For any real values c1,  c2  only the second case is possible and we see that \
for any such pair we find  numbers a, c  with fc[c1,c2][u] == a*Cosh[u/a + c].\
\>", "Text",
 CellChangeTimes->{{3.5924050900318103`*^9, 3.592405155695566*^9}, {
   3.592405412983282*^9, 3.592405454815675*^9}, {3.592405556992519*^9, 
   3.592405591679503*^9}, {3.5924056395522413`*^9, 3.5924058092159452`*^9}, 
   3.5924074485357094`*^9, 3.592459559078082*^9, {3.612077504922923*^9, 
   3.612077507044527*^9}}],

Cell["\<\
Now we show with the same argumentation that also the second solution f2[u]  \
is a catenary for all values of c1,c2:\
\>", "Text",
 CellChangeTimes->{{3.5924595963153477`*^9, 3.5924596781998916`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fd", "[", 
    RowBox[{"c1_", ",", "c2_"}], "]"}], "[", "u_", "]"}], " ", "=", " ", 
  RowBox[{
   RowBox[{"f2", "[", "u", "]"}], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"C", "[", "1", "]"}], "\[Rule]", "c1"}], ",", 
     RowBox[{
      RowBox[{"C", "[", "2", "]"}], "\[Rule]", "c2"}]}], "}"}]}]}]], "Input",
 CellChangeTimes->{{3.5923766072208424`*^9, 3.5923766754709625`*^9}, {
  3.592376706561817*^9, 3.592376714455431*^9}, {3.5923767748899374`*^9, 
  3.5923767911919656`*^9}, {3.5924580355170064`*^9, 3.592458056904644*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"fd", "[", 
   RowBox[{"0", ",", "0"}], "]"}], "[", "u", "]"}]], "Input",
 CellChangeTimes->{{3.5923767197594404`*^9, 3.592376729462657*^9}, 
   3.5924580763734784`*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Together", "[", 
   RowBox[{
    RowBox[{"meancurvature", "[", 
     RowBox[{"sfr", "[", 
      RowBox[{"fd", "[", 
       RowBox[{"c1", ",", "c2"}], "]"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{
  3.5923770400280027`*^9, {3.592402645102968*^9, 3.592402685762294*^9}, 
   3.592458198490493*^9, {3.6120775588834176`*^9, 3.612077567744233*^9}, 
   3.6148618605487413`*^9, 3.614863136677783*^9}],

Cell["\<\
To show that any function fd[c1,c2][u] is a catenary we consider the function \
\>", "Text",
 CellChangeTimes->{{3.592406927479906*^9, 3.5924069740795717`*^9}, {
   3.5924070154639387`*^9, 3.592407048239814*^9}, 3.592458346971554*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"ff", "[", "u_", "]"}], " ", ":=", " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"a", "*", 
     RowBox[{"Cosh", "[", 
      RowBox[{
       RowBox[{"u", "/", "a"}], "+", "c"}], "]"}]}], " ", "-", " ", 
    RowBox[{
     RowBox[{"fd", "[", 
      RowBox[{"c1", ",", "c2"}], "]"}], "[", "u", "]"}]}], ")"}]}]], "Input",
 CellChangeTimes->{{3.5924070587674155`*^9, 3.5924070657398143`*^9}, 
   3.592458381556814*^9, {3.616246614320464*^9, 3.616246623488989*^9}}],

Cell[BoxData[
 RowBox[{"ff", "[", "u", "]"}]], "Input",
 CellChangeTimes->{{3.6162466290293055`*^9, 3.616246632518505*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"a", "*", 
     RowBox[{"Cosh", "[", 
      RowBox[{
       RowBox[{"u", "/", "a"}], "+", "c"}], "]"}]}], " ", "-", " ", 
    RowBox[{
     RowBox[{"fd", "[", 
      RowBox[{"c1", ",", "c2"}], "]"}], "[", "u", "]"}]}], ")"}], "/.", 
  RowBox[{"u", "\[Rule]", " ", "0"}]}]], "Input",
 CellChangeTimes->{3.592391432480026*^9, 3.59245841888768*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"c1", "\[Element]", "Reals"}], ",", 
     RowBox[{"c2", "\[Element]", "Reals"}], ",", 
     RowBox[{"a", "\[Element]", "Reals"}], ",", 
     RowBox[{"c", "\[Element]", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.592391170736055*^9, 3.5923913059287877`*^9}, {
  3.5923913602958975`*^9, 3.592391377799898*^9}, {3.5923914969207115`*^9, 
  3.5923915113365364`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"a", "*", 
       RowBox[{"Cosh", "[", 
        RowBox[{
         RowBox[{"u", "/", "a"}], "+", "c"}], "]"}]}], " ", "-", " ", 
      RowBox[{
       RowBox[{"fd", "[", 
        RowBox[{"c1", ",", "c2"}], "]"}], "[", "u", "]"}]}], ")"}], ",", 
    "u"}], "]"}], "/.", 
  RowBox[{"u", "\[Rule]", " ", "0"}]}]], "Input",
 CellChangeTimes->{{3.5923910346082687`*^9, 3.5923910392705355`*^9}, {
   3.5923910704973216`*^9, 3.5923911040812426`*^9}, {3.5923911540981035`*^9, 
   3.592391157953324*^9}, 3.5924584526617393`*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"c1", "\[Element]", "Reals"}], ",", 
     RowBox[{"c", "\[Element]", "Reals"}], ",", 
     RowBox[{"c2", "\[Element]", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.592391170736055*^9, 3.5923913059287877`*^9}, {
  3.5923913602958975`*^9, 3.592391377799898*^9}}],

Cell[BoxData[
 RowBox[{"Reduce", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"a", " ", 
        RowBox[{"Cosh", "[", "c", "]"}]}], "+", 
       RowBox[{
        RowBox[{"Cosh", "[", 
         RowBox[{"c1", "+", 
          RowBox[{"c2", " ", 
           SuperscriptBox["\[ExponentialE]", "c1"]}]}], "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"Cosh", "[", "c1", "]"}]}], "+", 
          RowBox[{"Sinh", "[", "c1", "]"}]}], ")"}]}]}], "\[Equal]", "0"}], 
     ",", 
     RowBox[{
      RowBox[{
       RowBox[{"Sinh", "[", "c", "]"}], "-", 
       RowBox[{"Sinh", "[", 
        RowBox[{"c1", "+", 
         RowBox[{"c2", " ", 
          SuperscriptBox["\[ExponentialE]", "c1"]}]}], "]"}]}], "\[Equal]", 
      "0"}], ",", 
     RowBox[{"a", ">", "0"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "c"}], "}"}], ",", " ", "Reals"}], "]"}]], "Input",
 CellChangeTimes->{{3.5923790170437927`*^9, 3.5923791509614525`*^9}, {
   3.5923822007118883`*^9, 3.5923822065432215`*^9}, {3.5923905062420483`*^9, 
   3.5923905404490047`*^9}, {3.592391583304652*^9, 3.5923915990655537`*^9}, {
   3.5923916412429667`*^9, 3.5923916766269903`*^9}, {3.592404722684799*^9, 
   3.592404773292694*^9}, 3.5924585256698675`*^9, 3.592459458551506*^9, 
   3.6148618605643415`*^9, 3.614917792357362*^9}],

Cell["\<\
Only the second solution is possible. We test the equality of the functions  \
inserting the solution:\
\>", "Text",
 CellChangeTimes->{{3.616249053796994*^9, 3.616249127372203*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"ff", "[", "u", "]"}], "/.", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"a", "\[Rule]", 
       SqrtBox[
        FractionBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cosh", "[", 
            RowBox[{"c1", "+", 
             RowBox[{"c2", " ", 
              SuperscriptBox["\[ExponentialE]", "c1"]}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", 
              RowBox[{"Cosh", "[", "c1", "]"}]}], "+", 
             RowBox[{"Sinh", "[", "c1", "]"}]}], ")"}], "2"]}], 
         RowBox[{"1", "+", 
          SuperscriptBox[
           RowBox[{"Sinh", "[", 
            RowBox[{"c1", "+", 
             RowBox[{"c2", " ", 
              SuperscriptBox["\[ExponentialE]", "c1"]}]}], "]"}], "2"]}]]]}], 
      ",", " ", 
      RowBox[{"c", "->", 
       RowBox[{"Log", "[", 
        RowBox[{
         RowBox[{"Sinh", "[", 
          RowBox[{"c1", "+", 
           RowBox[{"c2", " ", 
            SuperscriptBox["\[ExponentialE]", "c1"]}]}], "]"}], "+", 
         SqrtBox[
          RowBox[{"1", "+", 
           SuperscriptBox[
            RowBox[{"Sinh", "[", 
             RowBox[{"c1", "+", 
              RowBox[{"c2", " ", 
               SuperscriptBox["\[ExponentialE]", "c1"]}]}], "]"}], "2"]}]]}], 
        "]"}]}]}], "}"}]}], ",", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"c1", "\[Element]", "Reals"}], ",", 
     RowBox[{"c2", "\[Element]", "Reals"}], ",", 
     RowBox[{"u", "\[Element]", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6162481074598675`*^9, 3.616248243639656*^9}, {
   3.6162483164488206`*^9, 3.6162483288805313`*^9}, 3.616248579127845*^9, 
   3.6162486162949705`*^9, {3.6162486790345592`*^9, 3.616248707883209*^9}, 
   3.61624882958317*^9, 3.6162489019123073`*^9}],

Cell[TextData[{
 "Thus we proved\n",
 StyleBox["Proposition",
  FontColor->RGBColor[0, 0, 1]],
 ". The only minimal surfaces of revolution are the catenoids, generated \
rotating catenaries.."
}], "Text",
 CellChangeTimes->{{3.5924597361539936`*^9, 3.592459803031311*^9}, {
  3.5924601472459154`*^9, 3.592460166199949*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["3.9.3. Surfaces of Revolution of Constant Gaussian Curvature", \
"Subsubsection",
 CellChangeTimes->{{3.5930088090318155`*^9, 3.593008813228223*^9}, {
   3.5934178586395583`*^9, 3.593417908497246*^9}, 3.6120777755369873`*^9, 
   3.612159908316609*^9, 3.61624922015751*^9, 3.616656611139438*^9, 
   3.6280853254044075`*^9}],

Cell["\<\
We assume that the surface of revolution is generated by the graph of a \
function f\
\>", "Text",
 CellChangeTimes->{{3.5934179940789967`*^9, 3.5934181363824463`*^9}, {
   3.593418242759033*^9, 3.593418244974237*^9}, 3.616301755428102*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sfrev", "[", 
   RowBox[{"id", ",", "f"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{3.5934182656442733`*^9}],

Cell["Its Gaussian curvature is", "Text",
 CellChangeTimes->{{3.59341829782713*^9, 3.5934183065475454`*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"gauss", "[", 
    RowBox[{"sfrev", "[", 
     RowBox[{"id", ",", "f"}], "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.593008824007842*^9, 3.59300892540802*^9}}],

Cell["\<\
Changing the order of the functions the Gaussian curvature is \
\>", "Text",
 CellChangeTimes->{{3.61647409352219*^9, 3.6164741247139745`*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"gauss", "[", 
    RowBox[{"sfrev", "[", 
     RowBox[{"f", ",", "id"}], "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6164741576198564`*^9, 3.6164741655013075`*^9}}],

Cell["\<\
If  f\.b4[u] = 0 then f[u] = u0 = const, and the surface is a plane\
\>", "Text",
 CellChangeTimes->{{3.616473593105568*^9, 3.616473629433646*^9}, {
   3.616473662185519*^9, 3.6164736992976418`*^9}, {3.616474248058029*^9, 
   3.6164742561944942`*^9}, 3.616487116010995*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fc", "[", "u_", "]"}], " ", "=", "1"}], ";", " ", 
  RowBox[{"ParametricPlot3D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"sfrev", "[", 
      RowBox[{"id", ",", "fc"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"u", ",", "0", ",", "2"}], "}"}], ",", " ", 
    RowBox[{"{", 
     RowBox[{"v", ",", "0", ",", 
      RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6164737205658584`*^9, 3.616473838821622*^9}}],

Cell["\<\
Changing the order of the arguments in sfrev, we obtain a circular cylinder:\
\>", "Text",
 CellChangeTimes->{{3.6164871579953957`*^9, 3.616487213555574*^9}, 
   3.6186383414936686`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fc", "[", "u_", "]"}], " ", "=", "1"}], ";", " ", 
  RowBox[{"ParametricPlot3D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"sfrev", "[", 
      RowBox[{"fc", ",", " ", "id"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"u", ",", "0", ",", "2"}], "}"}], ",", " ", 
    RowBox[{"{", 
     RowBox[{"v", ",", "0", ",", 
      RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6164737205658584`*^9, 3.616473838821622*^9}, {
  3.616474059494244*^9, 3.616474067365694*^9}}],

Cell[BoxData[
 RowBox[{"Clear", "[", "fc", "]"}]], "Input",
 CellChangeTimes->{{3.6164742984839134`*^9, 3.616474307804446*^9}}],

Cell["\<\
 If k =0, the differential equation in both cases reduces to f\.b4\.b4 [u] = \
0, which has the general solution\
\>", "Text",
 CellChangeTimes->{{3.616474377681443*^9, 3.6164744774261484`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"fl", "[", "u_", "]"}], " ", ":=", " ", 
  RowBox[{"u0", " ", "+", 
   RowBox[{"u", "*", "c0"}]}]}]], "Input",
 CellChangeTimes->{{3.6164744983583455`*^9, 3.616474559035816*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"gauss", "[", 
   RowBox[{"sfrev", "[", 
    RowBox[{"id", ",", "fl"}], "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.616474586925411*^9, 3.6164746220444202`*^9}}],

Cell["\<\
which yields  besides of the already mentioned developable surfaces  the \
circular cones; e.g. \
\>", "Text",
 CellChangeTimes->{{3.6164746434736457`*^9, 3.616474751481823*^9}, {
  3.6164872703158207`*^9, 3.6164873153713975`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fl", "[", "u_", "]"}], " ", "=", 
   RowBox[{"3", "*", "u"}]}], ";", " ", 
  RowBox[{"ParametricPlot3D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"sfrev", "[", 
      RowBox[{"id", ",", "fl"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"u", ",", 
      RowBox[{"-", "2"}], ",", "2"}], "}"}], ",", " ", 
    RowBox[{"{", 
     RowBox[{"v", ",", "0", ",", 
      RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6164737205658584`*^9, 3.616473838821622*^9}, {
  3.616474059494244*^9, 3.616474067365694*^9}, {3.6164747940862603`*^9, 
  3.616474876637982*^9}}],

Cell[BoxData[
 RowBox[{"Clear", "[", "fl", "]"}]], "Input",
 CellChangeTimes->{{3.6164873522455063`*^9, 3.6164873624790916`*^9}}],

Cell["Assuming k \[NotEqual]0 we define the curvature differential \
operator:", "Text",
 CellChangeTimes->{{3.616328440589955*^9, 3.6163284701016426`*^9}, {
   3.61632880241465*^9, 3.6163288045097694`*^9}, {3.616473145681977*^9, 
   3.6164732661538677`*^9}, 3.6164873978281136`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"diff", "[", 
    RowBox[{"f_", ",", "k_"}], "]"}], "[", "u_", "]"}], " ", ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "''"}], "[", "u", "]"}], "-", 
   RowBox[{"k", "*", " ", "u", "*", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{
         RowBox[{
          RowBox[{"f", "'"}], "[", "u", "]"}], "^", "2"}]}], ")"}], "^", 
      "2"}], "/", 
     RowBox[{
      RowBox[{"f", "'"}], "[", "u", "]"}]}]}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6163268268226523`*^9, 3.6163268268246527`*^9}, {
  3.616327879928887*^9, 3.616327923097356*^9}}],

Cell["\<\
Its kernel defines the meridians generating surfaces of revolution with \
constant curvature k:\
\>", "Text",
 CellChangeTimes->{{3.61632851358313*^9, 3.616328569550331*^9}, {
  3.616328825054945*^9, 3.6163288317103257`*^9}, {3.6164877106670074`*^9, 
  3.6164877260268855`*^9}}],

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"diff", "[", 
        RowBox[{"f", ",", "k"}], "]"}], "[", "u", "]"}], "\[Equal]", " ", 
      "0"}], ",", 
     RowBox[{
      RowBox[{"f", "[", "u0", "]"}], " ", "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"f", "'"}], "[", "u0", "]"}], "\[Equal]", " ", "c0"}]}], "}"}],
    ",", " ", 
   RowBox[{"f", "[", "u", "]"}], ",", "u"}], "]"}]], "Input",
 CellChangeTimes->{{3.593009068101471*^9, 3.5930091129671497`*^9}, {
   3.5930091433872027`*^9, 3.593009245723383*^9}, {3.593009299044276*^9, 
   3.593009307530691*^9}, 3.6148618605643415`*^9, 3.6148646599112577`*^9, 
   3.6163941993331203`*^9, {3.6163942501580095`*^9, 3.6163942809992633`*^9}, {
   3.616394314960523*^9, 3.6163943961430655`*^9}, {3.6164733180838375`*^9, 
   3.616473429445207*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"condsol", " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.6164896454916725`*^9, 3.6164896529330983`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fpos", "[", 
    RowBox[{"k_", ",", "c0_", ",", "u0_"}], "]"}], "[", "u_", "]"}], " ", "=", 
  RowBox[{
   RowBox[{"f", "[", "u", "]"}], "/.", 
   RowBox[{"condsol", "[", 
    RowBox[{"[", "2", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6163955471910877`*^9, 3.6163957105545745`*^9}, {
  3.6163957632826667`*^9, 3.6163958144663568`*^9}, {3.6164948205166836`*^9, 
  3.6164948762243814`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fneg", "[", 
    RowBox[{"k_", ",", "c0_", ",", "u0_"}], "]"}], "[", "u_", "]"}], " ", "=", 
  RowBox[{
   RowBox[{"f", "[", "u", "]"}], "/.", 
   RowBox[{"condsol", "[", 
    RowBox[{"[", "1", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6163955471910877`*^9, 3.6163957105545745`*^9}, {
   3.6163957632826667`*^9, 3.6163958144663568`*^9}, {3.616414076424296*^9, 
   3.616414086189913*^9}, {3.6164145646271534`*^9, 3.6164145816155834`*^9}, {
   3.6164948995776224`*^9, 3.6164949074868364`*^9}, 3.6164949413232956`*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"fpos", "[", 
     RowBox[{"k", ",", "c0", ",", "u0"}], "]"}], "[", "u", "]"}], ",", " ", 
   RowBox[{"c0", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616494960199329*^9, 3.6164950123970203`*^9}, 
   3.616495124764018*^9}],

Cell["We redefine", "Text",
 CellChangeTimes->{{3.616495492066663*^9, 3.616495498244274*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "fpos", "]"}], ";", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"fpos", "[", 
     RowBox[{"k_", ",", " ", "c0_", ",", " ", "u0_"}], "]"}], "[", "\n", 
    "    ", "u_", "]"}], " ", ":=", " ", "\n", "   ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"Sqrt", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"-", "k"}], ")"}], "*", 
          RowBox[{"u", "^", "2"}]}], " ", "-", " ", 
         RowBox[{
          RowBox[{"c0", "^", "2"}], "*", "\n", "         ", 
          RowBox[{"(", 
           RowBox[{"1", " ", "+", " ", 
            RowBox[{"k", "*", 
             RowBox[{"u", "^", "2"}]}]}], ")"}]}], " ", "+", " ", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", " ", "+", " ", 
            RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", "\n", 
          "         ", 
          RowBox[{"u0", "^", "2"}]}]}], "]"}], "*", 
       RowBox[{"Sqrt", "[", "\n", "       ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", " ", "-", " ", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1", " ", "+", " ", 
              RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
            RowBox[{"(", 
             RowBox[{"u", " ", "-", " ", "u0"}], ")"}], "*", "\n", 
            "          ", 
            RowBox[{"(", 
             RowBox[{"u", " ", "+", " ", "u0"}], ")"}]}]}], ")"}], "/", 
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1", " ", "+", " ", 
              RowBox[{"c0", "^", "2"}]}], ")"}], "*", "\n", "          ", "k",
             "*", 
            RowBox[{"u0", "^", "2"}]}]}], ")"}]}], "]"}], "*", 
       RowBox[{"EllipticE", "[", "\n", "       ", 
        RowBox[{
         RowBox[{"ArcSin", "[", 
          RowBox[{"u", "*", 
           RowBox[{"Sqrt", "[", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"1", " ", "+", " ", 
                 RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k"}], ")"}], "/", 
             "\n", "           ", 
             RowBox[{"(", 
              RowBox[{"1", " ", "+", " ", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"1", " ", "+", " ", 
                  RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", "\n", 
                "            ", 
                RowBox[{"u0", "^", "2"}]}]}], ")"}]}], "]"}]}], "]"}], ",", 
         " ", "\n", "       ", 
         RowBox[{"1", " ", "+", " ", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"1", " ", "+", " ", 
             RowBox[{"c0", "^", "2"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", 
              RowBox[{"c0", "^", "2"}]}], " ", "+", " ", "\n", "          ", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"1", " ", "+", " ", 
                RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
              RowBox[{"u0", "^", "2"}]}]}], ")"}]}]}]}], "]"}]}], " ", "+", 
      " ", "\n", "     ", 
      RowBox[{
       RowBox[{"Sqrt", "[", 
        RowBox[{"1", "/", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"c0", "^", "2"}], " ", "-", " ", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1", " ", "+", " ", 
              RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", "\n", 
            "          ", 
            RowBox[{"u0", "^", "2"}]}]}], ")"}]}], "]"}], "*", 
       RowBox[{"Sqrt", "[", "\n", "       ", 
        RowBox[{"1", "/", 
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1", " ", "+", " ", 
              RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
            RowBox[{"u0", "^", "2"}]}]}], ")"}]}], "]"}], "*", "\n", "      ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"c0", "^", "2"}]}], " ", "+", " ", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", " ", "+", " ", 
            RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
          RowBox[{"u0", "^", "2"}]}]}], ")"}], "*", "\n", "      ", 
       RowBox[{"Sqrt", "[", 
        RowBox[{
         RowBox[{"-", "1"}], " ", "+", " ", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", " ", "+", " ", 
            RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", "\n", 
          "         ", 
          RowBox[{"(", 
           RowBox[{"u", " ", "-", " ", "u0"}], ")"}], "*", 
          RowBox[{"(", 
           RowBox[{"u", " ", "+", " ", "u0"}], ")"}]}]}], "]"}], "*", "\n", 
       "      ", 
       RowBox[{"Sqrt", "[", 
        RowBox[{"1", " ", "-", " ", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1", " ", "+", " ", 
              RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
            RowBox[{"u", "^", "2"}]}], ")"}], "/", "\n", "         ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"c0", "^", "2"}]}], " ", "+", " ", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", " ", "+", " ", 
               RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", "\n", 
             "           ", 
             RowBox[{"u0", "^", "2"}]}]}], ")"}]}]}], "]"}], "*", 
       RowBox[{"EllipticE", "[", "\n", "       ", 
        RowBox[{
         RowBox[{"ArcSin", "[", 
          RowBox[{"u0", "*", 
           RowBox[{"Sqrt", "[", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"1", " ", "+", " ", 
                 RowBox[{"c0", "^", "2"}]}], ")"}], "*", "\n", "            ",
                "k"}], ")"}], "/", 
             RowBox[{"(", 
              RowBox[{"1", " ", "+", " ", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"1", " ", "+", " ", 
                  RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", "\n", 
                "            ", 
                RowBox[{"u0", "^", "2"}]}]}], ")"}]}], "]"}]}], "]"}], ",", 
         " ", "\n", "       ", 
         RowBox[{"1", " ", "+", " ", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"1", " ", "+", " ", 
             RowBox[{"c0", "^", "2"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", 
              RowBox[{"c0", "^", "2"}]}], " ", "+", " ", "\n", "          ", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"1", " ", "+", " ", 
                RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
              RowBox[{"u0", "^", "2"}]}]}], ")"}]}]}]}], "]"}]}]}], ")"}], 
    "/", "\n", "    ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Sqrt", "[", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", " ", "+", " ", 
            RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k"}], ")"}], "/", "\n", 
        "       ", 
        RowBox[{"(", 
         RowBox[{"1", " ", "+", " ", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"1", " ", "+", " ", 
             RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
           RowBox[{"u0", "^", "2"}]}]}], ")"}]}], "]"}], "*", "\n", "     ", 
      RowBox[{"Sqrt", "[", 
       RowBox[{
        RowBox[{"-", "1"}], " ", "+", " ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", 
           RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", "\n", "        ", 
         RowBox[{"(", 
          RowBox[{"u", " ", "-", " ", "u0"}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"u", " ", "+", " ", "u0"}], ")"}]}]}], "]"}], "*", "\n", 
      "     ", 
      RowBox[{"Sqrt", "[", 
       RowBox[{"1", " ", "-", " ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"1", " ", "+", " ", 
             RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
           RowBox[{"u", "^", "2"}]}], ")"}], "/", "\n", "        ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"c0", "^", "2"}]}], " ", "+", " ", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1", " ", "+", " ", 
              RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", "\n", 
            "          ", 
            RowBox[{"u0", "^", "2"}]}]}], ")"}]}]}], "]"}]}], 
     ")"}]}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.616495509850694*^9, 3.616495528352327*^9}, {
  3.616495560410383*^9, 3.6164956132788763`*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"fneg", "[", 
     RowBox[{"k", ",", "c0", ",", "u0"}], "]"}], "[", "u", "]"}], ",", " ", 
   RowBox[{"c0", "<", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616494960199329*^9, 3.6164950123970203`*^9}, 
   3.616495124764018*^9, {3.6164956739785824`*^9, 3.6164956824805975`*^9}}],

Cell["Now we redefine ", "Text",
 CellChangeTimes->{{3.616496134288591*^9, 3.6164961682654505`*^9}, 
   3.6164962078739204`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "fneg", "]"}], ";", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"fneg", "[", 
     RowBox[{"k_", ",", " ", "c0_", ",", " ", "u0_"}], "]"}], "[", "u_", 
    "]"}], " ", ":=", " ", "\n", "   ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"-", 
         RowBox[{"Sqrt", "[", 
          RowBox[{"1", "/", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"c0", "^", "2"}], " ", "-", " ", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"1", " ", "+", " ", 
                RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", "\n", 
              "           ", 
              RowBox[{"u0", "^", "2"}]}]}], ")"}]}], "]"}]}], ")"}], "*", 
       RowBox[{"Sqrt", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"-", "k"}], ")"}], "*", 
          RowBox[{"u", "^", "2"}]}], " ", "-", " ", "\n", "        ", 
         RowBox[{
          RowBox[{"c0", "^", "2"}], "*", 
          RowBox[{"(", 
           RowBox[{"1", " ", "+", " ", 
            RowBox[{"k", "*", 
             RowBox[{"u", "^", "2"}]}]}], ")"}]}], " ", "+", " ", "\n", 
         "        ", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", " ", "+", " ", 
            RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
          RowBox[{"u0", "^", "2"}]}]}], "]"}], "*", "\n", "      ", 
       RowBox[{"Sqrt", "[", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", " ", "-", " ", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1", " ", "+", " ", 
              RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", "\n", 
            "          ", 
            RowBox[{"(", 
             RowBox[{"u", " ", "-", " ", "u0"}], ")"}], "*", 
            RowBox[{"(", 
             RowBox[{"u", " ", "+", " ", "u0"}], ")"}]}]}], ")"}], "/", "\n", 
         "        ", 
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1", " ", "+", " ", 
              RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
            RowBox[{"u0", "^", "2"}]}]}], ")"}]}], "]"}], "*", "\n", "      ", 
       RowBox[{"EllipticE", "[", 
        RowBox[{
         RowBox[{"ArcSin", "[", "\n", "        ", 
          RowBox[{"u", "*", 
           RowBox[{"Sqrt", "[", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"1", " ", "+", " ", 
                 RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k"}], ")"}], "/", 
             "\n", "           ", 
             RowBox[{"(", 
              RowBox[{"1", " ", "+", " ", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"1", " ", "+", " ", 
                  RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", "\n", 
                "            ", 
                RowBox[{"u0", "^", "2"}]}]}], ")"}]}], "]"}]}], "]"}], ",", 
         " ", "\n", "       ", 
         RowBox[{"1", " ", "+", " ", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"1", " ", "+", " ", 
             RowBox[{"c0", "^", "2"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", 
              RowBox[{"c0", "^", "2"}]}], " ", "+", " ", "\n", "          ", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"1", " ", "+", " ", 
                RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
              RowBox[{"u0", "^", "2"}]}]}], ")"}]}]}]}], "]"}]}], " ", "+", 
      " ", "\n", "     ", 
      RowBox[{
       RowBox[{"Sqrt", "[", 
        RowBox[{"1", "/", 
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1", " ", "+", " ", 
              RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", "\n", 
            "          ", 
            RowBox[{"u0", "^", "2"}]}]}], ")"}]}], "]"}], "*", 
       RowBox[{"Sqrt", "[", "\n", "       ", 
        RowBox[{
         RowBox[{"-", "1"}], " ", "+", " ", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", " ", "+", " ", 
            RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
          RowBox[{"(", 
           RowBox[{"u", " ", "-", " ", "u0"}], ")"}], "*", "\n", "         ", 
          RowBox[{"(", 
           RowBox[{"u", " ", "+", " ", "u0"}], ")"}]}]}], "]"}], "*", 
       RowBox[{"Sqrt", "[", "\n", "       ", 
        RowBox[{"1", " ", "-", " ", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1", " ", "+", " ", 
              RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
            RowBox[{"u", "^", "2"}]}], ")"}], "/", "\n", "         ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"c0", "^", "2"}]}], " ", "+", " ", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", " ", "+", " ", 
               RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", "\n", 
             "           ", 
             RowBox[{"u0", "^", "2"}]}]}], ")"}]}]}], "]"}], "*", 
       RowBox[{"EllipticE", "[", "\n", "       ", 
        RowBox[{
         RowBox[{"ArcSin", "[", 
          RowBox[{"u0", "*", 
           RowBox[{"Sqrt", "[", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"1", " ", "+", " ", 
                 RowBox[{"c0", "^", "2"}]}], ")"}], "*", "\n", "            ",
                "k"}], ")"}], "/", 
             RowBox[{"(", 
              RowBox[{"1", " ", "+", " ", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"1", " ", "+", " ", 
                  RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", "\n", 
                "            ", 
                RowBox[{"u0", "^", "2"}]}]}], ")"}]}], "]"}]}], "]"}], ",", 
         " ", "\n", "       ", 
         RowBox[{"1", " ", "+", " ", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"1", " ", "+", " ", 
             RowBox[{"c0", "^", "2"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", 
              RowBox[{"c0", "^", "2"}]}], " ", "+", " ", "\n", "          ", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"1", " ", "+", " ", 
                RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
              RowBox[{"u0", "^", "2"}]}]}], ")"}]}]}]}], "]"}]}]}], ")"}], 
    "/", "\n", "    ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Sqrt", "[", 
       RowBox[{"1", "/", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"c0", "^", "2"}], " ", "-", " ", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"1", " ", "+", " ", 
             RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", "\n", 
           "         ", 
           RowBox[{"u0", "^", "2"}]}]}], ")"}]}], "]"}], "*", 
      RowBox[{"Sqrt", "[", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", " ", "+", " ", 
            RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k"}], ")"}], "/", "\n", 
        "       ", 
        RowBox[{"(", 
         RowBox[{"1", " ", "+", " ", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"1", " ", "+", " ", 
             RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
           RowBox[{"u0", "^", "2"}]}]}], ")"}]}], "]"}], "*", "\n", "     ", 
      RowBox[{"Sqrt", "[", 
       RowBox[{
        RowBox[{"-", "1"}], " ", "+", " ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", 
           RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", "\n", "        ", 
         RowBox[{"(", 
          RowBox[{"u", " ", "-", " ", "u0"}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"u", " ", "+", " ", "u0"}], ")"}]}]}], "]"}], "*", "\n", 
      "     ", 
      RowBox[{"Sqrt", "[", 
       RowBox[{"1", " ", "-", " ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"1", " ", "+", " ", 
             RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
           RowBox[{"u", "^", "2"}]}], ")"}], "/", "\n", "        ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"c0", "^", "2"}]}], " ", "+", " ", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1", " ", "+", " ", 
              RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", "\n", 
            "          ", 
            RowBox[{"u0", "^", "2"}]}]}], ")"}]}]}], "]"}]}], 
     ")"}]}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6164962222103453`*^9, 3.6164963217853203`*^9}}],

Cell[TextData[StyleBox["Properties of the Solutions",
 FontColor->RGBColor[0, 0, 1]]], "Text",
 CellChangeTimes->{{3.616553101773759*^9, 3.616553114331781*^9}, 
   3.616647345128044*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"fpos", "[", 
     RowBox[{"k", ",", "c0", ",", "u0"}], "]"}], "[", "u", "]"}], "+", 
   RowBox[{
    RowBox[{"fneg", "[", 
     RowBox[{"k", ",", "c0", ",", "u0"}], "]"}], "[", "u", "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6166467744322414`*^9, 3.616646897048457*^9}}],

Cell[TextData[{
 StyleBox["Lemma",
  FontColor->RGBColor[0, 0, 1]],
 ". The solutions fpos, fneg differ by sign only. ",
 "\[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.616647242682664*^9, 3.616647299076763*^9}, 
   3.6166473329288225`*^9, {3.616648383982584*^9, 3.6166483839885845`*^9}}],

Cell[TextData[{
 "Remember: According to the conditions, fpos must be applied for c0>0, and \
fneg in case c0< 0. \nWe consider the case c0 = 0, in which both conditions \
are satisfied.\n\n",
 StyleBox["Case c0 =0, u0=0:",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.6165531516938467`*^9, 3.6165533617482157`*^9}, {
  3.616554275800621*^9, 3.6165542928358507`*^9}, {3.6166477369539323`*^9, 
  3.6166477503387556`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"fpos", "[", 
   RowBox[{"k", ",", "0", ",", "0"}], "]"}], "[", "u", "]"}]], "Input",
 CellChangeTimes->{{3.616553366007023*^9, 3.6165533770206423`*^9}}],

Cell["\<\
The solutions with the initial conditions c0 = u0 = 0 are not contained in \
the family of solutions; thus the spheres are not  found by DSolve:\
\>", "Text",
 CellChangeTimes->{{3.616553530696512*^9, 3.6165536814551773`*^9}}],

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"diff", "[", 
        RowBox[{"f", ",", "k"}], "]"}], "[", "u", "]"}], "\[Equal]", "0"}], 
     ",", " ", 
     RowBox[{
      RowBox[{"f", "[", "0", "]"}], " ", "\[Equal]", " ", "0"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["f", "\[Prime]",
        MultilineFunction->None], "[", "0", "]"}], "==", "0"}]}], "}"}], ",", 
   RowBox[{"f", "[", "u", "]"}], ",", "u"}], "]"}]], "Input",
 CellChangeTimes->{{3.6165536891147904`*^9, 3.616553704340417*^9}, {
   3.6165537785497475`*^9, 3.6165538590770893`*^9}, {3.616553919901596*^9, 
   3.616553921211998*^9}, {3.616554017635768*^9, 3.616554023532578*^9}, 
   3.616554926477764*^9}],

Cell["Half-spheres are generated by the function", "Text",
 CellChangeTimes->{{3.616558753193776*^9, 3.616558772849811*^9}, {
  3.6186384991017866`*^9, 3.6186385277414246`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fsph", "[", "r_", "]"}], "[", "u_", "]"}], " ", ":=", 
  RowBox[{"r", "-", " ", 
   RowBox[{"Sqrt", "[", 
    RowBox[{
     RowBox[{"r", "^", "2"}], "-", 
     RowBox[{"u", "^", "2"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6165587812426257`*^9, 3.6165588315059137`*^9}, {
  3.616559024946254*^9, 3.616559026194256*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sfrev", "[", 
     RowBox[{"id", ",", 
      RowBox[{"fsph", "[", "1", "]"}]}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616558846871941*^9, 3.616558957226535*^9}}],

Cell["\<\
Of course, fsph satisfies the differential equation with the initial \
conditions\
\>", "Text",
 CellChangeTimes->{{3.6165591945653515`*^9, 3.616559238089428*^9}, {
   3.616559463010623*^9, 3.616559469157034*^9}, 3.6186385519728107`*^9}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"fsph", "[", "r", "]"}], "[", "0", "]"}], ",", " ", 
   RowBox[{"r", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6165592460922422`*^9, 3.6165593935125012`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"fsph", "[", "r", "]"}], "[", "u", "]"}], ",", "u"}], "]"}], "/.",
   " ", 
  RowBox[{"u", "\[Rule]", "0"}]}]], "Input",
 CellChangeTimes->{{3.616559401874116*^9, 3.616559447535396*^9}}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"diff", "[", 
     RowBox[{
      RowBox[{"fsph", "[", "r", "]"}], ",", 
      RowBox[{"1", "/", 
       RowBox[{"r", "^", "2"}]}]}], "]"}], "[", "u", "]"}], ",", 
   RowBox[{"r", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6165590731815386`*^9, 3.616559144224063*^9}}],

Cell["\<\
Also for arbitrary position u0 of the initial values with c0 = 0 Mathematica\
\[CloseCurlyQuote]s  DSolve doesn\[CloseCurlyQuote]t solve the equation:\
\>", "Text",
 CellChangeTimes->{{3.6165556473531213`*^9, 3.6165557078188276`*^9}, {
  3.6165595071275005`*^9, 3.6165595147247143`*^9}, {3.616559595376856*^9, 
  3.6165596105868826`*^9}}],

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"diff", "[", 
        RowBox[{"f", ",", "k"}], "]"}], "[", "u", "]"}], "\[Equal]", "0"}], 
     ",", " ", 
     RowBox[{
      RowBox[{"f", "[", "u0", "]"}], " ", "\[Equal]", " ", "0"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["f", "\[Prime]",
        MultilineFunction->None], "[", "u0", "]"}], "==", "0"}]}], "}"}], ",", 
   RowBox[{"f", "[", "u", "]"}], ",", "u"}], "]"}]], "Input",
 CellChangeTimes->{{3.6165536891147904`*^9, 3.616553704340417*^9}, {
   3.6165537785497475`*^9, 3.6165538590770893`*^9}, {3.616553919901596*^9, 
   3.616553921211998*^9}, {3.616554017635768*^9, 3.616554023532578*^9}, {
   3.616554839398411*^9, 3.616554862876452*^9}, 3.6165549113301373`*^9, {
   3.616554985024667*^9, 3.61655499828469*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"fpos", "[", 
   RowBox[{"k", ",", "0", ",", "u0"}], "]"}], "[", "u", "]"}]], "Input",
 CellChangeTimes->{{3.6165555090276785`*^9, 3.616555525906908*^9}, {
  3.6165560932331047`*^9, 3.6165560937947054`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"fpos", "[", 
   RowBox[{"k", ",", "0", ",", "u0"}], "]"}], "[", "u0", "]"}]], "Input",
 CellChangeTimes->{{3.6165553183660517`*^9, 3.6165553458377004`*^9}, {
  3.6165563652351823`*^9, 3.6165563753128*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"fpos", "[", 
      RowBox[{"k", ",", "0", ",", "u0"}], "]"}], "[", "u", "]"}], ",", " ", 
    "u"}], "]"}], "/.", " ", 
  RowBox[{"u", "\[Rule]", " ", "u0"}]}]], "Input",
 CellChangeTimes->{{3.616556427354491*^9, 3.616556490253802*^9}, 
   3.616556676284128*^9}],

Cell[TextData[{
 StyleBox["Lemma.",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox[" ",
  FontColor->RGBColor[1, 0, 0]],
 "The functions fpos,fneg give no solutions in the case c0 = 0, u0= 0; in the \
case  c=0, u0\[NotEqual]0 they give solutions, but the initial conditions are \
not confirmed; they appear as limits.."
}], "Text",
 CellChangeTimes->{{3.616556804843954*^9, 3.616556809586363*^9}, {
   3.6165568403808165`*^9, 3.616557030482751*^9}, {3.616647033782697*^9, 
   3.616647072673566*^9}, {3.616647142312088*^9, 3.616647179627353*^9}, {
   3.616648193332734*^9, 3.616648202115549*^9}, {3.616648234501206*^9, 
   3.616648235686808*^9}, {3.6166482872290497`*^9, 3.616648300709821*^9}, 
   3.61682560562577*^9, 3.6186385827415705`*^9}],

Cell[BoxData[
 RowBox[{"Limit", "[", 
  RowBox[{
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"fpos", "[", 
       RowBox[{"k", ",", "0", ",", "u0"}], "]"}], "[", "u", "]"}], ",", " ", 
     "u"}], "]"}], ",", " ", 
   RowBox[{"u", " ", "\[Rule]", " ", "u0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6166479442626963`*^9, 3.616647983106765*^9}}],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.6166483460414143`*^9, 3.616648346048414*^9}}],

Cell[TextData[StyleBox["Example. The Pseudosphere",
 FontColor->RGBColor[0, 0, 1]]], "Text",
 CellChangeTimes->{{3.6165025853310537`*^9, 3.616502599183878*^9}}],

Cell[BoxData[
 RowBox[{"?", "pseudosphere"}]], "Input",
 CellChangeTimes->{{3.616502650383168*^9, 3.6165026597899847`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pseudosphere", "[", "1", "]"}], "[", 
  RowBox[{"0", ",", 
   RowBox[{"2", "*", "Pi"}], ",", "0", ",", "Pi"}], "]"}]], "Input",
 CellChangeTimes->{{3.616502709444872*^9, 3.6165027856198053`*^9}}],

Cell["", "Text",
 CellChangeTimes->{{3.6165053658799376`*^9, 3.6165054637389097`*^9}, {
   3.616566645708686*^9, 3.6165666542575006`*^9}, 3.6165667056439915`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pseudosphere", "[", "a", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6165028578791323`*^9, 3.616502871778757*^9}, {
  3.6165029625865164`*^9, 3.616503026749429*^9}, {3.6165041297181664`*^9, 
  3.6165041330565724`*^9}, {3.6165667127420034`*^9, 3.6165667181864133`*^9}}],

Cell["To get a parameter representation  in the form", "Text",
 CellChangeTimes->{{3.6165674048528194`*^9, 3.6165674411852827`*^9}, 
   3.6165674871585636`*^9, 3.618638587836862*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sfrev", "[", 
   RowBox[{"id", ",", "f"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6165674459900913`*^9, 3.616567471137336*^9}}],

Cell["we permute the variables u,v and consider", "Text",
 CellChangeTimes->{{3.616505550521862*^9, 3.616505557292274*^9}, {
   3.6165675014325886`*^9, 3.616567520293022*^9}, 3.6186386083970385`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pseudosphere", "[", "a", "]"}], "[", 
  RowBox[{"0", ",", "u"}], "]"}]], "Input",
 CellChangeTimes->{{3.6165675315094414`*^9, 3.6165675827867317`*^9}}],

Cell["\<\
transforming the variable u setting x = a*Sin[u] we obtain with x replaced by \
u again\
\>", "Text",
 CellChangeTimes->{{3.616567647168045*^9, 3.6165676902865205`*^9}, {
  3.616567724731381*^9, 3.6165677556350355`*^9}, {3.616567907080101*^9, 
  3.616567916174917*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fpssph", "[", "a_", "]"}], "[", "u_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"Sqrt", "[", 
    RowBox[{
     RowBox[{"a", "^", "2"}], "-", 
     RowBox[{"u", "^", "2"}]}], "]"}], " ", "+", " ", 
   RowBox[{"a", "*", 
    RowBox[{"Log", "[", " ", 
     RowBox[{"u", "/", 
      RowBox[{"(", 
       RowBox[{"a", "+", 
        RowBox[{"Sqrt", "[", 
         RowBox[{
          RowBox[{"a", "^", "2"}], " ", "-", " ", 
          RowBox[{"u", "^", "2"}]}], "]"}]}], ")"}]}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.616504417148671*^9, 3.616504465087556*^9}, {
   3.616567776960273*^9, 3.6165678371139784`*^9}, 3.6165678964252825`*^9}],

Cell["This function generates the pseudospheres by", "Text",
 CellChangeTimes->{{3.6165679776858253`*^9, 3.616568022395504*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sfrev", "[", 
   RowBox[{"id", ",", 
    RowBox[{"fpssph", "[", "a", "]"}]}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6165680314435196`*^9, 3.6165680611459723`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sfrev", "[", 
     RowBox[{"id", ",", 
      RowBox[{"fpssph", "[", "1", "]"}]}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6165660142663765`*^9, 3.616566114028552*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sfrev", "[", 
     RowBox[{"id", ",", 
      RowBox[{"(", 
       RowBox[{"-", 
        RowBox[{"fpssph", "[", "1", "]"}]}], ")"}]}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6165660142663765`*^9, 3.6165662161619315`*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"%", ",", "%%"}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "4"}], ",", "4"}], "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616566249935991*^9, 3.61656627787564*^9}, 
   3.616566325674124*^9, {3.616566392411041*^9, 3.616566402129858*^9}, {
   3.616566463983967*^9, 3.6165664749195857`*^9}, {3.6165681795314384`*^9, 
   3.616568184476647*^9}}],

Cell["The function fpssph satisfies the initial conditions", "Text",
 CellChangeTimes->{{3.6165683003692503`*^9, 3.6165683577773514`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"fpssph", "[", "a", "]"}], "[", "a", "]"}]], "Input",
 CellChangeTimes->{{3.616568367574168*^9, 3.6165683886186056`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"fpssph", "[", "a", "]"}], "[", "u", "]"}], ",", "u"}], "]"}], "/.", 
  RowBox[{"u", "\[Rule]", "a"}]}]], "Input",
 CellChangeTimes->{{3.616568393485814*^9, 3.616568457258726*^9}}],

Cell["Consider the limit instead:", "Text",
 CellChangeTimes->{{3.6165687093083687`*^9, 3.616568729994005*^9}, 
   3.61656876122526*^9}],

Cell[BoxData[
 RowBox[{"Limit", "[", 
  RowBox[{
   RowBox[{"Evaluate", "[", 
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"fpssph", "[", "a", "]"}], "[", "u", "]"}], ",", "u"}], "]"}], 
    "]"}], ",", 
   RowBox[{"u", "->", "a"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616568495400793*^9, 3.616568499332*^9}, {
   3.6165685328564587`*^9, 3.6165686361442404`*^9}, 3.6165686718059025`*^9}],

Cell["Of course, it solves the differential equation", "Text",
 CellChangeTimes->{3.6165688647470417`*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"diff", "[", 
     RowBox[{
      RowBox[{"fpssph", "[", "a", "]"}], ",", 
      RowBox[{
       RowBox[{"-", "1"}], "/", 
       RowBox[{"a", "^", "2"}]}]}], "]"}], "[", "u", "]"}], ",", 
   RowBox[{"a", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6165688696298504`*^9, 3.616568946491185*^9}}],

Cell["\<\
But it is not contained in the families defined by fpos,fneg:\
\>", "Text",
 CellChangeTimes->{{3.616505712902547*^9, 3.616505779826665*^9}, {
  3.616568977098439*^9, 3.616568979142043*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"fpos", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "1"}], "/", 
     RowBox[{"a", "^", "2"}]}], ",", " ", "0", ",", "a"}], "]"}], "[", "u", 
  "]"}]], "Input",
 CellChangeTimes->{{3.6165048110805635`*^9, 3.6165048262437897`*^9}, {
  3.616504861593452*^9, 3.6165048756490765`*^9}, {3.6165689908732634`*^9, 
  3.616569003072485*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"gauss", "[", 
   RowBox[{"sfrev", "[", 
    RowBox[{"id", ",", " ", 
     RowBox[{"fpssph", "[", "a", "]"}]}], "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6165033554732065`*^9, 3.616503377469245*^9}, 
   3.616503425361329*^9, {3.616503627818485*^9, 3.6165036492685223`*^9}, {
   3.6165040282711887`*^9, 3.616504038239606*^9}, {3.6165690695598016`*^9, 
   3.6165691004322557`*^9}, 3.6165691331455135`*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", ",", " ", 
   RowBox[{"v", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616503449276171*^9, 3.6165034784170227`*^9}, 
   3.6165040615460467`*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"meancurvature", "[", 
     RowBox[{"sfrev", "[", 
      RowBox[{"id", ",", " ", 
       RowBox[{"fpssph", "[", "a", "]"}]}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
   RowBox[{"a", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6165033554732065`*^9, 3.616503377469245*^9}, 
   3.616503425361329*^9, {3.616503627818485*^9, 3.6165036492685223`*^9}, {
   3.6165040282711887`*^9, 3.616504038239606*^9}, {3.6165690695598016`*^9, 
   3.6165691004322557`*^9}, 3.6165691331455135`*^9, {3.6166489395359545`*^9, 
   3.6166490211446223`*^9}}],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"%", "/.", " ", 
    RowBox[{"a", "\[Rule]", "1"}]}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"u", ",", ".0001", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616649090407584*^9, 3.616649167744007*^9}, {
  3.6166492201040025`*^9, 3.616649235935908*^9}, {3.6166556733628006`*^9, 
  3.6166556957920837`*^9}, {3.6166561977777953`*^9, 3.616656227289483*^9}, {
  3.616656471546454*^9, 3.616656516074001*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"prcurvatures", "[", 
     RowBox[{"sfrev", "[", 
      RowBox[{"id", ",", " ", 
       RowBox[{"fpssph", "[", "a", "]"}]}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
   RowBox[{"a", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6165033554732065`*^9, 3.616503377469245*^9}, 
   3.616503425361329*^9, {3.616503627818485*^9, 3.6165036492685223`*^9}, {
   3.6165040282711887`*^9, 3.616504038239606*^9}, {3.6165690695598016`*^9, 
   3.6165691004322557`*^9}, 3.6165691331455135`*^9, {3.6166489395359545`*^9, 
   3.6166490211446223`*^9}, {3.6166493015136585`*^9, 3.616649308457056*^9}}],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"%", "/.", 
    RowBox[{"a", "\[Rule]", "1"}]}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"u", ",", ".0001", ",", "1.5"}], "}"}], ",", " ", 
   RowBox[{"PlotStyle", " ", "\[Rule]", "Red"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616649858609523*^9, 3.6166499597283063`*^9}, {
  3.6166500054969244`*^9, 3.6166500197367387`*^9}, {3.6166500609120936`*^9, 
  3.6166501297290297`*^9}, {3.6166501853292103`*^9, 3.6166502075764823`*^9}, {
  3.6166502487998405`*^9, 3.616650253008081*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
3.9.4. Surfaces of Revolution of Constant Positive Gaussian Curvature\
\>", "Subsubsection",
 CellChangeTimes->{{3.592460039870927*^9, 3.592460088511812*^9}, {
   3.592460175341565*^9, 3.5924602045760164`*^9}, {3.592743295809393*^9, 
   3.5927433008794017`*^9}, 3.59341846877223*^9, {3.6166566434732876`*^9, 
   3.616656645879425*^9}, 3.6167420110361643`*^9, 3.6170153528088784`*^9, 
   3.620482334629857*^9, 3.6280853261532087`*^9}],

Cell["\<\
In this and the following subsection we shall find surfaces of revolution for \
all values c0 \[NotEqual] 0. We distinguish the cases u0 = 0 and u0 \
\[NotEqual] 0. In the first case the origin is a singular point of the \
surface, since it lies on the rotation axis  \
\>", "Text",
 CellChangeTimes->{{3.616669104636664*^9, 3.6166694203500185`*^9}, {
  3.616669476541317*^9, 3.616669536351822*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"fpos", "[", 
     RowBox[{"k", ",", "c0", ",", "0"}], "]"}], "[", "u", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"c0", "\[NotEqual]", "0"}], ",", " ", 
     RowBox[{"k", ">", "0"}], ",", " ", 
     RowBox[{"u", "\[Element]", " ", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616669555695856*^9, 3.6166696676728525`*^9}, {
  3.6166697333333683`*^9, 3.616669744144187*^9}}],

Cell["Manual simplification yields", "Text",
 CellChangeTimes->{{3.616670096564406*^9, 3.616670134612873*^9}, {
  3.6166702113771152`*^9, 3.6166702354174905`*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"fp0", "[", 
     RowBox[{"k_", ",", " ", "c0_"}], "]"}], "[", "u_", "]"}], " ", ":=", " ",
    "\n", "  ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{"c0", "/", 
        RowBox[{"Sqrt", "[", 
         RowBox[{"k", "*", 
          RowBox[{"(", 
           RowBox[{"1", " ", "+", " ", 
            RowBox[{"c0", "^", "2"}]}], ")"}]}], "]"}]}], ")"}]}], ")"}], "*",
     "\n", "   ", 
    RowBox[{"EllipticE", "[", 
     RowBox[{
      RowBox[{"ArcSin", "[", 
       RowBox[{
        RowBox[{"Sqrt", "[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", " ", "+", " ", 
            RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k"}], "]"}], "*", "\n", 
        "      ", "u"}], "]"}], ",", " ", 
      RowBox[{"-", 
       RowBox[{"(", 
        RowBox[{"1", "/", 
         RowBox[{"c0", "^", "2"}]}], ")"}]}]}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.6166702562526817`*^9, 3.6166704501657734`*^9}, {
  3.6166705339745665`*^9, 3.61667054015092*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"gauss", "[", 
     RowBox[{"sfrev", "[", 
      RowBox[{"id", ",", " ", 
       RowBox[{"fp0", "[", 
        RowBox[{"k", ",", "c0"}], "]"}]}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"c0", "\[NotEqual]", "0"}], ",", " ", 
     RowBox[{"k", ">", "0"}], ",", " ", 
     RowBox[{"u", "\[Element]", " ", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6166704583212395`*^9, 3.6166705232119513`*^9}, 
   3.6166705710216856`*^9, 3.616670617478343*^9}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sfrev", "[", 
     RowBox[{"id", ",", 
      RowBox[{"fp0", "[", 
       RowBox[{"2", ",", "2"}], "]"}]}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "3"}], ",", "3"}], "}"}], ",", 
   RowBox[{"PlotPoints", " ", "\[Rule]", " ", "100"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6166780151621265`*^9, 3.6166782994893894`*^9}, {
   3.6166783444659615`*^9, 3.6166783458900433`*^9}, 3.6166786841703916`*^9, 
   3.6166787147221394`*^9, {3.6166787502021685`*^9, 3.616678774457556*^9}, {
   3.6166788196831427`*^9, 3.616678821785263*^9}, {3.616678856666258*^9, 
   3.61667885896939*^9}, {3.616678890130172*^9, 3.616678945665348*^9}, {
   3.616678976538114*^9, 3.6166790022815866`*^9}}],

Cell["\<\
Try the last cell with distinct values of c0 and k.  You get surfaces of \
revolution with positive curvature k of the conical type. By definition \
clearly is\
\>", "Text",
 CellChangeTimes->{{3.616678530133581*^9, 3.61667867119765*^9}, {
  3.616679025684925*^9, 3.6166790431089215`*^9}, {3.6166791032693624`*^9, 
  3.616679131340968*^9}, {3.61863870244693*^9, 3.6186387048230658`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fp0", "[", 
    RowBox[{"k", ",", 
     RowBox[{"-", "c0"}]}], "]"}], "[", "u", "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"fp0", "[", 
    RowBox[{"k", ",", 
     RowBox[{"-", "c0"}]}], "]"}], "[", "u", "]"}]}]], "Input",
 CellChangeTimes->{{3.6166791549483185`*^9, 3.616679225512355*^9}}],

Cell["Case u0 \[NotEqual] 0", "Text",
 CellChangeTimes->{{3.616679901846039*^9, 3.6166799211331415`*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"fpos", "[", 
     RowBox[{"k", ",", "c0", ",", "u0"}], "]"}], "[", "u", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u0", ">", "0"}], ",", 
     RowBox[{"c0", ">", "0"}], ",", " ", 
     RowBox[{"k", ">", "0"}], ",", " ", 
     RowBox[{"u", "\[Element]", " ", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616669555695856*^9, 3.6166696676728525`*^9}, {
  3.6166697333333683`*^9, 3.616669744144187*^9}, {3.6166799831776905`*^9, 
  3.616680007937107*^9}, {3.616680485873443*^9, 3.616680487080512*^9}, {
  3.616730007161146*^9, 3.61673001505476*^9}}],

Cell["We use this result to define", "Text",
 CellChangeTimes->{{3.6167304861131873`*^9, 3.6167304980784082`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fp1", "[", 
    RowBox[{"k_", ",", " ", "c0_", ",", " ", "u0_"}], "]"}], "[", "u_", "]"}],
   " ", ":=", " ", "\n", "  ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"Sqrt", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"-", "k"}], ")"}], "*", 
         RowBox[{"u", "^", "2"}]}], " ", "-", " ", 
        RowBox[{
         RowBox[{"c0", "^", "2"}], "*", "\n", "        ", 
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", 
           RowBox[{"k", "*", 
            RowBox[{"u", "^", "2"}]}]}], ")"}]}], " ", "+", " ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", 
           RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", "\n", "        ", 
         RowBox[{"u0", "^", "2"}]}]}], "]"}], "*", 
      RowBox[{"Sqrt", "[", "\n", "      ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", " ", "-", " ", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"1", " ", "+", " ", 
             RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
           RowBox[{"(", 
            RowBox[{"u", " ", "-", " ", "u0"}], ")"}], "*", "\n", "         ", 
           RowBox[{"(", 
            RowBox[{"u", " ", "+", " ", "u0"}], ")"}]}]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{"1", " ", "+", " ", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"1", " ", "+", " ", 
             RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", "\n", 
           "         ", 
           RowBox[{"u0", "^", "2"}]}]}], ")"}]}], "]"}], "*", 
      RowBox[{"EllipticE", "[", "\n", "      ", 
       RowBox[{
        RowBox[{"ArcSin", "[", 
         RowBox[{"u", "*", 
          RowBox[{"Sqrt", "[", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"1", " ", "+", " ", 
                RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k"}], ")"}], "/", 
            "\n", "          ", 
            RowBox[{"(", 
             RowBox[{"1", " ", "+", " ", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"1", " ", "+", " ", 
                 RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
               RowBox[{"u0", "^", "2"}]}]}], ")"}]}], "]"}]}], "]"}], ",", 
        " ", "\n", "      ", 
        RowBox[{"1", " ", "+", " ", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", " ", "+", " ", 
            RowBox[{"c0", "^", "2"}]}], ")"}], "/", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"c0", "^", "2"}]}], " ", "+", " ", "\n", "         ", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", " ", "+", " ", 
               RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
             RowBox[{"u0", "^", "2"}]}]}], ")"}]}]}]}], "]"}]}], " ", "+", 
     " ", "\n", "    ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"c0", "^", "2"}]}], " ", "+", " ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", 
           RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
         RowBox[{"u0", "^", "2"}]}]}], ")"}], "*", "\n", "     ", 
      RowBox[{"Sqrt", "[", 
       RowBox[{"1", "/", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"c0", "^", "2"}], " ", "+", " ", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "1"}], " ", "+", " ", 
             RowBox[{"c0", "^", "4"}]}], ")"}], "*", "k", "*", "\n", 
           "         ", 
           RowBox[{"u0", "^", "2"}]}], " ", "-", " ", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1", " ", "+", " ", 
              RowBox[{"c0", "^", "2"}]}], ")"}], "^", "2"}], "*", 
           RowBox[{"k", "^", "2"}], "*", "\n", "         ", 
           RowBox[{"u0", "^", "4"}]}]}], ")"}]}], "]"}], "*", 
      RowBox[{"Sqrt", "[", "\n", "      ", 
       RowBox[{
        RowBox[{"-", "1"}], " ", "+", " ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", 
           RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
         RowBox[{"(", 
          RowBox[{"u", " ", "-", " ", "u0"}], ")"}], "*", "\n", "        ", 
         RowBox[{"(", 
          RowBox[{"u", " ", "+", " ", "u0"}], ")"}]}]}], "]"}], "*", 
      RowBox[{"Sqrt", "[", "\n", "      ", 
       RowBox[{"1", " ", "-", " ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"1", " ", "+", " ", 
             RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
           RowBox[{"u", "^", "2"}]}], ")"}], "/", "\n", "        ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"c0", "^", "2"}]}], " ", "+", " ", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1", " ", "+", " ", 
              RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
            RowBox[{"u0", "^", "2"}]}]}], ")"}]}]}], "]"}], "*", "\n", 
      "     ", 
      RowBox[{"EllipticE", "[", 
       RowBox[{
        RowBox[{"ArcSin", "[", "\n", "       ", 
         RowBox[{"u0", "*", 
          RowBox[{"Sqrt", "[", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"1", " ", "+", " ", 
                RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k"}], ")"}], "/", 
            "\n", "          ", 
            RowBox[{"(", 
             RowBox[{"1", " ", "+", " ", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"1", " ", "+", " ", 
                 RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
               RowBox[{"u0", "^", "2"}]}]}], ")"}]}], "]"}]}], "]"}], ",", 
        " ", "\n", "      ", 
        RowBox[{"1", " ", "+", " ", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", " ", "+", " ", 
            RowBox[{"c0", "^", "2"}]}], ")"}], "/", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"c0", "^", "2"}]}], " ", "+", " ", "\n", "         ", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", " ", "+", " ", 
               RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
             RowBox[{"u0", "^", "2"}]}]}], ")"}]}]}]}], "]"}]}]}], ")"}], "/",
    "\n", "   ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Sqrt", "[", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", 
           RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "1"}], " ", "+", " ", "\n", "        ", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1", " ", "+", " ", 
              RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
            RowBox[{"(", 
             RowBox[{"u", " ", "-", " ", "u0"}], ")"}], "*", "\n", 
            "         ", 
            RowBox[{"(", 
             RowBox[{"u", " ", "+", " ", "u0"}], ")"}]}]}], ")"}]}], ")"}], 
       "/", 
       RowBox[{"(", 
        RowBox[{"1", " ", "+", " ", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", " ", "+", " ", 
            RowBox[{"c0", "^", "2"}]}], ")"}], "*", "\n", "        ", "k", 
          "*", 
          RowBox[{"u0", "^", "2"}]}]}], ")"}]}], "]"}], "*", 
     RowBox[{"Sqrt", "[", "\n", "     ", 
      RowBox[{"1", " ", "-", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", " ", "+", " ", 
            RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
          RowBox[{"u", "^", "2"}]}], ")"}], "/", "\n", "       ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"c0", "^", "2"}]}], " ", "+", " ", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"1", " ", "+", " ", 
             RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
           RowBox[{"u0", "^", "2"}]}]}], ")"}]}]}], "]"}]}], 
    ")"}]}]}]], "Input",
 CellChangeTimes->{{3.6167298286968327`*^9, 3.61672993555702*^9}, {
  3.6167304526511283`*^9, 3.6167304756923685`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"Evaluate", "[", 
    RowBox[{
     RowBox[{"sfrev", "[", 
      RowBox[{"id", ",", 
       RowBox[{"fp1", "[", 
        RowBox[{"1", ",", "2", ",", "1"}], "]"}]}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0.7", ",", "1.3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"PlotPoints", " ", "\[Rule]", " ", "100"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6167305403076825`*^9, 3.6167307478972473`*^9}, {
  3.6167308107965574`*^9, 3.616730822480978*^9}, {3.616731196585235*^9, 
  3.616731217005671*^9}, {3.616731885373245*^9, 3.616731895684863*^9}, {
  3.6167420367706366`*^9, 3.6167420449371033`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"Evaluate", "[", 
    RowBox[{
     RowBox[{"sfrev", "[", 
      RowBox[{"id", ",", 
       RowBox[{"-", 
        RowBox[{"fp1", "[", 
         RowBox[{"1", ",", "1", ",", "1"}], "]"}]}]}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0.7", ",", "1.3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"PlotPoints", " ", "\[Rule]", " ", "100"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6167305403076825`*^9, 3.6167307478972473`*^9}, {
   3.6167308107965574`*^9, 3.616730822480978*^9}, {3.616730915675542*^9, 
   3.6167309254723587`*^9}, 3.6167312361001043`*^9, {3.61673193398293*^9, 
   3.616731941361743*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"%", ",", "%%"}], "}"}], ",", " ", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6167309502140026`*^9, 3.6167309896664715`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
3.9.5. Surfaces of Revolution of Constant Negative Gaussian Curvature\
\>", "Subsubsection",
 CellChangeTimes->{{3.592460039870927*^9, 3.592460088511812*^9}, {
   3.592460175341565*^9, 3.5924602045760164`*^9}, {3.592743295809393*^9, 
   3.5927433008794017`*^9}, 3.592743439329645*^9, {3.592743475989709*^9, 
   3.5927434804045167`*^9}, 3.5934185161963134`*^9, {3.6126938340250387`*^9, 
   3.612693841775482*^9}, 3.6128496103122997`*^9, 3.6170153609043417`*^9, {
   3.620482324521039*^9, 3.6204823404486666`*^9}, 3.628085327416811*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"fpos", "[", 
     RowBox[{"k", ",", "c0", ",", "0"}], "]"}], "[", "u", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"c0", ">", "0"}], ",", " ", 
     RowBox[{"k", "<", "0"}], ",", " ", 
     RowBox[{"u", "\[Element]", " ", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616669555695856*^9, 3.6166696676728525`*^9}, {
  3.6166697333333683`*^9, 3.616669744144187*^9}, {3.6167321924753847`*^9, 
  3.6167322001505976`*^9}}],

Cell["Manual simplification leads to", "Text",
 CellChangeTimes->{{3.616732738679144*^9, 3.616732753233969*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fn0", "[", 
    RowBox[{"k_", ",", " ", "c0_"}], "]"}], "[", "u_", "]"}], " ", ":=", " ", 
  "\n", "  ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"I", "*", 
     RowBox[{"(", 
      RowBox[{"c0", "/", 
       RowBox[{"Sqrt", "[", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"-", "k"}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", 
           RowBox[{"c0", "^", "2"}]}], ")"}]}], "]"}]}], ")"}]}], ")"}], "*", 
   "\n", "   ", 
   RowBox[{"EllipticE", "[", 
    RowBox[{
     RowBox[{"ArcSin", "[", "\n", "     ", 
      RowBox[{
       RowBox[{"Sqrt", "[", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", 
           RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k"}], "]"}], "*", "u"}], 
      "]"}], ",", " ", "\n", "    ", 
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{"1", "/", 
        RowBox[{"c0", "^", "2"}]}], ")"}]}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6167327637639875`*^9, 3.616733006672014*^9}, {
  3.616733986712536*^9, 3.6167341589992385`*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"gauss", "[", 
     RowBox[{"sfrev", "[", 
      RowBox[{"id", ",", 
       RowBox[{"fn0", "[", 
        RowBox[{"k", ",", "c0"}], "]"}]}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"k", "<", "0"}], ",", " ", 
     RowBox[{"c0", ">", "0"}], ",", " ", 
     RowBox[{"u", "\[Element]", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6167330199476376`*^9, 3.6167331565102777`*^9}, 
   3.616733240797226*^9, 3.6167345350846987`*^9}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sfrev", "[", 
     RowBox[{"id", ",", 
      RowBox[{"fn0", "[", 
       RowBox[{
        RowBox[{"-", "2"}], ",", "2"}], "]"}]}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "3"}], ",", "3"}], "}"}], ",", 
   RowBox[{"PlotPoints", " ", "\[Rule]", " ", "100"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6166780151621265`*^9, 3.6166782994893894`*^9}, {
   3.6166783444659615`*^9, 3.6166783458900433`*^9}, 3.6166786841703916`*^9, 
   3.6166787147221394`*^9, {3.6166787502021685`*^9, 3.616678774457556*^9}, {
   3.6166788196831427`*^9, 3.616678821785263*^9}, {3.616678856666258*^9, 
   3.61667885896939*^9}, {3.616678890130172*^9, 3.616678945665348*^9}, {
   3.616678976538114*^9, 3.6166790022815866`*^9}, {3.6167349679230595`*^9, 
   3.6167349745998707`*^9}}],

Cell["\<\
This is a surface of revolution of constant negative curvature of conical \
shape. Try with other parameters c0>0, k<0.\
\>", "Text",
 CellChangeTimes->{{3.616735074408846*^9, 3.6167352156982946`*^9}, {
  3.616736829817529*^9, 3.616736840097947*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"fpos", "[", 
     RowBox[{"k", ",", "c0", ",", "u0"}], "]"}], "[", "u", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u0", ">", "0"}], ",", 
     RowBox[{"c0", ">", "0"}], ",", " ", 
     RowBox[{"k", "<", "0"}], ",", " ", 
     RowBox[{"u", "\[Element]", " ", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616669555695856*^9, 3.6166696676728525`*^9}, {
   3.6166697333333683`*^9, 3.616669744144187*^9}, {3.6166799831776905`*^9, 
   3.616680007937107*^9}, {3.616680485873443*^9, 3.616680487080512*^9}, {
   3.616730007161146*^9, 3.61673001505476*^9}, 3.61673585671902*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fn1", "[", 
    RowBox[{"k_", ",", " ", "c0_", ",", " ", "u0_"}], "]"}], "[", "u_", "]"}],
   " ", ":=", " ", "\n", "  ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"Sqrt", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"-", "k"}], ")"}], "*", 
         RowBox[{"u", "^", "2"}]}], " ", "-", " ", 
        RowBox[{
         RowBox[{"c0", "^", "2"}], "*", "\n", "        ", 
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", 
           RowBox[{"k", "*", 
            RowBox[{"u", "^", "2"}]}]}], ")"}]}], " ", "+", " ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", 
           RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", "\n", "        ", 
         RowBox[{"u0", "^", "2"}]}]}], "]"}], "*", 
      RowBox[{"Sqrt", "[", "\n", "      ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", " ", "-", " ", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"1", " ", "+", " ", 
             RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
           RowBox[{"(", 
            RowBox[{"u", " ", "-", " ", "u0"}], ")"}], "*", "\n", "         ", 
           RowBox[{"(", 
            RowBox[{"u", " ", "+", " ", "u0"}], ")"}]}]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{"1", " ", "+", " ", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"1", " ", "+", " ", 
             RowBox[{"c0", "^", "2"}]}], ")"}], "*", "\n", "         ", "k", 
           "*", 
           RowBox[{"u0", "^", "2"}]}]}], ")"}]}], "]"}], "*", 
      RowBox[{"EllipticE", "[", "\n", "      ", 
       RowBox[{
        RowBox[{"ArcSin", "[", 
         RowBox[{"u", "*", 
          RowBox[{"Sqrt", "[", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"1", " ", "+", " ", 
                RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k"}], ")"}], "/", 
            "\n", "          ", 
            RowBox[{"(", 
             RowBox[{"1", " ", "+", " ", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"1", " ", "+", " ", 
                 RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", "\n", 
               "            ", 
               RowBox[{"u0", "^", "2"}]}]}], ")"}]}], "]"}]}], "]"}], ",", 
        " ", "\n", "      ", 
        RowBox[{"1", " ", "+", " ", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", " ", "+", " ", 
            RowBox[{"c0", "^", "2"}]}], ")"}], "/", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"c0", "^", "2"}]}], " ", "+", " ", "\n", "         ", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", " ", "+", " ", 
               RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
             RowBox[{"u0", "^", "2"}]}]}], ")"}]}]}]}], "]"}]}], " ", "+", 
     " ", "\n", "    ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"c0", "^", "2"}]}], " ", "+", " ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", 
           RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
         RowBox[{"u0", "^", "2"}]}]}], ")"}], "*", "\n", "     ", 
      RowBox[{"Sqrt", "[", 
       RowBox[{"1", "/", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"c0", "^", "2"}], " ", "+", " ", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "1"}], " ", "+", " ", 
             RowBox[{"c0", "^", "4"}]}], ")"}], "*", "k", "*", "\n", 
           "         ", 
           RowBox[{"u0", "^", "2"}]}], " ", "-", " ", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1", " ", "+", " ", 
              RowBox[{"c0", "^", "2"}]}], ")"}], "^", "2"}], "*", 
           RowBox[{"k", "^", "2"}], "*", "\n", "         ", 
           RowBox[{"u0", "^", "4"}]}]}], ")"}]}], "]"}], "*", 
      RowBox[{"Sqrt", "[", "\n", "      ", 
       RowBox[{
        RowBox[{"-", "1"}], " ", "+", " ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", 
           RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
         RowBox[{"(", 
          RowBox[{"u", " ", "-", " ", "u0"}], ")"}], "*", "\n", "        ", 
         RowBox[{"(", 
          RowBox[{"u", " ", "+", " ", "u0"}], ")"}]}]}], "]"}], "*", 
      RowBox[{"Sqrt", "[", "\n", "      ", 
       RowBox[{"1", " ", "-", " ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"1", " ", "+", " ", 
             RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
           RowBox[{"u", "^", "2"}]}], ")"}], "/", "\n", "        ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"c0", "^", "2"}]}], " ", "+", " ", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1", " ", "+", " ", 
              RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", "\n", 
            "          ", 
            RowBox[{"u0", "^", "2"}]}]}], ")"}]}]}], "]"}], "*", 
      RowBox[{"EllipticE", "[", "\n", "      ", 
       RowBox[{
        RowBox[{"ArcSin", "[", 
         RowBox[{"u0", "*", 
          RowBox[{"Sqrt", "[", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"1", " ", "+", " ", 
                RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k"}], ")"}], "/", 
            "\n", "          ", 
            RowBox[{"(", 
             RowBox[{"1", " ", "+", " ", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"1", " ", "+", " ", 
                 RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", "\n", 
               "            ", 
               RowBox[{"u0", "^", "2"}]}]}], ")"}]}], "]"}]}], "]"}], ",", 
        " ", "\n", "      ", 
        RowBox[{"1", " ", "+", " ", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", " ", "+", " ", 
            RowBox[{"c0", "^", "2"}]}], ")"}], "/", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"c0", "^", "2"}]}], " ", "+", " ", "\n", "         ", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", " ", "+", " ", 
               RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
             RowBox[{"u0", "^", "2"}]}]}], ")"}]}]}]}], "]"}]}]}], ")"}], "/",
    "\n", "   ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Sqrt", "[", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", 
           RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k"}], ")"}], "/", "\n", 
       "      ", 
       RowBox[{"(", 
        RowBox[{"1", " ", "+", " ", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", " ", "+", " ", 
            RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
          RowBox[{"u0", "^", "2"}]}]}], ")"}]}], "]"}], "*", "\n", "    ", 
     RowBox[{"Sqrt", "[", 
      RowBox[{
       RowBox[{"-", "1"}], " ", "+", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", " ", "+", " ", 
          RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
        RowBox[{"(", 
         RowBox[{"u", " ", "-", " ", "u0"}], ")"}], "*", "\n", "       ", 
        RowBox[{"(", 
         RowBox[{"u", " ", "+", " ", "u0"}], ")"}]}]}], "]"}], "*", 
     RowBox[{"Sqrt", "[", "\n", "     ", 
      RowBox[{"1", " ", "-", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", " ", "+", " ", 
            RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
          RowBox[{"u", "^", "2"}]}], ")"}], "/", "\n", "       ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"c0", "^", "2"}]}], " ", "+", " ", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"1", " ", "+", " ", 
             RowBox[{"c0", "^", "2"}]}], ")"}], "*", "k", "*", 
           RowBox[{"u0", "^", "2"}]}]}], ")"}]}]}], "]"}]}], 
    ")"}]}]}]], "Input",
 CellChangeTimes->{{3.6167360784734097`*^9, 3.616736126459094*^9}, {
  3.6167361616995564`*^9, 3.6167361899824057`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"Chop", "[", 
    RowBox[{"Evaluate", "[", 
     RowBox[{
      RowBox[{"sfrev", "[", 
       RowBox[{"id", ",", 
        RowBox[{"-", 
         RowBox[{"fn1", "[", 
          RowBox[{
           RowBox[{"-", "1"}], ",", "1", ",", "1"}], "]"}]}]}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "]"}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0.6", ",", "1.5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"PlotPoints", " ", "\[Rule]", " ", "100"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6167305403076825`*^9, 3.6167307478972473`*^9}, {
   3.6167308107965574`*^9, 3.616730822480978*^9}, {3.616730915675542*^9, 
   3.6167309254723587`*^9}, 3.6167312361001043`*^9, {3.61673193398293*^9, 
   3.616731941361743*^9}, {3.616736247187706*^9, 3.6167362531781163`*^9}, 
   3.6167372556982775`*^9, {3.6167374562522297`*^9, 3.616737462866641*^9}, {
   3.6167393375375338`*^9, 3.6167393469131503`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"Chop", "[", 
    RowBox[{"Evaluate", "[", 
     RowBox[{
      RowBox[{"sfrev", "[", 
       RowBox[{"id", ",", 
        RowBox[{"fn1", "[", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "1", ",", "1"}], "]"}]}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "]"}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0.6", ",", "1.5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"PlotPoints", " ", "\[Rule]", " ", "100"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6167305403076825`*^9, 3.6167307478972473`*^9}, {
   3.6167308107965574`*^9, 3.616730822480978*^9}, {3.616730915675542*^9, 
   3.6167309254723587`*^9}, 3.6167312361001043`*^9, {3.61673193398293*^9, 
   3.616731941361743*^9}, {3.616736247187706*^9, 3.6167362531781163`*^9}, {
   3.616736370864723*^9, 3.616736383141945*^9}, {3.6167375152359333`*^9, 
   3.616737521881545*^9}, {3.6167392356849546`*^9, 3.616739283530239*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"%", ",", "%%"}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.61673673333136*^9, 3.6167367761222353`*^9}}],

Cell["\<\
This is a surface of revolution of constant negative curvature of cylindrical \
shape.\
\>", "Text",
 CellChangeTimes->{{3.616735074408846*^9, 3.6167352156982946`*^9}, {
   3.616736829817529*^9, 3.616736840097947*^9}, {3.616736877569213*^9, 
   3.6167368943860426`*^9}, 3.6167409857665224`*^9}],

Cell["\<\
Remark. Without Chop we get a small  gap in the near of u = 1:\
\>", "Text",
 CellChangeTimes->{{3.616739512538641*^9, 3.616739547155102*^9}, {
  3.6167396324041147`*^9, 3.6167396676201286`*^9}}],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"fn1", "[", 
     RowBox[{
      RowBox[{"-", "1"}], ",", "1", ",", "1"}], "]"}], "[", "u", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "4"}], ",", "4"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616736917396083*^9, 3.6167369687669735`*^9}, 
   3.616737339564025*^9}],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"fn1", "[", 
     RowBox[{
      RowBox[{"-", "1"}], ",", "1", ",", "1"}], "]"}], "[", "u", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", ".8", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616736917396083*^9, 3.6167369687669735`*^9}, 
   3.616737339564025*^9, {3.616738112280182*^9, 3.6167381739938903`*^9}, {
   3.6167382061299467`*^9, 3.6167382144759617`*^9}}],

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"fn1", "[", 
     RowBox[{
      RowBox[{"-", "1"}], ",", "1", ",", "1"}], "]"}], "[", "u", "]"}], ",", 
   " ", 
   RowBox[{"{", 
    RowBox[{"u", ",", ".980", ",", "1", ",", ".002"}], "}"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.616738267718855*^9, 3.616738352520604*^9}, {
  3.616738443577964*^9, 3.616738516352092*^9}, {3.616738649919526*^9, 
  3.616738679559578*^9}, {3.616738758012116*^9, 3.616738786372966*^9}}],

Cell["\<\
The reason for this are small numerical disturbances appearing as non-zero \
values of the imaginary part of fn1.  With Chop this gap vanishes:\
\>", "Text",
 CellChangeTimes->{{3.6167396986199017`*^9, 3.6167398206358805`*^9}, {
  3.6167403088608055`*^9, 3.616740313197054*^9}}],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"Chop", "[", 
    RowBox[{
     RowBox[{"fn1", "[", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "1", ",", "1"}], "]"}], "[", "u", "]"}], 
    "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", ".8", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616736917396083*^9, 3.6167369687669735`*^9}, 
   3.616737339564025*^9, {3.616738112280182*^9, 3.6167381739938903`*^9}, {
   3.6167382061299467`*^9, 3.6167382144759617`*^9}, {3.616739853455758*^9, 
   3.616739864927414*^9}}],

Cell["", "Text",
 CellChangeTimes->{{3.6167398968282385`*^9, 3.6167399110120497`*^9}, 
   3.6167403030844755`*^9}],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"Re", "[", 
    RowBox[{
     RowBox[{"fn1", "[", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "1", ",", "1"}], "]"}], "[", "u", "]"}], 
    "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "4"}], ",", "4"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616736917396083*^9, 3.6167369687669735`*^9}, {
   3.61673701240025*^9, 3.616737027750677*^9}, 3.6167373803892965`*^9}],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"Im", "[", 
    RowBox[{
     RowBox[{"fn1", "[", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "1", ",", "1"}], "]"}], "[", "u", "]"}], 
    "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "4"}], ",", "4"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616736917396083*^9, 3.6167369687669735`*^9}, {
   3.61673701240025*^9, 3.616737027750677*^9}, {3.616737059200332*^9, 
   3.61673706916875*^9}, 3.6167373899989133`*^9}],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"Re", "[", 
    RowBox[{
     RowBox[{"fn1", "[", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "1", ",", "1"}], "]"}], "[", "u", "]"}], 
    "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", ".7", ",", "1.2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616736917396083*^9, 3.6167369687669735`*^9}, {
   3.61673701240025*^9, 3.616737027750677*^9}, {3.616737059200332*^9, 
   3.61673706916875*^9}, 3.6167373899989133`*^9, {3.6167389102683835`*^9, 
   3.6167389627936754`*^9}, {3.6167390099525585`*^9, 3.616739015989769*^9}, {
   3.6167390809170833`*^9, 3.6167390811822834`*^9}}],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"Im", "[", 
    RowBox[{
     RowBox[{"fn1", "[", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "1", ",", "1"}], "]"}], "[", "u", "]"}], 
    "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", ".7", ",", "1.2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616736917396083*^9, 3.6167369687669735`*^9}, {
   3.61673701240025*^9, 3.616737027750677*^9}, {3.616737059200332*^9, 
   3.61673706916875*^9}, 3.6167373899989133`*^9, {3.6167389102683835`*^9, 
   3.6167389627936754`*^9}, {3.6167390099525585`*^9, 3.616739015989769*^9}, {
   3.6167390809170833`*^9, 3.6167390811822834`*^9}, {3.6167403595017023`*^9, 
   3.616740370496331*^9}}],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"Chop", "[", 
    RowBox[{"Im", "[", 
     RowBox[{
      RowBox[{"fn1", "[", 
       RowBox[{
        RowBox[{"-", "1"}], ",", "1", ",", "1"}], "]"}], "[", "u", "]"}], 
     "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", ".7", ",", "1.2"}], "}"}], ",", 
   RowBox[{"PlotStyle", " ", "\[Rule]", " ", "Red"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616736917396083*^9, 3.6167369687669735`*^9}, {
   3.61673701240025*^9, 3.616737027750677*^9}, {3.616737059200332*^9, 
   3.61673706916875*^9}, 3.6167373899989133`*^9, {3.6167389102683835`*^9, 
   3.6167389627936754`*^9}, {3.6167390099525585`*^9, 3.616739015989769*^9}, {
   3.6167390809170833`*^9, 3.6167390811822834`*^9}, {3.6167403595017023`*^9, 
   3.6167404183920703`*^9}}],

Cell["\<\
 Try with other parameters c0>0, k<0, u0. Depending on the initial conditions \
there may  appear surfaces of conical type also in case u0 \[NotEqual] 0\
\>", "Text",
 CellChangeTimes->{{3.616741006620715*^9, 3.616741020948535*^9}, {
   3.616741824790512*^9, 3.6167418861000185`*^9}, {3.616741923404152*^9, 
   3.6167419729889884`*^9}, 3.6186387530688257`*^9}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"Chop", "[", 
    RowBox[{"Evaluate", "[", 
     RowBox[{
      RowBox[{"sfrev", "[", 
       RowBox[{"id", ",", 
        RowBox[{"fn1", "[", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "1", ",", 
          RowBox[{"1", "/", "2"}]}], "]"}]}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "]"}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", ".8"}], ",", ".8"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"PlotPoints", " ", "\[Rule]", " ", "100"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6167305403076825`*^9, 3.6167307478972473`*^9}, {
   3.6167308107965574`*^9, 3.616730822480978*^9}, {3.616730915675542*^9, 
   3.6167309254723587`*^9}, 3.6167312361001043`*^9, {3.61673193398293*^9, 
   3.616731941361743*^9}, {3.616736247187706*^9, 3.6167362531781163`*^9}, {
   3.616736370864723*^9, 3.616736383141945*^9}, {3.6167375152359333`*^9, 
   3.616737521881545*^9}, {3.6167392356849546`*^9, 3.616739283530239*^9}, {
   3.6167410807459545`*^9, 3.616741133896995*^9}, {3.616741182321765*^9, 
   3.616741183848852*^9}, {3.616741481537879*^9, 3.6167414850570803`*^9}, {
   3.6167415796664915`*^9, 3.616741601689751*^9}, {3.6167416397939305`*^9, 
   3.6167416450332303`*^9}}],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"Re", "[", 
    RowBox[{
     RowBox[{"fn1", "[", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "1", ",", 
       RowBox[{"1", "/", "2"}]}], "]"}], "[", "u", "]"}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "3"}], ",", "3"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6167413030396695`*^9, 3.6167413615190144`*^9}}],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"Im", "[", 
    RowBox[{
     RowBox[{"fn1", "[", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "1", ",", 
       RowBox[{"1", "/", "2"}]}], "]"}], "[", "u", "]"}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "3"}], ",", "3"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6167413030396695`*^9, 3.6167413615190144`*^9}, {
  3.616741413770003*^9, 3.6167414366163096`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["3.9.6. Alfred Gray\[CloseCurlyQuote]s Exposition", "Subsubsection",
 CellChangeTimes->{{3.616761707824889*^9, 3.616761725544903*^9}, {
   3.6167618514484744`*^9, 3.616761855888728*^9}, 3.628085328540013*^9}],

Cell[TextData[{
 "In this subsection we reproduce Alfred Gray\[CloseCurlyQuote]s exposition, \
see [",
 ButtonBox["G94",
  BaseStyle->"Hyperlink",
  ButtonData->"Gray3"],
 "], p. 331-351. He considers surfaces of revolution in the form"
}], "Text",
 CellChangeTimes->{{3.616761864977248*^9, 3.6167619121849484`*^9}, {
  3.616762256056617*^9, 3.616762268648337*^9}, {3.616762315433013*^9, 
  3.6167623634807615`*^9}, {3.6167623992378063`*^9, 3.6167624170408244`*^9}, {
  3.616762447824585*^9, 3.616762528984227*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sfrev", "[", 
   RowBox[{"f", ",", "h"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6167625563717937`*^9, 3.616762562882166*^9}, {
  3.616762596579094*^9, 3.6167626241226687`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"gauss", "[", 
   RowBox[{"sfrev", "[", 
    RowBox[{"f", ",", "h"}], "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6168156415232797`*^9, 3.616815663410118*^9}}],

Cell[TextData[{
 "assuming that u is the arc length of the meridians: ",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox[
    RowBox[{
     SuperscriptBox["f", "\[Prime]",
      MultilineFunction->None], "[", "u", "]"}], "2"], " "}]],
  CellChangeTimes->{{3.616763093942541*^9, 3.616763101653982*^9}}],
 " + ",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox[
    RowBox[{
     SuperscriptBox["h", "\[Prime]",
      MultilineFunction->None], "[", "u", "]"}], "2"], " "}]],
  CellChangeTimes->{{3.616763093942541*^9, 3.616763101653982*^9}}],
 " = 1, we simplify"
}], "Text",
 CellChangeTimes->{{3.6167627698730054`*^9, 3.616762832176569*^9}, {
   3.61676314952072*^9, 3.616763207233021*^9}, {3.616763618840564*^9, 
   3.6167636504333706`*^9}, 3.616766019912897*^9, 3.616817422687608*^9, {
   3.616817534851805*^9, 3.6168175393758135`*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"%", "/.", 
    RowBox[{
     RowBox[{
      SuperscriptBox["h", "\[Prime]\[Prime]",
       MultilineFunction->None], "[", "u", "]"}], "->", " ", 
     RowBox[{
      RowBox[{"-", " ", 
       RowBox[{
        SuperscriptBox["f", "\[Prime]",
         MultilineFunction->None], "[", "u", "]"}]}], "*", 
      RowBox[{
       RowBox[{
        SuperscriptBox["f", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "u", "]"}], "/", 
       RowBox[{
        SuperscriptBox["h", "\[Prime]",
         MultilineFunction->None], "[", "u", "]"}]}]}]}]}], ",", 
   RowBox[{"{", " ", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["f", "\[Prime]",
          MultilineFunction->None], "[", "u", "]"}], "2"], " ", "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["h", "\[Prime]",
          MultilineFunction->None], "[", "u", "]"}], "2"]}], "  ", "==", 
      "1"}], ",", 
     RowBox[{"v", "\[Element]", " ", "Reals"}], ",", 
     RowBox[{
      RowBox[{"f", "[", "u", "]"}], ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616815692441769*^9, 3.616815734140642*^9}, {
   3.6168157725947104`*^9, 3.6168158224523973`*^9}, 3.6168158754300904`*^9, 
   3.616816933080748*^9, {3.6168169756220226`*^9, 3.61681703093972*^9}, {
   3.6168170698461885`*^9, 3.616817077193801*^9}, {3.616817155552739*^9, 
   3.6168172066272287`*^9}, 3.6168172513681073`*^9, {3.6168173069198046`*^9, 
   3.6168173135810165`*^9}, {3.6168173795067325`*^9, 3.6168173808639345`*^9}, 
   3.6168174826229134`*^9}],

Cell["\<\
We define simplification rules completed with the regularity condition f[u]>0\
\>", "Text",
 CellChangeTimes->{
  3.6168174336700277`*^9, {3.616817814872297*^9, 3.6168178310495253`*^9}, 
   3.618638761429304*^9}],

Cell[BoxData[
 RowBox[{"hfrules", " ", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["f", "\[Prime]",
         MultilineFunction->None], "[", "u", "]"}], 
       RowBox[{"2", " "}]], " ", "+", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["h", "\[Prime]",
         MultilineFunction->None], "[", "u", "]"}], "^", "2"}]}], " ", "==", 
     " ", "1"}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["h", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "u", "]"}], "*", 
       RowBox[{
        SuperscriptBox["h", "\[Prime]",
         MultilineFunction->None], "[", "u", "]"}]}], "+", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["f", "\[Prime]",
         MultilineFunction->None], "[", "u", "]"}], "*", 
       RowBox[{
        SuperscriptBox["f", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "u", "]"}]}]}], "\[Equal]", " ", 
     "0"}], ",", 
    RowBox[{
     RowBox[{"f", "[", "u", "]"}], ">", "0"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.616763093942541*^9, 3.616763101653982*^9}, {
  3.6167632193497143`*^9, 3.616763225317055*^9}, {3.616763584790616*^9, 
  3.616763593167095*^9}, {3.6167637325750685`*^9, 3.616763746788882*^9}, {
  3.616763851589876*^9, 3.616763855941125*^9}, {3.616766029566449*^9, 
  3.616766055494932*^9}, {3.6168168060341253`*^9, 3.6168168365945787`*^9}, {
  3.6168175677210627`*^9, 3.616817576035877*^9}, {3.6168176380615864`*^9, 
  3.6168177407253666`*^9}, {3.6168177737506247`*^9, 3.6168177833914413`*^9}}],

Cell["Corollary 18.9, p.333, is proved by the formulas", "Text",
 CellChangeTimes->{{3.616763273112789*^9, 3.6167633640009875`*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fund1", "[", 
      RowBox[{"sfrev", "[", 
       RowBox[{"f", ",", "h"}], "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"s1", ",", "s2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"s1", ",", "s2"}], "}"}]}], "]"}], ",", "hfrules"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6167633769727297`*^9, 3.616763411850724*^9}, {
   3.6167634592524357`*^9, 3.616763542812215*^9}, 3.6167636682603903`*^9, {
   3.6167637805648136`*^9, 3.616763782700936*^9}, 3.616766130622229*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fund2", "[", 
      RowBox[{"sfrev", "[", 
       RowBox[{"f", ",", "h"}], "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"s1", ",", "s2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"s1", ",", "s2"}], "}"}]}], "]"}], ",", "hfrules"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6167633769727297`*^9, 3.616763411850724*^9}, {
   3.6167634592524357`*^9, 3.616763542812215*^9}, 3.6167636682603903`*^9, {
   3.6167637805648136`*^9, 3.616763782700936*^9}, {3.6167638833616934`*^9, 
   3.616763891758174*^9}, {3.6167639523656397`*^9, 3.616763959157028*^9}, 
   3.6167640369744797`*^9, {3.6167661529985094`*^9, 3.6167661920377417`*^9}}],

Cell["Indeed, remember that we permuted u,v.", "Text",
 CellChangeTimes->{{3.6167658674081745`*^9, 3.61676593342495*^9}, 
   3.6167664090331535`*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"prcurvatures", "[", 
     RowBox[{"sfrev", "[", 
      RowBox[{"f", ",", "h"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", " ", "hfrules"}], "]"}]], "Input",
 CellChangeTimes->{{3.6167633769727297`*^9, 3.616763411850724*^9}, {
   3.6167634592524357`*^9, 3.616763542812215*^9}, 3.6167636682603903`*^9, {
   3.6167637805648136`*^9, 3.616763782700936*^9}, {3.6167638833616934`*^9, 
   3.616763891758174*^9}, {3.6167639523656397`*^9, 3.616763959157028*^9}, 
   3.6167640369744797`*^9, {3.616764252801824*^9, 3.6167642858527145`*^9}, {
   3.6167663533249674`*^9, 3.6167663604363737`*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"meancurvature", "[", 
     RowBox[{"sfrev", "[", 
      RowBox[{"f", ",", "h"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", " ", "hfrules"}], "]"}]], "Input",
 CellChangeTimes->{{3.6167633769727297`*^9, 3.616763411850724*^9}, {
   3.6167634592524357`*^9, 3.616763542812215*^9}, 3.6167636682603903`*^9, {
   3.6167637805648136`*^9, 3.616763782700936*^9}, {3.6167638833616934`*^9, 
   3.616763891758174*^9}, {3.6167639523656397`*^9, 3.616763959157028*^9}, 
   3.6167640369744797`*^9, {3.616764252801824*^9, 3.6167642858527145`*^9}, {
   3.6167663533249674`*^9, 3.6167663604363737`*^9}, {3.61676642925731*^9, 
   3.6167664451882215`*^9}}],

Cell["Consider", "Text",
 CellChangeTimes->{{3.6167671612251763`*^9, 3.6167671647043753`*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{
       SuperscriptBox["h", "\[Prime]",
        MultilineFunction->None], "[", "u", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        RowBox[{
         RowBox[{"f", "[", "u", "]"}], " ", 
         RowBox[{
          SuperscriptBox["f", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "u", "]"}]}]}], ")"}]}], 
     RowBox[{"f", "[", "u", "]"}]], "\[Equal]", " ", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["f", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "u", "]"}], "*", 
      RowBox[{
       SuperscriptBox["h", "\[Prime]",
        MultilineFunction->None], "[", "u", "]"}]}], "-", 
     FractionBox[
      RowBox[{
       SuperscriptBox["h", "\[Prime]",
        MultilineFunction->None], "[", "u", "]"}], 
      RowBox[{"f", "[", "u", "]"}]]}]}], ",", " ", "hfrules"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6167633769727297`*^9, 3.616763411850724*^9}, {
   3.6167634592524357`*^9, 3.616763542812215*^9}, 3.6167636682603903`*^9, {
   3.6167637805648136`*^9, 3.616763782700936*^9}, {3.6167638833616934`*^9, 
   3.616763891758174*^9}, {3.6167639523656397`*^9, 3.616763959157028*^9}, 
   3.6167640369744797`*^9, {3.616764252801824*^9, 3.6167642858527145`*^9}, {
   3.6167663533249674`*^9, 3.6167663604363737`*^9}, {3.61676642925731*^9, 
   3.6167664451882215`*^9}, {3.6167669272437935`*^9, 
   3.6167669744854956`*^9}, {3.616767010878577*^9, 3.616767044479499*^9}, 
   3.616767126944216*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"gauss", "[", 
     RowBox[{"sfrev", "[", 
      RowBox[{"f", ",", "h"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", " ", "hfrules"}], "]"}]], "Input",
 CellChangeTimes->{{3.6167633769727297`*^9, 3.616763411850724*^9}, {
   3.6167634592524357`*^9, 3.616763542812215*^9}, 3.6167636682603903`*^9, {
   3.6167637805648136`*^9, 3.616763782700936*^9}, {3.6167638833616934`*^9, 
   3.616763891758174*^9}, {3.6167639523656397`*^9, 3.616763959157028*^9}, 
   3.6167640369744797`*^9, {3.616764252801824*^9, 3.6167642858527145`*^9}, {
   3.6167663533249674`*^9, 3.6167663604363737`*^9}, {3.61676642925731*^9, 
   3.6167664451882215`*^9}, {3.616767194901102*^9, 3.6167672153722734`*^9}}],

Cell["\<\
\[FilledSquare]
Now we apply the last formula ans solve\
\>", "Text",
 CellChangeTimes->{{3.616819049895266*^9, 3.616819089846936*^9}}],

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     SuperscriptBox["f", "\[Prime]\[Prime]",
      MultilineFunction->None], "[", "u", "]"}], "\[Equal]", " ", 
    RowBox[{
     RowBox[{"-", "k"}], "*", 
     RowBox[{"f", "[", "u", "]"}]}]}], ",", 
   RowBox[{"f", "[", "u", "]"}], ",", "u"}], "]"}]], "Input",
 CellChangeTimes->{{3.6168183087535644`*^9, 3.6168183645548625`*^9}}],

Cell["Setting", "Text",
 CellChangeTimes->{{3.6168217096187725`*^9, 3.6168217128323784`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Sqrt", "[", "k", "]"}], " ", "=", 
   RowBox[{"1", "/", "a"}]}], ";", 
  RowBox[{
   RowBox[{"C", "[", "1", "]"}], " ", "=", 
   RowBox[{"b", "*", 
    RowBox[{"Cos", "[", "c", "]"}]}]}], ";", " ", 
  RowBox[{
   RowBox[{"C", "[", "2", "]"}], "=", " ", 
   RowBox[{
    RowBox[{"-", "b"}], "*", 
    RowBox[{"Sin", "[", "c", "]"}]}]}], ";"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6168217372464213`*^9, 3.6168218967567015`*^9}}],

Cell["\<\
leads to the general solution f[u] = b*Cos[u/a +c]; since the additive \
constant c means a translation along the z-axis  only, we may assume c = 0 \
and define as the solution for f\
\>", "Text",
 CellChangeTimes->{{3.616822046454564*^9, 3.61682214638834*^9}, {
   3.6168222017060375`*^9, 3.616822233389693*^9}, {3.6168222749682674`*^9, 
   3.616822329784403*^9}, {3.616822380673313*^9, 3.61682239320903*^9}, {
   3.616822686496805*^9, 3.6168227043848286`*^9}, 3.618638772788954*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fsol", "[", 
    RowBox[{"k_", ",", "b_"}], "]"}], "[", "u_", "]"}], " ", ":=", " ", 
  RowBox[{"b", "*", 
   RowBox[{"Cos", "[", 
    RowBox[{"u", "*", 
     RowBox[{"Sqrt", "[", "k", "]"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.616822420616598*^9, 3.6168225175151405`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox[
    RowBox[{"fsol", "[", 
     RowBox[{"k", ",", "b"}], "]"}], "\[Prime]\[Prime]",
    MultilineFunction->None], "[", "u", "]"}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{"-", "k"}], "*", 
   RowBox[{
    RowBox[{"fsol", "[", 
     RowBox[{"k", ",", "b"}], "]"}], "[", "u", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.616822784435407*^9, 3.616822831668109*^9}}],

Cell["\<\
Since changing the sign of fsol means a reflection at the y,z-plane only we \
may assume b>0;
the case b=0  degenerates to the z-axis, since h\.b4[u] = \[PlusMinus]1, h[u] \
= \[PlusMinus]u + c0:\
\>", "Text",
 CellChangeTimes->{{3.616822955943217*^9, 3.616823088680809*^9}, {
   3.6168232351451864`*^9, 3.616823284248995*^9}, 3.616823402856779*^9, {
   3.6168235962008376`*^9, 3.6168237090572925`*^9}, {3.616823977353638*^9, 
   3.616824004769206*^9}, {3.6168240378730993`*^9, 3.6168240444164743`*^9}, {
   3.6168240905691137`*^9, 3.61682411900174*^9}, {3.6168241508565617`*^9, 
   3.616824269849368*^9}, {3.616824377489525*^9, 3.6168243914573236`*^9}, {
   3.616824577209948*^9, 3.616824633353159*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"h1", "[", "u_", "]"}], " ", ":=", " ", 
   RowBox[{"u", "+", "1"}]}], ";", 
  RowBox[{"ParametricPlot3D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"sfrev", "[", 
      RowBox[{"zero", ",", "h1"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
    RowBox[{"{", 
     RowBox[{"u", ",", 
      RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"v", ",", "0", ",", 
      RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6168237141345825`*^9, 3.616823839297742*^9}, {
  3.616823883861291*^9, 3.61682393560425*^9}, {3.616824280708989*^9, 
  3.6168243321969337`*^9}}],

Cell["Analogously, the case k = 0  leads to ", "Text",
 CellChangeTimes->{{3.616824671409336*^9, 3.6168247003769927`*^9}, {
  3.6168247342969327`*^9, 3.6168247391772118`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"f0", "[", "b_", "]"}], "[", "u_", "]"}], " ", ":=", " ", "b"}], 
  ";", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"h0", "[", "c0_", "]"}], "[", "u_", "]"}], " ", ":=", " ", 
   RowBox[{"u", " ", "+", " ", "c0"}]}]}]], "Input",
 CellChangeTimes->{{3.616824747833707*^9, 3.6168248737079067`*^9}}],

Cell["and gives a circular cylinder", "Text",
 CellChangeTimes->{{3.616825074441388*^9, 3.6168250879691615`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sfrev", "[", 
     RowBox[{
      RowBox[{"f0", "[", 
       RowBox[{"1", "/", "2"}], "]"}], ",", 
      RowBox[{"h0", "[", "1", "]"}]}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6168237141345825`*^9, 3.616823839297742*^9}, {
  3.616823883861291*^9, 3.61682393560425*^9}, {3.616824280708989*^9, 
  3.6168243321969337`*^9}, {3.6168249119320927`*^9, 3.6168250119328127`*^9}}],

Cell["\<\
Remember the discussion of the developable surfaces of revolution at the \
beginning subsection 3.9.3. Thus we will suppose b>0 in the following, b ten \
is the maximal distance of the meridian to the z_axis. To find h[u] we \
remember\
\>", "Text",
 CellChangeTimes->{{3.6168257166171184`*^9, 3.616825794936598*^9}, {
   3.6168258277774763`*^9, 3.6168258771372995`*^9}, {3.6168266438661537`*^9, 
   3.6168266721447716`*^9}, {3.616826897673671*^9, 3.6168269301615295`*^9}, {
   3.6186387818694725`*^9, 3.6186388047167797`*^9}, 3.62808533275202*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"h\.b4", "[", "u", "]"}], " ", "\[Equal]", "  ", 
  RowBox[{"Sqrt", "[", 
   RowBox[{"1", "-", 
    RowBox[{
     RowBox[{"f\.b4", "[", "u", "]"}], "^", "2"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.616826684975505*^9, 3.6168267685152836`*^9}}],

Cell["with f\.b4[u] equal", "Text",
 CellChangeTimes->{{3.6168276321506805`*^9, 3.6168276803534374`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"fsol", "[", 
      RowBox[{"k", ",", "b"}], "]"}], "[", "t", "]"}], ",", "t"}], "]"}], "/.",
   " ", 
  RowBox[{"t", "\[Rule]", " ", "u"}]}]], "Input",
 CellChangeTimes->{{3.6168275053584285`*^9, 3.616827562586702*^9}}],

Cell["\<\
We assume h\.b4[u] \[GreaterEqual] 0; this can be reached always substituting \
u .> -u, if necessary.  Integration  \
\>", "Text",
 CellChangeTimes->{{3.6168273598091035`*^9, 3.6168274930327234`*^9}, 
   3.6168277062729197`*^9}],

Cell[BoxData[
 RowBox[{"Integrate", "[", 
  RowBox[{
   RowBox[{"Sqrt", "[", 
    RowBox[{"1", "-", 
     RowBox[{"k", "*", 
      RowBox[{"b", "^", "2"}], "*", 
      RowBox[{
       RowBox[{"Sin", "[", 
        RowBox[{
         RowBox[{"Sqrt", "[", "k", "]"}], "*", "t"}], "]"}], "^", "2"}]}]}], 
    "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", "u"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6168275053584285`*^9, 3.616827562586702*^9}, {
  3.6168277151474276`*^9, 3.616827777011966*^9}, {3.6168278227725835`*^9, 
  3.616827859483683*^9}, {3.61682790208412*^9, 3.616827911220642*^9}}],

Cell["\<\
We set the additive integration constant equal zero; another constant acts as \
a translation along the z-axis only, and define \
\>", "Text",
 CellChangeTimes->{{3.616828038160903*^9, 3.616828084720566*^9}, {
  3.616828184393267*^9, 3.616828257961475*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"hsol", "[", 
    RowBox[{"k_", ",", " ", "b_"}], "]"}], "[", "u_", "]"}], " ", ":=", " ", 
  "\n", "  ", 
  RowBox[{
   RowBox[{"EllipticE", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Sqrt", "[", "k", "]"}], "*", "u"}], ",", " ", 
     RowBox[{
      RowBox[{"b", "^", "2"}], "*", "k"}]}], "]"}], "/", 
   RowBox[{"Sqrt", "[", "k", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.616828268317067*^9, 3.6168283557230663`*^9}}],

Cell["Control:", "Text",
 CellChangeTimes->{{3.616828630960809*^9, 3.616828636009098*^9}, {
  3.6168286713051167`*^9, 3.616828702768916*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"gauss", "[", 
     RowBox[{"sfrev", "[", 
      RowBox[{
       RowBox[{"fsol", "[", 
        RowBox[{"k", ",", "b"}], "]"}], ",", 
       RowBox[{"hsol", "[", 
        RowBox[{"k", ",", "b"}], "]"}]}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "\[Element]", "Reals"}], ",", 
     RowBox[{"v", "\[Element]", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616828375117176*^9, 3.6168284650523195`*^9}, {
  3.6168285000513215`*^9, 3.6168285465979843`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sfrev", "[", 
   RowBox[{
    RowBox[{"fsol", "[", 
     RowBox[{"0", ",", "b"}], "]"}], ",", 
    RowBox[{"hsol", "[", 
     RowBox[{"0", ",", "b"}], "]"}]}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{3.616828801107541*^9}],

Cell["\<\
The developable surfaces are not contained in this family; the case k = b= 1 \
yields the unit sphere:\
\>", "Text",
 CellChangeTimes->{{3.616829005569236*^9, 3.6168290587852793`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sfrev", "[", 
     RowBox[{
      RowBox[{"fsol", "[", 
       RowBox[{"1", ",", "1"}], "]"}], ",", 
      RowBox[{"hsol", "[", 
       RowBox[{"1", ",", "1"}], "]"}]}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{
      RowBox[{"-", "Pi"}], "/", "2"}], ",", 
     RowBox[{"Pi", "/", "2"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6168287071561675`*^9, 3.6168287474044695`*^9}, {
  3.61682882626898*^9, 3.6168289348461905`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
3.9.6.a. Surfaces of Revolution of Constant Positive Gaussian Curvature\
\>", "Subsubsection",
 CellChangeTimes->{{3.6168293520900555`*^9, 3.6168294026009445`*^9}, {
   3.6168445489178305`*^9, 3.6168445611545305`*^9}, {3.617015288424196*^9, 
   3.6170153146076937`*^9}, {3.6170153724870043`*^9, 3.6170153975684385`*^9}, 
   3.6258046614906793`*^9, 3.629187774354987*^9}],

Cell["\<\
We assume k > 0 and distinguish three cases to obtain real solutions:\
\>", "Text",
 CellChangeTimes->{{3.616829477278216*^9, 3.6168295357855625`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"1.", " ", "b"}], "\[Equal]", 
   FractionBox["1", 
    SqrtBox["k"]]}], ",", " ", 
  RowBox[{
   RowBox[{"2.", "  ", "0"}], "<", "b", "<", 
   FractionBox["1", 
    SqrtBox["k"]]}], ",", 
  RowBox[{
   RowBox[{"3.", "  ", "b"}], ">", 
   SqrtBox["k"]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6168299083288703`*^9, 3.616829965910164*^9}}],

Cell["This integral must be real:", "Text",
 CellChangeTimes->{{3.616833170721469*^9, 3.616833187457426*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["\[Integral]", "0", "u"], 
   RowBox[{
    SqrtBox[
     RowBox[{"1", "-", 
      RowBox[{
       SuperscriptBox["b", "2"], " ", "k", " ", 
       RowBox[{
        SuperscriptBox["sin", "2"], "(", 
        RowBox[{
         SqrtBox["k"], " ", "t"}], ")"}]}]}]], 
    RowBox[{"\[DifferentialD]", "t"}]}]}], TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{3.6168301032840214`*^9}],

Cell[TextData[{
 StyleBox["Case 1. ",
  FontColor->RGBColor[0, 0, 1]],
 Cell[BoxData[
  RowBox[{
   SuperscriptBox["b", "2"], " ", "k"}]],
  Evaluatable->False,
  CellChangeTimes->{3.6168301032840214`*^9},
  FontWeight->"Bold",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox[" ",
  FontWeight->"Bold",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox["= 1. ",
  FontColor->RGBColor[0, 0, 1]],
 "\nThe integral is real for all values of t. In this case we have"
}], "Text",
 CellChangeTimes->{{3.616833199884137*^9, 3.616833220898339*^9}, {
   3.61683335012973*^9, 3.616833371824971*^9}, {3.6168335148811536`*^9, 
   3.616833533841238*^9}, {3.61683481034625*^9, 3.6168348377698183`*^9}, 
   3.6168350450976768`*^9, {3.616835086377038*^9, 3.6168351081372824`*^9}, 
   3.6168471838425083`*^9}],

Cell[BoxData[
 RowBox[{"Integrate", "[", 
  RowBox[{
   RowBox[{"Cos", "[", 
    RowBox[{
     RowBox[{"Sqrt", "[", "k", "]"}], "*", "t"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", "u"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.61683355485244*^9, 3.6168335917955527`*^9}, {
  3.6168336859399376`*^9, 3.6168337029559107`*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"EllipticE", "[", 
     RowBox[{"u", ",", "1"}], "]"}], "-", 
    RowBox[{"Sin", "[", "u", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"-", "Pi"}], "/", "2"}], "<", "u"}], ",", 
     RowBox[{"u", "<", 
      RowBox[{"Pi", "/", "2"}]}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616834449300599*^9, 3.6168345502833753`*^9}, {
  3.616834647315925*^9, 3.6168347160528564`*^9}, {3.6168347565251713`*^9, 
  3.616834757942252*^9}, {3.6168349378215413`*^9, 3.6168350062374544`*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sfrev", "[", 
     RowBox[{
      RowBox[{"fsol", "[", 
       RowBox[{
        RowBox[{"1", "/", 
         RowBox[{"r", "^", "2"}]}], ",", "r"}], "]"}], ",", 
      RowBox[{"hsol", "[", 
       RowBox[{
        RowBox[{"1", "/", 
         RowBox[{"r", "^", "2"}]}], ",", "r"}], "]"}]}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"-", "Pi"}], "/", "1"}], " ", "<", "u"}], ",", " ", 
     RowBox[{"u", "<", 
      RowBox[{"Pi", "/", "2"}]}], ",", " ", 
     RowBox[{"v", "\[Element]", "Reals"}], ",", 
     RowBox[{"r", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6168301493156543`*^9, 3.6168302037477674`*^9}, {
   3.6168302402198534`*^9, 3.6168303768376675`*^9}, 3.6168315516728644`*^9, {
   3.616832109871792*^9, 3.61683215251023*^9}, 3.616832244134471*^9, {
   3.6168323161585903`*^9, 3.6168323190147543`*^9}}],

Cell["here u has to be considered for  -Pi/2 < u/r <Pi/2.", "Text",
 CellChangeTimes->{{3.6168325282897234`*^9, 3.616832636000884*^9}}],

Cell[BoxData[
 RowBox[{"%", "//.", "  ", 
  RowBox[{"r", "\[Rule]", "2"}]}]], "Input",
 CellChangeTimes->{
  3.6168307265556707`*^9, 3.6168308348368635`*^9, 3.616830966524396*^9, {
   3.616830997092144*^9, 3.6168310186793785`*^9}, {3.61683105260732*^9, 
   3.6168310789158244`*^9}, {3.616832403811604*^9, 3.6168324092029123`*^9}}],

Cell["The solutions are spheres of radius r (=b).", "Text",
 CellChangeTimes->{{3.616832671361907*^9, 3.616832704545805*^9}}],

Cell[TextData[{
 StyleBox["Case 2.  ",
  FontColor->RGBColor[0, 0, 1]],
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["b", "2"], " ", "k"}], "<", "1"}], TraditionalForm]],
  CellChangeTimes->{{3.6168447609059553`*^9, 3.6168447853933563`*^9}},
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.6168446858536625`*^9, 3.616844702922639*^9}, {
  3.616844742825921*^9, 3.6168447467141438`*^9}, {3.61684479280978*^9, 
  3.616844810883814*^9}}],

Cell["\<\
In this case the integrand and the Integral are real for all values of t, \
resp. u.\
\>", "Text",
 CellChangeTimes->{{3.616844897241753*^9, 3.616844938938138*^9}, 
   3.6168451400106387`*^9}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sfrev", "[", 
     RowBox[{
      RowBox[{"fsol", "[", 
       RowBox[{"1", ",", 
        RowBox[{"1", "/", "2"}]}], "]"}], ",", 
      RowBox[{"hsol", "[", 
       RowBox[{"1", ",", 
        RowBox[{"1", "/", "2"}]}], "]"}]}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"3", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6168304350179954`*^9, 3.6168305159716253`*^9}, 
   3.6168306246138396`*^9, 3.616830700981208*^9, {3.61683123811693*^9, 
   3.6168312828994913`*^9}, {3.616831485061054*^9, 3.6168314905633693`*^9}, 
   3.6168316288302774`*^9, {3.616831870246086*^9, 3.6168318809086957`*^9}, 
   3.6168319324366426`*^9, {3.6168319719739046`*^9, 3.6168320241508884`*^9}, 
   3.616832444068907*^9, 3.616845214575904*^9, {3.616845276031419*^9, 
   3.6168453394710474`*^9}, 3.616845373118972*^9}],

Cell["\<\
These surfaces are infinite strings whose links have the shape of a spindle \
(or American football).
Experiment with other values of k,b.\
\>", "Text",
 CellChangeTimes->{{3.6168461611130114`*^9, 3.6168462607227087`*^9}, {
  3.6168462990349007`*^9, 3.616846328698597*^9}}],

Cell[TextData[{
 StyleBox["Case 3.  ",
  FontColor->RGBColor[0, 0, 1]],
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["b", "2"], " ", "k"}], ">", "1"}], TraditionalForm]],
  CellChangeTimes->{{3.6168447609059553`*^9, 3.6168447853933563`*^9}},
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.6168463583232913`*^9, 3.616846375572278*^9}}],

Cell["In this case the integrand is real for values t with", "Text",
 CellChangeTimes->{{3.616847201088495*^9, 3.616847246578097*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{"ArcSin", "[", 
     FractionBox["1", 
      RowBox[{"b", 
       SqrtBox["k"]}]], "]"}], 
    SqrtBox["k"]]}], "<", "t", "<", 
  FractionBox[
   RowBox[{"ArcSin", "[", 
    FractionBox["1", 
     RowBox[{"b", 
      SqrtBox["k"]}]], "]"}], 
   SqrtBox["k"]]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{
  3.6168475139593897`*^9, {3.616847759740448*^9, 3.6168477836828175`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", "\n", "  ", 
  RowBox[{
   RowBox[{
    RowBox[{"sfrev", "[", 
     RowBox[{
      RowBox[{"fsol", "[", 
       RowBox[{"1", ",", " ", "1.5"}], "]"}], ",", " ", 
      RowBox[{"hsol", "[", 
       RowBox[{"1", ",", " ", "1.5"}], "]"}]}], "]"}], "[", "\n", "   ", 
    RowBox[{"u", ",", " ", "v"}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"u", ",", " ", 
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{"ArcSin", "[", 
        RowBox[{"1", "/", "1.5"}], "]"}], ")"}]}], ",", " ", "\n", "   ", 
     RowBox[{"ArcSin", "[", 
      RowBox[{"1", "/", "1.5"}], "]"}]}], "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"v", ",", " ", "0", ",", " ", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6168304350179954`*^9, 3.6168305159716253`*^9}, 
   3.6168306246138396`*^9, 3.616830700981208*^9, {3.61683123811693*^9, 
   3.6168312828994913`*^9}, {3.616831485061054*^9, 3.6168314905633693`*^9}, 
   3.6168316288302774`*^9, {3.616831870246086*^9, 3.6168318809086957`*^9}, 
   3.6168319324366426`*^9, {3.6168319719739046`*^9, 3.6168320241508884`*^9}, 
   3.616832444068907*^9, 3.616845214575904*^9, {3.616845276031419*^9, 
   3.6168453394710474`*^9}, 3.616845373118972*^9, {3.616847657623607*^9, 
   3.6168476918065624`*^9}, {3.616847927586048*^9, 3.616847970522504*^9}, {
   3.6168486582318387`*^9, 3.6168486806661215`*^9}, {3.616849296034319*^9, 
   3.616849394718963*^9}}],

Cell["\<\
Entering larger limits for u  doesn't change the figure since only real \
points are plotted:\
\>", "Text",
 CellChangeTimes->{{3.616849593778349*^9, 3.616849646162345*^9}, {
  3.6186388573087873`*^9, 3.6186388846473513`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", "\n", "  ", 
  RowBox[{
   RowBox[{
    RowBox[{"sfrev", "[", 
     RowBox[{
      RowBox[{"fsol", "[", 
       RowBox[{"1", ",", " ", "1.5"}], "]"}], ",", " ", 
      RowBox[{"hsol", "[", 
       RowBox[{"1", ",", " ", "1.5"}], "]"}]}], "]"}], "[", " ", 
    RowBox[{"u", ",", " ", "v"}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"u", ",", " ", 
     RowBox[{"-", "5"}], ",", "5"}], "}"}], " ", ",", " ", 
   RowBox[{"{", 
    RowBox[{"v", ",", " ", "0", ",", " ", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.61684941687123*^9, 3.616849416921233*^9}, {
  3.6168494526232753`*^9, 3.616849541086335*^9}, {3.61684995066333*^9, 
  3.6168499651191573`*^9}}],

Cell["\<\
In this case the surface doesn\[CloseCurlyQuote]t meet the z-axis. The \
surfaces have the shape of a barrel.\
\>", "Text",
 CellChangeTimes->{{3.616849694275097*^9, 3.6168497543625336`*^9}, {
  3.616849820738899*^9, 3.616849824155094*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
3.9.6.b. Surfaces of Revolution of Constant Negative Gaussian Curvature\
\>", "Subsubsection",
 CellChangeTimes->{{3.6168293520900555`*^9, 3.6168294026009445`*^9}, {
   3.6168445489178305`*^9, 3.6168445611545305`*^9}, {3.616849902363568*^9, 
   3.616849913674215*^9}, {3.6170154058879147`*^9, 3.617015418936661*^9}, {
   3.625804683476118*^9, 3.6258046934289355`*^9}, 3.629187778676194*^9}],

Cell["\<\
Alfred Gray uses the following solution of the differential equation\
\>", "Text",
 CellChangeTimes->{{3.6169913911456676`*^9, 3.6169914250601273`*^9}}],

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"f", "''"}], "[", "u", "]"}], "-", 
     RowBox[{
      RowBox[{"f", "[", "u", "]"}], "/", 
      RowBox[{"a", "^", "2"}]}]}], "\[Equal]", "0"}], ",", 
   RowBox[{"f", "[", "u", "]"}], ",", "u"}], "]"}]], "Input",
 CellChangeTimes->{{3.616899810646412*^9, 3.616899899722568*^9}, 
   3.616899996115137*^9, {3.616900109948537*^9, 3.6169001203849554`*^9}, {
   3.6169007813113165`*^9, 3.616900806692561*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fsn", "[", 
    RowBox[{"a_", ",", "A_", ",", "B_"}], "]"}], "[", "u_", "]"}], " ", ":=", 
  " ", 
  RowBox[{
   RowBox[{"A", "*", 
    RowBox[{"Exp", "[", 
     RowBox[{"u", "/", "a"}], "]"}]}], " ", "+", " ", 
   RowBox[{"B", "*", 
    RowBox[{"Exp", "[", 
     RowBox[{
      RowBox[{"-", "u"}], "/", "a"}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.6169389368467135`*^9, 3.6169390613788366`*^9}, {
  3.6169390930036454`*^9, 3.6169391115237045`*^9}, {3.616998034301736*^9, 
  3.6169980472497587`*^9}}],

Cell[TextData[StyleBox["Case A=0, B>0.",
 FontColor->RGBColor[0, 0, 1]]], "Text",
 CellChangeTimes->{{3.6169397692433243`*^9, 3.6169398291707516`*^9}, {
  3.6169977852940984`*^9, 3.6169977862145*^9}}],

Cell["We transform the variable u:", "Text",
 CellChangeTimes->{{3.6169914945894494`*^9, 3.6169915299859114`*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"B", "*", 
     RowBox[{"Exp", "[", 
      RowBox[{
       RowBox[{"-", "u"}], "/", "a"}], "]"}]}], "/.", " ", 
    RowBox[{"u", "\[Rule]", 
     RowBox[{"w", "+", 
      RowBox[{"a", "*", 
       RowBox[{"Log", "[", "B", "]"}]}], "-", 
      RowBox[{"a", "*", 
       RowBox[{"Log", "[", "a", "]"}]}]}]}]}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"B", ">", "0"}], ",", 
     RowBox[{"a", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6169398777015276`*^9, 3.6169400590218983`*^9}, 
   3.616940481063038*^9}],

Cell["and set", "Text",
 CellChangeTimes->{{3.616991544103936*^9, 3.6169915524031506`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fsn1", "[", "a_", "]"}], "[", "u_", "]"}], ":=", 
  RowBox[{"a", "*", 
   RowBox[{"Exp", "[", 
    RowBox[{
     RowBox[{"-", "u"}], "/", "a"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6169405658588877`*^9, 3.616940632564703*^9}, 
   3.6169409437174997`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fsn1", "[", "a", "]"}], "'"}], "[", "u", "]"}]], "Input",
 CellChangeTimes->{{3.6169408458189006`*^9, 3.6169408906134624`*^9}}],

Cell["To obtain the h - function we calculate", "Text",
 CellChangeTimes->{{3.616991578002796*^9, 3.6169916349740963`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"hsn1", "[", "a_", "]"}], "[", "u_", "]"}], " ", ":=", " ", 
  RowBox[{"Integrate", "[", 
   RowBox[{
    RowBox[{"Sqrt", "[", 
     RowBox[{"1", "-", 
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{"-", 
        FractionBox[
         RowBox[{"2", "*", "t"}], "a"]}]]}], "]"}], ",", " ", 
    RowBox[{"{", 
     RowBox[{"t", ",", "0", ",", "u"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.616907470301609*^9, 3.616907561087802*^9}, {
  3.6169407225198483`*^9, 3.616940779999136*^9}, {3.616941000447745*^9, 
  3.616941006096068*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"hsn1", "[", "a", "]"}], "[", "u", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", ">", "0"}], ",", " ", 
     RowBox[{"u", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6169900368136888`*^9, 3.6169900521953163`*^9}, {
  3.616990255416873*^9, 3.616990312372573*^9}}],

Cell["We redefine", "Text",
 CellChangeTimes->{{3.616990656290777*^9, 3.616990661345186*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"hsn1", "[", "a_", "]"}], "[", "u_", "]"}], " ", ":=", " ", "\n", 
  "  ", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"-", "a"}], ")"}], "*", 
      RowBox[{"Sqrt", "[", 
       RowBox[{
        RowBox[{"-", "1"}], " ", "+", " ", "\n", "       ", 
        RowBox[{"E", "^", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"2", "*", "u"}], ")"}], "/", "a"}], ")"}]}]}], "]"}]}], 
     ")"}], "/", "\n", "    ", 
    RowBox[{"E", "^", 
     RowBox[{"(", 
      RowBox[{"u", "/", "a"}], ")"}]}]}], " ", "+", " ", "u", " ", "+", " ", 
   "\n", "   ", 
   RowBox[{"a", "*", 
    RowBox[{"Log", "[", 
     RowBox[{"1", " ", "+", " ", "\n", "      ", 
      RowBox[{
       RowBox[{"Sqrt", "[", 
        RowBox[{
         RowBox[{"-", "1"}], " ", "+", " ", 
         RowBox[{"E", "^", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"2", "*", "u"}], ")"}], "/", "a"}], ")"}]}]}], "]"}], 
       "/", "\n", "       ", 
       RowBox[{"E", "^", 
        RowBox[{"(", 
         RowBox[{"u", "/", "a"}], ")"}]}]}]}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.6169907012500563`*^9, 3.616990715492881*^9}}],

Cell["Try with  distinct parameters a", "Text",
 CellChangeTimes->{{3.616991665269349*^9, 3.616991726546257*^9}}],

Cell[BoxData[
 RowBox[{"pss1", "=", 
  RowBox[{"ParametricPlot3D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"sfrev", "[", 
      RowBox[{
       RowBox[{"fsn1", "[", ".2", "]"}], ",", 
       RowBox[{"hsn1", "[", ".2", "]"}]}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"u", ",", "0", ",", ".5"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"v", ",", "0", ",", 
      RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6169076266495514`*^9, 3.6169077454233446`*^9}, {
   3.616941068335628*^9, 3.616941081603387*^9}, 3.6169902080239897`*^9, {
   3.61699079488142*^9, 3.6169909004156055`*^9}, {3.6169926328322487`*^9, 
   3.61699263938426*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"gauss", "[", 
     RowBox[{"sfrev", "[", 
      RowBox[{
       RowBox[{"fsn1", "[", "a", "]"}], ",", 
       RowBox[{"hsn1", "[", "a", "]"}]}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", ">", "0"}], ",", " ", 
     RowBox[{"u", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6169917639863224`*^9, 3.616991862796896*^9}}],

Cell["\<\
Inverting the sign of the functions, clearly one gets a solution again\
\>", "Text",
 CellChangeTimes->{{3.616992879475482*^9, 3.616992947897202*^9}}],

Cell[BoxData[
 RowBox[{"pss2", "=", 
  RowBox[{"ParametricPlot3D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"sfrev", "[", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"fsn1", "[", ".2", "]"}]}], ",", 
       RowBox[{"-", 
        RowBox[{"hsn1", "[", ".2", "]"}]}]}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"u", ",", "0", ",", ".5"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"v", ",", "0", ",", 
      RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6169076266495514`*^9, 3.6169077454233446`*^9}, {
   3.616941068335628*^9, 3.616941081603387*^9}, 3.6169902080239897`*^9, {
   3.61699079488142*^9, 3.6169909004156055`*^9}, 3.616992481759183*^9, 
   3.6169926018349943`*^9, {3.6169926517706814`*^9, 3.6169926596018953`*^9}}],

Cell["<the surface is a pseudosphere:", "Text",
 CellChangeTimes->{{3.6169929865384703`*^9, 3.6169930012804956`*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"pss1", ",", "pss2"}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616992687151544*^9, 3.616992702626771*^9}, {
  3.616992735059228*^9, 3.6169927677948856`*^9}}],

Cell["", "Text"],

Cell["Changing both signs  acts as a reflection at the origin:", "Text",
 CellChangeTimes->{{3.616995201804761*^9, 3.616995245375637*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"sfrev", "[", 
    RowBox[{"f", ",", "h"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], " ", "+", " ", 
  RowBox[{
   RowBox[{"sfrev", "[", 
    RowBox[{
     RowBox[{"-", "f"}], ",", 
     RowBox[{"-", "h"}]}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], " "}]], "Input",
 CellChangeTimes->{{3.6169949972104015`*^9, 3.616995011780827*^9}}],

Cell["\<\
Changing the sign of f only acts as a reflection in the x, y - plane and \
yields the same surface :\
\>", "Text",
 CellChangeTimes->{{3.616995598435457*^9, 3.616995667247178*^9}, 
   3.6291879866203613`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sfrev", "[", 
   RowBox[{"f", ",", "h"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"sfrev", "[", 
   RowBox[{
    RowBox[{"-", "f"}], ",", "h"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6169954540104036`*^9, 3.616995462902419*^9}}],

Cell["\<\
Changing the sign at h only acts as a reflection at the x,y-plane:\
\>", "Text",
 CellChangeTimes->{{3.6169958078346252`*^9, 3.616995850719101*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"sfrev", "[", 
    RowBox[{"f", ",", 
     RowBox[{"-", "h"}]}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], " "}]], "Input",
 CellChangeTimes->{{3.6169949972104015`*^9, 3.616995011780827*^9}, {
   3.616995091543767*^9, 3.616995097331377*^9}, {3.616995129389434*^9, 
   3.616995140683853*^9}, 3.6169957434845123`*^9}],

Cell["\<\
The case B\[NotEqual]0, A=0 yields the same results by the variable transform 
u -> -u.\
\>", "Text",
 CellChangeTimes->{{3.616994639720174*^9, 3.6169946467869864`*^9}, {
   3.616994681824648*^9, 3.6169947215423174`*^9}, {3.6169948441117325`*^9, 
   3.6169948840166025`*^9}, 3.6169979055235095`*^9}],

Cell[TextData[{
 StyleBox["Case A",
  FontColor->RGBColor[0, 0, 1]],
 "\[NotEqual]",
 StyleBox["0, B",
  FontColor->RGBColor[0, 0, 1]],
 "\[NotEqual]",
 StyleBox["0.",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.6169397692433243`*^9, 3.6169398291707516`*^9}, {
  3.6169977852940984`*^9, 3.6169977862145*^9}, {3.616997819770159*^9, 
  3.6169978304093776`*^9}}],

Cell["We transform the variable ", "Text",
 CellChangeTimes->{{3.61699786607104*^9, 3.616997880563466*^9}}],

Cell[BoxData[
 RowBox[{"u", " ", "\[Rule]", " ", 
  RowBox[{"w", " ", "+", 
   RowBox[{
    RowBox[{"a", "/", "2"}], "*", 
    RowBox[{"Log", "[", 
     RowBox[{
      RowBox[{"Abs", "[", "B", "]"}], "/", 
      RowBox[{"Abs", "[", "A", "]"}]}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.6169979139163246`*^9, 3.6169979917916613`*^9}, 
   3.6169981497419386`*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fsn", "[", 
      RowBox[{"a", ",", "A", ",", "B"}], "]"}], "[", "u", "]"}], "/.", 
    RowBox[{"u", " ", "\[Rule]", " ", 
     RowBox[{"w", " ", "+", 
      RowBox[{
       RowBox[{"a", "/", "2"}], "*", 
       RowBox[{"Log", "[", 
        RowBox[{
         RowBox[{"Abs", "[", "B", "]"}], "/", 
         RowBox[{"Abs", "[", "A", "]"}]}], "]"}]}]}]}]}], ",", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", ">", "0"}], ",", " ", 
     RowBox[{"A", "\[NotEqual]", "0"}], ",", 
     RowBox[{"B", "\[NotEqual]", "0"}], ",", " ", 
     RowBox[{"w", "\[Element]", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616998096826646*^9, 3.616998136216715*^9}, {
  3.616998170255975*^9, 3.616998278083364*^9}}],

Cell[TextData[{
 "Thus we may assume Abs[A] == Abs[B].\n\n",
 StyleBox["Case A= B>0",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.6169983386894703`*^9, 3.6169983974703736`*^9}, {
   3.616998433646837*^9, 3.6169984349572396`*^9}, {3.616999099159606*^9, 
   3.6169991018428106`*^9}, 3.6170078639195557`*^9, 3.6170079205507946`*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"fsn", "[", 
     RowBox[{"a", ",", "A", ",", "A"}], "]"}], "[", "u", "]"}], "\[Equal]", 
   " ", 
   RowBox[{"2", "*", "A", "*", 
    RowBox[{"Cosh", "[", 
     RowBox[{"u", "/", "a"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616999248670269*^9, 3.616999372331686*^9}}],

Cell["Setting b= 2A we define", "Text",
 CellChangeTimes->{{3.6169994792074738`*^9, 3.616999497584306*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fsn2", "[", 
    RowBox[{"a_", ",", "b_"}], "]"}], "[", "u_", "]"}], " ", ":=", " ", 
  RowBox[{"b", "*", 
   RowBox[{"Cosh", "[", 
    RowBox[{"u", "/", "a"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6169993881501136`*^9, 3.6169994609554415`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fsn2", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "'"}], "[", "u", "]"}]], "Input",
 CellChangeTimes->{{3.616999512232732*^9, 3.6169995416387835`*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Integrate", "[", 
    RowBox[{
     RowBox[{"Sqrt", "[", 
      RowBox[{"1", "-", 
       RowBox[{
        RowBox[{"b", "^", "2"}], "*", 
        RowBox[{
         RowBox[{
          RowBox[{"Sinh", "[", 
           RowBox[{"t", "/", "a"}], "]"}], "^", "2"}], "/", 
         RowBox[{"a", "^", "2"}]}]}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", "0", ",", "u"}], "}"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"-", "a"}], "*", 
       RowBox[{"ArcSinh", "[", 
        RowBox[{"a", "/", "b"}], "]"}]}], "<", "u"}], ",", 
     RowBox[{"u", "<", 
      RowBox[{"a", "*", 
       RowBox[{"ArcSinh", "[", 
        RowBox[{"a", "/", "b"}], "]"}]}]}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6169996601209917`*^9, 3.6169999512955027`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"hsn2", "[", 
    RowBox[{"a_", ",", " ", "b_"}], "]"}], "[", "u_", "]"}], " ", ":=", " ", 
  "\n", "  ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", "I"}], ")"}], "*", "a", "*", 
   RowBox[{"EllipticE", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"I", "*", "u"}], ")"}], "/", "a"}], ",", " ", "\n", "    ", 
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"b", "^", "2"}], "/", 
        RowBox[{"a", "^", "2"}]}], ")"}]}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6170004523059826`*^9, 3.6170005089340825`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sfrev", "[", 
     RowBox[{
      RowBox[{"fsn2", "[", 
       RowBox[{"1", ",", ".2"}], "]"}], ",", 
      RowBox[{"hsn2", "[", 
       RowBox[{"1", ",", ".2"}], "]"}]}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", 
      RowBox[{"ArcSinh", "[", "5", "]"}]}], ",", 
     RowBox[{"ArcSinh", "[", "5", "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6169076266495514`*^9, 3.6169077454233446`*^9}, {
   3.616941068335628*^9, 3.616941081603387*^9}, 3.6169902080239897`*^9, {
   3.61699079488142*^9, 3.6169909004156055`*^9}, 3.616992481759183*^9, 
   3.6169926018349943`*^9, {3.6169926517706814`*^9, 3.6169926596018953`*^9}, {
   3.617000567075384*^9, 3.6170006327359*^9}, {3.617000673888772*^9, 
   3.6170006823439865`*^9}, 3.617000763510929*^9, {3.6170007966297874`*^9, 
   3.6170008213246307`*^9}}],

Cell["This is a surface of hyperboloidal type of  curvature -1:", "Text",
 CellChangeTimes->{{3.6170008768295283`*^9, 3.617000926858816*^9}, 
   3.6186391924136333`*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"gauss", "[", 
     RowBox[{"sfrev", "[", 
      RowBox[{
       RowBox[{"fsn2", "[", 
        RowBox[{"1", ",", 
         RowBox[{"2", "/", "10"}]}], "]"}], ",", 
       RowBox[{"hsn2", "[", 
        RowBox[{"1", ",", 
         RowBox[{"2", "/", "10"}]}], "]"}]}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", "\[Element]", " ", "Reals"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6170009298072214`*^9, 3.6170009576688704`*^9}, {
   3.6170009942197347`*^9, 3.61700104857023*^9}, {3.6170021872745433`*^9, 
   3.617002200114278*^9}, 3.6170076753987727`*^9}],

Cell[TextData[StyleBox["Case A = -B>0",
 FontColor->RGBColor[0, 0, 1]]], "Text",
 CellChangeTimes->{{3.6170078931032248`*^9, 3.617007928519251*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"fsn", "[", 
     RowBox[{"a", ",", "A", ",", 
      RowBox[{"-", "A"}]}], "]"}], "[", "u", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", ">", "0"}], ",", " ", 
     RowBox[{"A", ">", "0"}], ",", 
     RowBox[{"u", "\[Element]", " ", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.617008009144862*^9, 3.6170080423057585`*^9}, {
  3.6170081028262205`*^9, 3.6170081747313333`*^9}}],

Cell["Setting 2A = b we define", "Text",
 CellChangeTimes->{{3.6170082573680596`*^9, 3.617008298823431*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fsn3", "[", 
    RowBox[{"a_", ",", "b_"}], "]"}], "[", "u_", "]"}], " ", ":=", " ", 
  RowBox[{"b", "*", 
   RowBox[{"Sinh", "[", 
    RowBox[{"u", "/", "a"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.617008308011956*^9, 3.617008385794405*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fsn3", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "'"}], "[", "u", "]"}]], "Input",
 CellChangeTimes->{{3.6170083932528315`*^9, 3.617008410296807*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Integrate", "[", 
    RowBox[{
     RowBox[{"Sqrt", "[", 
      RowBox[{"1", " ", "-", " ", 
       RowBox[{
        RowBox[{"b", "^", "2"}], "*", 
        RowBox[{
         RowBox[{
          RowBox[{"Cosh", "[", 
           RowBox[{"t", "/", "a"}], "]"}], "^", "2"}], "/", 
         RowBox[{"a", "^", "2"}]}]}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", "0", ",", "u"}], "}"}]}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", ">", "0"}], ",", " ", 
     RowBox[{"b", ">", "0"}], ",", 
     RowBox[{"b", "\[LessEqual]", "a"}], ",", 
     RowBox[{"u", "\[Element]", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6170084693691854`*^9, 3.6170086802052445`*^9}, {
   3.617008986420759*^9, 3.6170090293732157`*^9}, 3.617009075739868*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"hsn3", "[", 
    RowBox[{"a_", ",", " ", "b_"}], "]"}], "[", "u_", "]"}], " ", ":=", " ", 
  "\n", "  ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", "I"}], ")"}], "*", 
   RowBox[{"Sqrt", "[", 
    RowBox[{
     RowBox[{"a", "^", "2"}], "-", 
     RowBox[{"b", "^", "2"}]}], "]"}], "*", "\n", "   ", 
   RowBox[{"EllipticE", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"I", "*", "u"}], ")"}], "/", "a"}], ",", " ", "\n", "    ", 
     RowBox[{
      RowBox[{"b", "^", "2"}], "/", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"a", "^", "2"}]}], " ", "+", " ", 
        RowBox[{"b", "^", "2"}]}], ")"}]}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6170131274146104`*^9, 3.6170131900961957`*^9}, {
   3.6170132542948675`*^9, 3.617013277140174*^9}, 3.617014579644656*^9}],

Cell["To get a real solution one has to regard the conditions", "Text",
 CellChangeTimes->{{3.6170133466241484`*^9, 3.617013365167209*^9}, {
  3.6170134139999847`*^9, 3.617013421839433*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", ">", "0"}], ",", 
   RowBox[{"b", ">", "0"}], ",", 
   RowBox[{"b", "\[LessEqual]", "a"}], ",", 
   RowBox[{"u", "\[LessEqual]", 
    RowBox[{"a", "*", 
     RowBox[{"ArcSinh", "[", 
      RowBox[{
       RowBox[{"Sqrt", "[", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"a", "^", "2"}], "-", 
          RowBox[{"b", "^", "2"}]}], ")"}], "]"}], "/", "b"}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "a"}], "*", 
     RowBox[{"ArcSinh", "[", 
      RowBox[{
       RowBox[{"Sqrt", "[", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"a", "^", "2"}], "-", 
          RowBox[{"b", "^", "2"}]}], ")"}], "]"}], "/", "b"}], "]"}]}], "<", 
    "u"}]}], "}"}]], "Input",
 CellChangeTimes->{{3.6170084693691854`*^9, 3.6170086802052445`*^9}, {
   3.617008986420759*^9, 3.6170090293732157`*^9}, 3.617009075739868*^9, {
   3.6170092980325823`*^9, 3.6170093610641875`*^9}, 3.6170133012135515`*^9, {
   3.6170134490309887`*^9, 3.617013501571994*^9}}],

Cell["Test with other parameters!", "Text",
 CellChangeTimes->{{3.617015088813779*^9, 3.6170151379445887`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sfrev", "[", 
     RowBox[{
      RowBox[{"fsn3", "[", 
       RowBox[{"1", ",", 
        RowBox[{"1", "/", "5"}]}], "]"}], ",", 
      RowBox[{"hsn3", "[", 
       RowBox[{"1", ",", 
        RowBox[{"1", "/", "5"}]}], "]"}]}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", " ", 
      RowBox[{"ArcSinh", "[", 
       SqrtBox["24"], "]"}]}], ",", " ", 
     RowBox[{"ArcSinh", "[", 
      SqrtBox["24"], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6169076266495514`*^9, 3.6169077454233446`*^9}, {
   3.616941068335628*^9, 3.616941081603387*^9}, 3.6169902080239897`*^9, {
   3.61699079488142*^9, 3.6169909004156055`*^9}, 3.616992481759183*^9, 
   3.6169926018349943`*^9, {3.6169926517706814`*^9, 3.6169926596018953`*^9}, {
   3.617000567075384*^9, 3.6170006327359*^9}, {3.617000673888772*^9, 
   3.6170006823439865`*^9}, 3.617000763510929*^9, {3.6170007966297874`*^9, 
   3.6170008213246307`*^9}, 3.6170145219403553`*^9, 3.617014597532679*^9, {
   3.617014628692461*^9, 3.617014645420418*^9}, {3.617014863190874*^9, 
   3.617014908262452*^9}}],

Cell["This is a surface of conical type with constant curvature", "Text",
 CellChangeTimes->{{3.6170149456085877`*^9, 3.6170149795755305`*^9}, 
   3.6186392130848155`*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"gauss", "[", 
     RowBox[{"sfrev", "[", 
      RowBox[{
       RowBox[{"fsn3", "[", 
        RowBox[{"1", ",", 
         RowBox[{"2", "/", "10"}]}], "]"}], ",", 
       RowBox[{"hsn3", "[", 
        RowBox[{"1", ",", 
         RowBox[{"2", "/", "10"}]}], "]"}]}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", "\[Element]", " ", "Reals"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6170009298072214`*^9, 3.6170009576688704`*^9}, {
   3.6170009942197347`*^9, 3.61700104857023*^9}, {3.6170021872745433`*^9, 
   3.617002200114278*^9}, 3.6170076753987727`*^9, {3.617015016756657*^9, 
   3.617015040035989*^9}}]
}, Closed]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["4. Absolute Differential and Christoffel Symbols", "Section",
 CellChangeTimes->{{3.5946282877913494`*^9, 3.594628341078397*^9}, 
   3.5946287521269073`*^9, {3.5947080886573257`*^9, 3.5947081111446123`*^9}, 
   3.598083945859253*^9, {3.598946107413493*^9, 3.598946113378834*^9}, {
   3.612849789938574*^9, 3.612849791376656*^9}, {3.6130221703369937`*^9, 
   3.613022178273448*^9}, 3.6130231613936787`*^9, {3.6167422011590385`*^9, 
   3.6167422040602045`*^9}, 3.617010591446561*^9, 3.6171012878039837`*^9, {
   3.625741401469241*^9, 3.62574140636765*^9}, 3.626535112124546*^9}],

Cell[TextData[{
 "In general, the differential and the derivatives of a tangent vector field \
on a surface are not tangent to the surface. The tangential component of the \
differential  of the vector field is called its ",
 StyleBox["absolute differential",
  FontColor->RGBColor[0, 0, 1]],
 "; we will calculate it in this section. The calculation leads to the ",
 StyleBox["Christoffel symbols",
  FontColor->RGBColor[0, 0, 1]],
 " which depend on the metric of the surface only and so belong to the inner \
geometry. These concepts play a fundamental role in the n-dimensional \
differential geometry of Riemannian manifolds and in Einstein\
\[CloseCurlyQuote]s general relativity theory. Therefore we begin this \
section considering m-dimensional submanifolds as immersions of m-dimensional \
parameter manifolds into the n-dimensional Euclidean space."
}], "Text",
 CellChangeTimes->{{3.5947073216646147`*^9, 3.5947073544090724`*^9}, {
   3.5947074072803535`*^9, 3.594707416416876*^9}, {3.594707450000797*^9, 
   3.594707526440169*^9}, {3.5947075823123646`*^9, 3.5947076930887003`*^9}, {
   3.5947077549602394`*^9, 3.5947077900162444`*^9}, {3.5947079466812053`*^9, 
   3.594708039528516*^9}, {3.5947081384651747`*^9, 3.5947082368408012`*^9}, {
   3.594708283920494*^9, 3.594708429712833*^9}, 3.594708506376218*^9, {
   3.594708537489998*^9, 3.5947085395871177`*^9}, 3.5949729165971603`*^9, {
   3.595133846274932*^9, 3.595133848630536*^9}, {3.5951339368798914`*^9, 
   3.5951339615747347`*^9}, {3.6130240656604*^9, 3.613024071877756*^9}, 
   3.6170751047813716`*^9, 3.61744219818544*^9, 3.6231320983087835`*^9}],

Cell[CellGroupData[{

Cell["\<\
4.1. The First Fundamental Form of an Immersion. Riemannian Metric\
\>", "Subsection",
 CellChangeTimes->{{3.5947089791212573`*^9, 3.5947089976963196`*^9}, {
   3.594883589600548*^9, 3.594883617945798*^9}, {3.594971429937128*^9, 
   3.5949714379675875`*^9}, 3.5949729180552435`*^9, {3.6234058500759945`*^9, 
   3.6234058518699975`*^9}, 3.6235877982564993`*^9}],

Cell[TextData[{
 "A smooth map f: ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["X", "m"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{"\[LongRightArrow]", " ", 
    SuperscriptBox["Y", "n"]}], TraditionalForm]]],
 ", m ",
 "\[LessEqual]",
 " n, of an m-dimensional manifold into an n-dimensional manifold is called \
an ",
 StyleBox["immersion",
  FontColor->RGBColor[0, 0, 1]],
 " if at any point x \[Element] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["X", "m"], TraditionalForm]]],
 " the differential "
}], "Text",
 CellChangeTimes->{{3.594709086392393*^9, 3.5947092776243305`*^9}, {
   3.5947093093291445`*^9, 3.5947095553792176`*^9}, {3.594709592893363*^9, 
   3.594709676260132*^9}, 3.594709830520955*^9, {3.594709960589394*^9, 
   3.594709960589394*^9}, {3.594710218598151*^9, 3.5947102185991516`*^9}, {
   3.6235878542449975`*^9, 3.62358786714622*^9}}],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["df", "x"], TraditionalForm]]],
 ": ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "x"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["X", "m"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[LongRightArrow]", " ", 
     SubscriptBox["T", 
      RowBox[{"f", "(", "x", ")"}]]}], 
    SuperscriptBox["Y", "n"]}], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.594709086392393*^9, 3.5947092776243305`*^9}, {
  3.5947093093291445`*^9, 3.5947095553792176`*^9}, {3.594709592893363*^9, 
  3.5947097207526765`*^9}}],

Cell[TextData[{
 "has maximal rank m. In the case m=n an immersion is named a",
 StyleBox[" local diffeomorphism",
  FontColor->RGBColor[0, 0, 1]],
 " a bijective immersion is then called a ",
 StyleBox["diffeomorphism",
  FontColor->RGBColor[0, 0, 1]],
 ". We name the image f(",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["X", "m"], TraditionalForm]]],
 ") of an immersion f  an ",
 StyleBox["m-dimensional submanifold",
  FontColor->RGBColor[0, 0, 1]],
 ", shortly an  ",
 StyleBox["m-surface",
  FontColor->RGBColor[0, 0, 1]],
 " in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["Y", "n"], TraditionalForm]]],
 ". Of course, for m=1 we speak about curves, and for m=2 about surfaces; ",
 StyleBox["hypersurfaces",
  FontColor->RGBColor[0, 0, 1]],
 " are submanifolds with m = n-1. For the following local considerations we \
assume that ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["Y", "n"], TraditionalForm]]],
 " = ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["E", "n"], TraditionalForm]]],
 " is the n-dimensional Euclidean space, and that  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["X", "m"], TraditionalForm]]],
 " = U is an open subset of the real vector space ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["R", "m"], TraditionalForm]]],
 " with Cartesian coordinates u[i], e.g."
}], "Text",
 CellChangeTimes->{{3.59470972972919*^9, 3.594709745073067*^9}, {
   3.59470993918517*^9, 3.594710073760867*^9}, {3.5947101193844767`*^9, 
   3.5947101879683995`*^9}, {3.59471027661747*^9, 3.5947103290484686`*^9}, {
   3.5947104053608336`*^9, 3.5947105040324774`*^9}, {3.5947105828459854`*^9, 
   3.5947106703929925`*^9}, {3.5947107725138335`*^9, 3.594710795201131*^9}, 
   3.59471086686523*^9, {3.594712076002389*^9, 3.5947120775364766`*^9}, {
   3.5947121190408506`*^9, 3.594712123329096*^9}, {3.5949713277202816`*^9, 
   3.594971351344633*^9}, 3.6174409203413515`*^9, {3.617442432065817*^9, 
   3.6174424531700244`*^9}, {3.617442959017957*^9, 3.6174429963860946`*^9}, {
   3.62358790107628*^9, 3.6235880783237915`*^9}, 3.623763201576983*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"m", "=", "3"}], ";", 
  RowBox[{
   RowBox[{"vec", "[", 
    RowBox[{"u", ",", "m"}], "]"}], " ", "\[Element]", " ", "U"}]}]], "Input",
 CellChangeTimes->{{3.594710822927717*^9, 3.5947108435468965`*^9}, {
  3.5947108983280296`*^9, 3.594710901706223*^9}, {3.5947109635867624`*^9, 
  3.594710990867323*^9}, {3.594711188715639*^9, 3.5947111972191253`*^9}, {
  3.5947112491310945`*^9, 3.594711271322364*^9}}],

Cell[TextData[{
 "For the dimension of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["Y", "n"], TraditionalForm]]],
 " we set  dim = n and define the coordinate representation of the immersion"
}], "Text",
 CellChangeTimes->{{3.5947121549129024`*^9, 3.5947122693124456`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", " ", 
   RowBox[{"m", ",", "n", ",", "dim", ",", "f", ",", "smf"}], "]"}], ";", 
  RowBox[{
   RowBox[{"smf", "[", "f_", "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"f", "[", "i", "]"}], "[", "##", "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "1", ",", "dim"}], "}"}]}], "]"}], "&"}]}], ";", " ", 
  RowBox[{"dim", " ", "=", " ", "4"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.594711374151245*^9, 3.5947114794592686`*^9}, {
   3.5947115852673206`*^9, 3.5947116073685846`*^9}, {3.594711639827441*^9, 
   3.594711671263239*^9}, {3.5947117575881767`*^9, 3.594711828853253*^9}, 
   3.594711998116934*^9, {3.5953194640529366`*^9, 3.5953194932676077`*^9}, {
   3.5953195538850746`*^9, 3.5953195709550505`*^9}, {3.5953196051960096`*^9, 
   3.5953196419811134`*^9}, {3.5955694749613805`*^9, 3.595569489786228*^9}, {
   3.5955695866347675`*^9, 3.5955696055618505`*^9}, {3.5955715900903587`*^9, 
   3.595571629682624*^9}, 3.614861861422343*^9, 3.614863501890024*^9, 
   3.617442593030024*^9, 3.6174431535420837`*^9, 3.6231340527618237`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"smf", "[", "f", "]"}], "[", 
  RowBox[{
   RowBox[{"t", "[", "1", "]"}], ",", 
   RowBox[{"t", "[", "2", "]"}], ",", 
   RowBox[{"t", "[", "3", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5955715213854294`*^9, 3.595571563568842*^9}, {
  3.5955716737611446`*^9, 3.595571675478243*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
  RowBox[{"x", ",", "y", ",", "z"}], "]"}]], "Input",
 CellChangeTimes->{{3.622975514380331*^9, 3.6229755568748055`*^9}}],

Cell[BoxData[
 RowBox[{"MatrixRank", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6235882514996953`*^9, 3.6235882772553406`*^9}}],

Cell[TextData[{
 StyleBox["Warning.",
  FontSlant->"Italic",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox[" Mathematica",
  FontSlant->"Italic"],
 " generically assume that the rank of a Function matrix is maximal;for \
concrete matrices this property needs a proof!"
}], "Text",
 CellChangeTimes->{{3.6235883066145926`*^9, 3.6235883315958357`*^9}, {
   3.6235887397904577`*^9, 3.6235888805313063`*^9}, 3.6237632074737935`*^9}],

Cell[TextData[{
 "The differential of the immersion smf[f]  at a point u has the matrix,  \
often named the ",
 StyleBox["Jacobian matrix",
  FontColor->RGBColor[0, 0, 1]],
 ":"
}], "Text",
 CellChangeTimes->{{3.5947126263848686`*^9, 3.594712689952505*^9}, {
   3.594713832024828*^9, 3.594713852521*^9}, 3.5947142700248795`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"jac", "[", "f_", "]"}], " ", ":=", 
  RowBox[{
   RowBox[{"Transpose", "[", " ", 
    RowBox[{
     RowBox[{"d", "[", "f", "]"}], "[", "##", "]"}], "]"}], "&"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.594713386074321*^9, 3.5947134565803537`*^9}, {
   3.5947136684854736`*^9, 3.5947136955560217`*^9}, {3.594878799409334*^9, 
   3.594878809003351*^9}, {3.59531976814933*^9, 3.5953197990600977`*^9}, {
   3.5955699854815807`*^9, 3.5955699916969357`*^9}, 3.5955700826891403`*^9, {
   3.595570128169742*^9, 3.595570177161544*^9}, {3.595570330490314*^9, 
   3.5955703838003635`*^9}, {3.595571733209545*^9, 3.595571767456504*^9}, 
   3.6148618614379425`*^9, 3.6149177932153635`*^9, 3.6231342894000406`*^9, 
   3.623134342974535*^9, 3.6231343934612236`*^9, {3.623134440761507*^9, 
   3.623134444630314*^9}}],

Cell[BoxData[
 RowBox[{"?", "jac"}]], "Input",
 CellChangeTimes->{{3.62314028648514*^9, 3.6231405273183627`*^9}, 
   3.623140582932461*^9, {3.6258056447520137`*^9, 3.6258056545176306`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"jac", "[", 
    RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
   RowBox[{"x", ",", "y", ",", "z"}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.623132306224349*^9, 3.6231323410164104`*^9}, {
   3.623132371424864*^9, 3.6231324023335185`*^9}, {3.62313247155484*^9, 
   3.62313248834247*^9}, {3.623132535612553*^9, 3.6231325774920273`*^9}, 
   3.623134473133564*^9, 3.623140264894702*^9}],

Cell[TextData[{
 StyleBox["Examples",
  FontColor->RGBColor[0, 0, 1]],
 ":"
}], "Text",
 CellChangeTimes->{{3.594713944363253*^9, 3.5947139681286125`*^9}, {
  3.623134525787657*^9, 3.623134529937264*^9}, {3.6231405953032827`*^9, 
  3.6231406050844994`*^9}}],

Cell[BoxData[
 RowBox[{"sf", "[", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5955700385656166`*^9, 3.5955700433108883`*^9}, {
  3.5955717961761465`*^9, 3.5955718060007086`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"jac", "[", "sf", "]"}], "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}]}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.594713474555382*^9, 3.5947135063091974`*^9}, 
   3.594713705411586*^9, {3.5955718260968575`*^9, 3.595571832905247*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "4"}], ";", 
  RowBox[{
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"smf", "[", "f", "]"}], "[", 
      RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"u", ",", "v", ",", "w"}], "}"}], "}"}]}], "]"}], "//", 
   "MatrixForm"}]}]], "Input",
 CellChangeTimes->{
  3.5947126945287666`*^9, {3.594712843322277*^9, 3.5947128956392694`*^9}, {
   3.5947129485562963`*^9, 3.5947129858044267`*^9}, 3.6231409108966846`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"jac", "[", 
    RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v", ",", "w"}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.623140829090141*^9, 3.6231408876682434`*^9}}],

Cell[TextData[{
 "The rows of  jac[smf[f]][u,...] are the coordinates of  mf = Length[{u}]",
 StyleBox[" tangent basis vectors",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.5948852693022985`*^9, 3.594885337053217*^9}, {
   3.594885368814873*^9, 3.5948853968481226`*^9}, {3.5948861634958687`*^9, 
   3.59488616987628*^9}, {3.5953198474808674`*^9, 3.595319851433093*^9}, 
   3.595596847808015*^9, {3.623140656252589*^9, 3.6231406633194017`*^9}}],

Cell[BoxData[
 RowBox[{"?", "tv"}]], "Input",
 CellChangeTimes->{{3.6258062304872465`*^9, 3.6258062321720495`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"tv", "[", "i_", "]"}], "[", "f_", "]"}], "[", "u__", "]"}], ":=",
    " ", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"IntegerQ", "[", "i", "]"}], "&&", 
      RowBox[{"i", ">", "0"}], "&&", 
      RowBox[{"i", "<=", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "u", "}"}], "]"}]}]}], ",", " ", "\n", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"Head", "[", 
          RowBox[{"f", "[", "u", "]"}], "]"}], "==", " ", "List"}], "&&", " ", 
        RowBox[{
         RowBox[{"Length", "[", 
          RowBox[{"f", "[", "u", "]"}], "]"}], ">", "1"}]}], ",", 
       RowBox[{
        RowBox[{"Transpose", "[", 
         RowBox[{
          RowBox[{"d", "[", "f", "]"}], "[", "u", "]"}], "]"}], "[", 
        RowBox[{"[", "i", "]"}], "]"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"d", "[", "f", "]"}], "[", "u", "]"}], "[", 
        RowBox[{"[", "i", "]"}], "]"}], ",", "\n", 
       RowBox[{
        RowBox[{
         RowBox[{"d", "[", "f", "]"}], "[", "u", "]"}], "[", 
        RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Wrong i = \>\"", ",", " ", "i"}], "]"}]}], "]"}]}], 
  ";"}]], "Input",
 Evaluatable->False],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"tv", "[", "2", "]"}], "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"jac", "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "[", 
   RowBox[{"[", "2", "]"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.594714541091384*^9, 3.5947145732912254`*^9}, {
   3.5947146047630253`*^9, 3.5947147040847063`*^9}, {3.5947147562046876`*^9, 
   3.5947148572694683`*^9}, {3.594878909670328*^9, 3.594878940402382*^9}, {
   3.594885417580559*^9, 3.594885424475771*^9}, {3.5953198901263065`*^9, 
   3.5953199276454525`*^9}, {3.5955706750110197`*^9, 
   3.5955707288821006`*^9}, {3.595571901346162*^9, 3.5955720037630196`*^9}, {
   3.5955720872657957`*^9, 3.595572121314743*^9}, {3.5955722913794703`*^9, 
   3.595572311092598*^9}, {3.595572402579831*^9, 3.5955724102522697`*^9}, {
   3.595572441820075*^9, 3.595572498956343*^9}, {3.5955725345643797`*^9, 
   3.595572831877385*^9}, 3.5955728836043434`*^9, 3.595572934572259*^9, {
   3.5955730768843985`*^9, 3.59557313163353*^9}, {3.595573173865946*^9, 
   3.5955731968172584`*^9}, {3.595573569185557*^9, 3.5955735798341656`*^9}, {
   3.625806292018156*^9, 3.6258063945389366`*^9}, 3.6258065815730667`*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->CompressedData["
1:eJwV0mtI02EYBfBtzqlptogyCYxERWzeCpqKaZNoyMxlmYsuKjJFzWWXkQpl
NrXWosGmJSi1WDTXUlvLZJiIRRfdcqUZZlqZlzX9m4ka4Zyj8354+HHgfDgf
nh15pYfz6TQaTYRbdDQ0jnlQPF2ETmqHabcDMryYFK9xv/gHcdnU88ofrlor
5thQXBN/YCNs7v1n4sBEzR1VFJTW17Oj4dk/gmRix5MXXcdh0LHVEhkcSiu/
qIT2hqaENqhcWhgahgxlUcs4FDXbZExPijfRKmNEwNzUursZJD8YKpLDWrUl
0gCdpQnTn6FjavcYnUXxtlYa4xiwUXRzxBO2sX75+8CWQ73lvpBmG9EQs43a
buLLYm6rH0yKXKOvh/pnkwFEyUR7MlEXGF9AHJyVXSWqb202E+M8bAPENXqt
i7gn9HusP2wvao4OIz3DeCpRLlGzwuGsNtaDAx3v+AohdEqMwkpoDJiSVsEQ
lncFkWX21RPTNeJOOdmTuUp7BFV7v123QlPCaMYKLAndZgv3oniXVHyvCFhn
6TxPNIR5OgTw5PM860EYskk1KoT6L+cmidOFoe5TMLxrg+AKdJflTmlhh6vq
qAFqit/mf4AFOYn9w5DRE1P4FypyZmpWYNDy7xQXTBq+XM30xi7qp18wtPcx
P3Fg5uP0OQHsrmU9zYJbrqVdEJGsPlEmhdwzg/c0cKFawc/ywR8c4e6SwpSH
7z/eh8H2mJ2v4Xy2zNwHnezB+SWonKk5vQLNsTf2uWEU981i4DqKN9Bv+bod
/geDDUMy
  "]]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tv", "[", "1", "]"}], "[", 
   RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.5947150536677017`*^9, 3.594715076507008*^9}, {
   3.6258059156658916`*^9, 3.6258059303611174`*^9}, 3.6258069533279223`*^9}],

Cell["\<\
The basis vector field tv[i][u,v,w] is the field of tangent vectors of the \
i-th coordinate line\
\>", "Text",
 CellChangeTimes->{{3.6235909145786867`*^9, 3.6235910952426043`*^9}, 
   3.6258067605187826`*^9, 3.625806814643278*^9}],

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"smf", "[", "f", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", "u"}], "]"}]], "Input",
 CellChangeTimes->{{3.6235910997354116`*^9, 3.6235911334626713`*^9}, 
   3.625806989085185*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tv", "[", 
    RowBox[{"-", "3"}], "]"}], "[", 
   RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.5947148725073395`*^9, 3.5947149377160697`*^9}, {
   3.5947149838037057`*^9, 3.5947149862728467`*^9}, {3.595573631609127*^9, 
   3.595573672792483*^9}, 3.6258068213824897`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tv", "[", "4", "]"}], "[", 
   RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.5947148725073395`*^9, 3.5947149377160697`*^9}, {
   3.5947149838037057`*^9, 3.5947149862728467`*^9}, {3.595573631609127*^9, 
   3.595573672792483*^9}, 3.6258068213824897`*^9, {3.625807060213711*^9, 
   3.625807068220525*^9}}],

Cell["\<\
To get the standard basis vector fields on the parameter manifold U we \
consider the identity function\
\>", "Text",
 CellChangeTimes->{
  3.623218571409417*^9, {3.62321866410478*^9, 3.623218743290519*^9}, {
   3.6232205419580784`*^9, 3.6232205534708986`*^9}, {3.6234021393998766`*^9, 
   3.6234021423014812`*^9}}],

Cell[BoxData[
 RowBox[{"?", "identity"}]], "Input",
 CellChangeTimes->{{3.6232183176593714`*^9, 3.623218328735391*^9}, {
  3.6232183670178585`*^9, 3.623218409403133*^9}, {3.625808351909989*^9, 
  3.6258083587448006`*^9}}],

Cell[BoxData[
 RowBox[{"identity", "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.623218415799144*^9, 3.6232184270623636`*^9}, {
  3.6235901130334764`*^9, 3.6235901154514804`*^9}}],

Cell[BoxData[
 RowBox[{"?", "\[Delta]u"}]], "Input",
 CellChangeTimes->{{3.6234024086875496`*^9, 3.6234024153331614`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tv", "[", "2", "]"}], "[", "identity", "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.6235916421633654`*^9, 3.6235916743774223`*^9}, 
   3.6236510505485196`*^9, 3.625806827731701*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Delta]u", "[", "2", "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.6232198810473175`*^9, 3.6232199227617908`*^9}, 
   3.6233820635357866`*^9, 3.6234022803149242`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Delta]u", "[", "4", "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.6232198810473175`*^9, 3.6232199227617908`*^9}, 
   3.6233820635357866`*^9, 3.6234022803149242`*^9, {3.625808600540027*^9, 
   3.6258086084492407`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"s1", "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{"s2", "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], " ", "}"}], "\[Equal]", " ", 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"s1", "[", 
     RowBox[{"u", ",", "v"}], "]"}], "*", 
    RowBox[{
     RowBox[{"\[Delta]u", "[", "1", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], " ", "+", 
   RowBox[{
    RowBox[{"s2", "[", 
     RowBox[{"u", ",", "v"}], "]"}], "*", 
    RowBox[{
     RowBox[{"\[Delta]u", "[", "2", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.623220292934841*^9, 3.623220442351904*^9}, {
   3.6233821187910833`*^9, 3.6233821254210954`*^9}, 3.623394381865259*^9, {
   3.623402346911441*^9, 3.6234023556318564`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "4"}], ";", 
  RowBox[{
   RowBox[{"d", "[", 
    RowBox[{
     RowBox[{"smf", "[", "f", "]"}], ",", "u", ",", "v", ",", "w"}], "]"}], 
   "[", 
   RowBox[{
    RowBox[{"\[Delta]u", "[", "2", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.623649818275008*^9, 3.6236498914547367`*^9}, {
  3.625808709314419*^9, 3.6258087135576262`*^9}}],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"tv", "[", "2", "]"}], "[", 
    RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v", ",", "w"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.623650074879859*^9, 3.6236500781558647`*^9}, {
   3.6236501122731247`*^9, 3.623650150009591*^9}, 3.6258068300405045`*^9}],

Cell[TextData[{
 "Thus we see\n",
 StyleBox["Lemma",
  FontColor->RGBColor[0, 0, 1]],
 ". The  basis vector fields \[Delta]u[i][u,v,...] of the parameter manifold \
U are mapped by  the differential of the immersion f onto the standard basis \
vector fields ",
 "tv[",
 "i][smf[f]][u,v,...]. It follows that the vectors with the same coordinates \
with respect to these bases correspond to each other under the differential \
of the immersion.\[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.6236502403493495`*^9, 3.6236502422213526`*^9}, {
   3.6236502782574162`*^9, 3.623650353168748*^9}, {3.62365039992203*^9, 
   3.6236506565268803`*^9}, {3.623650833283537*^9, 3.62365089192404*^9}, 
   3.6236512671715*^9, 3.623763249235067*^9, 3.625806832162109*^9}],

Cell[TextData[{
 "A ",
 StyleBox["tangent vector",
  FontColor->RGBColor[0, 0, 1]],
 " is a linear combination of the tangent basis vectors with the coordinates \
{s1,s2,...} \[Element] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["R", "m"], TraditionalForm]]],
 ". A ",
 StyleBox["tangent vector field",
  FontColor->RGBColor[0, 0, 1]],
 " is obtained if the coordinates are given as functions of the parameters :"
}], "Text",
 CellChangeTimes->{{3.594878538608076*^9, 3.5948785573125095`*^9}, {
   3.594886028196831*^9, 3.5948861049957657`*^9}, {3.5955967368876705`*^9, 
   3.5955967454471607`*^9}, {3.5959112371428165`*^9, 3.595911291181311*^9}, {
   3.5959113324745836`*^9, 3.595911408431117*^9}, {3.59591146137761*^9, 
   3.595911536491742*^9}, 3.5959116685147743`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"vv", "[", "\[Xi]_", "]"}], ":=", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Xi]", "[", "i", "]"}], "[", "##", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "##", "}"}], "]"}]}], "}"}]}], "]"}], "&"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5955963089271927`*^9, 3.595596389983829*^9}, {
   3.59559642496883*^9, 3.5955964744026575`*^9}, {3.59559657706553*^9, 
   3.595596688818921*^9}, {3.595596899914995*^9, 3.595596910010573*^9}, {
   3.5955969406903276`*^9, 3.5955969705550356`*^9}, {3.5955970131314707`*^9, 
   3.5955970157876225`*^9}, 3.6148618615003433`*^9, 3.6148635019056244`*^9, 
   3.6258098310601964`*^9}],

Cell[BoxData[
 RowBox[{"?", "vv"}]], "Input",
 CellChangeTimes->{{3.623141182697962*^9, 3.6231414076191573`*^9}, {
   3.6231414474148273`*^9, 3.623141539720189*^9}, 3.62314953740816*^9, {
   3.625809877347478*^9, 3.6258098893282986`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.595596485708304*^9, 3.595596552745138*^9}, {
  3.5955969877460194`*^9, 3.5955969919132576`*^9}}],

Cell[BoxData[
 RowBox[{"%", "==", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Delta]u", "[", "1", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], " ", "*", 
    RowBox[{
     RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "+", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"\[Delta]u", "[", "2", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "*", 
    RowBox[{
     RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.6236553641408057`*^9, 3.6236553680564127`*^9}, {
  3.6236554096304855`*^9, 3.6236555029810495`*^9}}],

Cell[TextData[{
 "Notice that this field is defined as a tangent vector field on the \
parameter manifold U. Its image under the differential of the immersion is a \
vector function on  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["R", "m"], TraditionalForm]]],
 " with values in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["E", "n"], TraditionalForm]]],
 ", e. g."
}], "Text",
 CellChangeTimes->{{3.5959117538781238`*^9, 3.5959119023591843`*^9}, {
   3.595911954619276*^9, 3.595911975195712*^9}, {3.5959121801956725`*^9, 
   3.595912259677812*^9}, 3.5960892898931456`*^9, 3.623655580669186*^9, 
   3.6237604876879764`*^9, {3.623760606001686*^9, 3.6237606248787193`*^9}, {
   3.6237606627867856`*^9, 3.6237606659847913`*^9}, {3.6237607313041058`*^9, 
   3.623760745484531*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", 
     RowBox[{"sf", ",", "u", ",", "v"}], "]"}], "[", 
    RowBox[{
     RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "]"}], "\[Equal]", " ", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"tv", "[", "i", "]"}], "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "*", 
      RowBox[{
       RowBox[{"\[Xi]", "[", "i", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "2"}], "}"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5959119820753245`*^9, 3.5959119820753245`*^9}, {
   3.595912052790249*^9, 3.5959121489800177`*^9}, {3.617779223413473*^9, 
   3.617779229123083*^9}, 3.6258068345197124`*^9}],

Cell[BoxData[
 RowBox[{"?", "tvf"}]], "Input",
 CellChangeTimes->{{3.623655892201733*^9, 3.623656206283085*^9}, 
   3.625806839108121*^9, {3.625809953468012*^9, 3.6258099614396257`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tvf", "[", "\[Xi]_", "]"}], "[", "ff_", "]"}], ":=", " ", 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"tv", "[", "i", "]"}], "[", "ff", "]"}], "[", "##", "]"}], 
      "*", 
      RowBox[{
       RowBox[{"\[Xi]", "[", "i", "]"}], "[", "##", "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "##", "}"}], "]"}]}], "}"}]}], "]"}], "&"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.594878617606615*^9, 3.5948787620316687`*^9}, 
   3.5948789981848836`*^9, {3.594879140737934*^9, 3.5948791442167397`*^9}, {
   3.594879176649197*^9, 3.5948793408394856`*^9}, {3.5948830153635397`*^9, 
   3.594883020963949*^9}, {3.594885506703515*^9, 3.594885512210325*^9}, {
   3.595322968315078*^9, 3.5953229945305777`*^9}, {3.59532303128568*^9, 
   3.595323059653302*^9}, {3.5955707720675707`*^9, 3.595570783931249*^9}, {
   3.5955708177081814`*^9, 3.5955708618827076`*^9}, {3.595570944371426*^9, 
   3.595570972020007*^9}, {3.5955737481397924`*^9, 3.5955738706988025`*^9}, {
   3.5955826733812866`*^9, 3.5955827043000555`*^9}, {3.6234046181574306`*^9, 
   3.6234046236486397`*^9}, {3.623579901734145*^9, 3.623579907131755*^9}, {
   3.6235800037115245`*^9, 3.623580008017132*^9}, 3.625806836844117*^9, 
   3.6258094752933683`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"tvf", "[", "\[Xi]", "]"}], "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{"d", "[", 
    RowBox[{"sf", ",", "u", ",", "v"}], "]"}], "[", 
   RowBox[{
    RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6235799730106707`*^9, 3.623579976832677*^9}, {
   3.6235800202943535`*^9, 3.6235800728352456`*^9}, {3.623580191801055*^9, 
   3.6235801931426573`*^9}, 3.62358022914752*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"dim", "=", " ", "4"}], ";", 
   RowBox[{
    RowBox[{
     RowBox[{"tvf", "[", "\[Xi]", "]"}], "[", 
     RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], 
  "\[IndentingNewLine]"}]], "Input",
 CellChangeTimes->{{3.594879354302309*^9, 3.594879363319125*^9}, {
   3.5948795443418427`*^9, 3.5948795935287294`*^9}, 3.5955708665119724`*^9, 
   3.6177792687627525`*^9, 3.6234047748129053`*^9, {3.6234051653439913`*^9, 
   3.6234051814120197`*^9}, {3.62357967113454*^9, 3.6235796763293495`*^9}, 
   3.6235802785060067`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{
    RowBox[{"smf", "[", "f", "]"}], ",", "u", ",", "v", ",", "w"}], "]"}], 
  "[", 
  RowBox[{
   RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
   RowBox[{"u", ",", "v", ",", "w"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6234048675082684`*^9, 3.62340494812921*^9}, {
   3.623405058452604*^9, 3.623405088747857*^9}, {3.623405211036472*^9, 
   3.6234052117852736`*^9}, {3.6234053308602824`*^9, 3.623405342061102*^9}, {
   3.6234056947621217`*^9, 3.6234057012829328`*^9}, {3.6235803733853736`*^9, 
   3.6235804249590645`*^9}, 3.6235805052056055`*^9}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", "-", "%%"}], "]"}]], "Input",
 CellChangeTimes->{{3.623405577044315*^9, 3.6234056031431603`*^9}}],

Cell["\<\
Tangent vectors of an m-surface are contained in the Euclidean n-space;  thus \
their scalar square and scalar products are defined:\
\>", "Text",
 CellChangeTimes->{{3.59488630057331*^9, 3.5948863791818476`*^9}, {
   3.623656329919303*^9, 3.623656349544137*^9}, 3.623763282962326*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"tvf", "[", "\[Eta]", "]"}], "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], ".", 
  RowBox[{
   RowBox[{
    RowBox[{"tvf", "[", "\[Eta]", "]"}], "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5948797188749495`*^9, 3.594879784707065*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"tvf", "[", "\[Xi]", "]"}], "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], ".", 
  RowBox[{
   RowBox[{
    RowBox[{"tvf", "[", "\[Eta]", "]"}], "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5948797188749495`*^9, 3.594879784707065*^9}, {
  3.594879824565135*^9, 3.5948798413663645`*^9}}],

Cell[TextData[{
 "This scalar product may be expressed by the ",
 StyleBox["first fundamental form ",
  FontColor->RGBColor[0, 0, 1]],
 "of the immersion, whose coordinates are defined by"
}], "Text",
 CellChangeTimes->{{3.5948864658712*^9, 3.5948865299093122`*^9}},
 CellTags->"fund1"],

Cell[BoxData[
 RowBox[{"?", "rgT"}]], "Input",
 CellChangeTimes->{{3.618641453594998*^9, 3.6186414580102506`*^9}, 
   3.6186415127083793`*^9, 3.6186416655541215`*^9}],

Cell["\<\
which we used already defining the first fundamental form of a surface.\
\>", "Text",
 CellChangeTimes->{{3.5948866048830442`*^9, 3.5948866612615433`*^9}, {
  3.596526134112959*^9, 3.596526134119959*^9}, {3.618641727448662*^9, 
  3.61864176011153*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"rgT", "[", 
    RowBox[{"1", ",", "2"}], "]"}], "[", 
   RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.595574517864818*^9, 3.5955745322806425`*^9}, 
   3.5955746182335587`*^9, {3.6186415960341454`*^9, 3.6186415985062866`*^9}, {
   3.625810531560831*^9, 3.6258105364592395`*^9}}],

Cell[BoxData[
 RowBox[{"?", "fund1"}]], "Input",
 CellChangeTimes->{{3.618641875834149*^9, 3.618641879318348*^9}, 
   3.623382204357234*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fund1", "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "[", 
  RowBox[{
   RowBox[{"vec", "[", 
    RowBox[{"s", ",", "2"}], "]"}], ",", 
   RowBox[{"vec", "[", 
    RowBox[{"t", ",", "2"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5948833076300526`*^9, 3.594883417126645*^9}, {
   3.5955757454070296`*^9, 3.5955757569936924`*^9}, 3.6186420187383223`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fund1", "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "[", 
  RowBox[{
   RowBox[{"vec", "[", 
    RowBox[{"s", ",", "4"}], "]"}], ",", 
   RowBox[{"vec", "[", 
    RowBox[{"t", ",", "2"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{3.5955757777128773`*^9, 3.5955767298893385`*^9, 
  3.5955769062254243`*^9, 3.6186426934259124`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "4"}], ";", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"tvf", "[", "\[Xi]", "]"}], "[", 
       RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v", ",", "w"}], "]"}], ".", 
     RowBox[{
      RowBox[{
       RowBox[{"tvf", "[", "\[Eta]", "]"}], "[", 
       RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], " ", "\[Equal]", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"fund1", "[", 
       RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v", ",", "w"}], "]"}], "[", 
     RowBox[{
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "i", "]"}], "[", 
         RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "3"}], "}"}]}], "]"}], ",", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Eta]", "[", "i", "]"}], "[", 
         RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "3"}], "}"}]}], "]"}]}], "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.594881401478305*^9, 3.594881598647051*^9}, {
   3.5948856462925606`*^9, 3.594885672282206*^9}, {3.594886933372821*^9, 
   3.594887200928891*^9}, {3.595576973645281*^9, 3.595576983931869*^9}, {
   3.5955828729667025`*^9, 3.595582949996108*^9}, {3.614861861547143*^9, 
   3.614861861547143*^9}, {3.614863501921224*^9, 3.614863501921224*^9}, 
   3.618642037261382*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "4"}], ";", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"tvf", "[", "\[Xi]", "]"}], "[", 
       RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v", ",", "w"}], "]"}], ".", 
     RowBox[{
      RowBox[{
       RowBox[{"tvf", "[", "\[Eta]", "]"}], "[", 
       RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], " ", "\[Equal]", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"fund1", "[", 
       RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v", ",", "w"}], "]"}], "[", 
     RowBox[{
      RowBox[{
       RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
       RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
      RowBox[{
       RowBox[{"vv", "[", "\[Eta]", "]"}], "[", 
       RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.594881401478305*^9, 3.594881598647051*^9}, {
   3.5948856462925606`*^9, 3.594885672282206*^9}, {3.594886933372821*^9, 
   3.594887200928891*^9}, {3.595576973645281*^9, 3.595576983931869*^9}, {
   3.5955828729667025`*^9, 3.595582949996108*^9}, {3.595912402636463*^9, 
   3.5959124815258017`*^9}, 3.61864269513101*^9}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fund1", "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{
     RowBox[{
      RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      RowBox[{"vv", "[", "\[Eta]", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "\[Equal]", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"tvf", "[", "\[Xi]", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ".", 
    RowBox[{
     RowBox[{
      RowBox[{"tvf", "[", "\[Eta]", "]"}], "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}]}], " ", "]"}]], "Input",
 CellChangeTimes->{{3.594881903097186*^9, 3.594881929071232*^9}, {
   3.594882044995035*^9, 3.59488209889313*^9}, {3.5948821483140163`*^9, 
   3.594882356324782*^9}, {3.5948857787367935`*^9, 3.5948858214808683`*^9}, 
   3.5948858581253324`*^9, {3.5948859196674404`*^9, 3.5948859277326546`*^9}, {
   3.5948859703207293`*^9, 3.594885975078738*^9}, {3.5955830629015665`*^9, 
   3.59558310002969*^9}, {3.595912582457979*^9, 3.595912628056859*^9}}],

Cell[TextData[{
 "The first fundamental form is a field of positive definite symmetric \
bilinear forms {\[Gamma]\[Gamma][i,j][u]}, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[Element]", 
    SuperscriptBox["X", "m"]}], TraditionalForm]]],
 ", where"
}], "Text",
 CellChangeTimes->{{3.5948875759067497`*^9, 3.5948877670538855`*^9}, 
   3.5949654316930485`*^9, 3.59496549099144*^9, {3.5949655491507664`*^9, 
   3.594965571823063*^9}, {3.594965717170377*^9, 3.5949657764947696`*^9}, {
   3.5949658208473063`*^9, 3.5949658766384974`*^9}, {3.5949659519598055`*^9, 
   3.5949659549429765`*^9}, {3.5949659930951586`*^9, 3.594966157918586*^9}, {
   3.5949661988769283`*^9, 3.59496644716713*^9}, {3.594966913414798*^9, 
   3.5949669701430426`*^9}, {3.5949716038810773`*^9, 
   3.5949716559840574`*^9}, {3.594971695811335*^9, 3.594971695811335*^9}, {
   3.5949717640392375`*^9, 3.594971771551667*^9}, {3.594971806319656*^9, 
   3.5949718596817083`*^9}, 3.5949729245216136`*^9, {3.595912681892554*^9, 
   3.5959127142470107`*^9}, {3.6130225413702154`*^9, 
   3.6130226045328283`*^9}, {3.6130226479333105`*^9, 
   3.6130227443038225`*^9}, {3.6130227886943617`*^9, 3.613022899241685*^9}, {
   3.613023003626655*^9, 3.613023014945303*^9}, {3.6170765536963167`*^9, 
   3.617076559561927*^9}, {3.6170773490949135`*^9, 3.617077383617774*^9}, {
   3.6170774835983496`*^9, 3.6170775130668015`*^9}, {3.6258117536571884`*^9, 
   3.6258117695380163`*^9}, 3.625811801834073*^9}],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\[Gamma]\[Gamma]", "[", 
        RowBox[{"i", ",", "j"}], "]"}], "[", "u", "]"}], "}"}], " ", ":"}], 
    " ", "=", "  ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"rgT", "[", 
        RowBox[{"i", ",", "j"}], "]"}], "[", "f", "]"}], "[", "u", "]"}], 
     "}"}]}], ",", "     ", 
   FormBox[
    RowBox[{"u", "\[Element]", 
     SuperscriptBox["X", "m"]}],
    TraditionalForm], ",", " "}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.617077552878072*^9, 3.617077649535841*^9}, 
   3.6186421722731037`*^9, 3.6186423423668327`*^9}],

Cell[TextData[{
 "is the  pulling back of the Euclidean  metric  from the m-subspace ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["df", "u"], TraditionalForm]]],
 "(",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "u"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["X", "m"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{")", " ", "\[Subset]", " ", 
    RowBox[{
     SubscriptBox["T", 
      RowBox[{"f", "(", "u", ")"}]], 
     SuperscriptBox["Y", "n"]}]}], TraditionalForm]]],
 " = ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["E", "n"], TraditionalForm]]],
 " by the differential ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["df", "u"], TraditionalForm]]],
 ". "
}], "Text",
 CellChangeTimes->{{3.5948875759067497`*^9, 3.5948877670538855`*^9}, 
   3.5949654316930485`*^9, 3.59496549099144*^9, {3.5949655491507664`*^9, 
   3.594965571823063*^9}, {3.594965717170377*^9, 3.5949657764947696`*^9}, {
   3.5949658208473063`*^9, 3.5949658766384974`*^9}, {3.5949659519598055`*^9, 
   3.5949659549429765`*^9}, {3.5949659930951586`*^9, 3.594966157918586*^9}, {
   3.5949661988769283`*^9, 3.59496644716713*^9}, {3.594966913414798*^9, 
   3.5949669701430426`*^9}, {3.5949716038810773`*^9, 
   3.5949716559840574`*^9}, {3.594971695811335*^9, 3.594971695811335*^9}, {
   3.5949717640392375`*^9, 3.594971771551667*^9}, {3.594971806319656*^9, 
   3.5949718596817083`*^9}, 3.5949729245216136`*^9, {3.595912681892554*^9, 
   3.5959127142470107`*^9}, {3.6130225413702154`*^9, 
   3.6130226045328283`*^9}, {3.6130226479333105`*^9, 
   3.6130227443038225`*^9}, {3.6130227886943617`*^9, 3.613022899241685*^9}, {
   3.613023003626655*^9, 3.613023014945303*^9}, {3.6170765536963167`*^9, 
   3.617076559561927*^9}, {3.6170773490949135`*^9, 3.617077383617774*^9}, 
   3.6170774835983496`*^9, {3.6170777038083363`*^9, 3.617077761263238*^9}, {
   3.6170784969761295`*^9, 3.617078513543359*^9}, {3.617078545913416*^9, 
   3.6170786173303413`*^9}, 3.6258118668921876`*^9, {3.625815875551713*^9, 
   3.625815876737315*^9}}],

Cell[TextData[{
 "If a field {\[Gamma]\[Gamma][i,j][u]} of symmetric non-degenerate bilinear \
forms on the manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["X", "m"], TraditionalForm]]],
 "is given  one names the pair \n[",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["X", "m"], TraditionalForm]]],
 ", \[Gamma]\[Gamma]] a ",
 StyleBox["pseudo-Riemannian manifold",
  FontColor->RGBColor[0, 0, 1]],
 ". With the scalar product defined by {\[Gamma]\[Gamma][i,j][u]}, any \
tangent space ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "u"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["X", "m"], TraditionalForm]]],
 " becomes an m-dimensional pseudo-Euclidean space; the norm of vectors and \
angles between vectors are defined by this scalar product. The index of the \
local scalar products  {\[Gamma]\[Gamma][i,j][u]}  is assumed to be constant; \
it is called the ",
 StyleBox["index",
  FontColor->RGBColor[0, 0, 1]],
 " of the pseudo-Riemannian manifold. The most important manifolds are those \
with index zero; they are called ",
 StyleBox["Riemannian manifolds",
  FontColor->RGBColor[0, 0, 1]],
 ". In particular the scalar products rgT induced by an immersion into a \
Euclidean space are positive definite and define Riemannian manifolds."
}], "Text",
 CellChangeTimes->{{3.5948875759067497`*^9, 3.5948877670538855`*^9}, 
   3.5949654316930485`*^9, 3.59496549099144*^9, {3.5949655491507664`*^9, 
   3.594965571823063*^9}, {3.594965717170377*^9, 3.5949657764947696`*^9}, {
   3.5949658208473063`*^9, 3.5949658766384974`*^9}, {3.5949659519598055`*^9, 
   3.5949659549429765`*^9}, {3.5949659930951586`*^9, 3.594966157918586*^9}, {
   3.5949661988769283`*^9, 3.59496644716713*^9}, {3.594966913414798*^9, 
   3.5949669701430426`*^9}, {3.5949716038810773`*^9, 
   3.5949716559840574`*^9}, {3.594971695811335*^9, 3.594971695811335*^9}, {
   3.5949717640392375`*^9, 3.594971771551667*^9}, {3.594971806319656*^9, 
   3.5949718596817083`*^9}, 3.5949729245216136`*^9, {3.595912681892554*^9, 
   3.5959127142470107`*^9}, {3.6130225413702154`*^9, 
   3.6130226045328283`*^9}, {3.6130226479333105`*^9, 
   3.6130227443038225`*^9}, {3.6130227886943617`*^9, 3.613022899241685*^9}, {
   3.613023003626655*^9, 3.613023014945303*^9}, {3.6170765536963167`*^9, 
   3.617076559561927*^9}, {3.6170773490949135`*^9, 3.617077383617774*^9}, {
   3.6170774835983496`*^9, 3.6170775130668015`*^9}, {3.6258117536571884`*^9, 
   3.6258117695380163`*^9}, 3.625811801834073*^9, {3.625811978816785*^9, 
   3.625811980501588*^9}, {3.6258120438768997`*^9, 3.625812054500518*^9}, {
   3.6258120938613873`*^9, 3.625812312019572*^9}, {3.6258159056773663`*^9, 
   3.6258159765512905`*^9}, {3.625816064881646*^9, 3.6258160695772543`*^9}, {
   3.625816122838748*^9, 3.6258161535396023`*^9}, {3.6258162567855835`*^9, 
   3.625816409453452*^9}, {3.625816475960369*^9, 3.6258165313580666`*^9}, {
   3.625816581249954*^9, 3.625816701779766*^9}, {3.62581676073427*^9, 
   3.625816773293292*^9}}],

Cell[TextData[{
 "Since the pseudo-Riemannian metric is non-degenerate,  the ",
 StyleBox["volume element  ",
  FontColor->RGBColor[0, 0, 1]],
 "dVol[ \[Gamma]\[Gamma]]",
 StyleBox[" ",
  FontColor->RGBColor[0, 0, 1]],
 "of  a pseudo-Riemannian manifold {",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["Y", "m"], TraditionalForm]]],
 ", \[Gamma]\[Gamma]}  is defined by (in eudiffgeov2.m)"
}], "Text",
 CellChangeTimes->{{3.5948875759067497`*^9, 3.5948877670538855`*^9}, 
   3.5949654316930485`*^9, 3.59496549099144*^9, {3.5949655491507664`*^9, 
   3.594965571823063*^9}, {3.594965717170377*^9, 3.5949657764947696`*^9}, {
   3.5949658208473063`*^9, 3.5949658766384974`*^9}, {3.5949659519598055`*^9, 
   3.5949659549429765`*^9}, {3.5949659930951586`*^9, 3.594966157918586*^9}, {
   3.5949661988769283`*^9, 3.59496644716713*^9}, {3.594966913414798*^9, 
   3.5949669701430426`*^9}, {3.5949716038810773`*^9, 
   3.5949716559840574`*^9}, {3.594971695811335*^9, 3.594971695811335*^9}, {
   3.5949717640392375`*^9, 3.594971771551667*^9}, {3.594971806319656*^9, 
   3.5949718596817083`*^9}, 3.5949729245216136`*^9, {3.595912681892554*^9, 
   3.5959127142470107`*^9}, {3.6130225413702154`*^9, 
   3.6130226045328283`*^9}, {3.6130226479333105`*^9, 
   3.6130227443038225`*^9}, {3.6130227886943617`*^9, 3.613022899241685*^9}, {
   3.613023003626655*^9, 3.613023014945303*^9}, {3.6170765536963167`*^9, 
   3.617076559561927*^9}, {3.6170773490949135`*^9, 3.617077383617774*^9}, 
   3.6170774835983496`*^9, {3.6170777038083363`*^9, 3.617077761263238*^9}, {
   3.6170784969761295`*^9, 3.617078513543359*^9}, {3.617078545913416*^9, 
   3.6170786173303413`*^9}, 3.6258118668921876`*^9, {3.625817552158662*^9, 
   3.6258175941091356`*^9}, {3.6258176391006145`*^9, 3.625817729435774*^9}, {
   3.6258178182175303`*^9, 3.625817837111163*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dVol", "[", "\[Gamma]\[Gamma]_", "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"m", ",", "i", ",", "j"}], "}"}], ",", 
     RowBox[{
      RowBox[{"m", " ", "=", " ", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "##", "}"}], "]"}]}], ";", 
      RowBox[{"Sqrt", "[", 
       RowBox[{"Abs", "[", 
        RowBox[{"Det", "[", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"\[Gamma]\[Gamma]", "[", 
             RowBox[{"i", ",", "j"}], "]"}], "[", "##", "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "m"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"j", ",", "m"}], "}"}]}], "]"}], "]"}], "]"}], "]"}]}]}], 
    "]"}], "&"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5721028398629103`*^9, 3.5721030771549273`*^9}, 
   3.5721031379326334`*^9, {3.586074746853404*^9, 3.5860747605491877`*^9}, {
   3.5860748158453503`*^9, 3.586074825492902*^9}, {3.5861549510774903`*^9, 
   3.586154964603264*^9}, {3.5862629631854534`*^9, 3.586262985017702*^9}, {
   3.5862630295142474`*^9, 3.5862630744648185`*^9}, {3.586604010786734*^9, 
   3.5866040355439777`*^9}, {3.594966564323831*^9, 3.5949665841789665`*^9}, {
   3.5949666975964537`*^9, 3.5949667458112116`*^9}, 3.5949668161242332`*^9, 
   3.594966903781247*^9, {3.595577348814739*^9, 3.5955774292283382`*^9}, {
   3.5955775238277493`*^9, 3.5955775729545593`*^9}, 3.5955776895562286`*^9, {
   3.614861861562743*^9, 3.614861861562743*^9}, 3.614863501936824*^9, 
   3.6149177932621636`*^9, {3.6170779225207205`*^9, 3.6170779402423515`*^9}, {
   3.6170779947644477`*^9, 3.617078006308468*^9}, {3.625817907532687*^9, 
   3.625817912181495*^9}, {3.62581797757981*^9, 3.6258180044898577`*^9}, {
   3.625818061994559*^9, 3.6258180965194197`*^9}, {3.6258186558124037`*^9, 
   3.625818718479714*^9}, 3.6258332272109594`*^9}],

Cell[BoxData[
 RowBox[{"?", "dVol"}]], "Input",
 CellChangeTimes->{{3.6258322301038055`*^9, 3.6258323374973946`*^9}, {
  3.6258324021937084`*^9, 3.6258324903514633`*^9}, {3.62583280084221*^9, 
  3.6258328070042205`*^9}}],

Cell["\<\
For immersion, or more general for pseudo-Riemannian manifolds of even index, \
the determinant of the bilinear form is always positive and the absolute \
function appearing in this definition may be omitted.  In particular, for \
immersions into Euclidean spaces the volume element may be defined by\
\>", "Text",
 CellChangeTimes->{{3.617078649513198*^9, 3.617078658857614*^9}, {
  3.6258183771069136`*^9, 3.6258186052185144`*^9}, {3.626855291683429*^9, 
  3.6268553122482653`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dV", "[", "ff_", "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"m", ",", "i", ",", "j"}], "}"}], ",", 
     RowBox[{
      RowBox[{"m", " ", "=", " ", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "##", "}"}], "]"}]}], ";", 
      RowBox[{"Sqrt", "[", 
       RowBox[{"Det", "[", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"rgT", "[", 
             RowBox[{"i", ",", "j"}], "]"}], "[", "ff", "]"}], "[", "##", 
           "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "m"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"j", ",", "m"}], "}"}]}], "]"}], "]"}], "]"}]}]}], "]"}], 
   "&"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5721028398629103`*^9, 3.5721030771549273`*^9}, 
   3.5721031379326334`*^9, {3.586074746853404*^9, 3.5860747605491877`*^9}, {
   3.5860748158453503`*^9, 3.586074825492902*^9}, {3.5861549510774903`*^9, 
   3.586154964603264*^9}, {3.5862629631854534`*^9, 3.586262985017702*^9}, {
   3.5862630295142474`*^9, 3.5862630744648185`*^9}, {3.586604010786734*^9, 
   3.5866040355439777`*^9}, {3.594966564323831*^9, 3.5949665841789665`*^9}, {
   3.5949666975964537`*^9, 3.5949667458112116`*^9}, 3.5949668161242332`*^9, 
   3.594966903781247*^9, {3.595577348814739*^9, 3.5955774292283382`*^9}, {
   3.5955775238277493`*^9, 3.5955775729545593`*^9}, 3.5955776895562286`*^9, {
   3.614861861562743*^9, 3.614861861562743*^9}, 3.614863501936824*^9, 
   3.6149177932621636`*^9, 3.6186421734671726`*^9, 3.6186423423708334`*^9, {
   3.625818195005393*^9, 3.6258181994670005`*^9}, {3.6258182295136538`*^9, 
   3.625818249528488*^9}}],

Cell[BoxData[
 RowBox[{"?", "dV"}]], "Input",
 CellChangeTimes->{{3.6258322301038055`*^9, 3.6258323374973946`*^9}, {
  3.6258324021937084`*^9, 3.6258324903514633`*^9}, {3.625832564594994*^9, 
  3.6258326752645884`*^9}, {3.62583293735345*^9, 3.6258329454030643`*^9}}],

Cell["Indeed, if \[Gamma]\[Gamma] is induced by an immersion smf,e.g.:", \
"Text",
 CellChangeTimes->{{3.617078676626045*^9, 3.617078760897393*^9}, {
   3.6258188542839527`*^9, 3.6258188724111843`*^9}, 3.6258334034806695`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "4"}], ";"}]], "Input",
 CellChangeTimes->{{3.6258190103994274`*^9, 3.62581901746624*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Gamma]sf", "[", 
    RowBox[{"i_", ",", "j_"}], "]"}], "[", 
   RowBox[{"u_", ",", "v_", ",", "w_"}], "]"}], ":=", "  ", 
  RowBox[{
   RowBox[{
    RowBox[{"rgT", "[", 
     RowBox[{"i", ",", "j"}], "]"}], "[", 
    RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v", ",", "w"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6170782340688677`*^9, 3.617078330508237*^9}, 
   3.6186421747842474`*^9, 3.618642342374833*^9, {3.625818771242007*^9, 
   3.6258187717880077`*^9}, {3.62581880727907*^9, 3.625818843439934*^9}, {
   3.6258191084017997`*^9, 3.6258191112254047`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"rgT", "[", 
    RowBox[{"1", ",", "1"}], "]"}], "[", 
   RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.6258190843143573`*^9, 3.625819086343361*^9}, {
  3.6258191323966417`*^9, 3.6258191372482505`*^9}}],

Cell["one has", "Text",
 CellChangeTimes->{{3.6170787827530317`*^9, 3.617078809553879*^9}, 
   3.625818885641008*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dVol", "[", "\[Gamma]sf", "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{3.625827948175063*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dV", "[", 
   RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Evaluate", "[", 
     RowBox[{"Det", "[", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Gamma]sf", "[", 
          RowBox[{"i", ",", "j"}], "]"}], "[", 
         RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "3"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"j", ",", "3"}], "}"}]}], "]"}], "]"}], "]"}], ">", "0"}], 
   ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"dVol", "[", "\[Gamma]sf", "]"}], "[", 
       RowBox[{"u", ",", "v", ",", "w"}], "]"}], "^", "2"}], " ", "\[Equal]", 
     " ", 
     RowBox[{
      RowBox[{
       RowBox[{"dV", "[", 
        RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v", ",", "w"}], "]"}], "^", "2"}]}], " ", "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6170783477618675`*^9, 3.617078415216386*^9}, {
   3.6258189088382483`*^9, 3.6258189615839415`*^9}, {3.625819192925748*^9, 
   3.6258192254538054`*^9}, 3.6258192694468827`*^9, {3.6258193283711863`*^9, 
   3.625819416467541*^9}, {3.6258243272556973`*^9, 3.62582434538173*^9}, {
   3.625824436614691*^9, 3.6258244606553335`*^9}, {3.625824533884862*^9, 
   3.6258245362092657`*^9}, {3.625828022791994*^9, 3.62582805438305*^9}, 
   3.6258281042271376`*^9, {3.6258281517302213`*^9, 3.625828256784806*^9}}],

Cell[TextData[{
 "after waiting a long time. Obviously, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has problems to verify this identity.\nThe area element of a surface is a \
special case of dV:"
}], "Text",
 CellChangeTimes->{{3.6170788288355126`*^9, 3.617078860394368*^9}, 
   3.617079026144659*^9, 3.6233823332446604`*^9, {3.625832093318765*^9, 
   3.625832164847891*^9}, {3.6258332563216105`*^9, 3.6258333000504875`*^9}, 
   3.626855323591485*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"dV", "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "\[Equal]", " ", 
   RowBox[{
    RowBox[{"dA", "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6170789222952766`*^9, 3.6170789370061026`*^9}, {
  3.617078990623397*^9, 3.6170790012158155`*^9}}],

Cell[TextData[{
 StyleBox["Example. ",
  FontColor->RGBColor[0, 0, 1]],
 "The volume of a 3-sphere in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["E", "4"], TraditionalForm]]],
 "."
}], "Text",
 CellChangeTimes->{{3.5949671433639503`*^9, 3.594967173390668*^9}, {
  3.5949684739990582`*^9, 3.5949685574148293`*^9}}],

Cell[BoxData[
 RowBox[{"?", "unit"}]], "Input",
 CellChangeTimes->{{3.5949685635881824`*^9, 3.594968565453289*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"unit", "[", "3", "]"}], "[", 
  RowBox[{"{", 
   RowBox[{"u", ",", "v", ",", "w"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.5949674880876675`*^9, 3.5949675122890515`*^9}}],

Cell[TextData[{
 "The hypersphere of radius r in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["E", "4"], TraditionalForm]]],
 " has the parameter representation"
}], "Text",
 CellChangeTimes->{{3.5949719474467278`*^9, 3.594972008231205*^9}, {
   3.5950507705444202`*^9, 3.5950507770028315`*^9}, 3.617441114508457*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"smh", "[", 
   RowBox[{"u_", ",", "v_", ",", "w_"}], "]"}], " ", "=", " ", 
  RowBox[{"r", "*", 
   RowBox[{
    RowBox[{"unit", "[", "3", "]"}], "[", 
    RowBox[{"{", 
     RowBox[{"u", ",", "v", ",", "w"}], "}"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5949675673111987`*^9, 3.594967639842347*^9}, {
  3.5955784262413645`*^9, 3.59557843927311*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Det", "[", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"rgT", "[", 
        RowBox[{"i", ",", "j"}], "]"}], "[", "smh", "]"}], "[", 
      RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "3"}], "}"}]}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.594967668807004*^9, 3.5949677562520056`*^9}, 
   3.5955784605383263`*^9, 3.6148618615783434`*^9, 3.614863501952424*^9, 
   3.618642175840308*^9, 3.618642342378834*^9}],

Cell[TextData[{
 "Indeed, all the upper - left submatrices have positive determinants, see ",
 ButtonBox["Math World",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://mathworld.wolfram.com/PositiveDefiniteMatrix.html"], None},
  ButtonNote->"http://mathworld.wolfram.com/PositiveDefiniteMatrix.html"],
 "."
}], "Text",
 CellChangeTimes->{{3.5949682501832566`*^9, 3.594968307255521*^9}, 
   3.5949683821998076`*^9, {3.5949684253162737`*^9, 3.594968425324274*^9}, 
   3.6174411573479075`*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Det", "[", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"rgT", "[", 
        RowBox[{"i", ",", "j"}], "]"}], "[", "smh", "]"}], "[", 
      RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "1"}], "}"}]}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5948873343559256`*^9, 3.5948874290636916`*^9}, 
   3.5949677892268915`*^9, {3.594967824028882*^9, 3.5949678300432262`*^9}, 
   3.5955785446821384`*^9, 3.614861861593943*^9, 3.6148635019680243`*^9, 
   3.6186421768573666`*^9, 3.6186423423828335`*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Det", "[", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"rgT", "[", 
        RowBox[{"i", ",", "j"}], "]"}], "[", "smh", "]"}], "[", 
      RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5948873343559256`*^9, 3.5948874290636916`*^9}, {
   3.5948874685785613`*^9, 3.5948875091386323`*^9}, 3.5949678425159397`*^9, 
   3.59557854662525*^9, 3.614861861609543*^9, 3.6148635019836245`*^9, 
   3.6186421777854195`*^9, 3.618642342387834*^9}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"dV", "[", "smh", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
   RowBox[{"r", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5950508613521795`*^9, 3.5950508830986176`*^9}, {
   3.5950509142830725`*^9, 3.5950509198678827`*^9}, 3.5955785474242954`*^9}],

Cell["The volume of the 3-sphere is obtained by an Integration:", "Text",
 CellChangeTimes->{{3.595051020410059*^9, 3.595051070501747*^9}, {
  3.6170769795926647`*^9, 3.6170769810434675`*^9}}],

Cell[BoxData[
 RowBox[{"?", "Integrate"}]], "Input",
 CellChangeTimes->{{3.5949685955800123`*^9, 3.5949686029754353`*^9}, 
   3.594970020744527*^9}],

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"r", ">", "0"}], ",", 
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", 
     RowBox[{"2", "*", "Pi"}]], 
    RowBox[{
     SubsuperscriptBox["\[Integral]", 
      RowBox[{
       RowBox[{"-", "Pi"}], "/", "2"}], 
      RowBox[{"Pi", "/", "2"}]], 
     RowBox[{
      SubsuperscriptBox["\[Integral]", 
       RowBox[{
        RowBox[{"-", "Pi"}], "/", "2"}], 
       RowBox[{"Pi", "/", "2"}]], 
      RowBox[{
       RowBox[{
        RowBox[{"dV", "[", "smh", "]"}], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}], 
       RowBox[{"\[DifferentialD]", "u"}], 
       RowBox[{"\[DifferentialD]", "v"}], 
       RowBox[{"\[DifferentialD]", "w"}]}]}]}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5949688159786186`*^9, 3.5949688817713814`*^9}, {
   3.5949689981080356`*^9, 3.5949690241395245`*^9}, {3.594969101804967*^9, 
   3.594969106379228*^9}, {3.5949691409732065`*^9, 3.594969173772083*^9}, 
   3.595578548961384*^9, {3.6130230655571976`*^9, 3.613023096004939*^9}}],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.6234030159098167`*^9, 3.6234030159098167`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["4.2. Absolute  Differential of Tangent Vector Fields", "Subsection",
 CellChangeTimes->{{3.5949730418953266`*^9, 3.5949731082711234`*^9}, {
   3.595077070522224*^9, 3.5950771362139397`*^9}, 3.613023169371135*^9, 
   3.6130232430183477`*^9, 3.6132201257627344`*^9, {3.6186449363901987`*^9, 
   3.6186449391503563`*^9}, 3.618719499291278*^9, {3.6237633284052057`*^9, 
   3.623763333506415*^9}, 3.624285203784027*^9, {3.62583352307448*^9, 
   3.625833526615686*^9}, 3.6260741531985583`*^9}],

Cell["\<\
We consider a general immersion smf[f[u...] into the n-dimensional Euclidean \
space, n = dim, e.g.\
\>", "Text",
 CellChangeTimes->{{3.595077178240413*^9, 3.595077208239266*^9}, {
   3.5951342404834337`*^9, 3.595134291859373*^9}, {3.595134540339585*^9, 
   3.595134634923995*^9}, {3.5953172126101613`*^9, 3.5953174521128597`*^9}, 
   3.596550784997245*^9, {3.596550826181317*^9, 3.596550833123329*^9}, {
   3.6128504259769535`*^9, 3.6128504380646443`*^9}, 3.6130232789364023`*^9, {
   3.6205365554152803`*^9, 3.620536569598091*^9}}],

Cell[BoxData[
 RowBox[{"?", "smf"}]], "Input",
 CellChangeTimes->{{3.6128502638746815`*^9, 3.612850266340822*^9}}],

Cell[TextData[StyleBox["Example.",
 FontColor->RGBColor[0, 0, 1]]], "Text",
 CellChangeTimes->{{3.613023386200537*^9, 3.6130233920648727`*^9}, {
  3.6205367297662525`*^9, 3.6205367311263304`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "4"}], ";", 
  RowBox[{
   RowBox[{"smf", "[", "f", "]"}], "[", 
   RowBox[{"u", ",", "v", ",", "w"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.612850301110811*^9, 3.612850367250594*^9}, {
   3.625893471726448*^9, 3.6258934976848936`*^9}, 3.627287506611168*^9}],

Cell["\<\
We fix the necessary assumptions and notations:  The immersion is \
smf[f][x[1],...,x[m]].\
\>", "Text",
 CellChangeTimes->{{3.595317712857774*^9, 3.595317728777684*^9}, {
  3.5953178514006977`*^9, 3.595317878616255*^9}, {3.5953179185605392`*^9, 
  3.595317922904788*^9}, {3.595318255584816*^9, 3.5953182694406085`*^9}, {
  3.5953234026079183`*^9, 3.5953234510956917`*^9}, {3.6130233386968203`*^9, 
  3.61302335163256*^9}, {3.62053664247826*^9, 3.620536666990662*^9}}],

Cell["\<\
The dimension of the parameter manifold is  mf = Length[{x}]. For the \
symbolic calculation we fix\
\>", "Text",
 CellChangeTimes->{{3.5953183285019865`*^9, 3.5953183859602733`*^9}, {
   3.5953185961762967`*^9, 3.5953186704725466`*^9}, {3.595319175793449*^9, 
   3.5953191760804653`*^9}, 3.595323351848015*^9, 3.5955692179186783`*^9, 
   3.612855570867403*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"dim", ",", "f"}], "]"}], ";", 
  RowBox[{"dim", "  ", "=", " ", "4"}], ";", 
  RowBox[{"mf", " ", "=", "2"}], ";", 
  RowBox[{
   RowBox[{"smf", "[", "f", "]"}], "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5953179973180437`*^9, 3.595318037226327*^9}, {
   3.595318077618637*^9, 3.5953182349966383`*^9}, {3.5953190274689655`*^9, 
   3.5953190679322796`*^9}, {3.5953233167050047`*^9, 3.595323324640459*^9}, 
   3.595918668773734*^9, {3.5959188539148593`*^9, 3.59591887143369*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "1", "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}]}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "2", "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}]}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "3", "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}]}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "4", "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}]}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6259912161217184`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["4.2.1. The Total Differential of a Tangent Vector Field", \
"Subsubsection",
 CellChangeTimes->{{3.595076945363204*^9, 3.5950769905720835`*^9}, 
   3.5953174793534184`*^9, {3.5953175263051033`*^9, 3.595317587064579*^9}, {
   3.595600920663969*^9, 3.5956009641674576`*^9}, {3.59560157753554*^9, 
   3.595601600231838*^9}, 3.613023249569722*^9, 3.613127844538868*^9, {
   3.6237634734230604`*^9, 3.6237634788362703`*^9}, 3.623765148910754*^9}],

Cell["\<\
One often needs the differentials of the coordinates which form mf fields of \
cobasis vectors:\
\>", "Text",
 CellChangeTimes->{{3.595324078111555*^9, 3.5953241174318037`*^9}, {
  3.595578890141898*^9, 3.5955789016865587`*^9}, {3.5955790255986457`*^9, 
  3.595579027053729*^9}, {3.6231300978748617`*^9, 3.623130169030587*^9}}],

Cell[BoxData[
 RowBox[{"?", "d"}]], "Input",
 CellChangeTimes->{{3.6231302219946804`*^9, 3.6231302227278814`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", "u", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], ".", 
  RowBox[{"vec", "[", 
   RowBox[{"s", ",", "2"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5951347602771645`*^9, 3.5951347844375467`*^9}, 
   3.5953240580574074`*^9, {3.6231303593921223`*^9, 3.6231303996129932`*^9}, {
   3.6231305640346823`*^9, 3.623130573316699*^9}, 3.623382546216234*^9, {
   3.6233825854971037`*^9, 3.6233825901771116`*^9}, {3.623382644683607*^9, 
   3.623382649597616*^9}, {3.623763568224427*^9, 3.62376357568124*^9}, 
   3.623763691215043*^9, 3.6237640462092667`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"v", ",", "u", ",", "v"}], "]"}], "[", 
  RowBox[{"vec", "[", 
   RowBox[{"s", ",", "2"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5951347602771645`*^9, 3.5951347844375467`*^9}, 
   3.5953240580574074`*^9, {3.6231303593921223`*^9, 3.6231303996129932`*^9}, {
   3.6231305640346823`*^9, 3.623130573316699*^9}, 3.623382546216234*^9, {
   3.6233825854971037`*^9, 3.6233825901771116`*^9}, {3.623382644683607*^9, 
   3.623382649597616*^9}, {3.623763568224427*^9, 3.62376357568124*^9}, 
   3.623763691215043*^9, 3.6237640462092667`*^9, {3.6258941443840294`*^9, 
   3.625894181839695*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", "v", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], ".", 
  RowBox[{"vec", "[", 
   RowBox[{"s", ",", "2"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5951347602771645`*^9, 3.5951347844375467`*^9}, 
   3.5953240580574074`*^9, {3.6231303593921223`*^9, 3.6231303996129932`*^9}, {
   3.6231305640346823`*^9, 3.6231305952991376`*^9}, {3.623382668473649*^9, 
   3.623382709922922*^9}, {3.6237637067838707`*^9, 3.623763710013076*^9}, 
   3.623764053135679*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{
    RowBox[{"x", "[", "4", "]"}], ",", 
    RowBox[{"x", "[", "1", "]"}], ",", " ", 
    RowBox[{"x", "[", "2", "]"}], ",", " ", 
    RowBox[{"x", "[", "3", "]"}], ",", " ", 
    RowBox[{"x", "[", "4", "]"}]}], "]"}], "[", 
  RowBox[{"vec", "[", 
   RowBox[{"u", ",", "4"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.623763770322782*^9, 3.6237638547657304`*^9}, {
   3.62376388871139*^9, 3.6237640096584024`*^9}, {3.625894278279064*^9, 
   3.6258942950802937`*^9}, 3.6258943272007504`*^9}],

Cell["We  consider the tangent vector field", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.5953235604669476`*^9, 3.595323569649472*^9}, {
   3.5953236189212904`*^9, 3.5953236679780965`*^9}, 3.595323711259572*^9, {
   3.5953239422667847`*^9, 3.5953239750656605`*^9}, {3.5955788337836742`*^9, 
   3.5955788569419994`*^9}, 3.613023988817005*^9}],

Cell[BoxData[
 RowBox[{"vx", ":=", " ", 
  RowBox[{
   RowBox[{"tvf", "[", "\[Xi]", "]"}], "[", 
   RowBox[{"smf", "[", "f", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5958287448867464`*^9, 3.595828758252511*^9}, {
  3.595828837669053*^9, 3.5958288760612493`*^9}}],

Cell[BoxData[
 RowBox[{"vx", "[", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5953239886694393`*^9, 3.595324018282133*^9}}],

Cell[TextData[{
 "The  ",
 StyleBox["total differential",
  FontColor->RGBColor[0, 0, 1]],
 " ",
 "d[vx][x[1],x[2]] ",
 "of the vector field vx is not tangential since it contains derivatives of \
second order of the immersion smf[f]:"
}], "Text",
 CellChangeTimes->{{3.5955793572066126`*^9, 3.5955793920946083`*^9}, {
   3.595579591878035*^9, 3.595579659893925*^9}, {3.595918977841477*^9, 
   3.595919013191139*^9}, {3.6131269223805656`*^9, 3.6131269457806063`*^9}, 
   3.61744117906815*^9, {3.623131398410351*^9, 3.623131400765955*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "vx", "]"}], "[", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.595324180746425*^9, 3.5953242505304165`*^9}}],

Cell[BoxData[
 RowBox[{"Dimensions", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.596089846720524*^9, 3.5960898684981613`*^9}}],

Cell["It may be identified with the built-in operation D", "Text",
 CellChangeTimes->{{3.612856097673535*^9, 3.6128561379208364`*^9}, {
   3.6128563706971507`*^9, 3.612856373881333*^9}, {3.623131453249448*^9, 
   3.6231314831879005`*^9}, 3.623764212879959*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", "vx", "]"}], "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}]}], "]"}], "==", 
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{"vx", "[", 
     RowBox[{
      RowBox[{"x", "[", "1", "]"}], ",", 
      RowBox[{"x", "[", "2", "]"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}]}], "}"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.595324180746425*^9, 3.5953242505304165`*^9}, {
  3.595324337972418*^9, 3.5953243435797386`*^9}}],

Cell["We introduce the vector of the coordinate differentials", "Text",
 CellChangeTimes->{{3.6237647328484726`*^9, 3.623764762036124*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"dx", "[", "1", "]"}], ",", 
   RowBox[{"dx", "[", "2", "]"}]}], "}"}]], "Input"],

Cell["\<\
The symbols dx[i] stand for the variable coordinates of a tangent vector of \
the parameter manifold U at the point {x[1],x[2]}.\
\>", "Text",
 CellChangeTimes->{{3.6237647935637794`*^9, 3.623764820661027*^9}, {
   3.6237649016815195`*^9, 3.623765002036496*^9}, 3.6258945242290964`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"vx", ",", 
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}]}], "]"}], "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"dx", "[", "1", "]"}], ",", 
    RowBox[{"dx", "[", "2", "]"}]}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.595324180746425*^9, 3.5953242505304165`*^9}, {
  3.595324337972418*^9, 3.5953243435797386`*^9}, {3.595579779459764*^9, 
  3.595579848568717*^9}}],

Cell["This coincides with", "Text",
 CellChangeTimes->{{3.5959193430693183`*^9, 3.5959193578893447`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "vx", "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}]}], "]"}], ".", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"dx", "[", "1", "]"}], ",", 
     RowBox[{"dx", "[", "2", "]"}]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.595324180746425*^9, 3.5953242505304165`*^9}, {
   3.595324337972418*^9, 3.5953243435797386`*^9}, {3.595579779459764*^9, 
   3.595579848568717*^9}, {3.595919143201768*^9, 3.59591915610299*^9}, {
   3.595919197489863*^9, 3.5959192019358706`*^9}, 3.625894658716933*^9}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", "==", "%%"}], "]"}]], "Input",
 CellChangeTimes->{{3.595919220655904*^9, 3.595919232387124*^9}, {
  3.6231295406692815`*^9, 3.623129560826517*^9}}],

Cell[BoxData[
 RowBox[{"Dimensions", "[", "%%", "]"}]], "Input",
 CellChangeTimes->{{3.595579998667302*^9, 3.5955800060637255`*^9}, 
   3.5959192788128057`*^9}],

Cell[TextData[{
 "This is a vector in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["E", "4"], TraditionalForm]]],
 " depending on the point smf[f][x[1],x[2]] of the immersion and on the \
tangent vector of the immersion defined by {dx[1],dx[2]} at this point"
}], "Text",
 CellChangeTimes->{{3.5955801071185055`*^9, 3.5955803651262627`*^9}, {
  3.613127181434941*^9, 3.6131271871302667`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{
    RowBox[{"smf", "[", "f", "]"}], ",", 
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}]}], "]"}], "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"dx", "[", "1", "]"}], ",", 
    RowBox[{"dx", "[", "2", "]"}]}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.5955803793400755`*^9, 3.595580438282447*^9}}],

Cell[TextData[{
 "Applying the differentiation rules we obtain the formal ",
 StyleBox["formula",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.5955984005678277`*^9, 3.595598438158978*^9}},
 CellTags->"dtt(1)"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", "1", ")"}], "   ", 
   RowBox[{"d", "[", "vx", "]"}]}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    UnderoverscriptBox["\[Sum]", "1", "mf"], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "[", 
       RowBox[{
        RowBox[{"tv", "[", "i", "]"}], "[", 
        RowBox[{"smf", "[", "f", "]"}], "]"}], "]"}], ")"}], "*", 
     RowBox[{"\[Xi]", "[", "i", "]"}]}]}], " ", "+", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"tv", "[", "i", "]"}], "[", 
     RowBox[{"smf", "[", "f", "]"}], "]"}], "*", 
    RowBox[{"d", "[", 
     RowBox[{"\[Xi]", "[", "i", "]"}], "]"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.595598463490427*^9, 3.5955984931051207`*^9}, {
   3.5955985511484404`*^9, 3.595598594362912*^9}, {3.5955986327071056`*^9, 
   3.5955986452108207`*^9}, {3.595598683170992*^9, 3.595598691723481*^9}, {
   3.595598737060074*^9, 3.595598913628173*^9}, 3.595599919420701*^9, 
   3.5956003320353017`*^9, {3.595920206461307*^9, 3.595920210028511*^9}, {
   3.6258068411361246`*^9, 3.6258068421813264`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"vv", "[", "s", "]"}], "[", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}]}], "]"}]], "Input"],

Cell["\<\
The left side of equation (1) evaluated for the tangent vector vv[s] is\
\>", "Text",
 CellChangeTimes->{{3.59559905624033*^9, 3.595599109839396*^9}, {
  3.6237644692070093`*^9, 3.6237644911874485`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", "vx", "]"}], "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}]}], "]"}], ".", 
  RowBox[{
   RowBox[{"vv", "[", "s", "]"}], "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5955923058844066`*^9, 3.595592346404724*^9}, {
  3.595597506634698*^9, 3.5955975114579735`*^9}, {3.595597557706619*^9, 
  3.595597587746337*^9}, {3.5955977418511515`*^9, 3.595597757699058*^9}}],

Cell["The sum of the first terms at the right side equals", "Text",
 CellChangeTimes->{{3.595599813111621*^9, 3.595599843183341*^9}, 
   3.5959198499349146`*^9}],

Cell[BoxData[
 RowBox[{"Sum", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"d", "[", 
        RowBox[{
         RowBox[{"tv", "[", "i", "]"}], "[", 
         RowBox[{"smf", "[", "f", "]"}], "]"}], "]"}], "[", 
       RowBox[{
        RowBox[{"x", "[", "1", "]"}], ",", 
        RowBox[{"x", "[", "2", "]"}]}], "]"}], ".", 
      RowBox[{
       RowBox[{"vv", "[", "s", "]"}], "[", 
       RowBox[{
        RowBox[{"x", "[", "1", "]"}], ",", 
        RowBox[{"x", "[", "2", "]"}]}], "]"}]}], ")"}], "*", 
    RowBox[{
     RowBox[{"\[Xi]", "[", "i", "]"}], "[", 
     RowBox[{
      RowBox[{"x", "[", "1", "]"}], ",", 
      RowBox[{"x", "[", "2", "]"}]}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "mf"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5956025399965897`*^9, 3.595602620998223*^9}, 
   3.595602703351933*^9, 3.625806843725729*^9}],

Cell[BoxData[
 RowBox[{"Dimensions", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.5956022073735647`*^9, 3.5956022187932177`*^9}}],

Cell["The second terms at the right side add up to", "Text",
 CellChangeTimes->{{3.5955999483073535`*^9, 3.595599979224122*^9}, {
   3.5956002120964413`*^9, 3.5956002139675484`*^9}, 3.5959199142395926`*^9}],

Cell[BoxData[
 RowBox[{"Sum", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", "i", "]"}], "[", 
      RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
     RowBox[{
      RowBox[{"x", "[", "1", "]"}], ",", 
      RowBox[{"x", "[", "2", "]"}]}], "]"}], "*", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"d", "[", 
        RowBox[{"\[Xi]", "[", "i", "]"}], "]"}], "[", 
       RowBox[{
        RowBox[{"x", "[", "1", "]"}], ",", 
        RowBox[{"x", "[", "2", "]"}]}], "]"}], ")"}], ".", 
     RowBox[{
      RowBox[{"vv", "[", "s", "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}]}], "]"}]}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "mf"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5956002571300173`*^9, 3.5956002597231655`*^9}, {
   3.5956003018525753`*^9, 3.595600311251113*^9}, {3.5956003819321556`*^9, 
   3.5956005168058696`*^9}, 3.625806845566532*^9}],

Cell[BoxData[
 RowBox[{"Dimensions", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.5956005282575245`*^9, 3.595600535904962*^9}}],

Cell[TextData[{
 "We check\n",
 StyleBox["Lemma",
  FontColor->RGBColor[0, 0, 1]],
 ". The total differential of a tangent vector field vx satisfies formula (1) \
above, for example"
}], "Text",
 CellChangeTimes->{{3.595600549015712*^9, 3.5956005533359594`*^9}, {
  3.5959200264320097`*^9, 3.5959200392477427`*^9}, {3.595920071487587*^9, 
  3.5959201138950124`*^9}, {3.5959202427443824`*^9, 3.5959202716950383`*^9}}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", "vx", "]"}], "[", 
     RowBox[{
      RowBox[{"x", "[", "1", "]"}], ",", 
      RowBox[{"x", "[", "2", "]"}]}], "]"}], ".", 
    RowBox[{
     RowBox[{"vv", "[", "s", "]"}], "[", 
     RowBox[{
      RowBox[{"x", "[", "1", "]"}], ",", 
      RowBox[{"x", "[", "2", "]"}]}], "]"}]}], "\[Equal]", "  ", 
   RowBox[{
    RowBox[{"Sum", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"d", "[", 
           RowBox[{
            RowBox[{"tv", "[", "i", "]"}], "[", 
            RowBox[{"smf", "[", "f", "]"}], "]"}], "]"}], "[", 
          RowBox[{
           RowBox[{"x", "[", "1", "]"}], ",", 
           RowBox[{"x", "[", "2", "]"}]}], "]"}], ".", 
         RowBox[{
          RowBox[{"vv", "[", "s", "]"}], "[", 
          RowBox[{
           RowBox[{"x", "[", "1", "]"}], ",", 
           RowBox[{"x", "[", "2", "]"}]}], "]"}]}], ")"}], "*", 
       RowBox[{
        RowBox[{"\[Xi]", "[", "i", "]"}], "[", 
        RowBox[{
         RowBox[{"x", "[", "1", "]"}], ",", 
         RowBox[{"x", "[", "2", "]"}]}], "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "mf"}], "}"}]}], "]"}], "+", " ", 
    RowBox[{"Sum", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"tv", "[", "i", "]"}], "[", 
         RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
        RowBox[{
         RowBox[{"x", "[", "1", "]"}], ",", 
         RowBox[{"x", "[", "2", "]"}]}], "]"}], "*", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"d", "[", 
           RowBox[{"\[Xi]", "[", "i", "]"}], "]"}], "[", 
          RowBox[{
           RowBox[{"x", "[", "1", "]"}], ",", 
           RowBox[{"x", "[", "2", "]"}]}], "]"}], ")"}], ".", 
        RowBox[{
         RowBox[{"vv", "[", "s", "]"}], "[", 
         RowBox[{
          RowBox[{"x", "[", "1", "]"}], ",", 
          RowBox[{"x", "[", "2", "]"}]}], "]"}]}]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "mf"}], "}"}]}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.595600564172579*^9, 3.5956005686658363`*^9}, {
   3.5956006043608775`*^9, 3.595600618411681*^9}, {3.595600654358737*^9, 
   3.5956006852045016`*^9}, 3.5956028655232086`*^9, 3.5956028962919683`*^9, {
   3.6258068469081345`*^9, 3.6258068477973366`*^9}}],

Cell["\<\
 In this formula the second derivatives of the immersion  are hidden, e,g,\
\>", "Text",
 CellChangeTimes->{{3.596090908988789*^9, 3.596090974852105*^9}}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"d", "[", 
    RowBox[{
     RowBox[{"tv", "[", "1", "]"}], "[", 
     RowBox[{"smf", "[", "f", "]"}], "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v", ",", "w"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5960904708776197`*^9, 3.5960905722621975`*^9}, {
   3.596090780007763*^9, 3.5960908043126054`*^9}, 3.6258068490765386`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["4.2.2. The Absolute Differential of a Tangent Vector Field", \
"Subsubsection",
 CellChangeTimes->{{3.595076945363204*^9, 3.5950769905720835`*^9}, 
   3.5953174793534184`*^9, {3.5953175263051033`*^9, 3.595317587064579*^9}, 
   3.5956008724322104`*^9, {3.5965213907316623`*^9, 3.5965214041788855`*^9}, 
   3.613127853739394*^9, 3.6132229815716324`*^9, 3.618719633997514*^9, 
   3.6187568869796286`*^9, 3.623765154089963*^9, 3.624285176327979*^9}],

Cell[TextData[{
 "For immersions the absolute differential will be defined as the orthogonal \
projection of the total differential of the vector field into the tangent \
space of the immersion at each point. First we describe this projection \
algebraically. Let ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["T", "x", "m"], TraditionalForm]]],
 " \[Subset] ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["E", "n"], " "}], TraditionalForm]]],
 "be an m.dimensional subspace of the Euclidean n-space, and {",
 Cell[BoxData[
  FormBox[
   SubscriptBox["b", "i"], TraditionalForm]]],
 "},  i = 1,...,m, an arbitrary basis of  ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["T", "x", "m"], TraditionalForm]]],
 " with scalar products"
}], "Text",
 CellChangeTimes->{{3.5956512820004816`*^9, 3.5956514597471943`*^9}, {
  3.59565153142932*^9, 3.595651640785512*^9}, {3.5956516917040014`*^9, 
  3.595651910665986*^9}, {3.5956520363086066`*^9, 3.595652063702255*^9}, {
  3.5956521682536383`*^9, 3.5956522065673056`*^9}, {3.5959237048705378`*^9, 
  3.595923706157612*^9}, {3.596521616120858*^9, 3.5965216273216777`*^9}}],

Cell[BoxData[
 RowBox[{
  SubscriptBox["g", "ij"], ":=", " ", 
  RowBox[{
   FormBox[
    SubscriptBox["b", "i"],
    TraditionalForm], ".", 
   FormBox[
    SubscriptBox["b", "j"],
    TraditionalForm]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5956521183335505`*^9, 3.595652155602016*^9}, {
  3.595652235962557*^9, 3.595652270813018*^9}, {3.5956523113418894`*^9, 
  3.595652327425517*^9}, {3.6238395209583635`*^9, 3.6238395286491766`*^9}, {
  3.623839642966178*^9, 3.6238396435901785`*^9}}],

Cell[TextData[{
 "Since the matrix {",
 Cell[BoxData[
  SubscriptBox["g", "ij"]],
  CellChangeTimes->{{3.5956521183335505`*^9, 3.595652155602016*^9}, {
   3.595652235962557*^9, 3.595652270813018*^9}, {3.5956523113418894`*^9, 
   3.595652327425517*^9}}],
 "} is symmetric and positive definite it has positive determinant: "
}], "Text",
 CellChangeTimes->{{3.5956523740071993`*^9, 3.5956524288724957`*^9}, 
   3.595652518980466*^9, {3.5956526437956047`*^9, 3.595652659380496*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Det", "[", 
   RowBox[{"Table", "[", 
    RowBox[{
     SubscriptBox["g", 
      RowBox[{"i", ",", "j"}]], ",", " ", 
     RowBox[{"{", 
      RowBox[{"i", ",", "mf"}], "}"}], ",", " ", 
     RowBox[{"{", 
      RowBox[{"j", ",", "mf"}], "}"}]}], "]"}], "]"}], ">", "0"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5956525363104568`*^9, 3.5956526283587217`*^9}, 
   3.614861861687543*^9, 3.6148635019836245`*^9, {3.6187389427887344`*^9, 
   3.6187389469851418`*^9}, 3.623839545216406*^9}],

Cell[TextData[{
 "Let vy = ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["p", "T"], TraditionalForm]]],
 "[vw]  \[Element] ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["T", "x", "m"], TraditionalForm]]],
 " be the orthogonal projection  of the vector  vw \[Element] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["E", "n"], TraditionalForm]]],
 ". Then the vector \nvy - vw is orthogonal to ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["T", "x", "m"], TraditionalForm]]],
 " :"
}], "Text",
 CellChangeTimes->{{3.595653006355342*^9, 3.595653056932235*^9}, {
   3.5956530998836913`*^9, 3.595653233396328*^9}, 3.595653599732281*^9, {
   3.5956536758766365`*^9, 3.5956537832277765`*^9}, {3.595830077103945*^9, 
   3.5958300802491245`*^9}, 3.597207643518685*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"vy", "-", "vw"}], ")"}], ".", 
    SubscriptBox["b", "i"]}], " ", "\[Equal]", " ", "0"}], ",", " ", 
  RowBox[{
   RowBox[{
    StyleBox["or",
     FontColor->GrayLevel[0]], "  ", 
    RowBox[{"vy", ".", 
     SubscriptBox["b", "i"]}]}], " ", "\[Equal]", " ", 
   RowBox[{
    RowBox[{"vw", ".", 
     SubscriptBox["b", "i"]}], "  ", "for", " ", "i", " ", 
    StyleBox[
     RowBox[{"=", " ", "1"}],
     FontColor->RGBColor[0, 0, 1]]}]}], 
  StyleBox[",",
   FontColor->RGBColor[0, 0, 1]], 
  StyleBox["  ",
   FontColor->RGBColor[0, 0, 1]], 
  StyleBox["...",
   FontColor->RGBColor[0, 0, 1]], 
  StyleBox[",",
   FontColor->RGBColor[0, 0, 1]], 
  StyleBox[" ",
   FontColor->RGBColor[0, 0, 1]], 
  RowBox[{
   StyleBox["mf",
    FontColor->RGBColor[0, 0, 1]], "."}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.595653794627428*^9, 3.5956538074371614`*^9}, {
   3.595653843238209*^9, 3.595653924069832*^9}, {3.5958300813321867`*^9, 
   3.5958300830922875`*^9}, 3.6187390440329123`*^9}],

Cell["Let the basis decomposition of vy be", "Text",
 CellChangeTimes->{{3.595654502819935*^9, 3.595654532756647*^9}}],

Cell[BoxData[
 RowBox[{"vy", " ", ":=", " ", 
  RowBox[{
   SubsuperscriptBox["\[Sum]", "1", "mf"], " ", 
   RowBox[{
    SubscriptBox["b", "i"], " ", 
    SubscriptBox["\[Eta]", "i"]}]}]}]], "Input",
 CellChangeTimes->{{3.5956545443553104`*^9, 3.595654548765563*^9}, {
  3.595654614775338*^9, 3.595654647116188*^9}, {3.5956547153400903`*^9, 
  3.595654823176258*^9}, {3.618739087650589*^9, 3.618739097431806*^9}, {
  3.618739128663061*^9, 3.618739144497089*^9}}],

Cell["Then vy is defined by the linear system", "Text",
 CellChangeTimes->{{3.5956548684278464`*^9, 3.595654892356215*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"vy", ".", 
   SubscriptBox["b", "j"]}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     SubsuperscriptBox["\[CapitalSigma]", "1", "m"], " ", 
     SubscriptBox["b", "i"], " ", 
     SubscriptBox["\[Eta]", "i"]}], ")"}], ".", 
   SubscriptBox["b", "j"]}], " ", "\[Equal]", " ", 
  RowBox[{"vw", ".", 
   SubscriptBox["b", "j"]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{
  3.595654941781042*^9, {3.595654975757985*^9, 3.5956550229336834`*^9}, {
   3.595655087159357*^9, 3.595655097189931*^9}, 3.5958300846603775`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubsuperscriptBox["\[CapitalSigma]", "1", "m"], " ", 
    SubscriptBox["g", "ji"], " ", 
    SubscriptBox["\[Eta]", "i"]}], " ", "\[Equal]", " ", 
   RowBox[{"vw", ".", 
    SubscriptBox["b", "j"]}]}], ",", " ", 
  RowBox[{"j", " ", "=", " ", "1"}], ",", "  ", "...", ",", " ", 
  RowBox[{"m", "."}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{
  3.595655150854*^9, {3.595655192325372*^9, 3.595655205189108*^9}, {
   3.595655252685825*^9, 3.595655302854694*^9}, {3.595655373710747*^9, 
   3.5956553766539154`*^9}, 3.5958300860184546`*^9}],

Cell[TextData[{
 "Now let {",
 Cell[BoxData[
  SubscriptBox["h", "ki"]],
  CellChangeTimes->{
   3.595655150854*^9, {3.595655192325372*^9, 3.595655205189108*^9}, {
    3.595655252685825*^9, 3.595655302854694*^9}, {3.595655373710747*^9, 
    3.5956553766539154`*^9}}],
 "} = Invers",
 "e[",
 "{",
 Cell[BoxData[
  SubscriptBox["g", "ji"]],
  CellChangeTimes->{
   3.595655150854*^9, {3.595655192325372*^9, 3.595655205189108*^9}, {
    3.595655252685825*^9, 3.595655302854694*^9}, {3.595655373710747*^9, 
    3.5956553766539154`*^9}}],
 "}] be the inverse matrix:"
}], "Text",
 CellChangeTimes->{{3.595655385316411*^9, 3.5956555192920732`*^9}, 
   3.6148618617187433`*^9, 3.6149177933713636`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubsuperscriptBox["\[CapitalSigma]", 
     RowBox[{"j", "=", "1"}], "m"], " ", 
    SubscriptBox["h", "kj"], " ", 
    SubscriptBox["g", "ji"]}], " ", "\[Equal]", " ", 
   SubscriptBox["\[Delta]", "ki"]}], ",", " ", "k", ",", 
  RowBox[{"i", " ", "=", " ", "1"}], ",", "...", ",", "m", ","}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5956555563181915`*^9, 3.5956556989513493`*^9}, {
  3.597207858081462*^9, 3.5972078683930798`*^9}}],

Cell[TextData[{
 "where ",
 Cell[BoxData[
  SubscriptBox["\[Delta]", "ki"]],
  CellChangeTimes->{{3.595655736429493*^9, 3.595655758124734*^9}, {
   3.595655816518074*^9, 3.5956558518540955`*^9}}],
 " are the coefficients of the identity matrix"
}], "Text",
 CellChangeTimes->{{3.5956562970605593`*^9, 3.5956563376038785`*^9}, {
  3.5972078829323053`*^9, 3.5972078829323053`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[Delta]", "ki"], " ", ":=", " ", 
   RowBox[{"KroneckerDelta", "[", 
    RowBox[{"k", ",", "i"}], "]"}]}], ","}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.595655736429493*^9, 3.595655758124734*^9}, {
  3.595655816518074*^9, 3.5956558518540955`*^9}, {3.597207915848363*^9, 
  3.5972079246779785`*^9}}],

Cell[BoxData[
 RowBox[{"KroneckerDelta", "[", 
  RowBox[{"k", ",", "k"}], "]"}]], "Input",
 CellChangeTimes->{3.595655988030884*^9}],

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"k", " ", "\[NotEqual]", " ", "j"}], ",", " ", 
   RowBox[{"Simplify", "[", 
    RowBox[{"KroneckerDelta", "[", 
     RowBox[{"k", ",", "j"}], "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5956560378387327`*^9, 3.595656081358222*^9}, {
  3.5956561420766945`*^9, 3.5956562544131203`*^9}}],

Cell["Then it follows", "Text",
 CellChangeTimes->{{3.595656395755204*^9, 3.5956564124361587`*^9}}],

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Eta]", "k"], " ", "\[Equal]", " ", 
  RowBox[{
   SubsuperscriptBox["\[CapitalSigma]", 
    RowBox[{"j", "=", "1"}], "mf"], " ", 
   SubsuperscriptBox["\[CapitalSigma]", 
    RowBox[{"i", "=", "1"}], "mf"], 
   SubscriptBox["h", "kj"], " ", 
   SubscriptBox["g", "ji"], 
   SubscriptBox["\[Eta]", "i"]}], " ", "\[Equal]", " ", 
  RowBox[{
   SubsuperscriptBox["\[CapitalSigma]", 
    RowBox[{"j", "=", "1"}], "mf"], " ", 
   SubscriptBox["h", "kj"], 
   RowBox[{"(", 
    RowBox[{"vw", ".", 
     SubscriptBox["b", "j"]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.595656458933818*^9, 3.595656464461134*^9}, {
  3.595656501646261*^9, 3.595656551702124*^9}, {3.5956565823028746`*^9, 
  3.5956566318537083`*^9}, {3.595656665518634*^9, 3.5956566976144695`*^9}, {
  3.5958299906780014`*^9, 3.5958299933341537`*^9}, {3.618739232044442*^9, 
  3.6187392404840574`*^9}}],

Cell[TextData[{
 StyleBox["Lemma 1.",
  FontColor->RGBColor[0, 0, 1]],
 " With the notations fixed above the orthogonal projection of vw \[Element] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["E", 
    RowBox[{"n", "  "}]], TraditionalForm]]],
 "into ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["T", "x", "m"], TraditionalForm]]],
 "   has the basis representation"
}], "Text",
 CellChangeTimes->{{3.5956567484743786`*^9, 3.5956567748518877`*^9}, {
   3.5956569489718466`*^9, 3.5956569986676893`*^9}, {3.5956570412541246`*^9, 
   3.5956570613162727`*^9}, {3.595658913543214*^9, 3.595658946260085*^9}, 
   3.5958299735670233`*^9, {3.5964511553473406`*^9, 3.596451158732547*^9}, {
   3.623840032748462*^9, 3.623840036289668*^9}, {3.6258953252125034`*^9, 
   3.6258953525593514`*^9}}],

Cell[BoxData[
 RowBox[{"vy", " ", "\[Equal]", " ", 
  RowBox[{
   SubscriptBox["p", 
    RowBox[{"T", Cell[""]}]], "[", "vw", "]"}], " ", "\[Equal]", "   ", 
  RowBox[{
   SubsuperscriptBox["\[CapitalSigma]", 
    RowBox[{"k", "=", "1"}], "m"], 
   SubscriptBox["b", "k"], 
   SubsuperscriptBox["\[CapitalSigma]", 
    RowBox[{"j", "=", "1"}], "m"], " ", 
   SubscriptBox["h", "kj"], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"vw", ".", 
      SubscriptBox["b", "j"]}], ")"}], ".", 
    StyleBox["\[FilledSquare]",
     FontColor->GrayLevel[0]]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5956570679866543`*^9, 3.595657161678013*^9}, 
   3.5956572516071568`*^9, {3.595657289167305*^9, 3.595657366046702*^9}, {
   3.5958300874755383`*^9, 3.595830088452594*^9}, {3.5964511915706043`*^9, 
   3.596451239915089*^9}, {3.5964513203956304`*^9, 3.596451327946044*^9}}],

Cell["\<\
We need the inverse matrix of the {rgT[i,j][sf]} to define the projections \
into the tangent spaces:\
\>", "Text",
 CellChangeTimes->{
  3.5960911255015697`*^9, {3.5965226917583475`*^9, 3.596522735204424*^9}, {
   3.5965227667944794`*^9, 3.5965227936733265`*^9}, 3.613128297635784*^9, 
   3.6186421820496635`*^9, 3.618642342405835*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"inverse", "[", 
     RowBox[{"i_", ",", "j_"}], "]"}], "[", "g_", "]"}], "[", "u__", "]"}], ":=",
   " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m", " ", "=", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "u", "}"}], "]"}]}], ",", "k", ",", "h", ",", "mM"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"mM", " ", "=", " ", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"g", "[", 
          RowBox[{"h", ",", "k"}], "]"}], "[", "u", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"h", ",", "m"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", "m"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Inverse", "[", "mM", "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"i", ",", "j"}], "]"}], "]"}]}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.62417013795344*^9, 3.62417016631429*^9}, {
   3.6241702057979593`*^9, 3.6241704585808034`*^9}, 3.624170521495714*^9, {
   3.6241705601837816`*^9, 3.624170561010583*^9}, {3.6241706254230967`*^9, 
   3.6241706872616053`*^9}, {3.624170740254898*^9, 3.624170785058177*^9}, 
   3.6241708448530817`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "inverse"}]], "Input",
 CellChangeTimes->{{3.62417231871247*^9, 3.624172440673485*^9}, {
   3.6241725619012976`*^9, 3.6241727013655424`*^9}, {3.6241727407712116`*^9, 
   3.6241728849934654`*^9}, 3.6241729245239344`*^9, {3.6259030986192017`*^9, 
   3.6259031014740067`*^9}}],

Cell[BoxData[
 StyleBox["\<\"If {g[h,k][u,v,...]}is a field of square matrices the rank of \
\\nwhich equals the number of variables u,v,..., then \
{inverse[i,j][g][u,v,...]} is \\nthe field of the inverse matrices.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.6259034271585803`*^9},
 CellTags->"Info3625907027-2112774"]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"invrgT", "[", 
     RowBox[{"i_", ",", "j_"}], "]"}], "[", "ff_", "]"}], "[", "u__", "]"}], ":=",
   " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"gg", ",", 
      RowBox[{"m", " ", "=", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "u", "}"}], "]"}]}], ",", "k", ",", "h", ",", "v"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"gg", "[", 
        RowBox[{"h_", ",", "k_"}], "]"}], "[", "v__", "]"}], " ", ":=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"rgT", "[", 
         RowBox[{"h", ",", "k"}], "]"}], "[", "ff", "]"}], "[", "v", "]"}]}], 
     " ", ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"inverse", "[", 
        RowBox[{"i", ",", "j"}], "]"}], "[", "gg", "]"}], "[", "u", "]"}]}]}],
    "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.624170884040351*^9, 3.624170900295579*^9}, {
   3.624171093969919*^9, 3.6241711500052176`*^9}, {3.6241713588583846`*^9, 
   3.6241714807257986`*^9}, {3.6241715200222673`*^9, 
   3.6241715235322742`*^9}, {3.6241715692559543`*^9, 3.624171601516811*^9}, {
   3.624171647536892*^9, 3.62417169243377*^9}, {3.624172071452036*^9, 
   3.624172114539312*^9}, 3.6241721566281857`*^9, 3.624172272286789*^9}],

Cell[BoxData[
 RowBox[{"?", "invrgT"}]], "Input",
 CellChangeTimes->{{3.623840427694356*^9, 3.623840456975607*^9}, {
   3.623840585831834*^9, 3.623840703877241*^9}, {3.623840781300177*^9, 
   3.623840815011836*^9}, {3.623840848551895*^9, 3.6238411098991537`*^9}, 
   3.623841177712473*^9, {3.6259035933450727`*^9, 3.6259035959814777`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"invrgT", "[", 
    RowBox[{"1", ",", "1"}], "]"}], "[", "sf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6241717142582088`*^9, 3.62417181163358*^9}, {
   3.624171854845656*^9, 3.624171911426955*^9}, 3.624171986026286*^9, 
   3.6241721716042123`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"invrgT", "[", 
         RowBox[{"i", ",", "j"}], "]"}], "[", 
        RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "3"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "3"}], "}"}]}], "]"}], ".", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"rgT", "[", 
         RowBox[{"i", ",", "j"}], "]"}], "[", 
        RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "3"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "3"}], "}"}]}], "]"}]}], "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.624172975380024*^9, 3.6241729882812467`*^9}, {
  3.6241730189821005`*^9, 3.6241730980586395`*^9}, {3.6241731311462975`*^9, 
  3.6241733147118196`*^9}}],

Cell[TextData[{
 StyleBox["Example.",
  FontColor->RGBColor[0, 0, 1]],
 " The sphere of radius r>0:"
}], "Text",
 CellChangeTimes->{{3.5965221197053423`*^9, 3.596522145663788*^9}, {
  3.596522200887885*^9, 3.596522218531516*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"rgT", "[", 
         RowBox[{"i", ",", "j"}], "]"}], "[", 
        RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "2"}], "}"}]}], "]"}], ",", 
    RowBox[{"r", ">", "0"}]}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.595660633590595*^9, 3.595660656702917*^9}, {
   3.595660697216234*^9, 3.595660726782925*^9}, {3.5956608079675684`*^9, 
   3.595660835895166*^9}, {3.596451415196997*^9, 3.5964514249782143`*^9}, {
   3.5965217603899117`*^9, 3.5965217906227646`*^9}, {3.5965219904279156`*^9, 
   3.5965220433744087`*^9}, {3.596522159454212*^9, 3.5965221709046326`*^9}, {
   3.5965222567671833`*^9, 3.5965222569387836`*^9}, 3.6148618617655435`*^9, 
   3.6148635020148244`*^9, 3.61864220699609*^9, 3.6186423424318366`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"invrgT", "[", 
         RowBox[{"i", ",", "j"}], "]"}], "[", 
        RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "2"}], "}"}]}], "]"}], ",", 
    RowBox[{"r", ">", "0"}]}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.595660633590595*^9, 3.595660656702917*^9}, {
   3.595660697216234*^9, 3.595660726782925*^9}, {3.5956608079675684`*^9, 
   3.595660835895166*^9}, {3.596451415196997*^9, 3.5964514249782143`*^9}, {
   3.5965217603899117`*^9, 3.5965217906227646`*^9}, {3.5965219904279156`*^9, 
   3.5965220433744087`*^9}, {3.596522159454212*^9, 3.5965221709046326`*^9}, 
   3.6148618617811437`*^9, 3.614863502030424*^9, 3.6187394898816953`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"%", ".", "%%"}], ",", 
    RowBox[{"r", ">", "0"}]}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.595660633590595*^9, 3.595660656702917*^9}, {
  3.595660697216234*^9, 3.595660726782925*^9}, {3.5956608079675684`*^9, 
  3.595660835895166*^9}, {3.596451415196997*^9, 3.5964514249782143`*^9}, {
  3.5965217603899117`*^9, 3.5965217906227646`*^9}, {3.5965219904279156`*^9, 
  3.5965220433744087`*^9}, {3.596522159454212*^9, 3.5965221709046326`*^9}, {
  3.5965222864852357`*^9, 3.5965223097448764`*^9}}],

Cell["\<\
OK. \[FilledRectangle]
\
\>", "Text",
 CellChangeTimes->{{3.596522418243067*^9, 3.596522439427904*^9}}],

Cell[TextData[{
 "Here is the definition of the ",
 StyleBox["field of tangent projections",
  FontColor->RGBColor[0, 0, 1]],
 " of the immersion ff[x...]:"
}], "Text",
 CellChangeTimes->{{3.5965229764900475`*^9, 3.596522995631281*^9}, {
  3.596523070556733*^9, 3.596523132165257*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"pT", "[", "ff_", "]"}], "[", "x__", "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"tv", "[", "k", "]"}], "[", "ff", "]"}], "[", "x", "]"}], "*", 
      RowBox[{"Sum", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"invrgT", "[", 
            RowBox[{"k", ",", "j"}], "]"}], "[", "ff", "]"}], "[", "x", "]"}],
          "*", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"tv", "[", "j", "]"}], "[", "ff", "]"}], "[", "x", "]"}],
            ".", "#"}], ")"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"j", ",", 
          RowBox[{"Length", "[", 
           RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}], "&"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5956711982508154`*^9, 3.5956714584426975`*^9}, {
   3.5956714915405903`*^9, 3.5956715370841956`*^9}, {3.595671585747979*^9, 
   3.595671613340557*^9}, {3.595671665565544*^9, 3.595671740565834*^9}, 
   3.6187394924557*^9, {3.6258068501217403`*^9, 3.625806851603743*^9}}],

Cell[BoxData[
 RowBox[{"?", "pT"}]], "Input",
 CellChangeTimes->{{3.623914453021508*^9, 3.6239147640236545`*^9}, 
   3.6239148609466248`*^9, {3.6259037669777794`*^9, 3.6259037710493865`*^9}}],

Cell[TextData[{
 StyleBox["Example.",
  FontColor->RGBColor[0, 0, 1]],
 " The unit sphere"
}], "Text",
 CellChangeTimes->{{3.6239149035814996`*^9, 3.623914907855907*^9}, {
  3.62391617373573*^9, 3.623916180974143*^9}}],

Cell["The projection of a tangent vector is the vector itself:", "Text",
 CellChangeTimes->{{3.6239163166475816`*^9, 3.623916342044426*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pTsph", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"pT", "[", 
       RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "[", 
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "1", "]"}], "[", 
       RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"u", ">", "0"}], ",", 
      RowBox[{"v", ">", "0"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6238523551404347`*^9, 3.623852415481341*^9}, {
  3.6238524574298143`*^9, 3.623852489550271*^9}, {3.623852532185146*^9, 
  3.6238525727764173`*^9}, {3.623852900501793*^9, 3.6238529160082197`*^9}, {
  3.6238546605091753`*^9, 3.6238546765772033`*^9}, {3.6238555562667484`*^9, 
  3.6238556039716325`*^9}, {3.623855655077322*^9, 3.623855675497758*^9}}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"%", "\[Equal]", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"amf", "[", "1", "]"}], "[", 
      RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", ">", "0"}], ",", 
     RowBox[{"v", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.623855727180649*^9, 3.6238557502842894`*^9}, {
  3.623916232891034*^9, 3.623916258365879*^9}}],

Cell["The projection of the normal vector is the null vector:", "Text",
 CellChangeTimes->{{3.6239163694068737`*^9, 3.623916397408923*^9}, {
  3.6239164858454785`*^9, 3.623916505673114*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pTsph", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"pT", "[", 
       RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "[", 
     RowBox[{
      RowBox[{
       RowBox[{"amf", "[", "3", "]"}], "[", 
       RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"u", ">", "0"}], ",", 
      RowBox[{"v", ">", "0"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6238523551404347`*^9, 3.623852415481341*^9}, {
  3.6238524574298143`*^9, 3.623852489550271*^9}, {3.623852532185146*^9, 
  3.6238525727764173`*^9}, {3.623852900501793*^9, 3.6238529160082197`*^9}, {
  3.6238546605091753`*^9, 3.6238546765772033`*^9}, {3.6238555562667484`*^9, 
  3.6238556039716325`*^9}, {3.623855655077322*^9, 3.623855675497758*^9}, {
  3.623855894679143*^9, 3.623855916862382*^9}}],

Cell["Here is a random vector", "Text",
 CellChangeTimes->{{3.623918289782647*^9, 3.623918299517064*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";", 
  RowBox[{"vr", " ", "=", " ", 
   RowBox[{"randomv", "[", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.623916512864726*^9, 3.6239165278563523`*^9}, {
  3.625895946421194*^9, 3.625895951959204*^9}}],

Cell["Its projection at the point sphero[1][u,v] is", "Text",
 CellChangeTimes->{{3.623918317160695*^9, 3.623918347237548*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pTsph", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"pT", "[", 
       RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "[", "vr", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"u", ">", "0"}], ",", 
      RowBox[{"v", ">", "0"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6238523551404347`*^9, 3.623852415481341*^9}, {
   3.6238524574298143`*^9, 3.623852489550271*^9}, {3.623852532185146*^9, 
   3.6238525727764173`*^9}, {3.623852900501793*^9, 3.6238529160082197`*^9}, {
   3.6238546605091753`*^9, 3.6238546765772033`*^9}, {3.6238555562667484`*^9, 
   3.6238556039716325`*^9}, {3.623855655077322*^9, 3.623855675497758*^9}, {
   3.623855894679143*^9, 3.623855916862382*^9}, 3.623856005816738*^9, {
   3.623856100042904*^9, 3.623856105986514*^9}, {3.623916544860382*^9, 
   3.623916545359583*^9}, {3.6239179299368153`*^9, 3.6239179399988327`*^9}, {
   3.623918078558276*^9, 3.6239180790262775`*^9}}],

Cell["\<\
The projections form a tangent vector field; we represent it by the arrows in \
the tangent planes:\
\>", "Text",
 CellChangeTimes->{{3.6239183852080145`*^9, 3.6239184414305134`*^9}, 
   3.625895918746746*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"arr", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Graphics3D", "[", 
   RowBox[{"{", 
    RowBox[{"Red", ",", 
     RowBox[{"Arrowheads", "[", "0.03", "]"}], ",", 
     RowBox[{"Arrow", "[", 
      RowBox[{"Tube", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"sphero", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], ",", 
          RowBox[{
           RowBox[{"pTsph", "[", 
            RowBox[{"u", ",", "v"}], "]"}], "+", 
           RowBox[{
            RowBox[{"sphero", "[", "1", "]"}], "[", 
            RowBox[{"u", ",", "v"}], "]"}]}]}], "}"}], ",", "0.003"}], "]"}], 
      "]"}]}], "}"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6238531941127977`*^9, 3.623853197966005*^9}, {
   3.6238543177141733`*^9, 3.623854450439206*^9}, {3.6238545220589323`*^9, 
   3.623854538423361*^9}, {3.623856261245988*^9, 3.6238562702160034`*^9}, {
   3.6238564022118354`*^9, 3.623856413896256*^9}, {3.6238564857393827`*^9, 
   3.6238565058478174`*^9}, {3.6239168142852554`*^9, 3.6239168464525123`*^9}, 
   3.6239168776837673`*^9, 3.6239169372758713`*^9, 3.6239169803475475`*^9, 
   3.6239170519360733`*^9, {3.6239171102645755`*^9, 3.6239171148353834`*^9}, 
   3.6239171473614407`*^9, {3.6239172534572268`*^9, 3.6239173025505133`*^9}, {
   3.623917342299383*^9, 3.6239173434849854`*^9}, 3.6239174393783536`*^9, 
   3.62391823446495*^9}],

Cell[BoxData[
 RowBox[{"arr", "[", 
  RowBox[{"0", ",", 
   RowBox[{"-", ".4"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6238545919938545`*^9, 3.623854609309885*^9}, {
   3.6238547609733515`*^9, 3.6238547685705647`*^9}, 3.623854808366235*^9, 
   3.623854918455628*^9, {3.623856316610485*^9, 3.6238563203700914`*^9}, {
   3.6238565596991124`*^9, 3.623856588685963*^9}, 3.6238566793573227`*^9, 
   3.623917383764256*^9, {3.6239181891936703`*^9, 3.623918209302106*^9}}],

Cell["Here is the picture of some vectors of this field:", "Text",
 CellChangeTimes->{{3.623918500975818*^9, 3.6239185237986584`*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"{", 
   RowBox[{"sph1", ",", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"arr", "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"u", ",", "0", ",", 
        RowBox[{"2", "*", "Pi"}], ",", " ", ".5"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"v", ",", 
        RowBox[{
         RowBox[{"-", "Pi"}], "/", "2"}], ",", 
        RowBox[{"Pi", "/", "2"}], ",", ".5"}], "}"}]}], "]"}]}], "}"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6238547317077*^9, 3.6238547444841223`*^9}, {
   3.6238548325618773`*^9, 3.623854852311512*^9}, {3.623854937628062*^9, 
   3.623854955848894*^9}, 3.6238550661606874`*^9, {3.6238563348157167`*^9, 
   3.623856337998122*^9}, {3.6238565368918724`*^9, 3.623856541883881*^9}, {
   3.6238566029151883`*^9, 3.6238566052551928`*^9}, 3.6238566950665503`*^9, 
   3.6239173995670834`*^9, 3.623917477972821*^9, {3.6239175532273536`*^9, 
   3.623917691833597*^9}, {3.6239178577866883`*^9, 3.6239178646351004`*^9}, {
   3.623917992212125*^9, 3.623917996845333*^9}}],

Cell[TextData[{
 "\[FilledSquare]",
 "\nNow we return to the general surface in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["E", "4"], TraditionalForm]]],
 " considered in the previous subsection: "
}], "Text",
 CellChangeTimes->{{3.595658977275859*^9, 3.59565902746873*^9}, {
  3.595660122941387*^9, 3.5956601380842533`*^9}, {3.59609110855994*^9, 
  3.5960911150339513`*^9}, {3.62400946336747*^9, 3.624009479934699*^9}, {
  3.6240096123165317`*^9, 3.624009617698541*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "4"}], ";", " ", 
  RowBox[{"mf", " ", "=", " ", "2"}], ";", " ", 
  RowBox[{"vx", " ", ":=", " ", 
   RowBox[{
    RowBox[{"tvf", "[", "\[Xi]", "]"}], "[", 
    RowBox[{"smf", "[", "f", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5956590461387978`*^9, 3.5956590587255173`*^9}, 
   3.595924528295635*^9}],

Cell[BoxData[
 RowBox[{"vx", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6241734515396605`*^9, 3.624173473254898*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"vv", "[", "s", "]"}], "[", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}]}], "]"}]], "Input"],

Cell["\<\
The orthogonal projection of the total differential yields hardly to handle \
expressions:\
\>", "Text",
 CellChangeTimes->{{3.595924809229704*^9, 3.595924857997493*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"pT", "[", 
    RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}]}], "]"}], "[", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "vx", "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}]}], "]"}], ".", 
   RowBox[{
    RowBox[{"vv", "[", "s", "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5956718156521287`*^9, 3.595671916986925*^9}, {
  3.595924878553669*^9, 3.5959248845610123`*^9}, {3.5959249303306303`*^9, 
  3.5959249361379623`*^9}}],

Cell["\<\
The simplification of this Output needs much time and brings little insight. \
We try the sphere example;\
\>", "Text",
 CellChangeTimes->{{3.5965233231091785`*^9, 3.5965233415892353`*^9}, {
  3.5965234585939274`*^9, 3.5965234706766186`*^9}, {3.5965236058293495`*^9, 
  3.5965236548851547`*^9}, {3.5965239391284127`*^9, 3.5965240097254505`*^9}, {
  3.596524051661849*^9, 3.5965240656206474`*^9}, {3.596524116821576*^9, 
  3.596524116826576*^9}, {3.596524164133282*^9, 3.596524171508704*^9}, {
  3.5965243170540285`*^9, 3.5965243252935*^9}, {3.62384177355552*^9, 
  3.623841797922763*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";", 
  RowBox[{"wx", ":=", " ", 
   RowBox[{
    RowBox[{"tvf", "[", "\[Xi]", "]"}], "[", 
    RowBox[{"sphero", "[", "r", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.623845509043992*^9, 3.623845583502923*^9}, {
  3.6238459238331203`*^9, 3.6238459275615273`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", "wx", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], ".", 
  RowBox[{
   RowBox[{"vv", "[", "s", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6238461248862734`*^9, 3.623846133559889*^9}, {
  3.6238461841039777`*^9, 3.623846196162799*^9}}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"pT", "[", 
      RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", "%", "]"}], ",", " ", 
   RowBox[{"r", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.623846233665265*^9, 3.623846306813793*^9}, {
  3.6238463410558534`*^9, 3.6238463519602723`*^9}}],

Cell[TextData[{
 "Now we return to to the general surface in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["E", "4"], TraditionalForm]]],
 " and apply  ",
 ButtonBox["formula (1)",
  BaseStyle->"Hyperlink",
  ButtonData->"dtt(1)"],
 " of the last subsection. We first calculate the tangential components of \
the total differentials of the  tangent basis vectors:"
}], "Text",
 CellChangeTimes->{{3.5965233231091785`*^9, 3.5965233415892353`*^9}, {
   3.5965234585939274`*^9, 3.5965234706766186`*^9}, {3.5965236058293495`*^9, 
   3.5965236548851547`*^9}, {3.5965239391284127`*^9, 
   3.5965240097254505`*^9}, {3.596524051661849*^9, 3.5965240656206474`*^9}, {
   3.596524116821576*^9, 3.596524116826576*^9}, {3.596524164133282*^9, 
   3.596524171508704*^9}, {3.5965243170540285`*^9, 3.5965243252935*^9}, 
   3.62384177355552*^9, {3.6238465789718714`*^9, 3.62384662976556*^9}, {
   3.6238467043960915`*^9, 3.6238467398393536`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "4"}], ";", " ", 
  RowBox[{"mf", " ", "=", " ", "2"}], ";", " ", 
  RowBox[{"vx", " ", ":=", " ", 
   RowBox[{
    RowBox[{"tvf", "[", "\[Xi]", "]"}], "[", 
    RowBox[{"smf", "[", "f", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5956590461387978`*^9, 3.5956590587255173`*^9}, 
   3.595924528295635*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Gamma]", "[", 
     RowBox[{"i_", ",", "j_", ",", "k_"}], "]"}], "[", "ff_", "]"}], "[", 
   "x__", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"tv", "[", "i", "]"}], "[", "ff", "]"}], "[", "x", "]"}], 
      ")"}], ".", 
     RowBox[{
      RowBox[{"d", "[", 
       RowBox[{
        RowBox[{"tv", "[", "j", "]"}], "[", "ff", "]"}], "]"}], "[", "x", 
      "]"}]}], ")"}], "[", 
   RowBox[{"[", "k", "]"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.595832267149208*^9, 3.5958323721202126`*^9}, {
   3.595832428807455*^9, 3.595832442513239*^9}, {3.595832489464924*^9, 
   3.5958325992172017`*^9}, {3.5958328395459476`*^9, 
   3.5958329541655035`*^9}, {3.5958329857303085`*^9, 3.5958329927137084`*^9}, 
   3.595833066441925*^9, {3.595833151066766*^9, 3.595833202040681*^9}, {
   3.595833410761619*^9, 3.595833418537064*^9}, {3.5958337753454723`*^9, 
   3.5958337836169453`*^9}, 3.5958373746493406`*^9, {3.6258068531481457`*^9, 
   3.625806853896947*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Gamma]", "[", 
    RowBox[{"1", ",", "2", ",", "1"}], "]"}], "[", 
   RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5958326835090227`*^9, 3.5958327630155706`*^9}, {
  3.595832965609158*^9, 3.5958329706304455`*^9}, {3.5958330067045083`*^9, 
  3.5958330097426825`*^9}}],

Cell["\[Gamma][i,j,k] is symmetric with respect to the last two indices:", \
"Text",
 CellChangeTimes->{{3.5958375539165945`*^9, 3.595837581947198*^9}}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"\[Gamma]", "[", 
        RowBox[{"k", ",", "j", ",", "i"}], "]"}], "[", 
       RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}]}], "]"}], "-", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Gamma]", "[", 
        RowBox[{"k", ",", "i", ",", "j"}], "]"}], "[", 
       RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}]}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "mf"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "mf"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "mf"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.59583346776388*^9, 3.595833575697053*^9}, {
   3.5958336323142915`*^9, 3.5958336479051833`*^9}, {3.5958336849463015`*^9, 
   3.5958337057534924`*^9}, {3.5958338055792017`*^9, 
   3.5958338282264967`*^9}, {3.596466574435828*^9, 3.5964666093798895`*^9}, 
   3.6148618620151443`*^9, 3.6148635020460243`*^9}],

Cell["\<\
The \[Gamma][i,j,k] may be expressed as the scalar product of the derivatives\
\>", "Text",
 CellChangeTimes->{{3.5958376134359984`*^9, 3.595837671195302*^9}, 
   3.596466437498788*^9}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"\[Gamma]", "[", 
        RowBox[{"i", ",", "j", ",", "k"}], "]"}], "[", 
       RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}]}], "]"}], "-", 
     RowBox[{"Sum", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"D", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"f", "[", "l", "]"}], "[", 
           RowBox[{
            RowBox[{"x", "[", "1", "]"}], ",", 
            RowBox[{"x", "[", "2", "]"}]}], "]"}], ",", 
          RowBox[{"x", "[", "i", "]"}]}], "]"}], "*", 
        RowBox[{"D", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"f", "[", "l", "]"}], "[", 
           RowBox[{
            RowBox[{"x", "[", "1", "]"}], ",", 
            RowBox[{"x", "[", "2", "]"}]}], "]"}], ",", 
          RowBox[{"x", "[", "j", "]"}], ",", 
          RowBox[{"x", "[", "k", "]"}]}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"l", ",", "dim"}], "}"}]}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "mf"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "mf"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "mf"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.595834301153547*^9, 3.595834330847245*^9}, {
   3.5958344074156246`*^9, 3.5958344117198706`*^9}, {3.595834470991261*^9, 
   3.595834498791851*^9}, {3.5958345504088035`*^9, 3.5958346806322517`*^9}, {
   3.5958347237637186`*^9, 3.5958347699433603`*^9}, {3.5958348271236305`*^9, 
   3.5958348924843693`*^9}, 3.614861862030744*^9, 3.6148635020616245`*^9}],

Cell["It follows that \[Gamma][i,j,k] may be defined by", "Text",
 CellChangeTimes->{{3.595835714644394*^9, 3.595835766867381*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Gamma]\[Gamma]\[Gamma]", "[", 
     RowBox[{"i_", ",", "j_", ",", "k_"}], "]"}], "[", "ff_", "]"}], " ", "[",
    "x__", "]"}], ":=", 
  RowBox[{
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{"ff", "[", "x", "]"}], ",", 
     RowBox[{
      RowBox[{"{", "x", "}"}], "[", 
      RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ".", " ", 
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{"ff", "[", "x", "]"}], ",", 
     RowBox[{
      RowBox[{"{", "x", "}"}], "[", 
      RowBox[{"[", "j", "]"}], "]"}], ",", 
     RowBox[{
      RowBox[{"{", "x", "}"}], "[", 
      RowBox[{"[", "k", "]"}], "]"}]}], "]"}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.5958360751920156`*^9, {3.5958362670469894`*^9, 3.595836268018045*^9}, {
   3.595836315208744*^9, 3.59583637003388*^9}, {3.5958366145528655`*^9, 
   3.595836615136899*^9}, {3.595836913992993*^9, 3.595837072362051*^9}, 
   3.595837191258851*^9, 3.5958373185941343`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Gamma]\[Gamma]\[Gamma]", "[", 
    RowBox[{"1", ",", "1", ",", "2"}], "]"}], "[", 
   RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.59583648699857*^9, 3.5958364893347034`*^9}}],

Cell["Check the statement:", "Text",
 CellChangeTimes->{{3.5965245375496407`*^9, 3.5965245477652245`*^9}}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"\[Gamma]", "[", 
        RowBox[{"i", ",", "j", ",", "k"}], "]"}], "[", 
       RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}]}], "]"}], "-", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Gamma]\[Gamma]\[Gamma]", "[", 
        RowBox[{"i", ",", "j", ",", "k"}], "]"}], "[", 
       RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}]}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "mf"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "mf"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "mf"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.595834301153547*^9, 3.595834330847245*^9}, {
   3.5958344074156246`*^9, 3.5958344117198706`*^9}, {3.595834470991261*^9, 
   3.595834498791851*^9}, {3.5958345504088035`*^9, 3.5958346806322517`*^9}, {
   3.5958347237637186`*^9, 3.5958347699433603`*^9}, {3.5958348271236305`*^9, 
   3.5958348924843693`*^9}, {3.5958361198955727`*^9, 
   3.5958361451940203`*^9}, {3.5958361819391212`*^9, 3.5958361936667924`*^9}, 
   3.5958362941475396`*^9, 3.614861862046344*^9, 3.6148635020616245`*^9}],

Cell[TextData[{
 "Now we remember the definition of the",
 ButtonBox[" first fundamental form",
  BaseStyle->"Hyperlink",
  ButtonData->"fund1"],
 " and  verify"
}], "Text",
 CellChangeTimes->{{3.595941251019602*^9, 3.5959412698656797`*^9}, {
  3.5964666958040414`*^9, 3.596466724040091*^9}, {3.596526324159829*^9, 
  3.5965263241648293`*^9}}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Table", " ", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"rgT", "[", 
          RowBox[{"i", ",", "j"}], "]"}], "[", 
         RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
        RowBox[{
         RowBox[{"x", "[", "1", "]"}], ",", 
         RowBox[{"x", "[", "2", "]"}]}], "]"}], ",", 
       RowBox[{"x", "[", "k", "]"}]}], "]"}], "-", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Gamma]", "[", 
        RowBox[{"i", ",", "k", ",", "j"}], "]"}], "[", 
       RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}]}], "]"}], "-", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Gamma]", "[", 
        RowBox[{"j", ",", "k", ",", "i"}], "]"}], "[", 
       RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}]}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "2"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5959412900918365`*^9, 3.595941430860888*^9}, {
   3.5959414627807136`*^9, 3.5959414712411976`*^9}, {3.5959415379840155`*^9, 
   3.595941634457533*^9}, 3.6186423861793385`*^9}],

Cell["Thus we have the relation (omitting the arguments)", "Text",
 CellChangeTimes->{{3.5959417091768064`*^9, 3.595941736120348*^9}, 
   3.5959433441283207`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[PartialD]", "k"], 
   SubscriptBox["g", 
    RowBox[{"ij", " "}]]}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"\[Gamma]", "[", 
    RowBox[{"i", ",", "k", ",", "j"}], "]"}], " ", "+", 
   RowBox[{"\[Gamma]", "[", 
    RowBox[{"j", ",", "k", ",", "i"}], "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5959417619478254`*^9, 3.5959417867222424`*^9}, {
  3.595941835226016*^9, 3.5959418868279676`*^9}}],

Cell["\<\
Permuting in this identity the indices i, j, k twice cyclically and calculate\
\>", "Text",
 CellChangeTimes->{{3.5959428318930225`*^9, 3.5959429012729907`*^9}, 
   3.595943368160695*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", "k"], 
     SubscriptBox["g", 
      RowBox[{"ij", " "}]]}], " ", "+", 
    RowBox[{
     SubscriptBox["\[PartialD]", "i"], 
     SubscriptBox["g", 
      RowBox[{"jk", " "}]]}], " ", "-", 
    RowBox[{
     SubscriptBox["\[PartialD]", "j"], 
     SubscriptBox["g", 
      RowBox[{"ki", " "}]]}]}], ")"}], "/", "2"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.595942934100868*^9, 3.5959429691548734`*^9}, {
  3.5959429992345943`*^9, 3.5959430300903587`*^9}}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Table", " ", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"D", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"rgT", "[", 
             RowBox[{"i", ",", "j"}], "]"}], "[", 
            RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
           RowBox[{
            RowBox[{"x", "[", "1", "]"}], ",", 
            RowBox[{"x", "[", "2", "]"}]}], "]"}], ",", 
          RowBox[{"x", "[", "k", "]"}]}], "]"}], " ", "+", " ", 
        RowBox[{"D", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"rgT", "[", 
             RowBox[{"j", ",", "k"}], "]"}], "[", 
            RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
           RowBox[{
            RowBox[{"x", "[", "1", "]"}], ",", 
            RowBox[{"x", "[", "2", "]"}]}], "]"}], ",", 
          RowBox[{"x", "[", "i", "]"}]}], "]"}], "-", 
        RowBox[{"D", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"rgT", "[", 
             RowBox[{"k", ",", "i"}], "]"}], "[", 
            RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
           RowBox[{
            RowBox[{"x", "[", "1", "]"}], ",", 
            RowBox[{"x", "[", "2", "]"}]}], "]"}], ",", 
          RowBox[{"x", "[", "j", "]"}]}], "]"}]}], ")"}], "/", "2"}], " ", 
     "-", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Gamma]", "[", 
        RowBox[{"j", ",", "k", ",", "i"}], "]"}], "[", 
       RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}]}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "2"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5959412900918365`*^9, 3.595941430860888*^9}, {
  3.5959414627807136`*^9, 3.5959414712411976`*^9}, {3.5959415379840155`*^9, 
  3.595941634457533*^9}, {3.595941961554242*^9, 3.595942003374634*^9}, {
  3.595942053413496*^9, 3.595942106702544*^9}, {3.5959421530311937`*^9, 
  3.595942184531996*^9}, {3.5959422245002813`*^9, 3.595942227412448*^9}, {
  3.5959423193967094`*^9, 3.5959423524365993`*^9}, {3.595943123101679*^9, 
  3.5959431250367894`*^9}, {3.6186423875404167`*^9, 3.6186423896215353`*^9}}],

Cell["We obtain ", "Text",
 CellChangeTimes->{{3.5959430503845196`*^9, 3.59594305948804*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Gamma]", "[", 
   RowBox[{"j", ",", "k", ",", "i"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", "k"], 
      SubscriptBox["g", 
       RowBox[{"ij", " "}]]}], " ", "+", 
     RowBox[{
      SubscriptBox["\[PartialD]", "i"], 
      SubscriptBox["g", 
       RowBox[{"jk", " "}]]}], " ", "-", 
     RowBox[{
      SubscriptBox["\[PartialD]", "j"], 
      SubscriptBox["g", 
       RowBox[{"ki", " "}]]}]}], ")"}], "/", "2"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.595943183946159*^9, 3.5959431903385243`*^9}, 
   3.5959432283076963`*^9}],

Cell["\<\
To get this result we used the symmetry of the \[Gamma][j,k,i] in the last \
two indices. We come to the  third definition of the \[Gamma]-symbols:\
\>", "Text",
 CellChangeTimes->{{3.595943419848652*^9, 3.5959434605369787`*^9}, {
   3.624174307341563*^9, 3.6241743281364*^9}, {3.6241743619728594`*^9, 
   3.6241743751704826`*^9}, {3.6241744088197412`*^9, 3.6241744186321588`*^9}, 
   3.624174449629413*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"christoffel1", "[", 
     RowBox[{"j_", ",", "k_", ",", "i_"}], "]"}], "[", "g_", "]"}], "[", 
   "x__", "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"g", "[", 
         RowBox[{"i", ",", "j"}], "]"}], "[", "x", "]"}], ",", 
       RowBox[{
        RowBox[{"{", "x", "}"}], "[", 
        RowBox[{"[", "k", "]"}], "]"}]}], "]"}], " ", "+", 
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"g", "[", 
         RowBox[{"j", ",", "k"}], "]"}], "[", "x", "]"}], ",", 
       RowBox[{
        RowBox[{"{", "x", "}"}], "[", 
        RowBox[{"[", "i", "]"}], "]"}]}], "]"}], " ", "-", 
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"g", "[", 
         RowBox[{"k", ",", "i"}], "]"}], "[", "x", "]"}], ",", 
       RowBox[{
        RowBox[{"{", "x", "}"}], "[", 
        RowBox[{"[", "j", "]"}], "]"}]}], "]"}]}], ")"}], "/", 
   "2"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.624174812798051*^9, 3.624174820644865*^9}, {
   3.624174855292526*^9, 3.6241748801745696`*^9}, {3.624174913621028*^9, 
   3.6241749353518667`*^9}, {3.6241750065035915`*^9, 3.6241750833181267`*^9}, 
   3.6241751637362676`*^9, {3.6241766686732793`*^9, 3.6241766761456923`*^9}}],

Cell[BoxData[
 RowBox[{"?", "christoffel1"}]], "Input",
 CellChangeTimes->{{3.6241751753270884`*^9, 3.624175231611987*^9}, {
   3.6241752747148623`*^9, 3.6241753353209686`*^9}, {3.6241754714593754`*^9, 
   3.624175537417291*^9}, 3.6241756021740046`*^9, 3.6241886148000774`*^9, {
   3.625904141139639*^9, 3.625904146958449*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Gamma]\[Gamma]", "[", 
     RowBox[{"j_", ",", "k_", ",", "i_"}], "]"}], "[", "ff_", "]"}], "[", 
   "x__", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"g", ",", "ii", ",", "jj", ",", "y"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"g", "[", 
        RowBox[{"ii_", ",", "jj_"}], "]"}], "[", "y__", "]"}], ":=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"rgT", "[", 
         RowBox[{"ii", ",", "jj"}], "]"}], "[", "ff", "]"}], "[", "y", 
       "]"}]}], ";", 
     RowBox[{
      RowBox[{
       RowBox[{"christoffel1", "[", 
        RowBox[{"j", ",", "k", ",", "i"}], "]"}], "[", "g", "]"}], "[", "x", 
      "]"}]}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.595943691303178*^9, 3.595943720562852*^9}, {
   3.595943755629857*^9, 3.5959438156432896`*^9}, {3.5959438577156963`*^9, 
   3.5959439199962587`*^9}, {3.595943963596752*^9, 3.5959440729940095`*^9}, {
   3.5959453340161357`*^9, 3.59594563392929*^9}, {3.5959456985059834`*^9, 
   3.5959457283136883`*^9}, {3.5959460610257187`*^9, 
   3.5959460813358803`*^9}, {3.618642390708598*^9, 3.6186423931307364`*^9}, {
   3.624176205742466*^9, 3.6241762142132807`*^9}, 3.624188702815432*^9}],

Cell[BoxData[
 RowBox[{"?", "\[Gamma]\[Gamma]"}]], "Input",
 CellChangeTimes->{{3.624188909578195*^9, 3.624188987047931*^9}, {
  3.6241890186691866`*^9, 3.6241890327872114`*^9}, {3.624189070258477*^9, 
  3.6241891921882915`*^9}, {3.624189232982363*^9, 3.6241892484107904`*^9}, {
  3.6259042402732134`*^9, 3.625904263922855*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Gamma]\[Gamma]", "[", 
    RowBox[{"1", ",", "1", ",", "2"}], "]"}], "[", 
   RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5959443675718584`*^9, 3.5959443703800187`*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Table", " ", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"\[Gamma]\[Gamma]", "[", 
        RowBox[{"i", ",", "j", ",", "k"}], "]"}], "[", 
       RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}], ",", 
       RowBox[{"x", "[", "3", "]"}]}], "]"}], "-", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Gamma]", "[", 
        RowBox[{"i", ",", "j", ",", "k"}], "]"}], "[", 
       RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}], ",", 
       RowBox[{"x", "[", "3", "]"}]}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "3"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5959440956543055`*^9, 3.595944199291233*^9}, {
   3.595944229936986*^9, 3.595944247207974*^9}, 3.5959443002400074`*^9, {
   3.624176360291938*^9, 3.624176362366741*^9}, {3.6241887502551155`*^9, 
   3.6241888414840755`*^9}}],

Cell[BoxData[
 RowBox[{"?", "inverse"}]], "Input",
 CellChangeTimes->{{3.6241894129608793`*^9, 3.624189416626886*^9}}],

Cell[TextData[{
 "The ",
 StyleBox["Christoffel symbols of the second kind",
  FontColor->RGBColor[0, 0, 1]],
 " ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[CapitalGamma]", "ij", "k"], TraditionalForm]]],
 " (== \[CapitalGamma][k,i,j]) are defined by",
 StyleBox[" ",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox["raising",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox[" the index ",
  FontColor->RGBColor[0, 0, 1]],
 "k of \[Gamma][k, i, j]:"
}], "Text",
 CellChangeTimes->{{3.5962632168845243`*^9, 3.59626338482813*^9}, {
   3.596263485246873*^9, 3.596263521619954*^9}, 3.596263699773144*^9, {
   3.624189284384454*^9, 3.6241892917788663`*^9}, {3.625898375657461*^9, 
   3.6258983965302978`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalGamma]", "[", 
    RowBox[{"k_", ",", "i_", ",", "j_"}], "]"}], "[", "g_", "]"}], ":=", 
  RowBox[{
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"inverse", "[", 
         RowBox[{"k", ",", "l"}], "]"}], "[", "g", "]"}], "[", "##", "]"}], 
      "*", 
      RowBox[{
       RowBox[{
        RowBox[{"christoffel1", "[", 
         RowBox[{"l", ",", "i", ",", "j"}], "]"}], "[", "g", "]"}], "[", "##",
        "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"l", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "##", "}"}], "]"}]}], "}"}]}], "]"}], "&"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5960018598938203`*^9, 3.596002000216067*^9}, {
   3.5960020444421444`*^9, 3.5960021548903384`*^9}, 3.596002568119464*^9, {
   3.5962627214502306`*^9, 3.5962627274874415`*^9}, {3.5962627993099675`*^9, 
   3.5962628115559893`*^9}, {3.59626286325448*^9, 3.596262865719284*^9}, 
   3.596355750517495*^9, 3.618739494686504*^9, {3.6241893439931583`*^9, 
   3.624189372213608*^9}, {3.6241894590775604`*^9, 3.6241894613395643`*^9}, 
   3.624189493818822*^9, 3.6241895469525146`*^9, {3.6241896057982187`*^9, 
   3.624189613957033*^9}, {3.6241896785743465`*^9, 3.6241896860467596`*^9}, 
   3.6241897275896325`*^9, {3.6241995606420655`*^9, 3.624199613325358*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "\[CapitalGamma]"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6241996712094603`*^9, 3.6241997055139203`*^9}, {
  3.624199745939592*^9, 3.6241998411535597`*^9}, {3.624261050068605*^9, 
  3.6242610515506077`*^9}, {3.6259045747672033`*^9, 3.625904585207622*^9}}],

Cell[BoxData[
 StyleBox["\<\"\[CapitalGamma][k,i,j][g][x,y,...] are the Christoffel symbols \
of the second kind of the \\npseudo-Riemannian metric {g[i,j][x,y,...]} in \
local coordinates x,y,...\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.6291867744780245`*^9},
 CellTags->"Info3629190374-5022241"]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalGamma]rgT", "[", 
    RowBox[{"k_", ",", "i_", ",", "j_"}], "]"}], "[", "ff_", "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"invrgT", "[", 
         RowBox[{"k", ",", "l"}], "]"}], "[", "ff", "]"}], "[", "##", "]"}], 
      "*", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Gamma]\[Gamma]", "[", 
         RowBox[{"l", ",", "i", ",", "j"}], "]"}], "[", "ff", "]"}], "[", 
       "##", "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"l", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "##", "}"}], "]"}]}], "}"}]}], "]"}], "&"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{
  3.624190278873401*^9, {3.624190400230015*^9, 3.6241904118676353`*^9}, {
   3.62419044278889*^9, 3.6241904443176923`*^9}, 3.6241904896417723`*^9, {
   3.6241996330487933`*^9, 3.624199634000395*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "\[CapitalGamma]rgT"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6242003660714846`*^9, 3.6242003976987405`*^9}, {
  3.624200439589814*^9, 3.624200449638232*^9}, {3.6242004811366873`*^9, 
  3.6242006614090047`*^9}, {3.6242006917706585`*^9, 3.6242006954366646`*^9}, {
  3.6259046928120117`*^9, 3.6259046977572203`*^9}}],

Cell[BoxData[
 StyleBox["\<\"\[CapitalGamma]rgT[j,k,i][f][x,y,...]  are the Christoffel \
symbols of the second  kind of the \\nRiemannian metric rgT[i,j][f][x,y,...] \
of the immersion f[x,y,...] into the Euclidean n-space.\"\>", "MSG"]], "Print",\
 "PrintUsage",
 CellChangeTimes->{3.6291867746340246`*^9},
 CellTags->"Info3629190374-5022241"]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "gg", "]"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"gg", "[", 
     RowBox[{"i_", ",", "j_"}], "]"}], "[", 
    RowBox[{"u_", ",", "v_", ",", "w_"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"rgT", "[", 
      RowBox[{"i", ",", "j"}], "]"}], "[", 
     RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6241897744053144`*^9, 3.6241898451378393`*^9}, {
  3.624189973762065*^9, 3.624190035740974*^9}, {3.62420018766817*^9, 
  3.62420022169783*^9}}],

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[CapitalGamma]", "[", 
       RowBox[{"i", ",", "j", ",", "k"}], "]"}], "[", "gg", "]"}], "[", 
     RowBox[{"u", ",", "v", ",", "w"}], "]"}], "-", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"\[CapitalGamma]rgT", "[", 
       RowBox[{"i", ",", "j", ",", "k"}], "]"}], "[", 
      RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "3"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6241899016879387`*^9, 3.6241899274279833`*^9}, {
  3.6241900617326193`*^9, 3.624190151200777*^9}, {3.624200113359239*^9, 
  3.624200174064946*^9}, {3.624200235269854*^9, 3.624200317038598*^9}}],

Cell[BoxData[
 RowBox[{"Clear", "[", "gg", "]"}]], "Input",
 CellChangeTimes->{{3.626010613026805*^9, 3.6260106180656137`*^9}}],

Cell["\<\
Clearly, the Christoffel symbols of the second kind are symmetric in the last \
two indices:\
\>", "Text",
 CellChangeTimes->{{3.62420089672362*^9, 3.6242009553107233`*^9}}],

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[CapitalGamma]", "[", 
       RowBox[{"i", ",", "j", ",", "k"}], "]"}], "[", "gg", "]"}], "[", 
     RowBox[{"u", ",", "v", ",", "w"}], "]"}], "-", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"\[CapitalGamma]", "[", 
       RowBox[{"i", ",", "k", ",", "j"}], "]"}], "[", "gg", "]"}], "[", 
     RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "3"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6241899016879387`*^9, 3.6241899274279833`*^9}, {
  3.6241900617326193`*^9, 3.624190151200777*^9}, {3.624200113359239*^9, 
  3.624200174064946*^9}, {3.624200235269854*^9, 3.624200317038598*^9}, {
  3.6242007774696093`*^9, 3.6242008250616937`*^9}}],

Cell[TextData[{
 StyleBox["Lemma 2. ",
  FontColor->RGBColor[0, 0, 1]],
 "The Christoffel symbols are defined  by the Riemannian metric rgT[i,j] \
only: isometric Riemannian manifolds (or immersions) have the same \
Christoffel symbols, if the isometry ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["F", "iso"], TraditionalForm]]],
 " is defined by equal coordinates: "
}], "Text",
 CellChangeTimes->{{3.596263795220603*^9, 3.5962638134276443`*^9}, {
   3.5962638465165367`*^9, 3.596263997252158*^9}, {3.5964667908862085`*^9, 
   3.596466795909417*^9}, 3.6186423947728305`*^9, {3.624201102529583*^9, 
   3.6242012044449625`*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["F", "iso"], "[", 
    RowBox[{"x", "[", 
     RowBox[{"u", ",", "v", ",", "..."}], "]"}], "]"}], " ", "=", 
   RowBox[{"y", "[", 
    RowBox[{"u", ",", "v", ",", "..."}], "]"}]}], TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6242013672094502`*^9, 3.6242014345169687`*^9}}],

Cell["They are symmetric with respect to the last two indices:", "Text",
 CellChangeTimes->{{3.596263795220603*^9, 3.5962638134276443`*^9}, {
   3.5962638465165367`*^9, 3.596263997252158*^9}, {3.5964667908862085`*^9, 
   3.596466795909417*^9}, 3.6186423947728305`*^9, {3.624201102529583*^9, 
   3.6242012044449625`*^9}}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"\[CapitalGamma]rgT", "[", 
        RowBox[{"k", ",", "i", ",", "j"}], "]"}], "[", 
       RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}]}], "]"}], " ", "-", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"\[CapitalGamma]rgT", "[", 
        RowBox[{"k", ",", "j", ",", "i"}], "]"}], "[", 
       RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}]}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "2"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.596092519613618*^9, 3.596092710557954*^9}, {
   3.5960927565312347`*^9, 3.5960927874972887`*^9}, {3.5960928282913604`*^9, 
   3.596092857510212*^9}, {3.5962629222849836`*^9, 3.596262936371808*^9}, 
   3.614861862061944*^9, 3.6148635020772247`*^9, {3.624201028498452*^9, 
   3.6242010438070793`*^9}}],

Cell["\[FilledRectangle]", "Text",
 CellChangeTimes->{{3.596526684213423*^9, 3.596526684219423*^9}}],

Cell[TextData[{
 StyleBox["Example",
  FontColor->RGBColor[0, 0, 1]],
 ": The unit sphere:"
}], "Text",
 CellChangeTimes->{{3.596264188339088*^9, 3.5962642170837317`*^9}, {
  3.5964670000357757`*^9, 3.5964670129369984`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Simplify", "[", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"i", ",", "j", ",", "k"}], "}"}], " ", "\[Rule]", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"\[CapitalGamma]rgT", "[", 
         RowBox[{"i", ",", "j", ",", "k"}], "]"}], "[", 
        RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", "2"}], "}"}]}], "]"}], "]"}], "//", 
  "TableForm"}]], "Input",
 CellChangeTimes->{{3.5964375688386*^9, 3.59643760902427*^9}, {
   3.5964376738579845`*^9, 3.596437674918786*^9}, {3.5964377439645076`*^9, 
   3.5964377584257326`*^9}, {3.5964378236026473`*^9, 3.5964378310594606`*^9}, 
   3.614861862077544*^9, 3.6148635020928245`*^9, 3.6242015198193197`*^9}],

Cell[TextData[{
 "\[FilledRectangle]\n",
 StyleBox["Example",
  FontColor->RGBColor[0, 0, 1]],
 ". A general surface:"
}], "Text",
 CellChangeTimes->{{3.596467059924281*^9, 3.5964670689566965`*^9}, {
  3.5965268092775755`*^9, 3.5965268485568223`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "3"}], ";", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"\[CapitalGamma]rgT", "[", 
      RowBox[{"1", ",", "2", ",", "1"}], "]"}], "[", "sf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.596002193906007*^9, 3.596002239302087*^9}, {
   3.596002307755007*^9, 3.5960023288774443`*^9}, 3.5960023753967257`*^9, {
   3.59600297016337*^9, 3.596002973298976*^9}, {3.596003011160242*^9, 
   3.5960030221738615`*^9}, {3.6187502576699853`*^9, 3.6187502629271946`*^9}, 
   3.6242015231421256`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"\[CapitalGamma]rgT", "[", 
      RowBox[{"1", ",", "1", ",", "2"}], "]"}], "[", 
     RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.596002193906007*^9, 3.596002239302087*^9}, {
   3.596002307755007*^9, 3.5960023288774443`*^9}, 3.5960023753967257`*^9, {
   3.59600297016337*^9, 3.596002973298976*^9}, {3.596003011160242*^9, 
   3.5960030221738615`*^9}, {3.5960030787083607`*^9, 3.5960031412332706`*^9}, 
   3.596003187549752*^9, {3.596264292944071*^9, 3.5962643306622286`*^9}, 
   3.624201525138929*^9}],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", "%%"}]], "Input",
 CellChangeTimes->{{3.5962643468541546`*^9, 3.5962643502293477`*^9}}],

Cell[TextData[{
 "\[FilledRectangle]",
 StyleBox["\nExample",
  FontColor->RGBColor[0, 0, 1]],
 ". The Euclidean space :"
}], "Text",
 CellChangeTimes->{{3.596438847572846*^9, 3.5964388776808987`*^9}, {
  3.5965269169567347`*^9, 3.5965269247021775`*^9}}],

Cell["\<\
The Euclidean space is immersed onto itself by the identity map\
\>", "Text",
 CellChangeTimes->{{3.596439093117277*^9, 3.596439130713343*^9}}],

Cell[BoxData[
 RowBox[{"identity", "[", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}], ",", 
   RowBox[{"x", "[", "3", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.62401515192708*^9, 3.6240151854827394`*^9}}],

Cell["The metric has the standard orthonormal shape", "Text",
 CellChangeTimes->{{3.5964391499637775`*^9, 3.596439188792245*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"rgT", "[", 
       RowBox[{"i", ",", "j"}], "]"}], "[", "identity", "]"}], "[", 
     RowBox[{
      RowBox[{"x", "[", "1", "]"}], ",", 
      RowBox[{"x", "[", "2", "]"}], ",", 
      RowBox[{"x", "[", "3", "]"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "3"}], "}"}]}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.5964381997037077`*^9, 3.596438289575466*^9}, {
   3.5964383447839627`*^9, 3.596438428010109*^9}, 3.5964384869314127`*^9, {
   3.5964386078784246`*^9, 3.596438784392735*^9}, {3.5965269616812925`*^9, 
   3.5965269678126435`*^9}, 3.6148618621087437`*^9, 3.614863502108424*^9, 
   3.6186423963049183`*^9, {3.6240152531712584`*^9, 3.624015256759265*^9}}],

Cell["Thus we define", "Text",
 CellChangeTimes->{{3.6242620209831104`*^9, 3.624262025787919*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"euclid", "[", 
   RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"rgT", "[", 
      RowBox[{"i", ",", "j"}], "]"}], "[", "identity", "]"}], "[", "##", 
    "]"}], "&"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6242616834453173`*^9, 3.624261702336951*^9}, {
  3.62426179294191*^9, 3.624261831770378*^9}}],

Cell[BoxData[
 RowBox[{"?", "euclid"}]], "Input",
 CellChangeTimes->{{3.624339607984866*^9, 3.6243396885278077`*^9}, 
   3.624339719977463*^9, {3.6259047650607386`*^9, 3.6259047709731493`*^9}}],

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"euclid", "[", 
     RowBox[{"i", ",", "j"}], "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "3"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6242618459352026`*^9, 3.624261910160516*^9}}],

Cell["All the Christoffel symbols are zero:", "Text",
 CellChangeTimes->{{3.5964392202731004`*^9, 3.5964392394299345`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Simplify", "[", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"i", ",", "j", ",", "k"}], "}"}], " ", "\[Rule]", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"\[CapitalGamma]rgT", "[", 
         RowBox[{"i", ",", "j", ",", "k"}], "]"}], "[", "identity", "]"}], 
       "[", 
       RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", "3"}], "}"}]}], "]"}], "]"}], "//", 
  "TableForm"}]], "Input",
 CellChangeTimes->{{3.5964375688386*^9, 3.59643760902427*^9}, {
   3.5964376738579845`*^9, 3.596437674918786*^9}, {3.5964377439645076`*^9, 
   3.5964377584257326`*^9}, {3.5964378236026473`*^9, 
   3.5964378310594606`*^9}, {3.596438962045847*^9, 3.596438993760703*^9}, 
   3.614861862124344*^9, 3.614863502108424*^9, {3.624015286555317*^9, 
   3.624015289472522*^9}, 3.624201528633335*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Simplify", "[", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"i", ",", "j", ",", "k"}], "}"}], " ", "\[Rule]", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"\[CapitalGamma]", "[", 
         RowBox[{"i", ",", "j", ",", "k"}], "]"}], "[", "euclid", "]"}], "[", 
       RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", "3"}], "}"}]}], "]"}], "]"}], "//", 
  "TableForm"}]], "Input",
 CellChangeTimes->{{3.5964375688386*^9, 3.59643760902427*^9}, {
   3.5964376738579845`*^9, 3.596437674918786*^9}, {3.5964377439645076`*^9, 
   3.5964377584257326`*^9}, {3.5964378236026473`*^9, 
   3.5964378310594606`*^9}, {3.596438962045847*^9, 3.596438993760703*^9}, 
   3.614861862124344*^9, 3.614863502108424*^9, {3.624015286555317*^9, 
   3.624015289472522*^9}, 3.624201528633335*^9, {3.6242620661295896`*^9, 
   3.624262087985228*^9}}],

Cell[TextData[{
 "\[FilledRectangle]\nNow we define the ",
 StyleBox["absolute differential",
  FontColor->RGBColor[0, 0, 1]],
 "  of a tangent vector field vw[x[1], x[2],... ] on the pseudo-Riemannian \
manifold with metric tensor g[i,j][x[1], x[2],... ] and local coordinates \
x[1], x[2],...:"
}], "Text",
 CellChangeTimes->{{3.5964393669757586`*^9, 3.5964394522767076`*^9}, {
   3.5964394822911606`*^9, 3.5964395605564985`*^9}, {3.596439649164654*^9, 
   3.5964396694290895`*^9}, {3.5964397029067483`*^9, 
   3.5964397744796743`*^9}, {3.596441447099012*^9, 3.596441496660299*^9}, {
   3.5964615275749636`*^9, 3.596461551396206*^9}, {3.5965270574777718`*^9, 
   3.5965270926457834`*^9}, 3.61744122138057*^9, 3.6242781546884165`*^9, {
   3.624278690346557*^9, 3.6242787603594804`*^9}, {3.6242787958183427`*^9, 
   3.624278806285961*^9}, {3.625904908828992*^9, 3.625904913821001*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DD", "[", "g_", "]"}], "[", "vw_", "]"}], "[", "x__", "]"}], ":=",
   " ", 
  RowBox[{"Transpose", "[", 
   RowBox[{"Simplify", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"D", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"vw", "[", "x", "]"}], "[", 
          RowBox[{"[", "k", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"{", "x", "}"}], "[", 
          RowBox[{"[", "j", "]"}], "]"}]}], "]"}], "+", 
       RowBox[{"Sum", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalGamma]", "[", 
             RowBox[{"k", ",", "i", ",", "j"}], "]"}], "[", "g", "]"}], "[", 
           "x", "]"}], "*", 
          RowBox[{
           RowBox[{"vw", "[", "x", "]"}], "[", 
           RowBox[{"[", "i", "]"}], "]"}]}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", 
           RowBox[{"Length", "[", 
            RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", 
        RowBox[{"Length", "[", 
         RowBox[{"{", "x", "}"}], "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"k", ",", 
        RowBox[{"Length", "[", 
         RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}], "]"}], 
   "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.596267909001898*^9, 3.596268002502246*^9}, {
   3.596268233767473*^9, 3.5962682417899323`*^9}, 3.596268272390682*^9, {
   3.5962683739424906`*^9, 3.596268434579959*^9}, {3.596268467832861*^9, 
   3.596268467847862*^9}, {3.596268546079336*^9, 3.59626858233541*^9}, {
   3.5962686311192007`*^9, 3.596268685327301*^9}, {3.596268721063345*^9, 
   3.5962687800967216`*^9}, {3.5962688200340056`*^9, 3.5962689711946516`*^9}, 
   3.596270942293791*^9, {3.5962709724375153`*^9, 3.5962709908775697`*^9}, {
   3.5962711252152534`*^9, 3.596271142601248*^9}, {3.5962712161694555`*^9, 
   3.5962712481452847`*^9}, {3.596271295953019*^9, 3.5962712978171253`*^9}, {
   3.5962714903281364`*^9, 3.5962715032818775`*^9}, {3.5962715560498953`*^9, 
   3.5962715738429136`*^9}, {3.5963609856954927`*^9, 
   3.5963610055046253`*^9}, {3.596361147280734*^9, 3.5963611482237883`*^9}, 
   3.5963690709758096`*^9, {3.59636910944601*^9, 3.5963691235168147`*^9}, {
   3.5963691564216967`*^9, 3.5963691658852386`*^9}, {3.5963695531093864`*^9, 
   3.5963695911245604`*^9}, {3.596369833029397*^9, 3.596369836182577*^9}, {
   3.5963701344546375`*^9, 3.5963701758290033`*^9}, {3.5963707897741194`*^9, 
   3.5963708379418745`*^9}, {3.5963708850625696`*^9, 
   3.5963709097469816`*^9}, {3.5963733257331676`*^9, 3.5963733297783995`*^9}, 
   3.614861862139944*^9, 3.6148635021240244`*^9, {3.624263265459696*^9, 
   3.6242632860049324`*^9}, {3.624280333201043*^9, 3.6242803452910643`*^9}, 
   3.625904928750227*^9}],

Cell[BoxData[
 RowBox[{"?", "DD"}]], "Input",
 CellChangeTimes->{{3.624284255422332*^9, 3.624284423185026*^9}, {
  3.624284507189603*^9, 3.624284581883534*^9}, {3.6259048914153614`*^9, 
  3.6259048955181684`*^9}, {3.625905113178152*^9, 3.6259051175149593`*^9}}],

Cell[TextData[{
 StyleBox["Example",
  FontColor->RGBColor[0, 0, 1]],
 ". Euclidean space\nIn this case the absolute differential coincides with \
the total differential of the vector field:"
}], "Text",
 CellChangeTimes->{{3.624263117477836*^9, 3.6242632398756514`*^9}, 
   3.624278292811059*^9, 3.6242783440883493`*^9, {3.624280409781577*^9, 
   3.6242804190947933`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DD", "[", "euclid", "]"}], "[", 
   RowBox[{"vv", "[", "\[Xi]", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.624262310051618*^9, 3.6242623672413187`*^9}}],

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"{", 
     RowBox[{"u", ",", "v"}], "}"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6242626554206247`*^9, 3.6242627572264037`*^9}, {
  3.6242628996390533`*^9, 3.6242629758139877`*^9}, {3.6242803663979015`*^9, 
  3.624280374385115*^9}}],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", "%%"}]], "Input",
 CellChangeTimes->{{3.624262981398797*^9, 3.624262983504801*^9}}],

Cell[TextData[{
 "\[FilledSquare]\n",
 "The tangent",
 StyleBox[" vector field",
  FontColor->RGBColor[0, 0, 1]],
 " on a surface in the Euclidean space is"
}], "Text",
 CellChangeTimes->{{3.5964653794113293`*^9, 3.5964654199402003`*^9}, 
   3.6130241204895363`*^9, 3.6238485533893394`*^9, {3.624263382725102*^9, 
   3.6242633857359076`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tvf", "[", "\[Xi]", "]"}], "[", "sf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{
  3.5963696782625446`*^9, 3.596370011736618*^9, {3.596370260543849*^9, 
   3.5963702620869374`*^9}, 3.5963713146021376`*^9, 3.5963734001034217`*^9, 
   3.59643714724786*^9, {3.5964415386243725`*^9, 3.596441564068017*^9}, {
   3.5964625091130877`*^9, 3.596462532559929*^9}, {3.5964647726950636`*^9, 
   3.5964647751910677`*^9}}],

Cell["\<\
This is the image of the tangent vector field vv[\[Xi]][u,v] under the \
differential of the immersion:\
\>", "Text",
 CellChangeTimes->{{3.5964655028543463`*^9, 3.5964655765332756`*^9}, 
   3.6130241253778157`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], ".", 
  RowBox[{
   RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5964629299238267`*^9, 3.5964629384414415`*^9}, {
  3.596464531128639*^9, 3.596464582265529*^9}}],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", 
  RowBox[{
   RowBox[{
    RowBox[{"tvf", "[", "\[Xi]", "]"}], "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5964645962899537`*^9, 3.5964646521848516`*^9}}],

Cell[TextData[{
 StyleBox["Example",
  FontColor->RGBColor[0, 0, 1]],
 ". A tangent vector field on the unit sphere.\nWe calculate its total \
differential"
}], "Text",
 CellChangeTimes->{{3.5964656072653294`*^9, 3.59646561355214*^9}, {
   3.5964671233539925`*^9, 3.596467138548419*^9}, 3.5964671699824743`*^9, {
   3.5964673422847767`*^9, 3.596467345123982*^9}, 3.5965274369334755`*^9, {
   3.5965274799579363`*^9, 3.5965274962528687`*^9}, 3.6132221202317195`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"d", "[", 
     RowBox[{
      RowBox[{"tvf", "[", "\[Xi]", "]"}], "[", 
      RowBox[{"sphero", "[", "1", "]"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ")"}], ".", 
  RowBox[{"{", 
   RowBox[{"s1", ",", "s2"}], "}"}]}]], "Input"],

Cell[TextData[{
 StyleBox["Statement.",
  FontVariations->{"Underline"->True},
  FontColor->RGBColor[0, 0, 1]],
 "\n",
 StyleBox["The absolute differential coincides with the tangent projection of \
the total differential:",
  FontSlant->"Italic"]
}], "Text",
 CellChangeTimes->{{3.5965275477568145`*^9, 3.5965275515490313`*^9}, {
   3.5965276068371935`*^9, 3.5965277272290792`*^9}, {3.5965280286613207`*^9, 
   3.5965280371008034`*^9}, 3.623848619034254*^9, 3.6242634401488028`*^9}],

Cell["Here is the mentioned tangent projection:", "Text",
 CellChangeTimes->{{3.6238487892461534`*^9, 3.6238488119129934`*^9}, 
   3.624279022985942*^9}],

Cell[BoxData[
 RowBox[{"pdv", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"pT", "[", 
      RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"d", "[", 
        RowBox[{
         RowBox[{"tvf", "[", "\[Xi]", "]"}], "[", 
         RowBox[{"sphero", "[", "1", "]"}], "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ")"}], ".", 
     RowBox[{"{", 
      RowBox[{"s1", ",", "s2"}], "}"}]}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.596461679628431*^9, {3.596461809795059*^9, 3.5964618218694806`*^9}, {
   3.596461958931321*^9, 3.5964619847337666`*^9}, {3.596462552715164*^9, 
   3.596462554743168*^9}, {3.5964636994287786`*^9, 3.5964637060743904`*^9}, {
   3.596463879593495*^9, 3.596463881980299*^9}, 3.5964639847844796`*^9}],

Cell["\<\
Calculate now the the absolute differential. The Riemannian metric is\
\>", "Text",
 CellChangeTimes->{{3.596527770333545*^9, 3.596527809484784*^9}, 
   3.6238487734745255`*^9, {3.624279040769973*^9, 3.6242790508787904`*^9}, 
   3.624279195350644*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"gsph", "[", 
    RowBox[{"i_", ",", "j_"}], "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"rgT", "[", 
     RowBox[{"i", ",", "j"}], "]"}], "[", 
    RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5962709980749817`*^9, 3.5962711091643353`*^9}, {
   3.59627160561073*^9, 3.596271620220566*^9}, {3.596370417199809*^9, 
   3.5963704177608414`*^9}, 3.5963713429137573`*^9, 3.596373557527426*^9, 
   3.596437289161309*^9, 3.5964418711857567`*^9, {3.596441937485873*^9, 
   3.596441940169078*^9}, {3.5964626737089767`*^9, 3.596462691680208*^9}, {
   3.5964640216941442`*^9, 3.5964640283085566`*^9}, {3.624263661653592*^9, 
   3.6242637508545485`*^9}, {3.624263795439427*^9, 3.6242638781819725`*^9}, {
   3.6242639871013637`*^9, 3.624263995837379*^9}, {3.6242640661623025`*^9, 
   3.62426407049911*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"gsph", "[", 
   RowBox[{"1", ",", "1"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6242640079586*^9, 3.6242640447278647`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DD", "[", "gsph", "]"}], "[", 
   RowBox[{"vv", "[", "\[Xi]", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5962709980749817`*^9, 3.5962711091643353`*^9}, {
   3.59627160561073*^9, 3.596271620220566*^9}, {3.596370417199809*^9, 
   3.5963704177608414`*^9}, 3.5963713429137573`*^9, 3.596373557527426*^9, 
   3.596437289161309*^9, 3.5964418711857567`*^9, {3.596441937485873*^9, 
   3.596441940169078*^9}, {3.5964626737089767`*^9, 3.596462691680208*^9}, {
   3.5964640216941442`*^9, 3.5964640283085566`*^9}, {3.624263661653592*^9, 
   3.6242637508545485`*^9}, {3.624263795439427*^9, 3.6242638781819725`*^9}, {
   3.6242639871013637`*^9, 3.624263995837379*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DD", "[", "gsph", "]"}], "[", 
    RowBox[{"vv", "[", "\[Xi]", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], ".", 
  RowBox[{"{", 
   RowBox[{"s1", ",", "s2"}], "}"}]}]], "Input",
 CellChangeTimes->{{3.5964627858575735`*^9, 3.5964628284456487`*^9}, 
   3.59646406489062*^9, {3.5964641314715376`*^9, 3.5964641589587855`*^9}, {
   3.596465191508999*^9, 3.5964652077018275`*^9}, 3.6148618621711445`*^9, 
   3.6149177936677647`*^9, {3.624277320835752*^9, 3.6242773228949556`*^9}, {
   3.624280472665288*^9, 3.6242804907925196`*^9}}],

Cell["Its image under the differential of the immersion is", "Text",
 CellChangeTimes->{{3.5965278541893415`*^9, 3.5965279038131795`*^9}, {
  3.6238489243111906`*^9, 3.623848953436442*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", 
     RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ".", "%"}], "]"}]], "Input",
 CellChangeTimes->{{3.596462848039283*^9, 3.5964628573992996`*^9}, {
   3.596462905369384*^9, 3.596462918832207*^9}, {3.5964629656010895`*^9, 
   3.5964630031659555`*^9}, 3.5964633152157035`*^9, {3.5964641983800545`*^9, 
   3.5964642202824936`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"diff\[Xi]", "=", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.5964630273147984`*^9, 3.596463049482437*^9}}],

Cell[BoxData[
 RowBox[{"Dimensions", "[", "diff\[Xi]", "]"}]], "Input",
 CellChangeTimes->{{3.624282151805437*^9, 3.6242821789338846`*^9}}],

Cell["Check the statement:", "Text",
 CellChangeTimes->{{3.596528165477146*^9, 3.5965281754927187`*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", " ", 
  RowBox[{"diff\[Xi]", " ", "-", "pdv"}], "]"}]], "Input",
 CellChangeTimes->{{3.596463140479797*^9, 3.5964631670622435`*^9}, {
  3.5964632076067147`*^9, 3.596463212895124*^9}}],

Cell[TextData[{
 "\[FilledRectangle]",
 StyleBox["\nExample",
  FontColor->RGBColor[0, 0, 1]],
 ". An arbitrary tangent vector field on a general surface\nWe repeat the \
last example for a general surface."
}], "Text",
 CellChangeTimes->{{3.5964656072653294`*^9, 3.59646561355214*^9}, {
  3.596467189139308*^9, 3.5964672158777547`*^9}, {3.5964673572296033`*^9, 
  3.596467407992092*^9}, {3.5965281927807074`*^9, 3.5965281982890224`*^9}, {
  3.624281781819587*^9, 3.6242818810981617`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"d", "[", 
     RowBox[{
      RowBox[{"tvf", "[", "\[Xi]", "]"}], "[", "sf", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ")"}], ".", 
  RowBox[{"{", 
   RowBox[{"s1", ",", "s2"}], "}"}]}]], "Input",
 CellChangeTimes->{3.596465746994775*^9, 3.6238490139801483`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"s2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
    RowBox[{"s1", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ",", 
   RowBox[{
    RowBox[{"s2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
    RowBox[{"s1", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ",", 
   RowBox[{
    RowBox[{"s2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
    RowBox[{"s1", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}]}], "}"}]], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{3.596464906262498*^9, 3.596465810658487*^9, 
  3.6132223900809937`*^9, 3.6187506160182147`*^9, 3.6238490139801483`*^9, 
  3.623849102323103*^9, 3.6242773940622807`*^9, 3.625899004880234*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"s2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
    RowBox[{"s1", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ",", 
   RowBox[{
    RowBox[{"s2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
    RowBox[{"s1", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ",", 
   RowBox[{
    RowBox[{"s2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
    RowBox[{"s1", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}]}], "}"}]], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{3.596464906262498*^9, 3.596465810658487*^9, 
  3.6132223900809937`*^9, 3.6187506160182147`*^9, 3.6238490139801483`*^9, 
  3.623849102323103*^9, 3.6242773940622807`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"s2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
    RowBox[{"s1", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ",", 
   RowBox[{
    RowBox[{"s2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
    RowBox[{"s1", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ",", 
   RowBox[{
    RowBox[{"s2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
    RowBox[{"s1", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}]}], "}"}]], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{3.596464906262498*^9, 3.596465810658487*^9, 
  3.6132223900809937`*^9, 3.6187506160182147`*^9, 3.6238490139801483`*^9, 
  3.623849102323103*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"s2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
    RowBox[{"s1", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ",", 
   RowBox[{
    RowBox[{"s2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
    RowBox[{"s1", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ",", 
   RowBox[{
    RowBox[{"s2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
    RowBox[{"s1", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ",", 
   RowBox[{
    RowBox[{"s2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "4", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "4", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "2"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "4", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "4", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
    RowBox[{"s1", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "4", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "4", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"\[Xi]", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "4", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "4", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"2", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}]}], "}"}]], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{3.596464906262498*^9, 3.596465810658487*^9, 
  3.6132223900809937`*^9, 3.6187506160182147`*^9, 3.6238490139801483`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"pdvsf", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"pT", "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"d", "[", 
        RowBox[{
         RowBox[{"tvf", "[", "\[Xi]", "]"}], "[", "sf", "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ")"}], ".", 
     RowBox[{"{", 
      RowBox[{"s1", ",", "s2"}], "}"}]}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.596461679628431*^9, {3.596461809795059*^9, 3.5964618218694806`*^9}, {
   3.596461958931321*^9, 3.5964619847337666`*^9}, {3.596462552715164*^9, 
   3.596462554743168*^9}, {3.5964636994287786`*^9, 3.5964637060743904`*^9}, {
   3.596463879593495*^9, 3.596463881980299*^9}, 3.5964639847844796`*^9, {
   3.5964657542643876`*^9, 3.596465755200389*^9}, 3.6238490139801483`*^9, {
   3.624282327087345*^9, 3.6242823275397453`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "3"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
         RowBox[{"2", " ", "s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", "s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "2"}], " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "2", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"2", " ", "s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"2", " ", "s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"s2", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                    RowBox[{"s1", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
               "+", 
               RowBox[{"s1", " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "2", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", "s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"2", " ", "s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"-", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"s2", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                    RowBox[{"s1", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
            "+", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
            "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", "s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "3"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", "s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], "+", 
         RowBox[{"s1", " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", "s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "2"}], " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"s2", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                    RowBox[{"s1", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
               "+", 
               RowBox[{"s1", " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "2"}], " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", "s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "-", 
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "3", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
               RowBox[{"s1", " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "3", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"2", " ", "s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
            "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", "s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "3"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
         RowBox[{"s1", " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "2"}], " ", "s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
         RowBox[{"2", " ", "s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", "s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", "s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
            "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "-", 
               RowBox[{"2", " ", "s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "-", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "-", 
               RowBox[{"2", " ", "s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", "s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"2", " ", "s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"\[Xi]", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"s2", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                    RowBox[{"s1", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
              ")"}]}]}], ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"]}], "+", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "2"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
        ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}]}], 
  "}"}]], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{
  3.5964619939689827`*^9, 3.5964625593139763`*^9, 3.596463708398794*^9, {
   3.5964639658616467`*^9, 3.5964639892460876`*^9}, 3.596465000127863*^9, 
   3.596465838644936*^9, 3.613222448908697*^9, 3.6187511898652225`*^9, 
   3.623849014042548*^9, 3.6238491531323924`*^9, 3.624277469566413*^9, 
   3.624280672626439*^9, 3.624282368006217*^9, 3.6258990626531353`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "3"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
         RowBox[{"2", " ", "s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", "s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "2"}], " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "2", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"2", " ", "s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"2", " ", "s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"s2", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                    RowBox[{"s1", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
               "+", 
               RowBox[{"s1", " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "2", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", "s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"2", " ", "s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"-", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"s2", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                    RowBox[{"s1", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
            "+", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
            "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", "s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "3"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", "s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], "+", 
         RowBox[{"s1", " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", "s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "2"}], " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"s2", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                    RowBox[{"s1", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
               "+", 
               RowBox[{"s1", " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "2"}], " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", "s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "-", 
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "3", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
               RowBox[{"s1", " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "3", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"2", " ", "s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
            "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", "s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "3"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
         RowBox[{"s1", " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "2"}], " ", "s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
         RowBox[{"2", " ", "s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", "s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", "s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
            "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "-", 
               RowBox[{"2", " ", "s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "-", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "-", 
               RowBox[{"2", " ", "s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", "s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"2", " ", "s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"\[Xi]", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"s2", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                    RowBox[{"s1", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
              ")"}]}]}], ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"]}], "+", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "2"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
        ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}]}], 
  "}"}]], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{
  3.5964619939689827`*^9, 3.5964625593139763`*^9, 3.596463708398794*^9, {
   3.5964639658616467`*^9, 3.5964639892460876`*^9}, 3.596465000127863*^9, 
   3.596465838644936*^9, 3.613222448908697*^9, 3.6187511898652225`*^9, 
   3.623849014042548*^9, 3.6238491531323924`*^9, 3.624277469566413*^9, 
   3.624280672626439*^9, 3.624282368006217*^9}]
}, Closed]],

Cell["The Riemannian metric remains unspecified:", "Text",
 CellChangeTimes->{{3.624279277235188*^9, 3.6242793228340683`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"gsf", "[", 
    RowBox[{"i_", ",", "j_"}], "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"rgT", "[", 
     RowBox[{"i", ",", "j"}], "]"}], "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6242775807322083`*^9, 3.624277624412285*^9}, {
  3.624277687077595*^9, 3.6242776970148125`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DD", "[", "gsf", "]"}], "[", 
   RowBox[{"vv", "[", "\[Xi]", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5962709980749817`*^9, 3.5962711091643353`*^9}, {
   3.59627160561073*^9, 3.596271620220566*^9}, {3.596370417199809*^9, 
   3.5963704177608414`*^9}, 3.5963713429137573`*^9, 3.596373557527426*^9, 
   3.596437289161309*^9, 3.5964418711857567`*^9, {3.596441937485873*^9, 
   3.596441940169078*^9}, {3.5964626737089767`*^9, 3.596462691680208*^9}, {
   3.5964640216941442`*^9, 3.5964640283085566`*^9}, 3.596465760769599*^9, 
   3.623849014042548*^9, 3.624277712131239*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"\[Xi]", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"\[Xi]", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"\[Xi]", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"\[Xi]", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"\[Xi]", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"\[Xi]", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "-", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"2", " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"\[Xi]", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "-", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"\[Xi]", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
           "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"\[Xi]", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
          ")"}]}]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
        RowBox[{"2", " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], "-", 
        RowBox[{"2", " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"\[Xi]", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "2"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "2"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "2"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "2"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "2"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "2"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "2"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], ")"}],
        "/", 
       RowBox[{"(", 
        RowBox[{"2", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], 
        ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "2"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], ")"}],
        "/", 
       RowBox[{"(", 
        RowBox[{"2", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], 
        ")"}]}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"\[Xi]", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], ")"}],
        "/", 
       RowBox[{"(", 
        RowBox[{"2", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], 
        ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], ")"}],
        "/", 
       RowBox[{"(", 
        RowBox[{"2", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], 
        ")"}]}]}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"\[Xi]", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "2"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"-", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "2"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}],
            "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"-", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "-", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "2"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "2"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], 
             ")"}]}]}], ")"}]}], ")"}], "/", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], ")"}],
        "/", 
       RowBox[{"(", 
        RowBox[{"2", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], 
        ")"}]}]}]}], "}"}]}], "}"}]], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{3.596271625477867*^9, 3.596355967841477*^9, 
  3.596356019727168*^9, 3.596361195033466*^9, 3.596370340497422*^9, 
  3.5963704187088957`*^9, 3.596371001424225*^9, 3.5963721256285257`*^9, 
  3.596372225653247*^9, 3.59637229808539*^9, 3.5963732860038958`*^9, 
  3.5963735584984818`*^9, 3.5964372904405107`*^9, 3.5964418616697397`*^9, 
  3.5964419425714817`*^9, 3.5964626943946133`*^9, 3.5964640348761673`*^9, 
  3.5964650330439205`*^9, 3.5964658604537745`*^9, 3.6132224740559406`*^9, 
  3.618751604825951*^9, 3.623849014042548*^9, 3.623849156798399*^9, 
  3.6242777236128592`*^9, 3.6242801018682365`*^9, 3.624280705121296*^9, 
  3.6258991231112423`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"\[Xi]", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"\[Xi]", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"\[Xi]", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"\[Xi]", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"\[Xi]", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"\[Xi]", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "-", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"2", " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"\[Xi]", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "-", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"\[Xi]", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
           "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"\[Xi]", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
          ")"}]}]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
        RowBox[{"2", " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], "-", 
        RowBox[{"2", " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"\[Xi]", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "2"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "2"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "2"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "2"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "2"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "2"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "2"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], ")"}],
        "/", 
       RowBox[{"(", 
        RowBox[{"2", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], 
        ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "2"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], ")"}],
        "/", 
       RowBox[{"(", 
        RowBox[{"2", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], 
        ")"}]}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"\[Xi]", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], ")"}],
        "/", 
       RowBox[{"(", 
        RowBox[{"2", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], 
        ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"2", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], ")"}],
        "/", 
       RowBox[{"(", 
        RowBox[{"2", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], 
        ")"}]}]}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"\[Xi]", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "2"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"-", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "2"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}],
            "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{"u", ",", "v"}], "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"-", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "-", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "2"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "2"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], 
             ")"}]}]}], ")"}]}], ")"}], "/", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
           RowBox[{"2", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], ")"}],
        "/", 
       RowBox[{"(", 
        RowBox[{"2", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "2"]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], 
        ")"}]}]}]}], "}"}]}], "}"}]], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{3.596271625477867*^9, 3.596355967841477*^9, 
  3.596356019727168*^9, 3.596361195033466*^9, 3.596370340497422*^9, 
  3.5963704187088957`*^9, 3.596371001424225*^9, 3.5963721256285257`*^9, 
  3.596372225653247*^9, 3.59637229808539*^9, 3.5963732860038958`*^9, 
  3.5963735584984818`*^9, 3.5964372904405107`*^9, 3.5964418616697397`*^9, 
  3.5964419425714817`*^9, 3.5964626943946133`*^9, 3.5964640348761673`*^9, 
  3.5964650330439205`*^9, 3.5964658604537745`*^9, 3.6132224740559406`*^9, 
  3.618751604825951*^9, 3.623849014042548*^9, 3.623849156798399*^9, 
  3.6242777236128592`*^9, 3.6242801018682365`*^9, 3.624280705121296*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"Dimensions", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6242801242854757`*^9, 3.6242801338014927`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DD", "[", "gsf", "]"}], "[", 
    RowBox[{"vv", "[", "\[Xi]", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], ".", 
  RowBox[{"{", 
   RowBox[{"s1", ",", "s2"}], "}"}]}]], "Input",
 CellChangeTimes->{{3.5964627858575735`*^9, 3.5964628284456487`*^9}, 
   3.59646406489062*^9, {3.5964641314715376`*^9, 3.5964641589587855`*^9}, {
   3.596465191508999*^9, 3.5964652077018275`*^9}, 3.596465764420006*^9, 
   3.6148618623271446`*^9, 3.6149177938081646`*^9, 3.623849014042548*^9, 
   3.6242777709901423`*^9, {3.624280734558548*^9, 3.6242807405957584`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"resa", " ", "=", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.6187544272921085`*^9, 3.618754491829422*^9}, 
   3.6238490140581484`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Dimensions", "[", "resa", "]"}]], "Input",
 CellChangeTimes->{{3.6187544994890356`*^9, 3.6187545076634502`*^9}, 
   3.6238490140581484`*^9}],

Cell[BoxData[
 RowBox[{"{", "2", "}"}]], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{3.6187545101906548`*^9, 3.6238490140581484`*^9, 
  3.623849157016799*^9, 3.62427780938181*^9, 3.624280767287405*^9, 
  3.6258991477456856`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Dimensions", "[", 
  RowBox[{
   RowBox[{"d", "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6187545853515863`*^9, 3.6187546150072384`*^9}, 
   3.6238490140581484`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3", ",", "2"}], "}"}]], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{3.6187546162708406`*^9, 3.6238490140581484`*^9, 
  3.6238491570479994`*^9, 3.624277827540242*^9, 3.6242807719362135`*^9, 
  3.6258991653133163`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3", ",", "2"}], "}"}]], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{3.6187546162708406`*^9, 3.6238490140581484`*^9, 
  3.6238491570479994`*^9, 3.624277827540242*^9, 3.6242807719362135`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"d", "[", "sf", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ")"}], ".", "resa"}], "]"}]], "Input",
 CellChangeTimes->{{3.596462848039283*^9, 3.5964628573992996`*^9}, {
   3.596462905369384*^9, 3.596462918832207*^9}, {3.5964629656010895`*^9, 
   3.5964630031659555`*^9}, 3.5964633152157035`*^9, {3.5964641983800545`*^9, 
   3.5964642202824936`*^9}, 3.5964657672904105`*^9, {3.6187545440739136`*^9, 
   3.6187545479583206`*^9}, {3.6187546327444696`*^9, 3.618754654662508*^9}, 
   3.6238490140581484`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "3"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
         RowBox[{"2", " ", "s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", "s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "2"}], " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "2", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"2", " ", "s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"2", " ", "s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"s2", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                    RowBox[{"s1", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
               "+", 
               RowBox[{"s1", " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "2", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", "s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"2", " ", "s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"-", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"s2", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                    RowBox[{"s1", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
            "+", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
            "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", "s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "3"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", "s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], "+", 
         RowBox[{"s1", " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", "s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "2"}], " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"s2", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                    RowBox[{"s1", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
               "+", 
               RowBox[{"s1", " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "2"}], " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", "s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "-", 
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "3", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
               RowBox[{"s1", " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "3", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"2", " ", "s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
            "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", "s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "3"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
         RowBox[{"s1", " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "2"}], " ", "s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
         RowBox[{"2", " ", "s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", "s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", "s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
            "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "-", 
               RowBox[{"2", " ", "s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "-", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "-", 
               RowBox[{"2", " ", "s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", "s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"2", " ", "s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"\[Xi]", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"s2", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                    RowBox[{"s1", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
              ")"}]}]}], ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"]}], "+", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "2"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
        ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}]}], 
  "}"}]], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{3.5964628601761045`*^9, 3.5964629208758106`*^9, 
  3.5964630043827577`*^9, 3.5964642259765034`*^9, 3.596465155488536*^9, 
  3.5964652184190464`*^9, 3.5964659045394516`*^9, 3.6132225529608793`*^9, 
  3.6187536418307295`*^9, 3.6187543484183702`*^9, 3.618756635460387*^9, 
  3.6238490140737486`*^9, 3.6238492111176944`*^9, 3.624277877319929*^9, 
  3.6242808193134966`*^9, 3.6242838893364887`*^9, 3.62589922391602*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "3"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
         RowBox[{"2", " ", "s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", "s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "2"}], " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "2", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"2", " ", "s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"2", " ", "s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"s2", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                    RowBox[{"s1", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
               "+", 
               RowBox[{"s1", " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "2", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", "s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"2", " ", "s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"-", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"s2", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                    RowBox[{"s1", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
            "+", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
            "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", "s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "3"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", "s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], "+", 
         RowBox[{"s1", " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", "s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "2"}], " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"s2", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                    RowBox[{"s1", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
               "+", 
               RowBox[{"s1", " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "2"}], " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", "s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "-", 
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "3", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
               RowBox[{"s1", " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "3", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"2", " ", "s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
            "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", "s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "3"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
         RowBox[{"s1", " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "2"}], " ", "s2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
         RowBox[{"2", " ", "s1", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"\[Xi]", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", "s2", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"1", ",", "0"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "-", 
         RowBox[{"2", " ", "s1", " ", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"2", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
            "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}]}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "-", 
               RowBox[{"2", " ", "s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "-", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "-", 
               RowBox[{"2", " ", "s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", "s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s2", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{"s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"2", " ", "s1", " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"\[Xi]", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
                    RowBox[{"u", ",", "v"}], "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"s2", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}], "+", 
                    RowBox[{"s1", " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
              ")"}]}]}], ")"}]}], "-", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"]}], "+", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"\[Xi]", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s2", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "-", 
            RowBox[{"s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"2", " ", "s1", " ", 
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"2", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "2"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "2"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"s2", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{"s1", " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"\[Xi]", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"s2", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], "+", 
               RowBox[{"s1", " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], ")"}]}]}], 
        ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"u", ",", "v"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}]}]}], ")"}]}]}], 
  "}"}]], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{3.5964628601761045`*^9, 3.5964629208758106`*^9, 
  3.5964630043827577`*^9, 3.5964642259765034`*^9, 3.596465155488536*^9, 
  3.5964652184190464`*^9, 3.5964659045394516`*^9, 3.6132225529608793`*^9, 
  3.6187536418307295`*^9, 3.6187543484183702`*^9, 3.618756635460387*^9, 
  3.6238490140737486`*^9, 3.6238492111176944`*^9, 3.624277877319929*^9, 
  3.6242808193134966`*^9, 3.6242838893364887`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"diff\[Xi]sf", "=", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.5964630273147984`*^9, 3.596463049482437*^9}, 
   3.6238490140737486`*^9, {3.624283864906846*^9, 3.6242838652188463`*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", " ", 
  RowBox[{"diff\[Xi]sf", " ", "-", "pdvsf"}], "]"}]], "Input",
 CellChangeTimes->{{3.596463140479797*^9, 3.5964631670622435`*^9}, {
   3.5964632076067147`*^9, 3.596463212895124*^9}, 3.6238490140737486`*^9, {
   3.624283873986062*^9, 3.6242838788532705`*^9}}],

Cell["\[FilledRectangle]", "Text",
 CellChangeTimes->{{3.5965283114394946`*^9, 3.5965283970773926`*^9}, {
  3.624279440973076*^9, 3.62427944351588*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DDimm", "[", "f_", "]"}], "[", "vw_", "]"}], "[", "x__", "]"}], ":=",
   " ", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "f", "]"}], "[", "x", "]"}], ".", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Simplify", "[", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"D", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"vw", "[", "x", "]"}], "[", 
           RowBox[{"[", "k", "]"}], "]"}], ",", 
          RowBox[{
           RowBox[{"{", "x", "}"}], "[", 
           RowBox[{"[", "j", "]"}], "]"}]}], "]"}], "+", 
        RowBox[{"Sum", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"\[CapitalGamma]rgT", "[", 
              RowBox[{"k", ",", "i", ",", "j"}], "]"}], "[", "f", "]"}], "[", 
            "x", "]"}], "*", 
           RowBox[{
            RowBox[{"vw", "[", "x", "]"}], "[", 
            RowBox[{"[", "i", "]"}], "]"}]}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", 
            RowBox[{"Length", "[", 
             RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", 
         RowBox[{"Length", "[", 
          RowBox[{"{", "x", "}"}], "]"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", 
         RowBox[{"Length", "[", 
          RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}], "]"}], 
    "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.624280924910082*^9, 3.6242809373433037`*^9}, {
   3.6242810276050625`*^9, 3.62428108878837*^9}, 3.6242812758014984`*^9, {
   3.6242813418832145`*^9, 3.6242813572648416`*^9}, {3.6242814124577384`*^9, 
   3.6242814491802025`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "DDimm"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.624284255422332*^9, 3.624284423185026*^9}, {
  3.624284507189603*^9, 3.624284581883534*^9}, {3.6242846431604414`*^9, 
  3.624284701379744*^9}, {3.6242847889294987`*^9, 3.6242848901288767`*^9}, {
  3.6242850307485237`*^9, 3.62428506862539*^9}, {3.6259053262975273`*^9, 
  3.6259053324751387`*^9}}],

Cell[BoxData[
 StyleBox["\<\"DD[[f][vw][x,y,...] is the absolute Differential of the vector \
field vw[x,y,...] \\non an immersion f[x,y,...]into the Euclidean n-space; \
here vw is a vector field on the \\nparameter manifold. and the images of \
DDim lie in the Euclidean n-space.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.6291867747900248`*^9},
 CellTags->"Info3629190374-5022241"]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{3.6242815731068206`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DDimm", "[", 
    RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
   RowBox[{"vv", "[", "\[Xi]", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6242814735630455`*^9, 3.624281483952664*^9}, {
  3.624281590500851*^9, 3.624281632667725*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"ddsph", "=", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.624281676347802*^9, 3.6242816864722195`*^9}}],

Cell[BoxData[
 RowBox[{"Dimensions", "[", "ddsph", "]"}]], "Input",
 CellChangeTimes->{{3.624281691261428*^9, 3.624281700637045*^9}}],

Cell["\<\
The result coincides with the vector field diff\[Xi] calculated by another \
method above:\
\>", "Text",
 CellChangeTimes->{{3.6242840929636464`*^9, 3.6242841567053585`*^9}}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"ddsph", ".", 
    RowBox[{"{", 
     RowBox[{"s1", ",", "s2"}], "}"}]}], " ", "\[Equal]", " ", "diff\[Xi]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.624281886308571*^9, 3.6242819463842764`*^9}, {
  3.6242819862891464`*^9, 3.6242819870379477`*^9}, {3.624282094210136*^9, 
  3.6242821195445805`*^9}, {3.6242822001031218`*^9, 3.6242822196187563`*^9}}],

Cell["The analogue calculation in case of a general surface:", "Text",
 CellChangeTimes->{{3.6242865842022524`*^9, 3.6242866075428934`*^9}}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DDimm", "[", "sf", "]"}], "[", 
    RowBox[{"vv", "[", "\[Xi]", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.624286240391248*^9, 3.6242862966761475`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"ddsf", " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.6242863394378223`*^9, 3.624286354710249*^9}}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"ddsf", ".", 
    RowBox[{"{", 
     RowBox[{"s1", ",", "s2"}], "}"}]}], "\[Equal]", " ", "diff\[Xi]sf"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.624286411463149*^9, 3.624286493690893*^9}}],

Cell["OK!", "Text",
 CellChangeTimes->{{3.6242866321909366`*^9, 3.624286635934943*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["4.2.3. Absolute Parallel Vector Fields", "Subsubsection",
 CellChangeTimes->{{3.595076945363204*^9, 3.5950769905720835`*^9}, 
   3.5953174793534184`*^9, {3.5953175263051033`*^9, 3.595317587064579*^9}, 
   3.5956008724322104`*^9, {3.5965213907316623`*^9, 3.5965214041788855`*^9}, {
   3.5972984401750546`*^9, 3.59729846072029*^9}, {3.5972984954771514`*^9, 
   3.5972985062723703`*^9}, {3.5973320886172123`*^9, 3.5973320905123205`*^9}, 
   3.5973952636966553`*^9, 3.6132229905260477`*^9, 3.6132797959577127`*^9, 
   3.6171013236210327`*^9, 3.6171717453084965`*^9, {3.6188032063086033`*^9, 
   3.6188032409562645`*^9}, 3.618817261348339*^9}],

Cell["\<\
Considering immersions we often will omit the word \
\[OpenCurlyDoubleQuote]tangent\[CloseCurlyDoubleQuote]; if not emphasized \
explicitly, e.g. the normal vector field e{3][sf][u,v], a vector field on an \
immersion is always meant to be a tangent vector field. Such a vector field \
\>", "Text",
 CellChangeTimes->{{3.5972985733524885`*^9, 3.5972986623350444`*^9}, {
   3.5972987038467174`*^9, 3.597298884011434*^9}, 3.5972989562083607`*^9, 
   3.6132230395101337`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "4"}], ";", " ", 
  RowBox[{"mf", " ", "=", " ", "2"}], ";", " ", 
  RowBox[{
   RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5956590461387978`*^9, 3.5956590587255173`*^9}, 
   3.595924528295635*^9, {3.597299146747095*^9, 3.5972992078212023`*^9}, {
   3.597300291508306*^9, 3.5973003191827545`*^9}, {3.5973004499577847`*^9, 
   3.5973004648714104`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}]}], "]"}], ",", 
   RowBox[{
    RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}]}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6259241442618017`*^9}]
}, Closed]],

Cell[TextData[{
 "is said to be ",
 StyleBox["absolute parallel",
  FontColor->RGBColor[0, 0, 1]],
 " if"
}], "Text",
 CellChangeTimes->{
  3.5972989670035796`*^9, {3.597299243826066*^9, 3.5972992583808913`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DDimm", "[", 
     RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
    RowBox[{"vv", "[", "\[Xi]", "]"}], "]"}], "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}], ",", 
    RowBox[{"x", "[", "3", "]"}]}], "]"}], "\[Equal]", " ", "0"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5972993146657906`*^9, 3.5972993494538517`*^9}, {
   3.597299402821545*^9, 3.597299430792394*^9}, 3.5972995294157677`*^9, 
   3.5973003459836016`*^9, 3.6132230887126207`*^9, {3.624339079502738*^9, 
   3.62433908661635*^9}}],

Cell["\<\
Here the 0 at the right side stands for the corresponding \
\[OpenCurlyDoubleQuote]null tensor\[CloseCurlyDoubleQuote], appearing as the \
null square matrix of order mf: Analogously the absolute parallel vector \
fields on a Riemannian manifold are defined:\
\>", "Text",
 CellChangeTimes->{{3.5973005157118998`*^9, 3.5973006704641714`*^9}, 
   3.5973830346646786`*^9, 3.6171684676959395`*^9, {3.624339772502755*^9, 
   3.6243398263072495`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DD", "[", "g", "]"}], "[", 
    RowBox[{"vv", "[", "\[Xi]", "]"}], "]"}], "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}], ",", 
    RowBox[{"x", "[", "3", "]"}]}], "]"}], "\[Equal]", " ", "0"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5972993146657906`*^9, 3.5972993494538517`*^9}, {
   3.597299402821545*^9, 3.597299430792394*^9}, 3.5972995294157677`*^9, 
   3.5973003459836016`*^9, 3.6132230887126207`*^9, {3.624339079502738*^9, 
   3.62433908661635*^9}, {3.624339889799361*^9, 3.6243399149310055`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6244466083670025`*^9, 3.6244466475230713`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Dimensions", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DDimm", "[", 
     RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
    RowBox[{"vv", "[", "\[Xi]", "]"}], "]"}], "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5973000987231674`*^9, 3.597300125648815*^9}, 
   3.597300366778438*^9, 3.597300478474634*^9, {3.6243394226409407`*^9, 
   3.624339423452142*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4", ",", "2"}], "}"}]], "Output",
 CellChangeTimes->{3.625924166774641*^9}]
}, Closed]],

Cell[TextData[{
 StyleBox["Example",
  FontColor->RGBColor[0, 0, 1]],
 ". For the ",
 StyleBox["Euclidean space",
  FontColor->RGBColor[0, 0, 1]],
 " the absolute differential coincides with the total differential. Therefore \
the absolute parallel vector fields are the vector fields with constant \
Cartesian coordinates."
}], "Text",
 CellChangeTimes->{{3.5973040529501977`*^9, 3.597304222288495*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DD", "[", "euclid", "]"}], "[", 
   RowBox[{"vv", "[", "\[Xi]", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.59730430471904*^9, 3.5973043671191497`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"\[Xi]", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"\[Xi]", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"\[Xi]", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"\[Xi]", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6259241810798664`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"{", 
     RowBox[{"u", ",", "v"}], "}"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6244463836798086`*^9, 3.6244463859574127`*^9}, {
   3.624446476546771*^9, 3.624446525858458*^9}, 3.6244466760711217`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"\[Xi]", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"\[Xi]", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"\[Xi]", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"\[Xi]", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6259241933902884`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Dimensions", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.624340005551564*^9, 3.6243400125091763`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "2"}], "}"}]], "Output",
 CellChangeTimes->{3.625924198054696*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"c", "[", "1", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", "a1"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"c", "[", "2", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", "a2"}], ";", 
  RowBox[{
   RowBox[{"vv", "[", "c", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5972997287685175`*^9, 3.597299731280122*^9}, {
  3.597299801963846*^9, 3.5972999486041036`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"a1", ",", "a2"}], "}"}]], "Output",
 CellChangeTimes->{3.625924205012308*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DD", "[", "euclid", "]"}], "[", 
   RowBox[{"vv", "[", "c", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5972999616457267`*^9, 3.5972999994601927`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6259242106283183`*^9}]
}, Closed]],

Cell["\<\
\[FilledRectangle]
Now we define a general pseudo-Riemannian metric:\
\>", "Text",
 CellChangeTimes->{{3.5973044572093077`*^9, 3.597304457224908*^9}, {
   3.597304491498168*^9, 3.59730455511508*^9}, 3.5973046113063784`*^9, {
   3.5973061652563076`*^9, 3.597306219747204*^9}, {3.597306396932315*^9, 
   3.597306402064724*^9}, 3.617441250860256*^9, 3.6244561042991996`*^9, {
   3.6244561896937494`*^9, 3.6244562186006002`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"grg", "[", 
   RowBox[{"h_", ",", "k_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"Normal", "[", 
     RowBox[{"Symmetrize", "[", 
      RowBox[{
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"gen", "[", 
           RowBox[{"i", ",", "j"}], "]"}], "[", "##", "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", 
           RowBox[{"Length", "[", 
            RowBox[{"{", "##", "}"}], "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"j", ",", 
           RowBox[{"Length", "[", 
            RowBox[{"{", "##", "}"}], "]"}]}], "}"}]}], "]"}], ",", 
       RowBox[{"Symmetric", "[", 
        RowBox[{"{", 
         RowBox[{"1", ",", "2"}], "}"}], "]"}]}], "]"}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"h", ",", "k"}], "]"}], "]"}], "&"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6244515694180346`*^9, 3.624451570775237*^9}, {
   3.624451612068509*^9, 3.6244516149701147`*^9}, {3.6244516620197973`*^9, 
   3.6244517282263136`*^9}, {3.6244523369705825`*^9, 3.624452351541008*^9}, {
   3.624452857184696*^9, 3.6244528590254993`*^9}, {3.6244529247952147`*^9, 
   3.6244529865713234`*^9}, {3.6244530195341816`*^9, 3.624453083712694*^9}, {
   3.6244531231183634`*^9, 3.6244531552700195`*^9}, {3.624454318844864*^9, 
   3.6244543516829214`*^9}, {3.624454392383393*^9, 3.6244544137242303`*^9}, {
   3.6244547280959826`*^9, 3.6244548118681297`*^9}, {3.6244551861439867`*^9, 
   3.6244552017596145`*^9}, {3.6244552418516846`*^9, 3.624455249542498*^9}, 
   3.6260057543665466`*^9, 3.6260106813705254`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"grg", "::", "usage"}], " ", "=", " ", 
   "\"\<grg[i,j][x,y,...] is a general, unspecified pseudo-Riemannian metric \
with respect to local coordinates {x,y,...}.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6245190611007977`*^9, 3.6245192090046577`*^9}, 
   3.6245192630275526`*^9, {3.6260057896070085`*^9, 3.626005797953023*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"grg", "[", 
   RowBox[{"1", ",", "3"}], "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.6244548894002657`*^9, 3.624454901459087*^9}, {
   3.624455275578944*^9, 3.624455279510151*^9}, 3.6244562481470523`*^9, 
   3.6260058374210925`*^9}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"gen", "[", 
      RowBox[{"1", ",", "3"}], "]"}], "[", 
     RowBox[{"u", ",", "v", ",", "w"}], "]"}], "+", 
    RowBox[{
     RowBox[{"gen", "[", 
      RowBox[{"3", ",", "1"}], "]"}], "[", 
     RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.6259243634445877`*^9, 3.62601068952934*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[CapitalGamma]", "[", 
      RowBox[{"1", ",", "2", ",", "1"}], "]"}], "[", "grg", "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}]}], "]"}], "-", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"\[CapitalGamma]", "[", 
      RowBox[{"1", ",", "1", ",", "2"}], "]"}], "[", "grg", "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.624453316667903*^9, 3.624453341019546*^9}, {
   3.6244544473266892`*^9, 3.624454513892006*^9}, 3.624454598475355*^9, {
   3.6244548284041586`*^9, 3.624454852599801*^9}, {3.626010744332236*^9, 
   3.6260107453462377`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6259244165862813`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalGamma]", "[", 
     RowBox[{"1", ",", "2", ",", "1"}], "]"}], "[", "grg", "]"}], "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{3.624455445120042*^9, 3.6260107469374404`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "2"}], " ", 
     RowBox[{
      RowBox[{"gen", "[", 
       RowBox[{"2", ",", "2"}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}]}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"gen", "[", 
        RowBox[{"1", ",", "1"}], "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}]}], "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"gen", "[", 
         RowBox[{"1", ",", "2"}], "]"}], "[", 
        RowBox[{
         RowBox[{"x", "[", "1", "]"}], ",", 
         RowBox[{"x", "[", "2", "]"}]}], "]"}], "+", 
       RowBox[{
        RowBox[{"gen", "[", 
         RowBox[{"2", ",", "1"}], "]"}], "[", 
        RowBox[{
         RowBox[{"x", "[", "1", "]"}], ",", 
         RowBox[{"x", "[", "2", "]"}]}], "]"}]}], ")"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"gen", "[", 
        RowBox[{"2", ",", "2"}], "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}]}], "]"}]}]}], ")"}], "/", 
  RowBox[{"(", 
   RowBox[{
    SuperscriptBox[
     RowBox[{
      RowBox[{"gen", "[", 
       RowBox[{"1", ",", "2"}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}]}], "]"}], "2"], "+", 
    RowBox[{"2", " ", 
     RowBox[{
      RowBox[{"gen", "[", 
       RowBox[{"1", ",", "2"}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}]}], "]"}], " ", 
     RowBox[{
      RowBox[{"gen", "[", 
       RowBox[{"2", ",", "1"}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}]}], "]"}]}], "+", 
    SuperscriptBox[
     RowBox[{
      RowBox[{"gen", "[", 
       RowBox[{"2", ",", "1"}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}]}], "]"}], "2"], "-", 
    RowBox[{"4", " ", 
     RowBox[{
      RowBox[{"gen", "[", 
       RowBox[{"1", ",", "1"}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}]}], "]"}], " ", 
     RowBox[{
      RowBox[{"gen", "[", 
       RowBox[{"2", ",", "2"}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}]}], "]"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.625924436339916*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"grg", "[", 
      RowBox[{"i", ",", "j"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.6244536869813538`*^9, 3.6244536974957724`*^9}, {
   3.6244537298034286`*^9, 3.624453735107438*^9}, {3.6244538621228614`*^9, 
   3.624453865071266*^9}, {3.6244563513724337`*^9, 3.6244563585172462`*^9}, {
   3.6244564158005466`*^9, 3.6244564548318152`*^9}, 3.626005839277496*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       RowBox[{"gen", "[", 
        RowBox[{"1", ",", "1"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], 
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"gen", "[", 
           RowBox[{"1", ",", "2"}], "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "+", 
         RowBox[{
          RowBox[{"gen", "[", 
           RowBox[{"2", ",", "1"}], "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], ")"}]}]},
     {
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"gen", "[", 
           RowBox[{"1", ",", "2"}], "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], "+", 
         RowBox[{
          RowBox[{"gen", "[", 
           RowBox[{"2", ",", "1"}], "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}], ")"}]}], 
      RowBox[{
       RowBox[{"gen", "[", 
        RowBox[{"2", ",", "2"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.6259244681991725`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Det", "[", "%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6244538793608913`*^9, 3.624453931511783*^9}, {
  3.624453984551876*^9, 3.624453987656282*^9}, {3.6244564941906843`*^9, 
  3.624456510445913*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "4"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", 
     SuperscriptBox[
      RowBox[{
       RowBox[{"gen", "[", 
        RowBox[{"1", ",", "2"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
    RowBox[{"2", " ", 
     RowBox[{
      RowBox[{"gen", "[", 
       RowBox[{"1", ",", "2"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{
      RowBox[{"gen", "[", 
       RowBox[{"2", ",", "1"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "-", 
    SuperscriptBox[
     RowBox[{
      RowBox[{"gen", "[", 
       RowBox[{"2", ",", "1"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
    RowBox[{"4", " ", 
     RowBox[{
      RowBox[{"gen", "[", 
       RowBox[{"1", ",", "1"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{
      RowBox[{"gen", "[", 
       RowBox[{"2", ",", "2"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.625924480554394*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Inverse", "[", "%%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6244566684585905`*^9, 3.6244566901894283`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"gen", "[", 
        RowBox[{"2", ",", "2"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "/", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          FractionBox["1", "4"]}], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"gen", "[", 
              RowBox[{"1", ",", "2"}], "]"}], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "+", 
            RowBox[{
             RowBox[{"gen", "[", 
              RowBox[{"2", ",", "1"}], "]"}], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], ")"}], "2"]}], "+", 
        RowBox[{
         RowBox[{
          RowBox[{"gen", "[", 
           RowBox[{"1", ",", "1"}], "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          RowBox[{"gen", "[", 
           RowBox[{"2", ",", "2"}], "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], ",", 
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"gen", "[", 
            RowBox[{"1", ",", "2"}], "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "+", 
          RowBox[{
           RowBox[{"gen", "[", 
            RowBox[{"2", ",", "1"}], "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              FractionBox["1", "4"]}], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 RowBox[{"gen", "[", 
                  RowBox[{"1", ",", "2"}], "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "+", 
                RowBox[{
                 RowBox[{"gen", "[", 
                  RowBox[{"2", ",", "1"}], "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], ")"}], "2"]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"gen", "[", 
               RowBox[{"1", ",", "1"}], "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              RowBox[{"gen", "[", 
               RowBox[{"2", ",", "2"}], "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], ")"}]}], ")"}]}]}],
     "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"gen", "[", 
            RowBox[{"1", ",", "2"}], "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}], "+", 
          RowBox[{
           RowBox[{"gen", "[", 
            RowBox[{"2", ",", "1"}], "]"}], "[", 
           RowBox[{"u", ",", "v"}], "]"}]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              FractionBox["1", "4"]}], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 RowBox[{"gen", "[", 
                  RowBox[{"1", ",", "2"}], "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}], "+", 
                RowBox[{
                 RowBox[{"gen", "[", 
                  RowBox[{"2", ",", "1"}], "]"}], "[", 
                 RowBox[{"u", ",", "v"}], "]"}]}], ")"}], "2"]}], "+", 
            RowBox[{
             RowBox[{
              RowBox[{"gen", "[", 
               RowBox[{"1", ",", "1"}], "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}], " ", 
             RowBox[{
              RowBox[{"gen", "[", 
               RowBox[{"2", ",", "2"}], "]"}], "[", 
              RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}], ")"}]}], ")"}]}], 
     ",", 
     RowBox[{
      RowBox[{
       RowBox[{"gen", "[", 
        RowBox[{"1", ",", "1"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "/", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          FractionBox["1", "4"]}], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"gen", "[", 
              RowBox[{"1", ",", "2"}], "]"}], "[", 
             RowBox[{"u", ",", "v"}], "]"}], "+", 
            RowBox[{
             RowBox[{"gen", "[", 
              RowBox[{"2", ",", "1"}], "]"}], "[", 
             RowBox[{"u", ",", "v"}], "]"}]}], ")"}], "2"]}], "+", 
        RowBox[{
         RowBox[{
          RowBox[{"gen", "[", 
           RowBox[{"1", ",", "1"}], "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}], " ", 
         RowBox[{
          RowBox[{"gen", "[", 
           RowBox[{"2", ",", "2"}], "]"}], "[", 
          RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}]}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.6259244898852105`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MatrixRank", "[", "%%%", "]"}]], "Input",
 CellChangeTimes->{{3.624456748393131*^9, 3.624456760311552*^9}}],

Cell[BoxData["2"], "Output",
 CellChangeTimes->{3.625924496267622*^9}]
}, Closed]],

Cell["\<\
Now we consider this general case and ask whether there exist absolute \
parallel vector fields. The system of linear partial differential equations \
of first order to find them is:\
\>", "Text",
 CellChangeTimes->{
  3.6244561411776648`*^9, {3.6244578001781783`*^9, 3.6244578152010045`*^9}}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"DD", "[", "grg", "]"}], "[", 
        RowBox[{"vv", "[", "\[Xi]", "]"}], "]"}], "[", 
       RowBox[{
        RowBox[{"x", "[", "1", "]"}], ",", 
        RowBox[{"x", "[", "2", "]"}]}], "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"i", ",", "j"}], "]"}], "]"}], "\[Equal]", " ", "0"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "mf"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "mf"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5973046678096776`*^9, 3.5973048127651324`*^9}, 
   3.597305117074867*^9, {3.5973055552952366`*^9, 3.597305559070443*^9}, 
   3.614861862498745*^9, 3.6148635021396246`*^9, 3.6188037315971227`*^9, 
   3.6188037658630824`*^9, {3.624355033463996*^9, 3.624355045413617*^9}, 
   3.624355133787772*^9, {3.624456851836912*^9, 3.6244568734897504`*^9}, {
   3.624456941880271*^9, 3.6244569555926948`*^9}, 3.624457008055587*^9, 
   3.626010748996644*^9}],

Cell["Remembering", "Text",
 CellChangeTimes->{{3.597306271258494*^9, 3.5973063053289537`*^9}, {
  3.6244579809980955`*^9, 3.6244579846017017`*^9}}],

Cell[CellGroupData[{

Cell[BoxData["mf"], "Input",
 CellChangeTimes->{{3.624457115180975*^9, 3.6244571177705793`*^9}}],

Cell[BoxData["2"], "Output",
 CellChangeTimes->{3.625924534429289*^9}]
}, Closed]],

Cell["\<\
and using the definition of the absolute differential we may rewrite this \
system as\
\>", "Text",
 CellChangeTimes->{{3.597306271258494*^9, 3.5973063053289537`*^9}, {
  3.6244579954437213`*^9, 3.624457998626127*^9}}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
         RowBox[{
          RowBox[{"x", "[", "1", "]"}], ",", 
          RowBox[{"x", "[", "2", "]"}]}], "]"}], "[", 
        RowBox[{"[", "k", "]"}], "]"}], ",", 
       RowBox[{"x", "[", "j", "]"}]}], "]"}], "\[Equal]", " ", 
     RowBox[{"-", 
      RowBox[{"Sum", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[CapitalGamma]", "[", 
            RowBox[{"k", ",", "i", ",", "j"}], "]"}], "[", "grg", "]"}], "[", 
          RowBox[{
           RowBox[{"x", "[", "1", "]"}], ",", 
           RowBox[{"x", "[", "2", "]"}]}], "]"}], "*", 
         RowBox[{
          RowBox[{
           RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
           RowBox[{
            RowBox[{"x", "[", "1", "]"}], ",", 
            RowBox[{"x", "[", "2", "]"}]}], "]"}], "[", 
          RowBox[{"[", "i", "]"}], "]"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "mf"}], "}"}]}], "]"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "mf"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "mf"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5973052297070646`*^9, 3.597305240642684*^9}, {
   3.5973052727631407`*^9, 3.5973053555524855`*^9}, {3.5973054039593706`*^9, 
   3.597305412757786*^9}, 3.5973054583410664`*^9, {3.597305510507558*^9, 
   3.5973055337047987`*^9}, {3.59730556880486*^9, 3.597305581394082*^9}, 
   3.597305692388277*^9, 3.6148618625143447`*^9, 3.614863502155225*^9, 
   3.6243547514291005`*^9, {3.6243551505578012`*^9, 3.624355151930604*^9}, 
   3.624355192272275*^9, {3.6244570688644934`*^9, 3.624457077054508*^9}, 
   3.6244576396850967`*^9, 3.6260107522414494`*^9}],

Cell[TextData[{
 "or, in traditional form, as a system of differential equations for the \
coordinates ",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox["\[Xi]", "k"], "[", "x", "]"}]],
  Evaluatable->False,
  CellChangeTimes->{{3.5973058712737913`*^9, 3.5973059305226955`*^9}, {
   3.597305962658752*^9, 3.5973060124072394`*^9}, {3.5973060436228943`*^9, 
   3.5973060926225805`*^9}}],
 " of the vector field "
}], "Text",
 CellChangeTimes->{{3.5973063359986076`*^9, 3.597306379600684*^9}, {
  3.5973064171031504`*^9, 3.597306458396423*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["\[PartialD]", "j"], 
    RowBox[{
     SuperscriptBox["\[Xi]", "k"], "[", "x", "]"}]}], " ", "\[Equal]", " ", 
   RowBox[{
    RowBox[{"-", " ", 
     SubsuperscriptBox["\[CapitalSigma]", 
      RowBox[{"i", "=", "1"}], "mf"]}], 
    RowBox[{
     RowBox[{
      RowBox[{"\[CapitalGamma]", "[", 
       RowBox[{"k", ",", "i", ",", "j"}], "]"}], "[", "grg", "]"}], "[", "x", 
     "]"}], " ", 
    RowBox[{
     SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}]}]}], ",", "    ", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"k", ",", "j", ",", "1", ",", "..", ",", "mf"}], "}"}], ".", 
   "       ", 
   RowBox[{"(", "1", ")"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5973058712737913`*^9, 3.5973059305226955`*^9}, {
   3.597305962658752*^9, 3.5973060124072394`*^9}, {3.5973060436228943`*^9, 
   3.5973060926225805`*^9}, {3.5973064685364404`*^9, 3.597306522949336*^9}, {
   3.5973134928002768`*^9, 3.597313703088646*^9}, {3.6243553099588814`*^9, 
   3.624355313094487*^9}, 3.6244574499575634`*^9, 3.6244577130520253`*^9, {
   3.6244580529610224`*^9, 3.624458069403451*^9}, 3.626010754051053*^9}],

Cell[TextData[{
 "Assume that we found a solution (",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SuperscriptBox["\[Xi]", "k"], "[", "x", "]"}], ",", " ", 
   RowBox[{"k", " ", "=", " ", "1"}], ",", "...", ",", "m"}]],
  Evaluatable->False,
  CellChangeTimes->{{3.5973058712737913`*^9, 3.5973059305226955`*^9}, {
   3.597305962658752*^9, 3.5973060124072394`*^9}, {3.5973060436228943`*^9, 
   3.5973060926225805`*^9}, {3.5973064685364404`*^9, 3.597306522949336*^9}}],
 "f) of this system. Then these functions satisfy also the following \
derivations of the equations:"
}], "Text",
 CellChangeTimes->{{3.5973065921510572`*^9, 3.5973067715513725`*^9}, 
   3.5973137235558815`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[PartialD]", "h"], 
   RowBox[{
    SubscriptBox["\[PartialD]", "j"], 
    RowBox[{
     SuperscriptBox["\[Xi]", "k"], "[", "x", "]"}]}]}], " ", "\[Equal]", " ", 
  RowBox[{"-", " ", 
   RowBox[{
    SubscriptBox["\[PartialD]", "h"], 
    RowBox[{"(", 
     RowBox[{
      SubsuperscriptBox["\[CapitalSigma]", 
       RowBox[{"i", "=", "1"}], "mf"], 
      RowBox[{
       RowBox[{
        RowBox[{"\[CapitalGamma]", "[", 
         RowBox[{"k", ",", "i", ",", "j"}], "]"}], "[", "grg", "]"}], "[", 
       "x", "]"}], " ", 
      RowBox[{
       SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}]}], ")"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5973058712737913`*^9, 3.5973059305226955`*^9}, {
   3.597305962658752*^9, 3.5973060124072394`*^9}, {3.5973060436228943`*^9, 
   3.5973060926225805`*^9}, {3.5973064685364404`*^9, 3.597306522949336*^9}, {
   3.597313407951728*^9, 3.5973134700554366`*^9}, {3.597313752103932*^9, 
   3.5973138567021155`*^9}, {3.5973138906945753`*^9, 3.5973138933309803`*^9}, 
   3.62445771568843*^9, 3.626010757545459*^9},
 FontSize->16],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"\[Equal]", "  ", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      SubsuperscriptBox["\[CapitalSigma]", 
       RowBox[{"i", "=", "1"}], "mf"]}], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[PartialD]", "h"], 
       RowBox[{
        RowBox[{
         RowBox[{"\[CapitalGamma]", "[", 
          RowBox[{"k", ",", "i", ",", "j"}], "]"}], "[", "grg", "]"}], "[", 
        "x", "]"}]}], ")"}], 
     RowBox[{
      SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}]}], "-", " ", 
    RowBox[{
     SubsuperscriptBox["\[CapitalSigma]", 
      RowBox[{"i", "=", "1"}], "mf"], 
     RowBox[{
      RowBox[{
       RowBox[{"\[CapitalGamma]", "[", 
        RowBox[{"k", ",", "i", ",", "j"}], "]"}], "[", "grg", "]"}], "[", "x",
       "]"}], " ", 
     RowBox[{
      SubscriptBox["\[PartialD]", "h"], 
      RowBox[{
       SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}]}]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.597313955278689*^9, 3.5973139664483085`*^9}, {
   3.5973140016887703`*^9, 3.597314030439621*^9}, {3.5973140614368753`*^9, 
   3.597314082777713*^9}, 3.5973141235249844`*^9, 3.597314177547879*^9, {
   3.5973142101831365`*^9, 3.597314235361581*^9}, {3.597315982396927*^9, 
   3.5973159873392096`*^9}, {3.6243553651517787`*^9, 
   3.6243553788330026`*^9}, {3.6244574555423727`*^9, 
   3.6244574570555754`*^9}, {3.624457717810034*^9, 3.624457722209241*^9}, {
   3.6260107592926617`*^9, 3.6260107606186647`*^9}},
 FontSize->16],

Cell["\<\
for  j ,k ,h= 1,..,mf. Inserting equation (1) with renamed indices  \
\>", "Text",
 CellChangeTimes->{{3.5973139027065964`*^9, 3.597313937791058*^9}, {
   3.5973141374558086`*^9, 3.597314150653432*^9}, 3.5973145527441382`*^9, {
   3.597314592071807*^9, 3.597314653442315*^9}, 3.597314768898118*^9, 
   3.597315097138294*^9, {3.617168754455643*^9, 3.61716875815285*^9}, 
   3.6171688253109674`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[PartialD]", "h"], 
   RowBox[{
    SuperscriptBox["\[Xi]", "l"], "[", "x", "]"}]}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{"-", 
    StyleBox[
     SubsuperscriptBox["\[CapitalSigma]", 
      RowBox[{"i", "=", "1"}], "mf"],
     FontSize->14]}], 
   RowBox[{
    RowBox[{
     RowBox[{"\[CapitalGamma]", "[", 
      RowBox[{"l", ",", "i", ",", "h"}], "]"}], "[", "grg", "]"}], "[", "x", 
    "]"}], 
   RowBox[{
    SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.597314922839188*^9, 3.597314928517598*^9}, 
   3.5973149670652657`*^9, {3.5973150064397354`*^9, 3.5973150128513465`*^9}, {
   3.597315389576408*^9, 3.5973154186548595`*^9}, 3.5973156631852884`*^9, {
   3.624355399549839*^9, 3.6243554015622425`*^9}, 3.624457465822791*^9, 
   3.6244577240812445`*^9, 3.6260107708366823`*^9},
 FontSize->16],

Cell["into the second term we obtain for the this term", "Text",
 CellChangeTimes->{
  3.597315106404711*^9, {3.6171688319097795`*^9, 3.6171688324869804`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", " ", 
    SubsuperscriptBox["\[CapitalSigma]", 
     RowBox[{"l", "=", "1"}], "mf"]}], 
   RowBox[{
    RowBox[{
     RowBox[{"\[CapitalGamma]", "[", 
      RowBox[{"k", ",", "l", ",", "j"}], "]"}], "[", "grg", "]"}], "[", "x", 
    "]"}], " ", 
   RowBox[{
    SubscriptBox["\[PartialD]", "h"], 
    RowBox[{
     SuperscriptBox["\[Xi]", "l"], "[", "x", "]"}]}]}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{"+", 
    SubsuperscriptBox["\[CapitalSigma]", 
     RowBox[{"l", "=", "1"}], "mf"]}], 
   SubsuperscriptBox["\[CapitalSigma]", 
    RowBox[{"i", "=", "1"}], "mf"], 
   RowBox[{
    RowBox[{
     RowBox[{"\[CapitalGamma]", "[", 
      RowBox[{"k", ",", "l", ",", "j"}], "]"}], "[", "grg", "]"}], "[", "x", 
    "]"}], 
   RowBox[{
    RowBox[{
     RowBox[{"\[CapitalGamma]", "[", 
      RowBox[{"l", ",", "i", ",", "h"}], "]"}], "[", "grg", "]"}], "[", "x", 
    "]"}], 
   RowBox[{
    SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5973146918963823`*^9, 3.597314697137992*^9}, {
   3.5973147316608524`*^9, 3.597314749491684*^9}, 3.597314788897353*^9, {
   3.5973148208618093`*^9, 3.597314844620651*^9}, 3.5973150316181793`*^9, {
   3.5973154540045214`*^9, 3.5973154840501738`*^9}, {3.597315550833891*^9, 
   3.597315574561533*^9}, {3.5973156251212215`*^9, 3.597315676570112*^9}, {
   3.6243554243694825`*^9, 3.624355459876145*^9}, {3.6244574673671937`*^9, 
   3.6244574700191984`*^9}, {3.6244577257816477`*^9, 
   3.6244577292916536`*^9}, {3.626010772287485*^9, 3.6260107749862895`*^9}}],

Cell[TextData[{
 "Adding the results for the two terms and collecting the terms at ",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}]],
  CellChangeTimes->{{3.5973146918963823`*^9, 3.597314697137992*^9}, {
    3.5973147316608524`*^9, 3.597314749491684*^9}, 3.597314788897353*^9, {
    3.5973148208618093`*^9, 3.597314844620651*^9}, 3.5973150316181793`*^9, {
    3.5973154540045214`*^9, 3.5973154840501738`*^9}, {3.597315550833891*^9, 
    3.597315574561533*^9}, {3.5973156251212215`*^9, 3.597315676570112*^9}}],
 " yields"
}], "Text",
 CellChangeTimes->{{3.597315754383049*^9, 3.5973157695774755`*^9}, {
  3.597315915735114*^9, 3.597315949999074*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[PartialD]", "h"], 
   RowBox[{
    SubscriptBox["\[PartialD]", "j"], 
    RowBox[{
     SuperscriptBox["\[Xi]", "k"], "[", "x", "]"}]}]}], " ", "\[Equal]", 
  RowBox[{
   RowBox[{"-", " ", 
    SubsuperscriptBox["\[CapitalSigma]", 
     RowBox[{"i", "=", "1"}], "mf"]}], 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", "h"], 
      RowBox[{
       RowBox[{
        RowBox[{"\[CapitalGamma]", "[", 
         RowBox[{"k", ",", "i", ",", "j"}], "]"}], "[", "grg", "]"}], "[", 
       "x", "]"}]}], "-", 
     RowBox[{
      SubsuperscriptBox["\[CapitalSigma]", 
       RowBox[{"l", "=", "1"}], "mf"], 
      RowBox[{
       RowBox[{
        RowBox[{"\[CapitalGamma]", "[", 
         RowBox[{"k", ",", "l", ",", "j"}], "]"}], "[", "grg", "]"}], "[", 
       "x", "]"}], 
      RowBox[{
       RowBox[{
        RowBox[{"\[CapitalGamma]", "[", 
         RowBox[{"l", ",", "i", ",", "h"}], "]"}], "[", "grg", "]"}], "[", 
       "x", "]"}]}]}], ")"}], 
   RowBox[{
    SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5973160039211583`*^9, 3.5973160378971014`*^9}, {
  3.5973160743631873`*^9, 3.5973160847617817`*^9}, {3.597316129803358*^9, 
  3.5973162206325536`*^9}, {3.5973162523453674`*^9, 3.5973163002021046`*^9}, {
  3.5973163303778305`*^9, 3.597316379267627*^9}, {3.624355489484997*^9, 
  3.624355507643429*^9}, {3.624457473201604*^9, 3.624457475775608*^9}, {
  3.6244577309452567`*^9, 3.6244577351572638`*^9}, {3.6260107762030916`*^9, 
  3.6260107782622957`*^9}}],

Cell[TextData[{
 "Permuting h and j and subtracting from the last equation the left side \
becomes zero, and we obtain a system of linear equations for the coordinates ",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}]],
  CellChangeTimes->{{3.5973160039211583`*^9, 3.5973160378971014`*^9}, {
   3.5973160743631873`*^9, 3.5973160847617817`*^9}, {3.597316129803358*^9, 
   3.5973162206325536`*^9}, {3.5973162523453674`*^9, 
   3.5973163002021046`*^9}, {3.5973163303778305`*^9, 3.597316379267627*^9}}],
 ":"
}], "Text",
 CellChangeTimes->{{3.5973174735432158`*^9, 3.5973175753910413`*^9}, {
   3.5973176149943066`*^9, 3.597317622350727*^9}, 3.5973279986752806`*^9, 
   3.61744129428474*^9, {3.618804143009654*^9, 3.618804146752868*^9}, 
   3.6188232318984947`*^9}],

Cell[BoxData[
 RowBox[{"0", " ", "=", 
  RowBox[{
   UnderoverscriptBox["\[Sum]", 
    RowBox[{"i", "=", "1"}], "mf"], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[PartialD]", "h"], 
       RowBox[{
        RowBox[{
         RowBox[{"\[CapitalGamma]", "[", 
          RowBox[{"k", ",", "i", ",", "j"}], "]"}], "[", "grg", "]"}], "[", 
        "x", "]"}]}], "-", 
      RowBox[{
       SubscriptBox["\[PartialD]", "j"], 
       RowBox[{
        RowBox[{
         RowBox[{"\[CapitalGamma]", "[", 
          RowBox[{"k", ",", "i", ",", "h"}], "]"}], "[", "grg", "]"}], "[", 
        "x", "]"}]}], "-", " ", 
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"l", "=", "1"}], "mf"], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalGamma]", "[", 
             RowBox[{"k", ",", "l", ",", "j"}], "]"}], "[", "grg", "]"}], "[",
            "x", "]"}], 
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalGamma]", "[", 
             RowBox[{"l", ",", "i", ",", "h"}], "]"}], "[", "grg", "]"}], "[",
            "x", "]"}]}], "-", " ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalGamma]", "[", 
             RowBox[{"k", ",", "l", ",", "h"}], "]"}], "[", "grg", "]"}], "[",
            "x", "]"}], 
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalGamma]", "[", 
             RowBox[{"l", ",", "i", ",", "j"}], "]"}], "[", "grg", "]"}], "[",
            "x", "]"}]}]}], ")"}]}]}], ")"}], "*", 
    RowBox[{
     SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5973280116780243`*^9, 3.5973280157692585`*^9}, {
   3.597328075913699*^9, 3.5973280771907716`*^9}, {3.597328179819642*^9, 
   3.5973283575508075`*^9}, 3.597328398054124*^9, {3.6243555266598625`*^9, 
   3.6243555695755377`*^9}, {3.624457481126418*^9, 3.624457494230441*^9}, {
   3.624457739135271*^9, 3.6244577508508916`*^9}, {3.62599134814475*^9, 
   3.625991350297554*^9}, {3.625991389609623*^9, 3.6259914949098077`*^9}, {
   3.6259915253142614`*^9, 3.625991539354286*^9}, 3.625991618711625*^9, {
   3.6259916918601537`*^9, 3.6259917018441715`*^9}, {3.625991864521257*^9, 
   3.6259918794816833`*^9}, 3.6259922141334715`*^9, 3.625992261089554*^9, {
   3.625992315455649*^9, 3.6259923329744797`*^9}, {3.625992396872192*^9, 
   3.6259924242658405`*^9}, 3.6260011022667756`*^9, {3.6260011462432528`*^9, 
   3.626001173870901*^9}, {3.626001322273962*^9, 3.6260013379519897`*^9}, {
   3.6260013718040495`*^9, 3.6260014063581095`*^9}, {3.6260014403193693`*^9, 
   3.626001453704193*^9}, {3.6260107796350975`*^9, 3.6260107865303097`*^9}}],

Cell[TextData[{
 "The coefficients of this linear system are the coordinates of the",
 StyleBox[" Riemannian curvature tensor",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox[", ",
  FontColor->GrayLevel[0]],
 "defined for a pseudo-Riemannian space with metric tensor g[i,j][x,y,...]"
}], "Text",
 CellChangeTimes->{{3.5973884413774414`*^9, 3.597388472552224*^9}, {
   3.5973885165367403`*^9, 3.5973885696807795`*^9}, 3.5973913039921737`*^9, {
   3.624355928769169*^9, 3.6243559380511847`*^9}, {3.6244583522007475`*^9, 
   3.62445836482117*^9}, {3.6244585643767204`*^9, 3.6244586082907977`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"R", "[", 
     RowBox[{"k_", ",", "i_", ",", "h_", ",", "j_"}], "]"}], "[", "g_", "]"}],
    "[", "x__", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"\[CapitalGamma]", "[", 
        RowBox[{"k", ",", "i", ",", "j"}], "]"}], "[", "g", "]"}], "[", "x", 
      "]"}], ",", 
     RowBox[{
      RowBox[{"{", "x", "}"}], "[", 
      RowBox[{"[", "h", "]"}], "]"}]}], "]"}], " ", "-", "  ", 
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"\[CapitalGamma]", "[", 
        RowBox[{"k", ",", "i", ",", "h"}], "]"}], "[", "g", "]"}], "[", "x", 
      "]"}], ",", 
     RowBox[{
      RowBox[{"{", "x", "}"}], "[", 
      RowBox[{"[", "j", "]"}], "]"}]}], "]"}], " ", "-", " ", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[CapitalGamma]", "[", 
           RowBox[{"k", ",", "l", ",", "j"}], "]"}], "[", "g", "]"}], "[", 
         "x", "]"}], "*", 
        RowBox[{
         RowBox[{
          RowBox[{"\[CapitalGamma]", "[", 
           RowBox[{"l", ",", "i", ",", "h"}], "]"}], "[", "g", "]"}], "[", 
         "x", "]"}]}], "-", " ", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[CapitalGamma]", "[", 
           RowBox[{"k", ",", "l", ",", "h"}], "]"}], "[", "g", "]"}], "[", 
         "x", "]"}], "*", 
        RowBox[{
         RowBox[{
          RowBox[{"\[CapitalGamma]", "[", 
           RowBox[{"l", ",", "i", ",", "j"}], "]"}], "[", "g", "]"}], "[", 
         "x", "]"}]}]}], ")"}], ",", 
     RowBox[{"{", 
      RowBox[{"l", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5973286677245483`*^9, 3.5973289520468106`*^9}, {
   3.597328985941749*^9, 3.5973290042307954`*^9}, 3.5973290495683885`*^9, {
   3.597329079818119*^9, 3.597329268394905*^9}, {3.624355833702602*^9, 
   3.624355897210313*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "R"}]], "Input",
 CellChangeTimes->{{3.6243685480756545`*^9, 3.624368611786166*^9}, {
  3.6243687165583916`*^9, 3.6243687976161337`*^9}, {3.624368838581806*^9, 
  3.6243689280011625`*^9}, {3.625925143972164*^9, 3.6259251490421734`*^9}}],

Cell[BoxData[
 StyleBox["\<\"R[k,h,i,j][g][x,y,...] is the Riemann curvature tensor of the \
pseudo-Riemannian \\nspace with metric tensor g[i,j][x,y,...].\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.6260016429793253`*^9},
 CellTags->"Info3626005242-1516062"]
}, Closed]],

Cell[TextData[{
 StyleBox["Remark",
  FontColor->RGBColor[0, 0, 1]],
 ". This well known expression coincides with the formula contained in \
Proposition 7.6, p.145 in \n",
 ButtonBox["S. Kobayashi, K. Nomizu I",
  BaseStyle->"Hyperlink",
  ButtonData->"K-N1"],
 ".\n\nThe integrability condition:"
}], "Text",
 CellChangeTimes->{{3.5973885997855015`*^9, 3.5973886194826283`*^9}, {
   3.5973886805381203`*^9, 3.597388763016838*^9}, {3.597388807320372*^9, 
   3.5973888593523483`*^9}, {3.5973892481165843`*^9, 3.597389248119584*^9}, {
   3.5974686722211447`*^9, 3.5974687529512863`*^9}, 3.597471054875729*^9, {
   3.6259248594520626`*^9, 3.625924876177292*^9}, {3.626001672478977*^9, 
   3.6260016794989896`*^9}}],

Cell[TextData[{
 StyleBox["Proposition 1",
  FontColor->RGBColor[0, 0, 1]],
 ". An absolute parallel vector field with coordinates ",
 Cell[BoxData[
  SubscriptBox[
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}], ")"}], 
   RowBox[{
    RowBox[{"i", "=", "1"}], ",", 
    RowBox[{"--", 
     RowBox[{",", "m"}]}]}]]],
  CellChangeTimes->{{3.5973280116780243`*^9, 3.5973280157692585`*^9}, {
    3.597328075913699*^9, 3.5973280771907716`*^9}, {3.597328179819642*^9, 
    3.5973283575508075`*^9}, 3.597328398054124*^9}],
 "  on a pseudo- Riemannian manifold satisfies the system of linear equations"
}], "Text",
 CellChangeTimes->{{3.597468651223508*^9, 3.5974686651231318`*^9}, {
  3.5974687616873016`*^9, 3.5974687668353105`*^9}, {3.59746886327468*^9, 
  3.5974690072941327`*^9}, {3.5974692417313447`*^9, 3.5974692445393496`*^9}, {
  3.59746943280048*^9, 3.59746948360977*^9}, {3.59747449972058*^9, 
  3.5974745004849815`*^9}, {3.624458646869665*^9, 3.6244587141837835`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"i", "=", "1"}], "m"], 
    RowBox[{
     RowBox[{"R", "[", 
      RowBox[{"k", ",", "i", ",", "j", ",", "h"}], "]"}], 
     RowBox[{
      SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}]}]}], "\[Equal]", " ", 
   "0"}], ",", "           ", 
  RowBox[{"(", 
   RowBox[{"k", ",", " ", "j", ",", " ", 
    RowBox[{"h", " ", "\[Equal]", " ", "1"}], ",", "...", ",", "m"}], 
   ")"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5974690605526266`*^9, 3.5974691322035522`*^9}, {
  3.597469170220819*^9, 3.5974692133236947`*^9}, {3.5974693011986494`*^9, 
  3.5974693148330727`*^9}}],

Cell["\<\
A necessary condition for the existence of such a non zero vector field is \
that the rank of this system is less the dimension mf of the manifold. \
Obviously, the set of all absolute parallel vector fields is a finite \
dimensional vector space under pointwise addition and multiplication with \
real numbers. \[FilledRectangle]\
\>", "Text",
 CellChangeTimes->{{3.597469516494627*^9, 3.597469627332822*^9}, 
   3.597471074609764*^9, {3.597471155386706*^9, 3.5974713113089795`*^9}, 
   3.624458742887834*^9}],

Cell[TextData[{
 "Repeating this considerations with the Riemannian metric rgT[f][x,y,...], \
DD replaced by DDimm and \[CapitalGamma] by \[CapitalGamma]rgT one obtains \
the result of Proposition 1 where the curvature tensor is now given by \nthe",
 StyleBox[" curvature tensor of the immersion",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox[" f, ",
  FontColor->GrayLevel[0]],
 "defined as"
}], "Text",
 CellChangeTimes->{{3.6244573675114183`*^9, 3.624457369913822*^9}, {
   3.624458808236349*^9, 3.6244588092191505`*^9}, {3.6244588694976563`*^9, 
   3.6244591352285233`*^9}, 3.624459228485487*^9, {3.624459263975549*^9, 
   3.6244592902303953`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Rimm", "[", 
     RowBox[{"k_", ",", "i_", ",", "h_", ",", "j_"}], "]"}], "[", "ff_", 
    "]"}], "[", "x__", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"\[CapitalGamma]rgT", "[", 
        RowBox[{"k", ",", "i", ",", "j"}], "]"}], "[", "ff", "]"}], "[", "x", 
      "]"}], ",", 
     RowBox[{
      RowBox[{"{", "x", "}"}], "[", 
      RowBox[{"[", "h", "]"}], "]"}]}], "]"}], " ", "-", "  ", 
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"\[CapitalGamma]rgT", "[", 
        RowBox[{"k", ",", "i", ",", "h"}], "]"}], "[", "ff", "]"}], "[", "x", 
      "]"}], ",", 
     RowBox[{
      RowBox[{"{", "x", "}"}], "[", 
      RowBox[{"[", "j", "]"}], "]"}]}], "]"}], " ", "-", " ", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[CapitalGamma]rgT", "[", 
           RowBox[{"k", ",", "l", ",", "j"}], "]"}], "[", "ff", "]"}], "[", 
         "x", "]"}], "*", 
        RowBox[{
         RowBox[{
          RowBox[{"\[CapitalGamma]rgT", "[", 
           RowBox[{"l", ",", "i", ",", "h"}], "]"}], "[", "ff", "]"}], "[", 
         "x", "]"}]}], "-", " ", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[CapitalGamma]rgT", "[", 
           RowBox[{"k", ",", "l", ",", "h"}], "]"}], "[", "ff", "]"}], "[", 
         "x", "]"}], "*", 
        RowBox[{
         RowBox[{
          RowBox[{"\[CapitalGamma]rgT", "[", 
           RowBox[{"l", ",", "i", ",", "j"}], "]"}], "[", "ff", "]"}], "[", 
         "x", "]"}]}]}], ")"}], ",", 
     RowBox[{"{", 
      RowBox[{"l", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.624355724048009*^9, 3.6243558027521477`*^9}}],

Cell[BoxData[
 RowBox[{"?", "Rimm"}]], "Input",
 CellChangeTimes->{{3.6243685480756545`*^9, 3.624368611786166*^9}, {
  3.6243687165583916`*^9, 3.6243687976161337`*^9}, {3.6259252523283553`*^9, 
  3.6259252596321683`*^9}}],

Cell[TextData[{
 StyleBox["Example",
  FontColor->RGBColor[0, 0, 1]],
 ". The spheres"
}], "Text",
 CellChangeTimes->{{3.5973893009176044`*^9, 3.597389332593416*^9}, 
   3.597391353152985*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Rimm", "[", 
      RowBox[{"1", ",", "2", ",", "1", ",", "2"}], "]"}], "[", 
     RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5973293281183205`*^9, 3.5973294162033587`*^9}, {
   3.5973295020032663`*^9, 3.597329553733225*^9}, 3.597329586043073*^9, 
   3.5973303088194137`*^9, 3.5973304210758343`*^9, {3.5973306909162683`*^9, 
   3.597330716155712*^9}, {3.6243690461245704`*^9, 3.6243690467953715`*^9}, {
   3.624517408340295*^9, 3.62451742267672*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.6260027628272924`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Rimm", "[", 
     RowBox[{"1", ",", "1", ",", "1", ",", "2"}], "]"}], "[", 
    RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5973307960902843`*^9, 3.597330833107401*^9}, {
  3.6243690742670197`*^9, 3.624369074859821*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6260027795973215`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Rimm", "[", 
     RowBox[{"2", ",", "1", ",", "1", ",", "2"}], "]"}], "[", 
    RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5973307960902843`*^9, 3.597330869385476*^9}, {
  3.6243690925034523`*^9, 3.6243690934394536`*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  SuperscriptBox[
   RowBox[{"Cos", "[", "v", "]"}], "2"]}]], "Output",
 CellChangeTimes->{3.626002784916931*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Rimm", "[", 
     RowBox[{"2", ",", "2", ",", "1", ",", "2"}], "]"}], "[", 
    RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5973307960902843`*^9, 3.597330869385476*^9}, {
  3.5973308994891977`*^9, 3.597330908523715*^9}, {3.6243691063562765`*^9, 
  3.6243691332351236`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6260027938089466`*^9}]
}, Closed]],

Cell["\<\
The coefficients  R[k, i, h, j] are anti-symmetric with respect to the last \
two  indices h, j:\
\>", "Text",
 CellChangeTimes->{{3.597389423456613*^9, 3.597389486240204*^9}, {
  3.5973895586333447`*^9, 3.5973896440652313`*^9}, {3.597389952256859*^9, 
  3.597389976545248*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"Rimm", "[", 
        RowBox[{"1", ",", "2", ",", "h", ",", "j"}], "]"}], "[", 
       RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"Rimm", "[", 
        RowBox[{"1", ",", "2", ",", "j", ",", "h"}], "]"}], "[", 
       RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"h", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.597330933249129*^9, 3.5973310703899727`*^9}, 
   3.597384757134715*^9, {3.5973896545658317`*^9, 3.5973896918149624`*^9}, 
   3.6148618625455446`*^9, 3.614863502170825*^9, {3.6243691561515636`*^9, 
   3.6243691657143803`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.626002809128174*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"FullSimplify", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Rimm", "[", 
        RowBox[{"h", ",", "j", ",", "1", ",", "2"}], "]"}], "[", 
       RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"h", ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.597330933249129*^9, 3.5973310703899727`*^9}, 
   3.597384757134715*^9, {3.5973896545658317`*^9, 3.59738979650895*^9}, {
   3.597389835566184*^9, 3.5973898447817116`*^9}, 3.614861862561145*^9, 
   3.614863502170825*^9, {3.6243691827340107`*^9, 3.624369183779212*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "1"},
     {
      RowBox[{"-", 
       SuperscriptBox[
        RowBox[{"Cos", "[", "v", "]"}], "2"]}], "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.626002820110593*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"FullSimplify", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Rimm", "[", 
         RowBox[{"1", ",", "i", ",", "1", ",", "2"}], "]"}], "[", 
        RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "]"}], "*", 
     RowBox[{"\[Xi]", "[", "i", "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "2"}], "}"}]}], "]"}], "\[Equal]", "0"}]], "Input",
 CellChangeTimes->{{3.5974714282935853`*^9, 3.59747143094559*^9}, {
  3.597471488338091*^9, 3.5974715248577547`*^9}, {3.597471588677467*^9, 
  3.597471677597623*^9}, {3.597471867231556*^9, 3.597471873611967*^9}, {
  3.6243692021560445`*^9, 3.6243692034352465`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Xi]", "[", "2", "]"}], "\[Equal]", "0"}]], "Output",
 CellChangeTimes->{3.6260028347278185`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"FullSimplify", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Rimm", "[", 
         RowBox[{"2", ",", "i", ",", "1", ",", "2"}], "]"}], "[", 
        RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "]"}], "*", 
     RowBox[{"\[Xi]", "[", "i", "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "2"}], "}"}]}], "]"}], "\[Equal]", "0"}]], "Input",
 CellChangeTimes->{{3.5974714282935853`*^9, 3.59747143094559*^9}, {
   3.597471488338091*^9, 3.5974715248577547`*^9}, {3.597471588677467*^9, 
   3.597471677597623*^9}, {3.597471793537027*^9, 3.5974718058298483`*^9}, 
   3.597471891193198*^9, {3.624369244198118*^9, 3.6243692455085206`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox[
     RowBox[{"Cos", "[", "v", "]"}], "2"]}], " ", 
   RowBox[{"\[Xi]", "[", "1", "]"}]}], "\[Equal]", "0"}]], "Output",
 CellChangeTimes->{3.6260028410302296`*^9}]
}, Closed]],

Cell["\<\
It follows: The only absolute parallel vector field on the sphere of radius r \
is the zero vector field.\[FilledRectangle]\
\>", "Text",
 CellChangeTimes->{{3.59747198351416*^9, 3.597472062653099*^9}, {
  3.6132240301052513`*^9, 3.6132240472582326`*^9}}],

Cell[TextData[{
 "Often one uses the covariant form of the curvature tensor defined by",
 StyleBox[" lowering the index",
  FontColor->RGBColor[0, 0, 1]],
 " k:"
}], "Text",
 CellChangeTimes->{{3.5973901479610524`*^9, 3.5973902036652384`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"covR", "[", 
     RowBox[{"k_", ",", "i_", ",", "h_", ",", "j_"}], "]"}], "[", "g_", "]"}],
    "[", "x__", "]"}], " ", ":=", "  ", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"grg", "[", 
       RowBox[{"k", ",", "l"}], "]"}], "[", "x", "]"}], "*", 
     RowBox[{
      RowBox[{
       RowBox[{"R", "[", 
        RowBox[{"l", ",", "i", ",", "h", ",", "j"}], "]"}], "[", "g", "]"}], 
      "[", "x", "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"l", ",", 
      RowBox[{"Length", "[", 
       RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5973286677245483`*^9, 3.5973289520468106`*^9}, {
   3.597328985941749*^9, 3.5973290042307954`*^9}, 3.5973290495683885`*^9, {
   3.597329079818119*^9, 3.597329268394905*^9}, {3.5973312970269356`*^9, 
   3.5973313689200478`*^9}, {3.5973314099923973`*^9, 3.597331434333789*^9}, 
   3.6186424048824086`*^9, {3.6243692892509975`*^9, 3.624369290296199*^9}, {
   3.6243710854089527`*^9, 3.624371086376154*^9}, 3.6245177926157703`*^9, {
   3.6245178256566277`*^9, 3.624517890256342*^9}, {3.624518944880594*^9, 
   3.6245189560970135`*^9}, 3.6260058406658983`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "covR"}]], "Input",
 CellChangeTimes->{{3.6245182784786234`*^9, 3.624518408161651*^9}, {
   3.624518443339713*^9, 3.6245184629177475`*^9}, 3.6245185108878317`*^9, {
   3.624518967344633*^9, 3.624518972835843*^9}, {3.6259257471472282`*^9, 
   3.6259257502360334`*^9}}],

Cell[BoxData[
 StyleBox["\<\"covR[i,j,k,l][g][x,y,...] is the covariant form of the \
Riemannian curvature tensor \\nof the pseudo-Riemannian metric \
g[i.j][x,y,...].\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.626002862324267*^9},
 CellTags->"Info3626006462-6741741"]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"covRimm", "[", 
     RowBox[{"k_", ",", "i_", ",", "h_", ",", "j_"}], "]"}], "[", "ff_", 
    "]"}], "[", "x__", "]"}], " ", ":=", "  ", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"rgT", "[", 
        RowBox[{"k", ",", "l"}], "]"}], "[", "ff", "]"}], "[", "x", "]"}], 
     "*", 
     RowBox[{
      RowBox[{
       RowBox[{"Rimm", "[", 
        RowBox[{"l", ",", "i", ",", "h", ",", "j"}], "]"}], "[", "ff", "]"}], 
      "[", "x", "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"l", ",", 
      RowBox[{"Length", "[", 
       RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5973286677245483`*^9, 3.5973289520468106`*^9}, {
   3.597328985941749*^9, 3.5973290042307954`*^9}, 3.5973290495683885`*^9, {
   3.597329079818119*^9, 3.597329268394905*^9}, {3.5973312970269356`*^9, 
   3.5973313689200478`*^9}, {3.5973314099923973`*^9, 3.597331434333789*^9}, 
   3.6186424048824086`*^9, {3.6243692892509975`*^9, 3.624369290296199*^9}, {
   3.6243710854089527`*^9, 3.624371086376154*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "covRimm"}]], "Input",
 CellChangeTimes->{{3.6245182784786234`*^9, 3.624518408161651*^9}, {
   3.624518443339713*^9, 3.6245184629177475`*^9}, {3.6245185399038825`*^9, 
   3.6245186395568576`*^9}, 3.624518684718937*^9, {3.625925729140796*^9, 
   3.625925736523609*^9}}],

Cell[BoxData[
 StyleBox["\<\"covRimm[i,j,k,l][f][x,y,...] is the covariant form of the \
Riemannian curvature tensor\\n of the immersion f[x,y,...] into the \
n-dimensional Euclidean space.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.626002878875896*^9},
 CellTags->"Info3626006478-6741741"]
}, Closed]],

Cell["\<\
The coefficients covR[h, j, 1, 2] are also antisymmetric with respect to the \
first two indices:\
\>", "Text",
 CellChangeTimes->{{3.597391451864631*^9, 3.5973916154249864`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"FullSimplify", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"covRimm", "[", 
        RowBox[{"h", ",", "j", ",", "1", ",", "2"}], "]"}], "[", 
       RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"h", ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.597330933249129*^9, 3.5973310703899727`*^9}, 
   3.597384757134715*^9, {3.5973896545658317`*^9, 3.59738979650895*^9}, {
   3.597389835566184*^9, 3.5973898447817116`*^9}, {3.5973903233100815`*^9, 
   3.597390324033123*^9}, 3.614861862576745*^9, 3.6148635021864247`*^9, {
   3.6243693300918694`*^9, 3.6243693315114717`*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", 
      RowBox[{
       SuperscriptBox["r", "2"], " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", "v", "]"}], "2"]}]},
     {
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox["r", "2"]}], " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", "v", "]"}], "2"]}], "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.626002892900321*^9}]
}, Closed]],

Cell["This is true for a general surface:", "Text",
 CellChangeTimes->{{3.597391626064595*^9, 3.59739163960837*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"FullSimplify", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"covRimm", "[", 
         RowBox[{"h", ",", "j", ",", "1", ",", "2"}], "]"}], "[", "sf", "]"}],
        "[", 
       RowBox[{"u", ",", "v"}], "]"}], "+", 
      RowBox[{
       RowBox[{
        RowBox[{"covRimm", "[", 
         RowBox[{"j", ",", "h", ",", "1", ",", "2"}], "]"}], "[", "sf", "]"}],
        "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"h", ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.597330933249129*^9, 3.5973310703899727`*^9}, 
   3.597384757134715*^9, {3.5973896545658317`*^9, 3.59738979650895*^9}, {
   3.597389835566184*^9, 3.5973898447817116`*^9}, {3.5973903233100815`*^9, 
   3.597390324033123*^9}, {3.5973904566657095`*^9, 3.597390511477844*^9}, 
   3.6148618625923448`*^9, 3.6148635022020245`*^9, {3.624369447232475*^9, 
   3.6243694839861393`*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0"},
     {"0", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.6259260514471645`*^9, 3.6260046600558243`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", "6"}], ";", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"FullSimplify", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"covRimm", "[", 
          RowBox[{"h", ",", "j", ",", "1", ",", "2"}], "]"}], "[", 
         RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"covRimm", "[", 
          RowBox[{"j", ",", "h", ",", "1", ",", "2"}], "]"}], "[", 
         RowBox[{"smf", "[", "f", "]"}], "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"h", ",", "2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "//", 
   "MatrixForm"}]}]], "Input",
 CellChangeTimes->{{3.5973905920774546`*^9, 3.5973907262311277`*^9}, {
   3.5973927065273943`*^9, 3.5973927241184*^9}, 3.597392767086858*^9, {
   3.6132779599386983`*^9, 3.6132779817299447`*^9}, 3.6148618626079445`*^9, 
   3.6148635022176247`*^9, {3.624519396813388*^9, 3.6245194394326625`*^9}, 
   3.6260046936582837`*^9, 3.6260047272139425`*^9, 3.626004777633231*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0"},
     {"0", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.6259289032085915`*^9, 3.626004403201373*^9, {3.626004716028723*^9, 
   3.6260047453723745`*^9}, 3.62600494778273*^9}]
}, Closed]],

Cell["and for the general pseudo-Riemannian space:", "Text",
 CellChangeTimes->{{3.597392845137322*^9, 3.597392874136981*^9}, {
  3.6245187925307264`*^9, 3.624518832341996*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"??", "grg"}]], "Input",
 CellChangeTimes->{{3.6260051421434712`*^9, 3.626005143079473*^9}, 
   3.626005860275133*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 StyleBox["\<\"grg[i,j][x,y,...] is a general, unspecified pseudo-Riemannian \
metric with respect to local coordinates {x,y,...}.\"\>", "MSG"]], "Print", \
"PrintUsage",
 CellChangeTimes->{3.6260058649239407`*^9},
 CellTags->"Info3626009464-6741741"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"grg", "[", 
          RowBox[{"h_", ",", "k_"}], "]"}], ":=", 
         RowBox[{
          RowBox[{
           RowBox[{"Normal", "[", 
            RowBox[{"Symmetrize", "[", 
             RowBox[{
              RowBox[{"Table", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"gen", "[", 
                  RowBox[{"i", ",", "j"}], "]"}], "[", "##1", "]"}], ",", 
                RowBox[{"{", 
                 RowBox[{"i", ",", 
                  RowBox[{"Length", "[", 
                   RowBox[{"{", "##1", "}"}], "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"j", ",", 
                  RowBox[{"Length", "[", 
                   RowBox[{"{", "##1", "}"}], "]"}]}], "}"}]}], "]"}], ",", 
              RowBox[{"Symmetric", "[", 
               RowBox[{"{", 
                RowBox[{"1", ",", "2"}], "}"}], "]"}]}], "]"}], "]"}], 
           "\[LeftDoubleBracket]", 
           RowBox[{"h", ",", "k"}], "\[RightDoubleBracket]"}], "&"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`grg],
  Editable->False]], "Print",
 CellChangeTimes->{3.626005864939541*^9},
 CellTags->"Info3626009464-6741741"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Context", "[", "grg", "]"}]], "Input",
 CellChangeTimes->{{3.6260052092547894`*^9, 3.6260052163060017`*^9}, 
   3.626005397219519*^9, 3.6260058737223563`*^9}],

Cell[BoxData["\<\"Global`\"\>"], "Output",
 CellChangeTimes->{3.626005217366803*^9, 3.626005398483122*^9, 
  3.626005874814358*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", "5"}], ";", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"FullSimplify", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"covR", "[", 
          RowBox[{"h", ",", "j", ",", "1", ",", "2"}], "]"}], "[", "grg", 
         "]"}], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"covR", "[", 
          RowBox[{"j", ",", "h", ",", "1", ",", "2"}], "]"}], "[", "grg", 
         "]"}], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"h", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "3"}], "}"}]}], "]"}], "//", 
   "MatrixForm"}]}]], "Input",
 CellChangeTimes->{{3.5973905920774546`*^9, 3.5973907262311277`*^9}, {
   3.5973927065273943`*^9, 3.5973927241184*^9}, 3.597392767086858*^9, {
   3.6132779599386983`*^9, 3.6132779737894907`*^9}, 3.6148618626235447`*^9, 
   3.614863502233225*^9, {3.6245194827539387`*^9, 3.6245195430324445`*^9}, {
   3.6260042812715592`*^9, 3.6260043001475925`*^9}, 3.626004421781006*^9, {
   3.626005918525635*^9, 3.626005921770441*^9}, {3.6260112284634857`*^9, 
   3.6260112853255863`*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0"},
     {"0", "0", "0"},
     {"0", "0", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.6260112132534595`*^9, 3.6260112375895023`*^9}, 
   3.6260120142992663`*^9}]
}, Closed]],

Cell[TextData[{
 "It follows\n",
 StyleBox["Lemma 1",
  FontColor->RGBColor[0, 0, 1]],
 ". The only essential component of the covariant curvature tensor of a \
surface in the n-dimensional Euclidean space is"
}], "Text",
 CellChangeTimes->{{3.5973929512323904`*^9, 3.597393035953236*^9}, {
  3.5973930819938693`*^9, 3.5973930855450726`*^9}, {3.597474539173049*^9, 
  3.5974745400154505`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "3"}], ";"}]], "Input",
 CellChangeTimes->{{3.6245212142321796`*^9, 3.6245212153865814`*^9}, {
  3.626012446092425*^9, 3.626012480396885*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"covRimm", "[", 
    RowBox[{"1", ",", "2", ",", "1", ",", "2"}], " ", "]"}], "[", "sf", "]"}],
   "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.597393063024784*^9, 3.597393122723199*^9}, {
   3.5973931779243565`*^9, 3.597393198355525*^9}, {3.613224376315054*^9, 
   3.6132243795782404`*^9}, 3.61880996300032*^9, {3.624520863746364*^9, 
   3.624520884853201*^9}, {3.626012530192172*^9, 3.6260125370717845`*^9}}],

Cell[BoxData[
 InterpretationBox[
  TagBox[
   FrameBox[GridBox[{
      {
       ItemBox[
        TagBox[
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             TemplateBox[{"20"},
              "OutputSizeLimit`Skeleton",
              DisplayFunction->(FrameBox[
                
                RowBox[{
                 "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                Background -> GrayLevel[0.75], 
                BaseStyle -> {
                 "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                  ShowStringCharacters -> False}, BaselinePosition -> 
                Baseline, ContentPadding -> False, FrameMargins -> 1, 
                FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], "+", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                FractionBox[
                 TemplateBox[{"1"},
                  "OutputSizeLimit`Skeleton",
                  DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], 
                 RowBox[{"2", " ", 
                  TemplateBox[{"1"},
                   "OutputSizeLimit`Skeleton",
                   DisplayFunction->(FrameBox[
                    
                    RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                    Background -> GrayLevel[0.75], 
                    BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                    Baseline, ContentPadding -> False, FrameMargins -> 1, 
                    FrameStyle -> GrayLevel[0.75], RoundingRadius -> 
                    7]& )]}]], "+", 
                TemplateBox[{"1"},
                 "OutputSizeLimit`Skeleton",
                 DisplayFunction->(FrameBox[
                   
                   RowBox[{
                    "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                   Background -> GrayLevel[0.75], 
                   BaseStyle -> {
                    "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                    ShowStringCharacters -> False}, BaselinePosition -> 
                   Baseline, ContentPadding -> False, FrameMargins -> 1, 
                   FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )]}], 
               ")"}], " ", 
              TemplateBox[{"1"},
               "OutputSizeLimit`Skeleton",
               DisplayFunction->(FrameBox[
                 
                 RowBox[{
                  "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                 Background -> GrayLevel[0.75], 
                 BaseStyle -> {
                  "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                   ShowStringCharacters -> False}, BaselinePosition -> 
                 Baseline, ContentPadding -> False, FrameMargins -> 1, 
                 FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )]}]}], 
            ")"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            TemplateBox[{"1"},
             "OutputSizeLimit`Skeleton",
             DisplayFunction->(FrameBox[
               
               RowBox[{
                "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
               Background -> GrayLevel[0.75], 
               BaseStyle -> {
                "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                 ShowStringCharacters -> False}, BaselinePosition -> Baseline,
                ContentPadding -> False, FrameMargins -> 1, FrameStyle -> 
               GrayLevel[0.75], RoundingRadius -> 7]& )], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             TemplateBox[{"16"},
              "OutputSizeLimit`Skeleton",
              DisplayFunction->(FrameBox[
                
                RowBox[{
                 "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                Background -> GrayLevel[0.75], 
                BaseStyle -> {
                 "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                  ShowStringCharacters -> False}, BaselinePosition -> 
                Baseline, ContentPadding -> False, FrameMargins -> 1, 
                FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )], "+", 
             TemplateBox[{"1"},
              "OutputSizeLimit`Skeleton",
              DisplayFunction->(FrameBox[
                
                RowBox[{
                 "\" \[CenterEllipsis]\"", #, "\"\[CenterEllipsis] \""}], 
                Background -> GrayLevel[0.75], 
                BaseStyle -> {
                 "Deploy", FontColor -> GrayLevel[1], FontSize -> Smaller, 
                  ShowStringCharacters -> False}, BaselinePosition -> 
                Baseline, ContentPadding -> False, FrameMargins -> 1, 
                FrameStyle -> GrayLevel[0.75], RoundingRadius -> 7]& )]}], 
            ")"}]}]}],
         Short[#, 5]& ],
        BaseStyle->{Deployed -> False},
        StripOnInput->False]},
      {GridBox[{
         {
          TagBox[
           TooltipBox[
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource[
                "FEStrings", "sizeBriefExplanation"], StandardForm],
               ImageSizeCache->{183., {7., 26.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarCategory",
             StripOnInput->False],
            StyleBox[
             DynamicBox[
              ToBoxes[
               FEPrivate`FrontEndResource["FEStrings", "sizeExplanation"], 
               StandardForm]], DynamicUpdating -> True, StripOnInput -> 
             False]],
           Annotation[#, 
            Style[
             Dynamic[
              FEPrivate`FrontEndResource["FEStrings", "sizeExplanation"]], 
             DynamicUpdating -> True], "Tooltip"]& ], 
          ButtonBox[
           PaneSelectorBox[{False->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeShowLess"], 
                StandardForm],
               ImageSizeCache->{167., {1., 26.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControl",
             StripOnInput->False], True->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeShowLess"], 
                StandardForm]],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControlActive",
             StripOnInput->False]}, Dynamic[
             CurrentValue["MouseOver"]],
            Alignment->Center,
            FrameMargins->0,
            ImageSize->{Automatic, 25}],
           Appearance->None,
           
           ButtonFunction:>OutputSizeLimit`ButtonFunction[
            Identity, 82, 26382165933215868772, 5/2],
           Enabled->True,
           Evaluator->Automatic,
           Method->"Queued"], 
          ButtonBox[
           PaneSelectorBox[{False->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeShowMore"], 
                StandardForm],
               ImageSizeCache->{189., {1., 26.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControl",
             StripOnInput->False], True->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeShowMore"], 
                StandardForm]],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControlActive",
             StripOnInput->False]}, Dynamic[
             CurrentValue["MouseOver"]],
            Alignment->Center,
            FrameMargins->0,
            ImageSize->{Automatic, 25}],
           Appearance->None,
           
           ButtonFunction:>OutputSizeLimit`ButtonFunction[
            Identity, 82, 26382165933215868772, 5 2],
           Enabled->True,
           Evaluator->Automatic,
           Method->"Queued"], 
          ButtonBox[
           PaneSelectorBox[{False->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeShowAll"], 
                StandardForm],
               ImageSizeCache->{141., {1., 26.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControl",
             StripOnInput->False], True->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeShowAll"], 
                StandardForm],
               ImageSizeCache->{255., {6., 22.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControlActive",
             StripOnInput->False]}, Dynamic[
             CurrentValue["MouseOver"]],
            Alignment->Center,
            FrameMargins->0,
            ImageSize->{Automatic, 25}],
           Appearance->None,
           
           ButtonFunction:>OutputSizeLimit`ButtonFunction[
            Identity, 82, 26382165933215868772, Infinity],
           Enabled->True,
           Evaluator->Automatic,
           Method->"Queued"], 
          ButtonBox[
           PaneSelectorBox[{False->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeChangeLimit"], 
                StandardForm],
               ImageSizeCache->{247., {1., 27.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControl",
             StripOnInput->False], True->
            StyleBox[
             StyleBox[
              DynamicBox[ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeChangeLimit"], 
                StandardForm],
               ImageSizeCache->{226., {1., 22.}}],
              StripOnInput->False,
              DynamicUpdating->True], "SuggestionsBarControlActive",
             StripOnInput->False]}, Dynamic[
             CurrentValue["MouseOver"]],
            Alignment->Center,
            FrameMargins->0,
            ImageSize->{Automatic, 25}],
           Appearance->None,
           ButtonFunction:>FrontEndExecute[{
              FrontEnd`SetOptions[
              FrontEnd`$FrontEnd, 
               FrontEnd`PreferencesSettings -> {"Page" -> "Evaluation"}], 
              FrontEnd`FrontEndToken["PreferencesDialog"]}],
           Evaluator->None,
           Method->"Preemptive"]}
        },
        AutoDelete->False,
        FrameStyle->GrayLevel[0.85],
        GridBoxDividers->{"Columns" -> {False, {True}}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{"Columns" -> {{2}}}]}
     },
     DefaultBaseStyle->"Column",
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxDividers->{
      "Columns" -> {{False}}, "ColumnsIndexed" -> {}, "Rows" -> {{False}}, 
       "RowsIndexed" -> {}},
     GridBoxItemSize->{
      "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], 
         Offset[1.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Background->RGBColor[0.9657, 0.9753, 0.9802],
    FrameMargins->{{12, 12}, {0, 15}},
    FrameStyle->GrayLevel[0.85],
    RoundingRadius->5,
    StripOnInput->False],
   Deploy,
   DefaultBaseStyle->"Deploy"],
  Out[82]]], "Output",
 CellChangeTimes->{
  3.626012418620776*^9, {3.626012464999658*^9, 3.6260124901625023`*^9}, 
   3.6260125409873915`*^9}]
}, Closed]],

Cell["\[FilledRectangle]", "Text",
 CellChangeTimes->{
  3.5973930498580313`*^9, {3.597393216582567*^9, 3.5973932327214904`*^9}}],

Cell[TextData[{
 "We return to dimension 3 and show\n",
 StyleBox["Lemma 2",
  FontColor->RGBColor[0, 0, 1]],
 ". The essential component of the curvature tensor is  related to the \
Gaussian curvature by the following formula"
}], "Text",
 CellChangeTimes->{{3.5973928878887672`*^9, 3.59739289807335*^9}, {
  3.5973932554967933`*^9, 3.597393345600947*^9}, {3.597393403425254*^9, 
  3.5973934148809094`*^9}, {3.597395110080869*^9, 3.5973951371294165`*^9}, {
  3.5974745575810814`*^9, 3.597474558517083*^9}, {3.613278341997551*^9, 
  3.613278344620701*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"covRimm", "[", 
    RowBox[{"1", ",", "2", ",", "1", ",", "2"}], "]"}], "[", "sf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dA", "[", "sf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"?", "gauss"}]], "Input",
 CellChangeTimes->{{3.6245215843896303`*^9, 3.6245215880868363`*^9}}],

Cell[BoxData[
 RowBox[{"sf", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6245216241384993`*^9, 3.6245216303785105`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"gauss", "[", "sf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"covRimm", "[", 
        RowBox[{"1", ",", "2", ",", "1", ",", "2"}], "]"}], "[", "sf", "]"}], 
      "[", 
      RowBox[{"u", ",", "v"}], "]"}], "-", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"gauss", "[", "sf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "*", 
      RowBox[{
       RowBox[{
        RowBox[{"dA", "[", "sf", "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "^", "2"}]}]}], ",", 
    RowBox[{"TimeConstraint", " ", "\[Rule]", "  ", "Infinity"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5973933860842624`*^9, 3.59739338866141*^9}, {
   3.597393435294077*^9, 3.597393445980688*^9}, 3.5973951782147665`*^9, {
   3.617169970290579*^9, 3.617170009321847*^9}, {3.6245210195906377`*^9, 
   3.62452102080744*^9}, {3.624526591111689*^9, 3.6245265993641033`*^9}, {
   3.624531878234537*^9, 3.6245319201674104`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6260152913658223`*^9}]
}, Closed]],

Cell["\[FilledRectangle]", "Text",
 CellChangeTimes->{{3.5973951992669706`*^9, 3.5973951992739706`*^9}}],

Cell["For the spheres we have", "Text",
 CellChangeTimes->{{3.597395222457297*^9, 3.597395232160852*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"covRimm", "[", 
      RowBox[{"1", ",", "2", ",", "1", ",", "2"}], "]"}], "[", 
     RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "/", 
   RowBox[{
    RowBox[{
     RowBox[{"dA", "[", 
      RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "^", "2"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5973866631347313`*^9, 3.5973866754074335`*^9}, {
   3.597386706204195*^9, 3.597386716163765*^9}, {3.5973867934351845`*^9, 
   3.597386821596795*^9}, 3.597386857140828*^9, {3.624521039511873*^9, 
   3.6245210410094757`*^9}}],

Cell[BoxData[
 FractionBox["1", 
  SuperscriptBox["r", "2"]]], "Output",
 CellChangeTimes->{3.6260712240438137`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"gauss", "[", 
    RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5973307960902843`*^9, 3.597330833107401*^9}, {
  3.5973319086279173`*^9, 3.597331910039998*^9}}],

Cell[BoxData[
 FractionBox["1", 
  SuperscriptBox["r", "2"]]], "Output",
 CellChangeTimes->{3.6260712295974236`*^9}]
}, Closed]],

Cell["Now we consider a surface in the Euclidean 3-space.", "Text",
 CellChangeTimes->{{3.597472852045686*^9, 3.59747288294934*^9}}],

Cell[CellGroupData[{

Cell[BoxData["dim"], "Input",
 CellChangeTimes->{{3.597472196189334*^9, 3.5974721967977347`*^9}}],

Cell[BoxData["3"], "Output",
 CellChangeTimes->{3.626071235603434*^9}]
}, Closed]],

Cell["The necessary conditions of proposition 1 are equivalent to", "Text",
 CellChangeTimes->{{3.597473422710288*^9, 3.597473472006375*^9}, {
   3.6132796320523376`*^9, 3.613279636212576*^9}, 3.61744137208919*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"covRimm", "[", 
          RowBox[{"k", ",", "i", ",", "1", ",", "2"}], "]"}], "[", "sf", 
         "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}], "/", 
       RowBox[{
        RowBox[{
         RowBox[{"dA", "[", "sf", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "^", "2"}]}], "*", 
      RowBox[{"\[Xi]", "[", "i", "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "2"}], "}"}]}], "]"}], "\[Equal]", " ", "0"}], ",", 
  " "}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.597472280023881*^9, 3.597472380628458*^9}, 
   3.5974726087632585`*^9, {3.597472647560527*^9, 3.5974726521625347`*^9}, {
   3.597472682535788*^9, 3.597472689774201*^9}, 3.597473403163454*^9, {
   3.5974734804459896`*^9, 3.5974735828601694`*^9}, 3.597473647475483*^9, {
   3.624521070977128*^9, 3.624521071569929*^9}}],

Cell["\<\
k = 1, 2. Denote the Gaussian curvature of the surface by K[u,v].. Using the \
last two Lemmas the system of equations reduce to\
\>", "Text",
 CellChangeTimes->{{3.5974736605327063`*^9, 3.5974738002309513`*^9}, {
  3.6174414045010443`*^9, 3.6174414118204627`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"K", "[", 
     RowBox[{"u", ",", "v"}], "]"}], 
    RowBox[{"\[Xi]", "[", "2", "]"}]}], "\[Equal]", " ", "0"}], ",", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     RowBox[{"K", "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], 
    RowBox[{"\[Xi]", "[", "1", "]"}]}], "\[Equal]", " ", "0"}], 
  ","}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5974738192629848`*^9, 3.597473901693529*^9}, 
   3.5974739416139994`*^9}],

Cell["and it follows:", "Text",
 CellChangeTimes->{{3.597474011439722*^9, 3.597474151465568*^9}, 
   3.6132794864470096`*^9}],

Cell[TextData[{
 StyleBox["Proposition 2",
  FontColor->RGBColor[0, 0, 1]],
 ". On a surface in the Euclidean 3-space exist non zero absolute parallel \
vector fields if and only if the surface is developable: K[u,v]\[Congruent] 0."
}], "Text",
 CellChangeTimes->{{3.597474011439722*^9, 3.597474151465568*^9}, {
  3.6132794864470096`*^9, 3.613279503380978*^9}, {3.613279693620859*^9, 
  3.613279711748896*^9}}],

Cell[TextData[{
 StyleBox["Proof.",
  FontColor->RGBColor[0, 0, 1]],
 " By Proposition 1 the condition is necessary. On the other hand, if it is \
satisfied, the surface is developable, thus locally isometric to an open \
region of the Euclidean plane; the images of constant vector fields are \
absolute parallel. \[FilledRectangle]"
}], "Text",
 CellChangeTimes->{{3.597474207298066*^9, 3.597474365856745*^9}, {
  3.597474445042484*^9, 3.5974744834809513`*^9}, {3.6174414537648616`*^9, 
  3.617441462916385*^9}}],

Cell[TextData[{
 StyleBox["Example.",
  FontColor->RGBColor[0, 0, 1]],
 " Flat tori in the  4-space."
}], "Text",
 CellChangeTimes->{{3.621077682746496*^9, 3.6210777464251385`*^9}, 
   3.621077781153125*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"torus4", "[", "b_", "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"Cos", "[", "b", "]"}], " ", 
     RowBox[{"Cos", "[", "u", "]"}]}], ",", 
    RowBox[{
     RowBox[{"Cos", "[", "b", "]"}], " ", 
     RowBox[{"Sin", "[", "u", "]"}]}], ",", 
    RowBox[{
     RowBox[{"Cos", "[", "v", "]"}], " ", 
     RowBox[{"Sin", "[", "b", "]"}]}], ",", 
    RowBox[{
     RowBox[{"Sin", "[", "b", "]"}], " ", 
     RowBox[{"Sin", "[", "v", "]"}]}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.454219651578125*^9, 3.45421967153125*^9}, 
   3.541836408381708*^9, 3.541836718180749*^9, {3.621077791602722*^9, 
   3.621077829255876*^9}}],

Cell["\<\
The stereographic projection permits to plot a conformal picture of this \
surface which isn\[CloseCurlyQuote]t flat, of course:\
\>", "Text",
 CellChangeTimes->{{3.621079052049816*^9, 3.621079126698086*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"sterproj", "[", 
    RowBox[{
     RowBox[{"torus4", "[", "0.3", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6210788544805155`*^9, 3.6210790255993032`*^9}, {
  3.6210797172378626`*^9, 3.6210797199650183`*^9}, {3.6210797528539*^9, 
  3.6210797595082803`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"fund1", "[", 
     RowBox[{"torus4", "[", "b", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"s1", ",", "s2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"s1", ",", "s2"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.62107784855398*^9, 3.6210779697319107`*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    SuperscriptBox["s1", "2"], "+", 
    SuperscriptBox["s2", "2"], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"s1", "-", "s2"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"s1", "+", "s2"}], ")"}], " ", 
     RowBox[{"Cos", "[", 
      RowBox[{"2", " ", "b"}], "]"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.6260713286575975`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Rimm", "[", 
    RowBox[{"1", ",", "2", ",", "1", ",", "2"}], "]"}], "[", 
   RowBox[{"torus4", "[", "b", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6210782245524855`*^9, 3.621078269756071*^9}, 
   3.6210785762195997`*^9, {3.621078617363953*^9, 3.6210786268834977`*^9}, 
   3.62108009099024*^9, {3.6245273680710535`*^9, 3.624527368695055*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6260713763312807`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"covRimm", "[", 
    RowBox[{"1", ",", "2", ",", "1", ",", "2"}], "]"}], "[", 
   RowBox[{"torus4", "[", "b", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6210782245524855`*^9, 3.621078269756071*^9}, 
   3.6210785762195997`*^9, {3.621078617363953*^9, 3.6210786268834977`*^9}, {
   3.621078673464162*^9, 3.6210786822146626`*^9}, {3.624527384607083*^9, 
   3.6245273853402843`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6260713822436914`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.621080447851651*^9, 3.6210804499227695`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.626071385504097*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", "s1"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", "s2"}], ";", 
  RowBox[{
   RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], " "}]], "Input",
 CellChangeTimes->{{3.62108019186901*^9, 3.6210801920270185`*^9}, {
  3.6210802857313786`*^9, 3.6210804271744685`*^9}, {3.621080459316307*^9, 
  3.621080521363856*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"s1", ",", "s2"}], "}"}]], "Output",
 CellChangeTimes->{3.626071389918905*^9}]
}, Closed]],

Cell["\<\
The vector fields with constant local coordinates are absolute parallel:\
\>", "Text",
 CellChangeTimes->{{3.621080579673191*^9, 3.6210806403776627`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DDimm", "[", 
    RowBox[{"torus4", "[", "b", "]"}], "]"}], "[", 
   RowBox[{"vv", "[", "\[Xi]", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6210801247241693`*^9, 3.6210801373638926`*^9}, {
  3.624527416524739*^9, 3.62452741732034*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6260713962213163`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "\[Xi]", "]"}], ";", 
  RowBox[{
   RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.621080660575818*^9, 3.6210806980599623`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Xi]", "[", "1", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{"\[Xi]", "[", "2", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6260714084205375`*^9}]
}, Closed]],

Cell["\<\
Thus the Riemann curvature tensor of this surface in the Euclidean 4-space is \
the null tensor, the surface is flat. Find a local isometry with the \
Euclidean plane. \[FilledSquare]\
\>", "Text",
 CellChangeTimes->{{3.6210795146792765`*^9, 3.6210795226497326`*^9}, {
   3.6210795725615873`*^9, 3.6210796670099897`*^9}, {3.6210798513535337`*^9, 
   3.6210798707376423`*^9}, 3.6259237255423155`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["4.2.4. Parallel Displacement Along Curves", "Subsubsection",
 CellChangeTimes->{{3.595076945363204*^9, 3.5950769905720835`*^9}, 
   3.5953174793534184`*^9, {3.5953175263051033`*^9, 3.595317587064579*^9}, 
   3.5956008724322104`*^9, {3.5965213907316623`*^9, 3.5965214041788855`*^9}, {
   3.5972984401750546`*^9, 3.59729846072029*^9}, {3.5972984954771514`*^9, 
   3.5972985062723703`*^9}, {3.5973320886172123`*^9, 3.5973320905123205`*^9}, 
   3.5973952636966553`*^9, {3.597729800827932*^9, 3.59772986233884*^9}, 
   3.6132798036301517`*^9, 3.6132847212454233`*^9, 3.6171717529369097`*^9, 
   3.61717393361794*^9, 3.6188231691395845`*^9, 3.6188258241208286`*^9, {
   3.6245360039693832`*^9, 3.6245360091017923`*^9}, 3.6245420308786135`*^9}],

Cell["\<\
In the last subsection we have seen that non zero absolute parallel vector \
fields exist only under certain integrability conditions expressed as \
restrictions to the curvature tensor. In dimension 1 these conditions don\
\[CloseCurlyQuote]t appear, Thus it is possible to define parallel vector \
functions along curves on the immersion (or Riemannian manifold). Aiming at \
this we first define the absolute derivative of a tangent vector function \
along a curve. For simplicity we consider curves on surfaces; the definition \
can be adapted with little change to immersions of arbitrary, not too high,  \
finite dimensions.
Let\
\>", "Text",
 CellChangeTimes->{{3.597735507062501*^9, 3.5977355331009903`*^9}, {
  3.5977355962056*^9, 3.597735634300779*^9}, {3.5977358303089895`*^9, 
  3.5977359083014507`*^9}, {3.59773603346961*^9, 3.597736128501045*^9}, {
  3.5977362858940477`*^9, 3.5977364648612843`*^9}, {3.5978475796643496`*^9, 
  3.5978476595364895`*^9}, {3.6132799006777024`*^9, 3.613279904972948*^9}, {
  3.6260716008312755`*^9, 3.626071601205676*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"cu", "[", "t_", "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"u", "[", "t", "]"}], ",", 
    RowBox[{"v", "[", "t", "]"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.5977318613653574`*^9, 3.597731863773495*^9}, {
   3.597736504464549*^9, 3.597736506142645*^9}, 3.597736550155163*^9, {
   3.6245408600029573`*^9, 3.6245408613757596`*^9}}],

Cell["be a smooth curve on the surface", "Text",
 CellChangeTimes->{{3.5977365672371397`*^9, 3.5977365831090474`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sf", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.597738312935988*^9, 3.597738318614313*^9}, {
  3.597817180004476*^9, 3.5978171949961023`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "1", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "2", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "3", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6260716494253607`*^9}]
}, Closed]],

Cell[TextData[{
 "We defined in eudiffgeov2.m the ",
 StyleBox["absolute derivative ",
  FontColor->RGBColor[0, 0, 1]],
 "DD1 of a vector function vw[t] given along a curve ",
 "cu",
 "[t] of the surface:"
}], "Text",
 CellChangeTimes->{{3.5978243856151857`*^9, 3.597824491751256*^9}, 
   3.597847681766528*^9, 3.624540411361769*^9, 3.6245407807392178`*^9, {
   3.6260721218721905`*^9, 3.626072135865415*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"cu", ",", "vw", ",", "ff", ",", "DD1", ",", "u", ",", "v"}], 
   "]"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"DD1", "[", 
     RowBox[{"ff_", ",", "cu_", ",", "vw_"}], "]"}], "[", "t_", "]"}], " ", ":=",
    " ", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "mf", ",", "\[CapitalGamma]\[CapitalGamma]", ",", "s", ",", " ", "i", 
       ",", "j", ",", "k", ",", "u", ",", "v"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"mf", " ", "=", " ", 
       RowBox[{"Length", "[", 
        RowBox[{"cu", "[", "t", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"\[CapitalGamma]\[CapitalGamma]", "[", 
         RowBox[{"i_", ",", "j_", ",", "k_"}], "]"}], "[", "s_", "]"}], ":=", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[CapitalGamma]rgT", "[", 
           RowBox[{"i", ",", "j", ",", "k"}], "]"}], "[", "ff", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "/.", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"u", "\[Rule]", " ", 
           RowBox[{
            RowBox[{"cu", "[", "s", "]"}], "[", 
            RowBox[{"[", "1", "]"}], "]"}]}], ",", 
          RowBox[{"v", "->", 
           RowBox[{
            RowBox[{"cu", "[", "s", "]"}], "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"D", "[", 
        RowBox[{
         RowBox[{"vw", "[", "t", "]"}], ",", "t"}], "]"}], " ", "+", " ", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"Sum", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"\[CapitalGamma]\[CapitalGamma]", "[", 
              RowBox[{"i", ",", "j", ",", "k"}], "]"}], "[", "t", "]"}], "*", 
            RowBox[{
             RowBox[{"vw", "[", "t", "]"}], "[", 
             RowBox[{"[", "j", "]"}], "]"}], "*", 
            RowBox[{"D", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"cu", "[", "t", "]"}], "[", 
               RowBox[{"[", "k", "]"}], "]"}], ",", "t"}], "]"}]}], ",", 
           RowBox[{"{", 
            RowBox[{"j", ",", "mf"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"k", ",", "mf"}], "}"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "mf"}], "}"}]}], "]"}]}]}]}], "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5977308704166784`*^9, 3.5977309528863955`*^9}, {
   3.597730990067522*^9, 3.5977310541191854`*^9}, {3.5977311130875587`*^9, 
   3.597731171388893*^9}, {3.5977312247049427`*^9, 3.5977316965809326`*^9}, {
   3.59773244291862*^9, 3.597732454294271*^9}, {3.597736676816407*^9, 
   3.5977366770154185`*^9}, 3.5977367373498697`*^9, {3.597736775207035*^9, 
   3.5977367948381577`*^9}, {3.597737366309844*^9, 3.5977374333886805`*^9}, 
   3.597737547742221*^9, {3.5977375864544353`*^9, 3.5977376044944677`*^9}, {
   3.5977379689903154`*^9, 3.5977380117027583`*^9}, {3.59773809127831*^9, 
   3.597738147071501*^9}, {3.597738183294573*^9, 3.5977381938471766`*^9}, {
   3.5977382930868526`*^9, 3.5977383028234096`*^9}, 3.597738333062139*^9, {
   3.59773856211124*^9, 3.597738563639327*^9}, {3.597738626846943*^9, 
   3.5977386858323164`*^9}, 3.597738739184368*^9, {3.597739039800562*^9, 
   3.597739048865081*^9}, 3.597739103775221*^9, {3.5977391974875813`*^9, 
   3.5977392430151854`*^9}, {3.5977416344573517`*^9, 
   3.5977416832421427`*^9}, {3.5977417284907303`*^9, 
   3.5977417331069946`*^9}, {3.597741787564109*^9, 3.5977418090453377`*^9}, 
   3.597742020350424*^9, {3.597742051182187*^9, 3.597742090246422*^9}, {
   3.5977421906331635`*^9, 3.5977421946103907`*^9}, 3.5977614023163023`*^9, 
   3.5977615118596945`*^9, {3.5977615457429543`*^9, 3.5977615563509727`*^9}, {
   3.5977616156154766`*^9, 3.597761781989769*^9}, {3.5977618977731724`*^9, 
   3.5977621593544316`*^9}, {3.5977621933936915`*^9, 3.597762210085721*^9}, {
   3.597762336165142*^9, 3.597762354136374*^9}, {3.597762521743068*^9, 
   3.597762586623582*^9}, 3.5977626705829296`*^9, 3.5977627045441895`*^9, 
   3.597762821466395*^9, {3.597762857174857*^9, 3.597762946890615*^9}, {
   3.597763103031289*^9, 3.597763144230962*^9}, 3.597763243556336*^9, {
   3.5977633091544514`*^9, 3.5977633095132523`*^9}, {3.597763359058939*^9, 
   3.5977635199420214`*^9}, {3.5977637409788094`*^9, 
   3.5977637779820747`*^9}, {3.597763824875757*^9, 3.5977638472461967`*^9}, {
   3.597763964184002*^9, 3.597764059406569*^9}, {3.5977641187802734`*^9, 
   3.5977641373911057`*^9}, 3.597764334138652*^9, 3.597764391999153*^9, {
   3.5977644843825154`*^9, 3.597764518515375*^9}, 3.5977645786378813`*^9, 
   3.5978143208626547`*^9, 3.597814672690072*^9, {3.5978148033247013`*^9, 
   3.5978148102199135`*^9}, {3.597814841201568*^9, 3.597814950963361*^9}, 
   3.5978151119556437`*^9, {3.5978152110938177`*^9, 3.5978152698903213`*^9}, {
   3.5978163646534443`*^9, 3.597816390253089*^9}, 3.5978164326227636`*^9, {
   3.5978164684872265`*^9, 3.5978164759596395`*^9}, {3.5978165061924925`*^9, 
   3.5978166048782663`*^9}, {3.597816688494413*^9, 3.5978167555121307`*^9}, 
   3.5978209977279816`*^9, {3.5978242016266623`*^9, 3.5978242030427427`*^9}, {
   3.5979349765451393`*^9, 3.5979350763378468`*^9}, {3.597935149242017*^9, 
   3.597935178465688*^9}, {3.61328063931495*^9, 3.613280694826125*^9}, {
   3.614861862639145*^9, 3.614861862654745*^9}, 3.614863502248825*^9, 
   3.614917793995365*^9, {3.6245410126180253`*^9, 3.6245410144432287`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "DD1"}]], "Input",
 CellChangeTimes->{{3.597847771310686*^9, 3.597847775990694*^9}, 
   3.597847810685155*^9, 3.597931689455128*^9, {3.6245404552134457`*^9, 
   3.6245404956175175`*^9}, 3.6245407109290953`*^9, {3.624540760833583*^9, 
   3.6245407743588066`*^9}, 3.624540881811795*^9, 3.62454095280752*^9, {
   3.626072002859582*^9, 3.626072006369588*^9}}],

Cell[BoxData[
 StyleBox["\<\"DD1[ff,cu,vw][t] is the absolute derivative of a vector \
function \\nvw[t] ={vw1[t],vw2[t]} defined along a curve with local \
coordinatess cu[t]={u[t],v[t]} \\nof a surface ff[u,v] in the n-dimensional \
Euclidean space. For higher dimensional \\nimmersions or Riemannian spaces \
create a new Module changing the definition of \[CapitalGamma]\[CapitalGamma] \
\\ncorresponding to the situation.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.6260725756457877`*^9},
 CellTags->"Info3626076175-1128913"]
}, Closed]],

Cell[TextData[{
 StyleBox["Examples",
  FontColor->RGBColor[0, 0, 1]],
 " :"
}], "Text",
 CellChangeTimes->{{3.5978245313115187`*^9, 3.597824542687169*^9}, 
   3.5979097307832108`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Gamma]", "[", "t_", "]"}], " ", ":=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}]}], ";", 
  RowBox[{
   RowBox[{"\[Xi]", "[", "t_", "]"}], " ", ":=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"s1", "[", "t", "]"}], ",", 
     RowBox[{"s2", "[", "t", "]"}]}], "}"}]}], ";", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"DD1", "[", 
     RowBox[{
      RowBox[{"sphero", "[", "r", "]"}], ",", "\[Gamma]", ",", "\[Xi]"}], 
     "]"}], "[", "t", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5977317427255716`*^9, 3.5977317893472385`*^9}, {
   3.597731905438878*^9, 3.5977319912867885`*^9}, 3.5977320952957373`*^9, {
   3.5977321323428564`*^9, 3.5977321664548073`*^9}, {3.597732200380748*^9, 
   3.59773233731258*^9}, {3.5977323700014496`*^9, 3.5977323837522364`*^9}, 
   3.5977615795014133`*^9, {3.5977631817334275`*^9, 3.597763189424241*^9}, 
   3.597814977779808*^9, {3.5978168288634596`*^9, 3.5978168347290697`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     SuperscriptBox["s1", "\[Prime]",
      MultilineFunction->None], "[", "t", "]"}], "-", 
    RowBox[{
     RowBox[{"Tan", "[", 
      RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"s2", "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["u", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}]}], "+", 
       RowBox[{
        RowBox[{"s1", "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["v", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}]}]}], ")"}]}]}], ",", 
   RowBox[{
    RowBox[{
     SuperscriptBox["s2", "\[Prime]",
      MultilineFunction->None], "[", "t", "]"}], "+", 
    RowBox[{
     RowBox[{"Cos", "[", 
      RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
     RowBox[{"s1", "[", "t", "]"}], " ", 
     RowBox[{"Sin", "[", 
      RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox["u", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.626072652335522*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Gamma]", "[", "t_", "]"}], " ", ":=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}]}], ";", 
  RowBox[{
   RowBox[{"\[Xi]", "[", "t_", "]"}], " ", ":=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"s1", "[", "t", "]"}], ",", 
     RowBox[{"s2", "[", "t", "]"}]}], "}"}]}], ";", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"DD1", "[", 
     RowBox[{"sf", ",", "\[Gamma]", ",", "\[Xi]"}], "]"}], "[", "t", "]"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5977317427255716`*^9, 3.5977317893472385`*^9}, {
   3.597731905438878*^9, 3.5977319912867885`*^9}, 3.5977320952957373`*^9, {
   3.5977321323428564`*^9, 3.5977321664548073`*^9}, {3.597732200380748*^9, 
   3.59773233731258*^9}, {3.5977323700014496`*^9, 3.5977323837522364`*^9}, 
   3.5977615795014133`*^9, {3.5977631817334275`*^9, 3.597763189424241*^9}, 
   3.597814977779808*^9, {3.5978153495284615`*^9, 3.5978153652688885`*^9}, 
   3.5978169355832467`*^9, 3.5978170612258673`*^9, 3.5978210148724117`*^9, 
   3.5978210893157425`*^9, 3.6171725651055365`*^9, 3.6171730417487736`*^9, 
   3.6245411324262357`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["s1", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"u", "[", "t", "]"}], ",", 
             RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"]}], ")"}]}], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"u", "[", "t", "]"}], ",", 
             RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"u", "[", "t", "]"}], ",", 
             RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"]}], ")"}]}]}], 
        ")"}]}], "+", 
      RowBox[{
       RowBox[{"s2", "[", "t", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["v", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"u", "[", "t", "]"}], ",", 
                RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"u", "[", "t", "]"}], ",", 
                RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"u", "[", "t", "]"}], ",", 
                RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}]}], 
              ")"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox["u", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"u", "[", "t", "]"}], ",", 
                RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"u", "[", "t", "]"}], ",", 
                RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"u", "[", "t", "]"}], ",", 
                RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}]}], 
              ")"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"s1", "[", "t", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["v", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"u", "[", "t", "]"}], ",", 
                RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"u", "[", "t", "]"}], ",", 
                RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"u", "[", "t", "]"}], ",", 
                RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}]}], 
              ")"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox["u", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"u", "[", "t", "]"}], ",", 
                RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"u", "[", "t", "]"}], ",", 
                RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"u", "[", "t", "]"}], ",", 
                RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}]}], 
              ")"}]}]}], ")"}]}]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"u", "[", "t", "]"}], ",", 
          RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"]}], ")"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{
         RowBox[{"u", "[", "t", "]"}], ",", 
         RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{
         RowBox[{"u", "[", "t", "]"}], ",", 
         RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{
         RowBox[{"u", "[", "t", "]"}], ",", 
         RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{
         RowBox[{"u", "[", "t", "]"}], ",", 
         RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{
         RowBox[{"u", "[", "t", "]"}], ",", 
         RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{
         RowBox[{"u", "[", "t", "]"}], ",", 
         RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"u", "[", "t", "]"}], ",", 
          RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"u", "[", "t", "]"}], ",", 
          RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"]}], ")"}]}]}], ")"}]}],
    ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["s2", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "3", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"u", "[", "t", "]"}], ",", 
             RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"]}], ")"}]}], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "2", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"u", "[", "t", "]"}], ",", 
             RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"f", "[", "1", "]"}], 
             TagBox[
              RowBox[{"(", 
               RowBox[{"0", ",", "1"}], ")"}],
              Derivative],
             MultilineFunction->None], "[", 
            RowBox[{
             RowBox[{"u", "[", "t", "]"}], ",", 
             RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"]}], ")"}]}]}], 
        ")"}]}], "+", 
      RowBox[{
       RowBox[{"s2", "[", "t", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["v", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], "+", 
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"]}], ")"}]}], 
               "-", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "2", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{
                   RowBox[{"u", "[", "t", "]"}], ",", 
                   RowBox[{"v", "[", "t", "]"}]}], "]"}]}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], "+", 
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"]}], ")"}]}]}], 
              ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "2", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{
                   RowBox[{"u", "[", "t", "]"}], ",", 
                   RowBox[{"v", "[", "t", "]"}]}], "]"}]}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "-", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], "+", 
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"]}], ")"}]}]}], 
              ")"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox["u", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "3", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{
                   RowBox[{"u", "[", "t", "]"}], ",", 
                   RowBox[{"v", "[", "t", "]"}]}], "]"}]}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], "+", 
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"]}], ")"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "2", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{
                   RowBox[{"u", "[", "t", "]"}], ",", 
                   RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "3", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{
                   RowBox[{"u", "[", "t", "]"}], ",", 
                   RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "-", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "-", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{
                   RowBox[{"u", "[", "t", "]"}], ",", 
                   RowBox[{"v", "[", "t", "]"}]}], "]"}]}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{
                   RowBox[{"u", "[", "t", "]"}], ",", 
                   RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "-", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}]}], 
              ")"}]}]}], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"s1", "[", "t", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["v", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "3", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{
                   RowBox[{"u", "[", "t", "]"}], ",", 
                   RowBox[{"v", "[", "t", "]"}]}], "]"}]}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], "+", 
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"]}], ")"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "2", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{
                   RowBox[{"u", "[", "t", "]"}], ",", 
                   RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "3", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{
                   RowBox[{"u", "[", "t", "]"}], ",", 
                   RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "-", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "-", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{
                   RowBox[{"u", "[", "t", "]"}], ",", 
                   RowBox[{"v", "[", "t", "]"}]}], "]"}]}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{
                   RowBox[{"u", "[", "t", "]"}], ",", 
                   RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "-", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}]}], 
              ")"}]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox["u", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "3", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{
                   RowBox[{"u", "[", "t", "]"}], ",", 
                   RowBox[{"v", "[", "t", "]"}]}], "]"}]}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], "+", 
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"]}], ")"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "2", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{
                   RowBox[{"u", "[", "t", "]"}], ",", 
                   RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "3", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{
                   RowBox[{"u", "[", "t", "]"}], ",", 
                   RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "-", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "-", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"u", "[", "t", "]"}], ",", 
               RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{
                   RowBox[{"u", "[", "t", "]"}], ",", 
                   RowBox[{"v", "[", "t", "]"}]}], "]"}]}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{
                   RowBox[{"u", "[", "t", "]"}], ",", 
                   RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"u", "[", "t", "]"}], ",", 
                  RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "-", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"u", "[", "t", "]"}], ",", 
                    RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}]}], 
              ")"}]}]}], ")"}]}]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "3", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"u", "[", "t", "]"}], ",", 
          RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"]}], ")"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{
         RowBox[{"u", "[", "t", "]"}], ",", 
         RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{
         RowBox[{"u", "[", "t", "]"}], ",", 
         RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "1", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{
         RowBox[{"u", "[", "t", "]"}], ",", 
         RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "3", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{
         RowBox[{"u", "[", "t", "]"}], ",", 
         RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{
         RowBox[{"u", "[", "t", "]"}], ",", 
         RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"f", "[", "2", "]"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{
         RowBox[{"u", "[", "t", "]"}], ",", 
         RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "2", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"u", "[", "t", "]"}], ",", 
          RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "1", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"f", "[", "1", "]"}], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{
          RowBox[{"u", "[", "t", "]"}], ",", 
          RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "2", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"f", "[", "3", "]"}], 
            TagBox[
             RowBox[{"(", 
              RowBox[{"1", ",", "0"}], ")"}],
             Derivative],
            MultilineFunction->None], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], "2"]}], ")"}]}]}], 
     ")"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.626072693862795*^9}]
}, Closed]],

Cell[TextData[{
 StyleBox["Example.",
  FontColor->RGBColor[0, 0, 1]],
 " Euclidean plane"
}], "Text",
 CellChangeTimes->{{3.624541561504989*^9, 3.6245415878066354`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Gamma]", "[", "t_", "]"}], " ", ":=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}]}], ";", 
  RowBox[{
   RowBox[{"\[Xi]", "[", "t_", "]"}], " ", ":=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"s1", "[", "t", "]"}], ",", 
     RowBox[{"s2", "[", "t", "]"}]}], "}"}]}], ";", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"DD1", "[", 
     RowBox[{"identity", ",", "\[Gamma]", ",", "\[Xi]"}], "]"}], "[", "t", 
    "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5977317427255716`*^9, 3.5977317893472385`*^9}, {
   3.597731905438878*^9, 3.5977319912867885`*^9}, 3.5977320952957373`*^9, {
   3.5977321323428564`*^9, 3.5977321664548073`*^9}, {3.597732200380748*^9, 
   3.59773233731258*^9}, {3.5977323700014496`*^9, 3.5977323837522364`*^9}, 
   3.5977615795014133`*^9, {3.5977631817334275`*^9, 3.597763189424241*^9}, 
   3.597814977779808*^9, {3.5978153495284615`*^9, 3.5978153652688885`*^9}, {
   3.5978154316938057`*^9, 3.5978154432534256`*^9}, 3.597816947501668*^9, {
   3.624541491679267*^9, 3.624541495501273*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SuperscriptBox["s1", "\[Prime]",
     MultilineFunction->None], "[", "t", "]"}], ",", 
   RowBox[{
    SuperscriptBox["s2", "\[Prime]",
     MultilineFunction->None], "[", "t", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6260727129260287`*^9}]
}, Closed]],

Cell["\<\
In the Euclidean plane with Cartesian coordinates the absolute derivative \
coincides with the standard derivative.
\[FilledRectangle]\
\>", "Text",
 CellChangeTimes->{{3.597824599532421*^9, 3.5978246899355917`*^9}}],

Cell[TextData[{
 StyleBox["Definition",
  FontColor->RGBColor[0, 0, 1]],
 ". The vector vw[t] is said to be",
 StyleBox[" parallel displaced",
  FontColor->RGBColor[0, 0, 1]],
 " along the curve \[Gamma][t], if its absolute derivative is zero: DD1[ff,\
\[Gamma],vw][t] \[Congruent] 0."
}], "Text",
 CellChangeTimes->{{3.597824912191304*^9, 3.5978250944877305`*^9}, {
  3.5978251325839095`*^9, 3.5978253947519045`*^9}, {3.613282563068982*^9, 
  3.6132825845572114`*^9}}],

Cell[TextData[{
 StyleBox["Proposition",
  FontColor->RGBColor[0, 0, 1]],
 ". Let \[Gamma][t0, t1] be the arc of the curve \[Gamma] between \
\[Gamma][t0], \[Gamma][t1]. Then for any tangent vector \n\[Xi]  \[Element] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", 
    RowBox[{"ff", "[", 
     RowBox[{"\[Gamma]", "[", "t0", "]"}], "]"}]], TraditionalForm]]],
 " with coordinates {s1, s2} there exists a uniquely defined vector function \
\[Xi][t], parallel displaced along \[Gamma][t], with  \[Xi][t0] == \[Xi].  \
The map"
}], "Text",
 CellChangeTimes->{{3.5978254006782436`*^9, 3.597825430982977*^9}, {
   3.5978254653109407`*^9, 3.5978256295313334`*^9}, {3.597827809533633*^9, 
   3.5978281050655365`*^9}, {3.597828142609684*^9, 3.5978284203455696`*^9}, 
   3.597848111297683*^9, {3.5978482153030653`*^9, 3.5978482290622897`*^9}, {
   3.597909746031082*^9, 3.597909760134889*^9}, {3.6132826504209785`*^9, 
   3.6132826656368484`*^9}, 3.617441476236147*^9}],

Cell[TextData[{
 "\[Gamma][t0, t1]:   \[Xi]  \[Element] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", 
    RowBox[{"ff", "[", 
     RowBox[{"\[Gamma]", "[", "t0", "]"}], "]"}]], TraditionalForm]]],
 "  \[RightTeeArrow] \[Xi][t1] \[Element] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", 
    RowBox[{"ff", "[", 
     RowBox[{"\[Gamma]", "[", "t1", "]"}], "]"}]], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.5978254006782436`*^9, 3.597825430982977*^9}, {
  3.5978254653109407`*^9, 3.5978256295313334`*^9}, {3.597827809533633*^9, 
  3.5978281050655365`*^9}, {3.597828142609684*^9, 3.5978284203455696`*^9}, {
  3.597848111297683*^9, 3.5978481458205433`*^9}, {3.5978491789011583`*^9, 
  3.597849186342371*^9}}],

Cell["\<\
is an isometry of Euclidean vector spaces, depending on the curve \[Gamma].\
\>", "Text",
 CellChangeTimes->{{3.5978254006782436`*^9, 3.597825430982977*^9}, {
  3.5978254653109407`*^9, 3.5978256295313334`*^9}, {3.597827809533633*^9, 
  3.5978281050655365`*^9}, {3.597828142609684*^9, 3.5978284203455696`*^9}, {
  3.597848111297683*^9, 3.59784815508696*^9}}],

Cell[TextData[{
 StyleBox["Proof",
  FontColor->RGBColor[0, 0, 1]],
 ". Analyzing the definition or looking at the examples we see that the \
equation "
}], "Text",
 CellChangeTimes->{{3.5978482749731703`*^9, 3.5978482821647835`*^9}, {
  3.597848335095676*^9, 3.5978483637373266`*^9}, {3.5978484072926025`*^9, 
  3.597848416122218*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DD1", "[", 
    RowBox[{"ff", ",", "\[Gamma]", ",", "\[Xi]"}], "]"}], "[", "t", "]"}], 
  " ", "\[Equal]", "  ", "0"}]], "Input",
 CellChangeTimes->{{3.597848445029069*^9, 3.5978484840291376`*^9}}],

Cell["\<\
Is a homogeneous system of ordinary linear differential equations of first \
order for the coordinates s[i][t] of \[Xi][t}. Therefore the solutions are \
uniquely defined by the initial conditions. Since\
\>", "Text",
 CellChangeTimes->{{3.597848493233154*^9, 3.597848651495432*^9}, {
  3.597848698732315*^9, 3.597848732397174*^9}, {3.618824036118561*^9, 
  3.618824043533985*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DD1", "[", 
    RowBox[{"\[Xi]", "+", "\[Eta]"}], "]"}], "\[Equal]", 
   RowBox[{
    RowBox[{"DD1", "[", "\[Xi]", "]"}], "+", 
    RowBox[{
     RowBox[{"DD1", "[", "\[Eta]", "]"}], " ", 
     StyleBox["and",
      FontColor->GrayLevel[0]], " ", 
     RowBox[{"DD1", "[", 
      RowBox[{"a", " ", "\[Xi]"}], "]"}]}]}], "\[Equal]", 
   RowBox[{"a", " ", 
    RowBox[{"DD1", "[", "\[Xi]", "]"}], " ", 
    StyleBox["for",
     FontColor->GrayLevel[0]], " ", "a"}]}], "\[Element]", 
  "\[DoubleStruckCapitalR]"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.597848753691211*^9, 3.5978489453375483`*^9}, {
  3.5978490043368516`*^9, 3.59784902028008*^9}}],

Cell[TextData[{
 "the map \[Gamma][t0, t1] defined above is a linear isomorphism. First we \
consider an  \n\n",
 StyleBox["Example",
  FontColor->RGBColor[0, 0, 1]],
 ". With"
}], "Text",
 CellChangeTimes->{{3.5978492407084665`*^9, 3.597849289240152*^9}, {
  3.597903092985965*^9, 3.597903098055974*^9}, {3.5979033790748672`*^9, 
  3.5979034785094423`*^9}, {3.597904186319085*^9, 3.597904199182821*^9}, {
  3.6260729171615877`*^9, 3.6260729195327916`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Gamma]", "[", "t_", "]"}], " ", ":=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}]}], ";", 
  RowBox[{
   RowBox[{"\[Xi]", "[", "t_", "]"}], " ", ":=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"s1", "[", "t", "]"}], ",", 
     RowBox[{"s2", "[", "t", "]"}]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{
  3.5978494117939672`*^9, 3.597849528528972*^9, {3.5978500298826528`*^9, 
   3.597850040709072*^9}, 3.5978504492269893`*^9}],

Cell["we calculate", "Text",
 CellChangeTimes->{3.5979035304575334`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"DD1", "[", 
    RowBox[{
     RowBox[{"sphero", "[", "r", "]"}], ",", "\[Gamma]", ",", "\[Xi]"}], 
    "]"}], "[", "t", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.597850718343062*^9, 3.59785075125912*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     SuperscriptBox["s1", "\[Prime]",
      MultilineFunction->None], "[", "t", "]"}], "-", 
    RowBox[{
     RowBox[{"Tan", "[", 
      RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"s2", "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["u", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}]}], "+", 
       RowBox[{
        RowBox[{"s1", "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["v", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}]}]}], ")"}]}]}], ",", 
   RowBox[{
    RowBox[{
     SuperscriptBox["s2", "\[Prime]",
      MultilineFunction->None], "[", "t", "]"}], "+", 
    RowBox[{
     RowBox[{"Cos", "[", 
      RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
     RowBox[{"s1", "[", "t", "]"}], " ", 
     RowBox[{"Sin", "[", 
      RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox["u", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6260729382216244`*^9}]
}, Closed]],

Cell["Using this result we define the ReplaceAll rules", "Text",
 CellChangeTimes->{{3.597903833581666*^9, 3.597903862207716*^9}, {
  3.597905596638751*^9, 3.597905628239558*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"srule", " ", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["s1", "\[Prime]",
        MultilineFunction->None], "[", "t", "]"}], "\[Rule]", " ", 
      RowBox[{
       RowBox[{"Tan", "[", 
        RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"s2", "[", "t", "]"}], " ", 
          RowBox[{
           SuperscriptBox["u", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}]}], "+", 
         RowBox[{
          RowBox[{"s1", "[", "t", "]"}], " ", 
          RowBox[{
           SuperscriptBox["v", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}]}]}], ")"}]}]}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["s2", "\[Prime]",
        MultilineFunction->None], "[", "t", "]"}], "\[Rule]", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"Cos", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}]}], " ", 
       RowBox[{"s1", "[", "t", "]"}], " ", 
       RowBox[{"Sin", "[", 
        RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox["u", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}]}]}], "}"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.5979020770185804`*^9, 3.5979021128986435`*^9}, 
   3.597903879695347*^9}],

Cell["\<\
Now we get the stated result: The scalar square of a parallel transferred \
vector is constant:\
\>", "Text",
 CellChangeTimes->{{3.5979039146550083`*^9, 3.5979039775387187`*^9}, {
  3.59790405527059*^9, 3.5979040727025867`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{"FullSimplify", "[", 
      RowBox[{"Sum", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"rgT", "[", 
            RowBox[{"i", ",", "j"}], "]"}], "[", 
           RowBox[{"sphero", "[", "r", "]"}], "]"}], "[", 
          RowBox[{
           RowBox[{"u", "[", "t", "]"}], ",", 
           RowBox[{"v", "[", "t", "]"}]}], "]"}], "*", 
         RowBox[{
          RowBox[{"\[Xi]", "[", "t", "]"}], "[", 
          RowBox[{"[", "i", "]"}], "]"}], "*", 
         RowBox[{
          RowBox[{"\[Xi]", "[", "t", "]"}], "[", 
          RowBox[{"[", "j", "]"}], "]"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}], ",", "t"}], "]"}], "/.",
    "srule"}], "]"}]], "Input",
 CellChangeTimes->{{3.5978492968529654`*^9, 3.5978493357282333`*^9}, {
   3.597849486299698*^9, 3.597849492602109*^9}, {3.5978495746270533`*^9, 
   3.59784975499457*^9}, {3.597850066464717*^9, 3.5978502509506407`*^9}, {
   3.597850302680332*^9, 3.5978503672956457`*^9}, {3.59785081986804*^9, 
   3.5978508536888995`*^9}, {3.5978510248056*^9, 3.597851033338815*^9}, {
   3.597903144169655*^9, 3.5979031639816895`*^9}, {3.597903226725*^9, 
   3.5979032503122416`*^9}, 3.6186424061704826`*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.626072973181286*^9}]
}, Closed]],

Cell["\<\
\[FilledRectangle]
Now we consider the general case:\
\>", "Text",
 CellChangeTimes->{{3.597904117111127*^9, 3.597904155526324*^9}}],

Cell["\<\
We define the ReplaceAll rules, analyzing the definition of DD1 and using \
DD1[sf,\[Gamma],\[Xi]][t]== 0:\
\>", "Text",
 CellChangeTimes->{{3.597903833581666*^9, 3.597903862207716*^9}, 
   3.597904443750809*^9, {3.5979055757595563`*^9, 3.597905583951025*^9}, {
   3.597905758415004*^9, 3.5979057915028963`*^9}, {3.5979058467290554`*^9, 
   3.5979058754867*^9}}],

Cell[BoxData[
 RowBox[{"sfrule", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["s1", "\[Prime]",
        MultilineFunction->None], "[", "t", "]"}], "\[Rule]", " ", 
      RowBox[{"-", 
       RowBox[{"Sum", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalGamma]rgT", "[", 
             RowBox[{"1", ",", "j", ",", "k"}], "]"}], "[", "sf", "]"}], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], "*", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "t", "]"}], "[", 
           RowBox[{"[", "j", "]"}], "]"}], "*", 
          RowBox[{"D", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"\[Gamma]", "[", "t", "]"}], "[", 
             RowBox[{"[", "k", "]"}], "]"}], ",", "t"}], "]"}]}], ",", 
         RowBox[{"{", 
          RowBox[{"j", ",", "2"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"k", ",", "2"}], "}"}]}], "]"}]}]}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["s2", "\[Prime]",
        MultilineFunction->None], "[", "t", "]"}], "\[Rule]", 
      RowBox[{"-", 
       RowBox[{"Sum", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalGamma]rgT", "[", 
             RowBox[{"2", ",", "j", ",", "k"}], "]"}], "[", "sf", "]"}], "[", 
           RowBox[{
            RowBox[{"u", "[", "t", "]"}], ",", 
            RowBox[{"v", "[", "t", "]"}]}], "]"}], "*", 
          RowBox[{
           RowBox[{"\[Xi]", "[", "t", "]"}], "[", 
           RowBox[{"[", "j", "]"}], "]"}], "*", 
          RowBox[{"D", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"\[Gamma]", "[", "t", "]"}], "[", 
             RowBox[{"[", "k", "]"}], "]"}], ",", "t"}], "]"}]}], ",", 
         RowBox[{"{", 
          RowBox[{"j", ",", "2"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"k", ",", "2"}], "}"}]}], "]"}]}]}]}], "}"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5979020770185804`*^9, 3.5979021128986435`*^9}, 
   3.597903879695347*^9, {3.5979044647410097`*^9, 3.5979044846771502`*^9}, {
   3.597904582635753*^9, 3.5979045980226336`*^9}, {3.59790463887097*^9, 
   3.597904648016493*^9}, {3.5979047164724083`*^9, 3.5979047360215263`*^9}, {
   3.597904808967698*^9, 3.5979048424166117`*^9}, {3.5979049171378856`*^9, 
   3.5979049321717453`*^9}, {3.597904975754238*^9, 3.597904989111002*^9}, {
   3.597905042449053*^9, 3.5979051553695116`*^9}, {3.5979052400023527`*^9, 
   3.597905252930092*^9}, {3.597905312498499*^9, 3.597905345545389*^9}, {
   3.6245418616027164`*^9, 3.6245418728191357`*^9}}],

Cell[TextData[{
 "Now we get the stated result: ",
 StyleBox["The scalar square of a parallel transferred vector is constant:",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.5979039146550083`*^9, 3.5979039775387187`*^9}, {
  3.59790405527059*^9, 3.5979040727025867`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{"FullSimplify", "[", 
      RowBox[{"Sum", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"rgT", "[", 
            RowBox[{"i", ",", "j"}], "]"}], "[", "sf", "]"}], "[", 
          RowBox[{
           RowBox[{"u", "[", "t", "]"}], ",", 
           RowBox[{"v", "[", "t", "]"}]}], "]"}], "*", 
         RowBox[{
          RowBox[{"\[Xi]", "[", "t", "]"}], "[", 
          RowBox[{"[", "i", "]"}], "]"}], "*", 
         RowBox[{
          RowBox[{"\[Xi]", "[", "t", "]"}], "[", 
          RowBox[{"[", "j", "]"}], "]"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}], ",", "t"}], "]"}], "/.",
    "sfrule"}], "]"}]], "Input",
 CellChangeTimes->{{3.5978492968529654`*^9, 3.5978493357282333`*^9}, {
   3.597849486299698*^9, 3.597849492602109*^9}, {3.5978495746270533`*^9, 
   3.59784975499457*^9}, {3.597850066464717*^9, 3.5978502509506407`*^9}, {
   3.597850302680332*^9, 3.5978503672956457`*^9}, {3.59785081986804*^9, 
   3.5978508536888995`*^9}, {3.5978510248056*^9, 3.597851033338815*^9}, {
   3.597903144169655*^9, 3.5979031639816895`*^9}, {3.597903226725*^9, 
   3.5979032503122416`*^9}, {3.5979053910299907`*^9, 3.59790541252422*^9}, 
   3.618642567905733*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.626073022383772*^9}]
}, Closed]],

Cell["\<\
Thus the map \[Gamma][t0, t1] defined above is an isometry, and therefore an \
isomorphy of Euclidean vector spaces.  All the algebraic properties defined \
using the first fundamental form are preserved, Remember the general identity \
for bilinear symmetric biforms, e.g. the Dot product:\
\>", "Text",
 CellChangeTimes->{{3.5979059861190276`*^9, 3.5979059936304574`*^9}, {
  3.597906034262781*^9, 3.597906090607004*^9}, {3.5979061594869437`*^9, 
  3.59790626781514*^9}, {3.5979064815023623`*^9, 3.5979064905028768`*^9}}],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"vec", "[", "x", "]"}], ".", 
    RowBox[{"vec", "[", "y", "]"}]}], " ", "\[Equal]", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"vec", "[", "x", "]"}], "+", 
         RowBox[{"vec", "[", "y", "]"}]}], ")"}], ".", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"vec", "[", "x", "]"}], "+", 
         RowBox[{"vec", "[", "y", "]"}]}], ")"}]}], "-", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"vec", "[", "x", "]"}], "-", 
         RowBox[{"vec", "[", "y", "]"}]}], ")"}], ".", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"vec", "[", "x", "]"}], "-", 
         RowBox[{"vec", "[", "y", "]"}]}], ")"}]}]}], ")"}], "/", "4"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.59790628075488*^9, 3.5979063105735855`*^9}, {
  3.5979063426894226`*^9, 3.5979064649564157`*^9}}],

Cell[TextData[{
 "We conclude\n",
 StyleBox["Corollary.",
  FontColor->RGBColor[0, 0, 1]],
 " Norm, angle and determinant, i.e. the area of the parallelogram spanned by \
the vectors, are invariant under parallel displacements.\n\[FilledRectangle]\n\
This result is a special case of ",
 ButtonBox["Ricci\[CloseCurlyQuote]s Lemma, ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://cms.math.ca/cjm/v16/cjm1964v16.0549-0560.pdf"], None},
  ButtonNote->"http://cms.math.ca/cjm/v16/cjm1964v16.0549-0560.pdf"],
 "see also ",
 ButtonBox["[B-R] ",
  BaseStyle->"Hyperlink",
  ButtonData->"B-R"],
 ", p.154, ",
 ButtonBox["[Kr]",
  BaseStyle->"Hyperlink",
  ButtonData->"Kr"],
 ", p.272."
}], "Text",
 CellChangeTimes->{{3.5979066223184166`*^9, 3.5979066356951814`*^9}, {
   3.597906685975057*^9, 3.597906703455057*^9}, {3.5979067557670493`*^9, 
   3.5979067829666047`*^9}, {3.597906823142903*^9, 3.5979068499664373`*^9}, {
   3.5979068902067385`*^9, 3.5979069783297787`*^9}, {3.5979070671908617`*^9, 
   3.5979070893111267`*^9}, {3.5979075377137737`*^9, 3.597907537721774*^9}, 
   3.59790761091096*^9, {3.597909307655009*^9, 3.59790933652666*^9}, {
   3.597909476022639*^9, 3.597909526774542*^9}, {3.5979100570878735`*^9, 
   3.5979100798951783`*^9}, {3.5979101121640244`*^9, 3.597910112173024*^9}, {
   3.597929142213434*^9, 3.597929152244008*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["4.2.5. Geodesics as Auto-Parallel Curves", "Subsubsection",
 CellChangeTimes->{{3.595076945363204*^9, 3.5950769905720835`*^9}, 
   3.5953174793534184`*^9, {3.5953175263051033`*^9, 3.595317587064579*^9}, 
   3.5956008724322104`*^9, {3.5965213907316623`*^9, 3.5965214041788855`*^9}, {
   3.5972984401750546`*^9, 3.59729846072029*^9}, {3.5972984954771514`*^9, 
   3.5972985062723703`*^9}, {3.5973320886172123`*^9, 3.5973320905123205`*^9}, 
   3.5973952636966553`*^9, {3.597729800827932*^9, 3.59772986233884*^9}, {
   3.597910399537461*^9, 3.597910436118553*^9}, {3.5979116271266747`*^9, 
   3.5979116482228813`*^9}, 3.613284728941863*^9, 3.6132874017987423`*^9, 
   3.6171739408407526`*^9, 3.617195815150076*^9, 3.6188258784059334`*^9, 
   3.618830173270586*^9, 3.6245420369782248`*^9, 3.6245441638235598`*^9}],

Cell[TextData[{
 StyleBox["Definition",
  FontColor->RGBColor[0, 0, 1]],
 ". A curve \[Gamma][t] on an immersion or in a Riemannian manifold is said \
to be ",
 StyleBox["auto-parallel",
  FontColor->RGBColor[0, 0, 1]],
 " or more commonly, a ",
 StyleBox["geodesic",
  FontColor->RGBColor[0, 0, 1]],
 ", if its tangent vector is parallel displaced along the curve.\n\n",
 StyleBox["Remark",
  FontColor->RGBColor[0, 0, 1]],
 ". Since the norm  of the   tangent vector depends on the parameter \
representation, the condition of the definition fixes also the type of the \
parameter t of the curve. In Riemannian geometry it is natural to assume that \
t is the arc length of the curve; remember that this is equivalent to the \
fact that the tangent vector has constant norm 1."
}], "Text",
 CellChangeTimes->{{3.597911657441409*^9, 3.597911675247427*^9}, {
   3.597911706147195*^9, 3.59791200922353*^9}, {3.597928848436631*^9, 
   3.5979288904770355`*^9}, {3.597928956356804*^9, 3.5979290724444437`*^9}, {
   3.6132847851830797`*^9, 3.6132847983008304`*^9}, 3.6132848615014453`*^9, 
   3.617441713205701*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Gamma]", "[", "t_", "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"u", "[", "t", "]"}], ",", 
    RowBox[{"v", "[", "t", "]"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.5977318613653574`*^9, 3.597731863773495*^9}, {
   3.597736504464549*^9, 3.597736506142645*^9}, 3.597736550155163*^9}],

Cell[TextData[{
 StyleBox["Example",
  FontColor->RGBColor[0, 0, 1]],
 ". The Euclidean plane"
}], "Text",
 CellChangeTimes->{{3.5979304075968103`*^9, 3.5979304432118473`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"DD1", "[", 
    RowBox[{"identity", ",", "\[Gamma]", ",", 
     SuperscriptBox["\[Gamma]", "\[Prime]",
      MultilineFunction->None]}], "]"}], "[", "t", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5979292059430795`*^9, 3.5979292420701456`*^9}, {
  3.597929272325876*^9, 3.597929298654382*^9}, {3.597929354638584*^9, 
  3.597929492815488*^9}, {3.597930356399882*^9, 3.597930367070492*^9}, {
  3.624543465207533*^9, 3.6245434701371417`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SuperscriptBox["u", "\[Prime]\[Prime]",
     MultilineFunction->None], "[", "t", "]"}], ",", 
   RowBox[{
    SuperscriptBox["v", "\[Prime]\[Prime]",
     MultilineFunction->None], "[", "t", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6260715429707737`*^9, 3.6260742338039*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{"%", "\[Equal]", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0"}], "}"}]}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}], ",", "t"}], "]"}]], "Input",
 CellChangeTimes->{{3.597929563264517*^9, 3.597929568107794*^9}, {
   3.5979298729262285`*^9, 3.597929965494523*^9}, 3.614861862717145*^9, 
   3.6148646599580584`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], "\[Rule]", 
     RowBox[{
      RowBox[{"C", "[", "1", "]"}], "+", 
      RowBox[{"t", " ", 
       RowBox[{"C", "[", "2", "]"}]}]}]}], ",", 
    RowBox[{
     RowBox[{"v", "[", "t", "]"}], "\[Rule]", 
     RowBox[{
      RowBox[{"C", "[", "3", "]"}], "+", 
      RowBox[{"t", " ", 
       RowBox[{"C", "[", "4", "]"}]}]}]}]}], "}"}], "}"}]], "Output",
 CellChangeTimes->{3.626071543095574*^9, 3.6260742383123083`*^9}]
}, Closed]],

Cell["\<\
In the Euclidean plane the lines with natural affine parameter t are the \
geodesics. \[FilledRectangle]\
\>", "Text",
 CellChangeTimes->{{3.5979305555082703`*^9, 3.5979306105174165`*^9}, {
  3.597930665788578*^9, 3.597930668044707*^9}}],

Cell[TextData[{
 StyleBox["Example",
  FontColor->RGBColor[0, 0, 1]],
 ". The spheres."
}], "Text",
 CellChangeTimes->{{3.597930716372471*^9, 3.597930736028595*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"DD1", "[", 
    RowBox[{
     RowBox[{"sphero", "[", "r", "]"}], ",", "\[Gamma]", ",", 
     SuperscriptBox["\[Gamma]", "\[Prime]",
      MultilineFunction->None]}], "]"}], "[", "t", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5979292059430795`*^9, 3.5979292420701456`*^9}, {
  3.597929272325876*^9, 3.597929298654382*^9}, {3.597929354638584*^9, 
  3.597929492815488*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"-", "2"}], " ", 
     RowBox[{"Tan", "[", 
      RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox["u", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}], " ", 
     RowBox[{
      SuperscriptBox["v", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}]}], "+", 
    RowBox[{
     SuperscriptBox["u", "\[Prime]\[Prime]",
      MultilineFunction->None], "[", "t", "]"}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"Cos", "[", 
      RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
     RowBox[{"Sin", "[", 
      RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["u", "\[Prime]",
        MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
    RowBox[{
     SuperscriptBox["v", "\[Prime]\[Prime]",
      MultilineFunction->None], "[", "t", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6260715432671747`*^9, 3.626074260089946*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{"%", "\[Equal]", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0"}], "}"}]}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}], ",", "t"}], "]"}]], "Input",
 CellChangeTimes->{{3.597929563264517*^9, 3.597929568107794*^9}, {
   3.5979298729262285`*^9, 3.597929965494523*^9}, 3.614861862732745*^9, 
   3.614864659973658*^9}],

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", "2"}], " ", 
        RowBox[{"Tan", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox["u", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["v", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}]}], "+", 
       RowBox[{
        SuperscriptBox["u", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        RowBox[{"Sin", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox["v", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}]}], "}"}], "\[Equal]", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0"}], "}"}]}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}], ",", "t"}], "]"}]], "Output",
 CellChangeTimes->{3.6260715433139744`*^9, 3.6260742636467524`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", "2"}], " ", 
        RowBox[{"Tan", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox["u", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["v", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}]}], "+", 
       RowBox[{
        SuperscriptBox["u", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        RowBox[{"Sin", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox["v", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}]}], "}"}], "\[Equal]", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0"}], "}"}]}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}], ",", "t"}], "]"}]], "Input",
 CellChangeTimes->{{3.614861862732745*^9, 3.6148618627483454`*^9}, 
   3.6148646599892583`*^9}],

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", "2"}], " ", 
        RowBox[{"Tan", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox["u", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["v", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}]}], "+", 
       RowBox[{
        SuperscriptBox["u", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        RowBox[{"Sin", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox["v", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}]}], "}"}], "\[Equal]", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0"}], "}"}]}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}], ",", "t"}], "]"}]], "Output",
 CellChangeTimes->{3.6260715433295746`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", "2"}], " ", 
        RowBox[{"Tan", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox["u", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["v", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}]}], "+", 
       RowBox[{
        SuperscriptBox["u", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        RowBox[{"Sin", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox["v", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", " ", "0"}], 
     ",", 
     RowBox[{
      RowBox[{"u", "[", "0", "]"}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{"v", "[", "0", "]"}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["u", "\[Prime]",
        MultilineFunction->None], "[", "0", "]"}], " ", "\[Equal]", " ", 
      "1"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["v", "\[Prime]",
        MultilineFunction->None], "[", "0", "]"}], "\[Equal]", "0"}]}], "}"}],
    ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}], ",", "t"}], "]"}]], "Input",
 CellChangeTimes->{{3.597930017299486*^9, 3.597930054979642*^9}, {
   3.5979301545233355`*^9, 3.5979301739564466`*^9}, 3.5989403761220384`*^9, 
   3.614861862763945*^9, 3.6148646599892583`*^9}],

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", "2"}], " ", 
        RowBox[{"Tan", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox["u", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["v", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}]}], "+", 
       RowBox[{
        SuperscriptBox["u", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        RowBox[{"Sin", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox["v", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{"u", "[", "0", "]"}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{"v", "[", "0", "]"}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["u", "\[Prime]",
        MultilineFunction->None], "[", "0", "]"}], "\[Equal]", "1"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["v", "\[Prime]",
        MultilineFunction->None], "[", "0", "]"}], "\[Equal]", "0"}]}], "}"}],
    ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}], ",", "t"}], "]"}]], "Output",
 CellChangeTimes->{3.6260715433607745`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", "2"}], " ", 
        RowBox[{"Tan", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox["u", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["v", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}]}], "+", 
       RowBox[{
        SuperscriptBox["u", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        RowBox[{"Sin", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox["v", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{"u", "[", "0", "]"}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{"v", "[", "0", "]"}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["u", "\[Prime]",
        MultilineFunction->None], "[", "0", "]"}], "\[Equal]", "1"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["v", "\[Prime]",
        MultilineFunction->None], "[", "0", "]"}], "\[Equal]", "0"}]}], "}"}],
    ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}], ",", "t"}], "]"}]], "Input",
 CellChangeTimes->{{3.614861862763945*^9, 3.6148618627795453`*^9}, 
   3.6148646600048585`*^9}],

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", "2"}], " ", 
        RowBox[{"Tan", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox["u", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["v", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}]}], "+", 
       RowBox[{
        SuperscriptBox["u", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        RowBox[{"Sin", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox["v", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{"u", "[", "0", "]"}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{"v", "[", "0", "]"}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["u", "\[Prime]",
        MultilineFunction->None], "[", "0", "]"}], "\[Equal]", "1"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["v", "\[Prime]",
        MultilineFunction->None], "[", "0", "]"}], "\[Equal]", "0"}]}], "}"}],
    ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}], ",", "t"}], "]"}]], "Output",
 CellChangeTimes->{3.6260715433763742`*^9}]
}, Closed]],

Cell["DSolve cannot solve these equations, NDSolve can:", "Text",
 CellChangeTimes->{{3.598011256632723*^9, 3.5980113116540194`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NDSolve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", "2"}], " ", 
        RowBox[{"Tan", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox["u", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}], " ", 
        RowBox[{
         SuperscriptBox["v", "\[Prime]",
          MultilineFunction->None], "[", "t", "]"}]}], "+", 
       RowBox[{
        SuperscriptBox["u", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        RowBox[{"Sin", "[", 
         RowBox[{"v", "[", "t", "]"}], "]"}], " ", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox["v", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", " ", "0"}], 
     ",", 
     RowBox[{
      RowBox[{"u", "[", "0", "]"}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{"v", "[", "0", "]"}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["u", "\[Prime]",
        MultilineFunction->None], "[", "0", "]"}], " ", "\[Equal]", " ", 
      "1"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["v", "\[Prime]",
        MultilineFunction->None], "[", "0", "]"}], "\[Equal]", "0"}]}], "}"}],
    ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.597930017299486*^9, 3.597930054979642*^9}, {
   3.5979301545233355`*^9, 3.5979302685808587`*^9}, 3.5980094359535255`*^9, {
   3.5980104154168453`*^9, 3.598010433559677*^9}, 3.598010516442623*^9, {
   3.598011025658717*^9, 3.598011037826739*^9}, {3.59894033925093*^9, 
   3.5989403593400784`*^9}, 3.6148618627795453`*^9, 3.614864660020458*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], "\[Rule]", 
     RowBox[{
      TagBox[
       RowBox[{"InterpolatingFunction", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"{", 
           RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
         ",", "\<\"<>\"\>"}], "]"}],
       False,
       Editable->False], "[", "t", "]"}]}], ",", 
    RowBox[{
     RowBox[{"v", "[", "t", "]"}], "\[Rule]", 
     RowBox[{
      TagBox[
       RowBox[{"InterpolatingFunction", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"{", 
           RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
         ",", "\<\"<>\"\>"}], "]"}],
       False,
       Editable->False], "[", "t", "]"}]}]}], "}"}], "}"}]], "Output",
 CellChangeTimes->{3.6260715435947747`*^9, 3.626074354470112*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"fu", "[", "t_", "]"}], " ", "=", 
  RowBox[{
   RowBox[{"u", "[", "t", "]"}], "/.", 
   RowBox[{
    RowBox[{"Flatten", "[", "%", "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5980105673455124`*^9, 3.598010700881747*^9}, {
  3.5980110599163775`*^9, 3.59801106141398*^9}}],

Cell[BoxData[
 RowBox[{
  TagBox[
   RowBox[{"InterpolatingFunction", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
     ",", "\<\"<>\"\>"}], "]"}],
   False,
   Editable->False], "[", "t", "]"}]], "Output",
 CellChangeTimes->{3.6260715436259747`*^9, 3.626074362550926*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"gu", "[", "t_", "]"}], " ", "=", 
  RowBox[{
   RowBox[{"v", "[", "t", "]"}], "/.", 
   RowBox[{
    RowBox[{"Flatten", "[", "%%", "]"}], "[", 
    RowBox[{"[", "2", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5980105673455124`*^9, 3.5980107410050173`*^9}, {
  3.598011081772016*^9, 3.598011083410019*^9}}],

Cell[BoxData[
 RowBox[{
  TagBox[
   RowBox[{"InterpolatingFunction", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
     ",", "\<\"<>\"\>"}], "]"}],
   False,
   Editable->False], "[", "t", "]"}]], "Output",
 CellChangeTimes->{3.626071543657175*^9, 3.626074366934534*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"r", "=", "1"}], ";", 
  RowBox[{"ParametricPlot3D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"sphero", "[", "r", "]"}], "[", 
     RowBox[{
      RowBox[{"fu", "[", "t", "]"}], ",", 
      RowBox[{"gu", "[", "t", "]"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"t", ",", "0", ",", 
      RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.59793530857513*^9, 3.597935328999298*^9}, {
   3.597935372503786*^9, 3.597935416552306*^9}, 3.5979361503762784`*^9, {
   3.5980107984599185`*^9, 3.5980108206743574`*^9}, {3.5980108954452887`*^9, 
   3.5980109588282003`*^9}, {3.5980111031908536`*^9, 3.5980111599125533`*^9}}],

Cell[BoxData[
 Graphics3DBox[{{}, {}, 
   {Hue[0.67, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxN2Xc41t//B3BS0bIrkUqJkkqRkuogkhUNKSEjRRlJicjM3rPIiEpW9l7H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     "]]}},
  Axes->True,
  Method->{},
  PlotRange->{{-0.9999998830731719, 0.9999999999999918}, {-0.9999998592131705,
    0.9999998782112116}, {-1., 1.}},
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.626071543688375*^9, 3.6260743713025417`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"equat1", "=", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.618826674612474*^9, 3.618826692702509*^9}}],

Cell[TextData[{
 StyleBox["T",
  FontSlant->"Italic"],
 "he spherical geodesics are the great circles,e.g. the equator"
}], "Text",
 CellChangeTimes->{{3.5979308517002115`*^9, 3.597930938316166*^9}, {
  3.59793601385347*^9, 3.5979360207808657`*^9}, {3.598011469978698*^9, 
  3.598011482271519*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"u", "[", "t_", "]"}], ":=", "t"}], ";", 
  RowBox[{
   RowBox[{"v", "[", "t_", "]"}], ":=", "0"}], ";", 
  RowBox[{"\[Gamma]", "[", "t", "]"}]}]], "Input",
 CellChangeTimes->{{3.597936287551124*^9, 3.5979362931424437`*^9}, {
  3.626074872047821*^9, 3.6260748756514273`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"t", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.6260715437039747`*^9, 3.62607439902379*^9, 
  3.6260748866026464`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"r", "=", "1"}], ";", 
  RowBox[{
   RowBox[{"u", "[", "s_", "]"}], ":=", "t"}], ";", 
  RowBox[{
   RowBox[{"v", "[", "t_", "]"}], ":=", "0"}], ";", 
  RowBox[{"ParametricPlot3D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"sphero", "[", "r", "]"}], "[", 
     RowBox[{
      RowBox[{"u", "[", "t", "]"}], ",", 
      RowBox[{"v", "[", "t", "]"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"t", ",", "0", ",", 
      RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
    RowBox[{"PlotStyle", " ", "\[Rule]", " ", "Red"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.59793530857513*^9, 3.597935328999298*^9}, {
   3.597935372503786*^9, 3.597935416552306*^9}, 3.5979361503762784`*^9, {
   3.618826730962697*^9, 3.618826739615192*^9}, {3.62607489978467*^9, 
   3.6260749054786797`*^9}}],

Cell[BoxData[
 Graphics3DBox[{{}, {}, 
   {RGBColor[1, 0, 0], Line3DBox[CompressedData["
1:eJxN2Xc4V+//B3BSoSGrEqmUpBKSROtGSlY0UEJGipIRiUghstcbLaNUsmf2
uM0oe2fv/fY+ycr+ne91/T7nPu9/XNfjOtd5H+d+3a/7+brefAbm14zW0dHR
qdHT0THgfzPW/vfBwFzBppqBk07n6f7/s15iE88y7m/SW6d3U+6A/zwz85Hp
H9wtg7/LNQWaEx6mfWymE/fzb0+IMic+JTyrTyW1CPeXMUXPIgNfEH5irDkq
HHePzv2b3+e6EP5gsbreCvf9Fk6buxNeEy5TL1W5EXdep49V1wP9CL+zt+Jd
6SoGvM2b62SKAwg/6r4U/gz3DrppY/VcCuEClA8qtSsYcJ5QyniU8IZwsy2i
53mWMSBxcesN9YAIwktNFErfL2GgNmk6dE/ZR8Kb56uc2HFn3Mhp71v0ifAh
kXefxxcwYEcRY9DP+Uy43KCO27l5/Htd6b1+xX0jPMPPB3v0BwMn4WK+nU8S
4V9yF9YsMAyM3dSoLpdLJlytr0zRlIaBG1p7ftQUIz9Z8dxfiYqBr+9UtOsK
UggXSWgciB/BAJT+VBiekUa4Vo37xGInBnRCXr9V+ZpJ+Hm34skfRRiQ3B4j
x+sI0fX2Cw++QQzIHp7hX15XSLiihmWjYz4GPrYe5Bi6hFwyINyYPRsDzDd8
B6NrkKtd7EyvSsLAsuDombmuIsK74+usfD9gYPDS1rKQpRLC9/KOlCqZ4tcn
KzbfF6sgnLfrxMPevzTgmpd/cdPFOsK5X4dvG06ZBJlPS8Tz2loJ39N/XuvK
eioQiNtTe5naTXj/r21qJ7PHwOBw260ypgHCUxgU/RyERgAfYyy9Itsw4bpL
GVuL1w2CRavyk3H/RlE9p17Yv+jTC+J3jNa8MZgg3IoybLeNvx24qcM5x5OT
hEuMORtr8TcAE2eTnoUkGuGh9INWFRsaQHfqbsNn35FP+Mszio3UA5tT1QrT
Wcj/mjwanI+pB76hWWdqipAPv2lSVhWpBz3yByiCjcgPbnvreEKqDjCv1POZ
zSD3TeyGW4OqgERcaOLJ4xjhF47qGI6pV4GeyjT/QnHkV+gBXcHOKuBMv5h+
QRL57Tuthtc+VIKsunc/xADySRUj58ORvwC957nRNGXkM+owMjyxAtCoXNYy
90j3n5eYYi0pBbdFzJVeUZADTuege66lYEoh8ihzCHK2wLnrmfKl4DKVztXt
LXK+F4IGilUlYJJj4OODMOS9jh6aks3FYPbltke/o5BD9kh36eFC8EMxsEMh
C/nxONO3DZtygPhehqCXzcgdHj5kdPTNBkeNT4dwtCLXzjG6c4A9G2RSeNdF
/CY9J1eLtPauLJD8zGJ7RAdyd+80Rs9DGaDwsOk+uT7kTCG73mOyqeC7RIUj
3wTy5O5PL+R/pACXttfTp6nIC9J6BD8opIAdf0arlCeRKztdOyehlgxSZb3j
b2HIVThVtZ/nxIM2nygurmnkjkGLRkaTX8DV5mDrQwvIjW0ouWbdn4H3+j61
JrIfVXF8XBsJuqJbxW0XkT/nP2X/IOUjEKGxcXxdQr5T7aW8wvhbsBJ9lZ+y
gjy2R+ceR1UIuMLnKPWP5IVsbPtaE4LAcraQ+o1V5NmXHpaqWPoDCZu4iCmS
ayhGFDBEeYM1O9UasTXk11mZJgxN3EGCLi/DQ5I3egfV8/W+AF/PLToWkpwy
qXey8N5TMH92qqyL5KY7zrMltz8CFw71s/8lueBNt6TE2lsgVrHcaIXkKkm7
13EKScMZu7CCNZKfP24r0btyFwZvucu7QPJfOValWv8ew7ju3c5jJPfn1uQp
tLSHy2pV1DqSfzoc2+M76wTnLtvpJJHcs2nym6SYG0wSPdb0iuTju0QlOs08
Yc/ctNpVkse0105tsA6Any3mtGtJ73PVO5x3jvoOFt9cV3SHtC41dedt+DaH
wvAGsweTy8hZGP7+kTscDv0uRnM/JvlZlr1zWWKfoNICi78eqR5M+rTiY8y+
Qut89sb6f8hn8wucNYcS4LFPZkMb/pLW911T2KaZRBg3b8C2fgr5NUYGVy36
ZKifai+/QKrzEshwfX5bCsTELzdWkPbFyJ4g1TmhNFijxPN4eRR5x23x96NG
mVA1SIMnowv5ad9Tuv0DmXDi0WMR6U7kX1cdlVv0s2A4xcG+uB258LlaoXid
bJga0JCXROoPm6FtxBb1XHhbXwQTqkc+6iz1wFUWQqrlKbPUYuR+21lo0KsE
Zi0abZ+MQM7eb5ZofbUS/nDfckbmAnLpidj9ZkGVMIvnycEgUh9+8S4v/k5r
Jfx4UZSr+wzy1xu80o/oVMG9TgPcmqQ+/+qtw0td42rY9SolrYsf+THzCkmh
57UwzmNe+Px65D8KBlUna+uhU8hA6o08dL6MnY4YTupvhsUb+h0OsCP/WTPe
BIO64G3O3oNWelTC9TUH/loJ9MO61+63qrXGCbc96/2I2WEI3jo1PcibOUJ4
02CejaHMKNwyx1hr0TNI+J9y/uK25nHYdc3osItZH+EMCx208CEqdKiTDzrh
2EF4SPdObEWHBtuHlT9VCDcRHv8hpknHkAY3tZqOX2NBbqn5dWu6MQ2+rlu0
a51sRPv6RKmVghUNsk/YBtTGI0845j3M7E6D7g/pohyOIB/vq+cdTaJBPSsn
zwY8F/znkWUB7KwLNJg7tjE3cTvKOV4ezz5prtAg4PSsPjRWi+onuk0zhB6D
Ap1CzGNRyLMy+CUWmDH4rsbLVm0/cqw04s0Nbgxe3VGukM5Vg+4jsJGqJ4VB
02pe1RGGKrS/CquuVFhgcM8WO8HHTeWE2wvFHMm3wuDpy0/V0oKRPylfE4u1
wWCrzc73NA3kzTMnfpg6YPBJSlbd9bYf6Nw8W9dPeY3BzvajTu1dZYQvVq/d
NgnFr5cs/zg8hPIhVOaiCRViMKtx+HbtIMqrzCentwsUY/BFYm/1jnvI97iX
aewsxWC2VI65xkgB4QG2hnz95Ri0+yNRVDCWT/jl6afVXLUYvHBmnOUULZfw
aT/OvZs6MRglkTjfN4vy89kPB+gWMQzWt2y5t0qH8vnwyrePf/5gcLbXLIuD
CeV8xvyIsJ4pDDLtGZU8wJKIzkHXjfFR0xj8xao/JMQdj3I14yBn2xwGj7x0
tpk6Hk34kOJhSuA8Bl1b6HwbT6H5ova3kqncPwxWvKDnSj4XRbiL60vhgAV8
fa2bHt9S+EL4LhNrWs0SBoWjt9Va6aF556CdIVBaxqC4cuVmvapwwhVKWcYh
7p/s+nnqW0MJl70opBy4gt8n9+i6CNpbwlOPmDL8wV3EoaZ8eSEE5YRYBs5L
qxgs981nUd8QTHjrv1HpDtzXA8u1OR403+lyc+hwr2Hwxy6D4y5XfFGudlBp
wM8zuDn6Hj+bkSc6jzo8PzrhvrjXovytvRvKYxWwPAZ34eU1Zp5AZ7Qvovvl
f+E+SieWefzqc8I7Myf2DeC+nt5RU8DxCcoJDM3XZnDPet/+7ajGQ8Jvmb/v
wnMAPFl9WJhJT5PwhVSpIjwHQIfT04fFUgD8z6u0UlcW/ve9fxiTVEMNCU8t
30Ch4s6kwiV++pMl4TtETjm14j4sevXWZWs7wifeK1Xk4N52jV08+vNLwhPG
FIxDcO9jLZMW9HlF+Ms6wTQJ3GdeLsRx6XkTvkd5n+4S/v5VBPOCS/j9Cc+1
FDHIwl3dmHfBZlcg4aafW57w4J6PVSrVrQshnOeJQ7UrXienyy4xSTeHER5y
8eaFrbgrTDooV/6MIJydf43dB6/DCU8Z+0M9Hwlf//JI4ZNFDBZKva8Jb4wk
PPJGmfoEXv8Vz5n81POiCG/ytQ28j+8vqTrdPUneiYSXHNxcfAffj1+H2izr
PiQRnrHn8swVfP/O7h02t1ZPJry7d06ffRL/fylTVMe7KYRfnRV6eGwUg/6s
LvOazmmEU2MtXb/j/UHkB+fMv7xMwhVlY/qYIQZLjk0z2ewoJHwqXlZoIA+D
BwNtlBpuIb+tst49LQeDJqJN04fDkP8ttX0ilYH3JckrRyoPFBEerLsm1R6P
9w2BNp5G0WLCmb660Vm8xeDZbSXOowqlhF8UPQSnjDFYx7Hv7NenFYQH/BSL
HDbCoO+6W3wt6chHzFgaGw3w99/M+I9uBvnLY2f43mhj0GKx6aycxU/CdU4t
vCpSxSBDTr/dC+NfhN/Mj90dJY738+8h1rmaVYS7fQ149f0fDcoqJcabc9UR
Lusqyrn7AQ3eOJTKf2moEdW/tkGGXy0VMvyKiBFv7yD8x2a1LR654zDITjS9
JKSPcJajtVoyeqPwqP/nm9q5g4SLx9qJshoPQREftgGBnyOEL/iyM7go9kPm
nxrOjTLjhKt72Sj8pHbB4xdldmsoUAl/W/y+FZ5ogYMHlFPkGGiEF88CNZ4d
LVA+L2g6lwl5RcTqPct/zfB5wL7BoyzIY49899qa3wzLezJ+zXIh7/DIddh4
qRmu+ot7HBdGHhxySZNeswk2HGOKl9dETnutv6bypAFWR2yjDn1GHl39uaZA
oRpejzS4EyeAoTo50TKymbEayvwU3Rh4BPlYRXuyWmkVvE9dV2gujFyloCa8
6HwV9JyQN2aVQB6tyL7uwolKOGA1azosh7x072psE/dP6G2irD2oh/xH+oU9
OYOlkHUL2MAXiPzVtxDD/Z9K4dib7btGg5DHvXpc+EqnFKqv2PNFvUH+fOIM
i2RLCaSktWxhDEPOypfKdbu8GB7wNpHU/oZ8wl564HN0IRS2ypC+noNceiGv
xb82C57mmfYyb0N+eFQz77RaFpTd8kHtXzvymjsJft11mfDsbMc+u07k6w8V
OnI2ZMDHyrH9Bj3ILx3RWzjb9B0e/GrV1zKIPDu26UxoazLkgBzO0jTk9j0e
o9OayXDyzLtWBQw5z+A4nV9yEmSv/nJC6Q/yvoNYf1lKAvyyS3X15F/kfusP
x3amxcDwu/da42aRL7nX7BrO+ATv57c94FpCfiHIpUg08yOclllmiSR5tJeK
Hc0sAnKm+2UcWEZuM61ya8r8Azw56LFt2wryEG4HWWOLd/B8R3qhLckfUszq
f1u8gQynuqzbSX53+4eeaEsKXHEpGHJdRZ41pa7L/DgABn5Viawl+cctCQyV
g74w3ttZn20NOeVhIcV+yBNeq1E+oETyxFQz04PDblBtNWzEgeQp52R9fg47
Q6kkk8QoktfG+UzU1zyHjx7jXZLkCVR2Jea6J3Dn78sX+klesFfPwibyITxh
rsI2Q/KBSaaKKAVN6F+W2LdC8kWzUllFQxlQc//B9zWSH/zpYv5U4y7Q63vh
sUDyPBeesM1HHgPvvD96EySvrpNfaBF8Bh45F55uJnnw4eDhag4nMJkzsyOT
5M+vNGGv2D2Ad1hsiwHJ2yo2+bey+QCfqzI5QiTX3zBvE+DuDya97n2kkd6/
0B7Vv7Wvg8Ch96HWWiT/JZZktNktFPiuGznfQKqTA+KH/jjbh4OHBnRhuiSv
66sLspP8CFrL8lQGSPX2LIx7sOtYJKBrCPnWvIjcIOb4RQuuKPCs/KW07j/k
71ifm7+mJoDNqflH1qaQ+yfO21GHE8EtB1opK8kDqPG7FPuSwHyX7Xdu0j4a
FD/JOT+eDBLs3YN2kvbjvzB3W5OeVGCT4XAnfQy5suija4/KM0CSK4uuE2m/
+wBBMw25TDA56POIqxv5LgGZGamiTOBrO9PyjdQ3PpdElWM5WeDc3/asJFL/
GXaWl+NIzAGZY8dKDBuR/86Z0t9LKQDcunOXdcqQs3CoP80/WgJCuP2Ov/uM
PJfzeNySWQlYlM7IZPpE8mt7DMRTS8BWIKdqHo7c6hDYGixZClypbul870jP
0/zkItvFMiCl22jF44scemyIHdQuB4JXhU0MbZBTS9YYrF5XAk4G+stK0qS+
dF5Ci7m/HjRUewamZaNz6kN++s5nN9vA6Mj7XZIGk4SXrwW8TnHsBdp3Jk/U
PpwgPEbVuP727ABoNSzlM2AYI5w3ObI/s3cY2FzcpFh9ZxjtrxKdezFRY4Ax
/9iDPL4B1LfPfh5Ta5oAbo9zykre9BC+2/BZcK/XJBDJvJFTTvmN6opy5orw
GA2sdP8u3+eLcot/Sttz2Uka2Cq7OuNljrx/eB+v6hQNGN7fyTuthnz4ekbU
1QUaOJ3N6ZDOgTyOLfPXEhMG+mbXfHaI1aJcMTzjNCCAAe/dSRcYLlUTftnh
uNmiNgYYD31RCDVGeay/M6uSWw8DPJdWXHyPIA9ubXA4YYiBjTJxIc+oKNd1
0GW3qJhgQEiw2EnaArnlTv8bAk8wsGnpBpfr03LCvzt/+W3khYFO6PFm9VUZ
yhVGIve5UzHwwUY7mjcU5dWZFxNFLt8xkLdBT5hXA/mUJb3VQAYGqsYqH+5k
RZ6ydLPZNRcDHBvPGCy7oDx8zVv4tUYpBjysSwW2txYQrravd8uhJgw0ju6w
CbbKQf1HotNhthkDsiy2T9eGs9HzVNFJ5bRiAMvFKPpayHXcwykCHRjgXfD/
ySKTRXjV0doSzz4MXKXf95uNJYNwba3GqJdUDDSYhm/tjULzQuD9AxFpkxiQ
+t3a0bMLOZ3Xnc5OGgYOuDRbtXujuaM/WZyffQoDhs7sWk5DaH4R7ebvZZzF
QMUz4T9xDrGEN0lT5qi472O76a9vEEP4ynZT1Yo5DHitY7Dedjma8PiYG+X6
//D1Ndykq86B5qbSbNOfe5cwYBMTLECJ+US4rZArNQP3uQfmypx+aC6zfCCn
JreMgQ7/71X7xsIJDx3Y2iazgoF6+Xdhs47vCf/b9mslBfd1XlyURoW3KI+x
BRlwrWLAXvA9Xywnmh8vmdZPlOPeeyEi50JsAOEy21hZ8fMe3BVQPMM67kv4
3lnMSg13sWmDYL8XnoQfqtLZ6Ya7vDRH58btbqi/nVFYTMG9vnnk/tNYJ3Qu
93/Z3YQ7P3vP/rIXz9A63r7niM/jIDvqiQElFs3jfyXc9i7hfir00YusWH3C
//ud/eBxn//9zl7wn/8fy77CiQ==
     "]]}},
  Axes->True,
  Method->{},
  PlotRange->{{-0.9999998830731719, 0.9999999999999918}, {-0.9999998592131705,
    0.9999998782112116}, {-1., 1.}},
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.626071543735175*^9, 3.626074409085808*^9, 
  3.6260749092694864`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"%", ",", "equat1"}], "}"}], ",", " ", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.618826747940668*^9, 3.618826776361294*^9}}],

Cell[BoxData[
 Graphics3DBox[{{{}, {}, 
    {RGBColor[1, 0, 0], Line3DBox[CompressedData["
1:eJxN2Xc4V+//B3BSoSGrEqmUpBKSROtGSlY0UEJGipIRiUghstcbLaNUsmf2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      "]]}}, {{}, {}, 
    {Hue[0.67, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxN2Xc41t//B3BS0bIrkUqJkkqRkuogkhUNKSEjRRlJicjM3rPIiEpW9l7H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      "]]}}},
  Axes->True,
  Method->{},
  PlotRange->All,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.6260715437507753`*^9, 3.626074440972264*^9, 
  3.626074921999109*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"r", "=", "1"}], ";", 
  RowBox[{
   RowBox[{"u", "[", "s_", "]"}], ":=", "t"}], ";", 
  RowBox[{
   RowBox[{"v", "[", "s_", "]"}], ":=", "0"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"norm", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"sphero", "[", "r", "]"}], "[", 
      RowBox[{
       RowBox[{"fu", "[", "t", "]"}], ",", 
       RowBox[{"gu", "[", "t", "]"}]}], "]"}], "-", 
     RowBox[{
      RowBox[{"sphero", "[", "r", "]"}], "[", 
      RowBox[{
       RowBox[{"u", "[", "t", "]"}], ",", 
       RowBox[{"v", "[", "t", "]"}]}], "]"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.6188268763600135`*^9, 3.618826890079798*^9}, {
  3.6188269471670637`*^9, 3.618826984440195*^9}, {3.61882710743423*^9, 
  3.6188271184888625`*^9}, {3.6188271596792183`*^9, 3.6188271684097176`*^9}, {
  3.6188272292031946`*^9, 3.6188272362896004`*^9}},
 NumberMarks->False],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxN0Qs0VHkcwPF5SMPKDpmRtQh5ZLxKkVyZrKJRc5iNJI9xpZis9yFD5nhv
ymvCypvyastjLbPZQkobB01LnnmsVdtbSY6diq1d/X97z7nnns/9/v/3/v/3
auLBHD8SgUBI+3h+upLbZfv/3B6/i/DpKGu30nnSZqV+ztvmXzsJMUmDydFR
YfB/JjhgD5X8l1qEglXHYGN7kzorhZmrTsPuewv1ioWlq87BOIEqY/nC+lWH
YNnzHk15wnZkqy6WrG24eNUf7/DZPk0hI8jf/eKXQdKeRh58vE+mGp9FTjNZ
JLQcfowcyriT21H5HFnie6JuIugVMn7d1Snr2Dyy7+vBjo7vF5DPmoqCq1IW
kUkdIosk/hKygoFj/1S+BNmh1PbCK+F7ZOW53r0C0TJywW+tRHIAgfnZmNru
lY1HiciMaersqQgS8pUaRf/mMDJy7KTnACNFCjkU2+FYkrMGWVJqhz/LkkYW
hxRwoyrWIlvaay1PlFOQu/dY1jCLZJArG89bzvwoi0zTNuR51X6B/DuRkqBy
TQ65mRJeJe5eh2xIo/LwLnnkuYxFJbOhL5Gfv1LvNBoDP+XYB+tPgmfp+d1q
j8BjJZZxlEUwfqjtzUtZKvKB40TPWwbg/fm5TTcMwazuzbLtxmAHBkd0dSvY
dq5CoX4neEeU3e3zLLBFzcjXP+ynMtlEC4nvtnuY+WhgeA4b+jZZYk+2E9jM
KndjJgfGbw3cHHX2IHTT4ut9p13BJv3Om1LdqExxq4/NtIYYM155yE9yh25o
yr8X7wFm+MjrC7yoTP8ItvJP1ncxA2FFXCwXuv5N8/vROFhvoYcRdRTWo6vj
nRBxDOZvcn0zEupPZb457GPHGOnFtFNTTYJ5MD5saFscHgi9Q2em1yUIutct
jIeFQS8gzl+U4cN+FGM9vrp4GsaTQresGa2BPuM+Pmb71+f13sMcHg783Ryk
wIy8EBfTryfGTob5+dNfKDC5K+TpFaIYY7cUsuiRimj+y/e8WPKi4up8MVY+
HP9zUPh61CX+1tzod+uZTdVmZVy5u5jah+wjoXwl9PxH9md6KFI09D1wR+5G
HWkamj/B3h65mwIedJnS5MvR0H47fc2inylBLxaM6/Xr0NB+NyQ2nHqqBz0n
JXlA2gB6WrqJwMYY+snCxKEGc+gLJW6GfTtgfcEVRglPdoKfVpJG1ljT0P8+
VjtspGUDnrl8OXHXbrBnQ/yo+zfwvpEmV5OoPdAPihjJ5+zB4lbCeP0+sGPb
fdNeR1jf7RuXUh4fgG7bJXgg5QQ2HRgwi3QDy83FzLgFgG/q9tqonwGzMqZY
9PT/vf/tvIt8JnjylkrgshC8hB/PnywAG5aSXpdcAucpW1Zo9IB5lCopTRk6
sqhsk/XNbHC/FdbFUVCG5/MKv+VmgO84lbqso29Axi2x6uoccMCLvrcP8sAh
5d52iufBApnEP2KLwUWj3aqcKvBQ9KGs9yIwqzUs2nkc7BwkdSdlAuymlUe/
NgU+nna1SXcWnOROePnuObhNko5XLYO76jQaxwkqyL144wqVDB7vGSyKWQte
KlAddqKCV9hXdFIUwdJkm4hflcBKJ3AFXRWwqsaC9xFVsNZAcl2WGtggVflD
lwZ4i1Wt4ztN8D+8elOA
     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{None, None},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->
   NCache[{{0, 2 Pi}, {0., 9.550499576785472*^-16}}, {{
     0, 6.283185307179586}, {0., 9.550499576785472*^-16}}],
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.6260715439223757`*^9}]
}, Closed]],

Cell["\<\
The numeric solution is quite near to the exact equator. Of course, explicit \
calculation confirmes that the equator is a geodesic:\
\>", "Text",
 CellChangeTimes->{{3.618828015437165*^9, 3.618828058413623*^9}, 
   3.6260749487843556`*^9, {3.6260749908732295`*^9, 3.6260750310745*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"u", "[", "s_", "]"}], ":=", "s"}], ";", 
  RowBox[{
   RowBox[{"v", "[", "s_", "]"}], ":=", "0"}], ";", " ", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"DD1", "[", 
     RowBox[{
      RowBox[{"sphero", "[", "r", "]"}], ",", "\[Gamma]", ",", 
      SuperscriptBox["\[Gamma]", "\[Prime]",
       MultilineFunction->None]}], "]"}], "[", "t", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.597930990691161*^9, 3.597931042054099*^9}, {
   3.5979310726468487`*^9, 3.597931145472014*^9}, 3.5979312951205735`*^9, {
   3.597931409520117*^9, 3.5979314723207088`*^9}, {3.597931552368287*^9, 
   3.59793155713656*^9}, 3.598011520257586*^9, 3.6188279817532387`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.6260715440627756`*^9}]
}, Closed]],

Cell["\<\
It follows: The equator and therefore all the great circles, are the \
geodesics of the sphere, since the rotation group preserving the sphere acts \
transitively on the set of all great circles. \[FilledRectangle]\
\>", "Text",
 CellChangeTimes->{{3.5979354457009735`*^9, 3.5979355381972637`*^9}, {
   3.5979361835101733`*^9, 3.597936190844593*^9}, 3.597936521357497*^9}],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"r", ",", "u", ",", "v", ",", "s", ",", "t", ",", "fu", ",", "gu"}],
   "]"}]], "Input",
 CellChangeTimes->{{3.597935683989602*^9, 3.597935730415258*^9}, 
   3.597936386831803*^9, {3.59801155100524*^9, 3.598011556324849*^9}}],

Cell[TextData[{
 "Applying the existence and uniqueness theorem for differential equations, \
we may state\n\n",
 StyleBox["Proposition",
  FontColor->RGBColor[0, 0, 1]],
 ". Through any point x of an immersion (or a Riemannian manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["X", "m"], TraditionalForm]]],
 ")  and any tangent unit vector vt \[Epsilon]",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SubscriptBox["T", "x"]}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["X", "m"], TraditionalForm]]],
 " there exists one and only one geodesic \[Gamma][t],  \[Gamma][0] = x, ",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Prime]",
    MultilineFunction->None], "[", "0", "]"}]],
  CellChangeTimes->{{3.5979292059430795`*^9, 3.5979292420701456`*^9}, {
   3.597929272325876*^9, 3.597929298654382*^9}, {3.597929354638584*^9, 
   3.597929492815488*^9}}],
 " = vt, defined for a maximal interval of values t, t the arc length of \
\[Gamma]."
}], "Text",
 CellChangeTimes->{{3.598076949768775*^9, 3.598077205905425*^9}, {
   3.598077253769163*^9, 3.598077273273278*^9}, {3.598077357521097*^9, 
   3.598077557343526*^9}, {3.5980775930175667`*^9, 3.5980777053379908`*^9}, {
   3.5980777669065123`*^9, 3.598077838916631*^9}, {3.5980778699544067`*^9, 
   3.598077876921805*^9}, {3.598077921365347*^9, 3.598077950626021*^9}, 
   3.5980779864740715`*^9, 3.6132850698693633`*^9, 3.6174417193890543`*^9}],

Cell[TextData[{
 StyleBox[" Example",
  FontColor->RGBColor[0, 0, 1]],
 ". Ruled surfaces."
}], "Text",
 CellChangeTimes->{{3.598013618960472*^9, 3.5980136605033455`*^9}, {
   3.5980775178572674`*^9, 3.5980775195133624`*^9}, 3.5980780173128347`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";"}]], "Input",
 CellChangeTimes->{{3.618828782127017*^9, 3.6188287890304117`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ruled", "[", 
    RowBox[{"x_", ",", "y_"}], "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"vec", "[", 
    RowBox[{"x", "[", "v", "]"}], "]"}], " ", "+", " ", 
   RowBox[{
    RowBox[{"vec", "[", 
     RowBox[{"y", "[", "v", "]"}], "]"}], "*", " ", "u"}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5872799484616065`*^9, 3.5872801346102533`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"rsf", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"ruled", "[", 
    RowBox[{"x", ",", "y"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5980123012573576`*^9, 3.598012356419055*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rsf", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.598012474604862*^9, 3.598012479799671*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"x", "[", "v", "]"}], "[", "1", "]"}], "+", 
    RowBox[{"u", " ", 
     RowBox[{
      RowBox[{"y", "[", "v", "]"}], "[", "1", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"x", "[", "v", "]"}], "[", "2", "]"}], "+", 
    RowBox[{"u", " ", 
     RowBox[{
      RowBox[{"y", "[", "v", "]"}], "[", "2", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"x", "[", "v", "]"}], "[", "3", "]"}], "+", 
    RowBox[{"u", " ", 
     RowBox[{
      RowBox[{"y", "[", "v", "]"}], "[", "3", "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6260715440939755`*^9, 3.626075090089404*^9}]
}, Closed]],

Cell["\<\
The generating lines are geodesics; later we shall see that any line \
contained in a surface is a geodesic.\
\>", "Text",
 CellChangeTimes->{{3.598013751248705*^9, 3.5980138273924384`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Gamma]", "[", "t_", "]"}], " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"t", ",", "v0"}], "}"}]}], ";", 
  RowBox[{
   RowBox[{"DD1", "[", 
    RowBox[{"rsf", ",", "\[Gamma]", ",", 
     SuperscriptBox["\[Gamma]", "\[Prime]",
      MultilineFunction->None]}], "]"}], "[", "t", "]"}]}]], "Input",
 CellChangeTimes->{{3.5980125593598113`*^9, 3.598012620574319*^9}, {
   3.5980126618363914`*^9, 3.598012676874818*^9}, 3.626075159634326*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.6260715461999793`*^9, 3.6260751652971363`*^9}]
}, Closed]],

Cell["\<\
Now we calculate the absolute derivative of the tangent vector at the \
directrix vec[x[v]]:\
\>", "Text",
 CellChangeTimes->{{3.5980139213202033`*^9, 3.598013941553439*^9}, {
   3.598015594407542*^9, 3.598015602878357*^9}, {3.5980744477356434`*^9, 
   3.5980745343625956`*^9}, 3.6174417367160454`*^9, {3.6188281664378014`*^9, 
   3.6188281699900045`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Gamma]", "[", "t_", "]"}], " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"0", ",", "t"}], "}"}]}], ";", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"DD1", "[", 
     RowBox[{"rsf", ",", "\[Gamma]", ",", 
      SuperscriptBox["\[Gamma]", "\[Prime]",
       MultilineFunction->None]}], "]"}], "[", "t", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5980125593598113`*^9, 3.598012620574319*^9}, {
   3.5980126618363914`*^9, 3.598012676874818*^9}, {3.598012716576887*^9, 
   3.5980127313969135`*^9}, {3.5980139548914623`*^9, 3.598013988774722*^9}, 
   3.598015612097973*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], "2"],
          "+", 
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], "2"],
          "+", 
         SuperscriptBox[
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], 
          "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
        ")"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
        ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
        ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"2", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "3", 
              "]"}]}]}], ")"}], "2"]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "2"]}], ")"}], 
         " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], 
            "2"], "+", 
           SuperscriptBox[
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], 
            "2"], "+", 
           SuperscriptBox[
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], 
            "2"]}], ")"}]}]}], ")"}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "2"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
        ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
        ")"}]}], "+", 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{
           RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
        ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"2", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], 
            "+", 
            RowBox[{
             RowBox[{
              RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
             RowBox[{
              RowBox[{
               SuperscriptBox["x", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "[", "3", 
              "]"}]}]}], ")"}], "2"]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "2"]}], ")"}], 
         " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], 
            "2"], "+", 
           SuperscriptBox[
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], 
            "2"], "+", 
           SuperscriptBox[
            RowBox[{
             RowBox[{
              SuperscriptBox["x", "\[Prime]",
               MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], 
            "2"]}], ")"}]}]}], ")"}]}], ")"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6260715491015844`*^9, 3.62607518476597*^9}]
}, Closed]],

Cell["\<\
We assume that v= t is the arc length of x, that y[v] is orthogonal to x\
\[CloseCurlyQuote], and that y.y ==1 is a unit vector, i.e. that the ruled \
surface is generated by a family of normals of the curve x[v]. This is \
equivalent to the Conditions\
\>", "Text",
 CellChangeTimes->{{3.598014139954587*^9, 3.5980141739002466`*^9}, {
   3.5980142117459135`*^9, 3.598014350960558*^9}, {3.598015630428005*^9, 
   3.598015689708109*^9}, 3.5980745908658953`*^9, 3.617174467076477*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"conds", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], 
         "2"]}], ")"}], "\[Equal]", "1"}], ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], "2"]}], ")"}], 
      "\[Equal]", "1"}], ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}], " ", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], "+", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}], " ", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], "+", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}], " ", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
       ")"}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], "+", 
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], "+", 
        RowBox[{
         RowBox[{
          RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "\[Prime]",
            MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], 
       ")"}], "\[Equal]", "0"}]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.598014358713772*^9, 3.5980144816731873`*^9}, {
   3.5980145294872713`*^9, 3.5980145784713573`*^9}, {3.5980146324630527`*^9, 
   3.59801463676866*^9}, {3.5980146712447205`*^9, 3.5980146860647464`*^9}, 
   3.598014725454816*^9, {3.5980147633784823`*^9, 3.5980147646732845`*^9}, {
   3.598014867867466*^9, 3.598014871814273*^9}}],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%%", ",", "conds"}], "]"}]], "Input",
 CellChangeTimes->{{3.598012757495759*^9, 3.5980127705529823`*^9}, {
   3.5980128218770723`*^9, 3.5980128789263725`*^9}, {3.5980148996915216`*^9, 
   3.5980149121091433`*^9}, 3.5980150010449*^9, {3.5980150367065625`*^9, 
   3.5980150390309663`*^9}, {3.5980157411413994`*^9, 
   3.5980157424830017`*^9}, {3.613285257919119*^9, 3.613285263830457*^9}, {
   3.6171931026017113`*^9, 3.6171931109477262`*^9}, {3.618828710063895*^9, 
   3.618828731983149*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], ",", 
   "0"}], "}"}]], "Input",
 CellChangeTimes->{3.6260752414252696`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"y", "[", "t", "]"}], "[", "1", "]"}], " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "t", "]"}], "[", "1", "]"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"y", "[", "t", "]"}], "[", "2", "]"}], " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "t", "]"}], "[", "2", "]"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"y", "[", "t", "]"}], "[", "3", "]"}], " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "t", "]"}], "[", "3", "]"}]}]}], ",", 
   "0"}], "}"}]], "Output",
 CellChangeTimes->{3.62607524148767*^9}]
}, Closed]],

Cell[TextData[{
 "Now vec[",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox["x", "\[Prime]\[Prime]",
    MultilineFunction->None], "[", "t", "]"}]],
  CellChangeTimes->{{3.598015889934461*^9, 3.598015935782942*^9}}],
 "] == k[t] *n[t] is the curvature vector of the curve vec[x[t]]; it is \
proportional to the principal normal vector n[t] with the curvature k[t] as \
factor; remember: t is the arc length. Since vec[y[t] is a unit normal \
vector. we have the interpretation where \[Alpha][t] denotes the angle \
between n[t] and vec[y[t]:"
}], "Text",
 CellChangeTimes->{{3.59801595197577*^9, 3.598016040630726*^9}, {
   3.5980160729071827`*^9, 3.598016191560991*^9}, {3.598016348949667*^9, 
   3.5980163923177433`*^9}, {3.5980746508636007`*^9, 3.598074668429231*^9}, 
   3.6171744572796593`*^9, {3.617193389361415*^9, 3.6171934129954567`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Gamma]", "[", "t_", "]"}], " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"0", ",", "t"}], "}"}]}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"DD1", "[", 
     RowBox[{"rsf", ",", "\[Gamma]", ",", 
      SuperscriptBox["\[Gamma]", "\[Prime]",
       MultilineFunction->None]}], "]"}], "[", "t", "]"}], "\[Equal]", 
   RowBox[{"{", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"k", "[", "t", "]"}], " ", 
      RowBox[{"Cos", "[", 
       RowBox[{"\[Alpha]", "[", "t", "]"}], "]"}]}], ",", "0"}], 
    "}"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5980164417854304`*^9, 3.5980164693506784`*^9}, {
  3.598016626224554*^9, 3.598016666847026*^9}, {3.6171932823608274`*^9, 
  3.617193289053239*^9}}],

Cell[TextData[{
 "It follows: ",
 StyleBox["Any curve is a geodesic on the normal surface generated by its \
binormals. ",
  FontColor->RGBColor[0, 0, 1]],
 "\[FilledRectangle]"
}], "Text",
 CellChangeTimes->{{3.598016655053405*^9, 3.598016715659511*^9}, 
   3.5980167597451887`*^9, {3.5980169228434753`*^9, 3.5980169797367754`*^9}, {
   3.5980170368952756`*^9, 3.5980170459588914`*^9}, {3.5980183077359076`*^9, 
   3.598018313476718*^9}, {3.598074739939757*^9, 3.598074742295361*^9}, 
   3.613285387981558*^9, 3.617193376413392*^9}],

Cell["\<\
Now we consider the special ruled surface generated by normals of the x-axis:\
\>", "Text",
 CellChangeTimes->{{3.6171938010149384`*^9, 3.617193859561841*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"rsf", "[", "w_", "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", 
  RowBox[{"{", 
   RowBox[{"u", ",", " ", 
    RowBox[{"v", "*", 
     RowBox[{"Cos", "[", 
      RowBox[{"w", "[", "u", "]"}], "]"}]}], ",", 
    RowBox[{"v", "*", 
     RowBox[{"Sin", "[", 
      RowBox[{"w", "[", "u", "]"}], "]"}]}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.598018362570004*^9, 3.5980185772263813`*^9}, 
   3.598074791778648*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"amf", "[", "3", "]"}], "[", 
    RowBox[{"rsf", "[", "w", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.59801860508803*^9, 3.5980186424188957`*^9}, {
   3.598074818563895*^9, 3.5980748315275173`*^9}, 3.6148618628107452`*^9, 
   3.614917794104565*^9, {3.6171935611489167`*^9, 3.6171935620381184`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"v", " ", 
      RowBox[{
       SuperscriptBox["w", "\[Prime]",
        MultilineFunction->None], "[", "u", "]"}]}], 
     SqrtBox[
      RowBox[{"1", "+", 
       RowBox[{
        SuperscriptBox["v", "2"], " ", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["w", "\[Prime]",
           MultilineFunction->None], "[", "u", "]"}], "2"]}]}]]]}], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"Sin", "[", 
      RowBox[{"w", "[", "u", "]"}], "]"}], 
     SqrtBox[
      RowBox[{"1", "+", 
       RowBox[{
        SuperscriptBox["v", "2"], " ", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["w", "\[Prime]",
           MultilineFunction->None], "[", "u", "]"}], "2"]}]}]]]}], ",", 
   FractionBox[
    RowBox[{"Cos", "[", 
     RowBox[{"w", "[", "u", "]"}], "]"}], 
    SqrtBox[
     RowBox[{"1", "+", 
      RowBox[{
       SuperscriptBox["v", "2"], " ", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["w", "\[Prime]",
          MultilineFunction->None], "[", "u", "]"}], "2"]}]}]]]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.6260715500531864`*^9, 3.6260755966066933`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Norm", "[", "%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5980186576289225`*^9, 3.5980186982669935`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"\[Sqrt]", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"Abs", "[", 
        RowBox[{"Cos", "[", 
         RowBox[{"w", "[", "u", "]"}], "]"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{"Abs", "[", 
        RowBox[{"Sin", "[", 
         RowBox[{"w", "[", "u", "]"}], "]"}], "]"}], "2"], "+", 
      RowBox[{"v", " ", 
       SuperscriptBox[
        RowBox[{"Abs", "[", 
         RowBox[{
          SuperscriptBox["w", "\[Prime]",
           MultilineFunction->None], "[", "u", "]"}], "]"}], "2"], " ", 
       RowBox[{"Conjugate", "[", "v", "]"}]}]}], ")"}]}], ")"}], "/", 
  RowBox[{"(", 
   SqrtBox[
    RowBox[{"Abs", "[", 
     RowBox[{"1", "+", 
      RowBox[{
       SuperscriptBox["v", "2"], " ", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["w", "\[Prime]",
          MultilineFunction->None], "[", "u", "]"}], "2"]}]}], "]"}]], 
   ")"}]}]], "Output",
 CellChangeTimes->{3.6260715507707872`*^9, 3.6260756006315002`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"w", "[", "u", "]"}], "\[Element]", " ", "Reals"}], ",", 
     RowBox[{"v", " ", "\[Element]", " ", "Reals"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["w", "\[Prime]",
        MultilineFunction->None], "[", "u", "]"}], "\[Element]", " ", 
      "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5980187581554985`*^9, 3.5980188628628826`*^9}, {
   3.598074896969633*^9, 3.598074903755645*^9}, 3.5980751467728715`*^9, {
   3.598075207238578*^9, 3.598075224086607*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.6260715508643875`*^9, 3.626075606777911*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"gauss", "[", 
     RowBox[{"rsf", "[", "w", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"w", "[", "u", "]"}], "\[Element]", " ", "Reals"}], ",", 
     RowBox[{"v", " ", "\[Element]", " ", "Reals"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["w", "\[Prime]",
        MultilineFunction->None], "[", "u", "]"}], "\[Element]", " ", 
      "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5980189149669743`*^9, 3.598018950675437*^9}, 
   3.598018999519123*^9, 3.5980749646425514`*^9, {3.5980752585314674`*^9, 
   3.598075273632294*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   SuperscriptBox[
    RowBox[{
     SuperscriptBox["w", "\[Prime]",
      MultilineFunction->None], "[", "u", "]"}], "2"], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "+", 
      RowBox[{
       SuperscriptBox["v", "2"], " ", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["w", "\[Prime]",
          MultilineFunction->None], "[", "u", "]"}], "2"]}]}], ")"}], 
    "2"]]}]], "Output",
 CellChangeTimes->{3.626071551769189*^9, 3.626075650255188*^9}]
}, Closed]],

Cell[TextData[{
 "It follows: ",
 StyleBox["The surface is developable, namely a part of a plane, iff w(u) == \
const.",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.59807502072465*^9, 3.598075092843577*^9}}],

Cell["\<\
Of course, the x-axis and the generating lines are geodesics:\
\>", "Text",
 CellChangeTimes->{{3.5980755458014727`*^9, 3.5980755807694726`*^9}, 
   3.613286069453536*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Gamma]", "[", "t_", "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0"}], "}"}]}], ";", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"DD1", "[", 
     RowBox[{
      RowBox[{"rsf", "[", "w", "]"}], ",", "\[Gamma]", ",", 
      SuperscriptBox["\[Gamma]", "\[Prime]",
       MultilineFunction->None]}], "]"}], "[", "t", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.598019304281258*^9, 3.5980193805497923`*^9}, {
  3.5980194122022476`*^9, 3.598019418036658*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.626071551847189*^9, 3.6260756835924463`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Gamma]", "[", "t_", "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{"uo", ",", "t"}], "}"}]}], ";", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"DD1", "[", 
     RowBox[{
      RowBox[{"rsf", "[", "w", "]"}], ",", "\[Gamma]", ",", 
      SuperscriptBox["\[Gamma]", "\[Prime]",
       MultilineFunction->None]}], "]"}], "[", "t", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.598019304281258*^9, 3.5980193805497923`*^9}, {
  3.5980194122022476`*^9, 3.598019418036658*^9}, {3.598019521090439*^9, 
  3.5980195443968797`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.6260715519719896`*^9, 3.626075687149253*^9}]
}, Closed]],

Cell["Let", "Text",
 CellChangeTimes->{{3.5980758476497374`*^9, 3.598075856841263*^9}, {
  3.6132867770860105`*^9, 3.613286807238735*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"w", "[", "u_", "]"}], " ", ":=", " ", "u"}]], "Input",
 CellChangeTimes->{{3.613286962941641*^9, 3.613286976054391*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"rsf", "[", "w", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "2"}], ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "2"}], ",", "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.613287147781213*^9, 3.6132872376243515`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"rsf1", "=", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.6132872841090107`*^9, 3.6132872905983815`*^9}}],

Cell["We consider the curve", "Text",
 CellChangeTimes->{{3.5980758476497374`*^9, 3.598075856841263*^9}, {
  3.6132867770860105`*^9, 3.613286807238735*^9}, {3.617193967841631*^9, 
  3.617193970571636*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"rsf", "[", "w", "]"}], "[", 
  RowBox[{"t", ",", "1"}], "]"}]], "Input",
 CellChangeTimes->{3.613286393944096*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"t", ",", 
   RowBox[{"Cos", "[", "t", "]"}], ",", 
   RowBox[{"Sin", "[", "t", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6260715520343895`*^9, 3.626075837970318*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"rsf", "[", "w", "]"}], "[", 
    RowBox[{"t", ",", "1"}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"t", ",", " ", 
     RowBox[{"-", "2"}], ",", "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6132869979946456`*^9, 3.6132870553199244`*^9}}],

Cell["It isn\[CloseCurlyQuote]t a geodesic:", "Text",
 CellChangeTimes->{{3.5980761396094365`*^9, 3.598076149833021*^9}, {
   3.613286460212886*^9, 3.6132864682773476`*^9}, 3.6132870776131997`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Gamma]", "[", "t_", "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{"t", ",", "1"}], "}"}]}], ";", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"DD1", "[", 
     RowBox[{
      RowBox[{"rsf", "[", "w", "]"}], ",", "\[Gamma]", ",", 
      SuperscriptBox["\[Gamma]", "\[Prime]",
       MultilineFunction->None]}], "]"}], "[", "t", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.598019304281258*^9, 3.5980193805497923`*^9}, {
  3.5980194122022476`*^9, 3.598019418036658*^9}, {3.598019521090439*^9, 
  3.5980195443968797`*^9}, {3.598019590884961*^9, 3.5980196226154175`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{"-", "1"}]}], "}"}]], "Output",
 CellChangeTimes->{3.62607155209679*^9, 3.6260758563315496`*^9}]
}, Closed]],

Cell["We try to find some geodesics.", "Text",
 CellChangeTimes->{{3.5980768531292477`*^9, 3.598076874889492*^9}}],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.617194795048684*^9, 3.617194806343104*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Gamma]", "[", "t_", "]"}], " ", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}]}], ";", 
  RowBox[{
   RowBox[{"dabs", "[", "t_", "]"}], " ", "=", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{"DD1", "[", 
      RowBox[{
       RowBox[{"rsf", "[", "w", "]"}], ",", "\[Gamma]", ",", 
       SuperscriptBox["\[Gamma]", "\[Prime]",
        MultilineFunction->None]}], "]"}], "[", "t", "]"}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5980199572984047`*^9, 3.598020047950164*^9}, {
   3.5980782588376493`*^9, 3.598078279077807*^9}, {3.6171941205814996`*^9, 
   3.617194139551133*^9}, 3.6171942578929405`*^9, {3.617194540331437*^9, 
   3.617194543716643*^9}, {3.617194818901126*^9, 3.6171948227699327`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    FractionBox[
     RowBox[{"2", " ", 
      RowBox[{"v", "[", "t", "]"}], " ", 
      RowBox[{
       SuperscriptBox["u", "\[Prime]",
        MultilineFunction->None], "[", "t", "]"}], " ", 
      RowBox[{
       SuperscriptBox["v", "\[Prime]",
        MultilineFunction->None], "[", "t", "]"}]}], 
     RowBox[{"1", "+", 
      SuperscriptBox[
       RowBox[{"v", "[", "t", "]"}], "2"]}]], "+", 
    RowBox[{
     SuperscriptBox["u", "\[Prime]\[Prime]",
      MultilineFunction->None], "[", "t", "]"}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{"v", "[", "t", "]"}]}], " ", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["u", "\[Prime]",
        MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
    RowBox[{
     SuperscriptBox["v", "\[Prime]\[Prime]",
      MultilineFunction->None], "[", "t", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.62607155220599*^9, 3.6260758847859993`*^9}]
}, Closed]],

Cell["Again a symbolic solution isn\[CloseCurlyQuote]t  available:", "Text",
 CellChangeTimes->{{3.598078644105685*^9, 3.5980786681610613`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"%", "[", 
       RowBox[{"[", "1", "]"}], "]"}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{"%", "[", 
       RowBox[{"[", "2", "]"}], "]"}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{"u", "[", "0", "]"}], "\[Equal]", " ", "1"}], ",", 
     RowBox[{
      RowBox[{"v", "[", "0", "]"}], "\[Equal]", " ", ".2"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}], ",", "t"}], "]"}]], "Input",
 CellChangeTimes->{{3.5980200989310536`*^9, 3.598020171689582*^9}, {
   3.5980202241992736`*^9, 3.59802039280437*^9}, {3.598020424176025*^9, 
   3.5980205019109616`*^9}, {3.5980205409734297`*^9, 
   3.5980205616278667`*^9}, {3.5980209521589527`*^9, 3.598020953484955*^9}, {
   3.5980210114702563`*^9, 3.598021015027063*^9}, 3.5980781057668943`*^9, {
   3.598078143270039*^9, 3.598078158902933*^9}, 3.614861862841945*^9, 
   3.6148646600360584`*^9, {3.6171945812503085`*^9, 3.6171945997987413`*^9}, {
   3.6171948391343613`*^9, 3.617194848946779*^9}}],

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       FractionBox[
        RowBox[{"2", " ", 
         RowBox[{"v", "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["v", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}]}], 
        RowBox[{"1", "+", 
         SuperscriptBox[
          RowBox[{"v", "[", "t", "]"}], "2"]}]], "+", 
       RowBox[{
        SuperscriptBox["u", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"v", "[", "t", "]"}]}], " ", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox["v", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{"u", "[", "0", "]"}], "\[Equal]", "1"}], ",", 
     RowBox[{
      RowBox[{"v", "[", "0", "]"}], "\[Equal]", "0.2`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}], ",", "t"}], "]"}]], "Output",
 CellChangeTimes->{3.62607155222159*^9, 3.6260758941928167`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       FractionBox[
        RowBox[{"2", " ", 
         RowBox[{"v", "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["v", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}]}], 
        RowBox[{"1", "+", 
         SuperscriptBox[
          RowBox[{"v", "[", "t", "]"}], "2"]}]], "+", 
       RowBox[{
        SuperscriptBox["u", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"v", "[", "t", "]"}]}], " ", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox["v", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{"u", "[", "0", "]"}], "\[Equal]", "1"}], ",", 
     RowBox[{
      RowBox[{"v", "[", "0", "]"}], "\[Equal]", "0.2`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}], ",", "t"}], "]"}]], "Input",
 CellChangeTimes->{{3.614861862841945*^9, 3.6148618628575454`*^9}, 
   3.6148646600516586`*^9}],

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       FractionBox[
        RowBox[{"2", " ", 
         RowBox[{"v", "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["v", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}]}], 
        RowBox[{"1", "+", 
         SuperscriptBox[
          RowBox[{"v", "[", "t", "]"}], "2"]}]], "+", 
       RowBox[{
        SuperscriptBox["u", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"v", "[", "t", "]"}]}], " ", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox["v", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{"u", "[", "0", "]"}], "\[Equal]", "1"}], ",", 
     RowBox[{
      RowBox[{"v", "[", "0", "]"}], "\[Equal]", "0.2`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}], ",", "t"}], "]"}]], "Output",
 CellChangeTimes->{3.62607155225279*^9, 3.626075899262825*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"dabs", "[", "t", "]"}], "[", 
       RowBox[{"[", "1", "]"}], "]"}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"dabs", "[", "t", "]"}], "[", 
       RowBox[{"[", "2", "]"}], "]"}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{"u", "[", "0", "]"}], "\[Equal]", " ", "1"}], ",", 
     RowBox[{
      RowBox[{"v", "[", "0", "]"}], "\[Equal]", " ", ".2"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["u", "\[Prime]",
        MultilineFunction->None], "[", "0", "]"}], "\[Equal]", " ", "1"}], 
     ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["v", "\[Prime]",
        MultilineFunction->None], "[", "0", "]"}], "\[Equal]", " ", "1"}]}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}], ",", "t"}], "]"}]], "Input",
 CellChangeTimes->{{3.5980200989310536`*^9, 3.598020171689582*^9}, {
   3.5980202241992736`*^9, 3.59802039280437*^9}, {3.598020424176025*^9, 
   3.5980205019109616`*^9}, {3.5980205409734297`*^9, 
   3.5980205616278667`*^9}, {3.5980209521589527`*^9, 3.598020953484955*^9}, {
   3.5980210114702563`*^9, 3.598021015027063*^9}, 3.5980781057668943`*^9, {
   3.598078143270039*^9, 3.598078158902933*^9}, {3.598079224186864*^9, 
   3.5980792763758492`*^9}, 3.614861862873145*^9, 3.6148646600672584`*^9}],

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       FractionBox[
        RowBox[{"2", " ", 
         RowBox[{"v", "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["v", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}]}], 
        RowBox[{"1", "+", 
         SuperscriptBox[
          RowBox[{"v", "[", "t", "]"}], "2"]}]], "+", 
       RowBox[{
        SuperscriptBox["u", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"v", "[", "t", "]"}]}], " ", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox["v", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{"u", "[", "0", "]"}], "\[Equal]", "1"}], ",", 
     RowBox[{
      RowBox[{"v", "[", "0", "]"}], "\[Equal]", "0.2`"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["u", "\[Prime]",
        MultilineFunction->None], "[", "0", "]"}], "\[Equal]", "1"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["v", "\[Prime]",
        MultilineFunction->None], "[", "0", "]"}], "\[Equal]", "1"}]}], "}"}],
    ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}], ",", "t"}], "]"}]], "Output",
 CellChangeTimes->{3.6260715522683897`*^9, 3.6260759033500323`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       FractionBox[
        RowBox[{"2", " ", 
         RowBox[{"v", "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["v", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}]}], 
        RowBox[{"1", "+", 
         SuperscriptBox[
          RowBox[{"v", "[", "t", "]"}], "2"]}]], "+", 
       RowBox[{
        SuperscriptBox["u", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"v", "[", "t", "]"}]}], " ", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox["v", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{"u", "[", "0", "]"}], "\[Equal]", "1"}], ",", 
     RowBox[{
      RowBox[{"v", "[", "0", "]"}], "\[Equal]", "0.2`"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["u", "\[Prime]",
        MultilineFunction->None], "[", "0", "]"}], "\[Equal]", "1"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["v", "\[Prime]",
        MultilineFunction->None], "[", "0", "]"}], "\[Equal]", "1"}]}], "}"}],
    ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}], ",", "t"}], "]"}]], "Input",
 CellChangeTimes->{{3.614861862873145*^9, 3.6148618628887453`*^9}, 
   3.614864660082858*^9}],

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       FractionBox[
        RowBox[{"2", " ", 
         RowBox[{"v", "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["v", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}]}], 
        RowBox[{"1", "+", 
         SuperscriptBox[
          RowBox[{"v", "[", "t", "]"}], "2"]}]], "+", 
       RowBox[{
        SuperscriptBox["u", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"v", "[", "t", "]"}]}], " ", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox["v", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{"u", "[", "0", "]"}], "\[Equal]", "1"}], ",", 
     RowBox[{
      RowBox[{"v", "[", "0", "]"}], "\[Equal]", "0.2`"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["u", "\[Prime]",
        MultilineFunction->None], "[", "0", "]"}], "\[Equal]", "1"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["v", "\[Prime]",
        MultilineFunction->None], "[", "0", "]"}], "\[Equal]", "1"}]}], "}"}],
    ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}], ",", "t"}], "]"}]], "Output",
 CellChangeTimes->{3.62607155229959*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"dabs", "[", "t", "]"}], "[", 
       RowBox[{"[", "1", "]"}], "]"}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"dabs", "[", "t", "]"}], "[", 
       RowBox[{"[", "2", "]"}], "]"}], "\[Equal]", "0"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "v"}], "}"}], ",", "t"}], "]"}]], "Input",
 CellChangeTimes->{{3.5980200989310536`*^9, 3.598020171689582*^9}, {
   3.5980202241992736`*^9, 3.59802039280437*^9}, {3.598020424176025*^9, 
   3.5980205019109616`*^9}, {3.5980205409734297`*^9, 
   3.5980205616278667`*^9}, {3.5980209521589527`*^9, 3.598020953484955*^9}, {
   3.5980210114702563`*^9, 3.598021015027063*^9}, 3.5980781057668943`*^9, {
   3.598078143270039*^9, 3.598078158902933*^9}, {3.598079224186864*^9, 
   3.598079328933855*^9}, 3.6148618628887453`*^9, 3.6148646600984583`*^9}],

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       FractionBox[
        RowBox[{"2", " ", 
         RowBox[{"v", "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["v", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}]}], 
        RowBox[{"1", "+", 
         SuperscriptBox[
          RowBox[{"v", "[", "t", "]"}], "2"]}]], "+", 
       RowBox[{
        SuperscriptBox["u", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"v", "[", "t", "]"}]}], " ", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox["v", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}]}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "v"}], "}"}], ",", "t"}], "]"}]], "Output",
 CellChangeTimes->{3.6260715523151903`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       FractionBox[
        RowBox[{"2", " ", 
         RowBox[{"v", "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["v", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}]}], 
        RowBox[{"1", "+", 
         SuperscriptBox[
          RowBox[{"v", "[", "t", "]"}], "2"]}]], "+", 
       RowBox[{
        SuperscriptBox["u", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"v", "[", "t", "]"}]}], " ", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox["v", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}]}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "v"}], "}"}], ",", "t"}], "]"}]], "Input",
 CellChangeTimes->{{3.6148618628887453`*^9, 3.6148618629043455`*^9}, 
   3.6148646600984583`*^9}],

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       FractionBox[
        RowBox[{"2", " ", 
         RowBox[{"v", "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["v", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}]}], 
        RowBox[{"1", "+", 
         SuperscriptBox[
          RowBox[{"v", "[", "t", "]"}], "2"]}]], "+", 
       RowBox[{
        SuperscriptBox["u", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"v", "[", "t", "]"}]}], " ", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["u", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox["v", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}]}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "v"}], "}"}], ",", "t"}], "]"}]], "Output",
 CellChangeTimes->{3.6260715523463902`*^9}]
}, Closed]],

Cell["We find a numeric solution:", "Text",
 CellChangeTimes->{{3.598078687809185*^9, 3.5980787026570344`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NDSolve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"dabs", "[", "t", "]"}], "[", 
       RowBox[{"[", "1", "]"}], "]"}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"dabs", "[", "t", "]"}], "[", 
       RowBox[{"[", "2", "]"}], "]"}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{"u", "[", "0", "]"}], "\[Equal]", " ", "1."}], ",", 
     RowBox[{
      RowBox[{"v", "[", "0", "]"}], "\[Equal]", " ", ".2"}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["u", "\[Prime]",
        MultilineFunction->None], "[", "0", "]"}], "\[Equal]", " ", "1."}], 
     ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["v", "\[Prime]",
        MultilineFunction->None], "[", "0", "]"}], "\[Equal]", " ", "1."}]}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5980200989310536`*^9, 3.598020171689582*^9}, {
   3.5980202241992736`*^9, 3.59802039280437*^9}, {3.598020424176025*^9, 
   3.5980205019109616`*^9}, {3.5980205409734297`*^9, 
   3.5980205616278667`*^9}, {3.5980209521589527`*^9, 3.598020953484955*^9}, {
   3.5980210114702563`*^9, 3.598021015027063*^9}, {3.5980783063833685`*^9, 
   3.5980783360160637`*^9}, {3.613286573172347*^9, 3.6132866043161287`*^9}, 
   3.6148618629043455`*^9, 3.6148646601140585`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], "\[Rule]", 
     RowBox[{
      TagBox[
       RowBox[{"InterpolatingFunction", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "1.`"}], ",", "1.`"}], "}"}], "}"}], 
         ",", "\<\"<>\"\>"}], "]"}],
       False,
       Editable->False], "[", "t", "]"}]}], ",", 
    RowBox[{
     RowBox[{"v", "[", "t", "]"}], "\[Rule]", 
     RowBox[{
      TagBox[
       RowBox[{"InterpolatingFunction", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "1.`"}], ",", "1.`"}], "}"}], "}"}], 
         ",", "\<\"<>\"\>"}], "]"}],
       False,
       Editable->False], "[", "t", "]"}]}]}], "}"}], "}"}]], "Output",
 CellChangeTimes->{3.62607155236199*^9, 3.6260759147848525`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"fu", "[", "t_", "]"}], " ", "=", 
  RowBox[{
   RowBox[{"u", "[", "t", "]"}], "/.", 
   RowBox[{
    RowBox[{"Flatten", "[", "%", "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5980105673455124`*^9, 3.598010700881747*^9}, {
  3.5980110599163775`*^9, 3.59801106141398*^9}}],

Cell[BoxData[
 RowBox[{
  TagBox[
   RowBox[{"InterpolatingFunction", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "1.`"}], ",", "1.`"}], "}"}], "}"}], ",", "\<\"<>\"\>"}],
     "]"}],
   False,
   Editable->False], "[", "t", "]"}]], "Output",
 CellChangeTimes->{3.62607155237759*^9, 3.626075919808061*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"gu", "[", "t_", "]"}], " ", "=", 
  RowBox[{
   RowBox[{"v", "[", "t", "]"}], "/.", 
   RowBox[{
    RowBox[{"Flatten", "[", "%%", "]"}], "[", 
    RowBox[{"[", "2", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5980105673455124`*^9, 3.5980107410050173`*^9}, {
  3.598011081772016*^9, 3.598011083410019*^9}}],

Cell[BoxData[
 RowBox[{
  TagBox[
   RowBox[{"InterpolatingFunction", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "1.`"}], ",", "1.`"}], "}"}], "}"}], ",", "\<\"<>\"\>"}],
     "]"}],
   False,
   Editable->False], "[", "t", "]"}]], "Output",
 CellChangeTimes->{3.6260715523931904`*^9, 3.6260759237548685`*^9}]
}, Closed]],

Cell["Here is the geodesic:", "Text",
 CellChangeTimes->{{3.5980787235852313`*^9, 3.5980787307136393`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"rsf", "[", "w", "]"}], "[", 
    RowBox[{
     RowBox[{"fu", "[", "t", "]"}], ",", 
     RowBox[{"gu", "[", "t", "]"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"PlotStyle", " ", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Thick", ",", "Red"}], "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.59793530857513*^9, 3.597935328999298*^9}, {
   3.597935372503786*^9, 3.597935416552306*^9}, 3.5979361503762784`*^9, {
   3.5980107984599185`*^9, 3.5980108206743574`*^9}, {3.5980108954452887`*^9, 
   3.5980109588282003`*^9}, {3.5980111031908536`*^9, 
   3.5980111599125533`*^9}, {3.598020677192869*^9, 3.598020773601039*^9}, {
   3.6132873225702105`*^9, 3.613287327424488*^9}, {3.617195528998773*^9, 
   3.617195531619578*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"geod1", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.598020793662674*^9, 3.5980208002614856`*^9}}],

Cell["Its position on the surface is shown here:", "Text",
 CellChangeTimes->{{3.5980788702026176`*^9, 3.5980788941529875`*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"{", 
   RowBox[{"rsf1", ",", "geod1"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.5980208669204025`*^9, 3.5980208873408384`*^9}}],

Cell["\<\
We control the deviation of the absolute differential from the null vector:\
\>", "Text",
 CellChangeTimes->{{3.6171956229421387`*^9, 3.6171956730962267`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Gamma]", "[", "t_", "]"}], "=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"fu", "[", "t", "]"}], ",", 
     RowBox[{"gu", "[", "t", "]"}]}], "}"}]}], ";", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"DD1", "[", 
     RowBox[{
      RowBox[{"rsf", "[", "w", "]"}], ",", "\[Gamma]", ",", 
      SuperscriptBox["\[Gamma]", "\[Prime]",
       MultilineFunction->None]}], "]"}], "[", "t", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.617195036006707*^9, 3.617195039142313*^9}, 
   3.6171950857239947`*^9, {3.61719513417768*^9, 3.617195172038946*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"2", " ", 
       RowBox[{
        TagBox[
         RowBox[{"InterpolatingFunction", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"-", "1.`"}], ",", "1.`"}], "}"}], "}"}], 
           ",", "\<\"<>\"\>"}], "]"}],
         False,
         Editable->False], "[", "t", "]"}], " ", 
       RowBox[{
        TagBox[
         RowBox[{"InterpolatingFunction", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"-", "1.`"}], ",", "1.`"}], "}"}], "}"}], 
           ",", "\<\"<>\"\>"}], "]"}],
         False,
         Editable->False], "[", "t", "]"}], " ", 
       RowBox[{
        TagBox[
         RowBox[{"InterpolatingFunction", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"-", "1.`"}], ",", "1.`"}], "}"}], "}"}], 
           ",", "\<\"<>\"\>"}], "]"}],
         False,
         Editable->False], "[", "t", "]"}]}], ")"}], "/", 
     RowBox[{"(", 
      RowBox[{"1", "+", 
       SuperscriptBox[
        RowBox[{
         TagBox[
          RowBox[{"InterpolatingFunction", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"-", "1.`"}], ",", "1.`"}], "}"}], "}"}], 
            ",", "\<\"<>\"\>"}], "]"}],
          False,
          Editable->False], "[", "t", "]"}], "2"]}], ")"}]}], "+", 
    RowBox[{
     TagBox[
      RowBox[{"InterpolatingFunction", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "1.`"}], ",", "1.`"}], "}"}], "}"}], 
        ",", "\<\"<>\"\>"}], "]"}],
      False,
      Editable->False], "[", "t", "]"}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       TagBox[
        RowBox[{"InterpolatingFunction", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "1.`"}], ",", "1.`"}], "}"}], "}"}], 
          ",", "\<\"<>\"\>"}], "]"}],
        False,
        Editable->False], "[", "t", "]"}]}], " ", 
     SuperscriptBox[
      RowBox[{
       TagBox[
        RowBox[{"InterpolatingFunction", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "1.`"}], ",", "1.`"}], "}"}], "}"}], 
          ",", "\<\"<>\"\>"}], "]"}],
        False,
        Editable->False], "[", "t", "]"}], "2"]}], "+", 
    RowBox[{
     TagBox[
      RowBox[{"InterpolatingFunction", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "1.`"}], ",", "1.`"}], "}"}], "}"}], 
        ",", "\<\"<>\"\>"}], "]"}],
      False,
      Editable->False], "[", "t", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6260715525335903`*^9, 3.626075967965346*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"%", "[", 
      RowBox[{"[", "1", "]"}], "]"}], "^", "2"}], " ", "+", " ", 
    RowBox[{
     RowBox[{"%", "[", 
      RowBox[{"[", "2", "]"}], "]"}], "^", "2"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.617195254859492*^9, 3.617195319771206*^9}, {
  3.6171953742309017`*^9, 3.6171953918901324`*^9}, {3.6171954242757893`*^9, 
  3.617195429735799*^9}}],

Cell["\<\
The deviation is comparatively small.
Find more geodesics with other initial conditions.\
\>", "Text",
 CellChangeTimes->{{3.5980789447378807`*^9, 3.598078967265169*^9}, {
  3.6171956903654566`*^9, 3.6171957148886995`*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["4.3. Geodesic Curvature", "Subsection",
 CellChangeTimes->{{3.595076945363204*^9, 3.5950769905720835`*^9}, 
   3.5953174793534184`*^9, {3.5953175263051033`*^9, 3.595317587064579*^9}, 
   3.5956008724322104`*^9, {3.5965213907316623`*^9, 3.5965214041788855`*^9}, {
   3.5972984401750546`*^9, 3.59729846072029*^9}, {3.5972984954771514`*^9, 
   3.5972985062723703`*^9}, {3.5973320886172123`*^9, 3.5973320905123205`*^9}, 
   3.5973952636966553`*^9, {3.597729800827932*^9, 3.59772986233884*^9}, {
   3.597910399537461*^9, 3.597910436118553*^9}, {3.5979116271266747`*^9, 
   3.5979116482228813`*^9}, {3.5988859626392813`*^9, 3.5988859896078234`*^9}, 
   3.613287406886033*^9, 3.6145881974242153`*^9, 3.6171958241200914`*^9, 
   3.617440712305024*^9, 3.624544170874772*^9, 3.6262678827716866`*^9, 
   3.626534442696803*^9, {3.626535821629798*^9, 3.6265358232210007`*^9}, 
   3.6267862094512415`*^9}],

Cell[TextData[{
 "Already in Section 3.4 we introduced the ",
 ButtonBox["geodesic curvature",
  BaseStyle->"Hyperlink",
  ButtonData->"geodc"],
 " of a curve on the surface sf[u;v]:"
}], "Text",
 CellChangeTimes->{{3.598943532606222*^9, 3.598943598786008*^9}, {
  3.5989437640264587`*^9, 3.59894377804226*^9}, {3.598943994802658*^9, 
  3.5989440320987916`*^9}, {3.614313265943533*^9, 3.6143132659465327`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"cx", "[", "s_", "]"}], " ", ":=", " ", 
   RowBox[{"sf", "[", 
    RowBox[{
     RowBox[{"hu", "[", "s", "]"}], ",", 
     RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], ";", " ", 
  RowBox[{
   RowBox[{"\[Gamma]", "[", "s_", "]"}], " ", ":=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"hu", "[", "s", "]"}], ",", 
     RowBox[{"hv", "[", "s", "]"}]}], "}"}]}]}]], "Input",
 CellChangeTimes->{{3.5911639056057415`*^9, 3.5911639586847773`*^9}, 
   3.5911644865179677`*^9, 3.618135579701305*^9, {3.6188395679109087`*^9, 
   3.618839573838919*^9}, 3.6188396059437757`*^9, {3.626534531721961*^9, 
   3.626534548803991*^9}}],

Cell["\<\
as the norm of the tangential projection of its curvature vector. We name it \
the \[OpenCurlyDoubleQuote]outer\[CloseCurlyDoubleQuote] definition of the \
geodesic curvature, since the embedding of the curve into the Euclidean space \
is explicitly used. In this subsection we will give the \
\[OpenCurlyDoubleQuote]inner\[CloseCurlyDoubleQuote] definition of the \
geodesic curvature  which is the norm of the absolute derivative of its \
tangent unit vector with respect to its arc length. This definition uses the \
Riemannian metric of  the surface only and can be easily generalized to \
curves in Riemannian spaces.  We show that both definitions yield the same \
geodesic curvature In the case of curves on surfaces in the Euclidean 3-space.\
\>", "Text",
 CellChangeTimes->{{3.5990257160305843`*^9, 3.5990257932975206`*^9}, 
   3.5990258639656444`*^9, 3.618135581074107*^9, {3.6181360567969427`*^9, 
   3.6181360986986165`*^9}, {3.6246050786922965`*^9, 3.624605079066697*^9}, {
   3.626534612011298*^9, 3.6265346489365625`*^9}, {3.6265347162274814`*^9, 
   3.6265347908056126`*^9}, {3.6265348261968427`*^9, 
   3.6265350229895887`*^9}, {3.626535059747254*^9, 3.626535096118918*^9}, {
   3.6265351591666293`*^9, 3.6265352018835044`*^9}, {3.6265352450391808`*^9, 
   3.626535258207604*^9}, {3.626535289427259*^9, 3.6265354337787137`*^9}, 
   3.6265354828*^9, 3.626535515033657*^9, 3.6268544534557924`*^9}],

Cell[CellGroupData[{

Cell["4.3.1. The Inner Definition of the Geodesic Curvature", "Subsubsection",
 CellChangeTimes->{{3.6265355476105146`*^9, 3.6265355961034*^9}}],

Cell[TextData[{
 "\nWe calculate the curvature vector of the curve cx[s] \[Element] ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["E", "3"], ",", " "}], TraditionalForm]]],
 "s the arc length, under this assumption:"
}], "Text",
 CellChangeTimes->{{3.5990257160305843`*^9, 3.5990257932975206`*^9}, 
   3.5990258639656444`*^9, 3.618135581074107*^9, {3.6181360567969427`*^9, 
   3.6181360986986165`*^9}, {3.6246050786922965`*^9, 3.624605079066697*^9}, {
   3.626534612011298*^9, 3.6265346489365625`*^9}, {3.6265347162274814`*^9, 
   3.6265347908056126`*^9}, {3.6265348261968427`*^9, 
   3.6265350229895887`*^9}, {3.626535059747254*^9, 3.626535096118918*^9}, {
   3.6265351591666293`*^9, 3.6265352018835044`*^9}, {3.6265352450391808`*^9, 
   3.626535258207604*^9}, {3.626535289427259*^9, 3.6265354337787137`*^9}, 
   3.6265354828*^9, 3.626535515033657*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"scond2", "[", "sf", "]"}], "[", "s", "]"}]], "Input",
 CellChangeTimes->{{3.626076262571863*^9, 3.626076268047473*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["hv", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}], "+", 
     RowBox[{"2", " ", 
      RowBox[{
       SuperscriptBox["hu", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{
       SuperscriptBox["hv", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["hu", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}]}], 
    "\[Equal]", "1"}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"2", " ", 
      RowBox[{
       SuperscriptBox["hv", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{
       SuperscriptBox["hv", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}], "+", 
     RowBox[{"2", " ", 
      RowBox[{
       SuperscriptBox["hv", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{
       SuperscriptBox["hu", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
     RowBox[{"2", " ", 
      RowBox[{
       SuperscriptBox["hu", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{
       SuperscriptBox["hv", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
     RowBox[{"2", " ", 
      RowBox[{
       SuperscriptBox["hu", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{
       SuperscriptBox["hu", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["hv", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "2"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
        RowBox[{"2", " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "2"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
        RowBox[{"2", " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "2"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], ")"}]}], 
     "+", 
     RowBox[{"2", " ", 
      RowBox[{
       SuperscriptBox["hu", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{
       SuperscriptBox["hv", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "2"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "2"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "2"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"2", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"2", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"2", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], ")"}]}], 
     "+", 
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["hu", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"2", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
        RowBox[{"2", " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"2", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
        RowBox[{"2", " ", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"2", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
       ")"}]}]}], "\[Equal]", "0"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.626076256862253*^9, 3.626076270325077*^9}, 
   3.626161682294077*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"scond2", "[", "sf", "]"}], "[", "s", "]"}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"cx", "[", "s", "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"s", ",", "2"}], "}"}]}], "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5989446013283496`*^9, 3.5989446663580694`*^9}, 
   3.6136210267017107`*^9, {3.6172830626050825`*^9, 3.6172830651634874`*^9}, 
   3.618135583492111*^9, 3.6246084942260823`*^9, {3.624629400197584*^9, 
   3.624629455671282*^9}, {3.6260763368279934`*^9, 3.6260763401975994`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["hv", "\[Prime]\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["hv", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "2"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
    RowBox[{
     RowBox[{
      SuperscriptBox["hu", "\[Prime]\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["hu", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{
      SuperscriptBox["hv", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["hu", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"2", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["hv", "\[Prime]\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["hv", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "2"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
    RowBox[{
     RowBox[{
      SuperscriptBox["hu", "\[Prime]\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["hu", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{
      SuperscriptBox["hv", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["hu", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"2", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["hv", "\[Prime]\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["hv", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "2"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
    RowBox[{
     RowBox[{
      SuperscriptBox["hu", "\[Prime]\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["hu", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{
      SuperscriptBox["hv", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["hu", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"2", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6260764402561755`*^9, 3.6261617306541624`*^9, 
  3.6261719311844788`*^9}]
}, Closed]],

Cell["\<\

Using this simplification we define a formula to calculate the curvature \
vector as the second derivative of the curve cx[s], s its arc length:\
\>", "Text",
 CellChangeTimes->{{3.59894497180654*^9, 3.598945037306286*^9}, {
   3.598945077202568*^9, 3.598945138706086*^9}, 3.6172010145564814`*^9, {
   3.617336690851932*^9, 3.6173367837032948`*^9}, {3.6173369022519894`*^9, 
   3.6173370104041758`*^9}, 3.617441792188218*^9, 3.6181355864405165`*^9, 
   3.618136128666269*^9, {3.6181361761059523`*^9, 3.618136187587572*^9}, 
   3.6246085348953533`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"d2cx", "[", "s_", "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["hv", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"hu", "[", "s", "]"}], ",", 
        RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["hv", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "2"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"hu", "[", "s", "]"}], ",", 
        RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
     RowBox[{
      RowBox[{
       SuperscriptBox["hu", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"hu", "[", "s", "]"}], ",", 
        RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
     RowBox[{"2", " ", 
      RowBox[{
       SuperscriptBox["hu", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{
       SuperscriptBox["hv", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"hu", "[", "s", "]"}], ",", 
        RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["hu", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"2", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"hu", "[", "s", "]"}], ",", 
        RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["hv", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"hu", "[", "s", "]"}], ",", 
        RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["hv", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "2"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"hu", "[", "s", "]"}], ",", 
        RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
     RowBox[{
      RowBox[{
       SuperscriptBox["hu", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"hu", "[", "s", "]"}], ",", 
        RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
     RowBox[{"2", " ", 
      RowBox[{
       SuperscriptBox["hu", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{
       SuperscriptBox["hv", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"hu", "[", "s", "]"}], ",", 
        RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["hu", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"2", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"hu", "[", "s", "]"}], ",", 
        RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["hv", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"hu", "[", "s", "]"}], ",", 
        RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["hv", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "2"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"hu", "[", "s", "]"}], ",", 
        RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
     RowBox[{
      RowBox[{
       SuperscriptBox["hu", "\[Prime]\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"hu", "[", "s", "]"}], ",", 
        RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
     RowBox[{"2", " ", 
      RowBox[{
       SuperscriptBox["hu", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{
       SuperscriptBox["hv", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"hu", "[", "s", "]"}], ",", 
        RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["hu", "\[Prime]",
         MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"2", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{
        RowBox[{"hu", "[", "s", "]"}], ",", 
        RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.5989449039436584`*^9, 3.5989449378745995`*^9}, 
   3.6181355885621204`*^9}],

Cell["Its tangential projection is", "Text",
 CellChangeTimes->{{3.5989458073863325`*^9, 3.5989458423543324`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"scond2", "[", "sf", "]"}], "[", "s", "]"}], ",", " ", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"pT", "[", "sf", "]"}], "[", 
       RowBox[{
        RowBox[{"hu", "[", "s", "]"}], ",", 
        RowBox[{"hv", "[", "s", "]"}]}], "]"}], "[", 
      RowBox[{"d2cx", "[", "s", "]"}], "]"}], ",", " ", 
     RowBox[{"s", ">", "0"}], ",", " ", 
     RowBox[{"TimeConstraint", " ", "\[Rule]", " ", "Infinity"}]}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.598945882152609*^9, 3.598945992372913*^9}, {
   3.5989466625182433`*^9, 3.5989466663334613`*^9}, {3.5989467172543736`*^9, 
   3.598946767455245*^9}, {3.598951350742051*^9, 3.5989513740263824`*^9}, 
   3.598957784087018*^9, 3.59895786157845*^9, 3.59895790475692*^9, 
   3.5990231499040775`*^9, {3.613443680308923*^9, 3.613443723037398*^9}, 
   3.613621042754139*^9, 3.618135592368527*^9, {3.626076655162153*^9, 
   3.626076666066572*^9}, {3.6261617859874597`*^9, 3.6261618344879446`*^9}, 
   3.62616193425012*^9}],

Cell[BoxData["$Aborted"], "Output",
 CellChangeTimes->{3.6261620445891137`*^9, 3.626182142915614*^9, 
  3.626258454414116*^9, 3.626509522079457*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"scond2", "[", "sf", "]"}], "[", "s", "]"}], ",", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"pT", "[", "sf", "]"}], "[", 
     RowBox[{
      RowBox[{"hu", "[", "s", "]"}], ",", 
      RowBox[{"hv", "[", "s", "]"}]}], "]"}], "[", 
    RowBox[{"d2cx", "[", "s", "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.598945882152609*^9, 3.598945992372913*^9}, {
   3.5989466625182433`*^9, 3.5989466663334613`*^9}, {3.5989467172543736`*^9, 
   3.598946767455245*^9}, {3.598951350742051*^9, 3.5989513740263824`*^9}, 
   3.598957784087018*^9, 3.59895786157845*^9, 3.59895790475692*^9, 
   3.5990231499040775`*^9, {3.613443680308923*^9, 3.613443723037398*^9}, 
   3.613621042754139*^9, 3.618135592368527*^9, {3.626076655162153*^9, 
   3.626076666066572*^9}, {3.6261617859874597`*^9, 3.6261618344879446`*^9}, 
   3.62616193425012*^9, {3.6265095320790744`*^9, 3.6265095549643145`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
           ")"}]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
          "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}]}], 
         ")"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
           ")"}]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
          "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}]}], 
         ")"}]}]}], ")"}]}], "+", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
           ")"}]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
          "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}]}], 
         ")"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
           ")"}]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
          "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}]}], 
         ")"}]}]}], ")"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
           ")"}]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
          "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}]}], 
         ")"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
           ")"}]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
          "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}]}], 
         ")"}]}]}], ")"}]}], "+", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
           ")"}]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
          "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}]}], 
         ")"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
           ")"}]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
          "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}]}], 
         ")"}]}]}], ")"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
           ")"}]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
          "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}]}], 
         ")"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
           ")"}]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
          "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}]}], 
         ")"}]}]}], ")"}]}], "+", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
           ")"}]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
          "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}]}], 
         ")"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "2", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "3", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
           ")"}]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
          "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"0", ",", "1"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "1", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "2", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox[
                RowBox[{"f", "[", "3", "]"}], 
                TagBox[
                 RowBox[{"(", 
                  RowBox[{"1", ",", "0"}], ")"}],
                 Derivative],
                MultilineFunction->None], "[", 
               RowBox[{
                RowBox[{"hu", "[", "s", "]"}], ",", 
                RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}]}], 
         ")"}]}]}], ")"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6265095667423353`*^9}]
}, Closed]],

Cell["The  result has been entered into the following definition:", "Text",
 CellChangeTimes->{{3.599023115459217*^9, 3.599023135427252*^9}, 
   3.5990262044050426`*^9, {3.6262679305857706`*^9, 3.6262679869954696`*^9}, {
   3.6262680268691397`*^9, 3.6262681088628836`*^9}, 3.626509452066534*^9, {
   3.6265098531276383`*^9, 3.6265099014877234`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"pTd2cx", "[", "s_", "]"}], " ", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"-", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
             ")"}]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
             ")"}]}]}], ")"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "1", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
             ")"}]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"-", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
             ")"}]}]}], ")"}]}]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}]}], 
       ")"}]}], ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"-", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
             ")"}]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
             ")"}]}]}], ")"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "2", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
             ")"}]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"-", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
             ")"}]}]}], ")"}]}]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}]}], 
       ")"}]}], ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"-", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
             ")"}]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
             ")"}]}]}], ")"}]}], "+", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"f", "[", "3", "]"}], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{
           RowBox[{"hu", "[", "s", "]"}], ",", 
           RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
             ")"}]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"-", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"f", "[", "1", "]"}], 
                  TagBox[
                   RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                   Derivative],
                  MultilineFunction->None], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "-", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"0", ",", "1"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "1", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "2", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"f", "[", "3", "]"}], 
                 TagBox[
                  RowBox[{"(", 
                   RowBox[{"1", ",", "0"}], ")"}],
                  Derivative],
                 MultilineFunction->None], "[", 
                RowBox[{
                 RowBox[{"hu", "[", "s", "]"}], ",", 
                 RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "2"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{"2", " ", 
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"2", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}]}]}], 
             ")"}]}]}], ")"}]}]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "1", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "2", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"0", ",", "1"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}], " ", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"f", "[", "3", "]"}], 
               TagBox[
                RowBox[{"(", 
                 RowBox[{"1", ",", "0"}], ")"}],
                Derivative],
               MultilineFunction->None], "[", 
              RowBox[{
               RowBox[{"hu", "[", "s", "]"}], ",", 
               RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"0", ",", "1"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "1", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "2", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
           SuperscriptBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"f", "[", "3", "]"}], 
              TagBox[
               RowBox[{"(", 
                RowBox[{"1", ",", "0"}], ")"}],
               Derivative],
              MultilineFunction->None], "[", 
             RowBox[{
              RowBox[{"hu", "[", "s", "]"}], ",", 
              RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}]}], 
       ")"}]}]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{3.599023047177897*^9, 3.5990230977843857`*^9, 
  3.61813559378813*^9}],

Cell[TextData[{
 "This is a tangent vector of the surface in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["E", "3"], TraditionalForm]]],
 ", the tangential projection of the curvature vector of the curve ",
 "cx[",
 "s]. By definition its norm is the ",
 StyleBox["geodesic curvature",
  FontColor->RGBColor[0, 0, 1]],
 " of the curve: the ",
 StyleBox["outer definition of the geodesic curvature",
  FontSlant->"Italic"],
 ". Now we show that it coincides with the ",
 StyleBox["inner definition of the geodesic curvature",
  FontSlant->"Italic"],
 ", contained in the\n\n",
 StyleBox["Proposition",
  FontColor->RGBColor[0, 0, 1]],
 ". ",
 StyleBox["The geodesic curvature is the norm of the absolute derivative of \
the tangent unit vector of the curve DD1[sf,\[Gamma],\[Gamma]\
\[CloseCurlyQuote]][s] with respect to the arc length s",
  FontSlant->"Italic"],
 ".  Thus a curve is a geodesic iff its geodesic curvature is zero.\n",
 StyleBox["Proof",
  FontColor->RGBColor[0, 0, 1]],
 "."
}], "Text",
 CellChangeTimes->{{3.59902627165676*^9, 3.599026696726307*^9}, {
   3.5990267275831614`*^9, 3.599026737676379*^9}, 3.5990267771600485`*^9, 
   3.6130242329889708`*^9, {3.6142281112104774`*^9, 3.6142281129576807`*^9}, {
   3.6173570308642416`*^9, 3.617357042295895*^9}, 3.6181355968925347`*^9, 
   3.626258361032352*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"pTd2cx", "[", "s", "]"}], ".", 
     RowBox[{"pTd2cx", "[", "s", "]"}]}], "-", 
    RowBox[{
     RowBox[{
      RowBox[{"fund1", "[", "sf", "]"}], "[", 
      RowBox[{
       RowBox[{"hu", "[", "s", "]"}], ",", 
       RowBox[{"hv", "[", "s", "]"}]}], "]"}], "[", 
     RowBox[{
      RowBox[{
       RowBox[{"DD1", "[", 
        RowBox[{"sf", ",", "\[Gamma]", ",", 
         RowBox[{"\[Gamma]", "'"}]}], "]"}], "[", "s", "]"}], ",", 
      RowBox[{
       RowBox[{"DD1", "[", 
        RowBox[{"sf", ",", "\[Gamma]", ",", 
         RowBox[{"\[Gamma]", "'"}]}], "]"}], "[", "s", "]"}]}], "]"}]}], ",", 
   RowBox[{
    RowBox[{"scond2", "[", "sf", "]"}], "[", "s", "]"}], ",", " ", 
   RowBox[{"TimeConstraint", " ", "\[Rule]", " ", "Infinity"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.598957931526451*^9, 3.5989579547757807`*^9}, {
   3.5990234271945643`*^9, 3.599023494118682*^9}, {3.5990235576575937`*^9, 
   3.5990236292773194`*^9}, {3.599023667949787*^9, 3.5990237026754484`*^9}, 
   3.613445355876666*^9, {3.613445387497921*^9, 3.6134454029107485`*^9}, {
   3.6134463165107527`*^9, 3.6134463361355877`*^9}, 3.6136210445325418`*^9, {
   3.6181355982965374`*^9, 3.618135599372939*^9}, 3.626076522171919*^9, {
   3.626076618470888*^9, 3.626076619843691*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.626077026536405*^9, 3.626164633413661*^9}]
}, Closed]],

Cell["\[FilledRectangle]", "Text",
 CellChangeTimes->{{3.5990269204618998`*^9, 3.5990269204618998`*^9}}],

Cell["\<\
The function pTd2cx[s] is used only once to prove the last proposition and to \
justify the following inner definition of  the geodesic curvature (up to \
sign) of the curve\
\>", "Text",
 CellChangeTimes->{{3.614146476570078*^9, 3.614146480048884*^9}, {
   3.614146537956186*^9, 3.614146561574627*^9}, 3.6141465923690815`*^9, {
   3.6142297053268776`*^9, 3.6142297166836977`*^9}, {3.626268201137046*^9, 
   3.6262682871555967`*^9}, {3.626268326717266*^9, 3.626268358837723*^9}, 
   3.6262684610491023`*^9, 3.626854496083068*^9}],

Cell[BoxData[
 RowBox[{"\[Gamma]", "[", "s", "]"}]], "Input",
 CellChangeTimes->{{3.6141466259559402`*^9, 3.6141466278123436`*^9}}],

Cell["\<\
on the surface ff[u,v], s an arbitrary parameter, by the Module (contained in \
eudiffgeov2.m):\
\>", "Text",
 CellChangeTimes->{{3.614146913948046*^9, 3.614146920047657*^9}, 
   3.6141480432028294`*^9, {3.626268401082597*^9, 3.626268447601879*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"geodcurvature", "[", 
    RowBox[{"ff_", ",", "hu_", ",", "hv_"}], "]"}], "[", "s_", "]"}], ":=", 
  " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "cuv", ",", "t", ",", "aa", ",", "bb", ",", "abc", ",", "vbc", ",", 
      "tt"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"cuv", "[", "t_", "]"}], "=", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"hu", "[", "t", "]"}], ",", 
        RowBox[{"hv", "[", "t", "]"}]}], "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"abc", "[", "t_", "]"}], " ", "=", 
      RowBox[{
       RowBox[{"D", "[", 
        RowBox[{
         RowBox[{"cuv", "[", "tt", "]"}], ",", "tt"}], "]"}], "/.", 
       RowBox[{"tt", "\[Rule]", "t"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"bb", "[", "t_", "]"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"fund1", "[", "ff", "]"}], "[", 
        RowBox[{
         RowBox[{"hu", "[", "t", "]"}], ",", 
         RowBox[{"hv", "[", "t", "]"}]}], "]"}], "[", 
       RowBox[{
        RowBox[{"abc", "[", "t", "]"}], ",", 
        RowBox[{"abc", "[", "t", "]"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"vbc", "[", "t_", "]"}], " ", "=", 
      RowBox[{
       RowBox[{"abc", "[", "t", "]"}], "/", 
       RowBox[{"Sqrt", "[", 
        RowBox[{"bb", "[", "t", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"aa", "[", "t_", "]"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"fund1", "[", "ff", "]"}], "[", 
        RowBox[{
         RowBox[{"hu", "[", "t", "]"}], ",", 
         RowBox[{"hv", "[", "t", "]"}]}], "]"}], "[", 
       RowBox[{
        RowBox[{
         RowBox[{"DD1", "[", 
          RowBox[{"ff", ",", "cuv", ",", "vbc"}], "]"}], "[", "t", "]"}], ",", 
        RowBox[{
         RowBox[{"DD1", "[", 
          RowBox[{"ff", ",", "cuv", ",", "vbc"}], "]"}], "[", "t", "]"}]}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Sqrt", "[", 
      RowBox[{
       RowBox[{"aa", "[", "s", "]"}], "/", 
       RowBox[{"(", 
        RowBox[{"bb", "[", "s", "]"}], ")"}]}], "]"}]}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6141469785009594`*^9, 3.6141471594612775`*^9}, {
   3.614147207197361*^9, 3.614147215184575*^9}, 3.6141472683494687`*^9, {
   3.6141473036055307`*^9, 3.6141473691880455`*^9}, {3.6141474491693864`*^9, 
   3.6141474805566416`*^9}, {3.6141475110702953`*^9, 3.614147680611393*^9}, {
   3.61414772454107*^9, 3.614147742309501*^9}, {3.6141477802799683`*^9, 
   3.6141477893903837`*^9}, {3.614147823055243*^9, 3.61414788398895*^9}, {
   3.6141479364050417`*^9, 3.6141479817075214`*^9}, {3.614151658259579*^9, 
   3.614151816007056*^9}, {3.614235429132931*^9, 3.614235440661351*^9}, {
   3.6142355823876004`*^9, 3.6142355873484087`*^9}, {3.614236105440919*^9, 
   3.61423614015098*^9}, {3.6142572420372305`*^9, 3.614257250040045*^9}, {
   3.614397469072534*^9, 3.614397515295415*^9}, {3.614397546121069*^9, 
   3.6143976442608414`*^9}, {3.6143978611324224`*^9, 3.6143978685112357`*^9}, 
   3.6143981191256757`*^9, {3.6144010659989595`*^9, 3.614401078382668*^9}, 
   3.614401116990876*^9, {3.614401159157288*^9, 3.614401252086603*^9}, {
   3.614401300351364*^9, 3.614401303728557*^9}, {3.614401342224759*^9, 
   3.6144014189041443`*^9}, {3.6144014551352167`*^9, 
   3.6144015880558195`*^9}, {3.614401629657199*^9, 3.614401720151375*^9}, 
   3.614401760607689*^9, {3.614401801610034*^9, 3.614401837968114*^9}, 
   3.6144019127433906`*^9, 3.6144914658978214`*^9, {3.614861863091546*^9, 
   3.614861863107146*^9}, 3.6148631367089825`*^9, 3.6149177943229656`*^9, {
   3.626509981671864*^9, 3.626510000735098*^9}, {3.626510387022976*^9, 
   3.626510398535796*^9}}],

Cell[BoxData[
 RowBox[{"?", "geodcurvature"}]], "Input",
 CellChangeTimes->{{3.62460558198038*^9, 3.6246057409134593`*^9}, {
  3.6246057820195312`*^9, 3.6246058622660723`*^9}, {3.6262687584324245`*^9, 
  3.6262687645632353`*^9}}],

Cell[TextData[{
 "If t is the arc length of the curve,  then bb[t] == 1, and the definition \
corresponds to the result of the proposition.  For arbitrary parameters the \
norming defines the tangent unit vector vbc[t], and ensures that the \
expression is invariant under a parameter transformations. Obviously, by this \
definition the geodesic curvature is always non-negative. \n",
 StyleBox["Remark.",
  FontColor->RGBColor[0, 0, 1]],
 " With an adapted definition of the derivative DD1 the geodesic curvature \
may be easily generalized to curves in a Riemann manifold. We leave this to \
the user. \n"
}], "Text",
 CellChangeTimes->{{3.59902816170928*^9, 3.599028196403741*^9}, {
   3.6141482435071816`*^9, 3.614148310478099*^9}, {3.614430191294223*^9, 
   3.61443019195026*^9}, {3.614430470191175*^9, 3.614430667510461*^9}, {
   3.6144307257667933`*^9, 3.6144308274956117`*^9}, {3.6174418306204166`*^9, 
   3.6174418465883303`*^9}, 3.6188394259034586`*^9, 3.6252210520663753`*^9, {
   3.6262689136370974`*^9, 3.626269023820091*^9}, {3.626269059013753*^9, 
   3.626269065596964*^9}, 3.626269134265447*^9, {3.6262691845852957`*^9, 
   3.626269185646098*^9}, 3.626269238225794*^9, 3.626854508188689*^9}],

Cell[TextData[{
 "We show\n",
 StyleBox["Lemma",
  FontColor->RGBColor[0, 0, 1]],
 ". ",
 StyleBox["The absolute derivative \
DD1[sf,\[Gamma],\[Gamma]\[CloseCurlyQuote]][s], s the arc length of the \
curve,  is orthogonal to the tangent vector \[Gamma]\[CloseCurlyQuote][s], s \
the arc length",
  FontSlant->"Italic"],
 "."
}], "Text",
 CellChangeTimes->{{3.599028227837796*^9, 3.599028350376012*^9}, {
   3.6144300781997547`*^9, 3.6144300966158075`*^9}, 3.617360986151471*^9, 
   3.617361063695906*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"scond2", "[", "sf", "]"}], "[", "s", "]"}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"fund1", "[", "sf", "]"}], "[", 
       RowBox[{
        RowBox[{"hu", "[", "s", "]"}], ",", 
        RowBox[{"hv", "[", "s", "]"}]}], "]"}], "[", 
      RowBox[{
       RowBox[{
        RowBox[{"DD1", "[", 
         RowBox[{"sf", ",", "\[Gamma]", ",", 
          RowBox[{"\[Gamma]", "'"}]}], "]"}], "[", "s", "]"}], ",", 
       RowBox[{
        RowBox[{"\[Gamma]", "'"}], "[", "s", "]"}]}], "]"}], ",", 
     RowBox[{"s", ">", "0"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5990270185548725`*^9, 3.5990270848237886`*^9}, {
   3.59902713120267*^9, 3.599027132887473*^9}, {3.5990278914232054`*^9, 
   3.599027893310809*^9}, 3.613449468246689*^9, 3.6134495002267447`*^9, 
   3.6134495381348114`*^9, 3.6134495822360888`*^9, 3.613449621048957*^9, 
   3.613621310653409*^9, 3.6188396723842926`*^9, {3.626258652175663*^9, 
   3.626258723545789*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.626258756352646*^9}]
}, Closed]],

Cell[TextData[{
 "\[FilledRectangle]\nNow, as in case of the plane, we can provide the non \
zero geodesic curvature with a sign: If the orientation of the tangent planes \
defined by the pair {\[Gamma]\[CloseCurlyQuote][s], DD1[sf,\[Gamma],\[Gamma]\
\[CloseCurlyQuote]][s]}  coincides with the orientation defined by the \
parameters {tv[1][sf],tv[2][sf]} we take the curvature as positive, and as \
negative if this is not the case. In this sense the",
 StyleBox[" oriented geodesic curvature",
  FontColor->RGBColor[0, 0, 1]],
 " is defined."
}], "Text",
 CellChangeTimes->{{3.599028678070187*^9, 3.5990289698842993`*^9}, {
   3.5990290095707693`*^9, 3.599029014515978*^9}, {3.599029058773256*^9, 
   3.599029104512536*^9}, 3.617600026709092*^9, 3.617600085848796*^9, {
   3.6188408953452406`*^9, 3.618840921288086*^9}, {3.626259940472726*^9, 
   3.6262599431715307`*^9}}],

Cell["\<\
As a test for the correctness of the definitions of curvature, normal, and \
geodesic curvature we show by a lengthy symbolic calculation that the next \
expression equals zero:\
\>", "Text",
 CellChangeTimes->{{3.6136215027053466`*^9, 3.613621632201174*^9}, {
   3.61414866399032*^9, 3.614148664349121*^9}, 3.614313586866888*^9, 
   3.6262610545486827`*^9, {3.6262695467855377`*^9, 3.626269568221976*^9}, {
   3.626532300900201*^9, 3.6265324546228714`*^9}, {3.6265325839281893`*^9, 
   3.626532633659482*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"normalcurvature", "[", 
     RowBox[{"sf", ",", "hu", ",", "hv"}], "]"}], "[", "s", "]"}], "^", "2"}],
   "+", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"geodcurvature", "[", 
     RowBox[{"sf", ",", "hu", ",", "hv"}], "]"}], "[", "s", "]"}], "^", "2"}],
   "-", 
  RowBox[{
   RowBox[{
    RowBox[{"curvature", "[", "cx", "]"}], "[", "s", "]"}], "^", 
   "2"}]}]], "Input",
 CellChangeTimes->{{3.599029134745389*^9, 3.599029405593065*^9}, 
   3.599040146992449*^9, {3.5990401854777164`*^9, 3.5990401961013346`*^9}, 
   3.613621085654214*^9, 3.613621484016514*^9, {3.6136232678952465`*^9, 
   3.6136232769900627`*^9}, {3.6136268909080105`*^9, 3.61362689666442*^9}, 
   3.614861863153946*^9, 3.6148631367089825`*^9, 3.626260981883755*^9, 
   3.6265224841950035`*^9, 3.626522529771884*^9, {3.6265226969417777`*^9, 
   3.62652278345953*^9}, {3.626522826281605*^9, 3.6265228681364784`*^9}, {
   3.626523443496689*^9, 3.6265234495807*^9}}],

Cell[BoxData[
 InterpretationBox[
  TagBox[
   PanelBox[GridBox[{
      {
       StyleBox[
        StyleBox[
         DynamicBox[ToBoxes[
           FEPrivate`FrontEndResource["FEStrings", "sizeExplanation"], 
           StandardForm],
          ImageSizeCache->{900., {7., 26.}}],
         StripOnInput->False,
         DynamicUpdating->True], "Panel",
        StripOnInput->False,
        Background->None]},
      {
       ItemBox[
        TagBox[
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "+", 
                    RowBox[{
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], " ", 
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}], 
                    "+", 
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}]}], 
                    ")"}]}]}], ")"}], "/", 
                  RowBox[{"(", 
                   RowBox[{"\[Sqrt]", 
                    RowBox[{"(", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                    RowBox[{
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], " ", 
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}]}], 
                    ")"}], "2"], "+", 
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                    ")"}], "2"], "+", 
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                    RowBox[{
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "[", 
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                    "]"}]}], "+", 
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}], 
                    ")"}], "2"]}], ")"}]}], ")"}]}], "-", 
                 FractionBox[
                  RowBox[{
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "+", 
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}], 
                    ")"}], " ", 
                   RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}], 
                  RowBox[{"2", " ", 
                   SuperscriptBox[
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                    RowBox[{
                    "\[LeftSkeleton]", "1", "\[RightSkeleton]"}]]}]]}], ")"}],
                "2"], "+", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 FractionBox[
                  RowBox[{
                   RowBox[{
                    RowBox[{
                    SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}], 
                   "+", 
                   RowBox[{"\[LeftSkeleton]", "3", "\[RightSkeleton]"}]}], 
                  SqrtBox[
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                    ")"}], "2"], "+", 
                    SuperscriptBox[
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                    "2"], "+", 
                    SuperscriptBox[
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                    "2"]}]]], "-", 
                 FractionBox[
                  RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                  RowBox[{"2", " ", 
                   RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}]]}], 
                ")"}], "2"], "+", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 FractionBox[
                  RowBox[{
                   RowBox[{
                    RowBox[{
                    SuperscriptBox["hv", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                    RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                   RowBox[{"\[LeftSkeleton]", "2", "\[RightSkeleton]"}], "+", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                    ")"}]}]}], 
                  SqrtBox[
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}], 
                    "+", 
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}], 
                    ")"}], "2"], "+", 
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                    ")"}], "2"], "+", 
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "+", 
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}], 
                    ")"}], "2"]}]]], "-", 
                 FractionBox[
                  RowBox[{
                   RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                   RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                   RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}], 
                  RowBox[{"2", " ", 
                   RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}]]}], 
                ")"}], "2"]}], ")"}], "/", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"], 
              "+", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "2", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"], 
              "+", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], " ", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "3", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}], ")"}], "2"]}],
              ")"}]}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{
                   RowBox[{"hu", "[", "s", "]"}], ",", 
                   RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "2", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                    ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{
                   RowBox[{"hu", "[", "s", "]"}], ",", 
                   RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}], 
                  "]"}], "2"], "+", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "3", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"1", ",", "0"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{
                   RowBox[{"hu", "[", "s", "]"}], ",", 
                   RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}], " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], ")"}], 
               "2"]}], "+", 
             RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "+", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "2"], 
                "+", 
                SuperscriptBox[
                 RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "2"], 
                "+", 
                SuperscriptBox[
                 RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "2"]}],
                ")"}], " ", 
              RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}]}], ")"}],
            "/", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "1", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{
                   RowBox[{"hu", "[", "s", "]"}], ",", 
                   RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"], "+", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "2", "]"}], 
                   TagBox[
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{
                   RowBox[{"hu", "[", "s", "]"}], ",", 
                   RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}], 
                  "]"}], "2"], "+", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"f", "[", "3", "]"}], 
                   TagBox[
                    RowBox[{"(", 
                    RowBox[{"0", ",", "1"}], ")"}],
                    Derivative],
                   MultilineFunction->None], "[", 
                  RowBox[{
                   RowBox[{"hu", "[", "s", "]"}], ",", 
                   RowBox[{"hv", "[", "s", "]"}]}], "]"}], "2"]}], ")"}]}], 
             "+", 
             RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "+", 
             RowBox[{
              SuperscriptBox[
               RowBox[{
                SuperscriptBox[
                 RowBox[{"\[LeftSkeleton]", "2", "\[RightSkeleton]"}], 
                 "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "2"], 
                "+", 
                RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "+", 
                RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}], 
               ")"}]}]}], ")"}]}], "+", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], ")"}], 
            "2"], "/", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"f", "[", "1", "]"}], 
                    TagBox[
                    RowBox[{"(", 
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                    ")"}],
                    Derivative],
                    MultilineFunction->None], "[", 
                   RowBox[{
                    RowBox[{"hu", "[", "s", "]"}], ",", 
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}], 
                   "]"}], "2"], "+", 
                 SuperscriptBox[
                  RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "2"], 
                 "+", 
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}],
                    MultilineFunction->None], "[", 
                   RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                   "]"}], "2"]}], ")"}]}], "+", 
              RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "+", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "2"], 
               " ", 
               RowBox[{"(", 
                RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                ")"}]}]}], ")"}], "2"]}]}],
         Short[#, 20]& ],
        Background->GrayLevel[1],
        BaseStyle->{Deployed -> False},
        Frame->True,
        FrameStyle->GrayLevel[0, 0.2],
        StripOnInput->False]},
      {
       RowBox[{
        ButtonBox[
         StyleBox[
          StyleBox[
           DynamicBox[ToBoxes[
             FEPrivate`FrontEndResource["FEStrings", "sizeShowLess"], 
             StandardForm],
            ImageSizeCache->{144., {1., 26.}}],
           StripOnInput->False,
           DynamicUpdating->True], "Panel",
          StripOnInput->False,
          Background->None],
         Appearance->Automatic,
         ButtonFunction:>OutputSizeLimit`ButtonFunction[
          Identity, 93, 24755564108025159747, 20/2],
         Enabled->True,
         Evaluator->Automatic,
         Method->"Queued"], "\[ThinSpace]", 
        ButtonBox[
         StyleBox[
          StyleBox[
           DynamicBox[ToBoxes[
             FEPrivate`FrontEndResource["FEStrings", "sizeShowMore"], 
             StandardForm],
            ImageSizeCache->{165., {1., 26.}}],
           StripOnInput->False,
           DynamicUpdating->True], "Panel",
          StripOnInput->False,
          Background->None],
         Appearance->Automatic,
         ButtonFunction:>OutputSizeLimit`ButtonFunction[
          Identity, 93, 24755564108025159747, 20 2],
         Enabled->True,
         Evaluator->Automatic,
         Method->"Queued"], "\[ThinSpace]", 
        ButtonBox[
         StyleBox[
          StyleBox[
           DynamicBox[ToBoxes[
             FEPrivate`FrontEndResource["FEStrings", "sizeShowAll"], 
             StandardForm],
            ImageSizeCache->{122., {1., 26.}}],
           StripOnInput->False,
           DynamicUpdating->True], "Panel",
          StripOnInput->False,
          Background->None],
         Appearance->Automatic,
         ButtonFunction:>OutputSizeLimit`ButtonFunction[
          Identity, 93, 24755564108025159747, Infinity],
         Enabled->True,
         Evaluator->Automatic,
         Method->"Queued"], "\[ThinSpace]", 
        ButtonBox[
         StyleBox[
          StyleBox[
           DynamicBox[ToBoxes[
             FEPrivate`FrontEndResource["FEStrings", "sizeChangeLimit"], 
             StandardForm],
            ImageSizeCache->{216., {1., 26.}}],
           StripOnInput->False,
           DynamicUpdating->True], "Panel",
          StripOnInput->False,
          Background->None],
         Appearance->Automatic,
         ButtonFunction:>FrontEndExecute[{
            FrontEnd`SetOptions[
            FrontEnd`$FrontEnd, 
             FrontEnd`PreferencesSettings -> {"Page" -> "Evaluation"}], 
            FrontEnd`FrontEndToken["PreferencesDialog"]}],
         Evaluator->None,
         Method->"Preemptive"]}]}
     },
     DefaultBaseStyle->"Column",
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxDividers->{
      "Columns" -> {{False}}, "ColumnsIndexed" -> {}, "Rows" -> {{False}}, 
       "RowsIndexed" -> {}},
     GridBoxItemSize->{
      "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[1.2]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    DefaultBaseStyle->{},
    FrameMargins->5],
   Deploy,
   DefaultBaseStyle->"Deploy"],
  Out[93]]], "Output",
 CellChangeTimes->{
  3.626522515497859*^9, 3.62652261310723*^9, {3.6265234308918667`*^9, 
   3.6265234523419046`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Together", "[", "%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.626523499890788*^9, 3.6265235053975973`*^9}, {
  3.6265236172341943`*^9, 3.626523629558216*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6265306077156725`*^9}]
}, Closed]],

Cell["\<\
In the next examples we consider some curves on specifier surfaces, calculate \
their curvatures and  control this relation again.\
\>", "Text",
 CellChangeTimes->{{3.6136216629488277`*^9, 3.6136217472669764`*^9}, {
   3.6136219986458178`*^9, 3.6136220101274376`*^9}, {3.6136316409915533`*^9, 
   3.6136316447511597`*^9}, {3.613631706792469*^9, 3.6136317217996955`*^9}, 
   3.613631784777006*^9, 3.626269306869915*^9, {3.6262693387115707`*^9, 
   3.626269339990773*^9}, {3.6262695332895145`*^9, 3.626269541169528*^9}, {
   3.6265230062435207`*^9, 3.6265230231539507`*^9}, {3.6265326641715364`*^9, 
   3.6265327806181417`*^9}, {3.6265328112917957`*^9, 3.6265328147590017`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["4.3.2. Circles on the Sphere", "Subsubsection",
 CellChangeTimes->{{3.6265356696031294`*^9, 3.626535718252015*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"sphero", "[", "r", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6144904863257933`*^9, 3.6144904977664475`*^9}, 
   3.6265861898818274`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"r", " ", 
    RowBox[{"Cos", "[", "u", "]"}], " ", 
    RowBox[{"Cos", "[", "v", "]"}]}], ",", 
   RowBox[{"r", " ", 
    RowBox[{"Cos", "[", "v", "]"}], " ", 
    RowBox[{"Sin", "[", "u", "]"}]}], ",", 
   RowBox[{"r", " ", 
    RowBox[{"Sin", "[", "v", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.626261078463525*^9, 3.6265104896555567`*^9, 
  3.626586192923833*^9}]
}, Closed]],

Cell[" We consider the circles", "Text",
 CellChangeTimes->{{3.6141488938255234`*^9, 3.6141489033727407`*^9}, {
   3.6143272794730387`*^9, 3.6143272834572663`*^9}, 3.6144904761582117`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"h1", "[", "t_", "]"}], " ", ":=", " ", "t"}], ";", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"h2", "[", "ca_", "]"}], "[", "t_", "]"}], " ", ":=", " ", "ca"}],
   ";"}]], "Input",
 CellChangeTimes->{{3.6141489619820433`*^9, 3.614149121149123*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sphero", "[", "1", "]"}], "[", 
    RowBox[{
     RowBox[{"h1", "[", "t", "]"}], ",", 
     RowBox[{
      RowBox[{"h2", "[", 
       RowBox[{"Pi", "/", "4"}], "]"}], "[", "t", "]"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", " ", 
   RowBox[{"PlotStyle", " ", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{"Thick", ",", " ", "Red"}], "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6141489619820433`*^9, 3.614149089215867*^9}, {
  3.6141491303999395`*^9, 3.614149285058611*^9}}],

Cell[BoxData[
 Graphics3DBox[{{}, {}, 
   {RGBColor[1, 0, 0], Thickness[Large], Line3DBox[CompressedData["
1:eJxF1Xk41VkYB3ATYZQYaSRqUJYnbkxP5lH3TifboGHsZO9hXFSMlNKMFuFm
yeBmS1xlHyVbtxljOdfYyn5tWXK524+4d6IGT0Yaf4xz/nif85zPH+c57/t8
n3M0/X9yCtwmISFhuVmSm5VDvkllPCSANKk9luMWdbLn/z3TtHvoQebm2r1g
VmRfCrbctTJnjHGZAE/M1TI7LJ4jHy65t59kQoAb3oZGauotyBuWNTiOPCGg
g/wc022dyKM1E0o9PwlAh0ubRvDUAPKyiD7lrH4+sLI5dLd8agR5j/RV6Qgj
Hig+mRtzcGEC+Ya1USfhPwMk30UcsDOexuc0sm8/sX0NRmz6y23Luchbk2k5
QSGvwHp3H5/iw0ce5rjqJhE9CM6ydTwDygTIE+Y/RqiVvwTx1lp0yToh8myu
sSHZuA0QdW5egbsI5EWVfreKvJuAyr3R0Tln7LRqRp/uuVqQZPAiOv029rXI
lLRbOfmgV17ucOcD7A41a+5yrnRob7Gys5WBXeSlbRWTXA3b2/fe8EvArp06
Netb3ABDjRX0q7yxe7ZL1R9wbIVx+f9wdNWw0/d6zzONXkJaQb80qwH3Zegx
Tg183wfXggyWgsyx0xcbdzXPD0NAEcw0KeL5/LB4qWT343FoUSFfwizkISev
U2VO+HOgKOkDTdw2g/xt/M2AOh0u3PPbkl5b3BTyhQS1nt8/8ODlwtgd70rH
cN6Uvg6ZCxNAI5ntqX+FDyFXkaJOyl0TQiWmb+zbpB7knyL3PK0hhPCOX4Gj
NexAzrexNvUgE3BZT/i4YpyFXHVo9u/oSwTcd4T3y48pOM/BV6zGbTIJKLFU
ZCBdiPM/MaklyHtIwNn1iusedxyQf6nef8Y2i4BdGe9lFXJL4JbvLojWrIog
4IfMOJI/6zly5p8L1D5jAmocebSoaNeCfN5gfjF88/5V2++bkVo7kPuo5gXl
xQih7/kLumtaPcjtrb44bu8ngOaWCfvYNUPIj7rYHFef5UGybvU0Z3oMeY0w
iUOjcGHiDbfhJscp5JlJduw1Sw6s1VsgP2XPII+95OSuxBmHRnOWcTvTechj
gkUXN/RGIDu5kgiUECDfaNRU/pzaD7tZBZoCEyHy0CaWTPa+Lngo6TT5WgX2
78NleM+YbbDiMx2fJWkCueQ7R5mTd5vgI7F7gdgJu72UrgkluQZaeqXEe8Rj
p2qrshzk78N82bZ0nXzsOYOqp5z1M8CzKMpHXwb2QpM3Oy+qVQOuvs63xYnY
ZenRzGz1BnDslbbYyg87X/Qiy/ZqKzDJi1xQUMUuBhQLp9MvAXdKJC+ox331
ftX7YoeIDcTnSfslO/F8ogKj3LqcXwH/tAlySwAf+TZl/oX646+BFaP26EwV
F7nFN4pmzqQZoHVOqce1eBp5szi02eUgD1B+Hhu5uGsSeZpz0UppJR8UDrj0
vnEbRe7KWpWLEghA1qDG9PJZNvIgUSm8MikEvSqHTzFCOpEP/Bphf4ZEAKlj
q5aXV3E+tXNfpyaGbJ4noH0XPvsH8qWwt/8O0wkA4ImMspVyPP+td7Tsetzm
/9K85f8B7oN8yA==
     "]]}},
  Axes->True,
  Method->{},
  PlotRange->{{-0.7071060615334048, 0.7071067811865417}, {-0.7068324037570819,
    0.7068801167644184}, {0., 1.414213562373095}},
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.626510517127205*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"circ", " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.6141493241678796`*^9, 3.6141493321082935`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sphero", "[", "1", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{
      RowBox[{"-", "Pi"}], "/", "2"}], ",", 
     RowBox[{"Pi", "/", "2"}]}], "}"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "None"}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"Opacity", "[", "0.7", "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6141493439487143`*^9, 3.6141494448340917`*^9}, {
  3.614149700861741*^9, 3.6141497219685783`*^9}, {3.6142600151061015`*^9, 
  3.6142600366653395`*^9}}],

Cell[BoxData[
 Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJx1m3lcz9n3x0N+Zsg6Y5kxDYVkkCWylE4MQtayi6HQNikMRihkq+zMhIhk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   "], {
    {Opacity[0.7], EdgeForm[None], 
     GraphicsGroup3DBox[Polygon3DBox[CompressedData["
1:eJxNm3f819Mexz/fYxMapCTtQUpLE/0KmaWhIdGyqTSISoNkZ5P2kNIgWRk3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       "]]]}, {}, {}, {}, {}},
   VertexNormals->CompressedData["
1:eJx1W2lYT93bbUJ5VGgwRCFkDEkifseQJBRlDJVSoVGpkBQhDSJEac5QSEil
kt9JExFFqdCkEhqllNLwnrXPyfX/8j4fni7Lbe2117n3ve99zm6yqZ2+uZCA
gEAb8z9h5ueHdVVdT4sv8L6MuWJQKc9bLjyA/1ppaVGnac/v+/FKhobuUYtw
5Q3iGWWifrfLPHlXxOZt/Flz7h9evbdn/Iit1rzo3umFNqIX/uFmDanNklut
+RPLNgYBF+Lw3Ss1t8SUefIbh5283/o/PBZhe64w4/IfOgkEqDLjDsYf2kh0
8s91TbVPVP+TPhg/oXyKN+J1Dt2gmXj+YPzJP79XgN/UuD6d4ecPxj8aV38Z
epIFbcMYPf/iqZi/HtBPtW159b+49FGraMz3ochWjZ//w6OgV7cA/vhndM1k
/PmHh+qwfvr9+WI1tXjps3+4xOqAS675lIpqyHgBD1eebfvrphCVVnrh1OVn
j93Jox677hhm/iSFWtzxiuBfpqXLq2c+p34rn9B7qZlLvT1aSPA96coa4w3v
UI47HMQzs19T8iJvCL707uKHEw3v0Ec79T8/Z3CJqBcEl31u/47hoU2q+jNe
MDzGG1ie84u/fmfGpZNdbtqYMePm1LDjNksTnXRgUHOAbKMafzWn0/lPXCXi
R3xMC2Li6W+lLwmuK1lNg39ipUABw08bcfw2b55vg55Xc12qGT1093hWj077
8GvQX+imKsropwf1Z2dYqWC+NoW6K5j50tIhrB7vwPsX4U9oat/nfcy4g/44
SLJ+ftqxULGp3YU/6Oca3sKWjOzPVMU6uyECHj48uXGynunFTXTVhq6kLJGP
VF/NO6mMsS+pplMCBDf93J2nsekdpWpwTPdj0gfq+opFBF/5bGyDCePLjgVP
i86M/ky9ujyW4HPdFddBx7ZZ26OBx6ydTPCcH8OOLNv0jubLCWWXMTzrJk8j
+IMdb2WzRT7SHWpjPPjMuNvVZQjetJropK/aHPaQbdTinxzL6pQIWCmO+AK3
jBgmnh6Mz79abAH+qJup7xh+epC/5XdtK57H1wrbGEYPPahnTNRG3l5G/33F
5rfAX3L6V5zxqMB8u31EVjDzpfMfDpwC3hlq9xz+CGjOq6OZcRs4f/o4P+vl
VZ5tNvfij+P83Ps+VC6qvZY6cUuWKSOXecNjPddmifygdx6da13Y/YUyaxnb
w+ilnmy9SfCCN2NM5b6UUSHVEfWhKlXUq3FtBA9LbbVUY5670UTjsZccaymH
jbO1gYvct8tDXh1+EdAZwOAWv/aT+AgT8d0TvpTRCUffTQLPu2EuBPcYuv8R
My5tdcCoEvMbXdWrBbzvA9FJh8zpvvPSX5f/+zarsyK4Lhbxhg73uzTga3Uq
Gbditcoq8DcNt1/E8NOD/Ak6cwOg59a1F92MHtqc09OaHngT+qt3lssy+ukY
tRgy7gvX18cw3/ENTe/Bk8fNd67FaEf40zXJpQLPM5Hzp4rzc5Vd9Pziyov8
YZyfYeN2X2N+UofeSDAVO4hXqOf2JJKZT6myZ4ubTz213dUqAXk70iOH4HMz
/u4MYXzx38k773ChlmpL/5IM3Jd2Gchm1r+WV6zAvH3fqQqxChLvWhkqgvWW
Hfi1SJnBR/++nQI8tfJ1AHSbmUmOAo/lzXTCs+q/5ZonfOppz8gjesjz0dHH
CM/FiUQnbaCWYTffeAvfWZfVKS0WpoF40UAdR+TtYHz34yJ78PPuik5k+OkV
NTUEL54sRaOObDGqKGX00KM4PTvMZPRzGP0yKhK9wHMXp5H4zSeFj2K+92sq
PcDTzM333uTFnfDH4tmxi8jzIzobCY/2eNZPo31zNLxlA/lvOD95U6zSmLym
mr1CBQU8QnivF1mKYn2K1xgrF3b/oLx37i5j1iW11CxsGPC6BXECeI5/m0pP
M/OjSjUXiwFfvqfYyoWpVzuXezY8kGui2mVDhgK3lj5fjfrpvoZOBp7glP8f
cNWnNo+Rh2XnjYjerMDfZNzElC2XmHHpc+XrNbAuRwdkDgEuP53opHf2jHWT
bdzG11nI6sy7vvwc4p1iF5oz8bQUF79s5ptI8NftXjIEz2GQ3ykr3Qd6Mt/c
fMrooR9wejxmj3kO/Wtu9dcCX7ljgzjw6K3OozDfKK3DLuAp4ua7LLlLHf58
EzGKQ91S5fzp5vzcd/1Mwf41Qfxszk/vfdLDQ1VaKRP1Sqb7COfF3Bx98SKz
3wRZ6+6HLy/Orb94lqmrnxePDwTeo1a4Duv/iJyWGvL2v+U9Z4Dn8C0X8pl9
Tspb7IjRwRYq/MXMa8Bjbhz+jf2mv0l+JXB+2hmCjzZxHIH1efDq6WDk+VuP
ESHAnUb9KMI8rRaGFZ5mxq3OmhwEXHc/0Un/aFnu0+u8kx8TxepsOzkvD/GT
opX6mHj6MxcvVlL+B3XhQ4LgXeSny8MZF4BPcda/BD2Thr3QYvTQaZye5Plu
/tDfOaTUAfg1Tn/L/s97MF+dP1bTmPnSoeKVAcAfNCY5wp8TGS0mjD90KefP
XnPWz2Kvd76Jq67zIzk/jXpJ30H144d7OG9AkLFbYAVvds9XXTyXW0Y/c58z
db1BdLNJZ81K3ouKzO3MOqA0tj+uxfo4Of7pRJXjWryE22MS/Zl9dp3B8mkY
55MFdW6W2wqeteTsC+gTVng3YN+lJoS1+AAfOKYxCfV22dni1eAJExBVAY/5
5p3PkLdqTzR6aWbc6wqqRhi3n+uPtuie9IbP7J8p3tnApQmIX2ZwcBbyZnbg
QRIf3jd6BPjbqr8ZoN6steubB/5dETpzocdrrccv9AH0kbtEz1O/RpkLjP6a
DbEKeJ6lpzyJ/qFK/daYb2tZ9nvw9Lh/UQCPfsdeQ/jzxIt3jfGHVng9lvgz
o4/1U10vlRLvuc5v5/y05fNE4cvv4PHM34fxhugtecj4T307MMcSz6thkeoF
5PPb7Yr3gMcoT9PG8x0SqayKfP65rSIR+Jb9dirIh3Nnu52Rt02mq+8C/7Ph
UgfyZ4Gw0grgFXNbYoE3LxIn+TznmAjJ59UzNW8Dz5J48R75eaP6TAHyObR2
VjxwlUyik95+lX8OPotsYHWucBd/ifjteiN7kc+D8fSY8C7kc7GgQyzy+UK3
UgJwTwXzAOhpH5m1Bnkrnpl7B/jKT6fPQ7+V8ItDwBs5/fwOUZLPt0ftm4p8
1jvbTfQPfL/sAH9u2JQaI5+zOX+0aNbPE6paPsjnPl1WZ+X0hmQ8lyO/NfsH
3EN4aw52yKIvkY6vmYX6c0NS4APqs1d0whjgO3aNGihg6pVyRvEp1OdxH79J
Af/xQdMS9U3N6vRXjB9wr0YO+L21iaS/dbZMTQReeOKgNHAR7cWkr4ikgtpR
nzsesPyN76rPo97u7xJcjPqs3KM9HvizWUQnrdKudwz1OceS1bnE2dUT8Tcs
w01Qn8WiH5FxRyZfDQf/PQlHIdTVVo4/rbP2HPQ8aQ1PxfN5zenZKGxBQ/93
lak1wJcfXCADXMr9syTq84dxh5zAI83Nd9WpY4vgz7CngndQn09x/gQrsX4G
Hr+Uj/pMcX7OrPx0Gev2zrgEZile432yrHmKdeJuubMJ8x87YdtD9Bu3eS/T
gBtGRW7B/jvmQY03+oSOc5rpwMu09vWi35jdv64P+Wlhr0Jws5LJwug3zlpr
vQfe8DOd8FyYL+GHfuBas5M4eF77nSLxi29UrcB8PnYu3YB+o9FrJsHlaohO
Wi7yqzX6DXcLVqd4FKWOeMPGEfboN5q4+LKlxjbgV5WvHI8+oU13N4nPGhf0
DP2G7CJeCfL8B6dH+ubJTeg3tnQa9AB/OKQ9FfgR01hHzFdd3dMNPL+4+Vrp
aJD8+PZfyHn0G/XmhwlexPlZLq+ujn6jhPNzRn6ONPq9P1MvM5X6Es9A/M8G
xLUVvTXHc8w27mxH//zq7tqNwGX3rTNEP9nS9qEcfe+xKa0kPp4+bob+0zTA
XQr9c3+OEsElfdZnoV+dlFfaivWXfvIHwZcHeeujv20fVysNnoc5foRfYK/Y
PeThgs8aZeifp6lGkvicAqKTDjBwj0D/TI1gdfomH7+JeH0Fu3b0z1O5+GNv
o5aA/0xewiz4LT+zjeDvlMR8oWdzsHg76spTTs8Jh4xQ6Ff4sVYC9Syn/B7R
s3si7Yj57tOVfQWeL2KOJF54vj45Xwy93lmC/tl6aDiJj+H85Bd/nIX+eRPn
Z8Wmkirk9dRtaV0D7j68xMO+Z5H37o/TYrFu5WeMHobzYPIkH4LPDPN+jvm/
cf66GnkueNaF4LpLdStwHrzqm5KDc5/Yfn+CH+/ZsQznwfunt1wBXi93jeCL
712zg77rlE0q8jbBwpLgE+kAMazDxuPbHXEenFQSTvDT24hOesmz4MM4D47j
dO5/rCOA+GfNq0NwHlRc6EdwQ9vkPeDPeKyTizzPuHiX4NaHxtXiPLhU3ToY
576vnJ6uiZ4LcB5scGvIAC7K6e8OPFOM+T7Ifq6KvBXg5hsgvDAR/izztivF
eTCR8+faZtbPI4vGPsJ5MIHTWf/hlSnO4zrWz5oH3F15kZkOregLdDo1duP8
buNfW8n4ROX9dST4mYpWQZz3vSP3zsb7jZ1zWTxRMW8U3g9M9x+P9wZUqMVh
gveJrT+L9wlyGyzvA198z4Xgzl87EpFXaXPWhuH9hv8NK4J/1RahUT8TQ3XX
4P2GS9Uhgi8uJzrpDGtva7zfOMzp/Jt3LxHxE1vW2MM/Zy7+9cJ9N8j7k9nr
7qFODPKvF3ozB3r8L55KwH44qGe44S0b6D//uPwT8EH9wfMVZDBfK4sAOdS/
USOOEtzfMs4C/oTnn3uMPHrB+ZNewvo5bpxCb2O7Cz+C03ksU8a4/JYfL0V7
yNiXchrLF7L9E3U4TU/jSoE3Lz2C2jKm/AhvEL+/27YusPwkb/EUlzVl5mf/
4Yf4a8fy3Q/wGo972/Y2+/7Du/SVvJ+7H+AbSGot+1/8YPGNoKvlJ/mnmzOS
ShkeFQ6/Jz45ghmXPynscZIMM+4gHp9DdPL/jGrZF53/O30Ql8sdEoj47IS4
KiaeP4gPzV3tBf7QKaFZDP8/XG6Mnjr0nBGsXcHo+YdPfShzGPr5DhsOAB/U
2dN0+Q/mWzzRYhEz33944H0PLfhz1CRnGuPPP1w5i/VTPvdRj8E09WeD+KMr
RaX8tqeUT5CDoopnMCUutt7dLb+V3vlq/ocnd9Kpkt7hSgIeVrwP1ToEP5+k
c/zz/mRK+3hGYH/lI2rpBDa+dGpynU1VLBUloO4sUZRK3cxi8WXG3f2fb+3m
yxWwfG84nur9PFe7qlj65Yc70oif/YDFFV/uGF6+P5neLP2yto/hD6dZ3Gl6
7X9021NaVcM8UJnR+S2AxSeLCw4DbmDR3cDg9CA+S6a8l9FJP4981MHw0GEc
j8TbB3oY9127lAIzLq3ZxOJijtSprMRtfCdWJ53P6Ww1TxezZebVVBlig/hv
OSze18DzhQ8zRwl4Mj7QSzgfcvLHVcLPMzZuSQsYPYN+Tn/Nzv9k4voLdyfs
5w/6+TH+aa6pZD014/WJ0c2K2dSlTW2LwyPrab33x0o/PvxCGS7cPVp7cTml
LVW+BPhVQfeXN6OLKKMaM6c/pnWU4kwldeAWGsdyJ2if5Ucsdb/C/Jlq1nAn
+PT8VZK3o4vosvxIJcSPWWFA+I/FuWt/eviFVnM72aPF8G968ozExzXubGH0
0Kbj4tc2MHpubpAh48bw4huA35YQ8mFwWurdHxI/9oThcvD8GO4ku3ZxOa3H
8fgLJzTcYsbdmGk7nxmXluXGfRgyJSxM7BS/gtVJN3I6X9WO/oF5mdC+Noif
zM1r8v3/auFDS5hhF/jVnZcRPfMcnV/Dt6iukGdNjB7FfHESPzpw/LnDBxuo
m32Co35PfEnZfyzrsRz6hX4+Nlpx1Lyv1JaJSRrP66opk4jaXuBZzrt+T/Ms
pW4NsVRyPPCd0muI+Avcyntl0XdjX/6a56n2e8Y2Ukkph0l8hl7EUSXPUtoy
pDvPgYl/u9CbxFuEi9xj+OnqufkBGQy/cPWwPuCRHy1TGT30orQ5SW2MHquW
o0RP613jROBeIhf++zXxJR2XE01wwU+t4eBpiqejGR5aZiqP8PdILNuEcd0L
fhYx49JvuHFj+c8MC23P8MNYnXQipzOpzUluOjOvXM2l8sy86A3cvHR3VKnA
Bw+54BmMD/Ruzofd4aJ+8E3o3PO7HYyecNs3RP/Xd0M22Dg0UyJirRctal5R
B+sOBOgofqBfxJlJ7gz7RiVv9WwX/FBDFW+NuwQ8RlCxoWjRJ+rN0mGr/dUa
qUPpWy4Cb96T8flt43m+0oIzn3RUWyjHxXsILjGVF/xh0Sf61cgbn88z8dfO
LPQHPsrB5RbDT4/1PqUOftkX9gR3l57qyeihtcOLi00ZPcmjtQmuq5vvBlys
ZPECs5pXtPnl+wTP1Jh8DTxrRqavZ3jo5TMHvIH79fgexLjKx77UM+PSV7hx
Nx1ZYJXc48V3Y3XShzidr6X/ShUz87KctHApMy9a+MN3wiO06Zw8fFgy/2w1
+N9wPoj8OaYP32YezFJgfKMtON/qE8t6zQRaKe1J5XpPN+ZTyt0ZXx4z9abt
62oLt/Lv1JTSsErPolrqb8D5CuDDDS7YjQj6TPWV9OzYcpvpS3Q3dQOfF6Fc
+XzBBb68ScB5po5QV3VCq4ArnpJZJBH0me60W9NkwMTX7E74CLxjRMvQE+Xf
6YLrgdLgL28J6AS+XkBfbZ9AKz1Ky3zfE0ZPyoLP74HXBrrPBz5ymM0rBqef
crii2souRif913f5DIaH/sTxyG/YK4RxI74f+8OMS1dw4zo9+RnW6+zNt2d1
0gGczopJsdcwr7+qBnrMvGhfbl4/viw6Ch/kZ5Tngr+F86FfWlOY0UMF5+2q
SmP0zOB8O+e7l/gZudWO+Pl4dYxwIlNnX3ZkmoNngfBi4qeh3fr/gHcmOBE/
GzSqiZ8vBE8KAjefklYBP09HjSd+mpr5CgD3VjZWxbx8p5xrhJ+Zl/9IAT+y
roH46XT9EvHzZuq3LkYPVeZnuwi+tYUFmsFP6+4LJN5hSug84C1GYXnw04rD
bzssJ35G+i4lfkZzPO/PBApiXOttMV3wM+p40XjEt0cEED9LWJ20Eaczz8Of
+Pk9WJ34mcPNK+Nn4hH4oLzFifipzfkQ1ZgvBD9/9bdUws94zrc3T0rWkryd
NvE81vtdi7Y7TN5SlduURiDPl2e2/MR6fPrX4C7w9uGT67HePwpV8rDem/el
EVxfKKwU6315wfYSrPeaw8tjgSesORKIdSfPSynFel+7UzwG+PAJGtFYp49+
Ny8C/zKbsbeBh/ASybpO/15RgPXe36IUD7x2rPsR4I7PTOZgvW+yzbwP/Hbc
/UuEZ/08bazHkf/5ED2hwj4WGFdrVUoN1ruUcjDRM8G12hLr/TKrk67idL6J
0pDEeleZs2YR1vuCi8rEhzNykhPgw+Yl5yvBP9Hp+U3gDRXrdeGb8dk2Oaz3
W5xv2dGup1FXJZIfi2M/eiecKbB/6Bfqt8K5KajDxVTxUuxHBtcnDzD1lrrS
FdKO/WiV5dVp2I9SIov7gV9ztXuH/cjNqckW+9GzSn0S7z/DxRn7Quy5hFzs
Rwq7DQTB73er4Q72kX2bGi9iPxr6eQ2JX//tZRL2HRnHykfYj16PNSN6Jmf+
eghcwdpWFPvRuZVLCT5/unQoeC7OPhKJ/ajtqiThl7Es0sW4cruOvcd+lHRL
nug8/XHTTuxHH1id9FNO52mH5HHYj8xuH5iA/SiJm9dJ45wF8EGQl6KE/Uif
8+H/2Y+ovJ/vM7Dv6yaME0e/1PRbgId+p2eJYTH6BIn1xuJcv7QceFSaTBb6
isTrOXbof7q8zEi85+OADPRL8Uvd/fHn3xruBA9SXyeKfmnVd9NJiH/asJsC
fnNdjSb6nNV+zzrQL+3YsofwG/zy+076pV2mq9AvWTuLkvgLHjl1wI91hp5G
v2TD4fJTT6uDR/nnMwn0M9cim8m4I+xTatEvPaqYPxP9Txo3buRcwyD0S02s
TvoXp/PjknW1mNf9fd6WiP99/T3Bv7u3VcKHkhsFP8Gv/9CV4ILFmVnw7U3T
5WT0S/cVOgh/WY79KfSr/aInPjD9KvW48KwH6t/HmHA39KtldkOLBtyteFvD
WVxKqmox+t5e9fnG6P/Nb7L4w6vjg9H/i1Y/UUI/v+7HGYJnDXWMQf8fHMfy
beB4ohJmzEMfnld/N0ecidfmxjVZLJWLvt1+/rb76P8XBLE8/ds8khmd9Olk
+7Xo/8/vZONTloc/Ah5eLRKG/n8Q35iwIgU8Pev/PkP/363oRfDk/FhhjCu3
bmohMy69jhv37vbqZej/O1id9EZO58TE288wr6qWUHn0/8O+sfhW7SIt+CD7
1m4D+v9BHz71JZ+DnzpV5yzR/ydy/NfCfs/EOTPjy4RXmcw5b/B8JzRZyQ33
RfTE/PNGbbX+dz+kYsI9C9wXUXIacz+mzPMfPja+8MXCCFde3siCbNzLGMRz
tw2T2BMx75nDBvZexuC9DmNXPZ9FEa78spKtO/433kjKTgT3PCYFrPp2+3/4
S1f5JONeiJ1oyWdxRs8gT92riY+AG06vXSqx1frfvRHTy0N+497JoVFXexie
f/dDpm84ugfjatycbIZ7K4N4atBYw9ztM5+9Y3X+4/l4/+IvzGt+nWfa/8ar
3zB1hg898qbBuOcyGK/h3OEJ3z717NBnfPsXfzBFSATr9kVGaxSzjqnv+4PI
ucN85tPjWLd/jLuOMeuPyt1xgpwXqh59FP3BrFtx/clCZsz6OL7fm8Rr7U3L
bWTWiVvMkI9YNzJ+ASQ+S5gfiHX1IFR7OHhcZPsI7rfv4Wasw+kdp3cy65hK
MvtM8AgN9pzztKTVVKdoMz+MO+/U5XivR3xKrpEb1n21SIQa8LKeXUfB/7ss
WAZ1wpnjn7i13x16TjVIfEZd+ZR0einw2WeubIL+mjNivcA1Lx0n56NOZ6Hr
mO+tall78PCfVpJ52dJ6I+CPj62zG+pcs+swwj+QLhyJ/SM9WHjGDKZOB350
I+eFR7klmtg/ZuqL/0CdtlRcRM4LsptVlv1k9o8Jw/++QZ3OCmkm8Tzhevd2
pl5bhswIBz4vXpHEf7wZ2oL9g/8ydy94TL6MJLjnyn192D+em/6+gf0h2f8K
4dmZyZ5nZgdes8V7b2PuXPN378wOxM/x0HnFxNOD8cMMN1eAP+3CZFvsV1vC
PpHzi9KknPnQk+8aF439TZnTo7nu7mXoN4wdkwf8VHY24Xkf1rAN87XkR38i
+yc334D7TrHwZ/tCpz7se07rpbB/0bXjre+jjxnlf2h7CdMvHKjaQc4LSs3D
6wSYPubMY/Hv6Bfkz31l+/+HCip7mT5malV/MPoF1zOSV4Bn1+kW7WP6Bq/R
d8yBP3flER7TJbvD0Weofj+jD57r0pOvAh8eGiiNvmTseJ076FNmSx8j545Z
Kuy5xTGt7zi+yzSpsecXhTfawxEf9GVkMRNPz+Tiyy9X+IH/4KmjJuibQjh+
jd+uztDTrz7MCn2W6N+yC8DTk976Qn/t0PYrwN+K3L0MvH3J107M97aM5ifw
rBc08AP+Q3poCvxp8bozh/GH9h3+hpyz0l7OSEM/3TVt6VBJpm+Ns6jNQz//
or8+7RTTT5+OitBl+msqXra6Cfie0xIPkph+eozvPD30rUf9xMn5Qk8i7GQq
07/yg+XGAF9tkPQDuK4odQr97jfT2Afof3nzLhKeS/vTZqA/3rGqR12cGbdi
y0kSL7WXPZ+Yj75Pvs++WseeU/qEpssjfu3Ohn1MPP2Ji9+3d7gt+C/Z59Do
3zdy/DZyzydBT8Ox6Ano91dxeq4XBIgnM/r5P1+tA+7I6d/26G8B5rv4R8Fy
nCciufl+vbPkBfzJedpZgHNACOePj9rWVOA3jW2HwDfVsyKkn9+V+IT4dm7L
PuKb1rEfpP8PlwmNh281CxN04Y/Rx6RenC/iV88nvpnY35QFnhMWQniqnkkS
3xplbIhvoZeDCc+a0cnEt07pTuLbLYXSBvAciWbPIbU9G4lvV03Z80h83GTi
W93seuJbNBffIfvZBvyBij7Et6scf/6mLcQ3382ziW9ZnJ5SEwXim1fwRuLb
Hk5/g0U+8U0j9ArxTY+bb9IYi1z4c/j3LOLbfM6f+wdO30UentS8tQXr9MXr
3Gj051kVgV+Qt3tr/tRhfW1aY036/54Ih3nI8xceI69hPd69z7sFPL7ArwDr
Ilbbdy/wdW4mJF68qj8U62jKh4JN4NG5f4PwXzpjNwrrbkrKyttYp+s+OJJz
DVXInjcUdN8fxTrdxZ07quT2DkO8apDmO6zTswLfyfnCa4mHL/i32mUZYX1p
c/wPRt11gJ4v8mUWWI9anJ6sYULe0D9StCYAuJ7yTaJfyL+4HfMVU7QuAc8G
br6FLuGP4Y/mEasZWKd1XnxybrotvzAcdc90vOZ07AsDFxJI3657ddVq1Mm3
rqO/oZ4bfFpGzgu7y1OWoq46Xt+Tj/rvdOoiifev4p1AHeZN2RkK3GT1WNLn
mx8WbUbdtv312hg8m4LGEh7FG5f/os6frV0UjX1h0po8wnPRmT1XHMpYZoN9
wZw7X1xuTviF+I1lX15iX4gSmEXie+NEKsGvbiNkh3puzO471PHeifOg54ZQ
RSTqvxGnR+yR90Xo96Lv5gA3XbGbnHdsbLZtxXzFv8eUgWcTN99ATzmyLzTv
kif7Qg/nT2GGSW8Xs88Kvwq8jj4kYPqoFei3b4957YR9eVdhmDP6h/9uSZD+
fFJHuCD28bUz1XrQbyg/jiX9+Zdo2+fY92fFqxcB/5x0msS/EFt4AX2CvZ3y
UPD4b1rGnoPmOW9AXyEw8vgW9CHeD2IJHq/Bnh9MChcZoQ+5xZ0j5r/9thrx
poUizuhDvDPHkHFzxIIcwT/NuXcU+oeTj0PJeadPefYJ6OlcJom+iJb2C1gG
fHlq4wboj7MZ9gd4kXgwiT9tcDoA852bnnUQPOKr4sm4v3xkhqEPkfexOYI+
JO1FMYl3OliiiD7ZW8NiUZb7gX/fZfItBJzxPSZ2QuD3q+Un/+Gr1KIF8D2m
ZlfaK3z3GMRTLIvoIqc5z4qz2e8eg/iRrdJXZfFdRGL8AcQP9uEWAX3C+I7y
Kumc2v/ym9od347vLmpRK+Lo/9GjXnhdH/g2u3zBDKafH8S1wuU78V1Hwbl5
Lb7zDOLZXak2GJd37J0DvgsN4qMVZTcPvJ3+bCb3HWkQjy6QU8K8+I+dn/9v
fNWlaafhg6ReWcn/8t+OZc9X34ZGrcH3hcHz2t9vS7/jfPV40lcVe+Yck2/M
fo84eODrZXK+GuhcV86cY+wH2O8RLdmThOYy56sLrwqm0Mw55lYVi0f9nO8+
nznPxB5MacX5JmMby5PUF7EG559eo/3pOA8VurPxi1b7rcd5qW5kzTV8P4nV
YPEXhez3Ff+fOqGyjdP5edx3kIIpi1YiPiShKJOJp2O4+JSwXQvBLx3b/Qrn
uUH+iNn7pkJP2cv6Dpz/aE6PYG/vHuhvag2YiO9Ctzn9kekbbmG+inL7luH7
0gpFFteX0fkFf9Ztlhdj/KFfc/68mbhxBu4h5VlqPczCPfMbFeSeW4eH4Rbc
57gxR3sf7iGo84QIfvkx/z/cQ3L6IyPyUK6JSvne5gv8llQ1eU+73pK9v7Y/
mr23tnn8mRf4Pm0UoRaMeziLfqn5AR8hLf4T9zDKpUSicJ9hwlQ9wv+4/pEx
7htNOd5fRzN6+lcYEp6vqlcMga+QiuFlZL+mBVay+GsNsWrwDF/TlYR7CJ9e
FJH7hLJjH1/DuLc/tN7EvZrqnQ3ngDsr+o4XLj7Hj2F10vs4nbGeGgsxL5tV
Ht2IXzb1Vyi51zdxzT74ULNr/CbcZyh4dJDc32uSVFKGb8fT09YxvtFDOd/y
Zli/x3fdA0OPGl9xzaeCyhf44X6a2cE5fzNFflDbNLe74l7D7tKVGrifNvaz
ngruz62T+NOM7+7nhLT24H7a665aY5wbufMe5SHA3k87HXLuD+4VOEwO9kD8
rJt7jBHvf+hGCO7PNLxVSMB9lGWRX6aDf4JS8X+Yp3ix1SHcu5rn4OQNPbKJ
W4YCrwvMeQfdaWdTCP5DKsqX3PsdUMnFPdpwsYYZ4FHsFMvDuOdeiPrhPpR3
9DQy7v72bPKe1og7l7pyOh+0eO7GvJIvva1F/IIXLmReF559G4F7HKqKcua4
h7LXXnAJ+G0S68vgm1JxS/NlRk9pYY0v9AQKOk+Hz6PWB8UjP3WX/yL3x3oK
XunjuejbipkhP+d7CZL7bLuEN4jhOVZs9BVEfs7XM30G3OTYcPLee8Fz9j5a
LndfTnTZMtz/oE+blF5Ffs7OMEwDPv7+jxbkleojuUjkp8SoInLPrbSkfg/y
MMh4Vy3yU2rtVcIzfFreDuCPOoWXIz9Hc/jjFeKV4Jk+2zQR+bnJRpfoTLdc
EYhxx802j0a+DY6rZyIzDvnpx+qkMzmdQz64qGBe48ck/UH8TSnTOOC6Nk2m
8MHl2DA95OcFqeAHwBPb9ebAt1cRI7SRn1qcbzp7XYNRB+qOjvhmx9QrZU+2
fqrM3LoHdWONoe8Q1MlQLxYfuWZO4hymzrzvsX6Derg+hsW7vdQXoC4NNd58
HfgoZfa9U2fMDwnUsa+9+S6ok3Pq2PgLBud78R7KaE/+etTJ3N0svjaefQ+1
pXzNNtRJHa6en3rd2Yb4jiYPZ9RJT0OWf6H2rV6853q/xugs6uQg/8KDj0vm
MXriF7+NQJ0s/o/FRV1FR6JO8ibNewl8A6ffYqu9LeYb8yeoCzyvzVj+xvfB
N+DPHuMtz/D+TFqLjT9r3xYg4xVHVfQO6/yWfJQKmVyXNaG9lV6ZqhYI/OQl
3bEMThcO1BI8Xm5pgzSDi0V/MkL8kDzXbAanpIcYNgP/4zfgUc/EC3P4eJr9
/ioXZ2P+pOQqPymVfQ+gvWTObeTl/NqaScfv5NGb+7+KYB1Onsd+X0x5SsX0
Ogfz+9XZ869LmrvXNv8WSmr4waPtPk/pSkPNKXFP3tFqRuz3MwmBykrdd9f5
9RvYc9/diz398XQrtdJBx31eeDqdI/T71OqkODo0gv0+tOHzURKfYM6ed274
85WO7myl0ifO/im+Kp0+mhP+pdQ+h1J7w37/qJ+edgN6rJ3ZPn+h5dwo6Bfd
ZTAB+msvzSX3Ku8NFEevT4qjbl3pMBQQ2MUbI8T6uZfP+vBCIYOpI9d5qZwP
JZ8SI8DzZOKtiwwP1XF0FbkPb6DM+hB9/qKggEAYT3gJ68NTBd0z8EEpP9Wk
w+cptWBh83j4EHrmo8wHk2bqwcvHPnNd0+hvcVrmNW5ltOke1h97K1UBAY9w
3k/On9U2e/7CH8mSavf54enUG86ft4WOM+CDqMdb4oPR2bJCxgf6ZTjrm/3Z
THJPO53zjRfYqIj43aVDByRWpVPKbiXEt6W7vIj/q4NrTsD/bwd7hTUZXw7k
s36+e3u+b2AglOfA+Xnk8Jfr8OHzkuV+8GH9mHfj4OeKjPVnMd8M2y1H8Nwl
ftybxcyXuu7AfkdZcnQIU5+Dec+r2HOEviOLR5R+N0O+PePwlGUmh+jiRirg
QuloJdlUOv+1eTH1u5IS9/o9yj3qAs+zNT+q+9Tuf+8n7785O5128ePNvLok
W/rOzn99o1Ha924dZh66SWkmYgt28HtfsXkeWbdbAjxuPopRPad2/3sfu1Xg
A8mHxfulWhE/mA+JqSzPgKVM+sCAIW+Qp944aybG3XzTqlLqzs5/75lNRz1f
ERFSTMWpPk3zLEygnPvHP8D58+KSTVm4Z2YnNFRbwfAOpaL8nvx+mRGdtSTl
bCEV3HVJ8sWuV1SE5ygSr51+3h33zyLarikuyXxOdTkXkPiR8xoX8bdnUj63
lFL3/1dEWb3siUf8wJJrKaZPUqgJ3nXSrsxz6buaQ+IPXkx6NVPIiT/8QdC5
9czzmy4zhvDveSTy6/Mta/46Kfb33nS533czE5hgSm/PpGPj2+zAb7pqEolf
En9OD/fJ7CJWJuB+WbbiWxKvPDUqitFPp/4xFstl9KscnkniebpPJHEvbXT5
ITvcUzuVk0fi/VqPWjD+0HSu5cyTjD8HItn5Grjod+J+m/quCW/lGH+sfrD8
Csf4poi//ve9FRNPd8hJkvjNUgltiN/YM19hguEdejB+jP2da9ATktMwjdFD
b+/qIv5MFh4mCD1f1CVO4v5dxeR3JD5zRPYqzLffhXJh5ktfl5pF+A92Xp+G
+Srbq6TjHt8NZfb3EFu1boXczbLnb2D9pAf9dIzrcclKtOBHsX7SGzk/+04/
0sfzKr319xH4bwSLkPjrZbbVeF7DtxYMZ54XHTEsn8SriN5ajXzQkcr7Df3y
EQokPm2ewUXkg/ykRZJMPtC8INZPxxMuh7FehF5FymC9pG+dmsesF/qry+HR
qDM825teqDPZZjlOTJ2hQo/Kx/goZVKWBUNnnChMpgS3tW9c6ddKt/ernDzO
/PnBYbPZvszfN179Q/C6arUT9qqXqWiZXP9tujnUT8XfBHdY4vDngEEcLbBn
9NUPy7KpzfM6CZ5Z2++6X59Pa3Qqd1jq86kUdzb+e9rfJCaO1nzy0ne/AVNf
RrHxelajTzO89IPXB00cVS/TO+3YcbsXze1mdNL8/eOmuBUm03lHWZ5cqcYz
+HPFUrOP+PuD1t0EV2g5r+bE/HvzkjA38LXdYPlXusfGWTLjHft6JA/jFxqz
eIGIWjT03SvoegO92ZzOoMxDmli/FkaSx5j1Sztx69da4XYe1m9AxLo6eSbf
HARY/1stix3gf/q1cUIzZFOpv+biWfC/MtSE+L+wdPlJZdc0qpmr8/O8NkxD
Heb91RSQZOqwZvac56jb7xc39KAOi/cdIXX+J1eH1WeKe6KuCj99ZIR9ZDRX
V6cGCUiC/8zQhSfAf6Mpjjzfu5KjbKHngdvbPiYfqHdc/eyz+jAX85oq5XQH
dclK3SMB9b17VeBVzKvbf54C6tJPU3tyL3T25frNeK4razUrDzL+vfCq1WP8
oaYPqZiF/GyNe9uF9V4225LwLJ53aBfy80bM66GoV6dmsfdL455reCDfhlfN
6GKeG+WQWk94Plx5sg7+r7/85BbjP7XapY7gpcPSp2K9fJmzKQb1x+vpIcIf
apJxgdS3e24NqD/zJJ0Iv0CQw2bk7dtWsyHIW/cr1YTnsMyn2XjuDU2pNzEP
SWVWf5ur6xPUQzntFU6oh2N8jxJ+Ide1WaiHAZ/Ye7Lu3P1Y20s3tyL/jW2z
9iL/4+O/EB7VmJ5Y5FvuNmtd4A84/hFlTZtQT1aojTeC/uOLrhH++OdXlFBP
tE1y8XsY1IcHtoR/Te6ZdKyjEGqVFXQWHWB5lmSdd0Cei6bq34V/Obk1BF9z
uPsS6ttGDYFfOYz/PwRYf/bp6zTiPvFuO3N91Nuc7az+ryWfNZDfbg3/JcLv
uzEsz56ZwuVYP743132Af83zWd/UNocaoN4uc1Yei/rcsPUw4accI5H/dPFY
3Vuoz9JB1oR/RdhsX6yrcWvL9bCuG9ez+iXqm3dgHZ6euOyxA+PfmB72+Vq9
ydYDv5e4hyHq+d5PzoRfU1IgB/yWT0w7GX56kP/S9DPbwSOntUod9eHS7yrC
85/T2jMYN8BH9TIzLj04bpPTWi/4o3o97z/Uz0y3aMKvUdNfDn++L9Hfh/o/
6E97S+R7+OAyIbaN8YGW8mJ1LjdfMgW+ndod/4bxjR70rcE9QwXP13VUhAXq
+dPwfYT/9bjcPtzzLrcvvYT9YvD5RnmLbsNznOOv/Rjj5HLPUbOiwxHPPX1r
yCHM4z333A38N/phf9nF5ic9mJ9jp7TqY39pZPOTHsxPenSPOPJw4GTwJvgk
upLLQ6+n5cjb33lmxsDP57DrTjmjdiXW1/V13pHQv7GBXb9lLd1PsL7WTn9L
fq9UmVtfA4vz7LGO1s1NCIfOkVyeZ1zeZYu8OVUzoxfz0mpj10WBdrgq6sOT
5HcV8D/+0HHCn6hab4X6YJ4w4xf8P8nVh/0PS/YiL/OOBJ2H36u4OhCVfc0H
eX9pWmwJ/Ivb/ZXg8yeeXYA6Nn553n7UZ2uujm0/9SgEdazq3ZY01OcHF9n7
8Dnm7juwrmItPl9i6hg9WMd2PUw+Drxkx4LxVsz8tDay+4LoD1kf6DFrkK1H
XRrcFxZ55lrCh5fDfQrw7/zbWfyHwPl38L+6z9YVdWBw3wkYORAN3SkDlYWY
x/FUdp/dWzJRiKlTdMTcOUNQDy+WsPG/Qy3P4jlVZy4px3o5toONv31pzEP4
rv13UgHyfHDfvDRjkin8mqYQKI6fa/S6CF6m9+UB8krZ0SoU/06f25ft7qUN
OcTwLlSu8sM4V0ex+nvleOF4jnkXVk+BrgZu35+zxfgRdF/tMBvG9Am0ENcn
jNG4cRjzPD7QHubCzPuNYA/BK4r366HeSS7sa2DWB5Uyhe3T8gJ29k9LLaRm
VT8Imbn/CbWscPKlD3QLPel8nT/q10bzbUXIl5i+4STee2d+4AGJPGq6Yf6j
g8xPPbnFJD6uidfvztSjIru5Lnj+1/mTSfxXbXuf6Qzvmj0jVk9nxtkUyvIr
L98ceIrJj1+6ivqI7+jrJ33gBTFFX6NnEbSIkLbMjJB31DDlZSReoasnFPl6
Ka7JEnrUZggS/u1Gn/et+JRJt/01plsE3lLPX84n8RZCytewfk7cNUvHejq8
UIPEH6qxG9kq8JaeaJ5ZQ33KpL4e1iDxix+y/eLit20HdIpW8iW4vjHWWlKe
0UGv+byjdOezCOqt/AIS73ek+SL4H+v5fGT46anUBBLvvnLcTGaetJpjTpzS
/ie0bqEqic+T2OcB/ZO3b3JA/bvoPZU9R1z8qcX4SLfOiBXAzzmcn3NFpm6G
P9nnRbaiHq+dMJvE90u+M5/B8LYK7B6LcexEtEn8+IDjWh6M/8/mSDiSfpzz
37HrRMI2Rrfr+QWKmMf7RE0S35N/MR7Pd/f2pFzouc09X1k/Z3/4Ujp9pBZ8
WjeLna/anQ3bkD+1z+e+Qn3lfV9M4gXqDX3h+6ovT1PwHPgTppN4rTj2PFP3
8I2bTtFJvhXnZ9merZJ4rv1Ha+P2Ms957/K1JP7/ABC2y6A=
    "]],
  Axes->True,
  Method->{},
  PlotRange->{{-1., 0.9999999999998993}, {-0.9999999999999748, 
   0.9999999999999748}, {-0.9999999999999748, 0.9999999999999748}},
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.626510528187624*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"sph1", " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.6141497356654024`*^9, 3.61414974589902*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"circ", ",", "sph1"}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.614149757833041*^9, 3.614149804227523*^9}}],

Cell[BoxData[
 Graphics3DBox[{{{}, {}, 
    {RGBColor[1, 0, 0], Thickness[Large], Line3DBox[CompressedData["
1:eJxF1Xk41VkYB3ATYZQYaSRqUJYnbkxP5lH3TifboGHsZO9hXFSMlNKMFuFm
yeBmS1xlHyVbtxljOdfYyn5tWXK524+4d6IGT0Yaf4xz/nif85zPH+c57/t8
n3M0/X9yCtwmISFhuVmSm5VDvkllPCSANKk9luMWdbLn/z3TtHvoQebm2r1g
VmRfCrbctTJnjHGZAE/M1TI7LJ4jHy65t59kQoAb3oZGauotyBuWNTiOPCGg
g/wc022dyKM1E0o9PwlAh0ubRvDUAPKyiD7lrH4+sLI5dLd8agR5j/RV6Qgj
Hig+mRtzcGEC+Ya1USfhPwMk30UcsDOexuc0sm8/sX0NRmz6y23Luchbk2k5
QSGvwHp3H5/iw0ce5rjqJhE9CM6ydTwDygTIE+Y/RqiVvwTx1lp0yToh8myu
sSHZuA0QdW5egbsI5EWVfreKvJuAyr3R0Tln7LRqRp/uuVqQZPAiOv029rXI
lLRbOfmgV17ucOcD7A41a+5yrnRob7Gys5WBXeSlbRWTXA3b2/fe8EvArp06
Netb3ABDjRX0q7yxe7ZL1R9wbIVx+f9wdNWw0/d6zzONXkJaQb80qwH3Zegx
Tg183wfXggyWgsyx0xcbdzXPD0NAEcw0KeL5/LB4qWT343FoUSFfwizkISev
U2VO+HOgKOkDTdw2g/xt/M2AOh0u3PPbkl5b3BTyhQS1nt8/8ODlwtgd70rH
cN6Uvg6ZCxNAI5ntqX+FDyFXkaJOyl0TQiWmb+zbpB7knyL3PK0hhPCOX4Gj
NexAzrexNvUgE3BZT/i4YpyFXHVo9u/oSwTcd4T3y48pOM/BV6zGbTIJKLFU
ZCBdiPM/MaklyHtIwNn1iusedxyQf6nef8Y2i4BdGe9lFXJL4JbvLojWrIog
4IfMOJI/6zly5p8L1D5jAmocebSoaNeCfN5gfjF88/5V2++bkVo7kPuo5gXl
xQih7/kLumtaPcjtrb44bu8ngOaWCfvYNUPIj7rYHFef5UGybvU0Z3oMeY0w
iUOjcGHiDbfhJscp5JlJduw1Sw6s1VsgP2XPII+95OSuxBmHRnOWcTvTechj
gkUXN/RGIDu5kgiUECDfaNRU/pzaD7tZBZoCEyHy0CaWTPa+Lngo6TT5WgX2
78NleM+YbbDiMx2fJWkCueQ7R5mTd5vgI7F7gdgJu72UrgkluQZaeqXEe8Rj
p2qrshzk78N82bZ0nXzsOYOqp5z1M8CzKMpHXwb2QpM3Oy+qVQOuvs63xYnY
ZenRzGz1BnDslbbYyg87X/Qiy/ZqKzDJi1xQUMUuBhQLp9MvAXdKJC+ox331
ftX7YoeIDcTnSfslO/F8ogKj3LqcXwH/tAlySwAf+TZl/oX646+BFaP26EwV
F7nFN4pmzqQZoHVOqce1eBp5szi02eUgD1B+Hhu5uGsSeZpz0UppJR8UDrj0
vnEbRe7KWpWLEghA1qDG9PJZNvIgUSm8MikEvSqHTzFCOpEP/Bphf4ZEAKlj
q5aXV3E+tXNfpyaGbJ4noH0XPvsH8qWwt/8O0wkA4ImMspVyPP+td7Tsetzm
/9K85f8B7oN8yA==
      "]]}}, GraphicsComplex3DBox[CompressedData["
1:eJx1m3lcz9n3x0N+Zsg6Y5kxDYVkkCWylE4MQtayi6HQNikMRihkq+zMhIhk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    "], {
     {Opacity[0.7], EdgeForm[None], 
      GraphicsGroup3DBox[Polygon3DBox[CompressedData["
1:eJxNm3f819Mexz/fYxMapCTtQUpLE/0KmaWhIdGyqTSISoNkZ5P2kNIgWRk3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        "]]]}, {}, {}, {}, {}},
    VertexNormals->CompressedData["
1:eJx1W2lYT93bbUJ5VGgwRCFkDEkifseQJBRlDJVSoVGpkBQhDSJEac5QSEil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     "]]},
  Axes->True,
  Method->{},
  PlotRange->All,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.6265105357848372`*^9}]
}, Closed]],

Cell["The geodesic curvature of the circle with parameter ca is", "Text",
 CellChangeTimes->{{3.6142629124679904`*^9, 3.6142629310944233`*^9}, {
  3.6265865355316343`*^9, 3.626586553596466*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"geodcurvature", "[", 
   RowBox[{
    RowBox[{"sphero", "[", "r", "]"}], ",", "h1", ",", 
    RowBox[{"h2", "[", "ca", "]"}]}], "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.614149876050049*^9, 3.614149963925003*^9}, {
   3.6141500332983255`*^9, 3.6141501013612447`*^9}, {3.6141501802973833`*^9, 
   3.6141502134162416`*^9}, 3.6141519955165715`*^9, {3.614236185172659*^9, 
   3.6142361856406603`*^9}, {3.6144907894011283`*^9, 3.6144908578020406`*^9}, 
   3.614861863216346*^9, 3.6148631367089825`*^9, 3.6188411396416693`*^9, 
   3.618841188157755*^9, {3.6188417041442614`*^9, 3.6188417117726746`*^9}, {
   3.618841825949275*^9, 3.618841866805747*^9}, 3.6265863265848675`*^9, {
   3.6265863956305885`*^9, 3.6265864207466326`*^9}, 3.6265866093665643`*^9, 
   3.6265871237462673`*^9}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], " ", 
  RowBox[{"\[Sqrt]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", 
           SuperscriptBox["r", "2"], " ", 
           RowBox[{"Cos", "[", "ca", "]"}], " ", 
           SuperscriptBox[
            RowBox[{"Cos", "[", "t", "]"}], "2"], " ", 
           RowBox[{"Sin", "[", "ca", "]"}]}], "+", 
          RowBox[{"2", " ", 
           SuperscriptBox["r", "2"], " ", 
           RowBox[{"Cos", "[", "ca", "]"}], " ", 
           RowBox[{"Sin", "[", "ca", "]"}], " ", 
           SuperscriptBox[
            RowBox[{"Sin", "[", "t", "]"}], "2"]}]}], ")"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["r", "2"], " ", 
          SuperscriptBox[
           RowBox[{"Cos", "[", "ca", "]"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox["r", "2"], " ", 
          SuperscriptBox[
           RowBox[{"Cos", "[", "t", "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", "ca", "]"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox["r", "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", "ca", "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", "t", "]"}], "2"]}]}], ")"}]}], ")"}], "/", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["r", "4"], " ", 
         SuperscriptBox[
          RowBox[{"Cos", "[", "ca", "]"}], "4"], " ", 
         SuperscriptBox[
          RowBox[{"Cos", "[", "t", "]"}], "2"]}], "+", 
        RowBox[{
         SuperscriptBox["r", "4"], " ", 
         SuperscriptBox[
          RowBox[{"Cos", "[", "ca", "]"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"Cos", "[", "t", "]"}], "4"], " ", 
         SuperscriptBox[
          RowBox[{"Sin", "[", "ca", "]"}], "2"]}], "+", 
        RowBox[{
         SuperscriptBox["r", "4"], " ", 
         SuperscriptBox[
          RowBox[{"Cos", "[", "ca", "]"}], "4"], " ", 
         SuperscriptBox[
          RowBox[{"Sin", "[", "t", "]"}], "2"]}], "+", 
        RowBox[{"2", " ", 
         SuperscriptBox["r", "4"], " ", 
         SuperscriptBox[
          RowBox[{"Cos", "[", "ca", "]"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"Cos", "[", "t", "]"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"Sin", "[", "ca", "]"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"Sin", "[", "t", "]"}], "2"]}], "+", 
        RowBox[{
         SuperscriptBox["r", "4"], " ", 
         SuperscriptBox[
          RowBox[{"Cos", "[", "ca", "]"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"Sin", "[", "ca", "]"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"Sin", "[", "t", "]"}], "4"]}]}], ")"}], "2"]}], 
    ")"}]}]}]], "Output",
 CellChangeTimes->{3.626586422696636*^9, 3.6265866142649727`*^9, 
  3.6265871281142755`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kg", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{"%", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"r", ">", "0"}], ",", 
      RowBox[{"ca", ">", "0"}], ",", 
      RowBox[{"t", ">", "0"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6265105621332836`*^9, 3.6265105864225264`*^9}, {
   3.626586445535076*^9, 3.6265864880919514`*^9}, 3.6265866220493865`*^9, 
   3.6265871373962917`*^9}],

Cell[BoxData[
 FractionBox[
  SqrtBox[
   SuperscriptBox[
    RowBox[{"Tan", "[", "ca", "]"}], "2"]], "r"]], "Output",
 CellChangeTimes->{3.626510587389728*^9, 3.6265864927251596`*^9, 
  3.626586626432994*^9, 3.6265871422323003`*^9}]
}, Closed]],

Cell["Only the great circles (ca=0) are geodesics.", "Text",
 CellChangeTimes->{{3.6142629436212454`*^9, 3.6142629704532924`*^9}, {
  3.6265866708306723`*^9, 3.626586679285887*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kn", " ", "=", " ", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"normalcurvature", "[", 
      RowBox[{
       RowBox[{"sphero", "[", "r", "]"}], ",", "h1", ",", 
       RowBox[{"h2", "[", "ca", "]"}]}], "]"}], "[", "t", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"Cos", "[", "ca", "]"}], ">", "0"}], ",", 
      RowBox[{"r", ">", "0"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6143273297369137`*^9, 3.614327353778289*^9}, {
   3.614327390450386*^9, 3.614327420572109*^9}, {3.6144908890498276`*^9, 
   3.6144909391846952`*^9}, 3.6144917093297453`*^9, 3.6144918945373383`*^9, 
   3.614861863263146*^9, 3.6148631367245827`*^9, {3.626523220275897*^9, 
   3.626523241710335*^9}, {3.626586790155282*^9, 3.6265867980176954`*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox["1", "r"]}]], "Output",
 CellChangeTimes->{
  3.6262612530590315`*^9, {3.6265867722464504`*^9, 3.626586798875697*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"sx", "[", "t_", "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"sphero", "[", "r", "]"}], "[", 
    RowBox[{
     RowBox[{"h1", "[", "t", "]"}], ",", 
     RowBox[{
      RowBox[{"h2", "[", "ca", "]"}], "[", "t", "]"}]}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"curvature", "[", "sx", "]"}], "[", "t", "]"}]}]], "Input",
 CellChangeTimes->{{3.61449096601723*^9, 3.614491046232818*^9}, {
   3.6144910803047667`*^9, 3.614491184156707*^9}, {3.614491760169653*^9, 
   3.614491772410353*^9}, 3.6148618632787457`*^9, 3.6148631367245827`*^9, {
   3.6265868432889752`*^9, 3.6265868849566483`*^9}}],

Cell[BoxData[
 FractionBox[
  SqrtBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"Cos", "[", "t", "]"}], "2"], "+", 
    SuperscriptBox[
     RowBox[{"Sin", "[", "t", "]"}], "2"]}]], 
  SqrtBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["r", "2"], " ", 
     SuperscriptBox[
      RowBox[{"Cos", "[", "ca", "]"}], "2"], " ", 
     SuperscriptBox[
      RowBox[{"Cos", "[", "t", "]"}], "2"]}], "+", 
    RowBox[{
     SuperscriptBox["r", "2"], " ", 
     SuperscriptBox[
      RowBox[{"Cos", "[", "ca", "]"}], "2"], " ", 
     SuperscriptBox[
      RowBox[{"Sin", "[", "t", "]"}], "2"]}]}]]]], "Output",
 CellChangeTimes->{3.626261261873047*^9, 3.6265868869846516`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kk", " ", "=", " ", 
  RowBox[{"Simplify", "[", 
   RowBox[{"%", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"t", ">", "0"}], ",", 
      RowBox[{"r", ">", "0"}], ",", 
      RowBox[{"ca", ">", "0"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6144912102411985`*^9, 3.6144912524796147`*^9}, {
   3.6144918028020916`*^9, 3.614491847393642*^9}, 3.6265869062350855`*^9}],

Cell[BoxData[
 FractionBox[
  SqrtBox[
   SuperscriptBox[
    RowBox[{"Sec", "[", "ca", "]"}], "2"]], "r"]], "Output",
 CellChangeTimes->{3.6262612685186586`*^9, 3.6265869077950883`*^9}]
}, Closed]],

Cell["Test of the curvature relation:", "Text",
 CellChangeTimes->{{3.6144922778622637`*^9, 3.614492295334263*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"kk", "^", "2"}], "-", 
  RowBox[{"kn", "^", "2"}], "-", 
  RowBox[{"kg", "^", "2"}]}]], "Input",
 CellChangeTimes->{{3.6144912677174864`*^9, 3.6144912827293453`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox["1", 
    SuperscriptBox["r", "2"]]}], "+", 
  FractionBox[
   SuperscriptBox[
    RowBox[{"Sec", "[", "ca", "]"}], "2"], 
   SuperscriptBox["r", "2"]], "-", 
  FractionBox[
   SuperscriptBox[
    RowBox[{"Tan", "[", "ca", "]"}], "2"], 
   SuperscriptBox["r", "2"]]}]], "Output",
 CellChangeTimes->{3.6262612763654723`*^9, 3.6265869188399076`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Together", "[", "%", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"r", ">", "0"}], ",", 
     RowBox[{"ca", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.614492018684439*^9, 3.614492069089322*^9}, {
  3.6265870882406054`*^9, 3.6265870939814157`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6262612805306797`*^9, 3.626586925157919*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["4.3.3. General Curves on the Unit Sphere", "Subsubsection",
 CellChangeTimes->{{3.626426940389179*^9, 3.626426942729183*^9}, {
  3.626439206942724*^9, 3.626439247705596*^9}, {3.626535746538865*^9, 
  3.6265357780861206`*^9}}],

Cell["The unit sphere", "Text",
 CellChangeTimes->{{3.6136216629488277`*^9, 3.6136217472669764`*^9}, {
  3.6136219986458178`*^9, 3.6136220101274376`*^9}, {3.6136316409915533`*^9, 
  3.6136316447511597`*^9}, {3.613631706792469*^9, 3.6136317217996955`*^9}, {
  3.613631784777006*^9, 3.6136317936534214`*^9}, {3.6265874323304095`*^9, 
  3.62658743254881*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"sphero", "[", "1", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6144904863257933`*^9, 3.6144904977664475`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", "u", "]"}], " ", 
    RowBox[{"Cos", "[", "v", "]"}]}], ",", 
   RowBox[{
    RowBox[{"Cos", "[", "v", "]"}], " ", 
    RowBox[{"Sin", "[", "u", "]"}]}], ",", 
   RowBox[{"Sin", "[", "v", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.626261078463525*^9, 3.6265874382896204`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", " ", "[", 
   RowBox[{"hu", ",", "hv", ",", "kk", ",", "kn", ",", "kg"}], "]"}], ";", 
  RowBox[{
   RowBox[{"sphercx", "[", "s_", "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{"sphero", "[", "1", "]"}], "[", 
    RowBox[{
     RowBox[{"hu", "[", "s", "]"}], ",", 
     RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6262696713989577`*^9, 3.626269769583931*^9}, {
   3.62627057233515*^9, 3.626270580473765*^9}, {3.6262715010427923`*^9, 
   3.6262715449472694`*^9}, {3.62627487396774*^9, 3.626274910835605*^9}, 
   3.6263327382028837`*^9, {3.626427055844982*^9, 3.626427070197007*^9}, {
   3.6265233742325673`*^9, 3.6265233838889847`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sphercx", "[", "s", "]"}]], "Input",
 CellChangeTimes->{{3.6262705892741804`*^9, 3.626270596951394*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", 
     RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
    RowBox[{"Cos", "[", 
     RowBox[{"hv", "[", "s", "]"}], "]"}]}], ",", 
   RowBox[{
    RowBox[{"Cos", "[", 
     RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
    RowBox[{"Sin", "[", 
     RowBox[{"hu", "[", "s", "]"}], "]"}]}], ",", 
   RowBox[{"Sin", "[", 
    RowBox[{"hv", "[", "s", "]"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6262705981369963`*^9, 3.626274916498415*^9, 
  3.6263326919332027`*^9, 3.626332747859301*^9, 3.6264178574881763`*^9, 
  3.6264270760782175`*^9, 3.626587453328047*^9, 3.626938260667557*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"kk", "[", "s_", "]"}], " ", ":=", 
  RowBox[{
   RowBox[{"curvature", "[", "sphercx", "]"}], "[", "s", "]"}]}]], "Input",
 CellChangeTimes->{{3.6265874900817113`*^9, 3.626587502592933*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kk", "[", "s", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"\[Sqrt]", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{
            RowBox[{"Sin", "[", 
             RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox["hv", "\[Prime]",
               MultilineFunction->None], "[", "s", "]"}], "2"]}], 
           SqrtBox[
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"Cos", "[", 
                RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "2"]}], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox["hv", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "2"]}]]]}], "+", 
         FractionBox[
          RowBox[{
           RowBox[{"Cos", "[", 
            RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
           RowBox[{
            SuperscriptBox["hv", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "s", "]"}]}], 
          SqrtBox[
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox["hu", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "2"]}], "+", 
            SuperscriptBox[
             RowBox[{
              SuperscriptBox["hv", "\[Prime]",
               MultilineFunction->None], "[", "s", "]"}], "2"]}]]], "-", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox["hv", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"-", "2"}], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
               RowBox[{"Sin", "[", 
                RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}]}], "+", 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], " ", 
               RowBox[{
                SuperscriptBox["hu", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}]}], "+", 
              RowBox[{"2", " ", 
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], " ", 
               RowBox[{
                SuperscriptBox["hv", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}]}]}], ")"}]}], 
           ")"}], "/", 
          RowBox[{"(", 
           RowBox[{"2", " ", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"]}], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], "2"]}], ")"}], 
             RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              RowBox[{"Cos", "[", 
               RowBox[{"hu", "[", "s", "]"}], "]"}]}], " ", 
             RowBox[{"Cos", "[", 
              RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox["hu", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "2"]}], "+", 
            RowBox[{"2", " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox["hu", "\[Prime]",
               MultilineFunction->None], "[", "s", "]"}], " ", 
             RowBox[{
              SuperscriptBox["hv", "\[Prime]",
               MultilineFunction->None], "[", "s", "]"}]}], "-", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
             RowBox[{"Cos", "[", 
              RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox["hv", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "2"]}], "-", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox["hu", "\[Prime]\[Prime]",
               MultilineFunction->None], "[", "s", "]"}]}], "-", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox["hv", "\[Prime]\[Prime]",
               MultilineFunction->None], "[", "s", "]"}]}]}], ")"}], "/", 
          RowBox[{"(", 
           SqrtBox[
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"Cos", "[", 
                RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "2"]}], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox["hv", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "2"]}]], ")"}]}], 
         "-", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}]}], " ", 
               RowBox[{"Sin", "[", 
                RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}]}], "-", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
               RowBox[{"Sin", "[", 
                RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"-", "2"}], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
               RowBox[{"Sin", "[", 
                RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}]}], "+", 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], " ", 
               RowBox[{
                SuperscriptBox["hu", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}]}], "+", 
              RowBox[{"2", " ", 
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], " ", 
               RowBox[{
                SuperscriptBox["hv", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}]}]}], ")"}]}], 
           ")"}], "/", 
          RowBox[{"(", 
           RowBox[{"2", " ", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"]}], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], "2"]}], ")"}], 
             RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              RowBox[{"Cos", "[", 
               RowBox[{"hv", "[", "s", "]"}], "]"}]}], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox["hu", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "2"]}], "-", 
            RowBox[{"2", " ", 
             RowBox[{"Cos", "[", 
              RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox["hu", "\[Prime]",
               MultilineFunction->None], "[", "s", "]"}], " ", 
             RowBox[{
              SuperscriptBox["hv", "\[Prime]",
               MultilineFunction->None], "[", "s", "]"}]}], "-", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox["hv", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "2"]}], "+", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
             RowBox[{"Cos", "[", 
              RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox["hu", "\[Prime]\[Prime]",
               MultilineFunction->None], "[", "s", "]"}]}], "-", 
            RowBox[{
             RowBox[{"Sin", "[", 
              RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
             RowBox[{
              SuperscriptBox["hv", "\[Prime]\[Prime]",
               MultilineFunction->None], "[", "s", "]"}]}]}], ")"}], "/", 
          RowBox[{"(", 
           SqrtBox[
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"Cos", "[", 
                RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
              SuperscriptBox[
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], "2"]}], "+", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox["hv", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "2"]}]], ")"}]}], 
         "-", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}]}], "-", 
              RowBox[{
               RowBox[{"Sin", "[", 
                RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
               RowBox[{"Sin", "[", 
                RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"-", "2"}], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
               RowBox[{"Sin", "[", 
                RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}]}], "+", 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
               RowBox[{
                SuperscriptBox["hu", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], " ", 
               RowBox[{
                SuperscriptBox["hu", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}]}], "+", 
              RowBox[{"2", " ", 
               RowBox[{
                SuperscriptBox["hv", "\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}], " ", 
               RowBox[{
                SuperscriptBox["hv", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", "s", "]"}]}]}], ")"}]}], 
           ")"}], "/", 
          RowBox[{"(", 
           RowBox[{"2", " ", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"]}], "+", 
               SuperscriptBox[
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], "2"]}], ")"}], 
             RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}], "2"]}], ")"}]}], 
   ")"}], "/", 
  RowBox[{"(", 
   RowBox[{"\[Sqrt]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"Cos", "[", 
         RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["hv", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "2"]}], "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{"Cos", "[", 
            RowBox[{"hv", "[", "s", "]"}], "]"}]}], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox["hu", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}]}], "-", 
         RowBox[{
          RowBox[{"Cos", "[", 
           RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox["hv", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}]}]}], ")"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"Cos", "[", 
           RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox["hu", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}]}], "-", 
         RowBox[{
          RowBox[{"Sin", "[", 
           RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
          RowBox[{
           SuperscriptBox["hv", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}]}]}], ")"}], "2"]}], 
     ")"}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.6265875450874076`*^9, 3.6269382768291855`*^9}]
}, Closed]],

Cell["kn= ?", "Text",
 CellChangeTimes->{{3.6264283126767893`*^9, 3.6264283200244026`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"kn", "[", "s_", "]"}], " ", ":=", 
  RowBox[{
   RowBox[{"normalcurvature", "[", 
    RowBox[{
     RowBox[{"sphero", "[", "1", "]"}], ",", "hu", ",", "hv"}], "]"}], "[", 
   "s", "]"}]}]], "Input",
 CellChangeTimes->{{3.6262697808793507`*^9, 3.626269850185673*^9}, {
   3.6262706253152437`*^9, 3.6262706316176553`*^9}, {3.626271402313818*^9, 
   3.6262714245634575`*^9}, {3.62627146016672*^9, 3.626271463349126*^9}, 
   3.6262715810029325`*^9, 3.626271640242237*^9, 3.626333487987001*^9, 
   3.626428299026766*^9, {3.6264342729636583`*^9, 3.626434282604475*^9}, 
   3.6265876197023387`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kn", "[", "s", "]"}]], "Input",
 CellChangeTimes->{{3.6265876294055557`*^9, 3.6265876457231846`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{"Cos", "[", 
       RowBox[{"hv", "[", "s", "]"}], "]"}]}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Cos", "[", 
             RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"Cos", "[", 
             RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"Cos", "[", 
             RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"Sin", "[", 
             RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}], "-", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"Cos", "[", 
             RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"Sin", "[", 
             RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "-", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"Sin", "[", 
             RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"Sin", "[", 
             RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{"\[Sqrt]", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"hv", "[", "s", "]"}], "]"}], "4"]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"hv", "[", "s", "]"}], "]"}], "4"], " ", 
             SuperscriptBox[
              RowBox[{"Sin", "[", 
               RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}], "+", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}]}]}], ")"}], "2"]}], 
           ")"}]}], ")"}]}], "-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
             RowBox[{"Cos", "[", 
              RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"hv", "[", "s", "]"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{"Sin", "[", 
               RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"hv", "[", "s", "]"}], "]"}]}]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "4"}], " ", 
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"hv", "[", "s", "]"}], "]"}], "3"], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"hv", "[", "s", "]"}], "]"}]}], "-", 
            RowBox[{"4", " ", 
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"hv", "[", "s", "]"}], "]"}], "3"], " ", 
             SuperscriptBox[
              RowBox[{"Sin", "[", 
               RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"hv", "[", "s", "]"}], "]"}]}], "+", 
            RowBox[{"2", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}]}]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}], "-", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "-", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}]}], ")"}]}]}], 
           ")"}]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{"2", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"Cos", "[", 
                RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
              SuperscriptBox[
               RowBox[{"Cos", "[", 
                RowBox[{"hv", "[", "s", "]"}], "]"}], "4"]}], "+", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"Cos", "[", 
                RowBox[{"hv", "[", "s", "]"}], "]"}], "4"], " ", 
              SuperscriptBox[
               RowBox[{"Sin", "[", 
                RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}], "+", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                 RowBox[{"Sin", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}]}], "+", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                 SuperscriptBox[
                  RowBox[{"Sin", "[", 
                   RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                 RowBox[{"Sin", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}]}]}], ")"}], "2"]}], 
            ")"}], 
           RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}], " ", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["hv", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "2"]}], "-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"Cos", "[", 
          RowBox[{"hv", "[", "s", "]"}], "]"}]}], " ", 
        RowBox[{"Sin", "[", 
         RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox["hu", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}]}], "-", 
       RowBox[{
        RowBox[{"Cos", "[", 
         RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
        RowBox[{"Sin", "[", 
         RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox["hv", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"Cos", "[", 
              RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
            RowBox[{
             SuperscriptBox["hu", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}]}], ")"}], "/", 
          RowBox[{"(", 
           RowBox[{"\[Sqrt]", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               SuperscriptBox[
                RowBox[{"Cos", "[", 
                 RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
               SuperscriptBox[
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}], "4"]}], "+", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}], "4"], " ", 
               SuperscriptBox[
                RowBox[{"Sin", "[", 
                 RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}], "+", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                  RowBox[{"Cos", "[", 
                   RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                  RowBox[{"Sin", "[", 
                   RowBox[{"hv", "[", "s", "]"}], "]"}]}], "+", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                  SuperscriptBox[
                   RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                  RowBox[{"Sin", "[", 
                   RowBox[{"hv", "[", "s", "]"}], "]"}]}]}], ")"}], "2"]}], 
             ")"}]}], ")"}]}], ")"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"2", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
               RowBox[{"Sin", "[", 
                RowBox[{"hv", "[", "s", "]"}], "]"}]}], ")"}], "/", 
             RowBox[{"(", 
              RowBox[{"\[Sqrt]", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                  SuperscriptBox[
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "4"]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "4"], " ", 
                  SuperscriptBox[
                   RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}], "+", 
                 SuperscriptBox[
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}]}], "+", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}]}]}], ")"}], "2"]}], 
                ")"}]}], ")"}]}], ")"}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "4"}], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "3"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}]}], "-", 
               RowBox[{"4", " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "3"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}]}]}], ")"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}], "-", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "-", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}]}], ")"}]}]}],
               ")"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"hv", "[", "s", "]"}], "]"}], "4"]}], "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"hv", "[", "s", "]"}], "]"}], "4"], " ", 
                 SuperscriptBox[
                  RowBox[{"Sin", "[", 
                   RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}], "+", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}]}], "+", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}]}]}], ")"}], "2"]}], 
               ")"}], 
              RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}], " ", 
        RowBox[{
         SuperscriptBox["hv", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}]}]}], ")"}]}], "-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", 
         RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
        RowBox[{"Cos", "[", 
         RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox["hu", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}]}], "-", 
       RowBox[{
        RowBox[{"Sin", "[", 
         RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
        RowBox[{"Sin", "[", 
         RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
        RowBox[{
         SuperscriptBox["hv", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Cos", "[", 
           RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
          RowBox[{
           SuperscriptBox["hu", "\[Prime]",
            MultilineFunction->None], "[", "s", "]"}]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{"\[Sqrt]", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"hv", "[", "s", "]"}], "]"}], "4"]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"hv", "[", "s", "]"}], "]"}], "4"], " ", 
             SuperscriptBox[
              RowBox[{"Sin", "[", 
               RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}], "+", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}]}]}], ")"}], "2"]}], 
           ")"}]}], ")"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"2", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
               RowBox[{"Sin", "[", 
                RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
               RowBox[{"Sin", "[", 
                RowBox[{"hv", "[", "s", "]"}], "]"}]}], ")"}], "/", 
             RowBox[{"(", 
              RowBox[{"\[Sqrt]", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                  SuperscriptBox[
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "4"]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "4"], " ", 
                  SuperscriptBox[
                   RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}], "+", 
                 SuperscriptBox[
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}]}], "+", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}]}]}], ")"}], "2"]}], 
                ")"}]}], ")"}]}], ")"}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"hu", "[", "s", "]"}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "4"}], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "3"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}]}], "-", 
               RowBox[{"4", " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "3"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}]}]}], ")"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}], "-", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "-", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}]}], ")"}]}]}],
               ")"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"hv", "[", "s", "]"}], "]"}], "4"]}], "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"hv", "[", "s", "]"}], "]"}], "4"], " ", 
                 SuperscriptBox[
                  RowBox[{"Sin", "[", 
                   RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}], "+", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}]}], "+", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}]}]}], ")"}], "2"]}], 
               ")"}], 
              RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}], " ", 
        RowBox[{
         SuperscriptBox["hv", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}]}]}], ")"}]}]}], ")"}], 
  "/", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", 
          RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Cos", "[", 
          RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", 
          RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", 
          RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], " ", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["hu", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "2"]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Cos", "[", 
         RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", 
          RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", 
          RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Sin", "[", 
          RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", 
          RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], " ", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["hv", "\[Prime]",
        MultilineFunction->None], "[", "s", "]"}], "2"]}]}], 
   ")"}]}]], "Output",
 CellChangeTimes->{3.6265876654104195`*^9, 3.626938288279606*^9}]
}, Closed]],

Cell["kg = ?", "Text",
 CellChangeTimes->{{3.6264283602256727`*^9, 3.626428363844879*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"kg", "[", "s_", "]"}], " ", ":=", 
  RowBox[{
   RowBox[{"geodcurvature", "[", 
    RowBox[{
     RowBox[{"sphero", "[", "1", "]"}], ",", "hu", ",", "hv"}], "]"}], "[", 
   "s", "]"}]}]], "Input",
 CellChangeTimes->{{3.6262723065954123`*^9, 3.626272358893704*^9}, 
   3.626333502151826*^9, 3.6264283515676575`*^9, {3.626434335550968*^9, 
   3.6264343443025837`*^9}, {3.62658778540583*^9, 3.6265877896802373`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"kg", "[", "s", "]"}], " "}]], "Input",
 CellChangeTimes->{{3.6264191133335824`*^9, 3.626419169852482*^9}, 
   3.626428376168901*^9, 3.626434711745629*^9, {3.62643603896716*^9, 
   3.6264360445363703`*^9}, {3.626587803096261*^9, 3.626587812253477*^9}}],

Cell[BoxData[
 RowBox[{"\[Sqrt]", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "2"}], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}]}], "-", 
               RowBox[{"2", " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}]}]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], " ", 
             RowBox[{
              SuperscriptBox["hu", "\[Prime]",
               MultilineFunction->None], "[", "s", "]"}], " ", 
             RowBox[{
              SuperscriptBox["hv", "\[Prime]",
               MultilineFunction->None], "[", "s", "]"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "4"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{"2", " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], " ", 
             RowBox[{"\[Sqrt]", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], 
                 " ", 
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"]}], "+", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], 
                 " ", 
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"]}]}], 
               ")"}]}]}], ")"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox["hu", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "/", 
           RowBox[{"(", 
            RowBox[{"\[Sqrt]", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], 
                " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"]}], "+", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], 
                " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"]}]}], 
              ")"}]}], ")"}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              SuperscriptBox["hu", "\[Prime]",
               MultilineFunction->None], "[", "s", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"-", "2"}], " ", 
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}]}], "-", 
                  RowBox[{"2", " ", 
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}]}]}], ")"}]}], 
               "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"-", "2"}], " ", 
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}]}], "+", 
                  RowBox[{"2", " ", 
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}]}], "+", 
                  RowBox[{"2", " ", 
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}]}]}], ")"}]}], 
               "+", 
               RowBox[{"2", " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], 
                " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], 
                " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}]}]}], ")"}]}], 
            ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], 
                 " ", 
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"]}], "+", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], 
                 " ", 
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"]}]}], 
               ")"}], 
              RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}], "2"]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"Cos", "[", 
           RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}]}]}], ")"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox["hu", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "2"]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"2", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "4"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{"2", " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], " ", 
             RowBox[{"\[Sqrt]", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], 
                 " ", 
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"]}], "+", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], 
                 " ", 
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"]}]}], 
               ")"}]}]}], ")"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "2"}], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"hv", "[", "s", "]"}], "]"}]}]}], ")"}], " ", 
             SuperscriptBox[
              RowBox[{
               SuperscriptBox["hv", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], "2"]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"2", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "4"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{"2", " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hu", "[", "s", "]"}], "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], " ", 
             RowBox[{"\[Sqrt]", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], 
                 " ", 
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"]}], "+", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], 
                 " ", 
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"]}]}], 
               ")"}]}]}], ")"}]}], "+", 
          RowBox[{
           RowBox[{
            SuperscriptBox["hv", "\[Prime]\[Prime]",
             MultilineFunction->None], "[", "s", "]"}], "/", 
           RowBox[{"(", 
            RowBox[{"\[Sqrt]", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], 
                " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"]}], "+", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], 
                " ", 
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"]}]}], 
              ")"}]}], ")"}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              SuperscriptBox["hv", "\[Prime]",
               MultilineFunction->None], "[", "s", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hu", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"-", "2"}], " ", 
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}]}], "-", 
                  RowBox[{"2", " ", 
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}]}]}], ")"}]}], 
               "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  SuperscriptBox["hv", "\[Prime]",
                   MultilineFunction->None], "[", "s", "]"}], "2"], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"-", "2"}], " ", 
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}]}], "+", 
                  RowBox[{"2", " ", 
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}]}], "+", 
                  RowBox[{"2", " ", 
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{
                    SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}]}]}], ")"}]}], 
               "+", 
               RowBox[{"2", " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], 
                " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hu", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], 
                " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}], " ", 
                RowBox[{
                 SuperscriptBox["hv", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}]}]}], ")"}]}], 
            ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], 
                 " ", 
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox["hu", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"]}], "+", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], 
                 " ", 
                 SuperscriptBox[
                  RowBox[{
                   SuperscriptBox["hv", "\[Prime]",
                    MultilineFunction->None], "[", "s", "]"}], "2"]}]}], 
               ")"}], 
              RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}], "2"]}]}], ")"}], 
    "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"hu", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["hu", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "2"]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"Cos", "[", 
           RowBox[{"hv", "[", "s", "]"}], "]"}], "2"], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"hu", "[", "s", "]"}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["hv", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "2"]}]}], ")"}]}], 
   ")"}]}]], "Output",
 CellChangeTimes->{3.6265878141254807`*^9, 3.6269383013524284`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Together", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"kk", "[", "s", "]"}], "^", "2"}], " ", "-", 
     RowBox[{
      RowBox[{"kn", "[", "s", "]"}], "^", "2"}], "-", 
     RowBox[{
      RowBox[{"kg", "[", "s", "]"}], "^", "2"}]}], "]"}], ",", 
   RowBox[{"s", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6262723763989353`*^9, 3.6262724099117937`*^9}, {
  3.6262750511094522`*^9, 3.6262750574430637`*^9}, {3.6264192519866257`*^9, 
  3.6264192857918854`*^9}, {3.626436118028099*^9, 3.6264361445637455`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.626436228008292*^9, 3.626587864014368*^9, 
  3.62693838771418*^9}]
}, Closed]],

Cell["OK.", "Text",
 CellChangeTimes->{{3.6264393374525537`*^9, 3.626439341508561*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["4.3.4. The Loxodromes", "Subsubsection",
 CellChangeTimes->{{3.6266047968208046`*^9, 3.6266047984900074`*^9}, 
   3.62677957012638*^9, {3.6267796293128843`*^9, 3.6267796330412908`*^9}, {
   3.626939234093667*^9, 3.626939236667671*^9}, 3.627111569371826*^9}],

Cell["\<\
Loxodromes are curves on a sphere cutting the meridians under a constant \
angle. We will find a parameter representation of the boxroomes.\
\>", "Text",
 CellChangeTimes->{{3.6266997272444143`*^9, 3.62669980431555*^9}, {
  3.6268546917543693`*^9, 3.626854707590397*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"??", "dirangle"}]], "Input",
 CellChangeTimes->{{3.6266815607199965`*^9, 3.626681565105604*^9}}],

Cell[CellGroupData[{

Cell["Global`dirangle", "Print", "PrintUsage",
 CellChangeTimes->{3.6266815662912064`*^9},
 CellTags->"Info3626685166-6153013"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{
          RowBox[{"dirangle", "[", 
           RowBox[{"ff_", ",", "hu_", ",", "hv_"}], "]"}], "[", "s_", "]"}], ":=", 
         RowBox[{"Assuming", "[", 
          RowBox[{
           RowBox[{"Simplify", "[", 
            RowBox[{"Evaluate", "[", 
             RowBox[{
              RowBox[{"scond2", "[", "ff", "]"}], "[", "s", "]"}], "]"}], 
            "]"}], ",", 
           RowBox[{"FullSimplify", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{
               RowBox[{"Cos", "[", "\[Tau]", "]"}], "\[Equal]", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"\[Sigma]", "[", "1", "]"}], "[", "ff", "]"}], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}], ".", 
                RowBox[{
                 SuperscriptBox["\[Gamma]", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}]}]}], ",", 
              RowBox[{
               RowBox[{"Sin", "[", "\[Tau]", "]"}], "\[Equal]", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"\[Sigma]", "[", "2", "]"}], "[", "ff", "]"}], "[", 
                 RowBox[{
                  RowBox[{"hu", "[", "s", "]"}], ",", 
                  RowBox[{"hv", "[", "s", "]"}]}], "]"}], ".", 
                RowBox[{
                 SuperscriptBox["\[Gamma]", "\[Prime]",
                  MultilineFunction->None], "[", "s", "]"}]}]}], ",", 
              RowBox[{"0", "\[LessEqual]", "\[Tau]"}], ",", 
              RowBox[{"\[Tau]", "<", 
               RowBox[{"2", " ", "\[Pi]"}]}]}], "}"}], "]"}]}], "]"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`dirangle],
  Editable->False]], "Print",
 CellChangeTimes->{3.6266815663068066`*^9},
 CellTags->"Info3626685166-6153013"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dirangle", "[", 
   RowBox[{
    RowBox[{"sphero", "[", "1", "]"}], ",", "hu", ",", "hv"}], "]"}], "[", 
  "s", "]"}]], "Input",
 CellChangeTimes->{{3.6266816929266295`*^9, 3.6266817328344994`*^9}, {
  3.62668180653603*^9, 3.626681822560258*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", "\[Tau]", "]"}], "\[Equal]", 
    RowBox[{
     SqrtBox[
      SuperscriptBox[
       RowBox[{"Cos", "[", 
        RowBox[{"hv", "[", "s", "]"}], "]"}], "2"]], " ", 
     RowBox[{
      SuperscriptBox["hu", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"Sin", "[", "\[Tau]", "]"}], "\[Equal]", 
    RowBox[{
     SuperscriptBox["hv", "\[Prime]",
      MultilineFunction->None], "[", "s", "]"}]}], ",", 
   RowBox[{"\[Tau]", "\[GreaterEqual]", "0"}], ",", 
   RowBox[{"\[Tau]", "<", 
    RowBox[{"2", " ", "\[Pi]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.626681776178376*^9, 3.6266818250094624`*^9}]
}, Closed]],

Cell["\<\
The solution hu, hv of this differental equation with constant \[Tau] are\
\>", "Text",
 CellChangeTimes->{{3.6266998655018573`*^9, 3.6266999297487707`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"hv", "[", "s_", "]"}], " ", ":=", 
  RowBox[{"s", "*", 
   RowBox[{"Sin", "[", "\[Tau]", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6266953290710692`*^9, 3.6266953701957417`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "1"}], "<", " ", "a", "<", "1"}], ",", 
   RowBox[{"Integrate", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Cos", "[", "\[Tau]", "]"}], "/", 
      RowBox[{"Cos", "[", 
       RowBox[{"s", "*", 
        RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ",", " ", "s"}], 
    "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.626682397567871*^9, 3.62668249922965*^9}, {
   3.62668253890452*^9, 3.6266825795455914`*^9}, {3.6266951065030775`*^9, 
   3.6266951253323107`*^9}, {3.626695197129637*^9, 3.6266952059934525`*^9}, {
   3.6266952711577673`*^9, 3.6266952744493732`*^9}, 3.6266953045292263`*^9, {
   3.626695458143097*^9, 3.626695484572543*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cos", "[", "\[Tau]", "]"}], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{"Csc", "[", "\[Tau]", "]"}]}], " ", 
     RowBox[{"Log", "[", 
      RowBox[{
       RowBox[{"Cos", "[", 
        RowBox[{
         FractionBox["1", "2"], " ", "s", " ", 
         RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
       RowBox[{"Sin", "[", 
        RowBox[{
         FractionBox["1", "2"], " ", "s", " ", 
         RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], "+", 
    RowBox[{
     RowBox[{"Csc", "[", "\[Tau]", "]"}], " ", 
     RowBox[{"Log", "[", 
      RowBox[{
       RowBox[{"Cos", "[", 
        RowBox[{
         FractionBox["1", "2"], " ", "s", " ", 
         RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
       RowBox[{"Sin", "[", 
        RowBox[{
         FractionBox["1", "2"], " ", "s", " ", 
         RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}]}], 
   ")"}]}]], "Output",
 CellChangeTimes->{3.6266825876264057`*^9, 3.6266955080817842`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"0", "<", " ", "\[Tau]", "<", 
    RowBox[{"2", "*", "Pi"}]}], ",", 
   RowBox[{"Simplify", "[", 
    RowBox[{"Together", "[", "%", "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6266826250246716`*^9, 3.626682673405357*^9}, {
  3.6266955731574993`*^9, 3.6266955992749453`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"Log", "[", 
      RowBox[{
       RowBox[{"Cos", "[", 
        RowBox[{
         FractionBox["1", "2"], " ", "s", " ", 
         RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
       RowBox[{"Sin", "[", 
        RowBox[{
         FractionBox["1", "2"], " ", "s", " ", 
         RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], "+", 
    RowBox[{"Log", "[", 
     RowBox[{
      RowBox[{"Cos", "[", 
       RowBox[{
        FractionBox["1", "2"], " ", "s", " ", 
        RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
      RowBox[{"Sin", "[", 
       RowBox[{
        FractionBox["1", "2"], " ", "s", " ", 
        RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
   ")"}]}]], "Output",
 CellChangeTimes->{3.626682677320964*^9, 3.626695604906555*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"hu", "[", "s_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"Cot", "[", "\[Tau]", "]"}], "*", "\n", "   ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"Log", "[", 
       RowBox[{
        RowBox[{"Cos", "[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "/", "2"}], ")"}], "*", "s", "*", 
          RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", "-", " ", "\n", 
        "       ", 
        RowBox[{"Sin", "[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "/", "2"}], ")"}], "*", "s", "*", 
          RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], " ", "+", 
     " ", "\n", "    ", 
     RowBox[{"Log", "[", 
      RowBox[{
       RowBox[{"Cos", "[", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "/", "2"}], ")"}], "*", "s", "*", 
         RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", "+", " ", 
       RowBox[{"Sin", "[", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "/", "2"}], ")"}], "*", "s", "*", 
         RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
    ")"}]}]}]], "Input",
 CellChangeTimes->{{3.6266929748531227`*^9, 3.6266931154963703`*^9}, {
   3.626693152362235*^9, 3.6266932223821588`*^9}, {3.626693277800456*^9, 
   3.6266932901712785`*^9}, 3.626693320329131*^9, {3.626693406916284*^9, 
   3.6266934795570116`*^9}, 3.6266935171092777`*^9, {3.62669355798835*^9, 
   3.6266935625903583`*^9}, {3.626695646982829*^9, 3.6266956926939096`*^9}, 
   3.6267022427949004`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sph2", " ", "=", " ", 
   RowBox[{"ParametricPlot3D", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"sphero", "[", "1", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"u", ",", "0", ",", 
       RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"v", ",", 
       RowBox[{
        RowBox[{"-", "Pi"}], "/", "2"}], ",", 
       RowBox[{"Pi", "/", "2"}]}], "}"}], " ", ",", 
     RowBox[{"PlotStyle", " ", "\[Rule]", " ", 
      RowBox[{"Opacity", "[", ".3", "]"}]}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.626700142750597*^9, 3.6267002763204327`*^9}, {
   3.6267003363729386`*^9, 3.6267003788558135`*^9}, 3.6267004280195007`*^9}],

Cell["Plot the curve using several values of  \[Tau] :", "Text",
 CellChangeTimes->{{3.6267000184877267`*^9, 3.6267000308741484`*^9}, {
  3.6267000844668946`*^9, 3.6267001355433846`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"sphero", "[", "1", "]"}], "[", 
     RowBox[{
      RowBox[{"hu", "[", "t", "]"}], ",", 
      RowBox[{"hv", "[", "t", "]"}]}], "]"}], "/.", 
    RowBox[{"\[Tau]", "\[Rule]", 
     RowBox[{"Pi", "/", "20"}]}]}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"t", ",", 
     RowBox[{"-", "85"}], ",", "85"}], "}"}], ",", " ", 
   RowBox[{"PlotStyle", " ", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{"Red", ",", "Thick"}], "}"}]}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6266935797689886`*^9, 3.626693649227311*^9}, {
   3.626694666091902*^9, 3.626694681382929*^9}, {3.6266947260332074`*^9, 
   3.6266947287008123`*^9}, 3.626694767844281*^9, {3.6266958384569664`*^9, 
   3.626695875681632*^9}, {3.6266968566601596`*^9, 3.6266968792957993`*^9}, {
   3.6266969845425844`*^9, 3.626696987225789*^9}, {3.6266970299728646`*^9, 
   3.6266971457360687`*^9}, 3.626700001327697*^9, 3.626700442467126*^9, 
   3.626779737436674*^9}],

Cell[BoxData[
 Graphics3DBox[{{}, {}, 
   {RGBColor[1, 0, 0], Thickness[Large], Line3DBox[CompressedData["
1:eJwNl3c8FA4YxjMziupHFFIpUYSMMuptIFuJbJVKEoUiKimRSGWkISEKKXtn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     "]], Line3DBox[CompressedData["
1:eJwNl2k4FFwUxxGioqRESFRUSgveNjmkIiV70UqWSELZVZJUskSULIVUhOz7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     "]], Line3DBox[CompressedData["
1:eJwVl3c01f8fx1HqS0pkVEYyGooUWqqXECotGipKQkqDkoYdkT1CKCOjMrMi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     "]], Line3DBox[CompressedData["
1:eJwNl3c4FXwUx42mVRlRhIpKRoMio5MklRZFgySREiohoZJR9igpSkS2hGTr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     "]], Line3DBox[CompressedData["
1:eJwVl3k4FIwTx+WOUOkiJclVqTeRCJPKlSIUUb1IrlwpKiJFenW4cxTKFYpC
5GZy5KZ1xbqWtWsXe6CQEj+/P+af7x/fZ56ZeWY+I3XVw9Sek4ODg74aXKtx
mth9UFOBC49zWCde4RJGUe7a5ZUVNsYeaV67N3tGSynZTPKQ6lkIOfpk7//1
fy12f7Pd9QA1CqgEYN5E+zvjcVmres9vso63mi38XV+ffiYzEsaGM/Nrl9nI
0FLK8+aJxgVFeTgnGIUZoQ33zv1lY6QU7d80xUTQ5JaMcdr7Ak+8Llky+sPG
vZ5rPOSuvgG3NyGh5kFJECbgqIm/2Fj+366F5LJwFNLhs/N+kAXzQaoGKXOr
PvNuBWnJOdghGBfq8zkSmo/zHZibYWPpKU/iJe8P6Kwf2a5gloIxVfuWyphs
XEz4ZP5mJRbZEbwS54w/4T2fm/WzdDa6cXdGXn6SBIZsN66ZXwUYEb8rNXGM
jYMBDjvM1xbAgOEd6xClbITw2YySITZOOaf/4nEtheAl14+pmfexvGA7W7eX
jRNsA1fJnaXwYdCbv/57Brg368TpEdjo7fNJKjr9EwwEG/2JFS8G/8cRVWWN
bHT2X9iQSXoDpNRqQ/XSKpDx4H3f/IWN0xM9frUB0egVHPTtR/EXiDwvTjQt
WfW/l6Xr3JaPPTJ9+E0cwV1gxcMsl41ZfPfF5YQrUfGzgX6LcQmE8uyLaX3L
RpmkTn7ZDzV4kujdnqWaA0+LLTyrEtiYWKXtqbtQh/IXTtzba+8HOSMPDipE
stFy2U04RvsrStVZeTlhBh7uG1ta95iNlTKc+YKJtSgd+tySurMEh5uX+a/7
spEkcKVKQ+sLfgpXLN9aUo1c0+HBx93Z+DplrWnGXBFuvNorcu1UPQ4XvsiI
sGWj0c8tGtcJGRgnEkGfd29C3WH9VKvzbJzjf5JjUuIN3mLCnDYbWpCeLJP/
WpeNf0XlNwlu/ACOVrF7t0m14GTuiNCVo2wc1z+ooG5XDo/JfYPm2U3YqO0y
F6PAxl8d13W0U2shyJVueo1Qj1zrZjIMxFf7KMNt0tjRCA9kJaMGqmqQU6XX
7q4AG7f7DFoPCLRBy7/FNub/lqOXRrnH7t8sPLLh6ecWUQIErZs0Fbufi5b9
nuv1J1jYzx3Oc4e/AxIiipw28j7BsG6esIleFmbdsOSo+rcD3gVcmZ5ZTIOG
5iIlvnoWUgsPhXFu64AMVRONpF3FsH1x4XRKAQtvX1WRH1YnAEfiG3tJZjXo
zbjLlCSzkOdbZeDt9DaYZSvi+rEGiDw9u/FMKAs1yLLupL4mKCoamFDWa4P1
+iEOV3xYaNly5m218lfI/XF2O9cfAlC87b0Z11jovVHw8EAZws9m+qaGiU7o
1ZiKXTRmofWfzclhPwvA5GdO1lv7bljnlrEnWJ2F/3WTa38lJIHUx+WpGcce
WK466xu+h4UuhzOm9e7GYuqvOx8uMnsg3PDL3DZhFgZLFBuat33CQPu/3vFz
PZD/m2tBaoGJv/z82x4OIQ6TxE7/E9QDIhMnR96TmEjYk0XS1KnHH32Sm9Ym
dINsXfvmwgYmrl3754aMYSseDuRlSpzqgm87Ni9r5jJxVMnY22OWgMkkEeLO
/zpgU1HmGoNYJt4QSeU44t+Fr5WFlg0G28G8QuVZjx8Tw/7ZmlVT3oOZnPeb
P/A3w0G+Z3yUq0wsji7RJ7ztxadt1tna2XUQdDKM6qXPxENGiVyzykQMvJRX
V1haCXvEITZEkYmBfuMKwdf6kdtjXVWEWz7InC18LrmRiTJ3EwYTDw2gX2FB
757oKIhvEdNSn2dg5pRxCf3VABZfcjl2figZv1d8lCISGejawsa45AG0qlb2
0Uj8jNGBtXlLFQwUuvUx5uipAdxTOXYv36oaoz4VO7x9w0DfrEesuJB+HKUd
v7m2oQGXeH996XrIQE6DecHXnkQE12H3F8ttaK27pBVix8Axqf8m9nH3YdDh
Yy8PLHRgym6dq7WnGEiYMxnz0OzGIKZdZqNlL/5Z35D1jYeBWYaVxREzHTj7
LLOVu4iIVffFiSnUKXwsVxyd1NWOTUbdVbc7VvOtITcu1U1hwZy8yMSlJmxz
I1zUyhzCZIEcrtG0KbyZodor/LwWh0JPcLgcJiHvwt8txoFTeKTPu6SEXoa4
uVnuntoIpqpbRunZTCGPxH+Mhhc5WGf77MJw/gguHVHha9Gcwp9pARdn+72R
bu4tqM45ilvC/vL3i09hnZzammzfTGBEXGg/t3cUeXJEFL0XJvFr//d4a1oJ
CKe+qdNd1Uf1SwSSuiaxKnCT+6Y1tXBA1eRqwqpP1ug/x0xzJ/HUGqU207tN
cDDpKe4uHEF7hSTLmKeTGOfU0f+y5Rv8Mg4qXIYR/OHJz+VoP4l7ZsuTpuc7
IWez4tuDxiQkrrP73gCT6M25g+Y30QOhWbWv9xOH0CctwKtEbBJfiJYkFl0e
gEybtzrT+/tx9m96kmHrBIZod5UM8A6D15CE+oO2Xty8oe5WVMoEcliFRhWr
j4ASY4P6yUM9KL9Ww+Ti7QkkREyq8+qR4cqONKWAknacC6t/em3nBP44mRt9
a9sYcM2mq5BtG9H+llNKzgwdRVRbesbrxuCy04bjeg3V6KDqo3ynjo6GFYt/
XAwpMHb/0QUd5mfU1vaS+xpLR4dp5/O22RSom7K9Jcl+g/PC+jZxTnTMlEl9
5EWhgIb919y97+JhPs+kgaVGx/EAKfk38xRY9Mw2qXMuhHNqF/VaBehoMSoq
Lk2mwMuI7T0C56th/c6Qtn8GaCiqdtLncSYFFKU57MrtG4Erwt1gRzYN2Z8M
vcL1KFB1wfxn8JVvoPL1Z06ULw1Psn9NQu0YrBulhlRv7YIO1UuMSAMaevZs
U/9oSgZhns0Bn18R4ZHq1YFD4+N4RujOQ0/nUQh0Yq01fj8IzknrAroLxtFH
Ppc2ZDsC+61MG82iSLC0I2Wc++E4Wnn0y1OWB4DWJ/nQ2Y0MZdWenKLi45jC
7SH53IQIJs8eOzBfjkHBzIeKWSoVj8Qt+Mrc/Q75H2sHVVbr9EyuWfdaPhWj
TX9YoxkBHNUurm+4PQ75iS9ZHHpUfGib7PNyTzM4KQt4jR+gwX61ZOkDG6jo
eOHtjmyPWgie4GHebqDB2fXSFjP9FJxTizTglikFoqrs4jDQIX1W8JVhOgXl
/e9qFV3KAElL547CODrklZ1f0XSj4OVi0zjumP9Q8uFnzcp2OuyirklsUqGg
ts99EwmFfJzR1jk2SaHDOskd/7H/jmH3ZVP18KuItvbPfS376RDWlEvM+zqG
hMKEGbn6ehTcdqjUNJcO3F6VH4VDx1BPcZflpuY2bD8asKRzjQ7zhlrSAufH
UGWIK0fcqhOtRx/lNS3SgBooZp4uPoY+E7vvJq0jorBXhGpE0zjsWcvjV5pB
RtmwlEyeE4P4/FFayweecXAiXSo64krGU2a2Rb5WJIyvpwsJyVKhLDtM0+IQ
GQdVfahnosn4elb2ccjqXJ95qt/zuGwU+y/7wOTEGOZZv/v0L3MUVILeHYu/
P4rlr4IrkvdRsT5VY4WaPQJmAWGvTE6M4g3jl1b7XWi4FLOQV9I3AK07mjmq
G0fQ9W4lSadyEm1fxKfEvWuH6Ff7BJ3SSPjyiEmwrdEUrpy73FTFagD32VKv
s7Ik1OHVF3ConULnAjPVIxuroZuDVyo2axj3uvkc+r2dgXPnB4K7jxQCb0B6
yKm9w5gf8cBhzIKB06KnbxwPiwcrByW5S9lD+Nu+5ATPHQZaCBUdlRNKxlZP
/XuD+4ZQ0r7ut40PAyekbqdQjhahfKQ7b0v2IOol7i3aZM3AgzbdRFelGnQ+
I3JWYd8gylf4d1rJM9BfgfFeRKsJOcVcvi++G0DTyjDRJz1TWFhIFFKQICAz
5dl+kB9Ac8ktNSKOU6i7TkZbIbQLNThUB/+87cePFVddHNQmMSKc5OCt2Y8e
FZkh1DdEfHbOvznq3gTu5D7+0vL2EL74fDhnqwQRNz46f7Q9nY7eleLrFAxH
8G9b5e2WuD488FR1USJvHOPTqwU4jo/hDV7e6zWhvah3rHHLUCYVD5ePJHI+
o6CCxoWElfWre65Pj8MylILKVUuskHoq7s8mVPRGf8fbhlEXrsqQca5e6MkV
QTpGG12zs43vwffjV7t+E0fwZKZW9fTGCdyapX5kq3gPpumGE+o5SNjRih5R
fJO4lJXx4kZCN8ZsUHE6ZUTEs5PtBpqfp9B6zj5lS1IXivzhjo02+45lWqRI
YTcGJvRKvnfc0YXPRA7l1el0Yb3138X7Ikx84Kfw41pSJz628OTTlSNgfYtW
yI5EJrIy/v2wfkcncrS06kjMNWGF1/yHpxtZOOLOIeiW2IHClaKthx/V4rpu
X/O7N1kYlpoNfts7MKN7ZPL9mjK8tzFu8l05C8t3GT84/oqAVdfj18h+eYfJ
QxP6P5ks3K24m6tiGwEp5DQhjujzcLfpdoHm2lVufqCyMqr4DY823imu2ZIN
Xw/mf/IXYiNvWsPlWoV2/GKcWCBbXQZFlwxYFas8ukE5M2qXTBvOHFZ07/pZ
CzLBvEubOliYll27ZVmyFYfP7NUL9G+G4ozFPXkRLFT6fczWRrwFL3vqaoz4
EqAy0VyqSY2FMHeaYbipGSXFYgNscrpATfVEy/tWJnYUH1b+KtSE0rwm36TG
iTBTKnrgRD4DBd1N526u1KO35qv0jolB2Bo8vHmAi4FerMc38ha+Yseu5q21
RBKMWlgpc56YQj3i9ZxQdh2aFn1S+l1Ghkb+QjGboAlUW0PzUxuuQfOa8Rgo
pAHjxlz8kiIVxcmvqO89KjH7qCnp3xY61EzQ+7Wnx/DrRqECyUvl6KIr8vFQ
xwQUTiS2SqeTkb/O8a2Ebileswh13ZYxBbHGP+flmkh46vXCnS7xzyg9c5s+
cosBddf4+nTGB3G9gmniG+4CvG0gV1OvyAT/LzOySSwitsyEu21j5aHK/tRq
ph0LdqQzCK+auvCzboS184N3OGLSKnKBxAKHQ+pJuckEnMfkkW6Tt5gr9DtE
VocNxMOWUbm9zZgfJbxI3p2CkYyzRPEYNlzME5yutqrDEH/h34a0l2hgpcD/
u50NzzYFnvB7XY6zXpyPbEIisTeYEZcwywYRYa2aC7Qc9HL56HPONBBPq9ES
x1fYIB8yPThLu4WMwx4Wv48a4C8vr5aOeTaIfVOxkGJngkFRo+IN8/vQsEbT
366XDfPpW6aEVu/MGaGQK5UT4TAb/uZDfgobagaU8k/uqgXS3JDTgFk8iFoB
c/TCqo/C2VbD5CbI3WxYf8QyGf4ZTtRq9WOB2sv39iZKXTA3vvSJVJQFZPfr
N3ZOM8GG7X+HbPwdtr70GRen5sAwfUtCoBETxt+du+dxhQiSfy2/uDzJg4K5
K0kX2qdg5Tm/VSmQgJau0KdTVAhpi3y/O6cnoVqaXiXGPwrdiv0WB1uL4F36
kar7HJPwRUzuoeg1MsR8V5krHC0Blu3J11smaTBLVQoy+kEBV41k7huClSBx
v46Tf4kMHjs+D7lemADv7dl7zHJqYFNhmkXlrVFIOaYdue/yJGhm6Myt6a6F
vPhUDXIJCTb53nDkNZyCJwWOj079qYPw6BP/aAoR4Vud5Ia4IQasffgiz0W/
AdwUtcc0Y3ugkt9frD6ACVLqvJUn3RuB1OfiYCLUCc9Drxvx8LHgA8dGx9jo
JiitOtjsOtUKbl4ztc03WaDiddLtSkkzpI1vJvi+/QrDlsvt6TUsqIrfFRk1
2AL1PxJzt72pgIZbOcnq8yxYZhrGHeRog+2DpaK06mzoaqISzgixIcY7Q++A
dDtkThuzjZVdMfRhdu9nPjY8yfR0C9H5BoPZwgbuKrno3X3J6zqVBYrvS6YS
DhPgRX3Qir5GFcapGXH6ZrDgZynv/u11BFAWz+1QuFyP55OlyoQNWaA1XUrW
Ot8Bwwf371jR7kTC1TV9C8CEx9IBzo63OuFA2rqYpdwedGisVZKPZMBGep2Y
A2cXeHgWeDfzrv5NF4OjYhqmgNXxi+NvRBdcKbm5L8qWhJSG3cb8tAnQn799
hpbTDUEcv+VDD46iuZh3wJcOOmQ3vktVVu+BLz0OXIaJZLRfvhAn/pYGSisV
osINPaDrmXa4Q42KwysvW5QFqPCrdKOOJek7GCqdaiM4juORIJcTN1+PwVAI
l+DX673wTtyGeuw+DfnJ862nd5Fh70+Thby5Xjggse9MvPUExq/E9j+mDkFn
5rTL37VE2B+8C+3kJzFnoD5BJrQfvq9HkYvRRAjjNmKY9Ezi8xQ3rgKJXmAF
8W2Qk+iHW+Ein237pvA8lK4w1hDAKvN80Jb9A2B0xP/0QUUGmrm3PNpd1QhN
HO/lDhUMwFbkEfVd/c+EX4/4iDtUg6eNekSh2iDcXPGfGfNloHJjr6UnXyH0
db8eTsJBWEn7I3vwFgM9NqsHEn69gPD6UtXJU0Og7plqBGcYKKLvX7Z1ORUH
wutrY5uGwFmBcVycl4F/eOJI7yVLUILT+WPy2WHgfO09SE9e5YQZQrCXQS3G
9lXI8nQMQ9lPk08LElP45Kv8z792zXh6m711tRkJWF8Go4t9J/GfOGepkGwC
hl5OCOvsIcG4x9JKZdkElsfZRe4T6kbuEMmFV/tGoPe7zs0PFBo+rOdtML/b
j+ulb0r8XOWyamUb55d/yUjTdFLm/ULGYe1D5y2yRuHNJ5N6vqejKHHKM5R3
fAx9t4rfCRwfhdSzct3RnCMofYW6c8MMBXN2UoYN95CB53Fi5LYAIsrfzLlj
tsodYfdIaoWvyeC9M9hzy/rvaPEw0INkS8PhuuqIcwNkyJe2tot+0olSUsXu
0j9oWO849yN46xh08KUP2oy0oXmlS9JuOzrOzFSLm5mNwRNXmvODyHrUrB30
48+nY+/arKqisDG46qT8kN+2Cq+o+FzfN0jHs/ZOCblNY+D7OCiIvjMXZz0W
14pQ6NhZIOChwU2BmZfX0xtdnXDktHekQyMdnfwoi5e0KJD/QNJZeiAbBPoV
Z7c8oeMQ683yursUcJd+4hrpVgHPaTf7jsrS8XLWiRNG+RT4bpcRfdzsK0zc
/+InnkzDbS7Kz3dOUiB9rvCMmHYruCuNg/bCOP5u1Mnw202FjEkHt2TfDpDg
rBZ02T+O26qLLjtaUeF/uTjSgA==
     "]]}},
  AutomaticImageSize->True,
  Axes->True,
  ImageSize->{365.6764864619431, 362.77900125627303`},
  Method->{},
  PlotRange->{All, All, All},
  PlotRangePadding->{Automatic, Automatic, Automatic},
  ViewPoint->{2.6408615991235784`, -1.6602321093658132`, 1.311289196670667},
  ViewVertical->{0.5720149453828716, -0.38062818570124207`, 
   0.7405595866007861}]], "Output",
 CellChangeTimes->{
  3.6266936581661267`*^9, 3.6266946857353363`*^9, 3.6266947311344166`*^9, 
   3.6266947692014837`*^9, 3.626695846787381*^9, 3.626695877865636*^9, 
   3.626696885788411*^9, 3.626696989768594*^9, {3.6266970337324715`*^9, 
   3.6266970610677195`*^9}, {3.626697091086172*^9, 3.6266971156250153`*^9}, 
   3.6266971484816732`*^9, 3.626700008254109*^9, 3.6267003990462494`*^9, 
   3.6267004439023285`*^9, 3.626779739027877*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"%", ",", "sph2"}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.626696892652423*^9, 3.626696960298142*^9}, 
   3.6267003919930363`*^9}],

Cell[BoxData[
 Graphics3DBox[{{{}, {}, 
    {RGBColor[1, 0, 0], Thickness[Large], Line3DBox[CompressedData["
1:eJwNl3c8FA4YxjMziupHFFIpUYSMMuptIFuJbJVKEoUiKimRSGWkISEKKXtn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      "]], Line3DBox[CompressedData["
1:eJwNl2k4FFwUxxGioqRESFRUSgveNjmkIiV70UqWSELZVZJUskSULIVUhOz7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      "]], Line3DBox[CompressedData["
1:eJwVl3c01f8fx1HqS0pkVEYyGooUWqqXECotGipKQkqDkoYdkT1CKCOjMrMi
82Xvfe19udzrXvd+IiuSn99f73Oe531e531e79d5vx+P7cZPdE052NjYdNjZ
2FatrKTdJfrp+hLwX8yen78YOzFo//DU8jKBVNIYoSp5FjdEOD5+fVcDEmZX
Df1byZ0v/ohXynXEnd3HG7SUrbGVP/j915V87yp2U5S6C8v7z27SDQ6A8ZB4
gYp/BPqmVAvss3+PAXI3yHNXA/Be7YZ715YIPDHOvfWwzEdY/71i68CeYPx4
f23ArUUCL3ccGKk4EQ0i5sksk4aPEOcp9bFnnsBoYnVygbMnZoZVuVsnfwOJ
l5f962YILDrJ9+XlUBJub/dBh4AgeO9Ms98/ReB6lrLPrEgq1uod6DBfH4Vv
Szyf8bAIJLY9/h49Fo7RNSJymrXpeKVbzcVgnEC7LUpaAh9C4UBHnG+pfBaS
YqxLtlMIdJHws1u9JgOierYQlVIp6NI5qGEwQCDdgFSixJkL5k8vzkXoemFG
StQOni4CW+Am++e3uVA95OCf9S4WlC1k3+xvIXCOTVDBtjMTYiy22ren/4AF
5sXn9TUErrnNGXJPIBa+/Tv913KpEC7InpDoLyHQ/Sjnoau8njgrKzhVnlYM
c5E28XdzCdxvvXPk7N00nNGM1Ln6CYEDO3cbpRF4PShqufNfPuoealYIVMoF
i0sf2lq/EOjxm8z77UYpiuLfNVLmqeBTqdaU/4nA91dX79XvKsef/MYSx466
w02XolMigQRu+Uveve98Bd5Zzn0lnxqHNpKnzCbdCGyX7WIPrS5DqUu1rioa
OcibZHbnhC2BYvQAtjUOxfjeuIw+ZVaCCYVrH3A8IXDJbP9IvksOmu4fl+7g
rcSsbKMfx40JDGk4TPMkf8Vn27wPZOrV4Lqqv69/XSZQhkHq2Dz2EAaW480+
cdWhLkfu9FYtAgu8rz11j0yFA06cYRGCdTidomHnfOj/89lpHaVVCHpPeJNq
LtVgkEqhsfZOAvsiYndkLpXDBYtIKzt6BUo0CS8+ESLwtMSGW49v14LfZGK6
/7YStJV9+XF+NYGR1Nsg59EIDV7toazX2cgxIx8xPMVCo4iANodvzaBqovnh
kUEMtk/lGMgPsVDdWVK06GILZOS0LAovvAfanp9H+upZ2LD276M+lRbQCVzu
NzHPhEmOvyETP1m4kS3ooK5lM/DoLlCPFBTDgWPPe27HsbD+ut+S5mwD7HC9
tSh2uQpkpw8/OujHQob94gSrrwbOCCWGG6g1wDE3+3zLVywUfiD71La4HJzl
qfR/e1pA/26oEu9dFrZ6mz/Pii2EkqiIXxaHSZDYtXiGX4eFZ3Tchnw80iAk
7GJAem4bGFUqWb9WYqGJx8yR0/2uQMRlK4p+awdvTdoubVEW6kTdVZ2y+4ou
ae9kvrF1wDGNFortKhZeUCUK87bn4fP8c5Ve5e1QwVG0Q4jOxEPq3raVm8tx
3PrW46z5NrgS7GMp3MxEhsVZ3fr6Wvyvzb39WDwJ2jTltJx+MHG7V8u7fJdm
9JJOuz3c1gIHpoIPXApnIt/wePGByySUTzRatbirCTaerSn0dGDiAYP5MdXa
dkx8fq+ebbQGZoSabOSMmag+lHb3rEM3fkl3W2RJ5sMpY4fctJ1M5A9+AYap
PfhzzcVIsw1JkPZL5XEgFxPjBSQE/9zrxVCvqWMfho2xRIj1ZZA+gZG4GCSa
2IsaS37Tx1gpyHbAjy2sbgLti9J5u573Ys8jk2SnG4V4onK9CCZN4GcDntsi
jT04erFZYG1ZBVacPR2m6zWBejKhL5xyunHIdrBP/XM9OrA2bL/8YAJza++w
Cx7uwoqNGb9Pf2hBkRNCN8u1JzDdapOI9LUObHmTT+JIbEPdpqWp2B0r51Gw
UGYXa8Nf1reSIx068VKj1offqyYwr/NdRVtgC4Ytqn8/xdOD199c/ZM5xEAn
gycVHa4N+MMtjNv2bB+mlZo3jBYw8MO6TEfeh1VIHF5zRfTgAE4ZhNR6hDLw
+Q/5mCe/ilGlLOm4QdUg1j2sT4x+xkCjHZrnf5zPwkd5TeF99kN4JL5Gds95
BhYpxhpF7A9Dt8Ndhi9pQ2jmtZVbdhcDe87XCIsQUWDRyP57TpKMO2IF10Wy
M1BW4a2Iu1E22HoX9hrJkVHJp3PKvYeOgaFG0b8NSmH8c8rQ8f/IqCoMHrQM
OmaueQBtljWQ9kq5NChrCKnHx0OLPen444Ze6fDeZqipWmfXqziEERJyxEZj
OqrOpBy960IC5gBfYA/vIMZQhQ16DtMxZWL+ytqsDrhkaye2bNqPuz6ZZ4jx
0lGnR+qilXsfeKhXZcwpdKPD2tBngnnjqOB6brxWbBA8J7WX76R34MJ67YA6
33HcOL+8qiBvCKRsushNsyTsnpxb+me8kn8ZVhxNGoaj1PVx5zrrcNWaZk0y
1zh+9PYX7X00AnoXr1at0q1AIiyJHtBHQ0PYEVTERYHCJ2yK7qKFePIn5WFZ
Kg3lR3VqCh0oQEtYLWx+PxVPe+bk2DjR8Fb9C7cb9RTwahe32utvifmxoznJ
l2gYlhlKNpuiACXJWmIsKBG4tdMNzCRpyDbP5sk+SYH9SQaOEon5QI/p+hA3
RcVbIaIVAdUU8M8a/Bu68v7tnfY3NC+lYs27l7vLXlHArOTJSA9fPcykOsdl
BFDxHfPCDmdOCsjbl7KnvWkBG280dTWiYqRKisnUwxG4+P5jhORYG6gEVn0a
kKeioYbSquA2MrByHyiO6/XCpdeCZgq1Y7i66nNq6fgQqGRIz16WHIBUaTEB
//AxpFwQlaisGQTNOfOaQdMh+DOevez5YAwrZVMMtXb2Au1c5dzGlmHoagkZ
v8I1hq5dyiNRwV3wT/4bBFBGQKJKLmxn1yhuKz32h22gHXr/CNeFD1JAcfIX
7cOXURSdvrPTiqcZfryived4NwZ00q2Ba2qjeJ/DtWF+ugYU53rfRylSIT2Z
rpbAO4qezTPSC3Fl4Lt3fZ5aGRWcDxhWfOqj4MidNNedxnng9Hz0r8VBGrzc
cOq0YgIFn/EKHml6mwg8WpVNxd40KH7alPnoOQUtLs4eCeHiwS9t57TWVdJg
SbScoXOSguvyzfx7ShLxZHaV2JFhGgRecu5u4aGgmd8h4UcVefhLazr8BpkG
5U1zZkudI1jTTapJ2liOEgL7jT+X0WCx6Itjc8wIxvSvqkrOqMV/Dxm7b3vQ
gAj6sFHn0QjqjoixfY5uxjXZ0rdXK9KAZ+b6f1aHRvB84jW9kB4SnvwvNUe8
hArrPZavqrKPYNj+twbZqzoxmzvqxzUlKrS0CrPyaofxeJqcTuzmHvRWeyD+
zGcM/OeSisnvh7Hc6+x6aaF+vE9vuneuYRScXY+W/TAYxkTFZ2NGt8moPhF2
THJlXva+ebr/PpOMe7ndmkUyhnHNgFGw2vIwmMYd9znzg4zjnvG7bH+PoG1I
g+gDChmAdfxPnx0ZBc4Op9gcH8N3gmEV2eMD4G54a3LzOjLu0lrKr2gdx+DO
UxOcCSS4lplyhEt6CF2PN6uaO9BxmewyqxTWDFyFlBox0iDqrZOPKuVjYPxX
B5ExRi00CTuqfzQcxDO75ptMvRlI2ySqHe9aDrGfFhne1AG0t3mrschkoNqu
5b1JnAWwMSO8ZdpyAAP/nf6qcXACix9vZMVkpEBBOwdn+59+XDQN8hQxncCC
K6d1m7heAeneobB9b/qxvtxkjdXrCezhmS5Q8/+K/yz8Ytdw96OHg6HmOZsJ
XKi+bJQunYszkKxiENCHv5Tr/mvSn0BZmmq7w7EybNCytD2wuQ+dJwt490hO
4MnoOE/fvhq86j/j4BXZi79Ft79NamWgoU7276sHm/HlPVeT+9K9mHixU/f7
AwbaXp5hD3hGQgbdQ6cxoQe98j4/SaDT8Yt1nvfu2A684+N7vUC+B+/96N0u
oUdHdddXyual3QiBe9OUM7vx86604/ODNOTtLpk2rhhEM8/Cjta8Lqza4v0J
OWlIijBb4NUlY+Qk9c/C8S5cdu6rbBCiYvF07WqNrGF8d3Pt+pziTky7acJx
6B8FN1ev/1CsM4q3P3oW/S7vwJLzj2csO0bwoo9Ry5zTGMJVRXyp2YGDdKPQ
lIhhlEYh0ZFYKq7X07IOr2rHx1M39MkLg1jQzxltlDGOG665kgqq27BavG+X
sUI/stVuHrzziY4nNlYxS7TbME6MfqFepwd1l7OYM1YMXP4S9OlhNQnfuE2K
z6m3YfdIWfXT/gnkY4Qn5Fe2YoV8UmysVAvmhI/svfqUiavC3rqanGrF0Ya9
0VtpdfjivH1vyS8mXpA5y8wvW/mfz176tb+iEJX6dlszv7GwLTnRyhGbUX/1
tZG6pjQ0tUhquURmodZlWylnaMa6J8S2G4d90fda073jqwj8V+hr9dqhCSP0
HyjEh8TAYqbJ75gNBCZmyKZa2DdiglvO4VDDbDigKR7xdmW/eOaWMz22DZhy
01+VN7MEgoy4G+dX6t8J6AmqfVWPpfVi7xLkqiHvAHNhWyILL57jGlJ9UYeb
JVRPDIY1ghyWm8kbsLD7vY7jSesV3ro8Gv7mVSsUkIr6zOaZ+FpZvbrZsgZf
bogfOhneDnss6XdEHJl4NciK//fDamzbcLRuIrkLVP+wBpJ/TeA27n05yeZV
qPGc7aJ61ADUh9LCDoQw8GncXYe5WxVIK+W8/7eIAvpf9aJvLFCxteHXbVOt
EiyzYkmP/hmFO7pzNHHqGF7l5G0sUC3G9WnfOrtlqHBf6lN0Stko+lsWK0U8
K0K263JfU86Ng/e6R5LOF0aQp2ZXi+nVPBy3OgHrtelgqUXq82AbxhG5Q89F
tX6i/XBp9GsFBvT/2ZQ6EjeEO+i/BC0OZaOePhznqpmAh5SLMBzSi+/TZWJ+
C2Wgh45dy8aXTCBZWjtdjurCyMV2FViThiceWDj95mfBkT1/1O1C2tE1O9rA
m56EHv1xNM0wFnBHX3xq79CKLyV/fJWs/oqz3NPDqdwEvFrmP7UusxHvSz/n
1YuPRXv7Hz/6zQg4rzRpEqxXjZqX1UJvrY5Cci2falAKAYtO7tn8IyVYYlMu
H1gYgv3jXGIxAwTEvStax1ufjU+l579XT3jjmtSzRaN/CCB2+Rqwucbi6ZJr
e4fzX+LypSoHnWUCsivXUqVeeIKObeHSo3IjmN/NntLKIuDB6WTihWEaRIt4
bhMqdoVzVaI+LtUrdZ6QPwUZFcJhiv/Eza+BELrqs+4rbwKOak1u891ZAewG
+gMGrHDY8mGWr/8oAVdcvrT986uDR87XDj0P/wyGPPEJYyQWSG6z/ew71QwZ
Bl5BloXxoG8WxiNQxQRr8RRn0vZOGKQl7TnBkQr6mRFav7YzYe6Q2zN5yR4Y
79Oxm+VLh3HpeNKw+QQEf1CPldvYD095gotdJTPBc+E4TbCQDi/N5lweXSXD
oVdr38So5YDpl6tkqt8YTJ3u/6B8fQy2VASFMewRGpk5BzUNVuYq0Crby5oK
nW5COiUVxRA5y5UuvpUC6U58t0850kBgMoPQqC+BrjERoaZHZPAJsts8cpsO
TVtM+O92lUHQJoH4d5xDsLkg7YKJEgOaRdjqp/vL4XPmeu4l6X5Yt1bivusM
A7gdJbbJjVTAy/g/sR4SndCUP3zungoTyOUsIaeJKnD9neBps6YNHO7waexD
JsyMiXAE/qqG7ajgYzXYDAfv0c98llvp35nJTUdW+EOiQUr3zc062Op4dv6u
GwtOpbBPW87VQvbWTV8fUssBSy5P7a1hQd2BgrPKC3XA3rmZO7mxAPYMpZ0s
mWFB9qxDpfPfeuCztzbgdPoOfrcKP86sI0DzUwKf7r8GMD7PL9159y3oTWrH
pa7k4qTvfZ+XG0HDjVvrZXkcFgpG7SBPsyBnPPjhQbZmsLknPUuy/Ykmy1Ov
00tZ8PSRwS1Fr2ZY4xBozVIpQyec45C0YUHFK6vRPsEWkAnPr5uWrsVFKTVN
M14WRP1M0lSIbgHdE6MxZq+aUcdLgVHty4QsxaBqWdlWeJ9lKZ3YRMLMO70l
SdMTcIzPOrMmsxXq0sRqVfg7UeSHZJH7yQno1jK+xH2cBK2FNAWji/3YHpfA
8yaQDvlxol2m59tALihxgk9qCMmx6k2rQ8eh+1PILduONtj6t8nkvC8ZV/3z
4j3tTgPHnGDSzlvtoK7w5cMLBQqWmH8SJu0Yg9yykZ6jDzsgp1zAkrbC1+5L
KW6p71a4fneVwPKbLqC+EC36eXMciWTFZwXT/XD2hG6O89puGP+VZJkiRkcP
6Zbg5KIeuMb6K+Pm1Q3el+vFpiroSBuQsVB82gkTktuquHh74MP9oxnVegzM
OD8u/Zy/Dd4dDl/FHdgDUtG3fT0rGUjJkBpMj2mG7RvGNF0FesFnUOdDgtgE
uoj8OeumUwsJa8WXn4f0wqZl86PpK5zw66D8lwMny+DEqg0RfcJ9sNbG1ljr
xQRalkWvG7n8E8iqEPQjtA+YZRkGcq8msJk3RiDneBxoHOO8+W9zP/y9eTxZ
9dYE/jYy9PPV8Mfwl54PskL7oTRsVZ3til8aSW9fa7w3A9ec6D/RLjwAVm+4
OmZbGLguB8ZX8xejq/bfq7dCBmBhck6KZsTA7yUP2D5OV+LcvRGlcwKDkGmm
WR5HouPA5Wh7p+wGVOALNYwKGASuEbKEpdk4CrC2m5wzbMdwF/vu/aeGgL/v
YmKFOw3/2KsvCb3owpibiXa0N0Nw0EHE2NqPiuYcpm0sq17cnD0zcLZ4CJDP
v77yyih2+EhWPXUYQna6FY/AUTK8zSp3ThGioKQOs2J3GRldnEKsbGzIoF5q
F/+0dBivceqZ5k8OY/qehhqnDDJEK2mHdBcPopS/4nm7NaNoUHsy/N3uYRC4
X/PrW1IfOs37nBgfH0X6Kr08F5Nh6FgQ7lt82Y03ZXMi09LH8EXaLjHJqGFw
dv+j6r23A3kfZEtlGFBRqHD/R+PuYfjumXnrVG0rgrHFC8Y4FZdcszdobBoB
399XlZyTG1Eq/Y3pa30auoYP6FXojMCW662DLglV6Jm2+m/kVxqmumVrj74d
gahimw3mQcW4xnn3p8IOGko7KjfEFI2AaPSxIL+JDGSGUueMRmnoza9e9nd2
BAwSbTrKygJR9ZXtvAqJhvx1BRum5Cmgev37+F9mDPAsER57omkoVGUb7GxG
AX4DzQzKixxwufBC67AODZu1bkBiBAUy3v13aEtUKTRfl1sd2knFfuUjsw/a
VrwqwYMqdrMG9g/E7/muRsW4j5xfSrlH4es+W7PdY00Qzvlshs93DM8KFh1L
Ux2FU60BqVLbO+A2T4cIWw0Fv1FMydpJo1DVa/d0E1s39DUvdCTmjeC6hJ72
ycFR8H65VvNuZi9YmxjVDvkPY9r0RYe9AmMwaTS0M1FtCNgfNxxKHR5Etqr1
zbq2Y5DlNLe24hwZEvWo2gH/+pAIOmp1LHUM3mWVXEk5OgxLainaOlPdKJMD
2ZlDY9Dw3/ENExkrffKyfyBvTcLdQ2pKWhpUEAwlEpUUKTBD8RTz1mrGSwev
mok9pYLV8USFyfcUkBOaVhPcVIu5Estx3lFUsK/dWn5qpR+fXBe1ftmXoc37
gl1B9VQo4qnY5/KbAtxZhIdhZi6Sltr2KP6hwu4zjWtkV3y8ok/Usbn0G3qn
Xqeby9DAPb4xhbni9exah/1yAx/CHgXZGKVLNHjSQz6Z40aBU0+ofvp8yXA3
Yi4w2I4Gzuh15Zs4BQSWBJreTOdBIFPrn+dXGugIGrwWCx4BnruPTxVWl8Gm
I692CrTSYN2nAJ80+jDkMWxu3KfUgN9MuZ7CXxoM/jyv0nGQDAb07Jw/ia2g
+ZFJEr8wDpk7NQIG9w3Bya11Mnln2uE1KyafeDEO83nqgswtA6AkXFJWn9cJ
kRW7Gy9Gj0OjSj73idkuKH3geV9trhf8BozSiolxEMiTe+Oycu93dpraz+f2
Q1TrSalGITqUyumfk9lJgvsL51tLjwxCTeLNtfeP0yFo/bnXCyueHpWfd1v/
yBCoPJDN9L9LB8Ub/ke+ZdTAW64mrri0IfjHIe2o4UEHoUEl9wmZMhDfFyl8
hp0Mccr5FS6pdLjrntF2bHUu2JT9C9oqu8IP5sYj50h0+My/7qfZ9a9Q/r3h
0aGV3Nja92DMHB2GtZ3fz2a+Qtc6zgaRlTr1D3O3OYkwwMyjtIPbLAXz2ByP
xaYOgc91U3bqCQZ4cd9ROPIqHznNNphvVByCIoa4ascdBoQfMIuaHy1DzuXk
yh3ig8C9y0ntnCsD9iQXHPYsr8Fd/Js+Rbr3g9Z/ilZnvzCgfE4l/qZWE753
e2pYFN8Li5umDzZXMYD3/E2/IflW9JNQSc+064aVgWbrpTHguh/rcOqbDuQQ
wXW1D9qAck3Z0Gn3BFj4Xou/cawLXzR3Sqv6tYDtaf22LacnYOTYWIByTDfe
C678eXdDA3BkfRE+tMJbV+7J3HiQ2oPfinXbn+ZUwqXfTpGdbhPg9WzT2DqD
XmyILzNLlyuG6yIWwwtxE+BClKQf+dqLXMMtrVHrMyFBk/d2dOkEdO2yW3vK
pxftG8KXiKYQcOqFwKrBlf98u1FVO08vVmzjz3vBEYnWVX8Wn/ydAD7W4Nz6
HT24TdieQ8MwC93SzcTDNjNhw5z+4aSeLlQ105bpuVqMKZKD9mpKTODZuZFb
Rr4TM+LLxL3FK7EjbiLa4gITDkX2FlTKtOPW6hSfpxZ1eLjr7EaBB0x4uUs1
rqy6FY3bv96Z5WtGdkfWVRVXJpQ0V23UkGjG1pOucxXCrWjguMROjmBC9W6b
0tbHtbhk6bfUkU/CpN6e6NU5K5wisCvL92A5poYKPuKfbkPBz1IV8U1MqDVa
OKVukY+b6h33TOe04776dI5yKhPeVukfTFFORlGhOBKVrQMpUpHErWUmiM17
1c1/MMfMvhDXr7XtaM+tJm0jxIKP4bGi6wO/weqI3NPhm9qxSOOICtcKD4bW
jazuLvgJmmYq3Pv6SSiec7dFWJ0FrLA68w/bS6GxjaaosLcVN2yhvv2sz4IX
MhIdhhpVUJLb4r6OqxlVIhUW4h+xQHx+u9WryDqYrGuX4WOrw67g2RbpNyz4
kT+k+Jy7CZ5yjPxqVKvAvuUjmWIhLCBVLs3HZzbDwPDPMy6cRSi7WVvTP4EF
6pb/tQidawHDfcVuC4NpmNSZr2xXwIK9LAF3Be0W2LXcOpy52RvVLRzlyY0s
IEwe7w+LboZ1UcXizfKxkBzdMFg0xIJJ2fjyru+N8IXLXPAK8wcInFZcFJhi
wX+trwOVmbWQqmwWqGpXDHEB/kojHAQAfL+utaUS9v/ZJrdvawXkX9qqtnvT
Sj5DTEqYFEMPn8xjm8BqmD1/s29MkoCcxE3iqRyZ8J2s+iyirhaYi+yNNxUJ
CJE/0W6qEQjxbLwfVI7WAemMTcs+dQIc99+ANKVYrAo/xEcpr4ErqgLBxroE
SFZ62I09W/HFfowWflYJjz+8qP1tRMCaj32+Lc+KcTzS7Zq4dQkUXbHfMfKY
AJUn3R6982W4drY5U1g7GwQvnLqjYEdAWk+TdM2KBwt43Ga+V46G/aGKB3vf
EeAqvafytWk5mjA5ms+0heIn2uTdkSACmOy+G5e4S7A7TnQIlzIwjWGZrB1N
gL5oeY7PsZwVDrzeYeJUgClO423cSQRcd5JseKYQi8UbPmSvDS5G6nh//O4f
BLwRie9T53wP9r43z0cwENkYWxlxRQRQHeSXsnalw5CG577nn3+idMfPZ3ZV
BOzndzy8sfEn1E2Lhh8kvuHpd6L8aU0EZFIeVhs9yQWpF3qLpFJ7uH95/iN0
EvDRghG2/moGOBfvvB3gnwy5/Tp02RVfzc14Wbo7MBiUdwqrXdDMgrqh+91W
FAIoXNqRR/OicTostmtTURrEMFjaAnQCtDnvLL12/Y7fDB7HKA8EQubqehF+
ggCf1F09Lz9/QQ39x7w/t3zGelW1wxa/CbgffF5ffcgJkhPeJlY/+4p9WhT3
bXMEzHwjvu9Vi4Vnr45zB9fewbjqkDn5BQLEwp5mJ925jfWzpwUKnkSARq//
i7C/BHQkPblSMeSFTtkLUxe7fDBO12Dx1j8CLoU+lNadvA0mNYZiiZfcIO9v
+yvHFa92tRXsv2Z6GQxHZUTuZBtiaoscc2ElH6t7o1w3oIL00jfdU8/JJxQV
njQtr+T/A6vv+AE=
      "]], Line3DBox[CompressedData["
1:eJwNl3c4FXwUx42mVRlRhIpKRoMio5MklRZFgySREiohoZJR9igpSkS2hGTr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      "]], Line3DBox[CompressedData["
1:eJwVl3k4FIwTx+WOUOkiJclVqTeRCJPKlSIUUb1IrlwpKiJFenW4cxTKFYpC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      "]]}}, GraphicsComplex3DBox[CompressedData["
1:eJy9vXVcVssT+I917e5uxO4EGexuxUKvioV9bezu1msHNna3sgjYiSAhiiIC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    "], {{
      {Opacity[0.3], EdgeForm[None], 
       GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJw1nHfgT9Ubxz93fe75NEgpFEkUomGUCglZKSvS/BXaEw0NlBbaexPaQ3tv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          "]], Polygon3DBox[CompressedData["
1:eJwtnAX4FUUbxTfmzu5VQFpQQRAQFBQLEUVCVEIFRBFsKVtRUBARBAMMbLG7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          "]], Polygon3DBox[CompressedData["
1:eJwt13nY1lMaB/Cn93ne530rYi655ppJmyUSwqiRmRajTVrfRGkjlShtWkTS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          "]]}]}, {}, {}, {}, {}}, {{}, {}, {}, {
       Line3DBox[{757, 1006, 1313, 1007, 2196, 1380, 1363, 1361, 1443, 1362, 
        2199, 770, 918, 2208, 784, 1023, 1203, 1024, 798, 2252, 1043, 1216, 
        1044, 812, 2255, 1063, 1229, 1064, 1323, 2345, 1083, 1242, 1084, 1330,
         2350, 1103, 1255, 1104, 2218, 1337, 1123, 1268, 1124, 2227, 843, 
        1143, 1281, 1144, 2236, 856, 1163, 1294, 1164, 2240, 870, 2273, 980, 
        884, 2279, 1395, 1396, 1397, 1398, 2374, 1182, 1347, 1183, 1304}], 
       Line3DBox[{758, 1008, 1314, 1009, 1381, 2369, 1366, 1364, 1445, 1365, 
        2200, 771, 919, 2209, 785, 1025, 1204, 2316, 1026, 799, 1045, 2292, 
        1217, 1046, 813, 2256, 1065, 1230, 1066, 1324, 2346, 1085, 1243, 1086,
         1331, 2351, 1105, 1256, 1106, 2219, 1338, 1125, 1269, 1126, 2228, 
        844, 1145, 1282, 1146, 857, 1165, 1295, 2339, 1166, 871, 981, 2242, 
        885, 2280, 1399, 1400, 1401, 1402, 1184, 1348, 1185, 1305}], 
       Line3DBox[{759, 1010, 1315, 641, 2370, 1369, 1367, 1446, 1368, 2201, 
        772, 920, 2210, 786, 1027, 1205, 2317, 1028, 800, 1047, 2293, 1218, 
        1048, 814, 1067, 2298, 1231, 1068, 1325, 2347, 1087, 1244, 1088, 1332,
         2352, 1107, 1257, 1108, 2220, 1339, 1127, 1270, 1128, 2229, 845, 
        1147, 1283, 2335, 1148, 858, 1167, 1296, 2340, 1168, 872, 982, 2243, 
        886, 1403, 1404, 1405, 2375, 679, 1349, 1186, 1306}], 
       Line3DBox[{760, 1011, 1316, 1383, 1382, 1372, 1370, 1447, 2385, 1371, 
        773, 921, 2211, 787, 1029, 1206, 2318, 1030, 801, 1049, 2294, 1219, 
        1050, 815, 1069, 2299, 1232, 1070, 1326, 1089, 2302, 1245, 1090, 1333,
         2353, 1109, 1258, 1110, 2221, 1340, 1129, 1271, 2333, 1130, 846, 
        1149, 1284, 2336, 1150, 859, 1169, 1297, 2341, 1170, 873, 983, 2244, 
        887, 1406, 2376, 1407, 1408, 1409, 2377, 1410, 1350, 1187, 1307}], 
       Line3DBox[{8, 508, 1879, 23, 738, 2100, 38, 1466, 53, 390, 1814, 68, 
        1482, 403, 83, 1496, 416, 98, 1510, 429, 113, 1525, 442, 1827, 128, 
        455, 1840, 143, 468, 1853, 158, 481, 1866, 173, 1544, 188, 1559, 203, 
        1573, 563, 218}], 
       Line3DBox[{380, 237, 1965, 630, 629, 2204, 777, 925, 2213, 791, 252, 
        2217, 805, 2253, 266, 819, 2258, 280, 525, 1896, 294, 539, 1910, 308, 
        1924, 553, 322, 2232, 850, 336, 2238, 863, 350, 2241, 877, 2277, 987, 
        891, 2282, 697, 698, 2060, 365, 498}], 
       Line3DBox[{764, 1015, 1320, 1016, 1384, 1376, 1374, 1454, 1375, 2205, 
        778, 926, 2214, 792, 1034, 1210, 2319, 1035, 806, 1054, 2295, 1223, 
        1055, 820, 2259, 1074, 1236, 1075, 1327, 2348, 1094, 1249, 1095, 1334,
         2354, 1114, 1262, 1115, 2222, 1341, 1134, 1275, 1135, 2233, 851, 
        1154, 1288, 1155, 2239, 864, 1174, 1301, 2342, 1175, 878, 988, 2245, 
        892, 2283, 1417, 1418, 1419, 1420, 2380, 1189, 1354, 1190, 1308}], 
       Line3DBox[{765, 1017, 1321, 648, 2371, 1379, 1377, 1455, 1378, 2206, 
        779, 927, 2215, 793, 1036, 1211, 2320, 1037, 807, 1056, 2296, 1224, 
        1057, 821, 1076, 2300, 1237, 1077, 1328, 2349, 1096, 1250, 1097, 1335,
         2355, 1116, 1263, 1117, 2223, 1342, 1136, 1276, 1137, 2234, 852, 
        1156, 1289, 2337, 1157, 865, 1176, 1302, 2343, 1177, 879, 989, 2246, 
        893, 1421, 1422, 1423, 2381, 707, 1355, 1191, 1309}], 
       Line3DBox[{766, 1018, 1322, 1019, 1387, 1385, 1456, 2386, 1386, 780, 
        928, 2216, 794, 1038, 1212, 2321, 1039, 808, 1058, 2297, 1225, 1059, 
        1357, 1078, 2301, 1238, 1079, 1329, 1098, 2303, 1251, 1099, 1336, 
        2356, 1118, 1264, 1119, 2224, 1343, 1138, 1277, 2334, 1139, 1358, 
        1158, 1290, 2338, 1159, 866, 1178, 1303, 2344, 1179, 880, 990, 2247, 
        894, 1424, 2382, 1425, 1426, 1427, 2383, 1428, 1356, 1192, 1193}], 
       Line3DBox[{901, 997, 1344, 1390, 2363, 1389, 895, 991, 2372, 1388, 881,
         977, 2270, 867, 971, 2310, 1291, 1160, 853, 965, 2306, 1278, 1140, 
        840, 959, 2304, 1265, 1120, 834, 2357, 953, 1252, 1100, 2264, 828, 
        947, 1239, 2331, 1080, 822, 941, 1226, 2327, 1060, 809, 935, 1213, 
        2322, 1040, 795, 929, 2286, 1200, 1020, 781, 915, 2248, 767, 909, 
        2365, 1437, 1439, 1438, 754, 907, 1429, 1310, 1003, 1194}], 
       Line3DBox[{902, 998, 1345, 1180, 1392, 2373, 896, 992, 1391, 882, 978, 
        2271, 868, 972, 2311, 1292, 1161, 854, 966, 2307, 1279, 1141, 841, 
        2225, 960, 1266, 1121, 835, 2358, 954, 1253, 1101, 2265, 829, 948, 
        1240, 1081, 2260, 823, 942, 1227, 2328, 1061, 810, 936, 1214, 2323, 
        1041, 796, 930, 2287, 1201, 1021, 782, 916, 2249, 768, 2197, 910, 
        1440, 1359, 1441, 755, 2366, 1431, 1430, 1311, 1004, 1195}], 
       Line3DBox[{903, 999, 1346, 1181, 2284, 1394, 897, 993, 1393, 2278, 883,
         979, 2272, 869, 973, 2312, 1293, 1162, 855, 2235, 967, 1280, 1142, 
        842, 2226, 961, 1267, 1122, 836, 2359, 955, 1254, 1102, 2266, 830, 
        949, 1241, 1082, 2261, 824, 943, 1228, 1062, 2254, 811, 937, 1215, 
        1042, 797, 931, 2288, 1202, 1022, 783, 2207, 917, 769, 2198, 911, 
        1442, 1360, 1444, 756, 1433, 2384, 1432, 1312, 1005, 1196}], 
       Line3DBox[{904, 1000, 1351, 1413, 2364, 1412, 898, 994, 2378, 1411, 
        888, 984, 2274, 874, 974, 2313, 1298, 1171, 860, 968, 2308, 1285, 
        1151, 847, 962, 2305, 1272, 1131, 837, 2360, 956, 1259, 1111, 2267, 
        831, 950, 1246, 2332, 1091, 825, 944, 1233, 2329, 1071, 816, 938, 
        1220, 2324, 1051, 802, 932, 2289, 1207, 1031, 788, 922, 2250, 774, 
        912, 2367, 1448, 1450, 1449, 761, 908, 1434, 1317, 1012, 1197}], 
       Line3DBox[{905, 1001, 1352, 692, 2379, 899, 995, 1414, 889, 985, 2275, 
        875, 975, 2314, 1299, 1172, 861, 969, 2309, 1286, 1152, 848, 2230, 
        963, 1273, 1132, 838, 2361, 957, 1260, 1112, 2268, 832, 951, 1247, 
        1092, 2262, 826, 945, 1234, 2330, 1072, 817, 939, 1221, 2325, 1052, 
        803, 933, 2290, 1208, 1032, 789, 923, 2251, 775, 2202, 913, 1451, 743,
         762, 2368, 720, 1318, 1013, 1198}], 
       Line3DBox[{906, 1002, 1353, 1188, 2285, 1416, 900, 996, 1415, 2281, 
        890, 986, 2276, 876, 976, 2315, 1300, 1173, 862, 2237, 970, 1287, 
        1153, 849, 2231, 964, 1274, 1133, 839, 2362, 958, 1261, 1113, 2269, 
        833, 952, 1248, 1093, 2263, 827, 946, 1235, 1073, 2257, 818, 940, 
        1222, 2326, 1053, 804, 934, 2291, 1209, 1033, 790, 2212, 924, 776, 
        2203, 914, 1452, 1373, 1453, 763, 1436, 1435, 1319, 1014, 1199}]}, {
       Line3DBox[CompressedData["
1:eJwNzskuA2AUxfErYnqMmqmap5UYXoCa2mJjIVY8R+mOZSOGiDQNUvNcYzyU
jd/in3PuOd93cxPr2+mtmojIoVIXka+PKDZENDdG7NZGzPEVXOEAe/KJpoiU
d2nzNW5wiFl5i75XN2++xR32ZUe0lfbpFvh7THlfkB3zbbRft8g/YFr3SE/Q
rhvQLfEbbnyip+iQD8qX+We84Ayd8oLbM/wr3lBCl3zS3iF/suYq3lFG0d5h
eY7/wCfO0e3PiHyF/8Kmdxc0KR+Vr/LfmLH3h16iRzemW+P/3PFLU7Jx2Q7+
AT68Li8=
        "]], 
       Line3DBox[{1458, 1947, 2108, 2365, 2094, 1949, 2197, 1593, 2095, 1951, 
        2198, 1594, 2096, 2183, 2199, 1595, 2097, 2184, 2200, 1596, 2098, 
        2185, 2201, 1597, 2099, 2385, 1956, 1598, 2100, 1958, 2109, 2367, 
        2101, 1960, 2202, 1599, 2102, 1962, 2203, 1600, 2103, 1964, 2204, 
        1601, 2104, 2186, 2205, 1602, 2105, 2187, 2206, 1603, 2106, 2386, 
        1990, 1604, 2107}], 
       Line3DBox[{1460, 2110, 2248, 1459, 2111, 2249, 1461, 2207, 1605, 1462, 
        2208, 1606, 1463, 2209, 1607, 1464, 2210, 1608, 1465, 2211, 1609, 
        1466, 2112, 2250, 1467, 2113, 2251, 1468, 2212, 1610, 1469, 2213, 
        1611, 1470, 2214, 1612, 1471, 2215, 1613, 1472, 2216, 1614, 1473}], 
       Line3DBox[{1474, 1675, 2114, 2286, 1808, 1676, 2115, 2287, 1809, 1677, 
        2116, 2288, 1810, 1024, 1615, 1811, 2316, 1678, 1616, 1812, 2317, 
        1679, 1617, 1813, 2318, 1680, 1618, 1814, 1681, 2117, 2289, 1815, 
        1682, 2118, 2290, 1816, 1683, 2119, 2291, 1817, 1684, 2217, 1619, 
        1818, 2319, 1685, 1620, 1819, 2320, 1686, 1621, 1820, 2321, 1687, 
        1622, 1475}], 
       Line3DBox[{1489, 1713, 1712, 2297, 1488, 1711, 1710, 2296, 1487, 1709, 
        1708, 2295, 1486, 1707, 2253, 1706, 1485, 2326, 1705, 1704, 1484, 
        2325, 1703, 1702, 1483, 2324, 1701, 1700, 1482, 1699, 1698, 2294, 
        1481, 1697, 1696, 2293, 1480, 1695, 1694, 2292, 1479, 1693, 2252, 
        2140, 1478, 1042, 1692, 1477, 2323, 1691, 1690, 1476, 2322, 1689, 
        1688, 1933}], 
       Line3DBox[{1503, 1936, 1736, 1928, 1735, 2301, 1502, 1734, 1733, 2300, 
        1501, 1732, 2259, 2145, 1500, 1731, 2258, 1730, 1499, 2144, 2257, 
        1729, 1498, 2330, 1728, 1727, 1497, 2329, 1726, 1725, 1496, 1724, 
        1723, 2299, 1495, 1722, 1721, 2298, 1494, 1720, 2256, 2143, 1493, 
        1719, 2255, 2142, 1492, 2141, 2254, 1718, 1491, 2328, 1717, 1716, 
        1490, 2327, 1715, 1934, 1714, 1992, 1935}], 
       Line3DBox[{1517, 1938, 1939, 1899, 1753, 2303, 1516, 1752, 1898, 2349, 
        2154, 1515, 1751, 1897, 2348, 2153, 1514, 1750, 1896, 1749, 1513, 
        2152, 2263, 1895, 1748, 1512, 2151, 2262, 1894, 1747, 1511, 2332, 
        1746, 1893, 1745, 1510, 1744, 1892, 1743, 2302, 1509, 1742, 1891, 
        2347, 2150, 1508, 1741, 1890, 2346, 2149, 1507, 1740, 1889, 2345, 
        2148, 1506, 2147, 2261, 1888, 1739, 1505, 2146, 2260, 1887, 1738, 
        1504, 2331, 1737, 1886, 1994, 1993, 1937}], 
       Line3DBox[{1519, 1941, 1942, 1914, 2357, 1623, 1821, 1755, 1915, 2358, 
        1624, 1822, 1757, 1916, 2359, 1625, 1823, 2159, 2218, 1917, 1626, 
        1824, 2161, 2219, 1918, 1627, 1825, 2163, 2220, 1919, 1628, 1826, 
        2165, 2221, 1920, 1629, 1827, 1763, 1921, 2360, 1630, 1828, 1765, 
        1922, 2361, 1631, 1829, 1767, 1923, 2362, 1632, 1830, 1770, 1924, 
        1633, 1831, 2170, 2222, 1925, 1634, 1832, 2172, 2223, 1926, 1635, 
        1833, 2174, 2224, 1927, 1930, 1636, 1929}], 
       Line3DBox[{1532, 1943, 1944, 1913, 2356, 2173, 1531, 1772, 1912, 2355, 
        2171, 1530, 1771, 1911, 2354, 2169, 1529, 1769, 1910, 1768, 1528, 
        2168, 2269, 1909, 1766, 1527, 2167, 2268, 1908, 1764, 1526, 2166, 
        2267, 1907, 1762, 1525, 1761, 1906, 2353, 2164, 1524, 1760, 1905, 
        2352, 2162, 1523, 1759, 1904, 2351, 2160, 1522, 1758, 1903, 2350, 
        2158, 1521, 2157, 2266, 1902, 1756, 1520, 2156, 2265, 1901, 1754, 
        1518, 2155, 2264, 1900, 1996, 1995, 1940}], 
       Line3DBox[{1533, 1997, 1998, 1945, 2120, 2304, 1834, 1773, 2225, 1637, 
        1835, 1774, 2226, 1638, 1836, 2175, 2227, 1639, 1837, 2176, 2228, 
        1640, 1838, 2177, 2229, 1641, 1839, 2333, 1775, 1642, 1840, 1776, 
        2121, 2305, 1841, 1777, 2230, 1643, 1842, 1778, 2231, 1644, 1843, 
        1779, 2232, 1645, 1844, 2178, 2233, 1646, 1845, 2179, 2234, 1647, 
        1846, 2334, 1780, 1932, 1968, 1648, 1931}], 
       Line3DBox[{1534, 1781, 2122, 2306, 1847, 1782, 2123, 2307, 1848, 1783, 
        2235, 1649, 1849, 2180, 2236, 1650, 1850, 1146, 1651, 1851, 2335, 
        1784, 1652, 1852, 2336, 1785, 1653, 1853, 1786, 2124, 2308, 1854, 
        1787, 2125, 2309, 1855, 1788, 2237, 1654, 1856, 1789, 2238, 1655, 
        1857, 2181, 2239, 1656, 1858, 2337, 1790, 1657, 1859, 2338, 1791, 
        1658, 1946}], 
       Line3DBox[{1535, 1792, 2126, 2310, 1860, 1793, 2127, 2311, 1861, 1794, 
        2128, 2312, 1862, 2182, 2240, 1659, 1863, 2339, 1795, 1660, 1864, 
        2340, 1796, 1661, 1865, 2341, 1797, 1662, 1866, 1798, 2129, 2313, 
        1867, 1799, 2130, 2314, 1868, 1800, 2131, 2315, 1869, 1801, 2241, 
        1663, 1870, 2342, 1802, 1664, 1871, 2343, 1803, 1665, 1872, 2344, 
        1804, 1666, 1536}], 
       Line3DBox[{1538, 2132, 2270, 1537, 2133, 2271, 1539, 2134, 2272, 1540, 
        2135, 2273, 1541, 2242, 1667, 1542, 2243, 1668, 1543, 2244, 1669, 
        1544, 2136, 2274, 1545, 2137, 2275, 1546, 2138, 2276, 1547, 2139, 
        2277, 1548, 2245, 1670, 1549, 2246, 1671, 1550, 2247, 1672, 1551}], 
       Line3DBox[{1553, 1999, 2000, 2372, 1552, 2004, 1391, 1554, 2009, 2278, 
        2190, 1555, 2191, 2279, 2015, 1556, 2192, 2280, 2021, 1557, 1403, 
        2026, 1558, 2376, 2032, 2033, 1559, 2039, 2040, 2378, 1560, 2046, 
        1414, 1561, 2052, 2281, 2193, 1562, 2058, 2282, 2059, 1563, 2194, 
        2283, 2065, 1564, 1421, 2071, 1565, 2382, 2077, 2078, 1566}], 
       Line3DBox[CompressedData["
1:eJwV0FdKQ1EUBdATJQ1MFBs2MM/eZmB+o4iK4AQEB6D/zsHBqZiIDRsWTBRs
YENXPhb7svflcXnJ5vb6VioiNpjLRLRnI4ZzEQkl1vIRs/pB5yH6GGBV3y9n
bD2yl05W9N2yi+nm92QHbRQpMKXPyTwN0mTJMGlrkcu+0yr/vCclgwlbWf+j
++WTb74Yt73LD15445Ul98dsT87PPNJgUV+Xo7Z7+cAtO+mIOzmiv5Y3XLDg
/pW8JLGdyXPq3nUsTzmhZDuSFfd3bYfONarN/2qb1+87H7BHgSL/KPUvzg==

        "]]}}},
    VertexNormals->CompressedData["
1:eJx0vHk41d33Pi4kKkohQ1EpQ0qlQsl5VSQkRFQSUYZCipBUFBkioRQZE2mg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     "]]},
  AutomaticImageSize->True,
  Axes->True,
  ImageSize->{329.8505931556609, 368.61631384235403`},
  Method->{},
  PlotRange->All,
  PlotRangePadding->{Automatic, Automatic, Automatic},
  ViewPoint->{0.8601287426063323, -2.8032946400863987`, 1.688702965889892},
  ViewVertical->{0.0014283890503982081`, -0.11457632998547021`, 
   0.9934134206420715}]], "Output",
 CellChangeTimes->{{3.626696934167096*^9, 3.6266969612185435`*^9}, 
   3.626697001657815*^9, 3.6266970401616826`*^9, {3.626697125612033*^9, 
   3.626697154113283*^9}, {3.626700392929038*^9, 3.6267004025874557`*^9}, 
   3.6267004508755407`*^9, 3.6267797508526974`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"lox", "[", "s_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"sphero", "[", "1", "]"}], "[", 
   RowBox[{
    RowBox[{"hu", "[", "s", "]"}], ",", 
    RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6266973167127695`*^9, 3.626697397514312*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{"lox", "[", "t", "]"}], ",", "t"}], "]"}], "/.", " ", 
  RowBox[{"t", "\[Rule]", " ", "s"}]}]], "Input",
 CellChangeTimes->{{3.626697404660125*^9, 3.6266974325061736`*^9}, {
   3.626700514781253*^9, 3.626700553350521*^9}, 3.626779821754822*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{"Cos", "[", 
       RowBox[{"s", " ", 
        RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], " ", 
     RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
     RowBox[{"Sin", "[", 
      RowBox[{
       RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"Log", "[", 
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
            RowBox[{"Sin", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], "+", 
         RowBox[{"Log", "[", 
          RowBox[{
           RowBox[{"Cos", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
           RowBox[{"Sin", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], ")"}]}],
       "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{
          RowBox[{
           RowBox[{"-", 
            FractionBox["1", "2"]}], " ", 
           RowBox[{"Cos", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
          RowBox[{
           FractionBox["1", "2"], " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
           RowBox[{"Sin", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], 
         RowBox[{
          RowBox[{"Cos", "[", 
           RowBox[{
            FractionBox["1", "2"], " ", "s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
          RowBox[{"Sin", "[", 
           RowBox[{
            FractionBox["1", "2"], " ", "s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]]}], "+", 
       FractionBox[
        RowBox[{
         RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{
            FractionBox["1", "2"], " ", "s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
          RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
         RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{
            FractionBox["1", "2"], " ", "s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], 
        RowBox[{
         RowBox[{"Cos", "[", 
          RowBox[{
           FractionBox["1", "2"], " ", "s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
         RowBox[{"Sin", "[", 
          RowBox[{
           FractionBox["1", "2"], " ", "s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]]}], ")"}]}], "-", 
    RowBox[{
     RowBox[{"Cos", "[", 
      RowBox[{
       RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"Log", "[", 
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
            RowBox[{"Sin", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], "+", 
         RowBox[{"Log", "[", 
          RowBox[{
           RowBox[{"Cos", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
           RowBox[{"Sin", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], ")"}]}],
       "]"}], " ", 
     RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
     RowBox[{"Sin", "[", 
      RowBox[{"s", " ", 
       RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"Cos", "[", 
      RowBox[{
       RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"Log", "[", 
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
            RowBox[{"Sin", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], "+", 
         RowBox[{"Log", "[", 
          RowBox[{
           RowBox[{"Cos", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
           RowBox[{"Sin", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], ")"}]}],
       "]"}], " ", 
     RowBox[{"Cos", "[", 
      RowBox[{"s", " ", 
       RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
     RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{
          RowBox[{
           RowBox[{"-", 
            FractionBox["1", "2"]}], " ", 
           RowBox[{"Cos", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
          RowBox[{
           FractionBox["1", "2"], " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
           RowBox[{"Sin", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], 
         RowBox[{
          RowBox[{"Cos", "[", 
           RowBox[{
            FractionBox["1", "2"], " ", "s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
          RowBox[{"Sin", "[", 
           RowBox[{
            FractionBox["1", "2"], " ", "s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]]}], "+", 
       FractionBox[
        RowBox[{
         RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{
            FractionBox["1", "2"], " ", "s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
          RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
         RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{
            FractionBox["1", "2"], " ", "s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], 
        RowBox[{
         RowBox[{"Cos", "[", 
          RowBox[{
           FractionBox["1", "2"], " ", "s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
         RowBox[{"Sin", "[", 
          RowBox[{
           FractionBox["1", "2"], " ", "s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]]}], ")"}]}], "-", 
    RowBox[{
     RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
     RowBox[{"Sin", "[", 
      RowBox[{
       RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"Log", "[", 
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
            RowBox[{"Sin", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], "+", 
         RowBox[{"Log", "[", 
          RowBox[{
           RowBox[{"Cos", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
           RowBox[{"Sin", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], ")"}]}],
       "]"}], " ", 
     RowBox[{"Sin", "[", 
      RowBox[{"s", " ", 
       RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"Cos", "[", 
     RowBox[{"s", " ", 
      RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
    RowBox[{"Sin", "[", "\[Tau]", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.626697433878976*^9, 3.626700556486127*^9, 
  3.6267022634357367`*^9, 3.6267797929415717`*^9, 3.626938512686*^9, 
  3.6270289718134594`*^9, 3.627042967839223*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", ",", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"s", ">", "0"}], ",", " ", 
     RowBox[{"\[Tau]", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{3.6267798516600747`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Cos", "[", "\[Tau]", "]"}], " ", 
     RowBox[{"Sin", "[", 
      RowBox[{
       RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Log", "[", 
          RowBox[{
           RowBox[{"Cos", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
           RowBox[{"Sin", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}], "-", 
         RowBox[{"Log", "[", 
          RowBox[{
           RowBox[{"Cos", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
           RowBox[{"Sin", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], ")"}]}],
       "]"}]}], "-", 
    RowBox[{
     RowBox[{"Cos", "[", 
      RowBox[{
       RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"Log", "[", 
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
            RowBox[{"Sin", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], "+", 
         RowBox[{"Log", "[", 
          RowBox[{
           RowBox[{"Cos", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
           RowBox[{"Sin", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], ")"}]}],
       "]"}], " ", 
     RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
     RowBox[{"Sin", "[", 
      RowBox[{"s", " ", 
       RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"Cos", "[", "\[Tau]", "]"}], " ", 
     RowBox[{"Cos", "[", 
      RowBox[{
       RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"Log", "[", 
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
            RowBox[{"Sin", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], "+", 
         RowBox[{"Log", "[", 
          RowBox[{
           RowBox[{"Cos", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
           RowBox[{"Sin", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], ")"}]}],
       "]"}]}], "+", 
    RowBox[{
     RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
     RowBox[{"Sin", "[", 
      RowBox[{
       RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Log", "[", 
          RowBox[{
           RowBox[{"Cos", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
           RowBox[{"Sin", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}], "-", 
         RowBox[{"Log", "[", 
          RowBox[{
           RowBox[{"Cos", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
           RowBox[{"Sin", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], ")"}]}],
       "]"}], " ", 
     RowBox[{"Sin", "[", 
      RowBox[{"s", " ", 
       RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"Cos", "[", 
     RowBox[{"s", " ", 
      RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
    RowBox[{"Sin", "[", "\[Tau]", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.6267798602400894`*^9, 3.6269385278180265`*^9, 
  3.6270289826086783`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"tlox", "[", "s_", "]"}], " ", ":=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Cos", "[", "\[Tau]", "]"}], " ", 
      RowBox[{"Sin", "[", 
       RowBox[{
        RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Log", "[", 
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
            RowBox[{"Sin", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}], "-", 
          RowBox[{"Log", "[", 
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
            RowBox[{"Sin", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
         ")"}]}], "]"}]}], "-", 
     RowBox[{
      RowBox[{"Cos", "[", 
       RowBox[{
        RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"Log", "[", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
             RowBox[{"Sin", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], "+", 
          RowBox[{"Log", "[", 
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
            RowBox[{"Sin", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
         ")"}]}], "]"}], " ", 
      RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
      RowBox[{"Sin", "[", 
       RowBox[{"s", " ", 
        RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"Cos", "[", "\[Tau]", "]"}], " ", 
      RowBox[{"Cos", "[", 
       RowBox[{
        RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"Log", "[", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
             RowBox[{"Sin", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], "+", 
          RowBox[{"Log", "[", 
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
            RowBox[{"Sin", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
         ")"}]}], "]"}]}], "+", 
     RowBox[{
      RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
      RowBox[{"Sin", "[", 
       RowBox[{
        RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Log", "[", 
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
            RowBox[{"Sin", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}], "-", 
          RowBox[{"Log", "[", 
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
            RowBox[{"Sin", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
         ")"}]}], "]"}], " ", 
      RowBox[{"Sin", "[", 
       RowBox[{"s", " ", 
        RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ",", 
    RowBox[{
     RowBox[{"Cos", "[", 
      RowBox[{"s", " ", 
       RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
     RowBox[{"Sin", "[", "\[Tau]", "]"}]}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.6267799208929963`*^9, 3.6267799608758664`*^9}, {
  3.627028318640312*^9, 3.627028328187529*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"tlox", "[", "s", "]"}], ".", 
    RowBox[{"tlox", "[", "s", "]"}]}], ",", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"s", ">", "0"}], ",", " ", 
     RowBox[{"\[Tau]", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.626697449622404*^9, 3.6266975061443033`*^9}, {
  3.627028353100773*^9, 3.627028376578814*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.6266975091551085`*^9, 3.6267005680321474`*^9, 
  3.626702269069347*^9, 3.626779971936286*^9, 3.6269385346976385`*^9, 
  3.627028380634821*^9, 3.627028988474289*^9, 3.6270429785096416`*^9, 
  3.627047928429536*^9, 3.6270503957766695`*^9, 3.6270510800562716`*^9, 
  3.627051242171756*^9, 3.6271064040311537`*^9}]
}, Closed]],

Cell["s is the arc length of the curve!", "Text",
 CellChangeTimes->{{3.626697551980184*^9, 3.626697596224862*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "amf"}]], "Input",
 CellChangeTimes->{{3.6267800303271885`*^9, 3.626780032635992*^9}}],

Cell[BoxData[
 StyleBox["\<\"amf[i][f][u,v]is the i-th vector, i = 1,2,3, of the \
orthonormal moving \\nframe adapted to the parameter representation of the \
surface f[u,v] in the \\nEuclidean 3-space.\"\>", "MSG"]], "Print", \
"PrintUsage",
 CellChangeTimes->{3.626780033899595*^9},
 CellTags->"Info3626783633-5796653"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"amf", "[", "1", "]"}], "[", 
      RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
     RowBox[{
      RowBox[{"hu", "[", "s", "]"}], ",", 
      RowBox[{"hv", "[", "s", "]"}]}], "]"}], ".", 
    RowBox[{"tlox", "[", "s", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"s", ">", "0"}], ",", " ", 
     RowBox[{"\[Tau]", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6266976120306897`*^9, 3.626697719008078*^9}, {
  3.626697759382949*^9, 3.6266978111956406`*^9}, {3.6267005783301654`*^9, 
  3.6267005847105765`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cos", "[", "\[Tau]", "]"}], " ", 
  RowBox[{"Cos", "[", 
   RowBox[{"s", " ", 
    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
  SqrtBox[
   SuperscriptBox[
    RowBox[{"Sec", "[", 
     RowBox[{"s", " ", 
      RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]]}]], "Output",
 CellChangeTimes->{3.6266978136916447`*^9, 3.6266981475416327`*^9, 
  3.6266983111511207`*^9, 3.6267005880489826`*^9, 3.626700642204678*^9, 
  3.626780049281222*^9, 3.6269385638540897`*^9, 3.627042990256462*^9, 
  3.6270479373215513`*^9, 3.627051088573886*^9, 3.627051260189788*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"%", "\[Equal]", " ", 
    RowBox[{
     RowBox[{"Cos", "[", "\[Tau]", "]"}], "*", 
     RowBox[{"Sign", "[", " ", 
      RowBox[{"Cos", "[", 
       RowBox[{"s", " ", 
        RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "]"}]}]}], ",", " ", 
   RowBox[{"Element", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"s", " ", ",", "\[Tau]"}], "}"}], ",", "Reals"}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6266978752211533`*^9, 3.626697939936067*^9}, {
  3.626700618114236*^9, 3.626700636571068*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{
  3.626697941059269*^9, {3.626698306268312*^9, 3.6266983143179264`*^9}, 
   3.626700657510305*^9, 3.626780056862835*^9}]
}, Closed]],

Cell["Thus the intersection angle is constant.", "Text",
 CellChangeTimes->{{3.626700676076338*^9, 3.6267006928171673`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"curvature", "[", "lox", "]"}], "[", "s", "]"}]], "Input",
 CellChangeTimes->{{3.626938880612646*^9, 3.6269389210323167`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"\[Sqrt]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"Sin", "[", "\[Tau]", "]"}], "4"], " ", 
       SuperscriptBox[
        RowBox[{"Sin", "[", 
         RowBox[{"s", " ", 
          RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"Cos", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{
            RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"Log", "[", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}], "-", 
              RowBox[{"Log", "[", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
             ")"}]}], "]"}]}], "-", 
         RowBox[{
          RowBox[{"Cos", "[", "\[Tau]", "]"}], " ", 
          RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{
            RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              "+", 
              RowBox[{"Log", "[", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
             ")"}]}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{"-", 
                   FractionBox["1", "2"]}], " ", 
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                 RowBox[{
                  FractionBox["1", "2"], " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
               "/", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], 
              ")"}]}], "+", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                FractionBox["1", "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
             "/", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{
          RowBox[{"Cos", "[", "\[Tau]", "]"}], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{
            RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"Log", "[", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}], "-", 
              RowBox[{"Log", "[", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
             ")"}]}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", 
                 FractionBox["1", "2"]}], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
             "/", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], "-", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                FractionBox["1", "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
             "/", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}]}], 
           ")"}], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], "2"], 
      "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{"Cos", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}]}], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"]}], "+", 
         RowBox[{
          RowBox[{"Cos", "[", "\[Tau]", "]"}], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{
            RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"Log", "[", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}], "-", 
              RowBox[{"Log", "[", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
             ")"}]}], "]"}], " ", 
          RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", 
                 FractionBox["1", "2"]}], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
             "/", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], "-", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                FractionBox["1", "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
             "/", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{
          RowBox[{"Cos", "[", "\[Tau]", "]"}], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{
            RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              "+", 
              RowBox[{"Log", "[", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
             ")"}]}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{"-", 
                   FractionBox["1", "2"]}], " ", 
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                 RowBox[{
                  FractionBox["1", "2"], " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
               "/", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], 
              ")"}]}], "+", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                FractionBox["1", "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
             "/", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}]}], 
           ")"}], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
     ")"}]}], ")"}], "/", 
  RowBox[{"(", 
   RowBox[{"\[Sqrt]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"Cos", "[", 
         RowBox[{"s", " ", 
          RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
       SuperscriptBox[
        RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"]}], "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{"Cos", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], " ", 
          RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{
            RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              "+", 
              RowBox[{"Log", "[", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
             ")"}]}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{"-", 
                   FractionBox["1", "2"]}], " ", 
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                 RowBox[{
                  FractionBox["1", "2"], " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
               "/", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], 
              ")"}]}], "+", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                FractionBox["1", "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
             "/", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}]}], 
           ")"}]}], "-", 
         RowBox[{
          RowBox[{"Cos", "[", 
           RowBox[{
            RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              "+", 
              RowBox[{"Log", "[", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
             ")"}]}], "]"}], " ", 
          RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], "2"], 
      "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"Cos", "[", 
           RowBox[{
            RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              "+", 
              RowBox[{"Log", "[", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
             ")"}]}], "]"}], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
          RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{"-", 
                   FractionBox["1", "2"]}], " ", 
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                 RowBox[{
                  FractionBox["1", "2"], " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
               "/", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], 
              ")"}]}], "+", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                FractionBox["1", "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
             "/", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}]}], 
           ")"}]}], "-", 
         RowBox[{
          RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{
            RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              "+", 
              RowBox[{"Log", "[", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
             ")"}]}], "]"}], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], "2"]}], 
     ")"}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.6270288788216963`*^9, 3.627029002982314*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", ",", " ", 
   RowBox[{"Element", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"s", " ", ",", "\[Tau]"}], "}"}], ",", "Reals"}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6267007717477064`*^9, 3.6267008002685566`*^9}}],

Cell[BoxData[
 FractionBox[
  SqrtBox[
   RowBox[{"3", "+", 
    RowBox[{"Cos", "[", 
     RowBox[{"2", " ", "s", " ", 
      RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
    RowBox[{"2", " ", 
     RowBox[{"Cos", "[", 
      RowBox[{"2", " ", "\[Tau]"}], "]"}], " ", 
     SuperscriptBox[
      RowBox[{"Sin", "[", 
       RowBox[{"s", " ", 
        RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}]], 
  RowBox[{"2", " ", 
   RowBox[{"Abs", "[", 
    RowBox[{"Cos", "[", 
     RowBox[{"s", " ", 
      RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "]"}]}]]], "Output",
 CellChangeTimes->{3.6267008057129664`*^9, 3.626780084162883*^9, 
  3.6270290244011517`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"kk", "[", "s_", "]"}], " ", ":=", " ", 
  FractionBox[
   SqrtBox[
    RowBox[{"3", "+", 
     RowBox[{"Cos", "[", 
      RowBox[{"2", " ", "s", " ", 
       RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
     RowBox[{"2", " ", 
      RowBox[{"Cos", "[", 
       RowBox[{"2", " ", "\[Tau]"}], "]"}], " ", 
      SuperscriptBox[
       RowBox[{"Sin", "[", 
        RowBox[{"s", " ", 
         RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}]], 
   RowBox[{"2", " ", 
    RowBox[{"Abs", "[", 
     RowBox[{"Cos", "[", 
      RowBox[{"s", " ", 
       RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "]"}]}]]}]], "Input",
 CellChangeTimes->{{3.6267008156677837`*^9, 3.6267008245305996`*^9}, {
  3.6270290406563807`*^9, 3.627029054166004*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"normalcurvature", "[", 
   RowBox[{
    RowBox[{"sphero", "[", "1", "]"}], ",", "hu", ",", "hv"}], "]"}], "[", 
  "s", "]"}]], "Input",
 CellChangeTimes->{{3.6269387244095716`*^9, 3.6269387351735907`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{"Cos", "[", 
       RowBox[{"s", " ", 
        RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], " ", 
     SuperscriptBox[
      RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Cos", "[", 
             RowBox[{
              RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"-", 
                 RowBox[{"Log", "[", 
                  RowBox[{
                   RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                   RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                "+", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               ")"}]}], "]"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"Cos", "[", 
             RowBox[{"s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"Cos", "[", 
             RowBox[{"s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"Sin", "[", 
             RowBox[{
              RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"-", 
                 RowBox[{"Log", "[", 
                  RowBox[{
                   RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                   RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                "+", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               ")"}]}], "]"}], "2"]}], "-", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"Cos", "[", 
             RowBox[{
              RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"-", 
                 RowBox[{"Log", "[", 
                  RowBox[{
                   RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                   RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                "+", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               ")"}]}], "]"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"Sin", "[", 
             RowBox[{"s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "-", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"Sin", "[", 
             RowBox[{
              RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"-", 
                 RowBox[{"Log", "[", 
                  RowBox[{
                   RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                   RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                "+", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               ")"}]}], "]"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"Sin", "[", 
             RowBox[{"s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}], ")"}], 
        "/", 
        RowBox[{"(", 
         RowBox[{"\[Sqrt]", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{
                RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", 
                   RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                  "+", 
                  RowBox[{"Log", "[", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                 ")"}]}], "]"}], "2"], " ", 
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"], " ", 
             SuperscriptBox[
              RowBox[{"Sin", "[", 
               RowBox[{
                RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", 
                   RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                  "+", 
                  RowBox[{"Log", "[", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                 ")"}]}], "]"}], "2"]}], "+", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
             "2"]}], ")"}]}], ")"}]}], "-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{
                RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", 
                   RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                  "+", 
                  RowBox[{"Log", "[", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                 ")"}]}], "]"}], "2"], " ", 
             RowBox[{"Cos", "[", 
              RowBox[{"s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{"s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{"Sin", "[", 
               RowBox[{
                RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", 
                   RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                  "+", 
                  RowBox[{"Log", "[", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                 ")"}]}], "]"}], "2"], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "4"}], " ", 
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{
                RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", 
                   RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                  "+", 
                  RowBox[{"Log", "[", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                 ")"}]}], "]"}], "2"], " ", 
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "3"], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "-", 
            RowBox[{"4", " ", 
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "3"], " ", 
             SuperscriptBox[
              RowBox[{"Sin", "[", 
               RowBox[{
                RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", 
                   RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                  "+", 
                  RowBox[{"Log", "[", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                 ")"}]}], "]"}], "2"], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
            RowBox[{"2", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
             " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"]}], "-", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "-", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}], 
              ")"}]}]}], ")"}]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{"2", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"Cos", "[", 
                RowBox[{
                 RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                   "+", 
                   RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                  ")"}]}], "]"}], "2"], " ", 
              SuperscriptBox[
               RowBox[{"Cos", "[", 
                RowBox[{"s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"]}], "+", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"Cos", "[", 
                RowBox[{"s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"], " ", 
              SuperscriptBox[
               RowBox[{"Sin", "[", 
                RowBox[{
                 RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                   "+", 
                   RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                  ")"}]}], "]"}], "2"]}], "+", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                 RowBox[{"Sin", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                 SuperscriptBox[
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                 RowBox[{"Sin", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
              "2"]}], ")"}], 
           RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}]}], "-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"Cos", "[", 
          RowBox[{"s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], " ", 
        RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
        RowBox[{"Sin", "[", 
         RowBox[{
          RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"Log", "[", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], "+", 
            RowBox[{"Log", "[", 
             RowBox[{
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
           ")"}]}], "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           FractionBox[
            RowBox[{
             RowBox[{
              RowBox[{"-", 
               FractionBox["1", "2"]}], " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
             RowBox[{
              FractionBox["1", "2"], " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
             RowBox[{"Sin", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]]}], "+", 
          FractionBox[
           RowBox[{
            RowBox[{
             FractionBox["1", "2"], " ", 
             RowBox[{"Cos", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
            RowBox[{
             FractionBox["1", "2"], " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], 
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
            RowBox[{"Sin", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]]}], ")"}]}], "-", 
       RowBox[{
        RowBox[{"Cos", "[", 
         RowBox[{
          RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"Log", "[", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], "+", 
            RowBox[{"Log", "[", 
             RowBox[{
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
           ")"}]}], "]"}], " ", 
        RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
        RowBox[{"Sin", "[", 
         RowBox[{"s", " ", 
          RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"Cos", "[", 
             RowBox[{"s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
           RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
           RowBox[{"Sin", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"-", 
                    FractionBox["1", "2"]}], " ", 
                   RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                  RowBox[{
                   FractionBox["1", "2"], " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                "/", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], 
               ")"}]}], "+", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
              "/", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}]}], 
            ")"}]}], ")"}], "/", 
         RowBox[{"(", 
          RowBox[{"\[Sqrt]", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"Cos", "[", 
                RowBox[{
                 RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                   "+", 
                   RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                  ")"}]}], "]"}], "2"], " ", 
              SuperscriptBox[
               RowBox[{"Cos", "[", 
                RowBox[{"s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"]}], "+", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"Cos", "[", 
                RowBox[{"s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"], " ", 
              SuperscriptBox[
               RowBox[{"Sin", "[", 
                RowBox[{
                 RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                   "+", 
                   RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                  ")"}]}], "]"}], "2"]}], "+", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                 RowBox[{"Sin", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                 SuperscriptBox[
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                 RowBox[{"Sin", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
              "2"]}], ")"}]}], ")"}]}]}], "+", 
       RowBox[{
        RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"2", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", 
                   RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                  "+", 
                  RowBox[{"Log", "[", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                 ")"}]}], "]"}], " ", 
              RowBox[{"Cos", "[", 
               RowBox[{"s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{"s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}], "/", 
            RowBox[{"(", 
             RowBox[{"\[Sqrt]", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"]}], "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"], " ", 
                 SuperscriptBox[
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"]}], "+", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                 "2"]}], ")"}]}], ")"}]}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{
               RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"-", 
                  RowBox[{"Log", "[", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                 "+", 
                 RowBox[{"Log", "[", 
                  RowBox[{
                   RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                   RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                ")"}]}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "4"}], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "3"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "-", 
               RowBox[{"4", " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "3"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                   RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"]}], "-", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "-", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}], 
                 ")"}]}]}], ")"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"]}], "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"], " ", 
                 SuperscriptBox[
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"]}], "+", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                 "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}]}]}], ")"}]}], "-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", 
         RowBox[{
          RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"Log", "[", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], "+", 
            RowBox[{"Log", "[", 
             RowBox[{
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
           ")"}]}], "]"}], " ", 
        RowBox[{"Cos", "[", 
         RowBox[{"s", " ", 
          RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
        RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           FractionBox[
            RowBox[{
             RowBox[{
              RowBox[{"-", 
               FractionBox["1", "2"]}], " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
             RowBox[{
              FractionBox["1", "2"], " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
             RowBox[{"Sin", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]]}], "+", 
          FractionBox[
           RowBox[{
            RowBox[{
             FractionBox["1", "2"], " ", 
             RowBox[{"Cos", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
            RowBox[{
             FractionBox["1", "2"], " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], 
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
            RowBox[{"Sin", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]]}], ")"}]}], "-", 
       RowBox[{
        RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
        RowBox[{"Sin", "[", 
         RowBox[{
          RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"Log", "[", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], "+", 
            RowBox[{"Log", "[", 
             RowBox[{
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
           ")"}]}], "]"}], " ", 
        RowBox[{"Sin", "[", 
         RowBox[{"s", " ", 
          RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Cos", "[", 
           RowBox[{
            RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              "+", 
              RowBox[{"Log", "[", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
             ")"}]}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
          RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{"-", 
                   FractionBox["1", "2"]}], " ", 
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                 RowBox[{
                  FractionBox["1", "2"], " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
               "/", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], 
              ")"}]}], "+", 
            FractionBox[
             RowBox[{
              RowBox[{
               FractionBox["1", "2"], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
              RowBox[{
               FractionBox["1", "2"], " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], 
             RowBox[{
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]]}], ")"}]}], 
         ")"}], "/", 
        RowBox[{"(", 
         RowBox[{"\[Sqrt]", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{
                RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", 
                   RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                  "+", 
                  RowBox[{"Log", "[", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                 ")"}]}], "]"}], "2"], " ", 
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"], " ", 
             SuperscriptBox[
              RowBox[{"Sin", "[", 
               RowBox[{
                RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", 
                   RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                  "+", 
                  RowBox[{"Log", "[", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                 ")"}]}], "]"}], "2"]}], "+", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
             "2"]}], ")"}]}], ")"}]}], "+", 
       RowBox[{
        RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"2", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{"s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", 
                   RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                  "+", 
                  RowBox[{"Log", "[", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                 ")"}]}], "]"}], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{"s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}], "/", 
            RowBox[{"(", 
             RowBox[{"\[Sqrt]", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"]}], "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"], " ", 
                 SuperscriptBox[
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"]}], "+", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                 "2"]}], ")"}]}], ")"}]}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{
               RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"-", 
                  RowBox[{"Log", "[", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                 "+", 
                 RowBox[{"Log", "[", 
                  RowBox[{
                   RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                   RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                ")"}]}], "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "4"}], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "3"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "-", 
               RowBox[{"4", " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "3"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                   RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"]}], "-", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "-", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}], 
                 ")"}]}]}], ")"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"]}], "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"], " ", 
                 SuperscriptBox[
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"]}], "+", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                 "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}]}]}], ")"}]}]}], 
   ")"}], "/", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{"Cot", "[", "\[Tau]", "]"}], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", 
          RowBox[{
           RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", 
              RowBox[{"Log", "[", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
             "+", 
             RowBox[{"Log", "[", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
            ")"}]}], "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Cos", "[", 
          RowBox[{"s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", 
          RowBox[{"s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", 
          RowBox[{
           RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", 
              RowBox[{"Log", "[", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
             "+", 
             RowBox[{"Log", "[", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
            ")"}]}], "]"}], "2"]}]}], ")"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{
           RowBox[{
            RowBox[{"-", 
             FractionBox["1", "2"]}], " ", 
            RowBox[{"Cos", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
           RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], 
          RowBox[{
           RowBox[{"Cos", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
           RowBox[{"Sin", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]]}], "+", 
        FractionBox[
         RowBox[{
          RowBox[{
           FractionBox["1", "2"], " ", 
           RowBox[{"Cos", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
          RowBox[{
           FractionBox["1", "2"], " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
           RowBox[{"Sin", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], 
         RowBox[{
          RowBox[{"Cos", "[", 
           RowBox[{
            FractionBox["1", "2"], " ", "s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
          RowBox[{"Sin", "[", 
           RowBox[{
            FractionBox["1", "2"], " ", "s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]]}], ")"}], "2"]}], 
    "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Cos", "[", 
         RowBox[{"s", " ", 
          RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", 
          RowBox[{
           RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", 
              RowBox[{"Log", "[", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
             "+", 
             RowBox[{"Log", "[", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
            ")"}]}], "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", 
          RowBox[{"s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Sin", "[", 
          RowBox[{
           RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", 
              RowBox[{"Log", "[", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
             "+", 
             RowBox[{"Log", "[", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
            ")"}]}], "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", 
          RowBox[{"s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}], ")"}]}]}], 
   ")"}]}]], "Output",
 CellChangeTimes->{3.626939097546627*^9, 3.6270290598912144`*^9, 
  3.6270430103492975`*^9, 3.627048024510104*^9, 3.6270506944391937`*^9, 
  3.627051332823515*^9, 3.6270523081684284`*^9, 3.62710645883405*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Together", "[", "%", "]"}], ",", " ", 
   RowBox[{"Element", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"s", " ", ",", "\[Tau]"}], "}"}], ",", "Reals"}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6267007717477064`*^9, 3.6267008002685566`*^9}, {
   3.626701007171522*^9, 3.626701018887142*^9}, 3.626707476428523*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   RowBox[{"Cos", "[", 
    RowBox[{"s", " ", 
     RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], " ", 
  SqrtBox[
   SuperscriptBox[
    RowBox[{"Sec", "[", 
     RowBox[{"s", " ", 
      RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]]}]], "Output",
 CellChangeTimes->{
  3.6267010008047104`*^9, 3.6267010994410844`*^9, {3.6267073454854918`*^9, 
   3.6267073741603427`*^9}, 3.6267801622878203`*^9, 3.627106514245347*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"kn", "[", "s_", "]"}], " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.6267073889199686`*^9, 3.6267073984535856`*^9}, 
   3.627106529502174*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"geodcurvature", "[", 
   RowBox[{
    RowBox[{"sphero", "[", "1", "]"}], ",", "hu", ",", "hv"}], "]"}], "[", 
  "s", "]"}]], "Input",
 CellChangeTimes->{{3.6270339335893745`*^9, 3.627033941826189*^9}}],

Cell[BoxData[
 RowBox[{"\[Sqrt]", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"Cos", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}], ")"}], 
       " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"2", " ", 
                 SuperscriptBox[
                  RowBox[{"Cot", "[", "\[Tau]", "]"}], "2"], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"]}]}], ")"}], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"-", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"-", 
                    FractionBox["1", "2"]}], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], 
                    ")"}]}], "+", 
                   RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}]}],
                   ")"}], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"-", 
                    FractionBox["1", "2"]}], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "2"], "/", 
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}], 
                    "2"]}], "-", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "2"], "/", 
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}], 
                    "2"]}], "+", 
                   RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"-", 
                    FractionBox["1", "4"]}], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"]}], "-", 
                    RowBox[{
                    FractionBox["1", "4"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], 
                   "-", 
                   RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"-", 
                    FractionBox["1", "4"]}], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"]}], "+", 
                    RowBox[{
                    FractionBox["1", "4"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}]}],
                   ")"}]}], "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cot", "[", "\[Tau]", "]"}], "2"], " ", 
                 SuperscriptBox[
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"-", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"-", 
                    FractionBox["1", "2"]}], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], 
                    ")"}]}], "+", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}]}],
                    ")"}], "2"], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"-", "2"}], " ", 
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "-", 
                   RowBox[{"2", " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}]}],
                 "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"-", "2"}], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
                   RowBox[{"2", " ", 
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
                   RowBox[{"2", " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], 
                  ")"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
             RowBox[{"2", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], "2"], " ", 
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"]}]}], ")"}], " ", 
                  SuperscriptBox[
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"-", 
                    FractionBox["1", "2"]}], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], 
                    ")"}]}], "+", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}]}],
                     ")"}], "2"]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"], " ", 
                  RowBox[{"(", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], "+", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}], 
                   ")"}]}]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}], ")"}]}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cot", "[", "\[Tau]", "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"]}]}], ")"}], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{"-", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"-", 
                    FractionBox["1", "2"]}], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                   "/", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], 
                  ")"}]}], "+", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                   RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                 "/", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                   RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}]}],
                ")"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}]}], 
            ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"2", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
               RowBox[{"2", " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}], 
              ")"}], " ", 
             RowBox[{"\[Sqrt]", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cot", "[", "\[Tau]", "]"}], "2"], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"]}]}], ")"}], " ", 
                 SuperscriptBox[
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"-", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"-", 
                    FractionBox["1", "2"]}], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], 
                    ")"}]}], "+", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}]}],
                    ")"}], "2"]}], "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}], 
                  ")"}]}]}], ")"}]}]}], ")"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"]}]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "2"}], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}]}], 
            ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"2", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
               RowBox[{"2", " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}], 
              ")"}], " ", 
             RowBox[{"\[Sqrt]", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cot", "[", "\[Tau]", "]"}], "2"], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"]}]}], ")"}], " ", 
                 SuperscriptBox[
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"-", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"-", 
                    FractionBox["1", "2"]}], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], 
                    ")"}]}], "+", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}]}],
                    ")"}], "2"]}], "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}], 
                  ")"}]}]}], ")"}]}]}], ")"}]}]}], ")"}], "2"]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"]}]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"-", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"-", 
                    FractionBox["1", "2"]}], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                   "/", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], 
                  ")"}]}], "+", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                   RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                 "/", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                   RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}]}],
                ")"}], " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"2", " ", 
                 SuperscriptBox[
                  RowBox[{"Cot", "[", "\[Tau]", "]"}], "2"], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"]}]}], ")"}], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"-", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"-", 
                    FractionBox["1", "2"]}], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], 
                    ")"}]}], "+", 
                   RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}]}],
                   ")"}], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"-", 
                    FractionBox["1", "2"]}], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "2"], "/", 
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}], 
                    "2"]}], "-", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "2"], "/", 
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}], 
                    "2"]}], "+", 
                   RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"-", 
                    FractionBox["1", "4"]}], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"]}], "-", 
                    RowBox[{
                    FractionBox["1", "4"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], 
                   "-", 
                   RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"-", 
                    FractionBox["1", "4"]}], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"]}], "+", 
                    RowBox[{
                    FractionBox["1", "4"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}]}],
                   ")"}]}], "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cot", "[", "\[Tau]", "]"}], "2"], " ", 
                 SuperscriptBox[
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"-", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"-", 
                    FractionBox["1", "2"]}], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], 
                    ")"}]}], "+", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}]}],
                    ")"}], "2"], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"-", "2"}], " ", 
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "-", 
                   RowBox[{"2", " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}]}],
                 "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"-", "2"}], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
                   RowBox[{"2", " ", 
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
                   RowBox[{"2", " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], 
                  ")"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
             RowBox[{"2", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], "2"], " ", 
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"]}]}], ")"}], " ", 
                  SuperscriptBox[
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"-", 
                    FractionBox["1", "2"]}], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], 
                    ")"}]}], "+", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}]}],
                     ")"}], "2"]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"], " ", 
                  RowBox[{"(", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], "+", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}], 
                   ")"}]}]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}], ")"}]}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"-", 
                    FractionBox["1", "2"]}], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                   RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                 "2"], "/", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                   RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}], 
                 "2"]}], "-", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                   RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                 "2"], "/", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                   RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}], 
                 "2"]}], "+", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"-", 
                    FractionBox["1", "4"]}], " ", 
                   RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"]}], "-", 
                  RowBox[{
                   FractionBox["1", "4"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                "/", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], 
               "-", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"-", 
                    FractionBox["1", "4"]}], " ", 
                   RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"]}], "+", 
                  RowBox[{
                   FractionBox["1", "4"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                "/", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}]}],
               ")"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"\[Sqrt]", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cot", "[", "\[Tau]", "]"}], "2"], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"]}]}], ")"}], " ", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"-", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"-", 
                    FractionBox["1", "2"]}], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], 
                    ")"}]}], "+", 
                   RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}]}],
                   ")"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"], " ", 
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}], 
                 ")"}]}]}], ")"}]}], ")"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Cos", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"-", 
                    FractionBox["1", "2"]}], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                  "/", 
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], 
                 ")"}]}], "+", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FractionBox["1", "2"], " ", 
                   RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                  RowBox[{
                   FractionBox["1", "2"], " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                "/", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}]}],
               ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "2"}], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "-", 
               RowBox[{"2", " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
             " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"Cos", "[", 
                 RowBox[{"s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}], 
              ")"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
               RowBox[{"2", " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}], 
              ")"}], " ", 
             RowBox[{"\[Sqrt]", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cot", "[", "\[Tau]", "]"}], "2"], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"]}]}], ")"}], " ", 
                 SuperscriptBox[
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"-", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"-", 
                    FractionBox["1", "2"]}], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], 
                    ")"}]}], "+", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
                    "/", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}]}],
                    ")"}], "2"]}], "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    "+", 
                    RowBox[{"Log", "[", 
                    RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                    RowBox[{"Sin", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
                    ")"}]}], "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}], 
                  ")"}]}]}], ")"}]}]}], ")"}]}]}], ")"}], "2"]}]}], ")"}], 
    "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"Cot", "[", "\[Tau]", "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"]}]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", 
                 FractionBox["1", "2"]}], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], 
             "/", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}]}], 
            ")"}]}], "+", 
          FractionBox[
           RowBox[{
            RowBox[{
             FractionBox["1", "2"], " ", 
             RowBox[{"Cos", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
            RowBox[{
             FractionBox["1", "2"], " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], 
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
            RowBox[{"Sin", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]]}], ")"}], 
        "2"]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"Cos", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}], 
        ")"}]}]}], ")"}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.627033949922603*^9, 3.627050708276418*^9, 
  3.6270523278088627`*^9, 3.6271065360697856`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Together", "[", "%", "]"}], ",", " ", 
   RowBox[{"Element", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"s", " ", ",", "\[Tau]"}], "}"}], ",", "Reals"}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6267007717477064`*^9, 3.6267008002685566`*^9}, {
   3.626701007171522*^9, 3.626701018887142*^9}, 3.626707476428523*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Abs", "[", 
   RowBox[{"Cos", "[", "\[Tau]", "]"}], "]"}], " ", 
  SqrtBox[
   SuperscriptBox[
    RowBox[{"Tan", "[", 
     RowBox[{"s", " ", 
      RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]]}]], "Output",
 CellChangeTimes->{3.626717607385458*^9, 3.626782717353908*^9, 
  3.627107177901713*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"kg", "[", "s_", "]"}], " ", ":=", 
  RowBox[{
   RowBox[{"Abs", "[", 
    RowBox[{"Cos", "[", "\[Tau]", "]"}], "]"}], " ", 
   SqrtBox[
    SuperscriptBox[
     RowBox[{"Tan", "[", 
      RowBox[{"s", " ", 
       RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]]}]}]], "Input",
 CellChangeTimes->{3.627107434787364*^9}],

Cell["\<\
 Since we know already that s is the arc length of the curve we may calculate \
kg[s] using another, more direct approach.\
\>", "Text",
 CellChangeTimes->{{3.626782725824723*^9, 3.626782824588496*^9}, 
   3.6268547425696583`*^9, {3.627107457547804*^9, 3.6271075211335154`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"clox", "[", "s_", "]"}], ":=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"hu", "[", "s", "]"}], ",", 
    RowBox[{"hv", "[", "s", "]"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.6267828895626106`*^9, 3.626782970183552*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{"clox", "[", "t", "]"}], ",", "t"}], "]"}], "/.", 
  RowBox[{"t", "\[Rule]", "s"}]}]], "Input",
 CellChangeTimes->{{3.6267829797619686`*^9, 3.6267830602737103`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           FractionBox["1", "2"]}], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{
            FractionBox["1", "2"], " ", "s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
          RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
         RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{
            FractionBox["1", "2"], " ", "s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], 
        RowBox[{
         RowBox[{"Cos", "[", 
          RowBox[{
           FractionBox["1", "2"], " ", "s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
         RowBox[{"Sin", "[", 
          RowBox[{
           FractionBox["1", "2"], " ", "s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]]}], "+", 
      FractionBox[
       RowBox[{
        RowBox[{
         FractionBox["1", "2"], " ", 
         RowBox[{"Cos", "[", 
          RowBox[{
           FractionBox["1", "2"], " ", "s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
         RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
        RowBox[{
         FractionBox["1", "2"], " ", 
         RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
         RowBox[{"Sin", "[", 
          RowBox[{
           FractionBox["1", "2"], " ", "s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], 
       RowBox[{
        RowBox[{"Cos", "[", 
         RowBox[{
          FractionBox["1", "2"], " ", "s", " ", 
          RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
        RowBox[{"Sin", "[", 
         RowBox[{
          FractionBox["1", "2"], " ", "s", " ", 
          RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]]}], ")"}]}], ",", 
   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.626783064142517*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tclox", "[", "s_", "]"}], " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.626783217381586*^9, 3.6267832392060246`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"fund1", "[", 
       RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "[", 
     RowBox[{
      RowBox[{"tclox", "[", "s", "]"}], ",", 
      RowBox[{"tclox", "[", "s", "]"}]}], "]"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"u", "->", 
       RowBox[{"hu", "[", "s", "]"}]}], ",", 
      RowBox[{"v", "->", 
       RowBox[{"hv", "[", "s", "]"}]}]}], "}"}]}], ",", " ", 
   RowBox[{"Element", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"s", " ", ",", "\[Tau]"}], "}"}], ",", "Reals"}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.626783116121808*^9, 3.6267831927959433`*^9}, {
   3.6267832573020563`*^9, 3.6267833060677423`*^9}, 3.626783339795001*^9, {
   3.62678337896667*^9, 3.626783446171588*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.62678336770345*^9, 3.626783454112002*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "DD1"}]], "Input",
 CellChangeTimes->{{3.6267834794152465`*^9, 3.6267834832060533`*^9}}],

Cell[BoxData[
 StyleBox["\<\"DD1[ff,cu,vw][t] is the absolute derivative of a vector \
function \\nvw[t] ={vw1[t],vw2[t]} defined along a curve with local \
coordinatess cu[t]={u[t],v[t]} \\nof a surface ff[u,v] in the n-dimensional \
Euclidean space. For higher dimensional \\nimmersions or Riemannian spaces \
create a new Module changing the definition of \[CapitalGamma]\[CapitalGamma] \
\\ncorresponding to the situation.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.626783484532056*^9},
 CellTags->"Info3626787084-5796653"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DD1", "[", 
   RowBox[{
    RowBox[{"sphero", "[", "1", "]"}], ",", "clox", ",", "tclox"}], "]"}], 
  "[", "s", "]"}]], "Input",
 CellChangeTimes->{{3.6267834924100695`*^9, 3.6267835044844904`*^9}, {
  3.626783545247362*^9, 3.6267835682574024`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", 
             FractionBox["1", "2"]}], " ", 
            RowBox[{"Cos", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
           RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], "2"], 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Cos", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
           RowBox[{"Sin", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}], "2"]], 
       "-", 
       FractionBox[
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{"Cos", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
           RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], "2"], 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Cos", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
           RowBox[{"Sin", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ")"}], "2"]], 
       "+", 
       FractionBox[
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           FractionBox["1", "4"]}], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{
            FractionBox["1", "2"], " ", "s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"]}], "-", 
         RowBox[{
          FractionBox["1", "4"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{
            FractionBox["1", "2"], " ", "s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], 
        RowBox[{
         RowBox[{"Cos", "[", 
          RowBox[{
           FractionBox["1", "2"], " ", "s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
         RowBox[{"Sin", "[", 
          RowBox[{
           FractionBox["1", "2"], " ", "s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]], "-", 
       FractionBox[
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           FractionBox["1", "4"]}], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{
            FractionBox["1", "2"], " ", "s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"]}], "+", 
         RowBox[{
          FractionBox["1", "4"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{
            FractionBox["1", "2"], " ", "s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], 
        RowBox[{
         RowBox[{"Cos", "[", 
          RowBox[{
           FractionBox["1", "2"], " ", "s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
         RowBox[{"Sin", "[", 
          RowBox[{
           FractionBox["1", "2"], " ", "s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]]}], ")"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Cos", "[", "\[Tau]", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              FractionBox["1", "2"]}], " ", 
             RowBox[{"Cos", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
            RowBox[{
             FractionBox["1", "2"], " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], 
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
            RowBox[{"Sin", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]]}], "+", 
         FractionBox[
          RowBox[{
           RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{"Cos", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
           RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
            RowBox[{"Sin", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], 
          RowBox[{
           RowBox[{"Cos", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
           RowBox[{"Sin", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "2"}], " ", 
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "-", 
         RowBox[{"2", " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"Cos", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}], ")"}]}], 
      ")"}], "/", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", 
          RowBox[{
           RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", 
              RowBox[{"Log", "[", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
             "+", 
             RowBox[{"Log", "[", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
            ")"}]}], "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Cos", "[", 
          RowBox[{"s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", 
          RowBox[{"s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", 
          RowBox[{
           RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", 
              RowBox[{"Log", "[", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
             "+", 
             RowBox[{"Log", "[", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
            ")"}]}], "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", 
          RowBox[{
           RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", 
              RowBox[{"Log", "[", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
             "+", 
             RowBox[{"Log", "[", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
            ")"}]}], "]"}], "4"], " ", 
        SuperscriptBox[
         RowBox[{"Cos", "[", 
          RowBox[{"s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", 
          RowBox[{"s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
       RowBox[{"2", " ", 
        SuperscriptBox[
         RowBox[{"Cos", "[", 
          RowBox[{
           RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", 
              RowBox[{"Log", "[", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
             "+", 
             RowBox[{"Log", "[", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
            ")"}]}], "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Cos", "[", 
          RowBox[{"s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", 
          RowBox[{
           RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", 
              RowBox[{"Log", "[", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
             "+", 
             RowBox[{"Log", "[", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
            ")"}]}], "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", 
          RowBox[{"s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", 
          RowBox[{"s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", 
          RowBox[{
           RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", 
              RowBox[{"Log", "[", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                RowBox[{"Sin", "[", 
                 RowBox[{
                  FractionBox["1", "2"], " ", "s", " ", 
                  RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
             "+", 
             RowBox[{"Log", "[", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
               RowBox[{"Sin", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", "s", " ", 
                 RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
            ")"}]}], "]"}], "4"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", 
          RowBox[{"s", " ", 
           RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}], ")"}]}]}], 
   ",", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Cot", "[", "\[Tau]", "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"]}]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           FractionBox[
            RowBox[{
             RowBox[{
              RowBox[{"-", 
               FractionBox["1", "2"]}], " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
             RowBox[{
              FractionBox["1", "2"], " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
             RowBox[{"Sin", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]]}], "+", 
          FractionBox[
           RowBox[{
            RowBox[{
             FractionBox["1", "2"], " ", 
             RowBox[{"Cos", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "-", 
            RowBox[{
             FractionBox["1", "2"], " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], 
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
            RowBox[{"Sin", "[", 
             RowBox[{
              FractionBox["1", "2"], " ", "s", " ", 
              RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]]}], ")"}], "2"], 
       " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", 
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
         RowBox[{"2", " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}]}], ")"}], 
     "/", 
     RowBox[{"(", 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
         RowBox[{"2", " ", 
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}], ")"}]}], 
      ")"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"]}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "2"}], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
         RowBox[{"2", " ", 
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "+", 
         RowBox[{"2", " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"s", " ", 
            RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], ")"}]}], ")"}], 
     "/", 
     RowBox[{"(", 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
         RowBox[{"2", " ", 
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{
             RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Log", "[", 
                 RowBox[{
                  RowBox[{"Cos", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
                  RowBox[{"Sin", "[", 
                   RowBox[{
                    FractionBox["1", "2"], " ", "s", " ", 
                    RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
               "+", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
                 RowBox[{"Sin", "[", 
                  RowBox[{
                   FractionBox["1", "2"], " ", "s", " ", 
                   RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
              ")"}]}], "]"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"s", " ", 
             RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}], ")"}]}], 
      ")"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.626783580753024*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", ",", 
   RowBox[{"Element", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"s", " ", ",", "\[Tau]"}], "}"}], ",", "Reals"}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6267836041842656`*^9, 3.626783641577531*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     RowBox[{"Cos", "[", "\[Tau]", "]"}]}], " ", 
    RowBox[{"Sec", "[", 
     RowBox[{"s", " ", 
      RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
    RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
    RowBox[{"Tan", "[", 
     RowBox[{"s", " ", 
      RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ",", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"Cos", "[", "\[Tau]", "]"}], "2"], " ", 
    RowBox[{"Tan", "[", 
     RowBox[{"s", " ", 
      RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.626783645945539*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"ttclox", "[", "s_", "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{"Cos", "[", "\[Tau]", "]"}]}], " ", 
     RowBox[{"Sec", "[", 
      RowBox[{"s", " ", 
       RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], " ", 
     RowBox[{"Sin", "[", "\[Tau]", "]"}], " ", 
     RowBox[{"Tan", "[", 
      RowBox[{"s", " ", 
       RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], ",", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"Cos", "[", "\[Tau]", "]"}], "2"], " ", 
     RowBox[{"Tan", "[", 
      RowBox[{"s", " ", 
       RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.6267836564287577`*^9, 3.6267836665219755`*^9}, {
  3.626785684354319*^9, 3.6267856907035303`*^9}, {3.626785750716836*^9, 
  3.626785786815299*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"fund1", "[", 
     RowBox[{"sphero", "[", "1", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "[", 
   RowBox[{
    RowBox[{"ttclox", "[", "s", "]"}], ",", 
    RowBox[{"ttclox", "[", "s", "]"}]}], "]"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"u", "->", 
     RowBox[{"hu", "[", "s", "]"}]}], ",", 
    RowBox[{"v", "->", 
     RowBox[{"hv", "[", "s", "]"}]}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.62678579860892*^9, 3.6267858667186394`*^9}, 
   3.626785916841528*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox[
    RowBox[{"Cos", "[", "\[Tau]", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"Sec", "[", 
     RowBox[{"s", " ", 
      RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"Sin", "[", "\[Tau]", "]"}], "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"Cos", "[", 
        RowBox[{
         RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"Log", "[", 
             RowBox[{
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], "+", 
           RowBox[{"Log", "[", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
             RowBox[{"Sin", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
          ")"}]}], "]"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"Cos", "[", 
        RowBox[{"s", " ", 
         RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"Cos", "[", 
        RowBox[{"s", " ", 
         RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"Sin", "[", 
        RowBox[{
         RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"Log", "[", 
             RowBox[{
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], "+", 
           RowBox[{"Log", "[", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
             RowBox[{"Sin", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
          ")"}]}], "]"}], "2"]}]}], ")"}], " ", 
   SuperscriptBox[
    RowBox[{"Tan", "[", 
     RowBox[{"s", " ", 
      RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"Cos", "[", "\[Tau]", "]"}], "4"], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"Cos", "[", 
       RowBox[{"s", " ", 
        RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"], "+", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"Cos", "[", 
        RowBox[{
         RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"Log", "[", 
             RowBox[{
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], "+", 
           RowBox[{"Log", "[", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
             RowBox[{"Sin", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
          ")"}]}], "]"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"Sin", "[", 
        RowBox[{"s", " ", 
         RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "+", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"Sin", "[", 
        RowBox[{
         RowBox[{"Cot", "[", "\[Tau]", "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"Log", "[", 
             RowBox[{
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "-", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", "s", " ", 
                RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], "+", 
           RowBox[{"Log", "[", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "+", 
             RowBox[{"Sin", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", "s", " ", 
               RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}]}], "]"}]}], 
          ")"}]}], "]"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"Sin", "[", 
        RowBox[{"s", " ", 
         RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}], ")"}], " ", 
   SuperscriptBox[
    RowBox[{"Tan", "[", 
     RowBox[{"s", " ", 
      RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]}]], "Output",
 CellChangeTimes->{3.626785924984742*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", ",", 
   RowBox[{"Element", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"s", " ", ",", "\[Tau]"}], "}"}], ",", "Reals"}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6267859667616153`*^9, 3.6267860305033274`*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox[
   RowBox[{"Cos", "[", "\[Tau]", "]"}], "2"], " ", 
  SuperscriptBox[
   RowBox[{"Tan", "[", 
    RowBox[{"s", " ", 
     RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}]], "Output",
 CellChangeTimes->{3.626786037258139*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"kg1", "[", "s_", "]"}], " ", ":=", " ", 
  RowBox[{"Sqrt", "[", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"Cos", "[", "\[Tau]", "]"}], "2"], " ", 
    SuperscriptBox[
     RowBox[{"Tan", "[", 
      RowBox[{"s", " ", 
       RowBox[{"Sin", "[", "\[Tau]", "]"}]}], "]"}], "2"]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.62678605491737*^9, 3.6267860772878094`*^9}, 
   3.626786139563119*^9, 3.6271075392919474`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"kg", "[", "s", "]"}], " ", "-", 
    RowBox[{"kg1", "[", "s", "]"}]}], ",", 
   RowBox[{"Element", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"s", " ", ",", "\[Tau]"}], "}"}], ",", "Reals"}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6271075669195957`*^9, 3.627107604344062*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{{3.627107607401667*^9, 3.627107636308518*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Together", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"kk", "[", "s", "]"}], "^", "2"}], " ", "-", 
     RowBox[{
      RowBox[{"kn", "[", "s", "]"}], "^", "2"}], " ", "-", 
     RowBox[{
      RowBox[{"kg", "[", "s", "]"}], "^", "2"}]}], " ", "]"}], ",", 
   RowBox[{"Element", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"s", " ", ",", "\[Tau]"}], "}"}], ",", "Reals"}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.626783673307987*^9, 3.626783756518533*^9}, {
   3.626783813302633*^9, 3.626783827186657*^9}, 3.6267838694471316`*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6267838807571516`*^9, 3.6267860944166393`*^9, 
  3.6267861466923313`*^9, 3.6271076413629265`*^9}]
}, Closed]],

Cell["OK!", "Text",
 CellChangeTimes->{{3.6267861581271515`*^9, 3.626786161387557*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["4.3.5. Another Curve on the Sphere", "Subsubsection",
 CellChangeTimes->{{3.626276300785455*^9, 3.6262763024390583`*^9}, {
   3.626437663195213*^9, 3.626437681915246*^9}, {3.6265357876021376`*^9, 
   3.626535792438146*^9}, 3.626780364932176*^9, {3.6271128923477497`*^9, 
   3.627112907978977*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", " ", "[", 
   RowBox[{"hu", ",", "hv", ",", "kk", ",", "kg", ",", "kn"}], "]"}], ";", 
  RowBox[{
   RowBox[{"hu", "[", "s_", "]"}], " ", "=", " ", "s"}], ";", " ", 
  RowBox[{
   RowBox[{"hv", "[", "s_", "]"}], " ", "=", " ", 
   RowBox[{"Cos", "[", "s", "]"}]}], ";", 
  RowBox[{
   RowBox[{"sphercx", "[", "s_", "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{"sphero", "[", "1", "]"}], "[", 
    RowBox[{
     RowBox[{"hu", "[", "s", "]"}], ",", 
     RowBox[{"hv", "[", "s", "]"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6262696713989577`*^9, 3.626269769583931*^9}, {
   3.62627057233515*^9, 3.626270580473765*^9}, {3.6262715010427923`*^9, 
   3.6262715449472694`*^9}, {3.62627487396774*^9, 3.626274910835605*^9}, 
   3.6263327382028837`*^9, {3.626588250146246*^9, 3.6265882709254827`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sphercx", "[", "s", "]"}]], "Input",
 CellChangeTimes->{{3.6262705892741804`*^9, 3.626270596951394*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", "s", "]"}], " ", 
    RowBox[{"Cos", "[", 
     RowBox[{"Cos", "[", "s", "]"}], "]"}]}], ",", 
   RowBox[{
    RowBox[{"Cos", "[", 
     RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
    RowBox[{"Sin", "[", "s", "]"}]}], ",", 
   RowBox[{"Sin", "[", 
    RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6262705981369963`*^9, 3.626274916498415*^9, 
  3.6263326919332027`*^9, 3.626332747859301*^9, 3.6264178574881763`*^9, 
  3.626439411412284*^9, 3.6265881729261103`*^9, 3.6265882861667094`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"sphercx", "[", "s", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"PlotStyle", " ", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Red", ",", "Thick"}], "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.626332700076417*^9, 3.6263327166904464`*^9}, {
  3.6263327568605165`*^9, 3.6263327771249523`*^9}, {3.6263332248613386`*^9, 
  3.6263332657178106`*^9}}],

Cell[BoxData[
 Graphics3DBox[{{}, {}, 
   {RGBColor[1, 0, 0], Thickness[Large], Line3DBox[CompressedData["
1:eJw123k4VO8XAHBFEmmjtEuSCmVJSnK0b5bKkiJJaCUkJERCKbsI2bNTdip1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     "]]}},
  AutomaticImageSize->True,
  Axes->True,
  ImageSize->{332.5270011025314, 366.7057606241775},
  Method->{},
  PlotRange->{{-0.5610963372145253, 0.5610962654964522}, {-0.999999718426374, 
   0.999999756422448}, {-0.841470921632056, 0.8414709848078921}},
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]},
  ViewPoint->{0.9237739730086653, -3.228900652904626, 0.4133306430254975},
  ViewVertical->{0.17077830036338873`, -0.4803919815441937, 
   1.0360468717194946`}]], "Output",
 CellChangeTimes->{3.6263333408787427`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"curv", " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.62633335644757*^9, 3.626333362547181*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"curv", ",", "sph1"}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.626333304639879*^9, 3.626333323281912*^9}, {
  3.626333369348793*^9, 3.626333387288824*^9}}],

Cell[BoxData[
 Graphics3DBox[{{{}, {}, 
    {RGBColor[1, 0, 0], Thickness[Large], Line3DBox[CompressedData["
1:eJw123k4VO8XAHBFEmmjtEuSCmVJSnK0b5bKkiJJaCUkJERCKbsI2bNTdip1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      "]]}}, GraphicsComplex3DBox[CompressedData["
1:eJx1m3lcz9n3x0N+Zsg6Y5kxDYVkkCWylE4MQtayi6HQNikMRihkq+zMhIhk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    "], {
     {Opacity[0.4], EdgeForm[None], 
      GraphicsGroup3DBox[Polygon3DBox[CompressedData["
1:eJxNm3f819Mexz/fYxMapCTtQUpLE/0KmaWhIdGyqTSISoNkZ5P2kNIgWRk3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        "]]]}, {}, {}, {}, {}},
    VertexNormals->CompressedData["
1:eJx1W2lYT93bbUJ5VGgwRCFkDEkifseQJBRlDJVSoVGpkBQhDSJEac5QSEil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     "]]},
  AutomaticImageSize->True,
  Axes->True,
  ImageSize->{374.20632387718615`, 358.51929988987405`},
  Method->{},
  PlotRange->All,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]},
  ViewPoint->{-3.1024447677623566`, 0.0598631230036527, -1.3495380207642291`},
  ViewVertical->{-0.7354819613950287, -0.6532438925696639, 
   0.17982964517282046`}]], "Output",
 CellChangeTimes->{{3.626333371953997*^9, 3.6263333899564285`*^9}}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"kk", "[", "s_", "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"curvature", "[", "sphercx", "]"}], "[", "t", "]"}], "/.", 
   RowBox[{"t", "\[Rule]", " ", "s"}]}]}]], "Input",
 CellChangeTimes->{{3.6262697808793507`*^9, 3.626269850185673*^9}, {
  3.6262706253152437`*^9, 3.6262706316176553`*^9}, {3.6262715558516884`*^9, 
  3.6262715729357185`*^9}, {3.626439396046256*^9, 3.6264394321603203`*^9}, {
  3.6265883015171366`*^9, 3.626588320159169*^9}, {3.6265887890335927`*^9, 
  3.6265888381112785`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kk", "[", "s", "]"}]], "Input",
 CellChangeTimes->{{3.626418694426047*^9, 3.626418752707749*^9}, {
  3.6264394420975375`*^9, 3.626439458461966*^9}, {3.6265883385516014`*^9, 
  3.6265883434812098`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"\[Sqrt]", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{
            SqrtBox["2"], " ", 
            RowBox[{"Cos", "[", "s", "]"}], " ", 
            RowBox[{"Cos", "[", 
             RowBox[{"Cos", "[", "s", "]"}], "]"}]}], 
           SqrtBox[
            RowBox[{"2", "-", 
             RowBox[{"Cos", "[", 
              RowBox[{"2", " ", "s"}], "]"}], "+", 
             RowBox[{"Cos", "[", 
              RowBox[{"2", " ", 
               RowBox[{"Cos", "[", "s", "]"}]}], "]"}]}]]]}], "-", 
         FractionBox[
          RowBox[{
           SqrtBox["2"], " ", 
           SuperscriptBox[
            RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
           RowBox[{"Sin", "[", 
            RowBox[{"Cos", "[", "s", "]"}], "]"}]}], 
          SqrtBox[
           RowBox[{"2", "-", 
            RowBox[{"Cos", "[", 
             RowBox[{"2", " ", "s"}], "]"}], "+", 
            RowBox[{"Cos", "[", 
             RowBox[{"2", " ", 
              RowBox[{"Cos", "[", "s", "]"}]}], "]"}]}]]], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"Cos", "[", 
             RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
            RowBox[{"Sin", "[", "s", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"2", " ", 
               RowBox[{"Sin", "[", 
                RowBox[{"2", " ", "s"}], "]"}]}], "+", 
              RowBox[{"2", " ", 
               RowBox[{"Sin", "[", "s", "]"}], " ", 
               RowBox[{"Sin", "[", 
                RowBox[{"2", " ", 
                 RowBox[{"Cos", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], ")"}], 
          "/", 
          RowBox[{"(", 
           RowBox[{
            SqrtBox["2"], " ", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{"2", "-", 
               RowBox[{"Cos", "[", 
                RowBox[{"2", " ", "s"}], "]"}], "+", 
               RowBox[{"Cos", "[", 
                RowBox[{"2", " ", 
                 RowBox[{"Cos", "[", "s", "]"}]}], "]"}]}], ")"}], 
             RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{
            SqrtBox["2"], " ", 
            RowBox[{"Cos", "[", "s", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"Cos", "[", 
               RowBox[{"Cos", "[", "s", "]"}], "]"}], "-", 
              RowBox[{
               RowBox[{"Cos", "[", "s", "]"}], " ", 
               RowBox[{"Sin", "[", 
                RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}]}], 
           SqrtBox[
            RowBox[{"2", "-", 
             RowBox[{"Cos", "[", 
              RowBox[{"2", " ", "s"}], "]"}], "+", 
             RowBox[{"Cos", "[", 
              RowBox[{"2", " ", 
               RowBox[{"Cos", "[", "s", "]"}]}], "]"}]}]]]}], "-", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SqrtBox["2"], " ", 
            RowBox[{"Sin", "[", "s", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"Cos", "[", "s", "]"}], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
               RowBox[{"Sin", "[", "s", "]"}]}], "+", 
              RowBox[{"2", " ", 
               RowBox[{"Sin", "[", "s", "]"}], " ", 
               RowBox[{"Sin", "[", 
                RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}]}], ")"}], 
          "/", 
          RowBox[{"(", 
           SqrtBox[
            RowBox[{"2", "-", 
             RowBox[{"Cos", "[", 
              RowBox[{"2", " ", "s"}], "]"}], "+", 
             RowBox[{"Cos", "[", 
              RowBox[{"2", " ", 
               RowBox[{"Cos", "[", "s", "]"}]}], "]"}]}]], ")"}]}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"Sin", "[", "s", "]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"Cos", "[", 
               RowBox[{"Cos", "[", "s", "]"}], "]"}], "-", 
              RowBox[{
               RowBox[{"Cos", "[", "s", "]"}], " ", 
               RowBox[{"Sin", "[", 
                RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"2", " ", 
               RowBox[{"Sin", "[", 
                RowBox[{"2", " ", "s"}], "]"}]}], "+", 
              RowBox[{"2", " ", 
               RowBox[{"Sin", "[", "s", "]"}], " ", 
               RowBox[{"Sin", "[", 
                RowBox[{"2", " ", 
                 RowBox[{"Cos", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], ")"}], 
          "/", 
          RowBox[{"(", 
           RowBox[{
            SqrtBox["2"], " ", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{"2", "-", 
               RowBox[{"Cos", "[", 
                RowBox[{"2", " ", "s"}], "]"}], "+", 
               RowBox[{"Cos", "[", 
                RowBox[{"2", " ", 
                 RowBox[{"Cos", "[", "s", "]"}]}], "]"}]}], ")"}], 
             RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SqrtBox["2"], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"-", 
                RowBox[{"Cos", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}]}], " ", 
               RowBox[{"Sin", "[", "s", "]"}]}], "-", 
              RowBox[{
               RowBox[{"Cos", "[", 
                RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
               SuperscriptBox[
                RowBox[{"Sin", "[", "s", "]"}], "3"]}], "+", 
              RowBox[{"3", " ", 
               RowBox[{"Cos", "[", "s", "]"}], " ", 
               RowBox[{"Sin", "[", "s", "]"}], " ", 
               RowBox[{"Sin", "[", 
                RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}]}], ")"}], 
          "/", 
          RowBox[{"(", 
           SqrtBox[
            RowBox[{"2", "-", 
             RowBox[{"Cos", "[", 
              RowBox[{"2", " ", "s"}], "]"}], "+", 
             RowBox[{"Cos", "[", 
              RowBox[{"2", " ", 
               RowBox[{"Cos", "[", "s", "]"}]}], "]"}]}]], ")"}]}], "-", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"Cos", "[", "s", "]"}], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
               RowBox[{"Sin", "[", 
                RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"2", " ", 
               RowBox[{"Sin", "[", 
                RowBox[{"2", " ", "s"}], "]"}]}], "+", 
              RowBox[{"2", " ", 
               RowBox[{"Sin", "[", "s", "]"}], " ", 
               RowBox[{"Sin", "[", 
                RowBox[{"2", " ", 
                 RowBox[{"Cos", "[", "s", "]"}]}], "]"}]}]}], ")"}]}], ")"}], 
          "/", 
          RowBox[{"(", 
           RowBox[{
            SqrtBox["2"], " ", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{"2", "-", 
               RowBox[{"Cos", "[", 
                RowBox[{"2", " ", "s"}], "]"}], "+", 
               RowBox[{"Cos", "[", 
                RowBox[{"2", " ", 
                 RowBox[{"Cos", "[", "s", "]"}]}], "]"}]}], ")"}], 
             RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}], "2"]}], ")"}]}], 
   ")"}], "/", 
  RowBox[{"(", 
   RowBox[{"\[Sqrt]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"Cos", "[", 
         RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
       SuperscriptBox[
        RowBox[{"Sin", "[", "s", "]"}], "2"]}], "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{"Cos", "[", 
            RowBox[{"Cos", "[", "s", "]"}], "]"}]}], " ", 
          RowBox[{"Sin", "[", "s", "]"}]}], "+", 
         RowBox[{
          RowBox[{"Cos", "[", "s", "]"}], " ", 
          RowBox[{"Sin", "[", "s", "]"}], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"Cos", "[", "s", "]"}], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}], "2"]}], ")"}]}], 
   ")"}]}]], "Output",
 CellChangeTimes->{3.6265883454000134`*^9, 3.626588843228088*^9, 
  3.6271116060786905`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"kk", "[", "s", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6263335604023285`*^9, 3.6263335987627954`*^9}, {
  3.626418774282587*^9, 3.6264187957482243`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"kn", "[", "s_", "]"}], ":=", 
  RowBox[{
   RowBox[{"normalcurvature", "[", 
    RowBox[{
     RowBox[{"sphero", "[", "1", "]"}], ",", "hu", ",", "hv"}], "]"}], "[", 
   "s", "]"}]}]], "Input",
 CellChangeTimes->{{3.6262697808793507`*^9, 3.626269850185673*^9}, {
   3.6262706253152437`*^9, 3.6262706316176553`*^9}, {3.626271402313818*^9, 
   3.6262714245634575`*^9}, {3.62627146016672*^9, 3.626271463349126*^9}, 
   3.6262715810029325`*^9, 3.626271640242237*^9, 3.626333487987001*^9, 
   3.626436637696212*^9, {3.6264394810664062`*^9, 3.6264394855280137`*^9}, {
   3.626588871854138*^9, 3.626588904270995*^9}, 3.6265889604466934`*^9, {
   3.6271116691964016`*^9, 3.627111682503225*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kn", "[", "s", "]"}]], "Input",
 CellChangeTimes->{{3.6264189328412657`*^9, 3.626418960000913*^9}, {
  3.626419062446293*^9, 3.626419083116329*^9}, {3.626436661860654*^9, 
  3.6264366832638917`*^9}, {3.6264394988972373`*^9, 3.6264395120792603`*^9}, {
  3.6265889178586187`*^9, 3.6265889221018267`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{"Cos", "[", 
       RowBox[{"Cos", "[", "s", "]"}], "]"}]}], " ", 
     SuperscriptBox[
      RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"Cos", "[", 
             RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"Cos", "[", 
             RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"Sin", "[", "s", "]"}], "2"]}], "-", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"Sin", "[", 
             RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "-", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"Sin", "[", 
             RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{"\[Sqrt]", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"Cos", "[", "s", "]"}], "]"}], "4"]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"Cos", "[", "s", "]"}], "]"}], "4"], " ", 
             SuperscriptBox[
              RowBox[{"Sin", "[", "s", "]"}], "2"]}], "+", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}], "2"]}], 
           ")"}]}], ")"}]}], "-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
             RowBox[{"Cos", "[", 
              RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{"Cos", "[", 
              RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "4"}], " ", 
             SuperscriptBox[
              RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"Cos", "[", "s", "]"}], "]"}], "3"], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "-", 
            RowBox[{"4", " ", 
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"Cos", "[", "s", "]"}], "]"}], "3"], " ", 
             SuperscriptBox[
              RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
             RowBox[{"Sin", "[", 
              RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
            RowBox[{"2", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "2"]}], "-", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "-", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}]}], ")"}]}]}], 
           ")"}]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{"2", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
              SuperscriptBox[
               RowBox[{"Cos", "[", 
                RowBox[{"Cos", "[", "s", "]"}], "]"}], "4"]}], "+", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"Cos", "[", 
                RowBox[{"Cos", "[", "s", "]"}], "]"}], "4"], " ", 
              SuperscriptBox[
               RowBox[{"Sin", "[", "s", "]"}], "2"]}], "+", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                 RowBox[{"Sin", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                 SuperscriptBox[
                  RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                 RowBox[{"Sin", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}], "2"]}], 
            ")"}], 
           RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}]}], "-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"Cos", "[", 
          RowBox[{"Cos", "[", "s", "]"}], "]"}]}], " ", 
        RowBox[{"Sin", "[", "s", "]"}]}], "+", 
       RowBox[{
        RowBox[{"Cos", "[", "s", "]"}], " ", 
        RowBox[{"Sin", "[", "s", "]"}], " ", 
        RowBox[{"Sin", "[", 
         RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"Cos", "[", 
             RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
           RowBox[{"Sin", "[", "s", "]"}]}], ")"}], "/", 
         RowBox[{"(", 
          RowBox[{"\[Sqrt]", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
              SuperscriptBox[
               RowBox[{"Cos", "[", 
                RowBox[{"Cos", "[", "s", "]"}], "]"}], "4"]}], "+", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"Cos", "[", 
                RowBox[{"Cos", "[", "s", "]"}], "]"}], "4"], " ", 
              SuperscriptBox[
               RowBox[{"Sin", "[", "s", "]"}], "2"]}], "+", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                 RowBox[{"Sin", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
                RowBox[{
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                 SuperscriptBox[
                  RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                 RowBox[{"Sin", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}], "2"]}], 
            ")"}]}], ")"}]}]}], "-", 
       RowBox[{
        RowBox[{"Sin", "[", "s", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"2", " ", 
              RowBox[{"Cos", "[", "s", "]"}], " ", 
              RowBox[{"Cos", "[", 
               RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{"Cos", "[", "s", "]"}], "]"}]}], ")"}], "/", 
            RowBox[{"(", 
             RowBox[{"\[Sqrt]", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"Cos", "[", "s", "]"}], "]"}], "4"]}], "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"Cos", "[", "s", "]"}], "]"}], "4"], " ", 
                 SuperscriptBox[
                  RowBox[{"Sin", "[", "s", "]"}], "2"]}], "+", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}], "2"]}], 
               ")"}]}], ")"}]}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Cos", "[", "s", "]"}], " ", 
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "4"}], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "3"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "-", 
               RowBox[{"4", " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "3"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                   RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", "s", "]"}], "2"]}], "-", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "-", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}]}], 
                 ")"}]}]}], ")"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"Cos", "[", "s", "]"}], "]"}], "4"]}], "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"Cos", "[", "s", "]"}], "]"}], "4"], " ", 
                 SuperscriptBox[
                  RowBox[{"Sin", "[", "s", "]"}], "2"]}], "+", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}], "2"]}], 
               ")"}], 
              RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}]}]}], ")"}]}], "-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", "s", "]"}], " ", 
        RowBox[{"Cos", "[", 
         RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
        RowBox[{"Sin", "[", 
         RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Cos", "[", "s", "]"}], " ", 
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{"\[Sqrt]", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"Cos", "[", "s", "]"}], "]"}], "4"]}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"Cos", "[", "s", "]"}], "]"}], "4"], " ", 
             SuperscriptBox[
              RowBox[{"Sin", "[", "s", "]"}], "2"]}], "+", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
               RowBox[{
                RowBox[{"Cos", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}], "2"]}], 
           ")"}]}], ")"}]}], "-", 
       RowBox[{
        RowBox[{"Sin", "[", "s", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"2", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
              RowBox[{"Sin", "[", "s", "]"}], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{"Cos", "[", "s", "]"}], "]"}]}], ")"}], "/", 
            RowBox[{"(", 
             RowBox[{"\[Sqrt]", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"Cos", "[", "s", "]"}], "]"}], "4"]}], "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"Cos", "[", "s", "]"}], "]"}], "4"], " ", 
                 SuperscriptBox[
                  RowBox[{"Sin", "[", "s", "]"}], "2"]}], "+", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}], "2"]}], 
               ")"}]}], ")"}]}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"Cos", "[", 
               RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
             RowBox[{"Sin", "[", "s", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "4"}], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "3"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "-", 
               RowBox[{"4", " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "3"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                   RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
                  RowBox[{
                   RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", "s", "]"}], "2"]}], "-", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "-", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}]}], 
                 ")"}]}]}], ")"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"Cos", "[", "s", "]"}], "]"}], "4"]}], "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"Cos", "[", "s", "]"}], "]"}], "4"], " ", 
                 SuperscriptBox[
                  RowBox[{"Sin", "[", "s", "]"}], "2"]}], "+", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                    RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
                   RowBox[{
                    RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}], "2"]}], 
               ")"}], 
              RowBox[{"3", "/", "2"}]]}], ")"}]}]}], ")"}]}]}], ")"}]}]}], 
   ")"}], "/", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
     SuperscriptBox[
      RowBox[{"Cos", "[", 
       RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"Cos", "[", 
       RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
     SuperscriptBox[
      RowBox[{"Sin", "[", "s", "]"}], "2"]}], "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Cos", "[", 
         RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", 
          RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", 
          RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}]}], ")"}]}]}], 
   ")"}]}]], "Output",
 CellChangeTimes->{3.626588923115828*^9, 3.6265889676851063`*^9, 
  3.627112248706219*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"kn", "[", "s", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}], ",", 
   RowBox[{"PlotStyle", " ", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{"Red", ",", "Thick"}], "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6263335604023285`*^9, 3.6263335987627954`*^9}, {
   3.626418774282587*^9, 3.6264187957482243`*^9}, 3.626419000919785*^9, {
   3.627112948882249*^9, 3.627112971595889*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"kg", "[", "s_", "]"}], ":=", 
  RowBox[{
   RowBox[{"geodcurvature", "[", 
    RowBox[{
     RowBox[{"sphero", "[", "1", "]"}], ",", "hu", ",", "hv"}], "]"}], "[", 
   "s", "]"}]}]], "Input",
 CellChangeTimes->{{3.6262723065954123`*^9, 3.626272358893704*^9}, 
   3.626333502151826*^9, {3.626436708660736*^9, 3.6264367142455463`*^9}, {
   3.62658899833916*^9, 3.6265890019271665`*^9}, {3.626589037354829*^9, 
   3.626589062626873*^9}, {3.627112263136245*^9, 3.6271122711078587`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kg", "[", "s", "]"}]], "Input",
 CellChangeTimes->{{3.6264191133335824`*^9, 3.626419169852482*^9}, {
  3.6264367308595753`*^9, 3.62643677326045*^9}, {3.6265890765576973`*^9, 
  3.626589082017707*^9}}],

Cell[BoxData[
 RowBox[{"\[Sqrt]", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"Cos", "[", 
           RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Sin", "[", "s", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                RowBox[{"Sin", "[", "s", "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "3"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"2", " ", 
                   RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{"Sin", "[", "s", "]"}], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "-", 
                  RowBox[{"2", " ", 
                   SuperscriptBox[
                    RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                   RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                   RowBox[{"Sin", "[", "s", "]"}], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "-", 
                  RowBox[{"2", " ", 
                   RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", "s", "]"}], "3"], " ", 
                   RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{"Cos", "[", "s", "]"}], " ", 
                RowBox[{"Sin", "[", "s", "]"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}]}], 
                 ")"}]}]}], ")"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
                 SuperscriptBox[
                  RowBox[{"Sin", "[", "s", "]"}], "2"]}], "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}]}], 
                  ")"}]}]}], ")"}], 
              RowBox[{"3", "/", "2"}]]}], ")"}]}], "-", 
          RowBox[{
           RowBox[{"Cos", "[", "s", "]"}], "/", 
           RowBox[{"(", 
            RowBox[{"\[Sqrt]", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                   SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}]}], 
                 ")"}]}]}], ")"}]}], ")"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "2"]}]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}]}], ")"}], 
           "/", 
           RowBox[{"(", 
            RowBox[{"2", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "4"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{"2", " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], " ", 
             RowBox[{"\[Sqrt]", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
                 SuperscriptBox[
                  RowBox[{"Sin", "[", "s", "]"}], "2"]}], "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}]}], 
                  ")"}]}]}], ")"}]}]}], ")"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "2"]}]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "2"}], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
               RowBox[{"2", " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}]}], ")"}], 
           "/", 
           RowBox[{"(", 
            RowBox[{"2", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "4"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{"2", " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], " ", 
             RowBox[{"\[Sqrt]", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
                 SuperscriptBox[
                  RowBox[{"Sin", "[", "s", "]"}], "2"]}], "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}]}], 
                  ")"}]}]}], ")"}]}]}], ")"}]}]}], ")"}], "2"]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", 
            RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", "s", "]"}], "2"]}]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
               RowBox[{"Sin", "[", "s", "]"}], " ", 
               RowBox[{"Sin", "[", 
                RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
              RowBox[{"2", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
               SuperscriptBox[
                RowBox[{"Sin", "[", "s", "]"}], "3"], " ", 
               RowBox[{"Sin", "[", 
                RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "+", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"2", " ", 
                  RowBox[{"Cos", "[", 
                   RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                  RowBox[{"Sin", "[", "s", "]"}], " ", 
                  RowBox[{"Sin", "[", 
                   RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "-", 
                 RowBox[{"2", " ", 
                  SuperscriptBox[
                   RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                  RowBox[{"Cos", "[", 
                   RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                  RowBox[{"Sin", "[", "s", "]"}], " ", 
                  RowBox[{"Sin", "[", 
                   RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "-", 
                 RowBox[{"2", " ", 
                  RowBox[{"Cos", "[", 
                   RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                  SuperscriptBox[
                   RowBox[{"Sin", "[", "s", "]"}], "3"], " ", 
                  RowBox[{"Sin", "[", 
                   RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}]}], "+", 
              RowBox[{"2", " ", 
               RowBox[{"Cos", "[", "s", "]"}], " ", 
               RowBox[{"Sin", "[", "s", "]"}], " ", 
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                  SuperscriptBox[
                   RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                  SuperscriptBox[
                   RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}]}], 
                ")"}]}]}], ")"}], "/", 
            RowBox[{"(", 
             RowBox[{"2", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                  SuperscriptBox[
                   RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
                  SuperscriptBox[
                   RowBox[{"Sin", "[", "s", "]"}], "2"]}], "+", 
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                  RowBox[{"(", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], "+", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}]}], 
                   ")"}]}]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}], ")"}]}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Sin", "[", "s", "]"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "2"}], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}]}], "-", 
               RowBox[{"2", " ", 
                RowBox[{"Cos", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                RowBox[{"Sin", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}]}]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"Cos", "[", 
                 RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}]}], ")"}]}], 
            ")"}], "/", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "4"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{"2", " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"Cos", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", "s", "]"}], "4"], " ", 
                SuperscriptBox[
                 RowBox[{"Sin", "[", 
                  RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}]}], ")"}], " ", 
             RowBox[{"\[Sqrt]", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Cos", "[", 
                   RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
                 SuperscriptBox[
                  RowBox[{"Sin", "[", "s", "]"}], "2"]}], "+", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                 RowBox[{"(", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"Cos", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
                   RowBox[{
                    SuperscriptBox[
                    RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"Sin", "[", 
                    RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}]}], 
                  ")"}]}]}], ")"}]}]}], ")"}]}]}], ")"}], "2"]}]}], ")"}], 
    "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", 
         RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Cos", "[", 
         RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], " ", 
       SuperscriptBox[
        RowBox[{"Sin", "[", "s", "]"}], "2"]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"Cos", "[", 
           RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", "s", "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Sin", "[", "s", "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", 
            RowBox[{"Cos", "[", "s", "]"}], "]"}], "2"]}]}], ")"}]}]}], 
     ")"}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.626589083281309*^9, 3.6271122790794725`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"kg", "[", "s", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6263335604023285`*^9, 3.6263335987627954`*^9}, {
   3.626418774282587*^9, 3.6264187957482243`*^9}, 3.626419000919785*^9, 
   3.6264192312697897`*^9, 3.626596579512866*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Together", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"kk", "[", "s", "]"}], "^", "2"}], " ", "-", 
     RowBox[{
      RowBox[{"kn", "[", "s", "]"}], "^", "2"}], "-", 
     RowBox[{
      RowBox[{"kg", "[", "s", "]"}], "^", "2"}]}], "]"}], ",", 
   RowBox[{"s", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6262723763989353`*^9, 3.6262724099117937`*^9}, {
  3.6262750511094522`*^9, 3.6262750574430637`*^9}, {3.6264192519866257`*^9, 
  3.6264192857918854`*^9}, {3.626436937965539*^9, 3.6264369527387652`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6262725949803205`*^9, 3.626276004736532*^9, 
  3.626420976616455*^9, 3.626436929947125*^9, 3.6264371868795767`*^9, 
  3.6264396302806683`*^9, 3.6265893053788996`*^9, 3.626604323852169*^9, 
  3.6271122973783045`*^9, 3.6271127845827603`*^9}]
}, Closed]],

Cell["OK!", "Text",
 CellChangeTimes->{{3.6271128414760604`*^9, 3.627112843238864*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["4.3.6. Further Examples", "Subsubsection",
 CellChangeTimes->{{3.626600317378045*^9, 3.626600325427659*^9}, {
   3.6266046150378838`*^9, 3.6266046193122916`*^9}, {3.626848682265691*^9, 
   3.626848696992117*^9}, 3.6268543822492666`*^9}],

Cell[TextData[{
 StyleBox["Example",
  FontColor->RGBColor[0, 0, 1]],
 ". Graphs of Functions"
}], "Text",
 CellChangeTimes->{{3.6136216629488277`*^9, 3.6136217472669764`*^9}, {
   3.6136219986458178`*^9, 3.6136220101274376`*^9}, {3.6136316409915533`*^9, 
   3.6136316447511597`*^9}, {3.613631706792469*^9, 3.6136317217996955`*^9}, {
   3.613631784777006*^9, 3.6136317936534214`*^9}, 3.6136319442872863`*^9, {
   3.6142629800161095`*^9, 3.6142629821221128`*^9}, 3.6143299951743555`*^9}],

Cell["\<\
We consider the graph of an unspecified function f[u,v]. In this case we \
obtain general formulas for all kind of curvatures of a general curve on the \
surface.\
\>", "Text",
 CellChangeTimes->{{3.613632210517354*^9, 3.6136322491742215`*^9}, {
  3.614329894398178*^9, 3.614329951790679*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"a", ",", "hu", ",", "hv", ",", "sx"}], "]"}], ";", 
  RowBox[{
   RowBox[{"hu", "[", "s_", "]"}], " ", "=", " ", "s"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"hv", "[", "a_", "]"}], "[", "s_", "]"}], ":=", " ", "a"}], ";", 
  RowBox[{
   RowBox[{"sx", "[", "s_", "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{"graph", "[", "f", "]"}], "[", 
    RowBox[{
     RowBox[{"hu", "[", "s", "]"}], ",", 
     RowBox[{
      RowBox[{"hv", "[", "a", "]"}], "[", "s", "]"}]}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6143294721366367`*^9, 3.614329624065303*^9}, {
   3.614494308370402*^9, 3.6144943725460725`*^9}, {3.614505389490672*^9, 
   3.614505523245307*^9}, {3.6145055567073655`*^9, 3.614505570107789*^9}, 
   3.6148618633255463`*^9, 3.6148631367245827`*^9, 3.6189119646725826`*^9, {
   3.6189120064650564`*^9, 3.6189120185394773`*^9}, {3.618912065011959*^9, 
   3.6189120878035994`*^9}, {3.618912435263009*^9, 3.6189124443734255`*^9}, {
   3.6189124810178895`*^9, 3.6189125661316395`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kk", " ", "=", " ", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"curvature", "[", "sx", "]"}], "[", "s", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6189121673325386`*^9, 3.618912199640196*^9}, 
   3.618912235660659*^9, 3.6189122782331333`*^9, {3.61891238421972*^9, 
   3.618912384906121*^9}, {3.618912595880892*^9, 3.6189125967232933`*^9}}],

Cell[BoxData[
 FractionBox[
  SqrtBox[
   FractionBox[
    SuperscriptBox[
     RowBox[{
      SuperscriptBox["f", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"2", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"s", ",", "a"}], "]"}], "2"], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["f", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"s", ",", "a"}], "]"}], "2"]}], ")"}], "2"]]], 
  SqrtBox[
   RowBox[{"1", "+", 
    SuperscriptBox[
     RowBox[{
      SuperscriptBox["f", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"s", ",", "a"}], "]"}], "2"]}]]]], "Output",
 CellChangeTimes->{3.6262613499040017`*^9, 3.6264219890582333`*^9, 
  3.6268487402665925`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kg", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"geodcurvature", "[", 
     RowBox[{
      RowBox[{"graph", "[", "f", "]"}], ",", "hu", ",", 
      RowBox[{"hv", "[", "a", "]"}]}], "]"}], "[", "s", "]"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6143294721366367`*^9, 3.614329624065303*^9}, {
   3.614494308370402*^9, 3.6144943725460725`*^9}, {3.614505389490672*^9, 
   3.614505523245307*^9}, {3.6145055567073655`*^9, 3.614505570107789*^9}, 
   3.6148618633255463`*^9, 3.6148631367245827`*^9, 3.6189119646725826`*^9, {
   3.618912026261491*^9, 3.618912029865097*^9}}],

Cell[BoxData[
 SqrtBox[
  FractionBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{
      SuperscriptBox["f", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"s", ",", "a"}], "]"}], "2"], " ", 
    SuperscriptBox[
     RowBox[{
      SuperscriptBox["f", 
       TagBox[
        RowBox[{"(", 
         RowBox[{"2", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"s", ",", "a"}], "]"}], "2"]}], 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["f", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"s", ",", "a"}], "]"}], "2"]}], ")"}], "3"], " ", 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"s", ",", "a"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["f", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"s", ",", "a"}], "]"}], "2"]}], ")"}]}]]]], "Output",
 CellChangeTimes->{3.626261368187234*^9, 3.626421994377843*^9, 
  3.626848747333405*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kn", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"normalcurvature", "[", 
     RowBox[{
      RowBox[{"graph", "[", "f", "]"}], ",", "hu", ",", 
      RowBox[{"hv", "[", "a", "]"}]}], "]"}], "[", "s", "]"}], " ", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.613621793911058*^9, 3.61362190860246*^9}, 
   3.6136224537922173`*^9, {3.6136225031195035`*^9, 3.613622523134339*^9}, {
   3.6136319875929623`*^9, 3.613631994222974*^9}, 3.61432944564779*^9, {
   3.614329681629404*^9, 3.6143297762279706`*^9}, {3.6145055851462154`*^9, 
   3.6145056105742607`*^9}, 3.6148618633255463`*^9, 3.6148631367245827`*^9, {
   3.6189121285040703`*^9, 3.618912134603681*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   SuperscriptBox["f", 
    TagBox[
     RowBox[{"(", 
      RowBox[{"2", ",", "0"}], ")"}],
     Derivative],
    MultilineFunction->None], "[", 
   RowBox[{"s", ",", "a"}], "]"}], 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "+", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["f", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"s", ",", "a"}], "]"}], "2"]}], ")"}], " ", 
   SqrtBox[
    RowBox[{"1", "+", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["f", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"s", ",", "a"}], "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox["f", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"s", ",", "a"}], "]"}], "2"]}]]}]]], "Output",
 CellChangeTimes->{3.626261373319643*^9, 3.62642199832465*^9, 
  3.626848769781844*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Together", "[", 
   RowBox[{
    RowBox[{"kk", "^", "2"}], "-", 
    RowBox[{"kn", "^", "2"}], "-", 
    RowBox[{"kg", "^", "2"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6145056477179255`*^9, 3.6145057008360186`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6262613787952523`*^9, 3.626422003051458*^9, 
  3.626848773588251*^9}]
}, Closed]],

Cell["The same for the specified function f3:", "Text",
 CellChangeTimes->{{3.6189278059856067`*^9, 3.6189278320064526`*^9}, 
   3.6189955701473665`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"f3", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"u", "^", "3"}], "+", 
   RowBox[{"v", "^", "3"}]}]}]], "Input",
 CellChangeTimes->{{3.618918953500458*^9, 3.6189189761204977`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"a", ",", "hu", ",", "hv", ",", "sx"}], "]"}], ";", 
  RowBox[{
   RowBox[{"hu", "[", "s_", "]"}], " ", "=", " ", "s"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"hv", "[", "a_", "]"}], "[", "s_", "]"}], ":=", " ", "a"}], ";", 
  RowBox[{
   RowBox[{"sx", "[", "s_", "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{"graph", "[", "f3", "]"}], "[", 
    RowBox[{
     RowBox[{"hu", "[", "s", "]"}], ",", 
     RowBox[{
      RowBox[{"hv", "[", "a", "]"}], "[", "s", "]"}]}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6143294721366367`*^9, 3.614329624065303*^9}, {
   3.614494308370402*^9, 3.6144943725460725`*^9}, {3.614505389490672*^9, 
   3.614505523245307*^9}, {3.6145055567073655`*^9, 3.614505570107789*^9}, 
   3.6148618633255463`*^9, 3.6148631367245827`*^9, 3.6189119646725826`*^9, {
   3.6189120064650564`*^9, 3.6189120185394773`*^9}, {3.618912065011959*^9, 
   3.6189120878035994`*^9}, {3.618912435263009*^9, 3.6189124443734255`*^9}, {
   3.6189124810178895`*^9, 3.6189125661316395`*^9}, {3.6189131345654373`*^9, 
   3.6189131457974577`*^9}, 3.6189190083501544`*^9, {3.6189274238941355`*^9, 
   3.618927442770169*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"graph", "[", "f3", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6189190252917843`*^9, 3.618919153118409*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kk", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"curvature", "[", "sx", "]"}], "[", "s", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.618927532517126*^9, 3.6189275639511814`*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{"6", " ", 
   SqrtBox[
    FractionBox[
     SuperscriptBox["s", "2"], 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{"9", " ", 
         SuperscriptBox["s", "4"]}]}], ")"}], "2"]]]}], 
  SqrtBox[
   RowBox[{"1", "+", 
    RowBox[{"9", " ", 
     SuperscriptBox["s", "4"]}]}]]]], "Output",
 CellChangeTimes->{3.626261436640154*^9, 3.626848832119554*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kg", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"geodcurvature", "[", 
     RowBox[{
      RowBox[{"graph", "[", "f3", "]"}], ",", "hu", ",", 
      RowBox[{"hv", "[", "a", "]"}]}], "]"}], "[", "s", "]"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6143294721366367`*^9, 3.614329624065303*^9}, {
   3.614494308370402*^9, 3.6144943725460725`*^9}, {3.614505389490672*^9, 
   3.614505523245307*^9}, {3.6145055567073655`*^9, 3.614505570107789*^9}, 
   3.6148618633255463`*^9, 3.6148631367245827`*^9, 3.6189119646725826`*^9, {
   3.618912026261491*^9, 3.618912029865097*^9}, 3.618913684263603*^9, {
   3.618916542438223*^9, 3.6189165488186345`*^9}, {3.6189198543552403`*^9, 
   3.618919862155254*^9}, 3.6189275059502797`*^9, {3.6189276139960694`*^9, 
   3.6189276170536747`*^9}}],

Cell[BoxData[
 RowBox[{"18", " ", 
  SqrtBox[
   FractionBox[
    RowBox[{
     SuperscriptBox["a", "4"], " ", 
     SuperscriptBox["s", "2"]}], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{"9", " ", 
         SuperscriptBox["s", "4"]}]}], ")"}], "3"], " ", 
     RowBox[{"(", 
      RowBox[{"1", "+", 
       RowBox[{"9", " ", 
        SuperscriptBox["a", "4"]}], "+", 
       RowBox[{"9", " ", 
        SuperscriptBox["s", "4"]}]}], ")"}]}]]]}]], "Output",
 CellChangeTimes->{3.6262614426149645`*^9, 3.626848835957161*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kn", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"normalcurvature", "[", 
     RowBox[{
      RowBox[{"graph", "[", "f3", "]"}], ",", "hu", ",", 
      RowBox[{"hv", "[", "a", "]"}]}], "]"}], "[", "s", "]"}], " ", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.613621793911058*^9, 3.61362190860246*^9}, 
   3.6136224537922173`*^9, {3.6136225031195035`*^9, 3.613622523134339*^9}, {
   3.6136319875929623`*^9, 3.613631994222974*^9}, 3.61432944564779*^9, {
   3.614329681629404*^9, 3.6143297762279706`*^9}, {3.6145055851462154`*^9, 
   3.6145056105742607`*^9}, 3.6148618633255463`*^9, 3.6148631367245827`*^9, {
   3.6189121285040703`*^9, 3.618912134603681*^9}, 3.6189276436205215`*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{"6", " ", "s"}], 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "+", 
     RowBox[{"9", " ", 
      SuperscriptBox["s", "4"]}]}], ")"}], " ", 
   SqrtBox[
    RowBox[{"1", "+", 
     RowBox[{"9", " ", 
      SuperscriptBox["a", "4"]}], "+", 
     RowBox[{"9", " ", 
      SuperscriptBox["s", "4"]}]}]]}]]], "Output",
 CellChangeTimes->{3.6262614508985786`*^9, 3.626848840169168*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Together", "[", 
   RowBox[{
    RowBox[{"kk", "^", "2"}], "-", 
    RowBox[{"kn", "^", "2"}], "-", 
    RowBox[{"kg", "^", "2"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6145056477179255`*^9, 3.6145057008360186`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.626261455016986*^9, 3.626848842852373*^9}]
}, Closed]],

Cell["OK.", "Text",
 CellChangeTimes->{{3.6145057514113073`*^9, 3.6145057689301386`*^9}}],

Cell[TextData[{
 StyleBox["Example",
  FontColor->RGBColor[0, 0, 1]],
 ". Helix Normal Surface"
}], "Text",
 CellChangeTimes->{{3.6136216629488277`*^9, 3.6136217472669764`*^9}, {
   3.6136219986458178`*^9, 3.6136220101274376`*^9}, {3.6136316409915533`*^9, 
   3.6136316447511597`*^9}, {3.613631706792469*^9, 3.6136317217996955`*^9}, {
   3.613631784777006*^9, 3.6136317936534214`*^9}, 3.6136319442872863`*^9, {
   3.613977412358617*^9, 3.6139774462886763`*^9}}],

Cell[TextData[{
 "We consider a normal surface of the helix, see Subsection ",
 ButtonBox["3.7.2",
  BaseStyle->"Hyperlink",
  ButtonData->"helix"],
 "."
}], "Text",
 CellChangeTimes->{{3.614330152547432*^9, 3.614330182062683*^9}, {
  3.6143302386127825`*^9, 3.614330280420856*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hel", "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.6147509470177927`*^9, 3.614750947048992*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{"Cos", "[", "t", "]"}], 
    SqrtBox["2"]], ",", 
   FractionBox[
    RowBox[{"Sin", "[", "t", "]"}], 
    SqrtBox["2"]], ",", 
   FractionBox["t", 
    SqrtBox["2"]]}], "}"}]], "Output",
 CellChangeTimes->{3.6264220279179015`*^9, 3.62685207513125*^9}]
}, Closed]],

Cell["The surface generated by the binormals:", "Text",
 CellChangeTimes->{{3.618981650960631*^9, 3.6189816787130795`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"cb", "[", "t_", "]"}], " ", ":=", " ", 
  RowBox[{"Pi", "/", "2"}]}]], "Input",
 CellChangeTimes->{{3.613977464712309*^9, 3.6139774850859447`*^9}, {
  3.613994501006675*^9, 3.613994505015882*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"hnsf", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"helixnormalcongr", "[", "cb", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
    RowBox[{"v", "\[Element]", "Reals"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6139774919343567`*^9, 3.613977501715574*^9}, {
   3.61397753935844*^9, 3.613977571853297*^9}, {3.6139776021953506`*^9, 
   3.6139776329898043`*^9}, {3.61433035108898*^9, 3.614330391461851*^9}, 
   3.6290988682429657`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"Cos", "[", "v", "]"}], "+", 
     RowBox[{"u", " ", 
      RowBox[{"Sin", "[", "v", "]"}]}]}], 
    SqrtBox["2"]], ",", 
   FractionBox[
    RowBox[{
     RowBox[{
      RowBox[{"-", "u"}], " ", 
      RowBox[{"Cos", "[", "v", "]"}]}], "+", 
     RowBox[{"Sin", "[", "v", "]"}]}], 
    SqrtBox["2"]], ",", 
   FractionBox[
    RowBox[{"u", "+", "v"}], 
    SqrtBox["2"]]}], "}"}]], "Output",
 CellChangeTimes->{3.626261499289864*^9, 3.6262615833272114`*^9, 
  3.626422069195574*^9, 3.6268520941320834`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"hnsf", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "3"}], ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.613977668760667*^9, 3.613977729663174*^9}, {
  3.614330411133486*^9, 3.6143304144874916`*^9}, {3.6143304498527536`*^9, 
  3.61433045337836*^9}, {3.6143307201856284`*^9, 3.614330757391694*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"hel1", " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.6143305134852657`*^9, 3.614330520380478*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"hel", "[", "t", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"PlotStyle", " ", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Thick", ",", " ", "Red"}], "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6143305234224825`*^9, 3.6143305311132965`*^9}, {
   3.6143306628399277`*^9, 3.614330695381585*^9}, {3.6143307681869125`*^9, 
   3.6143307872501464`*^9}, {3.6143308932991323`*^9, 3.6143309148271704`*^9}, 
   3.6147509693882313`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"hel2", " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.61433080043217*^9, 3.614330813395792*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"hel1", ",", "hel2"}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6143308164845977`*^9, 3.614330844439847*^9}}],

Cell["\<\
We consider  hnsf. The helix itself is a curve on the surface\
\>", "Text",
 CellChangeTimes->{{3.613632210517354*^9, 3.6136322491742215`*^9}, {
  3.613977780737664*^9, 3.613977788802878*^9}, {3.6143310270537677`*^9, 
  3.6143310468814025`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "sx", "]"}], ";", 
  RowBox[{
   RowBox[{"sx", "[", "s_", "]"}], " ", ":=", " ", 
   RowBox[{"hnsf", "[", 
    RowBox[{"0", ",", "s"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.613621793911058*^9, 3.61362190860246*^9}, 
   3.6136224537922173`*^9, {3.6136225031195035`*^9, 3.613622523134339*^9}, {
   3.6136319875929623`*^9, 3.613631994222974*^9}, {3.6139778317185535`*^9, 
   3.6139778583322*^9}, {3.6140535554544344`*^9, 3.614053565142051*^9}, 
   3.614057854151184*^9, {3.6140579446937437`*^9, 3.614057962664975*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sx", "[", "t", "]"}]], "Input",
 CellChangeTimes->{
  3.6140536423777866`*^9, 3.6140578620915985`*^9, {3.6140579046484733`*^9, 
   3.614057915490492*^9}, {3.614057981244608*^9, 3.6140579829762106`*^9}, 
   3.61450612840157*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{"Cos", "[", "t", "]"}], 
    SqrtBox["2"]], ",", 
   FractionBox[
    RowBox[{"Sin", "[", "t", "]"}], 
    SqrtBox["2"]], ",", 
   FractionBox["t", 
    SqrtBox["2"]]}], "}"}]], "Output",
 CellChangeTimes->{3.6262616368353057`*^9, 3.626422115808456*^9, 
  3.6268521420397673`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"curvature", "[", "sx", "]"}], "[", "t", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.613621793911058*^9, 3.61362190860246*^9}, 
   3.6136224537922173`*^9, {3.6136225031195035`*^9, 3.613622523134339*^9}, {
   3.6136319875929623`*^9, 3.613631994222974*^9}, {3.6139778317185535`*^9, 
   3.6139778583322*^9}, {3.613977953227167*^9, 3.613977974224804*^9}, 
   3.61399468710133*^9, {3.6140012428100443`*^9, 3.614001260500475*^9}, 
   3.614003200051882*^9, {3.614053692017074*^9, 3.6140537100195055`*^9}, 
   3.6143312564053707`*^9, 3.614506139196789*^9, 3.614861863387946*^9, 
   3.614863136740183*^9}],

Cell[BoxData[
 FractionBox["1", 
  SqrtBox["2"]]], "Output",
 CellChangeTimes->{3.626261642373315*^9, 3.6264221201764636`*^9, 
  3.626852145331373*^9}]
}, Closed]],

Cell["It is a geodesic on the surface:", "Text",
 CellChangeTimes->{{3.614331293034235*^9, 3.614331320037882*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"hu", ",", "hv"}], "]"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"hu", "[", "a_", "]"}], "[", "s_", "]"}], ":=", " ", "a"}], ";", 
  RowBox[{
   RowBox[{"hv", "[", "s_", "]"}], ":=", " ", "s"}], ";"}]], "Input",
 CellChangeTimes->{{3.599029134745389*^9, 3.599029405593065*^9}, 
   3.613476504428583*^9, 3.6136213128686132`*^9, {3.6136220816691637`*^9, 
   3.6136221609485025`*^9}, 3.6136226985098467`*^9, {3.6136230453764563`*^9, 
   3.613623055532074*^9}, {3.613632058448287*^9, 3.613632088088339*^9}, {
   3.613981056689015*^9, 3.6139810903850737`*^9}, {3.6140538078940773`*^9, 
   3.6140538380645304`*^9}, {3.614053909044655*^9, 3.6140539372963047`*^9}, {
   3.614057401001589*^9, 3.6140574170228167`*^9}, {3.6140574584564896`*^9, 
   3.614057531854618*^9}, {3.6140577454501934`*^9, 3.6140577665102305`*^9}, {
   3.614058011789461*^9, 3.6140580637219524`*^9}, {3.614331077410656*^9, 
   3.614331090155878*^9}, {3.6143311369871607`*^9, 3.6143311824612403`*^9}, 
   3.614861863387946*^9, 3.614863136740183*^9, {3.6189817649032307`*^9, 
   3.618981788786873*^9}, {3.6189818891262493`*^9, 3.618981900436269*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"geodcurvature", "[", 
     RowBox[{"hnsf", ",", 
      RowBox[{"hu", "[", "a", "]"}], ",", "hv"}], "]"}], "[", "t", "]"}], 
   "]"}], "/.", 
  RowBox[{"a", "\[Rule]", " ", "0"}]}]], "Input",
 CellChangeTimes->{{3.599029134745389*^9, 3.599029405593065*^9}, 
   3.613476504428583*^9, 3.6136213128686132`*^9, {3.6136220816691637`*^9, 
   3.6136221609485025`*^9}, 3.6136226985098467`*^9, {3.6136230453764563`*^9, 
   3.613623055532074*^9}, {3.613632058448287*^9, 3.613632088088339*^9}, {
   3.613981056689015*^9, 3.6139810903850737`*^9}, {3.6140538078940773`*^9, 
   3.6140538380645304`*^9}, {3.614053909044655*^9, 3.6140539372963047`*^9}, {
   3.614057401001589*^9, 3.6140574170228167`*^9}, {3.6140574584564896`*^9, 
   3.614057531854618*^9}, {3.6140577454501934`*^9, 3.6140577665102305`*^9}, {
   3.614058011789461*^9, 3.6140580637219524`*^9}, {3.614331077410656*^9, 
   3.614331090155878*^9}, {3.6143311369871607`*^9, 3.6143311824612403`*^9}, 
   3.614861863387946*^9, 3.614863136740183*^9, {3.6189817649032307`*^9, 
   3.618981846787775*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6262616664129577`*^9, 3.6264221325784855`*^9, 
  3.6268521546445894`*^9}]
}, Closed]],

Cell["Its normal curvature coincides with its curvature:", "Text",
 CellChangeTimes->{{3.614331335575509*^9, 3.61433136426396*^9}, 
   3.6174418910598736`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"normalcurvature", "[", 
     RowBox[{"hnsf", ",", 
      RowBox[{"hu", "[", "a", "]"}], ",", "hv"}], "]"}], "[", "t", "]"}], 
   "]"}], "/.", " ", 
  RowBox[{"a", "\[Rule]", " ", "0"}]}]], "Input",
 CellChangeTimes->{{3.599029134745389*^9, 3.599029405593065*^9}, 
   3.613476504428583*^9, 3.6136213128686132`*^9, {3.6136220816691637`*^9, 
   3.6136221609485025`*^9}, 3.6136226985098467`*^9, {3.6136230453764563`*^9, 
   3.613623055532074*^9}, {3.613632058448287*^9, 3.613632088088339*^9}, {
   3.613981056689015*^9, 3.6139810903850737`*^9}, {3.6140538078940773`*^9, 
   3.6140538380645304`*^9}, {3.614053909044655*^9, 3.6140539372963047`*^9}, {
   3.614057401001589*^9, 3.6140574170228167`*^9}, {3.6140574584564896`*^9, 
   3.614057531854618*^9}, {3.6140577454501934`*^9, 3.6140577665102305`*^9}, {
   3.614058011789461*^9, 3.6140580637219524`*^9}, {3.614331077410656*^9, 
   3.614331090155878*^9}, {3.6143311369871607`*^9, 3.614331232490528*^9}, 
   3.6148618634035463`*^9, 3.614863136740183*^9, {3.6189819294679203`*^9, 
   3.618981931745524*^9}, {3.618982920522061*^9, 3.618982951940516*^9}}],

Cell[BoxData[
 FractionBox["1", 
  SqrtBox["2"]]], "Output",
 CellChangeTimes->{3.6262616992198153`*^9, 3.626422142110102*^9, 
  3.6268521600577993`*^9}]
}, Closed]],

Cell["Clearly, the generating lines are geodesics too:", "Text",
 CellChangeTimes->{{3.614331549857486*^9, 3.6143315806051397`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hnsf", "[", 
  RowBox[{"s", ",", "ca"}], "]"}]], "Input",
 CellChangeTimes->{{3.6143314959437914`*^9, 3.6143315099214153`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"Cos", "[", "ca", "]"}], "+", 
     RowBox[{"s", " ", 
      RowBox[{"Sin", "[", "ca", "]"}]}]}], 
    SqrtBox["2"]], ",", 
   FractionBox[
    RowBox[{
     RowBox[{
      RowBox[{"-", "s"}], " ", 
      RowBox[{"Cos", "[", "ca", "]"}]}], "+", 
     RowBox[{"Sin", "[", "ca", "]"}]}], 
    SqrtBox["2"]], ",", 
   FractionBox[
    RowBox[{"ca", "+", "s"}], 
    SqrtBox["2"]]}], "}"}]], "Output",
 CellChangeTimes->{3.62626171371224*^9, 3.6264221508617177`*^9, 
  3.6268521654866085`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"hu", "[", "s_", "]"}], ":=", " ", "s"}], ";", 
  RowBox[{
   RowBox[{"hv", "[", "s_", "]"}], ":=", " ", "ca"}], ";", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"geodcurvature", "[", 
     RowBox[{"hnsf", ",", "hu", ",", "hv"}], "]"}], "[", "t", "]"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.599029134745389*^9, 3.599029405593065*^9}, 
   3.613476504428583*^9, 3.6136213128686132`*^9, {3.6136220816691637`*^9, 
   3.6136221609485025`*^9}, 3.6136226985098467`*^9, {3.6136230453764563`*^9, 
   3.613623055532074*^9}, {3.613632058448287*^9, 3.613632088088339*^9}, {
   3.613981056689015*^9, 3.6139810903850737`*^9}, {3.6140538078940773`*^9, 
   3.6140538380645304`*^9}, {3.614053909044655*^9, 3.6140539372963047`*^9}, {
   3.614057401001589*^9, 3.6140574170228167`*^9}, {3.6140574584564896`*^9, 
   3.614057531854618*^9}, {3.6140577454501934`*^9, 3.6140577665102305`*^9}, {
   3.614058011789461*^9, 3.6140580637219524`*^9}, {3.614331077410656*^9, 
   3.614331090155878*^9}, {3.6143311369871607`*^9, 3.6143311824612403`*^9}, {
   3.6143314227796626`*^9, 3.614331442981698*^9}, 3.6148618634191465`*^9, 
   3.614863136740183*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6262617234622574`*^9, 3.6264221543717237`*^9, 
  3.626852180462635*^9}]
}, Closed]],

Cell["For the parameter curves u = u0 we calculate", "Text",
 CellChangeTimes->{{3.618983158547279*^9, 3.618983177438912*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sx", "[", "s_", "]"}], ":=", 
  RowBox[{"hnsf", "[", 
   RowBox[{"uo", ",", "s"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.614331807290964*^9, 3.614331815195416*^9}, {
   3.6145062733882246`*^9, 3.614506286851048*^9}, {3.6145063753032036`*^9, 
   3.614506378017608*^9}, 3.614506412150468*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sc", "[", "s", "]"}]], "Input",
 CellChangeTimes->{{3.6189830814987435`*^9, 3.618983098299973*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   SuperscriptBox["s", "2"], ",", 
   SuperscriptBox["s", "3"]}], "}"}]], "Output",
 CellChangeTimes->{3.626261739545886*^9, 3.62642216359134*^9, 
  3.6268521914762545`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kk", " ", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"curvature", "[", "sx", "]"}], "[", "s", "]"}], ",", 
    RowBox[{"s", "\[Element]", "Reals"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6145063038550777`*^9, 3.614506385412021*^9}, 
   3.614506423351288*^9, 3.6148618634347467`*^9, 3.614863136740183*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   SqrtBox["2"], " ", 
   SqrtBox[
    RowBox[{"1", "+", 
     SuperscriptBox["uo", "2"]}]]}], 
  RowBox[{"2", "+", 
   SuperscriptBox["uo", "2"]}]]], "Output",
 CellChangeTimes->{3.626261769154738*^9, 3.6264221705177517`*^9, 
  3.6268521964526634`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "torsion"}]], "Input",
 CellChangeTimes->{{3.6143322148102727`*^9, 3.614332225668894*^9}}],

Cell[BoxData[
 StyleBox["\<\"torsion[curve][t] yields the torsion of the curve[t] in \\nthe \
Euclidean 3-space; it is defined only at points with non vanishing curvature.\
\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.626261783397563*^9},
 CellTags->"Info3626265383-1010981"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"torsion", "[", "sx", "]"}], "[", "s", "]"}], ",", 
   RowBox[{"s", "\[Element]", "Reals"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6145063038550777`*^9, 3.614506385412021*^9}, 
   3.614506423351288*^9, {3.614506500259423*^9, 3.614506531506278*^9}}],

Cell[BoxData[
 FractionBox[
  SqrtBox["2"], 
  RowBox[{"2", "+", 
   SuperscriptBox["uo", "2"]}]]], "Output",
 CellChangeTimes->{3.6264221821397724`*^9, 3.6268522064522805`*^9}]
}, Closed]],

Cell["\<\
It follows: The parameter curves u= const. are helices again, its curvature \
and torsion depend on uo.\
\>", "Text",
 CellChangeTimes->{{3.6143322874654284`*^9, 3.614332359217532*^9}, {
  3.614506634216858*^9, 3.6145066625777082`*^9}, {3.618983208092966*^9, 
  3.6189832084049664`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"hnsf", "[", 
    RowBox[{"1", ",", "t"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", " ", 
   RowBox[{"PlotStyle", " ", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{"Thick", ",", " ", "Green"}], "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6143317451024065`*^9, 3.614331772987002*^9}, {
  3.6143318641392155`*^9, 3.614331897460121*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"hel3", " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.6143319076417036`*^9, 3.614331921147476*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"hel1", ",", "hel2", ",", "hel3"}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6143308164845977`*^9, 3.614330844439847*^9}, {
  3.614331956171479*^9, 3.6143319583726053`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"hu", "[", "s_", "]"}], ":=", " ", "uo"}], ";", 
  RowBox[{
   RowBox[{"hv", "[", "s_", "]"}], ":=", " ", "s"}], ";", 
  RowBox[{"kg", " ", "=", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"geodcurvature", "[", 
       RowBox[{"hnsf", ",", "hu", ",", "hv"}], "]"}], "[", "s", "]"}], ",", 
     RowBox[{"s", "\[Element]", "Reals"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.599029134745389*^9, 3.599029405593065*^9}, 
   3.613476504428583*^9, 3.6136213128686132`*^9, {3.6136220816691637`*^9, 
   3.6136221609485025`*^9}, 3.6136226985098467`*^9, {3.6136230453764563`*^9, 
   3.613623055532074*^9}, {3.613632058448287*^9, 3.613632088088339*^9}, {
   3.613981056689015*^9, 3.6139810903850737`*^9}, {3.6140538078940773`*^9, 
   3.6140538380645304`*^9}, {3.614053909044655*^9, 3.6140539372963047`*^9}, {
   3.614057401001589*^9, 3.6140574170228167`*^9}, {3.6140574584564896`*^9, 
   3.614057531854618*^9}, {3.6140577454501934`*^9, 3.6140577665102305`*^9}, {
   3.614058011789461*^9, 3.6140580637219524`*^9}, {3.614331077410656*^9, 
   3.614331090155878*^9}, {3.6143311369871607`*^9, 3.6143311824612403`*^9}, {
   3.6143314227796626`*^9, 3.614331442981698*^9}, {3.6143316492429237`*^9, 
   3.6143316728042717`*^9}, 3.6143317151976967`*^9, 3.614506458201749*^9, 
   3.6145064916482077`*^9, {3.614506787409127*^9, 3.6145067899987316`*^9}, 
   3.6148618634815464`*^9, 3.614863136740183*^9}],

Cell[BoxData[
 SqrtBox[
  FractionBox[
   SuperscriptBox["uo", "2"], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"2", "+", 
      SuperscriptBox["uo", "2"]}], ")"}], "2"]]]], "Output",
 CellChangeTimes->{3.626261908884183*^9, 3.626422216553433*^9, 
  3.626852260163175*^9}]
}, Closed]],

Cell["\<\
The only geodesic parameter curve u == uo is the generating hel obtained for \
uo = 0.\
\>", "Text",
 CellChangeTimes->{{3.6143326361293707`*^9, 3.6143326704333324`*^9}, 
   3.6145069125525475`*^9, 3.614751027217533*^9, {3.617429872653062*^9, 
   3.6174299091727257`*^9}, {3.618983277747088*^9, 3.6189833075431404`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kn", " ", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"normalcurvature", "[", 
      RowBox[{"hnsf", ",", "hu", ",", "hv"}], "]"}], "[", "t", "]"}], ",", 
    RowBox[{"t", "\[Element]", "Reals"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.614332709739581*^9, 3.6143327141728344`*^9}, {
   3.614506937574991*^9, 3.614506939836995*^9}, 3.6148618634971466`*^9, 
   3.614863136740183*^9}],

Cell[BoxData[
 FractionBox["1", 
  SqrtBox[
   RowBox[{"2", "+", 
    SuperscriptBox["uo", "2"]}]]]], "Output",
 CellChangeTimes->{3.6262619168713975`*^9, 3.626422229641856*^9, 
  3.6268522759036026`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"Together", "[", 
    RowBox[{
     RowBox[{"kk", "^", "2"}], " ", "-", " ", 
     RowBox[{"kg", "^", "2"}], "-", 
     RowBox[{"kn", "^", "2"}]}], "]"}], ",", 
   RowBox[{"uo", "\[Element]", "Reals"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6143327450406*^9, 3.614332813123494*^9}, {
  3.614332923036781*^9, 3.6143329233798003`*^9}, {3.6145069563262243`*^9, 
  3.614507048615986*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6262619199602027`*^9, 3.626422247332287*^9, 
  3.6268522801312103`*^9}]
}, Closed]],

Cell["OK.", "Text",
 CellChangeTimes->{{3.6145070851824503`*^9, 3.614507087226054*^9}}],

Cell[TextData[{
 StyleBox["Example",
  FontColor->RGBColor[0, 0, 1]],
 ". Catenoids"
}], "Text",
 CellChangeTimes->{{3.6136216629488277`*^9, 3.6136217472669764`*^9}, {
   3.6136219986458178`*^9, 3.6136220101274376`*^9}, {3.6136316409915533`*^9, 
   3.6136316447511597`*^9}, {3.613631706792469*^9, 3.6136317217996955`*^9}, {
   3.613631784777006*^9, 3.6136317936534214`*^9}, 3.6136319442872863`*^9, {
   3.6136323634132223`*^9, 3.613632369544033*^9}, {3.6143337794257636`*^9, 
   3.6143337836740065`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"catenoid", "[", "a", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.61433378758123*^9, 3.6143337980828304`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", " ", 
    RowBox[{"Cos", "[", "u", "]"}], " ", 
    RowBox[{"Cosh", "[", 
     FractionBox["v", "a"], "]"}]}], ",", 
   RowBox[{"a", " ", 
    RowBox[{"Cosh", "[", 
     FractionBox["v", "a"], "]"}], " ", 
    RowBox[{"Sin", "[", "u", "]"}]}], ",", "v"}], "}"}]], "Output",
 CellChangeTimes->{3.6262672973548965`*^9, 3.6264222567703037`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"catenoid", "[", "1", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "2"}], ",", "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.614333822495227*^9, 3.614333879636495*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"gauss", "[", 
     RowBox[{"catenoid", "[", "ao", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "\[Element]", "Reals"}], ",", 
     RowBox[{"v", "\[Element]", "Reals"}], ",", 
     RowBox[{"ao", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.614334848950937*^9, 3.6143348515860877`*^9}, {
  3.614334951923826*^9, 3.6143349775872946`*^9}, {3.6143361902226534`*^9, 
  3.6143362040454435`*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   SuperscriptBox[
    RowBox[{"Sech", "[", 
     FractionBox["v", "ao"], "]"}], "4"], 
   SuperscriptBox["ao", "2"]]}]], "Output",
 CellChangeTimes->{3.626267308805317*^9, 3.626422269390726*^9, 
  3.626852344512523*^9}]
}, Closed]],

Cell["\<\
Remember that this is the only minimal surface of revolution:\
\>", "Text",
 CellChangeTimes->{{3.617430756535014*^9, 3.61743078288346*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"meancurvature", "[", 
     RowBox[{"catenoid", "[", "ao", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "\[Element]", "Reals"}], ",", 
     RowBox[{"v", "\[Element]", "Reals"}], ",", 
     RowBox[{"ao", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.614334848950937*^9, 3.6143348515860877`*^9}, {
   3.614334951923826*^9, 3.6143349775872946`*^9}, {3.6143361902226534`*^9, 
   3.6143362040454435`*^9}, 3.617430053285779*^9, {3.6174307303581676`*^9, 
   3.6174307367073793`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6262673210981383`*^9, 3.6264222811219463`*^9, 
  3.6268523576633463`*^9}]
}, Closed]],

Cell["The geodesic curvature of a meridian is", "Text",
 CellChangeTimes->{{3.61433633999422*^9, 3.61433636132944*^9}, 
   3.6143959895035353`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"h1", "[", "t_", "]"}], " ", ":=", "uo"}], ";", 
  RowBox[{
   RowBox[{"h2", "[", "t_", "]"}], ":=", " ", "t"}], ";", " ", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"geodcurvature", "[", 
      RowBox[{
       RowBox[{"catenoid", "[", "ao", "]"}], ",", "h1", ",", "h2"}], "]"}], 
     "[", "t", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"uo", "\[Element]", "Reals"}], ",", 
      RowBox[{"t", "\[Element]", "Reals"}], ",", 
      RowBox[{"ao", ">", "0"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.614335163132907*^9, 3.6143352204111834`*^9}, {
   3.614335253212059*^9, 3.614335367557599*^9}, 3.614335503949401*^9, 
   3.6145073265928745`*^9, 3.6148618635127463`*^9, 3.614863136755783*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6262673273693495`*^9, 3.6264222868471565`*^9, 
  3.626852363653757*^9}]
}, Closed]],

Cell[" The meridians are geodesics.", "Text",
 CellChangeTimes->{{3.6145072358007145`*^9, 3.6145072494975386`*^9}}],

Cell["Its normal curvature :", "Text",
 CellChangeTimes->{{3.6143363759432755`*^9, 3.6143363979705353`*^9}, {
  3.614507283645999*^9, 3.614507284753601*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kn", " ", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"normalcurvature", "[", 
      RowBox[{
       RowBox[{"catenoid", "[", "ao", "]"}], ",", "h1", ",", "h2"}], "]"}], 
     "[", "t", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"uo", "\[Element]", "Reals"}], ",", 
      RowBox[{"t", "\[Element]", "Reals"}], ",", 
      RowBox[{"ao", ">", "0"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.614335163132907*^9, 3.6143352204111834`*^9}, {
   3.614335253212059*^9, 3.614335367557599*^9}, 3.614335503949401*^9, {
   3.6143360854696617`*^9, 3.6143361025496387`*^9}, {3.614507348666913*^9, 
   3.614507356794527*^9}, 3.614861863528346*^9, 3.614863136755783*^9}],

Cell[BoxData[
 FractionBox[
  SuperscriptBox[
   RowBox[{"Sech", "[", 
    FractionBox["t", "ao"], "]"}], "2"], "ao"]], "Output",
 CellChangeTimes->{3.626267334670162*^9, 3.6264222965503736`*^9, 
  3.6268523729825735`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"sx", "[", "t_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"catenoid", "[", "ao", "]"}], "[", 
   RowBox[{"uo", ",", "t"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6143966521458993`*^9, 3.6143966702887306`*^9}, {
  3.6143967042499905`*^9, 3.6143967161684113`*^9}, {3.6143967976629543`*^9, 
  3.6143968015005617`*^9}, {3.6145073821445723`*^9, 3.6145073949989944`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sx", "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.614396727759232*^9, 3.6143967316436386`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"ao", " ", 
    RowBox[{"Cos", "[", "uo", "]"}], " ", 
    RowBox[{"Cosh", "[", 
     FractionBox["t", "ao"], "]"}]}], ",", 
   RowBox[{"ao", " ", 
    RowBox[{"Cosh", "[", 
     FractionBox["t", "ao"], "]"}], " ", 
    RowBox[{"Sin", "[", "uo", "]"}]}], ",", "t"}], "}"}]], "Output",
 CellChangeTimes->{3.6262673435465775`*^9, 3.6264223102627974`*^9, 
  3.6268523902518034`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kk", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"curvature", "[", "sx", "]"}], "[", "t", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"uo", "\[Element]", "Reals"}], ",", 
      RowBox[{"t", "\[Element]", "Reals"}], ",", 
      RowBox[{"ao", ">", "0"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.614396815415786*^9, 3.6143968606402655`*^9}, {
   3.614507412595825*^9, 3.6145074240930457`*^9}, 3.6148618635439463`*^9, 
   3.614863136755783*^9}],

Cell[BoxData[
 FractionBox[
  SuperscriptBox[
   RowBox[{"Sech", "[", 
    FractionBox["t", "ao"], "]"}], "2"], "ao"]], "Output",
 CellChangeTimes->{3.6262673477429852`*^9, 3.626422314599605*^9, 
  3.62685239404261*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"kk", "^", "2"}], "-", 
   RowBox[{"kn", "^", "2"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.614396897955531*^9, 3.614396932634392*^9}, 
   3.614397004098117*^9, 3.6143972530589542`*^9, {3.61450748368515*^9, 
   3.614507504776387*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.626267358007803*^9, 3.6264223186868124`*^9, 
  3.6268524045414286`*^9}]
}, Closed]],

Cell["The geodesic curvature of the generating circles  v = vo is", "Text",
 CellChangeTimes->{{3.6143366537701664`*^9, 3.6143366972176514`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"h1", "[", "t_", "]"}], " ", ":=", "t"}], ";", 
  RowBox[{
   RowBox[{"h2", "[", "t_", "]"}], ":=", " ", "vo"}], ";", " ", 
  RowBox[{"kg1", "=", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"geodcurvature", "[", 
       RowBox[{
        RowBox[{"catenoid", "[", "ao", "]"}], ",", "h1", ",", "h2"}], "]"}], 
      "[", "t", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"t", "\[Element]", "Reals"}], ",", 
       RowBox[{"vo", "\[Element]", "Reals"}], ",", 
       RowBox[{"ao", ">", "0"}]}], "}"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.614335163132907*^9, 3.6143352204111834`*^9}, {
   3.614335253212059*^9, 3.614335367557599*^9}, 3.614335503949401*^9, {
   3.6143365726285257`*^9, 3.6143365995500655`*^9}, {3.614507543948056*^9, 
   3.614507584570527*^9}, 3.6148618635595465`*^9, 3.614863136755783*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"Abs", "[", 
    RowBox[{"Tanh", "[", 
     FractionBox["vo", "ao"], "]"}], "]"}], " ", 
   RowBox[{"Sech", "[", 
    FractionBox["vo", "ao"], "]"}]}], "ao"]], "Output",
 CellChangeTimes->{3.6262673656050167`*^9, 3.6264223273760276`*^9, 
  3.6268524113274403`*^9}]
}, Closed]],

Cell["It is a geodesic for vo = 0 only.", "Text",
 CellChangeTimes->{{3.6143367097293673`*^9, 3.614336749553645*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"h1", "[", "t_", "]"}], " ", ":=", "t"}], ";", 
  RowBox[{
   RowBox[{"h2", "[", "t_", "]"}], ":=", " ", "vo"}], ";", " ", 
  RowBox[{"kn1", "=", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"normalcurvature", "[", 
       RowBox[{
        RowBox[{"catenoid", "[", "ao", "]"}], ",", "h1", ",", "h2"}], "]"}], 
      "[", "t", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"t", "\[Element]", "Reals"}], ",", 
       RowBox[{"vo", "\[Element]", "Reals"}], ",", 
       RowBox[{"ao", ">", "0"}]}], "}"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.614335163132907*^9, 3.6143352204111834`*^9}, {
   3.614335253212059*^9, 3.614335367557599*^9}, 3.614335503949401*^9, {
   3.6143365726285257`*^9, 3.6143365995500655`*^9}, {3.614507543948056*^9, 
   3.614507584570527*^9}, {3.6145076250993986`*^9, 3.6145076497474422`*^9}, 
   3.6148618635751467`*^9, 3.614863136755783*^9}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   SuperscriptBox[
    RowBox[{"Sech", "[", 
     FractionBox["vo", "ao"], "]"}], "2"], "ao"]}]], "Output",
 CellChangeTimes->{
  3.6262673889582577`*^9, 3.626422331775235*^9, {3.6268524331674786`*^9, 
   3.626852440624292*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"sx", "[", "t_", "]"}], " ", "=", " ", 
  RowBox[{
   RowBox[{"catenoid", "[", "ao", "]"}], "[", 
   RowBox[{"t", ",", "vo"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6145076705110784`*^9, 3.614507738542798*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"ao", " ", 
    RowBox[{"Cos", "[", "t", "]"}], " ", 
    RowBox[{"Cosh", "[", 
     FractionBox["vo", "ao"], "]"}]}], ",", 
   RowBox[{"ao", " ", 
    RowBox[{"Cosh", "[", 
     FractionBox["vo", "ao"], "]"}], " ", 
    RowBox[{"Sin", "[", "t", "]"}]}], ",", "vo"}], "}"}]], "Output",
 CellChangeTimes->{3.626267397210672*^9, 3.626422337828046*^9, 
  3.6268524487831063`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kk1", " ", "=", " ", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"curvature", "[", "sx", "]"}], "[", "t", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"t", "\[Element]", "Reals"}], ",", 
      RowBox[{"vo", "\[Element]", "Reals"}], ",", 
      RowBox[{"ao", ">", "0"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.614507752239622*^9, 3.614507799242504*^9}, 
   3.614861863590747*^9, 3.614863136755783*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{"Sech", "[", 
   FractionBox["vo", "ao"], "]"}], "ao"]], "Output",
 CellChangeTimes->{3.6262674030762825`*^9, 3.626422342773255*^9, 
  3.6268524524959126`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Together", "[", 
    RowBox[{
     RowBox[{"kk1", "^", "2"}], "-", 
     RowBox[{"kn1", "^", "2"}], "-", 
     RowBox[{"kg1", "^", "2"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"vo", "\[Element]", "Reals"}], ",", 
     RowBox[{"ao", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6145078278997545`*^9, 3.61450790487029*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6262674136219006`*^9, 3.626422347624863*^9, 
  3.626852458954324*^9}]
}, Closed]],

Cell["OK.", "Text",
 CellChangeTimes->{{3.6145079169291115`*^9, 3.614507921234719*^9}}],

Cell[TextData[{
 StyleBox["Example",
  FontColor->RGBColor[0, 0, 1]],
 ". Surfaces of revolution"
}], "Text",
 CellChangeTimes->{{3.6136216629488277`*^9, 3.6136217472669764`*^9}, {
   3.6136219986458178`*^9, 3.6136220101274376`*^9}, {3.6136316409915533`*^9, 
   3.6136316447511597`*^9}, {3.613631706792469*^9, 3.6136317217996955`*^9}, {
   3.613631784777006*^9, 3.6136317936534214`*^9}, 3.6136319442872863`*^9, {
   3.6136323634132223`*^9, 3.613632369544033*^9}, {3.614337018522029*^9, 
   3.614337032273816*^9}}],

Cell[TextData[{
 "We consider the special surfaces of revolution with  an unspecified \
function f[z], see Subsection ",
 ButtonBox["3.8.2",
  BaseStyle->"Hyperlink",
  ButtonData->"sfrev"],
 "."
}], "Text",
 CellChangeTimes->{{3.613632210517354*^9, 3.6136322491742215`*^9}, {
  3.614337045025545*^9, 3.614337100169699*^9}, {3.614337139193931*^9, 
  3.6143371486734734`*^9}, {3.6143371788692007`*^9, 3.614337178873201*^9}, {
  3.614337255633591*^9, 3.614337261929951*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"sfr", "[", "f", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{3.6143369034754486`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", "v", "]"}], " ", 
    RowBox[{"f", "[", "u", "]"}]}], ",", 
   RowBox[{
    RowBox[{"f", "[", "u", "]"}], " ", 
    RowBox[{"Sin", "[", "v", "]"}]}], ",", "u"}], "}"}]], "Output",
 CellChangeTimes->{3.626267419518711*^9, 3.626422376048113*^9, 
  3.6268524802327614`*^9}]
}, Closed]],

Cell["The meridians of a surface of revolution are geodesics:", "Text",
 CellChangeTimes->{{3.6145105285608983`*^9, 3.6145105522417397`*^9}, 
   3.617441960564849*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"sx", ",", "hu", ",", "hv"}], "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.613621793911058*^9, 3.61362190860246*^9}, 
   3.6136224537922173`*^9, {3.6136225031195035`*^9, 3.613622523134339*^9}, {
   3.6136319875929623`*^9, 3.613631994222974*^9}, 3.6136324917858477`*^9, {
   3.6139708453034763`*^9, 3.6139709222428117`*^9}, 3.6139709681380925`*^9, 
   3.613972610035379*^9, 3.6139765411438866`*^9, {3.6143373012522*^9, 
   3.6143373205003014`*^9}, 3.614337396340639*^9, {3.6143374303895864`*^9, 
   3.614337446300497*^9}, {3.61451029605809*^9, 3.6145103278509455`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"hu", "[", "s_", "]"}], ":=", " ", "s"}], ";", 
  RowBox[{
   RowBox[{"hv", "[", "s_", "]"}], " ", ":=", " ", "vo"}], ";", 
  RowBox[{"kg", " ", "=", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{"geodcurvature", "[", 
      RowBox[{
       RowBox[{"sfr", "[", "f", "]"}], ",", "hu", ",", "hv"}], "]"}], "[", 
     "s", "]"}], " ", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.613621793911058*^9, 3.61362190860246*^9}, 
   3.6136224537922173`*^9, {3.6136225031195035`*^9, 3.613622523134339*^9}, {
   3.6136319875929623`*^9, 3.613631994222974*^9}, 3.6136324917858477`*^9, {
   3.6139708453034763`*^9, 3.6139709222428117`*^9}, 3.6139709681380925`*^9, 
   3.613972610035379*^9, 3.6139765411438866`*^9, {3.614337351228059*^9, 
   3.6143373738843546`*^9}, {3.614339108452566*^9, 3.6143391480198298`*^9}, {
   3.61433920555312*^9, 3.6143392193249083`*^9}, {3.6145080441629343`*^9, 
   3.614508046003738*^9}, {3.6145097196370773`*^9, 3.6145097964828124`*^9}, 
   3.614861863606347*^9, 3.614863136755783*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.626267428083126*^9, 3.626422388590535*^9, 
  3.626852498079193*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kn", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"normalcurvature", "[", 
     RowBox[{
      RowBox[{"sfr", "[", "f", "]"}], ",", "hu", ",", "hv"}], "]"}], "[", "s",
     "]"}], " ", "]"}]}]], "Input",
 CellChangeTimes->{{3.613621793911058*^9, 3.61362190860246*^9}, 
   3.6136224537922173`*^9, {3.6136225031195035`*^9, 3.613622523134339*^9}, {
   3.6136319875929623`*^9, 3.613631994222974*^9}, 3.6136324917858477`*^9, {
   3.6139708453034763`*^9, 3.6139709222428117`*^9}, 3.6139709681380925`*^9, 
   3.613972610035379*^9, 3.6139765411438866`*^9, {3.614337351228059*^9, 
   3.6143373738843546`*^9}, {3.614339108452566*^9, 3.6143391480198298`*^9}, {
   3.6145080216832952`*^9, 3.6145080231028976`*^9}, 3.614861863621947*^9, 
   3.614863136771383*^9}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "s", "]"}], "3"], " ", 
    RowBox[{
     SuperscriptBox["f", "\[Prime]\[Prime]",
      MultilineFunction->None], "[", "s", "]"}]}], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "s", "]"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{"1", "+", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox["f", "\[Prime]",
           MultilineFunction->None], "[", "s", "]"}], "2"]}], ")"}]}], ")"}], 
    RowBox[{"3", "/", "2"}]]]}]], "Output",
 CellChangeTimes->{3.6262674358051395`*^9, 3.626422403441761*^9, 
  3.626852504506404*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"sx", "[", "s_", "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{"sfr", "[", "f", "]"}], "[", 
    RowBox[{"s", ",", "vo"}], "]"}]}], ";", 
  RowBox[{"kk", " ", "=", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"curvature", "[", "sx", "]"}], "[", "s", "]"}], ",", 
     RowBox[{"s", "\[Element]", "Reals"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6143402822007008`*^9, 3.6143403642143917`*^9}, {
   3.614340406997839*^9, 3.6143404223017144`*^9}, {3.6145101194969797`*^9, 
   3.614510178839484*^9}, 3.6145103356353593`*^9, 3.6148618636375465`*^9, 
   3.614863136771383*^9}],

Cell[BoxData[
 FractionBox[
  SqrtBox[
   FractionBox[
    SuperscriptBox[
     RowBox[{
      SuperscriptBox["f", "\[Prime]\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], "2"], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "+", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["f", "\[Prime]",
          MultilineFunction->None], "[", "s", "]"}], "2"]}], ")"}], "2"]]], 
  SqrtBox[
   RowBox[{"1", "+", 
    SuperscriptBox[
     RowBox[{
      SuperscriptBox["f", "\[Prime]",
       MultilineFunction->None], "[", "s", "]"}], "2"]}]]]], "Output",
 CellChangeTimes->{3.6262674414211493`*^9, 3.6264224167797847`*^9, 
  3.6268525439588737`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"%", "^", "2"}], "-", 
   RowBox[{"%%", "^", "2"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.614510431934328*^9, 3.61451046704999*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.62626744731796*^9, 3.6264224211477923`*^9, 
  3.6268525479836807`*^9}]
}, Closed]],

Cell["For the generating circles we obtain the following results:", "Text",
 CellChangeTimes->{{3.6145106139710484`*^9, 3.6145106484627085`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"sx", ",", "hu", ",", "hv"}], "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.613621793911058*^9, 3.61362190860246*^9}, 
   3.6136224537922173`*^9, {3.6136225031195035`*^9, 3.613622523134339*^9}, {
   3.6136319875929623`*^9, 3.613631994222974*^9}, 3.6136324917858477`*^9, {
   3.6139708453034763`*^9, 3.6139709222428117`*^9}, 3.6139709681380925`*^9, 
   3.613972610035379*^9, 3.6139765411438866`*^9, {3.6143373012522*^9, 
   3.6143373205003014`*^9}, 3.614337396340639*^9, {3.6143374303895864`*^9, 
   3.614337446300497*^9}, {3.61451029605809*^9, 3.6145103278509455`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"hu", "[", "s_", "]"}], ":=", " ", "uo"}], ";", 
  RowBox[{
   RowBox[{"hv", "[", "s_", "]"}], " ", ":=", " ", "s"}], ";", 
  RowBox[{"kg1", " ", "=", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"geodcurvature", "[", 
       RowBox[{
        RowBox[{"sfr", "[", "f", "]"}], ",", "hu", ",", "hv"}], "]"}], "[", 
      "s", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"s", "\[Element]", "Reals"}], ",", 
       RowBox[{
        RowBox[{"f", "[", "uo", "]"}], ">", "0"}]}], "}"}]}], " ", 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.613621793911058*^9, 3.61362190860246*^9}, 
   3.6136224537922173`*^9, {3.6136225031195035`*^9, 3.613622523134339*^9}, {
   3.6136319875929623`*^9, 3.613631994222974*^9}, 3.6136324917858477`*^9, {
   3.6139708453034763`*^9, 3.6139709222428117`*^9}, 3.6139709681380925`*^9, 
   3.613972610035379*^9, 3.6139765411438866`*^9, {3.614337351228059*^9, 
   3.6143373738843546`*^9}, {3.614339108452566*^9, 3.6143391480198298`*^9}, {
   3.61433920555312*^9, 3.6143392193249083`*^9}, {3.6145080441629343`*^9, 
   3.614508046003738*^9}, {3.6145097196370773`*^9, 3.6145097964828124`*^9}, {
   3.614510676839159*^9, 3.614510701627602*^9}, 3.614511028260976*^9, 
   3.6148618636531467`*^9, 3.614863136771383*^9}],

Cell[BoxData[
 SqrtBox[
  FractionBox[
   SuperscriptBox[
    RowBox[{
     SuperscriptBox["f", "\[Prime]",
      MultilineFunction->None], "[", "uo", "]"}], "2"], 
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "uo", "]"}], "2"], " ", 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["f", "\[Prime]",
         MultilineFunction->None], "[", "uo", "]"}], "2"]}], 
     ")"}]}]]]], "Output",
 CellChangeTimes->{{3.6262674575359783`*^9, 3.626267461623185*^9}, 
   3.626422432208212*^9, 3.6268525560956945`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kn1", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"normalcurvature", "[", 
     RowBox[{
      RowBox[{"sfr", "[", "f", "]"}], ",", "hu", ",", "hv"}], "]"}], "[", "s",
     "]"}], " ", "]"}]}]], "Input",
 CellChangeTimes->{{3.613621793911058*^9, 3.61362190860246*^9}, 
   3.6136224537922173`*^9, {3.6136225031195035`*^9, 3.613622523134339*^9}, {
   3.6136319875929623`*^9, 3.613631994222974*^9}, 3.6136324917858477`*^9, {
   3.6139708453034763`*^9, 3.6139709222428117`*^9}, 3.6139709681380925`*^9, 
   3.613972610035379*^9, 3.6139765411438866`*^9, {3.614337351228059*^9, 
   3.6143373738843546`*^9}, {3.614339108452566*^9, 3.6143391480198298`*^9}, {
   3.6145080216832952`*^9, 3.6145080231028976`*^9}, 3.6145107301132526`*^9, 
   3.614861863668747*^9, 3.614863136771383*^9}],

Cell[BoxData[
 FractionBox["1", 
  SqrtBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "uo", "]"}], "2"], " ", 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      SuperscriptBox[
       RowBox[{
        SuperscriptBox["f", "\[Prime]",
         MultilineFunction->None], "[", "uo", "]"}], "2"]}], 
     ")"}]}]]]], "Output",
 CellChangeTimes->{3.6262674699848*^9, 3.6264224385262227`*^9, 
  3.62685256483171*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"sx", "[", "s_", "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{"sfr", "[", "f", "]"}], "[", 
    RowBox[{"uo", ",", "s"}], "]"}]}], ";", 
  RowBox[{"kk1", " ", "=", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"curvature", "[", "sx", "]"}], "[", "s", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"s", "\[Element]", "Reals"}], ",", 
       RowBox[{
        RowBox[{"f", "[", "uo", "]"}], ">", "0"}]}], "}"}]}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6143402822007008`*^9, 3.6143403642143917`*^9}, {
   3.614340406997839*^9, 3.6143404223017144`*^9}, {3.6145101194969797`*^9, 
   3.614510178839484*^9}, 3.6145103356353593`*^9, {3.6145107478816833`*^9, 
   3.614510761375707*^9}, {3.614510957171651*^9, 3.614510976578085*^9}, 
   3.614861863684347*^9, 3.614863136771383*^9}],

Cell[BoxData[
 FractionBox["1", 
  RowBox[{"f", "[", "uo", "]"}]]], "Output",
 CellChangeTimes->{3.626267473479206*^9, 3.6264224433778315`*^9, 
  3.6268525695117188`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Together", "[", 
   RowBox[{
    RowBox[{"kk1", "^", "2"}], "-", 
    RowBox[{"kg1", "^", "2"}], " ", "-", 
    RowBox[{"kn1", "^", "2"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.614510431934328*^9, 3.61451046704999*^9}, {
  3.614510786335751*^9, 3.6145108526046677`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.626267476848812*^9, 3.6264224480578394`*^9, 
  3.6268525755801287`*^9}]
}, Closed]],

Cell["OK.", "Text",
 CellChangeTimes->{{3.6145113332883115`*^9, 3.614511335768716*^9}}],

Cell[TextData[{
 StyleBox["Example",
  FontColor->RGBColor[0, 0, 1]],
 ". Pseudosphere"
}], "Text",
 CellChangeTimes->{{3.6136216629488277`*^9, 3.6136217472669764`*^9}, {
   3.6136219986458178`*^9, 3.6136220101274376`*^9}, {3.6136316409915533`*^9, 
   3.6136316447511597`*^9}, {3.613631706792469*^9, 3.6136317217996955`*^9}, {
   3.613631784777006*^9, 3.6136317936534214`*^9}, 3.6136319442872863`*^9, {
   3.6136323634132223`*^9, 3.613632369544033*^9}, {3.614337018522029*^9, 
   3.614337032273816*^9}, {3.614520626709233*^9, 3.614520632524565*^9}}],

Cell["We consider the  surface of revolution named", "Text",
 CellChangeTimes->{{3.613632210517354*^9, 3.6136322491742215`*^9}, {
  3.614337045025545*^9, 3.614337100169699*^9}, {3.614337139193931*^9, 
  3.6143371486734734`*^9}, {3.6143371788692007`*^9, 3.614337178873201*^9}, {
  3.614337255633591*^9, 3.614337261929951*^9}, {3.614520784941283*^9, 
  3.614520803445341*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "pseudosphere"}]], "Input",
 CellChangeTimes->{{3.6145206567479506`*^9, 3.614520666822527*^9}}],

Cell[BoxData[
 StyleBox["\<\"{u,v}->pseudosphere[a][u,v] is the standard parametrization of \
a pseudosphere of constant negative curvature -1/a^2.  To plot try \
pseudosphere[1][0,3Pi/2,0.001,Pi-0.001].\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.6262674824180217`*^9},
 CellTags->"Info3626271082-4060188"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"pseudosphere", "[", "a", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{
  3.6143369034754486`*^9, {3.6145206979213057`*^9, 3.6145207228067293`*^9}, 
   3.614861863699947*^9, 3.614862881648535*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", " ", 
    RowBox[{"Cos", "[", "u", "]"}], " ", 
    RowBox[{"Sin", "[", "v", "]"}]}], ",", 
   RowBox[{"a", " ", 
    RowBox[{"Sin", "[", "u", "]"}], " ", 
    RowBox[{"Sin", "[", "v", "]"}]}], ",", 
   RowBox[{"a", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Cos", "[", "v", "]"}], "+", 
      RowBox[{"Log", "[", 
       RowBox[{"Tan", "[", 
        FractionBox["v", "2"], "]"}], "]"}]}], ")"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.626267485631627*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"pseudosphere", "[", "1", "]"}], "[", 
  StyleBox[
   RowBox[{"0", ",", " ", 
    RowBox[{"3", "*", 
     RowBox[{"Pi", "/", "2"}]}], ",", "0.001", ",", " ", 
    RowBox[{"Pi", "-", "0.001"}]}], "MSG"], 
  StyleBox["]", "MSG"]}]], "Input",
 CellChangeTimes->{{3.6145211179693313`*^9, 3.614521219566142*^9}, 
   3.6148618637155466`*^9, 3.614862882927737*^9}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"pseudosphere", "[", "5", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     StyleBox["0", "MSG"], 
     StyleBox[",", "MSG"], 
     StyleBox[" ", "MSG"], 
     StyleBox[
      RowBox[{
       StyleBox[
        StyleBox["2", "MSG",
         FontWeight->"Plain"],
        FontWeight->"Plain"], 
       StyleBox[
        StyleBox["*", "MSG",
         FontWeight->"Plain"],
        FontWeight->"Plain"], "Pi"}], "MSG"]}], 
    StyleBox["}", "MSG"]}], 
   StyleBox[",", "MSG"], 
   RowBox[{"{", 
    RowBox[{"v", ",", ".05", 
     StyleBox[",", "MSG"], 
     StyleBox[" ", "MSG"], "3.07"}], 
    StyleBox["}", "MSG"]}], ",", " ", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6145211179693313`*^9, 3.614521219566142*^9}, {
   3.61458697032903*^9, 3.61458724307563*^9}, {3.614587291074375*^9, 
   3.6145873808355093`*^9}, 3.6145874129723473`*^9, {3.6145875356043615`*^9, 
   3.6145876236113954`*^9}, {3.6145876565172777`*^9, 3.614587683412816*^9}, {
   3.6145877298114696`*^9, 3.6145877299794793`*^9}, {3.614587809876049*^9, 
   3.614587847756216*^9}, {3.6145878805000887`*^9, 3.614587970652245*^9}, {
   3.614588043044386*^9, 3.6145881118043184`*^9}, 3.614861863746747*^9, 
   3.6148628843317394`*^9, {3.626855000328114*^9, 3.626855037107379*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"pseudosphere", "[", "a", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{3.6189942016752872`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", " ", 
    RowBox[{"Cos", "[", "u", "]"}], " ", 
    RowBox[{"Sin", "[", "v", "]"}]}], ",", 
   RowBox[{"a", " ", 
    RowBox[{"Sin", "[", "u", "]"}], " ", 
    RowBox[{"Sin", "[", "v", "]"}]}], ",", 
   RowBox[{"a", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Cos", "[", "v", "]"}], "+", 
      RowBox[{"Log", "[", 
       RowBox[{"Tan", "[", 
        FractionBox["v", "2"], "]"}], "]"}]}], ")"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.626267505240862*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"gauss", "[", 
     RowBox[{"pseudosphere", "[", "a", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", ">", "0"}], ",", 
     RowBox[{"u", ">", "0"}], ",", 
     RowBox[{"v", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.614520815235016*^9, 3.6145208500470066`*^9}, 
   3.614521396431258*^9, {3.614521453503523*^9, 3.614521505631504*^9}, 
   3.614861863777947*^9, 3.614862885376941*^9}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox["1", 
   SuperscriptBox["a", "2"]]}]], "Output",
 CellChangeTimes->{3.626267511262472*^9, 3.626852615282199*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"prcurvatures", "[", 
     RowBox[{"pseudosphere", "[", "a", "]"}], "]"}], "[", " ", 
    RowBox[{"u", ",", " ", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", ">", "0"}], ",", 
     RowBox[{"u", ">", "0"}], ",", 
     RowBox[{"v", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.592315825904586*^9, 3.592315836169404*^9}, 
   3.592315870785865*^9, 3.592741650505703*^9, 3.5988843282017965`*^9, {
   3.612074334747755*^9, 3.612074353998189*^9}, {3.614573304329767*^9, 
   3.614573332097816*^9}, {3.614573363750272*^9, 3.614573366168276*^9}, 
   3.6148618637935467`*^9, 3.6148628872021446`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"Abs", "[", 
      RowBox[{"Cos", "[", "v", "]"}], "]"}], " ", 
     RowBox[{"Csc", "[", "v", "]"}]}], "a"], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{"Abs", "[", 
       RowBox[{"Cos", "[", "v", "]"}], "]"}], " ", 
      RowBox[{"Sec", "[", "v", "]"}], " ", 
      RowBox[{"Tan", "[", "v", "]"}]}], "a"]}]}], "}"}]], "Output",
 CellChangeTimes->{3.62626760102503*^9, 3.6268526322238283`*^9}]
}, Closed]],

Cell["\<\
We obtain the Gaussian curvature as the product of the principal curvatures:\
\>", "Text",
 CellChangeTimes->{{3.6145828150596323`*^9, 3.614582894979203*^9}, {
  3.6145868363523664`*^9, 3.614586850311165*^9}, {3.6174318859931164`*^9, 
  3.617431890201357*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"%", "[", 
     RowBox[{"[", "1", "]"}], "]"}], "*", 
    RowBox[{"%", "[", 
     RowBox[{"[", "2", "]"}], "]"}]}], ",", 
   RowBox[{"v", "\[Element]", " ", "Reals"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.617431089080798*^9, 3.61743118701777*^9}, 
   3.617431225752638*^9}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox["1", 
   SuperscriptBox["a", "2"]]}]], "Output",
 CellChangeTimes->{3.6262676076394415`*^9, 3.6268526370130367`*^9}]
}, Closed]],

Cell[TextData[{
 "It is constant negative, as we know already, see ",
 ButtonBox["Case1",
  BaseStyle->"Hyperlink",
  ButtonData->"pseusph"],
 " in Subsection 3.8.5."
}], "Text",
 CellChangeTimes->{{3.614582936275565*^9, 3.614582960251937*^9}, {
   3.614583309187895*^9, 3.614583327723955*^9}, {3.6145833661561527`*^9, 
   3.6145833803799667`*^9}, {3.6145834365011764`*^9, 3.6145834365021763`*^9}, 
   3.6145881791771717`*^9}],

Cell["OK.", "Text",
 CellChangeTimes->{{3.6145113332883115`*^9, 3.614511335768716*^9}}],

Cell["\<\
As for all surfaces of revolution the meridians are geodesics. For the other \
parameter curves, the circles, we obtain\
\>", "Text",
 CellChangeTimes->{{3.6174320470253267`*^9, 3.6174320687845716`*^9}, {
  3.618994352449552*^9, 3.6189944130712585`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"hu", "[", "s_", "]"}], ":=", "s"}], ";", 
  RowBox[{
   RowBox[{"hv", "[", "s_", "]"}], " ", ":=", " ", "uo"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"kg1", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"geodcurvature", "[", 
      RowBox[{
       RowBox[{"pseudosphere", "[", "a", "]"}], ",", "hu", ",", "hv"}], "]"}],
      "[", "s", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a", ">", "0"}], ",", 
      RowBox[{"s", ">", "0"}]}], "}"}]}], " ", "]"}]}]}], "Input",
 CellChangeTimes->{{3.613621793911058*^9, 3.61362190860246*^9}, 
   3.6136224537922173`*^9, {3.6136225031195035`*^9, 3.613622523134339*^9}, {
   3.6136319875929623`*^9, 3.613631994222974*^9}, 3.6136324917858477`*^9, {
   3.6139708453034763`*^9, 3.6139709222428117`*^9}, 3.6139709681380925`*^9, 
   3.613972610035379*^9, 3.6139765411438866`*^9, {3.614337351228059*^9, 
   3.6143373738843546`*^9}, {3.614339108452566*^9, 3.6143391480198298`*^9}, {
   3.6145080216832952`*^9, 3.6145080231028976`*^9}, 3.614521785152492*^9, {
   3.6145218322641864`*^9, 3.6145218887524176`*^9}, {3.614522956094466*^9, 
   3.614523048511752*^9}, {3.6145835550879593`*^9, 3.614583556488039*^9}, 
   3.6145835939061794`*^9, {3.6148618638559475`*^9, 3.6148618638559475`*^9}, 
   3.6148628927713547`*^9, 3.6148631367869825`*^9}],

Cell[BoxData[
 FractionBox["1", "a"]], "Output",
 CellChangeTimes->{3.626267628980279*^9, 3.626852650600661*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kn1", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"normalcurvature", "[", 
      RowBox[{
       RowBox[{"pseudosphere", "[", "a", "]"}], ",", "hu", ",", "hv"}], "]"}],
      "[", "s", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a", ">", "0"}], ",", 
      RowBox[{"s", ">", "0"}]}], "}"}]}], " ", "]"}]}]], "Input",
 CellChangeTimes->{{3.613621793911058*^9, 3.61362190860246*^9}, 
   3.6136224537922173`*^9, {3.6136225031195035`*^9, 3.613622523134339*^9}, {
   3.6136319875929623`*^9, 3.613631994222974*^9}, 3.6136324917858477`*^9, {
   3.6139708453034763`*^9, 3.6139709222428117`*^9}, 3.6139709681380925`*^9, 
   3.613972610035379*^9, 3.6139765411438866`*^9, {3.614337351228059*^9, 
   3.6143373738843546`*^9}, {3.614339108452566*^9, 3.6143391480198298`*^9}, {
   3.6145080216832952`*^9, 3.6145080231028976`*^9}, 3.614521785152492*^9, {
   3.6145218322641864`*^9, 3.6145218887524176`*^9}, {3.614522956094466*^9, 
   3.614523048511752*^9}, {3.6145835550879593`*^9, 3.614583556488039*^9}, 
   3.6145835939061794`*^9, {3.6148618638559475`*^9, 3.6148618638559475`*^9}, 
   3.6148628927713547`*^9, 3.6148631367869825`*^9, 3.617432148730144*^9, {
   3.6174321926216545`*^9, 3.617432218900158*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{
    SqrtBox[
     SuperscriptBox[
      RowBox[{"Cos", "[", "uo", "]"}], "2"]], " ", 
    RowBox[{"Csc", "[", "uo", "]"}]}], "a"]}]], "Output",
 CellChangeTimes->{3.6262676342218885`*^9, 3.6268526561854706`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"sx", "[", "s_", "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{"pseudosphere", "[", "a", "]"}], "[", 
    RowBox[{"s", ",", "uo"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"kk1", " ", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"curvature", "[", "sx", "]"}], "[", "s", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a", ">", "0"}], ",", 
      RowBox[{"s", ">", "0"}]}], "}"}]}], " ", "]"}]}]}], "Input",
 CellChangeTimes->{{3.6145864758237457`*^9, 3.614586558650483*^9}, {
   3.6145866388910723`*^9, 3.6145866895779715`*^9}, {3.614861863871547*^9, 
   3.614861863887147*^9}, 3.614862893816556*^9, 3.6148631367869825`*^9}],

Cell[BoxData[
 FractionBox["1", 
  SqrtBox[
   RowBox[{
    SuperscriptBox["a", "2"], " ", 
    SuperscriptBox[
     RowBox[{"Sin", "[", "uo", "]"}], "2"]}]]]], "Output",
 CellChangeTimes->{3.626267645063907*^9, 3.626852663330283*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"kn1", "^", "2"}], "+", 
    RowBox[{"kg1", "^", "2"}], "-", 
    RowBox[{"kk1", "^", "2"}]}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", ">", "0"}], ",", 
     RowBox[{"uo", ">", "0"}]}], "}"}]}], " ", "]"}]], "Input",
 CellChangeTimes->{{3.6145863523506837`*^9, 3.614586387312683*^9}, {
  3.6145867153694468`*^9, 3.6145867620811186`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.6262676493851147`*^9, 3.6268526677606907`*^9}]
}, Closed]],

Cell["OK.", "Text",
 CellChangeTimes->{{3.614586770251586*^9, 3.614586793679926*^9}}],

Cell[TextData[{
 "Using the local isometry of the pseudosphere with H. Poincar\[EAcute]\
\[CloseCurlyQuote]s model of the hyperbolic plane (see e.g. [",
 ButtonBox["OS3",
  BaseStyle->"Hyperlink",
  ButtonData->"OS3"],
 "],p. 255) one easily finds all geodesics of the pseudosphere. A good \
visualization  of the geodesics is the CDF-program ",
 ButtonBox["Pseudosphere Geodesics",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://demonstrations.wolfram.com/PseudosphereGeodesics/"], None},
  ButtonNote->"http://demonstrations.wolfram.com/PseudosphereGeodesics/"],
 " in Wolfram ",
 ButtonBox["Demonstrations Project.",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://demonstrations.wolfram.com/index.html"], None},
  ButtonNote->"http://demonstrations.wolfram.com/index.html"]
}], "Text",
 CellChangeTimes->{{3.620533656797402*^9, 3.620533693067466*^9}, {
   3.620533723331519*^9, 3.6205338124076757`*^9}, {3.6205338774597893`*^9, 
   3.6205340125560265`*^9}, {3.6205345720787644`*^9, 3.620534626349869*^9}, {
   3.620534741843527*^9, 3.620534741843527*^9}, {3.620535057918519*^9, 
   3.620535074262454*^9}, {3.620535119074215*^9, 3.6205351472078247`*^9}, 
   3.620535178017508*^9, {3.6205352215883846`*^9, 3.6205352215883846`*^9}}],

Cell[TextData[{
 StyleBox["Exercise",
  FontColor->RGBColor[0, 0, 1]],
 ". Study this CDF program and calculate the geodesic curvature of the curves \
on the pseudosphere; it must be zero."
}], "Text",
 CellChangeTimes->{{3.620535308386937*^9, 3.6205354134687214`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6178041659455204`*^9, 3.6178041730591326`*^9}}]
}, Closed]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["References", "Subtitle",
 CellChangeTimes->{{3.5881675431561174`*^9, 3.588167577236067*^9}, 
   3.5883163578378267`*^9, {3.591690059078165*^9, 3.591690069748584*^9}, {
   3.592463089910284*^9, 3.5924631116099224`*^9}, {3.5949735445390763`*^9, 
   3.594973561296035*^9}, {3.5980125441965847`*^9, 3.5980125441965847`*^9}, 
   3.598016745174763*^9, {3.614340028650199*^9, 3.61434006957854*^9}}],

Cell["\<\
[B-R]  W. Blaschke, H. Reichardt. Einf\[UDoubleDot]hrung in die \
Differentialgeometrie.2. Aufl. Springer-Verlag. \
Berlin.G\[ODoubleDot]ttingen.Heidelberg. 1960.\
\>", "Text",
 CellChangeTimes->{{3.5866621958935566`*^9, 3.5866623742377577`*^9}, 
   3.5875491110897255`*^9, {3.588167593643005*^9, 3.5881676056026893`*^9}},
 CellTags->"B-R"],

Cell[TextData[{
 "[G06] Alfred Gray, Simon Salamon, Elsa Abbena. Modern Differential Geometry \
of Curves and Surfaces with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ".\nThird ed. CRC Press. 2006. "
}], "Text",
 CellChangeTimes->{{3.4481738125625*^9, 3.448173840671875*^9}, {
   3.44817393246875*^9, 3.44817394509375*^9}, {3.448173979828125*^9, 
   3.448173980734375*^9}, {3.4481759745*^9, 3.448176007265625*^9}, 
   3.450237117953125*^9, {3.450237154*^9, 3.450237191546875*^9}, {
   3.450238953984375*^9, 3.450238956734375*^9}, 3.541836408394127*^9, 
   3.5418367181953497`*^9},
 CellTags->"Gray"],

Cell[TextData[{
 "[G06nb] Alfred Gray, Simon Salamon, Elsa Abbena. A series of 27 ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notebooks corresponding to the chapters of [G06], available from the \
website ",
 "\n",
 ButtonBox["http://alpha01.dm.unito.it/personalpages/abbena/gray/",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://alpha01.dm.unito.it/personalpages/abbena/gray/"], None},
  ButtonNote->"http://alpha01.dm.unito.it/personalpages/abbena/gray/"]
}], "Text",
 CellChangeTimes->{{3.599201219155733*^9, 3.599201261780171*^9}, {
  3.5992013329092393`*^9, 3.599201340627681*^9}, {3.5992013935957108`*^9, 
  3.5992014308198395`*^9}, {3.599201571187868*^9, 3.599201593564148*^9}, {
  3.599201693291852*^9, 3.5992016987161627`*^9}, {3.599201729499923*^9, 
  3.599201765534984*^9}},
 CellTags->"Gray2"],

Cell["\<\
[G94] Alfred Gray. Differentialgeometrie. Klassische Theorie in moderner \
Darstellung. (\[CapitalUDoubleDot]bersetzung aus dem Amerikanischen H. Gollek).
Spektrum Akademischer Verlag, Heidelberg.Berlin.Oxford. 1994.\
\>", "Text",
 CellChangeTimes->{{3.450237198875*^9, 3.45023720096875*^9}, {
   3.450237249703125*^9, 3.450237440828125*^9}, {3.450238940359375*^9, 
   3.45023894278125*^9}, {3.454066758375*^9, 3.4540668540625*^9}, 
   3.45407173278125*^9, 3.541836408394155*^9, 3.5418367181953783`*^9},
 CellTags->"Gray3"],

Cell["\<\
[Kr] Erwin Kreyszig. Differentialgeometrie. Akademische Verlagsgesellschaft \
Geest & Portig K. G. Leipzig 1957.\
\>", "Text",
 CellChangeTimes->{{3.5906430418435745`*^9, 3.5906431143993025`*^9}, {
  3.5906432147074785`*^9, 3.590643286436404*^9}},
 CellTags->"Kr"],

Cell["\<\
{K-N1] S. Kobayashi, K. Nomizu. Foundations of Differential Geometry, Vol.  \
I. Interscience Publishers. New York - London. 1963.\
\>", "Text",
 CellChangeTimes->{{3.597388908321149*^9, 3.5973889165686207`*^9}, {
  3.5973889798572407`*^9, 3.597389168017003*^9}},
 CellTags->"K-N1"],

Cell["\<\
[OS3] A. L. Onishchik, R. Sulanke. Projective and Cayley-Klein Geometries. \
Springer-Verlag. Berlin,Heidelberg. 2006.\
\>", "Text",
 CellChangeTimes->{{3.450266884375*^9, 3.45026689246875*^9}, {
   3.4502669551875*^9, 3.45026703496875*^9}, {3.450267067078125*^9, 
   3.450267115015625*^9}, 3.541836408394287*^9, 3.541836718195511*^9, {
   3.6205335043695345`*^9, 3.6205335122631483`*^9}},
 CellTags->"OS3"],

Cell[CellGroupData[{

Cell["Homepage", "Subsubtitle",
 CellChangeTimes->{
  3.445933589770261*^9, {3.4459337168429823`*^9, 3.445933720398094*^9}, 
   3.541836408394314*^9, 3.541836718195538*^9}],

Cell[TextData[ButtonBox["http://www-irm.mathematik.hu-berlin.de/~sulanke",
 BaseStyle->"Hyperlink",
 ButtonData->{
   URL["http://www-irm.mathematik.hu-berlin.de/~sulanke"], None}]], "Text",
 CellChangeTimes->{{3.4459337369318686`*^9, 3.4459338102072334`*^9}, {
   3.4465638794375*^9, 3.4465638794375*^9}, 3.54183640839435*^9, 
   3.541836718195591*^9},
 CellTags->"Home"],

Cell["e-mail: sulanke@mathematik.hu-berlin.de", "Text",
 CellChangeTimes->{{3.618995363041403*^9, 3.618995394834259*^9}}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowToolbars->"EditBar",
WindowSize->{1485, 917},
WindowMargins->{{-7, Automatic}, {Automatic, 0}},
ShowSelection->True,
Magnification:>2. Inherited,
FrontEndVersion->"10.0 for Microsoft Windows (64-bit) (December 4, 2014)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Info3627193388-6571891"->{
  Cell[24178, 502, 222, 4, 2147483647, "Print",
   CellTags->"Info3627193388-6571891"]},
 "Info3627193599-6571891"->{
  Cell[24849, 524, 2816, 43, 2147483647, "Print",
   CellTags->"Info3627193599-6571891"]},
 "Info3627194180-6571891"->{
  Cell[28348, 593, 587, 9, 2147483647, "Print",
   CellTags->"Info3627194180-6571891"],
  Cell[28938, 604, 5342, 133, 2147483647, "Print",
   CellTags->"Info3627194180-6571891"]},
 "Info3627196796-6571891"->{
  Cell[56499, 1296, 680, 14, 2147483647, "Print",
   CellTags->"Info3627196796-6571891"]},
 "Info3627639363-8560585"->{
  Cell[75530, 1808, 229, 4, 162, "Print",
   CellTags->"Info3627639363-8560585"]},
 "Info3627458084-5102015"->{
  Cell[116830, 2873, 292, 5, 1900544, "Print",
   CellTags->"Info3627458084-5102015"]},
 "Info3627197182-6571891"->{
  Cell[118532, 2925, 343, 6, 2147483647, "Print",
   CellTags->"Info3627197182-6571891"]},
 "Info3626671618-6153013"->{
  Cell[165216, 4134, 390, 6, 2147483647, "Print",
   CellTags->"Info3626671618-6153013"]},
 "Info3627286815-6629669"->{
  Cell[171224, 4296, 585, 9, 2147483647, "Print",
   CellTags->"Info3627286815-6629669"]},
 "Info3627289761-6629669"->{
  Cell[216977, 5788, 409, 6, 2147483647, "Print",
   CellTags->"Info3627289761-6629669"]},
 "Info3627108654-6604827"->{
  Cell[234545, 6304, 281, 5, 2147483647, "Print",
   CellTags->"Info3627108654-6604827"]},
 "Info3627290180-6629669"->{
  Cell[238738, 6439, 313, 5, 2147483647, "Print",
   CellTags->"Info3627290180-6629669"]},
 "Info3627309622-3465247"->{
  Cell[257829, 7016, 324, 6, 2147483647, "Print",
   CellTags->"Info3627309622-3465247"]},
 "Info3627309626-3465247"->{
  Cell[340729, 9409, 295, 5, 2147483647, "Print",
   CellTags->"Info3627309626-3465247"]},
 "Info3627312169-3465247"->{
  Cell[365245, 10139, 281, 5, 2147483647, "Print",
   CellTags->"Info3627312169-3465247"]},
 "Info3627312345-3465247"->{
  Cell[373637, 10397, 503, 8, 2147483647, "Print",
   CellTags->"Info3627312345-3465247"]},
 "g(i"->{
  Cell[383032, 10709, 2126, 45, 2147483647, "Text",
   CellTags->"g(i"]},
 "Info3627312854-3465247"->{
  Cell[387666, 10821, 342, 6, 2147483647, "Print",
   CellTags->"Info3627312854-3465247"]},
 "Info3627312921-3465247"->{
  Cell[388377, 10841, 297, 5, 2147483647, "Print",
   CellTags->"Info3627312921-3465247"]},
 "Info3629190373-5022241"->{
  Cell[406252, 11408, 229, 4, 140, "Print",
   CellTags->"Info3629190373-5022241"],
  Cell[897995, 26531, 359, 6, 140, "Print",
   CellTags->"Info3629190373-5022241"]},
 "ptr"->{
  Cell[419624, 11805, 388, 12, 2147483647, "Input",
   CellTags->"ptr"]},
 "Info3627376031-6375012"->{
  Cell[500366, 14213, 357, 6, 2147483647, "Print",
   CellTags->"Info3627376031-6375012"]},
 "Info3627376078-6375012"->{
  Cell[510233, 14549, 349, 6, 2147483647, "Print",
   CellTags->"Info3627376078-6375012"]},
 "Info3627376383-6375012"->{
  Cell[570540, 16305, 585, 9, 2147483647, "Print",
   CellTags->"Info3627376383-6375012"]},
 "Info3627445080-5102015"->{
  Cell[605398, 17382, 788, 15, 2147483647, "Print",
   CellTags->"Info3627445080-5102015"]},
 "Info3627445143-5102015"->{
  Cell[610119, 17535, 529, 12, 2147483647, "Print",
   CellTags->"Info3627445143-5102015"]},
 "Info3627445604-5102015"->{
  Cell[613211, 17636, 319, 6, 2147483647, "Print",
   CellTags->"Info3627445604-5102015"]},
 "Info3627446141-5102015"->{
  Cell[661753, 19239, 293, 5, 2147483647, "Print",
   CellTags->"Info3627446141-5102015"]},
 "Info3627459539-5102015"->{
  Cell[696951, 20418, 328, 6, 1946157056, "Print",
   CellTags->"Info3627459539-5102015"]},
 "bastr"->{
  Cell[732762, 21546, 386, 9, 2147483647, "Text",
   CellTags->"bastr"]},
 "intcond"->{
  Cell[907564, 26839, 129, 2, 76, "Subsubsection",
   CellTags->"intcond"]},
 "lem2.1.1"->{
  Cell[964516, 28554, 576, 11, 2147483647, "Text",
   CellTags->"lem2.1.1"]},
 "Info3627711720-5071683"->{
  Cell[974173, 28812, 231, 4, 2147483647, "Print",
   CellTags->"Info3627711720-5071683"]},
 "Info3627639472-8560585"->{
  Cell[976311, 28881, 433, 7, 2147483647, "Print",
   CellTags->"Info3627639472-8560585"]},
 "Info3627718148-3784632"->{
  Cell[1100548, 32627, 295, 5, 2147483647, "Print",
   CellTags->"Info3627718148-3784632"]},
 "polco"->{
  Cell[1116998, 33100, 244, 7, 2147483647, "Text",
   CellTags->"polco"]},
 "Info3627718626-1674846"->{
  Cell[1121564, 33234, 2652, 42, 2147483647, "Print",
   CellTags->"Info3627718626-1674846"]},
 "Info3627719831-1674846"->{
  Cell[1129840, 33439, 281, 5, 2147483647, "Print",
   CellTags->"Info3627719831-1674846"]},
 "Info3627720425-9230554"->{
  Cell[1148967, 34006, 283, 5, 2147483647, "Print",
   CellTags->"Info3627720425-9230554"]},
 "Info3627720445-9230554"->{
  Cell[1149787, 34033, 285, 5, 2147483647, "Print",
   CellTags->"Info3627720445-9230554"]},
 "Info3627720479-9230554"->{
  Cell[1152400, 34106, 2648, 42, 2147483647, "Print",
   CellTags->"Info3627720479-9230554"]},
 "Info3627744819-7142630"->{
  Cell[1162660, 34375, 275, 5, 2147483647, "Print",
   CellTags->"Info3627744819-7142630"]},
 "Info3627745043-7142630"->{
  Cell[1179124, 34872, 375, 6, 2147483647, "Print",
   CellTags->"Info3627745043-7142630"]},
 "Info3627749163-7142630"->{
  Cell[1191870, 35247, 205, 4, 2147483647, "Print",
   CellTags->"Info3627749163-7142630"]},
 "Info3627816879-6530549"->{
  Cell[1223728, 36121, 405, 6, 2147483647, "Print",
   CellTags->"Info3627816879-6530549"]},
 "Info3627817277-6530549"->{
  Cell[1252214, 36972, 280, 5, 2147483647, "Print",
   CellTags->"Info3627817277-6530549"]},
 "Info3627830615-6530549"->{
  Cell[1256561, 37096, 3360, 53, 2147483647, "Print",
   CellTags->"Info3627830615-6530549"]},
 "gaussK"->{
  Cell[1277949, 37645, 1208, 36, 830472192, "Input",
   Evaluatable->False,
   CellTags->"gaussK"]},
 "Info3627831290-6705403"->{
  Cell[1279478, 37692, 237, 4, 973078528, "Print",
   CellTags->"Info3627831290-6705403"]},
 "cij"->{
  Cell[1481640, 43540, 321, 8, 1644167168, "Text",
   CellTags->"cij"]},
 "Info3627905210-4669561"->{
  Cell[1482100, 43556, 319, 5, 1946157056, "Print",
   CellTags->"Info3627905210-4669561"]},
 "Info3627911044-4669561"->{
  Cell[1499374, 44037, 296, 5, 973078528, "Print",
   CellTags->"Info3627911044-4669561"]},
 "Info3627911413-4669561"->{
  Cell[1503653, 44146, 319, 5, 973078528, "Print",
   CellTags->"Info3627911413-4669561"]},
 "Info3627914288-4669561"->{
  Cell[1511688, 44392, 351, 6, 2147483647, "Print",
   CellTags->"Info3627914288-4669561"]},
 "Info3627977160-2492953"->{
  Cell[1548608, 45524, 373, 6, 629145600, "Print",
   CellTags->"Info3627977160-2492953"]},
 "Info3627987206-2313691"->{
  Cell[1675909, 49329, 269, 5, 243269632, "Print",
   CellTags->"Info3627987206-2313691"]},
 "Info3627987267-2313691"->{
  Cell[1676782, 49360, 213, 4, 169869312, "Print",
   CellTags->"Info3627987267-2313691"]},
 "Info3628063470-3762495"->{
  Cell[1780899, 52070, 316, 5, 78643200, "Print",
   CellTags->"Info3628063470-3762495"]},
 "Info3628063532-3762495"->{
  Cell[1796670, 52545, 667, 14, 61341696, "Print",
   CellTags->"Info3628063532-3762495"]},
 "Info3628055261-3902486"->{
  Cell[1848489, 54014, 128, 2, 10616832, "Print",
   CellTags->"Info3628055261-3902486"],
  Cell[1848620, 54018, 880, 24, 5636096, "Print",
   CellTags->"Info3628055261-3902486"]},
 "Info3628055436-3902486"->{
  Cell[1853296, 54146, 262, 5, 15204352, "Print",
   CellTags->"Info3628055436-3902486"]},
 "geodc"->{
  Cell[1862896, 54402, 1251, 28, 18350080, "Text",
   CellTags->"geodc"]},
 "Info3628063421-3762495"->{
  Cell[1874732, 54709, 251, 4, 15204352, "Print",
   CellTags->"Info3628063421-3762495"]},
 "Info3628065998-8815218"->{
  Cell[1896911, 55353, 242, 4, 15204352, "Print",
   CellTags->"Info3628065998-8815218"]},
 "Info3628088987-5827330"->{
  Cell[1937455, 56349, 288, 5, 15204352, "Print",
   CellTags->"Info3628088987-5827330"]},
 "Info3628089421-5827330"->{
  Cell[1960736, 57147, 2013, 34, 15466496, "Print",
   CellTags->"Info3628089421-5827330"]},
 "helix"->{
  Cell[2165886, 62853, 610, 8, 76, "Subsubsection",
   CellTags->"helix"],
  Cell[2344008, 67814, 626, 9, 60, "Subsubsection",
   CellTags->"helix"]},
 "Info3628233781-2217871"->{
  Cell[2173840, 63109, 373, 6, 9600, "Print",
   CellTags->"Info3628233781-2217871"]},
 "Info3628235851-2217871"->{
  Cell[2175958, 63171, 335, 6, 9600, "Print",
   CellTags->"Info3628235851-2217871"]},
 "Info3628839108-8634717"->{
  Cell[2292961, 66291, 375, 6, 9600, "Print",
   CellTags->"Info3628839108-8634717"]},
 "Info3629004696-3111076"->{
  Cell[2322336, 67168, 382, 6, 9600, "Print",
   CellTags->"Info3629004696-3111076"]},
 "Info3628232522-2217871"->{
  Cell[2375493, 68848, 265, 5, 464, "Print",
   CellTags->"Info3628232522-2217871"]},
 "Info3629190502-5022241"->{
  Cell[2397802, 69454, 825, 12, 504, "Print",
   CellTags->"Info3629190502-5022241"]},
 "sfrev"->{
  Cell[3163848, 82638, 580, 11, 196, "Text",
   CellTags->"sfrev"]},
 "fund1"->{
  Cell[3361275, 88168, 286, 7, 196, "Text",
   CellTags->"fund1"]},
 "dtt(1)"->{
  Cell[3403727, 89256, 230, 6, 2147483647, "Text",
   CellTags->"dtt(1)"]},
 "Info3625907027-2112774"->{
  Cell[3425884, 89893, 338, 6, 2147483647, "Print",
   CellTags->"Info3625907027-2112774"]},
 "Info3629190374-5022241"->{
  Cell[3469208, 91082, 315, 5, 140, "Print",
   CellTags->"Info3629190374-5022241"],
  Cell[3470867, 91129, 345, 6, 140, "Print",
   CellTags->"Info3629190374-5022241"],
  Cell[4793021, 127911, 400, 6, 140, "Print",
   CellTags->"Info3629190374-5022241"]},
 "Info3626005242-1516062"->{
  Cell[4842833, 129370, 277, 5, 2147483647, "Print",
   CellTags->"Info3626005242-1516062"]},
 "Info3626006462-6741741"->{
  Cell[4858501, 129826, 284, 5, 464, "Print",
   CellTags->"Info3626006462-6741741"]},
 "Info3626006478-6741741"->{
  Cell[4860269, 129875, 306, 5, 2147483647, "Print",
   CellTags->"Info3626006478-6741741"]},
 "Info3626009464-6741741"->{
  Cell[4866745, 130066, 265, 5, 2147483647, "Print",
   CellTags->"Info3626009464-6741741"],
  Cell[4867013, 130073, 1753, 46, 2147483647, "Print",
   CellTags->"Info3626009464-6741741"]},
 "Info3626076175-1128913"->{
  Cell[4909781, 131220, 545, 8, 2147483647, "Print",
   CellTags->"Info3626076175-1128913"]},
 "Info3626685166-6153013"->{
  Cell[5917204, 156778, 127, 2, 2147483647, "Print",
   CellTags->"Info3626685166-6153013"],
  Cell[5917334, 156782, 2463, 61, 2147483647, "Print",
   CellTags->"Info3626685166-6153013"]},
 "Info3626783633-5796653"->{
  Cell[6168406, 161278, 322, 6, 2147483647, "Print",
   CellTags->"Info3626783633-5796653"]},
 "Info3626787084-5796653"->{
  Cell[6520648, 169638, 543, 8, 2147483647, "Print",
   CellTags->"Info3626787084-5796653"]},
 "Info3626265383-1010981"->{
  Cell[6754880, 175175, 290, 5, 2147483647, "Print",
   CellTags->"Info3626265383-1010981"]},
 "Info3626271082-4060188"->{
  Cell[6786930, 176116, 322, 5, 2147483647, "Print",
   CellTags->"Info3626271082-4060188"]},
 "B-R"->{
  Cell[6801158, 176528, 350, 7, 98, "Text",
   CellTags->"B-R"]},
 "Gray"->{
  Cell[6801511, 176537, 609, 13, 136, "Text",
   CellTags->"Gray"]},
 "Gray2"->{
  Cell[6802123, 176552, 827, 18, 136, "Text",
   CellTags->"Gray2"]},
 "Gray3"->{
  Cell[6802953, 176572, 533, 9, 136, "Text",
   CellTags->"Gray3"]},
 "Kr"->{
  Cell[6803489, 176583, 274, 6, 98, "Text",
   CellTags->"Kr"]},
 "K-N1"->{
  Cell[6803766, 176591, 292, 6, 98, "Text",
   CellTags->"K-N1"]},
 "OS3"->{
  Cell[6804061, 176599, 417, 8, 98, "Text",
   CellTags->"OS3"]},
 "Home"->{
  Cell[6804678, 176616, 372, 7, 60, "Text",
   CellTags->"Home"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Info3627193388-6571891", 6805600, 176644},
 {"Info3627193599-6571891", 6805718, 176647},
 {"Info3627194180-6571891", 6805838, 176650},
 {"Info3627196796-6571891", 6806047, 176655},
 {"Info3627639363-8560585", 6806167, 176658},
 {"Info3627458084-5102015", 6806279, 176661},
 {"Info3627197182-6571891", 6806396, 176664},
 {"Info3626671618-6153013", 6806516, 176667},
 {"Info3627286815-6629669", 6806636, 176670},
 {"Info3627289761-6629669", 6806756, 176673},
 {"Info3627108654-6604827", 6806876, 176676},
 {"Info3627290180-6629669", 6806996, 176679},
 {"Info3627309622-3465247", 6807116, 176682},
 {"Info3627309626-3465247", 6807236, 176685},
 {"Info3627312169-3465247", 6807356, 176688},
 {"Info3627312345-3465247", 6807477, 176691},
 {"g(i", 6807579, 176694},
 {"Info3627312854-3465247", 6807682, 176697},
 {"Info3627312921-3465247", 6807803, 176700},
 {"Info3629190373-5022241", 6807924, 176703},
 {"ptr", 6808103, 176708},
 {"Info3627376031-6375012", 6808206, 176711},
 {"Info3627376078-6375012", 6808327, 176714},
 {"Info3627376383-6375012", 6808448, 176717},
 {"Info3627445080-5102015", 6808569, 176720},
 {"Info3627445143-5102015", 6808691, 176723},
 {"Info3627445604-5102015", 6808813, 176726},
 {"Info3627446141-5102015", 6808934, 176729},
 {"Info3627459539-5102015", 6809055, 176732},
 {"bastr", 6809159, 176735},
 {"intcond", 6809247, 176738},
 {"lem2.1.1", 6809339, 176741},
 {"Info3627711720-5071683", 6809446, 176744},
 {"Info3627639472-8560585", 6809567, 176747},
 {"Info3627718148-3784632", 6809688, 176750},
 {"polco", 6809793, 176753},
 {"Info3627718626-1674846", 6809897, 176756},
 {"Info3627719831-1674846", 6810021, 176759},
 {"Info3627720425-9230554", 6810143, 176762},
 {"Info3627720445-9230554", 6810265, 176765},
 {"Info3627720479-9230554", 6810387, 176768},
 {"Info3627744819-7142630", 6810511, 176771},
 {"Info3627745043-7142630", 6810633, 176774},
 {"Info3627749163-7142630", 6810755, 176777},
 {"Info3627816879-6530549", 6810877, 176780},
 {"Info3627817277-6530549", 6810999, 176783},
 {"Info3627830615-6530549", 6811121, 176786},
 {"gaussK", 6811229, 176789},
 {"Info3627831290-6705403", 6811359, 176793},
 {"cij", 6811461, 176796},
 {"Info3627905210-4669561", 6811563, 176799},
 {"Info3627911044-4669561", 6811685, 176802},
 {"Info3627911413-4669561", 6811806, 176805},
 {"Info3627914288-4669561", 6811927, 176808},
 {"Info3627977160-2492953", 6812049, 176811},
 {"Info3627987206-2313691", 6812170, 176814},
 {"Info3627987267-2313691", 6812291, 176817},
 {"Info3628063470-3762495", 6812412, 176820},
 {"Info3628063532-3762495", 6812532, 176823},
 {"Info3628055261-3902486", 6812653, 176826},
 {"Info3628055436-3902486", 6812863, 176831},
 {"geodc", 6812966, 176834},
 {"Info3628063421-3762495", 6813070, 176837},
 {"Info3628065998-8815218", 6813190, 176840},
 {"Info3628088987-5827330", 6813310, 176843},
 {"Info3628089421-5827330", 6813430, 176846},
 {"helix", 6813535, 176849},
 {"Info3628233781-2217871", 6813715, 176854},
 {"Info3628235851-2217871", 6813831, 176857},
 {"Info3628839108-8634717", 6813947, 176860},
 {"Info3629004696-3111076", 6814063, 176863},
 {"Info3628232522-2217871", 6814179, 176866},
 {"Info3629190502-5022241", 6814294, 176869},
 {"sfrev", 6814393, 176872},
 {"fund1", 6814474, 176875},
 {"dtt(1)", 6814555, 176878},
 {"Info3625907027-2112774", 6814660, 176881},
 {"Info3629190374-5022241", 6814782, 176884},
 {"Info3626005242-1516062", 6815068, 176891},
 {"Info3626006462-6741741", 6815191, 176894},
 {"Info3626006478-6741741", 6815307, 176897},
 {"Info3626009464-6741741", 6815430, 176900},
 {"Info3626076175-1128913", 6815648, 176905},
 {"Info3626685166-6153013", 6815771, 176908},
 {"Info3626783633-5796653", 6815989, 176913},
 {"Info3626787084-5796653", 6816112, 176916},
 {"Info3626265383-1010981", 6816235, 176919},
 {"Info3626271082-4060188", 6816358, 176922},
 {"B-R", 6816462, 176925},
 {"Gray", 6816539, 176928},
 {"Gray2", 6816620, 176931},
 {"Gray3", 6816702, 176934},
 {"Kr", 6816780, 176937},
 {"K-N1", 6816856, 176940},
 {"OS3", 6816933, 176943},
 {"Home", 6817010, 176946}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[580, 22, 116, 2, 201, "Title"],
Cell[699, 26, 306, 5, 111, "Subtitle"],
Cell[1008, 33, 493, 10, 98, "Text"],
Cell[CellGroupData[{
Cell[1526, 47, 200, 3, 95, "Subsection"],
Cell[1729, 52, 10180, 178, 1958, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[11946, 235, 145, 2, 95, "Subsection"],
Cell[12094, 239, 2962, 42, 478, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[15093, 286, 195, 3, 95, "Subsection"],
Cell[15291, 291, 719, 11, 424, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[16047, 307, 170, 2, 108, "Section"],
Cell[16220, 311, 316, 6, 60, "Text"],
Cell[CellGroupData[{
Cell[16561, 321, 507, 8, 60, "Input",
 InitializationCell->True],
Cell[17071, 331, 917, 16, 118, "Output"]
}, Closed]],
Cell[18003, 350, 94, 1, 50, "Text"],
Cell[18100, 353, 524, 8, 60, "Input"],
Cell[18627, 363, 603, 9, 136, "Text"],
Cell[19233, 374, 31, 0, 60, "Input"],
Cell[19267, 376, 917, 16, 174, "Text"],
Cell[20187, 394, 337, 6, 98, "Text"],
Cell[20527, 402, 153, 2, 60, "Text"],
Cell[CellGroupData[{
Cell[20705, 408, 161, 2, 95, "Subsection"],
Cell[20869, 412, 252, 6, 196, "Text"],
Cell[21124, 420, 473, 7, 170, "Input",
 InitializationCell->True],
Cell[21600, 429, 198, 4, 120, "Text"],
Cell[21801, 435, 166, 3, 120, "Input",
 InitializationCell->True],
Cell[21970, 440, 333, 6, 120, "Text"],
Cell[22306, 448, 216, 4, 120, "Input",
 InitializationCell->True],
Cell[22525, 454, 270, 5, 196, "Text"],
Cell[22798, 461, 324, 6, 120, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[23147, 471, 132, 2, 120, "Input"],
Cell[23282, 475, 86, 1, 2147483647, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[23417, 482, 171, 2, 75, "Subsection"],
Cell[23591, 486, 443, 8, 136, "Text"],
Cell[CellGroupData[{
Cell[24059, 498, 116, 2, 60, "Input"],
Cell[24178, 502, 222, 4, 2147483647, "Print",
 CellTags->"Info3627193388-6571891"]
}, Closed]],
Cell[24415, 509, 298, 7, 50, "Text"],
Cell[CellGroupData[{
Cell[24738, 520, 108, 2, 60, "Input"],
Cell[24849, 524, 2816, 43, 2147483647, "Print",
 CellTags->"Info3627193599-6571891"]
}, Closed]],
Cell[27680, 570, 431, 12, 75, "Text"],
Cell[CellGroupData[{
Cell[28136, 586, 187, 3, 60, "Input"],
Cell[CellGroupData[{
Cell[28348, 593, 587, 9, 2147483647, "Print",
 CellTags->"Info3627194180-6571891"],
Cell[28938, 604, 5342, 133, 2147483647, "Print",
 CellTags->"Info3627194180-6571891"]
}, Closed]]
}, Closed]],
Cell[34307, 741, 361, 6, 88, "Text"],
Cell[CellGroupData[{
Cell[34693, 751, 206, 3, 76, "Subsubsection"],
Cell[CellGroupData[{
Cell[34924, 758, 271, 5, 2147483647, "Input",
 InitializationCell->True],
Cell[35198, 765, 13773, 330, 140, "Print"]
}, Closed]],
Cell[48986, 1098, 240, 4, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[49251, 1106, 127, 2, 2147483647, "Input"],
Cell[49381, 1110, 94, 1, 2147483647, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[49524, 1117, 128, 2, 60, "Subsubsection"],
Cell[CellGroupData[{
Cell[49677, 1123, 147, 3, 120, "Input",
 InitializationCell->True],
Cell[49827, 1128, 6434, 157, 810, "Print"]
}, Closed]],
Cell[CellGroupData[{
Cell[56298, 1290, 198, 4, 198, "Input"],
Cell[56499, 1296, 680, 14, 2147483647, "Print",
 CellTags->"Info3627196796-6571891"]
}, Closed]],
Cell[57194, 1313, 290, 5, 88, "Input"],
Cell[CellGroupData[{
Cell[57509, 1322, 127, 2, 120, "Input"],
Cell[57639, 1326, 95, 1, 2147483647, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[57783, 1333, 130, 2, 60, "Subsubsection"],
Cell[57916, 1337, 235, 3, 120, "Text"],
Cell[CellGroupData[{
Cell[58176, 1344, 204, 4, 120, "Input",
 InitializationCell->True],
Cell[58383, 1350, 13019, 316, 1250, "Print"]
}, Closed]],
Cell[CellGroupData[{
Cell[71439, 1671, 213, 5, 120, "Input"],
Cell[71655, 1678, 2303, 78, 392, "Output"]
}, Closed]],
Cell[73973, 1759, 294, 5, 120, "Input"],
Cell[CellGroupData[{
Cell[74292, 1768, 127, 2, 120, "Input"],
Cell[74422, 1772, 95, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[74554, 1778, 189, 3, 88, "Input"],
Cell[74746, 1783, 126, 2, 118, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[74909, 1790, 191, 3, 120, "Input"],
Cell[75103, 1795, 138, 2, 118, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[75278, 1802, 249, 4, 120, "Input"],
Cell[75530, 1808, 229, 4, 162, "Print",
 CellTags->"Info3627639363-8560585"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[75808, 1818, 153, 2, 60, "Subsubsection"],
Cell[75964, 1822, 228, 5, 120, "Text"],
Cell[CellGroupData[{
Cell[76217, 1831, 118, 2, 120, "Input"],
Cell[CellGroupData[{
Cell[76360, 1837, 40467, 1034, 48447488, "Print"],
Cell[116830, 2873, 292, 5, 1900544, "Print",
 CellTags->"Info3627458084-5102015"]
}, Closed]]
}, Closed]],
Cell[117149, 2882, 286, 5, 88, "Input"],
Cell[CellGroupData[{
Cell[117460, 2891, 127, 2, 120, "Input"],
Cell[117590, 2895, 96, 1, 2147483647, "Output"]
}, Closed]],
Cell[117701, 2899, 245, 8, 88, "Input"],
Cell[117949, 2909, 441, 8, 196, "Text"],
Cell[CellGroupData[{
Cell[118415, 2921, 114, 2, 120, "Input"],
Cell[118532, 2925, 343, 6, 2147483647, "Print",
 CellTags->"Info3627197182-6571891"]
}, Closed]],
Cell[CellGroupData[{
Cell[118912, 2936, 131, 2, 88, "Input"],
Cell[119046, 2940, 85, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[119168, 2946, 388, 11, 88, "Input"],
Cell[119559, 2959, 75, 1, 2147483647, "Output"]
}, Closed]],
Cell[119649, 2963, 193, 2, 100, "Text"],
Cell[119845, 2967, 117, 2, 120, "Input"],
Cell[119965, 2971, 356, 11, 120, "Input"],
Cell[120324, 2984, 402, 11, 120, "Input"],
Cell[120729, 2997, 192, 4, 120, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[120958, 3006, 205, 3, 60, "Subsubsection"],
Cell[121166, 3011, 275, 6, 120, "Text"],
Cell[CellGroupData[{
Cell[121466, 3021, 116, 2, 120, "Input"],
Cell[CellGroupData[{
Cell[121607, 3027, 43606, 1105, 52772864, "Print"],
Cell[165216, 4134, 390, 6, 2147483647, "Print",
 CellTags->"Info3626671618-6153013"]
}, Closed]]
}, Closed]],
Cell[165633, 4144, 291, 5, 88, "Input"],
Cell[CellGroupData[{
Cell[165949, 4153, 127, 2, 120, "Input"],
Cell[166079, 4157, 98, 1, 2147483647, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[166226, 4164, 251, 3, 60, "Subsubsection"],
Cell[166480, 4169, 165, 3, 2147483647, "Text"],
Cell[166648, 4174, 121, 2, 2147483647, "Input"]
}, Closed]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[166830, 4183, 580, 7, 140, "Section"],
Cell[CellGroupData[{
Cell[167435, 4194, 336, 4, 95, "Subsection"],
Cell[167774, 4200, 408, 12, 124, "Text"],
Cell[168185, 4214, 461, 13, 120, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[168671, 4231, 250, 6, 120, "Input"],
Cell[168924, 4239, 379, 12, 2147483647, "Output"]
}, Closed]],
Cell[169318, 4254, 1767, 34, 480, "Text"],
Cell[CellGroupData[{
Cell[171110, 4292, 111, 2, 120, "Input"],
Cell[171224, 4296, 585, 9, 2147483647, "Print",
 CellTags->"Info3627286815-6629669"]
}, Closed]],
Cell[171824, 4308, 1106, 18, 252, "Text"],
Cell[172933, 4328, 1885, 58, 502, "Input",
 Evaluatable->False],
Cell[174821, 4388, 159, 5, 120, "Text"],
Cell[CellGroupData[{
Cell[175005, 4397, 154, 3, 120, "Input"],
Cell[175162, 4402, 124, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[175323, 4410, 176, 4, 88, "Input"],
Cell[175502, 4416, 101, 2, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[175640, 4423, 224, 5, 88, "Input"],
Cell[175867, 4430, 117, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[176021, 4438, 249, 5, 88, "Input"],
Cell[176273, 4445, 94, 2, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[176404, 4452, 175, 4, 88, "Input"],
Cell[176582, 4458, 115, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[176734, 4466, 199, 5, 88, "Input"],
Cell[176936, 4473, 117, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[177090, 4481, 143, 3, 88, "Input"],
Cell[177236, 4486, 72, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[177345, 4492, 283, 5, 88, "Input"],
Cell[177631, 4499, 92, 2, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[177760, 4506, 406, 7, 88, "Input"],
Cell[178169, 4515, 117, 3, 2147483647, "Output"]
}, Closed]],
Cell[178301, 4521, 321, 8, 100, "Text"],
Cell[178625, 4531, 113, 2, 120, "Input"],
Cell[CellGroupData[{
Cell[178763, 4537, 253, 7, 120, "Input"],
Cell[179019, 4546, 511, 19, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[179567, 4570, 175, 4, 88, "Input"],
Cell[179745, 4576, 511, 19, 2147483647, "Output"]
}, Closed]],
Cell[180271, 4598, 353, 6, 176, "Text"],
Cell[CellGroupData[{
Cell[180649, 4608, 377, 9, 120, "Input"],
Cell[181029, 4619, 541, 20, 2147483647, "Output"]
}, Closed]],
Cell[181585, 4642, 138, 1, 100, "Text"],
Cell[CellGroupData[{
Cell[181748, 4647, 283, 7, 120, "Input"],
Cell[182034, 4656, 1748, 63, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[183819, 4724, 294, 8, 88, "Input"],
Cell[184116, 4734, 73, 1, 2147483647, "Output"]
}, Closed]],
Cell[184204, 4738, 121, 1, 100, "Text"],
Cell[184328, 4741, 300, 8, 120, "Input"],
Cell[CellGroupData[{
Cell[184653, 4753, 232, 6, 120, "Input"],
Cell[184888, 4761, 2134, 75, 2147483647, "Output"]
}, Closed]],
Cell[187037, 4839, 229, 5, 100, "Text"],
Cell[CellGroupData[{
Cell[187291, 4848, 371, 10, 120, "Input"],
Cell[187665, 4860, 453, 9, 2147483647, "Message"],
Cell[188121, 4871, 561, 16, 2147483647, "Output"]
}, Closed]],
Cell[188697, 4890, 149, 2, 100, "Text"],
Cell[CellGroupData[{
Cell[188871, 4896, 555, 16, 120, "Input"],
Cell[189429, 4914, 2134, 75, 2147483647, "Output"]
}, Closed]],
Cell[191578, 4992, 203, 4, 176, "Text"],
Cell[191784, 4998, 140, 1, 120, "Text"],
Cell[191927, 5001, 428, 11, 120, "Input",
 Evaluatable->False],
Cell[192358, 5014, 271, 5, 120, "Text"],
Cell[192632, 5021, 365, 10, 120, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[193022, 5035, 513, 12, 120, "Input"],
Cell[193538, 5049, 757, 27, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[194332, 5081, 277, 8, 88, "Input"],
Cell[194612, 5091, 2033, 72, 2147483647, "Output"]
}, Closed]],
Cell[196660, 5166, 119, 1, 100, "Text"],
Cell[196782, 5169, 105, 3, 120, "Input"],
Cell[CellGroupData[{
Cell[196912, 5176, 838, 24, 120, "Input"],
Cell[197753, 5202, 75, 1, 2147483647, "Output"]
}, Closed]],
Cell[197843, 5206, 155, 2, 100, "Text"],
Cell[CellGroupData[{
Cell[198023, 5212, 1001, 28, 120, "Input"],
Cell[199027, 5242, 889, 15, 2147483647, "Message"],
Cell[199919, 5259, 889, 15, 2147483647, "Message"],
Cell[200811, 5276, 3664, 129, 2147483647, "Output"]
}, Closed]],
Cell[204490, 5408, 125, 1, 100, "Text"],
Cell[CellGroupData[{
Cell[204640, 5413, 902, 25, 120, "Input"],
Cell[205545, 5440, 75, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[205657, 5446, 262, 6, 88, "Input"],
Cell[205922, 5454, 2176, 73, 2147483647, "Output"]
}, Closed]],
Cell[208113, 5530, 602, 11, 100, "Text"],
Cell[CellGroupData[{
Cell[208740, 5545, 416, 8, 120, "Input"],
Cell[209159, 5555, 72, 1, 2147483647, "Output"]
}, Closed]],
Cell[209246, 5559, 1816, 43, 480, "Text"],
Cell[211065, 5604, 753, 14, 120, "Text"],
Cell[CellGroupData[{
Cell[211843, 5622, 180, 4, 120, "Input"],
Cell[212026, 5628, 334, 10, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[212397, 5643, 139, 4, 88, "Input"],
Cell[212539, 5649, 773, 25, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[213349, 5679, 234, 6, 88, "Input"],
Cell[213586, 5687, 406, 13, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[214029, 5705, 478, 10, 88, "Input"],
Cell[214510, 5717, 94, 1, 2147483647, "Output"]
}, Closed]],
Cell[214619, 5721, 218, 4, 100, "Text"],
Cell[214840, 5727, 542, 15, 120, "Input"],
Cell[215385, 5744, 599, 15, 120, "Input"],
Cell[215987, 5761, 277, 6, 120, "Input"],
Cell[216267, 5769, 369, 8, 120, "Text"],
Cell[CellGroupData[{
Cell[216661, 5781, 313, 5, 120, "Input"],
Cell[216977, 5788, 409, 6, 2147483647, "Print",
 CellTags->"Info3627289761-6629669"]
}, Closed]],
Cell[217401, 5797, 1753, 46, 242, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[219179, 5847, 126, 2, 120, "Input"],
Cell[219308, 5851, 91, 1, 2147483647, "Output"]
}, Closed]],
Cell[219414, 5855, 104, 1, 100, "Text"],
Cell[CellGroupData[{
Cell[219543, 5860, 715, 13, 120, "Input",
 InitializationCell->True],
Cell[220261, 5875, 1054, 24, 140, "Output"]
}, Closed]],
Cell[221330, 5902, 106, 1, 100, "Text"],
Cell[CellGroupData[{
Cell[221461, 5907, 302, 7, 120, "Input"],
Cell[221766, 5916, 811, 30, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[222614, 5951, 639, 12, 88, "Input",
 InitializationCell->True],
Cell[223256, 5965, 1616, 45, 140, "Output"]
}, Closed]],
Cell[224887, 6013, 416, 7, 100, "Text"],
Cell[CellGroupData[{
Cell[225328, 6024, 431, 13, 120, "Input"],
Cell[225762, 6039, 467, 14, 2147483647, "Output"]
}, Closed]],
Cell[226244, 6056, 192, 2, 100, "Text"],
Cell[CellGroupData[{
Cell[226461, 6062, 184, 4, 120, "Input"],
Cell[226648, 6068, 147, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[226832, 6076, 317, 9, 88, "Input"],
Cell[227152, 6087, 125, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[227314, 6095, 340, 8, 88, "Input"],
Cell[227657, 6105, 227, 7, 2147483647, "Output"]
}, Closed]],
Cell[227899, 6115, 572, 10, 252, "Text"],
Cell[228474, 6127, 266, 5, 120, "Text"],
Cell[228743, 6134, 123, 2, 120, "Input"],
Cell[228869, 6138, 421, 9, 120, "Input"],
Cell[229293, 6149, 147, 3, 120, "Input"],
Cell[CellGroupData[{
Cell[229465, 6156, 184, 5, 120, "Input"],
Cell[229652, 6163, 283, 8, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[229972, 6176, 747, 19, 166, "Input"],
Cell[230722, 6197, 348, 10, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[231107, 6212, 200, 4, 88, "Input"],
Cell[231310, 6218, 127, 3, 2147483647, "Output"]
}, Closed]],
Cell[231452, 6224, 762, 17, 88, "Input"],
Cell[232217, 6243, 175, 4, 120, "Input"],
Cell[232395, 6249, 927, 17, 348, "Text"],
Cell[233325, 6268, 899, 25, 198, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[234249, 6297, 293, 5, 120, "Input"],
Cell[234545, 6304, 281, 5, 2147483647, "Print",
 CellTags->"Info3627108654-6604827"]
}, Closed]],
Cell[CellGroupData[{
Cell[234863, 6314, 306, 7, 88, "Input"],
Cell[235172, 6323, 1847, 66, 2147483647, "Output"]
}, Closed]],
Cell[237034, 6392, 445, 9, 176, "Text"],
Cell[237482, 6403, 982, 26, 198, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[238489, 6433, 246, 4, 120, "Input"],
Cell[238738, 6439, 313, 5, 2147483647, "Print",
 CellTags->"Info3627290180-6629669"]
}, Closed]],
Cell[CellGroupData[{
Cell[239088, 6449, 322, 9, 88, "Input"],
Cell[239413, 6460, 451, 14, 2147483647, "Output"]
}, Closed]],
Cell[239879, 6477, 1266, 30, 166, "Input"],
Cell[241148, 6509, 676, 16, 120, "Input"],
Cell[241827, 6527, 554, 11, 120, "Input"],
Cell[CellGroupData[{
Cell[242406, 6542, 539, 11, 120, "Input",
 InitializationCell->True],
Cell[242948, 6555, 2927, 90, 140, "Output"]
}, Closed]],
Cell[245890, 6648, 301, 6, 176, "Text"],
Cell[CellGroupData[{
Cell[246216, 6658, 295, 8, 120, "Input"],
Cell[246514, 6668, 299, 9, 2147483647, "Output"]
}, Closed]],
Cell[246828, 6680, 624, 11, 176, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[247489, 6696, 543, 7, 75, "Subsection"],
Cell[CellGroupData[{
Cell[248057, 6707, 313, 4, 76, "Subsubsection"],
Cell[248373, 6713, 146, 3, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[248544, 6720, 144, 3, 2147483647, "Input"],
Cell[248691, 6725, 377, 12, 2147483647, "Output"]
}, Closed]],
Cell[249083, 6740, 114, 1, 2147483647, "Text"],
Cell[249200, 6743, 1232, 32, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[250457, 6779, 387, 8, 2147483647, "Input"],
Cell[250847, 6789, 3734, 130, 2147483647, "Output"]
}, Closed]],
Cell[254596, 6922, 317, 8, 2147483647, "Text"],
Cell[254916, 6932, 1247, 35, 2147483647, "Input",
 Evaluatable->False],
Cell[256166, 6969, 1289, 36, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[257480, 7009, 346, 5, 2147483647, "Input"],
Cell[257829, 7016, 324, 6, 2147483647, "Print",
 CellTags->"Info3627309622-3465247"]
}, Closed]],
Cell[CellGroupData[{
Cell[258190, 7027, 519, 10, 2147483647, "Input"],
Cell[258712, 7039, 16648, 512, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[275397, 7556, 495, 10, 2147483647, "Input"],
Cell[275895, 7568, 24540, 749, 2147483647, "Output"]
}, Closed]],
Cell[300450, 8320, 151, 2, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[300626, 8326, 952, 22, 2147483647, "Input"],
Cell[301581, 8350, 328, 9, 2147483647, "Output"]
}, Closed]],
Cell[301924, 8362, 122, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[302071, 8367, 522, 13, 2147483647, "Input"],
Cell[302596, 8382, 94, 1, 2147483647, "Output"]
}, Closed]],
Cell[302705, 8386, 411, 10, 2147483647, "Text"],
Cell[303119, 8398, 291, 7, 2147483647, "Text"],
Cell[303413, 8407, 1684, 43, 2147483647, "Input",
 InitializationCell->True],
Cell[305100, 8452, 667, 12, 2147483647, "Input",
 InitializationCell->True],
Cell[305770, 8466, 388, 7, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[306183, 8477, 1165, 26, 2147483647, "Input"],
Cell[307351, 8505, 451, 8, 2147483647, "Message"],
Cell[307805, 8515, 32491, 875, 2147483647, "Output"]
}, Closed]],
Cell[340311, 9393, 212, 7, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[340548, 9404, 178, 3, 2147483647, "Input"],
Cell[340729, 9409, 295, 5, 2147483647, "Print",
 CellTags->"Info3627309626-3465247"]
}, Closed]],
Cell[CellGroupData[{
Cell[341061, 9419, 210, 5, 2147483647, "Input"],
Cell[341274, 9426, 376, 9, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[341687, 9440, 901, 19, 2147483647, "Input"],
Cell[342591, 9461, 951, 29, 2147483647, "Output"]
}, Closed]],
Cell[343557, 9493, 561, 7, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[344143, 9504, 421, 9, 2147483647, "Input"],
Cell[344567, 9515, 512, 17, 2147483647, "Output"]
}, Closed]],
Cell[345094, 9535, 301, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[345420, 9544, 1376, 29, 2147483647, "Input"],
Cell[346799, 9575, 582, 11, 2147483647, "Message"],
Cell[347384, 9588, 298, 7, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[347719, 9600, 140, 4, 2147483647, "Input"],
Cell[347862, 9606, 151, 3, 2147483647, "Output"]
}, Closed]],
Cell[348028, 9612, 899, 20, 2147483647, "Input"],
Cell[348930, 9634, 137, 3, 2147483647, "Input"],
Cell[349070, 9639, 454, 12, 2147483647, "Input"],
Cell[349527, 9653, 145, 3, 2147483647, "Input"],
Cell[349675, 9658, 522, 13, 2147483647, "Input"],
Cell[350200, 9673, 142, 3, 2147483647, "Input"],
Cell[350345, 9678, 561, 15, 2147483647, "Input"],
Cell[350909, 9695, 148, 3, 2147483647, "Input"],
Cell[351060, 9700, 330, 7, 2147483647, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[351427, 9712, 285, 4, 60, "Subsubsection"],
Cell[351715, 9718, 387, 10, 2147483647, "Input"],
Cell[352105, 9730, 237, 4, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[352367, 9738, 333, 8, 2147483647, "Input"],
Cell[352703, 9748, 199, 7, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[352939, 9760, 277, 6, 2147483647, "Input"],
Cell[353219, 9768, 254, 10, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[353510, 9783, 299, 7, 2147483647, "Input"],
Cell[353812, 9792, 275, 11, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[354124, 9808, 1096, 25, 2147483647, "Input"],
Cell[355223, 9835, 304, 9, 2147483647, "Output"]
}, Closed]],
Cell[355542, 9847, 192, 2, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[355759, 9853, 503, 13, 2147483647, "Input"],
Cell[356265, 9868, 304, 9, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[356606, 9882, 513, 13, 2147483647, "Input"],
Cell[357122, 9897, 302, 9, 2147483647, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[357473, 9912, 417, 5, 60, "Subsubsection"],
Cell[357893, 9919, 137, 1, 2147483647, "Text"],
Cell[358033, 9922, 507, 11, 2147483647, "Input",
 InitializationCell->True],
Cell[358543, 9935, 175, 2, 2147483647, "Text"],
Cell[358721, 9939, 597, 13, 2147483647, "Input",
 InitializationCell->True],
Cell[359321, 9954, 110, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[359456, 9959, 323, 9, 2147483647, "Input"],
Cell[359782, 9970, 3082, 97, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[362901, 10072, 1248, 28, 2147483647, "Input"],
Cell[364152, 10102, 125, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[364314, 10110, 696, 17, 2147483647, "Input"],
Cell[365013, 10129, 75, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[365125, 10135, 117, 2, 2147483647, "Input"],
Cell[365245, 10139, 281, 5, 2147483647, "Print",
 CellTags->"Info3627312169-3465247"]
}, Closed]],
Cell[365541, 10147, 528, 9, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[366094, 10160, 853, 20, 2147483647, "Input"],
Cell[366950, 10182, 3851, 137, 2147483647, "Output"]
}, Closed]],
Cell[370816, 10322, 432, 7, 2147483647, "Text"],
Cell[371251, 10331, 2064, 55, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[373340, 10390, 294, 5, 2147483647, "Input"],
Cell[373637, 10397, 503, 8, 2147483647, "Print",
 CellTags->"Info3627312345-3465247"]
}, Closed]],
Cell[CellGroupData[{
Cell[374177, 10410, 228, 7, 2147483647, "Input"],
Cell[374408, 10419, 860, 32, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[375305, 10456, 277, 8, 2147483647, "Input"],
Cell[375585, 10466, 1541, 59, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[377163, 10530, 533, 12, 2147483647, "Input",
 InitializationCell->True],
Cell[377699, 10544, 907, 16, 140, "Print"]
}, Closed]],
Cell[CellGroupData[{
Cell[378643, 10565, 583, 12, 2147483647, "Input",
 InitializationCell->True],
Cell[379229, 10579, 1931, 67, 140, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[381197, 10651, 564, 15, 2147483647, "Input"],
Cell[381764, 10668, 75, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[381876, 10674, 1063, 29, 2147483647, "Input"],
Cell[382942, 10705, 75, 1, 2147483647, "Output"]
}, Closed]],
Cell[383032, 10709, 2126, 45, 2147483647, "Text",
 CellTags->"g(i"],
Cell[385161, 10756, 2231, 55, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[387417, 10815, 246, 4, 2147483647, "Input"],
Cell[387666, 10821, 342, 6, 2147483647, "Print",
 CellTags->"Info3627312854-3465247"]
}, Closed]],
Cell[388023, 10830, 209, 3, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[388257, 10837, 117, 2, 2147483647, "Input"],
Cell[388377, 10841, 297, 5, 2147483647, "Print",
 CellTags->"Info3627312921-3465247"]
}, Closed]],
Cell[CellGroupData[{
Cell[388711, 10851, 709, 18, 2147483647, "Input"],
Cell[389423, 10871, 365, 11, 2147483647, "Output"]
}, Closed]],
Cell[389803, 10885, 151, 2, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[389979, 10891, 1030, 29, 2147483647, "Input"],
Cell[391012, 10922, 73, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[391122, 10928, 533, 16, 2147483647, "Input"],
Cell[391658, 10946, 778, 23, 2147483647, "Output"]
}, Closed]],
Cell[392451, 10972, 225, 5, 2147483647, "Text"],
Cell[392679, 10979, 534, 17, 2147483647, "Input",
 InitializationCell->True],
Cell[393216, 10998, 339, 8, 2147483647, "Input",
 InitializationCell->True],
Cell[393558, 11008, 346, 6, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[393929, 11018, 672, 20, 2147483647, "Input"],
Cell[394604, 11040, 75, 1, 2147483647, "Output"]
}, Closed]],
Cell[394694, 11044, 341, 6, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[395060, 11054, 582, 16, 2147483647, "Input"],
Cell[395645, 11072, 5969, 204, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[401651, 11281, 1175, 30, 2147483647, "Input"],
Cell[402829, 11313, 75, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[402941, 11319, 1301, 31, 2147483647, "Input"],
Cell[404245, 11352, 75, 1, 2147483647, "Output"]
}, Closed]],
Cell[404335, 11356, 464, 13, 2147483647, "Text"],
Cell[404802, 11371, 1148, 26, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[405975, 11401, 274, 5, 2147483647, "Input",
 InitializationCell->True],
Cell[406252, 11408, 229, 4, 140, "Print",
 CellTags->"Info3629190373-5022241"]
}, Closed]],
Cell[406496, 11415, 530, 9, 2147483647, "Text"],
Cell[407029, 11426, 168, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[407222, 11435, 386, 9, 2147483647, "Input"],
Cell[407611, 11446, 269, 9, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[407917, 11460, 434, 9, 2147483647, "Input"],
Cell[408354, 11471, 141, 5, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[408532, 11481, 800, 19, 2147483647, "Input"],
Cell[409335, 11502, 144, 4, 2147483647, "Output"]
}, Closed]],
Cell[409494, 11509, 101, 1, 2147483647, "Text"],
Cell[409598, 11512, 802, 14, 2147483647, "Text"],
Cell[410403, 11528, 164, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[410592, 11537, 306, 8, 2147483647, "Input"],
Cell[410901, 11547, 187, 6, 2147483647, "Output"]
}, Closed]],
Cell[411103, 11556, 199, 2, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[411327, 11562, 359, 10, 2147483647, "Input"],
Cell[411689, 11574, 132, 3, 2147483647, "Output"]
}, Closed]],
Cell[411836, 11580, 190, 4, 2147483647, "Text"],
Cell[412029, 11586, 651, 17, 2147483647, "Input"],
Cell[412683, 11605, 632, 15, 2147483647, "Input"],
Cell[413318, 11622, 101, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[413444, 11627, 306, 8, 2147483647, "Input"],
Cell[413753, 11637, 233, 8, 2147483647, "Output"]
}, Closed]],
Cell[414001, 11648, 140, 2, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[414166, 11654, 689, 15, 2147483647, "Input"],
Cell[414858, 11671, 130, 3, 2147483647, "Output"]
}, Closed]],
Cell[415003, 11677, 261, 3, 2147483647, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[415301, 11685, 673, 9, 60, "Subsubsection"],
Cell[415977, 11696, 181, 2, 2147483647, "Text"],
Cell[416161, 11700, 129, 2, 2147483647, "Input"],
Cell[416293, 11704, 148, 3, 2147483647, "Input"],
Cell[416444, 11709, 481, 13, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[416950, 11726, 148, 3, 2147483647, "Input"],
Cell[417101, 11731, 411, 12, 2147483647, "Output"]
}, Closed]],
Cell[417527, 11746, 226, 5, 2147483647, "Text"],
Cell[417756, 11753, 736, 20, 2147483647, "Input",
 Evaluatable->False],
Cell[418495, 11775, 516, 14, 2147483647, "Input"],
Cell[419014, 11791, 607, 12, 2147483647, "Text"],
Cell[419624, 11805, 388, 12, 2147483647, "Input",
 CellTags->"ptr"],
Cell[420015, 11819, 296, 7, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[420336, 11830, 283, 5, 2147483647, "Input"],
Cell[420622, 11837, 655, 18, 2147483647, "Output"]
}, Closed]],
Cell[421292, 11858, 264, 3, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[421581, 11865, 586, 15, 2147483647, "Input"],
Cell[422170, 11882, 1310, 37, 2147483647, "Output"]
}, Closed]],
Cell[423495, 11922, 264, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[423784, 11931, 261, 5, 2147483647, "Input"],
Cell[424048, 11938, 10021, 303, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[434106, 12246, 1483, 36, 2147483647, "Input"],
Cell[435592, 12284, 125, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[435754, 12292, 883, 25, 2147483647, "Input"],
Cell[436640, 12319, 127, 3, 2147483647, "Output"]
}, Closed]],
Cell[436782, 12325, 462, 8, 2147483647, "Text"],
Cell[437247, 12335, 231, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[437503, 12344, 210, 5, 2147483647, "Input"],
Cell[437716, 12351, 1515, 50, 2147483647, "Output"]
}, Closed]],
Cell[439246, 12404, 185, 4, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[439456, 12412, 278, 8, 2147483647, "Input"],
Cell[439737, 12422, 997, 37, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[440771, 12464, 448, 11, 2147483647, "Input"],
Cell[441222, 12477, 1960, 62, 2147483647, "Output"]
}, Closed]],
Cell[443197, 12542, 337, 6, 2147483647, "Text"],
Cell[443537, 12550, 154, 3, 2147483647, "Input"],
Cell[443694, 12555, 465, 8, 2147483647, "Text"],
Cell[444162, 12565, 885, 21, 2147483647, "Input"],
Cell[445050, 12588, 159, 3, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[445234, 12595, 350, 10, 2147483647, "Input"],
Cell[445587, 12607, 6899, 240, 2147483647, "Output"]
}, Closed]],
Cell[452501, 12850, 146, 2, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[452672, 12856, 247, 5, 2147483647, "Input"],
Cell[452922, 12863, 6417, 215, 2147483647, "Output"]
}, Closed]],
Cell[459354, 13081, 580, 9, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[459959, 13094, 1919, 47, 2147483647, "Input"],
Cell[461881, 13143, 70, 1, 2147483647, "Output"]
}, Closed]],
Cell[461966, 13147, 162, 2, 2147483647, "Text"],
Cell[462131, 13151, 1756, 44, 2147483647, "Input"],
Cell[463890, 13197, 93, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[464008, 13202, 914, 25, 2147483647, "Input"],
Cell[464925, 13229, 69, 1, 2147483647, "Output"]
}, Closed]],
Cell[465009, 13233, 309, 5, 2147483647, "Text"],
Cell[465321, 13240, 2207, 53, 2147483647, "Input"],
Cell[467531, 13295, 91, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[467647, 13300, 805, 22, 2147483647, "Input"],
Cell[468455, 13324, 215, 7, 2147483647, "Output"]
}, Closed]],
Cell[468685, 13334, 591, 11, 2147483647, "Text"],
Cell[469279, 13347, 613, 16, 2147483647, "Input",
 Evaluatable->False],
Cell[469895, 13365, 789, 24, 2147483647, "Input"],
Cell[470687, 13391, 1157, 25, 2147483647, "Text"],
Cell[471847, 13418, 540, 16, 2147483647, "Input",
 Evaluatable->False],
Cell[472390, 13436, 1246, 19, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[473661, 13459, 291, 7, 2147483647, "Input"],
Cell[473955, 13468, 10443, 308, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[484435, 13781, 1535, 38, 2147483647, "Input"],
Cell[485973, 13821, 75, 1, 2147483647, "Output"]
}, Closed]],
Cell[486063, 13825, 897, 17, 2147483647, "Text"],
Cell[486963, 13844, 326, 4, 2147483647, "Text"],
Cell[487292, 13850, 217, 4, 2147483647, "Text"],
Cell[487512, 13856, 567, 16, 2147483647, "Input",
 Evaluatable->False],
Cell[488082, 13874, 528, 9, 2147483647, "Text"],
Cell[488613, 13885, 452, 10, 2147483647, "Text"],
Cell[489068, 13897, 372, 11, 2147483647, "Input"],
Cell[489443, 13910, 306, 6, 2147483647, "Text"],
Cell[489752, 13918, 526, 14, 2147483647, "Input"],
Cell[490281, 13934, 216, 5, 2147483647, "Text"],
Cell[490500, 13941, 280, 5, 2147483647, "Text"],
Cell[490783, 13948, 735, 19, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[491543, 13971, 239, 6, 2147483647, "Input"],
Cell[491785, 13979, 629, 18, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[492451, 14002, 930, 24, 2147483647, "Input"],
Cell[493384, 14028, 213, 7, 2147483647, "Output"]
}, Closed]],
Cell[493612, 14038, 437, 10, 2147483647, "Text"],
Cell[494052, 14050, 555, 16, 2147483647, "Input",
 Evaluatable->False],
Cell[494610, 14068, 644, 10, 2147483647, "Text"],
Cell[495257, 14080, 209, 5, 2147483647, "Input"],
Cell[495469, 14087, 1796, 32, 2147483647, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[497302, 14124, 559, 7, 60, "Subsubsection"],
Cell[497864, 14133, 297, 7, 2147483647, "Text"],
Cell[498164, 14142, 421, 12, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[498610, 14158, 206, 5, 2147483647, "Input"],
Cell[498819, 14165, 387, 12, 2147483647, "Output"]
}, Closed]],
Cell[499221, 14180, 728, 21, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[499974, 14205, 389, 6, 2147483647, "Input"],
Cell[500366, 14213, 357, 6, 2147483647, "Print",
 CellTags->"Info3627376031-6375012"]
}, Closed]],
Cell[CellGroupData[{
Cell[500760, 14224, 127, 2, 2147483647, "Input"],
Cell[500890, 14228, 89, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[501016, 14234, 307, 7, 2147483647, "Input"],
Cell[501326, 14243, 3680, 129, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[505043, 14377, 268, 7, 2147483647, "Input"],
Cell[505314, 14386, 3690, 129, 2147483647, "Output"]
}, Closed]],
Cell[509019, 14518, 888, 20, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[509932, 14542, 298, 5, 2147483647, "Input"],
Cell[510233, 14549, 349, 6, 2147483647, "Print",
 CellTags->"Info3627376078-6375012"]
}, Closed]],
Cell[510597, 14558, 129, 2, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[510751, 14564, 403, 8, 2147483647, "Input"],
Cell[511157, 14574, 46918, 1385, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[558112, 15964, 159, 3, 2147483647, "Input"],
Cell[558274, 15969, 70, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[558381, 15975, 485, 10, 2147483647, "Input"],
Cell[558869, 15987, 1502, 54, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[560408, 16046, 151, 3, 2147483647, "Input"],
Cell[560562, 16051, 1835, 63, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[562434, 16119, 205, 5, 2147483647, "Input"],
Cell[562642, 16126, 72, 1, 2147483647, "Output"]
}, Closed]]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[562775, 16134, 359, 5, 95, "Subsection"],
Cell[563137, 16141, 793, 15, 136, "Text"],
Cell[CellGroupData[{
Cell[563955, 16160, 393, 5, 76, "Subsubsection"],
Cell[564351, 16167, 572, 9, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[564948, 16180, 271, 8, 2147483647, "Input"],
Cell[565222, 16190, 550, 20, 2147483647, "Output"]
}, Closed]],
Cell[565787, 16213, 346, 4, 2147483647, "Text"],
Cell[566136, 16219, 4264, 78, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[570425, 16301, 112, 2, 2147483647, "Input"],
Cell[570540, 16305, 585, 9, 2147483647, "Print",
 CellTags->"Info3627376383-6375012"]
}, Closed]],
Cell[CellGroupData[{
Cell[571162, 16319, 181, 4, 2147483647, "Input"],
Cell[571346, 16325, 802, 27, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[572185, 16357, 253, 6, 2147483647, "Input"],
Cell[572441, 16365, 322, 10, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[572800, 16380, 224, 5, 2147483647, "Input"],
Cell[573027, 16387, 1774, 63, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[574838, 16455, 171, 4, 2147483647, "Input"],
Cell[575012, 16461, 139, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[575188, 16469, 307, 7, 2147483647, "Input"],
Cell[575498, 16478, 120, 2, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[575655, 16485, 440, 10, 2147483647, "Input"],
Cell[576098, 16497, 933, 32, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[577068, 16534, 476, 9, 2147483647, "Input"],
Cell[577547, 16545, 933, 32, 2147483647, "Output"]
}, Closed]],
Cell[578495, 16580, 263, 6, 2147483647, "Text"],
Cell[578761, 16588, 148, 3, 2147483647, "Input"],
Cell[578912, 16593, 172, 4, 2147483647, "Input"],
Cell[579087, 16599, 247, 3, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[579359, 16606, 630, 16, 2147483647, "Input"],
Cell[579992, 16624, 126, 2, 2147483647, "Output"]
}, Closed]],
Cell[580133, 16629, 255, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[580413, 16638, 230, 6, 2147483647, "Input"],
Cell[580646, 16646, 1822, 66, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[582505, 16717, 804, 24, 2147483647, "Input"],
Cell[583312, 16743, 73, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[583422, 16749, 1125, 32, 2147483647, "Input"],
Cell[584550, 16783, 72, 1, 2147483647, "Output"]
}, Closed]],
Cell[584637, 16787, 630, 16, 2147483647, "Input"],
Cell[585270, 16805, 168, 2, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[585463, 16811, 745, 20, 2147483647, "Input"],
Cell[586211, 16833, 1748, 63, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[587996, 16901, 206, 5, 2147483647, "Input"],
Cell[588205, 16908, 75, 1, 2147483647, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[588329, 16915, 325, 4, 60, "Subsubsection"],
Cell[588657, 16921, 364, 6, 2147483647, "Text"],
Cell[589024, 16929, 142, 3, 2147483647, "Input"],
Cell[589169, 16934, 131, 2, 2147483647, "Input"],
Cell[589303, 16938, 202, 5, 2147483647, "Input"],
Cell[589508, 16945, 275, 7, 2147483647, "Input"],
Cell[589786, 16954, 127, 2, 2147483647, "Input"],
Cell[589916, 16958, 148, 3, 2147483647, "Input"],
Cell[590067, 16963, 219, 2, 2147483647, "Text"],
Cell[590289, 16967, 166, 5, 2147483647, "Input"],
Cell[590458, 16974, 225, 6, 2147483647, "Input"],
Cell[590686, 16982, 624, 16, 2147483647, "Input"],
Cell[591313, 17000, 916, 23, 2147483647, "Input"],
Cell[592232, 17025, 256, 5, 2147483647, "Text"],
Cell[592491, 17032, 194, 5, 2147483647, "Input"],
Cell[592688, 17039, 222, 6, 2147483647, "Input"],
Cell[592913, 17047, 540, 12, 2147483647, "Input"],
Cell[593456, 17061, 487, 10, 2147483647, "Input"],
Cell[593946, 17073, 895, 24, 2147483647, "Input"],
Cell[594844, 17099, 128, 2, 2147483647, "Input"],
Cell[594975, 17103, 309, 6, 2147483647, "Text"],
Cell[595287, 17111, 833, 18, 2147483647, "Input",
 Evaluatable->False],
Cell[596123, 17131, 736, 16, 2147483647, "Input",
 Evaluatable->False],
Cell[596862, 17149, 288, 5, 2147483647, "Text"],
Cell[597153, 17156, 495, 12, 2147483647, "Input"],
Cell[597651, 17170, 202, 5, 2147483647, "Input"],
Cell[597856, 17177, 817, 19, 2147483647, "Input"],
Cell[598676, 17198, 120, 1, 2147483647, "Text"],
Cell[598799, 17201, 255, 7, 2147483647, "Input"],
Cell[599057, 17210, 266, 7, 2147483647, "Input"],
Cell[599326, 17219, 331, 8, 2147483647, "Input"],
Cell[599660, 17229, 317, 8, 2147483647, "Input"],
Cell[599980, 17239, 138, 4, 2147483647, "Text"],
Cell[600121, 17245, 366, 10, 2147483647, "Input"],
Cell[600490, 17257, 175, 4, 2147483647, "Input"],
Cell[600668, 17263, 131, 2, 2147483647, "Input"],
Cell[600802, 17267, 262, 7, 2147483647, "Input"],
Cell[601067, 17276, 172, 4, 2147483647, "Input"],
Cell[601242, 17282, 668, 14, 2147483647, "Input"],
Cell[601913, 17298, 253, 5, 2147483647, "Input"],
Cell[602169, 17305, 349, 7, 2147483647, "Input"],
Cell[602521, 17314, 483, 10, 2147483647, "Input"],
Cell[603007, 17326, 1395, 29, 2147483647, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[604439, 17360, 466, 6, 60, "Subsubsection"],
Cell[604908, 17368, 238, 4, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[605171, 17376, 224, 4, 2147483647, "Input"],
Cell[605398, 17382, 788, 15, 2147483647, "Print",
 CellTags->"Info3627445080-5102015"]
}, Closed]],
Cell[CellGroupData[{
Cell[606223, 17402, 214, 3, 2147483647, "Input"],
Cell[606440, 17407, 86, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[606563, 17413, 704, 15, 2147483647, "Input"],
Cell[607270, 17430, 284, 8, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[607591, 17443, 127, 2, 2147483647, "Input"],
Cell[607721, 17447, 2241, 79, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[609999, 17531, 117, 2, 2147483647, "Input"],
Cell[610119, 17535, 529, 12, 2147483647, "Print",
 CellTags->"Info3627445143-5102015"]
}, Closed]],
Cell[610663, 17550, 237, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[610925, 17559, 550, 14, 2147483647, "Input"],
Cell[611478, 17575, 955, 37, 2147483647, "Output"]
}, Closed]],
Cell[612448, 17615, 160, 2, 2147483647, "Text"],
Cell[612611, 17619, 454, 9, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[613090, 17632, 118, 2, 2147483647, "Input"],
Cell[613211, 17636, 319, 6, 2147483647, "Print",
 CellTags->"Info3627445604-5102015"]
}, Closed]],
Cell[CellGroupData[{
Cell[613567, 17647, 97, 2, 2147483647, "Input"],
Cell[613667, 17651, 72, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[613776, 17657, 132, 2, 2147483647, "Input"],
Cell[613911, 17661, 87, 1, 2147483647, "Output"]
}, Closed]],
Cell[614013, 17665, 239, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[614277, 17674, 180, 4, 2147483647, "Input"],
Cell[614460, 17680, 161, 4, 2147483647, "Output"]
}, Closed]],
Cell[614636, 17687, 124, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[614785, 17692, 209, 5, 2147483647, "Input"],
Cell[614997, 17699, 122, 3, 2147483647, "Output"]
}, Closed]],
Cell[615134, 17705, 89, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[615248, 17710, 389, 10, 2147483647, "Input"],
Cell[615640, 17722, 616, 22, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[616293, 17749, 465, 11, 2147483647, "Input"],
Cell[616761, 17762, 689, 25, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[617487, 17792, 237, 6, 2147483647, "Input"],
Cell[617727, 17800, 2531, 87, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[620295, 17892, 549, 15, 2147483647, "Input"],
Cell[620847, 17909, 75, 1, 2147483647, "Output"]
}, Closed]],
Cell[620937, 17913, 259, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[621221, 17922, 475, 14, 2147483647, "Input"],
Cell[621699, 17938, 427, 15, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[622163, 17958, 464, 13, 2147483647, "Input"],
Cell[622630, 17973, 425, 15, 2147483647, "Output"]
}, Closed]],
Cell[623070, 17991, 121, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[623216, 17996, 537, 12, 2147483647, "Input"],
Cell[623756, 18010, 6856, 223, 2147483647, "Output"]
}, Closed]],
Cell[630627, 18236, 144, 2, 2147483647, "Text"],
Cell[630774, 18240, 731, 19, 2147483647, "Input"],
Cell[631508, 18261, 667, 16, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[632200, 18281, 176, 3, 2147483647, "Input"],
Cell[632379, 18286, 73, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[632489, 18292, 480, 13, 2147483647, "Input"],
Cell[632972, 18307, 70, 1, 2147483647, "Output"]
}, Closed]],
Cell[633057, 18311, 96, 1, 2147483647, "Text"],
Cell[633156, 18314, 117, 2, 2147483647, "Input"],
Cell[633276, 18318, 135, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[633436, 18323, 321, 9, 2147483647, "Input"],
Cell[633760, 18334, 463, 12, 2147483647, "Output"]
}, Closed]],
Cell[634238, 18349, 114, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[634377, 18354, 205, 3, 2147483647, "Input"],
Cell[634585, 18359, 2185, 78, 2147483647, "Output"]
}, Closed]],
Cell[636785, 18440, 116, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[636926, 18445, 347, 10, 2147483647, "Input"],
Cell[637276, 18457, 2241, 79, 2147483647, "Output"]
}, Closed]],
Cell[639532, 18539, 169, 2, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[639726, 18545, 205, 4, 2147483647, "Input"],
Cell[639934, 18551, 73, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[640044, 18557, 293, 9, 2147483647, "Input"],
Cell[640340, 18568, 2185, 78, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[642562, 18651, 457, 12, 2147483647, "Input"],
Cell[643022, 18665, 117, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[643176, 18673, 437, 11, 2147483647, "Input"],
Cell[643616, 18686, 115, 3, 2147483647, "Output"]
}, Closed]],
Cell[643746, 18692, 645, 15, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[644416, 18711, 148, 3, 2147483647, "Input"],
Cell[644567, 18716, 236, 8, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[644840, 18729, 251, 8, 2147483647, "Input"],
Cell[645094, 18739, 2239, 79, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[647370, 18823, 1069, 27, 2147483647, "Input"],
Cell[648442, 18852, 2271, 79, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[650750, 18936, 1409, 37, 2147483647, "Input"],
Cell[652162, 18975, 75, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[652274, 18981, 1050, 28, 2147483647, "Input"],
Cell[653327, 19011, 213, 7, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[653577, 19023, 493, 12, 2147483647, "Input"],
Cell[654073, 19037, 236, 8, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[654346, 19050, 729, 18, 2147483647, "Input"],
Cell[655078, 19070, 236, 8, 2147483647, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[655363, 19084, 427, 5, 60, "Subsubsection"],
Cell[655793, 19091, 855, 17, 2147483647, "Text"],
Cell[656651, 19110, 4785, 118, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[661461, 19232, 289, 5, 1006632960, "Input"],
Cell[661753, 19239, 293, 5, 2147483647, "Print",
 CellTags->"Info3627446141-5102015"]
}, Closed]],
Cell[CellGroupData[{
Cell[662083, 19249, 129, 2, 738197504, "Input"],
Cell[662215, 19253, 89, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[662341, 19259, 392, 12, 738197504, "Input"],
Cell[CellGroupData[{
Cell[662758, 19275, 94, 1, 2147483647, "Print"],
Cell[662855, 19278, 92, 1, 2147483647, "Print"],
Cell[662950, 19281, 92, 1, 2147483647, "Print"],
Cell[663045, 19284, 92, 1, 2147483647, "Print"]
}, Closed]],
Cell[663152, 19288, 95, 2, 2147483647, "Output"]
}, Closed]],
Cell[663262, 19293, 233, 4, 1476395008, "Text"],
Cell[663498, 19299, 293, 6, 1644167168, "Text"],
Cell[CellGroupData[{
Cell[663816, 19309, 286, 8, 1006632960, "Input"],
Cell[664105, 19319, 261, 8, 1409286144, "Output"]
}, Closed]],
Cell[664381, 19330, 88, 1, 1006632960, "Text"],
Cell[CellGroupData[{
Cell[664494, 19335, 414, 11, 1006632960, "Input"],
Cell[664911, 19348, 276, 8, 1409286144, "Output"]
}, Closed]],
Cell[665202, 19359, 107, 1, 1006632960, "Text"],
Cell[CellGroupData[{
Cell[665334, 19364, 175, 4, 1006632960, "Input"],
Cell[665512, 19370, 252, 8, 1409286144, "Output"]
}, Closed]],
Cell[665779, 19381, 159, 5, 1006632960, "Text"],
Cell[665941, 19388, 418, 11, 1006632960, "Input"],
Cell[CellGroupData[{
Cell[666384, 19403, 253, 6, 1006632960, "Input"],
Cell[666640, 19411, 451, 14, 2147483647, "Output"]
}, Closed]],
Cell[667106, 19428, 315, 9, 738197504, "Input"],
Cell[CellGroupData[{
Cell[667446, 19441, 534, 14, 1006632960, "Input"],
Cell[667983, 19457, 97, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[668117, 19463, 197, 5, 738197504, "Input"],
Cell[668317, 19470, 143, 2, 721420288, "Print"]
}, Closed]],
Cell[CellGroupData[{
Cell[668497, 19477, 102, 3, 1006632960, "Input"],
Cell[668602, 19482, 301, 9, 1409286144, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[668940, 19496, 135, 4, 1006632960, "Input"],
Cell[669078, 19502, 97, 1, 1409286144, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[669212, 19508, 354, 8, 1006632960, "Input"],
Cell[669569, 19518, 146, 3, 1409286144, "Output"]
}, Closed]],
Cell[669730, 19524, 362, 6, 1644167168, "Text"],
Cell[CellGroupData[{
Cell[670117, 19534, 288, 8, 1006632960, "Input"],
Cell[670408, 19544, 305, 9, 1409286144, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[670750, 19558, 199, 5, 1006632960, "Input"],
Cell[670952, 19565, 349, 11, 1409286144, "Output"]
}, Closed]],
Cell[671316, 19579, 493, 9, 1644167168, "Text"],
Cell[671812, 19590, 213, 4, 1006632960, "Text"],
Cell[CellGroupData[{
Cell[672050, 19598, 457, 9, 1006632960, "Input"],
Cell[672510, 19609, 70, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[672617, 19615, 274, 7, 738197504, "Input"],
Cell[672894, 19624, 1748, 63, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[674679, 19692, 204, 5, 738197504, "Input"],
Cell[674886, 19699, 127, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[675050, 19707, 226, 5, 738197504, "Input"],
Cell[675279, 19714, 94, 1, 2147483647, "Print"]
}, Closed]],
Cell[CellGroupData[{
Cell[675410, 19720, 256, 6, 738197504, "Input"],
Cell[675669, 19728, 94, 1, 2147483647, "Print"]
}, Closed]],
Cell[CellGroupData[{
Cell[675800, 19734, 209, 5, 738197504, "Input"],
Cell[676012, 19741, 212, 6, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[676261, 19752, 177, 4, 738197504, "Input"],
Cell[676441, 19758, 94, 1, 2147483647, "Print"]
}, Closed]],
Cell[CellGroupData[{
Cell[676572, 19764, 314, 8, 738197504, "Input"],
Cell[676889, 19774, 404, 12, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[677330, 19791, 227, 5, 738197504, "Input"],
Cell[677560, 19798, 1811, 64, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[679408, 19867, 279, 6, 738197504, "Input"],
Cell[679690, 19875, 118, 1, 2147483647, "Print"]
}, Closed]],
Cell[CellGroupData[{
Cell[679845, 19881, 307, 7, 738197504, "Input"],
Cell[680155, 19890, 3865, 132, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[684057, 20027, 204, 5, 738197504, "Input"],
Cell[684264, 20034, 149, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[684450, 20042, 604, 16, 738197504, "Input"],
Cell[685057, 20060, 2342, 81, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[687436, 20146, 175, 4, 738197504, "Input"],
Cell[687614, 20152, 3127, 114, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[690778, 20271, 1009, 25, 738197504, "Input"],
Cell[691790, 20298, 149, 3, 2147483647, "Output"]
}, Closed]],
Cell[691954, 20304, 341, 6, 1476395008, "Text"],
Cell[692298, 20312, 521, 14, 1006632960, "Input",
 Evaluatable->False]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[692868, 20332, 418, 5, 95, "Subsection"],
Cell[CellGroupData[{
Cell[693311, 20341, 449, 6, 76, "Subsubsection"],
Cell[693763, 20349, 519, 9, 1644167168, "Text"],
Cell[694285, 20360, 2519, 50, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[696829, 20414, 119, 2, 1006632960, "Input"],
Cell[696951, 20418, 328, 6, 1946157056, "Print",
 CellTags->"Info3627459539-5102015"]
}, Closed]],
Cell[CellGroupData[{
Cell[697316, 20429, 133, 2, 1006632960, "Input"],
Cell[697452, 20433, 91, 1, 989855744, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[697580, 20439, 147, 2, 1006632960, "Input"],
Cell[697730, 20443, 70, 1, 989855744, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[697837, 20449, 400, 8, 1006632960, "Input"],
Cell[698240, 20459, 3744, 128, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[702021, 20592, 496, 10, 1006632960, "Input"],
Cell[702520, 20604, 5420, 165, 2147483647, "Output"]
}, Closed]],
Cell[707955, 20772, 105, 1, 1006632960, "Text"],
Cell[CellGroupData[{
Cell[708085, 20777, 514, 13, 2147483647, "Input"],
Cell[708602, 20792, 115, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[708754, 20800, 376, 8, 2147483647, "Input"],
Cell[709133, 20810, 117, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[709287, 20818, 428, 9, 2147483647, "Input"],
Cell[709718, 20829, 95, 1, 2147483647, "Print"]
}, Closed]],
Cell[CellGroupData[{
Cell[709850, 20835, 461, 10, 2147483647, "Input"],
Cell[710314, 20847, 93, 1, 2147483647, "Print"]
}, Closed]],
Cell[CellGroupData[{
Cell[710444, 20853, 296, 7, 2147483647, "Input"],
Cell[710743, 20862, 1462, 47, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[712242, 20914, 337, 9, 2147483647, "Input"],
Cell[712582, 20925, 290, 9, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[712909, 20939, 543, 15, 2147483647, "Input"],
Cell[713455, 20956, 115, 3, 2147483647, "Output"]
}, Closed]],
Cell[713585, 20962, 263, 7, 2147483647, "Text"],
Cell[713851, 20971, 1046, 29, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[714922, 21004, 1534, 41, 2147483647, "Input"],
Cell[716459, 21047, 147, 3, 2147483647, "Output"]
}, Closed]],
Cell[716621, 21053, 119, 1, 2147483647, "Text"],
Cell[716743, 21056, 500, 14, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[717268, 21074, 999, 32, 2147483647, "Input"],
Cell[718270, 21108, 73, 1, 2147483647, "Output"]
}, Closed]],
Cell[718358, 21112, 638, 18, 2147483647, "Input",
 Evaluatable->False],
Cell[718999, 21132, 138, 1, 2147483647, "Text"],
Cell[719140, 21135, 269, 8, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[719434, 21147, 1337, 37, 2147483647, "Input"],
Cell[720774, 21186, 146, 3, 2147483647, "Output"]
}, Closed]],
Cell[720935, 21192, 162, 4, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[721122, 21200, 783, 20, 2147483647, "Input"],
Cell[721908, 21222, 75, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[722020, 21228, 543, 14, 2147483647, "Input"],
Cell[722566, 21244, 8901, 262, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[731504, 21511, 648, 18, 2147483647, "Input"],
Cell[732155, 21531, 73, 1, 2147483647, "Output"]
}, Closed]],
Cell[732243, 21535, 516, 9, 2147483647, "Text"],
Cell[732762, 21546, 386, 9, 2147483647, "Text",
 CellTags->"bastr"],
Cell[733151, 21557, 645, 19, 2147483647, "Input",
 Evaluatable->False],
Cell[733799, 21578, 889, 25, 2147483647, "Input",
 Evaluatable->False],
Cell[734691, 21605, 154, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[734870, 21614, 2025, 51, 2147483647, "Input"],
Cell[736898, 21667, 75, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[737010, 21673, 1688, 39, 2147483647, "Input"],
Cell[738701, 21714, 73, 1, 2147483647, "Output"]
}, Closed]],
Cell[738789, 21718, 95, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[738909, 21723, 825, 22, 2147483647, "Input"],
Cell[739737, 21747, 73, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[739847, 21753, 679, 19, 2147483647, "Input"],
Cell[740529, 21774, 70, 1, 2147483647, "Output"]
}, Closed]],
Cell[740614, 21778, 282, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[740921, 21787, 230, 6, 2147483647, "Input"],
Cell[741154, 21795, 809, 30, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[742000, 21830, 230, 6, 2147483647, "Input"],
Cell[742233, 21838, 809, 30, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[743079, 21873, 789, 21, 2147483647, "Input"],
Cell[743871, 21896, 71, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[743979, 21902, 1818, 49, 2147483647, "Input"],
Cell[745800, 21953, 75, 1, 2147483647, "Output"]
}, Closed]],
Cell[745890, 21957, 177, 4, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[746092, 21965, 695, 19, 2147483647, "Input"],
Cell[746790, 21986, 70, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[746897, 21992, 1418, 37, 2147483647, "Input"],
Cell[748318, 22031, 69, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[748424, 22037, 1328, 36, 2147483647, "Input"],
Cell[749755, 22075, 98, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[749890, 22081, 596, 18, 2147483647, "Input"],
Cell[750489, 22101, 72, 1, 2147483647, "Output"]
}, Closed]],
Cell[750576, 22105, 217, 5, 2147483647, "Text"],
Cell[750796, 22112, 820, 19, 2147483647, "Input",
 Evaluatable->False],
Cell[751619, 22133, 735, 19, 2147483647, "Input",
 Evaluatable->False]
}, Closed]],
Cell[CellGroupData[{
Cell[752391, 22157, 329, 4, 60, "Subsubsection"],
Cell[752723, 22163, 245, 5, 2147483647, "Text"],
Cell[752971, 22170, 983, 24, 2147483647, "Input",
 Evaluatable->False],
Cell[753957, 22196, 398, 7, 2147483647, "Text"],
Cell[754358, 22205, 581, 13, 2147483647, "Input",
 Evaluatable->False],
Cell[754942, 22220, 324, 8, 2147483647, "Text"],
Cell[755269, 22230, 2684, 60, 2147483647, "Input",
 Evaluatable->False],
Cell[757956, 22292, 137, 3, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[758118, 22299, 339, 8, 2147483647, "Input"],
Cell[758460, 22309, 114, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[758611, 22317, 385, 8, 2147483647, "Input"],
Cell[758999, 22327, 96, 1, 2147483647, "Print"]
}, Closed]],
Cell[CellGroupData[{
Cell[759132, 22333, 476, 10, 2147483647, "Input"],
Cell[759611, 22345, 32624, 989, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[792272, 23339, 649, 16, 2147483647, "Input"],
Cell[792924, 23357, 117, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[793078, 23365, 405, 10, 2147483647, "Input"],
Cell[793486, 23377, 117, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[793640, 23385, 280, 7, 2147483647, "Input"],
Cell[793923, 23394, 117, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[794077, 23402, 286, 7, 2147483647, "Input"],
Cell[794366, 23411, 78337, 2329, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[872740, 25745, 601, 15, 2147483647, "Input"],
Cell[873344, 25762, 115, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[873496, 25770, 286, 7, 2147483647, "Input"],
Cell[873785, 25779, 20733, 635, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[894555, 26419, 508, 14, 2147483647, "Input"],
Cell[895066, 26435, 115, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[895218, 26443, 284, 7, 2147483647, "Input"],
Cell[895505, 26452, 117, 3, 2147483647, "Output"]
}, Closed]],
Cell[895637, 26458, 169, 2, 2147483647, "Text"],
Cell[895809, 26462, 1765, 56, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[897599, 26522, 393, 7, 2147483647, "Input",
 InitializationCell->True],
Cell[897995, 26531, 359, 6, 140, "Print",
 CellTags->"Info3629190373-5022241"]
}, Closed]],
Cell[898369, 26540, 342, 7, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[898736, 26551, 559, 17, 2147483647, "Input"],
Cell[899298, 26570, 148, 4, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[899483, 26579, 580, 16, 2147483647, "Input"],
Cell[900066, 26597, 136, 4, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[900239, 26606, 634, 17, 2147483647, "Input"],
Cell[900876, 26625, 146, 4, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[901059, 26634, 634, 17, 2147483647, "Input"],
Cell[901696, 26653, 146, 4, 2147483647, "Output"]
}, Closed]],
Cell[901857, 26660, 150, 3, 2147483647, "Input"],
Cell[902010, 26665, 297, 7, 2147483647, "Text"],
Cell[902310, 26674, 613, 14, 2147483647, "Input",
 Evaluatable->False],
Cell[902926, 26690, 589, 14, 2147483647, "Input",
 Evaluatable->False],
Cell[903518, 26706, 610, 15, 2147483647, "Input",
 Evaluatable->False],
Cell[904131, 26723, 617, 15, 2147483647, "Input",
 Evaluatable->False],
Cell[904751, 26740, 187, 6, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[904963, 26750, 1992, 66, 2147483647, "Input"],
Cell[906958, 26818, 124, 3, 2147483647, "Output"]
}, Closed]],
Cell[907097, 26824, 121, 2, 2147483647, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[907267, 26832, 272, 3, 95, "Subsection"],
Cell[CellGroupData[{
Cell[907564, 26839, 129, 2, 76, "Subsubsection",
 CellTags->"intcond"],
Cell[907696, 26843, 1459, 23, 2147483647, "Text"],
Cell[909158, 26868, 110, 2, 2147483647, "Input"],
Cell[909271, 26872, 368, 10, 2147483647, "Input",
 Evaluatable->False],
Cell[909642, 26884, 349, 9, 2147483647, "Input",
 Evaluatable->False],
Cell[909994, 26895, 418, 11, 2147483647, "Input",
 Evaluatable->False],
Cell[910415, 26908, 372, 10, 2147483647, "Input",
 Evaluatable->False],
Cell[910790, 26920, 370, 10, 2147483647, "Input",
 Evaluatable->False],
Cell[911163, 26932, 395, 10, 2147483647, "Input",
 Evaluatable->False],
Cell[911561, 26944, 334, 6, 2147483647, "Text"],
Cell[911898, 26952, 146, 3, 2147483647, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[912081, 26960, 329, 5, 60, "Subsubsection"],
Cell[912413, 26967, 139, 3, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[912577, 26974, 173, 5, 2147483647, "Input"],
Cell[912753, 26981, 29040, 987, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[941830, 27973, 799, 22, 2147483647, "Input"],
Cell[942632, 27997, 72, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[942741, 28003, 851, 23, 2147483647, "Input"],
Cell[943595, 28028, 70, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[943702, 28034, 1039, 30, 2147483647, "Input"],
Cell[944744, 28066, 72, 1, 2147483647, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[944865, 28073, 306, 4, 60, "Subsubsection"],
Cell[CellGroupData[{
Cell[945196, 28081, 847, 23, 2147483647, "Input"],
Cell[946046, 28106, 72, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[946155, 28112, 901, 24, 2147483647, "Input"],
Cell[947059, 28138, 72, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[947168, 28144, 897, 24, 2147483647, "Input"],
Cell[948068, 28170, 70, 1, 2147483647, "Output"]
}, Closed]]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[948199, 28178, 317, 4, 95, "Subsection"],
Cell[948519, 28184, 220, 4, 196, "Text"],
Cell[948742, 28190, 1601, 27, 504, "Text"],
Cell[950346, 28219, 2344, 33, 728, "Text"],
Cell[952693, 28254, 970, 24, 120, "Input",
 Evaluatable->False],
Cell[953666, 28280, 753, 20, 120, "Input",
 Evaluatable->False],
Cell[954422, 28302, 773, 21, 120, "Input",
 Evaluatable->False],
Cell[955198, 28325, 826, 21, 120, "Input",
 Evaluatable->False],
Cell[956027, 28348, 912, 15, 348, "Text"],
Cell[956942, 28365, 176, 5, 120, "Text"],
Cell[CellGroupData[{
Cell[957143, 28374, 568, 11, 120, "Input"],
Cell[957714, 28387, 215, 6, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[957966, 28398, 131, 2, 88, "Input"],
Cell[958100, 28402, 215, 6, 2147483647, "Output"]
}, Closed]],
Cell[958330, 28411, 205, 3, 100, "Text"],
Cell[CellGroupData[{
Cell[958560, 28418, 631, 12, 120, "Input"],
Cell[959194, 28432, 72, 1, 2147483647, "Output"]
}, Closed]],
Cell[959281, 28436, 633, 14, 88, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[959963, 28456, 272, 3, 140, "Section"],
Cell[CellGroupData[{
Cell[960260, 28463, 160, 2, 95, "Subsection"],
Cell[CellGroupData[{
Cell[960445, 28469, 376, 5, 76, "Subsubsection"],
Cell[960824, 28476, 1963, 34, 2147483647, "Text"],
Cell[962790, 28512, 778, 21, 2147483647, "Input",
 Evaluatable->False],
Cell[963571, 28535, 942, 17, 2147483647, "Text"],
Cell[964516, 28554, 576, 11, 2147483647, "Text",
 CellTags->"lem2.1.1"],
Cell[965095, 28567, 222, 5, 2147483647, "Text"],
Cell[965320, 28574, 277, 3, 2147483647, "Text"],
Cell[965600, 28579, 453, 7, 2147483647, "Text"],
Cell[966056, 28588, 663, 16, 2147483647, "Input",
 Evaluatable->False],
Cell[966722, 28606, 716, 11, 2147483647, "Text"],
Cell[967441, 28619, 445, 10, 2147483647, "Input",
 Evaluatable->False],
Cell[967889, 28631, 92, 1, 2147483647, "Text"],
Cell[967984, 28634, 1845, 48, 2147483647, "Input",
 Evaluatable->False],
Cell[969832, 28684, 425, 7, 2147483647, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[970294, 28696, 334, 4, 60, "Subsubsection"],
Cell[970631, 28702, 432, 10, 2147483647, "Text"],
Cell[971066, 28714, 457, 13, 2147483647, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[971548, 28731, 201, 5, 2147483647, "Input"],
Cell[971752, 28738, 644, 21, 2147483647, "Output"]
}, Closed]],
Cell[972411, 28762, 646, 16, 2147483647, "Text"],
Cell[973060, 28780, 289, 7, 2147483647, "Text"],
Cell[973352, 28789, 669, 15, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[974046, 28808, 124, 2, 2147483647, "Input"],
Cell[974173, 28812, 231, 4, 2147483647, "Print",
 CellTags->"Info3627711720-5071683"]
}, Closed]],
Cell[CellGroupData[{
Cell[974441, 28821, 154, 2, 2147483647, "Input"],
Cell[974598, 28825, 140, 2, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[974775, 28832, 171, 3, 2147483647, "Input"],
Cell[974949, 28837, 406, 11, 2147483647, "Output"]
}, Closed]],
Cell[975370, 28851, 432, 12, 2147483647, "Input"],
Cell[975805, 28865, 148, 3, 2147483647, "Input"],
Cell[975956, 28870, 210, 3, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[976191, 28877, 117, 2, 2147483647, "Input"],
Cell[976311, 28881, 433, 7, 2147483647, "Print",
 CellTags->"Info3627639472-8560585"]
}, Closed]],
Cell[CellGroupData[{
Cell[976781, 28893, 151, 3, 2147483647, "Input"],
Cell[976935, 28898, 263, 8, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[977235, 28911, 228, 5, 2147483647, "Input"],
Cell[977466, 28918, 244, 7, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[977747, 28930, 313, 7, 2147483647, "Input"],
Cell[978063, 28939, 437, 14, 2147483647, "Output"]
}, Closed]],
Cell[978515, 28956, 356, 10, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[978896, 28970, 731, 19, 2147483647, "Input"],
Cell[979630, 28991, 365, 12, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[980032, 29008, 522, 16, 2147483647, "Input"],
Cell[980557, 29026, 72, 1, 2147483647, "Output"]
}, Closed]],
Cell[980644, 29030, 777, 22, 2147483647, "Input"],
Cell[981424, 29054, 113, 1, 2147483647, "Text"],
Cell[981540, 29057, 772, 21, 2147483647, "Input"],
Cell[982315, 29080, 237, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[982577, 29089, 417, 9, 2147483647, "Input"],
Cell[982997, 29100, 656, 21, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[983690, 29126, 196, 4, 2147483647, "Input"],
Cell[983889, 29132, 295, 9, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[984221, 29146, 159, 3, 2147483647, "Input"],
Cell[984383, 29151, 337, 10, 2147483647, "Output"]
}, Closed]],
Cell[984735, 29164, 168, 2, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[984928, 29170, 182, 4, 2147483647, "Input"],
Cell[985113, 29176, 692, 23, 2147483647, "Output"]
}, Closed]],
Cell[985820, 29202, 133, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[985978, 29207, 301, 7, 2147483647, "Input"],
Cell[986282, 29216, 121, 2, 2147483647, "Output"]
}, Closed]],
Cell[986418, 29221, 122, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[986565, 29226, 303, 7, 2147483647, "Input"],
Cell[986871, 29235, 124, 2, 2147483647, "Output"]
}, Closed]],
Cell[987010, 29240, 248, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[987283, 29249, 718, 20, 2147483647, "Input"],
Cell[988004, 29271, 1298, 41, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[989339, 29317, 449, 11, 2147483647, "Input"],
Cell[989791, 29330, 224, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[990052, 29338, 554, 12, 2147483647, "Input"],
Cell[990609, 29352, 216, 4, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[990862, 29361, 1009, 29, 2147483647, "Input"],
Cell[991874, 29392, 174, 4, 2147483647, "Output"]
}, Closed]],
Cell[992063, 29399, 179, 2, 2147483647, "Text"],
Cell[992245, 29403, 429, 11, 2147483647, "Input"],
Cell[992677, 29416, 149, 3, 2147483647, "Input"],
Cell[992829, 29421, 112, 1, 2147483647, "Text"],
Cell[992944, 29424, 698, 19, 2147483647, "Input"],
Cell[993645, 29445, 140, 3, 2147483647, "Input"],
Cell[993788, 29450, 262, 6, 2147483647, "Input"],
Cell[994053, 29458, 210, 2, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[994288, 29464, 404, 11, 2147483647, "Input"],
Cell[994695, 29477, 1027, 34, 2147483647, "Output"]
}, Closed]],
Cell[995737, 29514, 149, 2, 2147483647, "Text"],
Cell[995889, 29518, 1204, 38, 2147483647, "Input"],
Cell[997096, 29558, 220, 4, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[997341, 29566, 148, 4, 2147483647, "Input"],
Cell[997492, 29572, 437, 9, 2147483647, "Message"],
Cell[997932, 29583, 475, 9, 2147483647, "Message"],
Cell[998410, 29594, 435, 9, 2147483647, "Message"],
Cell[998848, 29605, 462, 9, 2147483647, "Message"],
Cell[999313, 29616, 435, 9, 2147483647, "Message"],
Cell[999751, 29627, 517, 10, 2147483647, "Message"],
Cell[1000271, 29639, 462, 9, 2147483647, "Message"],
Cell[1000736, 29650, 519, 10, 2147483647, "Message"],
Cell[1001258, 29662, 164, 4, 2147483647, "Output"]
}, Closed]],
Cell[1001437, 29669, 176, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1001638, 29678, 467, 13, 2147483647, "Input"],
Cell[1002108, 29693, 94, 1, 2147483647, "Output"]
}, Closed]],
Cell[1002217, 29697, 238, 3, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1002480, 29704, 361, 10, 2147483647, "Input"],
Cell[1002844, 29716, 3532, 110, 2147483647, "Output"]
}, Closed]],
Cell[1006391, 29829, 107, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1006523, 29834, 131, 2, 2147483647, "Input"],
Cell[1006657, 29838, 70, 1, 2147483647, "Output"]
}, Closed]],
Cell[1006742, 29842, 139, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1006906, 29847, 724, 17, 2147483647, "Input"],
Cell[1007633, 29866, 972, 32, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1008642, 29903, 131, 2, 2147483647, "Input"],
Cell[1008776, 29907, 92, 1, 2147483647, "Output"]
}, Closed]],
Cell[1008883, 29911, 101, 1, 2147483647, "Text"],
Cell[1008987, 29914, 355, 7, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1009367, 29925, 409, 11, 2147483647, "Input"],
Cell[1009779, 29938, 75, 1, 2147483647, "Output"]
}, Closed]],
Cell[1009869, 29942, 217, 3, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1010111, 29949, 801, 22, 2147483647, "Input"],
Cell[1010915, 29973, 95, 1, 2147483647, "Output"]
}, Closed]],
Cell[1011025, 29977, 96, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1011146, 29982, 651, 18, 2147483647, "Input"],
Cell[1011800, 30002, 18160, 523, 2147483647, "Output"]
}, Closed]],
Cell[1029975, 30528, 260, 7, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[1030260, 30539, 705, 19, 2147483647, "Input"],
Cell[1030968, 30560, 15029, 432, 2147483647, "Output"]
}, Closed]],
Cell[1046012, 30995, 1217, 18, 2147483647, "Text"],
Cell[1047232, 31015, 1506, 43, 2147483647, "Input",
 InitializationCell->True],
Cell[1048741, 31060, 236, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1049002, 31069, 559, 12, 2147483647, "Input"],
Cell[1049564, 31083, 302, 9, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1049903, 31097, 563, 12, 2147483647, "Input"],
Cell[1050469, 31111, 7183, 219, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1057689, 31335, 592, 13, 2147483647, "Input"],
Cell[1058284, 31350, 9246, 281, 2147483647, "Output"]
}, Closed]],
Cell[1067545, 31634, 217, 4, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1067787, 31642, 291, 6, 2147483647, "Input"],
Cell[1068081, 31650, 19545, 576, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1087663, 32231, 243, 5, 2147483647, "Input"],
Cell[1087909, 32238, 11268, 347, 2147483647, "Output"]
}, Closed]],
Cell[1099192, 32588, 442, 9, 2147483647, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[1099671, 32602, 440, 6, 60, "Subsubsection"],
Cell[1100114, 32610, 281, 9, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1100420, 32623, 125, 2, 2147483647, "Input"],
Cell[1100548, 32627, 295, 5, 2147483647, "Print",
 CellTags->"Info3627718148-3784632"]
}, Closed]],
Cell[CellGroupData[{
Cell[1100880, 32637, 137, 2, 2147483647, "Input"],
Cell[1101020, 32641, 85, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1101142, 32647, 312, 7, 2147483647, "Input"],
Cell[1101457, 32656, 281, 8, 2147483647, "Output"]
}, Closed]],
Cell[1101753, 32667, 420, 9, 2147483647, "Input"],
Cell[1102176, 32678, 296, 7, 2147483647, "Text"],
Cell[1102475, 32687, 2214, 48, 2147483647, "Input",
 InitializationCell->True],
Cell[1104692, 32737, 111, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1104828, 32742, 557, 15, 2147483647, "Input"],
Cell[1105388, 32759, 75, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1105500, 32765, 531, 11, 2147483647, "Input"],
Cell[1106034, 32778, 73, 1, 2147483647, "Output"]
}, Closed]],
Cell[1106122, 32782, 222, 4, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1106369, 32790, 1287, 34, 2147483647, "Input"],
Cell[1107659, 32826, 99, 2, 2147483647, "Output"]
}, Closed]],
Cell[1107773, 32831, 122, 1, 2147483647, "Text"],
Cell[1107898, 32834, 1583, 29, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[1109506, 32867, 1408, 25, 2147483647, "Input"],
Cell[1110917, 32894, 127, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1111081, 32902, 501, 12, 2147483647, "Input"],
Cell[1111585, 32916, 764, 27, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1112386, 32948, 149, 3, 2147483647, "Input"],
Cell[1112538, 32953, 70, 1, 2147483647, "Output"]
}, Closed]],
Cell[1112623, 32957, 685, 11, 2147483647, "Text"],
Cell[1113311, 32970, 167, 3, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1113503, 32977, 549, 15, 2147483647, "Input"],
Cell[1114055, 32994, 1177, 39, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1115269, 33038, 131, 2, 2147483647, "Input"],
Cell[1115403, 33042, 374, 12, 2147483647, "Output"]
}, Closed]],
Cell[1115792, 33057, 89, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1115906, 33062, 365, 11, 2147483647, "Input"],
Cell[1116274, 33075, 73, 1, 2147483647, "Output"]
}, Closed]],
Cell[1116362, 33079, 187, 4, 2147483647, "Text"],
Cell[1116552, 33085, 443, 13, 2147483647, "Input",
 Evaluatable->False],
Cell[1116998, 33100, 244, 7, 2147483647, "Text",
 CellTags->"polco"],
Cell[1117245, 33109, 403, 11, 2147483647, "Input"],
Cell[1117651, 33122, 157, 3, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1117833, 33129, 489, 14, 2147483647, "Input"],
Cell[1118325, 33145, 193, 6, 2147483647, "Output"]
}, Closed]],
Cell[1118533, 33154, 106, 1, 2147483647, "Text"],
Cell[1118642, 33157, 319, 9, 2147483647, "Input"],
Cell[1118964, 33168, 127, 2, 2147483647, "Text"],
Cell[1119094, 33172, 368, 10, 2147483647, "Input"],
Cell[1119465, 33184, 443, 7, 2147483647, "Text"],
Cell[1119911, 33193, 562, 14, 2147483647, "Input"],
Cell[1120476, 33209, 683, 10, 2147483647, "Text"],
Cell[1121162, 33221, 257, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1121444, 33230, 117, 2, 2147483647, "Input"],
Cell[1121564, 33234, 2652, 42, 2147483647, "Print",
 CellTags->"Info3627718626-1674846"]
}, Closed]],
Cell[CellGroupData[{
Cell[1124253, 33281, 402, 12, 2147483647, "Input"],
Cell[1124658, 33295, 354, 11, 2147483647, "Output"]
}, Closed]],
Cell[1125027, 33309, 244, 5, 2147483647, "Input"],
Cell[1125274, 33316, 229, 3, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1125528, 33323, 224, 5, 2147483647, "Input"],
Cell[1125755, 33330, 451, 14, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1126243, 33349, 201, 5, 2147483647, "Input"],
Cell[1126447, 33356, 87, 1, 2147483647, "Output"]
}, Closed]],
Cell[1126549, 33360, 1082, 22, 2147483647, "Input"],
Cell[1127634, 33384, 178, 4, 2147483647, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[1127849, 33393, 510, 7, 60, "Subsubsection"],
Cell[1128362, 33402, 788, 15, 2147483647, "Text"],
Cell[1129153, 33419, 320, 9, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1129498, 33432, 339, 5, 2147483647, "Input"],
Cell[1129840, 33439, 281, 5, 2147483647, "Print",
 CellTags->"Info3627719831-1674846"]
}, Closed]],
Cell[CellGroupData[{
Cell[1130158, 33449, 133, 2, 2147483647, "Input"],
Cell[1130294, 33453, 89, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1130420, 33459, 176, 4, 2147483647, "Input"],
Cell[1130599, 33465, 382, 12, 2147483647, "Output"]
}, Closed]],
Cell[1130996, 33480, 403, 7, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1131424, 33491, 104, 1, 2147483647, "Input"],
Cell[1131531, 33494, 2563, 83, 2147483647, "Output"]
}, Closed]],
Cell[1134109, 33580, 133, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1134267, 33585, 860, 18, 2147483647, "Input"],
Cell[1135130, 33605, 1568, 51, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1136735, 33661, 322, 8, 2147483647, "Input"],
Cell[1137060, 33671, 70, 1, 2147483647, "Output"]
}, Closed]],
Cell[1137145, 33675, 386, 7, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1137556, 33686, 400, 8, 2147483647, "Input"],
Cell[1137959, 33696, 127, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1138123, 33704, 685, 15, 2147483647, "Input"],
Cell[1138811, 33721, 125, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1138973, 33729, 823, 18, 2147483647, "Input"],
Cell[1139799, 33749, 500, 16, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1140336, 33770, 301, 7, 2147483647, "Input"],
Cell[1140640, 33779, 75, 1, 2147483647, "Output"]
}, Closed]],
Cell[1140730, 33783, 188, 2, 2147483647, "Text"],
Cell[1140921, 33787, 1398, 40, 2147483647, "Input"],
Cell[1142322, 33829, 219, 4, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1142566, 33837, 858, 23, 2147483647, "Input"],
Cell[1143427, 33862, 73, 1, 2147483647, "Output"]
}, Closed]],
Cell[1143515, 33866, 223, 2, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1143763, 33872, 807, 22, 2147483647, "Input"],
Cell[1144573, 33896, 191, 6, 2147483647, "Output"]
}, Closed]],
Cell[1144779, 33905, 197, 2, 2147483647, "Text"],
Cell[1144979, 33909, 314, 10, 2147483647, "Input",
 Evaluatable->False],
Cell[1145296, 33921, 761, 15, 2147483647, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[1146094, 33941, 539, 7, 60, "Subsubsection"],
Cell[1146636, 33950, 176, 5, 2147483647, "Text"],
Cell[1146815, 33957, 1357, 24, 2147483647, "Text"],
Cell[1148175, 33983, 569, 14, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1148769, 34001, 195, 3, 2147483647, "Input"],
Cell[1148967, 34006, 283, 5, 2147483647, "Print",
 CellTags->"Info3627720425-9230554"]
}, Closed]],
Cell[CellGroupData[{
Cell[1149287, 34016, 137, 2, 2147483647, "Input"],
Cell[1149427, 34020, 89, 1, 2147483647, "Output"]
}, Closed]],
Cell[1149531, 34024, 101, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1149657, 34029, 127, 2, 2147483647, "Input"],
Cell[1149787, 34033, 285, 5, 2147483647, "Print",
 CellTags->"Info3627720445-9230554"]
}, Closed]],
Cell[CellGroupData[{
Cell[1150109, 34043, 208, 5, 2147483647, "Input"],
Cell[1150320, 34050, 239, 7, 2147483647, "Output"]
}, Closed]],
Cell[1150574, 34060, 779, 15, 2147483647, "Input",
 InitializationCell->True],
Cell[1151356, 34077, 762, 18, 2147483647, "Input"],
Cell[1152121, 34097, 132, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1152278, 34102, 119, 2, 2147483647, "Input"],
Cell[1152400, 34106, 2648, 42, 2147483647, "Print",
 CellTags->"Info3627720479-9230554"]
}, Closed]],
Cell[1155063, 34151, 582, 16, 2147483647, "Input"],
Cell[1155648, 34169, 293, 7, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[1155966, 34180, 631, 14, 2147483647, "Input"],
Cell[1156600, 34196, 229, 7, 2147483647, "Output"]
}, Closed]],
Cell[1156844, 34206, 448, 7, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1157317, 34217, 375, 9, 2147483647, "Input"],
Cell[1157695, 34228, 1005, 33, 2147483647, "Output"]
}, Closed]],
Cell[1158715, 34264, 206, 4, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1158946, 34272, 1010, 30, 2147483647, "Input"],
Cell[1159959, 34304, 142, 4, 2147483647, "Output"]
}, Closed]],
Cell[1160116, 34311, 527, 8, 2147483647, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[1160680, 34324, 557, 7, 60, "Subsubsection"],
Cell[1161240, 34333, 563, 14, 2147483647, "Text"],
Cell[1161806, 34349, 534, 15, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[1162365, 34368, 292, 5, 2147483647, "Input"],
Cell[1162660, 34375, 275, 5, 2147483647, "Print",
 CellTags->"Info3627744819-7142630"]
}, Closed]],
Cell[1162950, 34383, 135, 2, 2147483647, "Input"],
Cell[1163088, 34387, 223, 5, 2147483647, "Text"],
Cell[1163314, 34394, 341, 8, 2147483647, "Input",
 InitializationCell->True],
Cell[1163658, 34404, 332, 9, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1164015, 34417, 657, 16, 2147483647, "Input"],
Cell[1164675, 34435, 2314, 71, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1167026, 34511, 205, 4, 2147483647, "Input"],
Cell[1167234, 34517, 125, 3, 2147483647, "Output"]
}, Closed]],
Cell[1167374, 34523, 172, 2, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1167571, 34529, 1385, 34, 2147483647, "Input",
 InitializationCell->True],
Cell[1168959, 34565, 2479, 68, 140, "Output"]
}, Closed]],
Cell[1171453, 34636, 176, 4, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1171654, 34644, 390, 9, 2147483647, "Input"],
Cell[1172047, 34655, 4603, 143, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1176687, 34803, 709, 17, 2147483647, "Input"],
Cell[1177399, 34822, 125, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1177561, 34830, 726, 17, 2147483647, "Input"],
Cell[1178290, 34849, 125, 3, 2147483647, "Output"]
}, Closed]],
Cell[1178430, 34855, 547, 9, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1179002, 34868, 119, 2, 2147483647, "Input"],
Cell[1179124, 34872, 375, 6, 2147483647, "Print",
 CellTags->"Info3627745043-7142630"]
}, Closed]],
Cell[1179514, 34881, 387, 9, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[1179926, 34894, 242, 7, 2147483647, "Input"],
Cell[1180171, 34903, 502, 16, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1180710, 34924, 242, 7, 2147483647, "Input"],
Cell[1180955, 34933, 2401, 78, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1183393, 35016, 719, 17, 2147483647, "Input"],
Cell[1184115, 35035, 125, 3, 2147483647, "Output"]
}, Closed]],
Cell[1184255, 35041, 485, 10, 2147483647, "Text"],
Cell[1184743, 35053, 263, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1185031, 35062, 741, 21, 2147483647, "Input"],
Cell[1185775, 35085, 965, 30, 2147483647, "Output"]
}, Closed]],
Cell[1186755, 35118, 403, 10, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1187183, 35132, 1102, 24, 2147483647, "Input"],
Cell[1188288, 35158, 75, 1, 2147483647, "Output"]
}, Closed]],
Cell[1188378, 35162, 254, 5, 45056, "Text"],
Cell[1188635, 35169, 656, 18, 33280, "Input"],
Cell[1189294, 35189, 481, 10, 30720, "Text"],
Cell[1189778, 35201, 1315, 23, 90112, "Text"],
Cell[1191096, 35226, 628, 13, 50176, "Text"],
Cell[CellGroupData[{
Cell[1191749, 35243, 118, 2, 30720, "Input"],
Cell[1191870, 35247, 205, 4, 2147483647, "Print",
 CellTags->"Info3627749163-7142630"]
}, Closed]],
Cell[CellGroupData[{
Cell[1192112, 35256, 211, 5, 22528, "Input"],
Cell[1192326, 35263, 175, 5, 2147483647, "Output"]
}, Closed]],
Cell[1192516, 35271, 559, 14, 42496, "Input"],
Cell[1193078, 35287, 141, 3, 30720, "Input"],
Cell[CellGroupData[{
Cell[1193244, 35294, 213, 6, 30720, "Input"],
Cell[1193460, 35302, 346, 11, 2147483647, "Output"]
}, Closed]],
Cell[1193821, 35316, 860, 20, 42496, "Input"],
Cell[1194684, 35338, 258, 6, 30720, "Input"],
Cell[1194945, 35346, 403, 8, 50176, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[1195385, 35359, 664, 9, 60, "Subsubsection"],
Cell[1196052, 35370, 1222, 27, 2147483647, "Text"],
Cell[1197277, 35399, 372, 8, 2147483647, "Text"],
Cell[1197652, 35409, 379, 8, 2147483647, "Text"],
Cell[1198034, 35419, 640, 16, 2013265920, "Input",
 Evaluatable->False],
Cell[1198677, 35437, 583, 14, 2013265920, "Input",
 Evaluatable->False],
Cell[1199263, 35453, 566, 9, 2147483647, "Text"],
Cell[1199832, 35464, 516, 13, 2013265920, "Input"],
Cell[1200351, 35479, 213, 4, 2147483647, "Text"],
Cell[1200567, 35485, 616, 16, 2013265920, "Input"],
Cell[1201186, 35503, 237, 5, 2013265920, "Text"],
Cell[1201426, 35510, 516, 14, 2013265920, "Input"],
Cell[1201945, 35526, 1029, 17, 2147483647, "Text"],
Cell[1202977, 35545, 263, 7, 2013265920, "Input"],
Cell[1203243, 35554, 462, 9, 2147483647, "Text"],
Cell[1203708, 35565, 362, 10, 2013265920, "Input"],
Cell[1204073, 35577, 101, 1, 2013265920, "Text"],
Cell[1204177, 35580, 324, 10, 2013265920, "Input"],
Cell[1204504, 35592, 396, 12, 2013265920, "Input"],
Cell[1204903, 35606, 498, 8, 2147483647, "Text"],
Cell[1205404, 35616, 206, 5, 2013265920, "Input"],
Cell[1205613, 35623, 93, 1, 2013265920, "Text"],
Cell[1205709, 35626, 390, 12, 2013265920, "Input"],
Cell[1206102, 35640, 440, 14, 2013265920, "Input"],
Cell[1206545, 35656, 91, 1, 2013265920, "Text"],
Cell[1206639, 35659, 411, 12, 2013265920, "Input",
 InitializationCell->True],
Cell[1207053, 35673, 227, 5, 2013265920, "Text"],
Cell[1207283, 35680, 412, 11, 2013265920, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[1207720, 35695, 202, 4, 2013265920, "Input"],
Cell[1207925, 35701, 2159, 68, 2147483647, "Output"]
}, Closed]],
Cell[1210099, 35772, 326, 6, 2147483647, "Text"],
Cell[1210428, 35780, 861, 13, 2147483647, "Text"],
Cell[1211292, 35795, 528, 13, 2013265920, "Input"],
Cell[1211823, 35810, 698, 11, 2147483647, "Text"],
Cell[1212524, 35823, 609, 18, 2013265920, "Input"],
Cell[1213136, 35843, 300, 9, 2013265920, "Input"],
Cell[1213439, 35854, 138, 1, 2013265920, "Text"],
Cell[1213580, 35857, 662, 20, 2013265920, "Input"],
Cell[1214245, 35879, 347, 11, 2013265920, "Input"],
Cell[1214595, 35892, 245, 5, 2013265920, "Text"],
Cell[1214843, 35899, 289, 8, 2013265920, "Input"],
Cell[1215135, 35909, 1419, 20, 2147483647, "Text"],
Cell[1216557, 35931, 624, 19, 2013265920, "Input",
 Evaluatable->False],
Cell[1217184, 35952, 281, 5, 2013265920, "Text"],
Cell[1217468, 35959, 616, 19, 2013265920, "Input"],
Cell[1218087, 35980, 216, 4, 2013265920, "Text"],
Cell[1218306, 35986, 419, 12, 2013265920, "Input",
 Evaluatable->False],
Cell[1218728, 36000, 486, 8, 2147483647, "Text"],
Cell[1219217, 36010, 831, 24, 2147483647, "Input"],
Cell[1220051, 36036, 315, 6, 2013265920, "Text"],
Cell[1220369, 36044, 605, 10, 2147483647, "Text"],
Cell[1220977, 36056, 285, 8, 2013265920, "Input",
 Evaluatable->False],
Cell[1221265, 36066, 511, 8, 2147483647, "Text"],
Cell[1221779, 36076, 229, 5, 2013265920, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[1222045, 36086, 672, 9, 60, "Subsubsection"],
Cell[1222720, 36097, 856, 16, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1223601, 36117, 124, 2, 2013265920, "Input"],
Cell[1223728, 36121, 405, 6, 2147483647, "Print",
 CellTags->"Info3627816879-6530549"]
}, Closed]],
Cell[CellGroupData[{
Cell[1224170, 36132, 246, 6, 2013265920, "Input"],
Cell[1224419, 36140, 382, 11, 2147483647, "Output"]
}, Closed]],
Cell[1224816, 36154, 233, 7, 2013265920, "Input"],
Cell[1225052, 36163, 280, 7, 2013265920, "Input"],
Cell[1225335, 36172, 97, 1, 2013265920, "Text"],
Cell[1225435, 36175, 430, 11, 2013265920, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[1225890, 36190, 198, 4, 2013265920, "Input"],
Cell[1226091, 36196, 335, 10, 2147483647, "Output"]
}, Closed]],
Cell[1226441, 36209, 814, 19, 2147483647, "Input",
 InitializationCell->True],
Cell[1227258, 36230, 125, 1, 2013265920, "Text"],
Cell[1227386, 36233, 307, 7, 2013265920, "Input"],
Cell[1227696, 36242, 110, 1, 2013265920, "Text"],
Cell[CellGroupData[{
Cell[1227831, 36247, 744, 19, 2013265920, "Input",
 InitializationCell->True],
Cell[1228578, 36268, 1460, 38, 140, "Output"]
}, Closed]],
Cell[1230053, 36309, 227, 4, 2147483647, "Text"],
Cell[1230283, 36315, 592, 16, 2013265920, "Input"],
Cell[1230878, 36333, 286, 6, 2013265920, "Text"],
Cell[CellGroupData[{
Cell[1231189, 36343, 787, 20, 2013265920, "Input"],
Cell[1231979, 36365, 504, 17, 2147483647, "Output"]
}, Closed]],
Cell[1232498, 36385, 220, 2, 2013265920, "Text"],
Cell[1232721, 36389, 174, 4, 2013265920, "Input"],
Cell[1232898, 36395, 819, 24, 2147483647, "Input"],
Cell[1233720, 36421, 860, 22, 2147483647, "Input"],
Cell[1234583, 36445, 608, 14, 2013265920, "Input"],
Cell[1235194, 36461, 245, 5, 2013265920, "Text"],
Cell[1235442, 36468, 531, 10, 2013265920, "Input"],
Cell[1235976, 36480, 217, 4, 2013265920, "Text"],
Cell[CellGroupData[{
Cell[1236218, 36488, 1098, 24, 2013265920, "Input"],
Cell[1237319, 36514, 431, 8, 1442840576, "Message"],
Cell[1237753, 36524, 4761, 131, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1242551, 36660, 259, 7, 2013265920, "Input"],
Cell[1242813, 36669, 2493, 75, 2147483647, "Output"]
}, Closed]],
Cell[1245321, 36747, 737, 16, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1246083, 36767, 178, 4, 2013265920, "Input"],
Cell[1246264, 36773, 2313, 76, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1248614, 36854, 298, 8, 2013265920, "Input"],
Cell[1248915, 36864, 452, 13, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1249404, 36882, 133, 2, 2013265920, "Input"],
Cell[1249540, 36886, 70, 1, 2147483647, "Output"]
}, Closed]],
Cell[1249625, 36890, 341, 6, 2147483647, "Text"],
Cell[1249969, 36898, 449, 12, 2013265920, "Input"],
Cell[CellGroupData[{
Cell[1250443, 36914, 403, 10, 2013265920, "Input"],
Cell[1250849, 36926, 102, 3, 2147483647, "Output"]
}, Closed]],
Cell[1250966, 36932, 555, 16, 2147483647, "Text"],
Cell[1251524, 36950, 555, 14, 2013265920, "Text"],
Cell[CellGroupData[{
Cell[1252104, 36968, 107, 2, 2013265920, "Input"],
Cell[1252214, 36972, 280, 5, 2147483647, "Print",
 CellTags->"Info3627817277-6530549"]
}, Closed]],
Cell[CellGroupData[{
Cell[1252531, 36982, 164, 3, 2013265920, "Input"],
Cell[1252698, 36987, 151, 4, 2147483647, "Output"]
}, Closed]],
Cell[1252864, 36994, 154, 4, 2013265920, "Input"],
Cell[1253021, 37000, 226, 4, 2147483647, "Text"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1253296, 37010, 321, 4, 95, "Subsection"],
Cell[CellGroupData[{
Cell[1253642, 37018, 309, 5, 76, "Subsubsection"],
Cell[1253954, 37025, 108, 1, 503316480, "Text"],
Cell[1254065, 37028, 772, 21, 830472192, "Input",
 InitializationCell->True],
Cell[1254840, 37051, 141, 1, 503316480, "Text"],
Cell[1254984, 37054, 543, 13, 503316480, "Input"],
Cell[1255530, 37069, 317, 7, 503316480, "Input"],
Cell[1255850, 37078, 561, 10, 1140850688, "Text"],
Cell[CellGroupData[{
Cell[1256436, 37092, 122, 2, 503316480, "Input"],
Cell[1256561, 37096, 3360, 53, 2147483647, "Print",
 CellTags->"Info3627830615-6530549"]
}, Closed]],
Cell[1259936, 37152, 287, 6, 503316480, "Text"],
Cell[CellGroupData[{
Cell[1260248, 37162, 596, 15, 503316480, "Input"],
Cell[1260847, 37179, 191, 5, 536870912, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1261075, 37189, 122, 2, 503316480, "Input"],
Cell[1261200, 37193, 87, 1, 494927872, "Output"]
}, Closed]],
Cell[1261302, 37197, 143, 1, 503316480, "Text"],
Cell[CellGroupData[{
Cell[1261470, 37202, 671, 17, 503316480, "Input"],
Cell[1262144, 37221, 118, 3, 536870912, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1262299, 37229, 122, 2, 503316480, "Input"],
Cell[1262424, 37233, 87, 1, 494927872, "Output"]
}, Closed]],
Cell[1262526, 37237, 307, 6, 822083584, "Text"],
Cell[CellGroupData[{
Cell[1262858, 37247, 278, 7, 503316480, "Input"],
Cell[1263139, 37256, 318, 12, 780140544, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1263494, 37273, 346, 8, 503316480, "Input"],
Cell[1263843, 37283, 299, 11, 780140544, "Output"]
}, Closed]],
Cell[1264157, 37297, 2072, 30, 2147483647, "Text"],
Cell[1266232, 37329, 334, 6, 822083584, "Text"],
Cell[1266569, 37337, 453, 14, 503316480, "Input"],
Cell[1267025, 37353, 252, 5, 822083584, "Text"],
Cell[1267280, 37360, 498, 13, 503316480, "Input"],
Cell[1267781, 37375, 493, 13, 503316480, "Input"],
Cell[1268277, 37390, 513, 14, 503316480, "Input"],
Cell[1268793, 37406, 514, 14, 503316480, "Input"],
Cell[1269310, 37422, 324, 8, 503316480, "Input"],
Cell[1269637, 37432, 137, 1, 503316480, "Text"],
Cell[1269777, 37435, 770, 18, 503316480, "Input"],
Cell[1270550, 37455, 772, 18, 503316480, "Input"],
Cell[1271325, 37475, 853, 20, 503316480, "Input"],
Cell[1272181, 37497, 852, 20, 503316480, "Input"],
Cell[1273036, 37519, 460, 10, 503316480, "Input"],
Cell[1273499, 37531, 1100, 17, 2147483647, "Text"],
Cell[1274602, 37550, 422, 12, 754974720, "Input",
 Evaluatable->False],
Cell[1275027, 37564, 361, 7, 822083584, "Text"],
Cell[1275391, 37573, 743, 20, 503316480, "Input"],
Cell[1276137, 37595, 236, 4, 822083584, "Text"],
Cell[1276376, 37601, 984, 28, 754974720, "Input",
 Evaluatable->False],
Cell[1277363, 37631, 583, 12, 822083584, "Text"],
Cell[1277949, 37645, 1208, 36, 830472192, "Input",
 Evaluatable->False,
 CellTags->"gaussK"],
Cell[CellGroupData[{
Cell[1279182, 37685, 293, 5, 503316480, "Input"],
Cell[1279478, 37692, 237, 4, 973078528, "Print",
 CellTags->"Info3627831290-6705403"]
}, Closed]],
Cell[1279730, 37699, 610, 11, 1459617792, "Text"],
Cell[CellGroupData[{
Cell[1280365, 37714, 222, 5, 503316480, "Input"],
Cell[1280590, 37721, 168, 5, 494927872, "Output"]
}, Closed]],
Cell[1280773, 37729, 316, 6, 822083584, "Text"],
Cell[1281092, 37737, 363, 10, 503316480, "Input"],
Cell[1281458, 37749, 157, 3, 503316480, "Text"],
Cell[CellGroupData[{
Cell[1281640, 37756, 355, 9, 503316480, "Input"],
Cell[1281998, 37767, 138, 3, 687865856, "Output"]
}, Closed]],
Cell[1282151, 37773, 165, 3, 503316480, "Text"],
Cell[CellGroupData[{
Cell[1282341, 37780, 227, 5, 503316480, "Input"],
Cell[1282571, 37787, 117531, 3531, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1400139, 41323, 499, 10, 503316480, "Input"],
Cell[1400641, 41335, 79, 1, 981467136, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1400757, 41341, 499, 10, 369098752, "Input"],
Cell[1401259, 41353, 57864, 1615, 2147483647, "Output"]
}, Closed]],
Cell[1459138, 42971, 447, 9, 738197504, "Text"],
Cell[1459588, 42982, 525, 11, 822083584, "Text"],
Cell[1460116, 42995, 188, 4, 503316480, "Text"],
Cell[1460307, 43001, 165, 3, 503316480, "Input"],
Cell[1460475, 43006, 486, 11, 503316480, "Input"],
Cell[1460964, 43019, 176, 4, 503316480, "Input"],
Cell[1461143, 43025, 308, 7, 503316480, "Input"],
Cell[1461454, 43034, 379, 8, 503316480, "Input"],
Cell[1461836, 43044, 463, 10, 503316480, "Input"],
Cell[1462302, 43056, 331, 9, 645922816, "Text"],
Cell[1462636, 43067, 483, 9, 503316480, "Input"],
Cell[1463122, 43078, 225, 4, 822083584, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[1463384, 43087, 369, 5, 60, "Subsubsection"],
Cell[1463756, 43094, 609, 10, 1459617792, "Text"],
Cell[1464368, 43106, 319, 7, 822083584, "Text"],
Cell[1464690, 43115, 465, 12, 503316480, "Input",
 Evaluatable->False],
Cell[1465158, 43129, 930, 18, 1459617792, "Text"],
Cell[1466091, 43149, 1397, 35, 1468006400, "Input"],
Cell[1467491, 43186, 378, 6, 822083584, "Text"],
Cell[1467872, 43194, 464, 12, 503316480, "Input",
 Evaluatable->False],
Cell[1468339, 43208, 427, 9, 822083584, "Text"],
Cell[1468769, 43219, 1091, 29, 830472192, "Input",
 Evaluatable->False],
Cell[1469863, 43250, 1058, 29, 830472192, "Input",
 Evaluatable->False],
Cell[1470924, 43281, 339, 5, 503316480, "Input"],
Cell[1471266, 43288, 131, 2, 503316480, "Input"],
Cell[1471400, 43292, 98, 1, 503316480, "Text"],
Cell[1471501, 43295, 1265, 33, 830472192, "Input"],
Cell[1472769, 43330, 1153, 34, 1149239296, "Input"],
Cell[CellGroupData[{
Cell[1473947, 43368, 1282, 34, 1468006400, "Input"],
Cell[1475232, 43404, 75, 1, 494927872, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[1475356, 43411, 418, 5, 60, "Subsubsection"],
Cell[1475777, 43418, 568, 9, 1140850688, "Text"],
Cell[1476348, 43429, 888, 16, 1459617792, "Text"],
Cell[1477239, 43447, 3516, 63, 2147483647, "Text"],
Cell[1480758, 43512, 879, 26, 830472192, "Input",
 Evaluatable->False],
Cell[1481640, 43540, 321, 8, 1644167168, "Text",
 CellTags->"cij"],
Cell[CellGroupData[{
Cell[1481986, 43552, 111, 2, 1006632960, "Input"],
Cell[1482100, 43556, 319, 5, 1946157056, "Print",
 CellTags->"Info3627905210-4669561"]
}, Closed]],
Cell[1482434, 43564, 5000, 150, 2147483647, "Input",
 Evaluatable->False],
Cell[1487437, 43716, 114, 1, 1006632960, "Text"],
Cell[CellGroupData[{
Cell[1487576, 43721, 648, 16, 1006632960, "Input"],
Cell[1488227, 43739, 75, 1, 494927872, "Output"]
}, Closed]],
Cell[1488317, 43743, 555, 9, 822083584, "Text"],
Cell[1488875, 43754, 1297, 34, 830472192, "Input",
 Evaluatable->False],
Cell[1490175, 43790, 395, 7, 822083584, "Text"],
Cell[1490573, 43799, 716, 20, 503316480, "Input",
 Evaluatable->False],
Cell[1491292, 43821, 257, 3, 503316480, "Text"],
Cell[1491552, 43826, 796, 22, 503316480, "Input",
 Evaluatable->False],
Cell[1492351, 43850, 244, 6, 503316480, "Text"],
Cell[1492598, 43858, 792, 22, 503316480, "Input",
 Evaluatable->False],
Cell[1493393, 43882, 134, 1, 503316480, "Text"],
Cell[1493530, 43885, 824, 21, 503316480, "Input",
 Evaluatable->False],
Cell[1494357, 43908, 299, 6, 503316480, "Text"],
Cell[1494659, 43916, 424, 11, 503316480, "Input",
 Evaluatable->False],
Cell[1495086, 43929, 494, 8, 822083584, "Text"],
Cell[1495583, 43939, 901, 17, 2097152000, "Text"]
}, Closed]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1496545, 43963, 374, 5, 140, "Section"],
Cell[CellGroupData[{
Cell[1496944, 43972, 467, 6, 95, "Subsection"],
Cell[1497414, 43980, 149, 3, 120, "Input"],
Cell[1497566, 43985, 392, 12, 120, "Text"],
Cell[1497961, 43999, 1095, 27, 198, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1499081, 44030, 290, 5, 120, "Input"],
Cell[1499374, 44037, 296, 5, 973078528, "Print",
 CellTags->"Info3627911044-4669561"]
}, Closed]],
Cell[CellGroupData[{
Cell[1499707, 44047, 128, 2, 88, "Input"],
Cell[1499838, 44051, 88, 1, 494927872, "Output"]
}, Closed]],
Cell[1499941, 44055, 1232, 21, 480, "Text"],
Cell[CellGroupData[{
Cell[1501198, 44080, 2229, 54, 350, "Input"],
Cell[1503430, 44136, 72, 1, 494927872, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1503539, 44142, 111, 2, 88, "Input"],
Cell[1503653, 44146, 319, 5, 973078528, "Print",
 CellTags->"Info3627911413-4669561"]
}, Closed]],
Cell[1503987, 44154, 240, 7, 100, "Text"],
Cell[CellGroupData[{
Cell[1504252, 44165, 732, 20, 198, "Input"],
Cell[1504987, 44187, 70, 1, 494927872, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1505094, 44193, 774, 20, 88, "Input"],
Cell[1505871, 44215, 72, 1, 494927872, "Output"]
}, Closed]],
Cell[1505958, 44219, 578, 15, 176, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[1506573, 44239, 367, 4, 75, "Subsection"],
Cell[1506943, 44245, 212, 4, 120, "Text"],
Cell[CellGroupData[{
Cell[1507180, 44253, 1069, 28, 198, "Input"],
Cell[1508252, 44283, 70, 1, 1409286144, "Output"]
}, Closed]],
Cell[1508337, 44287, 695, 14, 252, "Text"],
Cell[1509035, 44303, 347, 10, 120, "Text"],
Cell[1509385, 44315, 2081, 68, 350, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1511491, 44387, 194, 3, 120, "Input"],
Cell[1511688, 44392, 351, 6, 2147483647, "Print",
 CellTags->"Info3627914288-4669561"]
}, Closed]],
Cell[1512054, 44401, 257, 4, 100, "Text"],
Cell[1512314, 44407, 625, 18, 120, "Input",
 Evaluatable->False],
Cell[1512942, 44427, 154, 5, 120, "Text"],
Cell[CellGroupData[{
Cell[1513121, 44436, 1613, 44, 274, "Input"],
Cell[1514737, 44482, 75, 1, 1409286144, "Output"]
}, Closed]],
Cell[1514827, 44486, 266, 3, 100, "Text"],
Cell[CellGroupData[{
Cell[1515118, 44493, 1247, 34, 274, "Input"],
Cell[1516368, 44529, 70, 1, 1409286144, "Output"]
}, Closed]],
Cell[1516453, 44533, 230, 8, 100, "Text"],
Cell[CellGroupData[{
Cell[1516708, 44545, 1892, 49, 274, "Input"],
Cell[1518603, 44596, 72, 1, 1409286144, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1518712, 44602, 859, 23, 88, "Input"],
Cell[1519574, 44627, 261, 9, 1778384896, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1519872, 44641, 792, 19, 166, "Input"],
Cell[1520667, 44662, 375, 12, 1778384896, "Output"]
}, Closed]],
Cell[1521057, 44677, 90, 1, 100, "Text"],
Cell[CellGroupData[{
Cell[1521172, 44682, 1211, 30, 274, "Input"],
Cell[1522386, 44714, 75, 1, 1409286144, "Output"]
}, Closed]],
Cell[1522476, 44718, 412, 12, 176, "Text"],
Cell[CellGroupData[{
Cell[1522913, 44734, 881, 23, 120, "Input"],
Cell[1523797, 44759, 196, 5, 1778384896, "Output"]
}, Closed]],
Cell[1524008, 44767, 106, 1, 100, "Text"],
Cell[1524117, 44770, 472, 11, 120, "Input"],
Cell[CellGroupData[{
Cell[1524614, 44785, 363, 11, 120, "Input"],
Cell[1524980, 44798, 70, 1, 1409286144, "Output"]
}, Closed]],
Cell[1525065, 44802, 408, 7, 176, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[1525510, 44814, 371, 6, 75, "Subsection"],
Cell[1525884, 44822, 322, 6, 503316480, "Text"],
Cell[CellGroupData[{
Cell[1526231, 44832, 1318, 41, 1468006400, "Input"],
Cell[1527552, 44875, 75, 1, 1409286144, "Output"]
}, Closed]],
Cell[1527642, 44879, 159, 3, 419430400, "Text"],
Cell[CellGroupData[{
Cell[1527826, 44886, 1154, 29, 830472192, "Input"],
Cell[1528983, 44917, 75, 1, 1409286144, "Output"]
}, Closed]],
Cell[1529073, 44921, 132, 1, 419430400, "Text"],
Cell[CellGroupData[{
Cell[1529230, 44926, 783, 22, 503316480, "Input"],
Cell[1530016, 44950, 75, 1, 1409286144, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1530128, 44956, 792, 21, 369098752, "Input"],
Cell[1530923, 44979, 75, 1, 1409286144, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1531035, 44985, 790, 21, 369098752, "Input"],
Cell[1531828, 45008, 70, 1, 1409286144, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1531935, 45014, 835, 23, 369098752, "Input"],
Cell[1532773, 45039, 71, 1, 1409286144, "Output"]
}, Closed]],
Cell[1532859, 45043, 119, 1, 419430400, "Text"],
Cell[CellGroupData[{
Cell[1533003, 45048, 3027, 81, 1786773504, "Input"],
Cell[1536033, 45131, 70, 1, 1409286144, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1536140, 45137, 1061, 28, 696254464, "Input"],
Cell[1537204, 45167, 72, 1, 1409286144, "Output"]
}, Closed]],
Cell[1537291, 45171, 126, 1, 419430400, "Text"],
Cell[1537420, 45174, 1251, 34, 830472192, "Input",
 InitializationCell->True],
Cell[1538674, 45210, 137, 1, 503316480, "Text"],
Cell[CellGroupData[{
Cell[1538836, 45215, 722, 17, 503316480, "Input"],
Cell[1539561, 45234, 72, 1, 1409286144, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[1539682, 45241, 463, 6, 75, "Subsection"],
Cell[CellGroupData[{
Cell[1540170, 45251, 118, 1, 76, "Subsubsection"],
Cell[1540291, 45254, 308, 6, 251658240, "Text"],
Cell[1540602, 45262, 302, 8, 251658240, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[1540929, 45274, 147, 3, 251658240, "Input"],
Cell[1541079, 45279, 301, 9, 352321536, "Output"]
}, Closed]],
Cell[1541395, 45291, 133, 1, 251658240, "Text"],
Cell[1541531, 45294, 377, 10, 251658240, "Input",
 InitializationCell->True],
Cell[1541911, 45306, 126, 1, 251658240, "Text"],
Cell[1542040, 45309, 373, 8, 251658240, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[1542438, 45321, 125, 2, 251658240, "Input"],
Cell[1542566, 45325, 571, 18, 352321536, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1543174, 45348, 196, 4, 251658240, "Input"],
Cell[1543373, 45354, 2827, 96, 666894336, "Output"]
}, Closed]],
Cell[1546215, 45453, 410, 7, 251658240, "Text"],
Cell[CellGroupData[{
Cell[1546650, 45464, 850, 22, 251658240, "Input"],
Cell[1547503, 45488, 73, 1, 352321536, "Output"]
}, Closed]],
Cell[1547591, 45492, 479, 15, 251658240, "Input"],
Cell[1548073, 45509, 390, 7, 251658240, "Text"],
Cell[CellGroupData[{
Cell[1548488, 45520, 117, 2, 251658240, "Input"],
Cell[1548608, 45524, 373, 6, 629145600, "Print",
 CellTags->"Info3627977160-2492953"]
}, Closed]],
Cell[CellGroupData[{
Cell[1549018, 45535, 179, 5, 251658240, "Input"],
Cell[1549200, 45542, 308, 9, 352321536, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1549545, 45556, 208, 5, 251658240, "Input"],
Cell[1549756, 45563, 3173, 103, 754974720, "Output"]
}, Closed]],
Cell[1552944, 45669, 304, 8, 251658240, "Input"],
Cell[CellGroupData[{
Cell[1553273, 45681, 143, 4, 251658240, "Input"],
Cell[1553419, 45687, 1681, 51, 666894336, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1555137, 45743, 1016, 24, 415236096, "Input"],
Cell[1556156, 45769, 128, 2, 352321536, "Output"]
}, Closed]],
Cell[1556299, 45774, 467, 8, 570425344, "Text"],
Cell[1556769, 45784, 717, 21, 251658240, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[1557511, 45809, 426, 8, 251658240, "Input"],
Cell[1557940, 45819, 489, 16, 352321536, "Output"]
}, Closed]],
Cell[1558444, 45838, 363, 7, 251658240, "Text"],
Cell[CellGroupData[{
Cell[1558832, 45849, 508, 9, 251658240, "Input"],
Cell[1559343, 45860, 5571, 170, 1417674752, "Output"]
}, Closed]],
Cell[1564929, 46033, 270, 7, 411041792, "Text"],
Cell[CellGroupData[{
Cell[1565224, 46044, 989, 26, 415236096, "Input"],
Cell[1566216, 46072, 1014, 31, 352321536, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1567267, 46108, 967, 24, 251658240, "Input"],
Cell[1568237, 46134, 23368, 697, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1591642, 46836, 1134, 27, 415236096, "Input"],
Cell[1592779, 46865, 22823, 701, 2147483647, "Output"]
}, Closed]],
Cell[1615617, 47569, 351, 6, 411041792, "Text"],
Cell[1615971, 47577, 2140, 48, 734003200, "Input",
 InitializationCell->True],
Cell[1618114, 47627, 247, 3, 251658240, "Text"],
Cell[CellGroupData[{
Cell[1618386, 47634, 935, 23, 251658240, "Input"],
Cell[1619324, 47659, 70, 1, 352321536, "Output"]
}, Closed]],
Cell[1619409, 47663, 319, 7, 251658240, "Text"],
Cell[CellGroupData[{
Cell[1619753, 47674, 489, 14, 251658240, "Input"],
Cell[1620245, 47690, 441, 14, 352321536, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1620723, 47709, 143, 3, 251658240, "Input"],
Cell[1620869, 47714, 1067, 32, 352321536, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1621973, 47751, 1014, 26, 415236096, "Input"],
Cell[1622990, 47779, 1014, 31, 352321536, "Output"]
}, Closed]],
Cell[1624019, 47813, 370, 6, 411041792, "Text"],
Cell[CellGroupData[{
Cell[1624414, 47823, 372, 9, 251658240, "Input"],
Cell[1624789, 47834, 1578, 48, 511705088, "Output"]
}, Closed]],
Cell[1626382, 47885, 115, 1, 251658240, "Text"],
Cell[CellGroupData[{
Cell[1626522, 47890, 1034, 24, 251658240, "Input"],
Cell[1627559, 47916, 72, 1, 352321536, "Output"]
}, Closed]],
Cell[1627646, 47920, 95, 1, 251658240, "Text"],
Cell[1627744, 47923, 171, 5, 251658240, "Text"],
Cell[1627918, 47930, 159, 3, 251658240, "Text"],
Cell[CellGroupData[{
Cell[1628102, 47937, 147, 3, 251658240, "Input"],
Cell[1628252, 47942, 335, 10, 352321536, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1628624, 47957, 508, 16, 251658240, "Input"],
Cell[1629135, 47975, 1344, 41, 511705088, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1630516, 48021, 1255, 30, 415236096, "Input"],
Cell[1631774, 48053, 70, 1, 352321536, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1631881, 48059, 1023, 31, 415236096, "Input"],
Cell[1632907, 48092, 561, 18, 352321536, "Output"]
}, Closed]],
Cell[1633483, 48113, 217, 4, 251658240, "Text"],
Cell[CellGroupData[{
Cell[1633725, 48121, 1244, 36, 574619648, "Input"],
Cell[1634972, 48159, 70, 1, 352321536, "Output"]
}, Closed]],
Cell[1635057, 48163, 211, 6, 570425344, "Text"],
Cell[CellGroupData[{
Cell[1635293, 48173, 1036, 31, 415236096, "Input"],
Cell[1636332, 48206, 72, 1, 352321536, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[1636453, 48213, 106, 1, 60, "Subsubsection"],
Cell[1636562, 48216, 473, 10, 411041792, "Text"],
Cell[1637038, 48228, 942, 27, 574619648, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[1638005, 48259, 369, 11, 251658240, "Input"],
Cell[1638377, 48272, 3501, 102, 1103101952, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1641915, 48379, 133, 2, 251658240, "Input"],
Cell[1642051, 48383, 910, 29, 545259520, "Output"]
}, Closed]],
Cell[1642976, 48415, 755, 12, 411041792, "Text",
 InitializationCell->True],
Cell[1643734, 48429, 2122, 50, 893386752, "Input",
 InitializationCell->True],
Cell[1645859, 48481, 100, 1, 251658240, "Text"],
Cell[CellGroupData[{
Cell[1645984, 48486, 780, 14, 251658240, "Input"],
Cell[1646767, 48502, 1169, 37, 629145600, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1647973, 48544, 450, 9, 251658240, "Input"],
Cell[1648426, 48555, 10356, 278, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1658819, 48838, 883, 21, 520093696, "Input"],
Cell[1659705, 48861, 8765, 257, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1668507, 49123, 1151, 25, 356515840, "Input"],
Cell[1669661, 49150, 1109, 35, 654311424, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1670807, 49190, 163, 5, 356515840, "Input"],
Cell[1670973, 49197, 707, 21, 457179136, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1671717, 49223, 1001, 29, 658505728, "Input"],
Cell[1672721, 49254, 521, 15, 490733568, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1673279, 49274, 1027, 15, 356515840, "Input"],
Cell[1674309, 49291, 182, 4, 352321536, "Output"]
}, Closed]],
Cell[1674506, 49298, 453, 6, 251658240, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[1674996, 49309, 289, 4, 60, "Subsubsection"],
Cell[1675288, 49315, 465, 6, 31457280, "Text"],
Cell[CellGroupData[{
Cell[1675778, 49325, 128, 2, 31457280, "Input"],
Cell[1675909, 49329, 269, 5, 243269632, "Print",
 CellTags->"Info3627987206-2313691"]
}, Closed]],
Cell[1676193, 49337, 441, 15, 23068672, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1676659, 49356, 120, 2, 31457280, "Input"],
Cell[1676782, 49360, 213, 4, 169869312, "Print",
 CellTags->"Info3627987267-2313691"]
}, Closed]],
Cell[1677010, 49367, 170, 3, 26214400, "Text"],
Cell[1677183, 49372, 379, 9, 31457280, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[1677587, 49385, 697, 11, 31457280, "Input"],
Cell[1678287, 49398, 101161, 2632, 2147483647, "Output"]
}, Closed]],
Cell[1779463, 52033, 386, 7, 46137344, "Text"],
Cell[1779852, 52042, 383, 8, 31457280, "Input",
 InitializationCell->True],
Cell[1780238, 52052, 516, 10, 51380224, "Text"],
Cell[CellGroupData[{
Cell[1780779, 52066, 117, 2, 31457280, "Input"],
Cell[1780899, 52070, 316, 5, 78643200, "Print",
 CellTags->"Info3628063470-3762495"]
}, Closed]],
Cell[CellGroupData[{
Cell[1781252, 52080, 129, 2, 31457280, "Input"],
Cell[1781384, 52084, 85, 1, 44040192, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1781506, 52090, 163, 4, 31457280, "Input"],
Cell[1781672, 52096, 14016, 413, 1226833920, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1795725, 52514, 568, 16, 43515904, "Input"],
Cell[1796296, 52532, 94, 1, 176160768, "Output"]
}, Closed]],
Cell[1796405, 52536, 122, 1, 26214400, "Text"],
Cell[CellGroupData[{
Cell[1796552, 52541, 115, 2, 31457280, "Input"],
Cell[1796670, 52545, 667, 14, 61341696, "Print",
 CellTags->"Info3628063532-3762495"]
}, Closed]],
Cell[CellGroupData[{
Cell[1797374, 52564, 133, 2, 31457280, "Input"],
Cell[1797510, 52568, 84, 1, 73924608, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1797631, 52574, 742, 20, 51904512, "Input"],
Cell[1798376, 52596, 7018, 210, 742391808, "Output"]
}, Closed]],
Cell[1805409, 52809, 212, 5, 26214400, "Text"],
Cell[CellGroupData[{
Cell[1805646, 52818, 1158, 27, 71827456, "Input"],
Cell[1806807, 52847, 103, 1, 176160768, "Output"]
}, Closed]],
Cell[1806925, 52851, 558, 9, 66060288, "Text"],
Cell[CellGroupData[{
Cell[1807508, 52864, 186, 4, 31457280, "Input"],
Cell[1807697, 52870, 561, 16, 176160768, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1808295, 52891, 195, 4, 23068672, "Input"],
Cell[1808493, 52897, 21868, 583, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1830398, 53485, 493, 14, 23068672, "Input"],
Cell[1830894, 53501, 11078, 309, 1300234240, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1842009, 53815, 507, 13, 23068672, "Input"],
Cell[1842519, 53830, 119, 2, 176160768, "Output"]
}, Closed]],
Cell[1842653, 53835, 263, 5, 26214400, "Text"],
Cell[CellGroupData[{
Cell[1842941, 53844, 609, 14, 31457280, "Input"],
Cell[1843553, 53860, 70, 1, 176160768, "Output"]
}, Closed]],
Cell[1843638, 53864, 89, 1, 26214400, "Text"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1843776, 53871, 303, 4, 95, "Subsection"],
Cell[1844082, 53877, 116, 1, 120, "Text"],
Cell[CellGroupData[{
Cell[1844223, 53882, 125, 2, 120, "Input"],
Cell[1844351, 53886, 569, 18, 7733248, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1844957, 53909, 79, 2, 88, "Input"],
Cell[1845039, 53913, 301, 9, 7733248, "Output"]
}, Closed]],
Cell[1845355, 53925, 98, 1, 100, "Text"],
Cell[CellGroupData[{
Cell[1845478, 53930, 146, 3, 120, "Input"],
Cell[1845627, 53935, 379, 12, 7733248, "Output"]
}, Closed]],
Cell[1846021, 53950, 664, 11, 252, "Text"],
Cell[CellGroupData[{
Cell[1846710, 53965, 1338, 31, 198, "Input"],
Cell[1848051, 53998, 70, 1, 7733248, "Output"]
}, Closed]],
Cell[1848136, 54002, 183, 2, 100, "Text"],
Cell[CellGroupData[{
Cell[1848344, 54008, 120, 2, 120, "Input"],
Cell[CellGroupData[{
Cell[1848489, 54014, 128, 2, 10616832, "Print",
 CellTags->"Info3628055261-3902486"],
Cell[1848620, 54018, 880, 24, 5636096, "Print",
 CellTags->"Info3628055261-3902486"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[1849549, 54048, 1832, 45, 166, "Input"],
Cell[1851384, 54095, 121, 2, 7733248, "Output"]
}, Closed]],
Cell[1851520, 54100, 1214, 29, 252, "Text"],
Cell[1852737, 54131, 405, 7, 196, "Text"],
Cell[CellGroupData[{
Cell[1853167, 54142, 126, 2, 7864320, "Input"],
Cell[1853296, 54146, 262, 5, 15204352, "Print",
 CellTags->"Info3628055436-3902486"]
}, Closed]],
Cell[1853573, 54154, 2425, 60, 25821184, "Input",
 Evaluatable->False],
Cell[1856001, 54216, 152, 2, 7864320, "Text"],
Cell[CellGroupData[{
Cell[1856178, 54222, 524, 15, 12976128, "Input"],
Cell[1856705, 54239, 69, 1, 7733248, "Output"]
}, Closed]],
Cell[1856789, 54243, 418, 7, 11534336, "Text"],
Cell[1857210, 54252, 289, 6, 7864320, "Text"],
Cell[CellGroupData[{
Cell[1857524, 54262, 643, 17, 12976128, "Input"],
Cell[1858170, 54281, 296, 9, 10616832, "Output"]
}, Closed]],
Cell[1858481, 54293, 547, 16, 5767168, "Input"],
Cell[1859031, 54311, 719, 12, 17825792, "Text"],
Cell[1859753, 54325, 342, 6, 12845056, "Text"],
Cell[1860098, 54333, 366, 7, 12845056, "Text"],
Cell[1860467, 54342, 736, 18, 7864320, "Input",
 Evaluatable->False],
Cell[1861206, 54362, 690, 10, 17825792, "Text"],
Cell[1861899, 54374, 994, 26, 7864320, "Input",
 Evaluatable->False],
Cell[1862896, 54402, 1251, 28, 18350080, "Text",
 CellTags->"geodc"],
Cell[1864150, 54432, 409, 9, 7864320, "Text"],
Cell[1864562, 54443, 425, 14, 8257536, "Input",
 Evaluatable->False],
Cell[1864990, 54459, 544, 17, 7864320, "Text"],
Cell[1865537, 54478, 389, 11, 7864320, "Input",
 Evaluatable->False],
Cell[1865929, 54491, 326, 6, 7864320, "Text"],
Cell[1866258, 54499, 614, 10, 12845056, "Text"],
Cell[1866875, 54511, 609, 18, 13369344, "Input",
 Evaluatable->False],
Cell[1867487, 54531, 1690, 43, 27656192, "Text"],
Cell[1869180, 54576, 502, 10, 17825792, "Text"],
Cell[1869685, 54588, 352, 4, 7864320, "Text"],
Cell[1870040, 54594, 271, 3, 7864320, "Text"],
Cell[1870314, 54599, 555, 16, 7864320, "Input",
 Evaluatable->False],
Cell[1870872, 54617, 1043, 18, 32768000, "Text"],
Cell[1871918, 54637, 537, 9, 12845056, "Text"],
Cell[1872458, 54648, 375, 11, 7864320, "Input",
 Evaluatable->False],
Cell[1872836, 54661, 519, 10, 12845056, "Text"],
Cell[1873358, 54673, 1170, 27, 17956864, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1874553, 54704, 176, 3, 7864320, "Input"],
Cell[1874732, 54709, 251, 4, 15204352, "Print",
 CellTags->"Info3628063421-3762495"]
}, Closed]],
Cell[1874998, 54716, 191, 2, 6553600, "Text"],
Cell[CellGroupData[{
Cell[1875214, 54722, 831, 23, 12976128, "Input"],
Cell[1876048, 54747, 263, 9, 10616832, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1876348, 54761, 621, 15, 5767168, "Input"],
Cell[1876972, 54778, 159, 5, 10616832, "Output"]
}, Closed]],
Cell[1877146, 54786, 469, 8, 11534336, "Text"],
Cell[CellGroupData[{
Cell[1877640, 54798, 449, 9, 7864320, "Input"],
Cell[1878092, 54809, 371, 13, 12058624, "Output"]
}, Closed]],
Cell[1878478, 54825, 1073, 19, 21495808, "Text"],
Cell[1879554, 54846, 1317, 24, 37748736, "Text"],
Cell[CellGroupData[{
Cell[1880896, 54874, 707, 18, 12976128, "Input"],
Cell[1881606, 54894, 213, 7, 44040192, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1881856, 54906, 658, 17, 10878976, "Input"],
Cell[1882517, 54925, 597, 21, 12058624, "Output"]
}, Closed]],
Cell[1883129, 54949, 398, 7, 11534336, "Text"],
Cell[CellGroupData[{
Cell[1883552, 54960, 517, 14, 7864320, "Input"],
Cell[1884072, 54976, 415, 14, 10616832, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1884524, 54995, 568, 15, 5767168, "Input"],
Cell[1885095, 55012, 407, 14, 12582912, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1885539, 55031, 639, 14, 5767168, "Input"],
Cell[1886181, 55047, 124, 3, 7733248, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1886342, 55055, 641, 14, 5767168, "Input"],
Cell[1886986, 55071, 162, 5, 12058624, "Output"]
}, Closed]],
Cell[1887163, 55079, 645, 13, 11534336, "Text"],
Cell[CellGroupData[{
Cell[1887833, 55096, 845, 23, 17956864, "Input"],
Cell[1888681, 55121, 79, 1, 7733248, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1888797, 55127, 906, 24, 15859712, "Input"],
Cell[1889706, 55153, 69, 1, 7733248, "Output"]
}, Closed]],
Cell[1889790, 55157, 719, 20, 11534336, "Text"],
Cell[1890512, 55179, 718, 20, 13893632, "Input",
 Evaluatable->False],
Cell[1891233, 55201, 371, 7, 7864320, "Text"],
Cell[1891607, 55210, 485, 9, 12845056, "Text"],
Cell[1892095, 55221, 724, 21, 7864320, "Input",
 Evaluatable->False],
Cell[1892822, 55244, 924, 14, 22806528, "Text"],
Cell[CellGroupData[{
Cell[1893771, 55262, 1365, 36, 12976128, "Input"],
Cell[1895139, 55300, 307, 9, 9961472, "Output"]
}, Closed]],
Cell[1895461, 55312, 467, 9, 11534336, "Text"],
Cell[1895931, 55323, 756, 21, 7864320, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1896712, 55348, 196, 3, 7864320, "Input"],
Cell[1896911, 55353, 242, 4, 15204352, "Print",
 CellTags->"Info3628065998-8815218"]
}, Closed]],
Cell[CellGroupData[{
Cell[1897190, 55362, 263, 4, 5767168, "Input"],
Cell[1897456, 55368, 89, 1, 7733248, "Output"]
}, Closed]],
Cell[1897560, 55372, 1178, 27, 11534336, "Text"],
Cell[CellGroupData[{
Cell[1898763, 55403, 1125, 25, 7864320, "Input"],
Cell[1899891, 55430, 28566, 652, 84541440, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1928494, 56087, 168, 3, 5767168, "Input"],
Cell[1928665, 56092, 70, 1, 7733248, "Output"]
}, Closed]],
Cell[1928750, 56096, 379, 8, 176, "Text"],
Cell[CellGroupData[{
Cell[1929154, 56108, 963, 23, 198, "Input"],
Cell[1930120, 56133, 124, 2, 7733248, "Output"]
}, Closed]],
Cell[1930259, 56138, 188, 4, 100, "Text"],
Cell[1930450, 56144, 485, 12, 120, "Input"],
Cell[1930938, 56158, 347, 7, 120, "Input"],
Cell[1931288, 56167, 263, 5, 120, "Text"],
Cell[CellGroupData[{
Cell[1931576, 56176, 1516, 38, 274, "Input"],
Cell[1933095, 56216, 183, 5, 11010048, "Output"]
}, Closed]],
Cell[1933293, 56224, 215, 4, 100, "Text"],
Cell[1933511, 56230, 221, 6, 120, "Text"],
Cell[1933735, 56238, 921, 25, 198, "Input"],
Cell[CellGroupData[{
Cell[1934681, 56267, 311, 8, 120, "Input"],
Cell[1934995, 56277, 363, 12, 13893632, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1935395, 56294, 380, 10, 88, "Input"],
Cell[1935778, 56306, 73, 1, 11010048, "Output"]
}, Closed]],
Cell[1935866, 56310, 292, 5, 176, "Text"],
Cell[1936161, 56317, 398, 7, 120, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[1936596, 56329, 336, 4, 75, "Subsection"],
Cell[1936935, 56335, 374, 6, 272, "Text"],
Cell[CellGroupData[{
Cell[1937334, 56345, 118, 2, 120, "Input"],
Cell[1937455, 56349, 288, 5, 15204352, "Print",
 CellTags->"Info3628088987-5827330"]
}, Closed]],
Cell[CellGroupData[{
Cell[1937780, 56359, 205, 5, 88, "Input"],
Cell[1937988, 56366, 181, 5, 11010048, "Output"]
}, Closed]],
Cell[1938184, 56374, 180, 2, 100, "Text"],
Cell[CellGroupData[{
Cell[1938389, 56380, 411, 9, 120, "Input"],
Cell[1938803, 56391, 968, 35, 17563648, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1939808, 56431, 409, 9, 88, "Input"],
Cell[1940220, 56442, 4946, 171, 54525952, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1945203, 56618, 461, 9, 88, "Input"],
Cell[1945667, 56629, 2389, 84, 30146560, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1948093, 56718, 815, 22, 166, "Input"],
Cell[1948911, 56742, 740, 21, 18350080, "Output"]
}, Closed]],
Cell[1949666, 56766, 333, 6, 176, "Text"],
Cell[1950002, 56774, 1606, 51, 292, "Input"],
Cell[CellGroupData[{
Cell[1951633, 56829, 126, 2, 120, "Input"],
Cell[1951762, 56833, 686, 24, 11665408, "Output"]
}, Closed]],
Cell[1952463, 56860, 457, 8, 252, "Text"],
Cell[CellGroupData[{
Cell[1952945, 56872, 1092, 29, 17956864, "Input"],
Cell[1954040, 56903, 302, 9, 11010048, "Output"]
}, Closed]],
Cell[1954357, 56915, 215, 6, 7864320, "Input"],
Cell[CellGroupData[{
Cell[1954597, 56925, 394, 9, 7864320, "Input"],
Cell[1954994, 56936, 1243, 45, 11665408, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1956274, 56986, 378, 10, 7864320, "Input"],
Cell[1956655, 56998, 144, 4, 11010048, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1956836, 57007, 416, 9, 7864320, "Input"],
Cell[1957255, 57018, 1817, 65, 17563648, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1959109, 57088, 378, 10, 7864320, "Input"],
Cell[1959490, 57100, 836, 31, 11010048, "Output"]
}, Closed]],
Cell[1960341, 57134, 248, 5, 7864320, "Text"],
Cell[CellGroupData[{
Cell[1960614, 57143, 119, 2, 7864320, "Input"],
Cell[1960736, 57147, 2013, 34, 15466496, "Print",
 CellTags->"Info3628089421-5827330"]
}, Closed]],
Cell[1962764, 57184, 140, 1, 7864320, "Text"],
Cell[1962907, 57187, 397, 10, 7864320, "Input"],
Cell[CellGroupData[{
Cell[1963329, 57201, 781, 17, 7864320, "Input"],
Cell[1964113, 57220, 1568, 49, 13893632, "Output"]
}, Closed]],
Cell[1965696, 57272, 360, 7, 12845056, "Text"],
Cell[1966059, 57281, 523, 15, 7864320, "Input"],
Cell[CellGroupData[{
Cell[1966607, 57300, 184, 4, 7864320, "Input"],
Cell[1966794, 57306, 136, 3, 11010048, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1966967, 57314, 698, 17, 166, "Input"],
Cell[1967668, 57333, 119, 3, 11010048, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1967824, 57341, 745, 18, 88, "Input"],
Cell[1968572, 57361, 96, 2, 11010048, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1968705, 57368, 845, 19, 166, "Input"],
Cell[1969553, 57389, 117, 3, 13893632, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1969707, 57397, 498, 15, 88, "Input"],
Cell[1970208, 57414, 191, 6, 11010048, "Output"]
}, Closed]],
Cell[1970414, 57423, 240, 6, 100, "Text"],
Cell[1970657, 57431, 510, 15, 120, "Input"],
Cell[1971170, 57448, 290, 6, 120, "Text"],
Cell[1971463, 57456, 582, 16, 120, "Input"],
Cell[1972048, 57474, 219, 6, 120, "Text"],
Cell[1972270, 57482, 605, 17, 120, "Input"],
Cell[1972878, 57501, 365, 10, 196, "Text"],
Cell[1973246, 57513, 584, 16, 120, "Input"],
Cell[1973833, 57531, 288, 5, 196, "Text"],
Cell[CellGroupData[{
Cell[1974146, 57540, 658, 16, 120, "Input"],
Cell[1974807, 57558, 2276, 73, 37879808, "Output"]
}, Closed]],
Cell[1977098, 57634, 227, 4, 88, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[1977362, 57643, 434, 5, 75, "Subsection"],
Cell[1977799, 57650, 136, 1, 120, "Text"],
Cell[CellGroupData[{
Cell[1977960, 57655, 203, 5, 120, "Input"],
Cell[1978166, 57662, 644, 21, 1933312, "Output"]
}, Closed]],
Cell[1978825, 57686, 158, 3, 100, "Text"],
Cell[1978986, 57691, 543, 12, 120, "Input"],
Cell[CellGroupData[{
Cell[1979554, 57707, 222, 4, 120, "Input"],
Cell[1979779, 57713, 695, 22, 2752512, "Output"]
}, Closed]],
Cell[1980489, 57738, 150, 2, 100, "Text"],
Cell[1980642, 57742, 740, 15, 120, "Input"],
Cell[CellGroupData[{
Cell[1981407, 57761, 298, 8, 120, "Input"],
Cell[1981708, 57771, 124, 2, 2752512, "Output"]
}, Closed]],
Cell[1981847, 57776, 200, 4, 100, "Text"],
Cell[1982050, 57782, 265, 3, 120, "Text"],
Cell[CellGroupData[{
Cell[1982340, 57789, 332, 8, 120, "Input"],
Cell[1982675, 57799, 622, 19, 1933312, "Output"]
}, Closed]],
Cell[1983312, 57821, 249, 3, 100, "Text"],
Cell[CellGroupData[{
Cell[1983586, 57828, 265, 6, 120, "Input"],
Cell[1983854, 57836, 395, 13, 1933312, "Output"]
}, Closed]],
Cell[1984264, 57852, 138, 1, 100, "Text"],
Cell[CellGroupData[{
Cell[1984427, 57857, 287, 7, 120, "Input"],
Cell[1984717, 57866, 776, 25, 1933312, "Output"]
}, Closed]],
Cell[1985508, 57894, 453, 8, 252, "Text"],
Cell[1985964, 57904, 1675, 50, 548, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[1987664, 57958, 167, 3, 240, "Input"],
Cell[1987834, 57963, 5289, 164, 9502720, "Output"]
}, Closed]],
Cell[1993138, 58130, 364, 6, 352, "Text"],
Cell[CellGroupData[{
Cell[1993527, 58140, 1503, 28, 240, "Input"],
Cell[1995033, 58170, 3355, 106, 5865472, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1998425, 58281, 226, 5, 176, "Input"],
Cell[1998654, 58288, 1782, 54, 3473408, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[2000473, 58347, 2025, 58, 484, "Input"],
Cell[2002501, 58407, 94, 1, 1933312, "Output"]
}, Closed]],
Cell[2002610, 58411, 155, 2, 200, "Text"],
Cell[CellGroupData[{
Cell[2002790, 58417, 1524, 29, 396, "Input"],
Cell[2004317, 58448, 75, 1, 1933312, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[2004429, 58454, 1458, 27, 332, "Input"],
Cell[2005890, 58483, 73, 1, 1933312, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[2006000, 58489, 2001, 43, 484, "Input"],
Cell[2008004, 58534, 73, 1, 1933312, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[2008114, 58540, 1459, 25, 176, "Input"],
Cell[2009576, 58567, 1242, 41, 1933312, "Output"]
}, Closed]],
Cell[2010833, 58611, 160, 2, 200, "Text"],
Cell[CellGroupData[{
Cell[2011018, 58617, 954, 17, 240, "Input"],
Cell[2011975, 58636, 583, 10, 3670016, "Message"],
Cell[2012561, 58648, 583, 10, 3670016, "Message"],
Cell[2013147, 58660, 5280, 160, 8716288, "Output"]
}, Closed]],
Cell[2018442, 58823, 366, 4, 200, "Text"],
Cell[2018811, 58829, 6296, 178, 4348, "Input"],
Cell[2025110, 59009, 507, 8, 392, "Text"],
Cell[2025620, 59019, 878, 19, 396, "Input"],
Cell[2026501, 59040, 351, 8, 240, "Input"],
Cell[2026855, 59050, 950, 20, 396, "Input"],
Cell[2027808, 59072, 366, 4, 240, "Text"],
Cell[2028177, 59078, 5389, 149, 3436, "Input"],
Cell[2033569, 59229, 413, 11, 240, "Input"],
Cell[2033985, 59242, 1002, 21, 396, "Input"],
Cell[2034990, 59265, 366, 4, 240, "Text"],
Cell[2035359, 59271, 13893, 377, 8300, "Input"],
Cell[2049255, 59650, 228, 5, 240, "Text"],
Cell[2049486, 59657, 1433, 30, 548, "Input"],
Cell[2050922, 59689, 366, 4, 240, "Text"],
Cell[2051291, 59695, 5657, 156, 3284, "Input",
 InitializationCell->True],
Cell[2056951, 59853, 91, 1, 240, "Text"],
Cell[CellGroupData[{
Cell[2057067, 59858, 600, 14, 240, "Input"],
Cell[2057670, 59874, 98, 1, 1933312, "Output"]
}, Closed]],
Cell[2057783, 59878, 636, 13, 352, "Text"],
Cell[2058422, 59893, 635, 18, 240, "Input",
 Evaluatable->False],
Cell[2059060, 59913, 112, 1, 240, "Text"],
Cell[2059175, 59916, 1680, 49, 444, "Input",
 Evaluatable->False],
Cell[2060858, 59967, 215, 3, 240, "Text"],
Cell[2061076, 59972, 949, 26, 240, "Input",
 Evaluatable->False],
Cell[2062028, 60000, 894, 15, 240, "Text"],
Cell[CellGroupData[{
Cell[2062947, 60019, 1832, 51, 700, "Input"],
Cell[2064782, 60072, 72, 1, 1933312, "Output"]
}, Closed]],
Cell[2064869, 60076, 657, 10, 352, "Text"],
Cell[2065529, 60088, 273, 6, 240, "Text"],
Cell[2065805, 60096, 1567, 41, 248, "Text"],
Cell[2067375, 60139, 148, 2, 240, "Text"],
Cell[2067526, 60143, 254, 6, 240, "Input"],
Cell[2067783, 60151, 2705, 57, 1152, "Text"],
Cell[2070491, 60210, 480, 8, 544, "Text"],
Cell[CellGroupData[{
Cell[2070996, 60222, 1410, 29, 396, "Input"],
Cell[2072409, 60253, 14980, 406, 16973824, "Output"]
}, Closed]],
Cell[2087404, 60662, 202, 4, 200, "Text"],
Cell[CellGroupData[{
Cell[2087631, 60670, 1519, 28, 396, "Input"],
Cell[2089153, 60700, 70, 1, 1933312, "Output"]
}, Closed]],
Cell[2089238, 60704, 274, 7, 176, "Input"],
Cell[CellGroupData[{
Cell[2089537, 60715, 1414, 26, 396, "Input"],
Cell[2090954, 60743, 4089, 127, 7274496, "Output"]
}, Closed]],
Cell[2095058, 60873, 5054, 145, 3220, "Input"],
Cell[2100115, 61020, 548, 13, 240, "Input"],
Cell[CellGroupData[{
Cell[2100688, 61037, 1518, 28, 396, "Input"],
Cell[2102209, 61067, 10605, 306, 13336576, "Output"]
}, Closed]],
Cell[2112829, 61376, 13120, 355, 7476, "Input"],
Cell[2125952, 61733, 265, 5, 240, "Text"],
Cell[CellGroupData[{
Cell[2126242, 61742, 782, 21, 548, "Input"],
Cell[2127027, 61765, 4280, 130, 7438336, "Output"]
}, Closed]],
Cell[2131322, 61898, 5282, 148, 1534, "Input"],
Cell[2136607, 62048, 119, 1, 120, "Text"],
Cell[2136729, 62051, 1277, 30, 274, "Input"],
Cell[2138009, 62083, 145, 1, 120, "Text"],
Cell[2138157, 62086, 968, 20, 198, "Input"],
Cell[2139128, 62108, 176, 2, 120, "Text"],
Cell[CellGroupData[{
Cell[2139329, 62114, 1107, 21, 198, "Input"],
Cell[2140439, 62137, 581, 10, 3670016, "Message"],
Cell[2141023, 62149, 11595, 322, 13533184, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[2152655, 62476, 1476, 31, 242, "Input"],
Cell[2154134, 62509, 10711, 308, 13402112, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[2164882, 62822, 478, 11, 88, "Input"],
Cell[2165363, 62835, 72, 1, 2752512, "Output"]
}, Closed]],
Cell[2165450, 62839, 85, 1, 100, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[2165572, 62845, 289, 4, 75, "Subsection"],
Cell[CellGroupData[{
Cell[2165886, 62853, 610, 8, 76, "Subsubsection",
 CellTags->"helix"],
Cell[2166499, 62863, 429, 10, 784, "Text"],
Cell[2166931, 62875, 216, 5, 480, "Input"],
Cell[CellGroupData[{
Cell[2167172, 62884, 121, 2, 480, "Input"],
Cell[2167296, 62888, 326, 9, 3776, "Output"]
}, Closed]],
Cell[2167637, 62900, 95, 1, 400, "Text"],
Cell[CellGroupData[{
Cell[2167757, 62905, 201, 5, 480, "Input"],
Cell[2167961, 62912, 5724, 188, 15936, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[2173722, 63105, 115, 2, 352, "Input"],
Cell[2173840, 63109, 373, 6, 9600, "Print",
 CellTags->"Info3628233781-2217871"]
}, Closed]],
Cell[2174228, 63118, 283, 7, 400, "Text"],
Cell[2174514, 63127, 842, 24, 792, "Input",
 InitializationCell->True],
Cell[2175359, 63153, 428, 9, 1392, "Text"],
Cell[CellGroupData[{
Cell[2175812, 63166, 143, 3, 480, "Input"],
Cell[2175958, 63171, 335, 6, 9600, "Print",
 CellTags->"Info3628235851-2217871"]
}, Closed]],
Cell[CellGroupData[{
Cell[2176330, 63182, 133, 2, 352, "Input"],
Cell[2176466, 63186, 111, 1, 3776, "Output"]
}, Closed]],
Cell[2176592, 63190, 123, 1, 400, "Text"],
Cell[2176718, 63193, 475, 11, 480, "Input"],
Cell[2177196, 63206, 667, 18, 792, "Input"],
Cell[2177866, 63226, 139, 1, 480, "Text"],
Cell[2178008, 63229, 304, 7, 480, "Input"],
Cell[2178315, 63238, 591, 10, 1088, "Text"],
Cell[2178909, 63250, 655, 19, 480, "Input"],
Cell[2179567, 63271, 94, 1, 480, "Text"],
Cell[CellGroupData[{
Cell[2179686, 63276, 173, 4, 480, "Input"],
Cell[2179862, 63282, 7311, 195, 23488, "Output"],
Cell[2187176, 63479, 68297, 1920, 128064, "Output"]
}, Closed]],
Cell[2255488, 65402, 388, 8, 704, "Text"],
Cell[2255879, 65412, 3402, 101, 1168, "Input"],
Cell[2259284, 65515, 191, 4, 480, "Text"],
Cell[CellGroupData[{
Cell[2259500, 65523, 1333, 42, 1096, "Input"],
Cell[2260836, 65567, 30973, 685, 50240, "Output"]
}, Closed]],
Cell[2291824, 66255, 143, 2, 400, "Text"],
Cell[CellGroupData[{
Cell[2291992, 66261, 384, 10, 480, "Input"],
Cell[2292379, 66273, 77, 1, 3776, "Output"]
}, Closed]],
Cell[2292471, 66277, 180, 2, 400, "Text"],
Cell[2292654, 66281, 163, 2, 480, "Text"],
Cell[CellGroupData[{
Cell[2292842, 66287, 116, 2, 480, "Input"],
Cell[2292961, 66291, 375, 6, 9600, "Print",
 CellTags->"Info3628839108-8634717"]
}, Closed]],
Cell[2293351, 66300, 574, 16, 664, "Input",
 Evaluatable->False],
Cell[2293928, 66318, 124, 1, 480, "Text"],
Cell[2294055, 66321, 1060, 31, 792, "Input",
 Evaluatable->False],
Cell[2295118, 66354, 304, 6, 480, "Text"],
Cell[CellGroupData[{
Cell[2295447, 66364, 1062, 31, 792, "Input"],
Cell[2296512, 66397, 671, 21, 5184, "Output"]
}, Closed]],
Cell[2297198, 66421, 117, 1, 400, "Text"],
Cell[CellGroupData[{
Cell[2297340, 66426, 488, 13, 480, "Input"],
Cell[2297831, 66441, 151, 2, 3776, "Output"]
}, Closed]],
Cell[2297997, 66446, 134, 1, 400, "Text"],
Cell[2298134, 66449, 468, 15, 480, "Input"],
Cell[2298605, 66466, 402, 8, 1088, "Text"],
Cell[2299010, 66476, 325, 9, 480, "Input"],
Cell[2299338, 66487, 97, 1, 480, "Text"],
Cell[2299438, 66490, 257, 5, 480, "Text"],
Cell[2299698, 66497, 314, 7, 784, "Text"],
Cell[2300015, 66506, 326, 9, 480, "Input"],
Cell[2300344, 66517, 301, 6, 784, "Text"],
Cell[CellGroupData[{
Cell[2300670, 66527, 1462, 42, 856, "Input"],
Cell[2302135, 66571, 245, 6, 5376, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[2302417, 66582, 1255, 38, 728, "Input"],
Cell[2303675, 66622, 96, 1, 5376, "Output"]
}, Closed]],
Cell[2303786, 66626, 2630, 46, 3744, "Text"],
Cell[2306419, 66674, 1165, 35, 1096, "Input"],
Cell[CellGroupData[{
Cell[2307609, 66713, 322, 8, 480, "Input"],
Cell[2307934, 66723, 3928, 114, 12672, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[2311899, 66842, 271, 7, 352, "Input"],
Cell[2312173, 66851, 1206, 38, 6720, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[2313416, 66894, 614, 18, 400, "Input"],
Cell[2314033, 66914, 6504, 190, 23808, "Output"]
}, Closed]],
Cell[2320552, 67107, 146, 3, 352, "Input"],
Cell[CellGroupData[{
Cell[2320723, 67114, 249, 6, 480, "Input"],
Cell[2320975, 67122, 1172, 36, 5824, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[2322184, 67163, 149, 3, 352, "Input"],
Cell[2322336, 67168, 382, 6, 9600, "Print",
 CellTags->"Info3629004696-3111076"]
}, Closed]],
Cell[2322733, 67177, 328, 9, 352, "Input"],
Cell[2323064, 67188, 399, 10, 800, "Text"],
Cell[CellGroupData[{
Cell[2323488, 67202, 289, 7, 480, "Input"],
Cell[2323780, 67211, 5346, 148, 19456, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[2329163, 67364, 507, 15, 400, "Input"],
Cell[2329673, 67381, 984, 31, 5632, "Output"]
}, Closed]],
Cell[2330672, 67415, 1135, 35, 720, "Input"],
Cell[2331810, 67452, 173, 2, 480, "Text"],
Cell[2331986, 67456, 478, 11, 480, "Input"],
Cell[2332467, 67469, 714, 11, 1392, "Text"],
Cell[2333184, 67482, 944, 23, 480, "Input"],
Cell[CellGroupData[{
Cell[2334153, 67509, 216, 5, 480, "Input"],
Cell[2334372, 67516, 3319, 101, 11968, "Output"]
}, Closed]],
Cell[2337706, 67620, 109, 1, 400, "Text"],
Cell[CellGroupData[{
Cell[2337840, 67625, 196, 4, 480, "Input"],
Cell[2338039, 67631, 371, 11, 6976, "Output"]
}, Closed]],
Cell[2338425, 67645, 122, 1, 400, "Text"],
Cell[2338550, 67648, 570, 15, 480, "Input"],
Cell[2339123, 67665, 324, 6, 784, "Text"],
Cell[2339450, 67673, 183, 5, 480, "Text"],
Cell[2339636, 67680, 154, 2, 480, "Text"],
Cell[2339793, 67684, 665, 18, 480, "Input"],
Cell[2340461, 67704, 158, 2, 480, "Text"],
Cell[2340622, 67708, 615, 16, 480, "Input"],
Cell[2341240, 67726, 121, 1, 480, "Text"],
Cell[2341364, 67729, 392, 10, 480, "Input"],
Cell[2341759, 67741, 123, 1, 480, "Text"],
Cell[CellGroupData[{
Cell[2341907, 67746, 360, 9, 480, "Input"],
Cell[2342270, 67757, 425, 14, 7744, "Output"]
}, Closed]],
Cell[2342710, 67774, 231, 6, 352, "Input"],
Cell[2342944, 67782, 472, 11, 480, "Input"],
Cell[2343419, 67795, 252, 6, 480, "Input"],
Cell[2343674, 67803, 297, 6, 1088, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[2344008, 67814, 626, 9, 60, "Subsubsection",
 CellTags->"helix"],
Cell[2344637, 67825, 341, 4, 480, "Text"],
Cell[2344981, 67831, 623, 13, 480, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[2345629, 67848, 150, 3, 480, "Input"],
Cell[2345782, 67853, 348, 12, 736, "Output"]
}, Closed]],
Cell[2346145, 67868, 204, 5, 480, "Text"],
Cell[CellGroupData[{
Cell[2346374, 67877, 453, 10, 480, "Input"],
Cell[2346830, 67889, 92, 1, 472, "Output"]
}, Closed]],
Cell[2346937, 67893, 764, 15, 480, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[2347726, 67912, 239, 5, 480, "Input"],
Cell[2347968, 67919, 1397, 48, 1584, "Output"]
}, Closed]],
Cell[2349380, 67970, 433, 8, 784, "Text"],
Cell[CellGroupData[{
Cell[2349838, 67982, 413, 10, 480, "Input"],
Cell[2350254, 67994, 516, 18, 736, "Output"]
}, Closed]],
Cell[2350785, 68015, 784, 18, 792, "Input"],
Cell[CellGroupData[{
Cell[2351594, 68037, 397, 9, 480, "Input"],
Cell[2351994, 68048, 317, 10, 784, "Output"]
}, Closed]],
Cell[2352326, 68061, 391, 7, 784, "Text"],
Cell[CellGroupData[{
Cell[2352742, 68072, 241, 6, 480, "Input"],
Cell[2352986, 68080, 156, 4, 928, "Output"]
}, Closed]],
Cell[2353157, 68087, 173, 4, 480, "Text"],
Cell[2353333, 68093, 255, 7, 480, "Input"],
Cell[CellGroupData[{
Cell[2353613, 68104, 217, 6, 480, "Input"],
Cell[2353833, 68112, 1830, 64, 2824, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[2355700, 68181, 334, 8, 480, "Input"],
Cell[2356037, 68191, 1421, 51, 1736, "Output"]
}, Closed]],
Cell[2357473, 68245, 1719, 57, 1952, "Input"],
Cell[2359195, 68304, 115, 1, 480, "Text"],
Cell[CellGroupData[{
Cell[2359335, 68309, 940, 20, 792, "Input"],
Cell[2360278, 68331, 123, 2, 472, "Output"]
}, Closed]],
Cell[2360416, 68336, 213, 3, 480, "Text"],
Cell[2360632, 68341, 525, 14, 480, "Input"],
Cell[2361160, 68357, 689, 17, 792, "Input"],
Cell[CellGroupData[{
Cell[2361874, 68378, 610, 15, 480, "Input"],
Cell[2362487, 68395, 961, 35, 1584, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[2363485, 68435, 610, 15, 480, "Input"],
Cell[2364098, 68452, 1092, 41, 1824, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[2365227, 68498, 515, 12, 480, "Input"],
Cell[2365745, 68512, 2424, 83, 3616, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[2368206, 68600, 245, 6, 480, "Input"],
Cell[2368454, 68608, 291, 10, 888, "Output"]
}, Closed]],
Cell[2368760, 68621, 539, 16, 760, "Input"],
Cell[CellGroupData[{
Cell[2369324, 68641, 162, 3, 480, "Input"],
Cell[2369489, 68646, 1054, 40, 1720, "Output"]
}, Closed]],
Cell[2370558, 68689, 368, 9, 480, "Input"],
Cell[CellGroupData[{
Cell[2370951, 68702, 343, 8, 480, "Input"],
Cell[2371297, 68712, 639, 23, 1464, "Output"]
}, Closed]],
Cell[2371951, 68738, 846, 28, 1760, "Input"],
Cell[CellGroupData[{
Cell[2372822, 68770, 288, 7, 480, "Input"],
Cell[2373113, 68779, 248, 9, 1080, "Output"]
}, Closed]],
Cell[2373376, 68791, 284, 6, 784, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[2373697, 68802, 330, 4, 60, "Subsubsection"],
Cell[2374030, 68808, 453, 9, 392, "Text"],
Cell[2374486, 68819, 759, 19, 240, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[2375270, 68842, 220, 4, 240, "Input"],
Cell[2375493, 68848, 265, 5, 464, "Print",
 CellTags->"Info3628232522-2217871"]
}, Closed]],
Cell[CellGroupData[{
Cell[2375795, 68858, 134, 2, 240, "Input"],
Cell[2375932, 68862, 115, 1, 336, "Output"]
}, Closed]],
Cell[2376062, 68866, 227, 5, 240, "Text"],
Cell[2376292, 68873, 301, 8, 240, "Input"],
Cell[2376596, 68883, 262, 5, 240, "Text"],
Cell[2376861, 68890, 275, 7, 240, "Input"],
Cell[2377139, 68899, 215, 2, 240, "Text"],
Cell[2377357, 68903, 408, 10, 240, "Input"],
Cell[2377768, 68915, 226, 5, 392, "Text"],
Cell[2377997, 68922, 303, 8, 240, "Input"],
Cell[2378303, 68932, 291, 7, 240, "Text"],
Cell[2378597, 68941, 149, 3, 240, "Input"],
Cell[2378749, 68946, 785, 14, 844, "Text"],
Cell[2379537, 68962, 477, 11, 240, "Input"],
Cell[2380017, 68975, 334, 6, 240, "Text"],
Cell[2380354, 68983, 1288, 32, 248, "Text"],
Cell[2381645, 69017, 340, 6, 392, "Text"],
Cell[2381988, 69025, 482, 13, 240, "Input",
 Evaluatable->False],
Cell[2382473, 69040, 322, 6, 392, "Text"],
Cell[2382798, 69048, 824, 21, 240, "Input",
 Evaluatable->False],
Cell[2383625, 69071, 290, 5, 392, "Text"],
Cell[2383918, 69078, 1369, 35, 240, "Input",
 Evaluatable->False],
Cell[2385290, 69115, 480, 10, 392, "Text"],
Cell[2385773, 69127, 441, 11, 240, "Input"],
Cell[2386217, 69140, 194, 3, 240, "Text"],
Cell[2386414, 69145, 740, 23, 396, "Input"],
Cell[2387157, 69170, 321, 6, 392, "Text"],
Cell[CellGroupData[{
Cell[2387503, 69180, 946, 28, 548, "Input"],
Cell[2388452, 69210, 75, 1, 336, "Output"]
}, Closed]],
Cell[2388542, 69214, 645, 15, 552, "Text"],
Cell[CellGroupData[{
Cell[2389212, 69233, 1284, 33, 396, "Input"],
Cell[2390499, 69268, 681, 22, 336, "Output"]
}, Closed]],
Cell[2391195, 69293, 996, 18, 1000, "Text"],
Cell[2392194, 69313, 710, 18, 696, "Text"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2392953, 69337, 263, 3, 95, "Subsection"],
Cell[2393219, 69342, 1313, 18, 288, "Text"],
Cell[CellGroupData[{
Cell[2394557, 69364, 335, 4, 76, "Subsubsection"],
Cell[2394895, 69370, 322, 7, 196, "Text"],
Cell[2395220, 69379, 640, 18, 120, "Input",
 InitializationCell->True],
Cell[2395863, 69399, 374, 8, 120, "Input"],
Cell[2396240, 69409, 262, 7, 120, "Text"],
Cell[2396505, 69418, 488, 10, 120, "Input"],
Cell[2396996, 69430, 126, 1, 120, "Text"],
Cell[2397125, 69433, 352, 7, 120, "Input"],
Cell[2397480, 69442, 178, 4, 120, "Input"],
Cell[CellGroupData[{
Cell[2397683, 69450, 116, 2, 120, "Input"],
Cell[2397802, 69454, 825, 12, 504, "Print",
 CellTags->"Info3629190502-5022241"]
}, Closed]],
Cell[CellGroupData[{
Cell[2398664, 69471, 254, 6, 88, "Input"],
Cell[2398921, 69479, 438, 14, 168, "Output"]
}, Closed]],
Cell[2399374, 69496, 412, 10, 88, "Input"],
Cell[CellGroupData[{
Cell[2399811, 69510, 183, 5, 120, "Input"],
Cell[2399997, 69517, 440, 14, 168, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[2400474, 69536, 243, 8, 88, "Input"],
Cell[2400720, 69546, 110432, 1787, 1188, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[2511189, 71338, 1224, 24, 88, "Input"],
Cell[2512416, 71364, 128785, 2088, 1166, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[2641238, 73457, 1176, 23, 88, "Input"],
Cell[2642417, 73482, 130910, 2118, 1166, "Output"]
}, Closed]],
Cell[2773342, 75603, 116, 1, 100, "Text"],
Cell[2773461, 75606, 257, 6, 120, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[2773743, 75616, 136, 4, 120, "Input"],
Cell[2773882, 75622, 301, 9, 168, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[2774220, 75636, 1576, 31, 88, "Input"],
Cell[2775799, 75669, 167326, 2731, 1838, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[2943162, 78405, 136, 4, 88, "Input"],
Cell[2943301, 78411, 268, 8, 168, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[2943606, 78424, 1398, 26, 88, "Input"],
Cell[2945007, 78452, 178346, 2927, 696, "Output"]
}, Closed]],
Cell[3123368, 81382, 206, 4, 176, "Text"],
Cell[CellGroupData[{
Cell[3123599, 81390, 674, 18, 120, "Input"],
Cell[3124276, 81410, 698, 22, 168, "Output"]
}, Closed]],
Cell[3124989, 81435, 327, 5, 176, "Text"],
Cell[CellGroupData[{
Cell[3125341, 81444, 553, 14, 120, "Input"],
Cell[3125897, 81460, 2138, 70, 262, "Output"]
}, Closed]],
Cell[3128050, 81533, 110, 1, 100, "Text"],
Cell[3128163, 81536, 533, 18, 120, "Input"],
Cell[3128699, 81556, 373, 7, 196, "Text"],
Cell[CellGroupData[{
Cell[3129097, 81567, 467, 14, 120, "Input"],
Cell[3129567, 81583, 555, 19, 178, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[3130159, 81607, 1096, 30, 242, "Input"],
Cell[3131258, 81639, 223, 7, 168, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[3131518, 81651, 465, 14, 88, "Input"],
Cell[3131986, 81667, 1396, 44, 268, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[3133419, 81716, 1291, 36, 242, "Input"],
Cell[3134713, 81754, 781, 25, 178, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[3135531, 81784, 1255, 32, 318, "Input"],
Cell[3136789, 81818, 520, 16, 168, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[3137346, 81839, 1093, 27, 318, "Input"],
Cell[3138442, 81868, 72, 1, 168, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[3138551, 81874, 1298, 33, 318, "Input"],
Cell[3139852, 81909, 231, 7, 222, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[3140120, 81921, 1194, 29, 242, "Input"],
Cell[3141317, 81952, 770, 24, 222, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[3142124, 81981, 420, 9, 88, "Input"],
Cell[3142547, 81992, 1883, 59, 268, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[3144467, 82056, 984, 26, 242, "Input"],
Cell[3145454, 82084, 781, 24, 222, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[3146272, 82113, 668, 15, 166, "Input"],
Cell[3146943, 82130, 69, 1, 168, "Output"]
}, Closed]],
Cell[3147027, 82134, 384, 8, 176, "Text"],
Cell[CellGroupData[{
Cell[3147436, 82146, 1074, 27, 274, "Input"],
Cell[3148513, 82175, 831, 26, 222, "Output"]
}, Closed]],
Cell[3149359, 82204, 733, 14, 328, "Text"],
Cell[CellGroupData[{
Cell[3150117, 82222, 1171, 29, 274, "Input"],
Cell[3151291, 82253, 561, 18, 168, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[3151889, 82276, 190, 5, 88, "Input"],
Cell[3152082, 82283, 328, 10, 168, "Output"]
}, Closed]],
Cell[3152425, 82296, 180, 4, 100, "Text"],
Cell[3152608, 82302, 314, 9, 120, "Input"],
Cell[3152925, 82313, 349, 6, 196, "Text"],
Cell[3153277, 82321, 120, 2, 120, "Input"],
Cell[CellGroupData[{
Cell[3153422, 82327, 1004, 24, 350, "Input"],
Cell[3154429, 82353, 584, 18, 168, "Output"]
}, Closed]],
Cell[3155028, 82374, 635, 10, 176, "Text"],
Cell[3155666, 82386, 526, 13, 120, "Input"],
Cell[3156195, 82401, 673, 10, 348, "Text"],
Cell[CellGroupData[{
Cell[3156893, 82415, 235, 6, 120, "Input"],
Cell[3157131, 82423, 2711, 82, 324, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[3159879, 82510, 927, 28, 184, "Input"],
Cell[3160809, 82540, 781, 24, 222, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[3161627, 82569, 1033, 29, 234, "Input"],
Cell[3162663, 82600, 831, 26, 222, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[3163543, 82632, 302, 4, 60, "Subsubsection"],
Cell[3163848, 82638, 580, 11, 196, "Text",
 CellTags->"sfrev"],
Cell[3164431, 82651, 114, 2, 120, "Input"],
Cell[3164548, 82655, 347, 10, 120, "Input",
 InitializationCell->True],
Cell[3164898, 82667, 151, 4, 120, "Input"],
Cell[3165052, 82673, 119, 1, 120, "Text"],
Cell[3165174, 82676, 268, 6, 120, "Input"],
Cell[3165445, 82684, 170, 3, 120, "Input"],
Cell[3165618, 82689, 195, 4, 120, "Text"],
Cell[3165816, 82695, 900, 22, 132, "Input"],
Cell[3166719, 82719, 107, 1, 120, "Text"],
Cell[3166829, 82722, 516, 15, 120, "Input"],
Cell[3167348, 82739, 383, 11, 120, "Input"],
Cell[3167734, 82752, 246, 5, 120, "Input"],
Cell[3167983, 82759, 384, 11, 120, "Input"],
Cell[3168370, 82772, 225, 5, 120, "Input"],
Cell[3168598, 82779, 662, 15, 196, "Text"],
Cell[3169263, 82796, 118, 2, 120, "Input"],
Cell[3169384, 82800, 277, 7, 120, "Input"],
Cell[3169664, 82809, 423, 12, 120, "Input"],
Cell[3170090, 82823, 144, 1, 120, "Text"],
Cell[3170237, 82826, 320, 9, 120, "Input"],
Cell[3170560, 82837, 1041, 30, 198, "Input"],
Cell[3171604, 82869, 722, 17, 120, "Input"],
Cell[3172329, 82888, 646, 16, 120, "Input"],
Cell[3172978, 82906, 528, 14, 120, "Input"],
Cell[3173509, 82922, 215, 5, 120, "Text"],
Cell[3173727, 82929, 553, 15, 120, "Input"],
Cell[3174283, 82946, 175, 4, 120, "Input"],
Cell[3174461, 82952, 400, 7, 196, "Text"],
Cell[3174864, 82961, 808, 19, 198, "Input"],
Cell[3175675, 82982, 112, 1, 120, "Text"],
Cell[3175790, 82985, 487, 12, 120, "Input"],
Cell[3176280, 82999, 474, 13, 120, "Input"],
Cell[3176757, 83014, 16, 0, 120, "Text"],
Cell[3176776, 83016, 221, 4, 120, "Text"],
Cell[3177000, 83022, 420, 12, 120, "Input"],
Cell[3177423, 83036, 497, 8, 272, "Text"],
Cell[3177923, 83046, 388, 12, 120, "Input"],
Cell[3178314, 83060, 487, 11, 120, "Input"],
Cell[3178804, 83073, 595, 17, 120, "Input"],
Cell[3179402, 83092, 385, 9, 120, "Input"],
Cell[3179790, 83103, 1329, 35, 218, "Input"],
Cell[3181122, 83140, 500, 8, 196, "Text"],
Cell[3181625, 83150, 211, 4, 196, "Text"],
Cell[3181839, 83156, 600, 15, 120, "Input"],
Cell[3182442, 83173, 203, 5, 120, "Input"],
Cell[3182648, 83180, 509, 12, 120, "Input"],
Cell[3183160, 83194, 244, 4, 120, "Text"],
Cell[3183407, 83200, 494, 13, 120, "Input"],
Cell[3183904, 83215, 123, 2, 120, "Input"],
Cell[3184030, 83219, 409, 12, 120, "Input"],
Cell[3184442, 83233, 487, 11, 120, "Input"],
Cell[3184932, 83246, 621, 17, 120, "Input"],
Cell[3185556, 83265, 385, 9, 120, "Input"],
Cell[3185944, 83276, 1381, 36, 218, "Input"],
Cell[3187328, 83314, 192, 4, 120, "Text"],
Cell[3187523, 83320, 1898, 52, 650, "Input"],
Cell[3189424, 83374, 323, 8, 196, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[3189784, 83387, 328, 5, 60, "Subsubsection"],
Cell[3190115, 83394, 250, 5, 120, "Text"],
Cell[3190368, 83401, 178, 5, 120, "Input"],
Cell[3190549, 83408, 108, 1, 120, "Text"],
Cell[3190660, 83411, 276, 7, 120, "Input"],
Cell[3190939, 83420, 153, 3, 120, "Text"],
Cell[3191095, 83425, 281, 7, 120, "Input"],
Cell[3191379, 83434, 283, 5, 120, "Text"],
Cell[3191665, 83441, 533, 15, 120, "Input"],
Cell[3192201, 83458, 196, 4, 120, "Text"],
Cell[3192400, 83464, 587, 16, 120, "Input"],
Cell[3192990, 83482, 127, 2, 120, "Input"],
Cell[3193120, 83486, 204, 4, 120, "Text"],
Cell[3193327, 83492, 212, 5, 120, "Input"],
Cell[3193542, 83499, 237, 6, 120, "Input"],
Cell[3193782, 83507, 241, 5, 120, "Text"],
Cell[3194026, 83514, 679, 19, 198, "Input"],
Cell[3194708, 83535, 129, 2, 120, "Input"],
Cell[3194840, 83539, 283, 4, 120, "Text"],
Cell[3195126, 83545, 657, 21, 120, "Input",
 InitializationCell->True],
Cell[3195786, 83568, 288, 6, 120, "Text"],
Cell[3196077, 83576, 897, 22, 120, "Input"],
Cell[3196977, 83600, 153, 3, 120, "Input"],
Cell[3197133, 83605, 444, 11, 120, "Input"],
Cell[3197580, 83618, 570, 12, 120, "Input"],
Cell[3198153, 83632, 312, 8, 120, "Input"],
Cell[3198468, 83642, 93, 1, 120, "Text"],
Cell[3198564, 83645, 8749, 236, 2630, "Input",
 InitializationCell->True],
Cell[3207316, 83883, 362, 8, 120, "Input"],
Cell[3207681, 83893, 128, 2, 120, "Text"],
Cell[3207812, 83897, 8830, 238, 2630, "Input",
 InitializationCell->True],
Cell[3216645, 84137, 186, 3, 120, "Text"],
Cell[3216834, 84142, 359, 10, 120, "Input"],
Cell[3217196, 84154, 293, 7, 120, "Text"],
Cell[3217492, 84163, 441, 9, 348, "Text"],
Cell[3217936, 84174, 187, 4, 120, "Input"],
Cell[3218126, 84180, 236, 4, 196, "Text"],
Cell[3218365, 84186, 768, 20, 120, "Input"],
Cell[3219136, 84208, 177, 2, 120, "Text"],
Cell[3219316, 84212, 373, 10, 120, "Input"],
Cell[3219692, 84224, 484, 14, 120, "Input"],
Cell[3220179, 84240, 247, 5, 120, "Text"],
Cell[3220429, 84247, 245, 6, 120, "Input"],
Cell[3220677, 84255, 269, 8, 120, "Input"],
Cell[3220949, 84265, 354, 10, 120, "Input"],
Cell[3221306, 84277, 348, 6, 196, "Text"],
Cell[3221657, 84285, 867, 21, 120, "Input"],
Cell[3222527, 84308, 241, 5, 120, "Input"],
Cell[3222771, 84315, 240, 5, 120, "Input"],
Cell[3223014, 84322, 346, 10, 120, "Input"],
Cell[3223363, 84334, 739, 14, 196, "Text"],
Cell[3224105, 84350, 365, 10, 120, "Input"],
Cell[3224473, 84362, 99, 1, 120, "Text"],
Cell[3224575, 84365, 160, 2, 120, "Text"],
Cell[3224738, 84369, 123, 2, 120, "Input"],
Cell[3224864, 84373, 230, 5, 120, "Input"],
Cell[3225097, 84380, 162, 2, 120, "Text"],
Cell[3225262, 84384, 338, 6, 120, "Input"],
Cell[3225603, 84392, 182, 2, 120, "Text"],
Cell[3225788, 84396, 202, 5, 120, "Input"],
Cell[3225993, 84403, 199, 2, 120, "Text"],
Cell[3226195, 84407, 186, 4, 120, "Input"],
Cell[3226384, 84413, 279, 6, 120, "Text"],
Cell[3226666, 84421, 689, 19, 120, "Input"],
Cell[3227358, 84442, 128, 1, 120, "Text"],
Cell[3227489, 84445, 239, 6, 120, "Input"],
Cell[3227731, 84453, 493, 14, 120, "Input"],
Cell[3228227, 84469, 547, 16, 120, "Input"],
Cell[3228777, 84487, 487, 12, 120, "Input"],
Cell[3229267, 84501, 138, 1, 120, "Text"],
Cell[3229408, 84504, 155, 3, 120, "Input"],
Cell[3229566, 84509, 263, 7, 120, "Input"],
Cell[3229832, 84518, 136, 2, 120, "Text"],
Cell[3229971, 84522, 414, 11, 120, "Input"],
Cell[3230388, 84535, 106, 1, 120, "Text"],
Cell[3230497, 84538, 383, 11, 120, "Input"],
Cell[3230883, 84551, 200, 4, 120, "Text"],
Cell[3231086, 84557, 371, 10, 120, "Input"],
Cell[3231460, 84569, 481, 10, 120, "Input"],
Cell[3231944, 84581, 213, 5, 120, "Input"],
Cell[3232160, 84588, 645, 14, 120, "Input"],
Cell[3232808, 84604, 476, 10, 120, "Input"],
Cell[3233287, 84616, 692, 14, 120, "Input"],
Cell[3233982, 84632, 541, 11, 120, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[3234560, 84648, 443, 7, 60, "Subsubsection"],
Cell[3235006, 84657, 409, 7, 2147483647, "Text"],
Cell[3235418, 84666, 482, 12, 2147483647, "Input"],
Cell[3235903, 84680, 163, 2, 2147483647, "Text"],
Cell[3236069, 84684, 1087, 33, 2147483647, "Input"],
Cell[3237159, 84719, 605, 16, 2147483647, "Input"],
Cell[3237767, 84737, 929, 21, 2147483647, "Input"],
Cell[3238699, 84760, 397, 7, 2147483647, "Text"],
Cell[3239099, 84769, 346, 10, 2147483647, "Input"],
Cell[3239448, 84781, 105, 1, 2147483647, "Text"],
Cell[3239556, 84784, 655, 15, 2147483647, "Input"],
Cell[3240214, 84801, 114, 1, 2147483647, "Text"],
Cell[3240331, 84804, 8269, 226, 2147483647, "Input"],
Cell[3248603, 85032, 815, 19, 2147483647, "Input"],
Cell[3249421, 85053, 818, 20, 2147483647, "Input"],
Cell[3250242, 85075, 255, 6, 2147483647, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[3250534, 85086, 544, 8, 60, "Subsubsection"],
Cell[3251081, 85096, 525, 13, 2147483647, "Input"],
Cell[3251609, 85111, 112, 1, 2147483647, "Text"],
Cell[3251724, 85114, 1096, 34, 2147483647, "Input"],
Cell[3252823, 85150, 590, 16, 2147483647, "Input"],
Cell[3253416, 85168, 1007, 23, 2147483647, "Input"],
Cell[3254426, 85193, 260, 5, 2147483647, "Text"],
Cell[3254689, 85200, 679, 15, 2147483647, "Input"],
Cell[3255371, 85217, 8288, 226, 2147483647, "Input"],
Cell[3263662, 85445, 1063, 24, 2147483647, "Input"],
Cell[3264728, 85471, 1056, 23, 2147483647, "Input"],
Cell[3265787, 85496, 247, 6, 2147483647, "Input"],
Cell[3266037, 85504, 304, 6, 2147483647, "Text"],
Cell[3266344, 85512, 205, 4, 2147483647, "Text"],
Cell[3266552, 85518, 373, 11, 2147483647, "Input"],
Cell[3266928, 85531, 455, 11, 2147483647, "Input"],
Cell[3267386, 85544, 497, 13, 2147483647, "Input"],
Cell[3267886, 85559, 288, 5, 2147483647, "Text"],
Cell[3268177, 85566, 546, 14, 2147483647, "Input"],
Cell[3268726, 85582, 114, 2, 2147483647, "Text"],
Cell[3268843, 85586, 459, 13, 2147483647, "Input"],
Cell[3269305, 85601, 508, 14, 2147483647, "Input"],
Cell[3269816, 85617, 646, 15, 2147483647, "Input"],
Cell[3270465, 85634, 698, 16, 2147483647, "Input"],
Cell[3271166, 85652, 798, 18, 2147483647, "Input"],
Cell[3271967, 85672, 370, 6, 2147483647, "Text"],
Cell[3272340, 85680, 1362, 29, 2147483647, "Input"],
Cell[3273705, 85711, 416, 12, 2147483647, "Input"],
Cell[3274124, 85725, 467, 13, 2147483647, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[3274628, 85743, 213, 2, 60, "Subsubsection"],
Cell[3274844, 85747, 515, 11, 196, "Text"],
Cell[3275362, 85760, 252, 6, 120, "Input"],
Cell[3275617, 85768, 235, 6, 120, "Input"],
Cell[3275855, 85776, 830, 22, 124, "Text"],
Cell[3276688, 85800, 1637, 42, 214, "Input"],
Cell[3278328, 85844, 222, 5, 120, "Text"],
Cell[3278553, 85851, 1613, 41, 132, "Input"],
Cell[3280169, 85894, 132, 1, 120, "Text"],
Cell[3280304, 85897, 634, 17, 120, "Input"],
Cell[3280941, 85916, 790, 19, 120, "Input"],
Cell[3281734, 85937, 149, 2, 120, "Text"],
Cell[3281886, 85941, 682, 13, 120, "Input"],
Cell[3282571, 85956, 734, 14, 120, "Input"],
Cell[3283308, 85972, 94, 1, 120, "Text"],
Cell[3283405, 85975, 1601, 40, 190, "Input"],
Cell[3285009, 86017, 774, 14, 120, "Input"],
Cell[3285786, 86033, 145, 4, 196, "Text"],
Cell[3285934, 86039, 407, 11, 120, "Input"],
Cell[3286344, 86052, 93, 1, 120, "Text"],
Cell[3286440, 86055, 490, 15, 120, "Input",
 Evaluatable->False],
Cell[3286933, 86072, 494, 8, 196, "Text"],
Cell[3287430, 86082, 329, 9, 120, "Input"],
Cell[3287762, 86093, 413, 12, 120, "Input"],
Cell[3288178, 86107, 714, 12, 196, "Text"],
Cell[3288895, 86121, 681, 19, 198, "Input"],
Cell[3289579, 86142, 175, 2, 120, "Text"],
Cell[3289757, 86146, 353, 10, 120, "Input"],
Cell[3290113, 86158, 113, 1, 120, "Text"],
Cell[3290229, 86161, 698, 18, 120, "Input"],
Cell[3290930, 86181, 558, 9, 272, "Text"],
Cell[3291491, 86192, 282, 7, 120, "Input"],
Cell[3291776, 86201, 105, 1, 120, "Text"],
Cell[3291884, 86204, 303, 9, 120, "Input"],
Cell[3292190, 86215, 239, 5, 120, "Text"],
Cell[3292432, 86222, 613, 16, 120, "Input"],
Cell[3293048, 86240, 267, 5, 196, "Text"],
Cell[3293318, 86247, 472, 14, 198, "Input"],
Cell[3293793, 86263, 141, 2, 120, "Text"],
Cell[3293937, 86267, 616, 17, 120, "Input"],
Cell[3294556, 86286, 293, 9, 120, "Input"],
Cell[3294852, 86297, 194, 4, 120, "Text"],
Cell[3295049, 86303, 683, 20, 198, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[3295769, 86328, 380, 6, 60, "Subsubsection"],
Cell[3296152, 86336, 161, 3, 120, "Text"],
Cell[3296316, 86341, 401, 14, 200, "Input",
 Evaluatable->False],
Cell[3296720, 86357, 109, 1, 120, "Text"],
Cell[3296832, 86360, 454, 15, 220, "Input",
 Evaluatable->False],
Cell[3297289, 86377, 780, 21, 200, "Text"],
Cell[3298072, 86400, 354, 9, 120, "Input"],
Cell[3298429, 86411, 608, 16, 120, "Input"],
Cell[3299040, 86429, 999, 27, 198, "Input"],
Cell[3300042, 86458, 135, 1, 120, "Text"],
Cell[3300180, 86461, 330, 6, 120, "Input"],
Cell[3300513, 86469, 125, 1, 120, "Text"],
Cell[3300641, 86472, 478, 13, 124, "Text"],
Cell[3301122, 86487, 202, 5, 120, "Text"],
Cell[3301327, 86494, 1067, 25, 198, "Input"],
Cell[3302397, 86521, 283, 6, 196, "Text"],
Cell[3302683, 86529, 379, 11, 124, "Text"],
Cell[3303065, 86542, 134, 1, 120, "Text"],
Cell[3303202, 86545, 455, 17, 250, "Input",
 Evaluatable->False],
Cell[3303660, 86564, 1458, 31, 350, "Input"],
Cell[3305121, 86597, 236, 5, 120, "Text"],
Cell[3305360, 86604, 745, 19, 198, "Input"],
Cell[3306108, 86625, 250, 5, 120, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[3306395, 86635, 400, 6, 60, "Subsubsection"],
Cell[3306798, 86643, 162, 3, 120, "Text"],
Cell[3306963, 86648, 508, 13, 120, "Input"],
Cell[3307474, 86663, 554, 16, 120, "Input"],
Cell[3308031, 86681, 200, 3, 120, "Text"],
Cell[3308234, 86686, 114, 1, 120, "Text"],
Cell[3308351, 86689, 614, 19, 120, "Input"],
Cell[3308968, 86710, 91, 1, 120, "Text"],
Cell[3309062, 86713, 308, 9, 120, "Input"],
Cell[3309373, 86724, 174, 4, 120, "Input"],
Cell[3309550, 86730, 123, 1, 120, "Text"],
Cell[3309676, 86733, 595, 16, 168, "Input"],
Cell[3310274, 86751, 375, 10, 120, "Input"],
Cell[3310652, 86763, 93, 1, 120, "Text"],
Cell[3310748, 86766, 1274, 40, 578, "Input"],
Cell[3312025, 86808, 113, 1, 120, "Text"],
Cell[3312141, 86811, 718, 18, 120, "Input"],
Cell[3312862, 86831, 484, 14, 120, "Input"],
Cell[3313349, 86847, 160, 3, 120, "Text"],
Cell[3313512, 86852, 815, 20, 198, "Input"],
Cell[3314330, 86874, 117, 1, 120, "Text"],
Cell[3314450, 86877, 302, 7, 120, "Input"],
Cell[3314755, 86886, 16, 0, 120, "Text"],
Cell[3314774, 86888, 138, 1, 120, "Text"],
Cell[3314915, 86891, 394, 12, 120, "Input"],
Cell[3315312, 86905, 218, 5, 120, "Text"],
Cell[3315533, 86912, 133, 4, 120, "Input"],
Cell[3315669, 86918, 221, 6, 120, "Input"],
Cell[3315893, 86926, 158, 3, 120, "Text"],
Cell[3316054, 86931, 367, 9, 120, "Input"],
Cell[3316424, 86942, 309, 6, 196, "Text"],
Cell[3316736, 86950, 381, 12, 120, "Text"],
Cell[3317120, 86964, 107, 1, 120, "Text"],
Cell[3317230, 86967, 366, 10, 120, "Input"],
Cell[3317599, 86979, 814, 22, 198, "Input"],
Cell[3318416, 87003, 351, 7, 272, "Text"],
Cell[3318770, 87012, 360, 10, 120, "Input"],
Cell[3319133, 87024, 107, 1, 120, "Text"],
Cell[3319243, 87027, 298, 8, 120, "Input"],
Cell[3319544, 87037, 197, 5, 120, "Input"],
Cell[3319744, 87044, 875, 27, 198, "Input"],
Cell[3320622, 87073, 611, 19, 274, "Input"],
Cell[3321236, 87094, 1056, 25, 198, "Input"],
Cell[3322295, 87121, 169, 2, 120, "Text"],
Cell[3322467, 87125, 694, 18, 198, "Input"],
Cell[3323164, 87145, 148, 2, 120, "Text"],
Cell[3323315, 87149, 488, 13, 120, "Input"],
Cell[3323806, 87164, 108, 1, 120, "Text"],
Cell[3323917, 87167, 294, 8, 120, "Input"],
Cell[3324214, 87177, 197, 5, 120, "Input"],
Cell[3324414, 87184, 852, 23, 198, "Input"],
Cell[3325269, 87209, 874, 26, 350, "Input"],
Cell[3326146, 87237, 190, 2, 120, "Text"],
Cell[3326339, 87241, 1036, 29, 198, "Input"],
Cell[3327378, 87272, 111, 1, 120, "Text"],
Cell[3327492, 87275, 1303, 31, 222, "Input"],
Cell[3328798, 87308, 171, 2, 120, "Text"],
Cell[3328972, 87312, 744, 19, 198, "Input"]
}, Closed]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[3329777, 87338, 582, 7, 140, "Section"],
Cell[3330362, 87347, 1619, 25, 288, "Text"],
Cell[CellGroupData[{
Cell[3332006, 87376, 370, 6, 95, "Subsection"],
Cell[3332379, 87384, 880, 25, 98, "Text"],
Cell[3333262, 87411, 620, 21, 65, "Text"],
Cell[3333885, 87434, 2059, 52, 250, "Text"],
Cell[3335947, 87488, 439, 9, 60, "Input"],
Cell[3336389, 87499, 276, 7, 60, "Text"],
Cell[3336668, 87508, 1175, 23, 99, "Input",
 InitializationCell->True],
Cell[3337846, 87533, 323, 8, 60, "Input"],
Cell[3338172, 87543, 214, 5, 60, "Input"],
Cell[3338389, 87550, 133, 2, 60, "Input"],
Cell[3338525, 87554, 425, 10, 98, "Text"],
Cell[3338953, 87566, 328, 8, 120, "Text"],
Cell[3339284, 87576, 850, 16, 120, "Input",
 Evaluatable->False],
Cell[3340137, 87594, 189, 3, 120, "Input"],
Cell[3340329, 87599, 448, 9, 120, "Input"],
Cell[3340780, 87610, 257, 7, 120, "Text"],
Cell[3341040, 87619, 259, 6, 120, "Input"],
Cell[3341302, 87627, 345, 8, 120, "Input"],
Cell[3341650, 87637, 541, 15, 120, "Input"],
Cell[3342194, 87654, 252, 6, 120, "Input"],
Cell[3342449, 87662, 465, 8, 120, "Text"],
Cell[3342917, 87672, 115, 2, 120, "Input"],
Cell[3343035, 87676, 1328, 40, 350, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[3344388, 87720, 1295, 24, 120, "Input",
 InitializationCell->True],
Cell[3345686, 87746, 876, 16, 168, "Output"]
}, Closed]],
Cell[3346577, 87765, 322, 7, 88, "Input"],
Cell[3346902, 87774, 241, 5, 120, "Text"],
Cell[3347146, 87781, 269, 7, 120, "Input"],
Cell[3347418, 87790, 394, 9, 120, "Input"],
Cell[3347815, 87801, 424, 9, 120, "Input"],
Cell[3348242, 87812, 325, 7, 120, "Text"],
Cell[3348570, 87821, 221, 4, 120, "Input"],
Cell[3348794, 87827, 215, 4, 120, "Input"],
Cell[3349012, 87833, 122, 2, 120, "Input"],
Cell[3349137, 87837, 272, 6, 120, "Input"],
Cell[3349412, 87845, 245, 5, 120, "Input"],
Cell[3349660, 87852, 297, 6, 120, "Input"],
Cell[3349960, 87860, 827, 24, 198, "Input"],
Cell[3350790, 87886, 444, 12, 120, "Input"],
Cell[3351237, 87900, 359, 8, 120, "Input"],
Cell[3351599, 87910, 757, 15, 424, "Text"],
Cell[3352359, 87927, 776, 18, 196, "Text"],
Cell[3353138, 87947, 768, 18, 120, "Input",
 Evaluatable->False],
Cell[3353909, 87967, 238, 4, 120, "Input"],
Cell[3354150, 87973, 229, 5, 120, "Input"],
Cell[3354382, 87980, 626, 18, 120, "Input"],
Cell[3355011, 88000, 782, 18, 196, "Text"],
Cell[3355796, 88020, 864, 24, 120, "Input"],
Cell[3356663, 88046, 186, 3, 120, "Input"],
Cell[3356852, 88051, 1421, 31, 198, "Input",
 Evaluatable->False],
Cell[3358276, 88084, 556, 14, 120, "Input"],
Cell[3358835, 88100, 608, 14, 198, "Input"],
Cell[3359446, 88116, 606, 13, 120, "Input"],
Cell[3360055, 88131, 153, 3, 120, "Input"],
Cell[3360211, 88136, 295, 5, 196, "Text"],
Cell[3360509, 88143, 355, 10, 120, "Input"],
Cell[3360867, 88155, 405, 11, 120, "Input"],
Cell[3361275, 88168, 286, 7, 196, "Text",
 CellTags->"fund1"],
Cell[3361564, 88177, 166, 3, 120, "Input"],
Cell[3361733, 88182, 262, 5, 120, "Text"],
Cell[3361998, 88189, 400, 9, 120, "Input"],
Cell[3362401, 88200, 140, 3, 120, "Input"],
Cell[3362544, 88205, 421, 11, 120, "Input"],
Cell[3362968, 88218, 394, 11, 120, "Input"],
Cell[3363365, 88231, 1590, 41, 350, "Input"],
Cell[3364958, 88274, 1285, 32, 274, "Input"],
Cell[3366246, 88308, 1159, 28, 198, "Input"],
Cell[3367408, 88338, 1460, 25, 196, "Text"],
Cell[3368871, 88365, 678, 22, 120, "Input",
 Evaluatable->False],
Cell[3369552, 88389, 2030, 47, 208, "Text"],
Cell[3371585, 88438, 2990, 57, 728, "Text"],
Cell[3374578, 88497, 1821, 32, 196, "Text"],
Cell[3376402, 88531, 1949, 40, 274, "Input",
 Evaluatable->False],
Cell[3378354, 88573, 220, 4, 120, "Input"],
Cell[3378577, 88579, 494, 8, 272, "Text"],
Cell[3379074, 88589, 1751, 39, 274, "Input",
 Evaluatable->False],
Cell[3380828, 88630, 266, 4, 120, "Input"],
Cell[3381097, 88636, 226, 3, 120, "Text"],
Cell[3381326, 88641, 146, 3, 120, "Input"],
Cell[3381475, 88646, 636, 15, 120, "Input"],
Cell[3382114, 88663, 323, 8, 120, "Input"],
Cell[3382440, 88673, 117, 2, 120, "Text"],
Cell[3382560, 88677, 169, 4, 120, "Input"],
Cell[3382732, 88683, 147, 4, 120, "Input"],
Cell[3382882, 88689, 1448, 36, 198, "Input"],
Cell[3384333, 88727, 463, 10, 98, "Text"],
Cell[3384799, 88739, 393, 10, 60, "Input"],
Cell[3385195, 88751, 317, 10, 62, "Text"],
Cell[3385515, 88763, 115, 2, 60, "Input"],
Cell[3385633, 88767, 212, 5, 60, "Input"],
Cell[3385848, 88774, 319, 8, 62, "Text"],
Cell[3386170, 88784, 390, 10, 60, "Input"],
Cell[3386563, 88796, 625, 16, 60, "Input"],
Cell[3387191, 88814, 500, 11, 60, "Text"],
Cell[3387694, 88827, 707, 17, 60, "Input"],
Cell[3388404, 88846, 704, 17, 60, "Input"],
Cell[3389111, 88865, 357, 8, 60, "Input"],
Cell[3389471, 88875, 192, 2, 60, "Text"],
Cell[3389666, 88879, 148, 3, 60, "Input"],
Cell[3389817, 88884, 1042, 27, 96, "Input"],
Cell[3390862, 88913, 101, 1, 60, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[3391000, 88919, 493, 6, 75, "Subsection"],
Cell[3391496, 88927, 543, 9, 60, "Text"],
Cell[3392042, 88938, 114, 2, 60, "Input"],
Cell[3392159, 88942, 196, 3, 60, "Text"],
Cell[3392358, 88947, 317, 7, 60, "Input"],
Cell[3392678, 88956, 478, 8, 60, "Text"],
Cell[3393159, 88966, 371, 7, 60, "Text"],
Cell[CellGroupData[{
Cell[3393555, 88977, 630, 14, 60, "Input"],
Cell[3394188, 88993, 721, 23, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[3394946, 89021, 447, 6, 60, "Subsubsection"],
Cell[3395396, 89029, 337, 6, 2147483647, "Text"],
Cell[3395736, 89037, 114, 2, 2147483647, "Input"],
Cell[3395853, 89041, 605, 12, 2147483647, "Input"],
Cell[3396461, 89055, 643, 12, 2147483647, "Input"],
Cell[3397107, 89069, 510, 11, 2147483647, "Input"],
Cell[3397620, 89082, 550, 13, 2147483647, "Input"],
Cell[3398173, 89097, 354, 5, 2147483647, "Text",
 InitializationCell->True],
Cell[3398530, 89104, 269, 6, 2147483647, "Input"],
Cell[3398802, 89112, 204, 5, 2147483647, "Input"],
Cell[3399009, 89119, 538, 12, 2147483647, "Text"],
Cell[3399550, 89133, 232, 6, 2147483647, "Input"],
Cell[3399785, 89141, 131, 2, 2147483647, "Input"],
Cell[3399919, 89145, 260, 3, 2147483647, "Text"],
Cell[3400182, 89150, 628, 19, 2147483647, "Input"],
Cell[3400813, 89171, 139, 1, 2147483647, "Text"],
Cell[3400955, 89174, 132, 4, 2147483647, "Input"],
Cell[3401090, 89180, 296, 5, 2147483647, "Text"],
Cell[3401389, 89187, 454, 12, 2147483647, "Input"],
Cell[3401846, 89201, 105, 1, 2147483647, "Text"],
Cell[3401954, 89204, 616, 15, 2147483647, "Input"],
Cell[3402573, 89221, 203, 4, 2147483647, "Input"],
Cell[3402779, 89227, 160, 3, 2147483647, "Input"],
Cell[3402942, 89232, 394, 9, 2147483647, "Text"],
Cell[3403339, 89243, 385, 11, 2147483647, "Input"],
Cell[3403727, 89256, 230, 6, 2147483647, "Text",
 CellTags->"dtt(1)"],
Cell[3403960, 89264, 1085, 27, 2147483647, "Input",
 Evaluatable->False],
Cell[3405048, 89293, 164, 5, 2147483647, "Input"],
Cell[3405215, 89300, 213, 4, 2147483647, "Text"],
Cell[3405431, 89306, 549, 14, 2147483647, "Input"],
Cell[3405983, 89322, 161, 2, 2147483647, "Text"],
Cell[3406147, 89326, 905, 27, 2147483647, "Input"],
Cell[3407055, 89355, 133, 2, 2147483647, "Input"],
Cell[3407191, 89359, 206, 2, 2147483647, "Text"],
Cell[3407400, 89363, 976, 28, 2147483647, "Input"],
Cell[3408379, 89393, 131, 2, 2147483647, "Input"],
Cell[3408513, 89397, 416, 9, 2147483647, "Text"],
Cell[3408932, 89408, 2391, 68, 2147483647, "Input"],
Cell[3411326, 89478, 164, 3, 2147483647, "Text"],
Cell[3411493, 89483, 393, 9, 2147483647, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[3411923, 89497, 451, 6, 60, "Subsubsection"],
Cell[3412377, 89505, 1132, 27, 2147483647, "Text"],
Cell[3413512, 89534, 507, 14, 2147483647, "Input",
 Evaluatable->False],
Cell[3414022, 89550, 479, 10, 2147483647, "Text"],
Cell[3414504, 89562, 534, 14, 2147483647, "Input",
 Evaluatable->False],
Cell[3415041, 89578, 766, 22, 2147483647, "Text"],
Cell[3415810, 89602, 1078, 35, 2147483647, "Input",
 Evaluatable->False],
Cell[3416891, 89639, 118, 1, 2147483647, "Text"],
Cell[3417012, 89642, 463, 10, 2147483647, "Input"],
Cell[3417478, 89654, 123, 1, 2147483647, "Text"],
Cell[3417604, 89657, 584, 16, 2147483647, "Input",
 Evaluatable->False],
Cell[3418191, 89675, 598, 15, 2147483647, "Input",
 Evaluatable->False],
Cell[3418792, 89692, 694, 20, 2147483647, "Text"],
Cell[3419489, 89714, 495, 12, 2147483647, "Input",
 Evaluatable->False],
Cell[3419987, 89728, 379, 9, 2147483647, "Text"],
Cell[3420369, 89739, 365, 9, 2147483647, "Input",
 Evaluatable->False],
Cell[3420737, 89750, 132, 3, 2147483647, "Input"],
Cell[3420872, 89755, 359, 8, 2147483647, "Input"],
Cell[3421234, 89765, 99, 1, 2147483647, "Text"],
Cell[3421336, 89768, 905, 22, 2147483647, "Input"],
Cell[3422244, 89792, 790, 19, 2147483647, "Text"],
Cell[3423037, 89813, 878, 22, 2147483647, "Input",
 Evaluatable->False],
Cell[3423918, 89837, 346, 7, 2147483647, "Text"],
Cell[3424267, 89846, 1288, 36, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[3425580, 89886, 301, 5, 2147483647, "Input"],
Cell[3425884, 89893, 338, 6, 2147483647, "Print",
 CellTags->"Info3625907027-2112774"]
}, Closed]],
Cell[3426237, 89902, 1352, 36, 2147483647, "Input",
 Evaluatable->False],
Cell[3427592, 89940, 339, 5, 2147483647, "Input"],
Cell[3427934, 89947, 332, 8, 2147483647, "Input"],
Cell[3428269, 89957, 1068, 31, 2147483647, "Input"],
Cell[3429340, 89990, 229, 6, 2147483647, "Text"],
Cell[3429572, 89998, 1044, 23, 2147483647, "Input"],
Cell[3430619, 90023, 970, 22, 2147483647, "Input"],
Cell[3431592, 90047, 590, 11, 2147483647, "Input"],
Cell[3432185, 90060, 113, 4, 2147483647, "Text"],
Cell[3432301, 90066, 285, 7, 2147483647, "Text"],
Cell[3432589, 90075, 1320, 37, 2147483647, "Input",
 Evaluatable->False],
Cell[3433912, 90114, 191, 3, 2147483647, "Input"],
Cell[3434106, 90119, 218, 6, 2147483647, "Text"],
Cell[3434327, 90127, 140, 1, 2147483647, "Text"],
Cell[3434470, 90130, 968, 24, 2147483647, "Input"],
Cell[3435441, 90156, 496, 14, 2147483647, "Input"],
Cell[3435940, 90172, 190, 2, 2147483647, "Text"],
Cell[3436133, 90176, 1017, 25, 2147483647, "Input"],
Cell[3437153, 90203, 105, 1, 2147483647, "Text"],
Cell[3437261, 90206, 265, 6, 2147483647, "Input"],
Cell[3437529, 90214, 127, 1, 2147483647, "Text"],
Cell[3437659, 90217, 1084, 23, 2147483647, "Input"],
Cell[3438746, 90242, 219, 5, 2147483647, "Text"],
Cell[3438968, 90249, 1462, 32, 2147483647, "Input"],
Cell[3440433, 90283, 467, 8, 2147483647, "Input"],
Cell[3440903, 90293, 134, 1, 2147483647, "Text"],
Cell[3441040, 90296, 1086, 24, 2147483647, "Input"],
Cell[3442129, 90322, 475, 11, 2147483647, "Text"],
Cell[3442607, 90335, 366, 9, 2147483647, "Input"],
Cell[3442976, 90346, 146, 3, 2147483647, "Input"],
Cell[3443125, 90351, 164, 5, 2147483647, "Input"],
Cell[3443292, 90358, 180, 4, 2147483647, "Text"],
Cell[3443475, 90364, 717, 21, 2147483647, "Input"],
Cell[3444195, 90387, 598, 10, 2147483647, "Text"],
Cell[3444796, 90399, 333, 8, 2147483647, "Input"],
Cell[3445132, 90409, 332, 9, 2147483647, "Input"],
Cell[3445467, 90420, 386, 10, 2147483647, "Input"],
Cell[3445856, 90432, 929, 19, 2147483647, "Text"],
Cell[3446788, 90453, 366, 9, 2147483647, "Input"],
Cell[3447157, 90464, 1128, 29, 2147483647, "Input",
 InitializationCell->True],
Cell[3448288, 90495, 439, 11, 2147483647, "Input"],
Cell[3448730, 90508, 152, 2, 2147483647, "Text"],
Cell[3448885, 90512, 1185, 31, 2147483647, "Input"],
Cell[3450073, 90545, 195, 4, 2147483647, "Text"],
Cell[3450271, 90551, 1717, 45, 2147483647, "Input"],
Cell[3451991, 90598, 131, 1, 2147483647, "Text"],
Cell[3452125, 90601, 996, 28, 2147483647, "Input",
 InitializationCell->True],
Cell[3453124, 90631, 348, 9, 2147483647, "Input"],
Cell[3453475, 90642, 106, 1, 2147483647, "Text"],
Cell[3453584, 90645, 1380, 33, 2147483647, "Input"],
Cell[3454967, 90680, 343, 9, 2147483647, "Text"],
Cell[3455313, 90691, 1430, 40, 2147483647, "Input"],
Cell[3456746, 90733, 162, 2, 2147483647, "Text"],
Cell[3456911, 90737, 473, 13, 2147483647, "Input",
 Evaluatable->False],
Cell[3457387, 90752, 197, 4, 2147483647, "Text"],
Cell[3457587, 90758, 567, 18, 2147483647, "Input",
 Evaluatable->False],
Cell[3458157, 90778, 2407, 62, 2147483647, "Input"],
Cell[3460567, 90842, 93, 1, 2147483647, "Text"],
Cell[3460663, 90845, 665, 21, 2147483647, "Input",
 Evaluatable->False],
Cell[3461331, 90868, 417, 7, 2147483647, "Text"],
Cell[3461751, 90877, 1352, 39, 2147483647, "Input",
 Evaluatable->False],
Cell[3463106, 90918, 327, 5, 2147483647, "Input"],
Cell[3463436, 90925, 1341, 34, 2147483647, "Input",
 Evaluatable->False],
Cell[3464780, 90961, 328, 5, 2147483647, "Input"],
Cell[3465111, 90968, 343, 9, 2147483647, "Input"],
Cell[3465457, 90979, 1209, 32, 2147483647, "Input"],
Cell[3466669, 91013, 118, 2, 2147483647, "Input"],
Cell[3466790, 91017, 703, 20, 2147483647, "Text"],
Cell[3467496, 91039, 1379, 32, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[3468900, 91075, 305, 5, 2147483647, "Input",
 InitializationCell->True],
Cell[3469208, 91082, 315, 5, 140, "Print",
 CellTags->"Info3629190374-5022241"]
}, Closed]],
Cell[3469538, 91090, 938, 27, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[3470501, 91121, 363, 6, 2147483647, "Input",
 InitializationCell->True],
Cell[3470867, 91129, 345, 6, 140, "Print",
 CellTags->"Info3629190374-5022241"]
}, Closed]],
Cell[3471227, 91138, 590, 16, 2147483647, "Input"],
Cell[3471820, 91156, 893, 23, 2147483647, "Input"],
Cell[3472716, 91181, 127, 2, 2147483647, "Input"],
Cell[3472846, 91185, 183, 4, 2147483647, "Text"],
Cell[3473032, 91191, 910, 23, 2147483647, "Input"],
Cell[3473945, 91216, 628, 14, 2147483647, "Text"],
Cell[3474576, 91232, 363, 10, 2147483647, "Input",
 Evaluatable->False],
Cell[3474942, 91244, 320, 4, 2147483647, "Text"],
Cell[3475265, 91250, 1207, 31, 2147483647, "Input"],
Cell[3476475, 91283, 100, 1, 2147483647, "Text"],
Cell[3476578, 91286, 224, 6, 2147483647, "Text"],
Cell[3476805, 91294, 956, 24, 2147483647, "Input"],
Cell[3477764, 91320, 251, 7, 2147483647, "Text"],
Cell[3478018, 91329, 605, 13, 2147483647, "Input"],
Cell[3478626, 91344, 714, 15, 2147483647, "Input"],
Cell[3479343, 91361, 127, 2, 2147483647, "Input"],
Cell[3479473, 91365, 254, 7, 2147483647, "Text"],
Cell[3479730, 91374, 153, 3, 2147483647, "Text"],
Cell[3479886, 91379, 248, 6, 2147483647, "Input"],
Cell[3480137, 91387, 129, 1, 2147483647, "Text"],
Cell[3480269, 91390, 861, 20, 2147483647, "Input"],
Cell[3481133, 91412, 98, 1, 2147483647, "Text"],
Cell[3481234, 91415, 396, 12, 2147483647, "Input",
 Evaluatable->False],
Cell[3481633, 91429, 193, 3, 2147483647, "Input"],
Cell[3481829, 91434, 382, 11, 2147483647, "Input"],
Cell[3482214, 91447, 123, 1, 2147483647, "Text"],
Cell[3482340, 91450, 1038, 26, 2147483647, "Input"],
Cell[3483381, 91478, 1077, 26, 2147483647, "Input"],
Cell[3484461, 91506, 885, 15, 2147483647, "Text"],
Cell[3485349, 91523, 2870, 64, 2147483647, "Input",
 Evaluatable->False],
Cell[3488222, 91589, 261, 4, 2147483647, "Input"],
Cell[3488486, 91595, 373, 8, 2147483647, "Text"],
Cell[3488862, 91605, 110, 3, 2147483647, "Input"],
Cell[3488975, 91610, 242, 6, 2147483647, "Input"],
Cell[3489220, 91618, 415, 11, 2147483647, "Input"],
Cell[3489638, 91631, 123, 2, 2147483647, "Input"],
Cell[3489764, 91635, 342, 9, 2147483647, "Text"],
Cell[3490109, 91646, 489, 10, 2147483647, "Input"],
Cell[3490601, 91658, 225, 5, 2147483647, "Text"],
Cell[3490829, 91665, 336, 9, 2147483647, "Input"],
Cell[3491168, 91676, 244, 6, 2147483647, "Input"],
Cell[3491415, 91684, 465, 9, 2147483647, "Text"],
Cell[3491883, 91695, 311, 10, 2147483647, "Input"],
Cell[3492197, 91707, 483, 11, 2147483647, "Text"],
Cell[3492683, 91720, 153, 2, 2147483647, "Text"],
Cell[3492839, 91724, 877, 22, 2147483647, "Input"],
Cell[3493719, 91748, 261, 5, 2147483647, "Text"],
Cell[3493983, 91755, 969, 20, 2147483647, "Input"],
Cell[3494955, 91777, 198, 5, 2147483647, "Input"],
Cell[3495156, 91784, 743, 13, 2147483647, "Input"],
Cell[3495902, 91799, 597, 13, 2147483647, "Input"],
Cell[3496502, 91814, 189, 2, 2147483647, "Text"],
Cell[3496694, 91818, 458, 10, 2147483647, "Input"],
Cell[3497155, 91830, 143, 3, 2147483647, "Input"],
Cell[3497301, 91835, 139, 2, 2147483647, "Input"],
Cell[3497443, 91839, 104, 1, 2147483647, "Text"],
Cell[3497550, 91842, 227, 4, 2147483647, "Input"],
Cell[3497780, 91848, 489, 10, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[3498294, 91862, 341, 10, 2147483647, "Input"],
Cell[3498638, 91874, 12793, 412, 2147483647, "Output"],
Cell[3511434, 92288, 12771, 412, 2147483647, "Output"],
Cell[3524208, 92702, 12747, 412, 2147483647, "Output"],
Cell[3536958, 93116, 16882, 546, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[3553877, 93667, 932, 22, 2147483647, "Input"],
Cell[3554812, 93691, 248589, 6887, 2147483647, "Output"],
Cell[3803404, 100580, 248565, 6887, 2147483647, "Output"]
}, Closed]],
Cell[4051984, 107470, 126, 1, 2147483647, "Text"],
Cell[4052113, 107473, 431, 12, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[4052569, 107489, 660, 12, 2147483647, "Input"],
Cell[4053232, 107503, 118398, 3230, 2147483647, "Output"],
Cell[4171633, 110735, 118371, 3229, 2147483647, "Output"]
}, Closed]],
Cell[4290019, 113967, 133, 2, 2147483647, "Input"],
Cell[4290155, 113971, 616, 13, 2147483647, "Input"],
Cell[4290774, 113986, 171, 4, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[4290970, 113994, 164, 3, 2147483647, "Input"],
Cell[4291137, 113999, 263, 6, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4291437, 114010, 243, 6, 2147483647, "Input"],
Cell[4291683, 114018, 291, 7, 2147483647, "Output"],
Cell[4291977, 114027, 264, 6, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4292278, 114038, 607, 12, 2147483647, "Input"],
Cell[4292888, 114052, 248638, 6887, 2147483647, "Output"],
Cell[4541529, 120941, 248617, 6887, 2147483647, "Output"]
}, Closed]],
Cell[4790161, 127831, 221, 4, 2147483647, "Input"],
Cell[4790385, 127837, 308, 5, 2147483647, "Input"],
Cell[4790696, 127844, 152, 2, 2147483647, "Text"],
Cell[4790851, 127848, 1748, 50, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[4792624, 127902, 394, 7, 2147483647, "Input",
 InitializationCell->True],
Cell[4793021, 127911, 400, 6, 140, "Print",
 CellTags->"Info3629190374-5022241"]
}, Closed]],
Cell[4793436, 127920, 154, 4, 2147483647, "Input"],
Cell[4793593, 127926, 324, 8, 2147483647, "Input"],
Cell[4793920, 127936, 139, 3, 2147483647, "Input"],
Cell[4794062, 127941, 133, 2, 2147483647, "Input"],
Cell[4794198, 127945, 184, 4, 2147483647, "Text"],
Cell[4794385, 127951, 415, 9, 2147483647, "Input"],
Cell[4794803, 127962, 140, 1, 2147483647, "Text"],
Cell[4794946, 127965, 280, 7, 2147483647, "Input"],
Cell[4795229, 127974, 148, 3, 2147483647, "Input"],
Cell[4795380, 127979, 255, 7, 2147483647, "Input"],
Cell[4795638, 127988, 87, 1, 2147483647, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[4795762, 127994, 644, 8, 60, "Subsubsection"],
Cell[4796409, 128004, 482, 8, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[4796916, 128016, 534, 12, 2147483647, "Input"],
Cell[4797453, 128030, 417, 13, 2147483647, "Output"]
}, Closed]],
Cell[4797885, 128046, 212, 7, 2147483647, "Text"],
Cell[4798100, 128055, 610, 15, 2147483647, "Input",
 Evaluatable->False],
Cell[4798713, 128072, 456, 8, 2147483647, "Text"],
Cell[4799172, 128082, 622, 14, 2147483647, "Input",
 Evaluatable->False],
Cell[4799797, 128098, 180, 4, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[4800002, 128106, 474, 12, 2147483647, "Input"],
Cell[4800479, 128120, 115, 3, 2147483647, "Output"]
}, Closed]],
Cell[4800609, 128126, 404, 10, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[4801038, 128140, 241, 6, 2147483647, "Input"],
Cell[4801282, 128148, 1215, 43, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4802534, 128196, 385, 10, 2147483647, "Input"],
Cell[4802922, 128208, 1215, 43, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4804174, 128256, 131, 2, 2147483647, "Input"],
Cell[4804308, 128260, 115, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4804460, 128268, 492, 14, 2147483647, "Input"],
Cell[4804955, 128284, 117, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4805109, 128292, 240, 6, 2147483647, "Input"],
Cell[4805352, 128300, 215, 7, 2147483647, "Output"]
}, Closed]],
Cell[4805582, 128310, 435, 8, 2147483647, "Text"],
Cell[4806020, 128320, 1627, 37, 2147483647, "Input",
 InitializationCell->True],
Cell[4807650, 128359, 407, 8, 2147483647, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[4808082, 128371, 311, 7, 2147483647, "Input"],
Cell[4808396, 128380, 435, 13, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4808868, 128398, 793, 20, 2147483647, "Input"],
Cell[4809664, 128420, 72, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4809773, 128426, 350, 9, 2147483647, "Input"],
Cell[4810126, 128437, 2846, 93, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4813009, 128535, 643, 15, 2147483647, "Input"],
Cell[4813655, 128552, 1641, 50, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4815333, 128607, 259, 5, 2147483647, "Input"],
Cell[4815595, 128614, 1047, 34, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4816679, 128653, 166, 3, 2147483647, "Input"],
Cell[4816848, 128658, 4993, 150, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4821878, 128813, 131, 2, 2147483647, "Input"],
Cell[4822012, 128817, 70, 1, 2147483647, "Output"]
}, Closed]],
Cell[4822097, 128821, 303, 6, 2147483647, "Text"],
Cell[4822403, 128829, 1057, 25, 2147483647, "Input"],
Cell[4823463, 128856, 148, 2, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[4823636, 128862, 96, 1, 2147483647, "Input"],
Cell[4823735, 128865, 70, 1, 2147483647, "Output"]
}, Closed]],
Cell[4823820, 128869, 228, 5, 2147483647, "Text"],
Cell[4824051, 128876, 1862, 46, 2147483647, "Input"],
Cell[4825916, 128924, 538, 13, 2147483647, "Text"],
Cell[4826457, 128939, 1170, 29, 2147483647, "Input",
 Evaluatable->False],
Cell[4827630, 128970, 676, 15, 2147483647, "Text"],
Cell[4828309, 128987, 1131, 29, 2147483647, "Input",
 Evaluatable->False],
Cell[4829443, 129018, 1498, 39, 2147483647, "Input",
 Evaluatable->False],
Cell[4830944, 129059, 407, 7, 2147483647, "Text"],
Cell[4831354, 129068, 913, 25, 2147483647, "Input",
 Evaluatable->False],
Cell[4832270, 129095, 159, 2, 2147483647, "Text"],
Cell[4832432, 129099, 1604, 41, 2147483647, "Input",
 Evaluatable->False],
Cell[4834039, 129142, 678, 13, 2147483647, "Text"],
Cell[4834720, 129157, 1583, 42, 2147483647, "Input"],
Cell[4836306, 129201, 789, 15, 2147483647, "Text"],
Cell[4837098, 129218, 2757, 65, 2147483647, "Input",
 Evaluatable->False],
Cell[4839858, 129285, 591, 11, 2147483647, "Text"],
Cell[4840452, 129298, 2089, 62, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[4842566, 129364, 264, 4, 2147483647, "Input"],
Cell[4842833, 129370, 277, 5, 2147483647, "Print",
 CellTags->"Info3626005242-1516062"]
}, Closed]],
Cell[4843125, 129378, 712, 15, 2147483647, "Text"],
Cell[4843840, 129395, 1008, 22, 2147483647, "Text"],
Cell[4844851, 129419, 679, 19, 2147483647, "Input",
 Evaluatable->False],
Cell[4845533, 129440, 520, 9, 2147483647, "Text"],
Cell[4846056, 129451, 650, 13, 2147483647, "Text"],
Cell[4846709, 129466, 1940, 59, 2147483647, "Input",
 Evaluatable->False],
Cell[4848652, 129527, 221, 4, 240, "Input"],
Cell[4848876, 129533, 192, 6, 240, "Text"],
Cell[CellGroupData[{
Cell[4849093, 129543, 671, 14, 240, "Input"],
Cell[4849767, 129559, 72, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4849876, 129565, 379, 9, 176, "Input"],
Cell[4850258, 129576, 72, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4850367, 129582, 381, 9, 176, "Input"],
Cell[4850751, 129593, 142, 4, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4850930, 129602, 432, 10, 176, "Input"],
Cell[4851365, 129614, 72, 1, 2147483647, "Output"]
}, Closed]],
Cell[4851452, 129618, 288, 6, 200, "Text"],
Cell[CellGroupData[{
Cell[4851765, 129628, 928, 24, 548, "Input"],
Cell[4852696, 129654, 213, 7, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4852946, 129666, 770, 18, 332, "Input"],
Cell[4853719, 129686, 714, 21, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4854470, 129712, 748, 18, 176, "Input"],
Cell[4855221, 129732, 133, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4855391, 129740, 773, 18, 176, "Input"],
Cell[4856167, 129760, 235, 7, 2147483647, "Output"]
}, Closed]],
Cell[4856417, 129770, 265, 5, 200, "Text"],
Cell[4856685, 129777, 243, 6, 240, "Text"],
Cell[4856931, 129785, 1247, 30, 396, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[4858203, 129819, 295, 5, 240, "Input"],
Cell[4858501, 129826, 284, 5, 464, "Print",
 CellTags->"Info3626006462-6741741"]
}, Closed]],
Cell[4858800, 129834, 1145, 30, 396, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[4859970, 129868, 296, 5, 240, "Input"],
Cell[4860269, 129875, 306, 5, 2147483647, "Print",
 CellTags->"Info3626006478-6741741"]
}, Closed]],
Cell[4860590, 129883, 189, 4, 200, "Text"],
Cell[CellGroupData[{
Cell[4860804, 129891, 829, 19, 396, "Input"],
Cell[4861636, 129912, 893, 27, 2147483647, "Output"]
}, Closed]],
Cell[4862544, 129942, 116, 1, 200, "Text"],
Cell[CellGroupData[{
Cell[4862685, 129947, 1075, 27, 396, "Input"],
Cell[4863763, 129976, 653, 18, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4864453, 129999, 1179, 29, 484, "Input"],
Cell[4865635, 130030, 727, 20, 2147483647, "Output"]
}, Closed]],
Cell[4866377, 130053, 177, 2, 200, "Text"],
Cell[CellGroupData[{
Cell[4866579, 130059, 141, 3, 240, "Input"],
Cell[CellGroupData[{
Cell[4866745, 130066, 265, 5, 2147483647, "Print",
 CellTags->"Info3626009464-6741741"],
Cell[4867013, 130073, 1753, 46, 2147483647, "Print",
 CellTags->"Info3626009464-6741741"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[4868815, 130125, 182, 3, 176, "Input"],
Cell[4869000, 130130, 131, 2, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4869168, 130137, 1249, 31, 2147483647, "Input"],
Cell[4870420, 130170, 715, 20, 2147483647, "Output"]
}, Closed]],
Cell[4871150, 130193, 393, 9, 2147483647, "Text"],
Cell[4871546, 130204, 188, 4, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[4871759, 130212, 483, 10, 2147483647, "Input"],
Cell[4872245, 130224, 13844, 337, 2147483647, "Output"]
}, Closed]],
Cell[4886104, 130564, 129, 2, 2147483647, "Text"],
Cell[4886236, 130568, 556, 11, 2147483647, "Text"],
Cell[4886795, 130581, 186, 5, 2147483647, "Input"],
Cell[4886984, 130588, 107, 3, 2147483647, "Input"],
Cell[4887094, 130593, 118, 2, 2147483647, "Input"],
Cell[4887215, 130597, 148, 3, 2147483647, "Input"],
Cell[4887366, 130602, 110, 3, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[4887501, 130609, 1016, 26, 2147483647, "Input"],
Cell[4888520, 130637, 72, 1, 2147483647, "Output"]
}, Closed]],
Cell[4888607, 130641, 104, 1, 2147483647, "Text"],
Cell[4888714, 130644, 105, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[4888844, 130649, 687, 17, 2147483647, "Input"],
Cell[4889534, 130668, 116, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4889687, 130676, 304, 7, 2147483647, "Input"],
Cell[4889994, 130685, 116, 3, 2147483647, "Output"]
}, Closed]],
Cell[4890125, 130691, 132, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[4890282, 130696, 97, 1, 2147483647, "Input"],
Cell[4890382, 130699, 70, 1, 2147483647, "Output"]
}, Closed]],
Cell[4890467, 130703, 214, 2, 2147483647, "Text"],
Cell[4890684, 130707, 974, 26, 2147483647, "Input",
 Evaluatable->False],
Cell[4891661, 130735, 275, 5, 2147483647, "Text"],
Cell[4891939, 130742, 489, 16, 2147483647, "Input",
 Evaluatable->False],
Cell[4892431, 130760, 125, 2, 2147483647, "Text"],
Cell[4892559, 130764, 410, 8, 2147483647, "Text"],
Cell[4892972, 130774, 514, 10, 2147483647, "Text"],
Cell[4893489, 130786, 207, 6, 2147483647, "Text"],
Cell[4893699, 130794, 743, 21, 2147483647, "Input"],
Cell[4894445, 130817, 218, 4, 2147483647, "Text"],
Cell[4894666, 130823, 641, 16, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[4895332, 130843, 414, 12, 2147483647, "Input"],
Cell[4895749, 130857, 431, 14, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4896217, 130876, 433, 9, 2147483647, "Input"],
Cell[4896653, 130887, 72, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4896762, 130893, 467, 10, 2147483647, "Input"],
Cell[4897232, 130905, 72, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4897341, 130911, 178, 4, 2147483647, "Input"],
Cell[4897522, 130917, 291, 9, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4897850, 130931, 559, 15, 2147483647, "Input"],
Cell[4898412, 130948, 117, 3, 2147483647, "Output"]
}, Closed]],
Cell[4898544, 130954, 164, 3, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[4898733, 130961, 325, 8, 2147483647, "Input"],
Cell[4899061, 130971, 333, 11, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4899431, 130987, 239, 6, 2147483647, "Input"],
Cell[4899673, 130995, 293, 9, 2147483647, "Output"]
}, Closed]],
Cell[4899981, 131007, 409, 7, 2147483647, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[4900427, 131019, 743, 9, 60, "Subsubsection"],
Cell[4901173, 131030, 1076, 17, 2147483647, "Text"],
Cell[4902252, 131049, 391, 9, 2147483647, "Input"],
Cell[4902646, 131060, 118, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[4902789, 131065, 195, 4, 2147483647, "Input"],
Cell[4902987, 131071, 379, 12, 2147483647, "Output"]
}, Closed]],
Cell[4903381, 131086, 409, 10, 2147483647, "Text"],
Cell[4903793, 131098, 5577, 110, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[4909395, 131212, 383, 6, 2147483647, "Input"],
Cell[4909781, 131220, 545, 8, 2147483647, "Print",
 CellTags->"Info3626076175-1128913"]
}, Closed]],
Cell[4910341, 131231, 184, 6, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[4910550, 131241, 1063, 25, 2147483647, "Input"],
Cell[4911616, 131268, 1143, 35, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[4912796, 131308, 1205, 27, 2147483647, "Input"],
Cell[4914004, 131337, 153663, 4061, 2147483647, "Output"]
}, Closed]],
Cell[5067682, 135401, 170, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5067877, 135410, 1158, 26, 2147483647, "Input"],
Cell[5069038, 135438, 303, 9, 2147483647, "Output"]
}, Closed]],
Cell[5069356, 135450, 226, 5, 2147483647, "Text"],
Cell[5069585, 135457, 469, 11, 2147483647, "Text"],
Cell[5070057, 135470, 964, 19, 2147483647, "Text"],
Cell[5071024, 135491, 722, 18, 2147483647, "Text"],
Cell[5071749, 135511, 367, 6, 2147483647, "Text"],
Cell[5072119, 135519, 336, 8, 2147483647, "Text"],
Cell[5072458, 135529, 244, 6, 2147483647, "Input"],
Cell[5072705, 135537, 391, 7, 2147483647, "Text"],
Cell[5073099, 135546, 707, 20, 2147483647, "Input",
 Evaluatable->False],
Cell[5073809, 135568, 457, 10, 2147483647, "Text"],
Cell[5074269, 135580, 562, 16, 2147483647, "Input"],
Cell[5074834, 135598, 72, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5074931, 135603, 276, 7, 2147483647, "Input"],
Cell[5075210, 135612, 1145, 35, 2147483647, "Output"]
}, Closed]],
Cell[5076370, 135650, 179, 2, 2147483647, "Text"],
Cell[5076552, 135654, 1358, 40, 2147483647, "Input"],
Cell[5077913, 135696, 239, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5078177, 135705, 1415, 34, 2147483647, "Input"],
Cell[5079595, 135741, 70, 1, 2147483647, "Output"]
}, Closed]],
Cell[5079680, 135745, 142, 4, 2147483647, "Text"],
Cell[5079825, 135751, 372, 7, 2147483647, "Text"],
Cell[5080200, 135760, 2740, 68, 2147483647, "Input"],
Cell[5082943, 135830, 291, 6, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5083259, 135840, 1424, 34, 2147483647, "Input"],
Cell[5084686, 135876, 70, 1, 2147483647, "Output"]
}, Closed]],
Cell[5084771, 135880, 533, 8, 2147483647, "Text"],
Cell[5085307, 135890, 960, 29, 2147483647, "Input"],
Cell[5086270, 135921, 1353, 30, 2147483647, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[5087660, 135956, 813, 10, 60, "Subsubsection"],
Cell[5088476, 135968, 1110, 23, 2147483647, "Text"],
Cell[5089589, 135993, 343, 8, 2147483647, "Input"],
Cell[5089935, 136003, 176, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5090136, 136012, 510, 10, 2147483647, "Input"],
Cell[5090649, 136024, 337, 9, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5091023, 136038, 453, 12, 2147483647, "Input"],
Cell[5091479, 136052, 523, 16, 2147483647, "Output"]
}, Closed]],
Cell[5092017, 136071, 247, 5, 2147483647, "Text"],
Cell[5092267, 136078, 165, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5092457, 136087, 442, 10, 2147483647, "Input"],
Cell[5092902, 136099, 1013, 30, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5093952, 136134, 451, 12, 2147483647, "Input"],
Cell[5094406, 136148, 1360, 39, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5095803, 136192, 1387, 40, 2147483647, "Input"],
Cell[5097193, 136234, 1336, 39, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5098566, 136278, 1937, 55, 2147483647, "Input"],
Cell[5100506, 136335, 1760, 51, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5102303, 136391, 1811, 52, 2147483647, "Input"],
Cell[5104117, 136445, 1760, 51, 2147483647, "Output"]
}, Closed]],
Cell[5105892, 136499, 133, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5106050, 136504, 2195, 60, 2147483647, "Input"],
Cell[5108248, 136566, 881, 28, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5109166, 136599, 345, 9, 2147483647, "Input"],
Cell[5109514, 136610, 351, 11, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5109902, 136626, 347, 9, 2147483647, "Input"],
Cell[5110252, 136637, 349, 11, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5110638, 136653, 683, 16, 2147483647, "Input"],
Cell[5111324, 136671, 7510, 129, 2147483647, "Output"]
}, Closed]],
Cell[5118849, 136803, 138, 3, 2147483647, "Input"],
Cell[5118990, 136808, 298, 7, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5119313, 136819, 321, 8, 2147483647, "Input"],
Cell[5119637, 136829, 165, 4, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5119839, 136838, 813, 21, 2147483647, "Input"],
Cell[5120655, 136861, 7533, 130, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5128225, 136996, 255, 6, 2147483647, "Input"],
Cell[5128483, 137004, 14622, 248, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5143142, 137257, 1038, 28, 2147483647, "Input"],
Cell[5144183, 137287, 2179, 44, 2147483647, "Output"]
}, Closed]],
Cell[5146377, 137334, 296, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5146698, 137343, 715, 16, 2147483647, "Input"],
Cell[5147416, 137361, 117, 3, 2147483647, "Output"]
}, Closed]],
Cell[5147548, 137367, 382, 6, 2147483647, "Text"],
Cell[5147933, 137375, 273, 5, 2147483647, "Input"],
Cell[5148209, 137382, 1435, 33, 2147483647, "Text"],
Cell[5149647, 137417, 249, 6, 2147483647, "Text"],
Cell[5149899, 137425, 147, 3, 2147483647, "Input"],
Cell[5150049, 137430, 457, 13, 2147483647, "Input",
 InitializationCell->True],
Cell[5150509, 137445, 294, 8, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[5150828, 137457, 145, 3, 2147483647, "Input"],
Cell[5150976, 137462, 668, 21, 2147483647, "Output"]
}, Closed]],
Cell[5151659, 137486, 200, 4, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5151884, 137494, 486, 12, 2147483647, "Input"],
Cell[5152373, 137508, 141, 3, 2147483647, "Output"]
}, Closed]],
Cell[5152529, 137514, 366, 7, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5152920, 137525, 629, 15, 2147483647, "Input"],
Cell[5153552, 137542, 11685, 343, 2147483647, "Output"]
}, Closed]],
Cell[5165252, 137888, 492, 8, 2147483647, "Text"],
Cell[5165747, 137898, 3522, 99, 2147483647, "Input"],
Cell[5169272, 137999, 547, 9, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[5169844, 138012, 838, 26, 2147483647, "Input"],
Cell[5170685, 138040, 836, 26, 2147483647, "Output"]
}, Closed]],
Cell[5171536, 138069, 844, 16, 2147483647, "Text"],
Cell[5172383, 138087, 760, 22, 2147483647, "Input",
 Evaluatable->False],
Cell[5173146, 138111, 533, 11, 2147483647, "Text"],
Cell[5173682, 138124, 169, 3, 2147483647, "Text"],
Cell[5173854, 138129, 477, 14, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[5174356, 138147, 429, 9, 2147483647, "Input"],
Cell[5174788, 138158, 1227, 41, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5176052, 138204, 166, 3, 2147483647, "Input"],
Cell[5176221, 138209, 1038, 32, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5177296, 138246, 623, 15, 2147483647, "Input"],
Cell[5177922, 138263, 94, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5178053, 138269, 720, 19, 2147483647, "Input"],
Cell[5178776, 138290, 523, 17, 2147483647, "Output"]
}, Closed]],
Cell[5179314, 138310, 229, 6, 2147483647, "Text"],
Cell[5179546, 138318, 181, 4, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5179752, 138326, 529, 14, 2147483647, "Input"],
Cell[5180284, 138342, 139, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5180460, 138350, 581, 15, 2147483647, "Input"],
Cell[5181044, 138367, 139, 3, 2147483647, "Output"]
}, Closed]],
Cell[5181198, 138373, 138, 2, 2147483647, "Text"],
Cell[5181339, 138377, 155, 3, 2147483647, "Input"],
Cell[5181497, 138382, 416, 12, 2147483647, "Input"],
Cell[5181916, 138396, 140, 3, 2147483647, "Input"],
Cell[5182059, 138401, 205, 3, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5182289, 138408, 149, 4, 2147483647, "Input"],
Cell[5182441, 138414, 211, 5, 2147483647, "Output"]
}, Closed]],
Cell[5182667, 138422, 338, 9, 2147483647, "Input"],
Cell[5183008, 138433, 197, 2, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5183230, 138439, 628, 15, 2147483647, "Input"],
Cell[5183861, 138456, 157, 4, 2147483647, "Output"]
}, Closed]],
Cell[5184033, 138463, 114, 1, 2147483647, "Text"],
Cell[5184150, 138466, 147, 3, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[5184322, 138473, 843, 21, 2147483647, "Input"],
Cell[5185168, 138496, 988, 30, 2147483647, "Output"]
}, Closed]],
Cell[5186171, 138529, 144, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5186340, 138534, 1150, 26, 2147483647, "Input"],
Cell[5187493, 138562, 1468, 42, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5188998, 138609, 1498, 43, 2147483647, "Input"],
Cell[5190499, 138654, 1466, 42, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5192002, 138701, 1477, 37, 2147483647, "Input"],
Cell[5193482, 138740, 1763, 51, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5195282, 138796, 1788, 52, 2147483647, "Input"],
Cell[5197073, 138850, 1736, 51, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5198846, 138906, 945, 21, 2147483647, "Input"],
Cell[5199794, 138929, 1237, 37, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5201068, 138971, 1290, 38, 2147483647, "Input"],
Cell[5202361, 139011, 1237, 37, 2147483647, "Output"]
}, Closed]],
Cell[5203613, 139051, 111, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5203749, 139056, 1543, 40, 2147483647, "Input"],
Cell[5205295, 139098, 906, 30, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5206238, 139133, 345, 9, 2147483647, "Input"],
Cell[5206586, 139144, 356, 12, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5206979, 139161, 347, 9, 2147483647, "Input"],
Cell[5207329, 139172, 361, 12, 2147483647, "Output"]
}, Closed]],
Cell[5207705, 139187, 107, 1, 2147483647, "Text"],
Cell[5207815, 139190, 881, 20, 2147483647, "Input"],
Cell[5208699, 139212, 144, 3, 2147483647, "Input"],
Cell[5208846, 139217, 128, 1, 2147483647, "Text"],
Cell[5208977, 139220, 181, 4, 2147483647, "Input"],
Cell[5209161, 139226, 169, 3, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5209355, 139233, 617, 16, 2147483647, "Input"],
Cell[5209975, 139251, 3019, 102, 2147483647, "Output"]
}, Closed]],
Cell[5213009, 139356, 530, 15, 2147483647, "Input"],
Cell[5213542, 139373, 233, 5, 2147483647, "Text"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[5213824, 139384, 897, 12, 95, "Subsection"],
Cell[5214724, 139398, 410, 9, 60, "Text"],
Cell[5215137, 139409, 668, 17, 60, "Input"],
Cell[5215808, 139428, 1422, 20, 288, "Text"],
Cell[CellGroupData[{
Cell[5217255, 139452, 144, 1, 76, "Subsubsection"],
Cell[5217402, 139455, 867, 16, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5218294, 139475, 154, 3, 2147483647, "Input"],
Cell[5218451, 139480, 33039, 973, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5251527, 140458, 631, 14, 2147483647, "Input"],
Cell[5252161, 140474, 7495, 247, 2147483647, "Output"]
}, Closed]],
Cell[5259671, 140724, 562, 10, 2147483647, "Text"],
Cell[5260236, 140736, 7807, 249, 2147483647, "Input"],
Cell[5268046, 140987, 114, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5268185, 140992, 1096, 24, 2147483647, "Input"],
Cell[5269284, 141018, 148, 2, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5269469, 141025, 956, 19, 2147483647, "Input"],
Cell[5270428, 141046, 234217, 6054, 2147483647, "Output"]
}, Closed]],
Cell[5504660, 147103, 349, 4, 2147483647, "Text"],
Cell[5505012, 147109, 189004, 4651, 2147483647, "Input"],
Cell[5694019, 151762, 1327, 33, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5695371, 151799, 1357, 32, 2147483647, "Input"],
Cell[5696731, 151833, 92, 1, 2147483647, "Output"]
}, Closed]],
Cell[5696838, 151837, 104, 1, 2147483647, "Text"],
Cell[5696945, 151840, 539, 9, 2147483647, "Text"],
Cell[5697487, 151851, 131, 2, 2147483647, "Input"],
Cell[5697621, 151855, 259, 5, 2147483647, "Text"],
Cell[5697883, 151862, 3844, 86, 2147483647, "Input",
 Evaluatable->False],
Cell[5701730, 151950, 229, 4, 2147483647, "Input"],
Cell[5701962, 151956, 1208, 19, 2147483647, "Text"],
Cell[5703173, 151977, 504, 14, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5703702, 151995, 1071, 26, 2147483647, "Input"],
Cell[5704776, 152023, 70, 1, 2147483647, "Output"]
}, Closed]],
Cell[5704861, 152027, 875, 15, 2147483647, "Text"],
Cell[5705739, 152044, 520, 9, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5706284, 152057, 993, 23, 2147483647, "Input"],
Cell[5707280, 152082, 24502, 601, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5731819, 152688, 215, 4, 2147483647, "Input"],
Cell[5732037, 152694, 72, 1, 2147483647, "Output"]
}, Closed]],
Cell[5732124, 152698, 686, 10, 2147483647, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[5732847, 152713, 121, 1, 60, "Subsubsection"],
Cell[CellGroupData[{
Cell[5732993, 152718, 208, 5, 2147483647, "Input"],
Cell[5733204, 152725, 426, 12, 2147483647, "Output"]
}, Closed]],
Cell[5733645, 152740, 188, 2, 2147483647, "Text"],
Cell[5733836, 152744, 293, 8, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[5734154, 152756, 636, 17, 2147483647, "Input"],
Cell[5734793, 152775, 2014, 39, 2147483647, "Output"]
}, Closed]],
Cell[5736822, 152817, 150, 3, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[5736997, 152824, 723, 19, 2147483647, "Input"],
Cell[5737723, 152845, 35774, 595, 2147483647, "Output"]
}, Closed]],
Cell[5773512, 153443, 147, 3, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[5773684, 153450, 251, 6, 2147483647, "Input"],
Cell[5773938, 153458, 37367, 623, 2147483647, "Output"]
}, Closed]],
Cell[5811320, 154084, 194, 2, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5811539, 154090, 826, 14, 2147483647, "Input"],
Cell[5812368, 154106, 2994, 84, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5815399, 154195, 430, 11, 2147483647, "Input"],
Cell[5815832, 154208, 233, 6, 2147483647, "Output"]
}, Closed]],
Cell[5816080, 154217, 181, 2, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5816286, 154223, 814, 18, 2147483647, "Input"],
Cell[5817103, 154243, 161, 4, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5817301, 154252, 630, 15, 2147483647, "Input"],
Cell[5817934, 154269, 674, 22, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5818645, 154296, 408, 10, 2147483647, "Input"],
Cell[5819056, 154308, 186, 5, 2147483647, "Output"]
}, Closed]],
Cell[5819257, 154316, 115, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5819397, 154321, 201, 5, 2147483647, "Input"],
Cell[5819601, 154328, 402, 13, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5820040, 154346, 345, 9, 2147483647, "Input"],
Cell[5820388, 154357, 94, 1, 2147483647, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[5820531, 154364, 231, 3, 60, "Subsubsection"],
Cell[5820765, 154369, 356, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5821146, 154378, 180, 4, 2147483647, "Input"],
Cell[5821329, 154384, 356, 10, 2147483647, "Output"]
}, Closed]],
Cell[5821700, 154397, 712, 15, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[5822437, 154416, 128, 2, 2147483647, "Input"],
Cell[5822568, 154420, 659, 17, 2147483647, "Output"]
}, Closed]],
Cell[5823242, 154440, 224, 5, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[5823491, 154449, 55, 1, 2147483647, "Input"],
Cell[5823549, 154452, 17567, 446, 2147483647, "Output"]
}, Closed]],
Cell[5841131, 154901, 91, 1, 2147483647, "Text"],
Cell[5841225, 154904, 626, 13, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[5841876, 154921, 125, 2, 2147483647, "Input"],
Cell[5842004, 154925, 30173, 755, 2147483647, "Output"]
}, Closed]],
Cell[5872192, 155683, 90, 1, 2147483647, "Text"],
Cell[5872285, 155686, 448, 10, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[5872758, 155700, 286, 5, 2147483647, "Input"],
Cell[5873047, 155707, 42556, 1019, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5915640, 156731, 589, 14, 2147483647, "Input"],
Cell[5916232, 156747, 116, 2, 2147483647, "Output"]
}, Closed]],
Cell[5916363, 156752, 87, 1, 2147483647, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[5916487, 156758, 263, 3, 60, "Subsubsection"],
Cell[5916753, 156763, 281, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5917059, 156772, 120, 2, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[5917204, 156778, 127, 2, 2147483647, "Print",
 CellTags->"Info3626685166-6153013"],
Cell[5917334, 156782, 2463, 61, 2147483647, "Print",
 CellTags->"Info3626685166-6153013"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[5919846, 156849, 281, 7, 2147483647, "Input"],
Cell[5920130, 156858, 729, 21, 2147483647, "Output"]
}, Closed]],
Cell[5920874, 156882, 167, 3, 2147483647, "Text"],
Cell[5921044, 156887, 214, 5, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[5921283, 156896, 729, 17, 2147483647, "Input"],
Cell[5922015, 156915, 1048, 31, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5923100, 156951, 348, 8, 2147483647, "Input"],
Cell[5923451, 156961, 906, 27, 2147483647, "Output"]
}, Closed]],
Cell[5924372, 156991, 1552, 40, 2147483647, "Input"],
Cell[5925927, 157033, 741, 19, 2147483647, "Input"],
Cell[5926671, 157054, 187, 2, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[5926883, 157060, 1078, 24, 2147483647, "Input"],
Cell[5927964, 157086, 46535, 771, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[5974536, 157862, 276, 7, 2147483647, "Input"],
Cell[5974815, 157871, 172900, 2819, 2147483647, "Output"]
}, Closed]],
Cell[6147730, 160693, 307, 8, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6148062, 160705, 313, 7, 2147483647, "Input"],
Cell[6148378, 160714, 9062, 249, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6157477, 160968, 248, 7, 2147483647, "Input"],
Cell[6157728, 160977, 4686, 131, 2147483647, "Output"]
}, Closed]],
Cell[6162429, 161111, 4906, 133, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6167360, 161248, 411, 11, 2147483647, "Input"],
Cell[6167774, 161261, 360, 5, 2147483647, "Output"]
}, Closed]],
Cell[6168149, 161269, 115, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[6168289, 161274, 114, 2, 2147483647, "Input"],
Cell[6168406, 161278, 322, 6, 2147483647, "Print",
 CellTags->"Info3626783633-5796653"]
}, Closed]],
Cell[CellGroupData[{
Cell[6168765, 161289, 652, 18, 2147483647, "Input"],
Cell[6169420, 161309, 603, 14, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6170060, 161328, 581, 16, 2147483647, "Input"],
Cell[6170644, 161346, 172, 3, 2147483647, "Output"]
}, Closed]],
Cell[6170831, 161352, 124, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[6170980, 161357, 160, 3, 2147483647, "Input"],
Cell[6171143, 161362, 29412, 736, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6200592, 162103, 293, 8, 2147483647, "Input"],
Cell[6200888, 162113, 675, 20, 2147483647, "Output"]
}, Closed]],
Cell[6201578, 162136, 783, 22, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6202386, 162162, 240, 6, 2147483647, "Input"],
Cell[6202629, 162170, 131246, 3105, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6333912, 165280, 401, 10, 2147483647, "Input"],
Cell[6334316, 165292, 467, 13, 2147483647, "Output"]
}, Closed]],
Cell[6334798, 165308, 204, 5, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6335027, 165317, 236, 6, 2147483647, "Input"],
Cell[6335266, 165325, 179935, 4136, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6515238, 169466, 401, 10, 2147483647, "Input"],
Cell[6515642, 169478, 339, 10, 2147483647, "Output"]
}, Closed]],
Cell[6515996, 169491, 357, 11, 2147483647, "Input"],
Cell[6516356, 169504, 288, 5, 2147483647, "Text"],
Cell[6516647, 169511, 259, 7, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6516931, 169522, 234, 6, 2147483647, "Input"],
Cell[6517168, 169530, 2135, 60, 2147483647, "Output"]
}, Closed]],
Cell[6519318, 169593, 179, 4, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6519522, 169601, 876, 25, 2147483647, "Input"],
Cell[6520401, 169628, 91, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6520529, 169634, 116, 2, 2147483647, "Input"],
Cell[6520648, 169638, 543, 8, 2147483647, "Print",
 CellTags->"Info3626787084-5796653"]
}, Closed]],
Cell[CellGroupData[{
Cell[6521228, 169651, 284, 7, 2147483647, "Input"],
Cell[6521515, 169660, 52665, 1344, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6574217, 171009, 286, 8, 2147483647, "Input"],
Cell[6574506, 171019, 633, 19, 2147483647, "Output"]
}, Closed]],
Cell[6575154, 171041, 845, 23, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6576024, 171068, 552, 17, 2147483647, "Input"],
Cell[6576579, 171087, 5844, 162, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6582460, 171254, 288, 8, 2147483647, "Input"],
Cell[6582751, 171264, 270, 8, 2147483647, "Output"]
}, Closed]],
Cell[6583036, 171275, 453, 12, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6583514, 171291, 378, 11, 2147483647, "Input"],
Cell[6583895, 171304, 94, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6584026, 171310, 621, 17, 2147483647, "Input"],
Cell[6584650, 171329, 147, 2, 2147483647, "Output"]
}, Closed]],
Cell[6584812, 171334, 87, 1, 2147483647, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[6584936, 171340, 303, 4, 60, "Subsubsection"],
Cell[6585242, 171346, 844, 19, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6586111, 171369, 128, 2, 2147483647, "Input"],
Cell[6586242, 171373, 600, 15, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6586879, 171393, 497, 12, 2147483647, "Input"],
Cell[6587379, 171407, 17108, 287, 2147483647, "Output"]
}, Closed]],
Cell[6604502, 171697, 145, 3, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6604672, 171704, 300, 7, 2147483647, "Input"],
Cell[6604975, 171713, 52476, 871, 2147483647, "Output"]
}, Closed]],
Cell[6657466, 172587, 538, 11, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6658029, 172602, 225, 4, 2147483647, "Input"],
Cell[6658257, 172608, 9212, 257, 2147483647, "Output"]
}, Closed]],
Cell[6667484, 172868, 377, 9, 2147483647, "Input"],
Cell[6667864, 172879, 721, 14, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6668610, 172897, 326, 5, 2147483647, "Input"],
Cell[6668939, 172904, 25623, 646, 2147483647, "Output"]
}, Closed]],
Cell[6694577, 173553, 565, 13, 2147483647, "Input"],
Cell[6695145, 173568, 517, 11, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6695687, 173583, 224, 4, 2147483647, "Input"],
Cell[6695914, 173589, 28616, 690, 2147483647, "Output"]
}, Closed]],
Cell[6724545, 174282, 450, 10, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6725020, 174296, 589, 14, 2147483647, "Input"],
Cell[6725612, 174312, 289, 4, 2147483647, "Output"]
}, Closed]],
Cell[6725916, 174319, 87, 1, 2147483647, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[6726040, 174325, 242, 3, 60, "Subsubsection"],
Cell[6726285, 174330, 486, 9, 2147483647, "Text"],
Cell[6726774, 174341, 303, 6, 2147483647, "Text"],
Cell[6727080, 174349, 1062, 24, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6728167, 174377, 382, 7, 2147483647, "Input"],
Cell[6728552, 174386, 1036, 37, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6729625, 174428, 614, 13, 2147483647, "Input"],
Cell[6730242, 174443, 1595, 56, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6731874, 174504, 697, 14, 2147483647, "Input"],
Cell[6732574, 174520, 1189, 43, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6733800, 174568, 282, 7, 2147483647, "Input"],
Cell[6734085, 174577, 119, 2, 2147483647, "Output"]
}, Closed]],
Cell[6734219, 174582, 153, 2, 2147483647, "Text"],
Cell[6734375, 174586, 254, 7, 2147483647, "Input"],
Cell[6734632, 174595, 1193, 26, 2147483647, "Input"],
Cell[6735828, 174623, 419, 12, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6736272, 174639, 225, 5, 2147483647, "Input"],
Cell[6736500, 174646, 428, 15, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6736965, 174666, 819, 16, 2147483647, "Input"],
Cell[6737787, 174684, 576, 19, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6738400, 174708, 722, 14, 2147483647, "Input"],
Cell[6739125, 174724, 419, 14, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6739581, 174743, 282, 7, 2147483647, "Input"],
Cell[6739866, 174752, 92, 1, 2147483647, "Output"]
}, Closed]],
Cell[6739973, 174756, 89, 1, 2147483647, "Text"],
Cell[6740065, 174759, 461, 9, 2147483647, "Text"],
Cell[6740529, 174770, 282, 8, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[6740836, 174782, 124, 2, 2147483647, "Input"],
Cell[6740963, 174786, 320, 11, 2147483647, "Output"]
}, Closed]],
Cell[6741298, 174800, 123, 1, 2147483647, "Text"],
Cell[6741424, 174803, 231, 5, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6741680, 174812, 555, 13, 2147483647, "Input"],
Cell[6742238, 174827, 589, 20, 2147483647, "Output"]
}, Closed]],
Cell[6742842, 174850, 540, 13, 2147483647, "Input"],
Cell[6743385, 174865, 148, 3, 2147483647, "Input"],
Cell[6743536, 174870, 578, 13, 2147483647, "Input"],
Cell[6744117, 174885, 145, 3, 2147483647, "Input"],
Cell[6744265, 174890, 253, 6, 2147483647, "Input"],
Cell[6744521, 174898, 255, 5, 2147483647, "Text"],
Cell[6744779, 174905, 569, 11, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6745373, 174920, 251, 5, 2147483647, "Input"],
Cell[6745627, 174927, 348, 12, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6746012, 174944, 665, 11, 2147483647, "Input"],
Cell[6746680, 174957, 151, 4, 2147483647, "Output"]
}, Closed]],
Cell[6746846, 174964, 114, 1, 2147483647, "Text"],
Cell[6746963, 174967, 1173, 20, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6748161, 174991, 1123, 20, 2147483647, "Input"],
Cell[6749287, 175013, 123, 2, 2147483647, "Output"]
}, Closed]],
Cell[6749425, 175018, 159, 2, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[6749609, 175024, 1176, 20, 2147483647, "Input"],
Cell[6750788, 175046, 153, 4, 2147483647, "Output"]
}, Closed]],
Cell[6750956, 175053, 132, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[6751113, 175058, 151, 3, 2147483647, "Input"],
Cell[6751267, 175063, 571, 20, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6751875, 175088, 1189, 22, 2147483647, "Input"],
Cell[6753067, 175112, 121, 2, 2147483647, "Output"]
}, Closed]],
Cell[6753203, 175117, 126, 1, 2147483647, "Text"],
Cell[6753332, 175120, 328, 7, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6753685, 175131, 123, 2, 2147483647, "Input"],
Cell[6753811, 175135, 213, 6, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6754061, 175146, 366, 8, 2147483647, "Input"],
Cell[6754430, 175156, 292, 10, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6754759, 175171, 118, 2, 2147483647, "Input"],
Cell[6754880, 175175, 290, 5, 2147483647, "Print",
 CellTags->"Info3626265383-1010981"]
}, Closed]],
Cell[CellGroupData[{
Cell[6755207, 175185, 331, 7, 2147483647, "Input"],
Cell[6755541, 175194, 177, 5, 2147483647, "Output"]
}, Closed]],
Cell[6755733, 175202, 297, 6, 2147483647, "Text"],
Cell[6756033, 175210, 481, 12, 2147483647, "Input"],
Cell[6756517, 175224, 148, 3, 2147483647, "Input"],
Cell[6756668, 175229, 317, 7, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6757010, 175240, 1461, 26, 2147483647, "Input"],
Cell[6758474, 175268, 280, 9, 2147483647, "Output"]
}, Closed]],
Cell[6758769, 175280, 330, 6, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[6759124, 175290, 441, 10, 2147483647, "Input"],
Cell[6759568, 175302, 204, 6, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6759809, 175313, 450, 11, 2147483647, "Input"],
Cell[6760262, 175326, 121, 2, 2147483647, "Output"]
}, Closed]],
Cell[6760398, 175331, 87, 1, 2147483647, "Text"],
Cell[6760488, 175334, 504, 10, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[6761017, 175348, 179, 4, 2147483647, "Input"],
Cell[6761199, 175354, 397, 11, 2147483647, "Output"]
}, Closed]],
Cell[6761611, 175368, 425, 12, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6762061, 175384, 551, 14, 2147483647, "Input"],
Cell[6762615, 175400, 264, 8, 2147483647, "Output"]
}, Closed]],
Cell[6762894, 175411, 150, 3, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[6763069, 175418, 637, 15, 2147483647, "Input"],
Cell[6763709, 175435, 123, 2, 2147483647, "Output"]
}, Closed]],
Cell[6763847, 175440, 147, 2, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[6764019, 175446, 791, 20, 2147483647, "Input"],
Cell[6764813, 175468, 121, 2, 2147483647, "Output"]
}, Closed]],
Cell[6764949, 175473, 115, 1, 2147483647, "Text"],
Cell[6765067, 175476, 157, 2, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[6765249, 175482, 737, 17, 2147483647, "Input"],
Cell[6765989, 175501, 222, 6, 2147483647, "Output"]
}, Closed]],
Cell[6766226, 175510, 406, 8, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6766657, 175522, 123, 2, 2147483647, "Input"],
Cell[6766783, 175526, 430, 12, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6767250, 175543, 523, 13, 2147483647, "Input"],
Cell[6767776, 175558, 219, 6, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6768032, 175569, 303, 7, 2147483647, "Input"],
Cell[6768338, 175578, 121, 2, 2147483647, "Output"]
}, Closed]],
Cell[6768474, 175583, 145, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[6768644, 175588, 904, 22, 2147483647, "Input"],
Cell[6769551, 175612, 314, 9, 2147483647, "Output"]
}, Closed]],
Cell[6769880, 175624, 117, 1, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[6770022, 175629, 960, 23, 2147483647, "Input"],
Cell[6770985, 175654, 272, 8, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6771294, 175667, 247, 6, 2147483647, "Input"],
Cell[6771544, 175675, 427, 12, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6772008, 175692, 478, 12, 2147483647, "Input"],
Cell[6772489, 175706, 197, 5, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6772723, 175716, 426, 12, 2147483647, "Input"],
Cell[6773152, 175730, 119, 2, 2147483647, "Output"]
}, Closed]],
Cell[6773286, 175735, 87, 1, 2147483647, "Text"],
Cell[6773376, 175738, 513, 10, 2147483647, "Text"],
Cell[6773892, 175750, 472, 11, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[6774389, 175765, 151, 4, 2147483647, "Input"],
Cell[6774543, 175771, 346, 10, 2147483647, "Output"]
}, Closed]],
Cell[6774904, 175784, 167, 2, 2147483647, "Text"],
Cell[6775074, 175788, 629, 10, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6775728, 175802, 1058, 21, 2147483647, "Input"],
Cell[6776789, 175825, 117, 2, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6776943, 175832, 791, 15, 2147483647, "Input"],
Cell[6777737, 175849, 698, 22, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6778472, 175876, 654, 16, 2147483647, "Input"],
Cell[6779129, 175894, 684, 22, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6779850, 175921, 202, 5, 2147483647, "Input"],
Cell[6780055, 175928, 120, 2, 2147483647, "Output"]
}, Closed]],
Cell[6780190, 175933, 145, 1, 2147483647, "Text"],
Cell[6780338, 175936, 629, 10, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6780992, 175950, 1325, 29, 2147483647, "Input"],
Cell[6782320, 175981, 571, 18, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6782928, 176004, 816, 15, 2147483647, "Input"],
Cell[6783747, 176021, 420, 14, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6784204, 176040, 874, 22, 2147483647, "Input"],
Cell[6785081, 176064, 170, 4, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6785288, 176073, 336, 8, 2147483647, "Input"],
Cell[6785627, 176083, 121, 2, 2147483647, "Output"]
}, Closed]],
Cell[6785763, 176088, 87, 1, 2147483647, "Text"],
Cell[6785853, 176091, 549, 10, 2147483647, "Text"],
Cell[6786405, 176103, 374, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[6786804, 176112, 123, 2, 2147483647, "Input"],
Cell[6786930, 176116, 322, 5, 2147483647, "Print",
 CellTags->"Info3626271082-4060188"]
}, Closed]],
Cell[CellGroupData[{
Cell[6787289, 176126, 261, 6, 2147483647, "Input"],
Cell[6787553, 176134, 526, 16, 2147483647, "Output"]
}, Closed]],
Cell[6788094, 176153, 391, 10, 2147483647, "Input"],
Cell[6788488, 176165, 1426, 36, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[6789939, 176205, 160, 4, 2147483647, "Input"],
Cell[6790102, 176211, 526, 16, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6790665, 176232, 543, 14, 2147483647, "Input"],
Cell[6791211, 176248, 155, 4, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6791403, 176257, 708, 16, 2147483647, "Input"],
Cell[6792114, 176275, 500, 15, 2147483647, "Output"]
}, Closed]],
Cell[6792629, 176293, 270, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[6792924, 176302, 357, 10, 2147483647, "Input"],
Cell[6793284, 176314, 159, 4, 2147483647, "Output"]
}, Closed]],
Cell[6793458, 176321, 426, 10, 2147483647, "Text"],
Cell[6793887, 176333, 87, 1, 2147483647, "Text"],
Cell[6793977, 176336, 264, 5, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[6794266, 176345, 1370, 29, 2147483647, "Input"],
Cell[6795639, 176376, 112, 2, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6795788, 176383, 1265, 24, 2147483647, "Input"],
Cell[6797056, 176409, 267, 8, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6797360, 176422, 725, 18, 2147483647, "Input"],
Cell[6798088, 176442, 234, 7, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6798359, 176454, 431, 12, 2147483647, "Input"],
Cell[6798793, 176468, 96, 1, 2147483647, "Output"]
}, Closed]],
Cell[6798904, 176472, 85, 1, 2147483647, "Text"],
Cell[6798992, 176475, 1254, 25, 2147483647, "Text"],
Cell[6800249, 176502, 269, 6, 2147483647, "Text"],
Cell[6800521, 176510, 176, 4, 2147483647, "Input",
 InitializationCell->True]
}, Closed]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[6800758, 176521, 397, 5, 111, "Subtitle"],
Cell[6801158, 176528, 350, 7, 98, "Text",
 CellTags->"B-R"],
Cell[6801511, 176537, 609, 13, 136, "Text",
 CellTags->"Gray"],
Cell[6802123, 176552, 827, 18, 136, "Text",
 CellTags->"Gray2"],
Cell[6802953, 176572, 533, 9, 136, "Text",
 CellTags->"Gray3"],
Cell[6803489, 176583, 274, 6, 98, "Text",
 CellTags->"Kr"],
Cell[6803766, 176591, 292, 6, 98, "Text",
 CellTags->"K-N1"],
Cell[6804061, 176599, 417, 8, 98, "Text",
 CellTags->"OS3"],
Cell[CellGroupData[{
Cell[6804503, 176611, 172, 3, 65, "Subsubtitle"],
Cell[6804678, 176616, 372, 7, 60, "Text",
 CellTags->"Home"],
Cell[6805053, 176625, 121, 1, 60, "Text"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
